<?php
require_once(ELC_CLASSES.'Relation.php');
/**
 *
 */
class Accountant extends Relation {
	/**
	 * @var relation Relation
	 */
	var $relation;
	/**
	 * @param $id
	 * @return boolean
	 */
	function set($id=null) {
		$id || $id = $this->id;

		$sql = "SELECT RELATION_ID FROM TBL_ACCOUNTANT WHERE RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();

		if(!($r = $q->fetch()))
			return false;

		$this->relation = new Relation($this->db, $id, true);
		return $this->complete = true;
	}

	function selectOverview() {
		$sql = "SELECT '' as CHECKBOX,
				R.RELATION_ID,
				R.`NAME` as 'Accountant',
				R.EMAIL,
				IP.INFO_PERSON_LAST_NAME AS 'Contact name',
				COUNT(DISTINCT RU.USER_ID) as 'Users',
				COUNT(DISTINCT AC.CRSO_RELATION_ID) as 'CRSOs'
			FROM TBL_ACCOUNTANT as A
			INNER JOIN TBL_RELATION as R
				ON A.RELATION_ID = R.RELATION_ID
			LEFT JOIN TBL_RELATION_USER as RU
				ON R.RELATION_ID = RU.RELATION_ID
			INNER JOIN TBL_INFO_PERSON IP
				ON R.CUSTOMER_CONTACT_ID = IP.INFO_PERSON_ID
			LEFT JOIN TBL_ACCOUNTANT_CRSO as AC
				ON A.RELATION_ID = AC.ACCOUNTANT_RELATION_ID
			GROUP BY A.RELATION_ID
			ORDER BY R.`NAME`";
		$q = $this->db->query($sql);
		$q->execute();
		return $q;
	}


	function selectUserOverview($accountant_id=null) {
		$accountant_id || $accountant_id = $this->relation->id;

		$sql = "SELECT
				U.USER_ID,
				U.USER_NAME,
				CONCAT_WS(' ', IP.INFO_PERSON_FIRST_NAME, IP.INFO_PERSON_MIDDLE_NAME, IP.INFO_PERSON_LAST_NAME) as CONTACT_NAME,
				U.USER_ACTIVE,
				RUI.IS_ADMIN
			FROM TBL_RELATION_USER as RUI
			INNER JOIN TBL_USER as U
				ON RUI.USER_ID = U.USER_ID
			INNER JOIN TBL_INFO_PERSON as IP
				ON U.INFO_PERSON_ID = IP.INFO_PERSON_ID
			WHERE RUI.RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($accountant_id);
		$q->execute();

		return $q;
	}

	/**
	 *
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_ACCOUNTANT';
		$this->objectmap['primary']['relation->id'] = 'RELATION_ID';
	}
	/**
	 * @static
	 */
	public static function getClearanceLevels() {
		return array(
			'AccountantClearanceDeclarationDetails',
			'AccountantClearanceRefundDetails',
			'AccountantClearanceRegistrationDetails'
		);
	}
}
/**#@+
 * Accountant specific Clearance levels.
 */
define('AccountantClearanceUserManagement', 1); // Access to the user management page.
define('AccountantClearanceDeclaration', 2); // Access to declaration related reports/interfaces.
define('AccountantClearanceDeclarationDetails', 4); // Access to declaration details in the reports.
define('AccountantClearanceRefund', 8); // Access to refund related reports/interfaces.
define('AccountantClearanceRefundDetails', 16); // Access to refund details in refund interfaces.
define('AccountantClearanceRegistrationDetails', 32); // Access to participant registration details.
/**#@-*/
?>