<?php
require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'Accountant.php');
/**
 * @author Marten Koedam <marten@basesoft.com>
 * @package WEEEBlackBox
 * @version $Id$
 */
class AccountantCRSO extends AbstractObject {
	/**
	 * @var integer
	 */
	var $id;
	/**
	 * @var CRSO
	 */
	var $crso;
	/**
	 * @var Accountant
	 */
	var $accountant;
	/**
	 * The clearance levels that are possible for this CRSO-Accountant relationship.
	 * @var integer
	 */
	var $available_clearance_levels;
	/**
	 * The start date for when this Accountant became active for this CRSO.
	 * @var string
	 */
	var $start_date;
	/**
	 * The end date on which this Accountant got laid off for this CRSO.
	 * @var string
	 */
	var $end_date;
	/**
	 * @var string
	 */
	var $created;
	/**
	 * @var string
	 */
	var $last_update;
	/**
	 * @param integer $id
	 * @return boolean
	 */
	function set($id=null) {
		$id || $id = $this->id;

		$sql = "SELECT ACCOUNTANT_CRSO_ID, CRSO_RELATION_ID, ACCOUNTANT_RELATION_ID
				AVAILABLE_CLEARANCE_LEVELS, START_DATE, END_DATE, CREATED, LAST_UPDATE
			FROM TBL_ACCOUNTANT_CRSO
			WHERE ACCOUNTANT_CRSO_ID = ?
			";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		return $this->_setFromQuery($q);
	}
	/**
	 * @param integer $accountant_id
	 * @param integer $crso_id
	 * @return boolean
	 */
	function setByAccountantAndCRSO($accountant_id=null, $crso_id=null) {
		$accountant_id || $accountant_id = $this->accountant->relation->id;
		$crso_id || $crso_id = $this->crso->relation->id;
		$sql = "SELECT ACCOUNTANT_CRSO_ID, CRSO_RELATION_ID, ACCOUNTANT_RELATION_ID,
				AVAILABLE_CLEARANCE_LEVELS, START_DATE, END_DATE, CREATED, LAST_UPDATE
			FROM TBL_ACCOUNTANT_CRSO
			WHERE ACCOUNTANT_RELATION_ID = ? AND CRSO_RELATION_ID = ?
			";
		$q = $this->db->query($sql);
		$q->bind_values($accountant_id, $crso_id);
		$q->execute();
		return $this->_setFromQuery($q);
	}
	/**
	 * @param Query $q
	 * @access protected
	 * @return boolean
	 */
	function _setFromQuery($q) {

		if(!($r = $q->fetch()))
			return false;

		$this->id = $r->ACCOUNTANT_CRSO_ID;
		$this->crso = new CRSO($this->db, array('relation'=>new Relation($this->db,$r->CRSO_RELATION_ID)));
		$this->accountant = new Accountant($this->db, array('relation'=>new Relation($this->db,$r->ACCOUNTANT_RELATION_ID)));
		$this->available_clearance_levels = $r->AVAILABLE_CLEARANCE_LEVELS;
		$this->start_date = $r->START_DATE;
		$this->end_date = $r->END_DATE;
		$this->created = $r->CREATED;
		$this->last_update = $r->LAST_UPDATE;
		return $this->complete = true;
	}
	/**
	 * Override update to remove any Clearance Levels to users linked to this Accountant-CRSO that are no longer legit.
	 */
	function update() {
		// TODO: implement
		parent::update();
	}
	/**
	 * Returns a resultset which contains the CRSOs which is linked to the supplied Accountant.
	 *
	 * Takes an optional $active_check boolean argument to get inactive Accountant-CRSO relations as well.
	 *
	 * @param integer $accountant_id
	 * @param integer $product_id
	 * @param boolean $active_check
	 * @return ResultSet
	 */
	function getAccountantCRSOs($accountant_id=null, $product_id=null, $active_check=true) {
		$accountant_id || $accountant_id = $this->accountant->relation->id;

		$values = array($accountant_id);
		$sql = "SELECT AC.ACCOUNTANT_CRSO_ID, AC.CRSO_RELATION_ID, AC.ACCOUNTANT_RELATION_ID,
				R.NAME, AC.AVAILABLE_CLEARANCE_LEVELS, AC.START_DATE, END_DATE, AC.CREATED, AC.LAST_UPDATE
			FROM TBL_ACCOUNTANT_CRSO as AC
			INNER JOIN TBL_RELATION as R
				ON R.RELATION_ID = AC.CRSO_RELATION_ID";
		if(is_numeric($product_id)) {
			$sql .= " INNER JOIN TBL_CRSO_PRODUCT CP ON CP.CRSO_RELATION_ID=AC.CRSO_RELATION_ID ";
		}
		$sql .= " WHERE ACCOUNTANT_RELATION_ID = ? ";
		if($active_check) {
			$date = currentDate();

			$values[] = $date->format('Y-m-d');
			$values[] = $date->format('Y-m-d');

			$sql .= " AND START_DATE <= ? AND (ISNULL(END_DATE) OR END_DATE > ?)";
		}
		if(is_numeric($product_id)) {
			$values[] = $product_id;

			$sql .= " AND CP.PRODUCT_ID=?";
		}
		$q = $this->db->query($sql);
		$q->bind_values($values);
		$q->execute();
		return $q;
	}

	/**
	 * Returns a resultset which contains the Accountants which are linked to the supplied CRSO.
	 *
	 * Takes an optional $active_check boolean argument to get inactive Accountant-CRSO relations as well.
	 *
	 * @param integer $crso_id
	 * @param integer $product_id
	 * @param boolean $active_check
	 * @return ResultSet
	 */
	function getCRSOAccountants($crso_id=null, $product_id=null, $active_check=true) {
		$crso_id || $crso_id = $this->crso->relation->id;

		$values = array($crso_id);
		$sql = "SELECT AC.ACCOUNTANT_CRSO_ID, AC.CRSO_RELATION_ID, AC.ACCOUNTANT_RELATION_ID,
				R.NAME, AC.AVAILABLE_CLEARANCE_LEVELS, AC.START_DATE, END_DATE, AC.CREATED, AC.LAST_UPDATE
			FROM TBL_ACCOUNTANT_CRSO as AC
			INNER JOIN TBL_RELATION as R
				ON R.RELATION_ID = AC.ACCOUNTANT_RELATION_ID";
		if(is_numeric($product_id)) {
			$sql .= " INNER JOIN TBL_CRSO_PRODUCT CP ON CP.CRSO_RELATION_ID=AC.CRSO_RELATION_ID ";
		}
		$sql .= " WHERE AC.CRSO_RELATION_ID = ? ";
		if($active_check) {
			$date = currentDate();

			$values[] = $date->format('Y-m-d');
			$values[] = $date->format('Y-m-d');

			$sql .= " AND START_DATE <= ? AND (ISNULL(END_DATE) OR END_DATE > ?)";
		}
		if(is_numeric($product_id)) {
			$values[] = $product_id;

			$sql .= " AND CP.PRODUCT_ID=?";
		}
		$q = $this->db->query($sql);
		$q->bind_values($values);
		$q->execute();
		return $q;
	}

	/**
	 * selects the crso masterdata for the current accountant (accountant soap method)
	 * @param $accountant_id
	 * @return ResultSet
	 */
	function selectAccountantCRSOMasterData($accountant_id=null) {
		$accountant_id || $accountant_id = $this->accountant->relation->id;

		$sql = "SELECT
			AC.CRSO_RELATION_ID, R.NAME,
			CONCAT(A.ADDRESS_STREET_NAME, ' ', A.ADDRESS_STREET_NUMBER) as ADDRESS, A.ADDRESS_ZIPCODE, A.CITY_NAME,
			CONCAT_WS(' ', I.INFO_PERSON_FIRST_NAME, I.INFO_PERSON_MIDDLE_NAME, I.INFO_PERSON_LAST_NAME) as CONTACT_NAME,

			R.PHONE, R.FAX, L.LANGUAGE_CODE, C.CODE AS COUNTRY_CODE,
			R.VAT_NO, R.EMAIL, R.WEBSITE,

			R.BANK_ACCOUNT_NUMBER, R.IBAN_NO, R.SWIFT_BIC_CODE,
			R.DD_BANK_ACCOUNT, R.DD_IBAN_NO, R.DD_SWIFT_BIC_CODE,
			R.PO_NO

			FROM TBL_ACCOUNTANT_CRSO AC
			INNER JOIN TBL_RELATION R ON AC.CRSO_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_ADDRESS A ON R.ADDRESS_ID = A.ADDRESS_ID
			INNER JOIN TBL_INFO_PERSON I ON R.CUSTOMER_CONTACT_ID = I.INFO_PERSON_ID
			INNER JOIN TBL_LANGUAGE L ON R.INVOICE_LANGUAGE_ID = L.LANGUAGE_ID
			INNER JOIN TBL_COUNTRY C ON A.COUNTRY_ID = C.COUNTRY_ID

			WHERE ACCOUNTANT_RELATION_ID = ? AND START_DATE <= ? AND (ISNULL(END_DATE) OR END_DATE > ?)";

		$date = currentDate()->format('Y-m-d');

		$q = $this->db->query($sql);
		$q->bind_values(array($accountant_id, $date, $date));
		$q->execute();
		return $q;
	}

	/**
	 * @param string $period
	 * @param integer $accountant_id
	 * @param integer $crso_id
	 * @return boolean
	 */
	function isAccountantForDate($period, $accountant_id=null, $crso_id=null) {
		if(!$this->setByAccountantAndCRSO($accountant_id, $crso_id))
		throw new Exception('Accountant is not part of the supplied CRSO');
		return (($this->start_date <= $period) && (!$this->end_date || ($this->end_date > $period)));
	}

	function selectOverview($accountant_id=null) {
		$accountant_id || $accountant_id = $this->accountant->relation->id;

		$sql = "SELECT AC.CRSO_RELATION_ID as `crso_id`, R.`NAME` as CRSO,
				AC.START_DATE, AC.END_DATE, AC.ACCOUNTANT_CRSO_ID as `Settings`
			FROM TBL_ACCOUNTANT_CRSO as AC
			INNER JOIN TBL_RELATION as R
				ON AC.CRSO_RELATION_ID = R.RELATION_ID
			WHERE AC.ACCOUNTANT_RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($accountant_id);
		$q->execute();

		return $q;
	}

	/**
	 *
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_ACCOUNTANT_CRSO';
		$this->objectmap['primary']['id'] = 'ACCOUNTANT_CRSO_ID';
		$this->objectmap['properties']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['accountant->relation->id'] = 'ACCOUNTANT_RELATION_ID';
		$this->objectmap['properties']['available_clearance_levels'] = 'AVAILABLE_CLEARANCE_LEVELS';
		$this->objectmap['properties']['start_date'] = 'START_DATE';
		$this->objectmap['properties']['end_date'] = 'END_DATE';
		$this->objectmap['properties']['created'] = 'CREATED';
		$this->objectmap['properties']['last_update'] = 'LAST_UPDATE';
	}


}
/**#@+
 * Accountant-CRSO Clearance Levels
 */
/**#@-*/
?>