<?php
require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES.'AccountantCRSO.php');
require_once(ELC_CLASSES.'User.php');
/**
 *
 * @author Marten Koedam <marten@basesoft.com>
 *
 */
class AccountantCRSOUser extends AbstractObject {
	/**
	 * @var AccountantCRSO
	 */
	var $accountant_crso;
	/**
	 * @var ELCUser
	 */
	var $user;
	/**
	 * Which Clearance levels (Bit-wise) does this relation have.
	 * @var integer
	 */
	var $clearance_level;
	/**
	 * @var string
	 */
	var $created;
	/**
	 * @var string
	 */
	var $last_update;
	/**
	 * @param integer $accountant_crso_id
	 * @param integer $user_id
	 * @return boolean
	 */
	function set($accountant_crso_id=null, $user_id=null) {
		$accountant_crso_id || $accountant_crso_id = $this->accountant_crso->id;
		$user_id || $user_id = $this->user->id;

		$sql = "SELECT ACCOUNTANT_CRSO_ID, USER_ID, CLEARANCE_LEVEL, CREATED, LAST_UPDATE
			FROM TBL_ACCOUNTANT_CRSO_USER
			WHERE ACCOUNTANT_CRSO_ID = ?
				AND USER_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($accountant_crso_id, $user_id);
		$q->execute();

		if(!($r = $q->fetch()))
			return false;

		$this->accountant_crso = new AccountantCRSO($this->db, $accountant_crso_id);
		$this->user = new ELCUser($this->db, $user_id);
		$this->clearance_level = $r->CLEARANCE_LEVEL;
		$this->created = $r->CREATED;
		$this->last_update = $r->LAST_UPDATE;

		return $this->complete = true;
	}
	/**
	 * Check if the user-accountant_crso relation has a certain clearance level.
	 * @param integer $clearance
	 * @return boolean
	 */
	function has($clearance) {
		if(!$this->isComplete())
			return false;
		return ($this->clearance_level & $clearance == $clearance);
	}

	function _initObjectmaps() {
		$this->objectmap['dbtable'] = 'TBL_ACCOUNTANT_CRSO_USER';
		$this->objectmap['primary']['accountant_crso->id'] = 'ACCOUNTANT_CRSO_ID';
		$this->objectmap['primary']['user->id'] = 'USER_ID';
		$this->objectmap['properties']['clearance_level'] = 'CLEARANCE_LEVEL';
		$this->objectmap['properties']['created'] = 'CREATED';
		$this->objectmap['properties']['last_update'] = 'LAST_UPDATE';

		$this->objectmap['updatetype'] = 'REPLACE';
	}
}
?>