<?php
require_once(IWARE.'AbstractObject.php');
/**
 * @package WEEE
 * @subpackage Declaration
 * @since 8-nov-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class AnnualConfirmation extends AbstractObject
{
	public $id;
	public $crso;
	public $participant;
	public $year;
	public $status;
	public $created;
	public $last_updated;
	public $change_request_status;
	public $product;
	public $review_status;

	function set($id=null)
	{
		$id || $id = $this->id;
		$sql = "SELECT ANNUAL_CONFIRMATION_ID, CRSO_RELATION_ID, PARTICIPANT_RELATION_ID,
				`YEAR`,`STATUS`,CREATED, LAST_UPDATED, PRODUCT_ID, REVIEW_STATUS
			FROM TBL_ANNUAL_CONFIRMATION
			WHERE ANNUAL_CONFIRMATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		return $this->setPropsByQuery($r);
	}

	function setByParticipantCRSOAndYear($participant_id=null,$crso_id=null,$year=null,$product_id=null)
	{
		$participant_id || $participant_id = $this->participant->relation->id;
		$crso_id || $crso_id = $this->crso->relation->id;
		$year || $year = $this->year;
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT ANNUAL_CONFIRMATION_ID, CRSO_RELATION_ID, PARTICIPANT_RELATION_ID,
				`YEAR`,`STATUS`,CREATED, LAST_UPDATED, PRODUCT_ID, REVIEW_STATUS
			FROM TBL_ANNUAL_CONFIRMATION
			WHERE PARTICIPANT_RELATION_ID = ?
				AND CRSO_RELATION_ID = ?
				AND `YEAR` = ?
				AND PRODUCT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id,$crso_id,$year,$product_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		return $this->setPropsByQuery($r);
	}

	protected function setPropsByQuery($r)
	{
		require_once(ELC_CLASSES.'CRSO.php');
		require_once(ELC_CLASSES.'Participant.php');
		require_once(ELC_CLASSES.'Product.php');

		$this->id = $r->ANNUAL_CONFIRMATION_ID;
		$this->crso = new CRSO($this->db,array('relation'=>new Relation($this->db,$r->CRSO_RELATION_ID)));
		$this->participant = new Participant($this->db,array('relation'=>new Relation($this->db,$r->PARTICIPANT_RELATION_ID)));
		$this->product = new Product($this->db,$r->PRODUCT_ID);
		$this->year = $r->YEAR;
		$this->status = $r->STATUS;
		$this->created = $r->CREATED;
		$this->last_updated = $r->LAST_UPDATED;
		$this->review_status = $r->REVIEW_STATUS;

		return $this->complete = true;
	}

	function insert($graceful=false)
	{
		$date = currentDate();
		$this->last_updated = $this->created = $date->format('Y-m-d H:i');

		return parent::insert($graceful);
	}

	function update($graceful=false)
	{
		$date = currentDate();
		$this->last_updated = $date->format('Y-m-d H:i');

		return parent::update($graceful);
	}

	public function getCRSOselect($participant_id=null,$return='q',$product_id=null)
	{
		$participant_id || $participant_id = $this->participant->relation->id;
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT AC.CRSO_RELATION_ID, R.`NAME` as CRSO_NAME
			FROM TBL_ANNUAL_CONFIRMATION as AC
			INNER JOIN TBL_RELATION as R
				ON AC.CRSO_RELATION_ID = R.RELATION_ID
			WHERE AC.PARTICIPANT_RELATION_ID = ?
				AND PRODUCT_ID = ?
			ORDER BY CRSO_NAME";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id,$product_id);
		$q->execute();
		if($return=='q')
			return $q;
		$rv = array();
		while($r=$q->fetch())
			$rv[$r->CRSO_RELATION_ID] = $r->CRSO_NAME;
		return $rv;
	}

	public function getYears($participant_id=null,$crso_id=null,$return='q',$status=false,$product_id=null)
	{
		$participant_id || $participant_id = $this->participant->relation->id;
		$crso_id || $crso_id = $this->crso->relation->id;
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT `YEAR` FROM TBL_ANNUAL_CONFIRMATION
			WHERE CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ?
				AND PRODUCT_ID = ? ";
		if($status)
			$sql .= " AND `STATUS` = ".$status." ";
		$sql .= " ORDER BY `YEAR`";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$participant_id,$product_id);
		$q->execute();
		if($return == 'q')
			return $q;
		$rv = array();
		while($r = $q->fetch())
			$rv[] = $r->YEAR;
		return $rv;
	}

	public function getCRSOYears($crso_id=null,$return='q',$status=false,$product_id=null)
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT DISTINCT `YEAR` FROM TBL_ANNUAL_CONFIRMATION
			WHERE CRSO_RELATION_ID = ?
				AND PRODUCT_ID = ? ";
		if($status)
			$sql .= " AND `STATUS` = ".$status." ";
		$sql .= " ORDER BY `YEAR`";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$product_id);
		$q->execute();
		if($return == 'q')
			return $q;
		$rv = array();
		while($r = $q->fetch())
			$rv[] = $r->YEAR;
		return $rv;
	}

	public function hasNoUnfiledMonths($participant_id=null,$crso_id=null,$year=null,$trust_id=null,$product_id=null)
	{
		$participant_id || $participant_id = $this->participant->relation->id;
		$crso_id || $crso_id = $this->crso->relation->id;
		$year || $year = $this->year;
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT MAX(D.DECLARATION_ID) as DECL_ID, D.`MONTH`
			FROM TBL_DECLARATION as D
			INNER JOIN TBL_DECLARATION_LINE as DL
				ON D.DECLARATION_ID = DL.DECLARATION_ID
			WHERE D.CRSO_RELATION_ID = ?
				AND D.PARTICIPANT_RELATION_ID = ?
				AND D.`YEAR` = ?
				AND D.`PRODUCT_ID` = ? ";
		if(is_null($trust_id))
			$sql .= " AND ISNULL(DL.TRUST_ID) ";
		else
			$sql .= " AND DL.TRUST_ID = ".$trust_id." ";
		$sql .= " GROUP BY D.`MONTH`";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$participant_id,$year, $product_id);
		$q->execute();
		if(!($r = $q->fetch())) // No Declarations where triggered. All Good.
			return array(true,array());

		require_once(ELC_CLASSES.'Declaration.php');

		$rvStatus = true;
		$declList = array();

		do
		{
			// get the status of the last declaration of the month.
			// If it hasnt been exported it is incorrect anyways.
			$d = new Declaration($this->db);
			$d->set($r->DECL_ID);
			$declList[$r->MONTH] = true;
			/*if(!$d->is_exported)
			{
				$declList[$r->MONTH] = $rvStatus = false;
				debug($r->MONTH.' is NOT exported ['.$d->id.']');
			}*/
			if($d->aggregated_status & DECLARATION_FORFEIT)
			{
				$declList[$r->MONTH] = $rvStatus = false;
				debug($r->MONTH.' is FORFEIT ['.$d->id.']');
			}
			if($d->aggregated_status == DECLARATION_OPEN)
			{
				$declList[$r->MONTH] = $rvStatus = false;
				debug($r->MONTH.' is OPEN ['.$d->id.']');
			}
		} while($r = $q->fetch());
		return array($rvStatus,$declList);
	}

	public function getDeclarationOverview($participant_id=null,$crso_id=null,$year=null,$trust_id=null,$product_id=null)
	{
		$participant_id || $participant_id = $this->participant->relation->id;
		$crso_id || $crso_id = $this->crso->relation->id;
		$year || $year = $this->year;
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT D.`MONTH`, SUM(DL.QUANTITY) as QUANTITY, SUM(DL.EXPORTED_AMOUNT) as AMOUNT, BIT_OR(DL.`STATUS`) as `STATUS`,
				SUM(DL.`WEIGHT`) as WEIGHT, SUM(DL.`WEIGHT_DEC`) as WEIGHT_DEC, DL.`BRAND_ID`,DL.`PRODUCT_CATEGORY_ID`,DL.`TRUST_ID`, '' as FILED,
				MIN(IFNULL(D.IS_EXPORTED,0)) as EXPORTED
			FROM TBL_DECLARATION as D
			INNER JOIN TBL_DECLARATION_LINE as DL
				ON D.DECLARATION_ID = DL.DECLARATION_ID
			WHERE D.CRSO_RELATION_ID = ?
				AND D.PARTICIPANT_RELATION_ID = ?
				AND D.`YEAR` = ?
				AND D.`PRODUCT_ID` = ? ";
		if(is_null($trust_id))
			$sql .= " AND ISNULL(DL.TRUST_ID) ";
		else
			$sql .= " AND TRUST_ID = ".$trust_id." ";
		$sql .= " GROUP BY D.`MONTH`, DL.`BRAND_ID`, DL.`PRODUCT_CATEGORY_ID`, DL.`TRUST_ID`";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$participant_id,$year,$product_id);
		$q->execute();
		return $q;
	}

	public function getRefundOverview($participant_id=null,$crso_id=null,$year=null,$product_id=null)
	{
		$participant_id || $participant_id = $this->participant->relation->id;
		$crso_id || $crso_id = $this->crso->relation->id;
		$year || $year = $this->year;
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT (MONTH(RE.EXPORT_DATE)-1) as `MONTH`, SUM(RRL.QUANTITY) as QUANTITY, SUM(RRL.EXPORTED_AMOUNT) as AMOUNT,
				SUM(RRL.`WEIGHT`) as WEIGHT, SUM(RRL.`WEIGHT_DEC`) as WEIGHT_DEC, RRL.`BRAND_ID`,RRL.`PRODUCT_CATEGORY_ID`
			FROM TBL_REFUND_EXPORT as RE
			INNER JOIN TBL_REFUND_REQUEST as RR
				ON RE.REFUND_EXPORT_ID = RR.REFUND_EXPORT_ID
			INNER JOIN TBL_REFUND_REQUEST_LINE as RRL
				ON RR.REFUND_REQUEST_ID = RRL.REFUND_REQUEST_ID
			WHERE RE.CRSO_RELATION_ID = ?
				AND RE.PARTICIPANT_RELATION_ID = ?
				AND ((MONTH(RE.EXPORT_DATE)=? AND YEAR(RE.EXPORT_DATE) = ?)
					OR (MONTH(RE.EXPORT_DATE)>? AND YEAR(RE.EXPORT_DATE) = ?))
				AND RE.`PRODUCT_ID` = ? ";
		$sql .= " GROUP BY RE.REFUND_EXPORT_ID, RRL.`BRAND_ID`, RRL.`PRODUCT_CATEGORY_ID`";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$participant_id,1,($year+1),1,$year,$product_id);
		$q->execute();
		return $q;
	}

	function getOverview($participant_id=null,$crso_id=null,$product_id=null)
	{
		$participant_id || $participant_id = $this->participant->relation->id;
		$crso_id || $crso_id = $this->crso->relation->id;
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT AC.ANNUAL_CONFIRMATION_ID, `YEAR`, `STATUS`,
				AC.CREATED, LAST_UPDATED, IFNULL(MAX(DOCUMENT_ID),0) as DOC_ID, '' as ALL_FILED
			FROM TBL_ANNUAL_CONFIRMATION as AC
			LEFT JOIN TBL_ANNUAL_CONFIRMATION_DOCUMENT as ACD
				ON AC.ANNUAL_CONFIRMATION_ID = ACD.ANNUAL_CONFIRMATION_ID
			WHERE CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ?
				AND PRODUCT_ID = ?
			GROUP BY AC.ANNUAL_CONFIRMATION_ID
			ORDER BY AC.`YEAR` DESC";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$participant_id,$product_id);
		$q->execute();
		return $q;
	}

	function getCRSOOverview($crso_id=null,$product_id=null,$year=false,$status=false,$review_status=false)
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT AC.ANNUAL_CONFIRMATION_ID as CHECKBOX, R.`NAME` as PARTICIPANT_NAME, AC.`STATUS`,
				IFNULL(MAX(ACD.DOCUMENT_ID),0) as AC_DOCUMENT,
				AC.LAST_UPDATED, AC.CHANGE_REQUEST_STATUS as REQUESTED_CHANGE,
				#AC.REVIEW_STATUS,
				CONCAT(
				IFNULL(
					(SELECT SUM(DL.QUANTITY)
					FROM TBL_DECLARATION as D
					INNER JOIN TBL_DECLARATION_LINE as DL
						ON D.DECLARATION_ID = DL.DECLARATION_ID
					WHERE D.PARTICIPANT_RELATION_ID = AC.PARTICIPANT_RELATION_ID
						AND D.CRSO_RELATION_ID = AC.CRSO_RELATION_ID
						AND D.PRODUCT_ID = AC.PRODUCT_ID
						AND D.`YEAR` = AC.`YEAR`)
				, ''),
					IFNULL(
				(SELECT SUM(RRL.QUANTITY)
					FROM TBL_REFUND_EXPORT as RE
					INNER JOIN TBL_REFUND_REQUEST as RR
						ON RE.REFUND_EXPORT_ID = RR.REFUND_EXPORT_ID
					INNER JOIN TBL_REFUND_REQUEST_LINE as RRL
						ON RR.REFUND_REQUEST_ID = RRL.REFUND_REQUEST_ID
					WHERE RE.PARTICIPANT_RELATION_ID = AC.PARTICIPANT_RELATION_ID
						AND RE.CRSO_RELATION_ID = AC.CRSO_RELATION_ID
						AND RE.PRODUCT_ID = AC.PRODUCT_ID
						AND ((MONTH(RE.EXPORT_DATE)=1 AND YEAR(RE.EXPORT_DATE) = (AC.YEAR+1))
							OR (MONTH(RE.EXPORT_DATE)>1 AND YEAR(RE.EXPORT_DATE) = AC.YEAR)))
				, '')) as HASH,
				AC.PARTICIPANT_RELATION_ID,
				AC.CRSO_RELATION_ID
			FROM TBL_ANNUAL_CONFIRMATION as AC
			INNER JOIN TBL_RELATION as R
				ON AC.PARTICIPANT_RELATION_ID = R.RELATION_ID
			LEFT JOIN TBL_ANNUAL_CONFIRMATION_DOCUMENT as ACD
				ON AC.ANNUAL_CONFIRMATION_ID = ACD.ANNUAL_CONFIRMATION_ID
			WHERE AC.CRSO_RELATION_ID = ?
				AND AC.PRODUCT_ID = ? ";
		if(is_numeric($year))
			$sql .= " AND AC.`YEAR` = ".$year." ";
		if(is_numeric($status))
			$sql .= " AND AC.`STATUS` = ".$status." ";
		if(is_numeric($review_status))
			$sql .= " AND AC.`REVIEW_STATUS` = ".$review_status." ";
		$sql .= " GROUP BY AC.ANNUAL_CONFIRMATION_ID
			 ORDER BY AC.REVIEW_STATUS, PARTICIPANT_NAME";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$product_id);
		$q->execute();
		return $q;
	}

	function getLastDocumentForConfirmation($id=null)
	{
		$id || $id = $this->id;

		$sql = "SELECT DOCUMENT_ID
			FROM TBL_ANNUAL_CONFIRMATION_DOCUMENT
			WHERE ANNUAL_CONFIRMATION_ID = ?
			ORDER BY CREATED DESC, DOCUMENT_ID DESC
			LIMIT 1";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		require_once(ELC_CLASSES.'Document.php');
		$document = new ELCDocument($this->db,$r->DOCUMENT_ID);
		return $document;
	}

	public function triggerNewRefundAnnualConfirmations($crso_id=null,$log,$product_id=null)
	{
		require_once(ELC_CLASSES.'CRSO.php');
		require_once(ELC_CLASSES.'Participant.php');
		require_once(ELC_CLASSES.'Product.php');
		require_once(ELC_CLASSES.'CRSOProduct.php');

		$crso_id || $crso_id = $this->crso->relation->id;
		$product_id || $product_id = $this->product->id;

		$d = currentDate();
		$year = $d->getYear()-1; // Last year! Durr
		$crso = new CRSO($this->db,array('relation'=>new Relation($this->db,$crso_id)));
		$cp = new CRSOProduct($this->db,array('crso'=>$crso));
		$cp->set(null,$product_id);

		$log->info(sprintf('Triggering Annual Refund Confirmations for CRSO [%s] and Product [%s]',$crso->relation->id,$product_id));

		$onlyExporters = false;
		if($cp->refund_export_day <= $cp->forfeit_day)
		{
			$log->info(' - Triggering only for EXPORTERS, Forfeit Day is at a later time.');
			$onlyExporters = true;
		}

		$recipients = array();
		// Only trigger for Exporters that have exports
		$sql = "SELECT DISTINCT RE.PARTICIPANT_RELATION_ID
			FROM TBL_REFUND_EXPORT as RE
			INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT as CPP
				ON RE.CRSO_RELATION_ID = CPP.CRSO_RELATION_ID
					AND RE.PARTICIPANT_RELATION_ID = CPP.PARTICIPANT_RELATION_ID
					AND RE.PRODUCT_ID = CPP.PRODUCT_ID
			WHERE CPP.ACTIVE = ?
				AND RE.CRSO_RELATION_ID = ?
				AND RE.PRODUCT_ID = ?
				AND ((YEAR(RE.EXPORT_DATE) = ? AND MONTH(RE.EXPORT_DATE) > ?) OR
					(YEAR(RE.EXPORT_DATE) = ? AND MONTH(RE.EXPORT_DATE) = ?)) ";
		if($onlyExporters) // No additional 'else' check necessary as Refund is ALWAYS triggered after Declaration.
			$sql .= " AND CPP.PRODUCER_EXPORTER = ".PARTICIPANT_EXPORTER." ";
		$sql .= "ORDER BY RE.PARTICIPANT_RELATION_ID";
		$q = $this->db->query($sql);
		$q->bind_values(PARTICIPANT_STATUS_ACTIVE, $crso_id,$product_id, $year, 1, ($year+1), 1);
		$q->execute();

		$recipients = array();
		$props = array('year'=>$year,'crso'=>$crso,'status'=>ANNUAL_CONFIRMATION_OPEN,'product'=>new Product($this->db,$product_id));
		$recipients = $this->triggerACandRecipients($q, $log, $recipients, $props);

		// if not OnlyExporters, then trigger also for Exporter+Producer where No Export for Refund have been done
		if($onlyExporters)
			return $this->sendACNotification($recipients,$props);

		$sql = "SELECT PARTICIPANT_RELATION_ID
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CPP
			WHERE CPP.ACTIVE = ?
				AND CRSO_RELATION_ID = ?
				AND PRODUCT_ID = ?
				AND PRODUCER_EXPORTER = ? ";
		$q = $this->db->query($sql);
		$q->bind_values(PARTICIPANT_STATUS_ACTIVE, $crso_id,$product_id,PARTICIPANT_PRODUCER+PARTICIPANT_EXPORTER);
		$q->execute();
		$recipients = $this->triggerACandRecipients($q, $log, $recipients, $props);
		$this->sendACNotification($recipients,$props);
	}

	public function triggerNewAnnualConfirmations($crso_id=null,$log,$product_id=null,$year=null,$participant_id=false)
	{
		require_once(ELC_CLASSES.'CRSO.php');
		require_once(ELC_CLASSES.'Participant.php');
		require_once(ELC_CLASSES.'Product.php');
		require_once(ELC_CLASSES.'CRSOProduct.php');

		$crso_id || $crso_id = $this->crso->relation->id;
		$product_id || $product_id = $this->product->id;

		$d = currentDate();
		$correctYear = $d->getYear()-1;
		$year = $year?$year:$correctYear; // Last year! Durr
		$crso = new CRSO($this->db,array('relation'=>new Relation($this->db,$crso_id)));
		$cp = new CRSOProduct($this->db,array('crso'=>$crso));
		$cp->set(null,$product_id);

		$log->info(sprintf('Triggering Annual Confirmations for CRSO [%s] and Product [%s]',$crso->relation->id,$product_id));

		$onlyProducers = false;
		if(($cp->forfeit_day < $cp->refund_export_day)&&($correctYear==$year)&&($d->getMonth()==1))
		{
			$log->info(' - Triggering only for PRODUCERS, Refund Export Day is at a later time.');
			$onlyProducers = true;
		}

		// Only for people that actually have had declarations triggered for them
		$sql = "SELECT DISTINCT D.PARTICIPANT_RELATION_ID
			FROM TBL_DECLARATION as D
			INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT as CPP
				ON D.PARTICIPANT_RELATION_ID = CPP.PARTICIPANT_RELATION_ID
					AND D.CRSO_RELATION_ID = CPP.CRSO_RELATION_ID
					AND D.PRODUCT_ID = CPP.PRODUCT_ID
			WHERE CPP.ACTIVE = ?
				AND D.CRSO_RELATION_ID = ?
				AND D.`YEAR` = ?
				AND D.PRODUCT_ID = ? ";
		if(is_numeric($participant_id))
			$sql .= " AND D.PARTICIPANT_RELATION_ID = ".$participant_id." ";
		if($onlyProducers) {
			$sql .= " AND CPP.PRODUCER_EXPORTER = ".PARTICIPANT_PRODUCER." ";
		} else {
			/*
			 * We have to make sure that the people that have Declarations over the previous year, are still a producer currently.
			 * It can happen that a participant changed from Producer & Refund Requester to a Refund Requester only.
			 * This will and has caused problems when the Annual Confirmation is later triggered during the
			 * refund export.
			 */
			$sql .= " AND CPP.PRODUCER_EXPORTER & ".PARTICIPANT_PRODUCER." ";
		}
		$sql .= " ORDER BY D.PARTICIPANT_RELATION_ID";
		$q = $this->db->query($sql);
		$q->bind_values(PARTICIPANT_STATUS_ACTIVE, $crso_id, $year, $product_id);
		$q->execute();

		$recipients = array();
		$props = array('year'=>$year,'crso'=>$crso,'status'=>ANNUAL_CONFIRMATION_OPEN,'product'=>new Product($this->db,$product_id));
		$recipients = $this->triggerACandRecipients($q, $log, $recipients, $props);

		$sql = "SELECT PARTICIPANT_RELATION_ID
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CPP
			WHERE CPP.ACTIVE = ?
				AND NO_DECLARATION = ?
				AND CRSO_RELATION_ID = ?
				AND PRODUCT_ID = ? ";
		if(is_numeric($participant_id))
			$sql .= " AND PARTICIPANT_RELATION_ID = ".$participant_id." ";
		if($onlyProducers)
			$sql .= " AND CPP.PRODUCER_EXPORTER = ".PARTICIPANT_PRODUCER." ";
		else
			$sql .= " AND CPP.PRODUCER_EXPORTER & ".PARTICIPANT_PRODUCER." ";
		$sql .= " ORDER BY PARTICIPANT_RELATION_ID";
		$q = $this->db->query($sql);
		$q->bind_values(PARTICIPANT_STATUS_ACTIVE, 1, $crso_id, $product_id);
		$q->execute();

		$recipients = $this->triggerACandRecipients($q, $log, $recipients, $props);
		$this->sendACNotification($recipients,$props);
	}

	protected function triggerACandRecipients($q, $log, $recipients, $props)
	{
		while($r = $q->fetch())
		{
			if(isset($recipients[$r->PARTICIPANT_RELATION_ID]))
				continue;
			$ac = new AnnualConfirmation($this->db,$props);
			$ac->participant = new Participant($this->db,array('relation'=>new Relation($this->db,$r->PARTICIPANT_RELATION_ID)));
			if($this->setByParticipantCRSOAndYear($r->PARTICIPANT_RELATION_ID, $props['crso']->relation->id, $props['year'], $props['product']->id)) {
				$log->info(sprinf(' - Annual Confirmation for Participant [%s] at CRSO [%s] for Product [%s] over Year [%s] already triggered.',
					$r->PARTICIPANT_RELATION_ID, $props['crso']->relation->id, $props['product']->id, $props['year'] ));
				continue;
			}
			$ac->insert();
			$log->info(sprintf(' - Triggered Annual Confirmation for Participant [%s]',$ac->participant->relation->id));
			// Send a notification to this participant.
			$recipients[$ac->participant->relation->id] = array('recipient'=>$ac->participant->relation,
				'callback'=>sprintf("go('/".SITE_PATH."scripts/elc/participant/InterfaceAnnualConfirmation.php?crso[id]=%s&year=%s')",$props['crso']->relation->id,$props['year']));
		}
		return $recipients;
	}

	protected function sendACNotification($recipients,$props)
	{
		if(count($recipients) == 0 )
			return;
		// only send notifications if crso doesnt have AC disabled by country settings
		require_once(ELC_CLASSES.'Country.php');
		$country = new Country($this->db,$props['crso']->getCountry(),true);
		if($country->has(HasAnnualConfirmationDisabled, $props['product']->id))
			return;

		require_once(ELC_CLASSES.'Notification.php');
		$langIf = IWRegistry::registry('langIf');

		$notificationSubject = $langIf->getText('Annual Confirmation Available');
		$notificationContentDesc = 'The Annual Confirmation for `{CRSO}` is available for approval';
		$langIf->getText($notificationContentDesc);

		$props['crso']->relation->set();
		Notification::send($this->db,
			array('subject'=>$notificationSubject,
				'sender'=>$props['crso']->relation,
                                'product'=>$props['product'],
				'message'=>
					$langIf->getText($notificationContentDesc,
						null,
						array ('CRSO'=>$props['crso']->relation->name,'YEAR'=>$props['year']),
						$props['crso']->default_language_id)
			),
			$recipients
		);
	}

	static public function translateStatus($langIf,$status)
	{
		switch($status)
		{
			case ANNUAL_CONFIRMATION_OPEN:
				return array($langIf->getText('Confirmation Open'), 'status_new');
			case ANNUAL_CONFIRMATION_CLOSED:
				return array($langIf->getText('Confirmation Closed'), 'status_verified');
			case ANNUAL_CONFIRMATION_REOPENED:
				return array($langIf->getText('Confirmation ReOpened'), 'status_resubmit');
		}
	}

	static public function translateReviewStatus($langIf,$status)
	{
		switch($status)
		{
			case AC_REVIEW_OPEN:
				return array($langIf->getText('Confirmation Unreviewed'), 'status_new');
			case AC_REVIEW_APPROVED:
				return array($langIf->getText('Confirmation Review Approved'), 'status_verified');
			case AC_REVIEW_REJECTED:
				return array($langIf->getText('Confirmation Review Rejected'), 'status_forfeit');
		}
	}

	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_ANNUAL_CONFIRMATION';
		$this->objectmap['primary']['id'] = 'ANNUAL_CONFIRMATION_ID';
		$this->objectmap['properties']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
		$this->objectmap['properties']['product->id'] = 'PRODUCT_ID';
		$this->objectmap['properties']['year'] = 'YEAR';
		$this->objectmap['properties']['status'] = 'STATUS';
		$this->objectmap['properties']['review_status'] = 'REVIEW_STATUS';
		$this->objectmap['properties']['created'] = 'CREATED';
		$this->objectmap['properties']['last_updated'] = 'LAST_UPDATED';
		$this->objectmap['properties']['change_request_status'] = 'CHANGE_REQUEST_STATUS';
	}

	function reopenAllAnnualConfirmationsWithoutDocument($crso_id, $year) {

		$sql = "SELECT AC.ANNUAL_CONFIRMATION_ID
			FROM TBL_ANNUAL_CONFIRMATION as AC
			LEFT JOIN TBL_ANNUAL_CONFIRMATION_DOCUMENT as ACD
				ON AC.ANNUAL_CONFIRMATION_ID = ACD.ANNUAL_CONFIRMATION_ID
			WHERE ISNULL(ACD.DOCUMENT_ID)
				AND AC.CRSO_RELATION_ID = ?
				AND AC.`YEAR` = ?";

		$q = $this->db->query($sql);
		$q->bind_values($crso_id, $year);
		$q->execute();

		while($r = $q->fetch()) {
			$ac = new AnnualConfirmation($this->db, $r->ANNUAL_CONFIRMATION_ID, true);
			$ac->change_request_status = ANNUAL_CONFIRMATION_CHANGE_APPROVED;
			$ac->status = ANNUAL_CONFIRMATION_REOPENED;
			$ac->update();
		}
	}

	function forceCloseAC($crso_id, $ac_id) {
		$sql = "SELECT AC.ANNUAL_CONFIRMATION_ID
			FROM TBL_ANNUAL_CONFIRMATION as AC
			WHERE AC.CRSO_RELATION_ID = ?
				AND AC.ANNUAL_CONFIRMATION_ID = ?
		";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, $ac_id);
		$q->execute();

		if(!($r = $q->fetch()))
			return false;

		$this->set($ac_id);
		$this->status = ANNUAL_CONFIRMATION_CLOSED;
		$this->change_request_status = 0;
		$this->update();

		info(sprintf(' - Force closed of Annual Confirmation %s by CRSO', $ac_id));
		return true;
	}

}
/**
 * @package WEEE
 * @subpackage Declaration
 */
class AnnualConfirmationDocument extends AbstractObject
{
	public $annual;
	public $document;
	public $created;

	function insert($graceful=false)
	{
		$date = currentDate();
		$this->created = $date->format('Y-m-d H:i');

		return parent::insert($graceful);
	}

	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_ANNUAL_CONFIRMATION_DOCUMENT';
		$this->objectmap['primary']['annual->id'] = 'ANNUAL_CONFIRMATION_ID';
		$this->objectmap['primary']['document->id'] = 'DOCUMENT_ID';
		$this->objectmap['properties']['created'] = 'CREATED';
	}
}

/**#@+
 *
 */
define('ANNUAL_CONFIRMATION_OPEN',1);
define('ANNUAL_CONFIRMATION_CLOSED',2);
define('ANNUAL_CONFIRMATION_REOPENED',3);

define('AC_REVIEW_OPEN',0);
define('AC_REVIEW_REJECTED',1);
define('AC_REVIEW_APPROVED',2);

define('ANNUAL_CONFIRMATION_CHANGE_REQUESTED',1);
define('ANNUAL_CONFIRMATION_CHANGE_APPROVED',2);
/**#@-*/
?>
