<?php
require_once(ELC_CLASSES.'CRSOProductDimension.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'ProductCategory.php');
require_once(ELC_CLASSES.'Brand.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'ProvisionalDeclaration.php');

/**
 * File description
 *
 * @package PACKAGE
 * @subpackage SUBPACKAGE
 * @since Nov 14, 2006
 * @author Martijn van den Broek <martijn@xs4all.nl>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */

class AnnualNewProvisional
{
	private $db;
	private $log;
	private $year;

	/**
	 * Constructor
	 */
	function AnnualNewProvisional($db, $log)
	{
		$this->db = $db;
		$this->log = $log;

		# date stuff
		$d = currentDate();
		$this->year = $d->getYear();
	}

	/**
	 * Starts making new provisionals for the given crso_id
	 */
	public function triggerNewYear($crso_id,$product_id)
	{
		$this->log->info(sprintf('AnnualNewProvisional::triggerNewYear() - Triggering newyear for CRSO %s', $crso_id));

		$sql = "SELECT PARTICIPANT_RELATION_ID
			FROM TBL_CRSO_PARTICIPANT_PRODUCT
			WHERE CRSO_RELATION_ID=?
				AND ACTIVE=?
				AND NO_DECLARATION=?
				AND PRODUCT_ID = ?
				AND PRODUCER_EXPORTER & ?
		";

		$q = $this->db->query($sql);
		$q->bind_values($crso_id, PARTICIPANT_STATUS_ACTIVE, 0, $product_id, PARTICIPANT_PRODUCER);
		$q->execute();

		$recipients = array();

		while ($r = $q->fetch())
		{
			$participant_id = $r->{'PARTICIPANT_RELATION_ID'};
			$this->processParticipant($crso_id, $participant_id,$product_id);

			$recipients[$participant_id] = array('recipient'=>new Relation($this->db, $participant_id),
				'callback'=>sprintf("go('/".SITE_PATH."scripts/elc/participant/InterfaceProvisionalDeclaration.php?crso[id]=%s&prov[year]=%s')",$crso_id, $this->year));
		}

		if (sizeof($recipients) == 0)
			return;

		require_once(ELC_CLASSES.'Notification.php');
		$langIf = IWRegistry::registry('langIf'); // FIXME

		$notificationSubject = $langIf->getText('Annual New Provisional Available');
		$notificationContentDesc = 'The Annual New Provisional for `{CRSO}` is available';
		$langIf->getText($notificationContentDesc);

		$crso = new CRSO($this->db, array('relation' => new Relation($this->db, $crso_id)));
		$crso->set();
		$crso->relation->set();

		Notification::send($this->db,
			array('subject'=>$notificationSubject,
				'sender'=>$crso->relation,
				'message'=>
					$langIf->getText($notificationContentDesc,
						null,
						array ('CRSO'=>$crso->relation->name,'YEAR'=>$this->year),
						$crso->default_language_id)
			),
			$recipients
		);

	}

	function processParticipant($crso_id, $participant_id,$product_id)
	{
		// first we make sure that no provisional is already made, and
		// if so, we break out of here
		$sql = "
			SELECT COUNT(*) AS CNT FROM TBL_PROVISIONAL_DECLARATION WHERE
			CRSO_RELATION_ID = ? AND PRODUCT_ID = ? AND PARTICIPANT_RELATION_ID = ?
			AND `YEAR` = ?
		";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, $product_id, $participant_id, $this->year);
		$q->execute();
		$r = $q->fetch();
		if((int)$r->CNT > 0)
		{
			$this->log->info(sprintf('AnnualNewProvisional - Provisional already exists part[%s], crso[%s], prod[%s] year[%s]',
				$participant_id,$crso_id, $product_id, $this->year));
			return;
		}


		// If there is no provisional for the year we reset the provisional change request status.
		$sql = "UPDATE TBL_CRSO_PARTICIPANT_PRODUCT SET PROVISIONAL_CHANGE_REQUEST_STATUS = ?
			WHERE PARTICIPANT_RELATION_ID = ? AND CRSO_RELATION_ID = ? AND PRODUCT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values(PROVISIONAL_CHANGE_REQUEST_OPEN, $participant_id, $crso_id, $product_id);
		$q->execute();

		$this->log->info(sprintf(' - Processing crso/participant %s/%s', $crso_id, $participant_id));
		$year = $this->year;

		# new provisional lines which will be added later
		$lines = array();

		# array that will contain open and closed months
		$open_months = array();
		$closed_months = 0;

		# lets go trough the declarations and fill the new provisional
		for ($month=1; $month<=12; $month++)
		{
			$this->log->info(sprintf('   - Working on month %s', $month));

			# TODO: change sql for multiple closed declarations
			$sql = "
				SELECT d.DECLARATION_ID as declaration_id, BIT_OR(dl.`STATUS`) as `status`
				FROM TBL_DECLARATION d
				INNER JOIN TBL_DECLARATION_LINE dl
					ON d.DECLARATION_ID=dl.DECLARATION_ID
				WHERE
					d.`YEAR`=?
					AND d.`MONTH`=?
					AND d.CRSO_RELATION_ID=?
					AND d.PARTICIPANT_RELATION_ID=?
					AND d.PRODUCT_ID = ?
				GROUP BY d.DECLARATION_ID
				HAVING `status` & (".implode(array(DECLARATION_NORMAL, DECLARATION_FORFEIT, DECLARATION_LATE, DECLARATION_RESUBMIT), " | ").")
				ORDER BY DECLARATION_ID DESC
			";

			$q = $this->db->query($sql);
			$q->bind_values($year-1, $month, $crso_id, $participant_id,$product_id);
			$q->execute();

			if (!($r = $q->fetch()))
			{
				$this->log->info('	 - No closed declaration found, we have to make up something for this month.');
				$open_months[] = $month;
				continue;
			}
			# take only the first, this is the last closed
			$declaration_id = $r->{'declaration_id'};
			$status = $r->{'status'};

			# get declaration details, process and add to new provisional
			$this->log->info(sprintf(sprintf('	 - A closed declaration for last year found, declaration id [%s] and has status [%s]', $declaration_id, $status)));

			$sql = "SELECT BRAND_ID, PRODUCT_CATEGORY_ID, CPD_ID, SUM(WEIGHT) as WEIGHT, SUM(WEIGHT_DEC) as WEIGHT_DEC, SUM(QUANTITY) as QUANTITY, TRUST_ID
				FROM TBL_DECLARATION d
				INNER JOIN TBL_DECLARATION_LINE dl
					ON d.DECLARATION_ID = dl.DECLARATION_ID
				WHERE d.`YEAR`=? AND d.`MONTH`=? AND d.CRSO_RELATION_ID=? AND d.PARTICIPANT_RELATION_ID=?
					AND d.`PRODUCT_ID` = ?
				GROUP BY dl.BRAND_ID, dl.PRODUCT_CATEGORY_ID, dl.CPD_ID, dl.TRUST_ID
			";
			$q = $this->db->query($sql);
			$q->bind_values($year-1, $month, $crso_id, $participant_id,$product_id);
			$q->execute();

			# we need to filter lines if not all are zero
			$all_zero = true;

			# Build an array of the lines of the last year declarations
			while ($r = $q->fetch())
			{
				// Validate Trust, in case the trust is terminated, skip this line.
				// See Trac Ticket #786
				if(!$this->validateTrust($r->TRUST_ID))
					continue;

				# TODO: better to use country settings maybe

				// check if the ProductCategory is active, else skip this line
				if (!empty($r->{'PRODUCT_CATEGORY_ID'})) {
					$pc = new ProductCategory($this->db);
					$pc->set($r->{'PRODUCT_CATEGORY_ID'});
					if (!$pc->isEnabled()) {
						$this->log->info(sprintf(' - skipping product_category [%s] because not enabled', $r->{'PRODUCT_CATEGORY_ID'}));
						continue;
					}
				}

				// check if the Dimensions is active, else skip this line
				if (!empty($r->{'CPD_ID'})) {
					$cpd = new CRSOProductDimension($this->db);
					$cpd->set($r->{'CPD_ID'});
					if (!$cpd->isEnabled()) {
						$this->log->info(sprintf(' - skipping dimension [%s] because not enabled', $r->{'CPD_ID'}));
						continue;
					}
				}

				$brand_id	= (empty($r->{'BRAND_ID'})) ? null : $r->{'BRAND_ID'};
				$product_category_id = (empty($r->{'PRODUCT_CATEGORY_ID'})) ? null : $r->{'PRODUCT_CATEGORY_ID'};
				$trust_id = (empty($r->{'TRUST_ID'}) ? null : $r->{'TRUST_ID'});
				$dimension = (empty($r->{'CPD_ID'}) ? null : $r->{'CPD_ID'});

				$weight = (empty($r->{'WEIGHT'}) ? null : (int) $r->{'WEIGHT'});
				$weight_dec = (empty($r->{'WEIGHT_DEC'}) ? null : $r->{'WEIGHT_DEC'});
				$quantity = (int) $r->{'QUANTITY'};

				if (empty($quantity))
				{

				}

				if ($quantity > 0)
					$all_zero = false;

				$lines[$month][$brand_id][$product_category_id][$trust_id][$dimension] = array('quantity'=>$quantity,'weight'=>$weight, 'weight_dec'=>$weight_dec);
			}

			# Filter zero lines if nessecary
			# TODO: what do we do with trust here?
			if (!$all_zero)
			{
				$this->log->info('	 - Not all lines are zero, filtering zero lines');
				foreach ($lines[$month] as $brand_id => $arr) {
					foreach ($arr as $product_category_id => $__arr) {
						foreach ($__arr as $trust_id => $___arr) {
							foreach($___arr as $dimension => $values) {
								if($values['quantity'] == 0)
								{
									$this->log->info(sprintf('	   - Removing line [%s][%s][%s][%s][%s]', $month, $brand_id, $product_category_id, $trust_id, $dimension));
									unset($lines[$month][$brand_id][$product_category_id][$trust_id][$dimension]);
								}
							}
						}
					}
				}
			}

			$closed_months++;
		}

		# Now build the prov_lines from the lines array
		list($totals, $prov_lines) = $this->buildProvLines($lines);

		# Get the average and add to lines...
		$this->log->info(' - Calculating values for empty months');

		$prov_lines = $this->fixOpenMonths($totals,$prov_lines,$open_months,$closed_months);

		# Now save for each trust (and null for itself) the new provisional
		$this->addNewProvisional($prov_lines,$crso_id,$participant_id,$year,$product_id);
	}

	private function addNewProvisional($prov_lines,$crso_id,$participant_id,$year,$product_id)
	{
		foreach ($prov_lines as $trust_id => $prov_lines)
		{
			$this->log->info(sprintf(' - Inserting new provisional for trust [%s]', $trust_id));
			$provisional = new ProvisionalDeclaration($this->db);
			$provisional->set($crso_id, $participant_id, $year, $trust_id,$product_id);
			$provisional->addLines($prov_lines);
			$provisional->insert();
		}
	}

	private function fixOpenMonths($totals,$prov_lines,$open_months,$closed_months)
	{
		foreach ($open_months as $month) {
			$this->log->info('   - Working on month '.$month);
			foreach ($totals as $brand_id => $arr) {
				foreach ($arr as $product_category_id => $__arr) {
					foreach ($__arr as $trust_id => $___arr) {
						foreach($___arr as $dimension => $values) {
							$quantity =  round($values['quantity'] / $closed_months);
							$weight = null;
							$weight_dec = null;

							if (!is_null($values['weight']))
								$weight =  round($values['weight'] / $closed_months);
							if (!is_null($values['weight_dec']))
								$weight_dec =  round($values['weight_dec'] / $closed_months);

							$prov_lines[$trust_id][] = $this->makeLine($month, $brand_id, $product_category_id, $dimension, $quantity, $weight, $weight_dec);
						}
					}
				}
			}
		}
		return $prov_lines;
	}

	private function buildProvLines($lines)
	{
		$prov_lines = $totals = array();
		foreach ($lines as $month => $_lines) {
			foreach ($_lines as $brand_id => $__lines) {
				if (!isset($totals[$brand_id]))
					$totals[$brand_id] = array();

				foreach ($__lines as $product_category_id => $___lines)	{
					foreach ($___lines as $trust_id => $____lines) {
						foreach($____lines as $dimension => $values) {
							# sum up totals
							foreach (array('quantity', 'weight', 'weight_dec') as $key) {
								if (!is_null($values[$key]))
								{
									if (!isset($totals[$brand_id][$product_category_id][$trust_id][$dimension][$key]))
										$totals[$brand_id][$product_category_id][$trust_id][$dimension][$key] = $values[$key];
									else
										$totals[$brand_id][$product_category_id][$trust_id][$dimension][$key] += $values[$key];
								}
								else
									$totals[$brand_id][$product_category_id][$trust_id][$dimension][$key] = null;
							}
							# now the current month
							$prov_lines[$trust_id][] = $this->makeLine($month, $brand_id, $product_category_id, $dimension, $values['quantity'], $values['weight'], $values['weight_dec']);
						}
					}
				}
			}
		}
		return array($totals,$prov_lines);
	}

	private function makeLine($month, $brand_id, $product_category_id, $dimension, $quantity, $weight, $weight_dec)
	{
		$line = array();
		$line['month'] = $month;

		if (!is_null($brand_id))
			$line['brand'] = new Brand($this->db, $brand_id);

		if (!is_null($product_category_id))
			$line['product_category'] = new ProductCategory($this->db, $product_category_id);

		if (!is_null($dimension))
			$line['dimension'] = new CRSOProductDimension($this->db, $dimension);

		if (!is_null($weight))
			$line['weight'] = $weight;

		if (!is_null($weight_dec))
			$line['weight_dec'] = $weight_dec;

		$line['quantity'] = $quantity;

		return $line;

	}
	/**
	 * This is necessary for the situation in which a participant applies in year x and
	 * gets activated between DeclarationExport and 1st of February in year x+1
	 *
	 * See trac #1392
	 *
	 * NOTE: This cannot include Trusts, trusts are only possible for active participants
	 *
	 * @param int $participant_id
	 * @param int $crso_id
	 * @param int $product_id
	 */
	public function processFromProvisional($participant_id,$crso_id,$product_id)
	{
		$this->log->info(sprintf('AnnualNewProvisional::processFromProvisional Started for Participant [%s] at CRSO [%s] for Product [%s]'
			,$participant_id,$crso_id,$product_id
			));
		// Get the provisional of previous year.
		$date = currentDate();
		$newyear = (int)$date->getYear(); // Current year
		$year = $newyear - 1; // Last year

		$open_months = array();
		$closed_months = 0;

		$trust_id = null;

		// The new year has every month
		for($month=1; $month<=12; $month++)
		{
			$q = $this->db->query($this->getProvisionalSQL());
			$q->bind_values($year,$month,$crso_id,$participant_id,$product_id);
			$q->execute();
			$rows = $q->fetchAll();
			if(!count($rows)) // No provisional for this month
			{
				$open_months[] = $month;
				continue;
			}

			$all_zero = true;

			foreach($rows as $r)
			{
				$brand_id	= (empty($r->{'BRAND_ID'})) ? null : $r->{'BRAND_ID'};
				$product_category_id = (empty($r->{'PRODUCT_CATEGORY_ID'})) ? null : $r->{'PRODUCT_CATEGORY_ID'};
				$dimension = (empty($r->{'CPD_ID'}) ? null : $r->{'CPD_ID'});

				$weight = (empty($r->{'WEIGHT'}) ? null : (int) $r->{'WEIGHT'});
				$weight_dec = (empty($r->{'WEIGHT_DEC'}) ? null : $r->{'WEIGHT_DEC'});
				$quantity = (int) $r->{'QUANTITY'};

				if ($quantity > 0)
					$all_zero = false;

				$lines[$month][$brand_id][$product_category_id][$trust_id][$dimension] = array('quantity'=>$quantity,'weight'=>$weight, 'weight_dec'=>$weight_dec);
			}

			$closed_months++;
		}

		list($totals, $prov_lines) = $this->buildProvLines($lines);
		$prov_lines = $this->fixOpenMonths($totals,$prov_lines,$open_months,$closed_months);
		$this->addNewProvisional($prov_lines,$crso_id,$participant_id,$newyear,$product_id);
	}

	private function getProvisionalSQL()
	{
		$sql = "SELECT BRAND_ID, PRODUCT_CATEGORY_ID, CPD_ID,
				SUM(WEIGHT) as WEIGHT, SUM(WEIGHT_DEC) as WEIGHT_DEC, SUM(QUANTITY) as QUANTITY
			FROM TBL_PROVISIONAL_DECLARATION as pd
			WHERE pd.`YEAR`=?
				AND pd.`MONTH`=?
				AND pd.CRSO_RELATION_ID=?
				AND pd.PARTICIPANT_RELATION_ID=?
				AND pd.`PRODUCT_ID` = ?
			GROUP BY pd.BRAND_ID, pd.PRODUCT_CATEGORY_ID, pd.CPD_ID";
		return $sql;
	}

	private function validateTrust($trust_id=null)
	{
		if(!$trust_id)
			return true;

		require_once(ELC_CLASSES.'Trust.php');
		$trust = new Trust($this->db,$trust_id,true);
		if($trust->approved==TRUST_APPROVED) // Only completely approved trusts are valid
			return true;
		return false;
	}
}














?>
