<?php
require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES.'Product.php');
/**
 * @author Marten Koedam <marten@basesoft.com>
 * @package ELC
 * @version $Id$
 */
class Brand extends AbstractObject
{
	var $crso;
	var $product;
	var $name;
	var $active;
	/**
	 * @param int $id
	 */
	function set($id='')
	{
		$id || $id = $this->id;
		$sql = "SELECT `NAME`, ACTIVE, PRODUCT_ID
			FROM TBL_BRAND
			WHERE BRAND_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		$this->name = $r->NAME;
		$this->active = $r->ACTIVE;
		$this->product = new Product($this->db, $r->PRODUCT_ID);
		$this->complete = true;
		return true;
	}

	function getOverview($crso_id=null, $product_id='')
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$product_id || $product_id = $this->product->id;
		$sql = "SELECT B.BRAND_ID, B.`NAME` as BRAND_NAME,
				B.ACTIVE, COUNT(DISTINCT PB.PARTICIPANT_RELATION_ID) as TIMES_USED
			FROM TBL_BRAND as B
			LEFT JOIN TBL_PARTICIPANT_BRAND as PB
				ON B.BRAND_ID = PB.BRAND_ID
			WHERE B.CRSO_RELATION_ID = ?
			AND B.PRODUCT_ID = ?
			GROUP BY B.BRAND_ID";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, $product_id);
		$q->execute();
		return $q;
	}
	/**
	 *
	 */
	function setByNameByCRSO($name,$crso_id='', $product_id='')
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$product_id || $product_id = $this->product->id;
		$sql = "SELECT BRAND_ID, ACTIVE, PRODUCT_ID
			FROM TBL_BRAND
			WHERE NAME = ?
				AND CRSO_RELATION_ID = ?
				AND PRODUCT_ID = ? ";
		$q = $this->db->query($sql);
		$q->bind_values($name,$crso_id, $product_id);
		if(!$q->execute())
			return false; // Todo: TriggerError
		if(!($r = $q->fetch()))
			return false;

		require_once(ELC_CLASSES.'CRSO.php');
		$this->crso = new CRSO($this->db,array('relation'=>new Relation($this->db,$crso_id)));
		$this->id = $r->BRAND_ID;
		$this->name = $name;
		$this->active = $r->ACTIVE;
		$this->product = new Product($this->db, $r->PRODUCT_ID);
		$this->complete = true;
		return true;
	}
	/**
	 *
	 */
	function getByCRSO($crso_id='', $product_id='')
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$product_id || $product_id = $this->product->id;
		$sql = "SELECT BRAND_ID, NAME, ACTIVE, PRODUCT_ID
			FROM TBL_BRAND
			WHERE CRSO_RELATION_ID = ?
			AND PRODUCT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, $product_id);
		if(!$q->execute())
			return false;
		return $q;
	}

	function select($crso_id=false, $product_id='')
	{
		$product_id || $product_id = $this->product->id;
		$sql = "SELECT BRAND_ID, `NAME` as BRAND_NAME
			FROM TBL_BRAND
			WHERE PRODUCT_ID= ".$product_id;
		if($crso_id)
			$sql .= " AND CRSO_RELATION_ID = ".$crso_id;
		$sql .= " ORDER BY BRAND_NAME";
		$q = $this->db->query($sql);
		$q->execute();
		return $q;
	}
	/**
	 *
	 */
	function getPublic($crso_id='', $type='', $product_id='')
	{
		require_once(ELC_CLASSES . 'Participant.php');

		$crso_id || $crso_id = $this->crso->relation->id;
		$product_id || $product_id = $this->product->id;
		$sql = "SELECT b.BRAND_ID, NAME, b.ACTIVE, b.PRODUCT_ID
			FROM TBL_BRAND b
			WHERE b.CRSO_RELATION_ID=?
			AND b.PRODUCT_ID = ?
			AND b.ACTIVE=1
			GROUP BY b.BRAND_ID";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, $product_id);
		if(!$q->execute())
			return false; // Todo: TriggerError
		return $q;
	}
	/**
	 *
	 */
	function activateByParticipant($participant_id,$crso_id, $product_id)
	{
		$sql = "SELECT BRAND_ID FROM TBL_PARTICIPANT_BRAND
			WHERE PARTICIPANT_RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id);
		$q->execute();
		$brands = array(-1);
		while($r = $q->fetch())
			$brands[] = $r->BRAND_ID;

		$sql = "UPDATE TBL_BRAND SET ACTIVE = ?
			WHERE BRAND_ID IN(".implode(',',$brands).")
				AND CRSO_RELATION_ID = ?
				AND PRODUCT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values(1,$crso_id, $product_id);
		return $q->execute();
	}

	/**
	 *
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_BRAND';
		$this->objectmap['primary']['id'] = 'BRAND_ID';
		$this->objectmap['properties']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['name'] = 'NAME';
		$this->objectmap['properties']['active'] = 'ACTIVE';
		$this->objectmap['properties']['product->id'] = 'PRODUCT_ID';
	}
}

?>
