<?php
require_once(ELC_CLASSES.'Relation.php');
/**
 * Collecting Recycling Service Organization
 * @author Marten Koedam <marten@basesoft.com>
 * @package I-Ware
 * @version $Id$
 */
class CRSO extends Relation
{
	/**
	 *
	 */
	var $relation;
	/**
	 *
	 */
	var $accepted_sla;
	var $default_language_id;
	var $reg_no;
	var $misc_1;
	/**
	 * The clearance levels that are possible for this CRSO. Given by sysadmin based on the contract.
	 * Available clearance levels can be redistributed by the CRSO-Admin user (or the one with the right Clearance)
	 * @var integer
	 */
	var $available_clearance_levels;
	/**
	 *
	 */
	function set($crso_id='')
	{
		$crso_id = $crso_id ? $crso_id : $this->relation->id;
		$sql = "SELECT * FROM TBL_CRSO
			WHERE CRSO_RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id);
		$q->execute();

		if(!($r = $q->fetch()))
			throw new Exception('CRSO not found: '.$crso_id);

		$this->relation = new Relation($this->db,$crso_id);
		$this->accepted_sla = $r->ACCEPTED_SLA;
		$this->default_language_id = $r->DEFAULT_LANGUAGE_ID;
		$this->reg_no = $r->CRSO_REG_NO;
		$this->misc_1 = $r->MISC_1;
		$this->available_clearance_levels = $r->AVAILABLE_CLEARANCE_LEVELS;
		return $this->complete = true;
	}

	function setByVAT($vat_no)
	{
		$sql = "SELECT CRSO_RELATION_ID
			FROM TBL_CRSO c
			INNER JOIN TBL_RELATION r ON c.CRSO_RELATION_ID = r.RELATION_ID
			WHERE r.VAT_NO = ?
		";
		$q = $this->db->query($sql);
		$q->bind_values($vat_no);
		$q->execute();

		if (!($r = $q->fetch()))
			return false;

		return $this->set($r->{'CRSO_RELATION_ID'});
	}

	/**
	 *
	 */
	function selectCRSOs($accepted_sla=1, $country_id='',$product_id = false)
	{
		$values = array();
		$sql = "SELECT C.CRSO_RELATION_ID, R.NAME, C.DEFAULT_LANGUAGE_ID,TBL_COUNTRY.`NAME` as COUNTRY_NAME
			FROM TBL_CRSO as C
			INNER JOIN TBL_CRSO_PRODUCT as CP
				ON C.CRSO_RELATION_ID = CP.CRSO_RELATION_ID
			INNER JOIN TBL_RELATION as R
				ON C.CRSO_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_ADDRESS as A
				USING(ADDRESS_ID)
			INNER JOIN TBL_COUNTRY
				USING(COUNTRY_ID)
			WHERE 1=1 ";

		if(is_numeric($product_id))
		{
			$sql .= " AND CP.PRODUCT_ID = ? ";
			$values[] = $product_id;
		}
		if ($accepted_sla != '') {
			$sql .= " AND ACCEPTED_SLA = ? ";
			$values[] = $accepted_sla;
		}
		if(is_array($country_id))
			$sql .= " AND TBL_COUNTRY.COUNTRY_ID IN(".implode(',',$country_id).") ";
		elseif ($country_id != '')
		{
			$sql .= " AND TBL_COUNTRY.COUNTRY_ID = ? ";
			$values[] = $country_id;
		}
		$sql .= " GROUP BY C.CRSO_RELATION_ID ORDER BY R.NAME";

		$q = $this->db->query($sql);
		$q->bind_values($values);
		$q->execute();
		return $q;
	}

	/**
	 *
	 */
	function selectOverview($country_ids,$product_id)
	{
		$sql = "SELECT '' as CHECKBOX, C.CRSO_RELATION_ID, '' as CRSO_SETTINGS, R.`NAME` as RELATION_NAME, COUNTRY.`NAME` as COUNTRY_NAME,
				R.`EMAIL`, ACCEPTED_SLA, COUNT(DISTINCT U.`USER_ID`) as USERS, '' as WEBSITE_LINK, COUNT(DISTINCT CP.PRODUCT_ID) as PRODUCTS
			FROM TBL_CRSO as C
			INNER JOIN TBL_CRSO_PRODUCT as CP
				ON C.CRSO_RELATION_ID = CP.CRSO_RELATION_ID
			INNER JOIN TBL_RELATION as R
				ON C.CRSO_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_ADDRESS as A
				USING(ADDRESS_ID)
			INNER JOIN TBL_COUNTRY as COUNTRY
				USING(COUNTRY_ID)
			LEFT JOIN TBL_RELATION_USER as RU
				ON R.RELATION_ID = RU.RELATION_ID
			LEFT JOIN TBL_USER as U
				ON RU.USER_ID = U.USER_ID
			GROUP BY R.RELATION_ID";
		$q = $this->db->query($sql);
		$q->execute();
		return $q;
	}
	/**
	 *
	 */
	function selectUserOverview($relation_id,$exclude=array())
	{
		$sql = "SELECT '' as CHECKBOX, U.`USER_ID`,
				U.`USER_NAME`,
				CONCAT( IFNULL(INFO_PERSON_FIRST_NAME,''),' ',
					IFNULL(INFO_PERSON_MIDDLE_NAME,''),' ',
					IFNULL(INFO_PERSON_LAST_NAME,'')) as CONTACT_NAME,
				TR.TRAC_USER_NAME,
				U.`USER_ACTIVE`, RU.IS_ADMIN,
				'' as CLEARANCE_LEVELS,
				COUNT(T.LANGUAGE_ID) as TRANSLATE
			FROM TBL_RELATION as R
			INNER JOIN TBL_RELATION_USER as RU
				USING(RELATION_ID)
			INNER JOIN TBL_USER as U
				USING(USER_ID)
			LEFT JOIN TBL_TRAC_USER TR
				ON TR.TRAC_USER_ID = U.TRAC_USER_ID
			INNER JOIN TBL_INFO_PERSON as IP
				USING(INFO_PERSON_ID)
			LEFT JOIN TBL_TRANSLATOR as T
				ON U.USER_ID = T.USER_ID
			WHERE R.RELATION_ID = ?
			".($exclude?' AND U.USER_ID NOT IN( '.implode(',',$exclude).' ) ':'')."
			GROUP BY U.USER_ID";
		$q = $this->db->query($sql);
		$q->bind_values($relation_id);
		$q->execute();
		return $q;
	}

	function selectCRSOUsersWithClearance($crso_id=null, $clearance) {

		$crso_id || $crso_id = $this->relation->id;
		$sql = "SELECT U.USER_ID, U.USER_NAME
			FROM TBL_USER as U
			INNER JOIN TBL_RELATION_USER as RU
				ON U.USER_ID = RU.USER_ID
			WHERE (RU.CLEARANCE_LEVEL & ? OR RU.IS_ADMIN = 1)
				AND RU.RELATION_ID = ?
				AND RU.STATUS = 1
				AND U.USER_ACTIVE = 1
		";
		$q = $this->db->query($sql);
		$q->bind_values($clearance, $crso_id);
		$q->execute();
		return $q;
	}

	/**
	 * Selects for an option array for a select box
	 * @return string $rv
	 */
	function select($selected=0)
	{
		$rv = '';
		$sql = "SELECT CRSO_RELATION_ID, R.`NAME` as RELATION_NAME, C.`NAME` as COUNTRY_NAME,
				IF(CRSO_RELATION_ID=".$selected.",' selected=\"selected\"','') as SELECTED
			FROM TBL_CRSO as CR
			INNER JOIN TBL_RELATION as R
				ON CR.CRSO_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_ADDRESS as A
				USING(ADDRESS_ID)
			INNER JOIN TBL_COUNTRY as C
				USING(COUNTRY_ID)
			ORDER BY R.`NAME`";
		$q = $this->db->query($sql);
		$q->execute();
		while($r = $q->fetch())
			$rv .= '<option value="'.$r->CRSO_RELATION_ID.'"'.$r->SELECTED.'>'.$r->RELATION_NAME.' ('.$r->COUNTRY_NAME.')</option>'."\n";
		return $rv;
	}

	function insert()
	{
		$this->created = date('Y-m-d H:i:s'); // FIXME

		info('Created new CRSO ['.$this->relation->id.']');
		return parent::insert();
	}
	function update()
	{
		info('Updated CRSO ['.$this->relation->id.']');
		return parent::update();
	}

	/**
	 * Get all the CRSO for the given countries, useful for Operator stuff on country levels
	 * @param mixed $country_ids
	 * @return ResultSet $q
	 */
	function getCRSOsByCountry($country_ids)
	{
		$country_ids = (array) $country_ids;
		$sql = "SELECT R.RELATION_ID, R.`NAME` as CRSO_NAME
			FROM TBL_CRSO as C
			INNER JOIN TBL_RELATION as R
				ON C.CRSO_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_ADDRESS as A
				USING(ADDRESS_ID)
			WHERE A.COUNTRY_ID IN (".implode(',',$country_ids).")
			ORDER BY CRSO_NAME";
		$q = $this->db->query($sql);
		$q->execute();
		return $q;
	}

	function getCountry($id='',$return='COUNTRY_ID')
	{
		if($id)
			return parent::getCountry($id,$return);
		return $this->relation->getCountry($id,$return);
	}

	/**
	 * @param String $sAccount G/L Account No.
	 * @param DelclarationType $oDeclarationType
	 */
	function setGLAccountForDeclarationType($sAccount, $oDeclarationType)
	{
		require_once(ELC_CLASSES.'CRSOGLAccount.php');
		$link = new CRSOGLAccount($this->db);
		$link->crso =& $this;
		$link->declaration_type = $oDeclarationType;
		$link->gl_account = $sAccount;
		$link->update();//is a replace
	}

	/**
	 * @param mixed mDeclarationType (Array of) DelclarationType, default all
	 * @return mixed (Array of) String G/L Account No. if array key is DelcarationType id
	 */
	function getGLAccount($mDeclarationType=null)
	{
		require_once(ELC_CLASSES.'CRSOGLAccount.php');
		if ( $mDeclarationType instanceof DeclarationType )
		{
			$link = new CRSOGLAccount($this->db, array('crso'=>$this, 'declaration_type'=>$mDeclarationType), true);
			$rv = $link->gl_account;
		}
		else
		{
			$link = new CRSOGLAccount($this->db, array('crso'=>$this));
			$rv = $link->getAccounts($mDeclarationType);
		}

		return $rv;
	}

	/**
	 * Selects all CRSOs for the given participant-user with the given participant type.
	 */
	function selectCRSOsByParticipantUser($user_id='', $producer_exporter='')
	{
		$sql = "SELECT DISTINCT c_r.RELATION_ID AS CRSO_ID, c_r.NAME as CRSO_NAME
			FROM TBL_RELATION c_r
			INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT cp
				ON c_r.RELATION_ID = cp.CRSO_RELATION_ID
			INNER JOIN TBL_RELATION p_r
				ON cp.PARTICIPANT_RELATION_ID = p_r.RELATION_ID
			INNER JOIN TBL_RELATION_USER ru
				ON p_r.RELATION_ID=ru.RELATION_ID
			WHERE ru.USER_ID=?
				AND cp.PRODUCER_EXPORTER & ?
			";

		$q = $this->db->query($sql);
		$q->bind_values($user_id, $producer_exporter);
		$q->execute();
		return $q;
	}

	function getRefundExportDay($product_id = 1)
	{
		require_once(ELC_CLASSES.'CRSOProduct.php');
		$cp = new CRSOProduct($this->db);
		$cp->crso =& $this;
		$cp->set(null,$product_id);
		//if we have no refund export day set then
		if(is_numeric($cp->refund_export_day) && $cp->refund_export_day > 0)
			return $cp->refund_export_day;
		return false;
	}
	/**
	 * @static
	 */
	static public function getClearanceLevels() {
		return array(
			'CRSOClearanceRegistrationDetails',
			'CRSOClearanceManagementReport',
			'CRSOClearanceDeclaration',
			'CRSOClearanceDeclarationDetails',
			'CRSOClearanceProvisionalDeclaration',
			'CRSOClearanceRefund',
			'CRSOClearanceRefundDetails',
			'CRSOClearanceDocumentAccess',
			'CRSOClearanceAnnualConfirmation',
			'CRSOClearanceBrandReport',
			'CRSOClearanceReminder',
			'CRSOClearanceAgeing',
			'CRSOClearanceLatePayment',
			'CRSOClearanceGuarantee',
			'CRSOClearanceBlockVat',
			'CRSOClearanceMaintenance',
			'CRSOClearanceUserManagement',
			'CRSOClearanceEAReports',
			'CRSOClearanceRemoteViewing'
		);
	}
	/**
	 * This makes sure that if the available clearance levels change, this reflects on the RUI.
	 */
	function setCRSOAvailableClearance($clearance) {
		if(!$this->isComplete())
			return;

		$sql = "UPDATE TBL_RELATION_USER SET CLEARANCE_LEVEL = CLEARANCE_LEVEL ^ %d WHERE RELATION_ID = ?";

		$all = $this->getClearanceLevels();
		foreach($all as $c) {
			$c = constant($c);
			if((($this->available_clearance_levels & $c) == $c) && !(in_array($c, $clearance))) {
				$q = $this->db->query(sprintf($sql, $c));
				$q->bind_values($this->relation->id);
				$q->execute();
			}
		}
		$this->available_clearance_levels = array_sum($clearance);
	}

	/**
	 *
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_CRSO';
		$this->objectmap['primary']['relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['accepted_sla'] = 'ACCEPTED_SLA';
		$this->objectmap['properties']['reg_no'] = 'CRSO_REG_NO';
		$this->objectmap['properties']['misc_1'] = 'MISC_1';
		$this->objectmap['properties']['available_clearance_levels'] = 'AVAILABLE_CLEARANCE_LEVELS';
		$this->objectmap['properties']['default_language_id'] = 'DEFAULT_LANGUAGE_ID';
		$this->objectmap['properties']['created'] = 'CREATED';

	}
}
/**#@+
 * CRSO Specific Clearance levels
 */
define('CRSOClearanceUserManagement', 1); // Can create/update users
define('CRSOClearanceRegistrationDetails', 2); // Can view pariticipant registration data
define('CRSOClearanceDeclaration', 4); // Access to Declaration related interfaces.
define('CRSOClearanceDeclarationDetails', 8); // Can view declaration details
define('CRSOClearanceRefund', 16); // Access to Refund related interfaces.
define('CRSOClearanceRefundDetails', 32); // Can view declaration details
define('CRSOClearanceManagementReport', 64); // Access to the management report data.
define('CRSOClearanceBlockVat', 128); // Access to block VATs
define('CRSOClearanceMaintenance', 256); // Access to Maintenance interfaces (settings, brands, etc)
define('CRSOClearanceProvisionalDeclaration', 512); // Access to Provisional Declaration related interfaces.
define('CRSOClearanceAgeing', 1024); // Access to the Ageing information
define('CRSOClearanceLatePayment', 2048); // Access to the late payment information
define('CRSOClearanceGuarantee', 4096); // Guarantee access
define('CRSOClearanceReminder', 8192); // Reminder access
define('CRSOClearanceAnnualConfirmation', 16384); // Access to the Annual Confirmation information.
define('CRSOClearanceDocumentAccess', 32768); // Access to the Document Access overview.
define('CRSOClearanceEAReports', 65536); // Access to EA Reports.
define('CRSOClearanceBrandReport', 131072); // Access to the Brand Report
define('CRSOClearanceRemoteViewing', 262144); // Access to Viewing participant actions remotely.
/**#@-*/
?>
