<?php
require_once(IWARE.'AbstractObject.php');
/**
 * @author Nick Loeve <nick@basesoft.com>
 * @package ELC
 * @version $Id$
 */
class CRSOAdminFee extends AbstractObject
{

	var $admin_fee_type;
	var $crso;
	var $start_date;
	var $fee;

	function set($crso_id=false, $admin_fee_type=false, $start_date=false)
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$admin_fee_type || $admin_fee_type = $this->admin_fee_type;
		$start_date || $start_date = $this->start_date;

		$sql = "SELECT *
			FROM TBL_CRSO_ADMIN_FEE
			WHERE CRSO_RELATION_ID = ?
				AND CRSO_ADMIN_FEE_TYPE = ?
				AND START_DATE = ?";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$admin_fee_type, $start_date);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		require_once(ELC_CLASSES.'CRSO.php');
		$this->crso = new CRSO($this->db, array('relation'=>new Relation($this->db,$crso_id)));
		$this->admin_fee_type = $r->{'CRSO_ADMIN_FEE_TYPE'};
		$this->start_date = $r->{'START_DATE'};
		$this->fee = $r->{'FEE'};

		return $this->complete = true;
	}


	function setToCurrentCRSOAdminFee($crso_id=false, $admin_fee_type=false)
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$admin_fee_type || $admin_fee_type = $this->admin_fee_type;

		$sql = "SELECT *
			FROM TBL_CRSO_ADMIN_FEE
			WHERE CRSO_RELATION_ID = ?
				AND CRSO_ADMIN_FEE_TYPE = ?
				AND START_DATE <= NOW()
			ORDER BY START_DATE DESC
			LIMIT 1";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$admin_fee_type);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		require_once(ELC_CLASSES.'CRSO.php');
		$this->crso = new CRSO($this->db, array('relation'=>new Relation($this->db,$crso_id)));
		$this->admin_fee_type = $r->{'CRSO_ADMIN_FEE_TYPE'};
		$this->start_date = $r->{'START_DATE'};
		$this->fee = $r->{'FEE'};
		return $this->complete = true;
	}

	function getCurrentFee($crso_id=false, $admin_fee_type=false)
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$admin_fee_type || $admin_fee_type = $this->admin_fee_type;

		$sql = "SELECT FEE
			FROM TBL_CRSO_ADMIN_FEE
			WHERE CRSO_RELATION_ID = ?
				AND CRSO_ADMIN_FEE_TYPE = ?
				AND START_DATE <= NOW()
			ORDER BY START_DATE DESC
			LIMIT 1";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$admin_fee_type);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		return $r->{'FEE'};
	}

	public static function getTypes()
	{
		return array(
			'CRSO Admin Fee Proof of Payment Review'=>CRSO_ADMIN_FEE_POP_REVIEW,
			'CRSO Admin Fee Refund Request Review'=>CRSO_ADMIN_FEE_REFUND_REVIEW,
			'CRSO Admin Fee Refund Export'=>CRSO_ADMIN_FEE_REFUND_EXPORT,
			'CRSO Admin Fee PoP Confirmation'=>CRSO_ADMIN_FEE_POP_CONFIRMATION
		);
	}

	/**
	 * Selects an fee overview by crso_id and admin_fee_type
	 */
	function selectCrsoTypeFee($crso_id, $admin_fee_type)
	{
		$sql = "SELECT START_DATE, FEE
			FROM TBL_CRSO_ADMIN_FEE
			WHERE CRSO_RELATION_ID=?
				AND CRSO_ADMIN_FEE_TYPE=?
			ORDER BY START_DATE";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, $admin_fee_type);
		$q->execute();
		return $q;
	}

	/**#@+
	 * @access private
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_CRSO_ADMIN_FEE';
		$this->objectmap['primary']['admin_fee_type'] = 'CRSO_ADMIN_FEE_TYPE';
		$this->objectmap['primary']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['primary']['start_date'] = 'START_DATE';
		$this->objectmap['properties']['fee'] = 'FEE';
		$this->objectmap['inserttype'] = 'REPLACE';
	}
	/**#@-*/
}

/**
 * CRSO_ADMIN_FEE_TYPE defines
 */
define('CRSO_ADMIN_FEE_POP_REVIEW', 1);
define('CRSO_ADMIN_FEE_REFUND_REVIEW', 2);
define('CRSO_ADMIN_FEE_REFUND_EXPORT', 3);
define('CRSO_ADMIN_FEE_POP_CONFIRMATION', 4);
?>