<?php
require_once(IWARE.'AbstractObject.php');
require_once(IWARE.'Document.php');
/**
 * @package
 * @subpackage
 * @since 1-okt-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class CRSOParticipantGuarantee extends AbstractObject
{
	/**#@+
	 * @access public
	 */
	/**
	 * @var CRSO $crso
	 */
	public $crso;
	/**
	 * @var Participant $participant
	 */
	public $participant;
	/**
	 * @var string $invoice_credit
	 */
	public $invoice_credit;
	/**
	 * @var int $guarantee_id
	 */
	public $id;
	/**
	 * @var int $year
	 */
	public $year;
	/**
	 * @var Document $document
	 */
	public $document;
	/**
	 * @var string $invoice_date
	 */
	public $invoice_date;
	/**
	 * @var string $import_date
	 */
	public $import_date;
	/**
	 * @var string $read_date
	 */
	public $read_date;
	/**
	 * @var int $status
	 */
	public $status;
	/**
	 * @var float $amount
	 */
	public $amount;
	/**#@-*/
	/**
	 *
	 */
	function set($crso_id=null,$invoice_credit=null,$id=null,$year=null)
	{
		$sql = "SELECT PARTICIPANT_RELATION_ID, DOCUMENT_ID,
				INVOICE_DATE, READ_DATE, `STATUS`, AMOUNT
			FROM TBL_CRSO_PARTICIPANT_GUARANTEE
			WHERE CRSO_RELATION_ID = ?
				AND INVOICE_CREDIT = ?
				AND GUARANTEE_ID = ?
				AND `YEAR` = ?";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$invoice_credit,$id,$year);
		$q->execute();
		$rows = $q->fetchAll();
		if(!count($rows))
			return false;
		$r = current($rows);

		$this->crso = new CRSO($this->db,array('relation'=>new Relation($this->db,$crso_id)));
		$this->participant = new Participant($this->db,array('relation'=>new Relation($this->db,$r->PARTICIPANT_RELATION_ID)));
		$this->document = new Document($this->db,$r->DOCUMENT_ID);

		$this->invoice_credit = $invoice_credit;
		$this->id = $id;
		$this->year = $year;
		$this->invoice_date = $r->INVOICE_DATE;
		$this->read_date = $r->READ_DATE;
		$this->status = $r->STATUS;
		$this->amount = $r->AMOUNT;

		return $this->complete = true;
	}

	function set_Doc($crso_id='',$participant_id='',$document_id='')
	{
		$sql = "SELECT PARTICIPANT_RELATION_ID, DOCUMENT_ID,
				INVOICE_DATE, READ_DATE, `STATUS`, AMOUNT,GUARANTEE_ID,INVOICE_CREDIT,`YEAR`
			FROM TBL_CRSO_PARTICIPANT_GUARANTEE
			WHERE CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ?
				AND DOCUMENT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$participant_id,$document_id);
		$q->execute();
		$rows = $q->fetchAll();
		if(!count($rows))
			return false;
		$r = current($rows);

		$this->crso = new CRSO($this->db,array('relation'=>new Relation($this->db,$crso_id)));
		$this->participant = new Participant($this->db,array('relation'=>new Relation($this->db,$r->PARTICIPANT_RELATION_ID)));
		$this->document = new Document($this->db,$r->DOCUMENT_ID);

		$this->invoice_credit = $r->INVOICE_CREDIT;
		$this->id = $r->GUARANTEE_ID;
		$this->year = $r->YEAR;
		$this->invoice_date = $r->INVOICE_DATE;
		$this->read_date = $r->READ_DATE;
		$this->status = $r->STATUS;
		$this->amount = $r->AMOUNT;

		return $this->complete = true;
	}

	/**
	 *
	 */
	function insert($graceful=false)
	{
		$d = currentDate();
		$this->import_date = $d->getYear().'-'.$d->getMonth().'-'.$d->getDay();
		parent::insert($graceful);
	}
	/**
	 *
	 */
	function markAsRead($group_id)
	{
		if(!$this->isComplete())
			$this->set();

		if($this->read_date||($group_id!=GROUP_PARTICIPANT))
			return false;

		$d = currentDate();
		$this->read_date = $d->getYear().'-'.$d->getMonth().'-'.$d->getDay();
		parent::update();
	}
	/**
	 *
	 */
	function isUnique()
	{
		$sql = "SELECT PARTICIPANT_RELATION_ID FROM TBL_CRSO_PARTICIPANT_GUARANTEE
			WHERE CRSO_RELATION_ID = ?
				AND INVOICE_CREDIT = ?
				AND GUARANTEE_ID = ?
				AND `YEAR` = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->crso->relation->id,$this->invoice_credit,$this->id,$this->year);
		$q->execute();
		$rows =  $q->fetchAll();
		return !((bool)count($rows));
	}
	/**
	 *
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_CRSO_PARTICIPANT_GUARANTEE';
		$this->objectmap['primary']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['primary']['invoice_credit'] = 'INVOICE_CREDIT';
		$this->objectmap['primary']['id'] = 'GUARANTEE_ID';
		$this->objectmap['primary']['year'] = 'YEAR';
		$this->objectmap['properties']['participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
		$this->objectmap['properties']['document->id'] = 'DOCUMENT_ID';
		$this->objectmap['properties']['invoice_date'] = 'INVOICE_DATE';
		$this->objectmap['properties']['import_date'] = 'IMPORT_DATE';
		$this->objectmap['properties']['read_date'] = 'READ_DATE';
		$this->objectmap['properties']['status'] = 'STATUS';
		$this->objectmap['properties']['amount'] = 'AMOUNT';
	}
}
?>