<?php
require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES.'Product.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'User.php');

/**
 * @author Marten Koedam <marten@basesoft.com>
 * @package I-Ware
 * @version $Id$
 */
class CRSOParticipantProduct extends AbstractObject
{
	/**
	 * @var CRSO
	 */
	var $crso;
	/**
	 * @var Participant
	 */
	var $participant;
	/**
	 * @var String ISO braatsel Mysql date
	 */
	var $obligation_start_date;
	/**
	 * Are declarations triggered for this CRSOParticipant?
	 * @var int $no_declaration
	 */
	var $no_declaration = 0;
	var $product;
	var $status;
	var $active;
	var $producer_exporter;

	var $provisional_change_request_status;

	var $general_manager;
	var $general_manager_authorized;
	var $participant_type_change_request;

	var $authorize_proof_of_payment = REFUND_AUTHORIZE_OPERATOR; // payments are _always_ authorized by operator
	var $authorize_refund_request_eu;
	var $authorize_refund_request_non_eu;

	var $allow_annual_confirmation;

	/**
	 *
	 */
	function set($crso_id='', $participant_id='', $product_id='')
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$participant_id || $participant_id = $this->participant->relation->id;
		$product_id || $product_id = $this->product->id;

		if(!$participant_id||!$crso_id || !$product_id)
			return false;

//				,PROVISIONAL_CHANGE_REQUEST_STATUS
		$sql = "SELECT `STATUS`, ACTIVE, PRODUCER_EXPORTER, OBLIGATION_START_DATE,
				NO_DECLARATION, GENERAL_MANAGER_ID, GENERAL_MANAGER_AUTHORIZED,
				AUTHORIZE_PROOF_OF_PAYMENT, AUTHORIZE_REFUND_REQUEST_EU, AUTHORIZE_REFUND_REQUEST_NON_EU,
				ALLOW_ANNUAL_CONFIRMATION,
				PROVISIONAL_CHANGE_REQUEST_STATUS,PARTICIPANT_TYPE_CHANGE_REQUEST, PRODUCT_ID
			FROM TBL_CRSO_PARTICIPANT_PRODUCT
			WHERE CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID =?
				AND PRODUCT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$participant_id, $product_id);

		if(!$q->execute())
			return false;
		if(!($r = $q->fetch()))
			return false;

		$this->participant = new Participant($this->db,array('relation' => new Relation($this->db,$participant_id)));
		$this->crso = new CRSO($this->db,array('relation' => new Relation($this->db,$crso_id)));
		$this->product = new Product($this->db, $r->PRODUCT_ID);

		$this->status = $r->STATUS;
		$this->active = $r->ACTIVE;
		$this->producer_exporter = $r->PRODUCER_EXPORTER;
		$this->obligation_start_date = $r->OBLIGATION_START_DATE;
		$this->no_declaration = $r->NO_DECLARATION;

		$this->general_manager = is_numeric($r->GENERAL_MANAGER_ID) ? new ELCUser($this->db, $r->GENERAL_MANAGER_ID) : null;
		$this->general_manager_authorized = ($r->GENERAL_MANAGER_AUTHORIZED == '') ? null : $r->GENERAL_MANAGER_AUTHORIZED; // can be null, 0 or 1

		$this->participant_type_change_request = $r->PARTICIPANT_TYPE_CHANGE_REQUEST;
		$this->provisional_change_request_status = $r->PROVISIONAL_CHANGE_REQUEST_STATUS;

		$this->authorize_refund_request_eu = $r->AUTHORIZE_REFUND_REQUEST_EU;
		$this->authorize_refund_request_non_eu = $r->AUTHORIZE_REFUND_REQUEST_NON_EU;

		$this->authorize_proof_of_payment = $r->AUTHORIZE_PROOF_OF_PAYMENT;

		$this->allow_annual_confirmation = $r->ALLOW_ANNUAL_CONFIRMATION;

		return $this->complete = true;
	}

	static public function createByIds($db, $participant_id, $crso_id, $product_id)
	{
		$participant = new Participant($db, array('relation'=>new Relation($db, $participant_id)));
		$crso = new CRSO($db, array('relation'=>new Relation($db, $crso_id)));
		$product = new Product($db, $product_id);

		$cp = new CRSOParticipantProduct($db, array('crso'=>$crso, 'participant'=>$participant, 'product'=>$product));
		return $cp;
	}

	function insert()
	{
		$rv = parent::insert();
		info(sprintf('Created new CRSO-Participant [%s/%s/%s]',
			$this->crso->relation->id, $this->participant->relation->id,$this->product->id));
		return $rv;
	}

	function update()
	{
		$rv = parent::update();
		info(sprintf('Updated CRSO-Participant [%s/%s/%s]',
			$this->crso->relation->id, $this->participant->relation->id, $this->product->id));
		return $rv;
	}

	/**
	 *
	 */
	function deleteFromCRSO($crso_id='', $participant_id='', $product_id='')
	{
		$crso_id || $crso_id = $this->crso->id;
		$participant_id || $participant_id = $this->participant->relation->id;
		$product_id || ($product_id = isset($this->product->id) ? $this->product->id : 1);

		if(!$participant_id||!$crso_id || !$product_id)
			return false;

		// ALWAYS check if the connection between the two can be removed.
		// CHECK TBL_TRUST, TBL_PROVISIONAL_DECLARATION, TBL_DECLARATION etc.

		$sql = "DELETE FROM TBL_CRSO_PARTICIPANT_PRODUCT
			WHERE CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ?
				AND PRODUCT_ID = ? ";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$participant_id, $product_id);
		return $q->execute();
	}
	/**
	 * @param int $participant_id
	 * @param string $return
	 * @return mixed $rv
	 */
	function getParticipantCRSOs($participant_id,$return='q',$producer_exporter=false, $xor=false,$product_id=null)
	{
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT CRSO_RELATION_ID, R.VAT_NO, R.`NAME` as CRSO_NAME,
				GENERAL_MANAGER_ID, GENERAL_MANAGER_AUTHORIZED,PRODUCER_EXPORTER
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CP
			INNER JOIN TBL_RELATION as R
				ON CP.CRSO_RELATION_ID = R.RELATION_ID
			WHERE PARTICIPANT_RELATION_ID = ?
				AND CP.PRODUCT_ID = ?
				AND CP.`STATUS` >= ?
				AND CP.ACTIVE = 1 ";
		if($producer_exporter && !$xor)
			$sql .= "AND PRODUCER_EXPORTER & ".$producer_exporter;
		else if($producer_exporter && $xor)
			$sql .= "AND PRODUCER_EXPORTER ^ ".$producer_exporter;
		$q = $this->db->query($sql);
		$q->bind_values($participant_id, $product_id, PARTICIPANT_STATUS_CONFIRMED);
		$q->execute();
		if($return=='q')
			return $q;
		$rv = array();
		while($r=$q->fetch())
			$rv[] = $r->CRSO_RELATION_ID;
		return $rv;
	}

	function getProductsForCRSOParticipant($participant_id=null,$crso_id=null,$producer_exporter=null)
	{
		$participant_id || $participant_id = $this->participant->relation->id;
		$crso_id || $crso_id = $this->crso->relation->id;

		$sql = "SELECT PRODUCT_ID
			FROM TBL_CRSO_PARTICIPANT_PRODUCT
			WHERE PARTICIPANT_RELATION_ID = ?
				AND CRSO_RELATION_ID = ? ";
		if($producer_exporter)
			$sql .= "AND PRODUCER_EXPORTER & ".$producer_exporter;
		$q = $this->db->query($sql);
		$q->bind_values($participant_id,$crso_id);
		$q->execute();
		return $q;
	}

	function getParticipantProducts($participant_id=null)
	{
		$participant_id || $participant_id = $this->participant->relation->id;

		require_once(ELC_CLASSES.'Participant.php'); // Only activated/approved products
		$sql = "SELECT DISTINCT CPP.PRODUCT_ID, P.`NAME` as PRODUCT_NAME
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CPP
			INNER JOIN TBL_PRODUCT as P
				ON CPP.PRODUCT_ID = P.PRODUCT_ID
			WHERE PARTICIPANT_RELATION_ID = ?
				AND `STATUS` >= ?
				AND `ACTIVE` = ?";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id, PARTICIPANT_STATUS_CONFIRMED, 1);
		$q->execute();
		return $q;
	}

	function getProducerProductsCRSO($participant_id=null)
	{
		$participant_id || $participant_id = $this->participant->relation->id;
		require_once(ELC_CLASSES.'Participant.php'); // Only activated/approved products
		$sql = "SELECT CPP.PRODUCT_ID,P.`NAME` as PRODUCT_NAME,
				CPP.CRSO_RELATION_ID,R.`NAME` as CRSO_NAME
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CPP
			INNER JOIN TBL_PRODUCT as P
				ON CPP.PRODUCT_ID = P.PRODUCT_ID
			INNER JOIN TBL_RELATION as R
				ON CPP.CRSO_RELATION_ID = R.RELATION_ID
			WHERE PARTICIPANT_RELATION_ID = ?
				AND `STATUS` >= ?
				AND `ACTIVE` = ?
				AND PRODUCER_EXPORTER & ".PARTICIPANT_PRODUCER;
		$q = $this->db->query($sql);
		$q->bind_values($participant_id, PARTICIPANT_STATUS_CONFIRMED, 1);
		$q->execute();
		$rv = array();
		while($r = $q->fetch())
		{
			$rv[$r->PRODUCT_ID]['name'] = $r->PRODUCT_NAME;
			$rv[$r->PRODUCT_ID]['crsos'][$r->CRSO_RELATION_ID] = $r->CRSO_NAME;
		}
		return $rv;
	}

	/**
	 *
	 */
	function getCRSOParticipants($crso_id,$return='q',$producer_exporter=false, $product_id=null, $is_active=1)
	{
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT PARTICIPANT_RELATION_ID, R.VAT_NO, R.`NAME` as PARTICIPANT_NAME
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CP
			INNER JOIN TBL_RELATION as R
				ON CP.PARTICIPANT_RELATION_ID = R.RELATION_ID
			WHERE CRSO_RELATION_ID = ?
			";

		if (!is_null($is_active))
			$sql .= "AND CP.`ACTIVE` =  ".(int) $is_active;

		$sql .= " AND CP.PRODUCT_ID = ? ";

		if($producer_exporter)
			$sql .= " AND PRODUCER_EXPORTER & ".$producer_exporter;
		$sql .= " ORDER BY R.`NAME`";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, $product_id);
		$q->execute();
		if($return=='q')
			return $q;
		$rv = array();
		while($r=$q->fetch())
			$rv[] = $r->PARTICIPANT_RELATION_ID;
		return $rv;
	}

	/**
	 * Translates a general manager status to a langIf text
	 * @param LanguageInterface $langIf
	 * @param int $status
	 * @return Array (Translation,CSSClassName)
	 */
	public static function translateGmStatus($langIf, $status)
	{
		switch($status)
		{
			case GENERAL_MANAGER_REQUEST:
				return array($langIf->getText('General Manager Requested'), 'status_new');
			case GENERAL_MANAGER_AUTHORIZED:
				return array($langIf->getText('General Manager Authorized'), 'status_verified');
		}
	}

	public static function translateAuthTypeForParticipant($langIf, $status)
	{
		switch($status)
		{
			case REFUND_AUTHORIZE_OPERATOR:
				return $langIf->getText('Operator');
			case REFUND_AUTHORIZE_GENERAL_MANAGER:
				return $langIf->getText('General Manager');
			case REFUND_AUTHORIZE_STATUTORY_AUDITOR:
				return $langIf->getText('Statutory Auditor');
			case REFUND_AUTHORIZE_EMAIL:
				return $langIf->getText('Importer via Email');
		}
		return '';
	}

	/**
	 * @access private
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_CRSO_PARTICIPANT_PRODUCT';
		$this->objectmap['primary']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['primary']['participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
		$this->objectmap['primary']['product->id'] = 'PRODUCT_ID';
		$this->objectmap['properties']['status'] = 'STATUS';
		$this->objectmap['properties']['active'] = 'ACTIVE';
		$this->objectmap['properties']['producer_exporter'] = 'PRODUCER_EXPORTER';
		$this->objectmap['properties']['obligation_start_date'] = 'OBLIGATION_START_DATE';
		$this->objectmap['properties']['no_declaration'] = 'NO_DECLARATION';

		$this->objectmap['properties']['general_manager->id'] = 'GENERAL_MANAGER_ID';
		$this->objectmap['properties']['general_manager_authorized'] = 'GENERAL_MANAGER_AUTHORIZED';

		$this->objectmap['properties']['provisional_change_request_status'] = 'PROVISIONAL_CHANGE_REQUEST_STATUS';
		$this->objectmap['properties']['participant_type_change_request'] = 'PARTICIPANT_TYPE_CHANGE_REQUEST';

		$this->objectmap['properties']['authorize_proof_of_payment'] = 'AUTHORIZE_PROOF_OF_PAYMENT';
		$this->objectmap['properties']['authorize_refund_request_eu'] = 'AUTHORIZE_REFUND_REQUEST_EU';
		$this->objectmap['properties']['authorize_refund_request_non_eu'] = 'AUTHORIZE_REFUND_REQUEST_NON_EU';

		$this->objectmap['properties']['allow_annual_confirmation'] = 'ALLOW_ANNUAL_CONFIRMATION';
	}

	/**
	 *
	 */
	function setObligationStartDate($date)
	{
		$this->obligation_start_date = $date;
	}

	function checkParticipantChangeProducer($participant_id, $product_id = 1)
	{
		$sql = "SELECT CRSO_RELATION_ID
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CP
			WHERE PARTICIPANT_RELATION_ID = ? AND PRODUCT_ID = ? AND PARTICIPANT_TYPE_CHANGE_REQUEST & ".PARTICIPANT_TYPE_CHANGE_PRODUCER_AUTHORIZED;
		$q = $this->db->query($sql);
		$q->bind_values($participant_id, $product_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
 		// get the first one, continue with next after this is done.
		return $r->CRSO_RELATION_ID;
	}

	function hasRefundSettings($validate=null){
		if(!$this->isComplete())
			$this->set();

		$validate || $validate = array('authorize_refund_request_eu', 'authorize_refund_request_non_eu');
		foreach($validate as $setting)
			if($this->$setting)
				return true;
		return false;
	}

	/**
	 * Returns true if any of the crsos belonging to the given participant have the HasCrsoParticipantLogin set.
	 * @param $participant_id
	 * @param $product_id
	 * @return boolean
	 */
	function hasCrsoParticipantLogin($participant_id, $product_id) {
		$crsos = $this->getParticipantCRSOs($participant_id, 'array', false, false, $product_id);
		foreach ($crsos as $crso_id) {

			$crso = new CRSO($this->db,array('relation'=>new Relation($this->db, $crso_id)));
			$crso->relation->set();
			$crso->relation->address->set();

			require_once(ELC_CLASSES.'Country.php');
			$country_id = $crso->relation->address->country_id;
			$country = new Country($this->db, $country_id, true);

			if ($country->has(HasCrsoParticipantLogin, $product_id)) {
				return true;
			}
		}

		return false;
	}

}
/**#@+
 * Participant Status
 */
define('PARTICIPANT_STATUS_ACTIVE',1);

define('PARTICIPANT_STATUS_NEW',1);
define('PARTICIPANT_STATUS_VERIFIED',2);
define('PARTICIPANT_STATUS_AUTHORIZED',3);
define('PARTICIPANT_STATUS_CONFIRMED',4);
define('PARTICIPANT_STATUS_PARTICIPANT_READY',5);

define('GENERAL_MANAGER_REQUEST', 1);
define('GENERAL_MANAGER_AUTHORIZED', 2);

define('REFUND_AUTHORIZE_OPERATOR', 1);
define('REFUND_AUTHORIZE_GENERAL_MANAGER', 2);
define('REFUND_AUTHORIZE_STATUTORY_AUDITOR', 3);
define('REFUND_AUTHORIZE_EMAIL', 4);

define('PROVISIONAL_CHANGE_REQUEST_OPEN',0);
define('PROVISIONAL_CHANGE_REQUEST_PENDING',1);
define('PROVISIONAL_CHANGE_REQUEST_AUTHORIZED',2);

define('PARTICIPANT_TYPE_CHANGE_EXPORTER',1);
define('PARTICIPANT_TYPE_CHANGE_PRODUCER',2);
define('PARTICIPANT_TYPE_CHANGE_PRODUCER_AUTHORIZED',4);


/**#@-*/
?>
