<?php
require_once(IWARE.'AbstractObject.php');
/**
 * @package
 * @subpackage
 * @since 20-apr-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class CRSOParticipantReminder extends AbstractObject
{
	/**#@+
	 * @access public
	 */
	/**
	 * @var CRSO $crso
	 */
	public $crso;
	/**
	 * @var Participant $participant
	 */
	public $participant;
	/**
	 * @var Document $document
	 */
	public $document;
	/**
	 * @var string $invoice_date
	 */
	public $invoice_date;
	/**
	 * @var string $import_date
	 */
	public $import_date;
	/**
	 * @var string $read_date
	 */
	public $read_date;
	/**
	 * @var int $status
	 */
	public $status;
	/**
	 * @var int $reminder_level
	 */
	public $reminder_level;
	/**#@-*/
	/**
	 * @param int $crso_id
	 * @param int $participant_id
	 * @param int $document_id
	 * @return bool
	 */
	function set($crso_id=null,$participant_id=null,$document_id=null)
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$participant_id || $participant_id = $this->participant->relation->id;
		$document_id || $document_id = $this->document->id;

		$sql = "SELECT INVOICE_DATE,IMPORT_DATE,READ_DATE,`STATUS`, REMINDER_LEVEL
			FROM TBL_CRSO_PARTICIPANT_REMINDER
			WHERE CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ?
				AND DOCUMENT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$participant_id,$document_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		require_once(ELC_CLASSES.'CRSO.php');
		require_once(ELC_CLASSES.'Participant.php');
		require_once(ELC_CLASSES.'Document.php');

		$this->crso = new CRSO($this->db,array('relation'=>new Relation($this->db,$crso_id)));
		$this->participant = new Participant($this->db,array('relation'=>new Relation($this->db,$participant_id)));
		$this->document = new ELCDocument($this->db,$document_id);
		$this->invoice_date = $r->INVOICE_DATE;
		$this->import_date = $r->IMPORT_DATE;
		$this->read_date = $r->READ_DATE;
		$this->status = $r->STATUS;
		$this->reminder_level = $r->REMINDER_LEVEL;

		return $this->complete = true;
	}

	function isUnique()
	{
		$sql = "SELECT INVOICE_DATE
			FROM TBL_CRSO_PARTICIPANT_REMINDER
			WHERE CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ?
				AND INVOICE_DATE = ?
				AND REMINDER_LEVEL = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->crso->relation->id,$this->participant->relation->id,$this->invoice_date,$this->reminder_level);
		$q->execute();
		if(!($r = $q->fetch()))
			return true;
		return false;
	}

	/**
	 *
	 */
	function insert($graceful=false)
	{
		$d = currentDate();
		$this->import_date = $d->getYear().'-'.$d->getMonth().'-'.$d->getDay();
		parent::insert($graceful);
	}
	/**
	 * Mark the current Reminder as being read. Only do this if the current user is a participant
	 */
	function markAsRead()
	{
		if(!$this->isComplete())
			$this->set();

		if($this->read_date||($_SESSION['session_group_group_id']!=GROUP_PARTICIPANT))
			return false;

		$d = currentDate();
		$this->read_date = $d->getYear().'-'.$d->getMonth().'-'.$d->getDay();
		parent::update();
	}

	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_CRSO_PARTICIPANT_REMINDER';
		$this->objectmap['primary']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['primary']['participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
		$this->objectmap['primary']['document->id'] = 'DOCUMENT_ID';
		$this->objectmap['properties']['invoice_date'] = 'INVOICE_DATE';
		$this->objectmap['properties']['import_date'] = 'IMPORT_DATE';
		$this->objectmap['properties']['read_date'] = 'READ_DATE';
		$this->objectmap['properties']['status'] = 'STATUS';
		$this->objectmap['properties']['reminder_level'] = 'REMINDER_LEVEL';
	}
}

?>
