<?php
require_once(IWARE.'AbstractObject.php');
/**
 * @package WEEE
 * @subpackage CRSO
 * @since 23-nov-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class CRSOProduct extends AbstractObject
{
	var $crso;
	var $product;
	var $payment_interest_rate;
	var $forfeit_start_date;
	var $forfeit_day;
	var $minimal_obligation_date;
	var $refund_export_day;
	var $refund_start_date;
	var $declaration_reminder_day;
	var $prov_decl_deviation_trigger_percentage;
	var $resubmit_deviation_trigger_percentage;
	var $ac_cut_off_month;
	var $created;

	function set($crso_id=null,$product_id=null)
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT * FROM TBL_CRSO_PRODUCT
			WHERE CRSO_RELATION_ID = ?
				AND PRODUCT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$product_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		require_once(ELC_CLASSES.'CRSO.php');
		require_once(ELC_CLASSES.'Product.php');
		$this->crso = new CRSO($this->db,array('relation'=>new Relation($this->db,$crso_id)));
		$this->product = new Product($this->db,$product_id);
		$this->payment_interest_rate = $r->PAYMENT_INTEREST_RATE;
		$this->forfeit_start_date = $r->FORFEIT_START_DATE;
		$this->forfeit_day = $r->FORFEIT_DAY;
		$this->minimal_obligation_date = $r->MINIMAL_OBLIGATION_DATE;
		$this->refund_export_day = $r->REFUND_EXPORT_DAY;
		$this->refund_start_date = $r->REFUND_START_DATE;
		$this->declaration_reminder_day = $r->DECLARATION_REMINDER_DAY;
		$this->prov_decl_deviation_trigger_percentage = $r->PROV_DECL_DEVIATION_TRIGGER_PERCENTAGE;
		$this->resubmit_deviation_trigger_percentage = $r->RESUBMIT_DEVIATION_TRIGGER_PERCENTAGE;
		$this->ac_cut_off_month = $r->AC_CUT_OFF_MONTH;
		$this->created = $r->CREATED;

		return $this->complete = true;
	}

	function getProducts($crso_id=null)
	{
		$crso_id || $crso_id = $this->crso->relation->id;

		$sql = "SELECT CP.PRODUCT_ID, P.`NAME` as PRODUCT_NAME
			FROM TBL_CRSO_PRODUCT as CP
			INNER JOIN TBL_PRODUCT as P
				ON CP.PRODUCT_ID = P.PRODUCT_ID
			WHERE CP.CRSO_RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id);
		$q->execute();
		return $q;
	}

	function getCRSOProductOverview($crso_id=null)
	{
		$crso_id || $crso_id = $this->crso->relation->id;

		$sql = "SELECT P.PRODUCT_ID, P.`NAME` as PRODUCT_NAME, IF(ISNULL(CP.CRSO_RELATION_ID),0,1) as CHECKBOX
			FROM TBL_PRODUCT as P
			LEFT JOIN TBL_CRSO_PRODUCT as CP
				ON P.PRODUCT_ID = CP.PRODUCT_ID
					AND CP.CRSO_RELATION_ID = ?
			ORDER BY PRODUCT_NAME";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id);
		$q->execute();
		return $q;
	}
	/**#@+
	 * Used in DeclarationReminderTrigger
	 */
	/**
	 *
	 */
	function &getPartipciantsWithOpenDeclarations($crso_id=null,$product_id=null)
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$product_id || $product_id = $this->product->id;

		$this->isComplete() || $this->set($crso_id,$product_id);

		require_once(IWARE.'form.php');
		$formDate = new IWFormDateTime('name');

		$formDate->setMySQLDate($this->forfeit_start_date);
		$minmonth = (int) date('m', $formDate->timestamp);
		$minyear = (int) date('Y', $formDate->timestamp);

		$participants = array();
		require_once(ELC_CLASSES.'Declaration.php');
		$sql = "SELECT (BIT_OR(DL.STATUS)) as DECLARATION_STATUS, D.PARTICIPANT_RELATION_ID
				FROM TBL_DECLARATION as D
				INNER JOIN TBL_DECLARATION_LINE as DL
					USING(DECLARATION_ID)
				INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT as CPP
					ON D.PARTICIPANT_RELATION_ID = CPP.PARTICIPANT_RELATION_ID
						AND D.CRSO_RELATION_ID = CPP.CRSO_RELATION_ID
						AND D.PRODUCT_ID = CPP.PRODUCT_ID
						AND CPP.ACTIVE = 1
				WHERE D.CRSO_RELATION_ID = ?
					AND D.PRODUCT_ID = ?
					AND (D.`MONTH` > ? OR D.`YEAR` > ?)
					AND CPP.NO_DECLARATION = ?
				GROUP BY D.DECLARATION_ID
				HAVING DECLARATION_STATUS = ".DECLARATION_OPEN;
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$product_id,$minmonth,$minyear,0);
		if(!$q->execute())
			trigger_error("Error executing query for open declarations", E_USER_ERROR);
		while($r = $q->fetch()) // Unique Participants ONLY!!
			$participants[$r->PARTICIPANT_RELATION_ID] = $r->PARTICIPANT_RELATION_ID;
		return $participants;
	}

	function getDeclarationReminderDay()
	{
		if(is_numeric($this->declaration_reminder_day) && $this->declaration_reminder_day > 0)
			return $this->declaration_reminder_day;
		return false;
	}
	/**#@-*/

	function updateForfeitDayForAllProducts($day,$country)
	{
		$sql = "SELECT PRODUCT_ID FROM TBL_CRSO_PRODUCT WHERE CRSO_RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->crso->relation->id);
		$q->execute();
		while($r=$q->fetch())
		{
			if(!$country->has(HasCrsoOneExportDay,$r->PRODUCT_ID))
				continue;
			$cp = new CRSOProduct($this->db,array('crso'=>$this->crso,'product'=>new Product($this->db,$r->PRODUCT_ID)));
			$cp->forfeit_day = $day;
			$cp->update();
		}
	}
	function getMaxExportDay($crso)
	{
		$sql = "SELECT MAX(IFNULL(FORFEIT_DAY,0)) AS DD, MAX(IFNULL(REFUND_EXPORT_DAY,0)) AS RD
				FROM TBL_CRSO_PRODUCT WHERE CRSO_RELATION_ID = ?
				GROUP BY CRSO_RELATION_ID";
		$q = $this->db->query($sql);
		$q->bind_values($crso);
		$q->execute();
		if(!$r = $q->fetch())
			return 0;
		if($r->DD > $r->RD)
			return $r->DD;
		return $r->RD;
	}

	function getNextCutOffDateDeclaration($crso_id=null,$product_id=null) {
		$crso_id || $crso_id = $this->crso->relation->id;
		$product_id || $product_id = $this->product->id;
		$sql = "SELECT FORFEIT_DAY FROM TBL_CRSO_PRODUCT
			WHERE CRSO_RELATION_ID = ?
				AND PRODUCT_ID = ?
				AND FORFEIT_DAY > 0";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$product_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		$fday = $r->FORFEIT_DAY;

		$date = currentDate();
		$year = $date->getYear();
		$month = $date->getMonth();
		$day = $date->getDay();

		if($fday < $day){
			if($month == 12){
				$year = $year + 1;
				$month = 1;
			}
			else {
				$month = $month + 1;
			}
		}
		return strftime('%x', mktime(1,1,1, $month, $fday, $year));

	}
	function getNextCutOffDateRefund($crso_id=null,$product_id=null) {
		$crso_id || $crso_id = $this->crso->relation->id;
		$product_id || $product_id = $this->product->id;
		$sql = "SELECT REFUND_EXPORT_DAY FROM TBL_CRSO_PRODUCT
			WHERE CRSO_RELATION_ID = ?
				AND PRODUCT_ID = ?
				AND REFUND_EXPORT_DAY > 0";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$product_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		$rday = $r->REFUND_EXPORT_DAY;

		$date = currentDate();
		$year = $date->getYear();
		$month = $date->getMonth();
		$day = $date->getDay();

		if($rday < $day){
			if($month == 12){
				$year = $year + 1;
				$month = 1;
			}
			else {
				$month = $month + 1;
			}
		}
		return strftime('%x', mktime(1,1,1, $month, $rday, $year));
	}
	function getNextCutOffDateAnnualConfirmation($crso_id=null,$product_id=null) {
		$crso_id || $crso_id = $this->crso->relation->id;
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT AC_CUT_OFF_MONTH,
				IF(REFUND_EXPORT_DAY IS NULL OR FORFEIT_DAY>REFUND_EXPORT_DAY, FORFEIT_DAY, REFUND_EXPORT_DAY) AS AC_CUT_OFF_DAY
			FROM TBL_CRSO_PRODUCT
			WHERE CRSO_RELATION_ID = ?
				AND PRODUCT_ID = ?
				AND AC_CUT_OFF_MONTH IS NOT NULL";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$product_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		$acday = $r->AC_CUT_OFF_DAY;
		$acmonth = $r->AC_CUT_OFF_MONTH;

		$date = currentDate();
		$year = $date->getYear();
		$month = $date->getMonth();
		$day = $date->getDay();

		if($month == $acmonth){
			if($acday < $day){
				$year = $year + 1;
			}
		}
		else
		if($acmonth < $month){
			$year = $year + 1;
		}
		return strftime('%x', mktime(1,1,1, $acmonth, $acday, $year));
	}

	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_CRSO_PRODUCT';
		$this->objectmap['primary']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['primary']['product->id'] = 'PRODUCT_ID';
		$this->objectmap['properties']['payment_interest_rate'] = 'PAYMENT_INTEREST_RATE';
		$this->objectmap['properties']['forfeit_start_date'] = 'FORFEIT_START_DATE';
		$this->objectmap['properties']['forfeit_day'] = 'FORFEIT_DAY';
		$this->objectmap['properties']['minimal_obligation_date'] = 'MINIMAL_OBLIGATION_DATE';
		$this->objectmap['properties']['refund_export_day'] = 'REFUND_EXPORT_DAY';
		$this->objectmap['properties']['refund_start_date'] = 'REFUND_START_DATE';
		$this->objectmap['properties']['declaration_reminder_day'] = 'DECLARATION_REMINDER_DAY';
		$this->objectmap['properties']['prov_decl_deviation_trigger_percentage'] = 'PROV_DECL_DEVIATION_TRIGGER_PERCENTAGE';
		$this->objectmap['properties']['resubmit_deviation_trigger_percentage'] = 'RESUBMIT_DEVIATION_TRIGGER_PERCENTAGE';
		$this->objectmap['properties']['ac_cut_off_month'] = 'AC_CUT_OFF_MONTH';
		$this->objectmap['properties']['created'] = 'CREATED';
	}
}
?>
