<?php
require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES . 'CRSO.php');
require_once(ELC_CLASSES.'Product.php');
/**
 * @package
 * @subpackage
 * @since 28-feb-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class CRSOProductDimension extends AbstractObject
{
	var $id;
	var $product;
	var $crso;
	var $name;

	/**
	 * @param int $id
	 */
	function set($id='')
	{
		$id || $id = $this->id;

		$sql = "SELECT PRODUCT_ID, CPD_ID, CRSO_RELATION_ID,
				NAME as DIMENSION_NAME,
				START_YEAR, END_YEAR
			FROM TBL_CRSO_PRODUCT_DIMENSION
			WHERE CPD_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		if(!$q->execute())
			return false;
		if(!($r = $q->fetch()))
			return false;

		$this->id = $id;
		$this->product = new Product($this->db,$r->PRODUCT_ID);
		$this->crso = new CRSO($this->db, array('relation'=>new Relation($this->db, $r->CRSO_RELATION_ID)));
		$this->name = $r->DIMENSION_NAME;
		$this->start_year = $r->START_YEAR;
		$this->end_year = $r->END_YEAR;
		return $this->complete = true;
	}
	/**
	 *
	 */
	function selectOverview($product_id, $crso_id, $year='')
	{
		$product_id = (array) $product_id;
		$sql = "SELECT CPD_ID, NAME as DIMENSION_NAME,
			START_YEAR AS START_YEAR,
			END_YEAR AS END_YEAR,
			END_YEAR AS DISABLE_DIMENSION
			FROM TBL_CRSO_PRODUCT_DIMENSION
			WHERE CRSO_RELATION_ID=?";
		if($product_id)
			$sql .= " AND PRODUCT_ID IN ( ".implode(',',(array) $product_id)." )";
		if($year)
			$sql .= sprintf(" AND START_YEAR <= %s AND (END_YEAR IS NULL OR END_YEAR > %s)", (int) $year, (int) $year);
		$q = $this->db->query($sql);
		$q->bind_values($crso_id);
		$q->execute();
		return $q;
	}
	/**
	 *
	 */
	function select($product_id='',$selectedindex='',$crso_id='', $year='')
	{
		$product_id || $product_id = $this->product->id;
		$crso_id || $crso_id = $this->crso->relation->id;

		$d = currentDate();
		$year || $year == $d->getYear();

		$rv = '';
		$sql = "SELECT CPD_ID, `NAME` as DIMENSION_NAME,
				IF(CPD_ID='".$selectedindex."',' selected=\"selected\"','') as SELECTED
			FROM TBL_CRSO_PRODUCT_DIMENSION
			WHERE PRODUCT_ID = ?
				AND CRSO_RELATION_ID = ?
				AND (END_YEAR IS NULL OR END_YEAR > ?)
						AND START_YEAR <= ?";
		$q = $this->db->query($sql);
		$q->bind_values($product_id,$crso_id, $year, $year);
		$q->execute();
		while($r = $q->fetch())
			$rv .='<option value="'.$r->CPD_ID.'"'.$r->SELECTED.'>'.$r->DIMENSION_NAME.'</option>';
		return $rv;
	}

	function getCrsoProductDimensions($crso_id, $product_id)
	{
		$sql = "SELECT CPD_ID, NAME
			FROM TBL_CRSO_PRODUCT_DIMENSION
			WHERE CRSO_RELATION_ID=?
			AND PRODUCT_ID=?;";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, $product_id);
		$q->execute();
		return $q;
	}

	/**
	 * Allow disabling of product categories, see #2528, #2530
	 */
	function disableDimension($cpd_id) {

		$crso_id = $this->crso->relation->id;
		$product_id = $this->product->id;

		$d = currentDate();
		$year = $d->getYear();
		$month = $d->getMonth();
		$day = $d->getDay();

		if ($month == 1)
			$end_year = $year;
		else
			$end_year = $year+1;

		$this->db->begin();

		# update end_date
		$q = $this->db->query("UPDATE TBL_CRSO_PRODUCT_DIMENSION SET END_YEAR = ? WHERE CPD_ID = ?");
		$q->bind_values($end_year, $cpd_id);
		$q->execute();

		# remove any provisional for this pc/end_year
		$q = $this->db->query("DELETE FROM TBL_PROVISIONAL_DECLARATION WHERE YEAR = ? AND CRSO_RELATION_ID = ? AND PRODUCT_ID = ? AND CPD_ID = ?");
		$q->bind_values($end_year, $crso_id, $product_id, $cpd_id);
		$q->execute();

		$this->db->commit();

		$log = getLog();
		$log->info(sprintf('Disabled dimension [%s] by setting end_year [%s] for crso [%s] and product [%s]', $cpd_id, $end_year, $crso_id, $product_id));

	}

	function isEnabled() {

		if (!$this->end_year)
			return true;

		$d = currentDate();
		$year = $d->getYear();

		return $year < $this->end_year;

	}


	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_CRSO_PRODUCT_DIMENSION';
		$this->objectmap['primary']['id'] = 'CPD_ID';
		$this->objectmap['properties']['product->id'] = 'PRODUCT_ID';
		$this->objectmap['properties']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['name'] = 'NAME';
		$this->objectmap['properties']['start_year'] = 'START_YEAR';
		$this->objectmap['properties']['end_year'] = 'END_YEAR';
		$this->objectmap['inserttype'] = 'REPLACE';
	}
}
?>
