<?php
require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES.'CRSO.php');

/**
 * CRSOVATBlock
 * @author Chris Wesseling <chris.wesseling@xs4all.nl>
 * @package {package}
 * @version $Id$
 */
class CRSOVATBlock extends AbstractObject
{
    var $crso;
    var $vat_no;

    /**
     * bitwise int containing all types blocked. (defined below)
     */
    var $types_blocked;

    function set($crso_id=null, $vat_no=null)
    {
        $crso_id || $crso_id = $this->crso_id;
        $vat_no || $vat_no = $this->vat_no;

        $sql = 'SELECT TYPES_BLOCKED
			FROM TBL_CRSO_VAT_BLOCK
			WHERE CRSO_RELATION_ID=? AND VAT_NO=?';

		$q = $this->db->query($sql);
		$q->bind_values($crso_id, $vat_no);
		$q->execute();

		$r = $q->fetch();
		$this->types_blocked = $r->TYPES_BLOCKED;

		return $this->complete = true;
    }

    function selectOverview($crso_id)
    {
    	$sql = 'SELECT VAT_NO, TYPES_BLOCKED, "" as REMOVE_BLOCK
			FROM TBL_CRSO_VAT_BLOCK WHERE CRSO_RELATION_ID=?
		';
		$q = $this->db->query($sql);
		$q->bind_values($crso_id);
		$q->execute();
		return $q;
    }

    public static function getTypes()
    {
    	return array(
    		CRSO_VAT_BLOCK_TYPE_NEW_PRODUCER=>'Block VAT for new Producers',
			CRSO_VAT_BLOCK_TYPE_NEW_EXPORTER=>'Block VAT for new Exporters',
			CRSO_VAT_BLOCK_TYPE_POP_PRODUCER=>'Block PoPs Producers',
			CRSO_VAT_BLOCK_TYPE_RR_IMPORTER=>'Blocks RR Importers',
			CRSO_VAT_BLOCK_TYPE_POE_PRODUCER=>'Block PoEs Producers'
		);
    }

    function isBlocked($vat_no, $crso_id, $block_type)
    {
    	$sql = "SELECT * FROM TBL_CRSO_VAT_BLOCK
			WHERE VAT_NO=? AND CRSO_RELATION_ID=? AND TYPES_BLOCKED & ?
		";
		$q = $this->db->query($sql);
		$q->bind_values($vat_no, $crso_id, $block_type);
		$q->execute();

		return (bool) ($r = $q->fetch());
    }

    /**#@+
     * @access private
     */
    function _initObjectMaps()
    {
		$this->objectmap['dbtable'] = 'TBL_CRSO_VAT_BLOCK';
		$this->objectmap['primary']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['primary']['vat_no'] = 'VAT_NO';
		$this->objectmap['properties']['types_blocked'] = 'TYPES_BLOCKED';

		$this->objectmap['inserttype'] = 'REPLACE';
    }
    /**#@-*/
}

/**
 * Blocks new producers with this VAT
 */
define('CRSO_VAT_BLOCK_TYPE_NEW_PRODUCER', 1);

/**
 * Blocks new exporters with this VAT
 */
define('CRSO_VAT_BLOCK_TYPE_NEW_EXPORTER', 2);

/**
 * Blocks proof of payments with producers of this vat
 */
define('CRSO_VAT_BLOCK_TYPE_POP_PRODUCER', 4);

/**
 * Blocks refund requests where the importer is this vat
 */
define('CRSO_VAT_BLOCK_TYPE_RR_IMPORTER', 8);

/**
 * Blocks proof of payments where the producer is this vat
 */
define('CRSO_VAT_BLOCK_TYPE_POE_PRODUCER', 16);

?>
