<?php

/**
 * Colors for demo
 * 0x1E1EA9
 * 0x631EA9
 * 0xA91EA9
 * 0xA91E63
 * 0x1E63A9
 * 0x2B2BDA
 * 0x5F5FE3
 * 0xA91E1E
 * 0x1EA9A9
 * 0xE3E35F
 * 0xDADA2B
 * 0xA9631E
 * 0x1EA963
 * 0x1EA91E
 * 0x63A91E
 * 0xA9A91E
 */

require_once(IWARE.'phpchartdir.php');

/**
 *  DashboardChartHandler
 */
class DashboardChartHandler
{
	private $settings;

	function __construct($handle, $userSettings)
	{
		debug($userSettings);

		if (!method_exists($this, $handle))
			throw new DashboardChartHandlerNotExistException();

		$this->$handle($userSettings);
	}

	private function printImage($chart)
	{
		header("Content-type: image/png");
		echo $chart->makeChart2(PNG);
	}

	function testChart()
	{
		# The data for the pie chart
		$data = array(25, 18, 15, 12, 8, 30, 35);

		# The labels for the pie chart
		$labels = array("Labor", "Licenses", "Taxes", "Legal", "Insurance", "Facilities",
		    "Production");

		# Create a PieChart object of size 360 x 300 pixels
		$c = new DashboardPieChart();

		# Set the pie data and the pie labels
		$c->setData($data, $labels);

		return $this->printImage($c);
	}


	function invoiceVSoutstanding($userSettings)
	{
		# The data for the pie chart
		$data = array((100 * $userSettings['crso']['id']) * $userSettings['product']['id'], (100 * $userSettings['crso']['id'] - 73) - $userSettings['product']['id'] * 5);

		# The labels for the pie chart
		$labels = array("Invoiced", "Outstanding");

		$c = new DashboardPieChart();

		# Set the pie data and the pie labels
		$c->setData($data, $labels);

		# output the chart
		return $this->printImage($c);

	}

	function documentViews()
	{
		$data = array(65, 35);
		$labels = array("Viewed", "Non-viewed");

		$c = new DashboardPieChart();
		$c->setData($data, $labels);

		return $this->printImage($c);
	}

	function refundChart()
	{
		$c = new DashboardXYChart();

		$labels = array('Confirmed PoP', 'Confirmed PoE', 'Next RR export');
		$c->xAxis->setLabels($labels);
		$c->yAxis->setTitle("Total number");

		$data = array(100, 150, 40);
		$c->setData($data);

		return $this->printImage($c);
	}

	function declarationQuantityChart()
	{
		$c = new DashboardXYChart();

		$labels = array("December", "January", " February", "March", "April");
		$c->xAxis->setLabels($labels);

		$actual = array(1000, 1250, 2450, 1470, 670);
		$provisional = array(850, 1560, 1790, 2110, 1230);

		$layer = $c->addBarLayer2(Side, 4);
		$layer->addDataSet($actual, 0x1E1EA9, "Actual");
		$layer->addDataSet($provisional, 0x631EA9, "Provisional");
		$layer->setBorderColor(Transparent, softLighting(Top));

		# Add a legend box at (50, 25) using horizontal layout. Use 10pts Arial Bold as font,
		# with transparent background.
		$legendObj = $c->addLegend(50, 5, false, "arialbd.ttf", 10);
		$legendObj->setBackground(Transparent);

		return $this->printImage($c);

	 }
}

/**
 * DashboardPieChart
 */
class DashboardPieChart extends PieChart
{
	function __construct($width=290, $height=240)
	{
		parent::__construct($width, $height);

		# Set the center of the pie at (180, 175) and the radius to 100 pixels
		$this->setPieSize(160, 135, 90);

		# The depths for the sectors
		$depths = array(10, 15);

		# Draw the pie in 3D with variable 3D depths
		$this->set3D2($depths);

		#set the colors
		$this->setColors2(DataColor, array(0x1E1EA9, 0x63A91E));

		# Set the start angle to 225 degrees may improve layout when the depths of the sector
		# are sorted in descending order, because it ensures the tallest sector is at the
		# back.
		$this->setStartAngle(0);
	}
}

/**
 * DashboardXYChart
 */

class DashboardXYChart extends XYChart
{
	function __construct($width=290, $height=240)
	{
		parent::__construct($width, $height);

		# Set the plotarea at (60, 40) and of size 500 x 280 pixels. Use a vertical gradient
		# color from light blue (eeeeff) to deep blue (0000cc) as background. Set border and
		# grid lines to white (ffffff).
		$this->setPlotArea(40, 5, $width-40, $height-30,
			$this->linearGradientColor(60, 40, 60, 280, 0xeeeeff, 0x0000cc), -1, 0xffffff, 0xffffff);

		$this->setColors2(DataColor, array(0x1E1EA9, 0x63A91E, 0xA91EA9, 0xA91E63));
	}

	function setData($data)
	{
		$barLayerObj = $this->addBarLayer3($data);
		$barLayerObj->setBorderColor(Transparent, softLighting(Left));
	}
}

class DashboardChartHandlerNotExistException extends IWException {};



?>