<?php
require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'CRSOProduct.php');
/**
 * @package ELC
 * @subpackage Declaration
 * @author Marten Koedam <marten@basesoft.com>
 * @since 2006-02-07
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class Declaration extends AbstractObject
{
	/**
	 * @var int $id
	 */
	var $id;
	/**
	 * @var CRSO $crso
	 */
	var $crso;
	/**
	 * @var Participant $participant
	 */
	var $participant;
	/**
	 * @var ELCUser $user
	 */
	var $user;
	var $product;
	/**
	 * @var int $month
	 */
	var $month;
	/**
	 * @var int $year
	 */
	var $year;
	/**
	 * @var Datetime $created
	 */
	var $created;
	/**
	 * @var Timestamp $last_update
	 */
	var $last_update;
	/**
	 * @var int $is_exported
	 */
	var $is_exported;
	/**
	 * @var int $has_critical_changes
	 */
	var $has_critical_changes;
	/**
	 * @var Array $lines
	 */
	var $lines = array();
	/**
	 * Bitwise status of all lines in a Declaration
	 * @var int $aggregated_status
	 */
	var $aggregated_status = DECLARATION_OPEN;
	/**
	 *
	 */
	var $document;
	/**
	 * @var Declaration $parent_declaration
	 */
	var $parent_declaration;
	/**
	 * Sets a declaration for given ID
	 * @param int $id
	 */
	function set($id='')
	{
		$id || $id = $this->id;
		$sql = "SELECT CRSO_RELATION_ID, PARTICIPANT_RELATION_ID,
				USER_ID, MONTH, YEAR, D.CREATED,
				D.LAST_UPDATE, IS_EXPORTED,
				BIT_OR(DL.STATUS) as AGGREGATED_STATUS,
				DOCUMENT_ID, PARENT_DECLARATION_ID, PRODUCT_ID
			FROM TBL_DECLARATION as D
			LEFT JOIN TBL_DECLARATION_LINE as DL
				USING(DECLARATION_ID)
			WHERE D.DECLARATION_ID = ?
			GROUP BY D.DECLARATION_ID";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		require_once(ELC_CLASSES.'CRSO.php');
		require_once(ELC_CLASSES.'Document.php');
		require_once(ELC_CLASSES.'Participant.php');
		require_once(ELC_CLASSES.'User.php');
		require_once(ELC_CLASSES.'Product.php');

		$this->id = $id;
		$this->crso = new CRSO($this->db, array('relation'=>new Relation($this->db,$r->CRSO_RELATION_ID)));
		$this->participant = new Participant($this->db, array('relation'=>new Relation($this->db,$r->PARTICIPANT_RELATION_ID)));
		$this->user = new ELCUser($this->db,$r->USER_ID);
		$this->product = new Product($this->db, $r->PRODUCT_ID);
		$this->document = new ELCDocument($this->db,$r->DOCUMENT_ID);
		$this->month = $r->MONTH;
		$this->year = $r->YEAR;
		$this->created = $r->CREATED;
		$this->last_update = $r->LAST_UPDATE;
		$this->is_exported = $r->IS_EXPORTED;
		$this->aggregated_status = $r->AGGREGATED_STATUS;
		$this->parent_declaration = ($r->PARENT_DECLARATION_ID) ? new Declaration($this->db, $r->PARENT_DECLARATION_ID) : null;

		// Set the lines
		$this->lines = array();
		$dl = new DeclarationLine($this->db);
		$q = $dl->getLines($id);
		while($r = $q->fetch())
			$this->lines[$r->LINE_INDEX] = new DeclarationLine($this->db,array('declaration'=>$this,'line_index'=>$r->LINE_INDEX));
		return $this->complete = true;
	}
	/**
	 * Set the creation date
	 */
	function insert()
	{
		$this->created = date('Y-m-d H:i:s'); // FIXME
		unset($this->last_update);
		return parent::insert();
	}
	/**
	 * Gets the lines with the data to be displayed in the Form interface for the Declaration
	 */
	function getFormLines($id='')
	{
		$id && $this->set($id);
		$lines = array();
		foreach($this->lines as $index => $line)
			$lines[] = $line->formatLine();
		return $lines;
	}
	/**
	 * @param int $trust_id
	 * @return ResultSet $q
	 */
	function getByTrust($trust_id='')
	{
		$trust_id || $trust_id = $this->trust->id;
		$sql = "SELECT DECLARATION_ID
			FROM TBL_DECLARATION_LINE
			WHERE TRUST_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($trust_id);
		$q->execute();
		return $q;
	}

	/**
	 * Copies a declaration to a credit, used for resubmission and late declarations
	 * Always a new one
	 * @return int $creditID
	 */
	function &copyToCredit($components=false)
	{
		$copy = clone $this;
		$copy->id = null;
		$copy->is_exported = null;
		$copy->export_date = null;
		$copy->document = null;
		$copy->parent_declaration =& $this;
		$copy->user = new ELCUser($this->db,$_SESSION['session_user_id']);

		//first we check if the credit for the month exists and is open
		if($copy->_hasUnexportedCredit()) {
			$copy = false;
			return $copy;
		}

		$copy->insert();
		$copy->insertLines(DECLARATION_CREDIT);

		if($components)
		{
			require_once(ELC_CLASSES.'DeclarationComponent.php');
			$dc = new DeclarationComponent($this->db);
			$dc->copyLines($this,$copy,true); // make the values negative
		}

		return $copy;
	}

	/**
	 * Check whether we have an unexported credit/resubmit for this month already
	 * @return bool
	 */
	function _hasUnexportedCredit()
	{
		$sql = "
				SELECT (BIT_OR(DL.STATUS)) as DECLARATION_STATUS
				FROM TBL_DECLARATION as D
				LEFT JOIN TBL_DECLARATION_LINE as DL USING(DECLARATION_ID)
				WHERE D.PARENT_DECLARATION_ID=?
				AND ISNULL(IS_EXPORTED)
				GROUP BY D.DECLARATION_ID
				HAVING DECLARATION_STATUS = ".DECLARATION_CREDIT."
				";
		$q = $this->db->query($sql);
		$q->bind_values($this->parent_declaration->id);
		$q->execute();
		if($r = $q->fetch())
			return true;
		return false;
	}

	/**
	 * Insert the userlines ($_POST) into a new declartion.
	 *
	 * This function always creates a new Credit, since the only time new Declarations are
	 * inserted through user input, is at a LATE or RESUBMIT
	 *
	 * @param Array $userlines
	 * @return int $debitID
	 */
	function insertDeclarationLines($userlines,$components=false,$manual=false)
	{
		require_once(ELC_CLASSES.'User.php');
		if(!$manual)
		{
			$credit = $this->copyToCredit($components);
			if($credit === false) //we already have a credit!
			{
				alert(sprintf("Declaration [%s] already has an unexported credit.", $this->id));
				return false;
			}
		}
		else
			$credit = false;
		$that = clone $this;
		$that->id = null;
		$that->is_exported = null;
		$that->export_date = null;
		$that->document = null;
		$that->parent_declaration =& $this;
		$that->user = new ELCUser($this->db,$_SESSION['session_user_id']);
		$that->insert();
		$that->insertLines();
		$that->updateDeclarationLines($userlines,$credit);

		if($components)
		{
			require_once(ELC_CLASSES.'DeclarationComponent.php');
			$dc = new DeclarationComponent($this->db);
			$dc->copyLines($this,$that);
			$dc->updateDeclarationComponents($that,$components);
		}

		return true;
	}
	/**
	 * Update the lines in a declaration by the user input
	 *
	 * @param Array $userlines<line_index> => array<column> => value
	 */
	function updateDeclarationLines($userlines,$credit=false)
	{
		if(!$credit && $creditID = $this->getCreditID()) // Check to see wether or not a Credit has been created for this Declaration
			$credit = new Declaration($this->db,$creditID,true);

		$quantitiesChanged = false;
		foreach($this->lines as $index => $line)
		{
			$this->lines[$index]->set();

			if(isset($userlines[$index-1]['weight'])) {
				$this->lines[$index]->weight = ($userlines[$index-1]['quantity']==0)?0:$userlines[$index-1]['weight'];
			} elseif(isset($userlines[$index-1]['weight_dec'])) {
				$this->lines[$index]->weight_dec = ($userlines[$index-1]['quantity']==0)?0:$userlines[$index-1]['weight_dec'];
			}
			// Get the original quantity and status
			list($orgquantity,$orgstatus,$orgweight,$orgweightdec) = $line->getLastExportedQuantityAndStatus();

			// check for empty lines
			if (!is_numeric($userlines[$index-1]['quantity']) && !is_null($line->status) && !($line->status & DECLARATION_OPEN))
				trigger_error('Declaration::updateDeclarationLines() - Cannot set empty quantity for non-open declaration line ', E_USER_ERROR);

			// If the quantity is different from the original, or is a forfeit, update both
			if(($userlines[$index-1]['quantity']!=$orgquantity)||$orgstatus==DECLARATION_FORFEIT)
			{
				$quantitiesChanged = true;
				info('Adding quantity ['.$userlines[$index-1]['quantity'].'!='.$orgquantity.'] for ['.$index.']');
				$this->lines[$index]->quantity = !is_numeric($userlines[$index-1]['quantity'])?DB_NULL:$userlines[$index-1]['quantity'];
				$this->lines[$index]->status = $line->nextStatus();
				if($credit && isset($credit->lines[$index])) // If a credit is available, set the quantity to the negative of the orginal quantity
				{
					info('Adding Credit quantity ['.(0-$orgquantity).'] for ['.$index.']');
					$credit->lines[$index]->set();
					$credit->lines[$index]->quantity = !is_numeric($userlines[$index-1]['quantity'])?DB_NULL:(0-$orgquantity);
					if(isset($userlines[$index-1]['weight'])&&is_numeric($userlines[$index-1]['weight'])&&is_numeric($orgweight)) {
						$credit->lines[$index]->weight = (0-$orgweight);
					} elseif(isset($userlines[$index-1]['weight_dec'])&&is_numeric($userlines[$index-1]['weight_dec'])&&is_numeric($orgweightdec)) {
						$credit->lines[$index]->weight_dec = (0-$orgweightdec);
					}

				}
			}
			else
			{
				$this->lines[$index]->quantity = DB_NULL;
				$this->lines[$index]->status = $orgstatus;

				$hasnoweight = (!isset($userlines[$index-1]['weight']) || ($userlines[$index-1]['weight']==$orgweight));
				$hasnoweightdec = (!isset($userlines[$index-1]['weight_dec']) || ($userlines[$index-1]['weight_dec']==$orgweightdec));
				// Check if the weight is not set or if it is still the same as previously.
				if($hasnoweight && $hasnoweightdec) {
					$this->lines[$index]->weight = DB_NULL;
					$this->lines[$index]->weight_dec = DB_NULL;
				// Check if weight is set and also set the quantity in case the weight has changed.
				} else {
					$this->lines[$index]->quantity = $orgquantity;
					$this->lines[$index]->status = $line->nextStatus();
				}


				// If a credit is available, set the quantity back to NULL
				if($credit && isset($credit->lines[$index]))
				{
					$credit->lines[$index]->set();
					$credit->lines[$index]->quantity = DB_NULL;
					// If the weight is set and it is the same as previously, nullify the value
					if($hasnoweight && $hasnoweightdec) {
						$credit->lines[$index]->weight = DB_NULL;
						$credit->lines[$index]->weight_dec = DB_NULL;
					} else {
						// The weight is set and is not the same as previous weight
						// This means we also have to credit the quantity
						$credit->lines[$index]->weight = (0-$orgweight);
						$credit->lines[$index]->weight_dec = (0-$orgweightdec);
						$credit->lines[$index]->quantity = (0-$orgquantity);
					}
				}
			}
		}
		if(sizeof($userlines)>sizeof($this->lines))
		{
			/**
			 * @todo THIS CHECK DOESNT TAKE INTO ACCOUNT HIDDEN LINE ELEMENT
			 */
			info('Declaration::updateDeclarationLines() - Adding extra lines...');
			// Loop and add extra lines, only to the current declaration
			$startIndex = sizeof($this->lines);
			$j = $startIndex+1;
			for($i=$startIndex;$i<sizeof($userlines);$i++)
			{
				if(!is_numeric($userlines[$i]['quantity']) )
					continue;
				$quantitiesChanged = true;
				$this->lines[$j] = new DeclarationLine($this->db,$userlines[$i]);
				if($this->lines[$j]->brand)
					$this->lines[$j]->brand = new Brand($this->db,$this->lines[$j]->brand['id']);

				if($this->lines[$j]->product_category && $this->lines[$j]->product_category['id']) {

					// check product category expiration
					$pc = new ProductCategory($this->db, $this->lines[$j]->product_category['id']);
					$pc->set();
					if ($this->year < $pc->start_year) {
						throw new Exception(sprintf('ProductCategory [%s] start_year [%s] not valid for Declaration [%s] with year [%s]', $pc->id, $pc->start_year, $this->id, $this->year));
					}
					if ($pc->end_year && $this->year >= $pc->end_year) {
						throw new Exception(sprintf('ProductCategory [%s] end_year [%s] not valid for Declaration [%s] with year [%s]', $pc->id, $pc->end_year, $this->id, $this->year));
					}

					$this->lines[$j]->product_category = $pc;
				}

				if($this->lines[$j]->dimension && $this->lines[$j]->dimension['id']) {
					$cpd = new CRSOProductDimension($this->db,$this->lines[$j]->dimension['id']);
					$cpd->set();

					// check dimension expiration
					if ($this->year < $cpd->start_year) {
						throw new Exception(sprintf('CRSOProductDimension [%s] start_year [%s] not valid for Declaration [%s] with year [%s]', $cpd->id, $cpd->start_year, $this->id, $this->year));
					}
					if ($cpd->end_year && $this->year >= $cpd->end_year) {
						throw new Exception(sprintf('CRSOProductDimension [%s] end_year [%s] not valid for Declaration [%s] with year [%s]', $cpd->id, $cpd->end_year, $this->id, $this->year));
					}

					$this->lines[$j]->dimension = $cpd;
				}

				if($this->lines[$j]->trust)
					$this->lines[$j]->trust = new Trust($this->db,$this->lines[$j]->trust);
				$this->lines[$j]->declaration =& $this;
				$this->lines[$j]->status = DECLARATION_DECLARED;
				$j++;
			}
		}

		$this->has_critical_changes = DECLARATION_HAS_CRITICAL_CHANGES;
		if($credit) {
			$credit->has_critical_changes = DECLARATION_HAS_CRITICAL_CHANGES;
		}

		// We are dealing with a resubmit; forfeits will set the quantitiesChanged flag always to true.
		if(!$quantitiesChanged && $credit) {
			// We keep a flag on both the declaration ($this) and on the credit ($credit).
			// This flag is checked on export to accountant:
			//   in case its set and the country setting to exclude weight-only changes
			//   no credit/invoice IDs are attached to this Declaration/Credit.
			$this->has_critical_changes = DECLARATION_HAS_NO_CRITICAL_CHANGES;
			$credit->has_critical_changes = DECLARATION_HAS_NO_CRITICAL_CHANGES;
			$credit->update();
		}

		$this->update(); // Update the current declaration (last_update field)
		$this->replaceLines();
		$this->executeResubmitDeviationTrigger();
		if($credit) { // If a credit is available, replace its lines
			$credit->replaceLines();
		}
	}

	function executeResubmitDeviationTrigger() {

		$d = new Declaration($this->db, $this->id, true);

		info(sprintf('Declaration::executeResubmitDeviationTrigger() - Declaration [%s]', $d->id));
		info(sprintf(' - Aggregated status is %s', $d->aggregated_status));
		// Check if we are dealing with a resubmit.
		if(($d->aggregated_status & DECLARATION_RESUBMIT)!=DECLARATION_RESUBMIT) {
			info(' - Declaration is not a Resubmit');
			return;
		}

		require_once(ELC_CLASSES.'CRSOProduct.php');

		$cp = new CRSOProduct($this->db);
		$cp->set($d->crso->relation->id, $d->product->id);
		// No resubmit trigger set, bail out.
		if(!$cp->resubmit_deviation_trigger_percentage) {
			info(sprintf(' - Bailing Out: No trigger percentage set for %s with product %s', $d->crso->relation->id, $d->product->id));
			return;
		}

		// Check the total of this declaration with the total of it's parent.
		$sql = "SELECT D.DECLARATION_ID, SUM(DL.QUANTITY) as NewQuantity, SUM(DL2.QUANTITY) as OriginalQuantity
			FROM TBL_DECLARATION as D
			INNER JOIN TBL_DECLARATION_LINE as DL
				ON D.DECLARATION_ID = DL.DECLARATION_ID
			INNER JOIN TBL_DECLARATION as D2
				ON D.PARENT_DECLARATION_ID = D2.DECLARATION_ID
			INNER JOIN TBL_DECLARATION_LINE as DL2
				ON D2.DECLARATION_ID = DL2.DECLARATION_ID
			WHERE D.DECLARATION_ID = ?
			GROUP BY D.DECLARATION_ID
		";
		$q = $this->db->query($sql);
		$q->bind_values($d->id);
		$q->execute();

		if(!($r = $q->fetch())) {
			return;
		}

		info(sprintf(' - Comparing quantities: %s - %s', $r->NewQuantity, $r->OriginalQuantity));

		// Now check the quantities.
		$deviation = 0;
		// If one of the two is zero, the deviation is a 100%
		if($r->NewQuantity==0 XOR $r->OriginalQuantity==0) {
			$deviation = 100;
		} elseif($r->NewQuantity > $r->OriginalQuantity) {
			$deviation = ceil(($r->NewQuantity/$r->OriginalQuantity)*100)-100;
		} elseif($r->NewQuantity < $r->OriginalQuantity) {
			$deviation = ceil(($r->OriginalQuantity/$r->NewQuantity)*100)-100;
		}
		// No issue, no need to inform the CRSO.
		if($deviation < $cp->resubmit_deviation_trigger_percentage) {
			info(sprintf(' - No trigger happening on deviation of "%s" with trigger set at "%s"', $deviation, $cp->resubmit_deviation_trigger_percentage));
			return;
		}
		$d->crso->set();
		$d->participant->relation->set();
		info(sprintf(' - Sending notification to CRSO for deviation "%s" with trigger set at "%s"', $deviation, $cp->resubmit_deviation_trigger_percentage));

		$langIf = IWRegistry::registry('langIf');
		// Send a notification informing the CRSO of the difference.
		require_once(ELC_CLASSES.'Notification.php');
		Notification::send($this->db,
			array('subject'=>$langIf->getText('Resubmission deviation notification triggered', null, array(), $d->crso->default_language_id),
				'sender'=>$d->participant->relation,
				'message'=>$langIf->getText(
					'The Declaration for {MONTH} in {YEAR} was resubmitted by {PARTICIPANT}, the original quantity was {ORIGINAL} with a resubmission value of {NEWQUANTITY}',
					null,
					array(
						'MONTH'=>strftime('%B', mktime(0, 0, 0, $d->month, 1)),
						'YEAR'=>$d->year,
						'PARTICIPANT'=>$d->participant->relation->name,
						'ORIGINAL'=>$r->OriginalQuantity,
						'NEWQUANTITY'=>$r->NewQuantity
					),
					$d->crso->default_language_id
				)
			),
			array(array('recipient'=>$d->crso->relation,
				'callback'=>sprintf("go('/%sscripts/elc/crso/InterfaceCRSODeclarationStatus.php?YEAR=%s&PARTICIPANT_RELATION_ID=%s&MONTH=%s')",
				SITE_PATH, $d->year, $d->participant->relation->id, $d->month
			)))
		);
	}

	/**
	 * Gets the credit ID for a certain YEAR and MONTH
	 * @return int $id DECLARATION_ID
	 */
	function getCreditID()
	{
		$sql = "SELECT d.DECLARATION_ID
			FROM TBL_DECLARATION as d
			INNER JOIN TBL_DECLARATION_LINE AS dl
				ON d.DECLARATION_ID = dl.DECLARATION_ID
			WHERE (ISNULL(d.IS_EXPORTED) OR d.IS_EXPORTED = 0)
				AND d.PARENT_DECLARATION_ID=?
			GROUP BY d.DECLARATION_ID
			HAVING BIT_OR(dl.STATUS)&?
		";

		$q = $this->db->query($sql);
		$q->bind_values(($this->parent_declaration?$this->parent_declaration->id:$this->id),
			DECLARATION_CREDIT);
		$q->execute();

		if(!($r = $q->fetch()))
			return null;
		return $r->DECLARATION_ID;
	}

	/**
	 * Adds Invoice numbers to ALL PENDING NON ZERO TOTAL declarations
	 *
	 * ZERO TOTAL happens if either the total quantity = 0 or if the fee for the month = 0
	 *
	 * @param int $crso_id
	 */
	function addDebitNumbersToPendingExport($crso_id, &$log, $products)
	{
		$date =& currentDate();
		$year = $date->getYear();
		$log->info(sprintf('Adding declaration debit for CRSO [%s]', $crso_id));
		//get startID
		$sql = "SELECT @debitno:=IFNULL(MAX(DECLARATION_DEBIT_ID), 0)
				FROM TBL_DECLARATION_DEBIT
				WHERE CRSO_RELATION_ID = ? AND `YEAR` = ?";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$year);
		$q->execute();

		$sql = "
					SELECT d.DECLARATION_ID, d.CRSO_RELATION_ID, d.PARTICIPANT_RELATION_ID, d.`MONTH`,d.`YEAR`,
						IFNULL(f_elcProductFee(d.PRODUCT_ID, d.CRSO_RELATION_ID, dl.PRODUCT_CATEGORY_ID, CONCAT(d.YEAR, '-', d.MONTH, '-01 00:00:00')),'-') as FEE
					FROM TBL_DECLARATION AS d
					INNER JOIN TBL_DECLARATION_LINE as dl
						ON dl.DECLARATION_ID = d.DECLARATION_ID
					LEFT JOIN TBL_DECLARATION_SET_LINE AS DSL
						ON DSL.DECLARATION_ID = d.DECLARATION_ID
					LEFT JOIN TBL_DECLARATION_DEBIT as db
						ON DSL.DECLARATION_SET_ID = db.DECLARATION_SET_ID
					WHERE d.CRSO_RELATION_ID = ?
						AND d.IS_EXPORTED = ?
						AND ISNULL(db.DECLARATION_DEBIT_ID)
						AND d.PRODUCT_ID IN (".join($products, ',').")
					GROUP BY dl.DECLARATION_ID, dl.PRODUCT_CATEGORY_ID
					HAVING SUM(dl.QUANTITY) > 0
					ORDER BY d.PARTICIPANT_RELATION_ID, d.`MONTH`
				";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, DECLARATION_EXPORT_PENDING);
		$q->execute();
		$last_year = $last_set_id = $last_month = $last_participant = false;

		// An array with all the previously INSERTED declaration ids.
		$declaration_ids = array();

		while($r = $q->fetch())
		{
			debug('Declaration::addDebitNumbersToPendingExport() - FEE = ['.$r->FEE.']');
			// We already added this declaration to the set, so there was a line with a fee and a quantity.
			if(in_array($r->DECLARATION_ID, $declaration_ids))
				continue;

			if(($r->FEE == '-')||($r->FEE=='0.0000000')) // See ticket #1279 and #2006
				continue;

			if(($last_participant != $r->PARTICIPANT_RELATION_ID) || ($last_month != $r->MONTH) || ($last_year != $r->YEAR))
			{
				//insert
				$sql = "INSERT INTO TBL_DECLARATION_SET (CREATED) VALUES(NOW())";
				$q1 = $this->db->query($sql);
				$q1->execute();
				$last_set_id = $q1->lastid();

				$sql = "INSERT INTO TBL_DECLARATION_DEBIT
					(CRSO_RELATION_ID,DECLARATION_SET_ID,DECLARATION_DEBIT_ID,`YEAR`)
					SELECT ".$crso_id.", ".$last_set_id.", @debitno:=@debitno+1 as DECLARATION_DEBIT_ID, ".$year;
				$q1 = $this->db->query($sql);
				$q1->execute();

				$last_participant = $r->PARTICIPANT_RELATION_ID;
				$last_month = $r->MONTH;
				$last_year = $r->YEAR;
			}
			$sql = "INSERT INTO TBL_DECLARATION_SET_LINE (DECLARATION_SET_ID, DECLARATION_ID) VALUES(?,?)";
			$q1 = $this->db->query($sql);
			$q1->bind_values($last_set_id,$r->DECLARATION_ID);
			$q1->execute();

			// Mark the declaration as used.
			$declaration_ids[] = $r->DECLARATION_ID;
		}
	}

	/**
	 * Adds Invoice Credit numbers to ALL PENDING SMALLER THAN ZERO TOTAL declarations
	 * @param int $crso_id
	 */
	function addCreditNumbersToPendingExport($crso_id, &$log, $products)
	{
		$date =& currentDate();
		$year = $date->getYear();
		$log->info(sprintf('Adding declaration debit for CRSO [%s]', $crso_id));
		//get startID
		$sql = "SELECT @creditno:=IFNULL(MAX(DECLARATION_CREDIT_ID), 0)
				FROM TBL_DECLARATION_CREDIT
				WHERE CRSO_RELATION_ID = ? AND `YEAR` = ?";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$year);
		$q->execute();

		$sql = "
					SELECT d.DECLARATION_ID, d.CRSO_RELATION_ID, d.PARTICIPANT_RELATION_ID, d.`MONTH`,d.`YEAR`,
						IFNULL(f_elcProductFee(d.PRODUCT_ID, d.CRSO_RELATION_ID, dl.PRODUCT_CATEGORY_ID, CONCAT(d.YEAR, '-', d.MONTH, '-01 00:00:00')),'-') as FEE
					FROM TBL_DECLARATION AS d
					INNER JOIN TBL_DECLARATION_LINE as dl
						ON dl.DECLARATION_ID = d.DECLARATION_ID
					LEFT JOIN TBL_DECLARATION_SET_LINE AS DSL
						ON DSL.DECLARATION_ID = d.DECLARATION_ID
					LEFT JOIN TBL_DECLARATION_CREDIT as dc
						ON DSL.DECLARATION_SET_ID = dc.DECLARATION_SET_ID
					WHERE d.CRSO_RELATION_ID = ?
						AND d.IS_EXPORTED = ?
						AND ISNULL(dc.DECLARATION_CREDIT_ID)
						AND d.PRODUCT_ID IN (".join($products, ',').")
					GROUP BY dl.DECLARATION_ID, dl.PRODUCT_CATEGORY_ID
					HAVING SUM(dl.QUANTITY) < 0
					ORDER BY d.PARTICIPANT_RELATION_ID, d.`MONTH`
				";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, DECLARATION_EXPORT_PENDING);
		$q->execute();
		$last_year = $last_set_id = $last_month = $last_participant = false;

		// An array with all the previously INSERTED declaration ids.
		$declaration_ids = array();

		while($r = $q->fetch())
		{
			debug('Declaration::addCreditNumbersToPendingExport() - FEE = ['.$r->FEE.']');

			// We already added this declaration to the set, so there was a line with a fee and a quantity.
			if(in_array($r->DECLARATION_ID, $declaration_ids))
				continue;

			if(($r->FEE == '-')||($r->FEE=='0.0000000')) // See ticket #1279 and #2006
				continue;

			if(($last_participant != $r->PARTICIPANT_RELATION_ID) || ($last_month != $r->MONTH) || ($last_year != $r->YEAR))
			{
				//insert
				$sql = "INSERT INTO TBL_DECLARATION_SET (CREATED) VALUES(NOW())";
				$q1 = $this->db->query($sql);
				$q1->execute();
				$last_set_id = $q1->lastid();

				$sql = "INSERT INTO TBL_DECLARATION_CREDIT
					(CRSO_RELATION_ID,DECLARATION_SET_ID,DECLARATION_CREDIT_ID,`YEAR`)
					SELECT ".$crso_id.", ".$last_set_id.", @creditno:=@creditno+1 as DECLARATION_CREDIT_ID, ".$year;
				$q1 = $this->db->query($sql);
				$q1->execute();

				$last_participant = $r->PARTICIPANT_RELATION_ID;
				$last_month = $r->MONTH;
				$last_year = $r->YEAR;
			}
			$sql = "INSERT INTO TBL_DECLARATION_SET_LINE (DECLARATION_SET_ID, DECLARATION_ID) VALUES(?,?)";
			$q1 = $this->db->query($sql);
			$q1->bind_values($last_set_id,$r->DECLARATION_ID);
			$q1->execute();

			// Mark the declaration as used.
			$declaration_ids[] = $r->DECLARATION_ID;
		}
	}
	/**#@+
	 * @access private
	 */
	/**
	 * @param string $type CREDIT|DEBIT
	 * @param int $crso_id CRSO_RELATION_ID
	 * @return int $type_ID
	 */
	function _getNewID($type,$crso_id,$year)
	{
		$sql = "SELECT MAX(DECLARATION_".$type."_ID) as NEWID
			FROM TBL_DECLARATION_".$type."
			WHERE CRSO_RELATION_ID = ?
				AND `YEAR` = ?";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$year);
		$q->execute();
		if(!($r = $q->fetch()))
			return 1;
		return (int) $r->NEWID+1;
	}
	/**#@-*/

	function getInvoiceID($id=null)
	{
		$id || $id = $this->id;

		$sql = "SELECT IF(ISNULL(dd.DECLARATION_DEBIT_ID),
						f_elcBuildInvoiceNoForInterfaces(D.CRSO_RELATION_ID,dc.`YEAR`,dc.DECLARATION_CREDIT_ID,true,false),
						f_elcBuildInvoiceNoForInterfaces(D.CRSO_RELATION_ID,dd.`YEAR`,dd.DECLARATION_DEBIT_ID,false,false)
				) as INVOICE_NR
			FROM TBL_DECLARATION as D
			LEFT JOIN TBL_DECLARATION_SET_LINE as dsl
				ON D.DECLARATION_ID = dsl.DECLARATION_ID
			LEFT JOIN TBL_DECLARATION_CREDIT dc ON dsl.DECLARATION_SET_ID=dc.DECLARATION_SET_ID
			LEFT JOIN TBL_DECLARATION_DEBIT dd ON dsl.DECLARATION_SET_ID=dd.DECLARATION_SET_ID
			WHERE D.DECLARATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		return $r->INVOICE_NR;
	}

	/**
	 * Get the Debit ID of a declaration
	 * @return int DECLARATION_DEBIT_ID
	 */
	function getDebitID()
	{
		$sql = "SELECT DECLARATION_DEBIT_ID
			FROM TBL_DECLARATION_DEBIT DD
			INNER JOIN TBL_DECLARATION_SET_LINE AS DSL
			ON DD.DECLARATION_SET_ID = DSL.DECLARATION_SET_ID
			WHERE DSL.DECLARATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		return $r->DECLARATION_DEBIT_ID;
	}

	function getByDebitID($debit_id, $crso_id=null,$year)
	{
		return $this->getByCreditOrDebitID($debit_id,$crso_id,'DEBIT',$year);
	}

	function getByCreditID($credit_id,$crso_id=null,$year)
	{
		return $this->getByCreditOrDebitID($credit_id,$crso_id,'CREDIT',$year);
	}

	function getByCreditOrDebitID($id,$crso_id,$type,$year)
	{
		$crso_id || $crso_id = $this->crso->relation->id;

		if (!is_numeric($crso_id))
			trigger_error('Declaration::getByDebitID() - CRSO is not numeric', E_USER_ERROR);

		$sql = "SELECT DSL.DECLARATION_ID
			FROM TBL_DECLARATION_".$type." as DT
			INNER JOIN TBL_DECLARATION_SET_LINE AS DSL
				ON DT.DECLARATION_SET_ID = DSL.DECLARATION_SET_ID
			WHERE DECLARATION_".$type."_ID = ?
				AND CRSO_RELATION_ID=?
				AND `YEAR` = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id, $crso_id,$year);
		$q->execute();
		$rv = array();
		while($r = $q->fetch())
			$rv[] = $r->DECLARATION_ID;
		return count($rv) ? $rv : false;
	}

	/**
	 * Checks wether or not the current Declaration is a credit
	 * @return bool
	 */
	function isCredit()
	{
		$sql = "SELECT *
			FROM TBL_DECLARATION d
			INNER JOIN TBL_DECLARATION_LINE dl
				ON d.DECLARATION_ID = dl.DECLARATION_ID
			WHERE d.DECLARATION_ID = ?
			GROUP BY d.DECLARATION_ID
			HAVING BIT_OR(dl.STATUS) = ?
		";
		$q = $this->db->query($sql);
		$q->bind_values($this->id, DECLARATION_CREDIT);
		$q->execute();
		return (bool) ($r = $q->fetch());
	}
	/**
	 * Checks if the aggregated status of the Declaration is a Forfeit
	 * @param int $id DECLARATION_ID
	 * @return bool
	 */
	function isForfeit($id='')
	{
		$id && $this->set($id);
		if(($this->aggregated_status & DECLARATION_FORFEIT)==DECLARATION_FORFEIT)
			return true;
		return false;
	}
	/**
	 * Checks if the aggregated status of the Declaration is a Late
	 * @param int $id DECLARATION_ID
	 * @return bool
	 */
	function isLate($id='')
	{
		$id && $this->set($id);
		if(($this->aggregated_status & DECLARATION_LATE)==DECLARATION_LATE)
			return true;
		return false;
	}
	/**
	 * Verifies that the Declaration is the newest Declaration of this month.
	 * @param int $id DECLARATION_ID
	 * @return bool
	 */
	function isNewestForMonth($id='')
	{
		$id && $this->set($id);

		if(!$this->parent_declaration)
		{
			$sql = "SELECT DECLARATION_ID
				FROM TBL_DECLARATION
				WHERE PARENT_DECLARATION_ID = ?";
			$q = $this->db->query($sql);
			$q->bind_values($id);
			$q->execute();
			if(!($r = $q->fetch()))
				return true;
			return false;
		}

		$sql = "SELECT MAX(DECLARATION_ID) as NEWEST
			FROM TBL_DECLARATION
			WHERE `MONTH` = ?
				AND `YEAR` = ?
				AND CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ?
				AND PRODUCT_ID = ? ";
		$q = $this->db->query($sql);
		$q->bind_values($this->month,$this->year,$this->crso->relation->id,$this->participant->relation->id, $this->product->id);
		$q->execute();
		$r = $q->fetch();
		if($r->NEWEST == $this->id)
			return true;
		return false;
	}

	public static function getDisplayStatus($value)
	{
		if(($value & DECLARATION_FORFEIT)==DECLARATION_FORFEIT)
			return DECLARATION_FORFEIT;
		if(($value & DECLARATION_LATE)==DECLARATION_LATE)
			return  DECLARATION_LATE;
		if(($value & DECLARATION_RESUBMIT)==DECLARATION_RESUBMIT)
			return  DECLARATION_RESUBMIT;
		if(($value & DECLARATION_NORMAL)==DECLARATION_NORMAL)
			return  DECLARATION_NORMAL;
		if(($value & DECLARATION_OPEN)==DECLARATION_OPEN)
			return  DECLARATION_OPEN;
		if(($value & DECLARATION_DECLARED)==DECLARATION_DECLARED)
			return  DECLARATION_DECLARED;
		if(($value & DECLARATION_CREDIT)==DECLARATION_CREDIT)
			return DECLARATION_CREDIT;
		return DECLARATION_OPEN;
	}

	/**
	 * Translates a bitwise status to a langIf text
	 * @param LanguageInterface $langIf
	 * @param int $status
	 * @return Array (Translation,CSSClassName)
	 */
	public static function translateStatus($langIf, $status)
	{
		switch($status)
		{
			case DECLARATION_OPEN:
				return array($langIf->getText('Declaration Open'), 'status_new');
			break;
			case DECLARATION_NORMAL:
				return array($langIf->getText('Declaration Normal'), 'status_verified');
			break;
			case DECLARATION_FORFEIT:
				return array($langIf->getText('Declaration Forfeit'), 'status_forfeit');
			break;
			case DECLARATION_LATE:
				return array($langIf->getText('Declaration Late'), 'status_late');
			break;
			case DECLARATION_DECLARED:
				return array($langIf->getText('Declaration Declared'), 'status_declared');
			break;
			case DECLARATION_RESUBMIT:
				return array($langIf->getText('Declaration Resubmit'), 'status_resubmit');
			break;
			case DECLARATION_CREDIT:
				return array($langIf->getText('Declaration Credit'), '');
			break;
		}
	}

	public static function getButtonAction($langIf, $status, $is_exported)
	{
		switch($status)
		{
			case DECLARATION_OPEN:
				return $langIf->getText('Declaration Declare');
			break;
			case ($status == DECLARATION_LATE) && (bool)$is_exported:
			case ($status == DECLARATION_DECLARED) && (bool)$is_exported:
			case ($status == DECLARATION_RESUBMIT) && (bool)$is_exported:
			case ($status == DECLARATION_NORMAL) && (bool)$is_exported:
				return  $langIf->getText('Declaration Re-Submit');
			break;
			case DECLARATION_FORFEIT:
				return $langIf->getText('Declaration Late');
			break;
			case ($status == DECLARATION_LATE) && !(bool)$is_exported:
			case ($status == DECLARATION_RESUBMIT) && !(bool)$is_exported:
			case ($status == DECLARATION_DECLARED) && !(bool)$is_exported:
			case DECLARATION_NORMAL:
				return $langIf->getText('Declaration Re-Declare');
			break;
		}
	}

	/**
	 * Always check if the declaration is for the current participant... Check outside this class.
	 * @param int $participant_id PARTICIPANT_RELATION_ID
	 * @param int $crso_id CRSO_RELATION_ID
	 * @param int $year
	 * @return ResultSet $q
	 */
	function getOverview($participant_id, $crso_id, $year='', $product_id=null)
	{
		$date =& currentDate();
		$year || $year = $date->getYear();

		$product_id || $product_id = $this->product->id;

		$sql = "SELECT D.DECLARATION_ID,
				IFNULL(CONCAT('D',DECLARATION_DEBIT_ID), IFNULL(CONCAT('C',DECLARATION_CREDIT_ID),'')) as DECLARATION_NUMBER,
				D.CRSO_RELATION_ID, R.NAME as CRSO_NAME, MONTH as PERIOD,
				DATE_FORMAT(MAX(DL.LAST_UPDATE),'%d-%m-%Y %H:%i') as LAST_UPDATED,
				DOCUMENT_ID as DOCUMENT,
				IFNULL(SUM(DL.QUANTITY), '-') as QUANTITY,
				(BIT_OR(DL.STATUS)) as DECLARATION_STATUS,
				'' as DECLARATION_ACTION, IS_EXPORTED,
				IFNULL(DD.`YEAR`, IFNULL(DC.`YEAR`,'')) as INVOICE_YEAR
			FROM TBL_DECLARATION as D
			INNER JOIN TBL_RELATION as R
				ON D.CRSO_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_RELATION as PR
				ON D.PARTICIPANT_RELATION_ID = PR.RELATION_ID
			INNER JOIN TBL_DECLARATION_LINE as DL
				ON D.DECLARATION_ID = DL.DECLARATION_ID
			LEFT JOIN TBL_DECLARATION_SET_LINE AS DSL
				ON D.DECLARATION_ID = DSL.DECLARATION_ID
			LEFT JOIN TBL_DECLARATION_DEBIT as DD
				ON DSL.DECLARATION_SET_ID = DD.DECLARATION_SET_ID
			LEFT JOIN TBL_DECLARATION_CREDIT as DC
				ON DSL.DECLARATION_SET_ID = DC.DECLARATION_SET_ID
			WHERE D.PARTICIPANT_RELATION_ID = ?
				AND D.CRSO_RELATION_ID = ?
				AND D.YEAR = ?
				AND D.PRODUCT_ID = ?
			GROUP BY D.DECLARATION_ID
			ORDER BY D.YEAR, D.MONTH, D.DECLARATION_ID";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id,$crso_id,$year, $product_id);
		$q->execute();
		return $q;
	}

	public function getLastUpdated($id=null)
	{
		$id || $id = $this->id;
		$sql = "SELECT DATE_FORMAT(MAX(DL.LAST_UPDATE),'%d-%m-%Y %H:%i') as LAST_UPDATED
			FROM TBL_DECLARATION as D
			INNER JOIN TBL_DECLARATION_LINE as DL
				ON D.DECLARATION_ID = DL.DECLARATION_ID
			WHERE D.DECLARATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if(!($r=$q->fetch()))
			return false;

		return $r->LAST_UPDATED;
	}

	public function getOpenDeclarationOverview($participant_id,$crso_id,$product_id)
	{
		require_once(ELC_CLASSES.'AnnualConfirmation.php');
		$sql = "SELECT D.DECLARATION_ID,
				D.CRSO_RELATION_ID, R.NAME as CRSO_NAME, D.MONTH as PERIOD,
				D.`YEAR` as DECLARATION_YEAR,
				IFNULL(SUM(DL.QUANTITY), '-') as QUANTITY,
				(BIT_OR(DL.STATUS)) as DECLARATION_STATUS,
				'' as DECLARATION_ACTION,
				D.PRODUCT_ID
			FROM TBL_DECLARATION as D
			INNER JOIN TBL_RELATION as R
				ON D.CRSO_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_RELATION as PR
				ON D.PARTICIPANT_RELATION_ID = PR.RELATION_ID
			INNER JOIN TBL_DECLARATION_LINE as DL
				ON D.DECLARATION_ID = DL.DECLARATION_ID
			LEFT JOIN TBL_DECLARATION as D2
				ON D.DECLARATION_ID = D2.PARENT_DECLARATION_ID
			LEFT JOIN TBL_ANNUAL_CONFIRMATION as AC
				ON D.CRSO_RELATION_ID = AC.CRSO_RELATION_ID
					AND D.PARTICIPANT_RELATION_ID = AC.PARTICIPANT_RELATION_ID
					AND D.PRODUCT_ID = AC.PRODUCT_ID
					AND D.YEAR = AC.YEAR
			WHERE D.PARTICIPANT_RELATION_ID = ?
				AND D.CRSO_RELATION_ID = ?
				AND D.PRODUCT_ID = ?
				AND ISNULL(D.PARENT_DECLARATION_ID)
				AND ISNULL(D2.DECLARATION_ID)
				AND (ISNULL(D.IS_EXPORTED) OR DL.STATUS & ".DECLARATION_FORFEIT.")
				AND (ISNULL(AC.STATUS) OR AC.STATUS != ".ANNUAL_CONFIRMATION_CLOSED.")
			GROUP BY D.DECLARATION_ID
			ORDER BY D.YEAR, D.MONTH, D.DECLARATION_ID";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id,$crso_id,$product_id);
		$q->execute();
		return $q;
	}

	/**
	 * Gets all the declarations within a certain month for a certain year
	 * @param int $month
	 * @param int $year
	 * @return Array $rv (DECLARATIONS)
	 */
	function getDeclarationLifeCycle($month='',$year='', $product_id='')
	{
		$month || $month = $this->month;
		$year || $year = $this->year;
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT DECLARATION_ID FROM TBL_DECLARATION
			WHERE MONTH = ?
				AND YEAR = ?
				AND CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ?
				AND PRODUCT_ID = ? ";
		$sql .= " ORDER BY DECLARATION_ID";
		$q = $this->db->query($sql);
		$q->bind_values($month,$year, $this->crso->relation->id,$this->participant->relation->id, $product_id);
		$q->execute();
		$rv = array();
		while($r = $q->fetch())
			$rv[] = new Declaration($this->db,$r->DECLARATION_ID);
		return $rv;
	}
	/**
	 * Updates are always valid till the declaration is exported
	 * @param int $id DECLARATION_ID
	 * @return bool
	 */
	function validateUpdate($id='')
	{
		return ! ((bool) $this->is_exported);
	}
	/**
	 * Validates wether or not it is valid to insert a new declaration for a certain month
	 * @todo make work with TRUSTs that start later but have an early obligation start date
	 * @return bool
	 */
	function validateInsert($trust)
	{
		$sql = "SELECT * FROM TBL_DECLARATION as D
			INNER JOIN TBL_DECLARATION_LINE as DL
				ON D.DECLARATION_ID = DL.DECLARATION_ID
			WHERE CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ?
				AND `MONTH` = ?
				AND `YEAR` = ?
				AND PRODUCT_ID = ? ";
		if($trust)
			$sql .= " AND TRUST_ID = ".$trust->id." ";
		else
			$sql .= " AND ISNULL(TRUST_ID) ";
		$sql .= " GROUP BY D.DECLARATION_ID";
		$q = $this->db->query($sql);
		$q->bind_values($this->crso->relation->id,$this->participant->relation->id,$this->month,$this->year, $this->product->id);
		$q->execute();
		if($r = $q->fetch())
			return false;
		return true;
	}
	/**
	 * Validates wether or not it should be possible for a participant to add a new line
	 * @return bool
	 */
	function validateNewLine()
	{
		if($this->isForfeit())
			return false;
		// Else check previous Declaration for this month-year
		// If its a credit that has not been exported yet and the status of the credit was FORFEIT
		if(!($id = $this->_getPreviousDeclaration('CREDIT')))
			return true;
		$d = new Declaration($this->db,$id);
		$d->set();
		if($d->isLate()&&!$d->is_exported)
			return false;
		return true;
	}
	/**
	 * @access private
	 * @param string $type
	 * @return int DECLARATION_ID
	 */
	function _getPreviousDeclaration($type='')
	{
		$sql = "SELECT MAX(D.DECLARATION_ID) as DECLARATION_ID
			FROM TBL_DECLARATION as D ";
		if($type)
		{
			$sql .= "
				INNER JOIN TBL_DECLARATION_SET_LINE AS DSL
				ON D.DECLARATION_ID = DSL.DECLARATION_ID
				INNER JOIN TBL_DECLARATION_".$type." as DT
				ON DSL.DECLARATION_SET_ID = DT.DECLARATION_SET_ID ";
		}
		$sql .= "WHERE D.DECLARATION_ID != ?
			AND D.`MONTH` = ?
			AND D.`YEAR` = ?
			AND D.PARTICIPANT_RELATION_ID = ?
			AND D.CRSO_RELATION_ID = ?
			AND D.PRODUCT_ID = ? ";
		$sql .= "GROUP BY D.CRSO_RELATION_ID, D.PARTICIPANT_RELATION_ID, D.`YEAR`, D.`MONTH`";
		$q = $this->db->query($sql);
		$q->bind_values($this->id,$this->month,$this->year,
			$this->participant->relation->id,$this->crso->relation->id, $this->product->id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		return $r->DECLARATION_ID;
	}
	/**
	 * Deletes a Declaration?
	 * @param int $id DECLARATION_ID
	 * @return bool
	 */
	function delete($id='')
	{
		$id || $id = $this->id;
		if(!$this->deleteLines($id))
			return false;

		$sql = "DELETE FROM TBL_DECLARATION
			WHERE DECLARATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		return $q->execute();
	}
	/**
	 * Deletes all the lines for a certain Declaration
	 * @param int $id DECLARATION_ID
	 * @return bool
	 */
	function deleteLines($id='')
	{
		$id || $id = $this->id;
		$sql = "DELETE FROM TBL_DECLARATION_LINE
			WHERE DECLARATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		return $q->execute();
	}
	/**
	 * Delete a single line from a declaration
	 * @param int $index
	 * @return bool
	 */
	function deleteLine($index)
	{
		if(isset($this->lines[$index]))
			return $this->lines[$index]->delete();
		return true;
	}
	/**
	 * Insert all the lines for the current Declaration
	 */
	function insertLines($status=null)
	{
		foreach($this->lines as $index => $bitbucket)
		{
			$line = &$this->lines[$index];
			$line->set();

			if (!is_null($status))
				$line->status = $status;
			$line->declaration = &$this;
			$line->line_index = $index;
			$line->created = null;
			$line->insert();
		}
	}
	/**
	 * Replacces all the lines for the current Declaration
	 */
	function replaceLines()
	{
		foreach($this->lines as $index => $bitbucket)
		{
			$line = &$this->lines[$index];

			$line->declaration = $this;
			$line->line_index = $index;
			$line->objectmap['inserttype'] = 'REPLACE';
			$line->insert();
		}
	}
	/**
	 * See trac ticket #1392
	 */
	private function checkNewProvisionalAndAC($product_id,$participant_id,$crso_id)
	{
		$date = currentDate();
		// has provisonal in previous year

		info(sprintf(' - checkNewProvisionalAndAc'));
		$sql = "SELECT * FROM TBL_PROVISIONAL_DECLARATION
			WHERE PARTICIPANT_RELATION_ID = ?
				AND CRSO_RELATION_ID = ?
				AND PRODUCT_ID = ?
				AND `YEAR` = ?";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id,$crso_id,$product_id,($date->getYear()-1));
		$q->execute();
		$rows = $q->fetchAll();
		if(!count($rows))
			return false;

		info(sprintf(' - Provisional Found'));

		// $day is past the export-date
		$cp = new CRSOProduct($this->db);
		$cp->set($crso_id,$product_id);

		if($cp->forfeit_day>$date->getDay())
			return false;

		info(sprintf(' - Needs new Provisional'));

		$log = getLog();
		info(sprintf(' - Process new Provisional'));
		// Trigger AnnualNewProvisional->processFromProvisional
		require_once(ELC_CLASSES.'AnnualNewProvisional.php');
		$anp = new AnnualNewProvisional($this->db,$log);
		$anp->processFromProvisional($participant_id,$crso_id,$product_id);
	}

	/**#@+
	 * Functions called by TaskRunner
	 */
	/**
	 * Only call this function from TaskDeclaration
	 * Insert all lines available for month from Provisional.
	 * @param array $firstTimeInsert
	 * @todo Trustee
	 */
	function insertNewDeclarations($firstTimeInsert=false, $product_id)
	{
		$log = getLog();
		$date = currentDate();
		$month = $date->getMonth()-1;
		$month = $month > 0 ? $month : 12;
		$year = $date->getYear();
		$year = $month == 12 ? $year - 1 : $year;
		info(sprintf('Declaration::insertNewDeclarations()'));
		info(sprintf(' - date: mm/yy: %s/%s', $month, $year));

		if(is_array($firstTimeInsert)&& $year<$date->getYear())
			$this->checkNewProvisionalAndAC($product_id,$firstTimeInsert['participant']->relation->id,$firstTimeInsert['crso']->relation->id);

		require_once(ELC_CLASSES.'ProvisionalDeclaration.php');
		require_once(ELC_CLASSES.'Trust.php');
		require_once(ELC_CLASSES.'DeclarationComponent.php');
		require_once(ELC_CLASSES.'Country.php');

		$pd = new ProvisionalDeclaration($this->db);
		$t = new Trust($this->db);

		$values = array(PARTICIPANT_STATUS_PARTICIPANT_READY,1,0, $product_id);
		$sql = "SELECT PARTICIPANT_RELATION_ID, CRSO_RELATION_ID
			FROM TBL_CRSO_PARTICIPANT_PRODUCT
			WHERE STATUS = ?
				AND ACTIVE = ?
				AND PRODUCER_EXPORTER & ".PARTICIPANT_PRODUCER. "
				AND NO_DECLARATION = ?
				AND PRODUCT_ID = ? ";
		if(is_array($firstTimeInsert))
		{
			$values[] = $firstTimeInsert['participant']->relation->id;
			$values[] = $firstTimeInsert['crso']->relation->id;
			$sql .= " AND PARTICIPANT_RELATION_ID = ?
				AND CRSO_RELATION_ID = ? ";
		}
		$sql .= 'ORDER BY CRSO_RELATION_ID ';

		$q = $this->db->query($sql);
		$q->bind_values($values);
		$q->execute();
		$rs = $q->fetchAll();

		debug(sprintf(' - found %s participants', count($rs)));
		info('Declaration::insertNewDeclarations() - Current Month: '.$month.' and current Year: '.$year);

		$crso = false;
		$recipients = array(); // notification receipients
		$dc = new DeclarationComponent($this->db);

		foreach($rs as $r)
		{
			// Get trusts, the participant id is the trustee id in table trust.
			$trusts = array(false);
			$query = $t->selectTrusts($r->PARTICIPANT_RELATION_ID,TRUST_APPROVED,$r->CRSO_RELATION_ID, $product_id);
			while($row = $query->fetch())
				$trusts[] = $row->TRUST_ID;

			$openMonths = is_array($firstTimeInsert) ?
				$this->_getOpenMonths($r->PARTICIPANT_RELATION_ID,$r->CRSO_RELATION_ID, $trusts, $product_id) :
					array($year=>array($month=>$month));

			$c = new CRSO($this->db,array('relation'=>new Relation($this->db,$r->CRSO_RELATION_ID, true)));
			$country = new Country($this->db,$c->getCountry(),true);

			// send notifications when crso switches in while
			if ($crso && ( $c->relation->id != $crso->relation->id )) {
				$this->_sendNotifications($crso, $recipients,$product_id);
				$recipients = array();
			}

			info('Declaration::insertNewDeclarations() - Open Months `'.sizeof($openMonths).'`');

			foreach($openMonths as $year => $months)
			{
				foreach($months as $month)
				{
					$values = array('month'=>$month,'year'=>$year, 'product' => new Product($this->db, $product_id));
					$d = new Declaration($this->db,$values);
					$d->crso =& $c;
					$d->participant = new Participant($this->db,array('relation'=>new Relation($this->db,$r->PARTICIPANT_RELATION_ID)));

					$i=1;
					$valid = false;
					foreach($trusts as $trust_id)
					{
						$trust = false;
						if($trust_id)
							$trust = new Trust($this->db,$trust_id);
						if(!$d->validateInsert($trust))
							continue;
						info('Declaration::validateInsert() - A valid Insert with trust: `'.((bool)$trust).'`');
						$valid = true;
						$log->info('New Declaration for Participant ['.$r->PARTICIPANT_RELATION_ID.
							'] at CRSO ['.$r->CRSO_RELATION_ID.'] month ['.$month.'] for year ['.
								$year.'] with Trust ['.($trust ? $trust->id : '-') .']');

						// only add line if provisional exists
						if (!$pd->setByMonth($month,$r->CRSO_RELATION_ID,$r->PARTICIPANT_RELATION_ID,$year,$trust_id, $product_id))
							continue;

						$lines = $pd->getLines();
						foreach($lines as $line)
						{
							$args = array('declaration'=>$this,
								'quantity'=>null,
								'weight'=>null,
								'weight_dec'=>null,
								'product_category'=>$line->product_category,
								'brand'=>$line->brand,
								'dimension'=>$line->dimension,
								'status'=>DECLARATION_OPEN,
								'trust'=>$trust
								);
							$d->lines[$i++] = new DeclarationLine($this->db,$args);
						}
					}

					//if(!$d->validateInsert()) {
					if(!$valid) {
						$log->notice('Declaration::insertNewDeclarations() - Not inserting declaration. Declaration not valid for CRSO ['.
							$r->CRSO_RELATION_ID.'] and Participant ['.$r->PARTICIPANT_RELATION_ID.']');
					}
					elseif (!sizeof($d->lines)) {
						$log->notice('Declaration::insertNewDeclarations() - Not inserting declaration. No lines for CRSO ['.
							$r->CRSO_RELATION_ID.'] and Participant ['.$r->PARTICIPANT_RELATION_ID.']');
					}
					else
					{
						$d->insert();
						$d->insertLines();

						// Check country settings for a Component setting.
						if($country->has(HasDeclarationComponent,$product_id))
						{
							$dc->setCRSOComponents($c->relation->id,$product_id);
							$dc->triggerNewDeclarationComponents($c->relation->id, $d);
						}

						$recipients[] = array('recipient'=>$d->participant->relation,
							'callback'=>"go('/".SITE_PATH."scripts/elc/participant/InterfaceDeclarationOverview.php')");
					}
				}

				isset($d) && $crso = $d->crso;
			}

		}

		// also send notification for last crso
		if($crso)
			$this->_sendNotifications($crso, $recipients, $product_id);
	}

	function _sendNotifications($crso, $recipients,$product_id)
	{
		$langIf = IWRegistry::registry('langIf'); // FIXME: dispatcher or something
		require_once(ELC_CLASSES.'Notification.php');
		require_once(ELC_CLASSES.'Product.php');
		Notification::send($this->db,
			array('subject'=>$langIf->getText('Notification: New Declaration'),
				'sender'=>$crso->relation,
				'product'=>new Product($this->db,$product_id),
				'message'=>
					$langIf->getText("Notification: New Declaration at CRSO `{CRSO}`", // fixme tra
						null,
						array ('CRSO'=>$crso->relation->name),
						$crso->default_language_id)
			),
			$recipients
		);

	}

	/**
	 * Sets some declarations to EXPORT_PENDING for the given CRSO
	 * Declarations are set to EXPORT_PENDING if:
	 *  - today is the FORFEIT_DAY (export day) for the crso
	 *  - IS_EXPORTED is NULL in TBL_DECLARATION
	 *  - STATUS not is DECLARATION_OPEN
	 *  - the crso forfeit day is today
	 */
	function exportDeclarations(&$log)
	{
		require_once(ELC_CLASSES.'Country.php');
		require_once(ELC_CLASSES.'DeclarationOneInvoice.php');
		$date =& currentDate();
		$currentDay = $date->getDay();
		$currentMonth = $date->getMonth();
		// LOOP ALL CRSO's
		$sql = "SELECT CRSO_RELATION_ID, PRODUCT_ID
			FROM TBL_CRSO_PRODUCT
			WHERE FORFEIT_DAY = ?";
		$q = $this->db->query($sql);
		$q->bind_values($currentDay);
		$q->execute();
		$rs = $q->fetchAll();
		if(!count($rs))
			$log->info('Declaration::exportDeclarations() - No Exports Scheduled for today');
		$crsos = array();
		foreach($rs as $r)
			$crsos[$r->CRSO_RELATION_ID][$r->PRODUCT_ID] = $r->PRODUCT_ID;

		$r = new Relation($this->db);
		foreach($crsos as $crso_id => $products)
		{
			$groupedProducts = array();
			$country = new Country($this->db,$r->getCountry($crso_id),true);

			foreach($products as $product_id)
			{
				$decl = $this;
				if($country->has(HasOneInvoice,$product_id))
					$decl = new DeclarationOneInvoice($this->db);

				if(!$country->has(HasCrsoOneExportDay,$product_id)) {
					$decl->_exportDeclarationsForCRSO($crso_id, $log, array($product_id));
				}
				else {
					$groupedProducts[] = $product_id;
				}
			}

			if(count($groupedProducts)>0)
				$decl->_exportDeclarationsForCRSO($crso_id, $log, $groupedProducts);

			reset($products);

			if($currentMonth==1) // January, trigger AnnualConfirmation, trigger New Provisionals
			{
				foreach($products as $product_id)
				{
					require_once(ELC_CLASSES.'AnnualConfirmation.php');
					$ac = new AnnualConfirmation($this->db);
					$ac->triggerNewAnnualConfirmations($crso_id,$log, $product_id);

					require_once(ELC_CLASSES.'AnnualNewProvisional.php');
					$anp = new AnnualNewProvisional($this->db, $log);
					$anp->triggerNewYear($crso_id, $product_id);
				}
			}
		}
	}
	/**
	 * @access private
	 */
	function _exportDeclarationsForCRSO($crso_id, &$log, $products)
	{
		$this->db->begin();
		$log->info(sprintf('Declaration::_exportDeclarationsForCRSO() - Starting export for CRSO [%s]', $crso_id));
		$date =& currentDate();
		//$lastmonth = ($date->getMonth()-1);
		$year = $date->getYear();

		$r = new Relation($this->db);
		$country = new Country($this->db,$r->getCountry($crso_id),true);

		$declaration_id = null;
		$dl = new DeclarationLine($this->db);
		// LOOP ALL NON-EXPORTED DECL.
		// LOOP LINES, UPD STATUS IF NECESSARY
		$sql = "SELECT D.DECLARATION_ID, D.`MONTH`, D.`YEAR`, DL.LINE_INDEX, D.PRODUCT_ID, D.HAS_CRITICAL_CHANGES
			FROM TBL_DECLARATION as D
			INNER JOIN TBL_DECLARATION_LINE as DL
				USING(DECLARATION_ID)
			WHERE CRSO_RELATION_ID = ?
				AND ISNULL(IS_EXPORTED)
				AND DL.`STATUS` != ?
				AND D.PRODUCT_ID IN (".join($products, ',').")
			ORDER BY DECLARATION_ID";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, DECLARATION_OPEN);
		$q->execute();
		while($r = $q->fetch())
		{
			# we are looping tru lines so only once change declaration to export_pending
			if($declaration_id != $r->DECLARATION_ID)
			{
				$export_status = DECLARATION_EXPORT_PENDING;
				// If only non-critical changes were made and the country setting to omit those from export is enabled, set correct status.
				if($r->HAS_CRITICAL_CHANGES == DECLARATION_HAS_NO_CRITICAL_CHANGES && $country->has(HasOmitExportOnOnlyWeightChanges, $r->PRODUCT_ID)) {
					$export_status = DECLARATION_EXPORT_OMITTED;
				}

				$declaration_id = $r->DECLARATION_ID;
				$sql = "UPDATE TBL_DECLARATION SET IS_EXPORTED = ?
					WHERE DECLARATION_ID = ?";
				$query = $this->db->query($sql);
				$query->bind_values($export_status, $declaration_id);
				$query->execute();
				$export = null;
			}

			# update line status to normal
			$dl->set($declaration_id,$r->LINE_INDEX);
			$dl->declaration = new Declaration($this->db,$declaration_id);
			if(($dl->status & DECLARATION_DECLARED) == DECLARATION_DECLARED)
			{
				$log->info('Changing declaration line ['.$declaration_id.', '.$r->LINE_INDEX.'] status to DECLARATION_NORMAL');
				$dl->status = DECLARATION_NORMAL;
				$dl->update();
			}
		}

		# ok .. declarations pending ... Now add Invoice numbers to them
		$this->addDebitNumbersToPendingExport($crso_id, $log, $products);
		$this->addCreditNumbersToPendingExport($crso_id, $log, $products);
		$this->db->commit();
	}


	/**
	 * selects all exported declarations for the given arguments
	 */
	function selectExportedDeclarations($crso_id, $year, $month) {

		$sql = "
			SELECT

				# invoice level

				IF(ISNULL(dd.DECLARATION_DEBIT_ID),
					f_elcBuildInvoiceNoForInterfaces(d.CRSO_RELATION_ID,dc.`YEAR`,dc.DECLARATION_CREDIT_ID,TRUE,FALSE),
					f_elcBuildInvoiceNoForInterfaces(d.CRSO_RELATION_ID,dd.`YEAR`,dd.DECLARATION_DEBIT_ID,FALSE,FALSE
				)) AS _INVOICE_NUMBER, -- used for ordening only

				IF(ISNULL(dd.DECLARATION_DEBIT_ID),
					dc.`YEAR`,
					dd.`YEAR`
				) as INVOICE_YEAR,

				IF(ISNULL(dd.DECLARATION_DEBIT_ID),
					dc.DECLARATION_CREDIT_ID,
					dd.DECLARATION_DEBIT_ID
				) as INVOICE_ID,

				p.NAME as PRODUCT_NAME,
				IF(ISNULL(dd.DECLARATION_DEBIT_ID),'credit','debit') AS TYPE,
				d.PARTICIPANT_RELATION_ID,
				d.EXPORT_DATE as DOCUMENT_DATE,

				# declaration level
				d.DECLARATION_ID AS DECLARATION_ID,
				d.YEAR,
				d.MONTH,
				DATE(dl.CREATED) as POSTING_DATE,

				IF(ISNULL(dd.DECLARATION_DEBIT_ID), dd2.`YEAR`, '') as ORIGINAL_INVOICE_YEAR,
				IF(ISNULL(dd.DECLARATION_DEBIT_ID), dd2.DECLARATION_DEBIT_ID, '') as ORIGINAL_INVOICE_ID,

				# line level
				IF(ISNULL(dd.DECLARATION_DEBIT_ID),
					IF(dl.STATUS = 64,
						ABS(dl.QUANTITY),
						-(ABS(dl.QUANTITY))
					),
					IF(dl.STATUS = 64,
						-(ABS(dl.QUANTITY)),
						ABS(dl.QUANTITY)
					)
				) as QUANTITY,

				ROUND(IF(ISNULL(dd.DECLARATION_DEBIT_ID),
					IF(dl.STATUS = 64,
						ABS(
							(IF(f_elcUseQuantityColumn(d.CRSO_RELATION_ID, d.PRODUCT_ID), dl.QUANTITY, dl.WEIGHT) *
								f_elcProductFee(d.PRODUCT_ID, d.CRSO_RELATION_ID, dl.PRODUCT_CATEGORY_ID, CONCAT(d.YEAR, '-', d.MONTH, '-01 00:00:00')))),
						-(ABS(
							(IF(f_elcUseQuantityColumn(d.CRSO_RELATION_ID, d.PRODUCT_ID), dl.QUANTITY, dl.WEIGHT) *
								f_elcProductFee(d.PRODUCT_ID, d.CRSO_RELATION_ID, dl.PRODUCT_CATEGORY_ID, CONCAT(d.YEAR, '-', d.MONTH, '-01 00:00:00')))))
					),
					IF(dl.STATUS = 64,
						-(ABS(
							(IF(f_elcUseQuantityColumn(d.CRSO_RELATION_ID, d.PRODUCT_ID), dl.QUANTITY, dl.WEIGHT) *
								f_elcProductFee(d.PRODUCT_ID, d.CRSO_RELATION_ID, dl.PRODUCT_CATEGORY_ID, CONCAT(d.YEAR, '-', d.MONTH, '-01 00:00:00'))))),
						ABS(
							(IF(f_elcUseQuantityColumn(d.CRSO_RELATION_ID, d.PRODUCT_ID), dl.QUANTITY, dl.WEIGHT) *
								f_elcProductFee(d.PRODUCT_ID, d.CRSO_RELATION_ID, dl.PRODUCT_CATEGORY_ID, CONCAT(d.YEAR, '-', d.MONTH, '-01 00:00:00'))))

					)
				), 2) as LINE_AMOUNT,

				ROUND(IF(ISNULL(dd.DECLARATION_DEBIT_ID), f_elcGetDeclarationCreditTotalAmount(d.CRSO_RELATION_ID, dc.DECLARATION_CREDIT_ID, dc.`YEAR`), f_elcGetDeclarationDebitTotalAmount(d.CRSO_RELATION_ID, dd.DECLARATION_DEBIT_ID, dd.`YEAR`)), 2) as TOTAL_AMOUNT,

				dl.STATUS,
				f_elcProductFee(d.PRODUCT_ID, d.CRSO_RELATION_ID, dl.PRODUCT_CATEGORY_ID, CONCAT(d.YEAR, '-', d.MONTH, '-01 00:00:00')) AS UNIT_COST,
				IFNULL(dl.WEIGHT, '') as WEIGHT,
				IFNULL(dl.WEIGHT_DEC, '') as WEIGHT_DEC,

				IFNULL(b.NAME, '') as BRAND_NAME,
				IFNULL(cpd.NAME, '') as CPD_NAME,
				IFNULL(pc.NAME, '') as CATEGORY_NAME,

				dl.STATUS as LINE_STATUS

			FROM TBL_DECLARATION AS d

			INNER JOIN TBL_RELATION AS cr
				ON cr.RELATION_ID = d.CRSO_RELATION_ID
			INNER JOIN TBL_ADDRESS AS ca
				ON cr.ADDRESS_ID = ca.ADDRESS_ID
			INNER JOIN TBL_COUNTRY AS cac
				ON cac.COUNTRY_ID = ca.COUNTRY_ID
			INNER JOIN TBL_DECLARATION_SET_LINE AS dsl
				ON d.DECLARATION_ID = dsl.DECLARATION_ID
			LEFT JOIN TBL_DECLARATION_DEBIT AS dd
				ON dsl.DECLARATION_SET_ID = dd.DECLARATION_SET_ID
			LEFT JOIN TBL_DECLARATION_CREDIT AS dc
				ON dsl.DECLARATION_SET_ID = dc.DECLARATION_SET_ID
			INNER JOIN TBL_DECLARATION_LINE AS dl
				ON dl.DECLARATION_ID = d.DECLARATION_ID
			LEFT JOIN TBL_PRODUCT_CATEGORY AS pc
				ON dl.PRODUCT_CATEGORY_ID = pc.PRODUCT_CATEGORY_ID
			INNER JOIN TBL_PRODUCT AS p
				ON d.PRODUCT_ID = p.PRODUCT_ID
			LEFT JOIN TBL_BRAND AS b
				ON b.BRAND_ID = dl.BRAND_ID
			LEFT JOIN TBL_CRSO_PRODUCT_DIMENSION cpd
				ON cpd.CPD_ID = dl .CPD_ID
			LEFT JOIN TBL_DECLARATION_SET_LINE AS dsl2
				ON d.PARENT_DECLARATION_ID = dsl2.DECLARATION_ID
			LEFT JOIN TBL_DECLARATION_DEBIT AS dd2
				ON dsl2.DECLARATION_SET_ID = dd2.DECLARATION_SET_ID

			WHERE d.IS_EXPORTED = ?
				AND d.CRSO_RELATION_ID = ?
				AND YEAR(d.EXPORT_DATE) = ? AND MONTH(d.EXPORT_DATE) = ?
				AND (ISNULL(dd.DECLARATION_DEBIT_ID) XOR ISNULL(dc.DECLARATION_CREDIT_ID))
			GROUP BY dl.DECLARATION_ID, dl.LINE_INDEX

			ORDER BY _INVOICE_NUMBER, p.PRODUCT_ID, d.DECLARATION_ID, dl.LINE_INDEX
		";

		$q = $this->db->query($sql);
		$q->bind_values(array(DECLARATION_EXPORT_EXPORTED, $crso_id, $year, $month));
		$q->execute();

		return $q;

	}

	/**
	 * Only call this function from TaskDeclaration
	 * @param Log &$log
	 */
	function forfeitDeclarations(&$log,$maxmonth='', $maxyear='', $product_id='')
	{
		$date =& currentDate();
		$maxmonth || $maxmonth = ($date->getMonth()-1);
		if($maxmonth == 0)
			$maxmonth = 12;
		$maxyear || $maxyear = $date->getYear();
		if($maxmonth == 12)
			$maxyear = $maxyear - 1;

		$product_id || $product_id = $this->product->id;

		$currentDay = $date->getDay();



		// A forfeit is triggered:
		// - if there are no lines for the month
		// - if the month is after the APPLICATION_START_DATE
		// - if the date is past the 10th of the next month
		// - if there are open lines

		require_once(IWARE.'form.php');
		$formDate = new IWFormDateTime('name');

		$sql = "SELECT FORFEIT_START_DATE, FORFEIT_DAY, CRSO_RELATION_ID
			FROM TBL_CRSO_PRODUCT
			WHERE NOT ISNULL(FORFEIT_START_DATE)
				AND FORFEIT_DAY = ?
				AND PRODUCT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($currentDay,$product_id);
		$q->execute();
		while($row = $q->fetch())
		{
			$formDate->setMySQLDate($row->FORFEIT_START_DATE);
			$minmonth = (int) date('m', $formDate->timestamp);
			$minyear = (int) date('Y', $formDate->timestamp);

			$sql = "SELECT (BIT_OR(DL.STATUS)) as DECLARATION_STATUS, D.DECLARATION_ID,
					D.CRSO_RELATION_ID, D.PARTICIPANT_RELATION_ID
				FROM TBL_DECLARATION as D
				INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT as CP
					ON D.CRSO_RELATION_ID = CP.CRSO_RELATION_ID
						AND D.PARTICIPANT_RELATION_ID = CP.PARTICIPANT_RELATION_ID
						AND D.PRODUCT_ID = CP.PRODUCT_ID
				LEFT JOIN TBL_DECLARATION_LINE as DL
					ON D.DECLARATION_ID = DL.DECLARATION_ID
				WHERE ((D.MONTH >= ? AND D.YEAR = ?) OR D.YEAR > ?)
					AND D.CRSO_RELATION_ID = ?
					AND CP.`ACTIVE` = ?
					AND D.PRODUCT_ID = ?
				GROUP BY D.DECLARATION_ID";
//				GROUP BY D.CRSO_RELATION_ID, D.PARTICIPANT_RELATION_ID, D.YEAR, D.MONTH";
			$q1 = $this->db->query($sql);
			$q1->bind_values($minmonth,$minyear,$minyear,$row->CRSO_RELATION_ID,1, $product_id);
			$q1->execute();
			while($r = $q1->fetch())
			{
				if(($r->DECLARATION_STATUS & DECLARATION_OPEN) == DECLARATION_OPEN)
					$this->_triggerForfeit($r->DECLARATION_ID, $log);
			}
		}
	}

	/**
	 * @returns int Total quantity from all lines
	 */
	function getTotalQuantity($id='')
	{
		$id || $id = $this->id;

		$sql = "SELECT SUM(QUANTITY) as TOTAL_QUANTITY FROM TBL_DECLARATION_LINE WHERE DECLARATION_ID=?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if ($r = $q->fetch())
			return $r->TOTAL_QUANTITY;
		else
			return 0;

	}

	/**
	 * @FIXME get from table field!!
	 * @return int totalAmount
	 */
	function getTotalAmount($id='')
	{
		$id || $id = $this->id;

		$sql = "SELECT (SUM(QUANTITY) * pf.FEE) as TOTAL_AMOUNT
			FROM TBL_DECLARATION_LINE dl
				INNER JOIN TBL_DECLARATION d
					ON dl.DECLARATION_ID = d.DECLARATION_ID
				INNER JOIN TBL_PRODUCT_FEE pf
					ON d.CRSO_RELATION_ID = pf.CRSO_RELATION_ID
			WHERE d.DECLARATION_ID=?
				AND pf.CRSO_RELATION_ID = ?";

		$q = $this->db->query($sql);
		$q->bind_values($id, $this->crso->relation->id);
		$q->execute();
		if ($r = $q->fetch())
			return $r->TOTAL_AMOUNT;
		else
			return 'error';
	}

	/**
	 * Only update the lines that have empty quantity
	 * @access private
	 */
	function _triggerForfeit($id, &$log)
	{
		$this->db->begin();
		$log->info('Forfeit triggered for Declaration ['.$id.']');
		require_once(ELC_CLASSES.'ProvisionalDeclaration.php');
		$this->set($id);
		$this->is_exported = DECLARATION_EXPORT_PENDING;//FIXME
		$this->update();

		$this->crso->set();
		$this->crso->relation->set();

		// Send a notifcation to the Participant,
		require_once(ELC_CLASSES.'Notification.php');
		$langIf = IWRegistry::registry('langIf');
		Notification::send($this->db,
			array('subject'=>$langIf->getText('Notification: Declaration Forfeit'),
				'sender'=>$this->crso->relation,
				'message'=>
					$langIf->getText("Notification: Declaration at CRSO `{CRSO}` was Forfeited", // fixme tra
						null,
						array ('CRSO'=>$this->crso->relation->name),
						$this->crso->default_language_id)),
			array(array('recipient'=>$this->participant->relation,
					'callback'=>"go('/".SITE_PATH."scripts/elc/participant/InterfaceDeclarationOverview.php')")));


                //send crso notification is there is 1 outstanding forfeit for this participant at this product
                // #279
                if($this->hasOneOutstandingForfeit($this->participant->relation->id, $this->crso->relation->id, $this->product->id))
                {
                    $this->participant->set();
                    $this->participant->relation->set();
                    Notification::send($this->db,
                        array('subject'=>$langIf->getText('Notification: Declaration Second Forfeit'),
				'message'=>
                                        $langIf->getText("Notification: Second outstanding Forfeit for {PARTICIPANT}",
                                        null,
                                        array ('PARTICIPANT'=>$this->participant->relation->name),
                                        $this->crso->default_language_id),
                                'product' => new Product($this->db, $this->product->id)),
			array(array('recipient'=>$this->crso->relation)
                        )
                    );
                }


		$pd = new ProvisionalDeclaration($this->db,array('crso'=>$this->crso,'participant'=>$this->participant,'year'=>$this->year, 'product'=>$this->product));
		foreach($this->lines as $line)
		{
			$line->set();
			if($line->status != DECLARATION_OPEN)
			{
				if($line->status == DECLARATION_DECLARED)
				{
					$line->status = DECLARATION_NORMAL;
					$line->update();
				}
				continue;
			}
			$pcid = isset($line->product_category->id)?$line->product_category->id:'';
			$bid = isset($line->brand->id)?$line->brand->id:'';
			$dimid = isset($line->dimension->id)?$line->dimension->id:'';
			$tid = isset($line->trust->id)?$line->trust->id:'';
			list($quantity, $weight, $weight_dec) = $pd->getQuantityForMonth($this->month,$pcid,$bid,$tid, null, $dimid);
			$log->info('Interest rate applied for line ['.$id.']['.$line->line_index.']: '.$this->_getInterestRate());
			$line->quantity = (int) ((int) $quantity * $this->_getInterestRate());
			$log->info('Forfeit quantity for line ['.$id.']['.$line->line_index.']: '.$line->quantity);
			$line->weight = $weight?(int) ((int) $weight * $this->_getInterestRate()):$weight;
			$line->weight_dec = $weight_dec?(float) ((float) $weight_dec * $this->_getInterestRate()):$weight_dec;
			$line->status = DECLARATION_FORFEIT;
			$line->update();
		}
		$this->db->commit();
	}

	/**
	 * Gets the interest rate for the current CRSO
	 * @todo get correct forfeit interest rate from database
	 */
	function _getInterestRate()
	{
		require_once(ELC_CLASSES.'DeclarationInterest.php');

		$di = new DeclarationInterest($this->db,array('crso'=>$this->crso, 'product'=>$this->product));
		$first = ($di->getCurrentRate(false,INTEREST_RATE_FORFEIT_FIRST, $this->product->id)/100);
		$second = $first*($di->getCurrentRate(false,INTEREST_RATE_FORFEIT_SECOND, $this->product->id)/100);
		$third = $first*($di->getCurrentRate(false,INTEREST_RATE_FORFEIT_THIRD, $this->product->id)/100);

		$sql = "SELECT COUNT(DISTINCT D.DECLARATION_ID) as FF FROM TBL_DECLARATION as D
			INNER JOIN TBL_DECLARATION_LINE as DL
				ON D.DECLARATION_ID = DL.DECLARATION_ID
			LEFT JOIN TBL_DECLARATION as D2
				ON D.DECLARATION_ID = D2.PARENT_DECLARATION_ID
			WHERE DL.STATUS = ?
				AND ISNULL(D2.PARENT_DECLARATION_ID)
				AND D.PARTICIPANT_RELATION_ID = ?
				AND D.CRSO_RELATION_ID = ?
				AND D.PRODUCT_ID = ?
				AND D.DECLARATION_ID != ?";
		$q = $this->db->query($sql);
		$q->bind_values(DECLARATION_FORFEIT,$this->participant->relation->id,
			$this->crso->relation->id,$this->product->id, $this->id);
		$q->execute();
		$r = $q->fetch();
		if($r->FF == 0)
			return $first;
		else if($r->FF == 1)
			return $second;
		return $third;
	}

	/**
	 * Get all the months that have no declaration whatsoever since the CRSOParticipant->obligation_start_date
	 * @access private
	 * @param int $participant_id
	 * @param int $crso_id
	 * @return array
	 */
	function _getOpenMonths($participant_id,$crso_id,$trusts, $product_id)
	{
		require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
		$cp = new CRSOParticipantProduct($this->db);

		$min_date = false;
		foreach ($trusts as $trust)
		{

			if ($trust) {
				if (is_string($trust) || is_int($trust)) {
					$trust = new Trust($this->db, $trust);
				} else if (get_class($trust) == 'Trust') {
					// pass
				} else {
					throw new Exception('error reading trust:'.$trust);
				}

				$trust->set();
				$cp->set($crso_id,$trust->subrogator->relation->id, $product_id);

			}
			else
				$cp->set($crso_id,$participant_id, $product_id);

			// pear date object
			$obligation_date = createDate($cp->obligation_start_date, "YYYY-MM-DD");

			//info('Declaration::_getOpenMonths() - obligation_date: ' . $cp->obligation_start_date);

			if (!$min_date)
				$min_date = $obligation_date;
			else if ($obligation_date->before($min_date))
				$min_date = $obligation_date;
		}

		if ($min_date->format("m/d/y") == "01/01/00")
		{
			trigger_error('Declaration::_getOpenMonths() - ERROR: empty min date', E_USER_NOTICE);
			return array();
		}

		$openMonths = array();
		$year = $min_date->getYear();
		$month = $min_date->getMonth();

		$date =& currentDate();
		$month = (int) $month;
		$currentYear = $date->getYear();
		$currentMonth = $date->getMonth();
	//	info(' - Current Month: '.$currentMonth.' for Year: '.$currentYear);
		if($year < $currentYear)
		{
			while($year!=$currentYear)
			{
				for($i=$month;$i<13;$i++)
					$openMonths[$year][$i] = $i;
				$year++;
				$month = 1;
			}
			for($i=1;$i < $currentMonth; $i++) // Always one month in the past
				$openMonths[$currentYear][$i] = $i;
		}
		else
			for($i=$month;$i < $currentMonth; $i++)
				$openMonths[$currentYear][$i] = $i;

		return $openMonths;
	}

	public static function getDeclarationTypes()
	{
		return array(DECLARATION_OPEN, DECLARATION_NORMAL, DECLARATION_FORFEIT, DECLARATION_LATE, DECLARATION_DECLARED, DECLARATION_RESUBMIT, DECLARATION_CREDIT);
	}

	/**#@-*/
	/**
	 * @access private
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_DECLARATION';
		$this->objectmap['primary']['id'] = 'DECLARATION_ID';
		$this->objectmap['properties']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
		$this->objectmap['properties']['user->id'] = 'USER_ID';
		$this->objectmap['properties']['product->id'] = 'PRODUCT_ID';
		$this->objectmap['properties']['document->id'] = 'DOCUMENT_ID';
		$this->objectmap['properties']['month'] = 'MONTH';
		$this->objectmap['properties']['year'] = 'YEAR';
		$this->objectmap['properties']['created'] = 'CREATED';
		$this->objectmap['properties']['last_update'] = 'LAST_UPDATE';
		$this->objectmap['properties']['is_exported'] = 'IS_EXPORTED';
		$this->objectmap['properties']['has_critical_changes'] = 'HAS_CRITICAL_CHANGES';
		$this->objectmap['properties']['export_date'] = 'EXPORT_DATE';
		$this->objectmap['properties']['parent_declaration->id'] = 'PARENT_DECLARATION_ID';
	}

	/**
	 * For the requested Credit/Debit IDs by the operator
	 */
	function insertRequestedInvoice($type,$invoice_id,$year)
	{
		$id = $this->_getNewId($type,$this->crso->relation->id,$year);
		$sql = "INSERT INTO TBL_DECLARATION_".$type."
			(CRSO_RELATION_ID,DECLARATION_".$type."_ID,REQUESTED_INVOICE_ID,`YEAR`)
			VALUES(?,?,?,?)";
		$q = $this->db->query($sql);
		$q->bind_values($this->crso->relation->id,$id,$invoice_id,$year);
		return $q->execute();
	}

	/**
	 * After the 'clone' method is called, this will be executed by php5
	 * Clone all the lines in the object so no errors occur
	 */
	function __clone()
	{
		foreach($this->lines as $index => $line)
			$this->lines[$index] = clone $line;
	}

	public static function getLastTriggeredMonth($db,$crso_id,$participant_id,$year=null, $product_id=null)
	{
		$date =& currentDate();
		$year || $year = $date->getYear();

		$sql = "SELECT MAX(`MONTH`) as MAXMONTH
			FROM TBL_DECLARATION
			WHERE CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ? AND `YEAR` = ?
				AND PRODUCT_ID= ? ";
		$q = $db->query($sql);
		$q->bind_values($crso_id,$participant_id,$year, $product_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return 0;
		return $r->MAXMONTH;
	}

	function hasManualDeclarations($participant_id=null,$crso_id=null,$product_id=null,$year=null)
	{
		$participant_id || $participant_id = $this->participant->relation->id;
		$crso_id || $crso_id = $this->crso->relation->id;
		$product_id || $product_id = $this->product->id;
		$year || $year = $this->year;
		$year = (int) $year;

		require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
		$cpp = new CRSOParticipantProduct($this->db);
		$cpp->set($crso_id,$participant_id,$product_id);
		if($cpp->status!=PARTICIPANT_STATUS_PARTICIPANT_READY)
			return false;

		$date = currentDate();
		$currentYear = $date->getYear();
		$maxMonth = ($currentYear == $year)?($date->getMonth()-1):12;
		/*if(($currentYear == ($year+1))&& $date->getMonth()==1)
			$maxMonth = 11;*/

		$startdate = explode('-',$cpp->obligation_start_date);

		$minMonth = 1;
		if($startdate[0] > $year)
			return false;

		if($startdate[0]==$year)
			$minMonth = (int)$startdate[1];

		$sql = "SELECT DISTINCT `MONTH` FROM TBL_DECLARATION
			WHERE PARTICIPANT_RELATION_ID = ?
				AND CRSO_RELATION_ID = ?
				AND PRODUCT_ID = ?
				AND `YEAR` = ?
				AND `MONTH` BETWEEN ? AND ?";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id,$crso_id,$product_id,$year,$minMonth,$maxMonth);
		$q->execute();
		$rs = $q->fetchAll();
		if((count($rs))==(($maxMonth-$minMonth)+1))
			return false;
		$months = array();
		foreach($rs as $r)
			$months[$r->MONTH] = $r->MONTH;

		$rv = array();
		for($i = $minMonth;$i<=$maxMonth;$i++)
			if(!isset($months[$i]))
				$rv[$i] = $i;

		return $rv;
	}


        function hasOneOutstandingForfeit($part_id, $crso_id, $product_id)
        {
            $sql = 'SELECT D.DECLARATION_ID
                FROM TBL_DECLARATION AS D
                INNER JOIN
                TBL_DECLARATION_LINE AS DL ON D.DECLARATION_ID = DL.DECLARATION_ID
                LEFT JOIN
                TBL_DECLARATION AS D2 ON D.DECLARATION_ID = D2.PARENT_DECLARATION_ID
                WHERE ISNULL(D2.DECLARATION_ID) AND
                D.PARTICIPANT_RELATION_ID = ?
                AND D.CRSO_RELATION_ID = ?
                AND D.PRODUCT_ID = ?
                GROUP BY D.DECLARATION_ID
                HAVING BIT_OR(DL.STATUS) = 4';

            $q = $this->db->query($sql);
            $q->bind_values($part_id, $crso_id, $product_id);
            $q->execute();
            $rows = $q->fetchAll();
            return count($rows) == 1;
        }

	function getGlAccount($declaration_id, $status) {

		$this->set($declaration_id);

		$this->parent_declaration && $this->parent_declaration->set();

		switch ($status)
		{
			case DECLARATION_NORMAL:
			case DECLARATION_DECLARED:
				$handle = 'DT_MONTHLY_DECLARATION';
			break;
			case DECLARATION_RESUBMIT:
				$handle = 'DT_RESUBMISSION';
			break;
			case DECLARATION_FORFEIT:
				$handle = 'DT_FORFEIT_DECLARATION';
			break;
			case DECLARATION_CREDIT:
				if ($this->parent_declaration && (($this->parent_declaration->aggregated_status & DECLARATION_FORFEIT) == DECLARATION_FORFEIT))
					$handle = 'DT_NEGATIVE_FORFEIT_DECLARATION';
				else
					$handle = 'DT_NEGATIVE_DECLARATION';
			break;
			case DECLARATION_LATE:
				$handle = 'DT_LATE_DECLARATION';
			break;

			/* TODO:
			DT_GUARANTEE_DOCUMENT_150:
			DT_GUARANTEE_DOCUMENT_200:
			DT_GUARANTEE_REIMBURSEMENT_DOCUMENT:
			*/

			case DECLARATION_OPEN:
				$handle = '';
			break;

			default:
				throw new ELCExporterException("Incorrect declaration status: ".$status);
		}

		if (!$handle) {
			return '';
		}

		$sql = "SELECT GL_ACCOUNT
			FROM TBL_CRSO_GLACCOUNT gc
			INNER JOIN TBL_DECLARATION_TYPE dt
				ON gc.DECLARATION_TYPE_ID = dt.DECLARATION_TYPE_ID
			WHERE gc.CRSO_RELATION_ID = ?
				AND dt.HANDLE = ?
		";

		$q = $this->db->query($sql);
		$q->bind_values(array($this->crso->relation->id, $handle));
		$q->execute();
		if ($r = $q->fetch()) {
			return $r->GL_ACCOUNT;
		}

		return '';
	}
}
/**
 * @package ELC
 * @subpackage Declaration
 * @author Marten Koedam <marten@basesoft.com>
 * @since 2006-13-02
 * @copyright 2006 BaseSoft B.V.
 */
class DeclarationLine extends AbstractObject
{
	/**
	 * @var Declaration $declaration
	 */
	var $declaration;
	/**
	 * @var int $line_index
	 */
	var $line_index;
	/**
	 * @var Brand $brand
	 */
	var $brand;
	/**
	 * @var Trust $trust
	 */
	var $trust;
	/**
	 * @var ProductCategory $product_category
	 */
	var $product_category;
	/**
	 * @var CRSOProductDimension $dimension
	 */
	var $dimension;
	/**
	 * @var int $weight
	 */
	var $weight;
	/**
	 * @var float $weight_dec
	 */
	var $weight_dec;
	/**
	 * @var int $quantity
	 */
	var $quantity;
	/**
	 * @var int $status
	 */
	var $status;
	/**
	 * @var Datetime $created
	 */
	var $created;
	/**
	 * @var Timestamp $last_update
	 */
	var $last_update;
	/**
	 * @param int $id
	 * @param int $index
	 */
	function set($id='',$index='')
	{
		$id || $id = $this->declaration->id;
		$index || $index = $this->line_index;

		$sql = "SELECT BRAND_ID, TRUST_ID, PRODUCT_CATEGORY_ID, CPD_ID, WEIGHT, WEIGHT_DEC,
				QUANTITY, STATUS, CREATED, LAST_UPDATE
			FROM TBL_DECLARATION_LINE
			WHERE DECLARATION_ID = ?
				AND LINE_INDEX = ?";

		$q = $this->db->query($sql);
		$q->bind_values($id, $index);
		if(!$q->execute(true))
			return false; // Trigger Error
		if(!($r = $q->fetch()))
			return false;

		require_once(ELC_CLASSES.'Brand.php');
		require_once(ELC_CLASSES.'ProductCategory.php');
		require_once(ELC_CLASSES.'Trust.php');
		require_once(ELC_CLASSES.'CRSOProductDimension.php');

		$this->line_index = $index;
		$this->brand = ($r->BRAND_ID)?new Brand($this->db,$r->BRAND_ID):'';
		$this->product_category = ($r->PRODUCT_CATEGORY_ID)?new ProductCategory($this->db,$r->PRODUCT_CATEGORY_ID):'';
		$this->dimension = ($r->CPD_ID)?new CRSOProductDimension($this->db,$r->CPD_ID):'';
		$this->trust = ($r->TRUST_ID)?new Trust($this->db,$r->TRUST_ID):'';

		$this->weight = is_numeric($r->WEIGHT)?(int)$r->WEIGHT:null;
		$this->weight_dec = is_numeric($r->WEIGHT_DEC)?$r->WEIGHT_DEC:null;
		$this->quantity = is_numeric($r->QUANTITY)?(int)$r->QUANTITY:null;
		$this->status = $r->STATUS;
		$this->created = $r->CREATED;
		$this->last_update = $r->LAST_UPDATE;

		return $this->complete = true;
	}
	/**
	 * Used for the create date.
	 */
	function insert()
	{
		$this->created = date('Y-m-d H:i:s');
		unset($this->last_update);
		return parent::insert();
	}
	/**
	 * Delete a line
	 * @param int $id
	 * @param int $index
	 */
	function delete($id='',$index='')
	{
		$id || $id = $this->declaration->id;
		$index || $index = $this->line_index;

		$sql = "DELETE FROM TBL_DECLARATION_LINE
			WHERE DECLARATION_ID = ?
				AND LINE_INDEX = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id,$index);
		return $q->execute();
	}
	/**
	 * Formats a line for the overview interface.
	 */
	function formatLine()
	{
		if(!$this->isComplete())
			$this->set();

		if ($this->trust)
			$this->trust->set();

		$this->product_category && $this->product_category->set();

		list($quantity,$weight, $weight_dec) = $this->getQuantityAndWeight(true);
		return array('ALTER'=>$this->_alterLine(),
				'TRANSACTION_ID' => (int)$this->line_index,
				'SUBROGATOR'=> ($this->trust) ? $this->trust->subrogator->relation->id : '',
				'TRUST_ID'=> ($this->trust) ? $this->trust->id : '',
				'VAT_NO' => '',
				'QUANTITY' => $quantity,
				'FEE' => $this->_getFee(),
				'BRAND_NAME' =>  $this->brand?(int)$this->brand->id:null,
				'PRODUCT_CATEGORY'=> $this->product_category?(int)$this->product_category->id:null,
				'PRODUCT_CATEGORY_DESCRIPTION'=> $this->product_category?$this->product_category->description:null,
				'CPD_ID'=> $this->dimension?(int)$this->dimension->id:null,
				'WEIGHT'=>$weight,
				'WEIGHT_DEC'=>$weight_dec,
				'TRANSACTION_STATUS' => (int)$this->status,
				'CHECKBOX'=>false);
	}
	/**
	 * This determines the next status of the declaration line
	 * @return int $status `STATUS`
	 */
	function nextStatus()
	{
		if(!$this->isComplete())
			$this->set();
		switch($this->status)
		{
			case DECLARATION_OPEN:
			case DECLARATION_DECLARED:
				return DECLARATION_DECLARED;
			case DECLARATION_LATE:
				if(!$this->declaration->is_exported)
					return DECLARATION_LATE;
			case DECLARATION_RESUBMIT:
			case DECLARATION_NORMAL:
				return DECLARATION_RESUBMIT;
			case DECLARATION_FORFEIT:
				return DECLARATION_LATE;
		}
	}
	/**
	 * Gets the last Exported quantity for the current line
	 * @return int $quantity
	 */
	function getLastExportedQuantity()
	{
		list($quantity,,) = $this->_getLast(true);
		return $quantity;
	}
	/**
	 * Gets the last Exported quantity and the status of the current line
	 * @return Array (QUANTITY,`STATUS`)
	 */
	function getLastExportedQuantityAndStatus()
	{
		list($quantity,$weight,$status,$weight_dec) = $this->_getLast(true); // true = only_exported
		return array($quantity,$status,$weight,$weight_dec);
	}
	/**#@+
	 * @access private
	 */
	/**
	 * Gets the fee for the current crso and declaration->month
	 */
	function _getFee()
	{
		require_once(ELC_CLASSES.'ProductFee.php');
		$pcf = new ProductCategoryFee($this->db);
		$datestring = $this->declaration->year.'-'.$this->declaration->month.'-01 00:00:00';
		return $pcf->getFee(($this->product_category?$this->product_category->id:null), $this->declaration->crso->relation->id, $datestring, $this->declaration->product->id);
	}
	/**
	 * Can this line be altered or not.
	 * @return bool
	 */
	function _alterLine()
	{
		if(!$this->isComplete())
			$this->set();
		$aggrstatus = $this->declaration->aggregated_status;
		switch($this->status)
		{
			case DECLARATION_OPEN:
			case DECLARATION_DECLARED:
			case DECLARATION_FORFEIT:
			case DECLARATION_RESUBMIT:
				return true;
			case DECLARATION_LATE:
			case DECLARATION_NORMAL:
				return !($aggrstatus & DECLARATION_FORFEIT);
		}
	}
	/**
	 * @param bool $only_exported Only select the ones that are exported
	 * @return Array (QUANTITY,WEIGHT,`STATUS`)
	 */
	function _getLast($only_exported=false,$noForfeitAndCredit=false)
	{
		// check if we have a quantity, if so break!
		// else check if parent_id is available and try there
		// if parent_id is NULL break

		if(!$this->declaration->parent_declaration)
			return array(null,null,DECLARATION_OPEN,null);
		$parent_id = $this->declaration->parent_declaration->id;
		while (true)
		{
			$sql = "SELECT dl.QUANTITY, dl.WEIGHT, dl.WEIGHT_DEC, dl.STATUS, d.PARENT_DECLARATION_ID
				FROM TBL_DECLARATION_LINE as dl
				INNER JOIN TBL_DECLARATION as d
					ON dl.DECLARATION_ID = d.DECLARATION_ID
				WHERE d.DECLARATION_ID = ?
					AND dl.LINE_INDEX = ? ".
					($only_exported?' AND NOT ISNULL(d.IS_EXPORTED) AND d.IS_EXPORTED != 0 ':'').
					($noForfeitAndCredit?' AND dl.STATUS NOT IN ( '.DECLARATION_FORFEIT.', '.DECLARATION_CREDIT.') ':'').
				" ORDER BY d.CREATED DESC";
			$q = $this->db->query($sql);
			$q->bind_values($parent_id,
				$this->line_index);

			$q->execute();

			if ( !($r = $q->fetch()) )
				break;


			if (is_numeric($r->QUANTITY))
				break;

			if (is_numeric($r->PARENT_DECLARATION_ID))
				$parent_id = $r->PARENT_DECLARATION_ID;
			else
				break;
		}

		if ( !$r )
			return array(null,null,DECLARATION_OPEN,null);// The status cannot be NULL
		return array((int)$r->QUANTITY,(is_numeric($r->WEIGHT)?(int)$r->WEIGHT:null),$r->STATUS,(is_numeric($r->WEIGHT_DEC)?$r->WEIGHT_DEC:null));
	}
	/**#@-*/
	/**
	 * Gets the last quantity and weight
	 * @return Array(QUANTITY,`STATUS`)
	 */
	function getQuantityAndWeight($noForfeitAndCredit=false)
	{
		if(!$this->isComplete())
			$this->set();
		if(is_numeric($this->quantity))
			return array($this->quantity,$this->weight,$this->weight_dec);

		list($quantity,$weight,$bitbucket,$weight_dec) = $this->_getLast(false,$noForfeitAndCredit);
		return array($quantity,$weight,$weight_dec);
	}
	/**
	 * Returns all the lines for the current Declaration. Usually 1 line, depends on CountrySettings
	 * @param int $id
	 */
	function getLines($id)
	{
		$sql = "SELECT LINE_INDEX
			FROM TBL_DECLARATION_LINE
			WHERE DECLARATION_ID = ?
			ORDER BY LINE_INDEX";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		return $q;
	}
	/**
	 * Verifies if a line can be updated or not .
	 * @param int $id
	 * @param int $line_index
	 */
	function validateLineUpdate($id='',$line_index='')
	{
		$id || $id = $this->declaration->id;
		$line_index || $line_index = $this->line_index;

		if($this->declaration->isForfeit()&& $this->status == DECLARATION_FORFEIT)
			return true;
		if(!$this->declaration->is_exported)
			return true;
		if(($this->status & DECLARATION_LATE)==DECLARATION_LATE)
			return true;
		return false;
	}
	/**
	 * @access private
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_DECLARATION_LINE';
		$this->objectmap['primary']['declaration->id'] = 'DECLARATION_ID';
		$this->objectmap['primary']['line_index'] = 'LINE_INDEX';
		$this->objectmap['properties']['brand->id'] = 'BRAND_ID';
		$this->objectmap['properties']['trust->id'] = 'TRUST_ID';
		$this->objectmap['properties']['product_category->id'] = 'PRODUCT_CATEGORY_ID';
		$this->objectmap['properties']['dimension->id'] = 'CPD_ID';
		$this->objectmap['properties']['weight'] = 'WEIGHT';
		$this->objectmap['properties']['weight_dec'] = 'WEIGHT_DEC';
		$this->objectmap['properties']['quantity'] = 'QUANTITY';
		$this->objectmap['properties']['status'] = 'STATUS';
		$this->objectmap['properties']['created'] = 'CREATED';
		$this->objectmap['properties']['last_update'] = 'LAST_UPDATE';
	}
}

/**#@+
 * BitWise Defines of the different Declaration Stati
 */
define('DECLARATION_OPEN',1);
define('DECLARATION_NORMAL',2);
define('DECLARATION_FORFEIT',4);
define('DECLARATION_LATE',8);
define('DECLARATION_DECLARED',16);
define('DECLARATION_RESUBMIT',32);
define('DECLARATION_CREDIT',64);
/**#@-*/
define('DECLARATION_EXPORT_PENDING', 1);
define('DECLARATION_EXPORT_EXPORTED', 2);
// This is used to exclude Declarations which only have changes in weight and the country setting to omit them from export is enabled.
define('DECLARATION_EXPORT_OMITTED', 3);

define('DECLARATION_HAS_NO_CRITICAL_CHANGES', 0);
define('DECLARATION_HAS_CRITICAL_CHANGES', 1);

?>
