<?php
require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES.'Declaration.php');
require_once(ELC_CLASSES.'ProductComponent.php');
/**
 * @package WEEE
 * @subpackage Declaration
 * @since 27-nov-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class DeclarationComponent
{
	var $db;
	protected $_lines = array();
	protected $_components = array();

	function DeclarationComponent($db)
	{
		$this->db = $db;
	}
	/**
	 * Get all the lines for a declaration
	 */
	function set($declaration_id)
	{
		$declaration = new Declaration($this->db,$declaration_id);

		$sql = "SELECT DECLARATION_ID, DC.COMPONENT_ID, QUANTITY, WEIGHT
			FROM TBL_DECLARATION_COMPONENT_LINE as DC
			INNER JOIN TBL_PRODUCT_COMPONENT as PC
				ON DC.COMPONENT_ID = PC.COMPONENT_ID
			WHERE DECLARATION_ID = ?
			ORDER BY PC.`COMPONENT_ID`";
		$q = $this->db->query($sql);
		$q->bind_values($declaration_id);
		$q->execute();
		$rs = $q->fetchAll();
		if(!count($rs))
			return false;

		foreach($rs as $r)
		{
			$line = array('quantity'=>$r->QUANTITY,'weight'=>$r->WEIGHT,'declaration'=>$declaration);
			$line['component'] = new ProductComponent($this->db,$r->COMPONENT_ID);
			$this->addLine($line);
		}

		return true;
	}

	function addLine($line,$fill=false)
	{
		$this->_lines[] = new DeclarationComponentLine($this->db,$line,$fill);
	}

	function getLines()
	{
		return $this->_lines;
	}

	function clearLines()
	{
		$this->_lines = array();
	}

	function getFormLines()
	{
		$rv = array();
		foreach($this->_lines as $line)
		{
			$line->component->set();
			$rv[] = array(
				'component'=>$line->component,
				'quantity'=>$line->quantity,
				'weight'=>$line->weight
				);
		}
		return $rv;
	}

	/**
	 * @todo trigger some sort of notification if No Rows!!
	 */
	function setCRSOComponents($crso_id,$product_id,$force=false)
	{
		if(isset($this->_components[$crso_id][$product_id])&&!$force)
			return;
		$c = new ProductComponent($this->db);
		$q = $c->getByProductCRSO($crso_id,$product_id);

		while($r = $q->fetch())
			$this->_components[$crso_id][$product_id][$r->COMPONENT_ID] = new ProductComponent($this->db,$r->COMPONENT_ID);
	}

	function triggerNewDeclarationComponents($crso_id,$declaration)
	{
		if(!isset($this->_components[$crso_id][$declaration->product->id]))
			return false;

		foreach($this->_components[$crso_id][$declaration->product->id] as $component)
		{
			$args['declaration'] =& $declaration;
			$args['component'] =& $component;
			$line = new DeclarationComponentLine($this->db,$args);
			$line->insert();
		}
		return true;
	}

	function updateDeclarationComponents($declaration,$lines)
	{
		foreach($lines as $cid => $line_C)
		{
			$line = new DeclarationComponentLine($this->db,$line_C);
			$line->declaration =& $declaration;
			$line->component = new ProductComponent($this->db,$cid);
			$line->update();
		}
	}

	function copyLines($from_decl,$to_decl,$makeNegative=false)
	{
		$this->set($from_decl->id);
		info(sprintf('Copying Components from Decl [%s] To Decl [%s]',$from_decl->id,$to_decl->id));
		foreach($this->_lines as $line)
		{
			$line->declaration =& $to_decl;
			//$line->objectmap['inserttype'] = 'REPLACE';
			if($makeNegative)
			{
				$line->quantity = is_numeric($line->quantity)?(0-$line->quantity):$line->quantity;
				$line->weight = is_numeric($line->weight)?(0-$line->weight):$line->weight;
			}
			$line->insert();
		}
	}
}

class DeclarationComponentLine extends AbstractObject
{
	var $declaration;
	var $component;
	var $quantity;
	var $weight;

	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_DECLARATION_COMPONENT_LINE';
		$this->objectmap['primary']['declaration->id'] = 'DECLARATION_ID';
		$this->objectmap['primary']['component->id'] = 'COMPONENT_ID';
		$this->objectmap['properties']['quantity'] = 'QUANTITY';
		$this->objectmap['properties']['weight'] = 'WEIGHT';
	}
}
?>
