<?php
/**
 * DeclarationController
 */

require_once(ELC_CLASSES.'WEEEController.php');
require_once(ELC_CLASSES.'AnnualConfirmation.php');
require_once(ELC_CLASSES.'RelationUser.php');
require_once(ELC_CLASSES.'ProductCategory.php');
require_once(ELC_CLASSES.'CRSOProductDimension.php');
require_once(ELC_CLASSES.'ProductComponent.php');
require_once(ELC_CLASSES.'Declaration.php');
require_once(ELC_CLASSES.'DeclarationComponent.php');

define('ERR_DECL_XML_PARTICIPANT_DENIED', 1);
define('ERR_DECL_XML_CRSO_PRODUCT_DENIED', 2);
define('ERR_DECL_XML_FIELD_REQUIRED', 3);
define('ERR_DECL_XML_FIELD_PROVIDED', 4);
define('ERR_DECL_XML_FIELD_UNKNOWN', 5);
define('ERR_DECL_XML_LINE_MISMATCH', 6);
define('ERR_DECL_XML_LATE_EXTRA_LINES', 7);
define('ERR_DECL_XML_LINES_DUPLICATE', 8);
define('ERR_DECL_XML_NEGATIVE_QUANTITY', 9);
define('ERR_DECL_XML_AC_CLOSED', 10);
define('ERR_DECL_XML_EMPTY_COMPONENTS', 10);
define('ERR_DECL_XML_MISSING_COMPONENTS', 11);
define('ERR_DECL_XML_NO_VALIDATE', 12);
define('ERR_DECL_XML_NEGATIVE_OR_EMPTY_WEIGHT', 13);
define('ERR_DECL_XML_NON_ZERO_WEIGHT', 14);
define('ERR_DECL_XML_NO_DECLARE_TRUST', 15);

class DeclarationController extends WEEEController
{
	protected $lang;

	/**
	 * Prepares an XML file for all open declarations (including forfeits) for the current logged in user.
	 * @return unknown_type
	 */
	public function prepareOpenDeclarationXML() {
		require_once(ELC_CLASSES.'Participant.php');

		$declaration = new Declaration($this->db);
		$sql = "SELECT CPP.PARTICIPANT_RELATION_ID as part_id, CPP.CRSO_RELATION_ID as crso_id, CPP.PRODUCT_ID as product_id
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CPP
			INNER JOIN TBL_RELATION_USER as RUI
				ON CPP.PARTICIPANT_RELATION_ID = RUI.RELATION_ID
			WHERE RUI.USER_ID = ? AND (RUI.CLEARANCE_LEVEL & ? = ? OR RUI.IS_ADMIN = 1) AND RUI.STATUS = 1";
		$q = $this->db->query($sql);
		$q->bind_values($this->user->id, ClearanceFeeRegistration, ClearanceFeeRegistration);
		$q->execute();

		$dom = new DOMDocument('1.0', 'UTF-8');
		$root = $dom->createElement('declarations');
		$dom->appendChild($root);

		while($r = $q->fetch()) {

			$declarations = $declaration->getOpenDeclarationOverview($r->part_id, $r->crso_id, $r->product_id);
			while($decl = $declarations->fetch()) {

				$declNode = $dom->createElement('declaration');

				$declaration->set($decl->DECLARATION_ID);

				$country = new WEEECountry($this->db, $declaration->crso->relation->getCountry(), true);

				$yearNode = $dom->createElement('year');
				$yearNode->appendChild($dom->createTextNode($declaration->year));
				$declNode->appendChild($yearNode);

				$monthNode = $dom->createElement('month');
				$monthNode->appendChild($dom->createTextNode($declaration->month));
				$declNode->appendChild($monthNode);

				$crsoNode = $dom->createElement('crso');
				$crsoNode->appendChild($dom->createTextNode($declaration->crso->relation->id));
				$declNode->appendChild($crsoNode);

				$participantNode = $dom->createElement('participant');
				$participantNode->appendChild($dom->createTextNode($declaration->participant->relation->id));
				$declNode->appendChild($participantNode);

				$productNode = $dom->createElement('product');
				$productNode->appendChild($dom->createTextNode($declaration->product->id));
				$declNode->appendChild($productNode);

				$lines = $dom->createElement('lines');
				$declNode->appendChild($lines);
				foreach($declaration->lines as $line) {

					$line->set();

					// dont add declaration if trust is found in any of the lines
					if ($line->trust && $line->trust->id)
						break 2; // exit while

					$lineNode = $dom->createElement('line');
					$lines->appendChild($lineNode);

					$quantity = $dom->createElement('quantity');
					$quantity->appendChild($dom->createTextNode($line->quantity));
					$lineNode->appendChild($quantity);

					if($country->has(HasDeclOfWeight, $declaration->product->id)) {
						$weight = $dom->createElement('weight');
						$weight->appendChild($dom->createTextNode($line->weight));
						$lineNode->appendChild($weight);
					}

					if($country->has(HasDeclOfWeightWithDecimals, $declaration->product->id)) {
						$weight_dec = $dom->createElement('weight_dec');
						$weight_dec->appendChild($dom->createTextNode($line->weight_dec));
						$lineNode->appendChild($weight_dec);
					}

					if($country->has(HasDeclByBrand, $declaration->product->id)) {
						$brand = $dom->createElement('brand');
						$lineNode->appendChild($brand);
						$brand->appendChild($dom->createTextNode($line->brand->id));
					}


					if($country->has(HasDeclByCategory, $declaration->product->id)) {
						$pc = $dom->createElement('product_category');
						$lineNode->appendChild($pc);
						$pc->appendChild($dom->createTextNode($line->product_category->id));
					}

					if($country->has(HasDeclarationDimension, $declaration->product->id)) {
						$dimension = $dom->createElement('dimension');
						$lineNode->appendChild($dimension);
						$dimension->appendChild($dom->createTextNode($line->dimension->id));
					}

				}
				/**
				 * Component declaration does NOT have WEIGHT_DEC
				 */
				if($country->has(HasDeclarationComponent, $declaration->product->id)) {

					$components = $dom->createElement('components');
					$declNode->appendChild($components);

					$dc = new DeclarationComponent($this->db);
					$dc->set($declaration->id); // set by declaration
					foreach($dc->getLines() as $line) {

						$component = $dom->createElement('component');
						$components->appendChild($component);

						$cid = $dom->createElement('component_id');
						$cid->appendChild($dom->createTextNode($line->component->id));
						$component->appendChild($cid);

						$quantity = $dom->createElement('quantity');
						$quantity->appendChild($dom->createTextNode($line->quantity));
						$component->appendChild($quantity);

						if($country->has(HasDeclOfWeight, $declaration->product->id)) {
							$weight = $dom->createElement('weight');
							$weight->appendChild($dom->createTextNode($line->weight));
							$component->appendChild($weight);
						}
					}
				}

				// add declNode to document
				$root->appendChild($declNode);
			}
		}
		$dom->formatOutput = true;
		return $dom->saveXML();


	}

	/**
	 * Parses the $xmlContents for declarations, validates them and inserts
	 * The return value is the a list of modified declaration objects (but rolled back in db if dryRun)
	 *
	 * @param unknown_type $xmlContents
	 * @param unknown_type $dryRun
	 */
	public function declareXml($xmlContents, $dryRun=true) {

		$langIf = IWRegistry::registry('langIf');
		$this->lang = array(
			ERR_DECL_XML_PARTICIPANT_DENIED => $langIf->getText('Permission denied declaring for participant'),
			ERR_DECL_XML_CRSO_PRODUCT_DENIED => $langIf->getText('Permission denied declaring for crso/product'),
			ERR_DECL_XML_FIELD_REQUIRED => $langIf->getText('Declaration by %s is required'),
			ERR_DECL_XML_FIELD_PROVIDED => $langIf->getText('Declaration by %s must be omitted'),
			ERR_DECL_XML_FIELD_UNKNOWN => $langIf->getText('The %s provided does not exist'),
			ERR_DECL_XML_LINE_MISMATCH => $langIf->getText('Line specification (%s) does not match database'),
			ERR_DECL_XML_LATE_EXTRA_LINES => $langIf->getText('A late declaration cannot contain extra lines'),
			ERR_DECL_XML_LINES_DUPLICATE => $langIf->getText('Declaration contains duplicate lines'),
			ERR_DECL_XML_NEGATIVE_QUANTITY => $langIf->getText('Quantity cant be negative'),
			ERR_DECL_XML_AC_CLOSED => $langIf->getText('The Annual Confirmation for this declaration is closed'),
			ERR_DECL_XML_MISSING_COMPONENTS => $langIf->getText('All declaration components must be supplied'),
			ERR_DECL_XML_NO_VALIDATE => $langIf->getText('XML does not validate with DeclarationSubmit.xsd: "%s" on line %s'),
			ERR_DECL_XML_NEGATIVE_OR_EMPTY_WEIGHT => $langIf->getText('Weight cant be negative or empty'),
			ERR_DECL_XML_NON_ZERO_WEIGHT => $langIf->getText('Weight cant be zero if quantity is not zero'),
			ERR_DECL_XML_NO_DECLARE_TRUST => $langIf->getText('Cannot declare for subrogation'),
		);

		// load xml document from string
		// http://nl.php.net/manual/en/class.domdocument.php
		$dom = new DOMDocument();
		@$dom->loadXML($xmlContents);

		// validate xml tru xsd or exception out with error details
		if (!@$dom->schemaValidate(SITE_HOMEDIR . 'scripts/elc/participant/xsd/DeclarationSubmit.xsd')) {
			$error = libxml_get_last_error();
			$msg = sprintf($this->lang[ERR_DECL_XML_NO_VALIDATE], str_replace("\n", '', $error->message), $error->line);
			throw new DeclarationXmlValidationException($msg);
		}

		// gets filled while parsing xml and contains all declaration info
		$declarations = array();

		// loop all declarations.. we know they are the childNodes of <declarations>
		foreach ($dom->getElementsByTagName('declaration') as $dIndex => $dElement) {

			$dObj = array();
			$dObj['year'] = (int) $this->_getXmlNodeValue($dElement, 'year');
			$dObj['month'] = (int) $this->_getXmlNodeValue($dElement, 'month');
			$dObj['crsoId'] = (int) $this->_getXmlNodeValue($dElement, 'crso');
			$dObj['participantId'] = (int) $this->_getXmlNodeValue($dElement, 'participant');
			$dObj['productId'] = (int) $this->_getXmlNodeValue($dElement, 'product');
			$dObj['lines'] = array();
			$dObj['components'] = array();

			// in case of failure this is the context
			$this->failureContext = array();
			$this->failureContext['declaration'] = $dObj;

			// check if the current logged in user has access to declare to the participant
			$rui = new RelationUser($this->db);
			if (!$rui->set($this->user->id, $dObj['participantId'])) {
				$this->addFailure(ERR_DECL_XML_PARTICIPANT_DENIED);
				continue;
			}

			// check if the crso/participant/product combo is valid
			require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
			$cpp = new CRSOParticipantProduct($this->db);
			if (!$cpp->set($dObj['crsoId'], $dObj['participantId'], $dObj['productId'])) {
				$this->addFailure(ERR_DECL_XML_CRSO_PRODUCT_DENIED);
				continue;
			}

			// get the current declaration in the database (else it will trow an exception)
			// this object is also used to check line validation
			$dbDeclaration = $this->getLastDeclaration($dObj['year'], $dObj['month'], $dObj['crsoId'], $dObj['participantId'], $dObj['productId']);

			// check if the AC for that year isnt closed
			$ac = new AnnualConfirmation($this->db);
			$ac->setByParticipantCRSOAndYear($dbDeclaration->participant->relation->id, $dbDeclaration->crso->relation->id, $dbDeclaration->year, $dbDeclaration->product->id);
			if($ac->status == ANNUAL_CONFIRMATION_CLOSED) {
				$this->addFailure(ERR_DECL_XML_AC_CLOSED);
			}

			// country settings is needed below in the lines loop
			$crso = new CRSO($this->db);
			$country = new Country($this->db, $crso->getCountry($dObj['crsoId']), true);

			// fill participant brands for check below
			$brands = array();
			$pb = new ParticipantBrand($this->db);
			$q = $pb->getBrands($dObj['participantId'], $dObj['crsoId'], $dObj['productId']);
			while ($row = $q->fetch()) {
				$brands[] = $row->BRAND_ID;
			}

			// fill product categories
			$categories = array();
			$pc = new ProductCategory($this->db);
			$q = $pc->getProductCategories($dObj['crsoId'], $dObj['productId']);
			while ($row = $q->fetch()) {
				$categories[] = $row->PRODUCT_CATEGORY_ID;
			}

			// fill dimensions
			$dimensions = array();
			$d = new CRSOProductDimension($this->db);
			$q = $d->getCrsoProductDimensions($dObj['crsoId'], $dObj['productId']);
			while ($row = $q->fetch()) {
				$dimensions[] = $row->CPD_ID;
			}

			// set up a list of line checks
			$checks = array(
				array('field'=>'brandId', 'csetting'=>HasDeclByBrand, 'list'=>$brands),
				array('field'=>'categoryId', 'csetting'=>HasDeclByCategory, 'list'=>$categories),
				array('field'=>'dimensionId', 'csetting'=>HasDeclarationDimension, 'list'=>$dimensions),
				array('field'=>'weight', 'csetting'=>HasDeclOfWeight),
				array('field'=>'weight_dec', 'csetting'=>HasDeclOfWeightWithDecimals),
			);

			// loop xml declaration lines
			foreach ($dElement->getElementsByTagName('line') as $lIndex => $lElement) {

				// get values from xml. type checking is performed by the xsd
				$lineObj = array(
					'quantity'=> $this->_getXmlNodeValue($lElement, 'quantity'),
					'weight' => $this->_getXmlNodeValue($lElement, 'weight', true),
					'weight_dec' => $this->_getXmlNodeValue($lElement, 'weight_dec', true),
					'brandId' => $this->_getXmlNodeValue($lElement, 'brand', true),
					'categoryId' => $this->_getXmlNodeValue($lElement, 'product_category', true),
					'dimensionId' => $this->_getXmlNodeValue($lElement, 'dimension', true)
				);

				if ($lineObj['quantity'] < 0) {
					$this->addFailure(ERR_DECL_XML_NEGATIVE_QUANTITY);
				}

				// in case of emergency have context
				$this->failureContext['line'] = $lineObj;

				// check each field if its applicable
				foreach ($checks as $check) {

					// check if the country has the setting and if its not supplied
					if((in_array($check['field'], array('weight', 'weight_dec'))) && $country->has($check['csetting'], $dObj['productId']) && ($lineObj[$check['field']] < 0)) {
						$this->addFailure(ERR_DECL_XML_NEGATIVE_OR_EMPTY_WEIGHT);
					} elseif((in_array($check['field'], array('weight', 'weight_dec'))) && $country->has($check['csetting'], $dObj['productId']) && ($lineObj[$check['field']] == 0) && ($lineObj['quantity'] > 0)) {
						$this->addFailure(ERR_DECL_XML_NON_ZERO_WEIGHT);
					} elseif ((!in_array($check['field'], array('weight', 'weight_dec'))) && $country->has($check['csetting'], $dObj['productId']) && !$lineObj[$check['field']]) {
						$this->addFailure(ERR_DECL_XML_FIELD_REQUIRED, array($check['field']));
					}

					// check if the country doesnt have the setting but it is provided
					if (!$country->has($check['csetting'], $dObj['productId']) && $lineObj[$check['field']]) {
						$this->addFailure(ERR_DECL_XML_FIELD_PROVIDED, array($check['field']));
					}

					// if it is provided check if it exists in the list
					if (isset($check['list']) && $lineObj[$check['field']] && !in_array($lineObj[$check['field']], $check['list'])) {
						$this->addFailure(ERR_DECL_XML_FIELD_UNKNOWN, array($check['field']));
					}
				}

				// All lines that are in the database declaration need to match their specification against what
				// is declared in the XML.
				if (isset($dbDeclaration->lines[$lIndex+1])) {
					$dbLine = $dbDeclaration->lines[$lIndex+1];
					$dbLine->set();

					if ($dbLine->brand && $dbLine->brand->id != $lineObj['brandId']) {
						$this->addFailure(ERR_DECL_XML_LINE_MISMATCH, array('brandId'));
					}

					if ($dbLine->product_category && $dbLine->product_category->id != $lineObj['categoryId']) {
						$this->addFailure(ERR_DECL_XML_LINE_MISMATCH, array('categoryId'));
					}

					if ($dbLine->dimension && $dbLine->dimension->id != $lineObj['dimensionId']) {
						$this->addFailure(ERR_DECL_XML_LINE_MISMATCH, array('dimensionId'));
					}

					// bail out if a line in the database is found with trust
					if ($dbLine->trust && $dbLine->trust->id) {
						$this->addFailure(ERR_DECL_XML_NO_DECLARE_TRUST);
					}

				}

				// finally add the line but in the correct format
				$dObj['lines'][] = array(
					'quantity' => $lineObj['quantity'],
					'weight' => $lineObj['weight'],
					'weight_dec' => $lineObj['weight_dec'],
					'brand' => array('id' => $lineObj['brandId']),
					'product_category' => array('id' => $lineObj['categoryId']),
					'dimension' => array('id' => $lineObj['dimensionId']),
					'weight' => $lineObj['weight'],
				);
			}

			// error if component declaration is active but no components are supplied
			if ($country->has(HasDeclarationComponent, $dObj['productId'])) {

				// loop and fill declaration components
				foreach ($dElement->getElementsByTagName('component') as $lIndex => $lElement) {

					$componentId = $this->_getXmlNodeValue($lElement, 'component_id');
					$quantity = $this->_getXmlNodeValue($lElement, 'quantity');
					$weight = $this->_getXmlNodeValue($lElement, 'weight');

					$dObj['components'][$componentId] = array(
						'quantity' => $quantity,
						'weight' => $weight,
					);
				}

				// all components in the declaration must be given
				$dc = new DeclarationComponent($this->db);
				$dc->set($dbDeclaration->id);
				if (sizeof($dObj['components']) != sizeof($dc->getLines())) {
					$this->addFailure(ERR_DECL_XML_MISSING_COMPONENTS);
				}

			}

			// when declaring for a late (current is forfeit) then dont allow extra lines
			if ($dbDeclaration->aggregated_status == DECLARATION_FORFEIT && sizeof($dObj['lines']) != sizeof($dbDeclaration->lines) ) {
				$this->addFailure(ERR_DECL_XML_LATE_EXTRA_LINES);
			}

			// lines must be unique
			$map = array();
			foreach ($dObj['lines'] as $line) {

				$brand = $line['brand']['id'];
				$pc = $line['product_category']['id'];
				$dim = $line['dimension']['id'];

				if (!isset($map[$brand]))
					$map[$brand] = array();

				if (!isset($map[$brand][$pc]))
					$map[$brand][$pc] = array();

				if (isset($map[$brand][$pc][$dim])) {
					$this->addFailure(ERR_DECL_XML_LINES_DUPLICATE);
					break;
				}

				$map[$brand][$pc][$dim] = true;
			}

			// finally add to declarations definition
			$declarations[] = $dObj;
		}

		// push out of failures if any
		if (!empty($this->failures))
			throw new DeclarationXmlFailuresException($this->failures);

		// Important: from here any exceptions are bugs!

		// start transaction, rollback later if dryRun is on
		$this->db->begin();

		// the return value of this method are declaration (and components) instances of all that have been touched
		$rvDeclarations = array();
		$rvComponents = array();

		// loop declarations and commit based on dryRun
		foreach ($declarations as $dObj) {

			// get the database declaration bases on the xml
			$dbDeclaration = $this->getLastDeclaration($dObj['year'], $dObj['month'], $dObj['crsoId'], $dObj['participantId'], $dObj['productId']);

			// validateUpdate() checks if the declaration is already exported
			if ($dbDeclaration->validateUpdate()) {

				// not exported: update lines and components if given
				info(sprintf('DeclarationController::declareXml: declare declaration [%s]. dryrun: %s', $dbDeclaration->id, (int) $dryRun));
				$dbDeclaration->updateDeclarationLines($dObj['lines']);
				if(sizeof($dObj['components']) > 0) {
					$dc = new DeclarationComponent($this->db);
					$dc->updateDeclarationComponents($dbDeclaration, $dObj['components']);
				}

			} else {
				// declaration already exported: resubmit or late creates a new declaration
				info(sprintf('DeclarationController::declareXml: resubmit/late declaration [%s]. dryrun: %s', $dbDeclaration->id, (int) $dryRun));
				$dbDeclaration->insertDeclarationLines($dObj['lines'], $dObj['components']);
			}

			// prepare components for resultset if any
			$dc = new DeclarationComponent($this->db);
			if ($dc->set($dbDeclaration->id)) {
				foreach ($dc->getLines() as $line)
					$line->component->set();
				$rvComponents[$dbDeclaration->id] = $dc;
			}

			// add to return value
			$rvDeclarations[] = $dbDeclaration;
		}

		if ($dryRun) {
			info('DeclarationController:declareXml: rolling back');
			$this->db->rollback();
		}
		else {
			info('DeclarationController:declareXml: committing');
			$this->db->commit();
		}

		// return declaration and components objects
		// notice that if rolled back this declarations dont match the db but they can be used by the interface
		// to show changes that will be applied if dryrun is off
		return array('declarations' => $rvDeclarations, 'components' => $rvComponents);
	}

	// failure methods

	protected $failureContext = array();
	protected $failures = array();
	protected function addFailure($error_code, $msg_args=array()) {
		$this->failures[] = array(
			'context' => $this->failureContext,
			'error_code' => $error_code,
			'message' => vsprintf($this->lang[$error_code], $msg_args),
		);
	}

	// xml wrappers

	protected function _getXmlNodeValue($domNode, $tagName, $optional=false) {
		$item = $domNode->getElementsByTagName($tagName)->item(0);

		if (!$item && !$optional)
			throw new Exception(sprintf('_getXmlNodeValue: required tag/value [%s] not found in domnode [%s]', $tagName, $domNode));

		if (!$item)
			return null;

		return $item->nodeValue;
	}

	protected function getLastDeclaration($year, $month, $crso_id, $participant_id, $product_id)
	{
		$sql = "SELECT MAX(DECLARATION_ID) as DECLARATION_ID
			FROM TBL_DECLARATION
			WHERE CRSO_RELATION_ID=?
				AND PARTICIPANT_RELATION_ID=?
				AND `YEAR`=?
				AND `MONTH`=?
				AND `PRODUCT_ID`=?";

		$q = $this->db->query($sql);
		$q->bind_values($crso_id, $participant_id, $year, $month, $product_id);
		$q->execute();

		$r = $q->fetch();
		$declaration_id = $r->DECLARATION_ID;

		if (empty($declaration_id))
			throw new Exception(sprintf('No declaration found for year [%s], month [%s], crso [%s], participant [%s], product [%s] ', $year, $month, $crso_id, $participant_id, $product_id));

		$d = new Declaration($this->db);
		$d->set($declaration_id);

		return $d;
	}

	/**
	 * gets all name/id mappings for the current user related to declarations
	 */
	public function getMappings() {
		$rv = array();

		// crsos
		$crso = new CRSO($this->db);
		$q = $crso->selectCRSOsByParticipantUser($this->user->id, PARTICIPANT_PRODUCER);
		while ($row = $q->fetch()) {
			$rv['crsos'][$row->CRSO_ID] = $row->CRSO_NAME;
		}

		// participants
		$p = new Participant($this->db);
		$q = $p->selectParticipantsByUser($this->user->id, PARTICIPANT_PRODUCER);
		$rv['participants'] = array();
		while ($row = $q->fetch()) {
			$rv['participants'][$row->PARTICIPANT_ID] = $row->PARTICIPANT_NAME;
		}

		// products
		$p = new Product($this->db);
		$q = $p->getProducts();
		while ($row = $q->fetch()) {
			$rv['products'][$row->PRODUCT_ID] = $row->PRODUCT_NAME;
		}

		// brands
		$b = new ParticipantBrand($this->db);
		foreach ($rv['crsos'] as $crso_id=>$v) {
			foreach ($rv['participants'] as $part_id=>$v) {
				foreach ($rv['products'] as $prod_id=>$v) {
					$q = $b->getBrands($part_id, $crso_id, $prod_id);
					while ($row = $q->fetch()) {
						$rv['brands'][$row->BRAND_ID] = $row->NAME;
					}
				}
			}
		}

		// product categories
		$pc = new ProductCategory($this->db);
		foreach ($rv['crsos'] as $crso_id=>$v) {
			foreach ($rv['products'] as $prod_id=>$v) {
				$q = $pc->getProductCategories($crso_id, $prod_id);
				while ($row = $q->fetch()) {
					$rv['categories'][$row->PRODUCT_CATEGORY_ID] = $row->NAME;
				}
			}
		}

		// dimensions
		$cpd = new CRSOProductDimension($this->db);
		foreach ($rv['crsos'] as $crso_id=>$v) {
			foreach ($rv['products'] as $prod_id=>$v) {
				$q = $cpd->getCrsoProductDimensions($crso_id, $prod_id);
				while ($row = $q->fetch()) {
					$rv['dimensions'][$row->CPD_ID] = $row->NAME;
				}
			}
		}

		// components
		$pc = new ProductComponent($this->db);
		foreach ($rv['crsos'] as $crso_id=>$v) {
			foreach ($rv['products'] as $prod_id=>$v) {
				$q = $pc->getByProductCRSO($crso_id, $prod_id);
				while ($row = $q->fetch()) {
					$rv['components'][$row->COMPONENT_ID] = $row->COMPONENT_NAME;
				}
			}
		}



		return $rv;
	}
}

/**
 * Exceptions
 */

class DeclarationException extends WEEEException {};
class DeclarationXmlValidationException extends DeclarationException {};

class DeclarationXmlFailuresException extends DeclarationException {

	public $failures;

	function __construct($failures, $code=0) {
		$this->failures = $failures;
		parent::__construct(print_r($failures, true), $code);
	}

};


?>
