<?php
require_once(IWARE.'AbstractObject.php');
/**
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @package ELC
 * @version $Id$
 */
class DeclarationInterest extends AbstractObject
{

	var $declaration_interest_type;
	var $crso;
	var $start_date;
	var $interest_rate;
	var $product;

	function set($crso_id=false, $declaration_interest_type=false, $start_date=false, $product_id='')
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$declaration_interest_type || $declaration_interest_type = $this->declaration_interest_type;
		$start_date || $start_date = $this->start_date;
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT *
			FROM TBL_DECLARATION_INTEREST
			WHERE CRSO_RELATION_ID = ?
				AND DECLARATION_INTEREST_TYPE = ?
				AND START_DATE = ?
				AND PRODUCT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$declaration_interest_type, $start_date, $product_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		require_once(ELC_CLASSES.'CRSO.php');
		$this->crso = new CRSO($this->db, array('relation'=>new Relation($this->db,$crso_id)));
		require_once(ELC_CLASSES.'Product.php');
		$this->product = new Product($this->db,$r->PRODUCT_ID);
		$this->declaration_interest_type = $r->{'DECLARATION_INTEREST_TYPE'};
		$this->start_date = $r->{'START_DATE'};
		$this->interest_rate = $r->{'INTEREST_RATE'};

		return $this->complete = true;
	}


	function setToCurrentCRSOAdminRate($crso_id=false, $declaration_interest_type=false, $product_id=false)
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$declaration_interest_type || $declaration_interest_type = $this->declaration_interest_type;
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT *
			FROM TBL_DECLARATION_INTEREST
			WHERE CRSO_RELATION_ID = ?
				AND DECLARATION_INTEREST_TYPE = ?
				AND PRODUCT_ID = ?
				AND START_DATE <= NOW()
			ORDER BY START_DATE DESC
			LIMIT 1";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$declaration_interest_type, $product_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		require_once(ELC_CLASSES.'CRSO.php');
		$this->crso = new CRSO($this->db, array('relation'=>new Relation($this->db,$crso_id)));
		require_once(ELC_CLASSES.'Product.php');
		$this->product = new Product($this->db,$r->PRODUCT_ID);
		$this->declaration_interest_type = $r->{'DECLARATION_INTEREST_TYPE'};
		$this->start_date = $r->{'START_DATE'};
		$this->interest_rate = $r->{'INTEREST_RATE'};
		return $this->complete = true;
	}

	function getCurrentRate($crso_id=false, $declaration_interest_type=false,$product_id=false)
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$declaration_interest_type || $declaration_interest_type = $this->declaration_interest_type;
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT INTEREST_RATE
			FROM TBL_DECLARATION_INTEREST
			WHERE CRSO_RELATION_ID = ?
				AND DECLARATION_INTEREST_TYPE = ?
				AND PRODUCT_ID = ?
				AND START_DATE <= NOW()
			ORDER BY START_DATE DESC
			LIMIT 1";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$declaration_interest_type,$product_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return 100;

		return $r->{'INTEREST_RATE'};
	}

	public static function getTypes()
	{
		return array(
			'Interest Rate Forfeit First'=>INTEREST_RATE_FORFEIT_FIRST,
			'Interest Rate Forfeit Second'=>INTEREST_RATE_FORFEIT_SECOND,
			'Interest Rate Forfeit Third'=>INTEREST_RATE_FORFEIT_THIRD,
		);
	}

	/**
	 * Selects an declaration interest overview by $crso_id and $declaration_interest_type
	 */
	function selectCrsoDeclIntType($crso_id, $declaration_interest_type,$product_id)
	{
		$sql = "SELECT START_DATE, INTEREST_RATE
			FROM TBL_DECLARATION_INTEREST
			WHERE CRSO_RELATION_ID=?
				AND DECLARATION_INTEREST_TYPE=?
				AND PRODUCT_ID = ?
			ORDER BY START_DATE";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, $declaration_interest_type, $product_id);
		$q->execute();
		return $q;
	}


	/**#@+
	 * @access private
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_DECLARATION_INTEREST';
		$this->objectmap['primary']['declaration_interest_type'] = 'DECLARATION_INTEREST_TYPE';
		$this->objectmap['primary']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['primary']['start_date'] = 'START_DATE';
		$this->objectmap['properties']['product->id'] = 'PRODUCT_ID';
		$this->objectmap['properties']['interest_rate'] = 'INTEREST_RATE';
		$this->objectmap['inserttype'] = 'REPLACE';
	}
	/**#@-*/
}

/**
 * INTEREST_RATE_FORFEIT_FIRST defines
 */
define('INTEREST_RATE_FORFEIT_FIRST', 1);
define('INTEREST_RATE_FORFEIT_SECOND', 2);
define('INTEREST_RATE_FORFEIT_THIRD', 3);


?>