<?php
require_once(ELC_CLASSES.'Declaration.php');
/**
 * HasOneInvoice is a country settings and not a ProductCountry setting
 * @package WEEE
 * @subpackage Declaration
 * @since 15-dec-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class DeclarationOneInvoice extends Declaration
{

/**
	 * Adds Invoice numbers to ALL PENDING NON ZERO TOTAL declarations
	 * @param int $crso_id
	 */
	function addDebitNumbersToPendingExport($crso_id, &$log, $products)
	{
		$log->info(sprintf('Adding Invoice Numbers for CRSO (setting = HasOneInvoice) [%s]', $crso_id));

		$sql = "
					SELECT d.DECLARATION_ID, d.CRSO_RELATION_ID, SUM(dl.QUANTITY) as QUANT,
							d.PARTICIPANT_RELATION_ID, d.`MONTH`, d.`YEAR`,
							IFNULL(dl.PRODUCT_CATEGORY_ID, '-') as cat,
							IFNULL(f_elcProductFee(d.PRODUCT_ID, d.CRSO_RELATION_ID, dl.PRODUCT_CATEGORY_ID, CONCAT(d.YEAR, '-', d.MONTH, '-01 00:00:00')),'0') as FEE
					FROM TBL_DECLARATION AS d
					INNER JOIN TBL_DECLARATION_LINE as dl
						ON dl.DECLARATION_ID = d.DECLARATION_ID
					LEFT JOIN TBL_DECLARATION_SET_LINE AS DSL
						ON DSL.DECLARATION_ID = d.DECLARATION_ID
					WHERE d.CRSO_RELATION_ID = ?
						AND d.IS_EXPORTED = ?
						AND ISNULL(DSL.DECLARATION_SET_ID)
						AND d.PRODUCT_ID IN (".join($products, ',').")
					GROUP BY dl.DECLARATION_ID, dl.PRODUCT_CATEGORY_ID
					HAVING SUM(dl.QUANTITY) != 0
					ORDER BY d.PARTICIPANT_RELATION_ID, d.`MONTH`, d.`YEAR` ASC
				";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, DECLARATION_EXPORT_PENDING);
		$q->execute();

		$perYearParticipantMonth = array();
		while($r = $q->fetch()) {
			$perYearParticipantMonth[$r->YEAR][$r->PARTICIPANT_RELATION_ID][$r->MONTH][$r->DECLARATION_ID][$r->cat] = array($r->QUANT, $r->FEE);
		}

		foreach($perYearParticipantMonth as $year => $yearlyDeclarations) {
			$this->_insertOneInvoiceCreditDebitLines($crso_id, $log, $yearlyDeclarations);
		}
	}
	/**
	 * This creates new invoice/credit ids per year/per month.
	 */
	private function _insertOneInvoiceCreditDebitLines($crso_id, &$log, $yearlyDeclarations)
	{
		// Get the current year, the debit/credit numbers are ALWAYS for the current year.
		$date =& currentDate();
		$year = $date->getYear();

		//get startID for debits
		$sql = "SELECT @debitno:=IFNULL(MAX(DECLARATION_DEBIT_ID), 0)
				FROM TBL_DECLARATION_DEBIT
				WHERE CRSO_RELATION_ID = ? AND `YEAR` = ?";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$year);
		$q->execute();
		//get startID for credits
		$sql = "SELECT @creditno:=IFNULL(MAX(DECLARATION_CREDIT_ID), 0)
				FROM TBL_DECLARATION_CREDIT
				WHERE CRSO_RELATION_ID = ? AND `YEAR` = ?";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$year);
		$q->execute();

		foreach($yearlyDeclarations as $part_id => $months)
		{
			foreach($months as $declarations)
			{
				$total_amount = 0;
				$individual_decl = array();
				foreach($declarations as $decl_id => $cats) {
					$individual_decl[$decl_id] = $decl_id;
					foreach($cats as $cat => $values) {
						$total_amount += ($values[0] * $values[1]);
					}
				}
				$log->info(sprintf(' - Total amount = %s',$total_amount));
				// We do not generate credit/debit ids for declarations with a total quantity or amount of 0. See ticket #2115, #2283
				if ($total_amount==0) {
					continue;
				}
				list($tbl,$type) = ($total_amount>0)?array('DECLARATION_DEBIT','debit'):array('DECLARATION_CREDIT','credit');

				$sql = "INSERT INTO TBL_DECLARATION_SET (CREATED) VALUES(NOW())";
				$q1 = $this->db->query($sql);
				$q1->execute();
				$last_set_id = $q1->lastid();
				$log->info(sprintf(' - Adding new `%s id` for declarations [%s] where the set is [%s]',$type,implode(',',array_keys($declarations)),$last_set_id));

				$sql = "INSERT INTO TBL_".$tbl."
					(CRSO_RELATION_ID,DECLARATION_SET_ID,".$tbl."_ID,`YEAR`)
					SELECT ".$crso_id.", ".$last_set_id.", @".$type."no:=@".$type."no+1 as ".$tbl."_ID, ".$year;
				$q1 = $this->db->query($sql);
				$q1->execute();

				foreach($individual_decl as $decl_id)
				{
					$sql = "INSERT INTO TBL_DECLARATION_SET_LINE (DECLARATION_SET_ID, DECLARATION_ID) VALUES(?,?)";
					$q1 = $this->db->query($sql);
					$q1->bind_values($last_set_id,$decl_id);
					$q1->execute();
				}
			}
		}
	}

	/**
	 * We did everything in the addDebit function.
	 */
	function addCreditNumbersToPendingExport($crso_id, &$log, $products)
	{

	}
}
?>
