<?php
require_once(IWARE.'Document.php');
/**
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @package ELC
 * @version $Id$
 */

define('APP_DOCUMENT_OBJECT', 'ELCDocument');

class ELCDocument extends Document
{
	/**
	 * @todo FIXME TICKET NUMBER #2122
	 */
	function hasPrivileges($privs=DOCUMENT_PRIV_READ)
	{
		/**
		 *  For IQUNIT
		 */
		if(defined('IN_IQUNIT') || php_sapi_name() == 'cli')
			return true;

		require_once(IWARE . 'group.php');
		switch ($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
				return true;

			case GROUP_ACCOUNTANT:

				if($this->_checkAccountantDocument($_SESSION['accountant_relation_id']))
					return true;

				break;
			case GROUP_CRSO:
				/**
				 * allow to view participant registration documents
				 */
				$sql = "SELECT *
					FROM TBL_CRSO_PARTICIPANT_DOCUMENT
					WHERE CRSO_RELATION_ID=?
					AND DOCUMENT_ID=?";

				$q = $this->db->query($sql);
				$q->bind_values($_SESSION['crso_relation_id'], $this->id);
				$q->execute();

				if ($r = $q->fetch())
					return true;
				if($this->_checkRelationDocument($_SESSION['crso_relation_id']))
					return true;

				// Check PoP Confirmation doc.
				if ($this->checkPoPConfirmationDocument($_SESSION['crso_relation_id'], true))
					return true;

				if ($this->checkRefundRequestConfirmationDocument($_SESSION['crso_relation_id'], true))
					return true;

				if($this->_checkProofOfPaymentDocument($_SESSION['crso_relation_id']))
					return true;

				if($this->_checkAnnualConfirmationDocument($_SESSION['crso_relation_id']))
					return true;

				if ($this->_checkRefundRequestDocument($_SESSION['crso_relation_id'])) {
					return true;
				}

				return false;
			case GROUP_PARTICIPANT:
				/**
				 * allow participant relation documents
				 */
				if($this->_checkRelationDocument($_SESSION['participant_ids']))
					return true;
				if($this->_checkDeclarationDocument($_SESSION['participant_ids']))
					return true;
				if ($this->checkRefundRequestConfirmationDocument($_SESSION['participant_ids']))
					return true;
				if ($this->checkPoPConfirmationDocument($_SESSION['participant_ids']))
					return true;
				if($this->_checkRefundExportDocument($_SESSION['participant_ids']))
					return true;
				if($this->_checkRequestedInvoiceDocument($_SESSION['participant_ids']))
					return true;
				if($this->_checkReminderDocument($_SESSION['participant_ids']))
					return true;
				if($this->_checkLatePaymentDocument($_SESSION['participant_ids']))
					return true;
				if($this->_checkGuaranteeDocument($_SESSION['participant_ids']))
					return true;
				if($this->_checkCreditViaProducer($_SESSION['participant_ids']))
					return true;
				if($this->_checkCRSOManuals($_SESSION['participant_ids']))
					return true;

			case GROUP_PUBLIC:
				/**
				 * allow crso document
				 */
				if (isset($_SESSION['public']['crso']['id']))
				{
					$sql = "SELECT * FROM TBL_CRSO_PUBLIC_DOCUMENT
						WHERE DOCUMENT_ID = ?
							AND CRSO_RELATION_ID = ?";
					$q = $this->db->query($sql);
					$q->bind_values($this->id, $_SESSION['public']['crso']['id']);
					$q->execute();
					if($r = $q->fetch())
						return true;
				}

		}

		if (!parent::hasPrivileges())
			return false;

		return true;
	}
	function _checkRelationDocument($relation_id)
	{
		$relation_id = is_array($relation_id) ? $relation_id : array($relation_id);
		$sql = "SELECT RELATION_ID
			FROM TBL_RELATION_DOCUMENT
			WHERE RELATION_ID IN (".implode(',',$relation_id).")
				AND DOCUMENT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		return (bool) ($r = $q->fetch());
	}

	function _checkProofOfPaymentDocument($crso_relation_id) {
		// Participant confirms access throught _checkRelationDocument

		$sql = "SELECT PARTICIPANT_RELATION_ID FROM TBL_PROOF_OF_PAYMENT
			WHERE DOCUMENT_ID = ? AND CRSO_RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->id, $crso_relation_id);
		$q->execute();
		return (bool) ($r = $q->fetch());

	}

	function _checkAnnualConfirmationDocument($crso_relation_id)
	{
		$sql = "SELECT AC.PARTICIPANT_RELATION_ID
			FROM TBL_ANNUAL_CONFIRMATION AC
			INNER JOIN TBL_ANNUAL_CONFIRMATION_DOCUMENT ACD
				ON ACD.ANNUAL_CONFIRMATION_ID = AC.ANNUAL_CONFIRMATION_ID
			WHERE ACD.DOCUMENT_ID = ? AND AC.CRSO_RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->id, $crso_relation_id);
		$q->execute();
		return (bool) ($r = $q->fetch());
	}

	function _checkAccountantDocument($accountant_id) {

		$sql = "SELECT DISTINCT PARTICIPANT_RELATION_ID
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CPP
			INNER JOIN TBL_ACCOUNTANT_CRSO as AC
				ON CPP.CRSO_RELATION_ID = AC.CRSO_RELATION_ID
			WHERE AC.ACCOUNTANT_RELATION_ID = ?
		";
		$q = $this->db->query($sql);
		$q->bind_values($accountant_id);
		$q->execute();
		$participants = array();
		while($r = $q->fetch())
			$participants[] = $r->PARTICIPANT_RELATION_ID;

		if(!count($participants))
			return false;

		if($this->_checkRelationDocument($participants))
			return true;
		if($this->_checkDeclarationDocument($participants))
			return true;
		if ($this->checkRefundRequestConfirmationDocument($participants))
			return true;
		if ($this->checkPoPConfirmationDocument($participants))
			return true;
		if($this->_checkRefundExportDocument($participants))
			return true;
		if($this->_checkRequestedInvoiceDocument($participants))
			return true;
		if($this->_checkReminderDocument($participants))
			return true;
		if($this->_checkLatePaymentDocument($participants))
			return true;
		if($this->_checkGuaranteeDocument($participants))
			return true;
		if($this->_checkCreditViaProducer($participants))
			return true;
		if($this->_checkCRSOManuals($participants))
			return true;


	}

	function _checkRequestedInvoiceDocument($participant_id)
	{
		$participant_id = is_array($participant_id) ? $participant_id : array($participant_id);
		$sql = "SELECT PARTICIPANT_RELATION_ID
			FROM TBL_REQUESTED_INVOICE
			WHERE PARTICIPANT_RELATION_ID IN (".implode(',',$participant_id).")
				AND DOCUMENT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		return (bool) ($r = $q->fetch());
	}
	function _checkDeclarationDocument($participant_id)
	{
		$participant_id = is_array($participant_id) ? $participant_id : array($participant_id);
		$sql = "SELECT DECLARATION_ID
			FROM TBL_DECLARATION
			WHERE DOCUMENT_ID = ?
				AND PARTICIPANT_RELATION_ID IN (".implode(',',$participant_id).")";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		return (bool) ($r = $q->fetch());
	}
	function _checkRefundExportDocument($participant_id)
	{
		$participant_id = is_array($participant_id) ? $participant_id : array($participant_id);
		$sql = "SELECT REFUND_EXPORT_ID
			FROM TBL_REFUND_EXPORT
			WHERE INVOICE_DOCUMENT_ID = ?
				AND PARTICIPANT_RELATION_ID IN (".implode(',',$participant_id).")";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		return (bool) ($r = $q->fetch());
	}

	function _checkRefundRequestDocument($crso_id) {

		$sql = "SELECT REFUND_REQUEST_ID
			FROM TBL_REFUND_REQUEST
			WHERE EXPORT_DOCUMENT_ID = ?
				AND CRSO_RELATION_ID  = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->id, $crso_id);
		$q->execute();
		return (bool) ($r = $q->fetch());

	}

	function checkRefundRequestConfirmationDocument($relation_id, $is_crso=false)
	{
		if($is_crso) {
			// Assert user has Clearance!
			require_once(ELC_CLASSES.'RelationUser.php');
			$ru = new RelationUser($this->db);
			if(!$ru->hasClearance(CRSOClearanceRefundDetails, $_SESSION['session_user_id'], $relation_id))
				return false;
		} else {
			$relation_id = is_array($relation_id) ? $relation_id : array($relation_id);
		}
		$sql = "SELECT rrc.REFUND_REQUEST_CONFIRMATION_ID
			FROM TBL_REFUND_REQUEST_CONFIRMATION rrc
			INNER JOIN TBL_REFUND_REQUEST rr ON rrc.REFUND_REQUEST_CONFIRMATION_ID = rr.REFUND_REQUEST_CONFIRMATION_ID
			WHERE rrc.DOCUMENT_ID = ? ";
		if($is_crso) {
			$sql .= " AND rr.CRSO_RELATION_ID = ".$relation_id;
		} else {
			$sql .= " AND rr.PARTICIPANT_RELATION_ID IN (".implode(',',$relation_id).") ";
		}
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		return (bool) ($r = $q->fetch());
	}

	function _checkReminderDocument($participant_id)
	{
		$participant_id = is_array($participant_id) ? $participant_id : array($participant_id);
		$sql = "SELECT PARTICIPANT_RELATION_ID
			FROM TBL_CRSO_PARTICIPANT_REMINDER
			WHERE DOCUMENT_ID = ?
				AND PARTICIPANT_RELATION_ID IN (".implode(',',$participant_id).")";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		return (bool) ($r = $q->fetch());
	}

	function _checkLatePaymentDocument($participant_id)
	{
		$participant_id = is_array($participant_id) ? $participant_id : array($participant_id);
		$sql = "SELECT PARTICIPANT_RELATION_ID
			FROM TBL_CRSO_PARTICIPANT_LATE_PAYMENT
			WHERE DOCUMENT_ID = ?
				AND PARTICIPANT_RELATION_ID IN (".implode(',',$participant_id).")";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		return (bool) ($r = $q->fetch());
	}

	function _checkGuaranteeDocument($participant_id)
	{
		$participant_id = is_array($participant_id) ? $participant_id : array($participant_id);
		$sql = "SELECT PARTICIPANT_RELATION_ID
			FROM TBL_CRSO_PARTICIPANT_GUARANTEE
			WHERE DOCUMENT_ID = ?
				AND PARTICIPANT_RELATION_ID IN (".implode(',',$participant_id).")";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		return (bool) ($r = $q->fetch());
	}

	function checkPoPConfirmationDocument($relation_id, $is_crso=false)
	{
		if($is_crso) {
			// Assert user has Clearance!
			require_once(ELC_CLASSES.'RelationUser.php');
			$ru = new RelationUser($this->db);
			if(!$ru->hasClearance(CRSOClearanceRefundDetails, $_SESSION['session_user_id'], $relation_id))
				return false;
		} else {
			$relation_id = is_array($relation_id) ? $relation_id : array($relation_id);
		}
		$sql = "SELECT PROOF_OF_PAYMENT_CONFIRMATION_ID
			FROM TBL_PROOF_OF_PAYMENT_CONFIRMATION
			WHERE DOCUMENT_ID = ? ";
		if($is_crso) {
			$sql .= " AND CRSO_RELATION_ID = ".$relation_id;
		} else {
			$sql .= " AND PARTICIPANT_RELATION_ID IN (".implode(',',$relation_id).")";
		}
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		return (bool) ($r = $q->fetch());
	}

	function _checkCreditViaProducer($participant_id)
	{
		$participant_id = is_array($participant_id) ? $participant_id : array($participant_id);
		$sql = "SELECT REFUND_REQUEST_CREDIT_ID
			FROM TBL_REFUND_EXPORT_CREDIT as rec
			INNER JOIN TBL_REFUND_EXPORT_SET_LINE as resl
				ON rec.REFUND_EXPORT_SET_ID = resl.REFUND_EXPORT_SET_ID
			INNER JOIN TBL_REFUND_EXPORT as re
				ON resl.REFUND_EXPORT_ID = re.REFUND_EXPORT_ID
			INNER JOIN TBL_REFUND_REQUEST as rr
				ON re.REFUND_EXPORT_ID = rr.REFUND_EXPORT_ID
			INNER JOIN TBL_REFUND_REQUEST_LINE as rrl
				ON rr.REFUND_REQUEST_ID = rrl.REFUND_REQUEST_ID
			WHERE re.INVOICE_DOCUMENT_ID = ?
				AND rrl.PRODUCER_RELATION_ID IN (".implode(',',$participant_id).")";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		return (bool) ($r = $q->fetch());
	}

	function _checkCRSOManuals($participant_id)
	{
		$participant_id = is_array($participant_id) ? $participant_id : array($participant_id);
		$sql = "SELECT DOCUMENT_ID
			FROM TBL_CRSO_SUPPORT_DOCUMENT as CSD
			INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT as CPP
				ON CSD.CRSO_RELATION_ID = CPP.CRSO_RELATION_ID
			WHERE CSD.DOCUMENT_ID = ?
				AND CPP.PARTICIPANT_RELATION_ID IN (".implode(',',$participant_id).")";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		return (bool) ($r = $q->fetch());
	}
}

?>
