<?php
require_once(IWARE.'IWChart.php');
/**
 * @package ELC
 * @subpackage Chart
 * @since 2/08/2006
 * @author Nick Loeve <nick@basesoft.com>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 * @todo DOCUMENTATION!!!
 * @todo add caching of images
 * @todo maybe make a ChartBuilder similar to ListBuilder
 */
class ELCChart extends IWBarChart
{
	var $translations;
	var $langIf;
	var $db;
	var $product_id;
	static private $hasEnoughParticipantsCheck = 'FALSE';

	/**
	 * @static
	 */
	public static function chartFactory(&$db, &$langIf, $chartType,$product_id)
	{
		$chartMap = array();
		require_once(IWARE.'group.php');
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
			case GROUP_ACCOUNTANT:
			case GROUP_CRSO:
				array_push($chartMap,
					ELC_CHART_TYPE_PUT_ON_MARKET,
					ELC_CHART_TYPE_VISIBLE_FEE,
					ELC_CHART_TYPE_OUTSTANDING_RECEIVABLES,
					ELC_CHART_TYPE_COLLECTED_RECEIVABLES);
			case GROUP_PARTICIPANT:
				break;
		}

		if(!in_array($chartType, $chartMap))
			return ELCChart::notFound($chartType);

		if(isset($_SESSION['crso_relation_id']))
		{
			if($_SESSION['crso_relation_id']!=$_REQUEST['crso']['id'])
				exit('You Dont Have Permission To View This Graph');
			require_once(ELC_CLASSES.'Relation.php');
			require_once(ELC_CLASSES.'Country.php');
			require_once(ELC_CLASSES.'CRSO.php');
			require_once(ELC_CLASSES.'RelationUser.php');
			$rel = new Relation($db,$_SESSION['crso_relation_id'],true);
			$country = new Country($db,$rel->getCountry(),true);

			// Relation user for Clearance level check.
			$ru = new RelationUser($db);

			if($country->has(HasCrsoDeclarationDetails,$product_id) && $ru->hasClearance(CRSOClearanceDeclarationDetails, $_SESSION['session_user_id'], $_SESSION['crso_relation_id']))
				self::$hasEnoughParticipantsCheck = 'FALSE';
			else
				self::$hasEnoughParticipantsCheck = 'TRUE';
		}


		$log = getLog();
		$log->info(sprintf('Creating ELCChart [%s].', $chartType));
		$chart = new ELCChart($db, $langIf, ELC_CHART_WIDTH, ELC_CHART_HEIGHT,$product_id);
		$chart->prepare($chartType);
		header("Content-type: image/png");
		echo $chart->makeChartToString();
	}
	/**
	 * @static
	 */
	public static function notFound($chartType=null)
	{
		$log = getLog();
		$log->info(sprintf('ELCChart [%s] requested, but denied access or not found.', $chartType));
		header('HTTP/1.1 404 Not Found', true, 404);
		exit();
	}
	/**
	 * @static
	 */
	public static function generateTranslations(&$langIf)
	{
		return array(
			ELC_CHART_TYPE_PUT_ON_MARKET => array(
				$langIf->getText('BarLegend: Amount recorded originally as put on the market'),
				$langIf->getText('BarLegend: Amount taking into account corrections in future months'),
				$langIf->getText('BarLegend: Provisional for the month'),
				$langIf->getText('AxisLabel: All Amounts In Units')
												),
			ELC_CHART_TYPE_VISIBLE_FEE  => array(
				$langIf->getText('BarLegend: Visible fee based upon actuals'),
				$langIf->getText('BarLegend: Visible fee based upon provisionals'),
				$langIf->getText('AxisLabel: All Amounts In Local Currency')
												),
			ELC_CHART_TYPE_OUTSTANDING_RECEIVABLES  => array(
				$langIf->getText('BarLegend: Outstanding receivables'),
				$langIf->getText('BarLegend: Theoretical outstanding receivables')
												),
			ELC_CHART_TYPE_COLLECTED_RECEIVABLES  => array(
				$langIf->getText('BarLegend: Collected Receivables')
												)
					);
	}

	function ELCChart(&$db, &$langIf, $width, $height,$product_id)
	{
		$this->db =& $db;
		$this->langIf =& $langIf;
		$this->product_id = $product_id;
		parent::IWBarChart($width, $height);
	}

	function getTranslations($chart_type=null)
	{
		$this->setTranslations();
		if(!is_null($chart_type) && isset($this->translations[$chart_type]))
			return $this->translations[$chart_type];
		return $this->translations;
	}

	function setTranslations()
	{
		if(!$this->translations)
			$this->translations = ELCChart::generateTranslations($this->langIf);
	}

	function setDataLabelStyle($font="", $fontSize=8, $fontColor=TextColor, $fontAngle=0)
	{
		$this->assertDataLayer();
		$this->dataLayer->setDataLabelStyle($font, $fontSize, $fontColor, $fontAngle);
	}

	function setBarWidth($widthAll, $widthSingle=-1)
	{
		$this->assertDataLayer();
		$this->dataLayer->setBarWidth($widthAll, $widthSingle);
	}

	function prepare($chartType)
	{
		switch($chartType)
		{
			case ELC_CHART_TYPE_PUT_ON_MARKET:
				$this->_buildPutOnMarket();
				break;
			case ELC_CHART_TYPE_VISIBLE_FEE:
				$this->_buildVisibleFee();
				break;
			case ELC_CHART_TYPE_OUTSTANDING_RECEIVABLES:
				$this->_buildOutstandingReceivables();
				break;
			case ELC_CHART_TYPE_COLLECTED_RECEIVABLES:
				$this->_buildCollectedReceivables();
				break;
		}
	}

	/**#@+*/

	function _monthLocale($month)
	{
		return strftime('%b', mktime(1, 1, 1, $month, 1) );
	}

	function _getMonthsLocale()
	{
		return array($this->_monthLocale(1), $this->_monthLocale(2), $this->_monthLocale(3), $this->_monthLocale(4), $this->_monthLocale(5), $this->_monthLocale(6), $this->_monthLocale(7), $this->_monthLocale(8), $this->_monthLocale(9), $this->_monthLocale(10), $this->_monthLocale(11), $this->_monthLocale(12));
	}

	function _buildPutOnMarket()
	{
		//need this in the real deal
		if(!isset($_REQUEST['crso']['id']) || !$crso_id = (int)$_REQUEST['crso']['id'])
			ELCChart::notFound();
		$data0 = array(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
		$data1 = array(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
		$data2 = array(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
		$labels = $this->_getMonthsLocale();

		$d = currentDate();
		$year =  isset($_REQUEST['year']) ? (int)$_REQUEST['year'] : $d->format('Y');
		for($i=1;$i<=12;$i++)
		{
			$sql = "
					SELECT $i as `MONTH`,
						IF(( NOT ".self::$hasEnoughParticipantsCheck." OR f_elcGetParticipantCountForDeclarationByStatus(?,$i,?,NULL,FALSE,FALSE,?)), f_elcGetDeclarationQuantityByStatus(?,$i,?,NULL,FALSE,FALSE,?, FALSE),NULL) as ORIGINAL_QUANTITY";
			$q = $this->db->query($sql);
			$q->bind_values((int)$_REQUEST['crso']['id'],$year, $this->product_id,(int)$_REQUEST['crso']['id'],$year, $this->product_id);
			$q->execute();
			if($r = $q->fetch())
				$data0[$r->MONTH-1] = $r->ORIGINAL_QUANTITY;

			$sql = "SELECT $i as `MONTH`,IF(( NOT ".self::$hasEnoughParticipantsCheck." OR COUNT(DISTINCT D.PARTICIPANT_RELATION_ID) > 2),SUM(DL.QUANTITY),NULL) as FUTURE_TOTAL_QUANTITY
				FROM TBL_DECLARATION as D
				INNER JOIN TBL_DECLARATION_LINE as DL
					USING(DECLARATION_ID)
				WHERE CRSO_RELATION_ID = ?
					AND `MONTH` = $i
					AND `YEAR` = ?
					AND D.PRODUCT_ID = ?
					AND NOT ISNULL(IS_EXPORTED)";
			$q = $this->db->query($sql);
			debug($q->tmp_sql);
			$q->bind_values($_REQUEST['crso']['id'],$year,$this->product_id);
			$q->execute();
			if($r = $q->fetch())
				$data1[$r->MONTH-1] = $r->FUTURE_TOTAL_QUANTITY;

			$sql = "
					SELECT $i as `MONTH`,
						IFNULL(f_elcGetProvisionalForDeclarationsTriggeredByCRSO(?,$i,?,FALSE,?),
						(SELECT
							IF(( NOT ".self::$hasEnoughParticipantsCheck." OR COUNT(DISTINCT PD.PARTICIPANT_RELATION_ID) > 2),SUM(PD.QUANTITY), NULL) AS PROVISIONAL_QUANTITY
							FROM TBL_PROVISIONAL_DECLARATION PD
							WHERE PD.CRSO_RELATION_ID=?
								AND PD.`MONTH` = $i
								AND PD.`YEAR` = ?
								AND PD.PRODUCT_ID = ?
							GROUP BY PD.`MONTH`
						)
						) as PROVISIONAL_QUANTITY";
			$q = $this->db->query($sql);
			$q->bind_values((int)$_REQUEST['crso']['id'],$year,$this->product_id,(int)$_REQUEST['crso']['id'],$year,$this->product_id);
			$q->execute();
			if($r = $q->fetch())
				$data2[$r->MONTH-1] = $r->PROVISIONAL_QUANTITY;
		}

		$this->chartObj->addLegend(0, 0, false);

		$this->setPlotArea(70, 60, 900, 300, 0xffffff);
		$this->setAxisTitle("y", $this->langIf->getText('AxisLabel: All Amounts In Units'));
		$this->setAxisLabels("x", $labels);

		//get translations
		$trans = $this->getTranslations(ELC_CHART_TYPE_PUT_ON_MARKET);
		$this->setBarWidth(70);
		$this->setDataLabelStyle("", 8, 0x000000, 90);
		$this->addDataSet($data0, 0x9999cc, $trans[0]); //sets the translated legend name
		$this->addDataSet($data1, 0x993366, $trans[1]);
		$this->addDataSet($data2, 0xffff99, $trans[2]);
	}

	function _buildVisibleFee()
	{
		//need this in the real deal

		if(!isset($_REQUEST['crso']['id']) || !$crso_id = (int)$_REQUEST['crso']['id'])
			ELCChart::notFound();

		$d = currentDate();
		$data0 = array(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
		$data1 = array(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);


		$d = currentDate();
		$year =  isset($_REQUEST['year']) ? (int)$_REQUEST['year'] : $d->format('%Y');

		for($i=1;$i<=12;$i++)
		{
			$sql = "SELECT $i as `MONTH`,
				IF(( NOT ".self::$hasEnoughParticipantsCheck." OR f_elcGetParticipantCountForProvisional(?,".($i-1).",?,?)),
					(f_elcGetProvisionalForDeclarationsTriggeredByCRSO(?,".($i-1).",?,FALSE,?)*f_elcProductFee(?,?,NULL,NOW())),NULL) as VISIBLE_FEE";
			$q = $this->db->query($sql);
			$q->bind_values($crso_id, $year,$this->product_id,$crso_id, $year,$this->product_id,$this->product_id,$crso_id);
			$q->execute();
			if($r = $q->fetch())
				$data0[$r->MONTH-1] = $r->VISIBLE_FEE;

			$sql = "SELECT $i as `MONTH`,
				IF(( NOT ".self::$hasEnoughParticipantsCheck." OR f_elcGetParticipantCountForVisibleFeeDeclarations(?,".($i-1).",?,FALSE,TRUE,?)),
					f_elcGetVisibleFeeInvoicedDeclarations(?,".($i-1).",?,FALSE,TRUE,?),NULL) as VISIBLE_FEE";
			$q = $this->db->query($sql);
			$q->bind_values($crso_id, $year, $this->product_id,$crso_id, $year,$this->product_id);
			$q->execute();
			if($r = $q->fetch())
				$data1[$r->MONTH-1] = $r->VISIBLE_FEE;
		}


		$labels = $this->_getMonthsLocale();

		$this->chartObj->addLegend(0, 0, false);

		$this->setPlotArea(70, 60, 900, 300, 0xffffff);
		$this->setAxisTitle("y", $this->langIf->getText('AxisLabel: All Amounts In Local Currency'));
		$this->setAxisLabels("x", $labels);
		$this->setBarWidth(70);
		$this->setDataLabelStyle("", 8, 0x000000, 90);

		$trans = $this->getTranslations(ELC_CHART_TYPE_VISIBLE_FEE);
		$this->addDataSet($data1, 0x9999cc, $trans[0]);
		$this->addDataSet($data0, 0x993366, $trans[1]);
	}

	function _buildOutstandingReceivables()
	{
		$data0 = array(0, 0, 0, 0, 230.89, 1287.79, 0, 0, 0, 0, 0, 0);
		$data1 = array(0, 0, 0, 0, 587.67, 915.87, 0, 0, 0, 0, 0, 0);
		$labels = $this->_getMonthsLocale();

		$this->chartObj->addLegend(0, 0, false);

		$this->setPlotArea(70, 60, 900, 300, 0xffffff);
		$this->setAxisTitle("y", "Amount (Thousands of EUR)");
		$this->setAxisLabels("x", $labels);
		$this->setBarWidth(70);
		$this->setDataLabelStyle("", 8, 0x000000, 90);


		$trans = $this->getTranslations(ELC_CHART_TYPE_OUTSTANDING_RECEIVABLES);
		$this->addDataSet($data0, 0x9999cc, $trans[0]);
		$this->addDataSet($data1, 0x993366, $trans[1]);
	}

	function _buildCollectedReceivables()
	{
		$data0 = array(0, 0, 0, 0, 0, 219.12, 466.23, 0, 0, 0, 0, 0);
		$labels = $this->_getMonthsLocale();

		$this->chartObj->addLegend(0, 0, false);

		$this->setPlotArea(70, 60, 900, 300, 0xffffff);
		$this->setAxisTitle("y", "Amount (Thousands of EUR - Receipts up to 11 July)");
		$this->setAxisLabels("x", $labels);
		$this->setBarWidth(70);
		$this->setDataLabelStyle("", 8, 0x000000, 90);

		$trans = $this->getTranslations(ELC_CHART_TYPE_COLLECTED_RECEIVABLES);
		$this->addDataSet($data0, 0x9999cc, $trans[0]);
	}

	/**#@-*/
}

define('ELC_CHART_WIDTH', 1000);
define('ELC_CHART_HEIGHT', 400);

//defines for different chart types in the system
//add charts here
define('ELC_CHART_TYPE_PUT_ON_MARKET', 1);
define('ELC_CHART_TYPE_VISIBLE_FEE', 2);
define('ELC_CHART_TYPE_OUTSTANDING_RECEIVABLES', 3);
define('ELC_CHART_TYPE_COLLECTED_RECEIVABLES', 4);
?>
