<?php
/**
 * @author Chris Wesseling <chris.wesseling@xs4all.nl>
 * @author Nick Loeve <nick@basesoft.com>
 * @package ELC
 * @version $Id$
 */

/**
 * Base class for export to Navision at PwC
 */
abstract class ELCExporter
{

	protected $db;
	protected $log;
	protected $langIf;

	public function __construct(IWDatabase $db, $log=null)
	{
		$this->db = $db;
		$this->log = is_null($log) ? getLog() : $log;
		$this->langIf = IWRegistry::registry('langIf');
	}

	protected function getText($handle, $language_id=null )
	{
		if($this->langIf)
			return $this->langIf->getText($handle, null, array(), $language_id);
		return $handle;
	}

	public function getLangIdForCRSO($crso_id)
	{
		$sql = "SELECT DEFAULT_LANGUAGE_ID FROM TBL_CRSO
					WHERE CRSO_RELATION_ID = ?
				";
		$q = $this->query($sql, array($crso_id));
		$r = $q->fetch();
		if($r->DEFAULT_LANGUAGE_ID)
			return $r->DEFAULT_LANGUAGE_ID;
		return 44; //english
	}

	abstract protected function doExport($crso_id);

	public function export()
	{
		$q = $this->query("SELECT CRSO_RELATION_ID as `crso_id` FROM TBL_CRSO");
		while ( $r = $q->fetch() )
		{
			try
			{
				$this->prepareInvoiceLines($r->crso_id);
				$this->doExport($r->crso_id);
			}
			catch(ELCExportException $eee)
			{
				$this->log->crit("Exception occured during export: ".$eee->getMessage()."\n\n".$eee->getTraceAsString());
				trigger_error("Exception occured during export: ".$eee->getMessage(), E_USER_NOTICE);
			}
		}
	}

	protected function query($sql, $bind_values=array())
	{
		$q = $this->db->query($sql);
		$q->bind_values($bind_values);
		//debug($q->tmp_sql);
		$q->execute();
		return $q;
	}

	protected function writefile($q, $fieldlengths, $filename, $processors=array(), $hide_cols=array(), $sort_col=false)
	{
		// FIXME: remove me when im not called anymore
	}

	protected function prepareInvoiceLines($crso_id)
	{
		//pass
	}

	public function fixAttrLength(&$value, $index, $lengthmap)
	{
		if ( isset($lengthmap[$index]) )
		{
			list($len, $start) = array_pad((array)$lengthmap[$index],2 ,0);
			$value = mb_substr($value, $start, $len, 'UTF-8');
		}
	}
}

class ELCExporterException extends Exception {}
