<?php
require_once(ELC_CLASSES.'ELCExporter.php');
/**
 * @package
 * @subpackage
 * @author Nick Loeve <nick@basesoft.com>
 * @since 24/10/2006
 * @copyright 2006 BaseSoft B.V.
 * $Id$
 */
class ELCExporter_RefundRequests extends ELCExporter
{
	static protected $invoice_numbers1;
	static protected $invoice_numbers2;
        static protected $colsForInvoice = array(
                            "INVOICE_LINE_INDEX", "`YEAR`",
                            "THIS IS RUNTIME SPECIFIC",
                            "CRSO_RELATION_ID", "REFUND_REQUEST_ID", "REFUND_LINE_INDEX", "PRODUCT_ID",
                            "JOURNAL_TEMPLATE", "CUSTOMER_NO", "CUSTOMER_NAME", "POSTING_DATE", "DOCUMENT_DATE",
                            "EXTERNAL_DOCUMENT_NO", "CURRENCY_CODE", "`TYPE`", "GL_ACCOUNT_NO", "DESCRIPTION",
                            "QUANTITY", "UNIT_COST", "LINE_AMOUNT_EXCL_VAT", "TOTAL_AMOUNT_EXCL_VAT",
                            "TOTAL_AMOUNT_INCL_VAT", "TOTAL_VAT_AMOUNT", "NAVISION_VAT_CODE", "VAT_EXTENDED_TEXT",
                            "DIMENSION_CODE_1", "DIMENSION_CODE_2", "DIMENSION_CODE_3", "DIMENSION_CODE_4",
                            "DIMENSION_CODE_5", "DIMENSION_CODE_6", "DIMENSION_CODE_7", "DIMENSION_CODE_8", "TARIFF_NO",
                            "NET_WEIGHT", "TRANSACTION_TYPE", "EXIT_POINT", "TRANSPORT_METHOD","REFUND_EXPORT_DATE",
                            "BILL_TO_CUSTOMER_NO","PRODUCT_DESCRIPTION", "DOCUMENT_REVIEW_NO", "REFUND_WEIGHT",
                            "REFUND_PRODUCT_CATEGORY", "REFUND_BRAND", "HASH_TOTAL"
                    );


	// {{{ protected function prepareInvoiceLines($crso_id)
	protected function prepareInvoiceLines($crso_id)
	{
		self::$invoice_numbers1 = array();
		self::$invoice_numbers2 = array();

		$this->db->begin();

		$sql = "
			SELECT
			'' AS INVOICE_LINE_INDEX,
			rec.REFUND_REQUEST_CREDIT_ID as REFUND_REQUEST_CREDIT_ID,
			red.REFUND_REQUEST_DEBIT_ID as REFUND_REQUEST_DEBIT_ID,
			IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID), rec.`YEAR`, red.`YEAR`) as `YEAR`,
			rr.CRSO_RELATION_ID as CRSO_RELATION_ID,
			re.PRODUCT_ID as PRODUCT_ID,
			IF(
				ISNULL(red.REFUND_REQUEST_DEBIT_ID),
				CONCAT(
					'RC',
					rr.CRSO_RELATION_ID,
					f_elcBuildInvoiceNoForExport(rr.CRSO_RELATION_ID,rec.`YEAR`,rec.REFUND_REQUEST_CREDIT_ID,TRUE,TRUE)
				),
				CONCAT(
					'RI',
					rr.CRSO_RELATION_ID,
					f_elcBuildInvoiceNoForExport(rr.CRSO_RELATION_ID,red.`YEAR`,red.REFUND_REQUEST_DEBIT_ID,FALSE,TRUE)
				)
			) as INVOICE_NUMBER,
			re.REFUND_EXPORT_ID,
			rrl.REFUND_REQUEST_ID as REFUND_REQUEST_ID,
			rrl.LINE_INDEX as REFUND_LINE_INDEX,
			CONCAT(IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID),'RC','RI'), rr.CRSO_RELATION_ID) as JOURNAL_TEMPLATE,
			CONCAT(cac.CODE,
				LPAD(rr.CRSO_RELATION_ID, 8, 0),
				LPAD(rr.PARTICIPANT_RELATION_ID, 8, 0)
			) as CUSTOMER_NO,
			'' as CUSTOMER_NAME,
			LAST_DAY(DATE_SUB('".(currentDate()->format('Y-m-d H:i:s'))."', INTERVAL 1 MONTH)) as POSTING_DATE,
			'".(currentDate()->format('Y-m-d H:i:s'))."' as DOCUMENT_DATE,
			'' as EXTERNAL_DOCUMENT_NO,
			'' as CURRENCY_CODE,
			'G/L Account' as `TYPE`,
				(SELECT GL_ACCOUNT
				FROM TBL_CRSO_GLACCOUNT gc
				INNER JOIN TBL_DECLARATION_TYPE dt
					ON gc.DECLARATION_TYPE_ID = dt.DECLARATION_TYPE_ID
				WHERE gc.CRSO_RELATION_ID = ?
					AND dt.HANDLE = ?)
			as GL_ACCOUNT_NO,
			CONCAT('PERIOD: ', IFNULL(rr.REFUND_EXPORT_DATE, ''))  as DESCRIPTION,
			IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID),
					IF(IFNULL(rrl.QUANTITY, 0) >= 0,
						ABS(IFNULL(rrl.QUANTITY, 0)),
						-(ABS(IFNULL(rrl.QUANTITY, 0)))
					),
					IF(IFNULL(rrl.QUANTITY, 0) >= 0,
						-(ABS(IFNULL(rrl.QUANTITY, 0))),
						ABS(IFNULL(rrl.QUANTITY, 0))
					)
			) as QUANTITY,
			f_elcProductFee(rr.PRODUCT_ID, rr.CRSO_RELATION_ID, rrl.PRODUCT_CATEGORY_ID, NOW()) as UNIT_COST,
			IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID),
					IF(IFNULL(rrl.QUANTITY, 0) >= 0,
						ABS(IFNULL(rrl.EXPORTED_AMOUNT, 0)),
						-(ABS(IFNULL(rrl.EXPORTED_AMOUNT, 0)))
					),
					IF(IFNULL(rrl.QUANTITY, 0) >= 0,
						-(ABS(IFNULL(rrl.EXPORTED_AMOUNT, 0))),
						ABS(IFNULL(rrl.EXPORTED_AMOUNT, 0))
					)
			) as LINE_AMOUNT_EXCL_VAT,
			f_elcGetTotalAmountForRefundExport(re.REFUND_EXPORT_ID) as TOTAL_AMOUNT_EXCL_VAT,
			'' as NAVISION_VAT_CODE,
			'' as VAT_EXTENDED_TEXT,
			'' as DIMENSION_CODE_1,
			'' as DIMENSION_CODE_2,
			'' as DIMENSION_CODE_3,
			'' as DIMENSION_CODE_4,
			'' as DIMENSION_CODE_5,
			'' as DIMENSION_CODE_6,
			'' as DIMENSION_CODE_7,
			'' as DIMENSION_CODE_8,
			'' as TARIFF_NO,
			IFNULL(rrl.WEIGHT, '') as NET_WEIGHT,
			'' as TRANSACTION_TYPE,
			'' as EXIT_POINT,
			'' as TRANSPORT_METHOD,
			rr.REFUND_EXPORT_DATE as REFUND_EXPORT_DATE,
			CONCAT(cac.CODE,
						LPAD(re.CRSO_RELATION_ID, 8, 0),
						LPAD(IFNULL(re.BILL_TO_CUSTOMER_NO,re.PARTICIPANT_RELATION_ID), 8, 0)
			) as BILL_TO_CUSTOMER_NO,
			p.`NAME` as PRODUCT_DESCRIPTION,
			(
				SELECT COUNT(redr2.REVIEW_COST)
					FROM TBL_REFUND_EXPORT_DOCUMENT_REVIEW as redr2
					WHERE redr2.REFUND_EXPORT_ID = re.REFUND_EXPORT_ID
			) as DOCUMENT_REVIEW_NO,
			rrl.WEIGHT as REFUND_WEIGHT,

			pc.`NAME` as REFUND_PRODUCT_CATEGORY,
			b.`NAME` as REFUND_BRAND,

			MD5(CONCAT(

					IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID),
							f_elcBuildInvoiceNoForInterfaces(re.CRSO_RELATION_ID,rec.`YEAR`,rec.REFUND_REQUEST_CREDIT_ID,TRUE,TRUE),
							f_elcBuildInvoiceNoForInterfaces(re.CRSO_RELATION_ID,red.`YEAR`,red.REFUND_REQUEST_DEBIT_ID,FALSE,TRUE)
					 ),
					CONCAT(cac.CODE,
						LPAD(re.CRSO_RELATION_ID, 8, 0),
						LPAD(re.PARTICIPANT_RELATION_ID, 8, 0)
					),
					REPLACE((CAST(f_elcGetTotalAmountForRefundExport(re.REFUND_EXPORT_ID) AS DECIMAL(10,2)) ), '.', ',')
				)) as HASH_TOTAL

			FROM TBL_REFUND_REQUEST as rr
			INNER JOIN TBL_PRODUCT as p
				on rr.PRODUCT_ID = p.PRODUCT_ID
			INNER JOIN TBL_REFUND_EXPORT as re
				ON re.REFUND_EXPORT_ID = rr.REFUND_EXPORT_ID
			INNER JOIN TBL_RELATION as cr
				ON cr.RELATION_ID = rr.CRSO_RELATION_ID
			INNER JOIN TBL_ADDRESS as ca
				ON cr.ADDRESS_ID = ca.ADDRESS_ID
			INNER JOIN TBL_COUNTRY as cac
				ON cac.COUNTRY_ID = ca.COUNTRY_ID
			INNER JOIN TBL_REFUND_EXPORT_SET_LINE resl
				ON re.REFUND_EXPORT_ID = resl.REFUND_EXPORT_ID
			LEFT JOIN TBL_REFUND_EXPORT_DEBIT as red
				ON red.REFUND_EXPORT_SET_ID = resl.REFUND_EXPORT_SET_ID
			LEFT JOIN TBL_REFUND_EXPORT_CREDIT as rec
				ON rec.REFUND_EXPORT_SET_ID = resl.REFUND_EXPORT_SET_ID
			INNER JOIN TBL_REFUND_REQUEST_LINE as rrl
				ON rr.REFUND_REQUEST_ID = rrl.REFUND_REQUEST_ID
			LEFT JOIN TBL_PRODUCT_CATEGORY as pc
				ON rrl.PRODUCT_CATEGORY_ID = pc.PRODUCT_CATEGORY_ID
			LEFT JOIN TBL_BRAND as b
				ON b.BRAND_ID = rrl.BRAND_ID
			LEFT JOIN TBL_REFUND_EXPORT_DOCUMENT_REVIEW as redr
				ON re.REFUND_EXPORT_ID = redr.REFUND_EXPORT_ID
			WHERE re.IS_EXPORTED = 1 -- Ready for export
				AND rr.CRSO_RELATION_ID = ?
			GROUP BY rr.REFUND_REQUEST_ID, rrl.LINE_INDEX
		";

		$bind_values = array($crso_id, 'DT_REFUND_REQUEST', $crso_id);
		$q = $this->query($sql, $bind_values);

		while ($row = $q->fetch())
		{
			//get em into invoice lines
			$this->insertInvoiceLine($row);
		}

		//get doc reviews
		//we do this as a seperate query, as there is refund exports that
		//might not be in the first query - cause they are only doc reviews fees
		// and there is not enough VStock to export RR's
		$sql = "
			SELECT
			'' AS INVOICE_LINE_INDEX,
			rec.REFUND_REQUEST_CREDIT_ID as REFUND_REQUEST_CREDIT_ID,
			red.REFUND_REQUEST_DEBIT_ID as REFUND_REQUEST_DEBIT_ID,
			IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID), rec.`YEAR`, red.`YEAR`) as `YEAR`,
			re.CRSO_RELATION_ID as CRSO_RELATION_ID,
			re.PRODUCT_ID as PRODUCT_ID,
			IF(
				ISNULL(red.REFUND_REQUEST_DEBIT_ID),
				CONCAT(
					'RC',
					re.CRSO_RELATION_ID,
					f_elcBuildInvoiceNoForExport(re.CRSO_RELATION_ID,rec.`YEAR`,rec.REFUND_REQUEST_CREDIT_ID,TRUE,TRUE)
				),
				CONCAT(
					'RI',
					re.CRSO_RELATION_ID,
					f_elcBuildInvoiceNoForExport(re.CRSO_RELATION_ID,red.`YEAR`,red.REFUND_REQUEST_DEBIT_ID,FALSE,TRUE)
				)
			) as INVOICE_NUMBER,
			re.REFUND_EXPORT_ID,
			NULL as REFUND_REQUEST_ID,
			NULL as REFUND_LINE_INDEX,
			CONCAT(IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID),'RC','RI'), re.CRSO_RELATION_ID) as JOURNAL_TEMPLATE,
			CONCAT(cac.CODE,
				LPAD(re.CRSO_RELATION_ID, 8, 0),
				LPAD(re.PARTICIPANT_RELATION_ID, 8, 0)
			) as CUSTOMER_NO,
			'' as CUSTOMER_NAME,
			LAST_DAY(DATE_SUB('".(currentDate()->format('Y-m-d H:i:s'))."', INTERVAL 1 MONTH)) as POSTING_DATE,
			'".(currentDate()->format('Y-m-d H:i:s'))."' as DOCUMENT_DATE,
			'' as EXTERNAL_DOCUMENT_NO,
			'' as CURRENCY_CODE,
			'G/L Account' as `TYPE`,
				(SELECT GL_ACCOUNT
				FROM TBL_CRSO_GLACCOUNT gc
				INNER JOIN TBL_DECLARATION_TYPE dt
					ON gc.DECLARATION_TYPE_ID = dt.DECLARATION_TYPE_ID
				WHERE gc.CRSO_RELATION_ID = ?
					AND dt.HANDLE = ?)
			as GL_ACCOUNT_NO,
			'".$this->getText('sum of document review fee', $this->getLangIdForCRSO($crso_id))."'  as DESCRIPTION, -- is this supposed to be a preferable combination of peroid, unit and weight information
			IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID), '-1', '1') as QUANTITY,
			'0,00' as UNIT_COST,
			(
				SELECT SUM(redr2.REVIEW_COST) FROM TBL_REFUND_EXPORT_DOCUMENT_REVIEW redr2 WHERE redr2.REFUND_EXPORT_ID = re.REFUND_EXPORT_ID
			) as LINE_AMOUNT_EXCL_VAT,
			IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID),
					-(ABS((SELECT SUM(redr2.REVIEW_COST) FROM TBL_REFUND_EXPORT_DOCUMENT_REVIEW redr2 WHERE redr2.REFUND_EXPORT_ID = re.REFUND_EXPORT_ID))),
					ABS((SELECT SUM(redr2.REVIEW_COST) FROM TBL_REFUND_EXPORT_DOCUMENT_REVIEW redr2 WHERE redr2.REFUND_EXPORT_ID = re.REFUND_EXPORT_ID))
			) as LINE_AMOUNT_EXCL_VAT,
			f_elcGetTotalAmountForRefundExport(re.REFUND_EXPORT_ID) as TOTAL_AMOUNT_EXCL_VAT,
			'' as NAVISION_VAT_CODE,
			'' as VAT_EXTENDED_TEXT,
			'' as DIMENSION_CODE_1,
			'' as DIMENSION_CODE_2,
			'' as DIMENSION_CODE_3,
			'' as DIMENSION_CODE_4,
			'' as DIMENSION_CODE_5,
			'' as DIMENSION_CODE_6,
			'' as DIMENSION_CODE_7,
			'' as DIMENSION_CODE_8,
			'' as TARIFF_NO,
			NULL as NET_WEIGHT,
			'' as TRANSACTION_TYPE,
			'' as EXIT_POINT,
			'' as TRANSPORT_METHOD,
			NULL as REFUND_EXPORT_DATE,
			CONCAT(cac.CODE,
				LPAD(re.CRSO_RELATION_ID, 8, 0),
				LPAD(IFNULL(re.BILL_TO_CUSTOMER_NO,re.PARTICIPANT_RELATION_ID), 8, 0)
			) as BILL_TO_CUSTOMER_NO,
			p.`NAME` as PRODUCT_DESCRIPTION,
			(
				SELECT COUNT(redr3.REVIEW_COST)
					FROM TBL_REFUND_EXPORT_DOCUMENT_REVIEW as redr3
					WHERE redr3.REFUND_EXPORT_ID = re.REFUND_EXPORT_ID
			) as DOCUMENT_REVIEW_NO,
			NULL as REFUND_WEIGHT,
			NULL as REFUND_PRODUCT_CATEGORY,
			NULL as REFUND_BRAND,
			MD5(CONCAT(

					IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID),
							f_elcBuildInvoiceNoForInterfaces(re.CRSO_RELATION_ID,rec.`YEAR`,rec.REFUND_REQUEST_CREDIT_ID,TRUE,TRUE),
							f_elcBuildInvoiceNoForInterfaces(re.CRSO_RELATION_ID,red.`YEAR`,red.REFUND_REQUEST_DEBIT_ID,FALSE,TRUE)
					 ),
					CONCAT(cac.CODE,
						LPAD(re.CRSO_RELATION_ID, 8, 0),
						LPAD(re.PARTICIPANT_RELATION_ID, 8, 0)
					),
					REPLACE((CAST(f_elcGetTotalAmountForRefundExport(re.REFUND_EXPORT_ID) AS DECIMAL(10,2)) ), '.', ',')
				)) as HASH_TOTAL
			FROM TBL_REFUND_EXPORT as re
			INNER JOIN TBL_PRODUCT as p
				on re.PRODUCT_ID = p.PRODUCT_ID
			INNER JOIN TBL_RELATION as cr
				ON cr.RELATION_ID = re.CRSO_RELATION_ID
			INNER JOIN TBL_ADDRESS as ca
				ON cr.ADDRESS_ID = ca.ADDRESS_ID
			INNER JOIN TBL_COUNTRY as cac
				ON cac.COUNTRY_ID = ca.COUNTRY_ID
			INNER JOIN TBL_REFUND_EXPORT_SET_LINE resl
				ON re.REFUND_EXPORT_ID = resl.REFUND_EXPORT_ID
			LEFT JOIN TBL_REFUND_EXPORT_DEBIT as red
				ON red.REFUND_EXPORT_SET_ID = resl.REFUND_EXPORT_SET_ID
			LEFT JOIN TBL_REFUND_EXPORT_CREDIT as rec
				ON rec.REFUND_EXPORT_SET_ID = resl.REFUND_EXPORT_SET_ID
			INNER JOIN TBL_REFUND_EXPORT_DOCUMENT_REVIEW as redr
				ON re.REFUND_EXPORT_ID = redr.REFUND_EXPORT_ID
			WHERE re.IS_EXPORTED = 1 -- Ready for export
				AND re.CRSO_RELATION_ID = ?
			GROUP BY re.REFUND_EXPORT_ID
		";

		$bind_values = array($crso_id, 'DT_REFUND_DOCUMENT_REVIEW', $crso_id);
		$q = $this->query($sql, $bind_values);

		while ($row = $q->fetch())
		{
			//get em into invoice lines
			$this->insertInvoiceLine($row);
		}

		//get refund fees
		$sql = "
			SELECT
			'' AS INVOICE_LINE_INDEX,
			rec.REFUND_REQUEST_CREDIT_ID as REFUND_REQUEST_CREDIT_ID,
			red.REFUND_REQUEST_DEBIT_ID as REFUND_REQUEST_DEBIT_ID,
			IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID), rec.`YEAR`, red.`YEAR`) as `YEAR`,
			re.CRSO_RELATION_ID as CRSO_RELATION_ID,
			re.PRODUCT_ID as PRODUCT_ID,
			IF(
				ISNULL(red.REFUND_REQUEST_DEBIT_ID),
				CONCAT(
					'RC',
					re.CRSO_RELATION_ID,
					f_elcBuildInvoiceNoForExport(re.CRSO_RELATION_ID,rec.`YEAR`,rec.REFUND_REQUEST_CREDIT_ID,TRUE,TRUE)
				),
				CONCAT(
					'RI',
					re.CRSO_RELATION_ID,
					f_elcBuildInvoiceNoForExport(re.CRSO_RELATION_ID,red.`YEAR`,red.REFUND_REQUEST_DEBIT_ID,FALSE,TRUE)
				)
			) as INVOICE_NUMBER,
			re.REFUND_EXPORT_ID,
			NULL as REFUND_REQUEST_ID,
			NULL as REFUND_LINE_INDEX,
			CONCAT(IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID),'RC','RI'), re.CRSO_RELATION_ID) as JOURNAL_TEMPLATE,
			CONCAT(cac.CODE,
				LPAD(re.CRSO_RELATION_ID, 8, 0),
				LPAD(re.PARTICIPANT_RELATION_ID, 8, 0)
			) as CUSTOMER_NO,
			'' as CUSTOMER_NAME,
			LAST_DAY(DATE_SUB('".(currentDate()->format('Y-m-d H:i:s'))."', INTERVAL 1 MONTH)) as POSTING_DATE,
			'".(currentDate()->format('Y-m-d H:i:s'))."' as DOCUMENT_DATE,
			'' as EXTERNAL_DOCUMENT_NO,
			'' as CURRENCY_CODE,
			'G/L Account' as `TYPE`,
				(SELECT GL_ACCOUNT
				FROM TBL_CRSO_GLACCOUNT gc
				INNER JOIN TBL_DECLARATION_TYPE dt
					ON gc.DECLARATION_TYPE_ID = dt.DECLARATION_TYPE_ID
				WHERE gc.CRSO_RELATION_ID = ?
					AND dt.HANDLE = ?)
			as GL_ACCOUNT_NO,
			'".$this->getText('refund fee', $this->getLangIdForCRSO($crso_id))."'  as `DESCRIPTION`, -- is this supposed to be a preferable combination of peroid, unit and weight information
			IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID), '-1', '1') as 'QUANTITY',
			IFNULL(re.REFUND_COST, '0.00') as UNIT_COST,
			IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID),
					-(ABS(IFNULL(re.REFUND_COST, '0.00'))),
					ABS(IFNULL(re.REFUND_COST, '0.00'))
			) as LINE_AMOUNT_EXCL_VAT,
			f_elcGetTotalAmountForRefundExport(re.REFUND_EXPORT_ID) as TOTAL_AMOUNT_EXCL_VAT,
			'' as NAVISION_VAT_CODE,
			'' as VAT_EXTENDED_TEXT,
			'' as DIMENSION_CODE_1,
			'' as DIMENSION_CODE_2,
			'' as DIMENSION_CODE_3,
			'' as DIMENSION_CODE_4,
			'' as DIMENSION_CODE_5,
			'' as DIMENSION_CODE_6,
			'' as DIMENSION_CODE_7,
			'' as DIMENSION_CODE_8,
			'' as TARIFF_NO,
			NULL as NET_WEIGHT,
			'' as TRANSACTION_TYPE,
			'' as EXIT_POINT,
			'' as TRANSPORT_METHOD,
			NULL as REFUND_EXPORT_DATE,
			CONCAT(cac.CODE,
				LPAD(re.CRSO_RELATION_ID, 8, 0),
				LPAD(IFNULL(re.BILL_TO_CUSTOMER_NO,re.PARTICIPANT_RELATION_ID), 8, 0)
			) as BILL_TO_CUSTOMER_NO,
			p.`NAME` as PRODUCT_DESCRIPTION,
			(
				SELECT COUNT(redr3.REVIEW_COST)
					FROM TBL_REFUND_EXPORT_DOCUMENT_REVIEW as redr3
					WHERE redr3.REFUND_EXPORT_ID = re.REFUND_EXPORT_ID
			) as DOCUMENT_REVIEW_NO,
			NULL as REFUND_WEIGHT,
			NULL as REFUND_PRODUCT_CATEGORY,
			NULL as REFUND_BRAND,
			MD5(CONCAT(

					IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID),
						f_elcBuildInvoiceNoForInterfaces(re.CRSO_RELATION_ID,rec.`YEAR`,rec.REFUND_REQUEST_CREDIT_ID,TRUE,TRUE),
						f_elcBuildInvoiceNoForInterfaces(re.CRSO_RELATION_ID,red.`YEAR`,red.REFUND_REQUEST_DEBIT_ID,FALSE,TRUE)
					 ),
					CONCAT(cac.CODE,
						LPAD(re.CRSO_RELATION_ID, 8, 0),
						LPAD(re.PARTICIPANT_RELATION_ID, 8, 0)
					),
					REPLACE((CAST(f_elcGetTotalAmountForRefundExport(re.REFUND_EXPORT_ID) AS DECIMAL(10,2)) ), '.', ',')
				)) as HASH_TOTAL
			FROM TBL_REFUND_EXPORT as re
			INNER JOIN TBL_PRODUCT as p
				on re.PRODUCT_ID = p.PRODUCT_ID
			 INNER JOIN TBL_RELATION as cr
			     ON cr.RELATION_ID = re.CRSO_RELATION_ID
			 INNER JOIN TBL_ADDRESS as ca
			     ON cr.ADDRESS_ID = ca.ADDRESS_ID
			 INNER JOIN TBL_COUNTRY as cac
			     ON cac.COUNTRY_ID = ca.COUNTRY_ID
			 INNER JOIN TBL_REFUND_EXPORT_SET_LINE resl
			     ON re.REFUND_EXPORT_ID = resl.REFUND_EXPORT_ID
			 LEFT JOIN TBL_REFUND_EXPORT_DEBIT as red
			     ON red.REFUND_EXPORT_SET_ID = resl.REFUND_EXPORT_SET_ID
			 LEFT JOIN TBL_REFUND_EXPORT_CREDIT as rec
			     ON rec.REFUND_EXPORT_SET_ID = resl.REFUND_EXPORT_SET_ID
			 WHERE re.IS_EXPORTED = 1 -- Ready for export
			     AND re.CRSO_RELATION_ID = ?
		";

		$bind_values = array($crso_id, 'DT_REFUND_ADMINISTRATIVE_FEE', $crso_id);
		$q = $this->query($sql, $bind_values);

		while ($row = $q->fetch())
		{
			//get em into invoice lines
			$this->insertInvoiceLine($row);
		}

		$this->updateRefundExportStatus($crso_id);
		$this->db->commit();
	}
	// }}}

	// {{{ private function insertInvoiceLine($row)
	protected function insertInvoiceLine($row)
	{
		//TODO: need to correct signage of amounts
		self::$colsForInvoice[2] = "REFUND_REQUEST_".(substr($row->JOURNAL_TEMPLATE, 0, 2) == 'RC' ? 'CREDIT' : 'DEBIT')."_ID";
    		$sql = "INSERT INTO TBL_REFUND_EXPORT_".(substr($row->JOURNAL_TEMPLATE, 0, 2) == 'RC' ? 'CREDIT' : 'DEBIT')."_LINE (";
		$columns = "";
		$bindValues = "";
		$values = array();

		foreach(self::$colsForInvoice as $col)
		{
			$columns .= $col.", ";
			$bindValues .= "?, ";
			$col = str_replace('`', '', $col);
			if(isset($row->{$col}))
			{
				if ($col == 'INVOICE_LINE_INDEX')
					$values[] = $this->processInvoiceLineNumber($row->INVOICE_NUMBER);
				else
					$values[] = $row->{$col};
			}
			else
			{
				$values[] = null;
			}
		}
		$sql .= substr($columns, 0, -2).") VALUES (".substr($bindValues, 0, -2).")";
		$q = $this->query($sql, $values);
	}
	// }}}

	protected function doExport($crso_id)
	{
		$this->db->begin();

		$this->query("UPDATE TBL_REFUND_EXPORT_CREDIT_LINE SET IS_EXPORTED = 1 WHERE CRSO_RELATION_ID = ? AND IS_EXPORTED = 0", array($crso_id));
		$this->query("UPDATE TBL_REFUND_EXPORT_DEBIT_LINE SET IS_EXPORTED = 1 WHERE CRSO_RELATION_ID = ? AND IS_EXPORTED = 0", array($crso_id));
		$this->db->commit();
	}

	private function updateRefundExportStatus($crso_id)
	{
		$date = currentDate();
		$vDate = $date->getYear().'-'.$date->getMonth().'-'.$date->getDay();
		$sql = "
					UPDATE
						TBL_REFUND_EXPORT
					SET
						EXPORT_DATE = ?,
						IS_EXPORTED = ?
					WHERE
						CRSO_RELATION_ID = ?
					AND
						IS_EXPORTED = ?

				";
		$q = $this->query($sql, array($vDate, REFUND_EXPORT_EXPORTED, $crso_id, REFUND_EXPORT_PENDING));
	}

	protected function processLineNumber($invoice_id)
	{
		if(!isset(self::$invoice_numbers1[$invoice_id]))
			return self::$invoice_numbers1[$invoice_id] = 1;
		return ++self::$invoice_numbers1[$invoice_id];
	}

	protected function processInvoiceLineNumber($invoice_id)
	{
		if(!isset(self::$invoice_numbers2[$invoice_id]))
			return self::$invoice_numbers2[$invoice_id] = 1;
		return ++self::$invoice_numbers2[$invoice_id];
	}
}
?>
