<?php
require_once(IWARE.'AbstractObject.php');
/**
 * @package ELC
 * @subpackage User
 * @since 28-aug-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class ELCGroupLanguage extends AbstractObject
{
	/**
	 * @var $group_id
	 */
	var $group_id;
	/**
	 * @var Language $language
	 */
	var $language;
	/**
	 * @var int $status
	 */
	var $status;
	/**
	 * @param int $group_id
	 * @param int $language_id
	 */
	function check($group_id=null,$language_id=null)
	{
		$group_id || $group_id = $this->group_id;
		$language_id || $language_id = $this->language_id;

		$sql = "SELECT * FROM TBL_GROUP_LANGUAGE
			WHERE GROUP_ID = ?
				AND LANGUAGE_ID = ?
				AND `STATUS` = ?";
		$q = $this->db->query($sql);
		$q->bind_values($group_id,$language_id,GROUP_LANGUAGE_ENABLED);
		$q->execute();
		return (bool)($r = $q->fetch());
	}
	/**
	 * Returns all the languages for a group in either a resultset or an array selected by status
	 * @param int $group_id
	 * @param string $return
	 * @return mixed
	 */
	function getLanguagesForGroupByStatus($group_id=null,$status=GROUP_LANGUAGE_ENABLED,$return='array')
	{
		$group_id || $group_id = $this->group_id;
		$sql = "SELECT LANGUAGE_ID
			FROM TBL_GROUP_LANGUAGE
			WHERE GROUP_ID = ?
				AND `STATUS` = ?";
		$q = $this->db->query($sql);
		$q->bind_values($group_id,$status);
		$q->execute();
		if($return == 'q')
			return $q;

		$rv = array();
		while($r = $q->fetch())
			$rv[$r->LANGUAGE_ID] = $r->LANGUAGE_ID;
		return $rv;
	}
	/**
	 * @param int $group_id
	 * return mixed
	 */
	function getEnabledLanguages($group_id=null)
	{
		return $this->getLanguagesForGroupByStatus($group_id);
	}
	function removeAllForGroup($group_id=null)
	{
		$group_id || $group_id = $this->group_id;
		$sql = "DELETE FROM TBL_GROUP_LANGUAGE
			WHERE GROUP_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($group_id);
		return $q->execute();
	}
	function insert()
	{
		$this->status = GROUP_LANGUAGE_ENABLED;
		return parent::insert();
	}
	/**
	 * @access protected
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_GROUP_LANGUAGE';
		$this->objectmap['primary']['group_id'] = 'GROUP_ID';
		$this->objectmap['primary']['language->id'] = 'LANGUAGE_ID';
		$this->objectmap['properties']['status'] = 'STATUS';
	}
}
/**#@+
 * Definitions for GROUP_LANGUAGES
 * For the future: add different stati
 */
define('GROUP_LANGUAGE_DISABLED',0);
define('GROUP_LANGUAGE_ENABLED',1);
/**#@-*/
?>
