<?php
require_once(IWARE.'ListBuilder.php');
require_once(IWARE.'group.php');
/**
 * @package ELC
 * @subpackage List
 * @since 12-jul-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class ELCListBuilder extends ListBuilder
{
	function ELCListBuilder(&$db,$langIf=false)
	{
		parent::ListBuilder($db, $langIf);
	}
}
/**
 * Add default callbacks or often used callbacks here. (crso/participant select etc.)
 * @package ELC
 * @subpackage List
 * @abstract
 */
class ELCDefaultList extends DefaultList
{
	var $statusmap = array();
	var $cache = array();
	function getCRSOSelect($rv='setValue')
	{
		$setValue = !empty($_REQUEST['CRSO_RELATION_ID'])?(int)$_REQUEST['CRSO_RELATION_ID']:false;
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_CRSO:
				$filter = '';
				$selectedValue = '';
				if(isset($_REQUEST['CRSO_RELATION_ID']) && ($_REQUEST['CRSO_RELATION_ID']!=$_SESSION['crso_relation_id']))
					throw new IWAJAXPermissionException('Incorrect CRSO Id supplied');
				$_REQUEST['CRSO_RELATION_ID'] = $setValue = $_SESSION['crso_relation_id'];
				break;
			case GROUP_PARTICIPANT:

				$remote = null;
				// Check for any active Remote Viewer connections.
				if(IWRegistry::isRegistered('remote_viewer')) {
					$remote = IWRegistry::registry('remote_viewer');
				}

				$sql = 'SELECT CRSO_RELATION_ID, R.`NAME`
						FROM TBL_CRSO_PARTICIPANT_PRODUCT as CP
						INNER JOIN TBL_RELATION as R
							ON CP.CRSO_RELATION_ID = R.RELATION_ID
						WHERE PARTICIPANT_RELATION_ID = ?
							AND CP.PRODUCT_ID = ? ';
				if($remote) {
					$sql .= " AND CP.CRSO_RELATION_ID = ".$remote->crso->relation->id;
				}
				$q = $this->db->query($sql);
				$q->bind_values($_SESSION['selected_participant_id'],$_SESSION['selected_product_id']);
				$q->execute();
				$rs = $q->fetchAll();
				if(count($rs)==1)
				{
					$r = $rs[0];
					$filter = '';
					$selectedValue = '';
					$_REQUEST['CRSO_RELATION_ID'] = $setValue = $r->CRSO_RELATION_ID;
				}
				break;
			case GROUP_ACCOUNTANT:
				// get all accountant's active CRSOs for the selected product
				require_once(ELC_CLASSES.'AccountantCRSO.php');
				$crso = new AccountantCRSO($this->db);
				$q = $crso->getAccountantCRSOs($_SESSION['accountant_relation_id'], $_SESSION['selected_product_id']);
				$rs = $q->fetchAll();
			break;
			default:
				require_once(ELC_CLASSES.'CRSO.php');
				$crso = new CRSO($this->db);
				$q = $crso->selectCRSOs(1,null,$_SESSION['selected_product_id']);
				$rs = $q->fetchAll();
				break;
		}
		if(isset($rs) && count($rs) == 1)
		{
			$setValue = $rs[0]->CRSO_RELATION_ID;
			$_REQUEST['CRSO_RELATION_ID'] = $setValue;
		}

		if(isset($$rv))
			return $$rv;
		$selectedValue = '';

		$select = new Form_Select('CRSO_RELATION_ID');
		$select->addOption(' - '.$this->getText('CRSO Select').' - ', '', false);
		foreach($rs as $r)
		{
			if($setValue==$r->CRSO_RELATION_ID)
				$selectedValue = $r->NAME;
			$select->addOption(((mb_strlen($r->NAME)>35)?mb_substr($r->NAME,0,35).'...':$r->NAME), $r->CRSO_RELATION_ID, false);
		}
		$select->setSelectedValue($setValue);

		$filter = $select->getHtml();

		return $$rv;
	}

	function getParticipantSelect($rv='setValue')
	{
		$setValue = !empty($_REQUEST['PARTICIPANT_RELATION_ID'])?(int)$_REQUEST['PARTICIPANT_RELATION_ID']:false;

		$crso_id = isset($_REQUEST['CRSO_RELATION_ID'])?(int)$_REQUEST['CRSO_RELATION_ID']:false;
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_CRSO:
				 $crso_id = $_SESSION['crso_relation_id'];
			break;
			case GROUP_PARTICIPANT:
				if(isset($_REQUEST['PARTICIPANT_RELATION_ID']) && ($_REQUEST['PARTICIPANT_RELATION_ID']!=$_SESSION['selected_participant_id']))
					throw new IWAJAXPermissionException('Incorrect Participant Id supplied');
				$_REQUEST['PARTICIPANT_RELATION_ID'] = $setValue = $_SESSION['selected_participant_id'];
				$selectedValue = $_SESSION['selected_participant_id'];
				if($setValue&&!isset($_SESSION['participant_ids'][$setValue]))
					throw new IWAJAXPermissionException('User Trying to Access Unauthorized Participant in List');
				$filter = '';
			break;
		}

		if(!is_numeric($crso_id))
			$filter = '';
		if(isset($$rv))
			return $$rv;
		$selectedValue = '';

		require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
		$p = new CRSOParticipantProduct($this->db);

		$producer_exporter = isset($this->producer_exporter)?$this->producer_exporter:false;

		$is_active = null;
		$q = $p->getCRSOParticipants($crso_id,'q',$producer_exporter, $_SESSION['selected_product_id'], $is_active);

		$select = new Form_Select('PARTICIPANT_RELATION_ID');
		$select->addOption(' - '.$this->getText('Participant Select').' - ', '', false);
		while($r = $q->fetch())
		{
			if($setValue==$r->PARTICIPANT_RELATION_ID)
				$selectedValue = $r->PARTICIPANT_NAME;
			if($_SESSION['session_group_group_id'] == GROUP_PARTICIPANT)
			{
				if(!in_array($r->PARTICIPANT_RELATION_ID, $_SESSION['participant_ids']))
					continue;
			}
			$name = $r->PARTICIPANT_NAME;
			$select->addOption(((mb_strlen($name)>35)?mb_substr($name,0,35).'...':$name), $r->PARTICIPANT_RELATION_ID, false);
		}
		$select->setSelectedValue($setValue);

		$filter = $select->getHtml();

		return $$rv;
	}

	function getMonthSelect($rv='setValue')
	{
		$setValue = !empty($_REQUEST['MONTH'])?(int)$_REQUEST['MONTH']:false;
		if(isset($$rv))
			return $$rv;
		$selectedValue = '';

		$select = new Form_Select('MONTH');
		$select->addOption(' - '.$this->getText('Month Select').' - ', '', false);
		for($i=1;$i<13;$i++)
		{
			if($setValue==$i)
				$selectedValue = $this->tfMonth(array('value'=>$i));
			$select->addOption($this->tfMonth(array('value'=>$i)), $i, false);
		}
		$select->setSelectedValue($setValue);
		$filter = $select->getHtml();
		return $$rv;
	}

	function getYearSelect($rv='setValue',$maxyear=false)
	{
		$date = currentDate();
		$year = (int)($maxyear?$maxyear:$date->getYear());

		$setValue = !empty($_REQUEST['YEAR']) ?
			(int)$_REQUEST['YEAR']:$year;

		if(isset($$rv))
			return $$rv;
		$selectedValue = '';

		$select = new Form_Select('YEAR');

		if($year != WEEE_APPLICATION_START_YEAR)
		{
			$select->addOption(' - '.$this->getText('Year Select').' - ', '', false);
		}
		else
		{
			$filter = '';
			$selectedValue = WEEE_APPLICATION_START_YEAR;
			return $$rv;
		}

		for($i=WEEE_APPLICATION_START_YEAR; $i<=$year; $i++)
		{
			if($setValue==$i)
				$selectedValue = $i;
			$select->addOption($i, $i, false);
		}

		$select->setSelectedValue($setValue);
		$filter = $select->getHtml();
		return $$rv;
	}

	function getProductSelect($rv='setValue')
	{
		$selectedValue = $filter = '';
		$setValue = $_SESSION['selected_product_id'];
		return $$rv;
	}

	function getDynamicProductSelect($rv='setValue')
	{
		$setValue = !empty($_REQUEST['PRODUCT_ID'])?(int)$_REQUEST['PRODUCT_ID']:false;
		if(isset($$rv))
			return $$rv;
		$selectedValue = '';

		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_PARTICIPANT:
				require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
				$cp = new CRSOParticipantProduct($this->db);
				$products = $cp->getParticipantProducts($_SESSION['selected_participant_id']);
			break;
			case GROUP_CRSO:
				require_once(ELC_CLASSES.'CRSOProduct.php');
				$cp = new CRSOProduct($this->db);
				$products = $cp->getProducts($_SESSION['crso_relation_id']);
			break;
			case GROUP_ACCOUNTANT:
				require_once(ELC_CLASSES.'AccountantCRSO.php');
				$accountant_crso = new AccountantCRSO($this->db);
				$ac = $accountant_crso->getAccountantCRSOs($_SESSION['accountant_relation_id']);

				$sql = "SELECT CP.PRODUCT_ID, P.`NAME` as PRODUCT_NAME
					FROM TBL_CRSO_PRODUCT as CP
					INNER JOIN TBL_PRODUCT as P
						ON CP.PRODUCT_ID = P.PRODUCT_ID
					WHERE CP.CRSO_RELATION_ID IN (";

				$crsos = array();
				while($r=$ac->fetch()){
					$crsos[] = $r->CRSO_RELATION_ID;
					$sql .= "?,";
				}
				$sql = substr($sql, 0, strlen($sql)-1).") GROUP BY CP.PRODUCT_ID";

				$products = $this->db->query($sql);
				$products->bind_values($crsos);
				$products->execute();
			break;
			case GROUP_PUBLIC:
			case GROUP_SYSADMIN:
			default:
				require_once(ELC_CLASSES.'Product.php');
				$p = new Product($this->db);
				$products = $p->getProducts();
			break;
		}

		$rs = $products->fetchAll();
		// preselect in case there's only one product
		if(isset($rs) && count($rs) == 1)
		{
			$setValue = $rs[0]->PRODUCT_ID;
			$_REQUEST['PRODUCT_ID'] = $setValue;
		}

		$select = new Form_Select('PRODUCT_ID');
		$select->addOption(' - '.$this->getText('Product Select').' - ', '', false);

		foreach($rs as $r)
		{
			if($setValue==$r->PRODUCT_ID)
				$selectedValue = $r->PRODUCT_ID;
			$select->addOption($r->PRODUCT_NAME, $r->PRODUCT_ID, false);
		}
		$select->setSelectedValue($setValue);
		$filter = $select->getHtml();
		return $$rv;
	}

	/**#@+
	 * Default ELC Table Factory Processors
	 * @param Array $args
	 */
	/**
	 * Shows a green `vinkje` or a red cross.
	 */
	function tfBooleanIcon($args)
	{
		if($this->_showType!=LIST_HTML_OUTPUT)
			return $args['value']?'true':'false';
		$img = ($args['value']) ? 'true.gif' : 'false.gif';
    	return '<img src="/'.SITE_PATH.'images/i-w/'.$img.'" />';
	}
	/**
	 * Number formats a weight with 3 decimals
	 */
    function tfValueProcessorWeightWithDecimals($args)
    {
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		return locale_nf($args['value'], 3);
    }
	/**
	 * Number formats a quantity with 2 decimals
	 */
    function tfValueProcessorQuantity($args)
    {
    	$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
    	return locale_nf($args['value'], 2);
    }
    /**
     * Number formats a quanitity with no decimals
     */
    function tfValueProcessorQuantityNoDecimals($args)
    {
    	$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
    	return locale_nf($args['value'], 0);
    }
    /**
     * Gets the month name thru the LOCALE
     */
    function tfMonth($args)
	{
		$args['extra'] = sprintf(' sort_val="%s" ', $args['value']);
		return strftime('%B', mktime(1, 1, 1, $args['value'], 1) );
	}
	function tfPercentageNoDecimals($args)
	{
    	$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
    	return locale_nf($args['value'], 0).'%';
	}
	function tfFixedFontWidth($args)
	{
		if($this->_showType!=LIST_HTML_OUTPUT)
			return $args['value'];
    	$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
    	return '<span style="font-family: monospace;font-size:9pt;">'.$args['value'].'</span>';
	}
	function tfDeclarationStatus($args)
	{
		if(isset($this->statusmap['declaration'])&&isset($this->statusmap['declaration'][$args['value']]))
			list($translatedStatus, $class) = $this->statusmap['declaration'][$args['value']];
		else
		{
			require_once(ELC_CLASSES.'Declaration.php');
			list($translatedStatus, $class) = Declaration::translateStatus($this->langIf,Declaration::getDisplayStatus($args['value']));
			$this->statusmap['declaration'][$args['value']] = array($translatedStatus,$class);
		}
		if($this->_showType==LIST_HTML_OUTPUT)
    		return '<span class="'.$class.'">'.$translatedStatus.'</span>';
    	return $translatedStatus;
	}

	function tfDate($args)
	{
		$bits  = explode('-', $args['value']);
		if(isset($args['extra']))
			$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		return strftime('%x', mktime(1,1,1, $bits[1], $bits[2], $bits[0]));
	}

	function tfMakePrettyDate($args)
	{
		return Form_Date::makePretty($args['value']);
	}

	function tfParticipantStatus($args)
	{
		require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
		require_once(ELC_CLASSES.'Participant.php');
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		list($trans,$class) = Participant::translateStatus($this->langIf, $args['value']);
		if($this->_showType==LIST_HTML_OUTPUT)
			return sprintf('<span class="%s">%s</span>',$class,$trans);
		return $trans;
	}

	function tfLongNames($args)
	{
		if($this->_showType!=LIST_HTML_OUTPUT)
			return $args['value'];
		return (mb_strlen($args['value'])>35)?mb_substr($args['value'],0,35).'...':$args['value'];
	}

	function tfVat($args)
	{
		require_once(IWARE.'VAT.php');
		$vat = new VAT($this->db);
		return $vat->getVATDisplay($args['value']);
	}

	function tfDocumentLink($args)
	{
		if(!is_numeric($args['value'])||($args['value']==0)||($this->_showType!=LIST_HTML_OUTPUT))
			return $this->tfBooleanIcon($args);
		if(!isset($args['hidename']))
		{
			require_once(ELC_CLASSES.'Document.php');
			$doc = new ELCDocument($this->db,$args['value'],true);
			$name = $doc->name;
		}
		else
			$name = isset($this->cache['tfDocumentLink'])?$this->cache['tfDocumentLink']:($this->cache['tfDocumentLink'] = $this->getText('Download'));
		return sprintf('<a href="/%sscripts/i-w/document/download.php?Document[id]=%d">%s</a>',SITE_PATH,$args['value'],$name);
	}

	function tfUnlinkDocumentLink($args)
	{
		if (!isset($args['document_id_column']))
			$args['document_id_column'] = 'DOCUMENT';

		if (!$args['row'][$args['document_id_column']])
			return '';

		$name = $this->getText('Unlink');

		return sprintf('<a href="/%s%s?id=%d" onclick="return confirm(\'%s\')">%s</a>',
			SITE_PATH, $args['url'], $args['value'], jsescape(gettext('Are you sure you want to unlink the document?'), "'"), $name);
	}

	function tfProductCategory($args)
	{
		if(!is_numeric($args['value']))
			return $args['value'];
		require_once(ELC_CLASSES.'ProductCategory.php');
		$pc = new ProductCategory($this->db,$args['value'],true);
		return $pc->name;
	}

	function tfBrand($args)
	{
		if(!is_numeric($args['value']))
			return $args['value'];
		require_once(ELC_CLASSES.'Brand.php');
		$b = new Brand($this->db,$args['value'],true);
		return $b->name;
	}

	function tfRefundRequestStatus($args)
	{
		require_once(ELC_CLASSES.'RefundRequest.php');
		list($trans,$class) = RefundRequest::translateStatus($this->langIf,$args['value']);
		if($this->_showType==LIST_HTML_OUTPUT)
			return sprintf('<span class="%s">%s</span>',$class,$trans);
		return $trans;
	}

	function tfTrustee($args)
	{
		/*if(!$args['value'])
			return '';*/

		require_once(ELC_CLASSES.'Trust.php');
		$trust = new Trust($this->db);
		$q = $trust->selectTrusts($args['row']['PARTICIPANT_RELATION_ID'],TRUST_APPROVED,(int)$_REQUEST['CRSO_RELATION_ID'],$_SESSION['selected_product_id']);
		$subrogators = '';
		$rs = $q->fetchAll();
		if(!count($rs))
			return '';
		foreach($rs as $r)
			$subrogators .= sprintf('%s<br />',$r->NAME);

		$this->trustee_lists[$args['row']['PARTICIPANT_RELATION_ID']] = sprintf(
			'<strong>%s</strong><br />%s',$this->getText('Subrogator List'),$subrogators);
		return sprintf('<img id="trusteeList_%s" src="/%simages/i-w/true.gif" />',
			$args['row']['PARTICIPANT_RELATION_ID'],
			SITE_PATH);
	}


	function thAlignRight($args) {
		return 'class="th-right"';
	}

    /**#@-*/
}
?>
