<?php
/**
 * Message Handling classes
 * @author Chris Wesseling <chris@basesoft.com>
 * @version $Id$
 * @package ELC
*/

/**
 * Meet the parent
 */
require_once(IWARE.'message_handler.php');

//Events
define('PARTICIPANT_REGISTRATION', 1);
define('PARTICIPANT_CONFIRMATION', 2); // we don't need this anymore. we send a single mail with a token
define('PARTICIPANT_CONFIRMATION_DAY_AFTER', 3);
define('NEW_USER', 4); // we don't need this anymore. we send a single mail with a token
define('NEW_USER_DAY_AFTER', 5);
define('LOST_PASSWORD', 6);
define('NEW_INVOICE', 7);
define('MESSAGE_REFUND_REQUEST', 8);
define('MESSAGE_UNREAD_NOTIFICATION_REMINDER', 9);
define('MESSAGE_DECLARATION_REMINDER', 10);
define('NEW_RR_INVOICE', 11);
define('AC_REMINDER', 12);
define('NEW_REMINDER_LATE_GUARANTEE', 13);

//RECEIVERS
define('RECEIVER_PARTICIPANT',1);
define('RECEIVER_CRSO',2);
define('RECEIVER_OPERATOR',3);
define('RECEIVER_SYSADMIN',4);
define('RECEIVER_USER',5);
define('RECEIVER_EXTERNAL',6);

/**
 *  Handles Message Events
 */
class ELCMessageHandler extends MessageHandler
{
	/**
	 * @return array (MESSAGE_EVENT_ID=>array(RECEIVER_ID, ...))
	 */
	function getReceivers()
	{
	}
	function setReceivers($event, $message_id, $receiver_ids=array())
	{
	}
	/**#@+
	 * @access private
	 */
	function _setOrigin()
	{
		$this->_sender = MAIL_FROM;
		$this->_senderName = 'WEEE Black Box';
		$this->_return_path = MAIL_FROM;
		return parent::_setOrigin();
	}
	/**
	 * Gets destinations.
	 * Currently only does E-mail
	 * Returns it in an array AND leaves them in $event->messages
	 * @todo refactor this code.. db_edit and db are 1 now..
	 * @param MessageEvent $event
	 * @param int MESSAGE_ID
	 * @param int MESSAGE_TYPE
	 * @returns array
	 */
	function _getDestinations(&$event, $message_id, $message_type)
	{
		$rv = array();
		// set DESTINATION query field
		switch ( $message_type )
		{
			case MESSAGE_EMAIL:
				$props = array('email', 'backup_email');
			break;
		}

		//get MESSAGE_RECEIVER_ID
		$sql = "SELECT MESSAGE_RECEIVER_ID
				FROM TBL_MESSAGE_EVENT_RECEIVER
				WHERE MESSAGE_EVENT_ID = ?
					AND MESSAGE_ID = ?
					AND MESSAGE_TYPE = ?";
		$qt = $this->db->query($sql);
		$qt->bind_values(array($event->id, $message_id, $message_type));
		$qt->execute();
		while ( $r = $qt->fetch() )
		{
			//PREPARE a DESTINATION query
			switch ( $r->MESSAGE_RECEIVER_ID )
			{
				case RECEIVER_PARTICIPANT:

					if (isset($event->params['user']))
					{
						$destination_entity =& $event->params['user'];
						$destination_entity->isComplete() || $destination_entity->set();
						$destination_entity =& $destination_entity->info_person;
					}
					else if (isset($event->params['info_person']))
						$destination_entity =& $event->params['info_person'];
					else
						trigger_error('No destination supplied', E_USER_WARNING);
				break;
				case RECEIVER_CRSO:
					$destination_entity =& $event->params['crso'];
					$destination_entity->isComplete() || $destination_entity->set();
					$destination_entity =& $destination_entity->relation;
					require_once(ELC_CLASSES.'RelationUser.php');
					$RelationUser = new RelationUser($this->db);
					$admin =& $RelationUser->getRelationAdmin($destination_entity->id);
					if ($admin->id)
					{
						$admin->set();
						$destination_entity =& $admin->info_person;
					}
					else
						$destination_entity = '';
				break;
				case RECEIVER_OPERATOR:
					$destination_entity =& $event->params['operator']; //fixme
				break;
				case RECEIVER_SYSADMIN:
					$destination_entity =& $l337h4xx0r; //fixme
				break;
				case RECEIVER_USER:
					$destination_entity =& $event->params['user'];
					$destination_entity->isComplete() || $destination_entity->set();
					$destination_entity =& $destination_entity->info_person;
				break;
				case RECEIVER_EXTERNAL:
					$destination_entity = $event->params['external'];
				break;
			}

			if ( ! is_object($destination_entity) )
			{
				trigger_error('ELCMessageHandler: OOPS, destination_entity is not an object on line '.__LINE__."\nTODO FIXME", E_USER_NOTICE);
				return $rv;
			}

			$destination_entity->isComplete() || $destination_entity->set();
			foreach ( $props as $prop )
			{
				if ( !empty($destination_entity->$prop) )
				{
				   	$dest = $destination_entity->$prop;

					$event->messages[$message_type][$dest][$message_id] = '';
					$event->destination_info[$dest] =& $destination_entity;
					$rv[] = $dest;
				}
			}
		}
		return $rv;
	}
	function _filterDestinations($dests, &$event, $message_id, $message_type)
	{
		return $dests; //never filter initiator
		//return parent::_filterDestinations($dests, $event, $message_id, $message_type);
	}
	/**
	 * Alter message in $event->messages
	 * @param array $dests
	 * @param MessageEvent $event
	 * @param int $message_id MESSAGE_ID
	 * @param int $message_type MESSAGE_TYPE
	 * @return array of Messages
	 */
	function _alterMessage($dests, &$event, $message_id, $message_type)
	{
		if ( !isset($event->params['relationuser']) )
		{
			require_once(ELC_CLASSES.'RelationUser.php');
			$RelationUser = new RelationUser($this->db);
		}
		else
			$RelationUser =& $event->params['relationuser'];

		//Participant
		if ( !empty($event->params['participant']) )
		{
			$o =& $event->params['participant'];
			$o->isComplete() || $o->set();
			$event->addReplacement('Url', $o->website);

			//infoperson stuff
			$o->relation->isComplete() || $o->relation->set();
			$event->addReplacement('CompanyName', $o->relation->name);
			$event->addReplacement('Email', $o->relation->email);

			//admin
			$admin =& $RelationUser->getRelationAdmin($o->relation->id);
			if ( !empty($admin->id) )
			{
				$admin->set();
				$admin->info_person->set();
				$event->addReplacement('AdminName', $admin->info_person->last_name);
			}
		}

		//CRSO
		if ( !empty($event->params['crso']) )
		{
			$o =& $event->params['crso'];
			$o->isComplete() || $o->set();

			//infoperson stuff
			$o->relation->isComplete() || $o->relation->set();
			$event->addReplacement('CrsoEmail', $o->relation->email);
			$event->addReplacement('CrsoCompanyName', $o->relation->name);

			//admin
			$admin =& $RelationUser->getRelationAdmin($o->relation->id);
			if ( !empty($admin->id) )
			{
				$admin->set();
				$admin->info_person->set();
				$event->addReplacement('CrsoAdminName', $admin->info_person->last_name);
			}
		}

		if(isset($RelationUser->relation))
		{
			$RelationUser->relation->isComplete() || $RelationUser->relation->set();
			$event->addReplacement('CompanyName', $RelationUser->relation->name);
		}

		//extra
		if ( isset($event->params['user']) )
		{
			$event->addReplacement('UserName', $event->params['user']->name);
			$event->addReplacement('LoginLink', 'http://'.SITE_SERVER.SITE_PATH.'scripts/elc/public/login.php?action=1&token=' . $event->params['user']->login_token);
		}

		// email to importer replacements
		if(isset($event->params['email_to_importer']))
		{
			$params = $event->params['email_to_importer'];
			$event->addReplacement('ImporterConfirmationLink','<a href="'.$params['confirmation_link'].'">'.$params['confirmation_link'].'</a>');
			$event->addReplacement('ImporterCountry', $params['importer_country']);
			$event->addReplacement('ImporterName', $params['importer_name']);
		}


		if(isset($event->params['subjects']))
		{
			$event->addReplacement('NotificationSubjects','<ul><li>'.join('</li><li>', $event->params['subjects']).'</li></ul>');
		}

		//alter
		return parent::_alterMessage($dests, $event, $message_id, $message_type);
	}

	/**#@-*/
	/**
	 * Prevent Flooding of the email server when testing...
	 * See ticket #1111
	 */
	/*
	function handleEvent(&$event, $dry_run)
	{
		if(!defined('TEST_SITE')||(!TEST_SITE))
			return parent::handleEvent($event,$dry_run);

		$testSiteSkips = array(
			//PARTICIPANT_CONFIRMATION_DAY_AFTER,
			//NEW_USER_DAY_AFTER,
			NEW_INVOICE,
			MESSAGE_UNREAD_NOTIFICATION_REMINDER,
			MESSAGE_DECLARATION_REMINDER,
			NEW_RR_INVOICE
			);

		if(in_array($event->id,$testSiteSkips))
			$dry_run = true;

		return parent::handleEvent($event,$dry_run);
	}
	*/
}
?>
