<?php
/**
 * ELCReportQuantityTable
 *
 * Makes a nice table showing pretty quantities for the given dataset.
 * Used in ParticipantsReport
 *
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @copyright (2006) BaseSoft B.V.
 * @since 02-06-2006
 * @version $Id$
 * @package I-Ware
 * @subpackage Core
 */
class ELCReportQuantityTable
{
	/**
	 * @access private
	 */
	var $_title = '';
	var $_subtitle = '';
	var $_data = array();
	var $_note = '';
	var $_totalRowIndex = null;
	var $_numtype = 'Int';
	var $rowProc = array();
	var $hidePrevOn = 1;
	var $hideCurrOn = 13;
	public static $langIf = null;
	public static $db = null;
	public static $props = null;

	/**
	 * Constructor, use the static function setUp() to setup this class
	 */
	function ELCReportQuantityTable()
	{
	}

	/**
	 * Setups the quantity tables
	 * @static
	 */
	public static function setUp($langIf=null, $db=null, $properties=null)
	{
		if (!is_null($langIf))
			ELCReportQuantityTable::langIf($langIf);

		if (!is_null($db))
			ELCReportQuantityTable::db($db);

		if (!is_null($properties))
			ELCReportQuantityTable::properties($properties);
	}

	/**
	 * @static
	 */
	public static function langIf($_langIf=null)
	{
		if (!is_null($_langIf))
			self::$langIf = $_langIf;
		return self::$langIf;
	}

	/**
	 * @static
	 */
	public static function db($_db=null)
	{
		if (!is_null($_db))
			self::$db = $_db;
		return self::$db;
	}

	/**
	 * @static
	 */
	public static function properties($_props=null)
	{
		if (!is_null($_props))
			self::$props = $_props;
		return self::$props;
	}

	function getProperty($name)
	{
		$props = ELCReportQuantityTable::properties();
		return $props[$name];
	}

	/**
	 * Sets the caption and subtitle for the object
	 */
	function setTitles($title, $subtitle)
	{
		$langIf = ELCReportQuantityTable::langIf();
		$this->_title = is_null($langIf) ? $title : $langIf->getText($title);
		$this->_subtitle = is_null($langIf) ? $subtitle : $langIf->getText($subtitle);
	}

	/**
	 * Sets the data like a TableFactory
	 * The first column will always be threated as a string and the caption is hidden
	 * The rest of the columns are aligned right and depending on type (int or float or string)
	 * a locale_nf is applied.
	 * @param Array $data
	 */
	function setData($data)
	{
		$this->_data = $data;
	}

	/**
	 * Returns the data, can be overridden in descendant class
	 * @access protected
	 */
	function getData()
	{
		return $this->_data;
	}

	/**
	 * Adds a note
	 * @param string $note
	 */
	function setNote($note)
	{
		$langIf = ELCReportQuantityTable::langIf();

		if (is_null($langIf))
			$this->_note = $note;
		else
			$this->_note = $langIf->getText($note);
	}

	/**
	 * Sets the row index to show the total, default is last row
	 * @param int $rowIndex
	 */
	function setTotalRow($rowIndex=null)
	{
		if (is_null($rowIndex))
			$rowIndex = sizeof($this->_data)-1;

		if(is_null($this->_totalRowIndex))
			$this->_totalRowIndex = array();
		$this->_totalRowIndex[] = $rowIndex;
	}

	/**
	 * builds and returns the html
	 * @return string $html
	 */
	function getHtml($thProcessor=false)
	{
		$data = $this->getData();
		$tb = new TableFactory($data);
		$thProcessor || $thProcessor = array($this, '_tfThProcessor');
		$tb->setThProcessor(TF_ALL, $thProcessor);
		$tb->setValueProcessor(TF_ALL, array($this, '_tfValueProcessor'));
		if (!empty($this->_note))
			$tb->setFooter($this->_note);

		$body = sprintf('<i>%s</i><br/>', $this->_subtitle);
		$body .= $tb->getXhtmlTable('ParticipantReportList');

		$d = array(array($body));
		$t = new TableFactory($d, ELCReportQuantityTable::langIf());
		$t->decapitate();
		$t->setCaption($this->_title,false);

		return $t->getXHTMLTable('Professional');
	}
	/**
	 * table factory th processor
	 */
	function _tfThProcessor($args)
	{
		# we dont want to see the first column
		# and the other columns have to be aligned right
		if ($args['j'] == 0)
			return 'style="visibility: hidden;"';
		else
			return 'style="text-align: right; width: 100px;"';
	}

	/**
	 * table factory value processor
	 */
	function _tfValueProcessor($args)
	{
		$v = $args['value'];

		# all value columns except the first one have to be align right
		if ($args['j'] != 0)
			$args['extra'] = 'style="text-align: right;" ';

		# translate the first column if we have langIf
		$langIf = ELCReportQuantityTable::langIf();
		if ($args['j'] == 0 && !is_null($langIf))
			$v = $langIf->getText($v);

		# apply the correct locale_nf

		if(($args['row']['']=='In percentage')&& $args['j']!=0)
			$v = locale_nf($v).'%';
		elseif (is_float($v)||(is_numeric($v) && $this->_numtype == 'Float'))
			$v = locale_nf($v, 2);
		elseif (is_numeric($v) && $this->_numtype == 'Int')
			$v = locale_nf($v);

		$args['value'] = $v;
		if($this->rowProc&&in_array($args['i'],$this->rowProc))
			$args = $this->executeValueProcessor($args);

		$v = $args['value'];
		if(empty($v))
			return ($args['j'] != 0)?'-':'';

		# see if we are total and make me pretty
		if (!is_null($this->_totalRowIndex) && in_array($args['i'],$this->_totalRowIndex) && $args['j'] != 0)
			$v = sprintf('<b>%s</b>', $v);

		return $v;
	}

	function getDataSet($stmt,$prev,$curr,$breakBefore = array(),$colorize = null)
	{
		$i = 0;
		$ds = array();
		$header = false;
		$lif = ELCReportQuantityTable::langIf();

		$hidePrev = $this->getProperty('month')==$this->hidePrevOn;
		$hideCurr = $this->getProperty('month')==$this->hideCurrOn;

		while($r=$stmt->fetch(PDO::FETCH_ASSOC))
		{
			if(in_array($r['label'],$breakBefore))
			{
				$row = array(''=>'&nbsp;',
					($hidePrev?'n/d':$this->getProperty($prev)) => '&nbsp;',
					($hideCurr?'n/d':$this->getProperty($curr)) => '&nbsp;',
					'YTD' => '&nbsp;');
				$ds[] = $row;
				$i++;
			}

			if($colorize&&in_array($r['label'],$colorize))
				$this->addRowProc($i);

			$row = array(
				''=>$r['label'],
				($hidePrev?'n/d':$this->getProperty($prev)) => ($hidePrev?'':$r['previous_month']),
				($hideCurr?'n/d':$this->getProperty($curr)) => ($hideCurr?'':$r['current_month']),
				'YTD' => $r['ytd']
			);
			$ds[] = $row;
			$i++;
		}
		return $ds;
	}

	function addRowProc($rowindex)
	{
		$this->rowProc[] = $rowindex;
	}

	function executeValueProcessor($args)
	{
		$extra = ' style="color: #999 !important; background-color: #E6E6E6 !important; %s" ';

		if($args['j']==0)
		{
			$args['value'] = '<span style="padding-left: 20px;">'.$args['value'].'</span>';
			$args['extra'] = sprintf($extra,'');
			if($args['row']['']=='Breakdown')
				$args['value'] = '<strong>Breakdown</strong>';
		}
		else
		{
			$args['extra'] = sprintf($extra,'text-align: right;');
			if($args['row']['']=='Breakdown')
				$args['value'] = '&nbsp;';
		}
		return $args;
	}
}
?>