<?php
require_once(IWARE.'IWSessionHandler.php');
/**
 * SessionHandler Decorator
 * @author Marten Koedam <marten@basesoft.com>
 * @package ELC
 * @version $Id$
 */
class ELCIWSessionHandlerDecorator extends IWSessionHandler
{
	/**
	 * @param SessionHandler $sh
	 */
	function ELCIWSessionHandlerDecorator($sh)
	{
		$this->handler = $sh;
	}
	function setLanguageInterface($langIf)
	{
		$this->handler->setLanguageInterface($langIf);
	}
	function getText($txt)
	{
		return $this->handler->getText($txt);
	}
	function _validate()
	{
		return $this->handler->_validate();
	}
	function _setDefault()
	{
		$this->handler->_setDefault();
	}

	/**
	 * common code that is executed by both login and loginAsUser functions
	 */
	function doLoginActions(&$thrownException)
	{
		$log = getLog();
		$this->_setLastLogin();

		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT)
		{
			$userId = $_SESSION['session_user_id'];
			$_SESSION['accountant_relation_id'] = null;

			require_once(ELC_CLASSES.'RelationUser.php');
			$ru = new RelationUser($this->handler->db);

			if (!$id = $ru->getDefaultRelation($userId)) {
				throw new IWException(sprintf('Failed to get default relation for accountant user [%s]: [%s]', $userId, $id));
			}

			$_SESSION['accountant_relation_id'] = $id;
			info('ELCSessionHandlerDecorator: accountant logged in as relation: '.$_SESSION['accountant_relation_id']);
		}

		elseif($_SESSION['session_group_group_id']==GROUP_CRSO)
		{
			if(!$this->_validateCRSO())
			{
				$thrownException .= $this->getText('This is not an active user');
				$this->_setDefault(); // Unset the current set session vars.
				return false;
			}
		}

		// Check if this participant is active at any CRSO
		elseif($_SESSION['session_group_group_id']==GROUP_PARTICIPANT)
		{
			if(!$this->_validateParticipant())
			{
				$thrownException .= $this->getText('This is not an active user');
				$log->info('['.$_SERVER['REMOTE_ADDR'].'] User Login `'.$_SESSION['session_user_name'].'` failed... Participant Not Active At Any CRSOs');
				$this->_setDefault(); // Unset the current set session vars.
				return false;
			}
			if(sizeof($_SESSION['participant_ids'])>1)
			{
				require_once(ELC_CLASSES.'RelationUser.php');
				$ru = new RelationUser($this->handler->db);
				$_SESSION['selected_participant_id'] = false;
				if($id = $ru->getDefaultRelation($_SESSION['session_user_id']))
					$_SESSION['selected_participant_id'] = $id;
			}
			else
				$_SESSION['selected_participant_id'] = current($_SESSION['participant_ids']);
			if($_SESSION['selected_participant_id'])
			{
				require_once(ELC_CLASSES.'RelationUser.php');
				$ru = new RelationUser($this->handler->db);
				$_SESSION['selected_product_id'] = $ru->getDefaultProduct($_SESSION['session_user_id'],$_SESSION['selected_participant_id']);
			}


			$this->_expireRemoteViewer();
		}

		ELCIWSessionHandlerDecorator::refreshSessionUserRelations($this->handler->db);

		if(function_exists('apc_store'))
		{
			$lookup = 'SFMenu'.$_SESSION['session_user_id'].'_'.
				(isset($_SESSION['selected_participant_id'])?$_SESSION['selected_participant_id']:0).'_'.
				(isset($_SESSION['selected_product_id'])?$_SESSION['selected_product_id']:0).APC_SUFFIX;
			apc_delete($lookup);
		}


	}

	/**
	 * @param string $user
	 * @param string $passwd
	 * @param string $thrownException
	 * @return bool
	 */
	function login($user,$passwd,&$thrownException)
	{
	    require_once(IWARE.'group.php');
	    if(!$this->handler->login($user,$passwd,$thrownException)) // Validate through parent function
			return false;

	    $this->doLoginActions($thrownException);

            log_elc_event('User login',
                $_SESSION['session_user_id'], isset($_SESSION['selected_product_id']) ? $_SESSION['selected_product_id']:null,
                isset($_SESSION['crso_relation_id']) ? $_SESSION['crso_relation_id'] : null,
                isset($_SESSION['selected_participant_id']) ? $_SESSION['selected_participant_id'] : null
            );

	    return true;
	}

	/**
	 * @param string $user
	 * @param string $thrownException
	 * @return bool
	 */
	function loginAsUser($user, &$thrownException)
	{
	    require_once(IWARE.'group.php');

		if(!$this->handler->loginAsUser($user, $thrownException)) // Validate through parent function
			return false;

	    $this->doLoginActions($thrownException);

            log_elc_event('User login via admin',
                $_SESSION['session_user_id'], isset($_SESSION['selected_product_id'])?$_SESSION['selected_product_id']:null,
                isset($_SESSION['crso_relation_id']) ? $_SESSION['crso_relation_id'] : null,
                isset($_SESSION['selected_participant_id']) ? $_SESSION['selected_participant_id'] : null
            );

            return true;
	}

	/**
	 * @param string $token
	 * @param string $thrownException
	 * @return bool
	 */
	function loginFromEmail($token, &$thrownException)
	{
            require_once(IWARE.'group.php');
            if (!$this->handler->loginFromEmail($token, $thrownException)) // Validate through parent function
                    return false;
            $this->doLoginActions($thrownException);

            log_elc_event('User login via email',
                $_SESSION['session_user_id'], isset($_SESSION['selected_product_id'])?$_SESSION['selected_product_id']:null,
                isset($_SESSION['crso_relation_id']) ? $_SESSION['crso_relation_id'] : null,
                isset($_SESSION['selected_participant_id']) ? $_SESSION['selected_participant_id'] : null
            );

            return true;
	}

	function changeProduct($productId)
	{
		require_once(IWARE .'group.php');

		// Validate if the selected product is in range:
		$validated = false;
		$store = true;
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_PARTICIPANT:
				// get the Products from TBL_CRSO_PARTICIPANT_PRODUCT WHERE PARTICIPANT_ID = selected_participant_id
				require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
				$cp = new CRSOParticipantProduct($this->handler->db);
				$q = $cp->getParticipantProducts($_SESSION['selected_participant_id']);
				$rel_id = $_SESSION['selected_participant_id'];
				while($r = $q->fetch())
					if($r->PRODUCT_ID==$productId)
						$validated = true;
			break;
			case GROUP_CRSO:
				// get the Products from TBL_CRSO_PRODUCT see Trac Ticket #814
				require_once(ELC_CLASSES.'CRSOProduct.php');
				$cp = new CRSOProduct($this->handler->db);
				$q = $cp->getProducts($_SESSION['crso_relation_id']);
				while($r = $q->fetch())
					if($r->PRODUCT_ID==$productId)
						$validated = true;
				$rel_id = $_SESSION['crso_relation_id'];
			break;
			case GROUP_SYSADMIN:
			case GROUP_PUBLIC:
				$store = false;
			case GROUP_ACCOUNTANT:
				// no product switching for accountants for now
				$store = false;
				$validated = true;
			break;
		}

		if($validated)
		{
			$_SESSION['selected_product_id'] = $productId;
			if($store)
			{
				// for the right relation/user store it!
				require_once(ELC_CLASSES.'RelationUser.php');
				require_once(ELC_CLASSES.'Product.php');
				$ru = new RelationUser($this->handler->db);
				$ru->set($_SESSION['session_user_id'],$rel_id);
				$ru->default_product = new Product($this->handler->db,$_SESSION['selected_product_id']);
				$ru->update();

				info(sprintf('RelationUser [%s/%s] changed to ProductId [%s]', $_SESSION['session_user_id'], $rel_id,$_SESSION['selected_product_id']));
			}

			return true;
		}
		else
			throw new IWException(sprintf('Failed to change product [%s]', $productId));
	}

	/**
	 * Refresh cache of relation ids in session
	 * @static
	 */
	function refreshSessionUserRelations(&$db)
	{
		$_SESSION['session_user_relations'] = array();

		// Load all relations for this user
		require_once(ELC_CLASSES.'RelationUser.php');
    	$rel_user = new RelationUser($db);
    	$relations = $rel_user->getUserRelations($_SESSION['session_user_id']);
    	while ($r = $relations->fetch())
	    {
	    	$_SESSION['session_user_relations'][] = array('id' => $r->{'RELATION_ID'}, 'name' => $r->{'RELATION_NAME'});
	    }
	}

	/**
	 * invalidates the one-time-login token given as parameter
	 * @param string $token
	 */
	function invalidateLoginToken($token){
		$this->handler->invalidateLoginToken($token);
	}

	/**
	 *
	 */
	function _setLastLogin()
	{
		$sql = "UPDATE TBL_USER SET USER_LAST_LOGIN = ?
			WHERE USER_ID = ?
				AND NOT ISNULL(USER_LAST_LOGIN)";
		$q = $this->handler->db->query($sql);
		$q->bind_values(date('Y-m-d H:i:s'),$_SESSION['session_user_id']);
		$q->execute();
	}

	/**
	 * If the current user is a CRSO-member, make sure that the CRSO has accepted the SLA
	 * If the user is connected to more then one RELATION and is a CRSO-member,
	 *  something went terribly wrong. Make sure this will be logged incase it happens.
	 */
	function _validateCRSO()
	{
		require_once(ELC_CLASSES.'CRSO.php');
		$crso = new CRSO($this->handler->db);
		if(!$relationIds = $crso->getByUserId($_SESSION['session_user_id']))
			return false;
		if(count($relationIds)>1)
		{
			$log = getLog();
			$log->crit('['.$_SERVER['REMOTE_ADDR'].'] User Login `'.$_SESSION['session_user_name'].'` failed... Too many Relations at one CRSOUser!');
			return false;
		}
		$r = $relationIds[0];
		$crso->set($r->RELATION_ID);
		if(!$crso->accepted_sla)
			return false;
		$_SESSION['crso_relation_id'] = $crso->relation->id;
		require_once(ELC_CLASSES.'RelationUser.php');
		$ru = new RelationUser($this->handler->db);
		$_SESSION['selected_product_id'] = $ru->getDefaultProduct($_SESSION['session_user_id'],$_SESSION['crso_relation_id']);
		return true;
	}
	/**
	 * A participant is ACTIVE if it is ACTIVE at atleast ONE CRSO,
	 * and has the STATUS of 'PARTICIPANT_STATUS_CONFIRMED'
	 * @todo If the participant user is active, and no CRSO have confirmed or actived,
	 *   goto a page where the user can apply at other CRSO's with a participant
	 */
	function _validateParticipant()
	{
		require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
		// Get all RELATION_USERs
		$sql = "SELECT CP.STATUS, CP.PARTICIPANT_RELATION_ID
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CP
			INNER JOIN TBL_RELATION as R
				ON CP.PARTICIPANT_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_RELATION_USER as RU
				USING(RELATION_ID)
			WHERE RU.USER_ID = ?
				AND CP.ACTIVE = ?
				AND RU.STATUS=1";
		$q = $this->handler->db->query($sql);
		$q->bind_values($_SESSION['session_user_id'],1);
		$q->execute();
		$rs = $q->fetchAll();
		if(!count($rs))
			return false;
		$return = false;

		foreach($rs as $r)
			if($r->STATUS >= PARTICIPANT_STATUS_CONFIRMED)
			{
				if(!isset($_SESSION['participant_ids']))
					$_SESSION['participant_ids'] = array();
				$_SESSION['participant_ids'][$r->PARTICIPANT_RELATION_ID] = $r->PARTICIPANT_RELATION_ID;
				$return = true;
			}
		return $return;
	}

	function _expireRemoteViewer() {
		require_once(ELC_CLASSES.'RemoteViewer.php');
		$rv = new RemoteViewer(IWRegistry::registry('db'));
		foreach($_SESSION['participant_ids'] as $participant_id)
			$rv->expireAll($participant_id, $_SESSION['session_user_id']);
	}

}

?>
