<?php
/**
 * SOAP server for the participants
 * @copyright 2006 BaseSoft B.V.
 */


class ELCSoapServer extends SoapServer
{
	/**
	 * Constructor
	 */
	function __construct($wsdl)
	{
		$errorHandler = IWRegistry::registry('errorHandler');
		$errorHandler->htmlOut = false;

		$exceptionHandler = IWRegistry::registry('exceptionHandler');
		$exceptionHandler->htmlOut = false;

		//info(sprintf('[soap] Starting ELCSoapServer for session_id [%s] ', session_id()));
		parent::__construct($wsdl, array(
			'encoding' => 'UTF-8',
			'classmap'=>ELCSoapServer::getClassMap(),
			'features' => SOAP_SINGLE_ELEMENT_ARRAYS
		));
	}

	/**
	 * Collect some debuging values and handle the soap request.
	 *
	 * @param string $request
	 * @return void
	 */
	public function handle($request = null)
	{
		// check variable HTTP_RAW_POST_DATA
		if (!isset($GLOBALS['HTTP_RAW_POST_DATA']))
		    $GLOBALS['HTTP_RAW_POST_DATA'] = file_get_contents('php://input');

		// check input param
		if (is_null($request))
		    $request = $GLOBALS['HTTP_RAW_POST_DATA'];

		// get soap namespace identifier
		// FIXME: extract from "<SOAP-ENV:Body><ns1:getCrsoMasterData/></SOAP-ENV:Body>"
		// debug('$request: '.$request);

		if (preg_match('#�:Envelope[^>]*xmlns:([^=]*)="http://www.weeeblackbox.com/"�im#',
			$request, $matches)) {
			$soapNameSpace = $matches[1];

			// grab called method from soap request
			$pattern = '#�<' . $soapNameSpace . ':([^> ]*)#�im';
			if (preg_match($pattern, $request, $matches))
				info(sprintf('[soap] ELCSoapServer::handle session_id [%s] calls method [%s] ', session_id(), $matches[1]));
		}

		// finaly call SoapServer::handle() - store result
		return parent::handle($request);
	}

	/**
	 * Includes all WSDL classes and returns an associative array of classmaps
	 * which can be used by a PHP-SOAP client aswell
	 */
	static function getClassMap()
	{
		require_once(ELC_CLASSES.'SoapProduct.php');
		require_once(ELC_CLASSES.'SoapProductCategory.php');
		require_once(ELC_CLASSES.'SoapBrand.php');
		require_once(ELC_CLASSES.'SoapCountry.php');
		require_once(ELC_CLASSES.'SoapDocument.php');
		require_once(ELC_CLASSES.'SoapProofOfPaymentLine.php');
		require_once(ELC_CLASSES.'SoapRefundLine.php');
		require_once(ELC_CLASSES.'SoapPopSupplier.php');
		require_once(ELC_CLASSES.'SoapPopProducer.php');
		require_once(ELC_CLASSES.'SoapPoeAdditionalDocument.php');
		require_once(ELC_CLASSES.'SoapCrsoMasterData.php');
		require_once(ELC_CLASSES.'SoapParticipantMasterData.php');
		require_once(ELC_CLASSES.'SoapDeclarationInvoice.php');
		require_once(ELC_CLASSES.'SoapRefundRequestInvoice.php');
		require_once(ELC_CLASSES.'SoapAgeingLine.php');

		$classMap = array();
		$classMap['Product'] = 'SoapProduct';
		$classMap['ProductCategory'] = 'SoapProductCategory';
		$classMap['Brand'] = 'SoapBrand';
		$classMap['Country'] = 'SoapCountry';
		$classMap['Document'] = 'SoapDocument';
		$classMap['ProofOfPaymentLine'] = 'SoapProofOfPaymentLine';
		$classMap['RefundLine'] = 'SoapRefundLine';
		$classMap['PopSupplier'] = 'SoapPopSupplier';
		$classMap['PopProducer'] = 'SoapPopProducer';
		$classMap['PoeAdditionalDocument'] = 'SoapPoeAdditionalDocument';
		$classMap['CrsoMasterData'] = 'SoapCrsoMasterData';
		$classMap['ParticipantMasterData'] = 'SoapParticipantMasterData';
		$classMap['DeclarationInvoice'] = 'SoapDeclarationInvoice';
		$classMap['Declaration'] = 'SoapDeclaration';
		$classMap['DeclarationLine'] = 'SoapDeclarationLine';
		$classMap['RefundRequestInvoice'] = 'SoapRefundRequestInvoice';
		$classMap['RefundDocumentReview'] = 'SoapRefundDocumentReview';
		$classMap['RefundFee'] = 'SoapRefundFee';
		$classMap['RefundRequest'] = 'SoapRefundRequest';
		$classMap['RefundRequestLine'] = 'SoapRefundRequestLine';
		$classMap['AgeingLine'] = 'SoapAgeingLine';

		$classMap['ArrayOfCrsoMasterData'] = 'SoapArray';
		$classMap['ArrayOfParticipantMasterData'] = 'SoapArray';
		$classMap['ArrayOfDeclarationInvoice'] = 'SoapArray';
		$classMap['ArrayOfDeclarations'] = 'SoapArray';
		$classMap['ArrayOfDeclarationLine'] = 'SoapArray';
		$classMap['ArrayOfRefundRequestInvoice'] = 'SoapArray';
		$classMap['ArrayOfRefundDocumentReviews'] = 'SoapArray';
		$classMap['ArrayOfRefundFees'] = 'SoapArray';
		$classMap['ArrayOfRefundRequests'] = 'SoapArray';
		$classMap['ArrayOfRefundRequestLine'] = 'SoapArray';
		$classMap['ArrayOfAgeingLines'] = 'SoapArray';

		return $classMap;
	}
}

/**
 * SoapArray is the base of all *SoapArray classes which have a $values property
 */
class SoapArray {
	public $values = array();
}

class ELCSoapServerParticipant extends ELCSoapServer
{
	function __construct($wsdl)
	{
		parent::__construct($wsdl);

		require_once(ELC_CLASSES.'ELCSoapHandler.php');
		$this->setClass('ELCSoapHandler');

	}
}

class ELCSoapServerAccountant extends ELCSoapServer
{
	function __construct($wsdl)
	{
		parent::__construct($wsdl);

		require_once(ELC_CLASSES.'ELCSoapHandlerAccountant.php');
		$this->setClass('ELCSoapHandlerAccountant');

	}
}

?>
