<?php
require_once(IWARE.'SuckerfishMenu.php');
require_once(ELC_CLASSES.'RelationUser.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'Country.php');
/**
 * Creates a suckerfish menu and does the SCRIPT rights for the different scripts
 * @author Marten Koedam <marten@basesoft.com>
 * @package ELC
 * @subpackage
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class ELCSuckerfishMenu extends SuckerfishMenu
{
	/**
	 * @access private
	 */
	function _postFilter()
	{
		parent::_postFilter();

		if($_SESSION['session_group_group_name']=='public')
			return;


		$ru = new RelationUser($this->dbApp);
		$cp = new CRSOParticipantProduct($this->dbApp);

		switch(strtolower($_SESSION['session_group_group_name']))
		{
			case 'participant':
				// Check clearance levels
				$remote_viewer = $ageing = $poe = $pop = $subrogation = $declaration = $reports = $producer = $exporter = $general_manager = $credit_producer = false;
				$ac_off = true;

				$countries = array();

				$q = $ru->getUserRelations($_SESSION['session_user_id']);
				$uk = false;
				$ro = false;
				// loop trough participants
				while($r=$q->fetch())
				{
					$q1 = $cp->getParticipantCRSOs($r->RELATION_ID,'q',false,false,$_SESSION['selected_product_id']);
					$rs = $q1->fetchAll();
					if(count($rs))
					{
						foreach($rs as $r1)
						{
							// Get Country Settings
							$crso = new CRSO($this->dbApp);
							$crso->set($r1->CRSO_RELATION_ID);
							$cid = $crso->getCountry();
							if($cid == '826')
								$uk = true;
							if($cid == 642)
								$ro = true;
							$countries[$cid] = new Country($this->dbApp,$cid,true);
							if($r1->PRODUCER_EXPORTER & PARTICIPANT_PRODUCER)
							{
								$producer = true;
								if($countries[$cid]->has(HasSubrogation,$_SESSION['selected_product_id']))
									$subrogation = true;
								if($countries[$cid]->has(HasCreditViaProducer,$_SESSION['selected_product_id']))
									$credit_producer = true;

							}
							if($r1->PRODUCER_EXPORTER & PARTICIPANT_EXPORTER)
							{
								if($countries[$cid]->has(HasRefundRequest,$_SESSION['selected_product_id']))
								{
									$exporter = true;
									if ($r1->GENERAL_MANAGER_ID == $_SESSION['session_user_id'] && $r1->GENERAL_MANAGER_AUTHORIZED == GENERAL_MANAGER_AUTHORIZED)
										$general_manager = true;
								}
							}
							if(!$countries[$cid]->has(HasAnnualConfirmationDisabled,$_SESSION['selected_product_id']))
								$ac_off = false;

							if($countries[$cid]->has(HasParticipantAgeingAccess,$_SESSION['selected_product_id']))
								$ageing = true;
							if($ru->hasClearance(ClearanceFeeRegistration,$_SESSION['session_user_id'],$r->RELATION_ID))
								$declaration = true;
							if($ru->hasClearance(ClearanceProofOfPayment,$_SESSION['session_user_id'],$r->RELATION_ID))
								$pop = true;
							if($ru->hasClearance(ClearanceProofOfExport,$_SESSION['session_user_id'],$r->RELATION_ID))
								$poe = true;
							if($ru->hasClearance(ClearanceRemoteViewer,$_SESSION['session_user_id'],$r->RELATION_ID))
								$remote_viewer = true;
						}
					}
					if($ru->hasClearance(ClearanceReporting,$_SESSION['session_user_id'],$r->RELATION_ID))
						$reports = true;
				}
				if(!$subrogation)
				{
					$this->unsetScript(MenuParticipantRegistration, MenuParticipantRelation);
					$this->unsetScript(MenuParticipantList, MenuParticipantTrustList);
				}
				if(!$declaration)
				{
					$this->unsetScript(MenuParticipantDeclaration,MenuParticipantDeclarationOverview);
					$this->unsetScript(MenuParticipantRegistration, MenuParticipantRelation);
				}
				if(!$reports)
					$this->unsetScript(0, MenuParticipantReports);
				if(!$producer)
				{
					$this->unsetScript(MenuParticipantRegistration, MenuParticipantRelation);
					$this->unsetScript(MenuParticipantDeclaration, MenuParticipantPutOnMarket);
					$this->unsetScript(MenuParticipantReports,MenuParticipantProvisional);
					$this->unsetScript(MenuParticipantList, MenuParticipantDeclarationList);
					$this->unsetScript(MenuParticipantList, MenuParticipantProvDeclCmpList);
					$this->unsetScript(MenuParticipantList, MenuParticipantProvDeclHistList);
				}
				if(!$exporter)
				{
					$this->unsetScript(MenuParticipantDeclaration, MenuParticipantRefund);
					$this->unsetScript(MenuParticipantRefund, MenuParticipantExportAuthorizeOverview);
					$this->unsetScript(MenuParticipantRefund, MenuParticipantExportConfirmationOverview);
					$this->unsetScript(MenuParticipantRefund, MenuParticipantExportAuthorizeOverview);
					$this->unsetScript(MenuParticipantAdmin, MenuParticipantRefundSettings);
					$this->unsetScript(MenuParticipantList, MenuParticipantRefundList);
					$this->unsetScript(MenuParticipantList, MenuParticipantRefundDocumentList);
				}
				if(!$credit_producer)
					$this->unsetScript(MenuParticipantList, MenuParticipantInvoicesList);
				if(!$ageing)
					$this->unsetScript(MenuParticipantList, MenuParticipantAgeingList);
				if($ac_off)
					$this->unsetScript(MenuParticipantDeclaration, MenuAnnualConfirmation);

				//EA registration data for UK
				if(!$uk) {
					$this->unsetScript(MenuParticipantAdmin, MenuParticipantUKXMLPreparation);
					$this->unsetScript(MenuParticipantList, MenuQuarterlyConfirmation);
				} else if(!$producer) {
					$this->unsetScript(MenuParticipantAdmin, MenuParticipantUKXMLPreparation);
				}

				$admin = $ru->isAdmin($_SESSION['session_user_id']);

				// Temporary on country, for Romania (642)
				if(!$ro || !$admin) {
					$this->unsetScript(MenuParticipantReports, 62610);
				}

				if (!$general_manager && !$admin)
				{
					// CLEARANCE CHECKS!!! Ticket #1270
					$deny_scripts = array(MenuParticipantPoPConfirmation,MenuParticipantExportAuthorizeOverview, MenuParticipantExportConfirmationOverview, MenuParticipantRefundRequestExportOverview);

					if(isset($this->scripts[MenuParticipantRefund]))
					{
						foreach ($this->scripts[MenuParticipantRefund] as $script_id => $bitbucket)
						{
							if (in_array($script_id, $deny_scripts)) {
								$this->unsetScript(MenuParticipantRefund, $script_id);
							}
						}
					}
					if(!$pop)
						$this->unsetScript(MenuParticipantRefund, MenuParticipantPoP);
					if(!$poe)
					{
						$this->unsetScript(MenuParticipantRefund, MenuParticipantPoE);
						$this->unsetScript(MenuParticipantRefund, MenuParticipantRefundOverview);
					}
					if(!$pop&&!$poe)
						$this->unsetScript(MenuParticipantDeclaration,MenuParticipantRefund);
				}
				if($general_manager&&!$admin)
				{
					// CLEARANCE CHECKS!!! Ticket #1270
					if(!$pop)
						$this->unsetScript(MenuParticipantRefund, MenuParticipantPoP);
					if(!$poe)
					{
						$this->unsetScript(MenuParticipantRefund, MenuParticipantPoE);
						$this->unsetScript(MenuParticipantRefund, MenuParticipantRefundOverview);
					}
				}
				if(!$admin) {
					if(!$remote_viewer) {
						$this->unsetScript(0, MenuParticipantAdmin);
					} else {
						$this->unsetScript(MenuParticipantAdmin, MenuParticipantRefundRequestExportOverview);
						$this->unsetScript(MenuParticipantAdmin, MenuParticipantRefundSettings);
						$this->unsetScript(MenuParticipantAdmin, MenuParticipantChangeRequest);
						$this->unsetScript(MenuParticipantAdmin, MenuParticipantUserOverview);
						$this->unsetScript(MenuParticipantAdmin, MenuParticipantUKXMLPreparation);
						$this->unsetScript(MenuParticipantAdmin, MenuParticipantBrandManagement);
						$this->unsetScript(MenuParticipantAdmin, MenuParticipantCompanyInformation);
					}
				}
			break;
			case 'crso':
				if(!$ru->isAdmin($_SESSION['session_user_id'])) {
					$this->unsetScript(MenuCRSOAdmin, 61310); // Email translation
					$this->unsetScript(MenuCRSOAdmin, 61330); // Public website
					$this->unsetScript(MenuCRSOAdmin, 61380); // Country Settings Overview
					$this->unsetScript(MenuCRSOReports, MenuAuthReporting);
				}

				// Set the Relation User for clearance level checking.
				$ru->set($_SESSION['session_user_id'], $_SESSION['crso_relation_id']);

				// Get country for CRSO.
				require_once(ELC_CLASSES.'Country.php');
				$r = new Relation($this->dbApp,$_SESSION['crso_relation_id']);
				$c = new Country($this->dbApp,$r->getCountry(),true);

				if(!$ru->hasClearance(CRSOClearanceMaintenance)) {
					$this->unsetScript(MenuCRSOAdmin, MenuCRSOAdminMaintainance);
					$this->unsetScript(MenuCRSOAdmin, MenuCRSOAdminSettings);
				} else {

					// Dont need to check this if we are not having access to the maintenance menu.
					if(!$c->has(HasDeclByCategory,$_SESSION['selected_product_id']))
					{
						$this->unsetScript(MenuCRSOAdminMaintainance, MenuCRSOAdminMaintainanceCategory);
						$this->unsetScript(MenuCRSOAdminMaintainance, MenuCRSOAdminMaintainanceCategoryFee);
					}
					if(!$c->has(HasDeclarationComponent,$_SESSION['selected_product_id']))
						$this->unsetScript(MenuCRSOAdminMaintainance, MenuCRSOAdminMaintainanceComponent);
				}

				if(!$c->has(HasCrsoRefundDetails,$_SESSION['selected_product_id'])) {
					$this->unsetScript(MenuCRSOReportRefund, MenuCRSOPoEDocumentList);
					$this->unsetScript(MenuCRSOReportRefund, MenuCRSOPopList);
					$this->unsetScript(MenuCRSOReportRefund, MenuCRSOPopDetailsList);
					$this->unsetScript(MenuCRSOReportRefund, MenuCRSOPoeDetailsList);
					$this->unsetScript(MenuCRSOReportRefund, MenuCRSOGenericRefundReport);
				}

				// VAT block access check.
				if(!$ru->hasClearance(CRSOClearanceBlockVat)) {
					$this->unsetScript(MenuCRSOAdmin, MenuCRSOAdminBlockVAT);
				}
				// Management Report access check.
				if(!$ru->hasClearance(CRSOClearanceManagementReport)) {
					$this->unsetScript(MenuCRSOReports, MenuCRSOManagementReport);
				}
				// User management access check.
				if(!$ru->hasClearance(CRSOClearanceUserManagement)) {
					$this->unsetScript(MenuCRSOAdmin, MenuCRSOAdminUserRegistration);
				}
				// Remote viewer access check.
				if(!$ru->hasClearance(CRSOClearanceRemoteViewing)) {
					$this->unsetScript(MenuCRSOAdmin, MenuCRSORVOverview);
				}

				// No clearance on Refund stuff, hide the whole reporting menu for it.
				if(!$ru->hasClearance(CRSOClearanceRefund)) {
					$this->unsetScript(MenuCRSOReports, MenuCRSOReportRefund);
					$this->unsetScript(MenuCRSORegistrations, MenuCRSORegistrationRefund);
				} else {

					// TODO: are there more interfaces we can disable in the Refund part when no details?

					if(!$ru->hasClearance(CRSOClearanceRefundDetails) || !$c->has(HasCrsoRefundDetails,$_SESSION['selected_product_id'])||!$c->has(HasRefundRequest,$_SESSION['selected_product_id'])) {
						$this->unsetScript(MenuCRSOReportRefund, MenuCRSOPopAndRRCmpList);
						$this->unsetScript(MenuCRSOReportRefund, MenuCRSOPopConfirmations);
						$this->unsetScript(MenuCRSOReportRefund, MenuCRSOPopAuthorization);
						$this->unsetScript(MenuCRSOReportRefund, MenuCRSOExportReview);
						$this->unsetScript(MenuCRSOReportRefund, MenuCRSOPoEDocumentList);
					}

				}
				// Check for Brand Overview access;
				if(!$ru->hasClearance(CRSOClearanceBrandReport)) {
					$this->unsetScript(MenuCRSOReports, MenuCRSOBrandOverview);
				}

				// Check for documents access:
				if(!$ru->hasClearance(CRSOClearanceDocumentAccess)) {
					$this->unsetScript(MenuCRSOReports, MenuCRSODocumentAccess);
				}
				// Check for Ageing
				if(!$ru->hasClearance(CRSOClearanceAgeing)) {
					$this->unsetScript(MenuCRSOReports, MenuCRSOAgeingReport);
				}
				// Check for Reminder
				if(!$ru->hasClearance(CRSOClearanceReminder)) {
					$this->unsetScript(MenuCRSOReports, MenuCRSOReminderReport);
				}
				// Check for Late Payment
				if(!$ru->hasClearance(CRSOClearanceLatePayment)) {
					$this->unsetScript(MenuCRSOReports, MenuCRSOLatePaymentReport);
				}
				// Check for Guarantee
				if(!$ru->hasClearance(CRSOClearanceGuarantee)) {
					$this->unsetScript(MenuCRSOReports, MenuCRSOGuaranteeReport);
				}

				// If the CRSO user doesnt have access to registration details, hide it.
				if(!$ru->hasClearance(CRSOClearanceRegistrationDetails)) {
					$this->unsetScript(0, MenuCRSORegistrations);
				} elseif(!$c->has(HasSubrogation,$_SESSION['selected_product_id'])) {
					// Only show trust menu if the country setting has subrogation
					$this->unsetScript(MenuCRSORegistrations, MenuCRSOTrusts);
				}

				// Check for access to declaration related reports.
				if(!$ru->hasClearance(CRSOClearanceDeclaration)) {
					$this->unsetScript(MenuCRSOReports, MenuCRSODeclaration);
					$this->unsetScript(MenuCRSORegistrations, MenuCRSORegistrationProducers);
				} else {

					if(!$ru->hasClearance(CRSOClearanceProvisionalDeclaration) || !$c->has(HasCrsoProvisionalDetails,$_SESSION['selected_product_id'])) {
						$this->unsetScript(MenuCRSODeclaration, MenuCRSOProvisionalList);
						$this->unsetScript(MenuCRSODeclaration, MenuCRSOProvisionalCmpList);
						$this->unsetScript(MenuCRSODeclaration, MenuCRSOProvisionalChanges);
					}
					if(!$c->has(HasCrsoDeclarationDetails,$_SESSION['selected_product_id']) || !$ru->hasClearance(CRSOClearanceDeclarationDetails)) {
						$this->unsetScript(MenuCRSODeclaration, MenuCRSOResubmitDeviationList);
						$this->unsetScript(MenuCRSODeclaration, MenuCRSOGenericDeclarationReport);
					}

				}

				// UK Related interfaces.
				if(($c->id != '826')||!$ru->hasClearance(CRSOClearanceEAReports))
					$this->unsetScript(0,MenuEAReports);

				// Temporary on country, for Italy (380)
				if(!in_array($c->id, array(380))) {
					$this->unsetScript(MenuCRSODeclaration, MenuCRSODeclarationYearOverview);
					$this->unsetScript(MenuCRSOReportRefund, MenuCRSORefundYearOverview);
				}

				// Temporary on country, for Romania (642)
				if(!in_array($c->id, array(642))) {
					$this->unsetScript(MenuCRSOReports, 61620);
					$this->unsetScript(MenuCRSOAdmin, 61610);
				}

				// Check for Trac access.
				$u = new ELCUser($this->dbApp, $_SESSION['session_user_id'], true);
				if(!$u->trac_user) {
					$this->unsetScript(MenuCRSOAdmin, MenuCRSOAdminTracTicketOverview);
				}


				// check if NOT EAR reporting
				if(!$c->has(HasDeEarReporting,$_SESSION['selected_product_id'])) {
					$this->unsetScript(MenuCRSOReports, MenuDeEarReporting);

					$authoririties = true;
					$has_pom_details = $c->has(HasCrsoDeclarationDetails,$_SESSION['selected_product_id']);
					$has_poe_details = $c->has(HasCrsoRefundDetails,$_SESSION['selected_product_id']);

					// authorities
					if(!$has_pom_details) {
						$this->unsetScript(MenuCRSOReports, MenuAuthPomMReporting);
						$this->unsetScript(MenuCRSOReports, MenuAuthPomYReporting);
						$this->unsetScript(MenuCRSOReports, MenuAuthNetReporting);
					}
					if(!$has_poe_details) {
						$this->unsetScript(MenuCRSOReports, MenuAuthPoeMReporting);
						$this->unsetScript(MenuCRSOReports, MenuAuthPoeYReporting);
						$this->unsetScript(MenuCRSOReports, MenuAuthNetReporting);
					}

					if (!$has_pom_details && !$has_poe_details) {
						$this->unsetScript(MenuCRSOReports, MenuAuthReporting);
					}
				} else {

					// make sure DE doesn't see the Authorities report
					$this->unsetScript(MenuCRSOReports, MenuAuthReporting);
				}


			break;
			case 'accountant':

				// remove Registration menu, cuz it only contains CRSO Overview
				// and for now we have remove script privileges from it for accountant
				$this->unsetScript(0, MenuRegistrationsAccountant);

				require_once(ELC_CLASSES.'AccountantCRSO.php');
				$crso = new AccountantCRSO($this->dbApp);
				$q = $crso->getAccountantCRSOs($_SESSION['accountant_relation_id'], null, false);
				$rs = $q->fetchAll();

				// if accountant has at least one crso from UK, show the EA related menus
				$uk = false;

				// if accountant has at least one crso which does not have AC disabled as country setting,
				// show AC related menus
				$ac_off = true;

				// ear reporting
				$ear = false;

				foreach($rs as $r)
				{
					$rel = new Relation($this->dbApp, $r->CRSO_RELATION_ID);
					$cid = $rel->getCountry();

					if($cid == '826')
						$uk = true;

					$cntr = new Country($this->dbApp, $cid, true);
					if(!$cntr->has(HasAnnualConfirmationDisabled,$_SESSION['selected_product_id']))
						$ac_off = false;

					if($cntr->has(HasDeEarReporting,$_SESSION['selected_product_id'])) {
						$ear = true;
					}

				}

				if($uk == false)
					$this->unsetScript(64300,MenuEAReportsAccountant);

				if($ac_off){
					$this->unsetScript(64300,MenuAccountantAnnualConfirmationReport);
					$this->unsetScript(64200,MenuAccountantAnnualConfirmationOverview);
				}

				if (!$ear) {
					$this->unsetScript(64300, MenuDeEarReportingAccountant);
				}

				break;


		}
	}


	function _build($current,$top=true)
	{
		$itemTemplate = "\n".'{ id:"sfMenu%s", url:"%s", text:"%s"'.($top ? '' : ', style: "ASubMenu"');
		$rv = '';
		foreach($current as $id => $script)
		{
			$path = $script['SCRIPT_PATH'];
			$path = $path?'/'.SITE_PATH.$path:'#';
			$name = $script['SCRIPT_NAME'];
			$tr = $this->getText('Menu ' . $name);
			if (empty($tr))
				$tr = 'Menu '.$this->getText('Menu'.$id);

			$rv .= sprintf($itemTemplate,$id,$path,$tr);

			$rv .= (isset($this->menu[$id]) ? ', submenu: { id: "'.$id.'submenu", itemdata: ['."\n\t".$this->_build($this->menu[$id],false)."\n".'] }' : '');

			$rv .= '},';

		}
		return substr($rv, 0, -1);

	}

	function display($class='', $target='',$secondMenuClass='', $lookup='menu')
	{
		$this->aClass = $class;
		$this->aTarget = $target;
		$this->secondClass = $secondMenuClass?'class="'.$secondMenuClass.'"':'';

		$rv = '';
		if(sizeof($this->$lookup)==0)
			return '<div id="mymenubar" class="yuimenubar">&nbsp;</div>';

		$rv = '
        <script type="text/javascript">

            WEEE.onDOMReady = function(p_sType) {

                var aItemData = [

                    '.$this->_build($this->{$lookup}[0]).'

                ];


                WEEE.oMenuBar = new YAHOO.widget.MenuBar("mymenubar", { lazyLoad: false, itemdata:aItemData });


                // Render the MenuBar instance and corresponding submenus

                WEEE.oMenuBar.render("sMenuMain");

            };


            // Add a "load" handler for the window

            YAHOO.util.Event.onDOMReady(WEEE.onDOMReady);
        </script>';
		return $rv;
	}

	public function getSiteMap($class='', $target='',$secondMenuClass='', $lookup='menu')
	{
		return parent::display($class,$target,$secondMenuClass,$lookup);
	}

	public function checkForPermission($path, $start=false)
	{
		$found = false;
		if(!$start)
			$start = $this->menu[0];
		//find the path in the menu, or return false
		foreach($start as $id => $script)
		{
			if($path == '/'.$script['SCRIPT_PATH'])
				return true; //we found it
			if(isset($this->menu[$id])?$this->checkForPermission($path, $this->menu[$id]):false)
				return true;
		}
		return false;
	}
}
/**@#+
 * Define some important menus
 */
define('MenuCRSOAdmin',61300);
define('MenuCRSORVOverview',61315);
define('MenuCRSOAdminBlockVAT', 61340);
define('MenuCRSOAdminUserRegistration', 61370);
define('MenuCRSOAdminTracTicketOverview',61360);
define('MenuCRSOAdminMaintainance',61320);
define('MenuCRSOAdminMaintainanceCategory',61390);
define('MenuCRSOAdminMaintainanceCategoryFee',61324);
define('MenuCRSOAdminMaintainanceBrand',61322);
define('MenuCRSOAdminMaintainanceComponent',61323);
define('MenuCRSOAdminSettings', 61350);

define('MenuCRSORegistrations',61200);
define('MenuCRSORegistrationProducers', 61210);
define('MenuCRSORegistrationRefund', 61211);
define('MenuCRSOTrusts',61250);

define('MenuCRSOReports', 61100);
define('MenuCRSODeclaration',61110);

define('MenuCRSOBrandOverview', 61230);

define('MenuCRSOProvisionalCmpList',61113);
define('MenuCRSOProvisionalChanges',61114);
define('MenuCRSOAnnualConfirmationReport', 61115);
define('MenuCRSOGenericDeclarationReport', 61117);
define('MenuCRSOProvisionalList',61118);
define('MenuCRSOResubmitDeviationList',61119);
define('MenuCRSODeclarationYearOverview',61121);

define('MenuCRSOManagementReport', 61120);
define('MenuCRSOReportRefund',61130);
define('MenuCRSOPopAndRRCmpList',61134);
define('MenuCRSOGenericRefundReport',61135);
define('MenuCRSOPopConfirmations',61136);
define('MenuCRSOPopAuthorization',61137);
define('MenuCRSOPopList',61510);
define('MenuCRSOPopDetailsList', 61530);
define('MenuCRSOPoeDetailsList', 61540);
define('MenuCRSOExportReview',61139);
define('MenuCRSOPoEDocumentList',61500);
define('MenuCRSORefundYearOverview',61143);

define('MenuCRSODocumentAccess', 61140);
define('MenuCRSOReminderReport', 61150);
define('MenuCRSOLatePaymentReport', 61160);
define('MenuCRSOGuaranteeReport', 61170);
define('MenuCRSOAgeingReport', 61180);

define('MenuParticipantAdmin',62500);
define('MenuParticipantUserOverview', 62510);
define('MenuParticipantChangeRequest', 62570);
define('MenuParticipantBrandManagement', 62580);
define('MenuParticipantCompanyInformation', 62595);

define('MenuParticipantRemoteViewer', 62505);
define('MenuParticipantDeclaration',62100);
define('MenuParticipantPutOnMarket',62105);
define('MenuParticipantReports',62200);
define('MenuParticipantDeclarationOverview',62110);
define('MenuParticipantRefund',62120);
define('MenuParticipantProvisional',62130);
define('MenuAnnualConfirmation',62140);
define('MenuQuarterlyConfirmation',62141);

define('MenuParticipantPoP',62121);
define('MenuParticipantPoE',62125);
define('MenuParticipantRefundOverview',62128);
define('MenuParticipantPoPConfirmation',62124);

define('MenuParticipantList',62210);
define('MenuParticipantDeclarationList',62211);
define('MenuParticipantRefundList',62212);
define('MenuParticipantRefundDocumentList',62216);
define('MenuParticipantProvDeclCmpList',62213);
define('MenuParticipantProvDeclHistList',62214);
define('MenuParticipantTrustList',62215);
define('MenuParticipantInvoicesList',62218);

define('MenuParticipantAgeingList', 62221);

define('MenuParticipantRegistration',62400); // For Subrogation
define('MenuParticipantRelation',62420); // For Subrogation

define('MenuParticipantPaymentAuthorizeOverview', 62520);
define('MenuParticipantExportAuthorizeOverview', 62530);
define('MenuParticipantExportConfirmationOverview', 62540);
define('MenuParticipantRefundSettings',62550);
define('MenuParticipantRefundRequestExportOverview',62560);

define('MenuAccountantAnnualConfirmationReport', 64318);
define('MenuAccountantAnnualConfirmationOverview', 64240);
define('MenuRegistrationsAccountant',64100);
//uk stuff
define('MenuParticipantUKXMLPreparation',62590);
define('MenuEAReports',61400);

define('MenuEAReportsAccountant',64485);

define('MenuDeEarReporting', 61270);
define('MenuDeEarReportingAccountant', 64500);


define('MenuAuthReporting', 61274);
define('MenuAuthPomMReporting', 61275);
define('MenuAuthPomYReporting', 61276);
define('MenuAuthPoeMReporting', 61277);
define('MenuAuthPoeYReporting', 61278);
define('MenuAuthNetReporting', 61279);

/**#@-*/
?>
