<?php
/**
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @since 20-01-2005
 * @version $Id$
 */
require_once(IWARE . 'FormBehaviour.php');

class FormBehaviourPluginElcRegMonths extends FormBehaviourPlugin
{
	var $_message = '';
	var $_countrySettings = 0;
	var $_handles = array('Please enter all required months', 'Only specify months after obligation date');

	function check()
	{
		$crso_id = isset($_REQUEST['crso']['id'])?(int)$_REQUEST['crso']['id']:$_SESSION['public']['crso']['id'];
		require_once(ELC_CLASSES . 'CRSO.php');
		require_once(ELC_CLASSES . 'Country.php');

		$this->_status = FB_STATUS_INVALID;

		// always valid if no months have to be specified
		$crso = new CRSO($this->_db);
		$countryId = $crso->getCountry($crso_id);
		$country = new Country($this->_db, $countryId, true);
		if ( !($country->settings & HasProvisionalPerMonth)) {
			// all good.. no need to specify by month
			$this->_status = FB_STATUS_VALID;
			return;
		}

		$start_date = $_POST['participant']['business_start_date'];
		$pieces = explode('-', $start_date);

		if (sizeof($pieces) != 3)
		return;

		$year = $pieces[2];
		$month = $pieces[1];
		$day = $pieces[0];


		$d =& currentDate();
		$current_year = $d->getYear();

		// If obligation date is last year, this whole year is required to specify
		$start_date = new IWDateTime($year.'-'.$month.'-'.$day);
		if ($start_date->getYear() != $current_year)
		$start_date = new IWDateTime($current_year.'-01-01');

		// Make an empty array with all required months
		$months = array();
		for ($i=$start_date->getMonth(); $i<12+1; $i++)
		$months[$i] = true;

		// See if more is specified than needed
		foreach ($_POST['provisional'] as $prov_line)
		{
			if (isset($prov_line['month']) && !isset($months[$prov_line['month']]))
			{
				// month was specified but not in array!
				$this->_message = $this->_handles[1];
				$this->_status = FB_STATUS_INVALID;
				return;
			}
		}

		// Loop trough the POST and remove month from $months if found.
		// If the array is empty at the end its all good
		foreach ($_POST['provisional'] as $prov_line)
		{
			if (isset($prov_line['month']) && isset($months[$prov_line['month']]))
			unset($months[$prov_line['month']]);
		}
		$this->_message = $this->_handles[0];
		$this->_status = (sizeof($months) == 0) ? FB_STATUS_VALID  : FB_STATUS_INVALID;

	}
}

/**
 * Checks if the sum of the individual quantities match the total quantity
 */
class FormBehaviourPluginElcTotalSum extends FormBehaviourPlugin
{
	var $_message = 'The total sum does not match the quantity';
	function check()
	{
		$crso_id = isset($_REQUEST['crso']['id'])?(int)$_REQUEST['crso']['id']:$_SESSION['public']['crso']['id'];
		$crso = new CRSO($this->_db);
		$countryId = $crso->getCountry($crso_id);
		$country = new Country($this->_db, $countryId, true);

		$hasSettings = false;
		foreach(array(HasDeclByCategory, HasDeclByBrand, HasDeclOfWeight, HasProvisionalPerMonth) as $setting)
		{
			if($country->has($setting,$_SESSION['selected_product_id']))
			$hasSettings = true;
		}

		// always valid if no details have to be specified
		if (!$hasSettings) {
			$this->_status = FB_STATUS_VALID;
			return;
		}

		$total = 0;
		foreach ($_POST['provisional'] as $prov_line)
		$total += (int) $prov_line['quantity'];

		$this->_status = ($total == $this->_value) ? FB_STATUS_VALID : FB_STATUS_INVALID;
	}
}

class FormBehaviourPluginElcVatUnique extends FormBehaviourPlugin
{
	var $_message = 'An application has already been made for the requested VAT';
	function check()
	{
		require_once(ELC_CLASSES.'Relation.php');
		$this->_status = (Relation::isUniqueVAT($this->_value, $this->_db)) ?
		FB_STATUS_VALID : FB_STATUS_INVALID;
	}
}

class FormBehaviourPluginElcProvDouble extends FormBehaviourPlugin
{
	var $_message = 'All declaration lines must be unique';
	function check()
	{
		// if no provisional submitted, all is good
		if (!isset($_POST['provisional'])) {
			$this->_status = FB_STATUS_VALID;
			return;
		}

		$prov = $_POST['provisional'];
		$prov2 = $_POST['provisional'];

		// remove weight and quantity (they are not part of unique)
		foreach ($prov as $key=>$bitbucket) {
			unset($prov[$key]['weight']);
			unset($prov2[$key]['weight']);
			unset($prov[$key]['quantity']);
			unset($prov2[$key]['quantity']);
		}
		$this->_status = FB_STATUS_VALID;

		foreach ($prov as  $key=>$provline)
		{
			foreach ($prov2 as  $key2=>$provline2)
			{
				if ($key != $key2 && $provline == $provline2)
				{
					$this->_status = FB_STATUS_INVALID;
					break 2;
				}
			}
		}
	}
}
class FormBehaviourPluginElcNotFuture extends FormBehaviourPlugin
{
	var $_message = 'Date cannot be in the future';

	function check($format='DD-MM-YYYY')
	{
		if(!($date = createDate($this->_value, $format)))
		return $this->_status = FB_STATUS_INVALID;
		$now = currentDate();

		// return: 0 if the dates are equal, -1 if d1 is before d2, 1 if d1 is after d2
		if ( IWDateTime::compare($date, $now) == 1 )
		$this->_status = FB_STATUS_INVALID;
		else
		$this->_status = FB_STATUS_VALID;
	}
}

class FormBehaviourPluginElcNotPast extends FormBehaviourPlugin
{
	var $_message = 'Date cannot be in the past';

	function check()
	{
		if(!($date = createDate($this->_value)))
		return $this->_status = FB_STATUS_INVALID;
		$now = currentDate();

		//debug('value: '.$this->_value);
		//debug(sprintf('comparing [%s] and [%s]', print_r($date, true), print_r($now, true)));

		// return: 0 if the dates are equal, -1 if d1 is before d2, 1 if d1 is after d2
		if ( IWDateTime::compare($date, $now) == -1 )
		$this->_status = FB_STATUS_INVALID;
		else
		$this->_status = FB_STATUS_VALID;
	}
}

class FormBehaviourPluginElcStartdate extends FormBehaviourPlugin
{
	var $_handles = array(
		'Obligation start date must be before next year.',
		'Obligation start date must be after minimal crso obligation date');
	var $_message = '';

	function check($crso_id='', $format='DD-MM-YYYY')
	{
		$date_now = currentDate();
		$obligation_date = new IWDateTime();
		if(!($obligation_date = createDate($this->_value, $format)))
		return $this->_status = FB_STATUS_INVALID;

		// check if date in future
		//debug('Comparing '.$obligation_date->getYear().' and '.$date_now->getYear());
		if ( $obligation_date->getYear() > $date_now->getYear() )
		{
			$this->_status = FB_STATUS_INVALID;
			$this->_message = $this->_handles[0];
			return;
		}

		// check for minimal crso date
		if (!$crso_id)
		$crso_id = isset($_REQUEST['crso']['id']) ? (int)$_REQUEST['crso']['id'] : $_SESSION['public']['crso']['id'];

		if (!is_numeric($crso_id))
		throw new IWException('FormBehaviourPluginElcStartdate::check() - Error while setting CRSO ('.$crso_id.')');

		require_once(ELC_CLASSES.'CRSOProduct.php');
		$crso = new CRSOProduct($this->_db);
		$crso->set($crso_id,$_SESSION['selected_product_id']);

		if (!empty($crso->minimal_obligation_date))
		{
			$crso_min_date = createDate($crso->minimal_obligation_date, 'YYYY-MM-DD');//new IWDateTime($crso->minimal_obligation_date);

			if ( IWDateTime::compare($obligation_date, $crso_min_date) == -1 ) {
				$this->_status = FB_STATUS_INVALID;
				$this->_message = $this->_handles[1];
				return;
			}
		}

		$this->_status = FB_STATUS_VALID;
		return true;
	}
}

class FormBehaviourPluginElcExporterDocumentdate extends FormBehaviourPluginElcStartdate
{
	var $_handles = array(
		'Proof of Payment document date must be before next year.',
		'Proof of Payment document date must be after minimal crso obligation date',
		'Proof of Payment document date must be after minimal crso refund start date');

	function check($crso_id='', $format='DD-MM-YYYY')
	{
		if(!parent::check($crso_id,$format)) {
			debug('parent returning false');
			return;
		}

		$obligation_date = new IWDateTime();
		$obligation_date = createDate($this->_value, $format);

		$crso = new CRSOProduct($this->_db);
		$crso->set($crso_id,$_SESSION['selected_product_id']);

		debug('crso date: '.$crso->refund_start_date);

		if (!empty($crso->refund_start_date))
		{
			$crso_min_date = new IWDateTime($crso->refund_start_date);

			if ( IWDateTime::compare($obligation_date, $crso_min_date) == -1 ) {
				$this->_status = FB_STATUS_INVALID;
				$this->_message = $this->_handles[2];
				return;
			}
		}
	}
}

/**
 * Various checks for the crso-participant relation
 *  - Checks if the participant is already registred at a crso
 *  - Checks if the participant is already registred at crso in the current country and if that is allowed to do twice
 */
class FormBehaviourPluginElcParticipantCrso extends FormBehaviourPlugin
{
	var $_handles = array(
		'You are already registred at this CRSO',
		'You can only apply to one CRSO in a country as producer');
	var $_message = '';

	function check()
	{
		// Can only check if we are participant already, new are always valid
		if (!isset($_REQUEST['participant']['id'])) {
			$this->_status = FB_STATUS_VALID;
			return;
		}

		$crso_id = isset($_REQUEST['crso']['id'])?(int)$_REQUEST['crso']['id']:$_SESSION['public']['crso']['id'];
		$crso = new CRSO($this->_db);

		$countryId = $crso->getCountry($crso_id);
		$country = new Country($this->_db, $countryId, true);

		// see if participant is already registred at a crso
		$cp = new CRSOParticipantProduct($this->_db);
		if ($cp->set($crso_id, (int)$_REQUEST['participant']['id'], $_SESSION['selected_product_id'])) {
			$this->_message = $this->_handles[0];
			$this->_status = FB_STATUS_INVALID;
			return;
		}

		// Check if this is a country setting and the user wants to apply as producer
		debug('starting country check..');
		if($country->has(HasParticipantOneCrso, $_SESSION['selected_product_id']) && ($_POST['admission'] & PARTICIPANT_PRODUCER))
		{
			$this->_status = FB_STATUS_VALID;

			// check if the participant is already registred in this country
			$p = new Participant($this->_db, $_POST['participant']);
			$q = $country->selectRegistredParticipants($_SESSION['selected_product_id']);

			while ($row = $q->fetch()) {
				if ($row->PARTICIPANT_RELATION_ID == $_REQUEST['participant']['id']) {
					debug('already registred in this country!');
					$this->_message = $this->_handles[1];
					$this->_status = FB_STATUS_INVALID;
					return;
				}
			}

			debug('not in same country');
		}
		else {
			debug('no country settings');
			$this->_status = FB_STATUS_VALID;
		}
	}
}

/**
 * Checks if the current date is before the value
 */
class FormBehaviourPluginElcForfeitStartDate extends FormBehaviourPluginDateNotInPast
{
	var $_message = 'Forfeit start data cannot be in the past';
}

/**
 * Checks if the current date is before the value
 */
class FormBehaviourPluginElcRefundStartDate extends FormBehaviourPluginDateNotInPast
{
	var $_message = 'Refund Request/Export start date cannot be in the past';
}


/**
 * Checks that username is unique
 */
class FormBehaviourPluginUsernameUnique extends FormBehaviourPlugin
{
	var $_message = '';
	var $_handles = array('Invalid UserName, User already exists');

	function check()
	{
		$sql = "SELECT USER_ID FROM TBL_USER
			WHERE USER_NAME = ? ";
		$q = $this->_db->query($sql);
		$q->bind_values($this->_value);
		$q->execute();
		if($r = $q->fetch())
		{
			$this->_status = FB_STATUS_INVALID;
			$this->_message = $this->_handles[0];
			return;
		}
		$this->_status = FB_STATUS_VALID;
	}
}


/**
 * Checks that the password is valid (ie. not weak)
 */
class FormBehaviourPluginPasswordStrong extends FormBehaviourPlugin
{
	var $_message = '';
	var $_handles = array(
							'it\'s WAY too short' => 'it\'s WAY too short',
							'it is too short' => 'it is too short',
							'it does not contain enough DIFFERENT characters' => 'it does not contain enough DIFFERENT characters',
							'it is all whitespace' => 'it is all whitespace',
							'it is too simplistic/systematic', 'it is too simplistic/systematic',
							'it looks like a National Insurance number.' => 'it looks like a National Insurance number.',
							'it is based on a dictionary word' => 'it is based on a dictionary word',
							'it is based on a (reversed) dictionary word' => 'it is based on a (reversed) dictionary word',
							'strong password' => 'strong password'
							);

							function check()
							{
								$message = '';
								if(!cracklibCheck($this->_value, $message))
								{
									$this->_status = FB_STATUS_INVALID;
									$this->_message = $this->_handles[$message];
								}
								else
								{
									$this->_status = FB_STATUS_VALID;
								}
							}
}

class FormBehaviourPluginElcVATBlock extends FormBehaviourPlugin
{
	var $_message = 'VAT number is blocked';

	function check($crso_id, $block_type)
	{
		require_once(ELC_CLASSES.'CRSOVATBlock.php');
		$v = new CRSOVATBlock($this->_db);
		if ($v->isBlocked($this->_value, $crso_id, $block_type))
			$this->_status = FB_STATUS_INVALID;
		else
			$this->_status = FB_STATUS_VALID;
	}
}


class FormBehaviourPluginElcVATBlockByID extends FormBehaviourPlugin {

	var $_message = 'VAT number is blocked for this action';

	function check($crso_id, $block_type, $message=null) {
		if($message) {
			$this->_message = $message;
		}
		require_once(ELC_CLASSES.'Relation.php');
		require_once(ELC_CLASSES.'CRSOVATBlock.php');
		$r = new Relation($this->_db, $this->_value, true);
		$v = new CRSOVATBlock($this->_db);
		if ($v->isBlocked($r->vat_no, $crso_id, $block_type))
			$this->_status = FB_STATUS_INVALID;
		else
			$this->_status = FB_STATUS_VALID;
	}

}


class FormBehaviourPluginElcProductUnique extends FormBehaviourPlugin
{
	var $_message = 'Product name must be unique';

	function check()
	{
		$sql = "SELECT PRODUCT_ID FROM TBL_PRODUCT
			WHERE LOWER(NAME) = LOWER(?) ";
		$q = $this->_db->query($sql);
		$q->bind_values(trim($this->_value));
		$q->execute();
		if($r = $q->fetch())
		{
			$this->_status = FB_STATUS_INVALID;
			return;
		}
		$this->_status = FB_STATUS_VALID;
	}
}

/**
 * Abstract class for weight and weight_dec
 */
class AbstractFormBehaviourPluginElcDeclarationWeight extends FormBehaviourPlugin {
	var $_message = '';

	function check()
	{
		if(!preg_match('/line\[(\d+)\]\['.$this->actor.'\]/',$this->input_name,$matches))
			return $this->_status = FB_STATUS_VALID;

		$index = $matches[1];
		$arrValues = $this->fb->getArrValues();
		$line = $arrValues['line'][$index];

		if(!isset($line[$this->actor]))
		return $this->_status = FB_STATUS_VALID;
		if(($line[$this->actor] == '0') && ($line['quantity'] == '0'))
		return $this->_status = FB_STATUS_VALID;
		if(is_numeric($line[$this->actor]) && ($line[$this->actor]==0)&&($line['quantity']>0))
		{
			$this->_message = 'Weight cannot be zero if quantity is more than zero';
			return $this->_status = FB_STATUS_INVALID;
		}
		if(is_numeric($line[$this->actor])&&empty($line['quantity']))
		{
			$this->_message = 'Weight cannot be set if no quantity is supplied';
			return $this->_status = FB_STATUS_INVALID;
		}
		if(is_numeric($line[$this->actor]) && ($line[$this->actor]>0)&&($line['quantity']==0))
		{
			$this->_message = 'Weight cannot be larger than zero if quantity equals zero';
			return $this->_status = FB_STATUS_INVALID;
		}
		if(empty($line[$this->actor])&&isset($line['quantity'])&&is_numeric($line['quantity']))
		{
			$this->_message = 'Weight cannot be empty if quantity is set';
			return $this->_status = FB_STATUS_INVALID;
		}
		if(!is_numeric($line[$this->actor])) {
			$this->_message = 'Weight is not a valid numerical value';
			return $this->_status = FB_STATUS_INVALID;
		}
		$this->_status = FB_STATUS_VALID;
	}
}

/**
 * important: FormBehaviourPluginElcDeclarationWeight is now also used for refunds
 */
class FormBehaviourPluginElcDeclarationWeight extends AbstractFormBehaviourPluginElcDeclarationWeight {
	var $actor = 'weight';
}

class FormBehaviourPluginElcDeclarationWeightDec extends AbstractFormBehaviourPluginElcDeclarationWeight {
	var $actor = 'weight_dec';
}

class FormBehaviourPluginElcDeclarationLineUnique extends FormBehaviourPlugin
{
	var $_message = 'Declaration Lines must be unique';

	function check()
	{
		$uniq_lines = array();

		foreach ($_POST['line'] as $line)
		{
			$prod_cat = (int)(isset($line['product_category'])) ? $line['product_category']['id'] : null;
			$brand = (int)(isset($line['brand'])) ? $line['brand']['id'] : null;
			$trust = (isset($line['trust'])) ? $line['trust'] : null;
			$dimension = (isset($line['dimension'])) ? $line['dimension']['id'] : null;

			if (!isset($uniq_lines[$prod_cat]))
			$uniq_lines[$prod_cat] = array();

			if (!isset($uniq_lines[$prod_cat][$brand]))
			$uniq_lines[$prod_cat][$brand] = array();

			if(!isset($uniq_lines[$prod_cat][$brand][$trust]))
			$uniq_lines[$prod_cat][$brand][$trust] = array();

			if(!isset($uniq_lines[$prod_cat][$brand][$trust][$dimension]))
			$uniq_lines[$prod_cat][$brand][$trust][$dimension] = array();

			// this actually checks for duplicates
			if ( is_numeric($line['quantity']) && sizeof($uniq_lines[$prod_cat][$brand][$trust][$dimension]) != 0 )
			{
				$this->_status = FB_STATUS_INVALID;
				return;
			}

			$uniq_lines[$prod_cat][$brand][$trust][$dimension] = (int)$line['quantity'];

		}

		$this->_status = FB_STATUS_VALID;

	}
}

class FormBehaviourPluginElcPopSupplyChainUnique extends FormBehaviourPlugin
{
	var $_message = 'VAT / invoice combination may only occur once';

	/**
	 * This checks if the $vat_no and $invoice_no are only once in the $supply_chain
	 */
	function check($vat_no)
	{
		$arrValues = $this->fb->getArrValues();
		$supply_chain = $arrValues['supply_chain'];

		$count = 0;
		foreach ($supply_chain as $key => $fields)
		{
			if ($fields['vat_no'] == $vat_no)
			$count++;
		}

		$this->_status = ($count == 1) ? FB_STATUS_VALID : FB_STATUS_INVALID;
	}
}

class FormBehaviourPluginUkGovRegistrationNoType extends FormBehaviourPluginRegex
{
	var $_regex = '/^(WEE\/)[A-Z]{2}[0-9]{4}[A-Z]{2}(\/PRO)$/';
	var $_message = 'Not a valid Registration Number';
}

class FormBehaviourPluginUkGovContactChannel extends FormBehaviourPluginRegex
{
	var $_regex = '/^[0-9]{10,35}$/';
	var $_message = 'Not a valid Phone Number';
}

class FormBehaviourPluginUkGovPostCode extends FormBehaviourPluginRegex
{
	//var $_regex = '/^(GIR 0AA)$|^((((?:(?=[^QVX])[A-Z])[0-9][0-9]?)|(((?:(?=[^QVX])[A-Z])(?:(?=[^IJZ])[A-Z])[0-9][0-9]?)|(((?:(?=[^QVX])[A-Z])[0-9][A-HJKSTUW])|((?:(?=[^QVX])[A-Z])(?:(?=[^IJZ])[A-Z])[0-9][ABEHMNPRVWXY]))))[ ]?[0-9](?:(?=[^CIKMOV])[A-Z]){2})$/';
	var $_regex = '/^[A-Z]{1,2}[0-9R][0-9A-Z]? [0-9][A-Z]{2}$/';
	var $_message = 'Not a valid UK PostCode';
}

class FormBehaviourPluginUkCountry extends FormBehaviourPlugin {
	function check() {
		$this->_status = (substr($this->_value, 0, 4) == 'UK -') ? FB_STATUS_VALID : FB_STATUS_INVALID;
	}
}

class FormBehaviourPluginNonUkCountry extends FormBehaviourPlugin {
	function check() {
		$this->_status = (substr($this->_value, 0, 4) != 'UK -') ? FB_STATUS_VALID : FB_STATUS_INVALID;
	}
}



class FormBehaviourPluginUkLogoUpload extends FormBehaviourPluginDocument
{
	function check()
	{
		// this will validate the upload with general requirements
		parent::check();

		if($this->_status == FB_STATUS_INVALID)
		return;

		//now we need to check for an image (as we allow pdfs etc also for the BB)
		// EA allows: jpg, png, gif only
		if(!preg_match('/^.*\.(jpg|jpeg|png|gif)$/i',$this->document->name))
		{
			$this->_message = "Image format is not supported by the EA";
			$this->_status = FB_STATUS_INVALID;
			return;
		}

		//now we need to check dimensions
		$sizeinfo = @getimagesize($this->document->temp_name);

		if(!$sizeinfo)
		{
			$this->_message = "Image format is corrupted!";
			$this->_status = FB_STATUS_INVALID;
			return;
		}

		//check width min and max
		if($sizeinfo[0] > 310 || $sizeinfo[0] < 120)
		{
			$this->_message = "Image width must be between 120px and 310px inclusive";
			$this->_status = FB_STATUS_INVALID;
			return;
		}

		//check height min and max
		if($sizeinfo[1] > 232 || $sizeinfo[1] < 120)
		{
			$this->_message = "Image height must be between 120px and 232px inclusive";
			$this->_status = FB_STATUS_INVALID;
			return;
		}

		$this->_status = FB_STATUS_VALID;
	}
}



?>
