<?php
require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES.'Relation.php');
require_once(ELC_CLASSES.'Product.php');
require_once(ELC_CLASSES.'User.php');
/**
 * Notification
 * @author Marten Koedam <marten@basesoft.com>
 * @author Nick Loeve <nick@basesoft.com>
 * @package ELC
 * @subpackage Core
 * @version $Id$
 */
class Notification extends AbstractObject
{
	var $id;
	var $product;
	var $sender;
	var $user;
	var $subject;
	var $message;
	var $type;
	var $sent_date;

	function set($id=null)
	{
		$id || $id = $this->id;
		$id = (int) $id;

		$sql = "SELECT n.SENDER_ID, n.USER_ID as SENDER_USER_ID, n.NOTIFICATION_ID,
				n.SUBJECT,  n.MESSAGE, n.`TYPE`, n.SENT_DATE, n.PRODUCT_ID
			FROM TBL_NOTIFICATION as n
			WHERE n.NOTIFICATION_ID = ?";

		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		$this->id = $r->NOTIFICATION_ID;
		$this->sender_id = $r->SENDER_ID;
		$this->sender_user_id = $r->SENDER_USER_ID;
		$this->subject = $r->SUBJECT;
		$this->message = $r->MESSAGE;
		$this->type = $r->TYPE;
		$this->sent_date = $r->SENT_DATE;
		$this->product = new Product($this->db, $r->PRODUCT_ID);

		return $this->complete = true;
	}

	function insert()
	{
		if (empty($this->message) && empty($this->subject)) {
			trigger_error('Notification::insert() - Not sending empty notification.', E_USER_NOTICE);
			return false;
		}

		if(isset($_SESSION['session_user_id']) && is_numeric($_SESSION['session_user_id']))
			$this->user = (object) array('id'=>$_SESSION['session_user_id']);

		if(!isset($this->product) && isset($_SESSION['selected_product_id']))
			$this->product = new Product($this->db, $_SESSION['selected_product_id']);

		$this->sent_date = date('Y-m-d H:i:s');

		return parent::insert();
	}


	/**
	 * @param Database $db
	 * @param Array $map
	 * @param Array $recipients
	 * @usage
	 * 		Notification::throw($this->db_app,
	 *			array('subject'=>$this->langIf->getText('Notification: Registration Confirmed'),
	 *				'sender'=>$ru->getRelationAdmin($cp->crso->relation->id),
	 *				'message'=>$this->langIf->getText("Notification: CRSO `{CRSO}` has confirmed the registration to participant `{PARTICIPANT}`", null,
	 *					array('CRSO'=>$cp->crso->relation->name, 'PARTICIPANT'=>$cp->participant->relation->name))),
	 *			array(array('recipient'=>$cp->participant->relation,
	 *				'user'=>$ru->getRelationAdmin($cp->participant->relation->id),
	 *				'callback'=>"go('InterfaceDeclarationSettings.php?script_parent_id=62100&participant[id]=".$cp->participant->relation->id."&crso[id]=".$cp->crso->relation->id."')"
	 *			)));
	 */
	static function send(&$db,$map,$recipients)
	{
		if (isset($map['sender']))
			if (get_class($map['sender']) != 'Relation')
				trigger_error(sprintf('Notification::send() - sender [%s] is not an Relation object', get_class($map['sender'])), E_USER_ERROR);

		if (isset($map['user']))
			if (get_class($map['user']) != 'ELCUser')
				trigger_error(sprintf('Notification::send() - user [%s] is not an ELCUser object', get_class($map['user'])), E_USER_ERROR);

		if (isset($map['product']))
			if ($map['product']!=DB_NULL)
				if(get_class($map['product']) != 'Product')
					trigger_error(sprintf('Notification::send() - product [%s] is not an Product object', get_class($map['product'])), E_USER_ERROR);

		if(!$recipients)
		{
			info('Notification::throw() - No recipients specified!');
			return false;
		}
		$notification = new Notification($db,$map);
		if ($notification->insert())
			$notification->_addRecipients($recipients);

		info(sprintf('Notification::throw() - Notification [%s] sent', $notification->id));
		return $notification->id;
	}

	/**
	 * @param int $uid User id
	 * @param array $relation_ids Relation ids (there maybe more than one)
	 * @param int $status
	 * @todo WILL A USER ALWAYS HAVE A RELATION ID?
	 */
	function select($uid, $relation_ids, $status=NOTIFICATION_STATUS_UNREAD,$limit='',$offset=0, $product_id='')
	{
		$product_id || $product_id = (isset($this->product->id) ? $this->product->id : 1);
		!count($relation_ids) && $relation_ids[] = 0;
		$sql = "SELECT DISTINCT '' as CHECKBOX, n.SENDER_ID, n.USER_ID as SENDER_USER_ID, n.NOTIFICATION_ID,
				n.SUBJECT,  n.MESSAGE, n.`TYPE`, nr.CALLBACK, nr.`STATUS`, n.SENT_DATE , nr.RECIPIENT_ID,
				nr.USER_ID as RECEIVER_USER_ID, nr.CLEARANCE_LEVEL
			FROM TBL_NOTIFICATION as n
			INNER JOIN TBL_NOTIFICATION_RECIPIENT as nr
				USING(NOTIFICATION_ID)
			WHERE (n.PRODUCT_ID = ? OR n.PRODUCT_ID IS NULL) AND ";
		if($relation_ids)
			$sql .= "(nr.RECIPIENT_ID IN (".(join($relation_ids, ',')).") OR ";
		$sql .= "(ISNULL(nr.RECIPIENT_ID) AND nr.USER_ID = ?)";
		if($relation_ids)
			$sql .= ")";
		$sql .= "
				AND (nr.USER_ID = ? OR ISNULL(nr.USER_ID)) ";
		if($status)
			$sql .= " AND nr.`STATUS` & ? ";
		$sql .= " ORDER BY n.NOTIFICATION_ID DESC, nr.`STATUS` ";
		if($limit)
			$sql .= "LIMIT ".((int)$offset).", ".((int)$limit);
		$q = $this->db->query($sql);
		$q->bind_values($product_id, $uid, $uid,$status);

		$q->execute();
		return $q;
	}

	/**
	 * Get the number of notifications for a relation/user based on status
	 * @todo WILL A USER ALWAYS HAVE A RELATION ID?
	 */
	function count($uid, $relation_ids, $status=NOTIFICATION_STATUS_UNREAD, $product_id='')
	{
		$product_id || $product_id = (isset($this->product->id) ? $this->product->id : 1);
		!count($relation_ids) && $relation_ids[] = 0;
		$sql = "SELECT DISTINCT COUNT(n.NOTIFICATION_ID) as `COUNT`
			FROM TBL_NOTIFICATION as n
			INNER JOIN TBL_NOTIFICATION_RECIPIENT as nr
				USING(NOTIFICATION_ID)
			WHERE n.PRODUCT_ID = ? AND (nr.RECIPIENT_ID IN (".(join($relation_ids, ',')).") OR (ISNULL(nr.RECIPIENT_ID) AND nr.USER_ID = ?))
				AND (nr.USER_ID = ? OR ISNULL(nr.USER_ID)) ";
		if($status)
			$sql .= " AND nr.`STATUS` = ? ";
		$q = $this->db->query($sql);
		$q->bind_values($product_id, $uid, $uid,$status);
		$q->execute();
		$r = $q->fetch();
		return $r->{'COUNT'};
	}

	/**
	 *
	 */
	function translateType($langIf,$type)
	{
		switch($type)
		{
			case NOTIFICATION_TYPE_MESSAGE:
				return $langIf->getText('Notification Type Message');
			case NOTIFICATION_TYPE_INFO:
				return $langIf->getText('Notification Type Information');
			case NOTIFICATION_TYPE_WARNING:
				return $langIf->getText('Notification Type Warning');
			case NOTIFICATION_TYPE_ERROR:
				return $langIf->getText('Notification Type Error');
		}
	}
	/**
	 *
	 */
	function translateStatus($langIf,$status)
	{
		return '';
	}
	/**#@+
	 * @access private
	 */
	/**
	 * @param Array $recipients (associative)
	 */
	function _addRecipients($recipients)
	{
		foreach($recipients as $recipient)
		{
			$recipient['notification'] = &$this;

			if (isset($recipient['recipient']))
				if (get_class($recipient['recipient']) != 'Relation')
					trigger_error(sprintf('Notification::_addRecipients() - recipient [%s] is not an Relation object', get_class($recipient['recipient'])), E_USER_ERROR);

			if (isset($recipient['user']))
				if (get_class($recipient['user']) != 'ELCUser')
					trigger_error(sprintf('Notification::_addRecipients() - user [%s] is not an ELCUser object', get_class($recipient['user'])), E_USER_ERROR);

			$recp = new NotificationRecipient($this->db, $recipient);
			$recp->insert();
		}
	}
	/**
	 *
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_NOTIFICATION';
		$this->objectmap['primary']['id'] = 'NOTIFICATION_ID';
		$this->objectmap['properties']['sender->id'] = 'SENDER_ID';
		$this->objectmap['properties']['user->id'] = 'USER_ID';
		$this->objectmap['properties']['product->id'] = 'PRODUCT_ID';
		$this->objectmap['properties']['subject'] = 'SUBJECT';
		$this->objectmap['properties']['message'] = 'MESSAGE';
		$this->objectmap['properties']['type'] = 'TYPE';
		$this->objectmap['properties']['sent_date'] = 'SENT_DATE';
	}
	/**#@-*/
}
/**
 * Recipient
 * @author Marten Koedam <marten@basesoft.com
 * @package ELC
 * @version $Id$
 */
class NotificationRecipient extends AbstractObject
{
	var $notification;
	var $recipient;
	var $user;
	var $status;
	var $callback;
	var $clearance_level;

	function set($notification_id, $uid, $relation_ids)
	{
		$sql = "
					SELECT
						*
					FROM
						TBL_NOTIFICATION_RECIPIENT
					WHERE
						NOTIFICATION_ID = ".($notification_id)."
					AND";
		if($relation_ids)
			$sql .= "	(RECIPIENT_ID IN (".(join($relation_ids, ',')).") OR";
		$sql .= " (ISNULL(RECIPIENT_ID) AND USER_ID = ?)";
		if($relation_ids)
			$sql .= ")";
		$sql .= "
					AND
						(USER_ID = ? OR ISNULL(USER_ID))
					LIMIT 1
					";
		$q = $this->db->query($sql);
		$q->bind_values($uid, $uid);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		$this->notification_id = $r->NOTIFICATION_ID;
		$this->recipient_id = $r->RECIPIENT_ID;
		$this->user_id = $r->USER_ID;
		$this->status = $r->STATUS;
		$this->callback = $r->CALLBACK;
		$this->clearance_level = $r->CLEARANCE_LEVEL;

		return $this->complete = true;
	}

	function markRead($notification_id, $uid, $relation_ids)
	{
		$sql = "
					UPDATE
						TBL_NOTIFICATION_RECIPIENT
					SET
						`STATUS` = ".NOTIFICATION_STATUS_READ."
					WHERE
						NOTIFICATION_ID = ".($notification_id)."
					AND";
		if($relation_ids)
			$sql .= "	(RECIPIENT_ID IN (".(join($relation_ids, ',')).") OR";
		$sql .= " (ISNULL(RECIPIENT_ID) AND USER_ID = ?)";
		if($relation_ids)
			$sql .= ")";
		$sql .= "
					AND
						(USER_ID = ? OR ISNULL(USER_ID))
					";
		$q = $this->db->query($sql);
		$q->bind_values($uid, $uid);
		$q->execute();
	}

	/**#@+
	 * @access private
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_NOTIFICATION_RECIPIENT';
		$this->objectmap['primary']['notification->id'] = 'NOTIFICATION_ID';
		$this->objectmap['primary']['recipient->id'] = 'RECIPIENT_ID';
		$this->objectmap['primary']['user->id'] = 'USER_ID';
		$this->objectmap['properties']['status'] = 'STATUS';
		$this->objectmap['properties']['callback'] = 'CALLBACK';
		$this->objectmap['properties']['clearance_level'] = 'CLEARANCE_LEVEL';
	}
	/**#@-*/
}

/**#@+
 * Types
 */
define('NOTIFICATION_TYPE_MESSAGE',1);
define('NOTIFICATION_TYPE_INFO',2);
define('NOTIFICATION_TYPE_WARNING',3);
define('NOTIFICATION_TYPE_ERROR',4);
/**#@-*/

/**#@+
 * Recipient Notification Status
 */
define('NOTIFICATION_STATUS_UNREAD',1);
define('NOTIFICATION_STATUS_READ',2);
define('NOTIFICATION_STATUS_CC',4);
define('NOTIFICATION_STATUS_BCC',8);
define('NOTIFICATION_STATUS_FORWARD',16);
/**#@-*/

/**#@+
 * Notification Inbox Page Limit
 */
define('NOTIFICATION_INBOX_PAGE_LIMIT',5);
/**#@-*/
?>