<?php
require_once(ELC_CLASSES.'Relation.php');
/**
 * @author Marten Koedam <marten@basesoft.com>
 * @package I-Ware
 * @version $Id$
 */
class Operator extends Relation
{
	/**
	 * @var int $id
	 */
	var $id;
	/**
	 *
	 */
	function selectByCountry($country_id)
	{
		$sql = "SELECT R.RELATION_ID, R.`NAME`
			FROM TBL_RELATION as R
			INNER JOIN TBL_OPERATOR as O
				USING(RELATION_ID)
			INNER JOIN TBL_OPERATOR_COUNTRY OC
				ON R.RELATION_ID=OC.RELATION_ID
			WHERE OC.COUNTRY_ID=?
				ORDER BY R.`NAME`";

		$q = $this->db->query($sql);
		$q->bind_values($country_id);
		$q->execute();
		return $q;
	}

	function selectOverview()
	{
		 $sql = "SELECT '' as CHECKBOX, R.RELATION_ID, R.`NAME` as COMPANY_NAME,
						R.EMAIL, IP.INFO_PERSON_LAST_NAME AS CONTACT_NAME,
						COUNT(DISTINCT U.`USER_ID`) as USERS,'' as COUNTRIES
			FROM TBL_RELATION as R
			INNER JOIN TBL_OPERATOR as O
				USING(RELATION_ID)
			LEFT JOIN TBL_RELATION_USER as RU
				ON R.RELATION_ID = RU.RELATION_ID
			INNER JOIN TBL_INFO_PERSON IP
				ON R.CUSTOMER_CONTACT_ID = IP.INFO_PERSON_ID
			LEFT JOIN TBL_USER as U
				ON RU.USER_ID = U.USER_ID
			GROUP BY R.RELATION_ID
			ORDER BY R.`NAME`";
		$q = $this->db->query($sql);
		$q->execute();
		return $q;
	}

	function selectUserOverview($relation_id)
	{
		$sql = "SELECT '' as CHECKBOX, U.`USER_ID`,
				U.`USER_NAME`,
				CONCAT( IFNULL(IP.INFO_PERSON_FIRST_NAME,''),' ',
					IFNULL(IP.INFO_PERSON_MIDDLE_NAME,''),' ',
					IFNULL(INFO_PERSON_LAST_NAME,'')) as CONTACT_NAME,
				U.`USER_ACTIVE`, RU.IS_ADMIN
				#COUNT(T.LANGUAGE_ID) as TRANSLATE
			FROM TBL_RELATION as R
			INNER JOIN TBL_RELATION_USER as RU
				USING(RELATION_ID)
			INNER JOIN TBL_USER as U
				USING(USER_ID)
			INNER JOIN TBL_INFO_PERSON as IP
				USING(INFO_PERSON_ID)
			LEFT JOIN TBL_TRANSLATOR as T
				ON U.USER_ID = T.USER_ID
			WHERE R.RELATION_ID = ?
			GROUP BY U.USER_ID";
		$q = $this->db->query($sql);
		$q->bind_values($relation_id);
		$q->execute();
		return $q;
	}

	function check_operator($id)
	{
		$sql = "SELECT RELATION_ID FROM TBL_OPERATOR WHERE RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		$r = $q->fetch();
		if(empty($r))
			return false;
		if($r->RELATION_ID == $id)
			return true;
		return false;
	}

	function getOperatorCountries($id='')
	{
		$sql = "SELECT COUNTRY_ID FROM TBL_OPERATOR_COUNTRY WHERE RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		return $q;
	}

	function getUserId($id='')
	{
		$id || $id = $this->id;
		$sql = "SELECT U.USER_ID
			FROM TBL_USER as U
			INNER JOIN TBL_RELATION_USER as RU
				USING(USER_ID)
			WHERE RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		$r = $q->fetch();

		return $r->USER_ID;
	}
	/**
	 *
	 */
	function delete($id='')
	{
		$id = $id || $this->id;
		require_once(ELC_CLASSES.'User.php');
		// Delete User;
		// Delete Operator Countries
		// Delete Operator..
		$user = new ELCUser($this->db,$this->getUserId($id));
		$user->delete();

		$sql = "DELETE FROM TBL_OPERATOR WHERE RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();

		require_once(ELC_CLASSES.'OperatorCountry.php');
		$oc = new OperatorCountry($this->db,array('operator'=>$this));
		$oc->deleteByOperator();
		return true;
	}

	function insert()
	{
		info(sprintf('Created new Operator [%s]',
			$this->relation->id));

		return parent::insert();
	}
	function update()
	{
		info(sprintf('Updated Operator [%s]',
			$this->relation->id));

		return parent::update();
	}
	/**
	 *
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_OPERATOR';
		$this->objectmap['primary']['relation->id'] = 'RELATION_ID';
	}
}
?>
