<?php
require_once(ELC_CLASSES.'Relation.php');
require_once(ELC_CLASSES.'ParticipantBrand.php');
/**
 * @author Marten Koedam <marten@basesoft.com>
 * @package I-Ware
 * @version $Id$
 */
class Participant extends Relation
{
	/**
	 * @var int $id
	 */
	var $id;

	function isA($type)
	{
		return $this->producer_exporter & $type;
	}

	function getBusinessStartYear(&$crso,$product_id)
	{
		require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
		$cp = new CRSOParticipantProduct($this->db,array('participant'=>$this,'crso'=>$crso));
		$cp->set(null,null,$product_id);
		return (int) substr($cp->obligation_start_date, 0, 4);
	}
	function getBusinessStartMonth(&$crso,$product_id)
	{
		require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
		$cp = new CRSOParticipantProduct($this->db,array('participant'=>$this,'crso'=>$crso));
		$cp->set(null,null,$product_id);

		return (int) substr($cp->obligation_start_date, 5, 2);
	}
	function getObligationStartDate($crso,$product_id)
	{
		require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
		$cp = new CRSOParticipantProduct($this->db,array('participant'=>$this,'crso'=>$crso));
		$cp->set(null,null,$product_id);
		return $cp->obligation_start_date;
	}
	/**
	 * @param string $date ISO-Date
	 * @deprecated
	 */
	function setBusinessStartDate($date)
	{

	}

	/**
	 *
	 */
	function selectByStatus($crso_id, $select = '*',$order='CP.STATUS, R.`NAME`',$status='', $type=false)
	{
		$sql = "SELECT ".$select."
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CP
			INNER JOIN TBL_PARTICIPANT as P
				USING(PARTICIPANT_RELATION_ID)
			INNER JOIN TBL_RELATION as R
				ON P.PARTICIPANT_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_ADDRESS as A
				USING(ADDRESS_ID)
			INNER JOIN TBL_COUNTRY as C
				USING(COUNTRY_ID)
			LEFT JOIN TBL_RELATION_USER as RU
				ON R.RELATION_ID = RU.RELATION_ID
			LEFT JOIN TBL_LANGUAGE as IL
				ON IL.LANGUAGE_ID = R.INVOICE_LANGUAGE_ID
			LEFT JOIN TBL_USER as U
				ON RU.USER_ID = U.USER_ID
			LEFT JOIN TBL_VAT V
				ON R.VAT_NO=V.VAT_NO
			WHERE CP.CRSO_RELATION_ID = ? ";
			if($status)
				$sql .= " AND CP.STATUS = '".$status."' ";
			if($type)
				$sql .= " AND CP.PRODUCER_EXPORTER & '".$type."'";
		$sql .= "GROUP BY CP.PARTICIPANT_RELATION_ID ";
		if ($order)
			$sql .= "ORDER BY " . $order;

		$q = $this->db->query($sql);
		$q->bind_values($crso_id,0);
		$q->execute();
		return $q;
	}
	/**
	 *
	 */
	function selectOverview($crso_id, $product_id)
	{
		$sql = "SELECT R.`NAME` as PARTICIPANT_NAME, P.PARTICIPANT_RELATION_ID, A.CITY_NAME, C.`NAME` as COUNTRY_NAME
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CP
			INNER JOIN TBL_PARTICIPANT as P
				USING(PARTICIPANT_RELATION_ID)
			INNER JOIN TBL_RELATION as R
				ON P.PARTICIPANT_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_ADDRESS as A
				ON R.ADDRESS_ID = A.ADDRESS_ID
			INNER JOIN TBL_COUNTRY as C
				ON A.COUNTRY_ID = C.COUNTRY_ID
			WHERE CP.CRSO_RELATION_ID = ?
				AND CP.PRODUCT_ID = ?
				AND CP.STATUS >= ?
				AND CP.PRODUCER_EXPORTER & ".PARTICIPANT_PRODUCER."
			ORDER BY R.`NAME`";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$product_id,PARTICIPANT_STATUS_CONFIRMED);
		$q->execute();
		return $q;
	}

	function selectByCountry($countryId)
	{
		$sql = "SELECT R.`NAME` as PARTICIPANT_NAME, R.RELATION_ID AS PARTICIPANT_ID
			FROM TBL_PARTICIPANT P
			INNER JOIN TBL_RELATION R ON P.PARTICIPANT_RELATION_ID=R.RELATION_ID
			INNER JOIN TBL_ADDRESS A ON R.ADDRESS_ID=A.ADDRESS_ID
			WHERE COUNTRY_ID=?
		";
		$q = $this->db->query($sql);
		$q->bind_values($countryId);
		$q->execute();
		return $q;
	}

	function selectByCRSO($crso_relation_id)
	{
		$sql = "SELECT R.`NAME` as PARTICIPANT_NAME, R.RELATION_ID AS PARTICIPANT_ID
			FROM TBL_PARTICIPANT P
			INNER JOIN TBL_RELATION R
				ON P.PARTICIPANT_RELATION_ID=R.RELATION_ID
			INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT CP
				ON P.PARTICIPANT_RELATION_ID=CP.PARTICIPANT_RELATION_ID
			WHERE
				CP.CRSO_RELATION_ID=?
			ORDER BY R.`NAME`";
		$q = $this->db->query($sql);
		$q->bind_values($crso_relation_id);
		$q->execute();
		return $q;
	}

	/**
	 * Gets all participant to which a participant user can apply to.
	 * -> exclude itself
	 * -> exclude not authorized
	 */
	function selectAvailableParticipants()
	{
		require_once(ELC_CLASSES . 'CRSOParticipantProduct.php');

		$sql = "SELECT PARTICIPANT_RELATION_ID
			FROM TBL_PARTICIPANT P
			INNER JOIN TBL_RELATION_USER RU
				ON P.PARTICIPANT_RELATION_ID=RU.RELATION_ID
				AND RU.USER_ID=?";
		$q = $this->db->query($sql);
		$q->bind_values($_SESSION['session_user_id']);
		$q->execute();

		$used = array(-1);
		while($r = $q->fetch())
			$used[] = $r->PARTICIPANT_RELATION_ID;

		$sql = "SELECT C.`NAME` AS COUNTRY_NAME, R.`NAME` as PARTICIPANT_NAME, R.RELATION_ID AS PARTICIPANT_ID
			FROM TBL_PARTICIPANT P
			INNER JOIN TBL_RELATION R ON P.PARTICIPANT_RELATION_ID=R.RELATION_ID
			INNER JOIN TBL_ADDRESS A ON R.ADDRESS_ID=A.ADDRESS_ID
			INNER JOIN TBL_COUNTRY C USING(COUNTRY_ID)
			INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT CP ON R.RELATION_ID=CP.PARTICIPANT_RELATION_ID AND CP.STATUS>=?
			WHERE P.PARTICIPANT_RELATION_ID NOT IN (".implode(',', $used).")
			GROUP BY P.PARTICIPANT_RELATION_ID
			ORDER BY COUNTRY_NAME, PARTICIPANT_NAME -- please keep this order for the apply-at-participant tree
		";
		$q = $this->db->query($sql);
		$q->bind_values(PARTICIPANT_STATUS_CONFIRMED);
		$q->execute();
		return $q;
	}

	/**
	 * Selects all Participants for the given or set user id.
	 */
	function selectParticipantsByUser($user_id='', $producer_exporter='')
	{
		$sql = "SELECT DISTINCT r.RELATION_ID as PARTICIPANT_ID, r.NAME as PARTICIPANT_NAME
			FROM TBL_RELATION r
			INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT cp ON
				 r.RELATION_ID = cp.PARTICIPANT_RELATION_ID
			INNER JOIN TBL_RELATION_USER ru ON
				r.RELATION_ID=ru.RELATION_ID
			WHERE
				ru.USER_ID=?
				AND cp.PRODUCER_EXPORTER & ?
		";
		$q = $this->db->query($sql);
		$q->bind_values($user_id, $producer_exporter);
		$q->execute();
		return $q;
	}

	/**
	 *
	 */
	function delete($id='')
	{
		$id || $id = $this->relation->id;

		/*
		$this->referencemap['dbtable'] = 'TBL_PROVISIONAL_DECLARATION';
		$this->referencemap['ondelete'] = INTERCLEAN_REF_CASCADE;
		$this->referencemap['fields']['id'] = 'DOSSIER_ID';
		*/
	}
	/**
	 * @static
	 */
	static function translateStatus($langIf, $status)
	{
		switch ( $status )
		{
			case PARTICIPANT_STATUS_NEW:
				return array($langIf->getText('PARTICIPANT_STATUS_NEW'),'status_new');
			case PARTICIPANT_STATUS_VERIFIED:
				return array($langIf->getText('PARTICIPANT_STATUS_VERIFIED'),'status_verified');;
			case PARTICIPANT_STATUS_CONFIRMED:
				return array($langIf->getText('PARTICIPANT_STATUS_CONFIRMED'),'status_confirmed');;
			case PARTICIPANT_STATUS_AUTHORIZED:
				return array($langIf->getText('PARTICIPANT_STATUS_AUTHORIZED'),'status_authorized');;
			case PARTICIPANT_STATUS_PARTICIPANT_READY:
				return array($langIf->getText('PARTICIPANT_STATUS_PARTICIPANT_READY'),'status_participant_ready');;
		}
	}
	/**
	 *
	 */
	function set($id='')
	{
		$id || $id = $this->relation->id;
		$sql = "SELECT *
			FROM TBL_PARTICIPANT
			WHERE PARTICIPANT_RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		$this->relation = new Relation($this->db,$id);

	//	$this->business_start_date = $r->BUSINESS_START_DATE;
	//	$this->producer_reg_no = $r->PRODUCER_REG_NO;
//		$this->producer_exporter = $r->PRODUCER_EXPORTER;
		return true;
	}

	function setByVAT($vat_no)
	{
		$sql = "SELECT PARTICIPANT_RELATION_ID
			FROM TBL_PARTICIPANT p
			INNER JOIN TBL_RELATION r ON p.PARTICIPANT_RELATION_ID = r.RELATION_ID
			WHERE r.VAT_NO = ?
		";
		$q = $this->db->query($sql);
		$q->bind_values($vat_no);
		$q->execute();

		if (!($r = $q->fetch()))
			return false;

		return $this->set($r->{'PARTICIPANT_RELATION_ID'});
	}

	function insert()
	{
		$this->created = date('Y-m-d H:i:s'); // FIXME

		info(sprintf('Created new Participant [%s]',
			$this->relation->id));
		return parent::insert();
	}
	function update()
	{
		info(sprintf('Updated Participant [%s]',
			$this->relation->id));
		return parent::update();
	}

	function selectMasterData($crso_id) {
		$sql = "
			SELECT
				P.PARTICIPANT_RELATION_ID, R.NAME,
				A.ADDRESS_STREET_NAME as ADDRESS, A.ADDRESS_ZIPCODE, A.CITY_NAME,
				CONCAT_WS(' ', I.INFO_PERSON_FIRST_NAME, I.INFO_PERSON_MIDDLE_NAME, I.INFO_PERSON_LAST_NAME) as CONTACT_NAME,
				R.PHONE, R.FAX, L.LANGUAGE_CODE, C.CODE AS COUNTRY_CODE,
				R.VAT_NO, R.EMAIL, R.WEBSITE,

				RI.NAME as SEND_TO_NAME,
				RIA.ADDRESS_STREET_NAME as SEND_TO_ADDRESS,
				RIA.ADDRESS_ZIPCODE as SEND_TO_ZIPCODE, RIA.CITY_NAME as SEND_TO_CITY_NAME,
				RIAC.CODE as SEND_TO_COUNTRY_CODE,
				CONCAT_WS(' ', RII.INFO_PERSON_FIRST_NAME, RII.INFO_PERSON_MIDDLE_NAME, RII.INFO_PERSON_LAST_NAME) as SEND_TO_CONTACT_NAME,
				R.BANK_ACCOUNT_NUMBER, R.IBAN_NO, R.SWIFT_BIC_CODE,
				R.DD_BANK_ACCOUNT, R.DD_IBAN_NO, R.DD_SWIFT_BIC_CODE,
				CP.PRODUCER_REG_NO, R.PO_NO

				FROM TBL_CRSO_PARTICIPANT CP
				INNER JOIN TBL_RELATION R ON CP.PARTICIPANT_RELATION_ID = R.RELATION_ID
				INNER JOIN TBL_PARTICIPANT P ON R.RELATION_ID = P.PARTICIPANT_RELATION_ID
				INNER JOIN TBL_ADDRESS A ON R.ADDRESS_ID = A.ADDRESS_ID
				INNER JOIN TBL_INFO_PERSON I ON R.CUSTOMER_CONTACT_ID = I.INFO_PERSON_ID
				INNER JOIN TBL_LANGUAGE L ON R.INVOICE_LANGUAGE_ID = L.LANGUAGE_ID
				INNER JOIN TBL_COUNTRY C ON A.COUNTRY_ID = C.COUNTRY_ID

				INNER JOIN TBL_RELATION_INVOICE as RI ON R.RELATION_ID = RI.RELATION_ID
				INNER JOIN TBL_ADDRESS as RIA ON RIA.ADDRESS_ID = RI.ADDRESS_ID
				INNER JOIN TBL_INFO_PERSON as RII ON RI.INFO_PERSON_ID = RII.INFO_PERSON_ID
				INNER JOIN TBL_COUNTRY as RIAC ON RIA.COUNTRY_ID = RIAC.COUNTRY_ID

				WHERE CP.CRSO_RELATION_ID = ?
		";

		$q = $this->db->query($sql);
		$q->bind_values($crso_id);
		$q->execute();

		return $q;
	}

	/**
	 * @param $ids
	 * @param $crso_id
	 * @param $product_id
	 */
	function export_masterdata($ids = array(),$crso_id='',$product_id='',$with_active=false)
	{
		$sql = "
			SELECT
				CP.PARTICIPANT_RELATION_ID AS PART,
				CP.CRSO_RELATION_ID AS CRSO,
				CP.PRODUCT_ID AS PROD,
				RR.NAME AS CRSO_NAME,
				PP.NAME AS PRODUCT,
				R.`NAME` as PARTICIPANT_NAME,
				IF(CP.PRODUCER_EXPORTER=1 OR CP.PRODUCER_EXPORTER=3,TRUE,FALSE) AS PRODUCER,
				IF(CP.PRODUCER_EXPORTER=2 OR CP.PRODUCER_EXPORTER=3,TRUE,FALSE) AS EXPORTER,
				CONCAT_WS(' ',A.ADDRESS_STREET_NAME,A.ADDRESS_STREET_NUMBER) AS COMPANY_ADDRESS,
				A.ADDRESS_ZIPCODE  AS COMPANY_POSTAL_CODE,
				A.CITY_NAME AS COMPANY_CITY,
				C.`NAME` as COMPANY_COUNTRY,
				REG.PRODUCER_REG_NO AS PRODUCER_REGISTRATION_NUMBER,
				CP.OBLIGATION_START_DATE,
				R.VAT_NO,
				R.PO_NO,
				R.BANK_ACCOUNT_NUMBER AS BANKACCOUNT,
				R.IBAN_NO AS IBAN,
				R.SWIFT_BIC_CODE AS SWIFT,
				R.DD_BANK_ACCOUNT AS DIRECT_DEBIT_BANKACCOUNT,
				R.DD_IBAN_NO AS DIRECT_DEBIT_IBAN,
				R.DD_SWIFT_BIC_CODE AS DIRECT_DEBIT_SWIFT,
				R.EMAIL,
				R.WEBSITE,
				R.PHONE,
				R.FAX,
				L.LANGUAGE AS COMPANY_LANGUAGE,
				CONCAT_WS(' ',IP.INFO_PERSON_TITLE,IP.INFO_PERSON_FIRST_NAME,IP.INFO_PERSON_MIDDLE_NAME,IP.INFO_PERSON_LAST_NAME) AS CONTACT_PERSON,
				CONCAT_WS(' ',UIP.INFO_PERSON_TITLE,UIP.INFO_PERSON_FIRST_NAME,UIP.INFO_PERSON_MIDDLE_NAME,UIP.INFO_PERSON_LAST_NAME) AS ADMIN_USER_NAME,
				UIP.INFO_PERSON_DEPARTMENT AS ADMIN_USER_DEPARTMENT,
				UIP.INFO_PERSON_POSITION AS ADMIN_USER_POSITION,
				UIP.INFO_PERSON_TELHOUSE AS ADMIN_USER_PHONE,
				UIP.INFO_PERSON_FAX AS ADMIN_USER_FAX,
				UIP.INFO_PERSON_EMAIL AS ADMIN_USER_EMAIL,
				CC.NAME AS ADMIN_USER_COUNTRY,
				LL.LANGUAGE AS ADMIN_USER_LANGUAGE,
				RI.NAME AS INVOICE_SENT_TO_COMPANY_NAME,
				CONCAT_WS(' ',ADR.ADDRESS_STREET_NAME,ADR.ADDRESS_STREET_NUMBER) AS INVOICE_SENT_TO_ADDRESS,
				ADR.CITY_NAME AS INVOICE_SENT_TO_CITY,
				ADR.ADDRESS_ZIPCODE AS INVOICE_SENT_TO_POSTAL_CODE,
				COU.NAME AS INVOICE_SENT_TO_COUNTRY,
				CONCAT_WS(' ',IIP.INFO_PERSON_TITLE,IIP.INFO_PERSON_FIRST_NAME,IIP.INFO_PERSON_MIDDLE_NAME,IIP.INFO_PERSON_LAST_NAME) AS INVOICE_SENT_TO_CONTACT_PERSON,
				IIP.INFO_PERSON_DEPARTMENT AS INVOICE_SENT_TO_CONTACT_DEPARTMENT,
				IIP.INFO_PERSON_EMAIL AS INVOICE_SENT_TO_CONTACT_EMAIL,
				IIP.INFO_PERSON_BACKUPEMAIL AS INVOICE_SENT_TO_CONTACT_BACKUPEMAIL
			";

		if ($with_active) {
			$sql .= ", CP.ACTIVE as ACTIVE ";
		}

		$sql .= "
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CP
			INNER JOIN TBL_PRODUCT PP
				ON CP.PRODUCT_ID = PP.PRODUCT_ID
			INNER JOIN TBL_RELATION as R
				ON CP.PARTICIPANT_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_RELATION RR
				ON CP.CRSO_RELATION_ID = RR.RELATION_ID
			LEFT JOIN TBL_ADDRESS as A
				ON R.ADDRESS_ID = A.ADDRESS_ID
			INNER JOIN TBL_COUNTRY as C
				ON A.COUNTRY_ID = C.COUNTRY_ID
			INNER JOIN TBL_INFO_PERSON IP
				ON IP.INFO_PERSON_ID=R.CUSTOMER_CONTACT_ID
			INNER JOIN TBL_LANGUAGE L
				ON R.INVOICE_LANGUAGE_ID=L.LANGUAGE_ID
			LEFT JOIN TBL_CRSO_PARTICIPANT REG
				ON CP.PARTICIPANT_RELATION_ID = REG.PARTICIPANT_RELATION_ID
				AND CP.CRSO_RELATION_ID = REG.CRSO_RELATION_ID
			INNER JOIN TBL_RELATION_USER RU
				ON RU.RELATION_ID=R.RELATION_ID
			INNER JOIN TBL_USER U
				ON U.USER_ID=RU.USER_ID
				AND RU.IS_ADMIN = 1
			INNER JOIN TBL_INFO_PERSON UIP
				ON U.INFO_PERSON_ID = UIP.INFO_PERSON_ID
			INNER JOIN TBL_LANGUAGE LL
				ON LL.LANGUAGE_ID=UIP.LANGUAGE_ID
			LEFT JOIN TBL_ADDRESS AA
				ON UIP.ADDRESS_ID=AA.ADDRESS_ID
			LEFT JOIN TBL_COUNTRY CC
				ON AA.COUNTRY_ID=CC.COUNTRY_ID
			INNER JOIN TBL_RELATION_INVOICE RI
				ON CP.PARTICIPANT_RELATION_ID = RI.RELATION_ID
			INNER JOIN TBL_ADDRESS ADR
				ON RI.ADDRESS_ID = ADR.ADDRESS_ID
			INNER JOIN TBL_INFO_PERSON IIP
				ON RI.INFO_PERSON_ID = IIP.INFO_PERSON_ID
			INNER JOIN TBL_COUNTRY AS COU
				ON ADR.COUNTRY_ID = COU.COUNTRY_ID ";
		if(count($ids) > 0 || $crso_id != '' || $product_id != '')
		{
			$sql .= " WHERE ";
			if(count($ids) > 0)
				$sql .= " CP.PARTICIPANT_RELATION_ID IN (" . str_repeat("?, ",count($ids)-1) . "?) AND";
			if($crso_id != '')
				$sql .= " CP.CRSO_RELATION_ID = ? AND";
			if($product_id != '')
				$sql .= " CP.PRODUCT_ID = ? AND";
			$sql = substr($sql,0,strlen($sql)-3);
		}
		$sql .= " ORDER BY RR.NAME";
		$q = $this->db->query($sql);
		if($crso_id != '')
			$ids[] = $crso_id;
		if($product_id != '')
			$ids[] = $product_id;
		if(count($ids) > 0)
			$q->bind_values($ids);
		$q->execute();
		$data = '';
		$arr = array();
		$pn = new ParticipantBrand($this->db);
		while($r = $q->fetch())
		{
			$qq = $pn->getBrands($r->PART,$r->CRSO, $r->PROD);
			$brands = '';
			while($rr = $qq->fetch())
				$brands .= $rr->NAME . ";";
			$brands = substr($brands,0,strlen($brands)-1);
			$a = array();
			foreach($r as $key=>$value)
			if($key != 'PART' && $key != 'CRSO' && $key != 'PROD')
			{
				$value = ($value=='1')?'True':$value;
				$value = ($value=='0')?'False':$value;
				$a[$key] = $value;
				if($key == 'COMPANY_LANGUAGE')
					$a['BRANDS'] = $brands;
			}
			$arr[] = $a;
		}
		require_once(IWARE.'IWCSVWriter.php');
		$csv = new IWCSVWriter($arr);
		$csv->eol = "\n";
		$csv->show_header = true;
		return $csv;
	}
	/**
	 *
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_PARTICIPANT';
		$this->objectmap['primary']['relation->id'] = 'PARTICIPANT_RELATION_ID';
//		$this->objectmap['properties']['business_start_date'] = 'BUSINESS_START_DATE';
//		$this->objectmap['properties']['producer_reg_no'] = 'PRODUCER_REG_NO';
//		$this->objectmap['properties']['producer_exporter'] = 'PRODUCER_EXPORTER';
		$this->objectmap['properties']['last_update'] = 'LAST_UPDATE';
		$this->objectmap['properties']['created'] = 'CREATED';
	}
	/**
	 * Used in InterfaceParticipantUserRegistration
	 * @static
	 */
	static public function getClearanceLevels()
	{
		return array(
			'ClearanceFeeRegistration',
			'ClearanceReporting',
			'ClearanceApplyAtCRSO',
			'ClearanceDeclarationSettings',
			'ClearanceProofOfPayment',
			'ClearanceProofOfExport',
			'ClearanceRemoteViewer');
	}
}
/**#@+
 * A participant can be a Producer, an Exporter or a combination of the two.
 * Use BITWISE operators to determine if it is either or both ($this->producer_exporter)
 */
define('PARTICIPANT_PRODUCER',1);
define('PARTICIPANT_EXPORTER',2);
/**#@-*/
/**#@+
 * ClearanceLevel Defines
 */
define('ClearanceFeeRegistration',1); // Declarations,RefundRequests
define('ClearanceReporting',2); // Reports of all kinds
define('ClearanceApplyAtCRSO',4); // Apply as an existing participant at a new CRSO
define('ClearanceDeclarationSettings', 8); // declaration settings
define('ClearanceProofOfPayment',16);
define('ClearanceProofOfExport', 32);
define('ClearanceRemoteViewer', 64); // Can invite CRSO to the view mode.
/**#@-*/
?>
