<?php
require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES.'Brand.php');
/**
 * @author Marten Koedam <marten@basesoft.com>
 * @package ELC
 * @version $Id$
 */
class ParticipantBrand extends AbstractObject
{

	function getAllParticipantBrands($participant_id, $crso_id, $active_only=false) {
		$sql = "SELECT B.BRAND_ID, B.`NAME`, PB.ACTIVE
			FROM TBL_BRAND as B
			INNER JOIN TBL_PARTICIPANT_BRAND as PB
				USING(BRAND_ID)
			WHERE PARTICIPANT_RELATION_ID = ?
				AND CRSO_RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id,$crso_id);
		$q->execute();
		$brands = array();
		while($r = $q->fetch()) {
			if($active_only && !$r->ACTIVE) continue;
			$brands[$r->BRAND_ID] = $r->NAME;
		}
		return $brands;
	}

	function deactivateBrands($participant_id, $brand_ids) {
		$sql = "UPDATE TBL_PARTICIPANT_BRAND
			SET ACTIVE = 0
			WHERE PARTICIPANT_RELATION_ID = ?
				AND BRAND_ID IN (".implode(', ', $brand_ids).")
			";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id);
		$q->execute();
	}

	/**
	 * Only authorized brands by the CRSO can be selected
	 */
	function getBrands($participant_id,$crso_id, $product_id)
	{
		$sql = "SELECT B.BRAND_ID, B.`NAME`
			FROM TBL_BRAND as B
			INNER JOIN TBL_PARTICIPANT_BRAND as PB
				USING(BRAND_ID)
			WHERE PARTICIPANT_RELATION_ID = ?
				AND CRSO_RELATION_ID = ?
			AND PRODUCT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id,$crso_id, $product_id);
		$q->execute();
		return $q;
	}
	function getBrandsIncludingUsed($participant_id,$crso_id,$product_id, $active_only=false)
	{
		$sql = "SELECT B.BRAND_ID, B.`NAME`, PB.ACTIVE
			FROM TBL_BRAND as B
			INNER JOIN TBL_PARTICIPANT_BRAND as PB
				ON B.BRAND_ID = PB.BRAND_ID
			WHERE PB.PARTICIPANT_RELATION_ID = ?
				AND B.CRSO_RELATION_ID = ?
				AND B.PRODUCT_ID = ?";
		// First we get all the participant brands.
		$q = $this->db->query($sql);
		$q->bind_values($participant_id, $crso_id, $product_id);
		$q->execute();

		$brands = array();
		while($r = $q->fetch()) {
			if($active_only && !$r->ACTIVE) continue;
			$brands[$r->BRAND_ID] = $r->NAME;
		}

		$used = array();
		// now check provisional
		$sql = "SELECT BRAND_ID
			FROM TBL_PROVISIONAL_DECLARATION
			WHERE PARTICIPANT_RELATION_ID = ?
				AND CRSO_RELATION_ID = ?
				AND PRODUCT_ID = ?
				AND NOT ISNULL(BRAND_ID)";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id, $crso_id, $product_id);
		$q->execute();
		while($r = $q->fetch()) {
			$used[] = $r->BRAND_ID;
		}

		// now check declarations
		$sql = "SELECT DISTINCT DL.BRAND_ID
			FROM TBL_DECLARATION_LINE as DL
			INNER JOIN TBL_DECLARATION as D
				ON DL.DECLARATION_ID = D.DECLARATION_ID
			WHERE D.PARTICIPANT_RELATION_ID = ?
				AND D.CRSO_RELATION_ID = ?
				AND D.PRODUCT_ID = ?
				AND NOT ISNULL(DL.BRAND_ID)";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id, $crso_id, $product_id);
		$q->execute();
		while($r = $q->fetch()) {
			if(!in_array($r->BRAND_ID, $used))
				$used[] = $r->BRAND_ID;
		}

		// now check refund requests
		$sql = "SELECT DISTINCT RRL.BRAND_ID
			FROM TBL_REFUND_REQUEST_LINE as RRL
			INNER JOIN TBL_REFUND_REQUEST as R
				ON RRL.REFUND_REQUEST_ID = R.REFUND_REQUEST_ID
			WHERE R.PARTICIPANT_RELATION_ID = ?
				AND R.CRSO_RELATION_ID = ?
				AND R.PRODUCT_ID = ?
				AND NOT ISNULL(RRL.BRAND_ID)";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id, $crso_id, $product_id);
		$q->execute();
		while($r = $q->fetch()) {
			if(!in_array($r->BRAND_ID, $used))
				$used[] = $r->BRAND_ID;
		}
		return array($brands, $used);
	}

	function deleteWhereNotUsed($participant_id,$crso_id,$product_id)
	{
		list($brands, $used) = $this->getBrandsIncludingUsed($participant_id,$crso_id,$product_id);
		if(count($used) == 0 )
			return;
		$sql = "DELETE FROM TBL_PARTICIPANT_BRAND WHERE PARTICIPANT_RELATION_ID = ? AND BRAND_ID NOT IN(".implode(',',$used).")";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id);
		return $q->execute();
	}

	/**
	 *
	 */
	function selectOverview($crso_id,$participant_id=false,$brand_id=false)
	{
		$sql = "SELECT R.`NAME` as PARTICIPANT_NAME, R.VAT_NO, B.`NAME` as BRAND_NAME
			FROM TBL_RELATION as R
			INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT as CP
				ON R.RELATION_ID = CP.PARTICIPANT_RELATION_ID
			INNER JOIN TBL_PARTICIPANT_BRAND as PB
				ON CP.PARTICIPANT_RELATION_ID = PB.PARTICIPANT_RELATION_ID
			INNER JOIN TBL_BRAND as B
				ON PB.BRAND_ID = B.BRAND_ID
					AND CP.CRSO_RELATION_ID = B.CRSO_RELATION_ID
			WHERE CP.CRSO_RELATION_ID = ? ";

		if($participant_id)
		{
			$participant_id = (array)$participant_id;
			$sql .= " AND CP.PARTICIPANT_RELATION_ID IN ( ".implode(',',$participant_id)." ) ";
		}
		if($brand_id)
		{
			$brand_id = (array)$brand_id;
			$sql .= " AND B.BRAND_ID IN ( ".implode(',',$brand_id)." ) ";
		}
		$sql .= "ORDER BY PARTICIPANT_NAME, BRAND_NAME";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id);
		$q->execute();
		return $q;
	}

	/**
	 *
	 */
	function select($crso_id,$participant_id='',$selected='')
	{
		$participant_id || $participant_id = $this->participant->relation->id;

		$sql = "SELECT B.BRAND_ID, B.`NAME`,
				IF(B.BRAND_ID='".$selected."',' selected=\"selected\"', '') as SELECTED
			FROM TBL_BRAND as B
			INNER JOIN TBL_PARTICIPANT_BRAND as PB
				USING(BRAND_ID)
			WHERE PARTICIPANT_RELATION_ID = ?
				AND CRSO_RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id,$crso_id);
		$q->execute();
		$rv = '';
		while($r = $q->fetch())
			$rv .= '<option value="'.$r->BRAND_ID.'"'.$r->SELECTED.'>'.$r->NAME.'</option>';
		return $rv;
	}
	/**
	 * @access private
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_PARTICIPANT_BRAND';
		$this->objectmap['primary']['participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
		$this->objectmap['primary']['brand->id'] = 'BRAND_ID';
		$this->objectmap['properties']['active'] = 'ACTIVE';
		$this->objectmap['inserttype'] = 'REPLACE';
	}
}
?>
