<?php
require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES.'Product.php');
/**
 * Example: TL, Lightbulb, Blacklight
 * @todo ProductCategories should be saved per CRSO and not in a free-for-all type of way
 *
 * @author Marten Koedam <marten@basesoft.com>
 * @package ELC
 * @version $Id$
 */
class ProductCategory extends AbstractObject
{
	var $id;
	var $product;
	var $crso;
	var $name;
	var $description;

	/**
	 * @param int $id
	 */
	function set($id='')
	{
		$id || $id = $this->id;

		$sql = "SELECT PRODUCT_ID, PC.PRODUCT_CATEGORY_ID, CRSO_RELATION_ID,
				PC.NAME as CATEGORY_NAME, PC.DESCRIPTION as CATEGORY_DESCRIPTION, PC.START_YEAR, PC.END_YEAR
			FROM TBL_PRODUCT_CATEGORY as PC
			WHERE PC.PRODUCT_CATEGORY_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		if(!$q->execute())
			throw new Exception('ProductCategory not found: '.$id);
		if(!($r = $q->fetch()))
			throw new Exception('ProductCategory not found: '.$id);

		$this->id = $id;
		$this->product = new Product($this->db,$r->PRODUCT_ID);
		$this->crso = new CRSO($this->db, array('relation'=>new Relation($this->db, $r->CRSO_RELATION_ID)));
		$this->name = $r->CATEGORY_NAME;
		$this->description = $r->CATEGORY_DESCRIPTION;
		$this->start_year = $r->START_YEAR;
		$this->end_year = $r->END_YEAR;
		return $this->complete = true;
	}

	function is_for_crso($crso_id)
	{
		if ($this->crso->relation->id == $crso_id)
			return true;
		return false;
	}

	/**
	 *
	 */
	function selectOverview($product_id, $crso_id, $year='')
	{
		$product_id = (array) $product_id;
		$sql = "SELECT PC.PRODUCT_CATEGORY_ID, PC.NAME as CATEGORY_NAME,
				PC.DESCRIPTION as CATEGORY_DESCRIPTION,
				PC.START_YEAR AS START_YEAR,
				PC.END_YEAR AS END_YEAR,
				PC.END_YEAR AS DISABLE_CATEGORY,
				'' as CURRENT_FEE
			FROM TBL_PRODUCT as P
			INNER JOIN TBL_PRODUCT_CATEGORY as PC
				ON P.PRODUCT_ID=PC.PRODUCT_ID
			WHERE P.COMPANY_ID = ? AND CRSO_RELATION_ID=?";
		if($product_id)
			$sql .= " AND P.PRODUCT_ID IN ( ".implode(',',(array) $product_id)." )";
		if($year)
			$sql .= sprintf(" AND PC.START_YEAR <= %s AND (PC.END_YEAR IS NULL OR PC.END_YEAR > %s)", (int) $year, (int) $year);
		$q = $this->db->query($sql);
		$q->bind_values($_SESSION['session_user_company_id'], $crso_id);
		$q->execute();
		return $q;
	}
	/**
	 *
	 */
	function select($product_id='',$selectedindex='',$crso_id='', $year='')
	{
		$product_id || $product_id = $this->product->id;
		$crso_id || $crso_id = $this->crso->relation->id;

		$d = currentDate();
		$year || $year == $d->getYear();

		$rv = '';
		$sql = "SELECT PC.PRODUCT_CATEGORY_ID, PC.NAME as CATEGORY_NAME,
				IF(PC.PRODUCT_CATEGORY_ID='".$selectedindex."',' selected=\"selected\"','') as SELECTED
			FROM TBL_PRODUCT_CATEGORY as PC
			WHERE PC.PRODUCT_ID = ?
				AND PC.CRSO_RELATION_ID = ?
				AND (PC.END_YEAR IS NULL OR PC.END_YEAR > ?)
				AND PC.START_YEAR <= ?
				";
		$q = $this->db->query($sql);
		$q->bind_values($product_id,$crso_id, $year, $year);
		$q->execute();
		while($r = $q->fetch())
			$rv .='<option value="'.$r->PRODUCT_CATEGORY_ID.'"'.$r->SELECTED.'>'.$r->CATEGORY_NAME.'</option>';
		return $rv;
	}

	function getProductCategories($crso_id, $product_id)
	{
		$sql = "SELECT PRODUCT_CATEGORY_ID, pc.NAME, pc.DESCRIPTION
			FROM TBL_PRODUCT_CATEGORY pc
			INNER JOIN TBL_PRODUCT USING (PRODUCT_ID)
			WHERE CRSO_RELATION_ID=?
			AND PRODUCT_ID=?;";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, $product_id);
		$q->execute();
		return $q;
	}


	/**
	 * Allow disabling of product categories, see #2528, #2530
	 */
	function disableCategory($product_category_id) {

		$crso_id = $this->crso->relation->id;
		$product_id = $this->product->id;

		$d = currentDate();
		$year = $d->getYear();
		$month = $d->getMonth();
		$day = $d->getDay();

		if ($month == 1)
			$end_year = $year;
		else
			$end_year = $year+1;

		$this->db->begin();

		# update end_date
		$q = $this->db->query("UPDATE TBL_PRODUCT_CATEGORY SET END_YEAR = ? WHERE PRODUCT_CATEGORY_ID = ?");
		$q->bind_values($end_year, $product_category_id);
		$q->execute();

		# remove any provisional for this pc/end_year
		$q = $this->db->query("DELETE FROM TBL_PROVISIONAL_DECLARATION WHERE YEAR = ? AND CRSO_RELATION_ID = ? AND PRODUCT_ID = ? AND PRODUCT_CATEGORY_ID = ?");
		$q->bind_values($end_year, $crso_id, $product_id, $product_category_id);
		$q->execute();

		$this->db->commit();

		$log = getLog();
		$log->info(sprintf('Disabled category [%s] by setting end_year [%s] for crso [%s] and product [%s]', $product_category_id, $end_year, $crso_id, $product_id));

	}

	/**
	 * returns true if the current category is disabled for the current year
	 * @return boolean
	 */
	function isEnabled() {

		if (!$this->end_year)
			return true;

		$d = currentDate();
		$year = $d->getYear();

		return $year < $this->end_year;

	}

	/**
	 *
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_PRODUCT_CATEGORY';
		$this->objectmap['primary']['id'] = 'PRODUCT_CATEGORY_ID';
		$this->objectmap['properties']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['product->id'] = 'PRODUCT_ID';
		$this->objectmap['properties']['name'] = 'NAME';
		$this->objectmap['properties']['description'] = 'DESCRIPTION';
		$this->objectmap['properties']['start_year'] = 'START_YEAR';
		$this->objectmap['properties']['end_year'] = 'END_YEAR';
	}
}
?>
