<?php
require_once(IWARE.'AbstractObject.php');
/**
 * @package ELC
 * @subpackage Product
 * @author Marten Koedam <marten@basesoft.com>
 * @since 2006-02-14
 * @version $Id$
 */
class ProductFee extends AbstractObject
{
	/**
	 * @var Product $product
	 */
	var $product;
	/**
	 * @var CRSO $crso
	 */
	var $crso;
	/**
	 * @var Datetime $start_date
	 */
	var $start_date;
	/**
	 * @var float $fee
	 */
	var $fee;
	/**
	 * @param int $id
	 * @param int $crso_id;
	 * @param datetime $date
	 */
	function getFee($id='',$crso_id='',$date='')
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		if(!$date)
		{
			$d = currentDate();
			$date = $d->getYear().'-'.$d->getMonth().'-01 00:00:00';
		}
		$id || $id = $this->product->id;
		$sql = "SELECT f_elcProductFee(?, ?, NULL, ?) as FEE";
		$q = $this->db->query($sql);
		$q->bind_values($id,$crso_id,$date);

		if(!$q->execute())
			return false; //trigger error
		if(!($r = $q->fetch()))
			return false;

		return $r->FEE;
	}

	function getAllFees($id='',$crso_id='')
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$id || $id = $this->product->id;
		$sql = "SELECT START_DATE, FEE
			FROM TBL_PRODUCT_FEE
			WHERE PRODUCT_ID = ?
				AND CRSO_RELATION_ID = ?
			ORDER BY START_DATE DESC";
		$q = $this->db->query($sql);
		$q->bind_values($id,$crso_id);

		if(!$q->execute())
			trigger_error("Error executing sql", E_USER_ERROR);
		return $q;
	}

	/**
	 * @access private
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_PRODUCT_FEE';
		$this->objectmap['primary']['product->id'] = 'PRODUCT_ID';
		$this->objectmap['primary']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['primary']['start_date'] = 'START_DATE';
		$this->objectmap['properties']['fee'] = 'FEE';
	}

	public static function hasFeeAlready($db, $product_id, $crso_id, $start_date)
	{
		$q = $db->query("SELECT
						FEE
						FROM TBL_PRODUCT_FEE
						WHERE START_DATE = ?
						AND CRSO_RELATION_ID = ?
						AND PRODUCT_ID = ?");
		$q->bind_values($start_date.' 00:00:00', $crso_id, $product_id);
		$q->execute();
		if($r = $q->fetch())
			return true;
		return false;
	}
}
/**
 * If the productcategoryfee is unavailable, get the productfee
 * @package ELC
 * @subpackage Product
 * @author Marten Koedam <marten@basesoft.com>
 * @since 2006-02-14
 */
class ProductCategoryFee extends AbstractObject
{
	/**
	 * @var ProductCategory $product_category
	 */
	var $product_category;
	/**
	 * @var CRSO $crso
	 */
	var $crso;
	/**
	 * @var Datetime $start_date
	 */
	var $start_date;
	/**
	 * @var float $fee
	 */
	var $fee;
	/**
	 * @param int $id
	 * @param int $crso_id
	 * @param int $date
	 */
	function getFee($id='',$crso_id='',$date='',$product_id=null)
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$id || $id = null; //needs null
		$product_id || $product_id = $_SESSION['selected_product_id'];
		if(!$date)
		{
			$d = currentDate();
			$date = $d->getYear().'-'.$d->getMonth().'-01 00:00:00';
		}


		$sql = "SELECT f_elcProductFee(?, ?, ?, ?) as FEE";
		$q = $this->db->query($sql);
		$q->bind_values($product_id,$crso_id,$id,$date);
		//debug($q->tmp_sql);
		$q->execute();

		$r = $q->fetch();
		return $r->FEE;
	}

	function getFeeOverview($product_id,$crso_id='')
	{
		$crso_id || $crso_id = $this->crso->relation->id;

		$sql = "SELECT PC.`NAME` as CATEGORY_NAME, '' as StartDate,
				PCF.FEE,
				MONTH(PCF.START_DATE) as Month, YEAR(PCF.START_DATE) as Year
			FROM TBL_PRODUCT_CATEGORY_FEE as PCF
			INNER JOIN TBL_PRODUCT_CATEGORY as PC
				USING(PRODUCT_CATEGORY_ID,CRSO_RELATION_ID)
			WHERE PC.CRSO_RELATION_ID = ?
				AND PC.PRODUCT_ID = ?
			ORDER BY CATEGORY_NAME, PCF.START_DATE
			";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$product_id);
		$q->execute();
		return $q;
	}

	public static function hasFeeAlready($db, $pc_id, $product_id, $crso_id, $start_date)
	{
		$q = $db->query(
				"SELECT FEE
				FROM TBL_PRODUCT_CATEGORY_FEE as PCF
				INNER JOIN TBL_PRODUCT_CATEGORY as PC
					ON PCF.PRODUCT_CATEGORY_ID = PC.PRODUCT_CATEGORY_ID
				WHERE START_DATE = ?
					AND PC.CRSO_RELATION_ID = ?
					AND PRODUCT_ID = ?
					AND PC.PRODUCT_CATEGORY_ID = ?"
			);
		$q->bind_values($start_date.' 00:00:00', $crso_id, $product_id, $pc_id);
		$q->execute();
		if($r = $q->fetch())
			return true;
		return false;
	}

	/**
	 * @access private
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_PRODUCT_CATEGORY_FEE';
		$this->objectmap['primary']['product_category->id'] = 'PRODUCT_CATEGORY_ID';
		$this->objectmap['primary']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['primary']['start_date'] = 'START_DATE';
		$this->objectmap['properties']['fee'] = 'FEE';
	}
}
?>
