<?php
/**
 * @package
 * @subpackage
 * @author Nick Loeve <nick@basesoft.com>
 * @since 16/03/2007
 * @copyright 2006 BaseSoft B.V.
 * $Id$
 */
require_once(ELC_CLASSES.'RefundRequestController.php');

class ProofOfExportController extends RefundRequestController
{
	static $instance;

	static public function instance(CRSOParticipantProduct $cpp, ELCUser $user, FormBehaviour $fb = null)
	{
		//based on crso-product we return the correct object to
		//call refund action on

		$db = IWRegistry::registry('db');

		$instance = null;

		if(!isset(self::$instance))
			self::$instance = $instance ? $instance : new ProofOfExportController($db, $cpp, $user, $fb);

		return self::$instance;
	}

	public function registerExportFormBehaviours()
	{
		$this->cpp->crso->relation->set();
		$this->cpp->crso->relation->address->set();
		$country = new Country($this->db, $this->cpp->crso->relation->address->country_id, true);

		$this->fb->addPlugin('elc_document_date', 'FormBehaviourPluginElcExporterDocumentdate');
		$this->fb->addPlugin('elc_not_future', 'FormBehaviourPluginElcNotFuture');
		$this->fb->addPlugin('elc_vat_block', 'FormBehaviourPluginElcVATBlock');
		$this->fb->addPlugin('elc_vat_block_by_id', 'FormBehaviourPluginElcVATBlockByID');
		$this->fb->addPlugin('elc_declaration_weight', 'FormBehaviourPluginElcDeclarationWeight');
		$this->fb->addPlugin('elc_declaration_weight_dec', 'FormBehaviourPluginElcDeclarationWeightDec');


		//basic form registration
		$this->fb->register('refund[export_country][id]', 'required', 'Item: ExportCountry');
		$this->fb->register('refund[importer_company_name]', 'required', 'Item: ImporterCompanyName');
		$this->fb->register('refund[importer_email]', 'required;email', 'Item: ImporterEmail');

		// importer VAT nr could be optional (#2556)
		$validation = sprintf('vat;elc_vat_block(%s, %s)',$this->cpp->crso->relation->id, CRSO_VAT_BLOCK_TYPE_RR_IMPORTER);
		if (!$country->has(HasOptionalRefundImporterVAT,$_SESSION['selected_product_id'])) {
			$validation = 'required;'.$validation;
		}
		$this->fb->register('refund[importer_vat_no]', $validation, 'Item: ImporterVAT');

		$this->fb->register('refund[identification_no]', 'required', 'Item: IdentificationNo');
		$this->fb->register('refund[office]', 'required', 'Item: Office');
		$this->fb->register('refund[commodity_code]', 'required', 'Item: CommodityCode');
		$this->fb->register('refund[export_document]', 'document', 'Item: ExportDocument');
		$this->fb->register('refund[refund_export_date]', 'required;date(YYYY-MM-DD);elc_not_future(YYYY-MM-DD);elc_document_date('.$this->cpp->crso->relation->id.',YYYY-MM-DD)', 'Item: ExportDate');
		if ($country->has(HasCreditViaProducer,$_SESSION['selected_product_id']))
      		$this->fb->register('producer[id]', 'required', 'Item: Producer');
	}

	public function getPoE($id = null, $properties = null, $lines = null, $countryId = null, $additionalDocs=null)
	{
		$rr  = new RefundRequest($this->db);

		if($id && !$rr->set($id))
			throw new ProofOfExportException(sprintf('Invalid RefundRequest: [%s]', $id ? $id : 'null'));

		//now we gotta feck with the cpp object
		if(!isset($this->cpp->crso->relation->id))
			$this->cpp->crso = $rr->crso;
		if(!isset($this->cpp->participant->relation->id))
			$this->cpp->participant = $rr->participant;
		if(!isset($this->cpp->product->id))
			$this->cpp->product = $rr->product;

		$this->checkAuthenticated();
		$this->checkRefundSettings();
		$this->checkClearance('ProofOfExport', ClearanceProofOfExport, $rr);


		$properties && $rr->setProperties($properties);

		if($lines)
		{
			$i=1;
			foreach ($lines as $line)
			{
				$rr->lines[$i] = new RefundRequestLine($this->db, $line);

				if (isset($line['producer_vat_no']))
				{
		    		if($relation_id = Relation::getByVAT($line['producer_vat_no'],$this->db))
		    			$rr->lines[$i]->producer_relation = new Relation($this->db, $relation_id);
		    		else
		    		{
		    			$rr->lines[$i]->producer_relation = new Relation($this->db);
		    			$rr->lines[$i]->producer_relation->vat_no = $line['producer_vat_no'];
		    		}
				}
				else if(isset($line['producer_relation']))
					$rr->lines[$i]->producer_relation = new Relation($this->db,$line['producer_relation']['id'] );

				$rr->lines[$i]->line_index = $i;
				$rr->lines[$i]->refund_request = $rr;
				$i++;
			}
		}

		// country can be changed in the form
		if($countryId)
			$rr->export_country = new Country($this->db, $countryId);

		$additional_documents = array();
		if($additionalDocs)
		{
			foreach($additionalDocs as $index => $posted_doc)
    		{
    			if(empty($posted_doc['invoice_no']))//empty first lines, means no docs
					continue;
			$it = new InterfaceItem(IWRegistry::registry('db'),IWRegistry::registry('db'),IWRegistry::registry('langIf'));
				$doc = $it->getDocument('additional_documents['.$index.'][invoice_document]', false, false);
    			$add_doc = new RefundRequestAdditionalDocument($this->db);
    			$add_doc->document = $doc;
    			$add_doc->invoice_number = $posted_doc['invoice_no'];
    			$add_doc->invoice_date = $posted_doc['invoice_date'];
    			$additional_documents[] = $add_doc;
    		}
		}

		return $rr;
	}

	public function proofOfExportEu($poeId, Array $refundLines, $exportDate, Country $exportCountry, $importerEmail, $importerVAT, $importerCompanyName, Array $additionalDocuments = null, $document = null)
	{
		$this->registerExportFormBehaviours();

		if ($exportCountry->region_id != REGION_EU)
			throw new ProofOfExportException('ExportCountry must be inside the EU');

		// fb checks
		$this->fb->arrValues['refund']['importer_company_name'] = $importerCompanyName;
		$this->fb->arrValues['refund']['importer_email'] = $importerEmail;
		$this->fb->arrValues['refund']['importer_vat_no'] = $importerVAT;
		$this->fb->unregister('refund[identification_no]');
		$this->fb->unregister('refund[office]');
		$this->fb->unregister('refund[commodity_code]');

		// Check if the EU form requires a document.
		$this->cpp->crso->relation->set();
		$this->cpp->crso->relation->address->set();
		$crso_country = new Country($this->db, $this->cpp->crso->relation->address->country_id, true);

		if (!$crso_country->has(HasRequiredPoEDocumentUploadEU,$_SESSION['selected_product_id'])) {
			$this->fb->unregister('refund[export_document]');
		} else {
			$this->fb->register('refund[export_document]', 'required;document', 'Item: ExportDocument');
			$this->fb->arrValues['refund']['export_document'] = $document?$document->id:'';
		}

		return $this->proofOfExport($poeId, $refundLines, $exportDate, $exportCountry, $importerEmail, $importerVAT, $importerCompanyName, DB_NULL, DB_NULL, DB_NULL, $document, $additionalDocuments);
	}

	public function proofOfExportNonEu($poeId, Array $refundLines, $exportDate, Country $exportCountry, $identificationNo, $office, $commodityCode, $document, Array $additionalDocuments = null)
	{
		$this->registerExportFormBehaviours();

		if ($exportCountry->region_id == REGION_EU)
			throw new ProofOfExportException('ExportCountry must be outside the EU');

		// fb checks
		$this->fb->arrValues['refund']['identification_no'] = $identificationNo;
		$this->fb->arrValues['refund']['office'] = $office;
		$this->fb->arrValues['refund']['commodity_code'] =  $commodityCode;
		if($document)
			$this->fb->arrValues['refund']['export_document'] =  $document->id;
		else
			$this->fb->unregister('refund[export_document]');
		$this->fb->unregister('refund[importer_company_name]');
		$this->fb->unregister('refund[importer_email]');
		$this->fb->unregister('refund[importer_vat_no]');

	 	return $this->proofOfExport($poeId, $refundLines, $exportDate, $exportCountry, DB_NULL, DB_NULL, DB_NULL, $identificationNo, $office, $commodityCode, $document, $additionalDocuments);
	}

	private function proofOfExport($poeId, Array $refundLines, $exportDate, Country $exportCountry, $importerEmail, $importerVAT, $importerCompanyName, $identificationNo, $office, $commodityCode, $document, Array $additionalDocuments = null)
	{

		$this->checkAuthenticated();
		$this->checkRefundSettings();
		$this->checkClearance('ProofOfExport', ClearanceProofOfExport);
		$this->checkProducerExporter('Exporter', PARTICIPANT_EXPORTER);

		// Formbehaviour
		$this->fb->arrValues['refund']['refund_export_date'] = $exportDate;
		$this->fb->arrValues['refund']['export_country']['id'] = $exportCountry->id;

		$this->cpp->crso->relation->set();
		$this->cpp->crso->relation->address->set();
		$crso_country = new Country($this->db, $this->cpp->crso->relation->address->country_id, true);

		if (sizeof($refundLines) == 0)
			throw new ProofOfExportException('You must supply at least one RefundLine');

		// Dynamic form behaviour for details
   		foreach ($refundLines as $key => $line)
   		{

			if ($crso_country->has(HasRefundInternalVAT,$_SESSION['selected_product_id'])) {
				$validation = sprintf('required;integer(1);elc_vat_block_by_id(%s, %s, %s)',$this->cpp->crso->relation->id, CRSO_VAT_BLOCK_TYPE_POE_PRODUCER, 'Producer blocked for PoE');
				$this->fb->register('line[' . $key . '][producer_relation][id]', $validation);
			} else {
				$this->fb->register('line[' . $key . '][producer_vat_no]', 'required;vat');
			}

   			$crso_country->has(HasDeclOfWeight,$_SESSION['selected_product_id']) && $this->fb->register('line['.$key.'][weight]', 'required;integer(0);elc_declaration_weight');
			$crso_country->has(HasDeclOfWeightWithDecimals,$_SESSION['selected_product_id']) && $this->fb->register('line['.$key.'][weight_dec]', 'required;elc_declaration_weight_dec');
   			$crso_country->has(HasDeclByBrand,$_SESSION['selected_product_id']) && $this->fb->register('line['.$key.'][brand]', 'required');
   			$crso_country->has(HasDeclByCategory,$_SESSION['selected_product_id']) && $this->fb->register('line['.$key.'][product_category]', 'required');

   			$this->fb->register('line['.$key.'][quantity]', 'required;integer(1)');
   		}

		// validate all refundLines
		foreach ($refundLines as $index=>$refundLine)
		{
			// check if the refundLines array contains valid objects
			if (get_class($refundLine) != 'RefundRequestLine')
				trigger_error(sprintf('refundLine[%s] is not a RefundRequestLine', $index), E_USER_ERROR);

			// Check if the producer has to be a participant for this country
   			if ($crso_country->has(HasRefundInternalVAT, $this->cpp->product->id))
   			{
   				// Check if $refundLine->producerVAT exists at all
				if (!$refundLine->producer_relation->setByVAT($refundLine->producer_relation->vat_no))
				{
					throw new ProofOfExportException(sprintf('You are obligated to supply a valid RefundLine->producerVAT: refundLine_%s_producerVAT [%s] does not exist',
						$index, $refundLine->producer_relation->vat_no));
				}
   				if($crso_country->has(HasCreditViaProducer,$_SESSION['selected_product_id']))
   					$this->fb->arrValues['producer']['id'] = $refundLine->producer_relation->id;

				// Check if the producer exists at $this->crso
   				$cpp = new CRSOParticipantProduct($this->db);
   				if (! ($cpp->set($this->cpp->crso->relation->id, $refundLine->producer_relation->id, $this->cpp->product->id) && $cpp->producer_exporter & PARTICIPANT_PRODUCER))
   				{
   					throw new ProofOfExportException(sprintf('You are obligated to supply a valid RefundLine->producerVAT: refundLine_%s_producerVAT [%s] is not a producer at current CRSO',
   						$index, $refundLine->producer_relation->vat_no));
   				}
   			}

   			// add a check for vat
			$this->fb->arrValues['line'][$index]['producer_vat_no'] = $refundLine->producer_relation->vat_no;
			$this->fb->arrValues['line'][$index]['producer_relation']['id'] = $refundLine->producer_relation->id;

  			// Add country-weight check
   			if ($crso_country->has(HasDeclOfWeight, $this->cpp->product->id))
				$this->fb->arrValues['line'][$index]['weight'] = $refundLine->weight;
   			else if (!empty($refundLine->weight))
   				throw new ProofOfExportException('Not needed to supply Weight for current CRSO / Country');

			// Add country-weight-as-dec check
			if ($crso_country->has(HasDeclOfWeightWithDecimals, $this->cpp->product->id))
				$this->fb->arrValues['line'][$index]['weight_dec'] = $refundLine->weight_dec;
			else if (!empty($refundLine->weight_dec))
				throw new ProofOfExportException('Not needed to supply Weight as decimals for current CRSO / Country');

  			// Add country-brand check
   			if ($crso_country->has(HasDeclByBrand, $this->cpp->product->id))
				$this->fb->arrValues['line'][$index]['brand'] = $refundLine->brand->id;
   			else if (!empty($refundLine->brand->id))
   				throw new ProofOfExportException('Not needed to supply Brand for current CRSO / Country');

  			// Add country-category check
   			if ($crso_country->has(HasDeclByCategory, $this->cpp->product->id))
				$this->fb->arrValues['line'][$index]['product_category'] = $refundLine->product_category->id;
   			else if (!empty($refundLine->product_category->id))
   				throw new ProofOfExportException('Not needed to supply ProductCategory for current CRSO / Country');

  			// Add quantity check
			$this->fb->arrValues['line'][$index]['quantity'] = $refundLine->quantity;
		}

		//additional documents
		if(!$crso_country->has(HasPoeAdditionalDocuments,$_SESSION['selected_product_id']) &&
		 	(!is_null($additionalDocuments) || count($additionalDocuments))
		)
		{
		 	debug('CRSO: '.print_r($this->cpp->crso->relation->id, true));
		 	debug('Participant: '.print_r($this->cpp->participant->relation->id, true));
		 	debug('Product: '.print_r($_SESSION['selected_product_id'], true));
		 	debug(var_export($crso_country->has(HasPoeAdditionalDocuments,$_SESSION['selected_product_id']), true));
		 	throw new ProofOfExportException('Not allowed to send additional documents for current CRSO / Country');
		}

		if($additionalDocuments)
		{
			foreach($additionalDocuments as $index => $value)
			{
				if(isset($_FILES['additional_documents'][$index]['invoice_document']))
				{
					$this->fb->register('additional_documents['.$index.'][invoice_document]', 'required(additional_documents['.$index.'][invoice_date],true);document');
					$this->fb->arrValues['additional_documents'][$index]['invoice_document'] = 1;
				}
				else
				{
					$this->fb->register('additional_documents['.$index.'][invoice_document]', 'required(additional_documents['.$index.'][invoice_date],true)');
					$this->fb->arrValues['additional_documents'][$index]['invoice_document'] = $value->document->id;
				}
				$this->fb->register('additional_documents['.$index.'][invoice_no]', 'required(additional_documents['.$index.'][invoice_document],true)');
				$this->fb->arrValues['additional_documents'][$index]['invoice_no'] = $value->invoice_number;
				$this->fb->register('additional_documents['.$index.'][invoice_date]', 'required(additional_documents['.$index.'][invoice_no],true)');
				$this->fb->arrValues['additional_documents'][$index]['invoice_date'] = $value->invoice_date;
			}
		}

		// validate values
		$this->fb->validateWithExceptions();

		// Start building the RefundRequest
		$rr = new RefundRequest($this->db);
		if($poeId)
			if(!$rr->set($poeId))
				throw new ProofOfExportException(sprintf('Invalid RefundRequest: [%s]', $poeId ? $poeId : 'null'));
		$rr->participant = $this->cpp->participant;
		$rr->crso = $this->cpp->crso;
		$rr->product = $this->cpp->product;
		$rr->status = REFUND_REQUEST_PENDING;
		$rr->export_country = $exportCountry;
		$rr->user = $this->user;
		$rr->refund_export_date = $exportDate;
    	$rr->importer_status = REFUND_IMPORTER_STATUS_PENDING;
    	$rr->importer_email = $importerEmail;
    	$rr->importer_vat_no = $importerVAT;
    	$rr->importer_company_name = $importerCompanyName;
    	$rr->identification_no = $identificationNo;
    	$rr->office = $office;
    	$rr->commodity_code = $commodityCode;

    	// authorisation type
		if ($rr->export_country->region_id == REGION_EU)
			$rr->authorisation_by_type = $this->cpp->authorize_refund_request_eu;
		else
			$rr->authorisation_by_type = $this->cpp->authorize_refund_request_non_eu;

		$this->db->begin();

		// if non-eu and we have a document, save it
		if ($document)
		{
			/**
			 * Here we check if the supplied document is different from the one
			 * in the database. If different we delete the old document
			 */
	    	if ($rr->export_document && $rr->export_document->id != $document->id)
		    	$rr->trashExportDocument();

    		$rr->export_document = $document;
    		$rd = new RelationDocument($this->db);
    		$rd->relation = $this->cpp->participant->relation;
    		$rd->document = $document;
    		$document->makePermanent();
			$rd->objectmap['inserttype'] = 'REPLACE';
			$rd->insert();
		}
		else
			$rr->export_document = DB_NULL;

		// insert the refund request
		if(!$poeId)
			$rr->insert();
		else
		{
			$rr->update();
			$rr->deleteLines(); //need to clear them out before resaving
		}

		// insert the lines
		foreach ($refundLines as $index => $refundLine)
		{
			// apearently all went good till here.. now lets see if the relation attached to the refundline is new then insert it
    		if($relation_id = Relation::getByVAT($refundLine->producer_relation->vat_no, $this->db))
    			$refundLine->producer_relation = new Relation($this->db, $relation_id);
    		else
    			$refundLine->producer_relation->insert();

			// start saving the refundline
			$refundLine->line_index = $index+1; // line indexes start from 1
			$refundLine->refund_request = $rr;
			$refundLine->insert();
		}

		//additional documents
		$docs = $poeId ? $rr->getAdditionalDocuments() : array();
		$reposted_docs = array();
		if($additionalDocuments)
		{
		    foreach($additionalDocuments as $posted_doc)
                    {
                            if(empty($posted_doc->invoice_number))//empty first lines, means no docs
                                    continue;
                            if(isset($posted_doc->document->id) && in_array($posted_doc->document->id, $docs))
                            {
                                    //skip it we have it linked already
                                    $reposted_docs[] = $posted_doc->document->id;
                                    continue;
                            }
                            //we have a new one
                            //save it

                            $posted_doc->document->makePermanent();
                            $posted_doc->refund_request = $rr;
                            $posted_doc->insert();
                            //make the relation document so they can access it
                            $posted_rd = new RelationDocument($this->db);
                            $posted_rd->relation = $rr->participant->relation;
                            $posted_rd->document = $posted_doc->document;
                            $posted_rd->objectmap['inserttype'] = 'REPLACE';
                                    $posted_rd->insert();

                    }
		}

		//remove docs not re-submitted
		$no_longer_needed = array_diff($docs, $reposted_docs);
		foreach($no_longer_needed as $delete)
		{
			RefundRequestAdditionalDocument::removeDocument($this->db, $delete);
		}

		$this->db->commit();

		// send importer authorisation email every time the PoE gets edited
		if($rr->authorisation_by_type==REFUND_AUTHORIZE_EMAIL || $rr->export_country->region_id == REGION_EU)
		{
			$external = new Relation($this->db);
			$external->email = $rr->importer_email;
			$external->name = $rr->importer_company_name;
			$external->setComplete();

			// generate a new token
			require_once(ELC_CLASSES.'RefundRequestImporterToken.php');
			$token = new RefundRequestImporterToken($this->db);
			$token->refund_request = $rr;
			$token->generate(); // this sets a new hash and sets a new expiry time
			$token->insert();

			require_once(IWARE.'encryption.php');
			$crypter = new Crypter(ENCRYPTION_KEY);
			$url = 'http://'.SITE_SERVER.SITE_PATH.'authorize.php?token='.urlencode($token->token);

			require_once(ELC_CLASSES.'ELCMessageHandler.php');
			MessageEvent::send(MESSAGE_REFUND_REQUEST,
				array(
					'external'=>$external, // used to send email,
					'participant'=>$rr->participant,
					'email_to_importer'=>
						array(
							'importer_name' => $external->name,
							'importer_country' => $rr->export_country->name,
							'confirmation_link' => $url
						)

				),
				'ELCMessageHandler');
		}

		return $rr;
	 }
}

/**
 * Exceptions
 *
 */
class ProofOfExportException extends WEEEException {};

?>
