<?php
require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES.'User.php');
/**
 * ProofOfPayment show the proof that the RefundRequester really bought/produced the $Product
 * @package ELC
 * @subpackage RefundRequest
 * @author Marten Koedam <marten@dalines.net>
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @version $Id$
 * @copyright 2006, BaseSoft B.V.
 */
class ProofOfPayment extends AbstractObject
{
	/**
	 * @var integer $id
	 */
	var $id;
	/**
	 * @var Participant $participant
	 */
	var $participant;
	/**
	 * @var CRSO $crso
	 */
	var $crso;
	var $product;
	/**
	 * @var Relation $producer_relation
	 */
	var $producer_relation;
	/**
	 * @var ELCDocument $document
	 */
	var $document;
	/**
	 * @var int $status
	 */
	var $status;
	/**
	 * @var MySQLDate $created
	 */
	var $created;
	/**
	 * @var MySQLDate $last_update
	 */
	var $last_update;
	/**
	 * @var string $remark
	 */
	var $remark;
	/**
	 * @var ELCUser $user
	 */
	var $user;
	/**
	 * @var ELCUser $user
	 */
	var $authorized_by;
	/**
	 * @var int authorisation_by_type
	 */
	var $authorisation_by_type;
	/**
	 * @var float $administrative_cost
	 */
	var $administrative_cost;
	/**
	 * @var Array $lines
	 */
	var $lines = array();
	/**
	 * @var ProofOfPaymentConfirmation $proof_of_payment_confirmation
	 */
	var $proof_of_payment_confirmation;
	/**
	 * @var Array
	 */
	var $supply_chain = array();

	/**
	 * Sets a ProofOfPayment by its id
	 * @param int $id
	 * @return bool
	 */
	function set($id=null)
	{
		$id || $id = $this->id;

		$sql = "SELECT PARTICIPANT_RELATION_ID, CRSO_RELATION_ID, PRODUCT_ID,
				`CREATED`, AUTHORISATION_BY_TYPE,
				PRODUCER_RELATION_ID, DOCUMENT_ID, `STATUS`, REMARK,
				LAST_UPDATE, USER_ID, AUTHORIZED_BY,
				ADMINISTRATIVE_COST, PROOF_OF_PAYMENT_CONFIRMATION_ID
			FROM TBL_PROOF_OF_PAYMENT
			WHERE PROOF_OF_PAYMENT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		$this->id = $id;

		require_once(ELC_CLASSES.'Participant.php');
		require_once(ELC_CLASSES.'CRSO.php');
		require_once(ELC_CLASSES.'Document.php');
		require_once(ELC_CLASSES.'Product.php');

		$this->participant = new Participant($this->db,array('relation'=> new Relation($this->db,$r->PARTICIPANT_RELATION_ID)));
		$this->crso = new CRSO($this->db,array('relation'=> new Relation($this->db,$r->CRSO_RELATION_ID)));
		$this->product = new Product($this->db,$r->PRODUCT_ID);

		$this->user = new ELCUser($this->db, $r->USER_ID);
		$this->authorized_by = new ELCUser($this->db, $r->AUTHORIZED_BY);
		$this->authorisation_by_type = $r->AUTHORISATION_BY_TYPE;
		$this->administrative_cost = $r->ADMINISTRATIVE_COST;

		$this->status = $r->STATUS;
		$this->remark = $r->REMARK;

		$this->created = $r->CREATED;
		$this->last_update = $r->LAST_UPDATE;

		$this->producer_relation = new Relation($this->db,$r->PRODUCER_RELATION_ID);


		if($r->DOCUMENT_ID)
		{
			$this->document = Document::createDocument($this->db);
			$this->document->id = $r->DOCUMENT_ID;
		}

		if($r->PROOF_OF_PAYMENT_CONFIRMATION_ID)
		{
			require_once(ELC_CLASSES.'ProofOfPaymentConfirmation.php');
			$this->proof_of_payment_confirmation = new ProofOfPaymentConfirmation($this->db,$r->PROOF_OF_PAYMENT_CONFIRMATION_ID);
		}

		// Set the lines
		$this->lines = array();
		$pl = new ProofOfPaymentLine($this->db);
		$q = $pl->getLines($id);
		while($r = $q->fetch()) {
			$this->lines[$r->LINE_INDEX] = new ProofOfPaymentLine($this->db,array('proof_of_payment'=>$this,'line_index'=>$r->LINE_INDEX), true);
		}

		// Set the supply chain
		$sp = new ProofOfPaymentSupplyChain($this->db);
		$q = $sp->getSupplyChain($id);
		while($r = $q->fetch())
		{
			$this->supply_chain[$r->SUPPLY_CHAIN_INDEX] = new ProofOfPaymentSupplyChain($this->db,
				array('pop'=>$this, 'supply_chain_index'=>$r->SUPPLY_CHAIN_INDEX), true);
		}

		return $this->complete = true;
	}

	/**
	 * Selects the fields used in the overview.
	 * @param int $participant_id
	 * @param int $crso_id
	 */
	function getOverview($participant_id='',$crso_id='', $product_id=null, $filter='')
	{
		$participant_id || $participant_id = $this->participant->relation->id;
		$crso_id || $crso_id = $this->crso->relation->id;
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT PoP.PROOF_OF_PAYMENT_ID, PoP.`CREATED`, INVOICE_ID,  PRODUCER_RELATION_ID,'' as COMPANY_NAME,
				'' as VAT_NO, PAYMENT_DATE as INVOICE_DATE, SUM(QUANTITY) as QUANTITY, FEE,
				SUM(QUANTITY*FEE) as AMOUNT, REMARK, STATUS, DOCUMENT_ID as DOCUMENT
				FROM TBL_PROOF_OF_PAYMENT as PoP
				INNER JOIN TBL_PROOF_OF_PAYMENT_LINE PoP_Line
					ON PoP.PROOF_OF_PAYMENT_ID = PoP_Line.PROOF_OF_PAYMENT_ID
				WHERE CRSO_RELATION_ID = ?
					AND PARTICIPANT_RELATION_ID = ?
					AND PRODUCT_ID = ?
					".$filter."
				GROUP BY PoP.PROOF_OF_PAYMENT_ID
				ORDER BY `STATUS`, `PAYMENT_DATE`
				";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$participant_id, $product_id);
		$q->execute();
		return $q;
	}

	/**
	 * Finds any invoice/id combination in pop lines (optional: excluding the given pop_id)
	 * @param int $invoice_id
	 * @param string $vat_no
	 * @return bool
	 */
	static function isUnique($db, $invoice_id, $vat_no, $pop_id=false)
	{
		$sql = "
			SELECT pop.PROOF_OF_PAYMENT_ID
			FROM TBL_PROOF_OF_PAYMENT as pop
			INNER JOIN TBL_PROOF_OF_PAYMENT_LINE as popline
				ON pop.PROOF_OF_PAYMENT_ID = popline.PROOF_OF_PAYMENT_ID
			INNER JOIN TBL_RELATION as R
				ON pop.PRODUCER_RELATION_ID = R.RELATION_ID
			WHERE
				popline.INVOICE_ID = ?
				AND R.VAT_NO = ?
			";

		$params = array($invoice_id, $vat_no);

		if ($pop_id) {
			$sql .= "AND pop.PROOF_OF_PAYMENT_ID != ?";
			$params[] = $pop_id;
		}

		$q = $db->query($sql);
		$q->bind_values($params);
		$q->execute();

		return !((bool) ($r = $q->fetch()));
	}

	/**
	 *
	 */
	function getDetails($id='')
	{

	}

	/**
	 * Selects all Participants that have pops. A crso filter can be added
	 * @param int $crso_id
	 * @return ResultSet $q
	 */
	function selectParticipants($crso_id='', $product_id='')
	{
		$product_id || $product_id = $this->product->id;

		$bind_values = array($product_id,PROOF_OF_PAYMENT_PENDING);
		$sql = "SELECT DISTINCT pop.PARTICIPANT_RELATION_ID, r.NAME
			FROM TBL_PROOF_OF_PAYMENT pop
			INNER JOIN TBL_RELATION r
				ON pop.PARTICIPANT_RELATION_ID = r.RELATION_ID
			WHERE PRODUCT_ID = ?
				AND pop.STATUS = ?
		";

		if ($crso_id) {
			$sql .= " AND CRSO_RELATION_ID=? ";
			$bind_values[] = $crso_id;
		}

		$sql .= " GROUP BY pop.PARTICIPANT_RELATION_ID ";

		$q = $this->db->query($sql);
		$q->bind_values($bind_values);
		$q->execute();
		return $q;
	}

	/**
	 * Selects an overview for proof of payment authorize.. used in the common interface authorize
	 * @param array $authorize_type (REFUND_AUTHORIZE_GENERAL_MANAGER | REFUND_AUTHORIZE_STATUTORY_AUDITOR | REFUND_AUTHORIZE_OPERATOR)
	 * @param int $authorisation_status (Not used in payment but only in refund request which uses same arguments)
	 * @param int crso_id
	 * @param int participant_id
	 */
	function selectAuthorizeOverview($authorize_type, $authorisation_status, $crso_id, $participant_id, $product_id)
	{
		$bind_values = array();
		$sql = "SELECT '' AS AUTHORIZE_CHECKBOX, pop.PROOF_OF_PAYMENT_ID, INVOICE_ID,
				r_pa.NAME as PARTICIPANT_COMPANY_NAME,
				-- r_pa.VAT_NO as PARTICIPANT_VAT_NO,
				r_cr.NAME as CRSO_COMPANY_NAME,
				-- r_cr.VAT_NO as CRSO_VAT_NO,
				r_pr.NAME as PRODUCER_COMPANY_NAME,
				-- r_pr.VAT_NO as PRODUCER_VAT_NO,
				PAYMENT_DATE as INVOICE_DATE,
				DOCUMENT_ID as DOCUMENT,
				pop.`CREATED`,
				SUM(pop_line.`QUANTITY`) AS QUANTITY,
				FEE,
				SUM(pop_line.`QUANTITY` * pop_line.`FEE`) as AMOUNT,
				pop.AUTHORISATION_BY_TYPE
				FROM TBL_PROOF_OF_PAYMENT as pop
				INNER JOIN TBL_PROOF_OF_PAYMENT_LINE pop_line
					ON pop.PROOF_OF_PAYMENT_ID = pop_line.PROOF_OF_PAYMENT_ID
				INNER JOIN TBL_RELATION r_pa
					ON pop.PARTICIPANT_RELATION_ID=r_pa.RELATION_ID
				INNER JOIN TBL_RELATION r_cr
					ON pop.CRSO_RELATION_ID=r_cr.RELATION_ID
				INNER JOIN TBL_RELATION r_pr
					ON pop.PRODUCER_RELATION_ID=r_pr.RELATION_ID
				WHERE pop.STATUS=?
					AND pop.AUTHORISATION_BY_TYPE in (".join($authorize_type, ',').")
					AND pop.CRSO_RELATION_ID=? ";
			if(is_array($participant_id))
				$sql .= " AND pop.PARTICIPANT_RELATION_ID IN (".implode(',',$participant_id).") ";
			else
				$sql .= " AND pop.PARTICIPANT_RELATION_ID=".((int)$participant_id)." ";
			$sql .= " AND pop.PRODUCT_ID=?
				GROUP BY pop.PROOF_OF_PAYMENT_ID
				ORDER BY r_pa.`NAME`, pop.`STATUS`, `PAYMENT_DATE`
		";

		$bind_values[] = PROOF_OF_PAYMENT_PENDING;
		$bind_values[] = $crso_id;
		//$bind_values[] = $participant_id;
		$bind_values[] = $product_id;

		$q = $this->db->query($sql);
		$q->bind_values($bind_values);
		$q->execute();
		return $q;
	}

	/**
	 * Translates the status from the define
	 * @param LanguageInterface $langIf
	 * @param int $status
	 * @return array (TranslatedTitle,CSSClassName)
	 */
	public static function translateStatus($langIf,$status)
	{
		switch($status)
		{
			case PROOF_OF_PAYMENT_PENDING:
				return array($langIf->getText('Registration Pending'), 'status_new');
			break;
			case PROOF_OF_PAYMENT_APPROVED:
				return array($langIf->getText('Registration Approved'), 'status_verified');
			break;
			case PROOF_OF_PAYMENT_REJECTED:
				return array($langIf->getText('Registration Rejected'), 'status_forfeit');
			break;
		}
	}
	/**
	 * Set the creation date
	 */
	function insert()
	{
		$date = currentDate();
		$this->created = $date->format('Y-m-d H:i');

		$rv = parent::insert();
		info(sprintf('Proof of Payment [%s] inserted', $this->id));
		return $rv;
	}

	/**
	 * Inserts the supply chain
	 * @param array $lines
	 */
	/*
	function insertSupplyChain($supply_chain)
	{
		$this->setSupplyChain($supply_chain, true);
	}
	*/

	/**
	 * Set all the supply_chain lines for the pop, or insert if $save = true
	 * @param array $supply_chain
	 * @param bool $save
	 */
	/*
	function setSupplyChain($supply_chain, $save=false)
	{
		$i=1;
		foreach ($supply_chain as $line)
		{
			$this->supply_chain[$i] = new ProofOfPaymentSupplyChain($this->db, $line);

			$this->supply_chain[$i]->supply_chain_index = $i;
			$this->supply_chain[$i]->pop = &$this;
			$save && $this->supply_chain[$i]->insert();
			$i++;
		}
	}
	*/

	/**
	 * Updates the supplychain for a proof of payment by deleting them all and then re-add them
	 * @param array $supply_chain
	 */
	/*
	function updateSupplyChain($supply_chain)
	{
		$this->deleteSupplyChain();
		$this->insertSupplyChain($supply_chain);
	}
	*/
	/**
	 * Delete all the lines for the current proof of payment
	 * @return ResultSet $q ?
	 */
	function deleteSupplyChain()
	{
		$sql = "DELETE FROM TBL_PROOF_OF_PAYMENT_SUPPLY_CHAIN WHERE PROOF_OF_PAYMENT_ID=?";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		return $q;
	}

	/**
	 * Updates a ProofOfPayment, notify the Logs
	 * @return bool
	 */
	function update()
	{
		$rv = parent::update();
		info(sprintf('Proof of Payment [%s] updated', $this->id));
		return $rv;
	}
	/**
	 * Deletes a ProofOfPayment
	 * @return bool
	 */
	function delete()
	{
		if(!$this->isDeletable())
			return false;
		parent::delete();
		if(isset($this->document)){
		  $this->document->set();
		  $this->document->delete();
		}
		return true;
	}
	/**
	 * ProofOfPayments can only be deleted while pending
	 *
	 * Rejected Payments can never be deleted since they have been reviewed
	 * by the Operator which brings in Administrative Costs for the document.
	 * @return bool
	 */
	function isDeletable()
	{
		if(!$this->isComplete())
			$this->set();

		return ($this->status == PROOF_OF_PAYMENT_PENDING);
		//|| $this->status == PROOF_OF_PAYMENT_REJECTED);
	}

	/**
	 * Create the exported_amount field in the db, so we get good FP precision
	 */
	function saveExportedAmount()
	{
		if(!$this->isComplete())
		{
			if(!$this->set())
				return false;
		}

		$sql = "
			UPDATE TBL_PROOF_OF_PAYMENT_LINE
			SET EXPORTED_AMOUNT = QUANTITY * FEE
			WHERE PROOF_OF_PAYMENT_ID=?";

		$q = $this->db->query($sql);
		$q->bind_values($this->id, $line_index);
		if(!$q->execute())
			return false;
		return true;
	}

	function getProducers($crso_id, $part_id, $status=PROOF_OF_PAYMENT_APPROVED, $product_id)
	{
		$product_id || $product_id = $this->product->id;

		$rv = array();
		$sql = '
				SELECT DISTINCT pop.PRODUCER_RELATION_ID
				FROM
					TBL_PROOF_OF_PAYMENT as pop
				WHERE
					CRSO_RELATION_ID = ?
				AND
					PARTICIPANT_RELATION_ID = ?
				AND
					`STATUS` = ?
				AND PRODUCT_ID = ?
				';
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, $part_id, $status,$product_id);
		$q->execute();
		while($r = $q->fetch())
			$rv[] = $r->PRODUCER_RELATION_ID;
		return $rv;
	}

	/**
	 * gets the count of proofofpayments for the given arguments
	 */
	function getCount($crso_id, $part_id, $product_id, $status, $producer_id=null)
	{
		$sql = 'SELECT DISTINCT COUNT(pop.PROOF_OF_PAYMENT_ID) as cnt
				FROM TBL_PROOF_OF_PAYMENT as pop
				WHERE
					CRSO_RELATION_ID = ?
				AND
					PARTICIPANT_RELATION_ID = ?
				AND
					PRODUCT_ID = ?
				AND
					`STATUS` = ?
		';

		$values = array($crso_id, $part_id,$product_id, $status);

		if ($producer_id) {
			$sql .= ' AND PRODUCER_RELATION_ID = ?';
			$values[] = $producer_id;
		}

		$q = $this->db->query($sql);
		$q->bind_values($values);
		$q->execute();
		$r = $q->fetch();

		return $r->{'cnt'};
	}

	function deleteLines()
	{
		$sql = "DELETE FROM TBL_PROOF_OF_PAYMENT_LINE WHERE PROOF_OF_PAYMENT_ID=?";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		return $q->rowCount();
	}

		/**
	 *  @access private
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_PROOF_OF_PAYMENT';
		$this->objectmap['primary']['id'] = 'PROOF_OF_PAYMENT_ID';
		$this->objectmap['properties']['participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
		$this->objectmap['properties']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['product->id'] = 'PRODUCT_ID';

		$this->objectmap['properties']['producer_relation->id'] = 'PRODUCER_RELATION_ID';
		$this->objectmap['properties']['document->id'] = 'DOCUMENT_ID';

		$this->objectmap['properties']['status'] = 'STATUS';
		$this->objectmap['properties']['remark'] = 'REMARK';
		$this->objectmap['properties']['created'] = 'CREATED';
		$this->objectmap['properties']['last_update'] = 'LAST_UPDATE';

		$this->objectmap['properties']['user->id'] = 'USER_ID';
		$this->objectmap['properties']['administrative_cost'] = 'ADMINISTRATIVE_COST';

		$this->objectmap['properties']['proof_of_payment_confirmation->id'] = 'PROOF_OF_PAYMENT_CONFIRMATION_ID';
		$this->objectmap['properties']['authorized_by->id'] = 'AUTHORIZED_BY';
		$this->objectmap['properties']['authorisation_by_type'] = 'AUTHORISATION_BY_TYPE';
	}
}



/**
 * ProofOfPaymentLine
 */
class ProofOfPaymentLine extends AbstractObject
{
	/**
	 * @var ProofOfPayment
	 */
	var $proof_of_payment;
	/**
	 * @var int $line_index
	 */
	var $line_index;
	/**
	 * @var int $quantity
	 */
	var $quantity;
	/**
	 * @var float $fee
	 */
	var $fee;
	/**
	 * @var invoice_id
	 */
	var $invoice_id;
	/**
	 * @var MySQLDate $payment_date
	 */
	var $payment_date;
	/**
	 * @var int $exported_amount
	 */
	var $exported_amount;
	/**
	 * @var MySQLDate $created
	 */
	var $created;
	/**
	 * @var MySQLDate $last_update
	 */
	var $last_update;

	/**
	 * @param int $id
	 * @param int $index
	 */
	function set($id='',$index='')
	{
		$id || $id = $this->proof_of_payment->id;
		$index || $index = $this->line_index;

		$sql = "SELECT QUANTITY, FEE, INVOICE_ID, PAYMENT_DATE, EXPORTED_AMOUNT, CREATED, LAST_UPDATE
			FROM TBL_PROOF_OF_PAYMENT_LINE
			WHERE PROOF_OF_PAYMENT_ID = ?
				AND LINE_INDEX = ?";

		$q = $this->db->query($sql);
		$q->bind_values($id, $index);
		if(!$q->execute(true))
			return false; // Trigger Error
		if(!($r = $q->fetch()))
			return false;

		$this->line_index = $index;
		$this->quantity = $r->QUANTITY;
		$this->fee = $r->FEE;
		$this->invoice_id = $r->INVOICE_ID;
		$this->payment_date = $r->PAYMENT_DATE;
		$this->exported_amount = $r->EXPORTED_AMOUNT;
		$this->created = $r->CREATED;
		$this->last_update = $r->LAST_UPDATE;

	}

	function insert()
	{
		$date = currentDate();
		$this->created = $date->format('Y-m-d H:i');

		$rv = parent::insert();
		info(sprintf('Proof of Payment Line [%s/%s] inserted', $this->proof_of_payment->id, $this->line_index));
		return $rv;
	}

	/**
	 * Returns all the lines for the gven PoP.id
	 * @param int $id
	 */
	function getLines($id)
	{
		$sql = "SELECT LINE_INDEX
			FROM TBL_PROOF_OF_PAYMENT_LINE
			WHERE PROOF_OF_PAYMENT_ID = ?
			ORDER BY LINE_INDEX";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		return $q;
	}

	// FIXME: move this to the ELCSoapHandler
	public static function fromSoapProofOfPaymentLine($db, $line)
	{
		$obj = new ProofOfPaymentLine($db);

		$obj->fee = $line->fee;
		$obj->quantity = $line->quantity;
		$obj->invoice_id = $line->invoiceId;
		$obj->payment_date = $line->paymentDate ;

		return $obj;
	}

	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_PROOF_OF_PAYMENT_LINE';
		$this->objectmap['primary']['proof_of_payment->id'] = 'PROOF_OF_PAYMENT_ID';
		$this->objectmap['primary']['line_index'] = 'LINE_INDEX';
		$this->objectmap['properties']['quantity'] = 'QUANTITY';
		$this->objectmap['properties']['fee'] = 'FEE';
		$this->objectmap['properties']['invoice_id'] = 'INVOICE_ID';
		$this->objectmap['properties']['payment_date'] = 'PAYMENT_DATE';
		$this->objectmap['properties']['exported_amount'] = 'EXPORTED_AMOUNT';
		$this->objectmap['properties']['created'] = 'CREATED';
		$this->objectmap['properties']['last_update'] = 'LAST_UPDATE';
	}
}

/**
 * ProofOfPaymentSupplyChain
 */
class ProofOfPaymentSupplyChain extends AbstractObject
{
	var $pop;
	var $supply_chain_index;
	var $company_name;
	var $vat_no;
	var $invoice_no;
	var $comment;

	function set($supply_chain_index='', $pop_id='')
	{
		$pop_id || $pop_id = $this->pop->id;
		$supply_chain_index || $supply_chain_index = $this->supply_chain_index;

		$sql = "SELECT COMPANY_NAME, VAT_NO, INVOICE_NO, COMMENT
			FROM TBL_PROOF_OF_PAYMENT_SUPPLY_CHAIN
			WHERE PROOF_OF_PAYMENT_ID = ?
				AND SUPPLY_CHAIN_INDEX = ?";

		$q = $this->db->query($sql);
		$q->bind_values($pop_id, $supply_chain_index);
		if(!$q->execute())
			return false; // Trigger Error
		if(!($r = $q->fetch()))
			return false;


		//$this->pop = new ProofOfPayment($this->db, $pop_id);
		$this->supply_chain_index = $supply_chain_index;

		$this->company_name = $r->COMPANY_NAME;
		$this->vat_no = $r->VAT_NO;
		$this->invoice_no = $r->INVOICE_NO;
		$this->comment = $r->COMMENT;

		return $this->complete = true;
	}


	/**
	 * Gets the whole chain for the given pop_id
	 * @param int $refund_request_id
	 * @return ResultSet $q
	 */
	function getSupplyChain($pop_id)
	{
		$sql = "SELECT SUPPLY_CHAIN_INDEX
			FROM TBL_PROOF_OF_PAYMENT_SUPPLY_CHAIN
			WHERE PROOF_OF_PAYMENT_ID = ?
			ORDER BY SUPPLY_CHAIN_INDEX";
		$q = $this->db->query($sql);
		$q->bind_values($pop_id);
		$q->execute();
		return $q;
	}

	public static function fromSoapSupplier($db, $soapSupplier)
	{
		$inst = new ProofOfPaymentSupplyChain($db);
		$inst->company_name = $soapSupplier->companyName;
		$inst->vat_no = $soapSupplier->vatNo;
		$inst->invoice_no = $soapSupplier->invoiceNo;
		$inst->comment = $soapSupplier->comment;
		return $inst;
	}

	/**
	 * Initializes the objectmaps used in AbstractObject
	 * @access protected
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_PROOF_OF_PAYMENT_SUPPLY_CHAIN';
		$this->objectmap['primary']['pop->id'] = 'PROOF_OF_PAYMENT_ID';
		$this->objectmap['primary']['supply_chain_index'] = 'SUPPLY_CHAIN_INDEX';

		$this->objectmap['properties']['company_name'] = 'COMPANY_NAME';
		$this->objectmap['properties']['vat_no'] = 'VAT_NO';
		$this->objectmap['properties']['invoice_no'] = 'INVOICE_NO';
		$this->objectmap['properties']['comment'] = 'COMMENT';
	}

}




/**#@+
 * Defines of the different Proof of Payment stati
 */
define('PROOF_OF_PAYMENT_PENDING',1);
define('PROOF_OF_PAYMENT_APPROVED',2);
define('PROOF_OF_PAYMENT_REJECTED',3);
/**#@-*/
?>