<?php
require_once(IWARE.'AbstractObject.php');
/**
 *
 *
 * @package WEEE
 * @subpackage Refund
 * @since 9-jan-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class ProofOfPaymentConfirmation extends AbstractObject
{
	/**#@+
	 * @access public
	 */
	public $id;
	public $document;
	public $product;
	public $crso;
	public $participant;
	public $review_status;
	public $process_status;
	public $auth_by_type;
	public $created;
	public $last_update;
	/**#@-*/
	/**
	 * @param int $id
	 * @return bool
	 */
	function set($id=null)
	{
		$id || $id = $this->id;
		$sql = "SELECT DOCUMENT_ID, PRODUCT_ID, CRSO_RELATION_ID, PARTICIPANT_RELATION_ID,
				REVIEW_STATUS, PROCESS_STATUS, AUTHORISATION_BY_TYPE,
				CREATED, LAST_UPDATE
			FROM TBL_PROOF_OF_PAYMENT_CONFIRMATION
			WHERE PROOF_OF_PAYMENT_CONFIRMATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		$this->id = $id;
		require_once(ELC_CLASSES.'Document.php');
		$this->document = new ELCDocument($this->db,$r->DOCUMENT_ID);
		$this->product = new Product($this->db,$r->PRODUCT_ID);
		$this->crso = new CRSO($this->db,array('relation'=>new Relation($this->db,$r->CRSO_RELATION_ID)));
		$this->participant = new Participant($this->db,array('relation'=>new Relation($this->db,$r->PARTICIPANT_RELATION_ID)));
		$this->review_status = $r->REVIEW_STATUS;
		$this->process_status = $r->PROCESS_STATUS;
		$this->auth_by_type = $r->AUTHORISATION_BY_TYPE;
		$this->created = $r->CREATED;
		$this->last_update = $r->LAST_UPDATE;

		return $this->complete = true;
	}

	function insert()
	{
		$this->created = $this->last_update =date('Y-m-d H:i');
		$this->review_status = POP_CONFIRMATION_UNREVIEWED;
		$this->process_status = POP_CONFIRMATION_OPEN;
		parent::insert();
	}

	function update()
	{
		$this->last_update = date('Y-m-d H:i');
		parent::update();
	}

	function getLatestOverview($auth_by,$crso_id=null,$participant_id=null,$product_id=null, $confirmation_id=null)
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$participant_id || $participant_id = $this->participant->relation->id;
		$product_id || $product_id = $this->product->id;
		$auth_by = (array) $auth_by;
		// Check for unprocessed PoPC's
		$confirmation_id || $confirmation_id = $this->getUnprocessedConfirmation($auth_by,$crso_id,$participant_id,$product_id);

		$bind_values = array($product_id,$crso_id,$participant_id, PROOF_OF_PAYMENT_REJECTED);
		if($confirmation_id)
			$bind_values[] = $confirmation_id;

		$sql = "
			SELECT
				pop.PROOF_OF_PAYMENT_ID as CHECKBOX,
				pop.PROOF_OF_PAYMENT_ID,
				popline.INVOICE_ID,
				p.`NAME` as PRODUCER,
				p.VAT_NO as PRODUCER_VAT_NO,
				SUM(popline.`QUANTITY`) AS QUANTITY,
				popline.`FEE`,
				SUM(popline.`QUANTITY` * popline.`FEE`) as AMOUNT,
				`STATUS`,
				popline.PAYMENT_DATE
			FROM TBL_PROOF_OF_PAYMENT as pop
			INNER JOIN TBL_PROOF_OF_PAYMENT_LINE popline
				ON pop.PROOF_OF_PAYMENT_ID = popline.PROOF_OF_PAYMENT_ID
			INNER JOIN TBL_RELATION as p
				ON pop.PRODUCER_RELATION_ID = p.RELATION_ID
			WHERE PRODUCT_ID = ?
				AND CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ?
				AND pop.`STATUS` != ?
				AND AUTHORISATION_BY_TYPE IN(".implode(',',$auth_by).")
				AND (ISNULL(PROOF_OF_PAYMENT_CONFIRMATION_ID)
		";
		if($confirmation_id)
			$sql .= " OR PROOF_OF_PAYMENT_CONFIRMATION_ID = ? ";
		$sql .= ")";

		$sql .="
			GROUP BY popline.PROOF_OF_PAYMENT_ID
			ORDER BY PAYMENT_DATE";
		$q = $this->db->query($sql);
		$q->bind_values($bind_values);
		$q->execute();
		return $q;
	}

	function getOperatorOverview($review_status, $crso_id)
	{
		$sql = "
			SELECT
				popc.PROOF_OF_PAYMENT_CONFIRMATION_ID as CHECKBOX,
				popc.PROOF_OF_PAYMENT_CONFIRMATION_ID,
				popc.DOCUMENT_ID as PoPCofirmationDocument,
				'' as CHECKSUM,
				r.`NAME` as COMPANY_NAME,
				r1.`NAME` as CRSO_NAME,
				SUM(popline.QUANTITY) as QUANTITY,
				popc.LAST_UPDATE
			FROM TBL_PROOF_OF_PAYMENT_CONFIRMATION as popc
			INNER JOIN TBL_PROOF_OF_PAYMENT as pop
				ON popc.PROOF_OF_PAYMENT_CONFIRMATION_ID = pop.PROOF_OF_PAYMENT_CONFIRMATION_ID
			INNER JOIN TBL_PROOF_OF_PAYMENT_LINE popline
				ON pop.PROOF_OF_PAYMENT_ID = popline.PROOF_OF_PAYMENT_ID
			INNER JOIN TBL_RELATION as r
				ON popc.PARTICIPANT_RELATION_ID = r.RELATION_ID
			INNER JOIN TBL_RELATION as r1
				ON popc.CRSO_RELATION_ID = r1.RELATION_ID
			WHERE popc.REVIEW_STATUS = ?
				AND popc.CRSO_RELATION_ID = ?
			GROUP BY popc.PROOF_OF_PAYMENT_CONFIRMATION_ID
			ORDER BY popc.LAST_UPDATE";
		$q = $this->db->query($sql);
		$q->bind_values($review_status, $crso_id);
		$q->execute();
		return $q;
	}

	/**
	 * @param int $crso_id
	 * @param int $participant_id
	 * @param int $product_id
	 * @return bool|int
	 */
	function getUnprocessedConfirmation($auth_by,$crso_id=null,$participant_id=null,$product_id=null)
	{
		$crso_id || $crso_id = $this->crso->relation->id;
		$participant_id || $participant_id = $this->participant->relation->id;
		$product_id || $product_id = $this->product->id;
		$auth_by = (array) $auth_by;

		$sql = "SELECT PROOF_OF_PAYMENT_CONFIRMATION_ID as id
			FROM TBL_PROOF_OF_PAYMENT_CONFIRMATION
			WHERE PRODUCT_ID = ?
				AND CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ?
				AND AUTHORISATION_BY_TYPE IN(".implode(',',$auth_by).")
				AND PROCESS_STATUS = ?";
		$q = $this->db->query($sql);
		$q->bind_values($product_id,$crso_id,$participant_id,POP_CONFIRMATION_OPEN);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		return $r->id;
	}

	public function nullifyPoPConfirmationId($id=null)
	{
		$id || $id = $this->id;
		if(!is_numeric($id))
			return false;

		$sql = "UPDATE TBL_PROOF_OF_PAYMENT
			SET PROOF_OF_PAYMENT_CONFIRMATION_ID = NULL
			WHERE PROOF_OF_PAYMENT_CONFIRMATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		return $q->execute();
	}

	public function getPopsByConfirmation($id=null,$return='q')
	{
		$id || $id = $this->id;
		$sql = "SELECT PROOF_OF_PAYMENT_ID FROM TBL_PROOF_OF_PAYMENT
			WHERE PROOF_OF_PAYMENT_CONFIRMATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if($return=='q')
			return $q;

		$rv = array();
		while($r = $q->fetch())
			$rv[] = $r->PROOF_OF_PAYMENT_ID;
		return $rv;
	}

	/**
	 * Will generate a checksum for confirmation documents
	 *
	 * @access public static
	 * @param array $popIds The proof of payment ids that you want to generate a checksum for
	 * @return string An md5 checksum
	 */
	static public function generateConfirmProofOfPaymentChecksum($popIds)
	{
		//we need this sort because the ids may be given to us in a different order
		array_walk($popIds, array('ProofOfPaymentConfirmation', '_forceNumeric'));
		sort($popIds, SORT_NUMERIC);

		$documentId = '';
		foreach($popIds as $approvedId)
			$documentId .= CONFIRM_PROOF_OF_PAYMENT_SALT.$approvedId;
		return md5($documentId);
	}

	/**
	 * * @access private static
	 */
	static private function _forceNumeric(&$value)
	{
		settype($value, 'integer');
	}

	static public function translateStatus($langIf,$status)
	{
		switch($status)
		{
			case POP_CONFIRMATION_UNREVIEWED:
				return array($langIf->getText('Registration Pending'), 'status_new');
			break;
			case POP_CONFIRMATION_APPROVED:
				return array($langIf->getText('Registration Approved'), 'status_verified');
			break;
			case POP_CONFIRMATION_REJECTED:
				return array($langIf->getText('Registration Rejected'), 'status_forfeit');
			break;
		}
	}

    function getAuthorizedById($id=null)
    {
    	$id || $id = $this->id;
    	$sql = "SELECT DISTINCT( AUTHORIZED_BY ) as USER_ID
			FROM TBL_PROOF_OF_PAYMENT
			WHERE PROOF_OF_PAYMENT_CONFIRMATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		return $r->USER_ID;
    }

	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_PROOF_OF_PAYMENT_CONFIRMATION';
		$this->objectmap['primary']['id'] = 'PROOF_OF_PAYMENT_CONFIRMATION_ID';
		$this->objectmap['properties']['document->id'] = 'DOCUMENT_ID';
		$this->objectmap['properties']['product->id'] = 'PRODUCT_ID';
		$this->objectmap['properties']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
		$this->objectmap['properties']['review_status'] = 'REVIEW_STATUS';
		$this->objectmap['properties']['process_status'] = 'PROCESS_STATUS';
		$this->objectmap['properties']['auth_by_type'] = 'AUTHORISATION_BY_TYPE';
		$this->objectmap['properties']['created'] = 'CREATED';
		$this->objectmap['properties']['last_update'] = 'LAST_UPDATE';
	}

	public static function closeOpenPaymentConfirmations($db,$crso_id,$product_id)
	{
		$sql = "UPDATE TBL_PROOF_OF_PAYMENT_CONFIRMATION SET PROCESS_STATUS = ?
			WHERE CRSO_RELATION_ID = ?
				AND PRODUCT_ID = ?";
		$q = $db->query($sql);
		$q->bind_values(POP_CONFIRMATION_CLOSED,$crso_id,$product_id);
		$q->execute();
	}
}
/**#@+
 * Confirmation Review Stati
 */
define('POP_CONFIRMATION_UNREVIEWED',0);
define('POP_CONFIRMATION_APPROVED',1);
define('POP_CONFIRMATION_REJECTED',2);
/**#@-*/
/**#@+
 * Confirmation Process Stati
 */
define('POP_CONFIRMATION_OPEN',0);
define('POP_CONFIRMATION_CLOSED',1);
/**#@-*/
?>
