<?php
require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES.'Trust.php');
require_once(ELC_CLASSES.'Product.php');
/**
 * ProvisionalDeclaration
 *
 * All the new declarations are based on the lines in the provisional declaration.
 * The only thing that isnt used in new declarations is the quantity and the weight.
 * On a forfeit trigger the quantity and the weight from the prov. is used to generate an invoice.
 *
 * @author Chris Wesseling <chris.wesseling@xs4all.nl>
 * @author Marten Koedam <marten@dalines.net>
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @package ELC
 * @subpackage Declaration
 * @version $Id$
 */
class ProvisionalDeclaration extends AbstractObject
{
	/**
	 * @var CRSO $crso
	 */
	var $crso;
	/**
	 * @var Participant $participant
	 */
	var $participant;

	var $product;
	/**
	 * @var Trust $trust
	 */
	var $trust;
	/**
	 * @var int $year
	 */
	var $year;
	/**
	 * @access private
	 * @var array $_lines
	 */
	var $_lines = array();
	/**
	 * Add lines to the current Prov. Decl.
	 * @param array $lines
	 */
	function addLines($lines)
	{
		foreach ( $lines as $line )
			$this->addLine($line);
	}
	/**
	 * Add a line, check wether or not the month is set in $args, else add lines for a whole year
	 * @param array $args
	 * @return mixed
	 */
	function addLine($args) //python or ruby style would be nice ;)
	{
		isset($args['year']) && $this->year = $args['year'];
		unset($args['year']);
		if ( empty($args['month']) )
			return $this->_addLinesForYear($args);
		else
			return $this->_addLine($args);
	}
	/**
	 * Set a Provisional Declaration including all its lines for a participant at a crso in a year (with Trust)
	 * @param int $crsoId
	 * @param int $participantId
	 * @param int $year
	 * @param int $trustId
	 * @return bool
	 */
	function set($crsoId='', $participantId='', $year='',$trustId='', $product_id='')
	{
		$crsoId = $crsoId ? $crsoId : $this->crso->relation->id;
		$participantId = $participantId ? $participantId : $this->participant->relation->id;
		$year = $year ? $year : $this->year;
		$product_id = $product_id ? $product_id : $this->product->id;
		$trustId = $trustId ? $trustId : (($this->trust instanceof Trust) ? $this->trust->id : null);
		$sql = "SELECT `MONTH`, `PRODUCT_CATEGORY_ID`, `BRAND_ID`, `CRSO_RELATION_ID`,
				`PARTICIPANT_RELATION_ID`, `QUANTITY`, `WEIGHT`, `CPD_ID`
			FROM TBL_PROVISIONAL_DECLARATION
			WHERE CRSO_RELATION_ID=?
				AND PARTICIPANT_RELATION_ID=?
				AND YEAR=?
				AND PRODUCT_ID = ? ";
		if(is_null($trustId))
			$sql .= "AND TRUST_ID IS NULL";
		else if($trustId)
			$sql .= "AND TRUST_ID = ".$trustId;
		$sql .= " ORDER BY `MONTH`";
		$q = $this->db->query($sql);
		$q->bind_values($crsoId, $participantId, $year, $product_id);

		$q->execute();

		$this->crso = new CRSO($this->db,array('relation'=>new Relation($this->db,$crsoId)));
		$this->participant = new Participant($this->db,array('relation'=>new Relation($this->db,$participantId)));
		$this->year = $year;
		$this->product = new Product($this->db, $product_id);
		$this->trust = $trustId?new Trust($this->db,$trustId):new Trust($this->db,array('id' => null));

		require_once(ELC_CLASSES.'Brand.php');
		require_once(ELC_CLASSES.'ProductCategory.php');
		require_once(ELC_CLASSES.'CRSOProductDimension.php');
		while ($row = $q->fetch())
		{
			$this->_addLine(array(
				'month'=>$row->{'MONTH'},
				'quantity'=>$row->{'QUANTITY'},
				'brand'=>new Brand($this->db, $row->{'BRAND_ID'}),
				'product_category'=>new ProductCategory($this->db, $row->{'PRODUCT_CATEGORY_ID'}),
				'dimension'=>new CRSOProductDimension($this->db,$row->CPD_ID),
				'weight'=>$row->{'WEIGHT'})
			);
		}
	}

	public function getAllProvisional($crsoId='', $participantId='',$trustId='', $product_id='')
	{
		$crsoId = $crsoId ? $crsoId : $this->crso->relation->id;
		$participantId = $participantId ? $participantId : $this->participant->relation->id;
		$product_id = $product_id ? $product_id : $this->product->id;
		$trustId = $trustId ? $trustId : (($this->trust instanceof Trust) ? $this->trust->id : null);
		$sql = "SELECT `MONTH`, `PRODUCT_CATEGORY_ID`, `BRAND_ID`, `CRSO_RELATION_ID`,
				`PARTICIPANT_RELATION_ID`, `QUANTITY`, `WEIGHT`, `CPD_ID`
			FROM TBL_PROVISIONAL_DECLARATION
			WHERE CRSO_RELATION_ID=?
				AND PARTICIPANT_RELATION_ID=?
				AND PRODUCT_ID = ? ";
		if(is_null($trustId))
			$sql .= "AND TRUST_ID IS NULL";
		else if($trustId)
			$sql .= "AND TRUST_ID = ".$trustId;
		$sql .= " ORDER BY `MONTH`";
		$q = $this->db->query($sql);
		$q->bind_values($crsoId, $participantId, $product_id);
		$q->execute();
		return $q;
	}
	/**
	 * Validates wether or not it is valid for this participant at this crso to have no declarations to declare
	 * Total provisional should be 0.
	 * This only applies to SUBROGATORS. See ticket #775.
	 *
	 * @param int $crsoId
	 * @param int $participantId
	 * @param int $year
	 * @return bool
	 */
	function validateNoDeclaration($crsoId='',$participantId='',$year='', $product_id='')
	{
		$crsoId = $crsoId ? $crsoId : $this->crso->relation->id;
		$participantId = $participantId ? $participantId : $this->participant->relation->id;
		$year || $year = $this->year;
		$product_id || $product_id = $this->product->id;

		// Check if the participant is a subrogator. See ticket #775
		require_once(ELC_CLASSES.'Trust.php');
		$trust = new Trust($this->db);
		$q = $trust->selectSubrogations($participantId,TRUST_APPROVED,$crsoId, $product_id);
		if(!($r = $q->fetch()))
			return false;

		$sql = "SELECT SUM(QUANTITY) as TOTALQUANTITY
			FROM TBL_PROVISIONAL_DECLARATION
			WHERE CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ?
				AND `YEAR` = ?
				AND PRODUCT_ID = ?
				AND ISNULL(TRUST_ID)";
		$q = $this->db->query($sql);
		$q->bind_values($crsoId,$participantId,$year, $product_id);
		$q->execute();
		$r = $q->fetch();

		return ($r->TOTALQUANTITY==0);
	}
	/**
	 * Set the lines for one month.
	 * Multiple lines will only be available in case of Brands/Product Categories
	 * @param int $month
	 * @param int $crsoId
	 * @param int $participantId
	 * @param int $year
	 * @param int $trustId
	 * @param int $product_id;
	 * @return bool
	 */
	function setByMonth($month, $crsoId='', $participantId='', $year='', $trustId='', $product_id='')
	{
		$crsoId = $crsoId ? $crsoId : $this->crso->relation->id;
		$participantId = $participantId ? $participantId : $this->participant->relation->id;
		$year = $year ? $year : $this->year;
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT `MONTH`, `PRODUCT_CATEGORY_ID`, `BRAND_ID`, `CRSO_RELATION_ID`,
				`PARTICIPANT_RELATION_ID`, `QUANTITY`, `WEIGHT`, `WEIGHT_DEC`, `TRUST_ID`,`CPD_ID`
			FROM TBL_PROVISIONAL_DECLARATION
			WHERE CRSO_RELATION_ID=?
				AND PARTICIPANT_RELATION_ID=?
				AND PRODUCT_ID = ?
				AND `YEAR`=?
				AND `MONTH`=? ";
		if($trustId)
			$sql .= " AND TRUST_ID = ".$trustId;
		else
			$sql .= " AND ISNULL(TRUST_ID)";

		$q = $this->db->query($sql);
		$q->bind_values($crsoId, $participantId, $product_id, $year,$month);
		$q->execute();
		$rs = $q->fetchAll();
		if (!count($rs))
			return false;

		require_once(ELC_CLASSES.'Brand.php');
		require_once(ELC_CLASSES.'ProductCategory.php');
		require_once(ELC_CLASSES.'CRSOProductDimension.php');
		$this->_lines = array();
		foreach($rs as $row)
		{
			$this->_addLine(array(
				'month'=>$row->{'MONTH'},
				'quantity'=>$row->{'QUANTITY'},
				'brand'=>new Brand($this->db, $row->{'BRAND_ID'}),
				'product_category'=>new ProductCategory($this->db, $row->{'PRODUCT_CATEGORY_ID'}),
				'dimension'=>new CRSOProductDimension($this->db,$row->CPD_ID),
				'weight'=>$row->{'WEIGHT'},
				'weight_dec'=>$row->{'WEIGHT_DEC'})
			);
		}

		return true;
	}
	/**
	 * Override the AbstractObject insert to add all the lines
	 * @param bool $graceful
	 * @return mixed
	 */
	function insert($graceful=false)
	{
		return $this->_doLines('insert', $graceful);
	}
	/**
	 * Override the AbstractObject update to update all the lines
	 * @param bool $graceful
	 * @return mixed
	 */
	function update($graceful=false)
	{
		return $this->_doLines('update', $graceful);
	}
	/**
	 * Override the AbstractObject delete to delete all the lines
	 * @param bool $graceful
	 * @return mixed
	 */
	function delete($graceful=false)
	{
		debug('ProvisionalDeclaration::delete');
		return $this->_doLines('delete', $graceful);
	}

	function getValidationKeyForTrust($trust_id='')
	{
		$trust_id || $trust_id = $this->trust->id;

		$sql = "SELECT CONCAT(QUANTITY,IFNULL(WEIGHT,''),
				IFNULL(CPD_ID,''),
				IFNULL(PRODUCT_CATEGORY_ID,''),
				IFNULL(BRAND_ID,''),CRSO_RELATION_ID,PARTICIPANT_RELATION_ID,`MONTH`,`YEAR`)
				as CONCAT_FIELD
			FROM TBL_PROVISIONAL_DECLARATION
			WHERE TRUST_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($trust_id);
		$q->execute();
		$rs = $q->fetchAll();
		if(!count($rs))
			return false;
		$rv = '';
		foreach($rs as $r)
			$rv .= $r->CONCAT_FIELD;
		return md5($rv);
	}

	/**
	 * What's this?
	 */
	function getLinesByExample()
	{
		//fixme ?
	}
	/**
	 * Get the quantity for a certain month
	 * This is based in a single line posibility, so everything that makes a line unique should be provided
	 * @param int $month
	 * @param int $product_category_id
	 * @param int $brand_id
	 * @param int $trust_id
	 * @return bool|array
	 */
	function getQuantityForMonth($month, $product_category_id='', $brand_id='', $trust_id='', $product_id='',$cpd_id='')
	{
		$product_id || $product_id = $this->product->id;

		$values = array($this->crso->relation->id, $this->participant->relation->id, $product_id,$this->year,$month);
		$sql = "SELECT QUANTITY, WEIGHT, WEIGHT_DEC
			FROM TBL_PROVISIONAL_DECLARATION
			WHERE CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ?
				AND PRODUCT_ID = ?
				AND `YEAR` = ?
				AND `MONTH` = ? ";
		if($product_category_id)
		{
			$values[] = $product_category_id;
			$sql .= " AND PRODUCT_CATEGORY_ID = ? ";
		}
		if($brand_id)
		{
			$values[] = $brand_id;
			$sql .= " AND BRAND_ID = ? ";
		}
		if($cpd_id)
		{
			$values[] = $cpd_id;
			$sql .= " AND CPD_ID = ? ";
		}

		if($trust_id)
		{
			$values[] = $trust_id;
			$sql .= " AND TRUST_ID = ? ";
		}
		else
			$sql .= " AND ISNULL(TRUST_ID) ";

		$q = $this->db->query($sql);
		$q->bind_values($values);
		$q->execute();
		if(!($r = $q->fetch()))
		{
			emerg('[ProvisionalDeclaration::getQuantityForMonth] Database is messed up! There should be at least ONE and only ONE line for this entry.');
			return false;
		}
		return array($r->QUANTITY, $r->WEIGHT, $r->WEIGHT_DEC);
	}
	/**
	 * Get the array for a TableFactory overview based on the country settings provided
	 * @param int $countrySettings
	 * @return array $rv
	 */
	function getOverview($countrySettings)
	{
		$rv = array();
		foreach ($this->getLines() as $line)
		{
			$newline = array();

			$newline['month'] = $line->month;
			$newline['quantity'] = $line->quantity;

			if ($countrySettings & HasDeclOfWeight)
				$newline['weight'] = $line->weight;

			if (($countrySettings & HasDeclByCategory) && isset($line->product_category->id)) {
				$line->product_category->set();
				$newline['product_category'] = $line->product_category;
			}

			if (($countrySettings & HasDeclarationDimension) && isset($line->dimension->id)) {
				$line->dimension->set();
				$newline['dimension'] = $line->dimension;
			}

			if (($countrySettings & HasDeclByBrand) && isset($line->brand->id)) {
				$line->brand->set();
				$newline['brand'] = $line->brand;
			}

			$rv[] = $newline;
		}

		return $rv;
	}
	/**
	 * @return array <ProvisionalDeclarationLine>
	 */
	function getLines()
	{
		return $this->_lines;
	}
	/**
	 * Get the number of distinct months for a certain participant/crso
	 * @todo Is this supposed to work with Trusts??
	 * @return bool|int
	 */
	function getDistinctMonthCount()
	{
		$sql = "SELECT COUNT(DISTINCT `MONTH`) as MONTHS
			FROM TBL_PROVISIONAL_DECLARATION
			WHERE CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ?
				AND PRODUCT_ID = ?
				AND `YEAR` = ?
				AND ".($this->trust->id?'TRUST_ID = '.$this->trust->id:'ISNULL(TRUST_ID)');
		$q = $this->db->query($sql);
		$q->bind_values($this->crso->relation->id,$this->participant->relation->id, $this->product->id, $this->year);
		$q->execute();
		if($r=$q->fetch())
			return $r->MONTHS;
		return false;
	}
	/**#@+
	 * Subrogation stuff
	 */
	/**
	 * Dont imitate, subrogate
	 * This is done by substracting the provided lines from the original provisional
	 *   that was still left for this participant at this crso
	 * @param array $lines
	 * @return mixed
	 */
	function subrogate($lines,$product_id)
	{
		require_once(ELC_CLASSES.'Brand.php');
		require_once(ELC_CLASSES.'ProductCategory.php');
		require_once(ELC_CLASSES.'Product.php');
		$this->product = new Product($this->db,$product_id);
		$prov = new ProvisionalDeclaration($this->db,array(
			'participant'=>$this->participant,
			'crso'=>$this->crso,
			'year'=>$this->year,
			'product' => $this->product));
		$prov->set();
		info(sprintf('ProvisonalDeclaration::subrogate() - Starting subrogation for Subrogator [%s] ', $this->participant->relation->id));
		//debug($lines);
		$startMonth = 13-$prov->getDistinctMonthCount(); // For all the months available
		foreach($lines as $index => $line)
		{
			if(!isset($line['month']))
				$this->_breakDownFromYear($line,$startMonth,'subrogateLine');
			else
				$this->subrogateLine($line);
			if(isset($line['brand']))
				$lines[$index]['brand'] = new Brand($this->db,$line['brand']);
			if(isset($line['product_category']))
				$lines[$index]['product_category'] = new ProductCategory($this->db,$line['product_category']);
			if(isset($line['dimension']))
				$lines[$index]['dimension'] = new CRSOProductDimension($this->db,$line['dimension']);
		}
		reset($lines);
		$this->addLines($lines);
		return $this->_doLines('insert',false);
	}
	/**
	 * Subrogates a single line,
	 * 		substracts the subrogated quantity from the original quantity.
	 * 		Same goes for weight, all based on the product_category/brand settings for this parts. country
	 * @param array $args
	 * @return bool
	 */
	function subrogateLine($args)
	{
		info('ProvisionalDeclaration::subrogateLine()');
		$values = array($this->year,$this->participant->relation->id,
			$this->crso->relation->id, $this->product->id, $args['month']);
		if(isset($args['product_category']))
		{
			$values[] = $args['product_category'];
			$product_category = ' PRODUCT_CATEGORY_ID = ? ';
		}
		else
			$product_category = ' ISNULL(PRODUCT_CATEGORY_ID) ';
		if(isset($args['dimension']))
		{
			$values[] = $args['dimension'];
			$dimension = ' CPD_ID = ? ';
		}
		else
			$dimension = ' ISNULL(CPD_ID) ';
		if(isset($args['brand']))
		{
			$values[] = $args['brand'];
			$brand = ' BRAND_ID = ? ';
		}
		else
			$brand = ' ISNULL(BRAND_ID) ';
		// Check for uniqueness of a line... else HOCUS BOGUS
		$sql = "SELECT QUANTITY FROM TBL_PROVISIONAL_DECLARATION
			WHERE `YEAR` = ?
				AND PARTICIPANT_RELATION_ID = ?
				AND CRSO_RELATION_ID = ?
				AND PRODUCT_ID = ?
				AND `MONTH` = ?
				AND ISNULL(TRUST_ID)
				AND ".$brand."
				AND ".$dimension."
				AND ".$product_category;

		$q = $this->db->query($sql);
		$q->bind_values($values);
		$q->execute();
		$rs = $q->fetchAll();
		if(count($rs)>1)
		{
			trigger_error('Provisional Declaration has multiple lines for used key',E_USER_WARNING);
			return false;
		}
		//info(sprintf(' - Updating Provisional set Quantity - [%s] and Weight - [%s] for month [%s]',$args['quantity'],$args['weight'],$args['month']));
		$sql = "UPDATE TBL_PROVISIONAL_DECLARATION
			SET QUANTITY = QUANTITY - ? ".
				(!empty($args['weight'])?", WEIGHT = WEIGHT - ".$args['weight']." ":"")."
			WHERE `YEAR` = ?
				AND PARTICIPANT_RELATION_ID = ?
				AND CRSO_RELATION_ID = ?
				AND PRODUCT_ID = ?
				AND `MONTH` = ?
				AND ISNULL(TRUST_ID)
				AND ".$brand."
				AND ".$dimension."
				AND ".$product_category;
		array_unshift($values,$args['quantity']);
		$q = $this->db->query($sql);
		$q->bind_values($values);
		return $q->execute();
	}
	/**
	 * Removes a subrogation, re-add the values back to the original participants provisional.
	 * @todo WEIGHT support
	 * @param Trust $trust
	 * @return bool
	 */
	function removeSubrogation($trust='')
	{
		$trust || $trust = $this->trust;
		$trust->set();

		$d = currentDate();
		$year = $d->getYear();
		$month = 1; // Get the last month in TBL_DECLARATION for this TRUST
		$sql = "SELECT MAX(`MONTH`) as MAXMONTH FROM TBL_DECLARATION as D
			INNER JOIN TBL_DECLARATION_LINE as DL
				ON D.DECLARATION_ID = DL.DECLARATION_ID
			WHERE TRUST_ID = ? AND `YEAR` = ?";
		$q = $this->db->query($sql);
		$q->bind_values($trust->id,$year);
		$q->execute();
		if($r = $q->fetch())
			$month = $r->MAXMONTH+1;
		if($month == 13) // Nothing will happen in this case.
			return true;
		// Select the subrogated provisional lines
		// Remaining months only (see 1.2.4.10)!!!
		$sql = "SELECT QUANTITY, WEIGHT,
					`YEAR`, `MONTH`,
					IFNULL(PRODUCT_CATEGORY_ID, 'ISNULL(PRODUCT_CATEGORY_ID)') as PRODUCT_CATEGORY_ID,
					IFNULL(CPD_ID, 'ISNULL(CPD_ID)') as CPD_ID,
					IFNULL(BRAND_ID, 'ISNULL(BRAND_ID)') as BRAND_ID
				FROM TBL_PROVISIONAL_DECLARATION
				WHERE TRUST_ID = ? AND `YEAR` = ? AND `MONTH` >= ?";
		$q = $this->db->query($sql);
		$q->bind_values($trust->id,$year,$month);
		$q->execute();
		while($r = $q->fetch())
		{

			$where = "WHERE CRSO_RELATION_ID = ?
					AND PARTICIPANT_RELATION_ID = ?
					AND PRODUCT_ID = ?
					AND ISNULL(TRUST_ID)
					AND ".(is_numeric($r->PRODUCT_CATEGORY_ID)?"PRODUCT_CATEGORY_ID = ":"").$r->PRODUCT_CATEGORY_ID."
					AND ".(is_numeric($r->CPD_ID)?"CPD_ID = ":"").$r->CPD_ID."
					AND ".(is_numeric($r->BRAND_ID)?"BRAND_ID = ":"").$r->BRAND_ID."
					AND `YEAR` = ".$r->YEAR."
					AND `MONTH` = ".$r->MONTH."
					";
			$values = array($trust->crso->relation->id,$trust->subrogator->relation->id, $trust->product->id);

			$sql = "SELECT PRODUCT_ID FROM TBL_PROVISIONAL_DECLARATION ".$where;
			$p = $this->db->query($sql);
			$p->bind_values($values);
			$p->execute();
			$rows = $p->fetchAll();
			if(count($rows))
			{
				// Update the original participant provisional
				$sql = "UPDATE TBL_PROVISIONAL_DECLARATION
					SET QUANTITY = QUANTITY + ".$r->QUANTITY." ";
				if(is_numeric($r->WEIGHT))
					$sql .= ", WEIGHT = WEIGHT + ".$r->WEIGHT." ";
				$sql .= $where;
				$p = $this->db->query($sql);
				$p->bind_values($values);
				$p->execute();
			}
			else
			{
				$replacements = '?, ?, ?, ?, ?, ?';
				$values = array($r->QUANTITY,$trust->crso->relation->id,
					$trust->subrogator->relation->id, $trust->product->id,
					$r->YEAR,$r->MONTH);

				is_numeric($r->PRODUCT_CATEGORY_ID) && ($replacements .= ', ?') && ($values[] = $r->PRODUCT_CATEGORY_ID);
				is_numeric($r->CPD_ID) && ($replacements .= ', ?') && ($values[] = $r->CPD_ID);
				is_numeric($r->BRAND_ID) && ($replacements .= ', ?') && ($values[] = $r->BRAND_ID);
				is_numeric($r->WEIGHT) && ($replacements .= ', ?') && ($values[] = $r->WEIGHT);

				// No original quantity, insert new, see Trac ticket #800
				$sql = "INSERT INTO TBL_PROVISIONAL_DECLARATION
					(QUANTITY,CRSO_RELATION_ID,PARTICIPANT_RELATION_ID,
						PRODUCT_ID, `YEAR`,`MONTH`
						".(is_numeric($r->PRODUCT_CATEGORY_ID)?', PRODUCT_CATEGORY_ID':'')."
						".(is_numeric($r->CPD_ID)?', CPD_ID':'')."
						".(is_numeric($r->BRAND_ID)?', BRAND_ID':'')."
						".(is_numeric($r->WEIGHT)?', WEIGHT':'')."
					)
					VALUES (".$replacements.")";
				$p = $this->db->query($sql);
				$p->bind_values($values);
				$p->execute();
			}


		}
		// Delete the subrogated provisional lines
		// Remaining months only (see 1.2.4.10)!!!
		$sql = "DELETE FROM TBL_PROVISIONAL_DECLARATION
			WHERE TRUST_ID = ? AND `YEAR` = ? AND `MONTH` >= ?";
		$q = $this->db->query($sql);
		$q->bind_values($trust->id,$year,$month);
		if(!$q->execute(true))
			return false;
		return true;
	}
	/**
	 * This changes the owner (PARTICIPANT_ID) of the provisional for the given trust.
	 * Used in crso subrogation approval
	 * @param int $new_participant_id
	 * @param int $trust_id
	 * @return bool
	 */
	function changeParticipantForTrust($new_participant_id, $trust_id='')
	{
		$trust_id = ($trust_id) ? $trust_id : $this->trust->id;

		$sql = 'UPDATE TBL_PROVISIONAL_DECLARATION
			SET PARTICIPANT_RELATION_ID=?
			WHERE
				TRUST_ID=?';
		$q = $this->db->query($sql);
		$q->bind_values($new_participant_id, $trust_id);
		if(!$q->execute())
			return false;
		return true;
	}
	/**
	 * Get all the lines, aggregated, therefore month is not stored in the lines array returned
	 * @param int $crsoId
	 * @param int $participantId
	 * @param int $year
	 * @param int $trustId
	 * @return bool|array
	 */
	function &getAggregateLinesByYear($crsoId='', $participantId='', $year='',$trustId='', $product_id='')
	{
		$crsoId = $crsoId ? $crsoId : $this->crso->relation->id;
		$participantId = $participantId ? $participantId : $this->participant->relation->id;
		$year = $year ? $year : $this->year;
		$trustId = $trustId ? $trustId : (($this->trust instanceof Trust)?$this->trust->id:null);
		$product_id || $product_id = $this->product->id;

		$sql = "SELECT
				  `PRODUCT_CATEGORY_ID`,
				  `BRAND_ID`,
				  `CPD_ID`,
				  sum(`QUANTITY`) AS QUANTITY,
				  sum(`WEIGHT`) AS WEIGHT,
				  sum(`WEIGHT_DEC`) AS WEIGHT_DEC
				FROM
				  TBL_PROVISIONAL_DECLARATION
				WHERE
				  CRSO_RELATION_ID=?
				AND
				  PARTICIPANT_RELATION_ID=?
				AND
				  PRODUCT_ID = ?
				AND
				  YEAR=?
				";
		if(is_null($trustId))
			$sql .= "AND ISNULL(TRUST_ID) ";
		else if($trustId)
			$sql .= "AND TRUST_ID = ".$trustId;
		$sql .= " GROUP BY  PRODUCT_CATEGORY_ID, BRAND_ID";
		$q = $this->db->query($sql);
		$q->bind_values($crsoId, $participantId, $product_id, $year);
		if(!$q->execute())
			return false;
		$lines = array();

		require_once(ELC_CLASSES.'Brand.php');
		require_once(ELC_CLASSES.'ProductCategory.php');
		require_once(ELC_CLASSES.'CRSOProductDimension.php');
		while($r = $q->fetch())
		{
			$l = new ProvisionalDeclarationLine($this->db, array(
				'month'=> 0, // not queried for as this an aggregate
				'quantity'=>$r->{'QUANTITY'},
				'brand'=>new Brand($this->db, $r->{'BRAND_ID'}),
				'product_category'=>new ProductCategory($this->db, $r->{'PRODUCT_CATEGORY_ID'}),
				'dimension'=>new CRSOProductDimension($this->db, $r->CPD_ID),
				'weight'=>$r->{'WEIGHT'},
				'weight_dec'=>$r->{'WEIGHT_DEC'}) );
			$l->declaration =& $this;
			$lines[] = $l;
		}
		return $lines;
	}
	/**#@-*/
	/**#@+
	 * @access private
	 */
	/**
	 * Does an $action for all the lines
	 * @param string $action
	 * @param bool $graceful
	 * @return bool
	 */
	function _doLines($action, $graceful)
	{
		$rv = 1;
		foreach ( $this->_lines as $line )
		{
			debug('Calling action '.$action.' on '.$line);
			$rv &= $line->$action($graceful);
		}
		return (bool) $rv;
	}
	/**
	 * Add the lines for a year.
	 * The months to be added are calculated based on the Obligation Start Date
	 *   (formerly known as Prince, errr... BusinessStartMonth)
	 * @param array $args
	 */
	function _addLinesForYear($args)
	{
		$month = 1;
		if ( $this->participant->getBusinessStartYear($this->crso,$this->product->id) == $this->year )
			$month = $this->participant->getBusinessStartMonth($this->crso,$this->product->id);

		if(isset($this->_startMonth)&& $this->_startMonth > $month)
			$month = $this->_startMonth;

		$this->_breakDownFromYear($args,$month);
	}
	/**
	 * @param array $args Available properties defined by country settings
	 * @param int $month Start month
	 * @param string $func Function to call on the line
	 */
	function _breakDownFromYear($args,$month,$func='addLine')
	{
		// Devide quantities and weights per month
		foreach ( array('quantity', 'weight', 'weight_dec') as $prop )
		{
			$q =& $args[$prop];

			if (!isset($$prop))
				$$prop = $q; // remember total

			$q = floor($q/(13-$month)); //floored
		}

		$args['month'] =& $month;
		$startMonth = $month;
		while ( $month <= 12 )
		{
			if ($month == 12) // If this is the last month of the year, add the remainder from the floored amount
			{
				foreach ( array('quantity', 'weight', 'weight_dec') as $prop )
				{
					$q =& $args[$prop];
					$q = $$prop - ($q * (12-$startMonth));
				}
			}

			$this->$func($args);
			$month++;
		}
	}
	// All data from $month is stored in TBL_PROVISIONAL_DECLARATION_HISTORY
	function saveHistoricalProv($month)
	{
		$user = new ELCUser($this->db,$_SESSION['session_user_id']);
		$pdh = new ProvisionalDeclarationHistory($this->db);
		$pdh->setProps(array('crso'=>$this->crso,'participant'=>$this->participant,'product'=> $this->product, 'year'=>$this->year,'user'=>$user));
		$lines = $this->getLines();
		foreach($lines as $line)
		{
			if($line->month < $month)
				continue;
			$pdh->addLine($line);
		}
		$pdh->insertLines();
	}

	/**
	 *
	 */
	function removeLinesFrom($month)
	{
		info('Deleting provisional lines for participant [%s], crso [%s], trust [%s], year [%s], month [%s]', $this->participant->relation->id, $this->crso->relation->id, $this->trust->id, $this->year, $month);

		$sql = sprintf("DELETE FROM TBL_PROVISIONAL_DECLARATION
			WHERE PARTICIPANT_RELATION_ID = ?
				AND CRSO_RELATION_ID = ? AND PRODUCT_ID = ? AND %s
				AND YEAR = ? AND MONTH >= ?",
			(($this->trust instanceof Trust && is_numeric($this->trust->id))?'TRUST_ID = '.$this->trust->id:'ISNULL(TRUST_ID)'));
		$q = $this->db->query($sql);
		$q->bind_values($this->participant->relation->id,$this->crso->relation->id, $this->product->id,$this->year,$month);
		return $q->execute();
	}
	/**
	 * Adds a single line to the _lines var
	 * @param array $args
	 */
	function _addLine($args)
	{
		$l = new ProvisionalDeclarationLine($this->db, $args );
		$l->declaration =& $this;
		$this->_lines[] = $l;
	}
	/**
	 * Initializes the objectmaps used in AbstractObject
	 */
	function _initObjectMaps()
	{
		/**
		 * array multidimensional ('parent'=>parentclassname=>property, 'children'=>ChildClass=>array('property'=>property, 'classfile'=>classpath, 'dbtable'=>linktablename, 'key'=>array(property=>linktablefieldname), 'properties'=>array(childproperty=>linktablefieldname)))
		 */

		$this->familymap['children']['ProvisionalDeclarationLine']['property'] = '_lines';
 		$this->familymap['children']['ProvisionalDeclarationLine']['classfile'] = ELC_CLASSES.'ProvisionalDeclaration.php';
 		$this->familymap['children']['ProvisionalDeclarationLine']['dbtable'] = 'TBL_PROVISIONAL_DECLARATION';
 		$this->familymap['children']['ProvisionalDeclarationLine']['key']['year'] = 'YEAR';
 		$this->familymap['children']['ProvisionalDeclarationLine']['key']['crso->relation->id'] = 'CRSO_RELATION_ID';
 		$this->familymap['children']['ProvisionalDeclarationLine']['key']['participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
 		$this->familymap['children']['ProvisionalDeclarationLine']['key']['product->id'] = 'PRODUCT_ID';
 		$this->familymap['children']['ProvisionalDeclarationLine']['properties']['month'] = 'MONTH';
		$this->familymap['children']['ProvisionalDeclarationLine']['properties']['brand'] = array('val'=>'BRAND_ID','class'=>'Brand');
		$this->familymap['children']['ProvisionalDeclarationLine']['properties']['product_category'] = array('val'=>'PRODUCT_CATEGORY_ID','class'=>'ProductCategory');
		$this->familymap['children']['ProvisionalDeclarationLine']['properties']['dimension'] = array('val'=>'CPD_ID','class'=>'CRSOProductDimension');
	}
	/**#@-*/
}
/**
 * @author Chris Wesseling <chris.wesseling@xs4all.nl>
 * @author Marten Koedam <marten@dalines.net>
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @package ELC
 * @subpackage Declaration
 * @version $Id$
 */
class ProvisionalDeclarationLine extends AbstractObject
{
	/**
	 * @var int $month
	 */
	var $month;
	/**
	 * @var ProvisionalDeclaration &$declaration
	 */
	var $declaration;
	/**
	 * @var Brand $brand
	 */
	var $brand;
	/**
	 * @var ProductCategory $product_category
	 */
	var $product_category;
	var $dimension;
	/**
	 * @var int $quantity
	 */
	var $quantity;
	/**
	 * @var int $weight
	 */
	var $weight;
	/**
	 * Calculations with the decimal weight column are ONLY done in SQL.
	 *
	 * @var float $weight_dec
	 */
	var $weight_dec;
	/**
	 * This should not be done... use dr. fill() on the parent
	 */
	function set()
	{
		return true; //pass
	}
	/**
	 * Initializes the objectmaps used in AbstractObject
	 * @access private
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_PROVISIONAL_DECLARATION';
		$this->objectmap['primary']['declaration->year'] = 'YEAR';
		$this->objectmap['primary']['month'] = 'MONTH';
		$this->objectmap['primary']['declaration->crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['primary']['declaration->participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
		$this->objectmap['primary']['declaration->product->id'] = 'PRODUCT_ID';
		$this->objectmap['primary']['declaration->trust->id'] = 'TRUST_ID';
		$this->objectmap['primary']['brand->id'] = 'BRAND_ID';
		$this->objectmap['primary']['product_category->id'] = 'PRODUCT_CATEGORY_ID';
		$this->objectmap['primary']['dimension->id'] = 'CPD_ID';
		$this->objectmap['properties']['quantity'] = 'QUANTITY';
		$this->objectmap['properties']['weight'] = 'WEIGHT';
		$this->objectmap['properties']['weight_dec'] = 'WEIGHT_DEC';

		$this->familymap['parent']['ProvisionalDeclaration'] = 'declaration';
	}

	public function __toString()
	{
		return 'DeclarationLine: '.$this->declaration->year.'/'.$this->month.' For CRSO['.$this->declaration->crso->relation->id.'], Participant ['.$this->declaration->participant->relation->id.']';
	}
}
?>
