<?php

require_once(ELC_CLASSES.'PwcExporter.php');
require_once(IWARE.'MockLanguageInterface.php');
require_once(ELC_CLASSES.'Declaration.php');

/**
 * PwcExporterDeclarations
 * @author martijn
 */
class PwcExporterDeclarations extends PwcExporter
{
	protected $hash_each_line_on_write = true;

	function exportCrso(SoapCrsoMasterData $crso, $year, $month, $day) {
		info('PwcExporterDeclarations: exportCrso: '.$crso->crsoId);

		// check if give day is the export date for the given crso, else continue
		if (!is_null($day)) {
			// TODO: FORFEIT_DAY is per product but export is per crso? wierd
			$sql = 'SELECT FORFEIT_DAY FROM TBL_CRSO_PRODUCT WHERE CRSO_RELATION_ID=?';
			$q = $this->_db->query($sql);
			$q->bind_values($crso->crsoId);
			$q->execute();
			$r = $q->fetch();
			if ($r->FORFEIT_DAY != $day) {
				info('PwcExporterDeclarations: skipping CRSO ['.$crso->crsoId.']: Day '.$day.' is not the export day');
				return;
			}
		}

		$date = new IWDateTime(sprintf('%s-%s-%s', $year, $month, $day));

		// get invoices for this month
		$invoices = $this->_soapClient->getDeclarations($crso->crsoId, $year, $month)->values;
		$data = array();

		// loopy
		foreach ($invoices as $i=>$inv) {
			$invLineIndex = 0;

			foreach ($inv->declarations->values as $decl) {
				$declLineIndex = 0;
				$totalQuantity = 0;

				$lines = array();

				foreach ($decl->lines->values as $line) {

					$invLineIndex++;
					$declLineIndex++;

					$customerNo = $crso->crsoCountryCode.
						str_pad($crso->crsoId, 8, '0', STR_PAD_LEFT).
						str_pad($inv->participantId, 8, '0', STR_PAD_LEFT);

					$postingDate = new DateTime($decl->postingDate);
					$documentDate = new DateTime($inv->documentDate);

					$periodEnd = new DateTime(($decl->month == 12 ? $decl->year+1 : $decl->year).'-'.($decl->month == 12 ? 1 : $decl->month+1).'-01');
					$periodEnd->modify('-1 day');

					$totalQuantity = $line->quantity;

					$invoiceNr = $this->buildInvoiceForExport($crso->crsoId, $inv->invoiceYear, $inv->invoiceId, ($inv->type == 'credit'), false );

					if ($inv->type == 'credit')
						$originalInvoiceNr = $this->buildInvoiceForExport($crso->crsoId, $decl->originalInvoiceYear, $decl->originalInvoiceId, ($inv->type == 'credit'), false );
					else
						$originalInvoiceNr = '';

					$lines[] = array(

						'invoice_no' => $invoiceNr,
						'invoice_line_no' => $invLineIndex,
						'declaration_id' => $decl->declarationId,
						'line_no' => $declLineIndex,
						'product_id'=> $this->getProductIdFromName($decl->product),
						'journal_template' => ($inv->type == 'debit' ? 'I' : 'C').$crso->crsoId,
						'customer_no' => $customerNo,
						'customer_name' => '',
						'posting_date' => $postingDate->format('d/m/Y'),
						'document_date' => $documentDate->format('d/m/Y'),
						'external_document_no' => '',
						'currency_code' => '',
						'type' => 'G/L Account',
						'gl_account_no' => $line->glAccount,
						'description' => '1/'.$decl->month.'/'.$decl->year.' - '.$periodEnd->format('d/n/Y'),
						'quantity'=> $line->quantity,
						'unit_cost' => number_format($line->unitCost, 8, ',', ''),
						'line_amount_excl_vat' => $line->amount != '' ? number_format($line->amount, 2, ',', '') : '',
						'total_amount_excl_vat' => number_format($inv->totalAmount, 2, ',', ''),
						'total_amount_incl_vat' => '',
						'total_vat_amount' => '',
						'navision_vat_code' => '',
						'vat_extended_text' => '',
						'dimension_code_1' => '',
						'dimension_code_2' => '',
						'dimension_code_3' => '',
						'dimension_code_4' => '',
						'dimension_code_5' => '',
						'dimension_code_6' => '',
						'dimension_code_7' => '',
						'dimension_code_8' => '',
						'tariff_no' => '',
						'net_weight' => (int) $line->weight,
						'transaction_type' => '',
						'exit_point' => '',
						'transport_method' => '',
						'decl_weight' => (int) $line->weight,
						'decl_product_category' => $line->productCategory,
						'decl_brand' => $line->brand,
						'decl_month' => $decl->month,
						'decl_year' => $decl->year,
						'decl_type' => $this->_translateLineStatus($line->status),
						'decl_total' => is_null($line->quantity) ? '' : $inv->type == 'credit' ? $line->quantity * -1 : $line->quantity,
						'original_invoice' => $originalInvoiceNr,
						'bill_to_customer_no' => $customerNo,
						'product_description' => $decl->product,
						'hash_total' => $this->_getHashTotal($inv->type, $invLineIndex, $date->getYear(), $inv->invoiceId, $crso->crsoId),
						'refund_export_date' => '',
						'document_review_no' => '',
						'refund_weight' => '',
						'refund_product_category' => '',
						'refund_brand' => '',
					);
				}

				// merge $lines into $data
				$data = array_merge($data, $lines);
			}
		}

		// pwc exports are ordered by invoice, line_no, declaration_id
		// so first keep track of these records in different arrays..
		$lineArray = array();
		foreach ($data as $line) {

			if (!isset($lineArray[$line['invoice_no']]))
				 $lineArray[$line['invoice_no']] = array();

			if (!isset($lineArray[$line['invoice_no']][$line['line_no']]))
				 $lineArray[$line['invoice_no']][$line['line_no']] = array();

			$lineArray[$line['invoice_no']][$line['line_no']][] = $line;
		}

		// then loop those arrays and add the lines.. also rewrite the
		// invoice line number
		$data = array();
		foreach ($lineArray as $invoices) {
			$invLineIndex = 0;

			foreach ($invoices as $lines) {

				foreach ($lines as $line) {
					$invLineIndex++;
					$line['invoice_line_no'] = $invLineIndex;
					$data[] = $line;
				}

			}
		}

		$fieldlengths = array();
		$processors = array();
		$date = new DateTime(sprintf('%s-%s-%s', $year, $month, $day));
		$filename = $crso->crsoId.'/'.sprintf('%s_TRASALES.xml', $date->format('Ymd'));

		$this->writefile($data, $fieldlengths, $filename, $processors);
	}

	/**
	 * translates the status received by soap to a DeclarationLine status.. this then is used to translate to a string
	 * @param unknown_type $status
	 */
	protected function _translateLineStatus($status) {

		$map = array(
			1 => DECLARATION_OPEN,
			2 => DECLARATION_NORMAL,
			3 => DECLARATION_FORFEIT,
			4 => DECLARATION_LATE,
			5 => DECLARATION_DECLARED,
			6 => DECLARATION_RESUBMIT,
			7 => DECLARATION_CREDIT
		);

		if (!$map[$status])
			throw new Exception('_translateLineStatus: no such status: '.$status);

		$lineStatusArray = Declaration::translateStatus($this->_langIf, $map[$status]);
		return $lineStatusArray[0];
	}

	/**
	 * gets the hash_total for the given arguments
	 * @param unknown_type $type
	 * @param unknown_type $invLineIndex
	 * @param unknown_type $year
	 * @param unknown_type $invNr
	 * @param unknown_type $crsoId
	 */
	protected function _getHashTotal($type, $invLineIndex, $year, $invNr, $crsoId) {

		if ($type == 'debit')
		{
			$sql = "SELECT HASH_TOTAL
				FROM TBL_DECLARATION_DEBIT_LINE
				WHERE INVOICE_LINE_INDEX=?
					AND YEAR=?
					AND DECLARATION_DEBIT_ID=?
					AND CRSO_RELATION_ID=?
				";
		} elseif ($type == 'credit')
		{
			$sql = "SELECT HASH_TOTAL
				FROM TBL_DECLARATION_CREDIT_LINE
				WHERE INVOICE_LINE_INDEX=?
					AND YEAR=?
					AND DECLARATION_CREDIT_ID=?
					AND CRSO_RELATION_ID=?
				";
		}
		else {
			throw new Exception('_getHashTotal: Unknown type: '.$type);
		}

		$q = $this->_db->query($sql);
		$q->bind_values($invLineIndex, $year, $invNr, $crsoId);
		$q->execute();

		if (!$row = $q->fetch())
			throw new Exception(sprintf('_getHashTotal: no credit/debit line for %s/%s/%s/%s', $invLineIndex, $year, $invNr, $crsoId));

		return $row->HASH_TOTAL;

	}



}

?>
