<?php

require_once(ELC_CLASSES.'PwcExporter.php');

/**
 * PwcExporterMasterData
 * @author martijnIQUNIT_WRITABLE_PATH
 *
 */
class PwcExporterMasterData extends PwcExporter
{

	protected static $existingParticipants = array();
	protected static $lastExports = array();

	public static function clearChangeControl()
	{
		self::$existingParticipants = array();
		self::$lastExports = array();
	}

	function export() {
		parent::export(null, null, null);
	}

	function exportCrso(SoapCrsoMasterData $crso, $year, $month, $day) {

		info('PwcExporterMasterData: working on crso: '.$crso->crsoId);

		$participantsData = array();

		// loop all participants for this crso
		$participantMasterData = $this->_soapClient->getParticipantMasterData($crso->crsoId)->values;
		foreach ($participantMasterData as $p) {

			info(sprintf('PwcExporterMasterData: working on participant "%s"', $p->participantName));

			$participantsData[] = array(

				'Customer No.' => $crso->crsoCountryCode.
					str_pad($crso->crsoId, 8, '0', STR_PAD_LEFT).
					str_pad($p->participantId, 8, '0', STR_PAD_LEFT),
				'Customer Name' => $p->participantName,
				'Customer Name 2' => $p->participantName,
				'Customer Address' => $p->participantAddress,
				'Customer Address 2' => $p->participantAddress,
				'Customer Post Code' => $p->participantPostalcode,
				'Customer City' => $p->participantCity,
				'Customer Contact'=>$p->participantContactName,
				'Customer Phone No.' => $p->participantPhone,
				'Customer Fax No.' => $p->participantFax,
				'Global Dimension Code 1' => '',
				'Global Dimension Code 2' => '',
				'Customer Currency Code' => '',
				'Customer Language Code' => $this->getNavisionLanguageCode($p->participantLanguageCode),
				'Customer Country Code' => $this->getNavisionCountryCode($p->participantCountryCode),
				'VAT Registration Number' => $p->participantVatNumber,
				'Customer e mail' => $p->participantEmail,
				'Customer Home Page' => $p->participantHomePage,
				'New Customer' => sprintf('%s;%s', $crso->crsoId, $p->participantVatNumber),
				'Send to Name' => $p->participantSendToName,
				'Send to Name 2' => $p->participantSendToName,
				'Send to Address' => $p->participantSendToAddress,
				'Send to Address 2' => $p->participantSendToAddress,
				'Send to Post Code' => $p->participantSendToPostalcode,
				'Send to City' => $p->participantSendToCity,
				'Send to Country Code' => $this->getNavisionCountryCode($p->participantSendToCountryCode),
				'Send to Contact' => $p->participantSendToContact,
				'ACCOUNT No 1' => $p->participantBankAccountNumber,
				'IBAN No 1' => $p->participantIbanNumber,
				'SWIFT No 1' => $p->participantSwiftNumber,
				'ACCOUNT No 2' => $p->participantDdBankAccountNumber,
				'IBAN No 2' => $p->participantDdIbanNumber,
				'SWIFT No 2' => $p->participantDdSwiftNumber,
				'Direct Debit' => ($p->participantDdBankAccountNumber ? 'yes' : 'no'),
				'Registration' => $p->participantRegistration,
				'PO Number' => $p->participantPoNumber
			);
		}

		$fieldlengths = array(
			'Customer No.'=>20,
			'Customer Name'=>30,
			'Customer Name 2'=>array(30,30),
			'Customer Address'=>30,
			'Customer Address 2'=>array(30,30),
			'Customer Post Code'=>20,
			'Customer City'=>30,
			'Customer Contact'=>30,
			'Customer Phone No.'=>30,
			'Customer Fax No.'=>30,
			'Global Dimension Code 1'=>20,
			'Global Dimension Code 2'=>20,
			'Customer Currency Code'=>10,
			'Customer Language Code'=>10,
			'Customer Country Code'=>10,
			'VAT Registration Number'=>20,
			'Customer e mail'=>80,
			'Customer Home Page'=>80,
			'Send to Name'=>30,
			'Send to Name 2'=>array(30,30),
			'Send to Address'=>30,
			'Send to Address 2'=>array(30,30),
			'Send to Post Code'=>20,
			'Send to City'=>30,
			'Send to Country Code'=>10,
			'Send to Contact'=>30,
			'ACCOUNT No 1'=>24,
			'IBAN No 1'=>50,
			'SWIFT No 1'=>50,
			'ACCOUNT No 2'=>24,
			'IBAN No 2'=>50,
			'SWIFT No 2'=>50,
			'Registration'=>30,
			'PO Number'=>30
			);

		$processors = array (
				'New Customer'=>'processTrackNew',
				'Customer Name' => 'recodeHTMLEntities',
				'Customer Name 2' => 'recodeHTMLEntities'
		);

		$date = currentDate();
		$filename = $crso->crsoId.'/'.strftime('%Y%m%d', $date->getTime()).'_MVEND.xml';

		$this->writefile($participantsData, $fieldlengths, $filename, $processors);
		info("Created export file: ".$filename);

		if(isset(self::$lastExports[$crso->crsoId]))
		{
			$diff = $this->diff(self::$lastExports[$crso->crsoId], $this->_path.$filename);
			file_put_contents($this->_path.$filename.'.diff', $diff);
			info("Created export diff file: ".$filename.'.diff');
		}
	}




	private function diff($last_export, $this_export)
	{
		debug('Running diff: '.sprintf('diff -u %s %s', $last_export, $this_export));
		$buffer = explode("\n", shell_exec(sprintf(
			'diff -u %s %s', $last_export, $this_export
		)));

        $buffer[0] = "--- LastExport";
		$buffer[1] = "+++ ThisExport";

		return implode("\n", $buffer);
	}

	protected static function generateExistingParticipants($crso_id, $writable_path)
	{
		self::$existingParticipants[$crso_id] = array();
		debug(sprintf('[masterdata] Generating previous masterdata participants for CRSO [%s]', $crso_id));
		if(is_dir($writable_path.$crso_id))
		{
			$it = new DirectoryIterator($writable_path.$crso_id);
			$files = array();
			foreach($it as $file)
			{
				if(!$it->isDot() && preg_match('/\d+_MVEND\.xml(?!\.diff)/',$file->getFilename()))
				{
					$files[] = $file->getFilename();
				}
			}
			if(count($files) < 1)
			{
				debug(sprintf('[masterdata] No previous masterdata exports for CRSO [%s]', $crso_id));
				return;
			}
			rsort($files);
			$lastFile = array_shift($files);
			self::$lastExports[$crso_id] = $writable_path.$crso_id.'/'.$lastFile;
			if(($lastExport = file_get_contents($writable_path.$crso_id.'/'.$lastFile))!== false)
			{
				preg_match_all('%<vat_registration_number>([a-zA-Z0-9\W]+)</vat_registration_number>%', $lastExport, $matches);
				self::$existingParticipants[$crso_id] = array_values($matches[1]);
			}
		}
		else
		{
			debug(sprintf('[masterdata] No writable directory for exports for CRSO [%s]', $crso_id));
		}
	}

	protected function recodeHTMLEntities($args)
	{
		return html_entity_decode($args, ENT_QUOTES, 'UTF-8');
	}

	protected function processTrackNew($args)
	{
		list ($cid, $pvat) = explode(';', $args);
		if(!isset(self::$existingParticipants[$cid]))
			self::generateExistingParticipants($cid, $this->_path);
		$existing = self::$existingParticipants[$cid];
		if(!in_array($pvat, $existing))
			return 'true';
		return 'false';
	}

}

?>
