<?php

require_once(ELC_CLASSES.'PwcExporter.php');
require_once(IWARE.'MockLanguageInterface.php');

/**
 * PwcExporterDeclarations
 * @author martijn
 */
class PwcExporterRefundRequests extends PwcExporter
{
	protected $hash_each_line_on_write = true;

	protected $baseLine = array(
		'invoice_no' => '',
		'invoice_line_no' => '',
		'declaration_id' => '',
		'line_no' => '',
		'product_id'=> '',
		'journal_template' => '',
		'customer_no' => '',
		'customer_name' => '',
		'posting_date' => '',
		'document_date' => '',
		'external_document_no' => '',
		'currency_code' => '',
		'type' => 'G/L Account',
		'gl_account_no' => '',
		'description' => '',
		'quantity'=> '',
		'unit_cost' => '',
		'line_amount_excl_vat' => '',
		'total_amount_excl_vat' => '',
		'total_amount_incl_vat' => '',
		'total_vat_amount' => '',
		'navision_vat_code' => '',
		'vat_extended_text' => '',
		'dimension_code_1' => '',
		'dimension_code_2' => '',
		'dimension_code_3' => '',
		'dimension_code_4' => '',
		'dimension_code_5' => '',
		'dimension_code_6' => '',
		'dimension_code_7' => '',
		'dimension_code_8' => '',
		'tariff_no' => '',
		'net_weight' => '',
		'transaction_type' => '',
		'exit_point' => '',
		'transport_method' => '',
		'decl_weight' => '',
		'decl_product_category' => '',
		'decl_brand' => '',
		'decl_month' => '',
		'decl_year' => '',
		'decl_type' => '',
		'decl_total' => '', // see down
		'original_invoice' => '',
		'bill_to_customer_no' => '',
		'product_description' => '',
		'hash_total' => '',
		'refund_export_date' => '',
		'document_review_no' => '',
		'refund_weight' => '',
		'refund_product_category' => '',
		'refund_brand' => '',
	);

	function exportCrso(SoapCrsoMasterData $crso, $year, $month, $day) {
		info('PwcExporterRefundRequests: exportCrso: '.$crso->crsoId);

		// check if give day is the export date for the given crso, else continue
		if (!is_null($day)) {
			// TODO: REFUND_EXPORT_DAY is per product but export is per crso? wierd
			$sql = 'SELECT REFUND_EXPORT_DAY FROM TBL_CRSO_PRODUCT WHERE CRSO_RELATION_ID=?';
			$q = $this->_db->query($sql);
			$q->bind_values($crso->crsoId);
			$q->execute();
			$r = $q->fetch();
			if ($r->REFUND_EXPORT_DAY != $day) {
				info('PwcExporterRefundRequests: skipping CRSO ['.$crso->crsoId.']: Day '.$day.' is not the export day');
				return;
			}
		}

		// get participant masterdata.. we need this for the participant invoice language
		// make a map of participantId => object
		$marterData = array();
		$soapMasterdata = $this->_soapClient->getParticipantMasterData($crso->crsoId)->values;
		foreach ($soapMasterdata as $participant) {
			$masterData[$participant->participantId] = $participant;
		}

		// get invoices for this month
		$invoices = $this->_soapClient->getRefundRequests($crso->crsoId, $year, $month)->values;
		$data = array();

		foreach ($invoices as $i=>$inv) {

			info(sprintf('PwcExporterRefundRequests: working on invoice %s/%s', $inv->invoiceYear, $inv->invoiceId));

			$invLineIndex = 0;

			// get the participant object from the masterdata map
			if (!isset($masterData[$inv->participantId]))
				throw new Exception('PwcExporterRefundRequests: participant ['.$inv->participantId.'] not in masterdata');
			$participant = $masterData[$inv->participantId];

			// get participant languageId from code.. passed into getText()
			$languageId = $this->getLanguageIdFromCode($participant->participantLanguageCode);
			//$languageId = 44;

			// build common vars

			$customerNo = $crso->crsoCountryCode.
				str_pad($crso->crsoId, 8, '0', STR_PAD_LEFT).
				str_pad($inv->participantId, 8, '0', STR_PAD_LEFT);

			$billToCustomerNo = $crso->crsoCountryCode.
				str_pad($crso->crsoId, 8, '0', STR_PAD_LEFT).
				str_pad($inv->billToParticipantId, 8, '0', STR_PAD_LEFT);

			$documentDate = new DateTime($inv->documentDate);

			// last day of the previous month is used in some places
			$lastMonthDate = new DateTime($year.'-'.$month.'-01');
			$lastMonthDate->modify('-1 day');

			// get total document review count which needs to go into each line anywho per product
			// its also used to build the extra refundFee lines which are per product
			$totalReviewCount = array();
			foreach ($inv->documentReviews->values as $docReview) {
				$productId = $this->getProductIdFromName($docReview->product);
				if (!isset($totalReviewCount[$productId]))
					$totalReviewCount[$productId] = 0;
				 $totalReviewCount[$productId] += $docReview->nrDocuments;
			}

			// copy baseline to a local baseline used for all lines and pre-fill it with common invoice data
			$baseLine = $this->baseLine;
			$baseLine['invoice_no'] = $this->buildInvoiceForExport($crso->crsoId, $inv->invoiceYear, $inv->invoiceId, ($inv->type == 'credit'), true);
			$baseLine['customer_no'] = $customerNo;
			$baseLine['journal_template'] = 'R'.($inv->type == 'debit' ? 'I' : 'C').$crso->crsoId;
			$baseLine['bill_to_customer_no'] = $billToCustomerNo;
			$baseLine['posting_date'] = $lastMonthDate->format('d/m/Y');
			$baseLine['document_date'] = $documentDate->format('d/m/Y');
			$baseLine['total_amount_excl_vat'] = number_format($inv->totalAmount, 2, ',', '');

			// this is a bit wierd.. keep the last $originalInvoiceNr outside the scope of the the rr loop
			// because its needed in the refund fee too
			$originalInvoiceNr = null;

			foreach ($inv->refundRequests->values as $j=>$rr) {
				//info('PwcExporterRefundRequests: working on refundRequest '.$rr->refundRequestId);

				$totalQuantity = 0;

				$rrPostingDate = new DateTime($rr->postingDate);

				$lines = array();
				foreach ($rr->lines->values as $rrIndex=>$rrLine) {

					$invLineIndex++;
					$productId = $this->getProductIdFromName($rr->product);

					$postingDate = new DateTime($rr->postingDate);

					$line = $baseLine; // copy
					$line['invoice_line_no'] = $invLineIndex;
					$line['declaration_id'] = $rr->refundRequestId;
					$line['line_no'] = $rrIndex+1;
					$line['product_id'] = $productId;
					$line['gl_account_no'] = $rrLine->glAccount;
					$line['description'] = 'PERIOD: ' . $postingDate->format('Y-m-d');
					$line['quantity'] = $rrLine->quantity;
					$line['unit_cost'] = number_format($rrLine->unitCost, 8, ',', '');
					$line['line_amount_excl_vat'] = number_format($rrLine->amount, 2, ',', '');
					$line['net_weight'] = (int) $rrLine->weight;
					$line['product_description'] = $rr->product;
					$line['hash_total'] = $this->_getHashTotal($inv->type, $invLineIndex, $year, $inv->invoiceId, $crso->crsoId);
					$line['document_review_no'] = isset($totalReviewCount[$productId]) ? $totalReviewCount[$productId] : 0;
					$line['refund_weight'] = $rrLine->weight;
					$line['refund_product_category'] = $rrLine->productCategory;
					$line['refund_export_date'] = $postingDate->format('d/m/Y');
					$line['refund_brand'] = $rrLine->brand;

					if ($rr->originalInvoiceId)
						$line['original_invoice'] = $originalInvoiceNr = $this->buildInvoiceNoForInterfaces($crso->crsoId, $rr->originalInvoiceYear, $rr->originalInvoiceId, false, false);
					else
						$line['original_invoice'] = $originalInvoiceNr = '';

					$lines[] = $line;
				}

				// merge $lines into $data
				$data = array_merge($data, $lines);
			}

			/**
			 * document reviews (summed per product)
			 */
			$docLines = array();
			$product = null;
			$line = null;


			info('PwcExporterRefundRequests: docReview count: '.count($inv->documentReviews));
			foreach ($inv->documentReviews->values as $docReview) {

				$invLineIndex++;

				$line = $baseLine; // copy
				$line['invoice_line_no'] = $invLineIndex;
				$line['description'] = $this->_langIf->getText('sum of document review fee', null, array(), $languageId);
				$line['product_id'] = $this->getProductIdFromName($docReview->product);
				$line['gl_account_no'] = $docReview->glAccount;
				$line['document_review_no'] = $docReview->nrDocuments;
				$line['quantity'] = $inv->type == 'debit' ? 1 : -1;
				$line['unit_cost'] = number_format(0, 8, ',', '');
				$line['line_amount_excl_vat'] = number_format($docReview->amount, 2, ',', '');
				$line['product_description'] = $docReview->product;
				$line['hash_total'] = $this->_getHashTotal($inv->type, $invLineIndex, $year, $inv->invoiceId, $crso->crsoId);
				$line['original_invoice'] = $originalInvoiceNr;

				$docLines[] = $line;
			}

			$data = array_merge($data, $docLines);

			//
			// refund fee
			//
			$feeLines = array();
			foreach ($inv->refundFees->values as $refundFee) {

				$invLineIndex++;

				$productId = $this->getProductIdFromName($refundFee->product);

				$line = $baseLine; // copy
				$line['invoice_line_no'] = $invLineIndex;
				$line['product_id'] = $productId;
				$line['gl_account_no'] = $refundFee->glAccount;

				$line['description'] = $this->_langIf->getText('refund fee', null, array(), $languageId);
				$line['quantity'] = $inv->type == 'debit' ? 1 : -1;
				$line['unit_cost'] = number_format($refundFee->fee, 8, ',', '');
				$line['line_amount_excl_vat'] = number_format($refundFee->amount, 2, ',', '');
				$line['product_description'] = $refundFee->product;
				$line['hash_total'] = $this->_getHashTotal($inv->type, $invLineIndex, $year, $inv->invoiceId, $crso->crsoId);
				$line['document_review_no'] = isset($totalReviewCount[$productId]) ? $totalReviewCount[$productId] : 0;
				$line['original_invoice'] = $originalInvoiceNr;

				info('PwcExporterRefundRequests: adding refundFee line with index '.$invLineIndex);
				$feeLines[] = $line;

			}
			$data = array_merge($data, $feeLines);

		}

		$date = new IWDateTime(sprintf('%s-%s-%s', $year, $month, $day));
		$filename = $crso->crsoId.'/'.strftime('%Y%m%d', $date->getTime()).'_TRASALES_RR.xml';

		$fieldlengths = array();
		$processors = array();
		$this->writefile($data, $fieldlengths, $filename, $processors);
	}



	/**
	 * gets the hash_total for the given arguments
	 * @param unknown_type $type
	 * @param unknown_type $invLineIndex
	 * @param unknown_type $year
	 * @param unknown_type $invNr
	 * @param unknown_type $crsoId
	 */
	protected function _getHashTotal($type, $invLineIndex, $year, $invNr, $crsoId) {

		if ($type == 'debit')
		{
			$sql = "SELECT HASH_TOTAL
				FROM TBL_REFUND_EXPORT_DEBIT_LINE
				WHERE INVOICE_LINE_INDEX=?
					AND YEAR=?
					AND REFUND_REQUEST_DEBIT_ID=?
					AND CRSO_RELATION_ID=?
				";
		} elseif ($type == 'credit')
		{
			$sql = "SELECT HASH_TOTAL
				FROM TBL_REFUND_EXPORT_CREDIT_LINE
				WHERE INVOICE_LINE_INDEX=?
					AND YEAR=?
					AND REFUND_REQUEST_CREDIT_ID=?
					AND CRSO_RELATION_ID=?
				";
		}
		else {
			throw new Exception('_getHashTotal: Unknown type: '.$type);
		}

		$q = $this->_db->query($sql);
		$q->bind_values($invLineIndex, $year, $invNr, $crsoId);
		$q->execute();

		if (!$row = $q->fetch())
			throw new Exception(sprintf('_getHashTotal: no credit/debit line for %s/%s/%s/%s/%s', $type, $invLineIndex, $year, $invNr, $crsoId));

		return $row->HASH_TOTAL;

	}



}

?>
