<?php
require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES.'ProofOfPayment.php');
require_once(ELC_CLASSES.'Product.php');
require_once(ELC_CLASSES.'RefundRequest.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'ProofOfPaymentExportQuantity.php');
require_once(ELC_CLASSES.'RefundExportDocumentReview.php');
require_once(ELC_CLASSES.'Document.php');
require_once(ELC_CLASSES.'Country.php');

/**
 *
 * @author Nick Loeve <nick@basesoft.com>
 * @package ELC
 * @subpackage RefundRequest
 * @copyright 2006, BaseSoft B.V.
 * @version $Id$
 */
class RefundExport extends AbstractObject
{
	/**
	 * @var int $id
	 */
    var $id;
    /**
     * @var ELCDocument $invoice_document
     */
    var $invoice_document;
    /**
     * @var string $export_date
     */
    var $export_date;
    /**
     * @var float $refund_cost
     */
    var $refund_cost;
    /**
     * @var bool $is_exported
     */
    var $is_exported;
    /**
     * @var Participant $participant
     */
    var $participant;
    /**
     * @var CRSO $crso
     */
    var $crso;
    var $product_id;
    var $bill_to_customer_no;
	/**
	 * Used for exporting the refunds. This calculates the amounts that will be exported etc.
	 * @static
	 * @param Database &$db
	 * @param int $participant_id
	 * @param int $crso_id
	 */
	public static function phaseOne(&$db, $part_id, $crso_id,$products)
	{
		$db->begin();
		$reArray = array();
		$crso = new CRSO($db, array('relation' => new Relation($db, array('id' => $crso_id))));
		$country = new Country($db, $crso->getCountry(), true);

		foreach($products as $product_id)
		{
			$totalRefundCost = 0;

			if($country->has(HasCreditViaProducer,$product_id))
			{
				if(!$country->has(HasRefundInternalVAT,$product_id))
					throw new IWException('Invalid Country Settings Combination');

				$re = new RefundExport($db);
				$rrExport = $re->getExportableRefundRequestsByProducer($crso_id, $part_id,$product_id);

				if(!is_array($rrExport))
					continue;

				// Loop through all RefundRequests, set it
				$rrProducer = RefundExport::buildRefundsByProducer($rrExport);
				foreach($rrProducer as $producer_id => $rrExport)
				{
					$re = new RefundExport($db);
					$re->is_exported = REFUND_EXPORT_PENDING;
					$re->participant = new Participant($db, array('relation' => new Relation($db, array('id' => $part_id))));
					$re->crso = $crso;
					$re->product = new Product($db,$product_id);
					$re->bill_to_customer_no = $producer_id;
					$totalRefundCost += RefundExport::buildRRForRefundExport($re,$rrExport,false);
					$re->id && $reArray[$producer_id][] = $re->id;
				}

				$re = new RefundExport($db);
				$re->is_exported = REFUND_EXPORT_PENDING;
				$re->participant = new Participant($db, array('relation' => new Relation($db, array('id' => $part_id))));
				$re->crso = $crso;
				$re->product = new Product($db,$product_id);
				RefundExport::buildDocReviewForRefundExport($re,$re->getExportableDocumentReviews($crso_id, $part_id,$product_id),$totalRefundCost);
				$re->id && $reArray[0][] = $re->id;
			}
			else
			{
				$re = new RefundExport($db);
				$re->is_exported = REFUND_EXPORT_PENDING;
				$re->participant = new Participant($db, array('relation' => new Relation($db, array('id' => $part_id))));
				$re->crso = $crso;
				$re->product = new Product($db,$product_id);

				if($country->has(HasRefundInternalVAT,$product_id))
					$rrExport = $re->getExportableRefundRequestsByProducer($crso_id, $part_id,$product_id);
				else
					$rrExport = $re->getExportableRefundRequests($crso_id, $part_id,$product_id);

				$docExport = $re->getExportableDocumentReviews($crso_id, $part_id,$product_id);

				RefundExport::buildRRForRefundExport($re,$rrExport);
				RefundExport::buildDocReviewForRefundExport($re,$docExport);
				$re->id && $reArray[0][] = $re->id;
			}
		}
		if(!count($reArray))
		{
			$db->commit();
			return;
		}

		foreach($reArray as $producer => $reArr)
			RefundExport::createDebitCredit($reArr,$crso_id);
		$db->commit();
	}

	public static function buildRRForRefundExport(RefundExport $re, $rrExport, $addRefundCost=true)
	{
		if(!is_array($rrExport)||!count($rrExport))
			return false;

		$re->id || $re->insert();

		foreach($rrExport as $rr)
		{
			debug(sprintf("RefundExport:: exporting RR [%s]", $rr));
			$re->setupExportRefundRequest($rr);
		}

		require_once(ELC_CLASSES.'CRSOAdminFee.php');
		$fee = new CRSOAdminFee(IWRegistry::registry('db'));
		$cost = $fee->getCurrentFee($re->crso->relation->id, CRSO_ADMIN_FEE_REFUND_EXPORT);
		$re->refund_cost = $addRefundCost?$cost:null;
		$re->update();
		return $cost;
	}

	public static function buildDocReviewForRefundExport(RefundExport $re, $docExport, $addRefundCost=false)
	{
		if(!is_array($docExport)||!count($docExport))
			return false;

		$re->id || $re->insert();
		foreach($docExport as $doc)
			$re->setUpExportDocumentReviews($doc);
		$addRefundCost && $re->refund_cost = $addRefundCost;
		$re->update();
	}

	public static function buildRefundsByProducer(Array $rrExport)
	{
		$rv = array();
		$db = IWRegistry::registry('db');
		$rr = new RefundRequest($db);

		foreach($rrExport as $rr_id)
		{
			$rr->set($rr_id);
			$producer_id = null;
			foreach($rr->lines as $line)
			{
				$producer_id || $producer_id = $line->producer_relation->id;
				if($producer_id != $line->producer_relation->id)
					continue 2;
			}
			$rv[$producer_id][] = $rr_id;
		}

		return $rv;
	}


	/**
	 * Sets a refund export by its ID
	 * @param int $id
	 * @return bool
	 */
    function set($id=null)
    {
        $id || $id = $this->id;
        $id = (int) $id;

        $sql = "SELECT REFUND_EXPORT_ID,EXPORT_DATE,REFUND_COST,INVOICE_DOCUMENT_ID,
				IS_EXPORTED,PARTICIPANT_RELATION_ID,CRSO_RELATION_ID, PRODUCT_ID, BILL_TO_CUSTOMER_NO
			FROM TBL_REFUND_EXPORT WHERE REFUND_EXPORT_ID=?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		$this->id = $r->REFUND_EXPORT_ID;
		if($r->INVOICE_DOCUMENT_ID)
		{
			$this->invoice_document = Document::createDocument($this->db);
			$this->invoice_document->set($r->INVOICE_DOCUMENT_ID);
		}
		else
		{
			$this->invoice_document = null;
		}
		require_once(ELC_CLASSES.'Product.php');
		$this->product = new Product($this->db,$r->PRODUCT_ID);
		$this->export_date = $r->EXPORT_DATE;
		$this->refund_cost = $r->REFUND_COST;
		$this->is_exported = $r->IS_EXPORTED;
		$this->participant = new Participant($this->db, array('relation' => new Relation($this->db, array('id' => $r->PARTICIPANT_RELATION_ID))));
		$this->crso = new CRSO($this->db, array('relation' => new Relation($this->db, array('id' => $r->CRSO_RELATION_ID))));
		$this->bill_to_customer_no = $r->BILL_TO_CUSTOMER_NO;
		return $this->complete = true;
    }

    /**
	 * Get the virtual stock for a CRSOParticipant
	 * @param int $crso_id CRSO relation id
	 * @param int $part_id Participant relation id
	 * @return mixed returns virtual stock or false on error
	 */
	function getVirtualStock($crso_id, $part_id, $producerId = null, $product_id)
	{
		$sql = "SELECT f_elcGetVirtualStock(?, ?, ?, ?) AS POPEQ_QUANTITY";
		$q = $this->db->query($sql);
		$q->bind_values($part_id, $crso_id, $producerId, $product_id);
		if(!$q->execute())
			return false;
		if(!($r = $q->fetch()))
			return 0;
		return (int)$r->{'POPEQ_QUANTITY'};
	}

	/**
	 * Gets the total amount for confirmed refund requests. Can be used to check
	 * together with getVirtualStock to check if there is sufficient stock
	 * available.
	 */
	function getRefundStock($crso_id, $part_id, $product_id, $producer_id=null)
	{
		$sql = "SELECT f_elcGetRefundStock(?, ?, ?, ?) AS QUANTITY";
		$values = array($part_id, $crso_id, $product_id, $producer_id);
		$q = $this->db->query($sql);
		$q->bind_values($values);
		if(!$q->execute())
			return false;
		if (!$r = $q->fetch())
			return 0;
		return (int) $r->{'QUANTITY'};
	}

	/**
	 * # Returns the quantity of refundStock - virtualStock. This can be positive or
	 * # negative (if you have more refund requests than virtual stock)
	 */
	function getRealStock($crso_id, $part_id, $product_id, $producer_id=null)
	{
		$sql = "SELECT f_elcGetRealVirtualStock(?, ?, ?, ?) AS QUANTITY";
		$values = array($part_id, $crso_id, $product_id, $producer_id);
		$q = $this->db->query($sql);
		$q->bind_values($values);
		if(!$q->execute())
			return false;
		if (!$r = $q->fetch())
			return 0;
		return (int) $r->{'QUANTITY'};
	}

    /**
	 * Get the RefundRequests that can be exported based on available virtual stock
	 * @param int $crso_id CRSO relation id
	 * @param int $part_id Participant relation id
	 * @return mixed returns array of RR ids or false on error
	 */
    function getExportableRefundRequests($crso_id, $part_id,$product_id)
    {
    	$rrForExport = array();

    	//get vStock
    	$vStock = $this->getVirtualStock($crso_id, $part_id,null, $product_id);
    	//debug(sprintf("RefundExport:: virtual stock for CRSO [%s] Participant [%s] is: [%s]", $crso_id, $part_id, $vStock));
    	if(!$vStock)
    		return false;

    	//get RefundRequests that are confirmed but not exported ORDER by creation time
    	$sql = "
				SELECT
					RR.REFUND_REQUEST_ID,
					SUM(RRL.QUANTITY) AS QUANTITY
				FROM
					TBL_REFUND_REQUEST RR
				INNER JOIN TBL_REFUND_REQUEST_LINE RRL USING(REFUND_REQUEST_ID)
				WHERE ISNULL(RR.REFUND_EXPORT_ID)
				AND RR.STATUS=?
				AND RR.CRSO_RELATION_ID = ?
				AND RR.PARTICIPANT_RELATION_ID = ?
				AND RR.PRODUCT_ID = ?
				GROUP BY RR.REFUND_REQUEST_ID
				ORDER BY RR.CREATED
				";
		$q = $this->db->query($sql);
		$q->bind_values(REFUND_REQUEST_CONFIRMED, $crso_id, $part_id, $product_id);
		if(!$q->execute())
			return false;
		debug(sprintf('RefundExport:: We are starting with %d virtual stock for CRSO [%s] Participant [%s]', $vStock, $crso_id, $part_id));
		while($r = $q->fetch())
		{
			if(((int)$r->{'QUANTITY'}) <= $vStock)
			{
				$rrForExport[] = $r->{'REFUND_REQUEST_ID'};
				$vStock -= (int)$r->{'QUANTITY'};
				debug(sprintf('RefundExport:: We are subtracting %d virtual stock for CRSO [%s] Participant [%s] and exporting RefundRequest [%s]', (int)$r->{'QUANTITY'}, $crso_id, $part_id, $r->{'REFUND_REQUEST_ID'}));
			}
			else
				break;
		}
		return  $rrForExport;
    }

	/**
	 * Selects id, quantities, amount and date from the refund export for the
	 * given arguments.
	 * Used in InterfaceRefundProcessStatus (refund index)
	 */
	function selectLastExportDetails($crso_id, $part_id, $product_id)
	{
		$sql = 'SELECT
					SUM(RRL.QUANTITY) AS QUANTITY,
					SUM(RRL.EXPORTED_AMOUNT) as AMOUNT,
					RR.REFUND_EXPORT_DATE,
					RE.INVOICE_DOCUMENT_ID as INVOICE_DOCUMENT
				FROM
					TBL_REFUND_REQUEST RR
				INNER JOIN
					TBL_REFUND_REQUEST_LINE RRL USING(REFUND_REQUEST_ID)
				INNER JOIN
					TBL_REFUND_EXPORT RE USING (REFUND_EXPORT_ID)
				WHERE RR.REFUND_EXPORT_ID = (
						SELECT MAX(RR.REFUND_EXPORT_ID)
						FROM
							TBL_REFUND_REQUEST RR
						WHERE
							RR.CRSO_RELATION_ID = :crso_id
							AND RR.PARTICIPANT_RELATION_ID = :part_id
							AND RR.PRODUCT_ID = :product_id
					)
				AND RR.CRSO_RELATION_ID = :crso_id
				AND RR.PARTICIPANT_RELATION_ID = :part_id
				AND RR.PRODUCT_ID = :product_id

				GROUP BY RR.PARTICIPANT_RELATION_ID
			';
		$q = $this->db->query($sql);
		$q->bindParam(':crso_id', $crso_id);
		$q->bindParam(':part_id', $part_id);
		$q->bindParam(':product_id', $product_id);

		$q->execute();
		return $q;
	}

    function getExportableRefundRequestsByProducer($crso_id, $part_id, $product_id)
    {
    	$vStock = $rrForExport = array();
    	$pop = new ProofOfPayment($this->db);
    	$producers = $pop->getProducers($crso_id, $part_id, PROOF_OF_PAYMENT_APPROVED, $product_id);

    	foreach($producers as $producerId)
    	{
	    	//get vStock
	    	$tempStock = $this->getVirtualStock($crso_id, $part_id, $producerId, $product_id);
	    	if($tempStock)
	    		$vStock[$producerId] = $tempStock;
    	}

    	if(count($vStock) < 1)
	    	return false;

	   	$sql = "
				SELECT
					RR.REFUND_REQUEST_ID,
					SUM(RRL.QUANTITY) AS QUANTITY,
					RRL.PRODUCER_RELATION_ID
				FROM
					TBL_REFUND_REQUEST RR
				INNER JOIN TBL_REFUND_REQUEST_LINE RRL USING(REFUND_REQUEST_ID)
				WHERE ISNULL(RR.REFUND_EXPORT_ID)
				AND RR.STATUS=?
				AND RR.CRSO_RELATION_ID = ?
				AND RR.PARTICIPANT_RELATION_ID = ?
				AND RR.PRODUCT_ID = ?
				GROUP BY RR.REFUND_REQUEST_ID, RRL.PRODUCER_RELATION_ID
				ORDER BY RR.CREATED, RR.REFUND_REQUEST_ID
				";
		$q = $this->db->query($sql);
		$q->bind_values(REFUND_REQUEST_CONFIRMED, $crso_id, $part_id, $product_id);
		if(!$q->execute())
			return false;
		$approvedRefund = $tempRRId = 0;
		$invalidProducers = array();

		while($r = $q->fetch())
		{
			if($tempRRId != $r->REFUND_REQUEST_ID)
			{
				if($approvedRefund > 0)
				{
					$rrForExport[] = $approvedRefund;
					debug(sprintf('RefundExport:: Exporting out the RR [%s]', $approvedRefund));
				}

				$approvedRefund = $tempRRId = $r->REFUND_REQUEST_ID;
			}
			if(isset($vStock[$r->PRODUCER_RELATION_ID])&&!in_array($r->PRODUCER_RELATION_ID,$invalidProducers)&&
				($vStock[$r->PRODUCER_RELATION_ID]>=$r->QUANTITY))
			{
				$vStock[$r->PRODUCER_RELATION_ID] -= $r->QUANTITY;
				debug(sprintf('RefundExport:: Enough virtual stock for producer - RR [%s], PROD [%s], VSTOCK [%s]', $r->REFUND_REQUEST_ID, $r->PRODUCER_RELATION_ID, $vStock[$r->PRODUCER_RELATION_ID]));
			}
			else
			{
				$approvedRefund = 0;
				$invalidProducers[$r->PRODUCER_RELATION_ID] = $r->PRODUCER_RELATION_ID;
				debug(sprintf('RefundExport:: Not enough virtual stock or invalid producer - RR [%s], PROD [%s], VSTOCK [%s]', $r->REFUND_REQUEST_ID, $r->PRODUCER_RELATION_ID, isset($vStock[$r->PRODUCER_RELATION_ID]) ? $vStock[$r->PRODUCER_RELATION_ID] : null));
			}
		}
		if($approvedRefund > 0)
		{
				$rrForExport[] = $approvedRefund;
				debug(sprintf('RefundExport:: Exporting out the RR [%s]', $approvedRefund));
		}
		return  $rrForExport;
    }


    /**
     * Sets up the export of refund request
     * @param int $rr_id REFUND_REQUEST_ID
     */
    function setupExportRefundRequest($rr_id)
    {
    	if(!$this->isComplete())
    		$this->set();

    	$rr = new RefundRequest($this->db, $rr_id, true);
    	//$this->_allocateQuantityAgainstProofOfPayment($rr->getQuantity());
    	$rr->refund_export =& $this;
    	$rr->update();
    	$rr->saveExportedAmount();
    }
    /**
     * Gets the exportable document reviews
     * @param int $crso_id
     * @param int $participant_id
     * @return bool|array $docsForExport
     */
    function &getExportableDocumentReviews($crso_id, $participant_id,$product_id)
    {
    	$docsForExport = array();
    	$sql = "
				SELECT DOCUMENT_ID FROM TBL_REFUND_EXPORT_DOCUMENT_REVIEW
				WHERE CRSO_RELATION_ID=? AND PARTICIPANT_RELATION_ID=? AND PRODUCT_ID = ?
				AND ISNULL(REFUND_EXPORT_ID)
					";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, $participant_id,$product_id);
		if(!$q->execute())
			return false;

		while($r = $q->fetch())
			$docsForExport[] = $r->{'DOCUMENT_ID'};

		return $docsForExport;
    }

    /**
     * returns all review_costs for the given refund_export_id
     */
    function getExportedDocumentReviews($refund_export_id) {
    	$sql = "
    		SELECT COUNT(r.DOCUMENT_ID) AS DOCUMENT_COUNT, SUM(REVIEW_COST) as REVIEW_COST, p.NAME as PRODUCT_NAME
    		FROM TBL_REFUND_EXPORT_DOCUMENT_REVIEW r
    		INNER JOIN TBL_PRODUCT p ON r.PRODUCT_ID = p.PRODUCT_ID
    		WHERE REFUND_EXPORT_ID=?
    		GROUP BY r.PRODUCT_ID ORDER BY r.PRODUCT_ID";

    	$q = $this->db->query($sql);
		$q->bind_values($refund_export_id);
		$q->execute();
		return $q;
    }

    function getExportedRefundFees($refund_export_id) {
    	$sql = "
    		SELECT
			p.NAME AS PRODUCT_NAME,
				(SELECT GL_ACCOUNT
				FROM TBL_CRSO_GLACCOUNT gc
				INNER JOIN TBL_DECLARATION_TYPE dt
					ON gc.DECLARATION_TYPE_ID = dt.DECLARATION_TYPE_ID
				WHERE gc.CRSO_RELATION_ID = 2
					AND dt.HANDLE = 'DT_REFUND_ADMINISTRATIVE_FEE')
			AS GL_ACCOUNT_NO,
			IFNULL(re.REFUND_COST, '0.00') AS REFUND_COST
			FROM TBL_REFUND_EXPORT AS re
			INNER JOIN TBL_PRODUCT AS p
				ON re.PRODUCT_ID = p.PRODUCT_ID
			 INNER JOIN TBL_RELATION AS cr
			     ON cr.RELATION_ID = re.CRSO_RELATION_ID
			 INNER JOIN TBL_ADDRESS AS ca
			     ON cr.ADDRESS_ID = ca.ADDRESS_ID
			 INNER JOIN TBL_COUNTRY AS cac
			     ON cac.COUNTRY_ID = ca.COUNTRY_ID
			 INNER JOIN TBL_REFUND_EXPORT_SET_LINE resl
			     ON re.REFUND_EXPORT_ID = resl.REFUND_EXPORT_ID
			 LEFT JOIN TBL_REFUND_EXPORT_DEBIT AS red
			     ON red.REFUND_EXPORT_SET_ID = resl.REFUND_EXPORT_SET_ID
			 LEFT JOIN TBL_REFUND_EXPORT_CREDIT AS rec
			     ON rec.REFUND_EXPORT_SET_ID = resl.REFUND_EXPORT_SET_ID
			 WHERE re.REFUND_EXPORT_ID = ?
    	";

    	$q = $this->db->query($sql);
		$q->bind_values($refund_export_id);
		$q->execute();
		return $q;

    }

    /**
     * Sets up the export of the document reviews by linking the right export id to the right document-review
     * @param int $doc_id DOCUMENT_ID
     */
    function setUpExportDocumentReviews($doc_id)
    {
    	$tempDoc = Document::createDocument($this->db);
    	$tempDoc->set($doc_id);
    	$doc = new RefundExportDocumentReview($this->db, array('document' => $tempDoc), true);
    	$doc->refund_export =& $this;
    	$doc->update();

    }
    /**
     * Calculates the total cost of a refund export
     * @return float
     * @todo Mkpretty
     */
    public static function calculateTotalCost($res)
    {
    	$db = IWRegistry::registry('db');
    	$re = new RefundExport($db);
    	$redr_cost = $rr_export_amount = $re_cost = $total_cost = 0;
    	foreach($res as $re_id)
    	{
    		$re->set($re_id);
	    	$re_cost += $re->refund_cost;
	    	debug(sprintf("RefundExport:: Refund Export Cost for Export [%s] is: %.2F", $re->id, $re_cost));
	    	$doc = new RefundExportDocumentReview($db);
	    	$redr_cost += $doc->getExportCost($re->id);
	    	$rr = new RefundRequest($db);
	    	$rr_export_amount += $rr->getExportAmount($re->id);
	    	debug(sprintf("RefundExport:: Total amount for Export [%s] is: %.2F", $re->id, ($re_cost + $redr_cost) - $rr_export_amount));
    	}
		$total_cost = ($re_cost + $redr_cost) - $rr_export_amount;
    	$total = sprintf("%.2F", $total_cost);
    	return $total;

    }
    /**
     * Creates the debit or credit, depending on the total cost (<0 is credit)
     */
    public static function createDebitCredit($res,$crso_id)
	{
		//get total cost
		$total = RefundExport::calculateTotalCost($res);
		if($total=='0.00')
			return;
		$db = IWRegistry::registry('db');
		$date = currentDate();

		//if we have a positive total we have debit
		$type = $total{0} == '-' ? 'CREDIT' : 'DEBIT';

		$sql = "INSERT INTO TBL_REFUND_EXPORT_SET (CREATED) VALUES(NOW())";
		$q1 = $db->query($sql);
		$q1->execute();
		$last_set_id = $q1->lastid();
		foreach($res as $refund_export)
		{
			$q1 = $db->query('INSERT INTO TBL_REFUND_EXPORT_SET_LINE(REFUND_EXPORT_SET_ID, REFUND_EXPORT_ID) VALUES (?, ?)');
			$q1->bind_values($last_set_id, $refund_export);
			$q1->execute();
		}

		//get startID
		$sql = "SELECT @creditdebitno:=IFNULL(MAX(REFUND_REQUEST_".strtoupper($type)."_ID), 0)
				FROM TBL_REFUND_EXPORT_".strtoupper($type)."
				WHERE CRSO_RELATION_ID = ? AND `YEAR` = '".$date->getYear()."'";
		$q = $db->query($sql);
		$q->bind_values($crso_id);
		$q->execute();
		// insert the new numbers
		$sql = "INSERT INTO TBL_REFUND_EXPORT_".strtoupper($type)."
				(REFUND_REQUEST_".strtoupper($type)."_ID, REFUND_EXPORT_SET_ID, CRSO_RELATION_ID, `YEAR`)
					SELECT
					@creditdebitno:=@creditdebitno+1 as REFUND_REQUEST_".strtoupper($type)."_ID,
					? as REFUND_EXPORT_SET_ID,
					? as CRSO_RELATION_ID,
					'".$date->getYear()."'
					";
		$q = $db->query($sql);
		$q->bind_values($last_set_id, $crso_id);
		$q->execute();
	}

    /**
     * Compatibility code with Declaration, used in ELCImporter
     * @param int $debit_id
     * @param int $crso_id
     * @return bool|int
     */
    function getByDebitID($debit_id, $crso_id=null,$year)
	{
		return $this->getByCreditOrDebitID($debit_id,$crso_id,'DEBIT',$year);
	}
	/**
	 * Compatibility code with Declaration, used in ELCImporter
	 */
	function getByCreditID($credit_id,$crso_id=null,$year)
	{
		return $this->getByCreditOrDebitID($credit_id,$crso_id,'CREDIT',$year);
	}
	/**
	 * Compatibility code with declaration, used in ELCImporter
	 * @param int $id
	 * @param int $crso_id
	 * @param string $type
	 * @return bool|int
	 */
	function getByCreditOrDebitID($id,$crso_id,$type,$year)
	{
		$crso_id || $crso_id = $this->crso->relation->id;

		if (!is_numeric($crso_id))
			trigger_error('RefundExport::getByDebitID() - CRSO is not numeric', E_USER_ERROR);

		$sql = "SELECT REFUND_EXPORT_ID
			FROM TBL_REFUND_EXPORT_".$type." as O
			INNER JOIN TBL_REFUND_EXPORT_SET_LINE as RESL
				ON O.REFUND_EXPORT_SET_ID = RESL.REFUND_EXPORT_SET_ID
			WHERE REFUND_REQUEST_".$type."_ID = ?
				AND CRSO_RELATION_ID=?
				AND `YEAR` = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id, $crso_id, $year);
		$q->execute();
		$rs = $q->fetchAll();
		if(!count($rs))
			return false;
		$rv = array();
		foreach($rs as $r)
			$rv[] = $r->REFUND_EXPORT_ID;
		return $rv;
	}

    /**
     * Initializes the objectmaps used in AbstractObject
     */
    function _initObjectMaps()
    {
        $this->objectmap['dbtable'] = 'TBL_REFUND_EXPORT';
        $this->objectmap['primary']['id'] = 'REFUND_EXPORT_ID';
        $this->objectmap['properties']['invoice_document->id'] = 'INVOICE_DOCUMENT_ID';
        $this->objectmap['properties']['product->id'] = 'PRODUCT_ID';
        $this->objectmap['properties']['export_date'] = 'EXPORT_DATE';
        $this->objectmap['properties']['refund_cost'] = 'REFUND_COST';
        $this->objectmap['properties']['is_exported'] = 'IS_EXPORTED';
        $this->objectmap['properties']['participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
        $this->objectmap['properties']['crso->relation->id'] = 'CRSO_RELATION_ID';
        $this->objectmap['properties']['bill_to_customer_no'] = 'BILL_TO_CUSTOMER_NO';
	}
    /**#@-*/
   	/**#@+
	 * For the requested Credit/Debit IDs by the operator, see RequestedInvoice.php
	 */
	/**
	 * Gets a new ID
	 * @access protected
	 * @param string $type CREDIT|DEBIT
	 * @param int $crso_id CRSO_RELATION_ID
	 * @return int $type_ID
	 */
	function _getNewID($type,$crso_id,$year)
	{
		$sql = "SELECT MAX(REFUND_REQUEST_".$type."_ID) as NEWID
			FROM TBL_REFUND_EXPORT_".$type."
			WHERE CRSO_RELATION_ID = ?
				AND `YEAR` = ?";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$year);
		$q->execute();
		if(!($r = $q->fetch()))
			return 1;
		return (int) $r->NEWID+1;
	}
	/**
	 * Inserts the requestedInvoice into the right table ($type)
	 * @param string $type
	 * @param int $invoice_id
	 * @return bool
	 */
	function insertRequestedInvoice($type,$invoice_id,$year)
	{
		$id = $this->_getNewId($type,$this->crso->relation->id,$year);
		$sql = "INSERT INTO TBL_REFUND_EXPORT_".$type."
			(CRSO_RELATION_ID,REFUND_REQUEST_".$type."_ID,REQUESTED_INVOICE_ID,`YEAR`)
			VALUES(?,?,?,?)";
		$q = $this->db->query($sql);
		$q->bind_values($this->crso->relation->id,$id,$invoice_id,$year);
		return $q->execute();
	}

	function selectExportedRefundRequests($crso_id, $year, $month) {

		require_once(ELC_CLASSES.'CRSOAdminFee.php');

		// this selects all exported refund requests
		$sql = "
			SELECT

			-- invoice level
			IF(
				ISNULL(red.REFUND_REQUEST_DEBIT_ID),
				CONCAT(
					'RC',
					re.CRSO_RELATION_ID,
					f_elcBuildInvoiceNoForExport(re.CRSO_RELATION_ID,rec.`YEAR`,rec.REFUND_REQUEST_CREDIT_ID,TRUE,TRUE)
				),
				CONCAT(
					'RI',
					re.CRSO_RELATION_ID,
					f_elcBuildInvoiceNoForExport(re.CRSO_RELATION_ID,red.`YEAR`,red.REFUND_REQUEST_DEBIT_ID,FALSE,TRUE)
				)
			) AS _INVOICE_NUMBER, -- for ordering only

			IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID),
				rec.`YEAR`,
				red.`YEAR`
			) as INVOICE_YEAR,

			IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID),
				rec.REFUND_REQUEST_CREDIT_ID,
				red.REFUND_REQUEST_DEBIT_ID
			) as INVOICE_ID,

			IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID),'credit','debit') AS TYPE,
			re.PARTICIPANT_RELATION_ID,
			re.CRSO_RELATION_ID,
			re.EXPORT_DATE as DOCUMENT_DATE,
			re.REFUND_EXPORT_ID,

			-- refund-request level
			rrl.REFUND_REQUEST_ID,
			p.NAME as PRODUCT_NAME,
			IFNULL(rr.REFUND_EXPORT_DATE, '') as POSTING_DATE,

			IFNULL(re.BILL_TO_CUSTOMER_NO, re.PARTICIPANT_RELATION_ID) as BILL_TO_PARTICIPANT_ID,

			IF(NOT ISNULL(re.BILL_TO_CUSTOMER_NO),
				(
					SELECT
					CONCAT(DI.`YEAR`, '-', DI.DECLARATION_DEBIT_ID)
					FROM TBL_DECLARATION as D
					INNER JOIN TBL_DECLARATION_SET_LINE AS DSL ON D.DECLARATION_ID = DSL.DECLARATION_ID
					INNER JOIN TBL_DECLARATION_DEBIT AS DI ON DSL.DECLARATION_SET_ID = DI.DECLARATION_SET_ID
					WHERE D.PARTICIPANT_RELATION_ID = re.BILL_TO_CUSTOMER_NO
					AND D.CRSO_RELATION_ID = re.CRSO_RELATION_ID
					AND D.PRODUCT_ID = re.PRODUCT_ID
					AND NOT ISNULL(D.EXPORT_DATE)
					AND D.EXPORT_DATE < ?
					ORDER BY D.DECLARATION_ID DESC
					LIMIT 1

				), '-'
			) AS ORIGINAL_INVOICE_DATA,

			IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID),
				recl.REFUND_PRODUCT_CATEGORY,
				redl.REFUND_PRODUCT_CATEGORY
			) as CATEGORY_NAME,

			cga3.GL_ACCOUNT as DOCUMENT_REVIEW_GLACCOUNT,

			-- line level
			IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID),
					IF(IFNULL(rrl.QUANTITY, 0) >= 0,
						ABS(IFNULL(rrl.QUANTITY, 0)),
						-(ABS(IFNULL(rrl.QUANTITY, 0)))
					),
					IF(IFNULL(rrl.QUANTITY, 0) >= 0,
						-(ABS(IFNULL(rrl.QUANTITY, 0))),
						ABS(IFNULL(rrl.QUANTITY, 0))
					)
			) as QUANTITY,

			f_elcProductFee(re.PRODUCT_ID, re.CRSO_RELATION_ID, rrl.PRODUCT_CATEGORY_ID, re.EXPORT_DATE) as UNIT_COST,

			re.PRODUCT_ID as A,
			re.CRSO_RELATION_ID as B,
			rrl.PRODUCT_CATEGORY_ID as C,
			re.EXPORT_DATE as D,

			IF(ISNULL(red.REFUND_REQUEST_DEBIT_ID),
					IF(IFNULL(rrl.QUANTITY, 0) >= 0,
						ABS(IFNULL(rrl.EXPORTED_AMOUNT, 0)),
						-(ABS(IFNULL(rrl.EXPORTED_AMOUNT, 0)))
					),
					IF(IFNULL(rrl.QUANTITY, 0) >= 0,
						-(ABS(IFNULL(rrl.EXPORTED_AMOUNT, 0))),
						ABS(IFNULL(rrl.EXPORTED_AMOUNT, 0))
					)
			) as LINE_AMOUNT_EXCL_VAT,

			f_elcGetTotalAmountForRefundExport(re.REFUND_EXPORT_ID) as TOTAL_AMOUNT_EXCL_VAT,

			rrl.WEIGHT,

			pc.NAME as CATEGORY_NAME,

			b.NAME as BRAND_NAME,

			cga.GL_ACCOUNT as REFUND_GLACCOUNT

			FROM TBL_REFUND_EXPORT as re

			-- gl account for refund_request
			LEFT JOIN TBL_CRSO_GLACCOUNT as cga
				ON re.CRSO_RELATION_ID = cga.CRSO_RELATION_ID
			INNER JOIN TBL_DECLARATION_TYPE as dt
				 ON cga.DECLARATION_TYPE_ID = dt.DECLARATION_TYPE_ID
				 AND dt.HANDLE='DT_REFUND_REQUEST'

			-- gl account for admin_fee
			LEFT JOIN TBL_CRSO_GLACCOUNT as cga2
				ON re.CRSO_RELATION_ID = cga2.CRSO_RELATION_ID
			INNER JOIN TBL_DECLARATION_TYPE as dt2
				 ON cga2.DECLARATION_TYPE_ID = dt2.DECLARATION_TYPE_ID
				 AND dt2.HANDLE='DT_REFUND_ADMINISTRATIVE_FEE'

			-- gl account for refund_request
			LEFT JOIN TBL_CRSO_GLACCOUNT as cga3
				ON re.CRSO_RELATION_ID = cga3.CRSO_RELATION_ID
			INNER JOIN TBL_DECLARATION_TYPE as dt3
				 ON cga3.DECLARATION_TYPE_ID = dt3.DECLARATION_TYPE_ID
				 AND dt3.HANDLE='DT_REFUND_DOCUMENT_REVIEW'

			INNER JOIN TBL_PRODUCT AS p
				ON re.PRODUCT_ID = p.PRODUCT_ID
			INNER JOIN TBL_RELATION AS cr
				ON cr.RELATION_ID = re.CRSO_RELATION_ID
			INNER JOIN TBL_ADDRESS AS ca
				ON cr.ADDRESS_ID = ca.ADDRESS_ID
			INNER JOIN TBL_COUNTRY AS cac
				ON cac.COUNTRY_ID = ca.COUNTRY_ID
			INNER JOIN TBL_REFUND_EXPORT_SET_LINE resl
				ON re.REFUND_EXPORT_ID = resl.REFUND_EXPORT_ID
			LEFT JOIN TBL_REFUND_REQUEST AS rr
				ON re.REFUND_EXPORT_ID = rr.REFUND_EXPORT_ID
			LEFT JOIN TBL_REFUND_EXPORT_DEBIT AS red
				ON red.REFUND_EXPORT_SET_ID = resl.REFUND_EXPORT_SET_ID
			LEFT JOIN TBL_REFUND_EXPORT_CREDIT AS rec
				ON rec.REFUND_EXPORT_SET_ID = resl.REFUND_EXPORT_SET_ID
			LEFT JOIN TBL_REFUND_REQUEST_LINE AS rrl
				ON rr.REFUND_REQUEST_ID = rrl.REFUND_REQUEST_ID
			LEFT JOIN TBL_PRODUCT_CATEGORY AS pc
				ON rrl.PRODUCT_CATEGORY_ID = pc.PRODUCT_CATEGORY_ID
			LEFT JOIN TBL_BRAND AS b
				ON b.BRAND_ID = rrl.BRAND_ID

			LEFT JOIN TBL_REFUND_EXPORT_CREDIT_LINE recl
				ON recl.REFUND_REQUEST_CREDIT_ID = rec.REFUND_REQUEST_CREDIT_ID
				AND recl.REFUND_REQUEST_ID = rr.REFUND_REQUEST_ID
				AND recl.REFUND_LINE_INDEX = rrl.LINE_INDEX

			LEFT JOIN TBL_REFUND_EXPORT_DEBIT_LINE redl
				ON redl.REFUND_REQUEST_DEBIT_ID = red.REFUND_REQUEST_DEBIT_ID
				AND redl.REFUND_REQUEST_ID = rr.REFUND_REQUEST_ID
				AND redl.REFUND_LINE_INDEX = rrl.LINE_INDEX

			WHERE re.IS_EXPORTED = 2 -- exported
				AND re.CRSO_RELATION_ID = ?
				AND YEAR(re.EXPORT_DATE) = ?
				AND MONTH(re.EXPORT_DATE) = ?

			ORDER BY TYPE DESC, _INVOICE_NUMBER, re.REFUND_EXPORT_ID, rr.REFUND_REQUEST_ID, rrl.LINE_INDEX
		";

		$q = $this->db->query($sql);
		$q->bind_values(array(sprintf('%s-%s-31 00:00:00', $year, $month), $crso_id, $year, $month));
		$q->execute();

		return $q;
	}

	/**#@-*/
}
/**#@+
 * Status defines
 */
define('REFUND_EXPORT_PENDING', 1);
define('REFUND_EXPORT_EXPORTED', 2);
/**#@-*/
?>
