<?php
require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES.'RefundExport.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'Document.php');
require_once(ELC_CLASSES.'Product.php');
/**
 * RefundExportDocumentReview
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @package ELC
 * @subpackage RefundRequest
 * @copyright 2006, BaseSoft B.V.
 * @version $Id$
 */
class RefundExportDocumentReview extends AbstractObject
{
	/**
	 * var Document $document
	 */
    var $document;
    /**
     * var RefundExport $refund_export
     */
    var $refund_export;
    /**
     * var int $review_cost
     */
    var $review_cost;
    /**
     * @var Participant $participant
     */
    var $participant;
    /**
     * @var CRSO $crso
     */
    var $crso;
    var $product;
	/**
	 * Sets the document review by document_id
	 * @param int $doc_id
	 * @return bool
	 */
    function set($doc_id=null)
    {
    	$doc_id || $doc_id = $this->document->id;

    	$sql = "
				SELECT * FROM TBL_REFUND_EXPORT_DOCUMENT_REVIEW WHERE DOCUMENT_ID=?
					";
		$q = $this->db->query($sql);
		$q->bind_values($doc_id);
		if(!$q->execute() || !($r = $q->fetch()))
			return false;

		$this->document = Document::createDocument($this->db);
		$this->document->set($r->DOCUMENT_ID);
		$this->refund_export = new RefundExport($this->db, $r->REFUND_EXPORT_ID);
		$this->review_cost = $r->REVIEW_COST;
		$this->product = new Product($this->db,$r->PRODUCT_ID);
		$this->participant = new Participant($this->db, array('relation' => new Relation($this->db, array('id' => $r->PARTICIPANT_RELATION_ID))));
		$this->crso = new CRSO($this->db, array('relation' => new Relation($this->db, array('id' => $r->CRSO_RELATION_ID))));
		return $this->complete = true;
    }
    /**
     * Gets the cost for the document review
     * @param int $re_id
     * @return bool|float
     */
    function getExportCost($re_id)
    {
    	$sql = "
				SELECT SUM(REVIEW_COST) AS TOTAL_REVIEW_COST
				FROM TBL_REFUND_EXPORT_DOCUMENT_REVIEW
				WHERE REFUND_EXPORT_ID=?
				";
		$q = $this->db->query($sql);
		$q->bind_values($re_id);
		if(!$q->execute())
			return false;
		$r = $q->fetch();
		debug(sprintf("RefundExportDocumentReview:: Total Export Cost for RefundExport [%s] is: %.2F", $re_id, $r->TOTAL_REVIEW_COST));
		return $r->TOTAL_REVIEW_COST;
    }

    /**#@+
     * @access protected
     */
    /**
     * Initializes the objectmaps used in AbstractObject
     */
    function _initObjectMaps()
    {
        $this->objectmap['dbtable'] = 'TBL_REFUND_EXPORT_DOCUMENT_REVIEW';
        $this->objectmap['primary']['document->id'] = 'DOCUMENT_ID';
        $this->objectmap['properties']['refund_export->id'] = 'REFUND_EXPORT_ID';
        $this->objectmap['properties']['product->id'] = 'PRODUCT_ID';
        $this->objectmap['properties']['review_cost'] = 'REVIEW_COST';
        $this->objectmap['properties']['crso->relation->id'] = 'CRSO_RELATION_ID';
        $this->objectmap['properties']['participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
    }
    /**#@-*/
}
?>