<?php
require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES.'RefundRequestAdditionalDocument.php');
/**
 * Handles the Individual refund requests
 * @package ELC
 * @subpackage RefundRequest
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @since 2006-15-05
 * @copyright 2006 BaseSoft B.V.
 */
class RefundRequest extends AbstractObject
{
	/**
	 * @var int $id
	 */
	var $id;
	/**
	 * @var Participant $participant
	 */
	var $participant;
	/**
	 * @var int $id
	 */
	var $crso;
	var $product;
	/**
	 * @var Country $export_country
	 */
	var $export_country;
	/**
	 * @var Document $export_document
	 */
	var $export_document;
	/**
	 * @var string importer_email
	 */
	var $importer_email;
	/**
	 * @var string importer_vat_no
	 */
	var $importer_vat_no;
	/**
	 * @var int $status
	 */
	var $status;
	/**
	 * @var Array<RefundRequestLine> $lines
	 */
	var $lines = array();
	/**
	 * @var string $importer_company_name
	 */
	var $importer_company_name;
	/**
	 * @var string $office
	 */
	var $office;
	/**
	 * @var string $commodity_code
	 */
	var $commodity_code;
	/**
	 * @var string $identification_no
	 */
	var $identification_no;
	/**
	 * @var ELCUser $user
	 */
	var $user;
	/**
	 * @var ELCUser $user
	 */
	var $authorized_by;
	/**
	 * @var int authorisation_by_type
	 */
	var $authorisation_by_type;
	/**
	 * @var string $refund_export_date
	 */
	var $refund_export_date;
	/**
	 * @var int $refund_request_confirmation_id
	 */
	var $refund_request_confirmation_id;

	var $additional_documents;

	/**
	 * Sets a refund request by its ID, join on RR_LINE
	 * This will also add the different refund_request_lines to the right property
	 * @param int $id
	 * @return bool
	 */
	function set($id='')
	{
		$id || $id = $this->id;
		$sql = "SELECT R.REFUND_REQUEST_ID, EXPORT_COUNTRY_ID, EXPORT_DOCUMENT_ID, REFUND_EXPORT_ID,
				PARTICIPANT_RELATION_ID, CRSO_RELATION_ID, PRODUCT_ID, IMPORTER_EMAIL,
				IMPORTER_VAT_NO, STATUS,
				IMPORTER_COMPANY_NAME, OFFICE, COMMODITY_CODE, IDENTIFICATION_NO,
				USER_ID, AUTHORIZED_BY, AUTHORISATION_BY_TYPE, REFUND_EXPORT_DATE, IMPORTER_STATUS
			FROM TBL_REFUND_REQUEST as R
			LEFT JOIN TBL_REFUND_REQUEST_LINE as RL
				ON R.REFUND_REQUEST_ID = RL.REFUND_REQUEST_ID
			WHERE R.REFUND_REQUEST_ID = ?
			GROUP BY R.REFUND_REQUEST_ID";

		$q = $this->db->query($sql);
		$q->bind_values($id);
		if(!$q->execute())
			return false; // trigger error
		if(!($r = $q->fetch()))
			return false;

		require_once(ELC_CLASSES . 'Country.php');
		require_once(ELC_CLASSES . 'Document.php');
		require_once(ELC_CLASSES . 'Participant.php');
		require_once(ELC_CLASSES . 'RefundExport.php');
		require_once(ELC_CLASSES . 'Product.php');

		$this->id = $id;
		$this->participant = new Participant($this->db, array('relation'=>new Relation($this->db, $r->PARTICIPANT_RELATION_ID)));
		$this->crso = new CRSO($this->db, array('relation'=>new Relation($this->db, $r->CRSO_RELATION_ID)));
		$this->product = new Product($this->db,$r->PRODUCT_ID);
		$this->export_country = new Country($this->db, $r->EXPORT_COUNTRY_ID);
		if($r->EXPORT_DOCUMENT_ID)
		{
			$this->export_document = Document::createDocument($this->db);
			$this->export_document->id = $r->EXPORT_DOCUMENT_ID;
		}
		else
			$this->export_document = null;

		$this->importer_email = $r->IMPORTER_EMAIL;
		$this->importer_vat_no = $r->IMPORTER_VAT_NO;
		$this->status = $r->STATUS;
		$this->importer_company_name = $r->IMPORTER_COMPANY_NAME;
		$this->office = $r->OFFICE;
		$this->commodity_code = $r->COMMODITY_CODE;
		$this->identification_no = $r->IDENTIFICATION_NO;
		$this->user = new ELCUser($this->db, $r->USER_ID);
		$this->refund_export_date = $r->REFUND_EXPORT_DATE;
		$this->refund_export = new RefundExport($this->db, array('id'=> $r->{'REFUND_EXPORT_ID'}));
		$this->authorized_by = new ELCUser($this->db, $r->AUTHORIZED_BY);
		$this->authorisation_by_type = $r->AUTHORISATION_BY_TYPE;
		$this->importer_status = $r->IMPORTER_STATUS;

		$this->lines = array();
		// Set the lines
		$rl = new RefundRequestLine($this->db);
		$q = $rl->getLines($id);
		while($r = $q->fetch())
		{
			$this->lines[$r->LINE_INDEX] = new RefundRequestLine($this->db,
				array('refund_request'=>$this, 'line_index'=>$r->LINE_INDEX), true);
		}

		// Set additional docs
		$this->additional_documents = array();
		$rl = new RefundRequestAdditionalDocument($this->db);
		$q = $rl->getLines($id);
		while($r = $q->fetch())
		{
			$doc = Document::createDocument($this->db);
			$doc->set($r->DOCUMENT_ID);
			$this->additional_documents[] = new RefundRequestAdditionalDocument($this->db,
				array('invoice_number'=>$r->INVOICE_NO, 'invoice_date'=>$r->INVOICE_DATE,
				'document' => $doc, 'refund_request'=>$this ));
		}
		return $this->complete = true;
	}
	/**
	 * Override parent insert for logging purposes, also to add the right creation datetime
	 * @return mixed
	 */
	function insert()
	{
		$date = currentDate();
		$this->created = $date->format('Y-m-d H:i');
		unset($this->last_update);
		$rv = parent::insert();
		info(sprintf('Created new RefundRequest [%s]', $this->id));
		return $rv;
	}
	/**
	 * Override the paretn update for loggin purposes
	 * @return mixed
	 */
	function update()
	{
		$rv = parent::update();
		info(sprintf('Updated RefundRequest [%s]', $this->id));
		return $rv;
	}
	/**
	 * Deletes a refund request, but only if isDeletable() returns true
	 */
	function delete()
	{
		if(!$this->isDeletable())
			return false;

		$sql = "DELETE FROM TBL_REFUND_REQUEST_IMPORTER_TOKEN WHERE REFUND_REQUEST_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();

		parent::delete();
		if(!$this->export_document)
			return;
		$this->export_document->set();
		$this->export_document->delete();
	}
	/**
	 * Checks wether or not the refund request can be delete
	 * @return bool
	 */
	function isDeletable()
	{
		if(!$this->isComplete())
			$this->set();

		return ($this->status == REFUND_REQUEST_PENDING ||
			$this->status == REFUND_REQUEST_REJECTED);
	}
	/**
	 * Insert all the lines provided
	 * @param array $lines
	 */
	/*
	function insertLines($lines)
	{
		$this->setLines($lines, true);
	}
	*/

	/**
	 * @deprecated use RefundRequestController
	 *
	 * Set all the lines for the refund request, or insert if $save = true
	 * @param array $lines
	 * @param bool $save
	 */
	/*
	function setLines($lines, $save=false)
	{
		$i=1;
		foreach ($lines as $line)
		{
			$this->lines[$i] = new RefundRequestLine($this->db, $line);

			if (isset($line['producer_vat_no']))
			{
	    		if($relation_id = Relation::getByVAT($line['producer_vat_no'],$this->db))
	    			$this->lines[$i]->producer_relation = new Relation($this->db, $relation_id);
	    		else
	    		{
	    			$this->lines[$i]->producer_relation = new Relation($this->db);
	    			$this->lines[$i]->producer_relation->vat_no = $line['producer_vat_no'];
	    			$save && $this->lines[$i]->producer_relation->insert();
	    		}
			}

			$this->lines[$i]->line_index = $i;
			$this->lines[$i]->refund_request = &$this;
			$save && $this->lines[$i]->insert();
			$i++;
		}
	}
	*/
	/**
	 * Updates the lines for a refund by deleting them all and then re-add them
	 * @param array $lines
	 */
	/*
	function updateLines($lines)
	{
		$this->deleteLines();
		$this->insertLines($lines);
	}
	*/
	/**
	 * Delete all the lines for the current refund request
	 * @return ResultSet $q ?
	 */
	function deleteLines()
	{
		$sql = "DELETE FROM TBL_REFUND_REQUEST_LINE WHERE REFUND_REQUEST_ID=?";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		return $q->rowCount();
	}

	/**
	 * Gets an overview of refund requests for the participant at crso
	 * @param int $participant_id
	 * @param int $crso_id
	 * @return ResultSet $q
	 */
	function getOverview($participant_id, $crso_id, $product_id, $filter)
	{
		$sql = "SELECT
					R.REFUND_REQUEST_ID, CREATED, IMPORTER_VAT_NO, C.NAME as COUNTRY_NAME, SUM(QUANTITY) as QUANTITY,
					SUM(
                                            IFNULL(RL.EXPORTED_AMOUNT, RL.QUANTITY * f_elcProductFee(
                                                R.PRODUCT_ID, R.CRSO_RELATION_ID, RL.PRODUCT_CATEGORY_ID, NOW()
                                            ))) AS AMOUNT, R.`STATUS`, R.EXPORT_DOCUMENT_ID as PROOF_OF_EXPORT, REFUND_EXPORT_DATE,
					IF(ISNULL(R.REFUND_EXPORT_ID),0,1) as IS_EXPORTED
				FROM TBL_REFUND_REQUEST R
				INNER JOIN TBL_REFUND_REQUEST_LINE RL
					ON R.REFUND_REQUEST_ID=RL.REFUND_REQUEST_ID
				INNER JOIN TBL_COUNTRY C
					ON R.EXPORT_COUNTRY_ID=C.COUNTRY_ID
				WHERE R.PARTICIPANT_RELATION_ID=? AND CRSO_RELATION_ID=? AND PRODUCT_ID = ?
				".$filter."
				GROUP BY R.REFUND_REQUEST_ID
				ORDER BY R.`STATUS`, R.REFUND_EXPORT_DATE
			";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id,$crso_id,$product_id);
		$q->execute();
		return $q;
	}

	/**
	 * Selects an overview for refund request authorize.. used in the common interface authorize
	 * @param array $authorize_type REFUND_AUTHORIZE_GENERAL_MANAGER | REFUND_AUTHORIZE_STATUTORY_AUDITOR | REFUND_AUTHORIZE_OPERATOR
	 * @param int $authorisation_status  REFUND_REQUEST_PENDING | REFUND_REQUEST_APPROVED | REFUND_REQUEST_CONFIRMED
	 * @param int crso_id
	 * @param int participant_id
	 * @return ResultSet $q
	 */
	function selectAuthorizeOverview($authorize_type, $authorisation_status, $crso_id, $participant_id, $product_id)
	{
		if (!is_array($authorisation_status) || sizeof($authorisation_status) == 0)
			trigger_error(sprintf('$authorisation_status [%s] must be array and cannot be empty', print_r($authorisation_status, true)), E_USER_ERROR);

		$bind_values = array();
		$sql = "SELECT
				'' AS AUTHORIZE_CHECKBOX,
				rr.REFUND_REQUEST_ID,
				IMPORTER_VAT_NO,
				rr_c.`NAME` as COUNTRY_NAME,
				SUM(QUANTITY) as QUANTITY,
				rr.`STATUS`,
				rr.EXPORT_DOCUMENT_ID as PROOF_OF_EXPORT,
				rr.REFUND_EXPORT_DATE,
				rr.REFUND_REQUEST_CONFIRMATION_ID

				FROM TBL_REFUND_REQUEST rr
				INNER JOIN TBL_REFUND_REQUEST_LINE rl
					ON rr.REFUND_REQUEST_ID=rl.REFUND_REQUEST_ID
				INNER JOIN TBL_RELATION p_r -- participant relation
					ON rr.PARTICIPANT_RELATION_ID = p_r.RELATION_ID
				INNER JOIN TBL_RELATION c_r -- crso relation
					ON rr.CRSO_RELATION_ID = c_r.RELATION_ID
				INNER JOIN TBL_COUNTRY rr_c
					ON rr.EXPORT_COUNTRY_ID = rr_c.COUNTRY_ID
				INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT cp
					ON rr.PARTICIPANT_RELATION_ID = cp.PARTICIPANT_RELATION_ID
						AND rr.CRSO_RELATION_ID = cp.CRSO_RELATION_ID
						AND rr.PRODUCT_ID = cp.PRODUCT_ID
				WHERE rr.STATUS IN (".implode(',', $authorisation_status).")
					AND rr.AUTHORISATION_BY_TYPE in (".join($authorize_type, ',').")";
			if(is_array($participant_id))
				$sql .= " AND rr.PARTICIPANT_RELATION_ID IN (".implode(',',$participant_id).") ";
			else
				$sql .= " AND rr.PARTICIPANT_RELATION_ID=".((int)$participant_id)." ";
			$sql .= " AND rr.CRSO_RELATION_ID=?
					AND rr.PRODUCT_ID = ?
				GROUP BY rr.REFUND_REQUEST_ID
				ORDER BY p_r.`NAME`, rr.`STATUS`, rr.REFUND_EXPORT_DATE

			";


		require_once(ELC_CLASSES.'Country.php');
		$bind_values[] = $crso_id;
		$bind_values[] = $product_id;

		$q = $this->db->query($sql);
		$q->bind_values($bind_values);
		$q->execute();
		return $q;
	}
	/**
	 * Selects the confirmation overview
	 * @param int $authorize_type
	 * @param int $authorisation_status
	 * @param int $crso_id
	 * @param int $participant_id
	 * @return ResultSet $q
	 */
	function selectConfirmationOverview($authorize_type, $authorisation_status, $crso_id, $participant_id, $product_id)
	{
		if (!is_array($authorisation_status) || sizeof($authorisation_status) == 0)
			trigger_error(sprintf('$authorisation_status [%s] must be array and cannot be empty', print_r($authorisation_status, true)), E_USER_ERROR);

		$bind_values = array();
		$sql = "SELECT
				'' AS AUTHORIZE_CHECKBOX,
				rr.REFUND_REQUEST_ID,
				IMPORTER_VAT_NO,
				SUM(QUANTITY) as QUANTITY,
				rr.`STATUS`,
				rr.EXPORT_DOCUMENT_ID as PROOF_OF_EXPORT,
				rr.REFUND_EXPORT_DATE,
				rr.REFUND_REQUEST_CONFIRMATION_ID,
				GROUP_CONCAT(pc.NAME SEPARATOR ',') AS PRODUCT_CATEGORY
				FROM TBL_REFUND_REQUEST rr
				INNER JOIN TBL_REFUND_REQUEST_LINE rl
					ON rr.REFUND_REQUEST_ID=rl.REFUND_REQUEST_ID
				LEFT JOIN TBL_PRODUCT_CATEGORY pc
					ON rl.PRODUCT_CATEGORY_ID = pc.PRODUCT_CATEGORY_ID
				INNER JOIN TBL_RELATION p_r -- participant relation
					ON rr.PARTICIPANT_RELATION_ID = p_r.RELATION_ID
				INNER JOIN TBL_RELATION c_r -- crso relation
					ON rr.CRSO_RELATION_ID = c_r.RELATION_ID
				INNER JOIN TBL_COUNTRY rr_c
					ON rr.EXPORT_COUNTRY_ID = rr_c.COUNTRY_ID
				INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT cp
					ON rr.PARTICIPANT_RELATION_ID = cp.PARTICIPANT_RELATION_ID
						AND rr.CRSO_RELATION_ID = cp.CRSO_RELATION_ID
						AND rr.PRODUCT_ID = cp.PRODUCT_ID
				LEFT JOIN TBL_REFUND_REQUEST_CONFIRMATION AS rrc
					ON rr.REFUND_REQUEST_CONFIRMATION_ID = rrc.REFUND_REQUEST_CONFIRMATION_ID
				WHERE rr.STATUS IN (".implode(',', $authorisation_status).")
					AND rr.AUTHORISATION_BY_TYPE in (".join($authorize_type, ',').")
					AND rr.PARTICIPANT_RELATION_ID=?
					AND rr.CRSO_RELATION_ID=?
					AND rr.PRODUCT_ID = ?
					AND ISNULL(rr.REFUND_EXPORT_ID)
					AND (ISNULL(rrc.PROCESS_STATUS) OR rrc.PROCESS_STATUS != 2)

				GROUP BY rr.REFUND_REQUEST_ID
				ORDER BY rr.`STATUS`, rr.REFUND_EXPORT_DATE
			";

		require_once(ELC_CLASSES.'Country.php');
		//$bind_values[] = $authorize_type;
		$d = &currentDate();
		//$bind_values[] = mktime(0, 0, 0, $d->getMonth(), 1, $d->getYear());
		$bind_values[] = $participant_id;
		$bind_values[] = $crso_id;
		$bind_values[] = $product_id;

		$q = $this->db->query($sql);
		$q->bind_values($bind_values);
		$q->execute();
		return $q;
	}

	function selectOverviewByRefundConfirmation($confirmation_id)
	{
		$sql = "SELECT rr.REFUND_REQUEST_ID,
				IMPORTER_VAT_NO,
				SUM(QUANTITY) as QUANTITY,
				rr.EXPORT_DOCUMENT_ID as PROOF_OF_EXPORT,
				rr.REFUND_EXPORT_DATE
			FROM TBL_REFUND_REQUEST rr
			INNER JOIN TBL_REFUND_REQUEST_LINE rl
				ON rr.REFUND_REQUEST_ID=rl.REFUND_REQUEST_ID
			WHERE rr.REFUND_REQUEST_CONFIRMATION_ID = ?
			GROUP BY rr.REFUND_REQUEST_ID
			ORDER BY rr.REFUND_EXPORT_DATE";
		$q = $this->db->query($sql);
		$q->bind_values($confirmation_id);
		$q->execute();
		return $q;
	}

	/**
	 * Selects the export country
	 * @param int $crso_id
	 * @param int $participant_id
	 * @return ResultSet $q
	 */
	function selectExportCountrySelect($crso_id=false,$participant_id=false, $product_id)
	{
		$values = array();
		$sql = "SELECT DISTINCT C.COUNTRY_ID, C.`NAME` as COUNTRY_NAME
			FROM TBL_REFUND_REQUEST as RR
			INNER JOIN TBL_COUNTRY as C
				ON RR.EXPORT_COUNTRY_ID = C.COUNTRY_ID ";
		if($crso_id)
		{
			$values[] = $crso_id;
			$sql .= " WHERE CRSO_RELATION_ID = ? ";
		}
		if($participant_id)
		{
			$values[] = $participant_id;
			$sql .= (($crso_id)?' AND ':' WHERE ')." PARTICIPANT_RELATION_ID = ? ";
		}
		$values[] = $product_id;
		$sql .= (($crso_id||$participant_id)?' AND ':' WHERE ')." PRODUCT_ID = ? ";
		$q = $this->db->query($sql);
		$q->bind_values($values);
		$q->execute();
		return $q;
	}

	/**
	 * Translates the status and gives the right css-class
	 * @static
	 * @param LanguageInterface $langIf
	 * @param int $status
	 * @return array (TRANSLATION,CSSCLASS)
	 */
	static public function translateStatus($langIf,$status)
	{
		switch($status)
		{
			case REFUND_REQUEST_PENDING:
				return array($langIf->getText('Registration Pending'), 'status_new');
			break;
			case REFUND_REQUEST_APPROVED:
				return array($langIf->getText('Registration Approved'), 'status_verified');
			break;
			case REFUND_REQUEST_REJECTED:
				return array($langIf->getText('Registration Rejected'), 'status_forfeit');
			break;
			case REFUND_REQUEST_CONFIRMED:
				return array($langIf->getText('Registration Confirmed'), 'status_verified');
			break;
			/*case REFUND_REQUEST_PENDING:
				return array($langIf->getText('Registration Export Pending'), 'status_verified');
			break;
			case REFUND_REQUEST_EXPORT_EXPORTED:
				return array($langIf->getText('Registration Export Exported'), 'status_verified');
			break;*/
		}
	}
	/**
	 * Gets the lines for the refund request form
	 * @return array
	 */
	function getFormLines()
	{
		$rv = array();

		if ($this->lines)
		{
			foreach ($this->lines as $i=>$line)
			{
				$line->refund_request = &$this;
				$line->set($i);
				$rv[] = array (
						'PRODUCER_RELATION'=>$line->producer_relation,
						'BRAND'=>(($line->brand instanceof Brand && isset($line->brand->id))?$line->brand->id:''),
						'PRODUCT_CATEGORY'=>(($line->product_category instanceof ProductCategory && isset($line->product_category->id))?$line->product_category->id:''),
						'QUANTITY'=>$line->quantity,
						'WEIGHT'=>$line->weight,
						'WEIGHT_DEC'=>$line->weight_dec,
						'VIRTUAL_STOCK'=>'',
						'DELETE'=>''
				);
			}
			return $rv;
		}

		$rv[] = array(
				'PRODUCER_RELATION'=>'',
				'BRAND'=>'',
				'PRODUCT_CATEGORY'=>'',
				'QUANTITY'=>'',
				'WEIGHT'=>'',
				'WEIGHT_DEC'=>'',
				'VIRTUAL_STOCK'=>'',
				'DELETE'=>''

		);
		return $rv;
	}
	/**
	 * Trashes the document for a refund request.
	 */
	function trashExportDocument()
	{
		$document_id = false;
		if (!$this->isComplete())
		{
			$sql = "SELECT EXPORT_DOCUMENT_ID FROM TBL_REFUND_REQUEST WHERE REFUND_REQUEST_ID=?";
			$q = $this->db->query($sql);
			$q->bind_values($this->id);
			$q->execute();

			if($row = $q->fetch())
				$document_id = $row->EXPORT_DOCUMENT_ID;
		}
		else
			$document_id = isset($this->export_document->id) ? $this->export_document->id : false;

		if (!$document_id)
			return;

		$sql = "UPDATE TBL_REFUND_REQUEST SET EXPORT_DOCUMENT_ID=NULL WHERE REFUND_REQUEST_ID=?";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();

		$doc = Document::createDocument($this->db);
		$doc->set($document_id);
		$doc->delete();
	}
	/**
	 * Gets the RefundRequestConfirmationID for non-exported refunds for participant at crso
	 * @param int $crso_id
	 * @param int $participant_id
	 * @return bool|int
	 */
	function getRefundRequestConfirmationId($crso_id, $participant_id,$product_id,$auth_by_type)
	{
		require_once(ELC_CLASSES.'RefundRequestConfirmation.php');
		$sql = "SELECT REFUND_REQUEST_CONFIRMATION_ID
			FROM TBL_REFUND_REQUEST_CONFIRMATION
			WHERE PROCESS_STATUS = ?
				AND CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ?
				AND PRODUCT_ID = ?
				AND AUTHORISATION_BY_TYPE = ?
		";

		$q = $this->db->query($sql);
		$q->bind_values(RR_CONFIRMATION_OPEN,$crso_id, $participant_id,$product_id,$auth_by_type);

		if (!$q->execute() || !($row = $q->fetch()))
			return false;

		if ($r = $q->fetch())
			trigger_error(sprintf('RefundRequest::getRefundRequestConfirmationId() - Should not return more than 1 rows for crso [%s] and participant [%s]', $crso_id, $participant_id));


		return $row->REFUND_REQUEST_CONFIRMATION_ID;

	}
	/**
	 * Nullifies the request confirmation ids for a certain id
	 * @param int $rr_confirmation_id
	 */
	function nullifyRequestConfirmationId($rr_confirmation_id)
	{
		$sql = "UPDATE TBL_REFUND_REQUEST
			SET REFUND_REQUEST_CONFIRMATION_ID=NULL
			WHERE REFUND_REQUEST_CONFIRMATION_ID=?";

		$q = $this->db->query($sql);
		$q->bind_values($rr_confirmation_id);
		$q->execute();

		debug('nullifyRequestConfirmationId() - affected rows: '.$q->affected_rows() );
	}
	/**
	 * Rollbacks the confirmed requests
	 * @param int $rr_confirmation_id
	 */
	function rollbackConfirmedRequests($rr_confirmation_id)
	{
		$sql = "UPDATE TBL_REFUND_REQUEST
			SET STATUS=".REFUND_REQUEST_APPROVED."
			WHERE REFUND_REQUEST_CONFIRMATION_ID=?
			AND STATUS=".REFUND_REQUEST_CONFIRMED;

		$q = $this->db->query($sql);
		$q->bind_values($rr_confirmation_id);
		$q->execute();

		debug('rollbackConfirmedRequests() - affected rows: '.$q->affected_rows() );
	}
	/**
	 * Gets the quantity for a refund request
	 * @param int $id
	 * @return int $quantity
	 */
	function getQuantity($id=false)
	{
		$id || $id = $this->id;
		$sql = "SELECT SUM(QUANTITY) AS QUANTITY
			FROM TBL_REFUND_REQUEST as R
			INNER JOIN TBL_REFUND_REQUEST_LINE as RL USING (REFUND_REQUEST_ID)
			WHERE R.REFUND_REQUEST_ID = ?
			";

		$q = $this->db->query($sql);
		$q->bind_values($id);
		if(!$q->execute())
			return false; // trigger error
		if(!($r = $q->fetch()))
			return false;

		return (int)$r->{'QUANTITY'};
	}

	function getAdditionalDocuments($id=null)
	{
		$id || $id = $this->id;
		$q = $this->db->query('SELECT DOCUMENT_ID FROM TBL_REFUND_REQUEST_ADDITIONAL_DOCUMENT WHERE REFUND_REQUEST_ID = ?');
		$q->bind_values($id);
		$q->execute();

		$docs = array();

		while($r = $q->fetch())
			$docs[] = $r->DOCUMENT_ID;
		return $docs;
	}

	/**
	 * Create the exported_amount field in the db, so we get good FP precision
	 * @return bool
	 */
	function saveExportedAmount()
	{
		if(!$this->isComplete())
		{
			if(!$this->set())
				return false;
		}

		debug(sprintf('Saving exported amount for RR lines - RR [%s]', $this->id));

		/**
		 * @todo THIS HAS HARDCODED PRODUCT ID
		 */

		$sql = "
				UPDATE TBL_REFUND_REQUEST_LINE RRL
				SET RRL.EXPORTED_AMOUNT = RRL.QUANTITY * f_elcProductFee(?, ?, RRL.PRODUCT_CATEGORY_ID, NOW())
				WHERE RRL.REFUND_REQUEST_ID=?
					";
		$q = $this->db->query($sql);
		$q->bind_values($this->product->id, $this->crso->relation->id, $this->id);
		if(!$q->execute())
			return false;
		return true;
	}
	/**
	 * Gets the exported amount for a refund export id
	 * Why is this here and not in RefundExport?
	 * @param int $re_id
	 * @return bool|float
	 */
	function getExportAmount($re_id)
	{
		$sql = "
				SELECT SUM(RRL.EXPORTED_AMOUNT) AS TOTAL_REVIEW_COST
				FROM TBL_REFUND_REQUEST RR
				INNER JOIN TBL_REFUND_REQUEST_LINE RRL USING(REFUND_REQUEST_ID)
				WHERE RR.REFUND_EXPORT_ID=?
				";
		$q = $this->db->query($sql);
		$q->bind_values($re_id);
		if(!$q->execute())
			return false;
		$r = $q->fetch();
		debug(sprintf("RefundRequest:: Total Export Amount for RefundExport [%s] is: %.2F", $re_id, $r->TOTAL_REVIEW_COST));
		return $r->TOTAL_REVIEW_COST;
	}

	function getAdditionalDocumentFormLines()
	{
		$lines = array();
		if(!isset($this->additional_documents))
			return $lines;
		foreach($this->additional_documents as $r)
		{
			$lines[] = array('INVOICE_NO' => $r->invoice_number, 'INVOICE_DATE' => $r->invoice_date,'INVOICE_DOCUMENT' => $r->document->id, 'DELETE' => '');
		}
		return $lines;
	}

	/**
	 * Initializes the objectmaps used in AbstractObject
	 * @access protected
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_REFUND_REQUEST';
		$this->objectmap['primary']['id'] = 'REFUND_REQUEST_ID';
		$this->objectmap['properties']['participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
		$this->objectmap['properties']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['product->id'] = 'PRODUCT_ID';
		$this->objectmap['properties']['export_document->id'] = 'EXPORT_DOCUMENT_ID';
		$this->objectmap['properties']['export_country->id'] = 'EXPORT_COUNTRY_ID';
		$this->objectmap['properties']['importer_email'] = 'IMPORTER_EMAIL';
		$this->objectmap['properties']['importer_vat_no'] = 'IMPORTER_VAT_NO';
		$this->objectmap['properties']['created'] = 'CREATED';
		$this->objectmap['properties']['last_update'] = 'LAST_UPDATE';
		$this->objectmap['properties']['status'] = 'STATUS';
		$this->objectmap['properties']['identification_no'] = 'IDENTIFICATION_NO';
		$this->objectmap['properties']['office'] = 'OFFICE';
		$this->objectmap['properties']['commodity_code'] = 'COMMODITY_CODE';
		$this->objectmap['properties']['importer_company_name'] = 'IMPORTER_COMPANY_NAME';
		$this->objectmap['properties']['refund_export->id'] = 'REFUND_EXPORT_ID';
		$this->objectmap['properties']['refund_export_date'] = 'REFUND_EXPORT_DATE';

		$this->objectmap['properties']['user->id'] = 'USER_ID';
		$this->objectmap['properties']['authorized_by->id'] = 'AUTHORIZED_BY';
		$this->objectmap['properties']['authorisation_by_type'] = 'AUTHORISATION_BY_TYPE';
		$this->objectmap['properties']['importer_status'] = 'IMPORTER_STATUS';

		$this->objectmap['properties']['refund_request_confirmation_id'] = 'REFUND_REQUEST_CONFIRMATION_ID';

	}

	/**
	 * Selects all Participants that have a payment. A crso filter can be added
	 * @param int $crso_id
	 * @return ResultSet $q
	 */
	function selectParticipants($crso_id='', $product_id)
	{
		$bind_values = array($product_id,REFUND_REQUEST_APPROVED);
		$sql = "SELECT DISTINCT rr.PARTICIPANT_RELATION_ID, r.NAME
			FROM TBL_REFUND_REQUEST rr
			INNER JOIN TBL_RELATION r
				ON rr.PARTICIPANT_RELATION_ID = r.RELATION_ID
			WHERE PRODUCT_ID = ?
				AND rr.STATUS <= ?
		";

		if ($crso_id) {
			$sql .= "AND CRSO_RELATION_ID=? ";
			$bind_values[] = $crso_id;
		}

		$sql .= " GROUP BY rr.PARTICIPANT_RELATION_ID ";

		$q = $this->db->query($sql);
		$q->bind_values($bind_values);
		$q->execute();
		return $q;
	}

	/**
	 * Will generate a checksum for confirmation documents
	 *
	 * @access public static
	 * @param array $refundIds The refund request ids that you want to generate a checksum for
	 * @return string An md5 checksum
	 */
	static public function generateConfirmProofOfExportChecksum($refundIds)
	{
		//we need this sort because the ids may be given to us in a different order
		array_walk($refundIds, array('RefundRequest', '_forceNumeric'));
		sort($refundIds, SORT_NUMERIC);

		$documentId = '';
		foreach($refundIds as $approvedId)
			$documentId .= CONFIRM_PROOF_OF_EXPORT_SALT.$approvedId;
		return md5($documentId);
	}

	/**
	 * * @access private static
	 */
	static private function _forceNumeric(&$value)
	{
		settype($value, 'integer');
	}

	function getCount($crso_id, $part_id, $product_id, $status)
	{
		$sql = 'SELECT COUNT(REFUND_REQUEST_ID) as cnt
				FROM TBL_REFUND_REQUEST
				WHERE
					CRSO_RELATION_ID = ?
				AND
					PARTICIPANT_RELATION_ID = ?
				AND
					PRODUCT_ID = ?
				AND
					`STATUS` = ?
		';

		$values = array($crso_id, $part_id,$product_id, $status);

		# FINISH ME

	}
}

require_once(ELC_CLASSES.'Brand.php');
require_once(ELC_CLASSES.'ProductCategory.php');
require_once(ELC_CLASSES.'Relation.php');
/**
 * Handles the Individual refund request lines
 * @package ELC
 * @subpackage RefundRequest
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @since 2006-15-05
 * @copyright 2006 BaseSoft B.V.
 */
class RefundRequestLine extends AbstractObject
{
	/**
	 * @var RefundRequest $refund_request
	 */
	var $refund_request;
	/**
	 * var int $line_index
	 */
	var $line_index;
	/**
	 * @var Relation $producer_relation
	 */
	var $producer_relation;
	/**
	 * @var Brand $brand
	 */
	var $brand;
	/**
	 * var ProductCategory $product_category
	 */
	var $product_category;
	/**
	 * var int $weight
	 */
	var $weight;
	/**
	 * var float $weight_dec
	 */
	var $weight_dec;
	/**
	 * var int $quantity
	 */
	var $quantity;
	/**
	 * @var int $exported_amount
	 */
	var $exported_amount;
	/**
	 * Sets a line by index and refund request id
	 * @param int $index
	 * @param int $id REFUND_REQUEST_ID
	 * @return bool
	 */
	function set($index='', $id='')
	{
		$id || $id = $this->refund_request->id;
		$index || $index = $this->line_index;

		$sql = "SELECT PRODUCER_RELATION_ID, BRAND_ID, WEIGHT_DEC,
				PRODUCT_CATEGORY_ID, WEIGHT, QUANTITY, EXPORTED_AMOUNT
			FROM TBL_REFUND_REQUEST_LINE
			WHERE REFUND_REQUEST_ID = ?
				AND LINE_INDEX = ?";

		$q = $this->db->query($sql);
		$q->bind_values($id, $index);
		if(!$q->execute())
			return false; // Trigger Error
		if(!$r = $q->fetch())
			return false;

		//print_r($r);

		$this->line_index = $index;
		$this->producer_relation = new Relation($this->db, $r->PRODUCER_RELATION_ID);
		require_once(ELC_CLASSES.'Brand.php');
		$this->brand = new Brand($this->db, $r->BRAND_ID);
		require_once(ELC_CLASSES.'ProductCategory.php');
		$this->product_category = new ProductCategory($this->db, $r->PRODUCT_CATEGORY_ID);
		$this->weight = $r->WEIGHT;
		$this->weight_dec = $r->WEIGHT_DEC;
		$this->quantity = $r->QUANTITY;
		$this->exported_amount = $r->EXPORTED_AMOUNT;

		return $this->complete = true;
	}
	/**
	 * Gets the lines for a refund request id
	 * @param int $refund_request_id
	 * @return ResultSet $q
	 */
	function getLines($refund_request_id)
	{
		$sql = "SELECT LINE_INDEX
			FROM TBL_REFUND_REQUEST_LINE
			WHERE REFUND_REQUEST_ID = ?
			ORDER BY LINE_INDEX";
		$q = $this->db->query($sql);
		$q->bind_values($refund_request_id);
		$q->execute();
		return $q;

	}

	/**
	 * Override parent insert function for brands,product_categories and producer_relations
	 */
	/*
	function insert()
	{
		$this->brand && $this->brand = new Brand($this->db, $this->brand);
		$this->product_category && $this->product_category = new ProductCategory($this->db, $this->product_category);
		if($this->producer_relation && !($this->producer_relation instanceof Relation))
			$this->producer_relation = new Relation($this->db, $this->producer_relation);

		parent::insert();
	}
	*/

	/**
	 * Initializes the objectmaps used in AbstractObject
	 * @access protected
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_REFUND_REQUEST_LINE';
		$this->objectmap['primary']['refund_request->id'] = 'REFUND_REQUEST_ID';
		$this->objectmap['primary']['line_index'] = 'LINE_INDEX';
		$this->objectmap['properties']['producer_relation->id'] = 'PRODUCER_RELATION_ID';
		$this->objectmap['properties']['brand->id'] = 'BRAND_ID';
		$this->objectmap['properties']['product_category->id'] = 'PRODUCT_CATEGORY_ID';
		$this->objectmap['properties']['weight'] = 'WEIGHT';
		$this->objectmap['properties']['weight_dec'] = 'WEIGHT_DEC';
		$this->objectmap['properties']['quantity'] = 'QUANTITY';
		$this->objectmap['properties']['exported_amount'] = 'EXPORTED_AMOUNT';
	}

	public static function fromSoapRefundLine($db, $soapRefundLine)
	{
		$obj = new RefundRequestLine($db);

		if ($soapRefundLine->producerVAT)
			$obj->producer_relation = new Relation($db, array('vat_no'=>$soapRefundLine->producerVAT));
		if ($soapRefundLine->brand)
			$obj->brand = new Brand($db, array('id'=>$soapRefundLine->brand->brandId));
		if ($soapRefundLine->productCategory)
			$obj->product_category = new ProductCategory($db, array('id'=>$soapRefundLine->productCategory->productCategoryId));
		if ($soapRefundLine->weight)
			$obj->weight = $soapRefundLine->weight;
		if (isset($soapRefundLine->weight_dec) && $soapRefundLine->weight_dec)
			$obj->weight_dec = $soapRefundLine->weight_dec;

		$obj->quantity = $soapRefundLine->quantity;

		return $obj;
	}

	public static function fromInterfaceRefundLine($db, $line)
	{
		$obj = new RefundRequestLine($db);

		if (isset($line['producer_relation']['id']))
			$obj->producer_relation = new Relation($db, $line['producer_relation']['id'], true);
		if (isset($line['brand']))
			$obj->brand = new Brand($db, $line['brand']);
		if (isset($line['product_category']))
			$obj->product_category = new ProductCategory($db, $line['product_category']);
		if (isset($line['weight']))
			$obj->weight = $line['weight'];
		if (isset($line['weight_dec']))
			$obj->weight_dec = $line['weight_dec'];
		if (isset($line['producer_vat_no']))
			if(!isset($obj->producer_relation))
				$obj->producer_relation = new Relation($db, array('vat_no'=>$line['producer_vat_no']));
			else
				$obj->producer_relation->vat_no = $line['producer_vat_no'];

		$obj->quantity = $line['quantity'];

		return $obj;
	}

}

/**#@+
 * Authorisation status
 */
define('REFUND_REQUEST_PENDING',   1); // Participant can make a request
define('REFUND_REQUEST_APPROVED',  2); // Depending on the the refund settings and type type it can be approved by GeneralManager, Statitory Auditor or importer email.
define('REFUND_REQUEST_REJECTED',  3); // Give it back to the participant
define('REFUND_REQUEST_CONFIRMED', 4); // GM or SA can confirm the request.

define('REFUND_IMPORTER_STATUS_PENDING', 0); //importer has not reviewed
define('REFUND_IMPORTER_STATUS_REJECTED', 1); //importer has reviewed and rejected
define('REFUND_IMPORTER_STATUS_APPROVED', 2); //importer has reviewed and approved
define('REFUND_IMPORTER_STATUS_RESOLVED', 2); //importer has reviewed and rejected, PwC has resolved the issue outside of the system
/**#@-*/
?>
