<?php
require_once(IWARE.'AbstractObject.php');
/**
 * A template file.
 * @author Chris Wesseling <chris.wesseling@xs4all.nl>
 * @author Marten Koedam <marten@dalines.net>
 * @copyright 2006, BaseSoft B.V.
 * @package ELC
 * @subpackage RefundRequest
 * @version $Id: AbstractObjectTemplate.php 2227 2006-02-27 11:11:39Z Marten Koedam $
 */
class RefundRequestConfirmation extends AbstractObject
{
	/**
	 * @var int $id
	 */
	var $id;
	/**
	 * @var ELCDocument $document
	 */
	var $document;
	/**
	 * @var string $created
	 */
	var $created;

	var $process_status;
	var $crso;
	var $participant;
	var $product;
	var $auth_by_type;

	/**
	 * @var int $review_status
	 */
	var $review_status;
	/**
	 * Sets a refund confirmation by its ID
	 * @param int $id
	 * @return bool
	 */
	function set($id='')
	{
		$id || $id = $this->id;

		$sql = "SELECT REFUND_REQUEST_CONFIRMATION_ID, DOCUMENT_ID, CREATED, PROCESS_STATUS,
			CRSO_RELATION_ID, PARTICIPANT_RELATION_ID, PRODUCT_ID, AUTHORISATION_BY_TYPE, REVIEW_STATUS
			FROM TBL_REFUND_REQUEST_CONFIRMATION
			WHERE REFUND_REQUEST_CONFIRMATION_ID=?";

		$q = $this->db->query($sql);
		$q->bind_values($id);
		if(!$q->execute())
			return false; // trigger error
		if(!($r = $q->fetch()))
			return false;


		$this->id = $id;
		require_once(ELC_CLASSES.'Document.php');
		require_once(ELC_CLASSES.'Participant.php');
		require_once(ELC_CLASSES.'CRSO.php');
		require_once(ELC_CLASSES.'Product.php');
		$this->document = Document::createDocument($this->db);
		$this->document->id = $r->DOCUMENT_ID;
		$this->created = $r->CREATED;
		$this->review_status = $r->REVIEW_STATUS;
		$this->participant = new Participant($this->db, array('relation'=>new Relation($this->db, $r->PARTICIPANT_RELATION_ID)));
		$this->crso = new CRSO($this->db, array('relation'=>new Relation($this->db, $r->CRSO_RELATION_ID)));
		$this->product = new Product($this->db,$r->PRODUCT_ID);
		$this->process_status = $r->PROCESS_STATUS;
		$this->auth_by_type = $r->AUTHORISATION_BY_TYPE;

		return $this->complete = true;
	}
	/**
	 * Override the insert function to add the right creation date
	 * @return mixed
	 */
	function insert()
	{
		$date = currentDate();
		$this->created = $date->format('Y-m-d H:i');
		$rv = parent::insert();
		info(sprintf('Created new RefundRequestConfirmation [%s]', $this->id));
		return $rv;
	}
	/**
	 * Override the parent for loggin purposes
	 * @return mixed;
	 */
	function update()
	{
		$rv = parent::update();
		info(sprintf('Updated RefundRequestConfirmation [%s]', $this->id));
		return $rv;
	}
	/**
	 * Gets all the refund requests for the confirmation id
	 * @param int $id
	 * @return array $rr
	 */
	function &getRefundRequestsForConfirmation($id=false)
	{
		$id || $id = $this->id;
		$sql = 'SELECT REFUND_REQUEST_ID FROM TBL_REFUND_REQUEST WHERE REFUND_REQUEST_CONFIRMATION_ID=?';
		$q = $this->db->query($sql);
		$q->bind_values($id);
		if(!$q->execute())
			return false;
		$rs = $q->fetchAll();
		if(!count($rs))
			return false;
		$rr = array();
		foreach($rs as $r)
			$rr[] = $r->{'REFUND_REQUEST_ID'};
		return $rr;
	}
	/**
	 * Gets the overview for the operator by status and CRSO
	 * @param int $status
	 * @param int $crso_id
	 * @return ResultSet $q
	 */
	function &getOverviewForOperator($status,$crso_id)
	{
		$sql = '
				SELECT
					\'\' AS CHECKBOX,
					RRC.REFUND_REQUEST_CONFIRMATION_ID,
					RRC.DOCUMENT_ID AS EXPORT_CONFIRMATION_DOCUMENT,
					\'\' AS `CHECKSUM`,
					P.NAME AS PARTICIPANT_NAME,
					C.NAME AS CRSO_NAME,
					SUM(RRL.QUANTITY) AS QUANTITY
				FROM
					TBL_REFUND_REQUEST_CONFIRMATION as RRC
				INNER JOIN
					TBL_REFUND_REQUEST RR
				ON (RRC.REFUND_REQUEST_CONFIRMATION_ID = RR.REFUND_REQUEST_CONFIRMATION_ID)
				INNER JOIN
					TBL_REFUND_REQUEST_LINE RRL
				ON (RR.REFUND_REQUEST_ID = RRL.REFUND_REQUEST_ID)
				INNER JOIN
					TBL_RELATION P
				ON (RRC.PARTICIPANT_RELATION_ID = P.RELATION_ID)
				INNER JOIN
					TBL_RELATION C
				ON (RRC.CRSO_RELATION_ID = C.RELATION_ID)
				INNER JOIN
					TBL_USER U
				ON (RR.AUTHORIZED_BY = U.USER_ID)
				WHERE
					RRC.REVIEW_STATUS = ?
				AND
					NOT ISNULL(RRC.DOCUMENT_ID)
				AND
					RR.CRSO_RELATION_ID = ?
				GROUP BY RRC.REFUND_REQUEST_CONFIRMATION_ID
					';
		$q = $this->db->query($sql);
		$q->bind_values($status,$crso_id);
		if(!$q->execute())
			return false;
		return $q;
	}
	/**
	 * Gets the right translation and css-class for the status provided
	 * @param LanguageInterface $langIf
	 * @param int $status
	 * @return array (TRANSLATION,CSSCLASS)
	 */
	public static function translateStatus($langIf,$status)
	{
		switch($status)
		{
			case RR_CONFIRMATION_UNREVIEWED:
				return array($langIf->getText('Refund Export Review: Unreviewed'), 'status_new');
			break;
			case RR_CONFIRMATION_APPROVED:
				return array($langIf->getText('Refund Export Review: Approved'), 'status_verified');
			break;
			case RR_CONFIRMATION_REJECTED:
				return array($langIf->getText('Refund Export Review: Rejected'), 'status_forfeit');
			break;
		}
	}

	public static function translateStatusForParticipant($langIf,$status)
	{
		switch($status)
		{
			case RR_CONFIRMATION_UNREVIEWED:
				return array($langIf->getText('Unreviewed'), 'status_new');
			break;
			case RR_CONFIRMATION_APPROVED:
				return array($langIf->getText('Approved'), 'status_verified');
			break;
			case RR_CONFIRMATION_REJECTED:
				return array($langIf->getText('Rejected'), 'status_forfeit');
			break;
		}
	}

	/**
	 * Gets the CRSO that is connected to this Confirmation for Document Review purposes
	 * @param int $id
	 * @return ResultSet $q
	 */
	function getCRSOForDocumentReview($id=false)
	{
		$id || $id = $this->id;
		$sql = '
				SELECT
					RRC.CRSO_RELATION_ID
				FROM
					TBL_REFUND_REQUEST_CONFIRMATION as RRC
				WHERE
					RRC.REFUND_REQUEST_CONFIRMATION_ID=?
				LIMIT 1
					';
		$q = $this->db->query($sql);
		$q->bind_values($id);
		if(!$q->execute() || !($r = $q->fetch()))
			return false;
		return $r->{'CRSO_RELATION_ID'};
	}

    /**#@+
     * @access private
     */
    /**
     * Initializes the objectmaps used in AbstractObject
     */
    function _initObjectMaps()
    {
        $this->objectmap['dbtable'] = 'TBL_REFUND_REQUEST_CONFIRMATION';
        $this->objectmap['primary']['id'] = 'REFUND_REQUEST_CONFIRMATION_ID';
        $this->objectmap['properties']['document->id'] = 'DOCUMENT_ID';
        $this->objectmap['properties']['created'] = 'CREATED';
        $this->objectmap['properties']['review_status'] = 'REVIEW_STATUS';
        $this->objectmap['properties']['participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
		$this->objectmap['properties']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['product->id'] = 'PRODUCT_ID';
		$this->objectmap['properties']['process_status'] = 'PROCESS_STATUS';
		$this->objectmap['properties']['auth_by_type'] = 'AUTHORISATION_BY_TYPE';
    }

    /**#@-*/

    public static function closeOpenRefundConfirmations($db,$crso_id,$product_id)
    {
    	$sql = "UPDATE TBL_REFUND_REQUEST_CONFIRMATION SET PROCESS_STATUS = ?
			WHERE PRODUCT_ID = ? AND CRSO_RELATION_ID = ?";
		$q = $db->query($sql);
		$q->bind_values(RR_CONFIRMATION_CLOSED,$product_id,$crso_id);
		return $q->execute();
    }

    function getAuthorizedById($id=null)
    {
    	$id || $id = $this->id;
    	$sql = "SELECT DISTINCT( AUTHORIZED_BY ) as USER_ID
			FROM TBL_REFUND_REQUEST as RR
			WHERE REFUND_REQUEST_CONFIRMATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		return $r->USER_ID;
    }
}
/**#@+
 * RefundRequest confirmation review status defines.
 * RefundRequest confirmation process status defines.
 */
define('RR_CONFIRMATION_UNREVIEWED', 0);
define('RR_CONFIRMATION_APPROVED', 1);
define('RR_CONFIRMATION_REJECTED', 2);

define('RR_CONFIRMATION_OPEN', 1);
define('RR_CONFIRMATION_CLOSED', 2);
/**#@-*/
?>
