<?php
/**
 * File description
 *
 * @package PACKAGE
 * @subpackage SUBPACKAGE
 * @since Aug 28, 2007
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */

require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES.'RefundRequest.php');

class RefundRequestImporterToken extends AbstractObject
{

	function set($refund_id)
	{
		$sql = "SELECT * FROM TBL_REFUND_REQUEST_IMPORTER_TOKEN
			WHERE REFUND_REQUEST_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($refund_id);
		$q->execute();

		if(!($r = $q->fetch()))
			return false;

		$this->refund_request = new RefundRequest($this->db, $refund_id);
		$this->token = $r->TOKEN;
		$this->expiry_date = $r->EXPIRE_DATE;

		return $this->complete = true;
	}

	function generate()
	{
		// generate a nice hash
		mt_srand( (double) microtime() * 1000000);
		$salt = pack("CCCC", mt_rand(), mt_rand(), mt_rand(), mt_rand());
		$this->token = sha1($salt);

		// and set the expiry date
		$date = currentDate();
		$date->modify('+2 weeks');
		$this->expire_date = $date->format("Y-m-d H:m");
	}

	function setByToken($token)
	{
		$sql = 'SELECT REFUND_REQUEST_ID
			FROM TBL_REFUND_REQUEST_IMPORTER_TOKEN
			WHERE TOKEN=:token
			AND EXPIRE_DATE > :now';
		$q = $this->db->query($sql);
		$q->bindValue('token', $token);
		$q->bindValue('now', currentDate()->format("Y-m-d H:M"));
		$q->execute();

		$row = $q->fetch();
		if (!$row)
			return false;

		return $this->set($row->{'REFUND_REQUEST_ID'});
	}

	function insert()
	{
		$rv = parent::insert();
		info(sprintf('Inserted new RefundRequestImporterToken for refund_request [%s]', $this->refund_request->id));
		return $rv;
	}

	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_REFUND_REQUEST_IMPORTER_TOKEN';
		$this->objectmap['primary']['refund_request->id'] = 'REFUND_REQUEST_ID';
		$this->objectmap['properties']['token'] = 'TOKEN';
		$this->objectmap['properties']['expire_date'] = 'EXPIRE_DATE';
		$this->objectmap['inserttype'] = 'REPLACE';
	}

}

?>
