<?php
require_once(IWARE.'AbstractObject.php');
/**
 * The relation object.
 * This is used at all of ELC, CRSO's, Participants etc.
 * @author Marten Koedam <marten@basesoft.com>
 * @package ELC
 * @subpackage Relation
 * @copyright 2006, BaseSoft B.V.
 * @version $Id$
 */
class Relation extends AbstractObject
{
	var $id;

	/**
	 * @var string $name
	 */
	var $name;
	/**
	 * @var string $phone
	 */
	var $phone;
	/**
	 * @var string $fax
	 */
	var $fax;
	/**
	 * @var string $vat_no
	 */
	var $vat_no;
	/**
	 * @var string $po_no
	 */
	var $po_no;
	/**
	 * @var string $bank_account_number
	 */
	var $bank_account_number;
	/**
	 * @var string $iban_no
	 */
	var $iban_no;
	/**
	 * @var string $swift_bic_code
	 */
	var $swift_bic_code;
	/**
	 * @var string $email
	 */
	var $email;
	/**
	 * @var string $website
	 */
	var $website;
	/**
	 * @var string $logo_url
	 */
	var $logo_url;
	/**
	 * @var string $dd_bank_account
	 */
	var $dd_bank_account;
	/**
	 * @var string $dd_iban_no
	 */
	var $dd_iban_no;
	/**
	 * @var string $dd_swift_bic_code
	 */
	var $dd_swift_bic_code;
	/**
	 * @var date $last_update
	 */
	var $last_update;
	/**
	 * @var date $created
	 */
	var $created;
	/**
	 * Sets the Relation by its relation id
	 * @param int $id
	 * @return bool
	 */
	function set($id='')
	{
		$id || $id = $this->id;
		$sql = "SELECT * FROM TBL_RELATION
			WHERE RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;


		require_once(IWARE.'nLanguage.php');
		require_once(IWARE.'InfoPerson.php');
		require_once(IWARE.'address.php');
		require_once(IWARE.'RelationType.php');

		$this->id = $id;

		$this->invoice_language = new nLanguage($this->db, $r->INVOICE_LANGUAGE_ID);
		$this->customer_contact = new InfoPerson($this->db, $r->CUSTOMER_CONTACT_ID);
		$this->address = new Address($this->db, array('address_id'=>$r->ADDRESS_ID));
		$this->relation_type = new RelationType($this->db, $r->RELATION_TYPE_ID);

		$this->name = $r->NAME;
		$this->phone = $r->PHONE;
		$this->fax = $r->FAX;
		$this->vat_no = $r->VAT_NO;
		$this->po_no = $r->PO_NO;
		$this->bank_account_number = $r->BANK_ACCOUNT_NUMBER;
		$this->iban_no = $r->IBAN_NO;
		$this->swift_bic_code = $r->SWIFT_BIC_CODE;
		$this->email = $r->EMAIL;
		$this->website = $r->WEBSITE;
		$this->logo_url = $r->LOGO_URL;
		$this->dd_bank_account = $r->DD_BANK_ACCOUNT;
		$this->dd_iban_no = $r->DD_IBAN_NO;
		$this->dd_swift_bic_code = $r->DD_SWIFT_BIC_CODE;
		$this->last_update = $r->LAST_UPDATE;
		$this->created = $r->CREATED;

		return $this->complete = true;
	}
	/**
	 * Override the insert function to add the creation time
	 * @return mixed
	 */
	function insert()
	{
		$this->created = (currentDate()->format('Y-m-d H:i:s'));
		return parent::insert();
	}

	function update()
	{
		$this->last_update = null;
		$rv = parent::update();
		if(get_class($this) == 'Relation')
		{
			info(sprintf('Updated Relation [%s]:', $this->id));
			info(sprintf('	- To: [%s] [%s] [%s] [%s] [%s]', $this->id, $this->name, $this->vat_no,
				$this->bank_account_number, $this->iban_no, $this->email));
		}
		return $rv;
	}
	/**
	 * Sets a relation by userId
	 * Why is this in here and not at RelationUser?
	 * @param int $user_id
	 * @return bool
	 */
	function setByUserId($user_id)
	{
		$rs = $this->getByUserId($user_id);
		if (!$rs)
			return false;
		if(count($rs) > 1)
			return false;
		$row = $rs[0];
		return $this->set($row->RELATION_ID);
	}
	/**
	 * Gets a relation by user_id
	 * What if there are more then one relations linked to this user?
	 * Why is this in here and not at RelationUser
	 * @param int $user_id
	 * @return bool|ResultSet
	 */
	function getByUserId($user_id)
	{
		$sql = "SELECT RELATION_ID, IS_ADMIN, CLEARANCE_LEVEL
			FROM TBL_RELATION_USER
			WHERE USER_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values(array($user_id));
		$q->execute(true);
		$rs = $q->fetchAll();
		if(!count($rs))
			return false;
		return $rs;
	}
	/**
	 * Select an overview of all the users/relations
	 * Why is this in here and not in RelationUser?
	 * @param bool|array $relation_ids
	 * @param string $select
	 * @return ResultSet $q
	 */
	function selectOverview($relation_ids=false, $select='')
	{
		$sql = "SELECT R.RELATION_ID, R.`NAME`, R.`EMAIL`, U.`USER_NAME`, U.`USER_ACTIVE`".$select."
			FROM TBL_RELATION as R
			LEFT JOIN TBL_RELATION_USER as RU
				ON R.RELATION_ID = RU.RELATION_ID
			LEFT JOIN TBL_USER as U
				ON RU.USER_ID = U.USER_ID
			WHERE 1=1 ";
		if($relation_ids)
			$sql .= "AND R.RELATION_ID IN( ".implode(',',$relation_ids)." )";
		$q = $this->db->query($sql);
		$q->execute();
		return $q;
	}
	/**
	 * Initializes the objectmaps used in AbstractObject
	 * @access protected
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_RELATION';
		$this->objectmap['primary']['id'] = 'RELATION_ID';
		$this->objectmap['properties']['invoice_language->id'] = 'INVOICE_LANGUAGE_ID';
		$this->objectmap['properties']['customer_contact->id'] = 'CUSTOMER_CONTACT_ID';
		$this->objectmap['properties']['address->address_id'] = 'ADDRESS_ID';
		$this->objectmap['properties']['relation_type->id'] = 'RELATION_TYPE_ID';
		$this->objectmap['properties']['name'] = 'NAME';
		$this->objectmap['properties']['phone'] = 'PHONE';
		$this->objectmap['properties']['fax'] = 'FAX';
		$this->objectmap['properties']['vat_no'] = 'VAT_NO';
		$this->objectmap['properties']['po_no'] = 'PO_NO';
		$this->objectmap['properties']['bank_account_number'] = 'BANK_ACCOUNT_NUMBER';
		$this->objectmap['properties']['iban_no'] = 'IBAN_NO';
		$this->objectmap['properties']['swift_bic_code'] = 'SWIFT_BIC_CODE';
		$this->objectmap['properties']['email'] = 'EMAIL';
		$this->objectmap['properties']['website'] = 'WEBSITE';
		$this->objectmap['properties']['logo_url'] = 'LOGO_URL';
		$this->objectmap['properties']['dd_bank_account'] = 'DD_BANK_ACCOUNT';
		$this->objectmap['properties']['dd_iban_no'] = 'DD_IBAN_NO';
		$this->objectmap['properties']['dd_swift_bic_code'] = 'DD_SWIFT_BIC_CODE';
		$this->objectmap['properties']['last_update'] = 'LAST_UPDATE';
		$this->objectmap['properties']['created'] = 'CREATED';


	}

	/**
	 * Get the country ID for the Relation
	 * @param int $id
	 * @param string $return
	 * @return mixed
	 */
	function getCountry($id='',$return='COUNTRY_ID')
	{
		$id || $id = $this->id;
		$sql = "SELECT C.COUNTRY_ID, C.`NAME` as COUNTRY_NAME
			FROM TBL_COUNTRY as C
			INNER JOIN TBL_ADDRESS as A
				USING(COUNTRY_ID)
			INNER JOIN TBL_RELATION as R
				USING(ADDRESS_ID)
			WHERE R.RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		if(!$q->execute())
			throw new IWException('Error executign query');

		if(!($r = $q->fetch())) {
			info(sprintf('Relation [%s] does not have a country', $id));
			return false;
		}

		return $r->{$return};
	}

	/**
	 * Checks wether or not the Vat is unique
	 *
	 * @static
	 * @param string $vat
	 * @param Database $db
	 * @return bool
	 */
	public static function isUniqueVAT($vat,$db='')
	{
		$sql = "SELECT VAT_NO
			FROM TBL_RELATION
			WHERE REPLACE(VAT_NO, ' ', '') = ?";
		$q = $db->query($sql);
		$q->bind_values(str_replace(' ', '', $vat));
		$q->execute();
		if($r = $q->fetch())
			return false;
		return true;
	}
	/**
	 * Gets a relationId by VAT
	 * @param string $vat
	 * @param Database $db
	 * @return int
	 */
	static public function getByVAT($vat,$db)
	{
		$sql = "SELECT RELATION_ID
			FROM TBL_RELATION
			WHERE VAT_NO = ?";
		$q = $db->query($sql);
		$q->bind_values($vat);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		return $r->RELATION_ID;
	}

	function setByVAT($vat_no)
	{
		$sql = "SELECT RELATION_ID
			FROM TBL_RELATION r
			WHERE r.VAT_NO = ?
		";
		$q = $this->db->query($sql);
		$q->bind_values($vat_no);
		$q->execute();

		if (!($r = $q->fetch()))
			return false;

		return $this->set($r->{'RELATION_ID'});
	}

}
?>