<?php
require_once(IWARE.'AbstractObject.php');
/**
 * Manages the relation between Relations (CRSO/Participants/Operators) and the users connected to it
 *
 * @todo Make this part of IWARE?
 * @author Marten Koedam <marten@basesoft.com>
 * @package ELC
 * @subpackage User
 * @version $Id$
 */
class RelationUser extends AbstractObject
{
	/**
	 * @var Relation $relation
	 */
	var $relation;
	/**
	 * @var ELCUser $user
	 */
	var $user;
	/**
	 * @var bool $is_admin
	 */
	var $is_admin;
	/**
	 * @var int $clearance_level
	 */
	var $clearance_level; // If Clearance Level=0, the user cannot access for this participant yet. I dont think this is true
	/**
	 * @var int $status
	 */
	var $status;
	/**
	 * @var bool $default_relation
	 */
	var $default_relation;
	var $default_product;
	/**
	 * Sets a relation-user object by both the userId and relationId
	 * @param int $user_id
	 * @param int $relation_id
	 * @return bool
	 */
	function set($user_id='',$relation_id='')
	{
		$user_id || $user_id = $this->user->id;
		$relation_id || $relation_id = $this->relation->id;
		$sql = "SELECT IS_ADMIN, CLEARANCE_LEVEL, STATUS, DEFAULT_RELATION, DEFAULT_PRODUCT_ID
			FROM TBL_RELATION_USER
			WHERE RELATION_ID = ?
				AND USER_ID = ?";

		$q = $this->db->query($sql);
		$q->bind_values($relation_id,$user_id);
		$q->execute();

		if(!($r = $q->fetch()))
			return false;

		require_once(ELC_CLASSES.'User.php');
		require_once(ELC_CLASSES.'Relation.php');
		require_once(ELC_CLASSES.'Product.php');
		$this->user = new ELCUser($this->db,$user_id);
		$this->relation = new Relation($this->db,$relation_id);
		$this->default_product = new Product($this->db,$r->DEFAULT_PRODUCT_ID);
		$this->default_relation = $r->DEFAULT_RELATION;
		$this->is_admin = $r->IS_ADMIN;
		$this->clearance_level = $r->CLEARANCE_LEVEL;
		$this->status = $r->STATUS;
		return $this->complete = true;
	}
	/**
	 * There can be only one.
	 *
	 * If there is more than one Admin user for a relation an EMERG is triggered. This is really something bad.
	 * @param int $user_id
	 * @param int $relation_id
	 * @param bool $none
	 * @return bool
	 */
	function validateAdmin($user_id='',$relation_id='',$none=true)
	{
		$user_id || $user_id = isset($this->user->id)?$this->user->id:'';
		$relation_id || $relation_id = $this->relation->id;
		$sql = "SELECT USER_ID, IS_ADMIN
			FROM TBL_RELATION_USER as RU
			WHERE RELATION_ID = ? AND IS_ADMIN = ?";
		$q = $this->db->query($sql);
		$q->bind_values($relation_id,1);
		$q->execute();
		if(!($r = $q->fetch()))
			return $none;
		if($r2 = $q->fetch())
		{
			$log = getLog();
			$log->emerg('more than one ADMIN users for RELATION ['.$relation_id.']');
			exit('Something went wrong, please contact the webmaster');
		}
		;
		if($r->USER_ID == $user_id)
			return true;
		return false;
	}
	/**
	 * Checks wether or not the user is administrator.
	 *
	 * Checks wether or not the user is administrator. If the relation_id is provided,
	 * only check for that relation_id
	 *
	 * @param int $user_id
	 * @param int $relation_id
	 * @return bool
	 */
	function isAdmin($user_id='',$relation_id='')
	{
		if($relation_id)
			return $this->validateAdmin($user_id,$relation_id);
		$user_id || $user_id = $this->user->id;
		$sql = "SELECT COUNT(RELATION_ID) as AMT
			FROM TBL_RELATION_USER
			WHERE USER_ID = ?
				AND IS_ADMIN = ? AND NOT ISNULL(IS_ADMIN)
				AND STATUS = ?";
		$q = $this->db->query($sql);
		$q->bind_values($user_id,1,1);
		$q->execute();
		$r = $q->fetch();
		return (bool) $r->AMT;
	}
	/**
	 * Sets the admin user for a relation
	 *
	 * @param int $relation_id
	 * @param int $user_id
	 * @return bool
	 */
	function setAdmin($relation_id, $user_id)
	{
		$this->db->begin();
		$q = $this->db->query('UPDATE TBL_RELATION_USER SET IS_ADMIN=NULL WHERE RELATION_ID=?');
		$q->bind_values($relation_id);
		$q2 = $this->db->query('UPDATE TBL_RELATION_USER SET IS_ADMIN=1, STATUS=1 WHERE RELATION_ID=? AND USER_ID=?');
		$q2->bind_values($relation_id, $user_id);
		if(!$q->execute() || !$q2->execute())
		{
			$this->db->rollback();
			return false;
		}
		$this->db->commit();
		return true;
	}

	/**
	 * Gets the default relation for a user.
	 * Returns bool false if it cannot find any default user.
	 * This is a setting for a multi-relation user
	 * @param int $user_id
	 * @return bool|int
	 */
	function getDefaultRelation($user_id=null)
	{
		$user_id || $user_id = $this->user->id;
		$sql = "SELECT RELATION_ID
			FROM TBL_RELATION_USER
			WHERE USER_ID = ?
				AND DEFAULT_RELATION = ?
				AND NOT ISNULL(DEFAULT_RELATION)";
		$q = $this->db->query($sql);
		$q->bind_values($user_id,1);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		return $r->RELATION_ID;
	}

	function getDefaultProduct($user_id=null,$relation_id=null)
	{
		$user_id || $user_id = $this->user->id;
		$relation_id || $relation_id = $this->relation->id;
		$sql = "SELECT DEFAULT_PRODUCT_ID
			FROM TBL_RELATION_USER
			WHERE USER_ID = ?
				AND RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($user_id,$relation_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		$allowedProducts = array();
		switch($_SESSION['session_group_group_id']) {
			case GROUP_SYSADMIN:
				return $r->DEFAULT_PRODUCT_ID;
			case GROUP_ACCOUNTANT:
			case GROUP_CRSO:
				$sql = "
					SELECT `PRODUCT_ID`
					FROM TBL_CRSO_PRODUCT
					WHERE `CRSO_RELATION_ID` = ?
				";
				$q = $this->db->query($sql);
				$q->bind_values($relation_id);
				$q->execute();
				while($row = $q->fetch())
					$allowedProducts[] = $row->PRODUCT_ID;
			break;
			case GROUP_PARTICIPANT:
				$sql = "
					SELECT `PRODUCT_ID`
					FROM TBL_CRSO_PARTICIPANT_PRODUCT
					WHERE `PARTICIPANT_RELATION_ID` = ?
						AND `ACTIVE` = ?
						AND `STATUS` >= ?
					";
				$q = $this->db->query($sql);
				$q->bind_values($relation_id, 1, 4);
				$q->execute();
				while($row = $q->fetch())
					$allowedProducts[] = $row->PRODUCT_ID;
		}
		if(in_array($r->DEFAULT_PRODUCT_ID, $allowedProducts)) {
			return $r->DEFAULT_PRODUCT_ID;
		}

		return array_pop($allowedProducts);
	}


	/**
	 * Sets the default relation for a user
	 * First removes the former default relation, if any
	 * @param int $user_id
	 * @param int $relation_id
	 * @return bool
	 */
	function setDefaultRelation($user_id=null,$relation_id=null)
	{
		$user_id || $user_id = $this->user->id;
		$relation_id || $relation_id = $this->relation->id;

		// first lets see if the user has rights for this participant
		$rui = new RelationUser($this->db);
		if (!$rui->set($user_id, $relation_id) && $rui->status = 1 )
			throw new IWException(sprintf('Cannot change default relation: RelationUser [%s/%s] does not exist or is not active', $user_id, $relation_id));

		// Remove any former default relations for this user.
		$sql = "UPDATE TBL_RELATION_USER
			SET DEFAULT_RELATION = NULL
			WHERE USER_ID = ?
				AND RELATION_ID != ?";
		$q = $this->db->query($sql);
		$q->bind_values($user_id,$relation_id);
		$q->execute();

		$sql = "UPDATE TBL_RELATION_USER
			SET DEFAULT_RELATION = ?
			WHERE USER_ID = ?
				AND RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values(1,$user_id,$relation_id);
		return $q->execute();
	}

	/**
	 * Deletes all the relations for a user
	 * @param int $user_id
	 * @return bool
	 */
	function deleteByUserId($user_id='')
	{
		$user_id || $user_id = $this->user->id;
		$sql = "DELETE FROM TBL_RELATION_USER
			WHERE USER_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($user_id);
		return $q->execute(true);
	}
	/**
	 * Deletes a user-relation
	 * @param int $user_id
	 * @param int $relation_id
	 * @return bool
	 */
	function deleteByUserRelation($user_id='',$relation_id='')
	{
		$user_id || $user_id = $this->user->id;
		$relation_id || $relation_id = $this->relation->id;
		$sql = "DELETE FROM TBL_RELATION_USER
			WHERE USER_ID = ?
				AND RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($user_id,$relation_id);
		return $q->execute(true);
	}
	/**
	 * Gets the admin user for the relation
	 * @param int $relation_id
	 * @return ELCUser $user
	 */
	function getRelationAdmin($relation_id='', $silently=false)
	{
		require_once(ELC_CLASSES.'User.php');
		$q = $this->selectByRelation($relation_id,1);

		if(!($r = $q->fetch()))
		{
			info('No Admin user for Relation');
			return;
		}
		if($r2 = $q->fetch())
		{
			$log = getLog();
			$log->emerg('more than one ADMIN users for RELATION ['.$relation_id.']');
			if(!$silently)
				exit('Something went wrong, please contact the webmaster');
			return false;
		}
		$user = new ELCUser($this->db,$r->USER_ID);
		return $user;
	}
	/**
	 * Gets all the users for a relation, posibility to select only admin or non-admins
	 * @param int $relation_id
	 * @param int $is_admin
	 * @return ResultSet $q
	 */
	function selectByRelation($relation_id='', $is_admin='')
	{
		$relation_id || $relation_id = $this->relation->id;
		$sql = "SELECT RU.USER_ID,IS_ADMIN, CLEARANCE_LEVEL, USER_NAME, INFO_PERSON_EMAIL, USER_ACTIVE, RU.STATUS, U.ALLOW_CRSO_LOGIN
			FROM TBL_RELATION_USER as RU
			INNER JOIN TBL_USER as U
				USING(USER_ID)
			INNER JOIN TBL_INFO_PERSON USING (INFO_PERSON_ID)
			WHERE RU.RELATION_ID = ? ";
		if($is_admin)
			$sql .= "AND IS_ADMIN = '".$is_admin."' ";
		$sql .= "ORDER BY USER_NAME";
		$q = $this->db->query($sql);
		$q->bind_values($relation_id);
		$q->execute();
		return $q;
	}
	/**
	 * Used for overview to see at which relations this user is active
	 * @param int $user_id
	 * @return ResultSet $q
	 */
	function selectRegistrationOverview($user_id='')
	{
		$user_id || $user_id = $this->user->id;
		$user_id = (int) $user_id;

		$sql = "SELECT P.PARTICIPANT_RELATION_ID, RU.`STATUS` as RU_STATUS, R1.`NAME` as PARTICIPANT_NAME,
				R1.VAT_NO as PARTICIPANT_VAT, RU.IS_ADMIN,
				R2.RELATION_ID as CRSO_ID, R2.`NAME` as CRSO_NAME,
				C2.`NAME` as CRSO_COUNTRY,
				R2.VAT_NO as CRSO_VAT,
				PR.PRODUCT_ID as PRODUCT_ID, PR.NAME as PRODUCT_NAME,
				CP.`STATUS` as CP_STATUS, CP.ACTIVE as CP_ACTIVE,
				IF(CP.PRODUCER_EXPORTER & ".PARTICIPANT_PRODUCER.",1,0) as PRODUCER,
				IF(CP.PRODUCER_EXPORTER & ".PARTICIPANT_EXPORTER.",1,0) as EXPORTER,
				'' as DECLARATION_SETTINGS, '' as COMPANY_DETAILS
			FROM TBL_PARTICIPANT as P
			INNER JOIN TBL_RELATION as R1
				ON P.PARTICIPANT_RELATION_ID = R1.RELATION_ID
			INNER JOIN TBL_RELATION_USER as RU
				ON R1.RELATION_ID = RU.RELATION_ID
					AND RU.USER_ID = ?
			INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT as CP
				ON P.PARTICIPANT_RELATION_ID = CP.PARTICIPANT_RELATION_ID
			INNER JOIN TBL_PRODUCT PR
				ON CP.PRODUCT_ID = PR.PRODUCT_ID
			INNER JOIN TBL_RELATION as R2
				ON CP.CRSO_RELATION_ID = R2.RELATION_ID
			INNER JOIN TBL_ADDRESS as A2
				ON R2.ADDRESS_ID = A2.ADDRESS_ID
			INNER JOIN TBL_COUNTRY as C2
				ON A2.COUNTRY_ID = C2.COUNTRY_ID
			GROUP BY PR.PRODUCT_ID, R1.RELATION_ID, R2.RELATION_ID
			ORDER BY R1.`NAME`, R2.`NAME`";
		$q = $this->db->query($sql);
		$q->bind_values($user_id);
		$q->execute();
		return $q;
	}
	/**
	 * Selects all the users in an overview for the admin user
	 * @param int $user_id
	 * @return ResultSet $q
	 */
	function selectUserOverviewForAdmin($user_id='')
	{
		$user_id || $user_id = $this->user->id;
		$sql = "SELECT '' as CHECKBOX,
				R.`NAME` as PARTICIPANT_NAME,
				R.VAT_NO as PARTICIPANT_VAT,
				U.USER_ID,
				CONCAT(IP.INFO_PERSON_FIRST_NAME,' ',
					IP.INFO_PERSON_MIDDLE_NAME,' ',
					IP.INFO_PERSON_LAST_NAME) as FULL_NAME,
				IP.INFO_PERSON_EMAIL,
				U.USER_NAME,
				RU2.CLEARANCE_LEVEL, RU2.STATUS,
				RU2.RELATION_ID as PARTICIPANT_ID,
				RU2.IS_ADMIN,
				-- IF(ISNULL(CP.GENERAL_MANAGER_ID), '', CP.GENERAL_MANAGER_AUTHORIZED)
				-- CP.GENERAL_MANAGER_AUTHORIZED,
				'' AS SETTINGS
			FROM TBL_RELATION_USER as RU1
			INNER JOIN TBL_RELATION_USER as RU2
				USING(RELATION_ID)
			INNER JOIN TBL_USER as U
				ON RU2.USER_ID=U.USER_ID
			INNER JOIN TBL_INFO_PERSON as IP
				USING(INFO_PERSON_ID)
			INNER JOIN TBL_RELATION as R
				ON RU2.RELATION_ID = R.RELATION_ID

			LEFT JOIN TBL_CRSO_PARTICIPANT_PRODUCT CP
				ON U.USER_ID = CP.GENERAL_MANAGER_ID

			WHERE RU1.USER_ID = ?
				AND RU1.IS_ADMIN = ?
			GROUP BY PARTICIPANT_ID, USER_ID
			ORDER BY PARTICIPANT_ID, RU2.STATUS, FULL_NAME";
		$q = $this->db->query($sql);
		$q->bind_values($user_id,1);
		$q->execute();
		return $q;
	}
	/**
	 * Get all the relations for user
	 * @param int $user_id
	 * @return ResultSet $q
	 */
	function getUserRelations($user_id='')
	{
		$user_id || $user_id = $this->user->id;
		$sql = "SELECT R.RELATION_ID, R.`NAME` as RELATION_NAME
			FROM TBL_RELATION_USER as RU
			INNER JOIN TBL_RELATION as R
				USING(RELATION_ID)
			WHERE RU.USER_ID = ?
				AND RU.`STATUS` = ?";
		$q = $this->db->query($sql);
		$q->bind_values($user_id, 1);
		$q->execute();
		return $q;
	}
	/**
	 * Checks to see if the user has clearance for this clearance typ at this relation
	 * @param int $clearance
	 * @param int $user_id
	 * @param int $relation_id
	 * @return bool
	 */
	function hasClearance($clearance,$user_id='',$relation_id='')
	{
		$user_id || $user_id = $this->user->id;
		$relation_id || $relation_id = $this->relation->id;

		if(!$this->set($user_id, $relation_id)) {
			//debug(sprintf('hasClearance() - No clearance: this->set(%s, %s) return false', $user_id,$relation_id));
			return false;
		}

		if($this->is_admin) {
			//debug(sprintf('hasClearance() - Clearance: user is admin'));
			return true;
		}

		if(($this->clearance_level & $clearance)==$clearance) {
			//debug(sprintf('hasClearance() - Clearance: user has clearance set'));
			return true;
		}

		return false;
	}
	/**#@+
	 * @access private
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_RELATION_USER';
		$this->objectmap['primary']['relation->id'] = 'RELATION_ID';
		$this->objectmap['primary']['user->id'] = 'USER_ID';
		$this->objectmap['properties']['is_admin'] = 'IS_ADMIN';
		$this->objectmap['properties']['clearance_level'] = 'CLEARANCE_LEVEL';
		$this->objectmap['properties']['status'] = 'STATUS';
		$this->objectmap['properties']['default_relation'] = 'DEFAULT_RELATION';
		$this->objectmap['properties']['default_product->id'] = 'DEFAULT_PRODUCT_ID';
	}
	/**#@-*/
}
?>