<?php
require_once(IWARE.'AbstractObject.php');
/**
 * Note that participant can only have one pending/active remote viewer at a time.
 * If they do another request for Remote Viewing, any other entries should be instantly expired.
 *
 * @author marten
 *
 */
class RemoteViewer extends AbstractObject {

	var $id;
	var $crso;
	var $participant;
	var $user;
	var $content;
	var $request_status;
	var $request_datetime;
	var $last_updated;
	/**
	 * @param $id
	 * @return boolean
	 */
	function set($id=null) {
		$id || $id = $this->id;

		$sql = "SELECT REMOTE_VIEWER_ID, CRSO_RELATION_ID, PARTICIPANT_RELATION_ID, USER_ID,
				REQUEST_STATUS, REQUEST_DATETIME, LAST_UPDATE
			FROM TBL_REMOTE_VIEWER
			WHERE REMOTE_VIEWER_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();

		return $this->_setFromQuery($q);
	}
	/**
	 * @param $crso_id
	 * @param $participant_id
	 * @return boolean
	 */
	function setByCRSOParticipant($crso_id = null, $participant_id = null) {
		$crso_id || $crso_id = $this->crso->relation->id;
		$participant_id || $participant_id = $this->participant->relation->id;

		$sql = "SELECT REMOTE_VIEWER_ID, CRSO_RELATION_ID, PARTICIPANT_RELATION_ID, USER_ID,
				REQUEST_STATUS, REQUEST_DATETIME, LAST_UPDATE
			FROM TBL_REMOTE_VIEWER
			WHERE CRSO_RELATION_ID = ? AND PARTICIPANT_RELATION_ID = ?
			";
		$q = $this->db->query($sql);
		$q->bind_values($crso_relation_id, $participant_relation_id);
		$q->execute();

		return $this->_setFromQuery($q);
	}
	/**
	 * @param Query $q
	 * @access protected
	 * @return boolean
	 */
	protected function _setFromQuery($q) {

		if(!($r = $q->fetch()))
			return false;

		$this->id = $r->REMOTE_VIEWER_ID;
		$this->crso = new CRSO($this->db, array('relation' => new Relation($this->db, $r->CRSO_RELATION_ID)));
		$this->participant = new Participant($this->db, array('relation' => new Relation($this->db, $r->PARTICIPANT_RELATION_ID)));
		$this->user = new ELCUser($this->db, $r->USER_ID);
		$this->request_status = $r->REQUEST_STATUS;
		$this->request_datetime = $r->REQUEST_DATETIME;
		$this->last_updated = $r->LAST_UPDATE;

		return $this->complete = true;
	}
	/**
	 * @param $crso_id
	 * @param $participant_id
	 * @param $user_id
	 * @return boolean
	 */
	function invite($crso_id = null, $participant_id = null, $user_id = null) {
		$crso_id || $crso_id = $this->crso->relation->id;
		$participant_id || $participant_id = $this->participant->relation->id;
		$user_id || $user_id = $this->user->id;

		$this->expireAll($participant_id, $user_id);

		$this->request_status = REMOTE_VIEWER_REQUEST_PENDING;
		$this->request_datetime = date('Y-m-d H:i:s');
		$this->crso = new CRSO($this->db, array('relation' => new Relation($this->db, $crso_id)));
		$this->participant = new Participant($this->db, array('relation' => new Relation($this->db, $participant_id)));
		$this->user = new ELCUser($this->db, $user_id);
		$rv = parent::insert();

		// Now send a notification to the CRSO, only users that have the right clearance.
		$users = $this->crso->selectCRSOUsersWithClearance(null, CRSOClearanceRemoteViewing);
		$recipients = array();
		while($r = $users->fetch()) {
			$recipients[] = array(
 				'user'=>new ELCUser($this->db, $r->USER_ID),
 				'callback'=>"go('/".SITE_PATH."scripts/elc/crso/InterfaceRVOverview.php')"
 			);
		}

		require_once(ELC_CLASSES.'Notification.php');
  		Notification::send($this->db,
 			array(
 				'sender'=>$this->participant->relation,
 				'subject'=>IWRegistry::registry('langIf')->getText('Notification: Remote Viewer Invitation'),
 				'message'=>IWRegistry::registry('langIf')->getText("Notification: Participant has invited you to observe their actions")),
 			$recipients);

		return $rv;
	}
	/**
	 * @param $participant_id
	 * @param $user_id
	 * @param $newstatus
	 * @return boolean
	 */
	function expireAll($participant_id = null, $user_id = null, $newstatus = REMOTE_VIEWER_REQUEST_EXPIRED) {
		$participant_id || $participant_id = $this->participant->relation->id;
		$user_id || $user_id = $this->user->id;

		$sql = "UPDATE TBL_REMOTE_VIEWER SET REQUEST_STATUS = ?
			WHERE PARTICIPANT_RELATION_ID = ? AND USER_ID = ?
			AND REQUEST_STATUS IN(?, ?)";
		$q = $this->db->query($sql);
		$q->bind_values($newstatus, $participant_id, $user_id, REMOTE_VIEWER_REQUEST_PENDING, REMOTE_VIEWER_REQUEST_ACTIVE);
		return $q->execute();
	}
	/**
	 * This expires all outstanding invitations which are more than an hour old.
	 * @return boolean
	 */
	function expireAllPendingInvites() {
		$date =& currentDate();
		$time = date('H:i:s');

		$sql = "UPDATE TBL_REMOTE_VIEWER SET REQUEST_STATUS = ?
			WHERE REQUEST_STATUS = ? AND REQUEST_DATETIME < ('".$date->format('Y-m-d')." ".$time."' - INTERVAL 1 HOUR)";
		$q = $this->db->query($sql);
		$q->bind_values(REMOTE_VIEWER_REQUEST_EXPIRED, REMOTE_VIEWER_REQUEST_PENDING);
		return $q->execute();
	}
	/**
	 * This expires all inactives, so active RV's with a last update which is more than an hour ago.
	 * @return boolean
	 */
	function expireAllInactiveInvites() {
		$date =& currentDate();
		$time = date('H:i:s');

		$sql = "UPDATE TBL_REMOTE_VIEWER SET REQUEST_STATUS = ?
			WHERE REQUEST_STATUS = ? AND LAST_UPDATE < ('".$date->format('Y-m-d')." ".$time."' - INTERVAL 1 HOUR)";
		$q = $this->db->query($sql);
		$q->bind_values(REMOTE_VIEWER_REQUEST_EXPIRED, REMOTE_VIEWER_REQUEST_ACTIVE);
		return $q->execute();
	}
	/**
	 *
	 * @param $participant_id
	 * @param $user_id
	 * @return unknown_type
	 */
	function hasPendingOrActiveConnection($participant_id = null, $user_id = null) {
		$participant_id || $participant_id = $this->participant->relation->id;
		$user_id || $user_id = $this->user->id;

		$sql = "SELECT RV.REMOTE_VIEWER_ID, RV.REQUEST_STATUS, R.NAME as CRSO
			FROM TBL_REMOTE_VIEWER as RV
			INNER JOIN TBL_RELATION as R
				ON RV.CRSO_RELATION_ID = R.RELATION_ID
			WHERE RV.PARTICIPANT_RELATION_ID = ?
				AND RV.USER_ID = ?
				AND RV.REQUEST_STATUS IN (?, ?)";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id, $user_id, REMOTE_VIEWER_REQUEST_PENDING, REMOTE_VIEWER_REQUEST_ACTIVE);
		$q->execute();

		$rv = $q->fetchAll();
		// We should only ever have one active or pending rv entry.
		if(count($rv)!=1) {
			$this->expireAll($participant_id, $user_id);
			return false;
		}
		return $rv[0];
	}
	/**
	 *
	 */
	function crsoHasActiveConnection($crso_id = null) {
		$crso_id || $crso_id = $this->crso->relation->id;

		$date =& currentDate();
		$time = date('H:i:s');

		$sql = "SELECT REMOTE_VIEWER_ID
			FROM TBL_REMOTE_VIEWER
			WHERE CRSO_RELATION_ID = ?
				AND REQUEST_STATUS = ?
				AND LAST_UPDATE > ('".$date->format('Y-m-d')." ".$time."' - INTERVAL 1 HOUR)
			";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, REMOTE_VIEWER_REQUEST_ACTIVE);
		$q->execute();
		$rv = $q->fetchAll();
		return (count($rv) > 0);
	}
	/**
	 * @param $participant_id
	 * @param $user_id
	 * @return resultset
	 */
	function getParticipantOverview($participant_id = null, $user_id= null) {
		$participant_id || $participant_id = $this->participant->relation->id;
		$user_id || $user_id = $this->user->id;

		$sql = "SELECT RV.REMOTE_VIEWER_ID as Id, R1.NAME as CRSO,
				RV.REQUEST_STATUS, RV.REQUEST_DATETIME, RV.LAST_UPDATE, '' as ACTION
			FROM TBL_REMOTE_VIEWER as RV
			INNER JOIN TBL_RELATION as R1
				ON RV.CRSO_RELATION_ID = R1.RELATION_ID
			WHERE RV.PARTICIPANT_RELATION_ID = ?
				AND RV.USER_ID = ?
			ORDER BY RV.LAST_UPDATE DESC, RV.REQUEST_DATETIME DESC";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id, $user_id);
		$q->execute();
		return $q;
	}
	/**
	 *
	 * @param $crso_id
	 * @return resultset
	 */
	function getCRSOOverview($crso_id=null) {
		$crso_id || $crso_id = $this->crso->relation->id;

		$date =& currentDate();
		$time = date('H:i:s');

		$sql = "SELECT RV.REMOTE_VIEWER_ID as Id, R.NAME as Participant,
				U.USER_NAME as User,
				CONCAT_WS(' ', IP.INFO_PERSON_FIRST_NAME, IP.INFO_PERSON_MIDDLE_NAME, IP.INFO_PERSON_LAST_NAME) as Person,
				INFO_PERSON_EMAIL as Email,
				RV.REQUEST_STATUS, RV.LAST_UPDATE, '' as View
			FROM TBL_REMOTE_VIEWER as RV
			INNER JOIN TBL_RELATION as R
				ON RV.PARTICIPANT_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_USER as U
				ON RV.USER_ID = U.USER_ID
			INNER JOIN TBL_INFO_PERSON as IP
				ON U.INFO_PERSON_ID = IP.INFO_PERSON_ID
			WHERE RV.CRSO_RELATION_ID = ?
				AND RV.REQUEST_STATUS IN (?, ?)
				AND RV.LAST_UPDATE > ('".$date->format('Y-m-d')." ".$time."' - INTERVAL 1 HOUR)
			ORDER BY RV.REQUEST_DATETIME
			";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, REMOTE_VIEWER_REQUEST_PENDING, REMOTE_VIEWER_REQUEST_ACTIVE);
		$q->execute();
		return $q;
	}

	function getLatestBody($id, $crso_id) {

		$sql = "SELECT REQUEST_STATUS, CONTENT FROM TBL_REMOTE_VIEWER WHERE REMOTE_VIEWER_ID = ? AND CRSO_RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id, $crso_id);
		$q->execute();

		if(!$r = $q->fetch()) {
			throw new IWException('No permission to view this data.');
		}
		if(!in_array($r->REQUEST_STATUS, array(REMOTE_VIEWER_REQUEST_PENDING, REMOTE_VIEWER_REQUEST_ACTIVE))) {
			throw new IWException('No permission to view this data.');
		}

		// If it is pending, set it to active.
		if($r->REQUEST_STATUS==REMOTE_VIEWER_REQUEST_PENDING) {
			$sql = "UPDATE TBL_REMOTE_VIEWER SET REQUEST_STATUS = ? WHERE REMOTE_VIEWER_ID = ?";
			$q = $this->db->query($sql);
			$q->bind_values(REMOTE_VIEWER_REQUEST_ACTIVE, $id);
			$q->execute();

			info(sprintf('CRSO User [%s, %s] started observing Remote Viewer request with ID [%s]',
				 $crso_id, $_SESSION['session_user_id'], $id
			));
		}

		return $r->CONTENT;
	}

	function saveBody($content, $participant_id, $user_id) {

		if(!$r = $this->hasPendingOrActiveConnection($participant_id, $user_id)) {
			return false;
		}

		$sql = "UPDATE TBL_REMOTE_VIEWER SET CONTENT = ? WHERE REMOTE_VIEWER_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($content, $r->REMOTE_VIEWER_ID);
		return $q->execute();
	}

	/**
	 * @param $langIf
	 * @param $status
	 * @return mixed
	 */
	function translateStatus($langIf, $status) {
		switch($status) {
			case REMOTE_VIEWER_REQUEST_PENDING:
				return array($langIf->getText('Request pending'), 'status_new');
			case REMOTE_VIEWER_REQUEST_ACTIVE:
				return array($langIf->getText('Request active'), 'status_verified');
			case REMOTE_VIEWER_REQUEST_CANCELLED:
				return array($langIf->getText('Request cancelled'), 'status_late');
			case REMOTE_VIEWER_REQUEST_EXPIRED:
				return array($langIf->getText('Request expired'), 'status_forfeit');
		}
		return false;
	}

	/**
	 * @see classes/i-w/AbstractObject#_initObjectMaps()
	 */
	function _initObjectMaps() {
		$this->objectmap['dbtable'] = 'TBL_REMOTE_VIEWER';
		$this->objectmap['primary']['id'] = 'REMOTE_VIEWER_ID';
		$this->objectmap['properties']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
		$this->objectmap['properties']['user->id'] = 'USER_ID';
		$this->objectmap['properties']['content'] = 'CONTENT';
		$this->objectmap['properties']['request_status'] = 'REQUEST_STATUS';
		$this->objectmap['properties']['request_datetime'] = 'REQUEST_DATETIME';
		$this->objectmap['properties']['last_updated'] = 'LAST_UPDATE';
	}

}

define('REMOTE_VIEWER_REQUEST_PENDING', 1);
define('REMOTE_VIEWER_REQUEST_ACTIVE', 2);
define('REMOTE_VIEWER_REQUEST_CANCELLED', 3);
define('REMOTE_VIEWER_REQUEST_EXPIRED', 4);

?>
