<?php
require_once(IWARE.'AbstractObject.php');
/**
 * This object handles the trusts between Participants
 *
 * A trust is made when one participant (the subrogator) moves part or all of his/her provisional
 *  to another participant (trustee)
 * This trustee then needs to make the declaration for the subrogator.
 * If a subrogator has no provisional left he/she can chose to disable declarations alltogether.
 *
 * @package ELC
 * @subpackage Subrogation
 * @author Marten Koedam <marten@basesoft.com>
 * @since 2006-02-14
 * @version $Id$
 */
class Trust extends AbstractObject
{
	/**
	 * @var int $id TRUST_ID
	 */
	var $id;
	/**
	 * @var Participant $subrogator
	 */
	var $subrogator;
	/**
	 * @var CRSO $crso
	 */
	var $crso;
	/**
	 * @var Participant $trustee
	 */
	var $trustee;

	/**
	 * @var Product $product
	 */
	var $product;
	/**
	 * @var int $approved
	 */
	var $approved;
	/**
	 * @var ELCDocument $document
	 */
	var $document;
	/**
	 * @var int $terminate_request_status
	 */
	var $terminate_request_status;
	/**
	 * @var string $created
	 */
	var $created;
	/**
	 * @var string $last_update
	 */
	var $last_update;
	/**
	 * Sets the Trust by a TrustId
	 * @param int $id
	 * @return bool
	 */
	function set($id='')
	{
		$id || $id = $this->id;
		$id = (int) $id;

		$sql = "SELECT SUBROGATOR_RELATION_ID, TRUSTEE_RELATION_ID,
				CRSO_RELATION_ID, APPROVED, DOCUMENT_ID,
				TERMINATE_REQUEST_STATUS, CREATED, LAST_UPDATE, PRODUCT_ID
			FROM TBL_TRUST
			WHERE TRUST_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();

		if(!($r = $q->fetch()))
			return false;

		require_once(ELC_CLASSES.'Product.php');
		require_once(ELC_CLASSES.'CRSO.php');
		require_once(ELC_CLASSES.'Participant.php');
		require_once(ELC_CLASSES.'Document.php');

		$this->id = $id;
		$this->subrogator = new Participant($this->db,array('relation'=>new Relation($this->db,$r->SUBROGATOR_RELATION_ID)));
		$this->trustee = new Participant($this->db,array('relation'=>new Relation($this->db,$r->TRUSTEE_RELATION_ID)));
		$this->crso = new CRSO($this->db,array('relation'=>new Relation($this->db,$r->CRSO_RELATION_ID)));
		$this->approved = $r->APPROVED;
		$this->document = $r->DOCUMENT_ID?new ELCDocument($this->db,$r->DOCUMENT_ID):null;
		$this->terminate_request_status = $r->TERMINATE_REQUEST_STATUS;
		$this->created = $r->CREATED;
		$this->last_update = $r->LAST_UPDATE;
		$this->product = new Product($this->db, $r->PRODUCT_ID);
		return $this->complete = true;
	}
	/**
	 * Adds the created date to the default insert of AbstractObject
	 * @return mixed
	 */
	function insert()
	{
		$this->created = date('Y-m-d H:i:s'); // FIXME
		return parent::insert();
	}
	/**
	 * Selects all trusts for the given subrogator and status
	 * @param int $subrogator_id
	 * @param int $approved
	 * @return ResultSet $q
	 */
	function selectSubrogations($subrogator_id='',$approved='',$crso_id='', $product_id='')
	{
		$subrogator_id || $subrogator_id = $this->subrogator->relation->id;
		$product_id || $product_id = $this->product->id;
		$sql = "SELECT TRUST_ID, TRUSTEE_RELATION_ID, CRSO_RELATION_ID, APPROVED, R.`NAME`, T.PRODUCT_ID
			FROM TBL_TRUST T
			INNER JOIN TBL_RELATION as R
				ON T.TRUSTEE_RELATION_ID = R.RELATION_ID
			WHERE SUBROGATOR_RELATION_ID = ?
			AND T.PRODUCT_ID = ? ";
		if($approved)
			$sql .= " AND APPROVED = '".$approved."'";
		if($crso_id)
			$sql .= " AND CRSO_RELATION_ID = ".$crso_id;
		$q = $this->db->query($sql);
		$q->bind_values($subrogator_id, $product_id);
		$q->execute();
		return $q;
	}
	/**
	 * Selects a number of pending subrogations
	 * @param int $subrogator_id
	 * @param int $crso_id
	 * @return int
	 */
	function countPendingSubrogations($subrogator_id=null,$crso_id=null, $product_id='')
	{
		$subrogator_id || $subrogator_id = isset($this->subrogator) ? $this->subrogator->relation->id : null;
		$crso_id || $crso_id = isset($this->crso) ? $this->crso->relation->id : null;
		$product_id || $product_id = $this->product->id;
		$sql = "SELECT COUNT(TRUSTEE_RELATION_ID) AS CNT
			FROM TBL_TRUST
			WHERE SUBROGATOR_RELATION_ID = ?
				AND PRODUCT_ID = ?
				AND CRSO_RELATION_ID = ?
				AND `APPROVED` != ?";
		$q = $this->db->query($sql);
		$q->bind_values($subrogator_id, $product_id, $crso_id,TRUST_APPROVED);
		$q->execute();
		$r = $q->fetch();
		return (int)$r->CNT;
	}

	/**
	 * Selects an overview for the given subrogator
	 * @param int $subrogator_id
	 * @return ResultSet $q
	 */
	function selectSubrogationOverview($subrogator_id='', $product_id='')
	{
		$subrogator_id || $subrogator_id = $this->subrogator->relation->id;
		$product_id || $product_id = $this->product->id;
		$sql = "SELECT '' as CHECKBOX, TRUST_ID,
				TRUSTEE_RELATION_ID, TR.`NAME` as TRUSTEE_NAME, TR.VAT_NO as TRUSTEE_VAT,
				CRSO_RELATION_ID, CR.`NAME` as CRSO_NAME, CR.VAT_NO as CRSO_VAT, '' as DETAILS,
				APPROVED,
				APPROVED & ".TRUST_SUBROGATOR_APPROVED." as TRUST_SUBROGATOR_APPROVED,
				APPROVED & ".TRUST_TRUSTEE_APPROVED." as TRUST_TRUSTEE_APPROVED,
				APPROVED & ".TRUST_CRSO_APPROVED." as TRUST_CRSO_APPROVED,
				IFNULL(DOCUMENT_ID,0) as AGREEMENT_DOCUMENT
			FROM TBL_TRUST as T
			INNER JOIN TBL_RELATION as TR
				ON T.TRUSTEE_RELATION_ID = TR.RELATION_ID
			INNER JOIN TBL_RELATION as CR
				ON T.CRSO_RELATION_ID = CR.RELATION_ID
			WHERE SUBROGATOR_RELATION_ID = ?
			AND T.PRODUCT_ID = ?
			ORDER BY TR.`NAME`, CR.`NAME`";
		$q = $this->db->query($sql);
		$q->bind_values($subrogator_id, $product_id);
		$q->execute();
		return $q;
	}
	/**
	 * Selects all trusts for the given trustee
	 * @param int $trustee_id
	 * @param int $approved
	 * @return ResultSet $q
	 */
	function selectTrusts($trustee_id='',$approved='',$crso_id='', $product_id='')
	{
		$trustee_id || $trustee_id = $this->trustee->relation->id;
		$product_id || $product_id = $this->product->id;
		$sql = "SELECT TRUST_ID, SUBROGATOR_RELATION_ID, R.`NAME`, R.VAT_NO, CRSO_RELATION_ID, APPROVED
			FROM TBL_TRUST as T
			INNER JOIN TBL_RELATION as R
				ON T.SUBROGATOR_RELATION_ID = R.RELATION_ID
			WHERE TRUSTEE_RELATION_ID = ?
			AND PRODUCT_ID = ? ";
		if($approved)
			$sql .= " AND APPROVED = '".$approved."'";
		if($crso_id)
			$sql .= " AND CRSO_RELATION_ID = ".$crso_id;
		$q = $this->db->query($sql);
		$q->bind_values($trustee_id, $product_id);
		$q->execute();
		return $q;
	}

	/**
	 * Selects an overview for the given trustee_id
	 * @param int $trustee_id
	 * @return ResultSet $q
	 */
	function selectTrustOverview($trustee_id='', $product_id='')
	{
		$trustee_id || $trustee_id = $this->trustee->relation->id;
		$product_id || $product_id = $this->product->id;
		$sql = "SELECT TRUST_ID,
				SUBROGATOR_RELATION_ID, SR.`NAME` as SUBROGATOR_NAME, SR.VAT_NO as SUBROGATOR_VAT,
				CRSO_RELATION_ID, CR.`NAME` as CRSO_NAME, CR.VAT_NO as CRSO_VAT, '' as DETAILS,
				APPROVED & ".TRUST_SUBROGATOR_APPROVED." as TRUST_SUBROGATOR_APPROVED,
				APPROVED & ".TRUST_TRUSTEE_APPROVED." as TRUST_TRUSTEE_APPROVED,
				APPROVED & ".TRUST_CRSO_APPROVED." as TRUST_CRSO_APPROVED,
				APPROVED,
				IFNULL(DOCUMENT_ID,0) as AGREEMENT_DOCUMENT
			FROM TBL_TRUST as T
			INNER JOIN TBL_RELATION as SR
				ON T.SUBROGATOR_RELATION_ID = SR.RELATION_ID
			INNER JOIN TBL_RELATION as CR
				ON T.CRSO_RELATION_ID = CR.RELATION_ID
			WHERE TRUSTEE_RELATION_ID = ?
			AND T.PRODUCT_ID = ?
			ORDER BY SR.`NAME`, CR.`NAME`";
		$q = $this->db->query($sql);
		$q->bind_values($trustee_id, $product_id);
		$q->execute();
		return $q;
	}
	/**
	 * Hocus Bogus?
	 * @param int $subrogator_id
	 * @param int $trustee_id
	 * @param int $crso_id
	 * @param int $product_id
	 */
	function getSubrogationStartMonth($subrogator_id='',$trustee_id='',$crso_id='', $product_id='')
	{
		// Get the earliest month possible.
		// If both subrogator and trustee are not PARTICIPANT_READY,
		// the earliest month possible is the start_month of the Subrogator.

		// If the Trustee has already started and the start month
		// of the Subrogator is smaller then the start month of the Trustee,
		// use the Trustee start month, else use the Subrogator start month

		// If the Subrogator has already started use the next available month

		// If both have been started, use the next month.
	}

	/**
	 * Selects all trusts for the given subrogator and crso_id
	 * @param int $participant_id
	 * @param int $crso_id
	 * @return ResultSet $q
	 */
	function selectAvailableTrustees($participant_id,$crso_id, $product_id='')
	{
		$exclude = array($participant_id);
		$sql = "SELECT TRUSTEE_RELATION_ID
			FROM TBL_TRUST
			WHERE SUBROGATOR_RELATION_ID = ?
				AND PRODUCT_ID = ?
				AND CRSO_RELATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($participant_id, $product_id, $crso_id);
		$q->execute();
		while($r=$q->fetch())
			$exclude[] = $r->TRUSTEE_RELATION_ID;

		$sql = "SELECT R.`NAME` as PARTICIPANT_NAME, P.PARTICIPANT_RELATION_ID, R.`VAT_NO`
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CP
			INNER JOIN TBL_PARTICIPANT as P
				USING(PARTICIPANT_RELATION_ID)
			INNER JOIN TBL_RELATION as R
				ON P.PARTICIPANT_RELATION_ID = R.RELATION_ID
			WHERE P.PARTICIPANT_RELATION_ID NOT IN (".implode(',',$exclude).")
				AND CP.CRSO_RELATION_ID = ?
				AND CP.STATUS >= ?
				AND CP.PRODUCER_EXPORTER & ".PARTICIPANT_PRODUCER."
				AND CP.PRODUCT_ID = ?
			ORDER BY PARTICIPANT_NAME";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,PARTICIPANT_STATUS_CONFIRMED, $product_id);
		$q->execute();
		return $q;
	}

	/**
	 * Selects an overview for the CRSO Trust overview
	 * @param int $crso_id
	 * @return ResultSet $q
	 */
	function selectCrsoOverview($crso_id, $product_id='')
	{
		$sql = "SELECT TRUST_ID,
			SUBROGATOR_RELATION_ID, SR.`NAME` as SUBROGATOR_NAME, SR.VAT_NO AS SUBROGATOR_VAT_NO,
			TRUSTEE_RELATION_ID, TR.`NAME` as TRUSTEE_NAME, TR.VAT_NO AS TRUSTEE_VAT_NO,
			APPROVED & ".TRUST_SUBROGATOR_APPROVED." as TRUST_SUBROGATOR_APPROVED,
			APPROVED & ".TRUST_TRUSTEE_APPROVED." as TRUST_TRUSTEE_APPROVED,
			APPROVED & ".TRUST_CRSO_APPROVED." as TRUST_CRSO_APPROVED,
			IF(ISNULL(DOCUMENT_ID),0,1) as AGREEMENT_DOCUMENT
			FROM TBL_TRUST as T
			INNER JOIN TBL_RELATION as SR
				ON T.SUBROGATOR_RELATION_ID = SR.RELATION_ID
			INNER JOIN TBL_RELATION as TR
				ON T.TRUSTEE_RELATION_ID = TR.RELATION_ID
			WHERE
				CRSO_RELATION_ID=?
			AND
				T.PRODUCT_ID = ?
			ORDER BY T.LAST_UPDATE";

		$q = $this->db->query($sql);
		$q->bind_values($crso_id, $product_id);
		$q->execute();
		return $q;
	}

	/**
	 * Delete a trust and move the original provisional back to the Subrogator.
	 * @param int $trust_id
	 * @return bool
	 */
	function delete($trust_id='')
	{
		$trust_id && $this->set($trust_id);
		$trust_id = $this->id;
		if($this->approved == TRUST_APPROVED)
			return false;

		// Start transaction
		$this->db->begin();

		// Check declaration, if any rows, unable to remove!
		require_once(ELC_CLASSES.'Declaration.php');
		$decl = new Declaration($this->db,array('trust'=>$this));
		$q = $decl->getByTrust();
		$rs = $q->fetchAll();
		if(count($rs))
			return false;

		// Check and remove/re-add provisional
		require_once(ELC_CLASSES.'ProvisionalDeclaration.php');
		$prov = new ProvisionalDeclaration($this->db,array('trust'=>$this));
		if(!$prov->removeSubrogation())
		{
			$this->db->rollback();
			return false;
		}

		// Delete the Trust alltogether.
		$sql = "DELETE FROM TBL_TRUST
			WHERE TRUST_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($trust_id);
		if(!$q->execute(true))
		{
			$this->db->rollback();
			return false;
		}

		$this->db->commit();

		return true;
	}
	/**
	 * Translates the approval status
	 * @todo finish me
	 */
	function translateApproval($approval,$langIf)
	{
		switch($approval)
		{
			case TRUST_SUBROGATOR_APPROVED:
			break;
			case TRUST_TRUSTEE_APPROVED:
			break;
			case TRUST_CRSO_APPROVED:
			break;
			case TRUST_APPROVED:
			break;
		}
	}
	/**#@+
	 * @access private
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_TRUST';
		$this->objectmap['primary']['id'] = 'TRUST_ID';
		$this->objectmap['properties']['subrogator->relation->id'] = 'SUBROGATOR_RELATION_ID';
		$this->objectmap['properties']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['trustee->relation->id'] = 'TRUSTEE_RELATION_ID';
		$this->objectmap['properties']['product->id'] = 'PRODUCT_ID';
		$this->objectmap['properties']['approved'] = 'APPROVED';
		$this->objectmap['properties']['document->id'] = 'DOCUMENT_ID';
		$this->objectmap['properties']['terminate_request_status'] = 'TERMINATE_REQUEST_STATUS';
		$this->objectmap['properties']['created'] = 'CREATED';
		$this->objectmap['properties']['last_update'] = 'LAST_UPDATE';
	}
	/**#@-*/
}
/**#@+
 * Approval Status defines
 */
define('TRUST_DISABLED',0);
define('TRUST_SUBROGATOR_APPROVED',1);
define('TRUST_TRUSTEE_APPROVED',2);
define('TRUST_CRSO_APPROVED',4);
define('TRUST_APPROVED',7);
/**#@-*/
/**#@+
 * Request for Termination Status defines
 */
define('TRUST_TERMINATION_SUBROGATOR_APPROVED',1);
define('TRUST_TERMINATION_TRUSTEE_APPROVED',2);
define('TRUST_TERMINATION_CRSO_APPROVED',4);
define('TRUST_TERMINATION_APPROVED',7);
/**#@-*/
?>