<?php
require_once(IWARE.'AbstractObject.php');
require_once(ELC_CLASSES.'Trac.php');
/**
 * The User
 * @author Marten Koedam <marten@basesoft.com>
 * @package ELC
 * @subpackage Core
 * @version $Id$
 * @todo Move to I-Ware, and only leave ELC specific things in a child class
 */
class ELCUser extends AbstractObject
{
	/**
	 * @var string $cleartext
	 */
	var $cleartext='';
	/**
	 * @var string $password
	 */
	var $password='';
	/**
	 * @var string $salt
	 */
	var $salt='';
	/**
	 * @var string $name;
	 */
	var $name='';
	/**
	 * @var string $last_login
	 */
	var $last_login = null;
	/**
	 * @var string $last_password_update The date when the password was last set
	 */
	var $last_password_update = null;
	/**
	 * @var InfoPerson $info_person
	 */
	var $info_person;
	/**
	 * @var int $active
	 */
	var $active;
	/**
	 * @var string $default_page
	 */
	var $default_page;
	/**
	 * @var int $application_id
	 */
	var $application_id;
	/**
	 * @var int $currency_id
	 */
	var $currency_id;
	/**
	 * @var string $login_token It is not stored anywhere, just used temporarily until we send the email with the login token
	 */
	var $login_token='';
	/**
	 * @var TracUser $trac_user Linked TracUser object
	 */
	var $trac_user = null;
	/**
	 * Sets a user by its ID
	 * @param int $id
	 * @return bool
	 */
	function set($id='')
	{
		$id || $id = $this->id;
		$sql = "SELECT  USER_NAME,USER_PASSWORD,USER_ACTIVE,USER_DEFAULT_PAGE,USER_LAST_LOGIN,
			      LAST_PASSWORD_UPDATE,APPLICATION_ID,CURRENCY_ID,INFO_PERSON_ID,USER_SALT,
			      TRAC_USER_ID, ALLOW_CRSO_LOGIN
			      FROM TBL_USER WHERE USER_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();

		if(!($r = $q->fetch()))
			return false;

		$this->id = $id;
		require_once(IWARE.'InfoPerson.php');
		require_once(ELC_CLASSES.'Trac.php');

		$this->info_person = new InfoPerson($this->db,$r->INFO_PERSON_ID);
		$this->name = $r->USER_NAME;
		$this->password = $r->USER_PASSWORD;
		$this->salt = $r->USER_SALT;
		$this->active = $r->USER_ACTIVE;
		$this->default_page = $r->USER_DEFAULT_PAGE;
		$this->last_login = $r->USER_LAST_LOGIN;
		$this->last_password_update = $r->LAST_PASSWORD_UPDATE;
		$this->application_id = $r->APPLICATION_ID;
		$this->currency_id = $r->CURRENCY_ID;
		$this->trac_user =  $r->TRAC_USER_ID ? new TracUser($this->db, $r->TRAC_USER_ID) : null;
		$this->allow_crso_login = $r->ALLOW_CRSO_LOGIN;
		return $this->complete = true;
	}
	/**
	 * Deletes a user and its connections.
	 * @todo Remove Translator, UserOpenBlocks, UserLang, TRANSACTIONS
	 * @param int $id
	 * @return bool;
	 */
	function delete($id='')
	{
		$this->set($id);
		$this->info_person->set();

		// Delete GroupUser
		require_once(IWARE.'GroupUser.php');
		$gu = new GroupUser($this->db,array('user'=>$this));
		$gu->deleteByUserId();

		// Delete RelationUser
		require_once(ELC_CLASSES.'RelationUser.php');
		$ru = new RelationUser($this->db,array('user'=>$this));
		$ru->deleteByUserId();

		// Delete Translator
		require_once(IWARE.'Translator.php');
		$t = new Translator($this->db,array('user'=>$this));
		$t->deleteByUser();

		// Delete UserLang

		// Delete User
		$sql = "DELETE FROM TBL_USER
			WHERE USER_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();

		// Delete InfoPerson
		$this->info_person->delete();

		// Delete Address
		require_once(IWARE.'address.php');
		$address = new Address($this->db,$this->info_person->address_id);
		$address->delete();

		return true;
	}
	/**
	 * Verifies if the username is a unique one in the system.
	 *
	 * This is NOT a case sensitive check, so we CANNOT have `bruno` and `Bruno` otherwise that gets confusing
	 * @param string $name
	 * @param int $exclude // This is used for changing of a username, to exclude the original
	 * @return bool
	 */
	function isUniqueUserName($name='',$exclude='')
	{
		$name || $name = $this->name;
		$sql = "SELECT USER_ID FROM TBL_USER
			WHERE USER_NAME = ? ";
		if($exclude)
			$sql .= "AND USER_ID != '".$exclude."'";
		$q = $this->db->query($sql);
		$q->bind_values($name);
		$q->execute();
		if($r = $q->fetch())
			return false;
		return true;
	}

	/**
	 * Encrypts this->password..
	 * @todo maybe remove $this->cleartext otherwise it might show up in a ticket?
	 * @param string $sClearText  (default: this->cleartext)
	 */
	function setPassword($sClearText)
	{
		if (CRYPT_BLOWFISH == 1)
		{
			$u1 = sha1(uniqid());
			$this->salt = "\$2a\$07\$" . $u1 . "\$";
			$this->password = crypt($sClearText,$this->salt);
		}
		else
			$this->password = sha1($sClearText);
		$this->last_password_update = date('Y-m-d H:i:s');
	}
	/**
	 * Also sets this->cleartext
	 * @todo maybe remove $this->cleartext otherwise it might show up in a ticket?
	 * @return string ClearText password
	 */
	function getPassword()
	{
		// DEPRECATED ! WE STORE PASSWORDS AS A HASH
		//$this->_setCrypter();
		//return $this->cleartext = $this->_crypter->decrypt($this->password);
		info('Calling deprecated getPassword(). (from User.php)');
		throw new IWUserException('Permission Denied');
	}

	/**
	 * Updates the password info of the user and sets a login token. Used when users request a new password.
	 * @return bool generated token
	 */
	function setPasswordInfoAndToken()
	{
		# we don't change the password now, because we check if the new password that they choose is different from the old one
		//$new_password = $this->generatePasswd();
		//$this->setPassword($new_password);
		$this->last_password_update = date('0000-00-00 00:00:00'); // mark the password as expired
		$login_token = $this->createLoginToken($this->id);
		return $login_token;
	}

	/**
	 * Creates a new login token for the given user and sets the $login_token variable and returns the token value
	 * @param integer user_id The id of the user for which we create the login token
	 * @return string The token that is created for the user (sha1 hash)
	 */
	function createLoginToken($user_id)
	{
		do
		{
			$rand_token = sha1(uniqid());
		}
		while ($this->loginTokenExists($rand_token));
		$sql = "INSERT INTO TBL_USER_ONE_TIME_LOGIN (`TOKEN`, `USER_ID`, `GENERATED`, `ACTIVE`) VALUES (?, ?, ?, 1)";
		$q = $this->db->query($sql);
		$q->bind_values($rand_token, $user_id, date('Y-m-d H:i:s'));
		$q->execute();
		return $this->login_token = $rand_token;
	}

	/**
	 * Check if a login token already exists in the database
	 * @param string token
	 * @return bool
	 */
	function loginTokenExists($token)
	{
		$sql = "SELECT USER_ID FROM TBL_USER_ONE_TIME_LOGIN WHERE TOKEN = ?";
		$q = $this->db->query($sql);
		$q->bind_values($token);
		$q->execute();
		if ($r = $q->fetch())
			return true;
		return false;
	}

	/**
	 * Check if a user is allowed to request a new password or not
	 * @param integer $user_id Id of the user we want to check
	 * @return boolean True if the user can request a new password, false if the user recently requested a new password
	 */
	function canRequestNewPassword($user_id='')
	{
		$user_id || $user_id = $this->id;
		$sql = "SELECT UNIX_TIMESTAMP(NOW()) - UNIX_TIMESTAMP(GENERATED) as SECONDS
			FROM TBL_USER_ONE_TIME_LOGIN
			WHERE GENERATED IN (SELECT MAX(GENERATED) FROM TBL_USER_ONE_TIME_LOGIN WHERE USER_ID=?)";
		$q = $this->db->query($sql);
		$q->bind_values($user_id);
		$q->execute();
		if (($r = $q->fetch()) && ($r->SECONDS < NEW_PASSWORD_REQUEST_TIMEOUT))
			return false; // he recently requested a new password
		return true; // doesn't have any previous tokens, so he can ask for a new password
	}

	/**
	 * Method used for invalidating all the login tokens of a user with the given id
	 * @param integer $user_id The id of the user for which we want to invalidate any active token
	 */
	function invalidateAllTokens($user_id='')
	{
		$user_id || $user_id = $this->id;
		$sql = "UPDATE TBL_USER_ONE_TIME_LOGIN SET ACTIVE = 0 WHERE USER_ID = ? AND ACTIVE = 1";
		$q = $this->db->query($sql);
		$q->bind_values($user_id);
		$q->execute();
	}

	/**
	 * Verifies if this is the first login or not.
	 * @return bool
	 */
	function verifyLastLogin($id='')
	{
		debug('Checking for first login');
		$this->set($id);
		if($this->last_login)
		{
			$_SESSION['user_first_login'] = false;
			return true;
		}
		return false;
	}

	/**
	 * Verifies if the password expired or not
	 * @return bool True if the password is not expired, false if it is expired
	 */
	function verifyPasswordNotExpired($id='')
	{
		debug('Checking if password expired');
		$this->set($id);

		$expiry_period = array(
			GROUP_SYSADMIN => PASSWORD_EXPIRES_SYSADMIN,
			GROUP_ACCOUNTANT => PASSWORD_EXPIRES_ACCOUNTANT,
			GROUP_CRSO => PASSWORD_EXPIRES_CRSO,
			GROUP_PARTICIPANT => PASSWORD_EXPIRES_PARTICIPANT,
			GROUP_PUBLIC => PASSWORD_EXPIRES_PUBLIC
		);

		$expires_in = $expiry_period[$_SESSION['session_group_group_id']];

		// it is expired if the date is set to '0000-00-00 00:00:00'
		if ($this->last_password_update == '0000-00-00 00:00:00')
			return false;

 		// it doesn't expire if it's null or expiry period is 0
		if (!$this->last_password_update || ($expires_in == 0))
		{
			$_SESSION['user_password_expired'] = false;
			return true;
		}

		$expiry_timestamp = strtotime($this->last_password_update);
		$expiry_timestamp += 86400 * $expires_in; // add the number of days
		$expiry_date = date("Y-m-d H:i:s", $expiry_timestamp);

		if ($expiry_date > date('Y-m-d H:i:s'))
		{
			$_SESSION['user_password_expired'] = false;
			return true;
		}
		return false;
	}

	/**
	 * Weak password check, not the same as previous password.
	 * At least 2 digits, 1 uppercase alpha and 1 lowercase alpha
	 *
	 * @todo make work with the crack_opendict to check for realwords.
	 * @param string $passwd
	 * @return boolean
	 */
	public static function validatePassword($passwd='')
	{
		return cracklibCheck($passwd, $message);
	}

	/**
	 * Return the minimum length of the passwor based on universe level, or false if it can't be determined
	 * @param integer $group The group for which we want to know the minimum password length (GROUP_SYSADMIN, etc...). If not specified, it's taken from the SESSION
	 * @return mixed False if the universe is not passed as a parameter or it can't be determined, or the minimum length
	 */
	public static function getMinimumPasswordLength($group='')
	{
		$group || $group = $_SESSION['session_group_group_id'];
		if (!$group)
			return false;
		$min_password_length = array(
			GROUP_SYSADMIN => PASSWORD_LENGTH_SYSADMIN,
			GROUP_ACCOUNTANT => PASSWORD_LENGTH_ACCOUNTANT,
			GROUP_CRSO => PASSWORD_LENGTH_CRSO,
			GROUP_PARTICIPANT => PASSWORD_LENGTH_PARTICIPANT,
			GROUP_PUBLIC => PASSWORD_LENGTH_PUBLIC
		);
		return $min_password_length[$group];
	}

	/**
	 * Check the lenght of the password, depending on the universe. If minimum length can't be set, use 8
	 * @param string $passwd
	 * @return boolean True if the password has the minimum length required
	 */
	public static function validatePasswordLength($passwd='')
	{
		$min_length = ELCUser::getMinimumPasswordLength();
		$min_length || $min_length = 8;

		$passwd || $passwd = $this->password;

		return mb_strlen($passwd) >= $min_length;
	}

	/**
	 * Generates a random password that complies to the weakpassword check
	 * @param int $size
	 * @param int $digits
	 * @param int $uppercase
	 * @param int $lowercase
	 * @return string $password
	 */
	public static function generatePasswd($size='',$digits=2,$uppercase=1,$lowercase=1)
	{
		if (!$size)
		{
			if (isset($_SESSION['session_group_group_id']))
			{
				$min_password_length = array(
					GROUP_SYSADMIN => PASSWORD_LENGTH_SYSADMIN,
					// TODO: add ACCOUNTANT requirements.
					GROUP_CRSO => PASSWORD_LENGTH_CRSO,
					GROUP_PARTICIPANT => PASSWORD_LENGTH_PARTICIPANT,
					GROUP_PUBLIC => PASSWORD_LENGTH_PUBLIC
				);
				$size = $min_password_length[$_SESSION['session_group_group_id']];
			}
			else
			{
				$size = 8;
			}
		}

		$characterArray = array();
		while($digits)
		{
			$characterArray[] = round(rand(0,9));
			$digits--;
			$size--;
		}
		while($uppercase)
		{
			$characterArray[] = ELCUser::_getCharacter(true);
			$uppercase--;
			$size--;
		}
		while($lowercase)
		{
			$characterArray[] = ELCUser::_getCharacter();
			$lowercase--;
			$size--;
		}
		if($size<=0)
		{
			shuffle($characterArray);
			return implode('',$characterArray);
		}
		while($size)
		{
			$type = round(rand(0,2));
			if($type==0)
				$characterArray[] = round(rand(0,9));
			elseif($type==1)
				$characterArray[] = ELCUser::_getCharacter();
			elseif($type==2)
				$characterArray[] = ELCUser::_getCharacter(true);
			$size--;
		}

		shuffle($characterArray);
		return implode('',$characterArray);
	}
	/**
	 * Gets a character
	 * @param bool $upper // Uppercase or lowercase switch
	 * @return char
	 */
	public static function _getCharacter($upper=false)
	{
		if($upper)
			return strtoupper(chr(round(rand(97,122))));
		return chr(round(rand(97,122)));
	}
	/**
	 * Sets a user by its email address //uniqueness?
	 * @param string $email
	 * @return Array (ELCUser, ELCUser, ELCUser)
	 */
	function getByEmail($email)
	{
		require_once(IWARE.'InfoPerson.php');
		$sql = "SELECT TBL_USER.USER_ID
			FROM TBL_USER
			INNER JOIN TBL_INFO_PERSON
				USING(INFO_PERSON_ID)
			WHERE INFO_PERSON_EMAIL = ?";
		$q = $this->db->query($sql);
		$q->bind_values($email);
		$q->execute();
		$rs = $q->fetchAll();
		if(!count($rs))
			return false;
		$rv = array();
		foreach($rs as $r) {
			$rv[] = new ELCUser($this->db, $r->USER_ID, true);
		}
		return $rv;
	}
	/**
	 * Sets a user by its name
	 * @todo Refactor with set()
	 * @param string $name
	 * @return bool
	 */
	function setByName($name)
	{
		$sql = "SELECT * FROM TBL_USER
			WHERE BINARY USER_NAME = ?";
		$q = $this->db->query($sql);
		$q->bind_values($name);
		if(!$q->execute(true))
			return false;
		if(!($r = $q->fetch()))
			return false;

		require_once(IWARE.'InfoPerson.php');

		$this->info_person = new InfoPerson($this->db,$r->INFO_PERSON_ID);
		$this->id = $r->USER_ID;
		$this->name = $r->USER_NAME;
		$this->password = $r->USER_PASSWORD;
		$this->salt = $r->USER_SALT;
		$this->active = $r->USER_ACTIVE;
		$this->default_page = $r->USER_DEFAULT_PAGE;
		$this->last_login = $r->USER_LAST_LOGIN;
		$this->last_password_update = $r->LAST_PASSWORD_UPDATE;
		$this->application_id = $r->APPLICATION_ID;
		$this->currency_id = $r->CURRENCY_ID;
		$this->trac_user =  $r->TRAC_USER_ID ? new TracUser($this->db, $r->TRAC_USER_ID) : null;
		$this->allow_crso_login = $r->ALLOW_CRSO_LOGIN;
		return $this->complete = true;
	}
	/**
	 * Select all users for a certain group
	 * @param int $group_id
	 * @param string $select
	 * @param int $exclude
	 */
	function selectByGroup($group_id, $select='', $exclude=1)
	{
		$sql = "SELECT '' as CHECKBOX, U.USER_ID, USER_NAME, USER_ACTIVE,
				COUNT(T.LANGUAGE_ID) as TRANSLATE ".$select."
			FROM TBL_USER as U
			INNER JOIN TBL_GROUP_USER as G
				USING(USER_ID)
			LEFT JOIN TBL_TRANSLATOR as T
				USING(USER_ID)
			WHERE G.GROUP_ID = ? ";
		if($exclude)
			$sql .= "AND U.USER_ID != '".$exclude."' ";
		$sql .= "GROUP BY USER_ID
			ORDER BY USER_NAME";
		$q = $this->db->query($sql);
		$q->bind_values($group_id);
		$q->execute();
		return $q;
	}
	public static function checkInGroup($db,$group_id,$user_id)
	{
		$sql = "SELECT GROUP_ID, USER_ID FROM TBL_GROUP_USER
			WHERE GROUP_ID = ? AND USER_ID = ?";
		$q = $db->query($sql);
		$q->bind_values($group_id,$user_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		return true;
	}
	/**
	 * Selects all the users for a certain relation
	 * @param int $relation_id
	 * @return ResultSet $q
	 */
	function selectByRelation($relation_id)
	{
		$sql = "SELECT U.USER_ID, USER_NAME
			FROM TBL_USER U
			INNER JOIN TBL_RELATION_USER RU
				ON U.USER_ID=RU.USER_ID
			WHERE
				RU.RELATION_ID=?";
		$q = $this->db->query($sql);
		$q->bind_values($relation_id);
		$q->execute();
		return $q;
	}
	/**
	 * Overriding for informational purposes
	 * @return mixed
	 */
	function insert()
	{
		info(sprintf('Created new User [%s]',
			$this->name));
		return parent::insert();
	}
	/**
	 * Overriding for informational purposes
	 * @return mixed
	 */
	function update()
	{
		info(sprintf('Updated User [%s]',
			$this->name));
		return parent::update();
	}

	function &getAdminUsersForOperator($type=false,$only_admin=true)
	{
		require_once(IWARE.'group.php');
		$sql = "
			SELECT U.USER_ID, U.USER_NAME, GU.GROUP_ID, RU.RELATION_ID
			FROM TBL_USER AS U
			INNER JOIN TBL_RELATION_USER AS RU
			ON U.USER_ID = RU.USER_ID
			INNER JOIN TBL_GROUP_USER AS GU
			ON U.USER_ID = GU.USER_ID
			WHERE
				U.USER_ACTIVE = ?
			AND
				RU.STATUS = ?";
		if($only_admin)
			$sql .="
				AND
					RU.IS_ADMIN = ?";
		$sql .="
			AND
				GU.GROUP_ID IN (?, ?, ?)
			ORDER BY GU.GROUP_ID, U.USER_NAME
				";
		$q = $this->db->query($sql);
		if($only_admin)
			$q->bind_values(1, 1, 1, GROUP_CRSO, GROUP_PARTICIPANT,GROUP_ACCOUNTANT);
		else
			$q->bind_values(1, 1, GROUP_CRSO, GROUP_PARTICIPANT,GROUP_ACCOUNTANT);
		$q->execute();
		if($type)
			return $q;
		$rv = array('CRSO' => array(), 'Participant' => array());
		while($r = $q->fetch())
		{
			if($r->GROUP_ID == GROUP_CRSO)
				$rv['CRSO'][] = array('loginname' => $r->USER_NAME,
									  'id' => $r->RELATION_ID);
			else
				$rv['Participant'][] = array('loginname' => $r->USER_NAME,
					 						 'id' => $r->RELATION_ID);
		}
		return $rv;
	}

	function getUsers()
	{
		$sql = "SELECT U.USER_ID,U.USER_NAME,IP.INFO_PERSON_EMAIL as EMAIL
			FROM TBL_USER as U
			INNER JOIN TBL_INFO_PERSON as IP
				ON U.INFO_PERSON_ID = IP.INFO_PERSON_ID
			ORDER BY U.USER_NAME";

		$q = $this->db->query($sql);
		$q->execute();
		return $q;
	}

	/**
	 * @access protected
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_USER';
		$this->objectmap['primary']['id'] = 'USER_ID';
		$this->objectmap['properties']['info_person->id'] = 'INFO_PERSON_ID';
		$this->objectmap['properties']['name'] = 'USER_NAME';
		$this->objectmap['properties']['password'] = 'USER_PASSWORD';
		$this->objectmap['properties']['salt'] = 'USER_SALT';
		$this->objectmap['properties']['active'] = 'USER_ACTIVE';
		$this->objectmap['properties']['default_page'] = 'USER_DEFAULT_PAGE';
		$this->objectmap['properties']['last_login'] = 'USER_LAST_LOGIN';
		$this->objectmap['properties']['last_password_update'] = 'LAST_PASSWORD_UPDATE';
		$this->objectmap['properties']['application_id'] = 'APPLICATION_ID';
		$this->objectmap['properties']['currency_id'] = 'CURRENCY_ID';
		$this->objectmap['properties']['trac_user->id'] = 'TRAC_USER_ID';
		$this->objectmap['properties']['allow_crso_login'] = 'ALLOW_CRSO_LOGIN';
	}
}

?>
