<?php
/**
 * FileDescription
 *
 * @package PACKAGE
 * @subpackage SUBPACKAGE
 * @since Dec 15, 2006
 * @author Martijn van den Broek <mjbroek@gmail.com>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
require_once(ELC_CLASSES.'User.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');

class WEEEController
{
	protected $db;
	protected $cpp;
	protected $user;
	protected $fb;

	/**
	 * some arguments are optional because not all functions need them. For
	 * example authenticate()
	 */
	function __construct(IWDatabase $db, CRSOParticipantProduct $cpp, ELCUser $user, FormBehaviour $fb = null)
	{
		$this->db = $db;
		$this->user = $user;
		$this->cpp = $cpp;

		if(is_null($fb))
		{
			require_once(IWARE.'FormBehaviour.php');
			require_once(ELC_CLASSES.'FormBehaviourPlugins.php');
			$this->fb = new FormBehaviour(IWRegistry::registry('langIf'), IWRegistry::registry('db'));
			$this->fb->setValues(array()); // we dont want this FormBehaviour to work on any $_POST data
		}
		else
			$this->fb = $fb;

		$this->fb->setValues(array()); // we dont want this FormBehaviour to work on any $_POST data
	}


	static $instance;
	static public function instance(CRSOParticipantProduct $cpp, ELCUser $user)
	{
		$db = IWRegistry::registry('db');

		if(!isset(self::$instance))
			self::$instance = new WEEEController($db, $cpp, $user);

		return self::$instance;

	}

	/**
	 * Checks for the current participant for clearance
	 * @throws IWException
	 */
	protected function checkClearance($title, $level)
	{
		require_once(ELC_CLASSES.'RelationUser.php');
        $ru = new RelationUser($this->db);

		if(!$ru->hasClearance($level, $this->user->id, $this->cpp->participant->relation->id))
		{
			throw new WEEEClearanceException(sprintf('The user [%s] does not have clearance %s for the Participant [%s]',
				$this->user->id, $title, $this->cpp->participant->relation->id ));
		}
	}

	/**
	 * Checks the current CRSOParticipant for ProducerExporter
	 * @throws IWException
	 */
	protected function checkProducerExporter($title, $producer_exporter)
	{
		if(! ($this->cpp->producer_exporter & $producer_exporter) )
			throw new WEEEProducerExporterException(sprintf('Participant [%s] is not an %s', $this->cpp->participant->relation->id, $title));
	}

	/**
	 * Checks if the current partiipant is authenticated
	 * @throws IWException
	 */
	protected function checkAuthenticated()
	{
		if (!$this->cpp->complete && !$this->cpp->set())
			throw new WEEEAuthenticateException(sprintf('Authentication failure. CRSO [%s] Participant [%s] Product [%s] does not exist',
				$this->cpp->crso->relation->id, $this->cpp->participant->relation->id, $this->cpp->product->id));

		if (!$this->user->complete && !$this->user->set())
			throw new WEEEAuthenticateException(sprintf('Authentication failure. User [%s] does not exist', $this->user->id));

		if (!$this->user->active)
			throw new WEEEAuthenticateException(sprintf('Authentication failure. User [%s] is not active', $this->user->id));
	}

	/**
	 * returns the participant object from the cpp
	 */
	public function getCpp()
	{
		if (!$this->cpp)
			throw new IWException('Unable to get cpp, it is probably never set');

		return $this->cpp;
	}

	public function getUser()
	{
		if (!$this->user)
			throw new IWException('Unable to get user, it is probably never set');

		return $this->user;
	}
}

/**
 * Exceptions thrown in this class
 */
class WEEEException extends IWException {};
class WEEEClearanceException extends WEEEException {};
class WEEEProducerExporterException extends WEEEException {};
class WEEEAuthenticateException extends WEEEException {};

?>