<?php
require_once(IWARE.'AbstractObject.php');
/**
 * @author Marten Koedam <marten@basesoft.com>
 * @package I-Ware
 * @version $Id$
 */
class WEEECountry extends AbstractObject
{
	/**
	 * @var int $id
	 */
	var $id;
	/**
	 * @var string $name
	 */
	var $name = '';
	/**
	 * @var int $code
	 */
	var $code = '';
	/**
	 * @var int $settings
	 */
	var $settings = 0;
	/**
	 * @var int $settingsArray
	 */
	var $settingsArray = array('HasRegNumberAsProducer', 'HasDeclByCategory', 'HasDeclByBrand',
		'HasDeclOfWeight', 'HasProvisionalPerMonth', 'HasDocumentUpload', 'HasCrsoProvisionalDetails',
		'HasParticipantOneCrso','HasCrsoDeclarationDetails','HasRefundInternalVAT','HasCrsoRefundDetails',
		'HasRefundRequest','HasSubrogation','HasDeclarationComponent','HasCrsoOneExportDay','HasOneInvoice',
		'HasPopSupplyChain','HasCreditViaProducer','HasDeclarationDimension', 'HasPoeAdditionalDocuments',
		'HasParticipantAgeingAccess', 'HasProofOfPaymentMultiline', 'HasCRSORefundConfirmations',
		'HasAnnualConfirmationDisabled', 'HasSendInvoicesAsAttachment', 'HasGenericReportByParticipant',
		'HasDeEarReporting', 'HasWeightAsBasisForAmount', 'HasRequiredPoEDocumentUploadEU',
		'HasDeclOfWeightWithDecimals', 'HasOptionalRefundImporterVAT', 'HasOmitExportOnOnlyWeightChanges',
		'HasParticipantCannotEditCompany', 'HasCrsoParticipantLogin');

	var $legal_notice;
	var $ISO_3166_a2;

	/**
	 * @var int $id
	 */
	function set($id='')
	{
		$id || $id = $this->id;

		$sql = "SELECT `NAME`, `CODE`, SETTINGS, COUNTRY_LOCALE, REGION_ID, ISO_3166_a2, LEGAL_NOTICE
			FROM TBL_COUNTRY
			WHERE COUNTRY_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);

		if(!$q->execute())
			return false; // Throw
		if (!($r = $q->fetch()))
			return false;

		$this->id = $id;
		$this->name = $r->NAME;
		$this->code = $r->CODE;
		$this->settings = (float) $r->SETTINGS;
		$this->locale = $r->COUNTRY_LOCALE;
		$this->region_id = $r->REGION_ID;
		$this->legal_notice = $r->LEGAL_NOTICE;
		$this->ISO_3166_a2 = $r->ISO_3166_a2;
		return $this->complete = true;
	}
	/**
	 * @param Array $country_ids
	 */
	function selectOverview($country_ids = array())
	{
		$sql = "SELECT COUNTRY_ID, `CODE`, `NAME` as COUNTRY_NAME, SETTINGS,
			'' as ".implode(", '' as ",$this->settingsArray).",
			IF(ISNULL(LEGAL_NOTICE),'add','update') as LEGAL_NOTICE,
			'' as ProductSettings
			FROM TBL_COUNTRY ";
		if($country_ids)
			$sql .= " WHERE COUNTRY_ID IN( ".implode(',',$country_ids)." ) ";
		$sql .= "ORDER BY `NAME`";
		$q = $this->db->query($sql);
		$q->execute();
		return $q;
	}
	/**
	 * @param Array $country_ids
	 */
	function select($country_ids=array())
	{
		$sql = "SELECT COUNTRY_ID, SUBSTRING(TBL_COUNTRY.`NAME`, 1, 20) AS `NAME`, `CODE`, `REGION_ID`
			FROM TBL_COUNTRY";
		if($country_ids)
			$sql .= " WHERE COUNTRY_ID IN ( ".implode(',',$country_ids)." ) ";
		$sql .= " ORDER BY `NAME`";
		$q = $this->db->query($sql);
		$q->execute();
		return $q;
	}

	function selectForInspection()
	{
		$sql = "SELECT
					c.COUNTRY_ID,
					SUBSTRING(c.`NAME`, 1, 20) AS `NAME`,
					c.`CODE`,
					c.`REGION_ID`
				FROM
					TBL_COUNTRY AS c
			ORDER BY c.`NAME`";
		$q = $this->db->query($sql);
		$q->execute();
		return $q;
	}


	/**
	 *
	 */
	function _selectByRegion($region_id,$return='q')
	{
		$sql = "SELECT COUNTRY_ID, `NAME`, `CODE`
			FROM TBL_COUNTRY
			WHERE REGION_ID = ?
			ORDER BY `CODE`, `NAME`";
		$q = $this->db->query($sql);
		$q->bind_values($region_id);
		$q->execute();
		if($return=='q')
			return $q;
		$rv = array();
		while($r = $q->fetch())
			$rv[$r->COUNTRY_ID] = $r->NAME;
		return $rv;
	}
	/**
	 * Check if the current country already has provisional declarations,
	 * if it does, then dont allow the country settings to be updated;
	 *
	 */
	function validateUpdate()
	{
		$sql = "SELECT COUNT(PD.QUANTITY) as PDS
			FROM TBL_PROVISIONAL_DECLARATION as PD
			INNER JOIN TBL_CRSO as C
				ON PD.CRSO_RELATION_ID = C.CRSO_RELATION_ID
			INNER JOIN TBL_CRSO_PRODUCT as CP
				ON PD.CRSO_RELATION_ID = CP.CRSO_RELATION_ID
					AND PD.PRODUCT_ID = CP.PRODUCT_ID
			INNER JOIN TBL_RELATION as R
				ON C.CRSO_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_ADDRESS as A
				USING(ADDRESS_ID)
			WHERE A.COUNTRY_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		$r = $q->fetch();
		if($r->PDS>0)
			return false;
		return true;
	}

	/**
	 * @param int $check (defined below)
	 */
	function has($check, $product_id=null)
	{
		// TODO: Remove when all the HAS calls have changed properly.
		// $product_id || $product_id = $_SESSION['selected_product_id'];
		require_once(ELC_CLASSES.'CountryProduct.php');

		$cp = new CountryProduct($this->db,array('country'=>$this));

		if(!$product_id || (!$cp->set(null,$product_id))) {
			$settings = $this->settings;
		} else {
			$settings = $cp->settings;
		}

		if (PHP_INT_SIZE == 4) {
			// 32 bit
			$settings = gmp_init((string) $settings);
			$check = gmp_init((string) $check);
			return gmp_cmp(gmp_and($settings, $check), $check) == 0;

		} elseif (PHP_INT_SIZE == 8) {
			// 64 bit
			return ($settings & $check) == $check;

		} else {
			throw new Exception('invalid PHP_INT_SIZE');
		}
	}

	/**
	 * Selects all participants registred in this country
	 */
	function selectRegistredParticipants($product_id)
	{
		$sql = "SELECT p.PARTICIPANT_RELATION_ID
			FROM TBL_PARTICIPANT p
				INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT cp USING (PARTICIPANT_RELATION_ID)
				INNER JOIN TBL_RELATION r ON cp.CRSO_RELATION_ID=r.RELATION_ID
				INNER JOIN TBL_ADDRESS USING (ADDRESS_ID)
				INNER JOIN TBL_COUNTRY c USING (COUNTRY_ID)
				WHERE c.COUNTRY_ID=?
					AND cp.PRODUCT_ID=?";

		$q = $this->db->query($sql);
		$q->bind_values($this->id, $product_id);
		$q->execute();
		return $q;
	}

	/**
	 * Selects all CRSOs with ID and NAME for the given country_id
	 * @param int $country_id
	 * @return Query $q
	 */
	function selectCRSOs($id='')
	{
		$id || $id = $this->id;

		$sql = "SELECT c.CRSO_RELATION_ID, r.NAME
			FROM TBL_RELATION r
			INNER JOIN TBL_CRSO c
				ON r.RELATION_ID=c.CRSO_RELATION_ID
			INNER JOIN TBL_ADDRESS a
				ON a.ADDRESS_ID=r.ADDRESS_ID
			WHERE a.COUNTRY_ID=?
		";

		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();

		return $q;
	}

	/**
	 * Fills a <select> with available countries split by region
	 * @param InterfaceItem $interface_item
	 * @param int $selected_country_id
	 */
	function fillInterfaceItem($interface_item, $selected_country_id)
	{
    	$q = $this->select();
    	$interface_item->firstOption = array('value'=>'','text'=>'- '.$interface_item->langIf->getText('Select Country').' -');
    	$group_eu = array();
    	$group_world = array();
    	while ($row = $q->fetch())
    	{
    		$option = array('text'=>$row->NAME, 'value'=>$row->COUNTRY_ID,
				'selected'=>($row->COUNTRY_ID == $selected_country_id) ? 'selected' : false);
			if ($row->REGION_ID == REGION_EU )
				$group_eu[] = $option;
			else
				$group_world[] = $option;
    	}

    	$interface_item->optgroups[] = array($interface_item->langIf->getText('Europe'), $group_eu);
    	$interface_item->optgroups[] = array($interface_item->langIf->getText('World'), $group_world);

	}

	/**
	 * Gets all available country settings
	 */
	function getSettings()
	{
		return $this->settingsArray;
	}

	/**#@+
	 * @access protected
	 */
	/**
	 *
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_COUNTRY';
		$this->objectmap['primary']['id'] = 'COUNTRY_ID';
		$this->objectmap['properties']['name'] = 'NAME';
		$this->objectmap['properties']['code'] = 'CODE';
		$this->objectmap['properties']['settings'] = 'SETTINGS';
		$this->objectmap['properties']['locale'] = 'COUNTRY_LOCALE';
		$this->objectmap['properties']['legal_notice'] = 'LEGAL_NOTICE';
		$this->objectmap['properties']['region_id'] = 'REGION_ID'; //@todo make object, make n:n relation
		$this->objectmap['properties']['ISO_3166_a2'] = 'ISO_3166_a2';
	}
	/**
	 * References to ITEM ids
	 *  I dont know another way to do this yet EEP!!!!!!!
	 */
	function _initSettingsItemIdMap()
	{
		$this->itemmap[HasDeclByCategory]					= 449;
		$this->itemmap[HasDeclByBrand]						= 450;
		$this->itemmap[HasDeclOfWeight]						= 454;
		$this->itemmap[HasProvisionalPerMonth]				= 448;
		$this->itemmap[HasDocumentUpload]					= '';
		$this->itemmap[HasRegNumberAsProducer]				= 422;
		$this->itemmap[HasCrsoProvisionalDetails]			= '';
		$this->itemmap[HasParticipantOneCrso]       		= '';
		$this->itemmap[HasCrsoDeclarationDetails]			= '';
		$this->itemmap[HasRefundInternalVAT] 				= 864;
		$this->itemmap[HasCrsoRefundDetails] 				= '';
		$this->itemmap[HasRefundRequest] 					= '';
		$this->itemmap[HasSubrogation]		 				= '';
		$this->itemmap[HasDeclarationComponent]				= '';
		$this->itemmap[HasCrsoOneExportDay]					= '';
		$this->itemmap[HasOneInvoice]			    		= '';
		$this->itemmap[HasPopSupplyChain]           		= '';
		$this->itemmap[HasCreditViaProducer]        		= '';
		$this->itemmap[HasDeclarationDimension]				= 1300;
		$this->itemmap[HasPoeAdditionalDocuments]			= '';
		$this->itemmap[HasParticipantAgeingAccess]			= '';
		$this->itemmap[HasProofOfPaymentMultiline]			= '';
		$this->itemmap[HasCRSORefundConfirmations] 			= '';
		$this->itemmap[HasAnnualConfirmationDisabled] 		= '';
        $this->itemmap[HasSendInvoicesAsAttachment] 		= '';
        $this->itemmap[HasGenericReportByParticipant]   	= '';
        $this->itemmap[HasDeEarReporting]   				= '';
        $this->itemmap[HasWeightAsBasisForAmount] 			= '';
		$this->itemmap[HasRequiredPoEDocumentUploadEU] 		= '';
		$this->itemmap[HasDeclOfWeightWithDecimals]			= 1609;
		$this->itemmap[HasOptionalRefundImporterVAT]		= '';
		$this->itemmap[HasOmitExportOnOnlyWeightChanges]	= '';
		$this->itemmap[HasParticipantCannotEditCompany]		= '';
		$this->itemmap[HasCrsoParticipantLogin]		= '';
	}
	/**#@-*/
	/**
	 *
	 */
	function initSettingsItemIdMap()
	{
		$this->_initSettingsItemIdMap();
	}
	var $itemmap = array();

	function getLegalNotice($id=false)
	{
		$id || $id = $this->id;
		$sql = "SELECT LEGAL_NOTICE
			FROM TBL_COUNTRY
			WHERE COUNTRY_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		$r = $q->fetch();
		return $r->LEGAL_NOTICE;
	}
}
?>
