<?php
/**
 * Created by PhpStorm.
 * User: marten
 * Date: 25/1/17
 * Time: 10:19
 */
require_once(ELC_CLASSES.'Relation.php');
require_once(ELC_CLASSES.'RelationUser.php');
require_once(ELC_CLASSES.'CRSOParticipant.php');
require_once(ELC_CLASSES.'WEEEController.php');
require_once(ELC_CLASSES.'ro/WasteReport.php');

// Doesn't extend the WEEEController as this controller is used only by the CRSO users. No need for CPP objects
class WasteReportController {

	function __construct(IWDatabase $db, CRSO $crso, ELCUser $user)
	{
		$this->db = $db;
		$this->user = $user;
		$this->crso = $crso;
		$this->log = getLog();
	}

	private function checkAuthenticated() {
		$ru = new RelationUser($this->db);

		// $ru->checkClearance()e
		if(!$ru->isAdmin($this->user->id, $this->crso->relation->id)) { // Should this be a clearance level?
			throw new WEEEException('Only the CRSO admin user is allowed to insert new Waste Report data.');
		}

	}

	private function validateParticipant($participant_id) {

		$cp = new CRSOParticipant($this->db);
		if(!$cp->set($this->crso->relation->id, $participant_id)) {
			throw new WEEEException('Invalid participant ID provided');
		}

	}

	function insert($participant_id, $year, $month, $weight) {
		$this->checkAuthenticated();
		$this->_insert($participant_id, $year, $month, $weight);
	}

	private function _insert($participant_id, $year, $month, $weight) {
		$this->validateParticipant($participant_id);
		$participant = new Participant($this->db, array('relation'=>new Relation($this->db, $participant_id)));

		$wr = new WasteReport($this->db, array('crso'=>$this->crso, 'participant'=>$participant));
		$wr->year = $year;
		$wr->month = $month;
		$wr->weight = $weight;

		$wr->insert();
		$this->log->info(sprintf('Inserted new Waste Report information for Participant %s, year %s, month %s with weight %s', $participant_id, $year, $month, $weight));

	}

	private function getParticipantByVatNumber($vat_no) {
		return Relation::getByVAT($vat_no, $this->db);
	}

	function upload($file) {

		$this->checkAuthenticated();

		$this->db->begin();

		$csv = array_map('str_getcsv', file($file));
		if(sizeof($csv[0]) != 5) {
			throw new WEEEException('Incorrect number of columns in CSV file detected.');
		}
		$csv[0] = array('name', 'vat_no', 'month', 'year', 'weight');
		array_walk($csv, function(&$a) use ($csv) {
			$a = array_combine($csv[0], $a);
		});
		array_shift($csv); // We don't need the first row.
		$name = null;
		$vat_no = null;
		$month = null;
		$year = null;
		$weight = null;

		foreach($csv as $index => $row) {

			extract($row); // creates: $name, $vat_no, $month, $year, $weight
			$relation_id = $this->getParticipantByVatNumber($vat_no);
			if(!$relation_id) {
				$this->db->rollback();
				throw new WEEEException(sprintf('The participant with VAT number "%s" could not be found; Line %s', $vat_no, $index+1));
			}
			$this->_insert($relation_id, $year, $month, $weight);

		}

		$this->db->commit();

	}

}