<?php
/**
 * Created on Jan 8, 2008
 *
 * @author Nicoleta Militaru <nicoleta@basesoft.com>
 **/

require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'uk/UKCRSODataLine.php');
require_once(ELC_CLASSES.'uk/UKCRSOSupplier.php');
require_once(ELC_CLASSES.'uk/uk_define.php');

//for the status
define('DATA_READY',1);
define('PUBLISHED',2);
define('CRSO_DATA_EXPORTED',3);
define('CRSO_DATA_ACCEPTED',4);

class UKNoDataException extends IWUserException {}


class UKCRSOData extends AbstractObject
{
	var $id;
	var $crso_id;
	var $status;
	var $year;
	var $quarter;
	var $last_update;
	var $last_exported;

	function setID($data_id='')
	{
		$data_id || $data_id = $this->id;

		$sql = " SELECT CRSO_RELATION_ID,DATA_ID,`YEAR`,`QUARTER`,`STATUS`,LAST_UPDATE,LAST_EXPORTED
				FROM TBL_UK_CRSO_DATA
				WHERE DATA_ID = ? ";
		$q = $this->db->query($sql);
		$q->bind_values($data_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		$this->id = $r->DATA_ID;
		$this->crso_id = $r->CRSO_RELATION_ID;
		$this->status = $r->STATUS;
		$this->year = $r->YEAR;
		$this->quarter = $r->QUARTER;
		$this->last_update = $r->LAST_UPDATE;
		$this->last_exported = $r->LAST_EXPORTED;
		return $this->complete = true;
	}
	function set($crso, $year, $quarter)
	{
		$sql = " SELECT CRSO_RELATION_ID,DATA_ID,`YEAR`,`QUARTER`,`STATUS`,LAST_UPDATE,LAST_EXPORTED
				FROM TBL_UK_CRSO_DATA
				WHERE CRSO_RELATION_ID=? AND YEAR =? AND QUARTER=?";
		$q = $this->db->query($sql);
		$q->bind_values($crso,$year,$quarter);
		$q->execute();
		if(!($r = $q->fetch())) {
			throw new UKNoDataException('No data available for year '.$year.' and quarter '.$quarter);
		}
		$this->id = $r->DATA_ID;
		$this->crso_id = $r->CRSO_RELATION_ID;
		$this->status = $r->STATUS;
		$this->year = $r->YEAR;
		$this->quarter = $r->QUARTER;
		$this->last_update = $r->LAST_UPDATE;
		$this->last_exported = $r->LAST_EXPORTED;
		return $this->complete = true;
	}

	function insert()
	{
		$this->last_update = date('Y-m-d H:i:s');
		$this->status = DATA_READY;
		parent::insert();
	}

	public function updateStatus($status,$last_exp='')
	{
		if(!isset($this->id))
			return;
		//don't let last_update change
		$la = $this->last_update;

		$this->status = $status;
		if($last_exp != '')
			$this->last_exported = $last_exp;
		parent::update();

		$this->last_update = $la;
		parent::update();

	}
	function getYears($crso)
	{
		$years = array();
		$date = currentDate();
		$y = $date->getYear();
		for($i=START_YEAR;$i<=$y;$i++)
			$years[] = $i;
		return $years;
	}
	function getQuarters($crso,$year)
	{
		$quarters = array();
		$date = currentDate();
		$y = $date->getYear();
		if($year == START_YEAR)
		{
			for($i=START_QUARTER;$i<=4;$i++)
				$quarters[] = $i;
			return $quarters;
		}
		if($year == $y)
		{
			$month = $date->getMonth();
			$quarter = ceil((int)$month / 3);
			for($i=1;$i<=$quarter;$i++)
				$quarters[] = $i;
			return $quarters;
		}
		else
			$quarters = array('1','2','3','4');

		return $quarters;

		$sql = " SELECT `QUARTER` FROM TBL_UK_CRSO_DATA WHERE CRSO_RELATION_ID=? and `YEAR`=? ORDER BY `QUARTER`";
		$q = $this->db->query($sql);
		$q->bind_values($crso,$year);
		$q->execute();
		return $q;
	}
	function _prettyQuarter($q='')
	{
		if($q == '')
			$q = (isset($this->quarter))?$this->quarter:false;
		switch($q)
		{
			case 1:
				return "Quarter 1: January - March";
				break;
			case 2:
				return "Quarter 2: April - June";
				break;
			case 3:
				return "Quarter 3: July - September";
				break;
			case 4:
				return "Quarter 4: October - December";
				break;
		}
	}

	function createCRSODataXML($file, $preview=false, $per_year=false, $returnXml=false, $xsd_version)
	{
		info('createCRSODataXML crso_id: '.$this->crso_id);

		$crso_rel = new CRSO($this->db);
		$crso_rel->set($this->crso_id);

		$dom = new DOMDocument('1.0', 'UTF-8');
		$dom->formatOutput = true;
		if($preview)
		{
			$frag = $dom->createDocumentFragment();
			$css = "/stylesheets/elc/actuals-rep-style.css";
			$frag->appendXML('<?xml-stylesheet href="'.$css.'" type="text/css"?>');
			$dom->appendChild($frag);
		}

		$root = $dom->appendChild($dom->createElement('weee:SchemeReturn'));
		$root->setAttribute('xmlns:weee','http://www.environment-agency.gov.uk/WEEE/XMLSchema/SchemeReturns');
		$root->setAttribute('xmlns:xsi','http://www.w3.org/2001/XMLSchema-instance');
		$root->setAttribute('xsi:schemaLocation','http://www.environment-agency.gov.uk/WEEE/XMLSchema/SchemeReturns EA_SCHEME_RETURNS.xsd');

		if ($xsd_version == 2)
			$elem = $root->appendChild($dom->createElement('weee:XSDVersion','2.0'));
		else if ($xsd_version == 3)
			$elem = $root->appendChild($dom->createElement('weee:XSDVersion','3.35'));

		$elem = $root->appendChild($dom->createElement('weee:ApprovalNo',$crso_rel->reg_no));

		if($preview)
		{
			$rel = new Relation($this->db);
			$rel->set($this->crso_id);
			$elem = $root->appendChild($dom->createElement('weee:Scheme',htmlentities($rel->name)));
		}

		$elem = $root->appendChild($dom->createElement('weee:ComplianceYear',$this->year));
		if($preview == true && $per_year)
			$elem = $root->appendChild($dom->createElement('weee:ReturnPeriod',"Yearly report"));
		else
			$elem = $root->appendChild($dom->createElement('weee:ReturnPeriod',$this->_prettyQuarter()));

		$s = new UKCRSOSupplier($this->db);
		$q = $s->selectOverview($this->crso_id);
		$suppliers = array();
		while($r = $q->fetch())
			$suppliers[$r->SUPPLIER_ID] = array('approval_no'=>$r->APPROVAL_NO, 'name'=>$r->NAME);

		$dl = new UKCRSODataLine($this->db);
		if($per_year == false)
			$q = $dl->selectOverview($this->id);
		else {
			// report for year 2010 should use only recycle data set for Q4
			// because the transition from v1.3 to v2.0 was made starting from 2011
			// so CRSO was asked to supply recycle data in new format for all 2010 as Q4 data
			if($this->year == 2010){
				// for yearly reports, default $quarter is set to 4
				$q = $dl->selectOverview($this->id);
			}
			else
				$q = $dl->selectYearlyOverview($this->crso_id, $this->year);
		}
		$tonnes = array();
		while($r = $q->fetch())
		{
			$sid = 'ALL';
			if($r->SUPPLIER_ID)
				$sid = 	$r->SUPPLIER_ID;
			$tonnes[$r->TYPE][$sid][$r->CATEGORY_NAME][$r->OBLIGATION_TYPE] = $r->TONNES;
		}

		foreach($tonnes as $type=>$ss){

			if ($xsd_version == 3)
				$type_node = $root->appendChild($dom->createElement('weee:'.$dl->translateType($type, $xsd_version)));

			foreach($ss as $supplier=>$key){

				if ($xsd_version == 2)
					$type_node = $root->appendChild($dom->createElement('weee:'.$dl->translateType($type, $xsd_version)));

				if(strcmp($supplier,'ALL') != 0){
					if(!array_key_exists($supplier, $suppliers)) {
						continue;
					}
					if($type == ATF){
						$d_elem = $type_node->appendChild($dom->createElement('weee:DeliveredToFacility'));
						$ap = $d_elem->appendChild($dom->createElement('weee:AATFApprovalNo',$suppliers[$supplier]['approval_no']));
						$nm = $d_elem->appendChild($dom->createElement('weee:FacilityName', htmlspecialchars($suppliers[$supplier]['name'])));
					}
					if($type == AE){
						$d_elem = $type_node->appendChild($dom->createElement('weee:DeliveredToOperator'));
						$ap = $d_elem->appendChild($dom->createElement('weee:AEApprovalNo',$suppliers[$supplier]['approval_no']));
						$nm = $d_elem->appendChild($dom->createElement('weee:OperatorName', htmlspecialchars($suppliers[$supplier]['name'])));
					}

				}

				$returns_node = $type_node;

				if ($xsd_version == 3 && ($type == ATF || $type == AE ))
					$returns_node = $d_elem->appendChild($dom->createElement('weee:Returns'));

				foreach($key as $categ=>$key1)
					foreach($key1 as $oblig=>$tone)
					{
						$elem = $returns_node->appendChild($dom->createElement('weee:Return'));
						$categ_node = $elem->appendChild($dom->createElement('weee:CategoryName',$dl->translateCategoryName($categ, $xsd_version)));
						$oblig_node = $elem->appendChild($dom->createElement('weee:ObligationType',$dl->translateObligationType($oblig)));
						$tonnes = $elem->appendChild($dom->createElement('weee:TonnesReturnValue',$tone));
					}
			}
		}
		$elem = $root->appendChild($dom->createElement('weee:ProducerList'));
		if($returnXml) return $dom;
		$dom->save($file);
	}

	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_UK_CRSO_DATA';
		$this->objectmap['primary']['id'] = 'DATA_ID';
		$this->objectmap['properties']['crso_id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['year'] = 'YEAR';
		$this->objectmap['properties']['quarter'] = 'QUARTER';
		$this->objectmap['properties']['status'] = 'STATUS';
		$this->objectmap['properties']['last_update'] = 'LAST_UPDATE';
		$this->objectmap['properties']['last_exported'] = 'LAST_EXPORTED';
	}
}
?>
