<?php
/**
 * Created on Jan 8, 2008
 *
 * @author Nicoleta Militaru <nicoleta@basesoft.com>
 **/

require_once(ELC_CLASSES.'uk/UKCRSOData.php');

define('UK_EA_CATEGORY_LEN', 14);
define('UK_EA_OBLIG_LEN', 2);

//type
define('DCF',1);
define('ATF',2);
define('AE',3);
define('REUSE',4); // not used since schema v2.0
// used only since schema v2.0
define('REG32',5);
define('REG39',6);

class UKCRSODataLine extends AbstractObject
{
	var $id;
	var $data_id;
	var $type;
	var $supplier_id;
	var $category_name;
	var $obligation_type;
	var $last_update;
	var $tonnes;

	function set($dl_id='')
	{
		$dl_id || $dl_id = $this->id;

		$sql = " SELECT DATA_ID,DATA_LINE_ID,`TYPE`,SUPPLIER_ID,CATEGORY_NAME,OBLIGATION_TYPE,TONNES,
						LAST_UPDATE
				FROM TBL_UK_CRSO_DATA_LINE
				WHERE DATA_LINE_ID = ? ";
		$q = $this->db->query($sql);
		$q->bind_values($dl_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		$this->id = $r->DATA_LINE_ID;
		$this->data_id = $r->DATA_ID;
		$this->type = $r->TYPE;
		$this->supplier_id = $r->SUPPLIER_ID;
		$this->category_name = $r->CATEGORY_NAME;
		$this->obligation_type = $r->OBLIGATION_TYPE;
		$this->last_update = $r->LAST_UPDATE;
		$this->tonnes = $r->TONNES;
		return $this->complete = true;
	}

	function selectOverview($data_id)
	{
		$sql = "SELECT `TYPE`,SUPPLIER_ID, CATEGORY_NAME,OBLIGATION_TYPE,TONNES
					FROM TBL_UK_CRSO_DATA_LINE
				WHERE DATA_ID = ? ORDER BY `TYPE`";
		$q = $this->db->query($sql);
		$q->bind_values($data_id);
		$q->execute();
		return $q;
	}

	function selectYearlyOverview($crso_id, $year)
	{
		$sql = "SELECT `TYPE`,SUPPLIER_ID, CATEGORY_NAME,OBLIGATION_TYPE,SUM(TONNES) AS TONNES
				FROM TBL_UK_CRSO_DATA_LINE DL
				INNER JOIN TBL_UK_CRSO_DATA D ON DL.DATA_ID=D.DATA_ID
				WHERE D.CRSO_RELATION_ID=? AND D.YEAR=?
				GROUP BY TYPE, SUPPLIER_ID, CATEGORY_NAME, OBLIGATION_TYPE";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id, $year);
		$q->execute();
		return $q;
	}

	function insertLines($data_id,$tonnes)
	{
		$DCF = DCF;
		$ATF = ATF;
		$AE = AE;
		$REUSE = REUSE;

		foreach ($tonnes as $type => $ss) {
			foreach ($ss as $supplier => $key) {
				foreach ($key as $categ => $key1) {
					foreach ($key1 as $oblig => $tone) {
						$s = $supplier;
						if (strcmp($supplier, 'ALL') == 0)
							$s = null;
						$sql = "INSERT INTO TBL_UK_CRSO_DATA_LINE
								(DATA_ID,`TYPE`, SUPPLIER_ID, CATEGORY_NAME,OBLIGATION_TYPE,TONNES)
								VALUES (?,?,?,?,?,?)";
						$q = $this->db->query($sql);
						$q->bind_values($data_id, $type, $s, $categ, $oblig, $tone);
						$q->execute();
					}

				}
			}
		}
	}
	function updateLines($data_id,$tonnes)
	{

		foreach($tonnes as $type=>$ss)
			foreach($ss as $supplier=>$key){
				$s = $supplier;
				if(strcmp($supplier,'ALL') == 0)
					$s = null;
				// is this a new supplier?
				if($s != null){
					$sql = "SELECT DATA_LINE_ID
								FROM TBL_UK_CRSO_DATA_LINE
							WHERE DATA_ID = ? AND `TYPE`=? AND SUPPLIER_ID=?";
					$q = $this->db->query($sql);
					$q->bind_values($data_id, $type, $s);
					$q->execute();
					$r = $q->fetch();
					if(!$r){
						$ins = array();
						$ins[$type][$supplier] = $tonnes[$type][$supplier];
						$this->insertLines($data_id,$ins);
						continue;
					}
				}
				foreach($key as $categ=>$key1)
					foreach($key1 as $oblig=>$tone)
					{
                        if($s == null){
                            $sql = "UPDATE TBL_UK_CRSO_DATA_LINE SET TONNES=?
                                        WHERE DATA_ID=? AND `TYPE`=? AND SUPPLIER_ID IS NULL
                                        AND CATEGORY_NAME=? AND OBLIGATION_TYPE=?
                                    ";
                            $q = $this->db->query($sql);
                            $p = $q->bind_values($tone,$data_id,$type,$categ,$oblig);
                        }
                        else {
                            $sql = "UPDATE TBL_UK_CRSO_DATA_LINE SET TONNES=?
                                        WHERE DATA_ID=? AND `TYPE`=? AND SUPPLIER_ID=?
                                        AND CATEGORY_NAME=? AND OBLIGATION_TYPE=?
                                    ";
                            $q = $this->db->query($sql);
                            $p = $q->bind_values($tone,$data_id,$type,$s,$categ,$oblig);
                        }
                        $q->execute();
					}
			}
		//check if anything was actualy changed
		$sql = "SELECT MAX(LAST_UPDATE) as LA FROM TBL_UK_CRSO_DATA_LINE WHERE DATA_ID=? GROUP BY DATA_ID";
		$q = $this->db->query($sql);
		$q->bind_values($data_id);
		$q->execute();
		$r = $q->fetch();
		$la = $r->LA;

		$data = new UKCRSOData($this->db);
		$data->setID($data_id);
		if($la > $data->last_update)
		{
			$data->last_update = $la;
			$data->update();
		}
	}
	public function translateType($type, $xsd_version)
	{
		switch($type)
		{
			case DCF:
				return 'CollectedFromDCF';
			case ATF:
				return $xsd_version == 2 ? 'DeliveredToATF' : 'DeliveredToAATF';
			case AE:
				return 'DeliveredToAE';
			case REUSE:
				return 'ForReuse';
			case REG32:
				return $xsd_version == 2 ? 'WEEEReturnedReg32' : 'B2CWEEEFromDistributors';
			case REG39:
				return $xsd_version == 2 ? 'WEEETakenBackReg39' : 'B2CWEEEFromFinalHolders';
		}
	}
	public function translateCategoryName($categ, $xsd_version=3)
	{
		switch($categ)
		{
			case 1:
				return 'Lighting Equipment';
			case 2:
				return 'Small Household Appliances';
			case 3:
				return $xsd_version == 2 ? 'Gas Discharge Lamps' : 'Gas Discharge Lamps and LED light sources';
			case 4:
				return 'Large Household Appliances';
			case 5:
				return 'IT and Telecomms Equipment';
			case 6:
				return 'Consumer Equipment';
			case 7:
				return 'Electrical and Electronic Tools';
			case 8:
				return 'Toys Leisure and Sports';
			case 9:
				return 'Medical Devices';
			case 10:
				return 'Monitoring and Control Instruments';
			case 11:
				return 'Automatic Dispensers';
			case 12:
				return 'Display Equipment';
			case 13:
				return 'Cooling Appliances Containing Refrigerants';
			case 14:
				return 'Photovoltaic Panels';
		}

		throw new Exception('no such category: '.$categ);

	}
	public function translateObligationType($oblig)
	{
		switch($oblig)
		{
			case 1:
				return 'B2C';
			case 2:
				return 'B2B';
		}
	}

	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_UK_DATA_LINE';
		$this->objectmap['primary']['id'] = 'DATA_LINE_ID';
		$this->objectmap['properties']['data_id'] = 'DATA_ID';
		$this->objectmap['properties']['type'] = 'TYPE';
		$this->objectmap['properties']['category_name'] = 'CATEGORY_NAME';
		$this->objectmap['properties']['obligation_type'] = 'OBLIGATION_TYPE';
		$this->objectmap['properties']['last_update'] = 'LAST_UPDATE';
		$this->objectmap['properties']['tonnes'] = 'TONNES';
	}
}
?>
