<?php
/**
 * Created on Dec 13, 2007
 *
 * @author Nicoleta Militaru <nicoleta@basesoft.com>
 **/

require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'uk/UKRegistrationData.php');

class UKPartnerName extends AbstractObject
{
	var $id;
	var $crso;
	var $participant;
	var $partner_name;

	function set($partner_id='')
	{
		$partner_id || $partner_id = $this->id;

		$sql = " SELECT CRSO_RELATION_ID,PARTICIPANT_RELATION_ID,PARTNER_ID,PARTNER_NAME
				FROM TBL_UK_PARTNER_NAME
				WHERE PARTNER_ID = ? ";
		$q = $this->db->query($sql);
		$q->bind_values($$partner_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		$this->id = $r->PARTNER_ID;
		$this->crso = new CRSO($this->db,array('relation'=> new Relation($this->db,$r->CRSO_RELATION_ID)));
		$this->participant = new Participant($this->db,array('relation'=> new Relation($this->db,$r->PARTICIPANT_RELATION_ID)));
		$this->partner_name = $r->PARTNER_NAME;
		return $this->complete = true;
	}

	public function getOverview($crso_id='',$participant_id='')
	{
		if($crso_id == '')
			$crso_id = (isset($this->crso->relation->id))?$this->crso->relation->id:false;
		if($participant_id == '')
			$participant_id = (isset($this->participant->relation->id))?$this->participant->relation->id:false;
		$sql = " SELECT CRSO_RELATION_ID,PARTICIPANT_RELATION_ID,PARTNER_ID,PARTNER_NAME
				FROM TBL_UK_PARTNER_NAME
				WHERE CRSO_RELATION_ID = ?
					AND PARTICIPANT_RELATION_ID = ?	";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$participant_id);
		$q->execute();
		return $q;
	}

	public function setID($partner)
	{
		$sql = "SELECT PARTNER_ID FROM TBL_UK_PARTNER_NAME WHERE CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ? AND PARTNER_NAME = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->crso->relation->id,$this->participant->relation->id,$partner);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		$this->id = $r->PARTNER_ID;
	}

	public function insert()
	{
		$a = parent::insert();
		$this->_notifyParent();
		return $a;
	}

	function update()
	{
		$sql = "UPDATE TBL_UK_PARTNER_NAME SET PARTNER_NAME = ? WHERE PARTNER_ID = ? ";
		$q = $this->db->query($sql);
		$q->bind_values($this->partner_name,$this->id);
		$q->execute();
		$count = $q->rowCount();
		if($count > 0)
			$this->_notifyParent();
	}

	function deletePartners($partners = array())
	{
		if(count($partners) == 0)
			return;
		$sql = "DELETE FROM TBL_UK_PARTNER_NAME WHERE PARTNER_NAME IN (" . str_repeat("?,",count($partners)-1) . "?)
				AND CRSO_RELATION_ID = ? AND PARTICIPANT_RELATION_ID = ?";
		$q = $this->db->query($sql);
		$partners[] = $this->crso->relation->id;
		$partners[] = $this->participant->relation->id;
		$q->bind_values($partners);
		$q->execute();
		$count = $q->rowCount();
		if($count > 0)
			$this->_notifyParent();
	}

	function _notifyParent()
	{
		$reg = new UKRegistrationData($this->db);
		$reg->set($this->crso->relation->id,$this->participant->relation->id);
		$date = date('Y-m-d H:i:s');
		$reg->childUpdate($date);
	}

	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_UK_PARTNER_NAME';
		//$this->objectmap['primary']['id'] = 'PARTNER_ID';
		$this->objectmap['properties']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
		$this->objectmap['properties']['partner_name'] = 'PARTNER_NAME';
	}
}
?>
