<?php
/**
 * Created on Jan 2, 2008
 *
 * @author Nicoleta Militaru <nicoleta@basesoft.com>
 **/

require_once(ELC_CLASSES.'CRSOParticipant.php');
require_once(ELC_CLASSES.'CRSOProduct.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'Relation.php');
require_once(IWARE.'address.php');
require_once(IWARE.'InfoPerson.php');
require_once(ELC_CLASSES.'uk/UKRegistrationData.php');
require_once(ELC_CLASSES.'uk/UKSicCode.php');
require_once(ELC_CLASSES.'uk/UKPartnerName.php');
require_once(ELC_CLASSES.'uk/UKCRSOData.php');
require_once(ELC_CLASSES.'uk/UKReportData.php');
require_once(ELC_CLASSES.'uk/uk_define.php');

//for the status of the report
define('DATA_EXPORTED',1);
define('ACCEPTED',2);
define('REJECTED',3);

//report type
define('REG_REPORT',1);
define('ACTUALS_REPORT',2);

class UKReport extends AbstractObject
{

	var $id;
	var $report_type;
	var $file_name;
	var $export_date;
	var $status;
	var $crso_id;
	var $reason;
	var $year;
	var $quarter;

	/**
	 * Returns a dict with active EA products per year used in the 'returns' export.
	 * dict key is TBL_PRODUCT.id
	 * @param $year
	 * @return array
	 */
	function get_product_map($year) {

		if ($year < 2016) {
			return array(
				1 => "Gas Discharge Lamps",
				2 => "Lighting Equipment",
				3 => "Small Household Appliances",
			);
		}

		// notice: product 4 (LED) isnt used in EA
		return array(
			1 => "Gas Discharge Lamps and LED light sources",
			2 => "Lighting Equipment",
			3 => "Small Household Appliances",
			5 => "Toys Leisure and Sports",
			6 => "Monitoring and Control Instruments",
			7 => "IT and Telecomms Equipment",
			8 => "Large Household Appliances"
		);


	}

	function set($rep_id = '')
	{
		$rep_id || $rep_id = $this->id;

		$sql = " SELECT REPORT_ID,REPORT_TYPE,FILE_NAME,EXPORT_DATE,`STATUS`,`YEAR`,`QUARTER`,
					CRSO_RELATION_ID,REJECT_REASON
				 FROM TBL_UK_REPORTS
				 WHERE REPORT_ID = ? ";
		$q = $this->db->query($sql);
		$q->bind_values($rep_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		$this->id = $r->REPORT_ID;
		$this->report_type = $r->REPORT_TYPE;
		$this->file_name = $r->FILE_NAME;
		$this->export_date = $r->EXPORT_DATE;
		$this->status = $r->STATUS;
		$this->year = $r->YEAR;
		$this->quarter = $r->QUARTER;
		$this->crso_id = $r->CRSO_RELATION_ID;
		$this->reason = $r->REJECT_REASON;
		return $this->complete = true;
	}
	static public function translateReportType($langIf,$type)
	{
		switch($type)
		{
			case REG_REPORT:
				return array($langIf->getText('Producer Reg Scheme'), 'font-weight:bold;');
			case ACTUALS_REPORT:
				return array($langIf->getText('Weee Actuals Scheme'), 'font-weight:bold;color:#7E7E7E;');
		}
	}
	static public function translateStatus($langIf,$status)
	{
		switch($status)
		{
			case DATA_EXPORTED:
				return array($langIf->getText('Exported'), 'status_declared');
			case ACCEPTED:
				return array($langIf->getText('Accepted'), 'status_verified');
			case REJECTED:
				return array($langIf->getText('Rejected'), 'status_denied');
		}
	}
	public function hasUnconfirmedReport($crso,$type)
	{
		$sql = " SELECT COUNT(REPORT_ID) AS REPNO
				 FROM TBL_UK_REPORTS
				 WHERE CRSO_RELATION_ID = ? AND REPORT_TYPE = ? AND `STATUS` = 1";
		$q = $this->db->query($sql);
		$q->bind_values($crso,$type);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		if($r->REPNO > 0)
			return false;
		return true;
	}

	public function exportPR($crso,$year,$preview = true)
	{

		$xsd_version = 2;
		if ($year >= 2016)
			$xsd_version = 3;

		$d = date('Y-m-d H:i:s');

		$crso_rel = new CRSO($this->db);
		$crso_rel->set($crso);

		if(!$preview)
		{
			$temp = explode(" ",$d);
			$file_date = str_replace("-","",$temp[0]);
			$folder = UK_EXPORT_PATH.UK_EXPORT_FILES;
			$file_name = str_replace(array("/"," "),array("",""),$crso_rel->reg_no).$file_date;

			if ($xsd_version == 3)
				$file_name .= 'cp'.$year;

			$images_folder = 'images';

			$nd = str_replace(array(" ","-",":"),array("","",""),$d);
			$folder = $folder.$nd."/";
			if(!is_dir($folder))
				mkdir($folder,0777);

			$file_archive = $folder.$file_name.'.ZIP';
			$file_xml = $folder.$file_name.'.XML';

			//we have to create a zip file that contains the producers logos also
			$zip = new ZipArchive();
			if($zip->open($file_archive, ZipArchive::CREATE) !== TRUE) {
				throw new Exception('failed to create zip: '.$file_archive);
			}

			if(!$zip->addEmptyDir($images_folder)) {
				throw new Exception('Directory already exists');
			}

			//insert the report
			$this->file_name = $nd.'/'.$file_name.'.ZIP';
			$this->export_date = $d;
			$this->report_type = REG_REPORT;
			$this->crso_id = $crso;
			$this->status = DATA_EXPORTED;
			$this->year = $year;

			$this->insert();
		}
		$reg = new UKRegistrationData($this->db);
		$sic = new UKSicCode($this->db);
		$partner = new UKPartnerName($this->db);

		//CRSO DATA ------------------------------------------------------------------------------
		$sql = "SELECT U.INFO_PERSON_ID, R.ADDRESS_ID, R.NAME
				FROM TBL_RELATION R
				INNER JOIN TBL_RELATION_USER RU
					ON R.RELATION_ID = RU.RELATION_ID
				INNER JOIN TBL_USER U
					ON U.USER_ID = RU.USER_ID
				WHERE R.RELATION_ID = ?
				AND RU.IS_ADMIN = 1";

		$q = $this->db->query($sql);
		$q->bind_values($crso);
		$q->execute();
		if(!($r = $q->fetch())) {
			throw new Exception('query failed');
		}
		$address_id = $r->ADDRESS_ID;
		$info_person_id = $r->INFO_PERSON_ID;
		$crso_name = $r->NAME;

		$addr = new Address($this->db);
		$addr->set($address_id);

		$ip = new InfoPerson($this->db);
		$ip->set($info_person_id);

		$dom = new DOMDocument('1.0', 'UTF-8');
		//$dom->formatOutput = true;

		if($preview)
		{
			$frag = $dom->createDocumentFragment();
			$css = "/stylesheets/elc/producer-reg-style.css";
			$frag->appendXML('<?xml-stylesheet href="'.$css.'" type="text/css"?>');
			$dom->appendChild($frag);
		}

		$root = $dom->appendChild($dom->createElement('weee:scheme'));
		$root->setAttribute('xmlns:weee','http://www.environment-agency.gov.uk/WEEE/XMLSchema');
		$root->setAttribute('xmlns:xsi','http://www.w3.org/2001/XMLSchema-instance');
		$root->setAttribute('xsi:schemaLocation','http://www.environment-agency.gov.uk/WEEE/XMLSchema EA_PROD_REG.xsd');

		if ($xsd_version == 2)
			$elem = $root->appendChild($dom->createElement('weee:XSDVersion','2.2'));
		else if ($xsd_version == 3)
			$elem = $root->appendChild($dom->createElement('weee:XSDVersion','3.06'));

		$elem = $root->appendChild($dom->createElement('weee:approvalNo',$crso_rel->reg_no));
		$elem = $root->appendChild($dom->createElement('weee:complianceYear',$year));
		$elem = $root->appendChild($this->makeDomNode($dom,'weee:tradingName',$crso_name));

		$sb = $root->appendChild($dom->createElement('weee:schemeBusiness'));

		$comp = $sb->appendChild($dom->createElement('weee:company'));
			$cd = $comp->appendChild($this->makeDomNode($dom,'weee:companyName',$crso_name));
			$cno = $comp->appendChild($this->makeDomNode($dom,'weee:companyNumber',$crso_rel->misc_1));

		//END OF CRSO DATA -----------------------------------------------------------------------------

		//get the settings for the participants that will not be reported
		if(!$preview)
		{
			$q = $reg->getDataSetNotReportedPr($crso);
			while($r = $q->fetch())
			{
				$part[] = $r->PARTICIPANT_RELATION_ID;
				//add the settings for this participant to report_data
				$rd = new UKReportData($this->db);
				$rd->report_id = $this->id;
				$rd->crso_id = $crso;
				$rd->participant_id = $r->PARTICIPANT_RELATION_ID;
				$rd->pr_status = $r->PR_STATUS;
				$rd->weee_status = $r->WEEE_STATUS;
				$rd->vat_registered = $r->VAT_REGISTERED;
				$rd->insert();
			}
		}

		$producerList = $root->appendChild($dom->createElement('weee:producerList'));

		//PRODUCERS that will be reported
		$q = $reg->getDataSetPR($crso);
		while($r = $q->fetch())
		{
			$reg->set($crso,$r->PARTICIPANT_RELATION_ID);
			if(!$preview)
			{
				//add the settings for this participant to report_data
				$rd = new UKReportData($this->db);
				$rd->report_id = $this->id;
				$rd->crso_id = $crso;
				$rd->participant_id = $r->PARTICIPANT_RELATION_ID;
				$rd->pr_status = $reg->pr_status;
				$rd->weee_status = $reg->weee_status;
				$rd->vat_registered = $reg->vat_registered;
				$rd->insert();
			}

			$producer = $producerList->appendChild($dom->createElement('weee:producer'));
			if($r->PR_STATUS == REG_NEW)
			{
				$elem = $producer->appendChild($dom->createElement('weee:status','I'));
				$elem = $producer->appendChild($dom->createElement('weee:registrationNo',''));
			}
			else
			{
				$elem = $producer->appendChild($dom->createElement('weee:status','A'));
				$elem = $producer->appendChild($dom->createElement('weee:registrationNo',$r->PRN));
			}
			$elem = $producer->appendChild($this->makeDomNode($dom,'weee:tradingName',$r->TRADING_NAME));
			$elem = $producer->appendChild($dom->createElement('weee:SICCodeList'));

			$qq = $sic->getOverview($crso,$r->PARTICIPANT_RELATION_ID);
			$x = 0;
			while($rr = $qq->fetch())
			{
				$s = $elem->appendChild($this->makeDomNode($dom,'weee:SICCode',$rr->SIC_CODE));
				$x += 1;
			}
			if($x == 0)
				$s = $elem->appendChild($dom->createElement('weee:SICCode'));

			$elem = $producer->appendChild($dom->createElement('weee:VATRegistered',$r->VAT_REGISTERED));
			$elem = $producer->appendChild($dom->createElement('weee:annualTurnover',$r->ANNUAL_TURNOVER));
			if($r->ANNUAL_TURNOVER <= 1000000)
				$elem = $producer->appendChild($dom->createElement('weee:annualTurnoverBand','Less than or equal to one million pounds'));
			else
				$elem = $producer->appendChild($dom->createElement('weee:annualTurnoverBand','Greater than one million pounds'));

			if ($xsd_version == 3) {

				switch ($r->PLACED_ON_MARKET) {
					case PLACED_ON_MARKET_LARGE:
						$xml_value = 'More than or equal to 5T EEE placed on market';
						break;
					case PLACED_ON_MARKET_SMALL:
						$xml_value = 'Less than 5T EEE placed on market';
						break;
					default:
						header('Location: /scripts/elc/public/error.php?msg=' . urlencode('Invalid placed on market for producer: '.$r->TRADING_NAME));
						exit(0);


				}

				$elem = $producer->appendChild($dom->createElement('weee:eeePlacedOnMarketBand', $xml_value));
			}

			$obligation_type = 'Both';
			if($r->OBLIGATION_TYPE == OBLIGATION_TYPE_B2B) {
				$obligation_type = 'B2B';
			} elseif($r->OBLIGATION_TYPE == OBLIGATION_TYPE_B2C) {
				$obligation_type = 'B2C';
			}

			$elem = $producer->appendChild($dom->createElement('weee:obligationType',$obligation_type));

			if ($xsd_version == 2) {
				$elem = $producer->appendChild($dom->createElement('weee:producerIdentificationMark'));

				if (!$preview) {
					$new_logo = $images_folder . '/' . $r->PARTICIPANT_RELATION_ID . '_graphic' . substr($r->LOGO, strrpos($r->LOGO, '.'));
					$gr = $elem->appendChild($this->makeDomNode($dom, 'weee:graphic', $new_logo));
					info('adding logo' . UK_EXPORT_PATH . UK_EXPORT_LOGO . $r->PARTICIPANT_RELATION_ID . '/' . $r->LOGO);

					$logo_file = UK_EXPORT_PATH . UK_EXPORT_LOGO . $r->PARTICIPANT_RELATION_ID . '/' . $r->LOGO;
					if (file_exists($logo_file))
						$zip->addFile($logo_file, $new_logo);
					else
						info('WARNING: logo file does not exist: ' . $logo_file);
				} else
					$gr = $elem->appendChild($this->makeDomNode($dom, 'weee:graphic', $r->LOGO));
			}

			if ($xsd_version == 3) {

				// brands
				$brandList = $producer->appendChild($dom->createElement('weee:producerBrandNames'));

				$pb = new ParticipantBrand($this->db);
				$brands = $pb->getAllParticipantBrands($r->PARTICIPANT_RELATION_ID, $crso, true);

				foreach($brands as $id => $name) {
					$brand = $brandList->appendChild(
						$this->makeDomNode($dom, 'weee:brandname', $name)
					);
				}
			}

			$prodB = $producer->appendChild($dom->createElement('weee:producerBusiness'));
				$cfn = $prodB->appendChild($dom->createElement('weee:correspondentForNotices'));
				$cd = $cfn->appendChild($dom->createElement('weee:contactDetails'));
				$s = $cd->appendChild($this->makeDomNode($dom,'weee:title',$r->CFN_TITLE));
				$s = $cd->appendChild($this->makeDomNode($dom,'weee:forename',$r->CFN_FORENAME));
				$s = $cd->appendChild($this->makeDomNode($dom,'weee:surname',$r->CFN_SURNAME));
				$s = $cd->appendChild($this->makeDomNode($dom,'weee:phoneLandLine',$r->CFN_PHONE));
				$s = $cd->appendChild($this->makeDomNode($dom,'weee:phoneMobile',$r->CFN_MOBILE));
				$s = $cd->appendChild($this->makeDomNode($dom,'weee:fax',$r->CFN_FAX));
				$s = $cd->appendChild($this->makeDomNode($dom,'weee:email',$r->CFN_EMAIL));
				$cda = $cd->appendChild($dom->createElement('weee:address'));
					$a = $cda->appendChild($this->makeDomNode($dom,'weee:primaryName',$r->CFN_BUILDING_NO));
					$a = $cda->appendChild($this->makeDomNode($dom,'weee:secondaryName',$r->CFN_BUILDING));
					$a = $cda->appendChild($this->makeDomNode($dom,'weee:streetName',$r->CFN_STREET));
					$a = $cda->appendChild($this->makeDomNode($dom,'weee:town',$r->CFN_CITY));

					if ($xsd_version == 2)
						$a = $cda->appendChild($this->makeDomNode($dom,'weee:postTown',$r->CFN_COUNTRY));

					$a = $cda->appendChild($this->makeDomNode($dom,'weee:locality',$r->CFN_LOCALITY));
					$a = $cda->appendChild($this->makeDomNode($dom,'weee:administrativeArea',$r->CFN_ADMINISTRATIVE_AREA));

					if ($xsd_version == 3)
						$a = $cda->appendChild($this->makeDomNode($dom,'weee:country',$r->CFN_COUNTRY));

					if (substr($r->CFN_COUNTRY, 0, 5) == 'UK - ')
						$a = $cda->appendChild($this->makeDomNode($dom,'weee:postCode',$r->CFN_POSTAL_CODE));
					else
						$a = $cda->appendChild($this->makeDomNode($dom,'weee:internationalPostCode',$r->CFN_POSTAL_CODE));


			if($r->BUSINESS_TYPE == BUSINESS_COMPANY)
			{
				$elem = $prodB->appendChild($dom->createElement('weee:company'));
				$cn = $elem->appendChild($this->makeDomNode($dom,'weee:companyName',$r->BUSINESS_NAME));
				$cno = $elem->appendChild($this->makeDomNode($dom,'weee:companyNumber',$r->BUSINESS_NUMBER));
				$el = $elem->appendChild($dom->createElement('weee:registeredOffice'));
			}
			else
			{
				$elem = $prodB->appendChild($dom->createElement('weee:partnership'));
				$pn = $elem->appendChild($this->makeDomNode($dom,'weee:partnershipName',$r->BUSINESS_NAME));
				$pl = $elem->appendChild($dom->createElement('weee:partnershipList'));

				$qq = $partner->getOverview($crso,$r->PARTICIPANT_RELATION_ID);
				while($rr = $qq->fetch())
					$s = $pl->appendChild($this->makeDomNode($dom,'weee:partner',$rr->PARTNER_NAME));

				$el = $elem->appendChild($dom->createElement('weee:principalPlaceOfBusiness'));
			}
			$cd = $el->appendChild($dom->createElement('weee:contactDetails'));
			$s = $cd->appendChild($this->makeDomNode($dom,'weee:title',$r->BUSINESS_PERSON_TITLE));
			$s = $cd->appendChild($this->makeDomNode($dom,'weee:forename',$r->BUSINESS_PERSON_FORENAME));
			$s = $cd->appendChild($this->makeDomNode($dom,'weee:surname',$r->BUSINESS_PERSON_SURNAME));
			$s = $cd->appendChild($this->makeDomNode($dom,'weee:phoneLandLine',$r->BUSINESS_PHONE));
			$s = $cd->appendChild($this->makeDomNode($dom,'weee:phoneMobile',$r->BUSINESS_MOBILE));
			$s = $cd->appendChild($this->makeDomNode($dom,'weee:fax',$r->BUSINESS_FAX));
			$s = $cd->appendChild($this->makeDomNode($dom,'weee:email',$r->BUSINESS_EMAIL));
			$cda = $cd->appendChild($dom->createElement('weee:address'));
				$a = $cda->appendChild($this->makeDomNode($dom,'weee:primaryName',$r->BUSINESS_BUILDING_NO));
				$a = $cda->appendChild($this->makeDomNode($dom,'weee:secondaryName',$r->BUSINESS_BUILDING));
				$a = $cda->appendChild($this->makeDomNode($dom,'weee:streetName',$r->BUSINESS_STREET));
				$a = $cda->appendChild($this->makeDomNode($dom,'weee:town',$r->BUSINESS_CITY));

				if ($xsd_version == 2)
					$a = $cda->appendChild($this->makeDomNode($dom,'weee:postTown',$r->BUSINESS_COUNTRY));

				$a = $cda->appendChild($this->makeDomNode($dom,'weee:locality',$r->BUSINESS_LOCALITY));
				$a = $cda->appendChild($this->makeDomNode($dom,'weee:administrativeArea',$r->BUSINESS_ADMINISTRATIVE_AREA));

				if ($xsd_version == 3)
					$a = $cda->appendChild($this->makeDomNode($dom,'weee:country',$r->BUSINESS_COUNTRY));

				if (substr($r->BUSINESS_COUNTRY, 0, 5) == 'UK - ')
					$a = $cda->appendChild($this->makeDomNode($dom,'weee:postCode',$r->BUSINESS_POSTAL_CODE));
				else
					$a = $cda->appendChild($this->makeDomNode($dom,'weee:internationalPostCode',$r->BUSINESS_POSTAL_CODE));



			if ($xsd_version == 3) {

				// authorized representative
				$ar_el = $producer->appendChild($dom->createElement('weee:authorisedRepresentative'));

				if (trim($r->AR_NAME) != '') {

					$producer_el = $ar_el->appendChild($dom->createElement('weee:overseasProducer'));
					$producer_el->appendChild($dom->createElement('weee:overseasProducerName', $r->AR_NAME));

					$cd = $producer_el->appendChild($dom->createElement('weee:overseasContact'));

					$s = $cd->appendChild($this->makeDomNode($dom,'weee:title',$r->AR_TITLE));
					$s = $cd->appendChild($this->makeDomNode($dom,'weee:forename',$r->AR_FORENAME));
					$s = $cd->appendChild($this->makeDomNode($dom,'weee:surname',$r->AR_SURNAME));
					$s = $cd->appendChild($this->makeDomNode($dom,'weee:phoneLandLine',$r->AR_PHONE));
					$s = $cd->appendChild($this->makeDomNode($dom,'weee:phoneMobile',$r->AR_MOBILE));
					$s = $cd->appendChild($this->makeDomNode($dom,'weee:fax',$r->AR_FAX));
					$s = $cd->appendChild($this->makeDomNode($dom,'weee:email',$r->AR_EMAIL));
					$cda = $cd->appendChild($dom->createElement('weee:address'));
						$a = $cda->appendChild($this->makeDomNode($dom,'weee:primaryName',$r->AR_BUILDING_NO));
						$a = $cda->appendChild($this->makeDomNode($dom,'weee:secondaryName',$r->AR_BUILDING));
						$a = $cda->appendChild($this->makeDomNode($dom,'weee:streetName',$r->AR_STREET));
						$a = $cda->appendChild($this->makeDomNode($dom,'weee:town',$r->AR_CITY));
						$a = $cda->appendChild($this->makeDomNode($dom,'weee:locality',$r->AR_LOCALITY));
						$a = $cda->appendChild($this->makeDomNode($dom,'weee:administrativeArea',$r->AR_ADMINISTRATIVE_AREA));
						$a = $cda->appendChild($this->makeDomNode($dom,'weee:country',$r->AR_COUNTRY));

						if (substr($r->CFN_COUNTRY, 0, 5) == 'UK - ')
							$a = $cda->appendChild($this->makeDomNode($dom,'weee:postCode',$r->AR_POSTAL_CODE));
						else
							$a = $cda->appendChild($this->makeDomNode($dom,'weee:internationalPostCode',$r->AR_POSTAL_CODE));

				}

				// ceaseToExistDate
				if ($r->CEASE_TO_EXIST_DATE)
					$elem = $producer->appendChild($dom->createElement('weee:ceaseToExistDate', $r->CEASE_TO_EXIST_DATE));

				// sellingTechnique
				switch ($r->SELLING_TECHNIQUE) {
					case SELLING_TECHNIQUE_DIRECT:
						$xml_value = 'Direct Selling to End User';
						break;
					case SELLING_TECHNIQUE_INDIRECT:
						$xml_value = 'Indirect Selling to End User';
						break;
					case SELLING_TECHNIQUE_BOTH:
						$xml_value = 'Both';
						break;
					default:
						header('Location: /scripts/elc/public/error.php?msg=' . urlencode('Invalid selling technique for producer: '.$r->TRADING_NAME));
						exit(0);

				}
				$elem = $producer->appendChild($dom->createElement('weee:sellingTechnique', $xml_value));
			}

			if(!$preview)
			{
				$reg->updatePRStatusExported();
				$reg->setLastExportPR($d);
			}
		}

		$preview_file = UK_EXPORT_PATH.UK_EXPORT_TEMPLATES.'PREVIEW_PR.XML';

		if($preview)
			$dom->save($preview_file);
		else
			if (!$dom->save($file_xml)) {
				throw new Exception('failed to write to xml: '.$file_xml);
			}


		if(!$preview)
		{
			if (!file_exists($file_xml))
				throw new Exception('File not found: '.$file_xml);

			$zip->addFile($file_xml,basename($file_xml));
			$zip->close();
			unlink($file_xml);
			info('export Registration Report for CRSO relation '.$crso.', year: '.$year);
			return $file_archive;
		}
		else
			return $preview_file;

	}

	private function makeDomNode($dom, $label, $value)
	{
		$node = $dom->createTextNode($value);
		$t = $dom->createElement($label);
		$t->appendChild($node);
		return $t;
	}

	function getDataForActiveQuarters($crso, $producer, $year, $oblig='B2B'){

		$preview = $this->isGracePeriod($crso, $year);

		// check current status
		$sql = "SELECT UK.WEEE_STATUS
                FROM TBL_UK_REGISTRATION_DATA UK
                WHERE UK.CRSO_RELATION_ID = ?
                AND UK.PARTICIPANT_RELATION_ID = ?";
        $qr = $this->db->query($sql);
        $qr->bind_values($crso, $producer);
        $qr->execute();
        $current_status = $qr->fetch()->WEEE_STATUS;
        // we get all data for all quarters when producer was active or had no PRN (if current status is active)

        // if it is not currently active, check if it was active for at least one quarter and add data
        // for previous quarters when no PRN
        $check_no_prn = false;
        if($current_status == 2)
            $check_no_prn = true;

		$rep =  "
			SELECT REPORT_ID, WEEE_STATUS
			FROM TBL_UK_REPORT_DATA
			WHERE REPORT_ID = ( SELECT MAX(REPORT_ID) FROM  TBL_UK_REPORTS
								WHERE CRSO_RELATION_ID=? AND YEAR=? AND QUARTER=? AND STATUS=2)
			AND PARTICIPANT_RELATION_ID=?";

		$months = array();

		// check quarter 4
        $qr = $this->db->query($rep);
        $qr->bind_values($crso, $year, 4, $producer);
        $qr->execute();
        $rid = $qr->fetch();
        if($rid && $rid->REPORT_ID) {
            if(($rid->WEEE_STATUS == 2) || ($rid->WEEE_STATUS == 1 && $check_no_prn == true))
                $months = array_merge($months, array(10,11,12));
            if($rid->WEEE_STATUS == 2)
                $check_no_prn = true;
        }
        else {
            // for quarter 4, if no report was exported, get current status for producer
            if($current_status == 2)
                $months = array_merge($months, array(10,11,12));
        }

        // check quarter 3
        $qr = $this->db->query($rep);
        $qr->bind_values($crso, $year, 3, $producer);
        $qr->execute();
        $rid = $qr->fetch();
        if($rid && $rid->REPORT_ID) {
            if(($rid->WEEE_STATUS == 2) || ($rid->WEEE_STATUS == 1 && $check_no_prn == true))
                $months = array_merge($months, array(7,8,9));
            if($rid->WEEE_STATUS == 2)
                $check_no_prn = true;
        }

		// check quarter 2
        $qr = $this->db->query($rep);
        $qr->bind_values($crso, $year, 2, $producer);
        $qr->execute();
        $rid = $qr->fetch();
        if($rid && $rid->REPORT_ID) {
            if(($rid->WEEE_STATUS == 2) || ($rid->WEEE_STATUS == 1 && $check_no_prn == true))
                $months = array_merge($months, array(4,5,6));
            if($rid->WEEE_STATUS == 2)
                $check_no_prn = true;
        }

		// check quarter 1
		$qr = $this->db->query($rep);
		$qr->bind_values($crso, $year, 1, $producer);
		$qr->execute();
		$rid = $qr->fetch();
		if($rid && $rid->REPORT_ID) {
            if(($rid->WEEE_STATUS == 2) || ($rid->WEEE_STATUS == 1 && $check_no_prn == true))
                $months = array_merge($months, array(1,2,3));
        }

		// producer wasn't active in any of the months and also it is not currently active, so return null
		if(count($months) == 0)
			return null;
		if(strcmp($oblig, 'B2B') == 0)
			$n = 'Non-Household';
		else
			$n = 'Household';

		$declarations_nhh = 'SELECT D.PRODUCT_ID, P.NAME AS PRODUCT_NAME, CPD.NAME AS DIMENSION_NAME,
								DL.PRODUCT_CATEGORY_ID, R.RELATION_ID,
								R.NAME AS RELATION_NAME, CAST(SUM(IFNULL(DL.WEIGHT, 0)) / 1000 AS DECIMAL(10,3)) AS TONNAGE
							FROM TBL_DECLARATION D
							INNER JOIN TBL_DECLARATION_LINE DL ON D.DECLARATION_ID = DL.DECLARATION_ID
							INNER JOIN TBL_RELATION R ON D.PARTICIPANT_RELATION_ID = R.RELATION_ID
							INNER JOIN TBL_PRODUCT P ON D.PRODUCT_ID = P.PRODUCT_ID
							INNER JOIN TBL_CRSO_PRODUCT_DIMENSION CPD
								ON D.PRODUCT_ID = CPD.PRODUCT_ID AND D.CRSO_RELATION_ID = CPD.CRSO_RELATION_ID AND DL.CPD_ID = CPD.CPD_ID
							WHERE D.CRSO_RELATION_ID=?
							AND D.YEAR = ? AND D.MONTH IN (';
		foreach($months as $month)
			$declarations_nhh .= sprintf('%s,', $month);
		$declarations_nhh = substr($declarations_nhh, 0, strlen($declarations_nhh) - 1);
		$declarations_nhh .= ')';
		if(!$preview) {
			$declarations_nhh .= ' AND D.IS_EXPORTED > 0 ';
		}
		$declarations_nhh .= ' AND D.PARTICIPANT_RELATION_ID = ?
							AND CPD.NAME="'.$n.'"
							GROUP BY D.PRODUCT_ID, DL.CPD_ID, DL.PRODUCT_CATEGORY_ID';
        $q = $this->db->query($declarations_nhh);
		$q->bind_values($crso, $year, $producer);
		$q->execute();
		$declarationNhhRows = $q->fetchAll();

		$product_map = $this->get_product_map($year);

		foreach ($product_map as $key=>$label)
			$tonnes[$label] = 0;

		foreach($declarationNhhRows as $row)
			$tonnes[$product_map[$row->PRODUCT_ID]] += $row->TONNAGE;

		return $tonnes;
	}

	/**
	 * This will subtract the refunds credited with an export date within the quarter from the producer PoM.
	 *
	 * Rules are as follows:
	 *  - Export date as supplied by Refunder should be within the quarter.
	 *  - The refund should have been created within the year applicable.
	 *  - Only refunds which have actually been credited are used within the calculation.
	 */
	protected function _processNettingOff($crso_id, $producer_id, $year, $quarter, $eaData) {

		switch($quarter) {
			case 1:
				$months = array(3); // January and February are excluded (#2566)
				break;
			case 2:
				$months = array(4, 5, 6);
				break;
			case 3:
				$months = array(7, 8, 9);
				break;
			case 4:
				$months = array(10, 11, 12);
				break;
		}

		$months = implode(',', $months);

		$values = array(
			2,
			$year,
			$year,
			$crso_id,
			$producer_id
		);

		$sql = sprintf("
			SELECT r.PRODUCT_ID, CAST(SUM(IFNULL(rrl.WEIGHT, 0)) / 1000 AS DECIMAL(10,3)) AS TONNAGE
			FROM TBL_REFUND_REQUEST_LINE as rrl
			INNER JOIN TBL_REFUND_REQUEST as r
				ON rrl.REFUND_REQUEST_ID = r.REFUND_REQUEST_ID
			INNER JOIN TBL_REFUND_EXPORT as re
				ON r.REFUND_EXPORT_ID = re.REFUND_EXPORT_ID
			WHERE re.IS_EXPORTED = ?
				AND YEAR(r.REFUND_EXPORT_DATE) = ?
				AND MONTH(r.REFUND_EXPORT_DATE) IN (%s)
				AND YEAR(re.EXPORT_DATE) = ?
				AND r.CRSO_RELATION_ID = ?
				AND rrl.PRODUCER_RELATION_ID = ?
			GROUP BY r.PRODUCT_ID
		", $months);

		$q = $this->db->query($sql);
		$q->bind_values($values);
		$q->execute();
		$refundRows = $q->fetchAll();

		$product_map = $this->get_product_map($year);

		foreach($refundRows as $row) {
			$dim = 'B2C'; // Refunds are always subtracted from the B2C PoM.

			if (!isset($product_map[ $row->PRODUCT_ID ])) {
				info("_processNettingOff: product $row->PRODUCT_ID not active in this xsd version, skipping");
				continue;
			}

			$ea = $product_map[ $row->PRODUCT_ID ];

			// Check if theses keys exists -> if no waste was produced, no need to subtract refunds (as negative values are not allowed).
			if(!array_key_exists($ea, $eaData)) continue;
			if(!array_key_exists($dim, $eaData[$ea])) continue;

			$eaData[$ea][$dim] -= $row->TONNAGE;

			// We are not allowed to export negative values to the EA, if this results in a negative value, make it 0.
			if($eaData[$ea][$dim] < 0) {
				$eaData[$ea][$dim] = 0;
			}
		}
		return $eaData;
	}

	public function exportWeee($crso, $year, $quarter, $preview = true, $returnXml = false)
	{
		$xsd_version = 2;
		if ($year >= 2016)
			$xsd_version = 3;

		$per_year = false;
		if($quarter == null){
			$per_year = true;
			$quarter = 4;
		}
		$d = date('Y-m-d H:i:s');

		$crso_rel = new CRSO($this->db);
		$crso_rel->set($crso);

		// if not preview, then add an entry to the DB
		if(!$preview)
		{
			$temp = explode(" ",$d);
			$file_date = str_replace("-","",$temp[0]);
			$folder = UK_EXPORT_PATH.UK_EXPORT_FILES;
			$file_name = str_replace(array("/"," "),array("",""),$crso_rel->reg_no).'_'.$file_date;

			$nd = str_replace(array(" ","-",":"),array("","",""),$d);
			$folder = $folder.$nd."/";
			if(!is_dir($folder))
				mkdir($folder,0777);

			$file = $folder.$file_name.'_RETURN.XML';

			//add the report
			$this->file_name = $nd.'/'.$file_name.'_RETURN.XML';;
			$this->export_date = $d;
			$this->report_type = ACTUALS_REPORT;
			$this->crso_id = $crso;
			$this->status = DATA_EXPORTED;
			$this->year = $year;
			if($per_year == false)
				$this->quarter = $quarter;

			$this->insert();
		}

		$reg = new UKRegistrationData($this->db);
		$crso_data = new UKCRSOData($this->db);
		$crso_data->crso_id = $crso;

		$crso_file = UK_EXPORT_PATH.UK_EXPORT_TEMPLATES.$crso."_xml.xml";
		$crso_data->set($crso, $year, $quarter);
		$crso_data->createCRSODataXML($crso_file, $preview, $per_year, false, $xsd_version);

		//now we gotta fill in the templates for each producer and insert them into the returns xml doc
		//load the template file
		$template = new DOMDocument('1.0', 'UTF-8');
		$template->formatOutput = true;
		$template->load($crso_file);

		//create an xpath for getting our nodes
		$xpath = new DOMXPath($template);
		$xpath->registerNamespace('weee', 'http://www.environment-agency.gov.uk/WEEE/XMLSchema/SchemeReturns');

		//get the producer list
		$nodeList = $xpath->query('//weee:ProducerList');
		$producerListNode = $nodeList->item(0); //this list should only return one node
		$total_tonnes = 0;

		$qd = $reg->getDataSetWeee($crso, $year, $quarter, $per_year);
		while($r = $qd->fetch())
		{
			$obligation_type = $r->OBLIGATION_TYPE ? $r->OBLIGATION_TYPE : OBLIGATION_TYPE_BOTH;
			$report_b2b = ($obligation_type & OBLIGATION_TYPE_B2B) == OBLIGATION_TYPE_B2B;
			$report_b2c = ($obligation_type & OBLIGATION_TYPE_B2C) == OBLIGATION_TYPE_B2C;

			// dont export a producer that registered after the year this repot is for
			$x = getdate(strtotime($r->CREATED));
			$yy = $x['year'];
			if($yy > $year)
			 continue;

			$participant = $r->PARTICIPANT_RELATION_ID;
			$prn = $r->PRN;
			$business_name = $r->BUSINESS_NAME;

			// boolean to check if a certain producer is to be reported or not
			$to_report = false;

			if(!$preview)
			{
				//add the settings for this participant to report_data
				$rd = new UKReportData($this->db);
				$rd->report_id = $this->id;
				$rd->crso_id = $crso;
				$rd->participant_id = $r->PARTICIPANT_RELATION_ID;
				$rd->pr_status = $r->PR_STATUS;
				$rd->weee_status = $r->WEEE_STATUS;
				$rd->vat_registered = $r->VAT_REGISTERED;
				$rd->insert();
			}

			$product_map = $this->get_product_map($year);

			foreach ($product_map as $product_id => $ea_label) {
				$products[$ea_label]['B2B'] = 0.000;
				$products[$ea_label]['B2C'] = 0.000;
			}


			if($quarter == 4 || $per_year == true){
				// this is already for the whole year, so no point in checking
				$tonnes = $this->getDataForActiveQuarters($crso, $participant, $year, 'B2B');
				if($tonnes != null){

					foreach ($product_map as $product_id => $ea_label) {
						$products[$ea_label]['B2B'] = $tonnes[$ea_label];
					}

					$to_report = true;
				}
			}
			if($per_year == true) {
				$tonnes = $this->getDataForActiveQuarters($crso, $participant, $year, 'B2C');
				if($tonnes != null){

					foreach ($product_map as $product_id => $ea_label) {
						$products[$ea_label]['B2C'] = $tonnes[$ea_label];
					}

					$to_report = true;
				}
			}
			else {
				if($r->WEEE_STATUS == 2) {

					$to_report = true;
					// B2C weights
					$declarations = 'SELECT D.PRODUCT_ID, P.NAME AS PRODUCT_NAME, CPD.NAME AS DIMENSION_NAME,
									DL.PRODUCT_CATEGORY_ID, R.RELATION_ID,
									R.NAME AS RELATION_NAME, CAST(SUM(IFNULL(DL.WEIGHT, 0)) / 1000 AS DECIMAL(10,3)) AS TONNAGE
								FROM TBL_DECLARATION D
								INNER JOIN TBL_DECLARATION_LINE DL ON D.DECLARATION_ID = DL.DECLARATION_ID
								INNER JOIN TBL_RELATION R ON D.PARTICIPANT_RELATION_ID = R.RELATION_ID
								INNER JOIN TBL_PRODUCT P ON D.PRODUCT_ID = P.PRODUCT_ID
								INNER JOIN TBL_CRSO_PRODUCT_DIMENSION CPD
									ON D.PRODUCT_ID = CPD.PRODUCT_ID AND D.CRSO_RELATION_ID = CPD.CRSO_RELATION_ID AND DL.CPD_ID = CPD.CPD_ID
								WHERE D.CRSO_RELATION_ID=? AND
								D.YEAR = ? AND D.MONTH IN (?,?,?) ';
					if(!$this->isGracePeriod($crso, $year, $quarter)) {
						$declarations .= ' AND D . IS_EXPORTED > 0 ';
					}
					$declarations .= ' AND D.PARTICIPANT_RELATION_ID = ?
								AND CPD.NAME="Household"
								GROUP BY D.PRODUCT_ID, DL.CPD_ID, DL.PRODUCT_CATEGORY_ID';

					$d_params = array();
					switch($quarter)
					{
						case 1:
							$d_params = array($crso,$year,'1','2','3', $participant);
							break;
						case 2:
							$d_params = array($crso,$year,'4','5','6', $participant);
							break;
						case 3:
							$d_params = array($crso,$year,'7','8','9', $participant);
							break;
						case 4:
							$d_params = array($crso,$year,'10','11','12', $participant);
							break;
					}
					$q = $this->db->query($declarations);
					$q->bind_values($d_params);
					$q->execute();
					$declarationRows = $q->fetchAll();

					foreach($declarationRows as $row)
					{
						$products[ $product_map[ $row->PRODUCT_ID ] ]['B2C'] += $row->TONNAGE;
					}

					// Netting off (Put on the Market - Refund)
					$products = $this->_processNettingOff($crso, $participant, $year, $quarter, $products);
				}
			}

			if($to_report) {

				foreach ($product_map as $product_id => $ea_label) {
					$products[$ea_label]['B2B'] = ($products[$ea_label]['B2B'] > 0) ? $products[$ea_label]['B2B']: 0.000;
					$products[$ea_label]['B2C'] = ($products[$ea_label]['B2C'] > 0) ? $products[$ea_label]['B2C']: 0.000;
				}

				//get totals for preview
				if($preview)
				{
					foreach ($product_map as $product_id => $ea_label) {
						$total_tonnes += $products[$ea_label]['B2B'];
						$total_tonnes += $products[$ea_label]['B2C'];
					}
				}


				$frag = $template->createDocumentFragment();

				$prod = $frag->appendChild($template->createElement('weee:Producer'));

				//we keep the first node so we can insert a node with the totals before it for the preview
				if(!isset($before_node))
					$before_node = $prod;

				$prod->appendChild($template->createElement('weee:RegistrationNo', $prn));
				$prod->appendChild($this->makeDomNode($template,'weee:ProducerCompanyName', $business_name));

				$returns = $prod;
				if ($xsd_version == 3)
					$returns = $prod->appendChild($template->createElement('weee:Returns'));

				foreach ($product_map as $product_id => $ea_label) {

					if($report_b2b) {
						$ret = $returns->appendChild($template->createElement('weee:Return'));
						$ret->appendChild($template->createElement('weee:CategoryName', $ea_label));
						$ret->appendChild($template->createElement('weee:ObligationType', 'B2B'));
						$ret->appendChild($template->createElement('weee:TonnesReturnValue', $products[$ea_label]['B2B']));
					}
					if($report_b2c) {
						$ret = $returns->appendChild($template->createElement('weee:Return'));
						$ret->appendChild($template->createElement('weee:CategoryName', $ea_label));
						$ret->appendChild($template->createElement('weee:ObligationType', 'B2C'));
						$ret->appendChild($template->createElement('weee:TonnesReturnValue', $products[$ea_label]['B2C']));
					}
				}

				$producerListNode->appendChild($frag);
				if(!$preview)
				{

					foreach ($product_map as $product_id => $ea_label) {

						// $product_id, $obligation_type, $tonnage
						$rd->insertReportDataLine($product_id, OBLIGATION_TYPE_B2B, $products[$ea_label]['B2C']);
						$rd->insertReportDataLine($product_id, OBLIGATION_TYPE_B2C, $products[$ea_label]['B2B']);
					}

					$reg->set($crso,$participant);
					$reg->updateWEEEStatusExported();
					$reg->setLastExportWeee($d);
				}
			}
		}

		if($preview)
		{
			if($returnXml) return $template;
			if(isset($before_node)) {
				$tot = $producerListNode->insertBefore($template->createElement('weee:TonnesTotal', $total_tonnes), $before_node);
			} else {
				$tot = $producerListNode->appendChild($template->createElement('weee:TonnesTotal', $total_tonnes));
			}
			$preview_file = UK_EXPORT_PATH.UK_EXPORT_TEMPLATES.'PREVIEW.XML';
			$template->save($preview_file);
			return $preview_file;
		}
		else
		{
			// if crso data was already reported and accepted, we dont change its status
			if($per_year == false && $crso_data->status != CRSO_DATA_ACCEPTED)
				$crso_data->updateStatus(CRSO_DATA_EXPORTED,$d);

			$template->save($file);

			// this pretty formats the xml
			$template->preserveWhiteSpace = false;
			$template->formatOutput = true;
			$template->load($file);
			$template->save($file);


			info('export Actuals Report for CRSO relation '.$crso.', year:'.$year.' quarter:'.$quarter);
			return $file;
		}
	}

	function getYearsReport($crso,$rep_type='')
	{
		$sql = " SELECT DISTINCT `YEAR`
				 FROM TBL_UK_REPORTS
				 WHERE CRSO_RELATION_ID = ? ";
		if($rep_type != '')
			$sql .= " AND REPORT_TYPE = ? ";
		$sql .= "ORDER BY `YEAR`";
		$q = $this->db->query($sql);
		if($rep_type != '')
			$q->bind_values($crso,$rep_type);
		else
			$q->bind_values($crso);
		$q->execute();
		return $q;
	}
	function getQuartersWeeeReport($crso,$year)
	{
		$sql = " SELECT DISTINCT `QUARTER`
				 FROM TBL_UK_REPORTS
				 WHERE CRSO_RELATION_ID = ? AND `YEAR`=? AND REPORT_TYPE = 2 ORDER BY `QUARTER`";
		$q = $this->db->query($sql);
		$q->bind_values($crso,$year);
		$q->execute();
		return $q;
	}

	/**
	 * Returns a boolean on whether or not we are in the declaration period of the last month of the quarter.
	 * This is the period between the 1st of the month of the new quarter and the export day.
	 */
	function isGracePeriod($crso, $year, $quarter=false) {

		$date = currentDate();

		$currentYear = $date->getYear();
		$month = $date->getMonth();
		$day = $date->getDay();

		$cp = new CRSOProduct($this->db);
		$export_day = $cp->getMaxExportDay((int)$crso);

		// We are checking per year (so january next year)
		if(!$quarter||$quarter==4) {
			if($currentYear != ($year+1)) {
				return false;
			}
			if($month != 1) {
				return false;
			}
			if($day < $export_day) {
				return true;
			}
			return false;
		}
		$export_month = ($quarter*3)+1;
		if($currentYear != $year) {
			return false;
		}
		if($month == $export_month && $day < $export_day) {
			return true;
		}
		return false;

	}

	function getYearsForWeeeExport($crso)
	{
		$date = currentDate();
		$year = $date->getYear();
		$month = $date->getMonth();
		$quarter = ceil((int)$month / 3);

		//number of quarters for the start year
		$q_no_start = 5 -START_QUARTER;
		$q_no = 4;
		$day = $date->getDay();
		$cp = new CRSOProduct($this->db);
		$export_day = $cp->getMaxExportDay((int)$crso);
		$export_month = (($quarter - 1)*3)+1;


		if($quarter == 1)
		{
			$year = $year - 1;
			if($month == $export_month && $day < $export_day)
			{
				$q_no = $q_no - 1;
				$q_no_start = $q_no_start - 1;
			}
		}
		if($quarter == 2)
		{
			if($month == $export_month && $day < $export_day)
				$year = $year - 1;
		}

		$sql = "SELECT QUARTER FROM TBL_UK_REPORTS
				WHERE CRSO_RELATION_ID = ? AND REPORT_TYPE=2 AND STATUS IN (1,2) AND `YEAR`=?";
		$years =array();
		for($i=START_YEAR;$i<=$year;$i++)
		{
			$q = $this->db->query($sql);
			$q->bind_values($crso,$i);
			$q->execute();
			$qn = $q->fetchAll();
			$cqn = count($qn);
			if($cqn == 0)
			{
				if($i == START_YEAR && $q_no_start == 0)
					continue;
				else
					$years[] = $i;
			}
			else
				if($i == START_YEAR)
				{
					if($cqn < $q_no_start)
						$years[] = $i;
				}
				else
				{
					if(!in_array('4',$qn))
						$years[] = $i;
					else
					if($cqn < $q_no)
						$years[] = $i;
				}
		}
		return $years;
	}
	function getQuartersForWeeeExport($crso,$year,$allowGrace=false)
	{
		$all_quarters = array();
		$date = currentDate();

		$cp = new CRSOProduct($this->db);
		$export_day = $cp->getMaxExportDay((int)$crso);
		$day = $date->getDay();
		$month = $date->getMonth();
		$quarter = ceil((int)$month / 3)-1;
		$export_month = ($quarter*3)+1;

		if($year == START_YEAR)
		{
			$q = 4;
			if($quarter == 0 && $day < $export_day && $month == $export_month)
				$q = 3;
			for($i=START_QUARTER;$i<=$q;$i++)
				$all_quarters[] = $i;
		}
		elseif($year == $date->getYear())
		{
			if($quarter == 0)
				return;
			if($day < $export_day && $month == $export_month && !$allowGrace) {
				$quarter = $quarter - 1;
			}
			for($i=1;$i<=$quarter;$i++)
				$all_quarters[] = $i;
		}
		else
		{
			if($quarter == 0 && $day<$export_day && $month == $export_month && !$allowGrace)
				$all_quarters = array('1','2','3');
			else
				$all_quarters = array('1','2','3','4');
		}
		return $all_quarters;
	}

	function hasAllYearReports($crso,$year){
		$sql = "SELECT DISTINCT QUARTER FROM TBL_UK_REPORTS
				WHERE CRSO_RELATION_ID = ? AND REPORT_TYPE=2 AND STATUS=2
				AND `YEAR`=?";
		$q = $this->db->query($sql);
		$q->bind_values($crso,$year);
		$q->execute();
		$qs = array();
		while($r = $q->fetch())
			$qs[] = $r->QUARTER;
		if(in_array(1, $qs) && in_array(2, $qs) && in_array(3, $qs) && in_array(4, $qs))
			return true;
		return false;
	}

	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_UK_REPORTS';
		$this->objectmap['primary']['id'] = 'REPORT_ID';
		$this->objectmap['properties']['report_type'] = 'REPORT_TYPE';
		$this->objectmap['properties']['file_name'] = 'FILE_NAME';
		$this->objectmap['properties']['crso_id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['export_date'] = 'EXPORT_DATE';
		$this->objectmap['properties']['status'] = 'STATUS';
		$this->objectmap['properties']['year'] = 'YEAR';
		$this->objectmap['properties']['quarter'] = 'QUARTER';
		$this->objectmap['properties']['reason'] = 'REJECT_REASON';
	}
}
?>
