<?php
/**
 * Created on Dec 13, 2007
 *
 * @author Nicoleta Militaru <nicoleta@basesoft.com>
 **/

require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'uk/UKRegistrationData.php');

class UKSicCode extends AbstractObject
{
	var $id;
	var $crso;
	var $participant;
	var $sic_code;

	function set($sic_id='')
	{
		$sic_id || $sic_id = $this->id;

		$sql = " SELECT CRSO_RELATION_ID,PARTICIPANT_RELATION_ID,SIC_CODE_ID,SIC_CODE
				FROM TBL_UK_SIC_CODE
				WHERE SIC_CODE_ID = ? ";
		$q = $this->db->query($sql);
		$q->bind_values($sic_id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		$this->id = $r->SIC_CODE_ID;
		$this->crso = new CRSO($this->db,array('relation'=> new Relation($this->db,$r->CRSO_RELATION_ID)));
		$this->participant = new Participant($this->db,array('relation'=> new Relation($this->db,$r->PARTICIPANT_RELATION_ID)));
		$this->sic_code = $r->SIC_CODE;
		return $this->complete = true;

	}

	public function getOverview($crso_id='',$participant_id='')
	{
		if($crso_id == '')
			$crso_id = (isset($this->crso->relation->id))?$this->crso->relation->id:false;
		if($participant_id == '')
			$participant_id = (isset($this->participant->relation->id))?$this->participant->relation->id:false;
		$sql = " SELECT CRSO_RELATION_ID,PARTICIPANT_RELATION_ID,SIC_CODE_ID,SIC_CODE
				FROM TBL_UK_SIC_CODE
				WHERE CRSO_RELATION_ID = ?
					AND PARTICIPANT_RELATION_ID = ?	";
		$q = $this->db->query($sql);
		$q->bind_values($crso_id,$participant_id);
		$q->execute();
		return $q;
	}

	public function setID($sic)
	{
		$sql = "SELECT SIC_CODE_ID FROM TBL_UK_SIC_CODE WHERE CRSO_RELATION_ID = ?
				AND PARTICIPANT_RELATION_ID = ? AND SIC_CODE = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->crso->relation->id,$this->participant->relation->id,$sic);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		$this->id = $r->SIC_CODE_ID;
	}

	public function insert()
	{
		$a = parent::insert();
		$this->_notifyParent();
		return $a;
	}

	function update()
	{
		$sql = "UPDATE TBL_UK_SIC_CODE SET SIC_CODE = ? WHERE SIC_CODE_ID = ? ";
		$q = $this->db->query($sql);
		$q->bind_values($this->sic_code,$this->id);
		$q->execute();
		$count = $q->rowCount();
		if($count > 0)
			$this->_notifyParent();
	}

	function deleteCodes($codes = array())
	{
		if(count($codes) == 0)
			return;
		$sql = "DELETE FROM TBL_UK_SIC_CODE WHERE SIC_CODE IN (" . str_repeat("?,",count($codes)-1) . "?)
				AND CRSO_RELATION_ID = ? AND PARTICIPANT_RELATION_ID = ?";
		$q = $this->db->query($sql);
		$codes[] = $this->crso->relation->id;
		$codes[] = $this->participant->relation->id;
		$q->bind_values($codes);
		$q->execute();
		$count = $q->rowCount();
		if($count > 0)
			$this->_notifyParent();
	}

	function _notifyParent()
	{
		$reg = new UKRegistrationData($this->db);
		$reg->set($this->crso->relation->id,$this->participant->relation->id);
		$date = date('Y-m-d H:i:s');
		$reg->childUpdate($date);
	}

	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_UK_SIC_CODE';
		$this->objectmap['primary']['id'] = 'SIC_CODE_ID';
		$this->objectmap['properties']['crso->relation->id'] = 'CRSO_RELATION_ID';
		$this->objectmap['properties']['participant->relation->id'] = 'PARTICIPANT_RELATION_ID';
		$this->objectmap['properties']['sic_code'] = 'SIC_CODE';
	}
}
?>
