<?php
/**
 * DefaultReportChart
 * Generates a default-report-chart like we know from DefaultReport
 *
 * <code>
 * 	$c = new DefaultReportChart( $langIf );
 * 	$c->setTitle('Vraag 11');
 *
 * 	//$c->addValue('Ding 1', 25);
 * 	//$c->addValue('Ding 2', 40);
 * 	$c->setValues( array( array('aap', 4), array('bier', 5) )  );
 *
 * 	//$c->setTotal(100);
 * 	echo $c->getHtml();
 * </code>
 *
 *
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @version $Id$
 * @package I-Ware
 *
 */
class DefaultReportChart
{
	/**#@+
	 * @access private
	 */
	var $_values = array();
	var $_total  = 0;
	var $_title  = 'TITLE';
	var $_subtitle = '';

	var $_langIf = null;
	/**#@-*/

	/**
	 * @param LanguageInterface langIf
	 */
	function DefaultReportChart(&$langIf)
	{
		$this->_langIf = $langIf;
	}

	/**
	 * @param string
	 */
	function setTitle($title)
	{
		$this->_title = $title;
	}

	/**
	 * @param string
	 */
	function setSubTitle($subtitle)
	{
		$this->_subtitle = $subtitle;
	}

	/**
	 * Sets the values all in once using an array or query object.
	 * @param array|query
	 * <code>$c->setValues( array( array('aap', 4), array('bier', 5) ) );</code>
	 */
	function setValues($arr)
	{
		if (is_object($arr))
		{
			while ($row = $arr->fetch())
			{
				list ($title, $value) = (array) $row;
				$this->addValue($title, $value);
			}
		}
		else
		{
			foreach ($arr as $row)
			{
				list ($title, $value) = (array) $row;
				$this->addValue($title, $value);
			}

		}
	}

	/**
	 * Adds a row to the chart
	 * @param string title
	 * @param string value
	 */
	function addValue($title, $value)
	{
		$this->_values[] = array($this->_subtitle => $this->_langIf->getText($title), 'VALUE'=>(int) $value);
		$this->_total +=   $value;
	}

	/**
	 * Sets the total value of the chart in case you dont want the total to be summed up by addValue.
	 * This should be done after setValues() or addValue() and  before getHtml()
	 */
	function setTotal($total)
	{
		$this->_total = $total;
	}

	/**
	 * Creates the chart and returns the HTML
	 * @return string HTML
	 */
	function getHtml()
	{
		foreach ($this->_values as $key => $arr)
		{
			//-- add empty column where we can draw the bars in
			$this->_values[$key]['PERCENT_BAR'] = '';

			//-- calculate % based on total
			if ($this->_total > 0) #bug
				$this->_values[$key]['PERCENT'] = ($arr['VALUE'] / $this->_total) * 100;
			else
				$this->_values[$key]['PERCENT'] = 0;
		}

		$rv = '';
		$carpenter = new TableFactory($this->_values, $this->_langIf);

		// create footer
		$marscompany = new TableFactory(array(array('0%', '20%', '40%', '60%', '80%')));
		$marscompany->decapitate();
		$marscompany->width = ' width="200px"';
		$marscompany->columnModifier(explode('@', substr(str_repeat("@class=\"DefaultPercentageBar\" width=\"40px\"", 5),1))); //array_fill.....
		$carpenter->setFooter(array($this->_langIf->getText('Total'),array('return '.$this->_total.';'), $marscompany->getXHTMLTable(array()),''));

		// modify fixed style
		$rightalign = ' style="text-align: right;"';
		$carpenter->columnModifier(array( $this->_subtitle =>' style="width: 100%;"', 'VALUE'=>$rightalign.' width="50px"', 'PERCENT'=>$rightalign.' width="50px"'));

		// create bars
		$spacer = mkDot(1,15);
		$carpenter->setValueProcessor('PERCENT_BAR', '$marscompany = new TableFactory(array(array(\''.$spacer.'\',\'\')));
			$marscompany->decapitate();
			$marscompany->width = \' width="200px"\';
			$marscompany->columnModifier(array(0=>\' class="__BARCLASS__" width="\'.(($row[\'PERCENT\']*2)?($row[\'PERCENT\']*2):1).\'px"\'));
			return str_replace(\'__BARCLASS__\', \'DefaultBar\'.($i%19), $marscompany->getXHTMLTable(array()));');
		$carpenter->setValueProcessor('PERCENT', 'return number_format($value, 2, \',\', \'.\');');

		$rv .= '<div class="DefaultQuestionContainer" style="width: 100%;">';
		$rv .= '<div class="DefaultAnswerTableContainer"><h3 class="Question">'.$this->_title.'</h3>';
		$rv .= $carpenter->getXHTMLTable( array('table'=>'DefaultAnswerTable', 'td'=>'DefaultAnswer', 'th'=>'DefaultAnswerHead', 'tf'=>'DefaultAnswerFoot'), 2);
		$rv .= '</div>';
		$rv .= '</div>';

		return $rv;

	}
}

?>
