<?php
require_once(IWARE.'AbstractObject.php');
/**
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @package ELC
 * @version $Id$
 */
class Document extends AbstractObject
{
	var $id;
	var $name;
	var $md5;
	var $temp_session_id;
	var $size;
	var $temp_name = null;
	var $file_name = null;

	/**
	 * @param string $id
	 */
	function set($id='')
	{
		$id || $id = $this->id;
		$this->id = $id;

		if (!$this->hasPrivileges()) {

			if (isset($_SERVER['HTTP_REFERER']))
			{
				//crit("No permission to view document (".$id.")");
				header('HTTP/1.1 403 Forbidden');
				throw new IWUserException('No permission to view document ('.$id.')');

			}

			header_message('session_timed_out');
		}

		$sql = "SELECT DOCUMENT_ID, NAME, MD5, CREATED, TEMP_SESSION_ID
			FROM TBL_DOCUMENT
			WHERE DOCUMENT_ID=?";
		$q = $this->db->query($sql);
		$q->bind_values($id);

		if(!$q->execute())
			return false; // Throw

		if (!($r = $q->fetch()))
			throw new IWException("Document ($id) not found in database");

		$this->name = $r->NAME;
		$this->md5 = $r->MD5;
		$this->file_name = $this->_getFilenameFromId($this->id);
		$this->created = $r->CREATED;
		$this->temp_session_id = $r->TEMP_SESSION_ID;

		if(defined('TEST_SITE') && TEST_SITE)
			$this->size = 100;
		elseif (!file_exists(DOCUMENT_REPOSITORY . $this->file_name))
			throw new IWException("Document ($id) not found in repository");
		else
			$this->size = filesize(DOCUMENT_REPOSITORY.$this->file_name);

		return $this->complete = true;
	}

	/**
	 *
	 */
	function delete($id='')
	{
		$delete_document = true;
		try
		{
			parent::delete();
		}
		catch(Exception $e)
		{
			if (strpos($e->getMessage(), 'Integrity constraint violation') > 0)
			{
				// if it's an  Integrity constraint violation, we don't phisically delete the document
				// (because it was downloaded already)
				$delete_document = false;
			}
			else
			{
				throw $e;
			}
		}

		if ($delete_document)
		{
			$this->file_name = $this->getFileName();
			if (file_exists(DOCUMENT_REPOSITORY . $this->file_name))
				unlink(DOCUMENT_REPOSITORY . $this->file_name);
		}
		return true;
	}

	/**
	 * @param string $temp_session_id
	 * @return Array $rv
	 */
	function getBySessionId($temp_session_id)
	{
		$rv = array();
		$sql = "SELECT DOCUMENT_ID FROM TBL_DOCUMENT
			WHERE TEMP_SESSION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($temp_session_id);
		$q->execute();
		$rs = $q->fetchAll();
		if(!count($rs))
			return false;
		foreach($rs as $r)
			$rv[] = $r->DOCUMENT_ID;
		return $rv;
	}

	function insert()
	{
		$this->created = date('Y-m-d H:i:s');

		if (!parent::insert())
			return false;

		if (!is_null($this->temp_name))
		{
			if (!file_exists($this->temp_name))
				trigger_error("Tempfile ($this->temp_name) does not exist", E_USER_ERROR);

			if (empty($this->name))
				trigger_error("Name cannot be empty", E_USER_ERROR);

			$this->file_name = $this->_getFilenameFromId($this->id);
			$destination = DOCUMENT_REPOSITORY . $this->file_name; //don't need basename as we generate it

			if (!copy($this->temp_name, $destination)) // for mr unit test
				trigger_error("Could not copy tempfile ($this->temp_name) to $destination", E_USER_ERROR);


			if (!file_exists(DOCUMENT_REPOSITORY.$this->_getFilenameFromId($this->id)))
				trigger_error("moved file does not exist (".DOCUMENT_REPOSITORY.$this->_getFilenameFromId($this->id).") ", E_USER_ERROR);

			$this->md5 = md5_file(DOCUMENT_REPOSITORY.$this->_getFilenameFromId($this->id));
		}

		info(sprintf('Inserted document [%s]. md5: [%s]', $this->id, $this->md5));

		return $this->update();
	}

	function update()
	{
		info(sprintf('Updated document [%s]', $this->id));
		return parent::update();
	}

	function getMimeType()
	{
		return ''; // FIXME
	}

	static public function &createDocument(&$db)
	{
		if (file_exists(SITE_HOMEDIR . SITE_PATH.'classes/'.APP_PATH.'Document.php'))
		{
			require_once(SITE_HOMEDIR . SITE_PATH.'classes/'.APP_PATH.'Document.php');
			$class = APP_DOCUMENT_OBJECT;
			$that = new $class($db);
			return $that;
		}

		$that = new Document($db);
		return $that;
	}

	/**
	 */
	function hasPrivileges()
	{

		if(defined('IN_IQUNIT') || php_sapi_name() == 'cli' || defined('IN_TASK_RUNNER'))
			return true;

		// check temp session
		$sql = "SELECT DOCUMENT_ID FROM TBL_DOCUMENT WHERE TEMP_SESSION_ID=?";
		$q = $this->db->query($sql);
		$q->bind_values(session_id());
		$q->execute();
		if ($r = $q->fetch())
			return true;

		// default no rights
		return false;
	}

	function getFileName()
	{
		return $this->_getFilenameFromId($this->id);
	}

	function makePermanent()
	{
		$this->temp_session_id = DB_NULL;
		$this->update();
	}

	/**
	 * @access private
	 */
	function _getFilenameFromId($id)
	{
		return str_pad($id, 8, '0', STR_PAD_LEFT);
	}
	/**#@+
	 * @access protected
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_DOCUMENT';
		$this->objectmap['primary']['id'] = 'DOCUMENT_ID';
		$this->objectmap['properties']['name'] = 'NAME';
		$this->objectmap['properties']['md5'] = 'MD5';
		$this->objectmap['properties']['created'] = 'CREATED';
		$this->objectmap['properties']['temp_session_id'] = 'TEMP_SESSION_ID';
	}
}
/**#@+
 * Defines
 */

# FIXME, dont use session
if ( !defined('DOCUMENT_REPOSITORY') )
	define('DOCUMENT_REPOSITORY', '/home/clients/'.(isset($_SESSION['session_user_company_name']) ? $_SESSION['session_user_company_name'] : 'AbstractCompany').'/documents/');

define('DOCUMENT_PRIV_READ', 1);
define('DOCUMENT_PRIV_WRITE', 2);
define('DOCUMENT_PRIV_DELETE', 4);
/**#@-*/
?>
