<?php
require_once(IWARE.'AbstractObject.php');
/**
 *
 * @author Marten Koedam <marten@basesoft.com>
 * @package I-Ware
 * @subpackage FAQ
 * @version $Id$
 */
class FAQSection extends AbstractObject
{
	/**
	 *
	 */
	var $id;
	/**
	 * @var nLanguage $default_language
	 */
	var $default_language;
	/**
	 *
	 */
	var $position;
	/**
	 *
	 */
	var $handle;
	/**
	 *
	 */
	var $faqs = array();
	/**
	 * @var FAQSectionLang $section_lang
	 */
	var $section_lang;
	/**
	 * @param int $id
	 */
	function set($id=null,$language_id='',$fill=true)
	{
		$id || $id = $this->id;
		$id = (int) $id;

		$sql = "SELECT DEFAULT_LANGUAGE_ID, POSITION, HANDLE, PRODUCT_ID
			FROM TBL_FAQ_SECTION
			WHERE FAQ_SECTION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		if(!$q->execute(true))
			return false; // ThrowException
		if(!($r = $q->fetch()))
			return false;

		require_once(IWARE.'nLanguage.php');
		require_once(ELC_CLASSES.'Product.php'); // EEEP!! TODO: Remove this some day

		$this->id = $id;
		$this->default_language = new nLanguage($this->db,$r->DEFAULT_LANGUAGE_ID);
		$this->product = new Product($this->db,$r->PRODUCT_ID);
		$this->position = $r->POSITION;
		$this->handle = $r->HANDLE;

		if($fill)
		{
			$language = $language_id?new nLanguage($this->db,$language_id):$this->default_language;
			$this->section_lang = new FAQSectionLang($this->db,array('language'=>$language,'section'=>$this));
			if(!$this->section_lang->set())
			{
				$this->section_lang->language = $this->default_language;
				$this->section_lang->set();
			}

			require_once(IWARE.'FAQ.php');

			$q = $this->getFAQs();
			$this->faqs = array();
			while($r = $q->fetch())
			{
				$faq = new FAQ($this->db);
				$faq->set($r->FAQ_ID,$this->section_lang->language->id);
				$this->faqs[] = $faq;
			}
		}

		return $this->complete = true;
	}
	/**
	 *
	 */
	function insert()
	{
		$this->position = $this->_getNextPosition();
		parent::insert();
	}
	/**
	 * @param int $id
	 */
	function getFAQs($id=null)
	{
		$id || $id = $this->id;
		$id = (int) $id;

		$sql = "SELECT FAQ_ID
			FROM TBL_FAQ
			WHERE FAQ_SECTION_ID = ?
			ORDER BY POSITION";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		return $q;
	}
	/**
	 *
	 */
	function getSections($product_id)
	{
		$sql = "SELECT FAQ_SECTION_ID, HANDLE
			FROM TBL_FAQ_SECTION
			WHERE PRODUCT_ID = ?
			ORDER BY POSITION";
		$q = $this->db->query($sql);
		$q->bind_values($product_id);
		$q->execute();
		return $q;
	}
	/**
	 *
	 */
	function getSectionsByGroup($group_id='',$product_id)
	{
		$group_id || $group_id = $_SESSION['session_group_group_id'];

		$sql = "SELECT FS.FAQ_SECTION_ID
			FROM TBL_FAQ_SECTION as FS
			INNER JOIN TBL_GROUP_FAQ_SECTION as GFS
				USING(FAQ_SECTION_ID)
			WHERE GROUP_ID = ?
				AND PRODUCT_ID = ?
			ORDER BY `POSITION`";
		$q = $this->db->query($sql);
		$q->bind_values($group_id,$product_id);
		$q->execute();
		return $q;
	}
	/**#@+
	 * @access private
	 */
	/**
	 *
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_FAQ_SECTION';
		$this->objectmap['primary']['id'] = 'FAQ_SECTION_ID';
		$this->objectmap['properties']['default_language->id'] = 'DEFAULT_LANGUAGE_ID';
		$this->objectmap['properties']['product->id'] = 'PRODUCT_ID';
		$this->objectmap['properties']['position'] = 'POSITION';
		$this->objectmap['properties']['handle'] = 'HANDLE';
	}
	/**
	 *
	 */
	function _getNextPosition()
	{
		$sql = "SELECT MAX(`POSITION`) as `POS`
			FROM TBL_FAQ_SECTION";
		$q = $this->db->query($sql);
		$q->execute();
		if(!($r = $q->fetch()))
			return 1;
		return ((int) $r->{'POS'})+1;
	}
	/**#@-*/
}
/**
 * @author Marten Koedam <marten@basesoft.com>
 * @package I-Ware
 */
class FAQSectionLang extends AbstractObject
{
	/**
	 *
	 */
	var $section;
	/**
	 *
	 */
	var $language;
	/**
	 *
	 */
	var $title;
	/**
	 *
	 */
	function set($section_id=null,$language_id=null)
	{
		$section_id || $section_id = $this->section->id;
		$language_id || $language_id = $this->language->id;

		$sql = "SELECT TITLE
			FROM TBL_FAQ_SECTION_LANG
			WHERE FAQ_SECTION_ID = ?
				AND LANGUAGE_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($section_id,$language_id);
		if(!$q->execute(true))
			return false; // ThrowException
		if(!($r = $q->fetch()))
			return false;

		require_once(IWARE.'nLanguage.php');

		$this->language = new nLanguage($this->db,$language_id);
		$this->section = new FAQSection($this->db,$section_id);
		$this->title = $r->TITLE;

		return $this->complete = true;
	}
	/**#@+
	 * @access private
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_FAQ_SECTION_LANG';
		$this->objectmap['primary']['section->id'] = 'FAQ_SECTION_ID';
		$this->objectmap['primary']['language->id'] = 'LANGUAGE_ID';
		$this->objectmap['properties']['title'] = 'TITLE';
	}
	/**#@-*/
}
/**
 * @author Marten Koedam <marten@basesoft.com>
 * @package I-Ware
 */
class GroupFAQSection extends AbstractObject
{
	/**
	 * @var Group $group_id
	 */
	var $group_id;
	/**
	 * @var FAQSection $section
	 */
	var $section;
	/**
	 * Select the section for a group
	 * @param int $group_id
	 */
	function getSections($group_id=null)
	{
		$group_id || $group_id = $this->group_id;
		$group_id = (int) $group_id;

		$sql = "SELECT FAQ_SECTION_ID
			FROM TBL_GROUP_FAQ_SECTION
			WHERE GROUP_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($group_id);
		$q->execute();
		return $q;
	}
	/**
	 * Select the groups for a section
	 * @param int $section_id
	 */
	function getGroups($section_id=null)
	{
		$section_id || $section_id = $this->section->id;
		$section_id = (int) $section_id;

		$sql = "SELECT GROUP_ID
			FROM TBL_GROUP_FAQ_SECTION
			WHERE FAQ_SECTION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($section_id);
		$q->execute();
		return $q;
	}
	/**
	 *
	 */
	function deleteFromSection($section_id=null,$group_id=false)
	{
		$section_id || $section_id = $this->section->id;
		$section_id = (int) $section_id;

		$values = array($section_id);
		$sql = "DELETE FROM TBL_GROUP_FAQ_SECTION WHERE FAQ_SECTION_ID = ? ";
		if($group_id)
		{
			$values[] = $group_id;
			$sql .= " AND GROUP_ID = ?";
		}
		$q = $this->db->query($sql);
		$q->bind_values($values);
		return $q->execute();
	}
	/**#@+
	 * @access private
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_GROUP_FAQ_SECTION';
		$this->objectmap['primary']['group_id'] = 'GROUP_ID'; // FIXME group needs to be abstract object
		$this->objectmap['primary']['section->id'] = 'FAQ_SECTION_ID';
	}
	/**#@-*/
}
?>