<?php
/**
 * @package I-Ware
 * @since 18-11-2005
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright 2005, BaseSoft B.V.
 * @example
		$FB = new FormBehaviour($langIf);
		// Register the field user[name] to be obliged and word-characters only
		$FB->register('user[name]','alpha;required','User Name');

 */

define ('FB_STATUS_UNDECIDED', -1);
define ('FB_STATUS_INVALID', 0);
define ('FB_STATUS_VALID', 1);

class FormBehaviour
{
	var $_plugins;
	var $_failures;
	var $debug = false;

	protected static $fb;

	/**
	 * All the registered elements on this page
	 * @var Array $elements
	 */
	var $_elements;
	/**
	 * @var LanguageInterface $langIf
	 */
	var $langIf;

	/**
	 * Array with values where formbehaviour will work on. This will be $_POST
	 * by default. Use addValue(s) to write in this
	 */
	public $arrValues;

	/**
	 * Constructor
	 * @param LanguageInterface $langIf
	 */
	function __construct($langIf=null, $db=null)
	{
		$this->debug && debug('FormBehaviour::construct()');

		$this->_plugins = array();
		$this->_failures = array();
		$this->_elements = array();

		$this->langIf = $langIf;
		$this->db = $db;

		$this->addPlugin('required', 'FormBehaviourPluginRequired');
		$this->addPlugin('year', 'FormBehaviourPluginYear');
		$this->addPlugin('month', 'FormBehaviourPluginMonth');
		$this->addPlugin('day', 'FormBehaviourPluginDay');
		$this->addPlugin('phone', 'FormBehaviourPluginPhone');
		$this->addPlugin('url', 'FormBehaviourPluginUrl');
		$this->addPlugin('numeric', 'FormBehaviourPluginNumeric');
		$this->addPlugin('alpha', 'FormBehaviourPluginAlpha');
		$this->addPlugin('alphanumeric', 'FormBehaviourPluginAlphaNumeric');
		$this->addPlugin('email', 'FormBehaviourPluginEmail');
		$this->addPlugin('date', 'FormBehaviourPluginDate');
		$this->addPlugin('vat', 'FormBehaviourPluginVAT');
		$this->addPlugin('iban', 'FormBehaviourPluginIBAN');
		$this->addPlugin('integer', 'FormBehaviourPluginInteger');
		$this->addPlugin('document', 'FormBehaviourPluginDocument');
		$this->addPlugin('length', 'FormBehaviourPluginLength');
		$this->addPlugin('ip', 'FormBehaviourPluginIP');
		$this->addPlugin('ipwildcard','FormBehaviourPluginIPWildcard');
		$this->addPlugin('float', 'FormBehaviourPluginFloat');
		$this->addPlugin('brand', 'FormBehaviourPluginBrand');
		$this->addPlugin('companyname', 'FormBehaviourPluginCompanyName');
		$this->addPlugin('personname', 'FormBehaviourPluginPersonName');
		$this->addPlugin('address', 'FormBehaviourPluginAddress');
		$this->addPlugin('zipcode', 'FormBehaviourPluginZipCode');
		$this->addPlugin('cityname', 'FormBehaviourPluginCityName');
		$this->addPlugin('swiftcode', 'FormBehaviourPluginSwiftCode');
		$this->addPlugin('bankaccount', 'FormBehaviourPluginBankAccount');
		$this->addPlugin('ponumber', 'FormBehaviourPluginPONumber');
		$this->addPlugin('departmentname', 'FormBehaviourPluginDepartmentName');
		$this->addPlugin('question', 'FormBehaviourPluginQuestion');
		$this->addPlugin('emailsubject', 'FormBehaviourPluginEmailSubject');
		$this->addPlugin('producerregno', 'FormBehaviourPluginProdRegistrationNo');
		$this->addPlugin('comment', 'FormBehaviourPluginComment');
		$this->addPlugin('cronmask', 'FormBehaviourPluginCronMask');
		$this->addPlugin('decimal', 'FormBehaviourPluginFloat');
		$this->addPlugin('productfee', 'FormBehaviourPluginProductFee');
		$this->addPlugin('glaccount', 'FormBehaviourPluginGLAccounts');
		$this->addPlugin('productcategory', 'FormBehaviourPluginProductCategory');
		$this->addPlugin('productcomponent', 'FormBehaviourPluginProductComponent');
		$this->addPlugin('invoicenumber', 'FormBehaviourPluginInvoiceNumber');
		$this->addPlugin('approvalatf', 'FormBehaviourPluginApprovalNoATF');
		$this->addPlugin('approvalae', 'FormBehaviourPluginApprovalNoAe');

		// by default we act on $_POST
		$values = $_POST;

		// merge $_FILES into $_POST, we only need the key and value
		foreach ($_FILES as $key => $file)
		{
			if(isset($values[$key]) && is_array($values[$key]))
			{
				//
				foreach($file['name'] as $key2 => $upload)
					if(isset($values[$key][$key2])&& is_array($values[$key][$key2]))
						$values[$key][$key2] = array_merge($values[$key][$key2],$upload);
					else
						$values[$key][$key2] = $upload;
			}
			else
				$values[$key] = $file['name'];
		}

		$this->setValues($values);
	}

	/**
	 * Tells formbehaviour to behave on this values. The argument should be an
	 * associative array like POST data
	 */
	public function setValues($arrValues)
	{
		$this->arrValues = $arrValues;
	}

	/**
	 * Sets a pair of name and value in the $arrValues array on
	 * which FormBehaviour will behave. If the argument register is passed it
	 * will register this value with the given behaviour.
	 */
	public function addValue($name, $value, $behaviour=false)
	{
		$this->debug && debug(sprintf('FormBehaviour: addValue(name=%s, value=%s, behaviour=%s))', $name, $value, $behaviour));
		$this->arrValues[$name] = $value;

		if ($behaviour !== false )
			$this->register($name, $behaviour);
	}

	public function clearValues()
	{
		$this->arrValues = array();
	}

	static function createFormBehaviour(&$langIf, $db=null)
	{
		if (!isset(self::$fb))
			self::$fb = new FormBehaviour($langIf, $db);

		return self::$fb;
	}

	function addPlugin($behaviour, $classPlugin)
	{
		$this->debug && debug(sprintf('FormBehaviour: Adding plugin [%s] for behaviour [%s]', $classPlugin, $behaviour));
		$this->_plugins[$behaviour] = $classPlugin;
	}

	function removePlugin($behaviour)
	{
		unset($this->_plugins[$behaviour]);
	}

	/**
	 * Validates the form and returns a string of errors
	 * @return bool valid
	 */
	function validate()
	{
		$this->debug && debug('Formbehaviour - starting validate()'."\n\n");
		// cache translation
		foreach ($this->_plugins as $plugin)
		{
			$p = new $plugin('', $this->db, $this);
			if ( $this->langIf )
				foreach ($p->getHandles() as $handle)
					$this->langIf->getText($handle);
		}

		// reset failures so we can run validate multiple times
		$this->failures = array();

		//echo '<pre>';
		//echo print_r($this->_elements, true);

		foreach($this->_elements as $name => $attr)
			$this->_behave($name, $this->getValue($name), $attr['behaviour']);

		$failures = count($this->_failures);
		$this->debug && debug('Finished validation.. number of failures: '.$failures);
		return ($failures == 0);

	}

	function validateWithExceptions()
	{
		if (!$this->validate())
			throw new FormBehaviourException($this->getFailures());
	}

	/**#@+
	 * @access protected
	 */
	/**
	 * Behave pls
	 * @param mixed $value
	 * @param string $behaviour
	 * @return string $rv
	 */
	function _behave($input_name, $value, $behaviour)
	{
		$this->debug && debug("\n\n".'FormBehaviour - behaving for '.$input_name.' with value '.$value.' and behaviour '.$behaviour);

		$rv = '';
		@$attr = explode(';',$behaviour);
		foreach($attr as $functionCall)
		{
			$reg = '/([^(]*)(?:\((.*)\))?/';
			preg_match($reg, $functionCall, $matches);
			$behaviour = $matches[1];

			if (empty($behaviour))
			{
				$this->debug && debug(' - no behaviour for you, bailing out');
				continue;
			}

			if (!isset($this->_plugins[$behaviour]))
				info("No plugin found for '$functionCall'");
			else if (!class_exists($this->_plugins[$behaviour]))
				trigger_error('Form behaviour plugin does not exist: '.$this->_plugins[$functionCall], E_USER_NOTICE);
			else
			{
				if ($value === '' && $behaviour != 'required')
					continue;

				$this->debug && debug(sprintf('Formbehaviour: new %s with value [%s]', $this->_plugins[$behaviour], $value));
				$inst = new $this->_plugins[$behaviour]($value, $this->db, $this, $input_name);
				$inst->debug = $this->debug;

				if (isset($matches[2]))
				{
					$args = "'".implode("','",explode(',', $matches[2]))."'";
					$e = '$inst->check('.$args.');';
					eval($e);
				}
				else
					$inst->check();

				$status = $inst->getStatus();

				if ($status == FB_STATUS_INVALID)
				{
					$this->debug && debug(' - marking status as INVALID');
					$this->_failures[$input_name] = (is_null($this->langIf)) ?
						$inst->getMessage() : $this->langIf->getText($inst->getMessage());
				}
				else if ($status == FB_STATUS_VALID)
					$this->debug && debug(' - marking status as VALID');
				else
					$this->debug && debug(' - marking status as '.$status);

			}
		}
	}

	/**
	 * clears failure for an element_name
	 */
	function clearFailure($element_name)
	{
		unset($this->_failures[$element_name]);
	}

	/**
	 * unsets all failures
	 */
	function clearFailures()
	{
		$this->_failures = array();
	}

	public function setFailures($failures)
	{
		$this->_failures = $failures;
	}

	/**
	 * Extracts the value from postdata for a name (info_person[email_address])
	 * @static
	 * @param string $name;
	 * @todo replace with inputStrToArr
	 */
	public function getValue($name)
	{
		$arrValues = $this->arrValues;

		$value = '';
		if(false===strpos($name,'[')&&isset($arrValues[$name]))
			$value = $arrValues[$name];
		elseif(false!==strpos($name,'[]'))
		{
			$postVar = "['".implode("']['", explode('[',str_replace(']','',str_replace('[]','',$name))))."']";
			$test = '';
			eval('$test = isset($arrValues'.$postVar.')?$arrValues'.$postVar.':"";');
			if($test !== '')
				$value = $test;
		}
		else
		{
			// The name is an array, eval it and check Post data
			$postVar = "['".implode("']['", explode('[',str_replace(']','',$name)))."']";
			$test = '';
			eval('$test = isset($arrValues'.$postVar.')?$arrValues'.$postVar.':"";');
			if($test !== '')
				$value = $test;
		}

		$this->debug && debug('FormBehavour - getValue() value for '.$name.'.: '.$value);

		return $value;
	}

	/**
	 * Returns all registred values, like POST. So _never_ access POST directy from plugins but use this function.
	 * This way all plugins can be used in other places as well (like soap)
	 */
	public function getArrValues()
	{
		return $this->arrValues;
	}

	/**#@-*/
	/**
	 * Register an element with a behaviour
	 * @param string $name
	 * @param string $behaviour
	 * @param string displayname
	 */
	function register($name, $behaviour, $displayname = '')
	{
		$this->debug && debug(sprintf('FormBehaviour::register() - name [%s], behaviour [%s], displayname [%s]', $name, $behaviour, $displayname));

		if(empty($name))
			return;

		$this->_elements[$name]['behaviour'] = $behaviour;
		$this->_elements[$name]['displayname'] = $displayname ? $displayname:'';
	}

	function appendBehaviour($name, $behaviour)
	{
		if(!isset($this->_elements[$name]))
			return $this->register($name, $behaviour);
		$this->_elements[$name]['behaviour'] .= ';'.$behaviour;
	}

	/**
	 *
	 */
	function unregister($name,$behaviour='')
	{
		//debug(sprintf('FormBehaviour::unregister() - [%s], [%s], [%s]', $name, $behaviour));

		if(!$behaviour)
		{
			unset($this->_elements[$name]);
			return;
		}

		$behaviours = explode(';',$this->_elements[$name]['behaviour']);
		$behaviours = array_flip($behaviours);
		unset($behaviours[$behaviour]);
		$behaviours = array_flip($behaviours);
		$this->_elements[$name]['behaviour'] = implode(';',$behaviours);
	}
	/**
	 *
	 */
	function registerAll($all=array())
	{

	}
	/**
	 * @return String $rv
	 */
	function getJavaScriptFailures()
	{
		$rv = "addEvent(window,'load',
			function()
			{\n";
		foreach($this->_failures as $name => $error)
			$rv .= "try{
				fb.displayError(document.getElementsByName('".$name."')[0],'".jsescape($error, "'")."');
				}
				catch(e){}\n";
		$rv .= "\n});\n";
		return $rv;
	}

	/**
	 * @return String $rv
	 */
	function getJavaScriptFailuresInline()
	{
		$rv = "";
		foreach($this->_failures as $name => $error)
			$rv .= "try{
				fb.displayError(document.getElementsByName('".$name."')[0],'".jsescape($error, "'")."');
				}
				catch(e){}\n";
		return $rv;
	}

	/**
	 * returns all failures
	 * @return array<array(id=>name)>
	 */
	function getFailures()
	{
		return $this->_failures;
	}

	/**
	 * returns the first failure as a string
	 * @return array(id=>name)
	 * @deprecated use exceptions
	 */
	function getFailure()
	{
		if (sizeof($this->_failures) == 0)
			return false;

		return key($this->_failures) . ': ' .array_pop($this->_failures);
	}

	/**#@-*/
	/**
	 * For external usage of protected functions
	 */
	function callMethod($method,$value,$attr='')
	{
		$rv = '';
		if(method_exists($this,'_'.$method))
			eval('$rv = $this->_'.$method.'('.$value.($attr?','.$attr:'').');');
		return $rv;
	}

	function getBehaviour($name)
	{
		if(isset($this->_elements[$name]['behaviour']))
			return $this->_elements[$name]['behaviour'];
		return '';
	}
}

class FormBehaviourPlugin
{
	var $_status = FB_STATUS_INVALID;
	var $_handles = array();
	var $_message = '';
	var $debug = false;
	var $_value;

	protected $fb; // reference to formbehaviour
	protected $input_name; // string form input name

	function FormBehaviourPlugin($value, $db=null, $fb, $input_name=null)
	{
		$this->_value = $value;
		$this->_db = $db;
		$this->fb = $fb;
		$this->input_name = $input_name;
	}

	function check()
	{
		$this->_status = FB_STATUS_VALID;
	}

	function getStatus()
	{
		return $this->_status;
	}

	function getMessage()
	{
		return $this->_message;
	}

	function getHandles()
	{
		$handles = $this->_handles;
		$handles[] = $this->getMessage();
		$handles = array_unique($handles);

		return $handles;
	}

}

class FormBehaviourPluginRegex extends FormBehaviourPlugin
{
	var $_trim = false;
	function check()
	{
		$this->_status = preg_match($this->_regex, $this->_trim ? trim($this->_value) : $this->_value) ? FB_STATUS_VALID : FB_STATUS_INVALID;
	}
}

class FormBehaviourPluginYear extends FormBehaviourPluginRegex
{
	var $_regex = '/^([0-9]){4}$/';
	var $_message = 'Not a valid year';
}

class FormBehaviourPluginMonth extends FormBehaviourPluginRegex
{
	var $_regex = '/^(1[0-2]|[1-9])$/';
	var $_message = 'Not a valid month';
}

class FormBehaviourPluginDay extends FormBehaviourPluginRegex
{
	var $_regex = '/([1-9]|[1-2][0-9]|3[01])/';
	var $_message = 'Not a valid day';
}

class FormBehaviourPluginPhone extends FormBehaviourPluginRegex
{
	var $_regex = '/^\+?(?:[\-0-9\s#*]|\(\d+\))+$/';
	var $_message = 'Not a valid phone or fax number';
	var $_trim = true;
}

class FormBehaviourPluginAlpha extends FormBehaviourPluginRegex
{
	var $_regex = '/^[a-z]+$/i';
	var $_message = 'Only letters are valid';
}

class FormBehaviourPluginNumeric extends FormBehaviourPluginRegex
{
	var $_regex = '/^[0-9]+$/';
	var $_message = 'Only numbers are valid';
}

class FormBehaviourPluginAlphaNumeric extends FormBehaviourPluginRegex
{
	var $_regex = '/^[\pL\d]+$/';
	var $_message = 'Only numbers and letters are valid';
}

class FormBehaviourPluginUrl extends FormBehaviourPlugin
{
	var $_message = 'Not a valid URL';

	function check()
	{
		$this->_status = FB_STATUS_INVALID;
		$value = $this->_value;

		if(filter_var($value, FILTER_VALIDATE_URL) === false)
			return;
		$this->_status = FB_STATUS_VALID;
	}
}

class FormBehaviourPluginEmail extends FormBehaviourPlugin
{
	var $_message = 'Not a valid Email';
	function check()
	{
		$this->_status = FB_STATUS_INVALID;
		$value = $this->_value;

		if(filter_var($value, FILTER_VALIDATE_EMAIL) === false)
			return;

		$this->_status = FB_STATUS_VALID;
	}
}

class FormBehaviourPluginRequired extends FormBehaviourPlugin
{
	var $_message = 'Field is required';

	/**
	 * @param string $dependentName Name of Element on which required is based on.
	 *               If dependant name does not have a value, this element is required.
	 * @param string $empty 'Boolean' in case this is 'true' element is required if dependName has a value
	 * @param string $dependantValue The element only is required in case the dependentName.value
	 *               is in dependentValue ie 'braat|aap|turu' Element is not required if dependentName.value = kwak
	 */
	function check($dependentName='', $empty='false', $dependentValue=false)
	{

		$this->debug && debug(sprintf("FormBehaviourPluginRequired:: check(depenentName='%s', empty='%s', dependentValue='%s')", $dependentName, $empty, $dependentValue));
		$this->debug && debug(sprintf(' - value: '.$this->_value));

		$this->_status = FB_STATUS_INVALID;
		if (is_string($this->_value))
			$this->_value = trim($this->_value);

		if ($empty == 'false')
			$empty = false;

		if($dependentName)
		{
			$val = $this->fb->getValue($dependentName);
			$this->debug && debug(' - dependentName value: '.$val);

			$fieldNoValue = ($this->_value==='');
			$this->debug && debug('fieldNoValue='.$fieldNoValue);

			if($dependentValue)
			{
				$dependentValue = explode('|', $dependentValue);
				if(in_array($val, $dependentValue) && $fieldNoValue)
				{
					$this->debug && debug(sprintf(' - dependentValue and fieldNoValue is set but val [%s] is not in dependentValues [%s], returning invalid.', $val, print_r($dependentValue, true)));
					return;
				}

				$this->debug && debug(sprintf(' - dependentValue is set, returning valid '));

				$this->_status = FB_STATUS_VALID;
				return;
			}

			// normal
			if((!$empty)&&empty($val)&& $fieldNoValue)
				return;

			//
			if ($empty && empty($val))
			{
				$this->_status = FB_STATUS_VALID;
				return;
			}

			if ($empty && $val && $fieldNoValue)
				return;

			$this->_status = FB_STATUS_VALID;
		}
		elseif($this->_value==='')
			return;

		$this->_status = FB_STATUS_VALID;
	}
}

class FormBehaviourPluginDate extends FormBehaviourPlugin
{
	var $_message = 'Not a valid date';
	var $_handles = array('Invalid format', 'Invalid date');

	function check($format='DD-MM-YYYY')
	{
		switch ($format)
		{
			case 'DD-MM-YYYY':
				$DATE_FORMAT = '/(\d{2})\-(\d{2})\-(\d{4})/';
				break;

			case 'YYYY-MM-DD':
				$DATE_FORMAT = '/(\d{4})\-(\d{2})\-(\d{2})/';
				break;

			default:
				throw new IWException('FormBehaviourPluginDate::check() - invalid format: '.$format);
		}

		if (preg_match($DATE_FORMAT, $this->_value, $matches))
		{
			// extract pieces of date and save them so we can compare later
			$v_pieces = explode('-', $this->_value);
			$f_pieces = array_flip(explode('-', $format));
			$year = $v_pieces[ $f_pieces['YYYY'] ];
			$month = $v_pieces[ $f_pieces['MM'] ];
			$day = $v_pieces[ $f_pieces['DD'] ];

			// make a date object from the value and format
			$date = createDate($this->_value, $format);

			// see if they match
			if ( $date->getYear() == $year &&
				strlen($year) == 4 &&
				$date->getMonth() == $month &&
				$date->getDay() == $day )
			{
				$this->_status = FB_STATUS_VALID;
			}
			else
			{
				$this->_status = FB_STATUS_INVALID;
			}
		}
		else
		{
			$this->_message = $this->_handles[0];
			$this->_status = FB_STATUS_INVALID;
		}
	}
}

/**
 * generic class for checking strings. can be extended if other rules need to be enforced
 */
class FormBehaviourPluginString extends FormBehaviourPlugin
{
	public $_message = '';
	public $_handles = array('Not a valid string', 'String too short', 'String too long');
	protected $_regex = '/^[a-zA-Z0-9 ]+$/'; // only letters, digits and spaces are allowed, no other characters
	protected $_trim = true;

	function check($min=null, $max=null)
	{
		$this->_value = $this->_trim ? trim($this->_value) : $this->_value;

		$this->_status = FB_STATUS_INVALID;

		if (!is_null($min) && strlen($this->_value) < $min){
			$this->_message = $this->_handles[1];
			return;
		}
		if (!is_null($max) && strlen($this->_value) > $max){
			$this->_message = $this->_handles[2];
			return;
		}

		// always check for UTF8 strings
		$this->_regex = $this->_regex.'u';

		$this->_status = preg_match($this->_regex, $this->_value) ? FB_STATUS_VALID : FB_STATUS_INVALID;
		$this->_message = $this->_handles[0];
	}
}

/**
 * plugin for checking company names
 */
class FormBehaviourPluginCompanyName extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid company name', 'Company name too short', 'Company name too long');
	protected $_regex = '/^[\d\pL- +,.&\(\)\/"\'*]{1,255}$/'; // utf8 letters, digits, spaces, ,.-()+
}

/**
 * Plugin used for validating person names
 */
class FormBehaviourPluginPersonName extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid name', 'Name too short', 'Name too long');
	protected $_regex = '/^[\d\pL- .,\/*\(\)&]+$/'; // digits, only utf8 letters, space, -./,()&
}

/**
 * Plugin used for validating city names
 */
class FormBehaviourPluginCityName extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid city name', 'City name too short', 'City name too long');
	protected $_regex = '/^[\d\pL- ,.\(\)]+$/'; // digits, only utf8 letters, space, -().,
}

/**
 * Plugin used for validating brand names
 */
class FormBehaviourPluginBrand extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid brand name', 'Brand name too short', 'Brand name too long');
	protected $_regex = '/^[\pL\d .,\-\(\)\'&\/]+$/'; // only letters, digits and spaces are allowed, no other characters
}

/**
 * Plugin used for validating addresses
 */
class FormBehaviourPluginAddress extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid address', 'Address too short', 'Address too long');
	protected $_regex = '/^[\d\pL- \',.\(\)\/&]+$/'; // utf8 letters, digits, spaces, ()/&'.,-
}

/**
 * Plugin used for validating zip-codes
 */
class FormBehaviourPluginZipCode extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid postal code', 'Postal code too short', 'Postal code too long');
	protected $_regex = '/^[a-zA-Z0-9 -]+$/'; // only letters, digits, space, hyphen
}

/**
 * Plugin used for validating swift-bic codes
 */
class FormBehaviourPluginSwiftCode extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid SWIFT code', 'SWIFT code too short', 'SWIFT code too long');
	protected $_regex = '/^[a-zA-Z0-9 \/]{1,12}$/'; // only letters, digits, space
}

/**
 * Plugin used for validating bankaccount numbers
 */
class FormBehaviourPluginBankAccount extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid bankaccount', 'Bankaccount too short', 'Bankaccount too long');
	protected $_regex = '/^[\d\pL .\-\/#]+$/'; // only letters, digits, space, dot, hyphen
}

/**
 * Plugin used for validating post office numbers
 */
class FormBehaviourPluginPONumber extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid PO no.', 'PO no. too short', 'PO no. too long');
	protected $_regex = '/^[a-zA-Z0-9 ]+$/'; // only letters, digits, space
}

/**
 * Plugin used for validating department names
 */
class FormBehaviourPluginDepartmentName extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid department name', 'Name too short', 'Name too long');
	protected $_regex = '/^[\d\pL- ,.\/&\(\)]+$/'; // only utf8 letters, digits, space, ,.-/
}

/**
 * Plugin used for validating questions
 */
class FormBehaviourPluginQuestion extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid question', 'Question too short', 'Question too long');
	protected $_regex = '/^[\d\pL- ,`.?\'\"\/]+$/'; // only utf8 letters, digits, space, comma, dot, hyphen, question mark
}

/**
 * Plugin used for validating email subject
 */
class FormBehaviourPluginEmailSubject extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid email subject', 'Subject too short', 'Subject too long');
	protected $_regex = '/^[\d\pL- ,`.?@:;!\'\/\{\}_\(\)]+$/'; // only utf8 letters, digits, space, some other chars...
}

/**
 * Plugin used for validating product registration numbers
 */
class FormBehaviourPluginProdRegistrationNo extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid producer reg. no.', 'No. too short', 'No. too long');
	protected $_regex = '#^[a-zA-Z0-9- \\\/]+$#'; // only letters, digits, space, hyphen, /, \
}

/**
 * Plugin used for validating comments
 */
class FormBehaviourPluginComment extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid comment', 'Comment too short', 'Comment too long');
	protected $_regex = '/^[\d\pL- ,.?:;!]+$/'; // only utf8 letters, digits, space, some other chars...
}

/**
 * Plugin used for validating cron masks
 */
class FormBehaviourPluginCronMask extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid cron mask', 'Cron mask too short', 'Cron mask too long');
	protected $_regex = '/^([\d *,-])+$/'; // only digits, space, comma, hyphen, asterix
}

/**
 * Plugin used for validating GL Accounts
 */
class FormBehaviourPluginGLAccounts extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid value', 'Value too short', 'Value too long');
	protected $_regex = '/^([A-Z0-9_])+$/'; // only digits, uppercase and underscore
}

/**
 * Plugin used for validating Product Categories
 */
class FormBehaviourPluginProductCategory extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid product category', 'Value too short', 'Value too long');
	protected $_regex = '/^([\d\pL .-_,])+$/'; // UTF8 chars, space, _,.-
}

/**
 * Plugin used for validating Product Components
 */
class FormBehaviourPluginProductComponent extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid product component', 'Value too short', 'Value too long');
	protected $_regex = '/^([\d\pL .-_,])+$/'; // UTF8 chars, space, _,.-
}

/**
 * Plugin used for validating Invoice Numbers
 */
class FormBehaviourPluginInvoiceNumber extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid invoice number', 'Invoice number too short', 'Invoice number too long');
	protected $_regex = '/^([A-Za-z0-9 \/\\\])+$/'; // letters, numbers, slash and space
}

/**
 * Plugin used for validating ATF aproval no
 */
class FormBehaviourPluginApprovalNoATF extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid ATF approval number', 'Too short', 'Too long');
	protected $_regex = '/^((WEE\/)[A-Z]{2}[0-9]{4}[A-Z]{2})(\/ATF)?$/';
}

/**
 * Plugin used for validating AE aproval no
 */
class FormBehaviourPluginApprovalNoAE extends FormBehaviourPluginString
{
	public $_handles = array('Not a valid AE approval number', 'Too short', 'Too long');
	protected $_regex = '/^((WEE\/)[A-Z]{2}[0-9]{4}[A-Z]{2})(\/AE|\/EXP)?$/';
}

class FormBehaviourPluginVAT extends FormBehaviourPlugin
{
	public $_message = '';
	public $_handles = array();
	//protected $_regex = '/^[a-zA-Z0-9 ]+$/'; // only letters and digits are allowed, no spaces and other characters
	protected $_regex = '/^([A-Z]{2})([0-9A-Z]{2,12})$/'; // only capitals and numbers
	protected $_trim = true;

	function check($checkUrl=null)
	{
		$this->_status = preg_match($this->_regex, $this->_trim ? trim($this->_value) : $this->_value) ? FB_STATUS_UNDECIDED : FB_STATUS_INVALID;

		if ($this->_status == FB_STATUS_UNDECIDED) {
			// check using the site, because we didn't find any unauthorised characters
			require_once(IWARE.'VAT.php');
			require_once(IWARE.'VATChecker.php');
			$this->vatChecker = new VATChecker($checkUrl);
			$this->_handles = $this->vatChecker->getErrors();

			if ($this->vatChecker->check($this->_value))
				return $this->_status = FB_STATUS_VALID;

			$this->_message = $this->vatChecker->getLastError();
			$undecided[] = 'Unable to connect to the VAT number validation site';
			$undecided[] = 'Internal error';
			$this->_status = in_array($this->_message, $undecided)?FB_STATUS_UNDECIDED:FB_STATUS_INVALID;
		} else {
			// it failed, because we have unauthorized characters in the VAT (only A-Z, 0-9 are allowed)
			$this->_message = "Invalid VAT format";
		}

		if ($this->_status == FB_STATUS_UNDECIDED)
		{
			$vat = new VAT($this->_db, array('vat_no'=>$this->_value), true);

			if ($vat->isComplete())
			{
				$status = (int) $vat->status;

				if ($status == VAT_INVALID) {
					$this->_status = FB_STATUS_INVALID;
					$this->_message = $this->vatChecker->getError(0);
				}
				else if ($status == VAT_PENDING)
					$this->_status = FB_STATUS_UNDECIDED;
				else if ($status == VAT_VALID)
					$this->_status = FB_STATUS_VALID;
			}
			else
			{
				$vat->status = VAT_PENDING;
				$vat->insert();
			}
		}
	}
}

class FormBehaviourPluginIBAN extends FormBehaviourPlugin
{
	var $_message = '';
	var $_handles = array();

	function FormBehaviourPluginIBAN($value, $db=null, $fb)
	{
		parent::FormBehaviourPlugin($value, $db, $fb);

		require_once(IWARE.'IBAN.php');
		$this->checker = new IBAN($value);

		$errorCodes = array (1,2,3,4,5,6,7,8);

		foreach ($errorCodes as $errorCode)
			$this->_handles[] = $this->checker->errorMessage($errorCode);
	}

	function check()
	{
		if ($this->checker->validate())
			return $this->_status = FB_STATUS_VALID;

		$this->_message = $this->checker->errorMessage($this->checker->getErrorcode());
		$this->_status = FB_STATUS_INVALID;
	}

}
/**
 *
 */
class FormBehaviourPluginDateNotInPast extends FormBehaviourPlugin
{
	var $_message = 'Date cannot be in the past';

	function check()
	{
		$this->_status = FB_STATUS_INVALID;

		$pieces = explode('-', $this->_value);
		if (sizeof($pieces) != 3) {
			trigger_error('Invalid date: ' . $this->_value, E_USER_NOTICE);
			return;
		}

		$year = $pieces[2];
		$month = $pieces[1];
		$day = $pieces[0];

	     //return int 0 if the dates are equal, -1 if d1 is before d2, 1 if d1 is after d2
		if (IWDateTime::compare(currentDate(), $start_date = new IWDateTime($year.'-'.$month.'-'.$day)) == -1)
			$this->_status = FB_STATUS_VALID;
	}
}

/**
 *
 */
class FormBehaviourPluginInteger extends FormBehaviourPlugin
{
	var $_message = '';
	var $_handles = array('Not a valid integer', 'Integer is not in valid range');

	function check($min=null, $max=null)
	{
		$this->_status = FB_STATUS_INVALID;
		$value = $this->_value;

		$min = ($min === 'null' ? null : $min); //hack for backwards compat
		$max = ($max === 'null' ? null : $max);

		if(filter_var($value, FILTER_VALIDATE_INT) === false)
		{
			$this->_message = $this->_handles[0];
			return;
		}
		if(!is_null($min) || !is_null($max))
		{
			$options = array('options'=>array());
			if(!is_null($min))
				$options['options']['min_range'] = $min;
			if(!is_null($max))
				$options['options']['max_range'] = $max;

			if(filter_var($value, FILTER_VALIDATE_INT, $options) === false)
			{
				$this->_message = $this->_handles[1];
				return;
			}
		}

		$this->_status = FB_STATUS_VALID;
	}
}

/**
 * Tries to get a document and returns the message on failure
 */
class FormBehaviourPluginDocument extends FormBehaviourPlugin
{
        protected $document = null;

        function check()
	{
		$langIf = IWRegistry::registry('langIf'); // This assumes LANGIF is available so get it from the Registry
		require_once(INTERFACE_OBJ.'InterfaceItem.php');
		$item = new InterfaceItem($this->_db, $this->_db, $langIf);
		try {
			$this->document = $item->getDocument($this->input_name, false, false);
		}
		catch (IWFileUploadException $e)
		{
			$this->_message = $e->getMessage();
			$this->_status = FB_STATUS_INVALID;
			return;
		}

		$this->_status = FB_STATUS_VALID;
	}
}

class FormBehaviourPluginLength extends FormBehaviourPlugin
{
	var $_handles = array('Value too short', 'Value too long');

	function check($minlength=0, $maxlength=null)
	{
		if (mb_strlen($this->_value) < $minlength)
		{
			$this->_message = $this->_handles[0];
			$this->_status = FB_STATUS_INVALID;
			return;
		}

		if ($maxlength && (mb_strlen($this->_value) > $maxlength))
		{
			$this->_message = $this->_handles[1];
			$this->_status = FB_STATUS_INVALID;
			return;
		}

		$this->_status = FB_STATUS_VALID;
	}
}


class FormBehaviourPluginFloat extends FormBehaviourPlugin
{
	var $_message = 'Value is not a decimal number';

	function check($decimal_sep='.')
	{
		$this->_status = FB_STATUS_INVALID;
		$value = $this->_value;

		if(filter_var($value, FILTER_VALIDATE_FLOAT,
			array('options'=>array('decimal' => $decimal_sep))) === false)
		{
			return;
		}

		$this->_status = FB_STATUS_VALID;
	}
}

/**
 * Plugin used for validating a product fee
 */
class FormBehaviourPluginProductFee extends FormBehaviourPluginFloat
{
	var $_handles = array(
		'The product fee is too large',
		'The product fee must be a positive number'
	);

	function check($decimal_sep='.')
	{
		parent::check($decimal_sep);
		if ($this->_status == FB_STATUS_INVALID)
		{
			return;
		}

		if ($this->_value > MAX_PRODUCT_FEE_VALUE)
		{
			$this->_status = FB_STATUS_INVALID;
			$this->_message = $this->_handles[0];
			return;
		}

		if ($this->_value < 0)
		{
			$this->_status = FB_STATUS_INVALID;
			$this->_message = $this->_handles[1];
			return;
		}

		$this->_status = FB_STATUS_VALID;
	}
}

class FormBehaviourPluginIP extends FormBehaviourPlugin
{
	var $_message = 'Value is not a valid/allowed IP address';

	function check($allowPrivate=false, $allowReserved=false, $allowIPv6=false)
	{
		$this->_status = FB_STATUS_INVALID;
		$value = $this->_value;

		$options = 0;
		if(!$allowPrivate)
			$options = $options | FILTER_FLAG_NO_PRIV_RANGE;
		if(!$allowReserved)
			$options = $options | FILTER_FLAG_NO_RES_RANGE;
		if(!$allowIPv6)
			$options = $options | FILTER_FLAG_IPV4;

		if(filter_var($value, FILTER_VALIDATE_IP, $options) === false)
			return;

		$this->_status = FB_STATUS_VALID;
	}
}

class FormBehaviourPluginIPWildcard extends FormBehaviourPlugin
{
	var $_message = 'Value is not a valid/allowed IP address';

	function check()
	{
		$this->_status = FB_STATUS_INVALID;
		$value = $this->_value;

		$pattern = '/^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(?:(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(?:(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)?)?)?$/';
		if(!preg_match($pattern,$value))
			return;
		$this->_status = FB_STATUS_VALID;
	}
}

/**
 * FormBehaviourException
 *
 * Can be trown from $fb->validateWithExceptions
 */
class FormBehaviourException extends IWException
{
	private $failures;

	function __construct(Array $failures)
	{
		$message = "Please correct the following error(s): \n";
		foreach ($failures as $field => $error)
			$message .= sprintf(" - %s: %s \n", $field, $error);

		$this->failures = $failures;

		parent::__construct($message, 0);
	}

	function getFailures()
	{
		return $this->failures;
	}

};

?>
