<?php
/**
 * @package I-Ware
 * @subpackage Security
 * @since 22-jun-2006
 * @author Chris Wesseling <chris.wesseling@xs4all.nl>
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class GPGCrypter
{
	/**
	 * @access private
	 * @var #Resource $__res
	 */
	var $__res;
	/**
	 * @access private
	 * @var string $__fingerprint
	 */
	var $__fingerprint;
	/**
	 * Initializes the GPGCryter
	 * @param string $key
	 */
	function GPGCrypter($fingerprint='',$home='')
	{
		$fingerprint || $fingerprint = GPG_FINGERPRINT;
		$home || $home = GPG_HOME;

		$this->init($fingerprint,$home);
	}
	/**
	 * Init the Gnupg resource and add the fingerprint
	 * @param string $fingerprint
	 * @param string $home /location/to/.gpg/file
	 */
	function init($fingerprint,$home)
	{
		putenv('GNUPGHOME='.$home);
		$this->setKey($fingerprint);
		$this->__res = gnupg_init();
		gnupg_addsignkey($this->__res,$this->__fingerprint,'');
	}
	/**
	 * Sets, or re-sets the fingerprint
	 * @param string $fingerprint
	 */
	function setKey($fingerprint)
	{
		$this->__fingerprint = $fingerprint;
	}
	/**
	 * Signs a string and returns the fully signed string
	 * @param string $string
	 */
	function sign($string)
	{
		return gnupg_sign($this->__res, $string);
	}
	/**
	 * Decrypts a signed string and returns the result if the validity is larger then Marginal
	 * @param string $signed_string
	 * @param string $signature
	 */
	function decrypt($signed_string, $signature)
	{
		$signature = '-----BEGIN PGP SIGNATURE-----
Version: GnuPG v1.4.1 (GNU/Linux)

'.$signature.'
-----END PGP SIGNATURE-----';
		$plain = '';
		if(!($info = gnupg_verify($this->__res, $signed_string, $signature, $plain)))
			return false;
		if($info[0]['validity']<=GNUPG_VALIDITY_MARGINAL)
			return false;
		return $plain;
	}
	/**
	 * Synonym for `sign`
	 * @param string $signed_string
	 */
	function encrypt($s)
	{
		return $this->sign($s);
	}
}
?>
