<?php
/**
 * @package IWare
 * @since 16-aug-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class HTTPAuthenticate
{
	protected $realm = 'Test Realm';
	protected $groups = array();
	protected $authenticationType = 'Basic';

	function __construct($db,$groups)
	{
		if(empty($groups)||!is_array($groups))
			return;

		$this->db = $db;
		$this->groups = array();
		foreach($groups as $define)
			defined($define) && $this->groups[] = (int)constant($define);

		if(!$this->{'authenticate'.$this->authenticationType}())
		{
			header('HTTP/1.1 403 Forbidden');
			exit('Permission Denied');
		}
	}

	public function setAuthenticationType($type)
	{
		$this->authenticationType = $type;
	}

	protected function authenticateBasic()
	{
		if(empty($_SERVER['PHP_AUTH_USER']))
		{
			header('HTTP/1.1 401 Unauthorized');
			header(
				sprintf('WWW-Authenticate: Basic realm="%s"',
					$this->realm)
				);
			die('Unauthorized Access');
		}

		$users = $this->get_valid_users();

		if(!array_key_exists($_SERVER['PHP_AUTH_USER'],$users))
			return false;

		list($passwd,$salt) = $users[$_SERVER['PHP_AUTH_USER']];

		if (CRYPT_BLOWFISH == 1)
			$password = crypt($_SERVER['PHP_AUTH_PW'],$salt);
		else
			$password = sha1($_SERVER['PHP_AUTH_PW']);

		if($passwd != $password)
			return false;

		return true;
	}

	protected function authenticateDigest()
	{
		if(empty($_SERVER['PHP_AUTH_DIGEST']))
		{
			header('HTTP/1.1 401 Unauthorized');
			header(
				sprintf('WWW-Authenticate: Basic realm="%s",qop="auth",nonce="%s",opaque="%s"',
					$this->realm,
					uniqid(),
					md5($this->realm)
				));
			die('Unauthorized Access');
		}

		$users = $this->get_valid_users();
		if(!($data = $this->http_digest_parse($_SERVER['PHP_AUTH_DIGEST'])))
			return false;
		if(!isset($users[$data['username']]))
			return false;

		$A1 = md5($data['username'] . ':' . $this->realm . ':' . $users[$data['username']]);
		$A2 = md5($_SERVER['REQUEST_METHOD'].':'.$data['uri']);
		$valid_response = md5($A1.':'.$data['nonce'].':'.$data['nc'].':'.$data['cnonce'].':'.$data['qop'].':'.$A2);

		if ($data['response'] != $valid_response)
			return false;
		return true;
	}

	protected function get_valid_users()
	{
		require_once(IWARE.'encryption.php');
		$crypter = new Crypter(ENCRYPTION_KEY);

		$sql = "SELECT U.USER_NAME, U.USER_PASSWORD, U.USER_SALT
			FROM TBL_USER as U
			INNER JOIN TBL_GROUP_USER as GU
				ON U.USER_ID = GU.USER_ID
			WHERE GU.GROUP_ID IN (".implode(',',$this->groups).")";
		$q = $this->db->query($sql);
		$q->execute();
		$rows = $q->fetchAll();
		$rv = array();
		foreach($rows as $r)
		{
			if($this->authenticationType=='Digest')
				$rv[$r->USER_NAME] = $crypter->decrypt($r->USER_PASSWORD);
			else
				$rv[$r->USER_NAME] = array($r->USER_PASSWORD,$r->USER_SALT);
		}
		return $rv;
	}

	protected function http_digest_parse($txt)
	{
		// protect against missing data
		$needed_parts = array('nonce'=>1, 'nc'=>1, 'cnonce'=>1, 'qop'=>1, 'username'=>1, 'uri'=>1, 'response'=>1);
		$data = array();

		preg_match_all('@(\w+)=[\'"]?(.*)[\'"]?(,|$)@U', $txt, $matches, PREG_SET_ORDER);

		foreach ($matches as $m) {
		    $data[$m[1]] = ($m[2]{0}=='"') ? substr($m[2],1) : $m[2];
		    unset($needed_parts[$m[1]]);
		}

		return $needed_parts ? false : $data;
	}

	public static function readHTTPDigestAuthenticatedFile($host,$file,$username,$password)
	{
	    if (!$fp=fsockopen($host,80, $errno, $errstr, 15))
	        return false;

	    //first do the non-authenticated header so that the server
	    //sends back a 401 error containing its nonce and opaque
	    $out = "GET /$file HTTP/1.1\r\n";
	       $out .= "Host: $host\r\n";
	       $out .= "Connection: Close\r\n\r\n";

	     fwrite($fp, $out);

	    //read the reply and look for the WWW-Authenticate element
	    while (!feof($fp))
	    {
	        $line=fgets($fp, 512);

	        if (strpos($line,"WWW-Authenticate:")!==false)
	            $authline=trim(substr($line,18));
	    }

	    fclose($fp);

	    //split up the WWW-Authenticate string to find digest-realm,nonce and opaque values
	    //if qop value is presented as a comma-seperated list (e.g auth,auth-int) then it won't be retrieved correctly
	    //but that doesn't matter because going to use 'auth' anyway
	    $authlinearr=explode(",",$authline);
	    $autharr=array();

	    foreach ($authlinearr as $el)
	    {
	        $elarr=explode("=",$el);
	        //the substr here is used to remove the double quotes from the values
	        $autharr[trim($elarr[0])]=substr($elarr[1],1,strlen($elarr[1])-2);
	    }

	    foreach ($autharr as $k=>$v)
	        echo("$k ==> $v\r\n");

	    //these are all the vals required from the server
	    $nonce=$autharr['nonce'];
	    $opaque=$autharr['opaque'];
	    $drealm=$autharr['Digest realm'];

	    //client nonce can be anything since this authentication session is not going to be persistent
	    //likewise for the cookie - just call it MyCookie
	    $cnonce="sausages";

	    //calculate the hashes of A1 and A2 as described in RFC 2617
	    $a1="$username:$drealm:$password";$a2="GET:/$file";
	    $ha1=md5($a1);$ha2=md5($a2);

	    //calculate the response hash as described in RFC 2617
	    $concat = $ha1.':'.$nonce.':00000001:'.$cnonce.':auth:'.$ha2;
	    $response=md5($concat);

	    //put together the Authorization Request Header
	    $out = "GET /$file HTTP/1.1\r\n";
	       $out .= "Host: $host\r\n";
	    $out .= "Connection: Close\r\n";
	    $out .= "Cookie: cookie=MyCookie\r\n";
	    $out .= "Authorization: Digest username=\"$username\", realm=\"$drealm\", qop=\"auth\", algorithm=\"MD5\", uri=\"/$file\", nonce=\"$nonce\", nc=00000001, cnonce=\"$cnonce\", opaque=\"$opaque\", response=\"$response\"\r\n\r\n";

	    if (!$fp=fsockopen($host,80, $errno, $errstr, 15))
	        return false;

	    fwrite($fp, $out);

	    //read in a string which is the contents of the required file
	    while (!feof($fp))
	    {
	        $str.=fgets($fp, 512);
	    }

	    fclose($fp);

	    return $str;
	}
}
?>