<?php

// For new countries use:
// http://en.wikipedia.org/wiki/IBAN

class IBAN
{
    /**
     * String containing the IBAN to be processed
     * @var     string
     */
    var $_iban = '';

    /**
     * Integer containing errorcode of last validation
     * @var     integer
     */
    var $_errorcode = 0;

    static protected $_iban_countrycode_countryname = array(
                    'AD' => 'Andorra',
                    'AT' => 'Austria',
                    'BE' => 'Belgium',
                    'CH' => 'Swiss',
                    'CZ' => 'Czech Republic',
                    'DE' => 'Germany',
                    'DK' => 'Denmark',
                    'ES' => 'Spain',
                    'FI' => 'Finland',
                    'FR' => 'France',
                    'GB' => 'Great Britain',
                    'GI' => 'Gibraltar',
                    'GR' => 'Greece',
                    'HU' => 'Hungary',
                    'IE' => 'Ireland',
                    'IS' => 'Iceland',
                    'IT' => 'Italy',
                    'LU' => 'Luxembourg',
                    'NL' => 'The Netherlands',
                    'NO' => 'Norwegian',
                    'PL' => 'Poland',
                    'PT' => 'Portugal',
                    'RO' => 'Romania',
                    'SE' => 'Sweden',
                    'SI' => 'Slovenia',
                    'EE' => 'Estonia',
                    'LV' => 'Latvia',
                    'SK' => 'Slovakia'
                );
    static protected $_iban_countrycode_length = array(
                    'AD' => 24,
                    'AT' => 20,
                    'BE' => 16,
                    'CH' => 21,
                    'CZ' => 24,
                    'DE' => 22,
                    'DK' => 18,
                    'ES' => 24,
                    'FI' => 18,
                    'FR' => 27,
                    'GB' => 22,
                    'GI' => 23,
                    'GR' => 27,
                    'HU' => 28,
                    'IE' => 22,
                    'IS' => 26,
                    'IT' => 27,
                    'LU' => 20,
                    'NL' => 18,
                    'NO' => 15,
                    'PL' => 28,
                    'PT' => 25,
                    'RO' => 24,
                    'SE' => 24,
                    'SI' => 19,
                    'EE' => 20,
                    'LV' => 21,
                    'SK' => 24
                );
    static protected $_iban_countrycode_bankcode = array(
                    'AD' => array('start' =>  4, 'length' =>  8), // first 4 chars are bankcode, last 4 chars are the branch
                    'AT' => array('start' =>  4, 'length' =>  5),
                    'BE' => array('start' =>  4, 'length' =>  3),
                    'CH' => array('start' =>  4, 'length' =>  5),
                    'CZ' => array('start' =>  4, 'length' =>  4),
                    'DE' => array('start' =>  4, 'length' =>  8),
                    'DK' => array('start' =>  4, 'length' =>  4),
                    'ES' => array('start' =>  4, 'length' =>  8),
                    'FI' => array('start' =>  4, 'length' =>  6),
                    'FR' => array('start' =>  4, 'length' => 10),
                    'GB' => array('start' =>  4, 'length' =>  4),
                    'GI' => array('start' =>  4, 'length' =>  4),
                    'GR' => array('start' =>  4, 'length' =>  7), // first 3 chars bankcode, last 4 chars branch
                    'HU' => array('start' =>  4, 'length' =>  7), // first 3 chars bankcode, last 4 chars branch, followed by 1 char (checksum)
                    'IE' => array('start' =>  4, 'length' => 10), // first 4 chars bankcode, last 6 chars branch
                    'IS' => array('start' =>  4, 'length' =>  4),
                    'IT' => array('start' =>  4, 'length' => 11),
                    'LU' => array('start' =>  4, 'length' =>  3),
                    'NL' => array('start' =>  4, 'length' =>  4),
                    'NO' => array('start' =>  4, 'length' =>  4),
                    'PL' => array('start' =>  4, 'length' =>  8),
                    'PT' => array('start' =>  4, 'length' =>  8),
                    'RO' => array('start' =>  4, 'length' =>  4),
                    'SE' => array('start' =>  4, 'length' =>  3),
                    'SI' => array('start' =>  4, 'length' =>  5),
                    'LV' => array('start' =>  4, 'length' =>  4),
                    'EE' => array('start' =>  4, 'length' =>  4),
                    'SK' => array('start' =>  4, 'length' =>  4)
                );
    static protected $_iban_countrycode_bankaccount = array(
                    'AD' => array('start' => 12, 'length' => 12),
                    'AT' => array('start' =>  9, 'length' => 11),
                    'BE' => array('start' =>  7, 'length' =>  9),
                    'CH' => array('start' =>  9, 'length' => 12),
                    'CZ' => array('start' =>  8, 'length' => 16),
                    'DE' => array('start' => 12, 'length' => 10),
                    'DK' => array('start' =>  8, 'length' => 10),
                    'ES' => array('start' => 12, 'length' => 12),
                    'FI' => array('start' => 10, 'length' =>  8),
                    'FR' => array('start' => 14, 'length' => 13),
                    'GB' => array('start' =>  8, 'length' => 14),
                    'GI' => array('start' =>  8, 'length' => 15),
                    'GR' => array('start' => 11, 'length' => 16),
                    'HU' => array('start' => 12, 'length' => 15), // followed by 1 char (checksum)
                    'IE' => array('start' => 14, 'length' =>  8),
                    'IS' => array('start' =>  8, 'length' => 18),
                    'IT' => array('start' => 15, 'length' => 12),
                    'LU' => array('start' =>  8, 'length' => 13),
                    'NL' => array('start' =>  8, 'length' => 10),
                    'NO' => array('start' =>  8, 'length' =>  7),
                    'PL' => array('start' => 12, 'length' => 16),
                    'PT' => array('start' => 12, 'length' => 13),
                    'RO' => array('start' =>  8, 'length' => 16),
                    'SE' => array('start' =>  7, 'length' => 17),
                    'SI' => array('start' =>  7, 'length' =>  8), // followed by 1 char (checksum)
                    'LV' => array('start' =>  8, 'length' => 13),
                    'EE' => array('start' =>  8, 'length' => 12),
                    'SK' => array('start' =>  8, 'length' => 16)
                );
    static protected $_iban_countrycode_regex = array(
                    'AD' => '/^AD[0-9]{2}[0-9]{8}[A-Z0-9]{12}$/',
                    'AT' => '/^AT[0-9]{2}[0-9]{5}[0-9]{11}$/',
                    'BE' => '/^BE[0-9]{2}[0-9]{3}[0-9]{9}$/',
                    'CH' => '/^CH[0-9]{2}[0-9]{5}[A-Z0-9]{12}$/',
                    'CZ' => '/^CZ[0-9]{2}[0-9]{4}[0-9]{16}$/',
                    'DE' => '/^DE[0-9]{2}[0-9]{8}[0-9]{10}$/',
                    'DK' => '/^DK[0-9]{2}[0-9]{4}[0-9]{10}$/',
                    'ES' => '/^ES[0-9]{2}[0-9]{8}[0-9]{12}$/',
                    'FI' => '/^FI[0-9]{2}[0-9]{6}[0-9]{8}$/',
                    'FR' => '/^FR[0-9]{2}[0-9]{10}[A-Z0-9]{13}$/',
                    'GB' => '/^GB[0-9]{2}[A-Z]{4}[0-9]{14}$/',
                    'GI' => '/^GI[0-9]{2}[A-Z]{4}[A-Z0-9]{15}$/',
                    'GR' => '/^GR[0-9]{2}[0-9]{7}[A-Z0-9]{16}$/',
                    'HU' => '/^HU[0-9]{2}[0-9]{7}[0-9]{1}[0-9]{15}[0-9]{1}$/',
                    'IE' => '/^IE[0-9]{2}[A-Z0-9]{4}[0-9]{6}[0-9]{8}$/',
                    'IS' => '/^IS[0-9]{2}[0-9]{4}[0-9]{18}$/',
                    'IT' => '/^IT[0-9]{2}[A-Z]{1}[0-9]{10}[A-Z0-9]{12}$/',
                    'LU' => '/^LU[0-9]{2}[0-9]{3}[A-Z0-9]{13}$/',
                    'NL' => '/^NL[0-9]{2}[A-Z]{4}[0-9]{10}$/',
                    'NO' => '/^NO[0-9]{2}[0-9]{4}[0-9]{7}$/',
                    'PL' => '/^PL[0-9]{2}[0-9]{8}[0-9]{16}$/',
                    'PT' => '/^PT[0-9]{2}[0-9]{8}[0-9]{13}$/',
                    'RO' => '/^RO[0-9]{2}[A-Z]{4}[A-Z0-9]{16}$/',
                    'SE' => '/^SE[0-9]{2}[0-9]{3}[0-9]{17}$/',
                    'SI' => '/^SI[0-9]{2}[0-9]{5}[0-9]{8}[0-9]{2}$/',
                    'LV' => '/^LV[0-9]{2}[A-Z0-9]{4}[A-Z0-9]{13}$/',
                    'EE' => '/^EE[0-9]{2}[0-9]{4}[0-9]{12}$/',
                    'SK' => '/^SK[0-9]{2}[0-9]{4}[0-9]{16}$/',
                );

    static public $_errorMessages = array(
                1   => 'no error',
                2   => 'unknown error',
                3   => 'IBAN generally invalid',
                4   => 'IBAN is too short',
                5   => 'IBAN is too long',
                6   => 'IBAN countrycode is invalid',
                7   => 'IBAN has invalid format',
                8   => 'IBAN checksum is invalid'
            );

    function __construct($iban = '')
    {
        $this->_iban = strtoupper($iban);
    }

    function validate()
    {
        $iban = $this->_iban;

        $errorcode=1;

        if (strlen($iban) <= 4) {
            $errorcode = 4;
        } elseif (!isset( self::$_iban_countrycode_countryname[ substr($iban,0,2) ] )) {
            $errorcode = 6;
        } elseif (strlen($iban) < self::$_iban_countrycode_length[ substr($iban,0,2) ]) {
            $errorcode = 4;
        } elseif (strlen($iban) > self::$_iban_countrycode_length[ substr($iban,0,2) ]) {
            $errorcode = 5;
        } elseif (!preg_match(self::$_iban_countrycode_regex[ substr($iban,0,2) ],$iban)) {
            $errorcode = 7;
        } else {
            $iban_replace_chars = range('A','Z');
            foreach (range(10,35) as $tempvalue) {
                $iban_replace_values[]=strval($tempvalue);
            }

            // move first 4 chars (countrycode and checksum) to the end of the string
            $tempiban = substr($iban, 4).substr($iban, 0, 4);
            $tempiban = str_replace($iban_replace_chars, $iban_replace_values, $tempiban);
            $tempcheckvalue = intval(substr($tempiban, 0, 1));
            for ($strcounter = 1; $strcounter < strlen($tempiban); $strcounter++) {
                $tempcheckvalue *= 10;
                $tempcheckvalue += intval(substr($tempiban,$strcounter,1));
                $tempcheckvalue %= 97;
            }

            // checkvalue of 1 indicates correct IBAN checksum
            if ($tempcheckvalue != 1) {
                $errorcode=8;
            } else {
                $errorcode=1;
            }
        }

        $this->_errorcode=$errorcode;
        return ($errorcode == 1);
    }

    function getErrorcode()
    {
        return $this->_errorcode;
    }

    function errorMessage($value)
    {
        return isset(self::$_errorMessages[$value]) ?
        	self::$_errorMessages[$value] : self::$_errorMessages[2];
    }
}
?>
