<?php
require_once(IWARE.'IWFileWriter.php');
/**
 * A CSV-file writer.
 *
 * Implements CommaSeperatedValue file as stated in http://www.ietf.org/rfc/rfc4180.txt
 * @author Chris Wesseling <chris.wesseling@xs4all.nl>
 * @package I-Ware
 * @version $Id$
 */
class IWCSVWriter implements IWFileWriter
{
	public $delimiter = ';';
	public $enclosure = '"';
	public $enclose_all = false;
	public $eol = "\r\n";
	public $show_header = false;
	private $ds;

	public function __construct($ds=null)
	{
		!empty($ds) && $this->setData($ds);
	}


	function getString()
	{
		$rv = '';
		if ( $this->show_header )
			$rv .= $this->processRow(array_keys($this->ds[0])).$this->eol;
		foreach ( $this->ds as $row )
			$rv .= $this->processRow($row).$this->eol;

		return $rv;
	}

	function write($filename)
	{
		return file_put_contents($filename, $this->getString());
	}

	function setData($dataset)
	{
		do
		{
			if ( is_array($dataset) )
			{
				if ( empty($dataset) || !is_array(current($dataset)) )
					break;//error
				$this->ds = $dataset;
			}
			elseif ( $dataset instanceof IWQuery )
				while ( $row = $dataset->fetch() )
					$this->ds[] = (array) $row;
			else
				break;//error

			return true;
		}while(false);

		throw new IWCSVWriterException('Dataset is invalid format or type');
	}

	function processRow($aRow)
	{
		foreach ( $aRow as $key=>$bitbucket )
		{
			$value =& $aRow[$key];
			if ( $this->enclose_all || preg_match('/['.$this->enclosure.$this->delimiter.$this->eol.']/',$value) )
				$value = $this->enclosure.str_replace($this->enclosure, str_repeat($this->enclosure,2), $value).$this->enclosure;
		}
		return implode($this->delimiter, $aRow);
	}
}

class IWCSVWriterException extends IWFileWriterException{}
?>
