<?php
require_once(IWARE.'AbstractObject.php');
/**
 * @package IW
 * @subpackage CustomQuery
 * @author Marten Koedam <marten@dalines.net>
 * @copyright 2007, BaseSoft Solutions B.V.
 * @since 11th of September 2007
 */
class IWCustomQueryDynamicQuery extends AbstractObject
{
	/**
	 * @var int
	 */
	public $id;
	/**
	 * @var string
	 */
	public $name;
	/**
	 * @var string
	 */
	public $query;
	/**
	 * @var string
	 */
	public $where;
	/**
	 * @var string
	 */
	public $description;
	/**
	 * @var string
	 */
	public $order_by;
	/**
	 * @var string
	 */
	public $group_by;
	/**
	 * @access protected
	 * @var array
	 */
	protected $filters;
	/**
	 * @param int $id
	 * @return bool
	 */
	public function set($id=null)
	{
		$id || $id = $this->id;
		return $this->_set($id);
	}
	/**
	 * @param string $name
	 * @return bool
	 */
	public function setByName($name=null)
	{
		$name || $name = $this->name;
		return $this->_set($name);
	}
	/**
	 * @param mixed $setter
	 * @return bool
	 */
	protected function _set($setter)
	{
		$field = is_numeric($setter)?'DYNAMIC_REPORT_ID':'NAME';
		$sql = "SELECT DYNAMIC_REPORT_ID, `NAME`, `QUERY`, `WHERE`,
				`DESCRIPTION`, ORDER_BY, GROUP_BY
			FROM TBL_DYNAMIC_REPORT
			WHERE $field = ?";
		$q = $this->db->query($sql);
		$q->bind_values($setter);
		$q->execute();

		$r = $q->fetch();

		$this->id = $r->DYNAMIC_REPORT_ID;
		$this->name = $r->NAME;
		$this->query = $r->QUERY;
		$this->where = $r->WHERE;
		$this->description = $r->DESCRIPTION;
		$this->order_by = $r->ORDER_BY;
		$this->group_by = $r->GROUP_BY;

		$this->setFilters();

		return $this->complete = true;
	}
	/**
	 * Get the overview of all available dynamic reports
	 * @return ResultSet $q
	 */
	public function getOverview()
	{
		$sql = "SELECT DYNAMIC_REPORT_ID, `NAME` as Report, `DESCRIPTION` as ReportDescription
			FROM TBL_DYNAMIC_REPORT
			ORDER BY DYNAMIC_REPORT_ID";
		$q = $this->db->query($sql);
		$q->execute();
		return $q;
	}
	/**
	 * Sets all the filters linked to the current Dynamic Report
	 */
	protected function setFilters()
	{
		$this->filters = array();

		$sql = "SELECT FILTER_INDEX
			FROM TBL_DYNAMIC_REPORT_FILTER
			WHERE DYNAMIC_REPORT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		while($r = $q->fetch())
		{
			$this->filters[$r->FILTER_INDEX] = new IWCustomQueryDynamicQueryFilter($this->db,
				array('dynamic_query'=>$this,'index'=>$r->FILTER_INDEX),true);
		}
	}
	/**
	 * Returns the array containing the IWCustomQueryDynamicReportFilters
	 * @return Array (IWCustomQueryDynamicReportFilter [, IWCustomQueryDynamicReportFilter [, ...]] )
	 */
	public function getFilters()
	{
		return $this->filters;
	}
	/**
	 * Gets the overview of the filters linked to the current Dynamic Report
	 * @return ResultSet $q
	 */
	function getFilterOverview()
	{
		$sql = "SELECT FILTER_INDEX, `DISPLAYNAME`, `FIELDNAME`, TABLE_ALIAS, REQUIRED
			FROM TBL_DYNAMIC_REPORT_FILTER
			WHERE DYNAMIC_REPORT_ID = ?
			ORDER BY FILTER_INDEX";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();
		return $q;
	}
	/**
	 * @access protected
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_DYNAMIC_REPORT';
		$this->objectmap['primary']['id'] = 'DYNAMIC_REPORT_ID';
		$this->objectmap['properties']['name'] = 'NAME';
		$this->objectmap['properties']['query'] = 'QUERY';
		$this->objectmap['properties']['where'] = 'WHERE';
		$this->objectmap['properties']['description'] = 'DESCRIPTION';
		$this->objectmap['properties']['group_by'] = 'GROUP_BY';
		$this->objectmap['properties']['order_by'] = 'ORDER_BY';
		$this->objectmap['inserttype'] = 'REPLACE';
	}
	/**
	 * Deletes the Dynamic Report, its filters and all the linked Presets
	 */
	function delete()
	{
		$this->db->begin();
		// Select the preset id to delete.
		$sql = "SELECT PRESET_ID FROM TBL_CUSTOM_QUERY_PRESET WHERE CUSTOM_QUERY = ?";
		$q = $this->db->query($sql);
		$q->bind_values('WEEECustomQueryDynamicReport_'.$this->id);
		$q->execute();
		$rows = $q->fetchAll();
		if(count($rows))
		{
			$r = current($rows);
			$preset_id = $r->PRESET_ID;
			// Delete all user linked presets.
			$sql = "DELETE FROM TBL_USER_CUSTOM_QUERY WHERE PRESET_ID = ?";
			$q = $this->db->query($sql);
			$q->bind_values($preset_id);
			$q->execute();

			// Delete all user defined presets filters.
			$sql = "DELETE FROM TBL_CUSTOM_QUERY_PRESET_FILTER WHERE PRESET_ID = ?";
			$q = $this->db->query($sql);
			$q->bind_values($preset_id);
			$q->execute();

			// Delete all user defined presets.
			$sql = "DELETE FROM TBL_CUSTOM_QUERY_PRESET WHERE PRESET_ID = ?";
			$q = $this->db->query($sql);
			$q->bind_values($preset_id);
			$q->execute();
		}

		// Delete all filters.
		$sql = "DELETE FROM TBL_DYNAMIC_REPORT_FILTER WHERE DYNAMIC_REPORT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();

		// Delete the actual query
		$sql = "DELETE FROM TBL_DYNAMIC_REPORT WHERE DYNAMIC_REPORT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		$q->execute();

		$this->db->commit();
	}

	function exportToPhp()
	{
		if(!$this->isComplete())
			$this->set();

		$filterContent = "
			\$query = %s;
			\$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
				\$query,'%s','%s','%s');
			";
		$fields = '';
		foreach($this->getFilters() as $filter)
		{
			$query = (substr($filter->query,0,6)=='filter')?"\$this->getFilter('".$filter->query."');":'"'.$filter->query.'"';
			$fields .= sprintf($filterContent,
				$query,
				$filter->fieldname,
				$filter->displayname,
				$filter->table_alias
				);
		}

		$valueProcessors = '';


		$output =
"<?php
/**
 * @package CustomQuery
 * @subpackage
 * @author
 * @copyright %s, BaseSoft Solutions B.V.
 */
class WEEECustomQuery... extends IWCustomQueryObject
{
	public function isPrivileged()
	{
		require_once(IWARE.'group.php');
		switch(\$_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
				return true;
		}
		return false;
	}

	public function getQuery()
	{
		\$sql = \"%s\";
		return \$sql;
	}

	public function getWhere()
	{
		%s
	}
	public function registerFields(IWCustomQueryFactory \$cq)
	{
		%s
	}

	public function tfValueProcessor(\$args=false)
	{
		if(!\$args)
			return array(\$this,'tfValueProcessor');

		require_once(ELC_CLASSES.'ELCListBuilder.php');
		\$list = new ELCDefaultList(IWRegistry::registry('db'),IWRegistry::registry('langIf'));
		if(!empty(\$_POST['export']))
			\$list->setShowType(LIST_BUTTON_CSV_EXPORT);

		\$this->_cols = \$args['visibleColumns'];
		switch(\$args['head'])
		{
			%s
		}
		return \$args['value'];
	}
	public function getSQLGroupBy()
	{
		return \" %s\";
	}

	public function getSQLOrderBy()
	{
		return \" %s\";
	}

	public function tfAllowSorting()
	{
		return true;
	}
}
?>";

		echo highlight_string(sprintf($output,
			date('Y'),
			$this->query,
			($this->where?"return 'AND ".preg_replace('/^WHERE/','',$this->obj->where)."';":''),
			$fields,
			$valueProcessors,
			$this->group_by,
			$this->order_by
			),true);
		exit;
	}
}
/**
 * @package IW
 * @subpackge CustomQuery
 * @author Marten Koedam <marten@dalines.net>
 */
class IWCustomQueryDynamicQueryFilter extends AbstractObject
{
	/**
	 * @var IWCustomQueryDynamicQuery
	 */
	public $dynamic_query;
	/**
	 * @var int
	 */
	public $index;
	/**
	 * @var string
	 */
	public $fieldname;
	/**
	 * @var string
	 */
	public $displayname;
	/**
	 * @var string
	 */
	public $table_alias;
	/**
	 * @var string
	 */
	public $query;
	/**
	 *
	 */
	public $required;
	/**
	 * @param int $dynamic_query->id
	 * @param int $index
	 * @return bool
	 */
	public function set($id=null,$index=null)
	{
		$id || $id = $this->dynamic_query->id;
		$index || $index = $this->index;

		$sql = "SELECT FIELDNAME,DISPLAYNAME,TABLE_ALIAS,`QUERY`,REQUIRED
			FROM TBL_DYNAMIC_REPORT_FILTER
			WHERE DYNAMIC_REPORT_ID = ?
				AND FILTER_INDEX = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id,$index);
		$q->execute();

		$rows = $q->fetchAll();
		if(!count($rows))
			return false;
		$r = current($rows);

		$this->dynamic_query = new IWCustomQueryDynamicQuery($this->db,$id);
		$this->fieldname = $r->FIELDNAME;
		$this->displayname = $r->DISPLAYNAME;
		$this->table_alias = $r->TABLE_ALIAS;
		$this->query = $r->QUERY;
		$this->required = $r->REQUIRED;

		return $this->complete = true;
	}
	/**
	 * Insert a filter, and make sure you get the next index in line.
	 */
	function insert()
	{
		empty($this->index) && $this->index = $this->getNextIndex();
		parent::insert();
	}
	/**
	 * @access protected
	 * @return int
	 */
	protected function getNextIndex()
	{
		$sql = "SELECT (MAX(FILTER_INDEX)+1) as NEWINDEX
			FROM TBL_DYNAMIC_REPORT_FILTER
			WHERE DYNAMIC_REPORT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->dynamic_query->id);
		$q->execute();
		$rows = $q->fetchAll();
		if(!count($rows))
			return 1;

		$r = current($rows);
		if(empty($r->NEWINDEX))
			return 1;

		return $r->NEWINDEX;
	}
	/**
	 * @access protected
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_DYNAMIC_REPORT_FILTER';
		$this->objectmap['primary']['dynamic_query->id'] = 'DYNAMIC_REPORT_ID';
		$this->objectmap['primary']['index'] = 'FILTER_INDEX';
		$this->objectmap['properties']['fieldname'] = 'FIELDNAME';
		$this->objectmap['properties']['displayname'] = 'DISPLAYNAME';
		$this->objectmap['properties']['table_alias'] = 'TABLE_ALIAS';
		$this->objectmap['properties']['query'] = 'QUERY';
		$this->objectmap['properties']['required'] = 'REQUIRED';
		$this->objectmap['inserttype'] = 'REPLACE';
	}
}
?>
