<?php
require_once(IWARE.'IWCustomQueryPreset.php');
/**
 * @package IW
 * @subpackage QueryBuilder
 *
 * Below is an example based on the TBL_RELATION from the WEEE-project
 * <code>
 * 	$cqf = new IWCustomQueryFactory();
 *	$cqf->setField('RELATION_TYPE_ID','Relation Type',FIELD_INPUT_DROPDOWN,array(1=>'Operator',3=>'CRSO',2=>'Participant'));
 *	$cqf->setField('NAME','Company',FIELD_INPUT_TEXT);
 *
 *	$cqf->setQuery('SELECT RELATION_ID, `NAME`, VAT_NO, WEBSITE FROM TBL_RELATION ');
 *	$cqfd = new IWCustomQueryFilterDisplay($cqf);
 *
 *	$show_style[] = 'filtertable';
 *	$script_header = '<script type="text/javascript" src="'.REL_IWARE_INCLUDES.'IWCustomQueryFactory.js"></script>
 *				<script type="text/javascript"><!--
 *					'.$cqfd->getProperties().'
 *					var CQF = new IWCustomQueryFactory(properties);
 *					'.$cqfd->getSelectedFilters().'
 *				--></script>';
 *
 *	$ds = $cqf->getDataSet($db);
 *	echo $cqfd->display().'<br /><br />';
 *	$tf = new TableFactory($ds,$langIf);
 *	echo $tf->getXHTMLTable('Professional',2);
 * </code>
 *
 */
/**
 * IWCustomQueryBuilder
 * @todo make the autogenerate more funcional
 *
 * @author Marten Koedam <marten@dalines.org>
 * @package IW
 * @subpackage QueryBuilder
 * @since 21 december 2006
 * @version $Id$
 */
class IWCustomQueryFactory
{
	/**#@+
	 * @access protected
	 */
	protected $db = null;
	protected $tbls = array();
	protected $fields = array();
	protected $query = '';
	protected $where = '';
	protected $sql_groupBy = '';
	protected $sql_orderBy = '';
	/**
	 * Mapping of Database types to certain filter types
	 * @var array $dbTypeMap
	 */
	protected $dbTypeMap = array(
		'int(10) unsigned'=>FIELD_INPUT_TEXT,
		'varchar(.*'=>FIELD_INPUT_DROPDOWN,
		'tinyint .*'=>FIELD_INPUT_TEXT
		);
	protected $settings;
	protected $handle;
	protected $requirements = false;
	/**#@-*/
	public $obj = false;
	/**
	 * Constructor
	 * <p>The Database param for the constructor needs to be of a type
	 * that supports 'DESC' as a query method. The chosen type is PDO at the moment.
	 *
	 * @param PDO $db
	 * @param IWCustomQueryObject $obj
	 */
	public function __construct($db=null,$obj=null,$load=false)
	{
		$db && $this->setDB($db);
		$obj && $this->setObject($obj);
		($obj && $load) && $this->loadFromObject();
	}
	/**#@+
	 * Function to use when Factory is initalized with an Object.
	 *
	 * This also include calls to tf->valueprocessors, tf->footer etc.
	 *
	 * @see TableFactory
	 */
	/**
	 * @param IWCustomQueryObject $obj
	 */
	public function setObject(IWCustomQueryObject $obj)
	{
		$this->obj = $obj;
	}
	/**
	 *
	 */
	public function loadFromObject()
	{
		if(!$this->obj)
			return new IWException('IWCustomQueryFactory::loadFromObject() - Object has not been initalized');

		// Check the rights on the CustomQuery.
		if(!$this->obj->isPrivileged())
			return new IWException('IWCustomQueryFactory::loadFromObject() - No Access to CustomQuery');

		//$this->obj->setFactory($this);
		$this->requirements = $this->obj->getRequiredFilters();
		$this->setQuery($this->obj->getQuery());
		$this->setWhere($this->obj->getWhere());
		$this->setSQLGroupBy($this->obj->getSQLGroupBy());
		$this->setSQLOrderBy($this->obj->getSQLOrderBy());
		$this->obj->registerFields($this);
		$this->setHandle((isset($this->obj->handle)?$this->obj->handle:get_class($this->obj)));
	}
	/**
	 * @access private
	 */
	public function __call($method,$args)
	{
		if(substr($method,0,2) == 'tf')
			return $this->obj->$method($args);
		return $this->$method($args); // Throws exception... method doesnt exist
	}
	/**@#-*/
	public function setHandle($handle)
	{
		$this->handle = $handle;
	}
	public function getHandle()
	{
		return $this->handle;
	}
	/**
	 * Set the PDO database pointer
	 * @param PDO $db
	 */
	public function setDB($db)
	{
		$this->db = $db;
	}
	/**
	 * Sets the actually query that needs to be executed.
	 * <p>This should not include any where statements
	 * or stuff that goes after WHERE statements in a query
	 *
	 * @param string $query
	 */
	public function setQuery($query)
	{
		if(substr($query,0,6)!='SELECT')
			throw new IWUserException('Incorrect Query in CustomQuery');
		$this->query = $query;
	}
	/**
	 * Sets the group by for the query
	 * <p> NOTE: This should include the actual GROUP BY part of the string
	 * So for example:
	 *
	 * <code>
	 * $cqf->setSQLGroupBy('GROUP BY D.DECLARATION_ID');
	 * </code>
	 *
	 * @param string $gb
	 */
	public function setSQLGroupBy($gb)
	{
		$this->sql_groupBy = ' '.$gb;
	}
	/**
	 *
	 */
	public function setSQLOrderBy($ob)
	{
		$this->sql_orderBy = ' '.$ob;
	}

	public function storeSettingsToDB($db)
	{
		if(!$this->handle)
			return false;

		$cqp = new IWCustomQueryPreset($db);
		$cqp->custom_query = $this->handle;
		$cqp->group_by = $this->settings['groupby'];
		$cqp->descending = $this->settings['descending'];
		$cqp->is_public = empty($_POST['is_public'])?'No':'Yes';
		$cqp->name = $_POST['preset_name'];

		foreach($this->fields as $fieldHandle => $bitbucket)
		{
			if(!isset($this->settings[$fieldHandle]))
				continue;
			foreach($this->settings[$fieldHandle] as $value)
			{
				$cqf = new IWCustomQueryPresetFilter($db);
				$cqf->type = empty($this->settings[$fieldHandle.'_mode'])?'':$this->settings[$fieldHandle.'_mode'];
				$cqf->value = $value;
				$cqf->handle = $fieldHandle;
				$cqp->addFilter($cqf);
			}
		}
		$cqp->insert();
		// Add it to the user ...
		if(empty($_POST['is_public']))
		{
			$ucq = new IWUserCustomQuery($db);
			$ucq->preset = $cqp;
			$ucq->user = new ELCUser($db,$_SESSION['session_user_id']);
			$ucq->is_default = empty($_POST['is_default'])?'No':'Yes';
			$ucq->insert();
		}
	}

	public function loadSettingsFromDB($db)
	{
		// If the options for the query have changed since the last time this one was loaded, give an error message.
		if(!$this->handle)
			return false;

		$ucq = new IWUserCustomQuery($db);
		$cqp = new IWCustomQueryPreset($db);

		$id = isset($_POST['preset'])?$_POST['preset']:$ucq->getDefaultPresetId($this->handle,$_SESSION['session_user_id']);
		if(!$id)
			return false;

		if(!$cqp->set($id))
			return false;

		if(!$cqp->hasAccess($_SESSION['session_user_id'],$id))
			return false;

		$this->settings['groupby'] = $cqp->group_by;
		$this->settings['descending'] = $cqp->descending;
		foreach($cqp->getFilters() as $ucqf)
		{
			if(isset($this->fields[$ucqf->handle]))
			{
				$this->settings[$ucqf->handle.'_mode'] = $ucqf->type;
				$this->settings[$ucqf->handle][] = $ucqf->value;
			}
		}
	}

	public function getPresets()
	{
		$cqp = new IWCustomQueryPreset(IWRegistry::registry('db'));
		$q = $cqp->getPresets($_SESSION['session_user_id'],$this->handle);
		return $q->fetchAll();
	}

	/**
	 * This should be the only place where the _GET is read.
	 */
	public function loadSettings($db)
	{
		if(empty($_POST['filter_query'])||!empty($_POST['preset']))
			return $this->loadSettingsFromDB($db);

		$_SESSION['CQCurrrentPage'] = array();

		if(!empty($_POST['groupby']))
			array_key_exists($_POST['groupby'],$this->fields) && $this->settings['groupby'] = $_POST['groupby'];
		else
			$this->settings['groupby'] = '';
		$this->settings['descending'] = empty($_POST['descending'])?'':(bool)$_POST['descending'];
		foreach($this->fields as $fieldHandle => $bitbucket)
		{
			if(isset($_POST[$fieldHandle]))
			{
				$this->settings[$fieldHandle]=$_POST[$fieldHandle];
				if(!empty($_POST[$fieldHandle.'_mode']))
					$this->settings[$fieldHandle.'_mode'] = $_POST[$fieldHandle.'_mode'];
			}
		}

		if(!empty($_POST['save_query']))
			$this->storeSettingsToDB($db);
	}

	public function getSettings()
	{
		return $this->settings;
	}

	public function getRequiredFields()
	{
		return $this->requirements;
	}

	public function getRowCount($db)
	{
		if(!$this->getLimit('default'))
			return false;
		if(!empty($this->settings['groupby']))
			return $this->getGroupedRowCount($db);
		if(method_exists($this->obj,'getRowCount'))
			return $this->obj->getRowCount($this->query,$this->getWhere(),$this->sql_groupBy);

		$where = $this->getWhere();
		$query = $this->obj->rowCntQuery?$this->obj->rowCntQuery:preg_replace('/SELECT(.*)FROM/is','SELECT COUNT(*) as CNT FROM',$this->query);
		$q = $db->query($query.$where);
		$q->execute();
		$r = $q->fetch();
		return $r->CNT;
	}

	protected function getGroupedRowCount($db)
	{
		$groupby = $this->settings['groupby'];
		list($groups,$where) = $this->getGroupedWhere();

		if(method_exists($this->obj,'getGroupedRowCount'))
			return $this->obj->getGroupedRowCount($this->query,$groups,$where,$this->sql_groupBy,$groupby,$this->fields);

		$rv = array();
		$src = $this->fields[$groupby]['tbl']?$this->fields[$groupby]['tbl'].'.'.'`'.$groupby.'`':'`'.$groupby.'`';
		foreach($groups['options'] as $value => $label)
		{
			$val = (($value==DB_NULL)?sprintf(' ISNULL(%s) ',$src):(($value=='NOT'.DB_NULL)?sprintf(' NOT ISNULL(%s) ',$src):sprintf('%s = %s',$src,$value)));
			$newwhere = ($where?$where.' AND ':' WHERE ').$val;
			$query = $this->obj->rowCntQuery?$this->obj->rowCntQuery:preg_replace('/SELECT(.*)FROM/is','SELECT COUNT(*) as CNT FROM',$this->query);
			$q = $db->query($query.$newwhere);
			$q->execute();
			$r = $q->fetch();
			$rv[$label] = $r->CNT;
		}
		return $rv;
	}

	/**
	 * Call this to get the dataset(s)
	 * This will be multiple datasets in case a groupby has been initalized
	 *
	 * @param Database $db
	 * @return mixed
	 */
	public function getDataSet($db,$limit=false)
	{
		if(!$this->checkRequirements())
			return array();

		if(!empty($this->settings['groupby']))
                    return $this->getGroupedDataSets($db,$limit);

		$where = $this->getWhere();
		$q = $db->query($this->query.$where.$this->sql_groupBy.$this->sql_orderBy.($limit?$this->getLimit('default'):''));
		$q->execute();
		return $q;
	}

	protected function checkRequirements()
	{
		if(!$this->requirements)
			return true;

		foreach($this->requirements as $required)
			if(!isset($this->settings[$required]))
				return false;

		return true;
	}

	protected function setWhere($where)
	{
		$this->where = $where;
	}

	protected function getWhere()
	{
		$where = $this->where;
		foreach($this->fields as $fieldHandle => $values)
		{
			if(isset($this->settings[$fieldHandle]))
				$where .= $this->filterFieldWithOperator((!empty($this->settings[$fieldHandle.'_mode'])?$this->settings[$fieldHandle.'_mode']:''),$fieldHandle, $values);
		}
		$where = $where?' WHERE '.substr($where,4):'';
		return $where;
	}

	protected function getGroupedWhere()
	{
		$groupby = $this->settings['groupby'];
		$groups = array();
		if(!isset($this->settings[$groupby]))
			$groups = $this->fields[$groupby];

		$where = $this->where;
		foreach($this->fields as $fieldHandle => $values)
		{
			if(isset($this->settings[$fieldHandle])&&($groupby!=$fieldHandle))
				$where .= $this->filterFieldWithOperator((!empty($this->settings[$fieldHandle.'_mode'])?$this->settings[$fieldHandle.'_mode']:''),$fieldHandle, $values);
			elseif(isset($this->settings[$fieldHandle])&&($groupby==$fieldHandle))
			{
				// The mode is empty, so that means we use EQUAL in the query, only use the selected fields!
				if(empty($this->settings[$fieldHandle.'_mode']))
				{
					foreach($this->settings[$fieldHandle] as $value)
						$groups['options'][$value] = $values['options'][$value];
				}
				else
				{
					// The mode is NOT equal, so it would be weird if we used those to group by: they are not selected!
					foreach($values['options'] as $value => $content)
						if(!in_array($value,$this->settings[$fieldHandle]))
							$groups['options'][$value] = $content;
				}
			}
		}
		$where = $where?' WHERE '.substr($where,4):'';
		return array($groups,$where);
	}

	protected function getLimit($group)
	{
		if(!$this->obj->limit||!empty($_POST['export']))
			return '';

		if(isset($_POST['page'])&&isset($_POST['page'][md5($group)]))
			$_SESSION['CQCurrrentPage'][md5($group)] = $this->obj->currentpage[$group] = $_POST['page'][md5($group)];
		elseif(!isset($this->currentpage[$group]))
			$this->obj->currentpage[$group] = isset($_SESSION['CQCurrrentPage'][md5($group)])?$_SESSION['CQCurrrentPage'][md5($group)]:0;

		$start = $this->obj->currentpage[$group]*$this->obj->limit;
		return ' LIMIT '.$start.', '.$this->obj->limit;
	}
	/**
	 * This is the label for the groupby that has been selected from the interface
	 * It looks up the display label of the filter from the local variable $fields
	 * @return string
	 */
	public function getGroupByLabel()
	{
		if(empty($this->settings['groupby']))
			return '';
		return $this->fields[$this->settings['groupby']]['display'];
	}
	/**
	 * Get the grouped data set in case the groupby has been set
	 *
	 * <p>The groups can only consist of fields that are of the types: FIELD_INPUT_MULTI or FIELD_INPUT_DROPDOWN
	 * In case the groupby was also a selected filter, the following applies:
	 *  - If the mode of the filter was empty (meaning the = in the query), use only those fields that have been selected
	 *  - If the mode of the filter is not empty, use everything but the ones that have been selected
	 * @param Database $db
	 * @return array $rv
	 */
	protected function getGroupedDataSets($db,$limit=false)
	{
		$hide_empty_sets = empty($_REQUEST['hide_empty_sets'])?false:true;
		$groupby = $this->settings['groupby'];
		list($groups, $where) = $this->getGroupedWhere();
		$rv = array();
		$src = $this->fields[$groupby]['tbl']?$this->fields[$groupby]['tbl'].'.'.'`'.$groupby.'`':'`'.$groupby.'`';
		// Now add the options to the where statement, or create a new where statement if the original statement was empty
		foreach($groups['options'] as $value => $label)
		{
			$val = (($value==DB_NULL)?sprintf(' ISNULL(%s) ',$src):(($value=='NOT'.DB_NULL)?sprintf(' NOT ISNULL(%s) ',$src):sprintf('%s = %s',$src,$value)));
			$newwhere = ($where?$where.' AND ':' WHERE ').$val;
			$q = $db->query($this->query.$newwhere.$this->sql_groupBy.$this->sql_orderBy.($limit?$this->getLimit($label):''));
			$q->execute();
			if($hide_empty_sets && $q->affected_rows() == 0) {
				continue;
			}
			$rv[$label] = $q;
		}
		// Do we want it ascending or descending
		if(empty($this->settings['descending']))
			ksort($rv);
		else
			krsort($rv);
		return $rv;
	}
	/**
	 * Creates the where statements from the selected filters
	 *
	 * <p>If the selected handle has a tbl index set on its values,
	 * we need to apply that infront of the lookup field, cause apparently the
	 * value can be ambiguous within the query
	 *
	 * @param string $mode
	 * @param string $handle
	 * @param array $values
	 */
	protected function filterFieldWithOperator($mode,$handle,$values)
	{
		$rv = "AND (";
		$src = $values['tbl']?$values['tbl'].'.'.'`'.$handle.'`':''.$handle.'';
		// selects can be "" (is) and ! (not is)

		if(is_array($values['callback']))
		{
			list($obj,$function) = $values['callback'];
			return $obj->$function($this,$mode,$handle,$values);
		}

		switch($values['type'])
		{
			// Text inputs can consist of a range of modes
			// texts can be ~ (contains), !~ (not contains), ^ (begins with), $ (ends with), "" (is) and ! (is not)
			case FIELD_INPUT_TEXT:
				$andModes = array('','!~');
				$operator = in_array($mode,$andModes)?'AND ':' OR ';
				$ops = $val = '';
				foreach($this->settings[$handle] as $value)
				{
					$value = addslashes($value);
					switch($mode)
					{
						case '': // Means EQUAL
							$ops = '=';
						case '!': // Means NOT EQUAL
							$ops || $ops = '!=';
							$rv .= sprintf("%s %s '%s' %s",$src,$ops,$value,$operator);
						break;
						case '~': // Means CONTAINS
							$ops = 'LIKE';
						case '!~': // Means NOT CONTAINS
							$ops || $ops = 'NOT LIKE';
							$rv .= sprintf("%s %s '%s' %s",$src,$ops,'%'.$value.'%',$operator);
						break;
						case '^': // Means BEGINS WITH
							$val = $value.'%';
						case '$': // Means ENDS WITH
							$val || $val = '%'.$value;
							$ops = 'LIKE ';
							$rv .= sprintf("%s %s '%s' %s",$src,$ops,$val,$operator);
						break;
					}

				}
				return substr($rv,0,-4).')';
			break;
			case FIELD_INPUT_NUMBER:
				$andModes = array('','!~');
				$operator = in_array($mode,$andModes)?'AND ':' OR ';
				$ops = $val = '';
				foreach($this->settings[$handle] as $value)
				{
					$db = IWRegistry::registry('db');
					$value = $db->quote($value);
					switch($mode)
					{
						case '': // Means EQUAL
							$ops = '=';
						case '!': // Means NOT EQUAL
							$ops || $ops = '!=';
							$rv .= sprintf("%s %s '%s' %s",$src,$ops,$value,$operator);
						break;
						case '~': // Means EQUAL with OR
							$ops = '=';
						case '!~': // Means NOT EQUAL with OR
							$ops || $ops = '!=';
							$rv .= sprintf("%s %s '%s' %s",$src,$ops,$value,$operator);
						break;
						case '^': // Means LARGER THAN
							$ops = '>';
						case '$': // Means ENDS WITH
							$ops || $ops = '<';
							$rv .= sprintf("%s %s '%s' %s",$src,$ops,$value,$operator);
						break;
					}

				}
				return substr($rv,0,-4).')';
			break;
			case FIELD_INPUT_PERIOD:
				$andModes = array('','~','!~');
				$operator = in_array($mode,$andModes)?'AND ':' OR ';
				$ops = $val = '';

				$betweenModes = array('~','!~');
				if(in_array($mode,$betweenModes))
				{
					$rv .= $src.' '.(($mode{0}=='!')?'NOT ':'').' BETWEEN ';
					if(count($this->settings[$handle])>2)
						return '';
				}
				$valuesProvided = false;
				foreach($this->settings[$handle] as $value)
				{
					$db = IWRegistry::registry('db');

					if(!preg_match('/^\d{4}-\d{2}-\d{2}$/',$value))
						continue;
					$valuesProvided = true;
					switch($mode)
					{
						case '': // Means EQUAL
							$ops = '=';
						case '!': // Means NOT EQUAL
							$ops || $ops = '!=';
							$rv .= sprintf("%s %s '%s' %s",$src,$ops,$value,$operator);
						break;
						case '~': // Means BETWEEN
							$ops = '=';
						case '!~': // Means NOT BETWEEN
							$ops || $ops = '!=';
							$rv .= sprintf("'%s' %s",$value,$operator);
						break;
						case '^': // Means LARGER THAN
							$ops = '>';
						case '$': // Means ENDS WITH
							$ops || $ops = '<';
							$rv .= sprintf("%s %s '%s' %s",$src,$ops,$value,$operator);
						break;
					}

				}
				if(!$valuesProvided)
					return '';
				return substr($rv,0,-4).')';
			break;
			// Multi and dropdown work the same way, only their display is different
			// They have two modes, EQUAL or NOT EQUAL
			case FIELD_INPUT_MULTI:
			case FIELD_INPUT_DROPDOWN:
				// the third %s can only be one of the values from Options
				$operator = $mode?'AND ':' OR ';
				$mode = $mode?'!=':'=';
				foreach($this->settings[$handle] as $value)
				{
					if(!isset($values['options'][$value]))
						throw new IWException('Someones trying to do something naughty');
					if($value == DB_NULL)
						$rv .= sprintf("ISNULL(%s) %s",$src,$operator);
					elseif($value == 'NOT'.DB_NULL)
						$rv .= sprintf("NOT ISNULL(%s) %s",$src,$operator);
					else
						$rv .= sprintf("%s %s '%s' %s",$src,$mode,$value,$operator);
				}
				return substr($rv,0,-4).')';
			break;
		}
	}
	/**
	 * Returns the array of local fields that have been set
	 * @return array
	 */
	public function getFields()
	{
		return $this->fields;
	}
	/**
	 *
	 */
	public function setFields($fields)
	{
		$this->fields = $fields?$fields:array();
	}
	/**
	 * Set a new field in the local fields array
	 * <p>This includes the fields options if applicable.
	 * Options is an array where the key is the input-value and the value is the input-display
	 * @param string $fieldHandle
	 * @param string $fieldDisplay // The displayed title of the field
	 * @param int $fieldType // DROPDOWN, MULTI or TEXT
	 * @param array|bool $options
	 * @param bool $reset
	 * @param string|bool $tbl
	 */
	public function setField($fieldHandle,$fieldDisplay,$fieldType,$options=false,$reset=false,$tbl=false,$callback=false)
	{
		$reset && $this->fields = array();

		$this->fields[$fieldHandle] = array('display'=>$fieldDisplay,'type'=>$fieldType,'options'=>$options,'tbl'=>$tbl,'callback'=>$callback);
	}
	/**
	 * You might want to choose to let people only use NON-text inputs.
	 * Most likely only useful for AutoGenerated fields
	 *
	 * Example
	 * <code>
	 *  // All fields of the type TEXT will be removed from the fields array
	 * $cqf->removeFieldsByType(FIELD_INPUT_TEXT);
	 * </code>
	 * @param int $fieldType // DROPDOWN, MULTI or TEXT
	 */
	public function removeFieldsByType($fieldType)
	{
		foreach($this->fields as $handle => $values)
			if($values['type']==$fieldType)
				unset($this->fields[$handle]);
	}
	/**#@+
	 * Function usable for AutoGeneration from tbls
	 * @todo fixme, cleanup and make useful
	 */
	/**
	 * Set all the tables that we want to check and auto-use
	 * Add multiple tables if you use joins.
	 * @param array $tbls (TblHandle in Query=>(TblName,TblType))
	 */
	public function setTables($tbls)
	{
		$this->tbls = $tbls;
	}
	/**
	 * Remove certain fields from the Fields array
	 * This is useful if they were filled by the autoGenerate functions and you dont want certain fields to be available
	 * @param array $fields
	 */
	public function filterFields($fields)
	{
		$this->fields = array_diff_key($this->fields,$fields);
	}
	/**
	 * Auto Generates the fields that can be used for the filters from the Description of the table.
	 * @param bool $reset
	 */
	public function autoGenerateFieldsFromTables($reset=true)
	{
		// If we need to reset the fields, do so
		$reset && $this->fields = array();

		// Loop all the tables, create $fields array
		foreach($this->tbls as $tblHandle => $tblName)
			$this->fields = array_merge($this->fields,$this->generateFieldsFromTable($tblHandle,$tblName));
	}
	/**
	 * @param string $tblHandle The Name for the table used in the query (FROM kwak AS tuut)
	 * @param string $tblName
	 * @return array $fields
	 */
	protected function generateFieldsFromTable($tblHandle,$tblName)
	{
		$fields = array();
		if(!$this->db)
			throw new IWException('No Database Pointer Set');

		$stmt = $this->db->prepare(sprintf("DESC %s",$tblName));
		$stmt->execute();
		while($r = $stmt->fetch())
		{
			// Create a reverse regular expression => loop through dbTypeMap and see if it matches
			if(isset($this->dbTypeMap[$r['Type']]))
				$fields[$r['Field']] = array('display'=>$r['Field'],'type'=>$this->dbTypeMap[$r['Type']],'options'=>false,'tbl'=>$tblHandle);
		}
		return $fields;
	}
	/**#@-*/
	/**
	 * Gets a filter field from a provided query, the query needs to contain a field named VAL and a field named DISP
	 *
	 * Example:
	 * <code>
	 *  // Gets filterable field from the following: (All participants from a certain CRSO)
	 * $cqf->getFilterFromFieldInTable($db,
	 *	'SELECT DISTINCT PARTICIPANT_RELATION_ID as VAL, R.`NAME` as DISP FROM TBL_CRSO_PARTICIPANT_PRODUCT as CP
	 * 	INNER JOIN TBL_RELATION as R ON CP.PARTICIPANT_RELATION_ID = R.RELATION_ID WHERE CRSO_RELATION_ID = 96 ORDER BY R.`NAME`',
	 * 	'PARTICIPANT_RELATION_ID','Participant','CP');
	 * </code>
	 * @param Database $db
	 * @param string $query
	 * @param string $field
	 * @param string $label
	 * @param string|bool $tbl
	 */
	public function getFilterFromFieldInTable($db, $query, $field, $label = '', $tbl = false)
	{
		$label = $label?$label:$field;
		$q = $db->query($query);
		$q->execute();
		$rs = $q->fetchAll();
		$type = (count($rs)>4)?FIELD_INPUT_DROPDOWN:FIELD_INPUT_MULTI;
		$options = array();
		foreach($rs as $r)
			$options[$r->VAL] = $r->DISP;
		$this->setField($field,$label,$type,$options,false,$tbl);
	}
	/**
	 *
	 */
	public function resetInternals()
	{
		if($this->obj)
			$this->obj->resetInternals();

	}
	/**
	 *
	 */
	public function exportQuery($tf,$export='csv')
	{

		if(preg_match('/WEEECustomQueryDynamicReport_/',$this->handle))
			$fileTitle = $this->obj->getName().'_Export';
		else
			$fileTitle = $this->handle.'_Export';
		$ds = $tf->getFilteredDS();
		switch($export)
		{
			case 'csv':
				require_once(IWARE.'IWCSVWriter.php');
				$writer = new IWCSVWriter($ds);
				$rv = $writer->getString();
				$contentType = 'text/plain';
				$fileTitle .= '.txt';
			break;
			case 'xml':
				require_once(IWARE.'IWXMLWriter.php');
				$writer = new IWXMLWriter($ds);
				$rv = $writer->getFormattedData();
				$contentType = 'application/octet-stream';
				$fileTitle .= '.xml';
			break;
			case 'masterdata':
				// This now happens through SOAP
			break;
		}
		header("Cache-control: ");
	    header("Pragma: ");
	    header(sprintf("Content-Type: %s",$contentType));
	    header(sprintf("Content-Disposition: attachment; filename=\"%s\" ", $fileTitle));
	    header("Content-transfer-Encoding: binary\n");
	    echo $rv;
		exit();
        }

        public function displayAdditionalFilterSettings() {
           return $this->obj->displayAdditionalFilterSettings();
        }

        public function getShowExportFunctions() {
            return $this->obj->getShowExportFunctions();
        }
}
/**
 * @author Marten Koedam <marten@dalines.org>
 * @package IW
 * @subpackage QueryBuilder
 * @since 21 december 2006
 */
class IWCustomQueryFilterDisplay
{
	/**#@+
	 * @access protected
	 */
	protected $cqf;
	protected $langIf = null;
	/**
	 * This is for the stuff used in IWCustomQueryFactory.js
	 * @var array
	 */
	protected $typeMap = array(
		FIELD_INPUT_TEXT => 'text',
		FIELD_INPUT_DROPDOWN => 'select',
		FIELD_INPUT_MULTI => 'radio',
		FIELD_INPUT_NUMBER => 'number',
		FIELD_INPUT_PERIOD => 'period',
		);
	protected $group = false;
	/**#@-*/
	/**
	 * @param IWCustomQueryFactory $cqf
	 * @param LanguageInterface|null $langIf
	 */
	public function __construct(IWCustomQueryFactory $cqf, LanguageInterface $langIf = null)
	{
		$this->cqf = $cqf;
		$this->langIf = $langIf;
	}
	/**
	 * This is a function to translate texts.
	 * <p>If a LanguageInterface has been provided it will be translated,
	 * else it will just return the original text
	 * @param string $txt
	 */
	protected function getText($txt)
	{
		$this->langIf && $txt = $this->langIf->getText($txt);
		return $txt;
	}
	/**
	 * Displays a filter form based on the filters provided and filters selected
	 * @todo make pretty
	 */
	public function display($hidden='')
	{
		$fields = $this->cqf->getFields();
		if(!count($fields))
			return '';

		$langIf = IWRegistry::registry('langIf');

		$group = $this->renderGroupBySelect();
		$rv = '
			<script type="text/javascript">
				function saveQuery()
				{


					MessageBox.prototype.Question(\''.$langIf->getText('Save as Preset').'\',\''.$this->getSaveOptions().'\', function(){

						CQF.savePreset($$(\'preset_name\').value,$$(\'is_default_chk\').checked,$$(\'is_public_chk\').checked);

					});


				}
			</script>
			<form action="'.$_SERVER['REQUEST_URI'].'" method="post" id="custom_query_form">
			<input type="hidden" name="filter_query" value="true" />
			'.$hidden.'
			<fieldset>
				<legend>'.$langIf->getText('Filters').'</legend>
				<table class="filtertable" id="filtertable" cellpadding="2">
					<tbody>
					<tr><th></th><td></td><td width="100%"></td><td></td></tr>
					</tbody>
					<tfoot><td colspan="4" style="text-align: right;">'.$langIf->getText('Add filter').' '.$this->renderFilterSelect().'</td></tfoot>
				</table>
			</fieldset>
			<table width="100%">
				<tr><td>'.($group?$langIf->getText('Group Result By').' '.$group.' <input type="checkbox" value="1" name="descending" disabled="disabled" /> <label for="descending">'.$langIf->getText('Descending').'</label>':'').
					($group?' <input type="checkbox" value="1" '.(empty($_REQUEST['hide_empty_sets'])?'':'checked="checked"').' name="hide_empty_sets" /> <label for="hide_empty_sets">'.$langIf->getText('Hide empty sets').'</label>':'').
					'</td><td style="text-align: right;">'.$this->renderPresetSelect().' <input type="button" onclick="saveQuery();" value="'.$langIf->getText('Save as Preset').'"> <input type="submit" value="'.$langIf->getText('Update').'"></td></tr>
                        <tr><td colspan="2">'.$this->cqf->displayAdditionalFilterSettings().'</td></tr>
                        </table>
			</form>
			';

		return $rv;
	}
	/**
	 *
	 */
	private function getSaveOptions()
	{
		$show_public_opt = sprintf('
				<tr>
					<td>%s</td>
					<td><input type="checkbox" value="1" name="is_public_chk" /></td>
				</tr>
			',
			$this->getText('This preset is publicly available')
			);
		$rv = sprintf(
			'%s<br />
			%s
			<table cellpadding="4">
				<tr>
					<td>%s</td>
					<td><input type="text" value="" name="preset_name" /></td>
				</tr>
			%s
				<tr>
					<td>%s</td>
					<td><input type="checkbox" value="1" name="is_default_chk" /></td>
				</tr>
			</table>
			%s
			',
			$this->getText('This will save the current settings as a preset'),
			(($_SESSION['session_group_group_id']!=GROUP_SYSADMIN)?'<input type="hidden" value="0" name="is_public_chk" />':''),
			$this->getText('Name of the preset'),
			(($_SESSION['session_group_group_id']==GROUP_SYSADMIN)?$show_public_opt:''),
			$this->getText('Use this preset as default'),
			$this->getText('Do you want to save the current settings?')
			);
		return jsescape($rv);
	}

	/**
	 * Gets the javascript part that is neccessary to generate previously selected filters
	 * This way you dont have to reselect everything if you wanna make your custom query a little more detailed.
	 * @return string
	 */
	public function getSelectedFilters()
	{
		$rv = '';
		$fields = $this->cqf->getFields();
		$settings = $this->cqf->getSettings();
		foreach($fields as $fieldHandle => $values)
		{
			if(!isset($settings[$fieldHandle]))
				continue;
			if($values['type']==FIELD_INPUT_MULTI)
			{
				$vals = '';
				foreach($settings[$fieldHandle] as $value)
					$vals .= '"'.$value.'": true, ';
				$rv .= sprintf('{handle: "%s", value: {%s}, mode: "%s"}, ',$fieldHandle, substr($vals,0,-2), '');
			}
			else
				foreach($settings[$fieldHandle] as $value) // Check for type == if multi, only once
					$rv .= sprintf('{handle: "%s", value: "%s", mode: "%s"}, ',$fieldHandle, $value, (!empty($settings[$fieldHandle.'_mode'])?$settings[$fieldHandle.'_mode']:''));
		}
		$filters = $rv?'['.substr($rv,0,-2).']':'false';
		$groupby = empty($settings['groupby'])?'false':"'".$settings['groupby']."'";
		$descending = empty($settings['descending'])?'false':'true';
		return 'addEvent(window, "load",function(){CQF.rebuildFilters('.$filters.','.$groupby.','.$descending.')});';
	}
	/**
	 * Gets the properties of all available filters.
	 * These are used to build the correct interface on the fly in javascript
	 * @return string $rv
	 */
	public function getProperties()
	{
		$rv = 'var properties={';
		$fields = $this->cqf->getFields();
		if(!count($fields))
			return $rv.'};';
		foreach($fields as $value => $content)
		{
			$rv .= sprintf('"%s": { type: "%s", label: "%s"%s }, ',
				$value,
				$this->typeMap[$content['type']],
				$content['display'],
				((!in_array($content['type'],array(FIELD_INPUT_TEXT,FIELD_INPUT_NUMBER,FIELD_INPUT_PERIOD)))?$this->getOptions($content['options']):'')
				);
		}
		$rv = substr($rv,0,-2).'};';
		return $rv;
	}
	/**
	 * Gets the javascript version of the options for a certain filter field
	 * @param array $options
	 * @return string $rv
	 */
	protected function getOptions($options)
	{
		if(!count($options))
			return '';
		$rv = ', options: [';
		foreach($options as $value => $label)
			$rv .= '{value: "'.$value.'", text: "'.$label.'"}, ';
		return substr($rv,0,-2).']';
	}
	/**
	 * Renders the filter selection.
	 * The onchange event triggers the addFilter on the JavaScript version of IWCustomQueryFactory
	 * @return string
	 */
	protected function renderFilterSelect()
	{
		$select = new Form_Select('filter');
		$select->addOption('','');
		$fields = $this->cqf->getFields();
		$required = $this->cqf->getRequiredFields();

		foreach($fields as $value => $values)
			$select->addOption($this->getText($values['display']).(isset($required[$value])?'*':''),$value);
		$select->onChange('CQF.addFilter(this);');
		return $select->getHtml();
	}
	/**
	 * Renders a select for the group by
	 * <p>This is based on all available filters,
	 * but only uses the non-TEXT types
	 * @return string
	 */
	protected function renderGroupBySelect()
	{
		$select = new Form_Select('groupby');
		$select->addOption('','');
		$fields = $this->cqf->getFields();
		$len = false;
		foreach($fields as $value => $values)
		{
			if(!in_array($values['type'],array(FIELD_INPUT_TEXT,FIELD_INPUT_NUMBER,FIELD_INPUT_PERIOD)))
			{
				$select->addOption($this->getText($values['display']),$value);
				$len = true;
			}
		}
		if(!$len)
			return '';
		$select->onChange('CQF.setGroupBy(this);');
		return $select->getHtml();
	}
	/**
	 *
	 */
	protected function renderPresetSelect()
	{
		$fields = array();
		$select = new Form_Select('preset');
		$select->addOption('','');
		$fields = $this->cqf->getPresets();
		if(!$fields)
			return '';

		foreach($fields as $row)
			$select->addOption($row->PRESET_NAME,$row->PRESET_ID);

		$select->onChange('CQF.setPreset(this);');
		return $select->getHtml();
	}
	/**
	 *
	 */
	public function renderPaging($rows, $group)
	{
		if(!$this->cqf->obj)
			return '';
		if(!$this->cqf->obj->limit)
			return '';

		if(!$rows)
			return '';

		if(!array_key_exists($group, $this->cqf->obj->currentpage))
			return '';

		$this->group = $group;
		$cp = $this->cqf->obj->currentpage[$group];

		if(($pages = ceil($rows/$this->cqf->obj->limit))==1)
			return '';
		$rpl = '';
		$rv = '<div style="text-align: right; margin-bottom: 2px;">[ %s ]</div>';

		if(!empty($_POST['orderBy']))
		{
			for($i = 1;$i<=$pages; $i++)
			{
				$rpl .= (($i-1)==$cp)?'<strong>'.$i.'</strong> ':sprintf('<a href="#%s" onclick="CQF.gotoPage(\'%s\',%s,\'%s\',\'%s\')">%s</a> ',$group,md5($group),($i-1),
					$_POST['orderBy'],
					$_POST['direction'],
					$i);
			}
		}
		else
		{
			for($i = 1;$i<=$pages; $i++)
				$rpl .= (($i-1)==$cp)?'<strong>'.$i.'</strong> ':sprintf('<a href="#%s" onclick="CQF.gotoPage(\'%s\',%s)">%s</a> ',$group,md5($group),($i-1),$i);
		}
		return sprintf($rv,$rpl);
	}
	/**
	 * This was added for sorting purposes.
	 */
	public function tfHeaderProcessor(&$args)
	{
		if(!$this->cqf->obj)
			return '';
		$rv = $this->cqf->obj->tfHeaderProcessor($args);
		$direction = 'ASC';
		$img = '';
		if(!empty($_POST['orderBy']))
		{
			if($_POST['orderBy']==$args['head'])
			{
				$direction = ($_POST['direction']=='ASC')?'DESC':'ASC';
				$img = sprintf('<img src="/%simages/i-w/if_arrow_%s.gif" align="absmiddle" /> ',
					SITE_PATH,
					(($_POST['direction']=='ASC')?'down':'up')
					);
			}
		}
		if($this->group)
		{
			$cp = $this->cqf->obj->currentpage[$this->group];
			$args['headerText'] = sprintf('%s<a href="javascript:CQF.orderBy(\'%s\',\'%s\',\'%s\',\'%s\')">%s</a>',
				$img,
				$args['head'],
				$direction,
				md5($this->group),
				$cp,
				$args['headerText']);
		}
		else
		{
			$args['headerText'] = sprintf('%s<a href="javascript:CQF.orderBy(\'%s\',\'%s\')">%s</a>',
				$img,
				$args['head'],
				$direction,
				$args['headerText']);
		}
		return $rv?$rv:'';
	}
}
/**
 * @abstract
 */
abstract class IWCustomQueryObject
{
	public $limit = false;
	public $currentpage = array('default'=>0);
        public $rowCntQuery = false;
        protected $showExportFunctions = true;

	public function __call($method,$args)
	{
		return false; // Dont flunk if function doesnt exist.
	}
	/**
	 * Default server side order option. This will only be used if $this->tfAllowSorting() return true;
	 */
	public function getSQLOrderBy()
	{
		$order = empty($_POST['orderBy'])?false:$_POST['orderBy'];
		if(!$order)
			return false;
		$direction = empty($_POST['direction'])?'ASC':$_POST['direction'];

		return ' ORDER BY '.$order.' '.$direction;
	}

	public function tfHeaderProcessor(&$args)
	{
		$args['headerText'] = IWRegistry::registry('langIf')->getText($args['head']);
	}

	public function tfDescription()
	{
		return '';
	}

	public function getFilter($filter)
	{
		return $this->$filter();
	}

	public function getShowExportFunctions() {
		return $this->showExportFunctions;
	}

	/**#@+
	 * Default filter function for Custom Query's
	 * @access private
	 */
	/**
	 *
	 */
	private function filterCRSO()
	{
		require_once(IWARE.'group.php');
		$where = '';
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_ACCOUNTANT:
				$where = ' WHERE ac.ACCOUNTANT_RELATION_ID = '.$_SESSION['accountant_relation_id'].' ';
			break;
		}
		$rv = 'SELECT ac.CRSO_RELATION_ID as VAL, r.`NAME` as DISP
				FROM TBL_ACCOUNTANT_CRSO as ac
				INNER JOIN TBL_RELATION as r
					ON ac.CRSO_RELATION_ID = r.RELATION_ID
				'.$where.'
				ORDER BY r.`NAME`';
		return $rv;
	}
	/**
	 *
	 */
	private function filterProduct()
	{
		$join = isset($_REQUEST['CRSO_RELATION_ID'])?
			'INNER JOIN TBL_CRSO_PRODUCT as cp ON p.PRODUCT_ID = cp.PRODUCT_ID and CRSO_ID IN('.implode(',',$_REQUEST['CRSO_RELATION_ID']).') ':'';
		return 'SELECT p.PRODUCT_ID as VAL, p.`NAME` as DISP
			FROM TBL_PRODUCT as p
			'.$join.'
			ORDER BY p.`NAME`';
	}
	/**#@-*/
}
/**#@+
 * Different types of filter inputs
 */
define('FIELD_INPUT_TEXT', 1);
define('FIELD_INPUT_DROPDOWN', 2);
define('FIELD_INPUT_MULTI', 3);
define('FIELD_INPUT_NUMBER',4);
define('FIELD_INPUT_PERIOD',5);
/**#@-*/
?>
