<?php
require_once(IWARE.'AbstractObject.php');
/**
 * @package IW
 * @subpackage CustomQuery
 * @since 12-june-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class IWCustomQueryPreset extends AbstractObject
{
	/**#@+
	 * @access public
	 */
	public $id;
	public $name;
	public $custom_query;
	public $is_public;
	public $group_by;
	public $descending;
	/**#@-*/

	/**
	 * @access protected
	 * @var array $filters (IWUserCustomQueryFilter[, IWUserCustomQueryFilter[, etc]]);
	 */
	protected $filters = array();

	/**
	 * @param int $id
	 * @return bool
	 */
	public function set($id=null)
	{
		$id || $id = $this->id;

		$sql = "SELECT CUSTOM_QUERY, `NAME`, IS_PUBLIC,
				GROUP_BY, DESCENDING
			FROM TBL_CUSTOM_QUERY_PRESET
			WHERE PRESET_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		$this->id = $id;
		$this->custom_query = $r->CUSTOM_QUERY;
		$this->name = $r->NAME;
		$this->is_public = $r->IS_PUBLIC;
		$this->group_by = $r->GROUP_BY;
		$this->descending = $r->DESCENDING;


		$this->filters = array();

		$sql = "SELECT FILTER_INDEX as `index`,
				FILTER_TYPE as `type`, FILTER_VALUE as `value`, FILTER_HANDLE as `handle`
			FROM TBL_CUSTOM_QUERY_PRESET_FILTER
			WHERE PRESET_ID = ?
			ORDER BY FILTER_INDEX";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		while($r = $q->fetchAssoc())
		{
			$props = $r;
			$props['preset'] = $this;
			$this->filters[] = new IWCustomQueryPresetFilter($this->db,$props);
		}

		return $this->complete = true;
	}
	/**
	 *
	 */
	public function getPresets($user_id,$custom_query)
	{
		$sql = "SELECT CQP.PRESET_ID, `NAME` as PRESET_NAME
			FROM TBL_CUSTOM_QUERY_PRESET as CQP
			LEFT JOIN TBL_USER_CUSTOM_QUERY as CQU
				ON CQP.PRESET_ID = CQU.PRESET_ID
			WHERE (IS_PUBLIC = ?
				OR CQU.USER_ID = ?) AND CUSTOM_QUERY = ?
			ORDER BY PRESET_NAME";
		$q = $this->db->query($sql);
		$q->bind_values('Yes',$user_id,$custom_query);
		$q->execute();
		return $q;
	}

	public function hasAccess($user_id,$preset_id=null)
	{
		$sql = "SELECT CQP.PRESET_ID, `NAME` as PRESET_NAME
			FROM TBL_CUSTOM_QUERY_PRESET as CQP
			LEFT JOIN TBL_USER_CUSTOM_QUERY as CQU
				ON CQP.PRESET_ID = CQU.PRESET_ID
			WHERE CQP.PRESET_ID = ?
				AND (IS_PUBLIC = ? OR CQU.USER_ID = ?)
			ORDER BY PRESET_NAME";
		$q = $this->db->query($sql);
		$q->bind_values($preset_id,'Yes',$user_id);
		$q->execute();
		$r = $q->fetchAll();
		return (bool)count($r);
	}

	/**
	 * @param array $filters
	 */
	public function fillFilters($filters)
	{
		$this->filters = array();
		foreach($filters as $index=>$props)
		{
			$props['preset'] = $this;
			$props['index'] = $index;
			$this->filters[] = new IWCustomQueryPresetFilter($this->db,$props);
		}
	}
	/**
	 * Adds a filter as the next indexed one to the local variable $filters
	 * @param IWUserCustomQueryFilter $cqf
	 */
	public function addFilter(IWCustomQueryPresetFilter $cqf)
	{
		$this->filters[] = $cqf;
	}
	/**
	 * Sets the filters
	 * @param array $filters (IWUserCustomQueryFilter[, IWUserCustomQueryFilter[, etc]]);
	 */
	public function setFilters($filters)
	{
		$this->filters = $filters;
	}
	/**
	 * Gets all the filters that are current registered to the local variable $filters
	 */
	public function getFilters()
	{
		return $this->filters;
	}
	/**
	 * Inserts the Prevs and the Filter
	 */
	function insert()
	{
		$this->deleteFilters();
		parent::insert();
		foreach($this->filters as $index => $cqf)
		{
			$cqf->preset = $this;
			$cqf->index = $index;
			$cqf->insert();
		}
	}
	/**
	 * Removes all the filters for the current user and query
	 */
	function deleteFilters()
	{
		$sql = "DELETE FROM TBL_CUSTOM_QUERY_PRESET_FILTER
			WHERE PRESET_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->id);
		return $q->execute();
	}
	/**
	 *
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_CUSTOM_QUERY_PRESET';
		$this->objectmap['primary']['id'] = 'PRESET_ID';
		$this->objectmap['properties']['custom_query'] = 'CUSTOM_QUERY';
		$this->objectmap['properties']['name'] = 'NAME';
		$this->objectmap['properties']['is_public'] = 'IS_PUBLIC';
		$this->objectmap['properties']['group_by'] = 'GROUP_BY';
		$this->objectmap['properties']['descending'] = 'DESCENDING';
	}
}

/**
 * @package IW
 * @subpackage CustomQuery
 * @since 8-jan-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class IWUserCustomQuery extends AbstractObject
{
	/**#@+
	 * @access public
	 */
	public $user;
	public $preset;
	public $is_default;
	/**#@-*/

	function insert($graceful=false)
	{
		if($this->is_default=='Yes')
			$this->removePreviousDefault();
		parent::insert($graceful);
	}

	function removePreviousDefault()
	{
		$this->preset->isComplete() || $this->preset->set();
		$sql = "SELECT PRESET_ID FROM TBL_CUSTOM_QUERY_PRESET
			WHERE CUSTOM_QUERY = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->preset->custom_query);
		$q->execute();
		$r = $q->fetchAll();
		if(!count($r))
			return;

		$sql = "UPDATE TBL_USER_CUSTOM_QUERY
			SET IS_DEFAULT = ?
			WHERE USER_ID = ? AND PRESET_ID = ?";
		$q = $this->db->query($sql);
		foreach($r as $row)
		{
			$q->bind_values('No',$this->user->id,$row->PRESET_ID);
			$q->execute();
		}
	}

	function getDefaultPresetId($custom_query,$user_id=null)
	{
		$user_id || $user_id = $this->user->id;

		$sql = "SELECT UCQ.PRESET_ID
			FROM TBL_USER_CUSTOM_QUERY as UCQ
			INNER JOIN TBL_CUSTOM_QUERY_PRESET as CQP
				ON UCQ.PRESET_ID = CQP.PRESET_ID
			WHERE USER_ID = ?
				AND IS_DEFAULT = ?
				AND CUSTOM_QUERY = ?";
		$q = $this->db->query($sql);
		$q->bind_values($user_id,'Yes',$custom_query);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;
		return $r->PRESET_ID;
	}

	/**
	 *
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_USER_CUSTOM_QUERY';
		$this->objectmap['primary']['user->id'] = 'USER_ID';
		$this->objectmap['primary']['preset->id'] = 'PRESET_ID';
		$this->objectmap['properties']['is_default'] = 'IS_DEFAULT';
	}
}
/**
 * @package IW
 * @subpackage CustomQuery
 * @since 8-jan-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class IWCustomQueryPresetFilter extends AbstractObject
{
	/**#@+
	 * @access public
	 */
	public $preset;
	public $index;
	public $handle;
	public $type;
	public $value;
	/**#@-*/
	/**
	 * @access protected
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_CUSTOM_QUERY_PRESET_FILTER';
		$this->objectmap['primary']['preset->id'] = 'PRESET_ID';
		$this->objectmap['primary']['index'] = 'FILTER_INDEX';
		$this->objectmap['properties']['handle'] = 'FILTER_HANDLE';
		$this->objectmap['properties']['type'] = 'FILTER_TYPE';
		$this->objectmap['properties']['value'] = 'FILTER_VALUE';
		$this->objectmap['inserttype'] = 'REPLACE';
	}
}
?>
