<?php
/**
 * Based on Zend.php
 * @package IW
 * @subpackage Registry
 * @since 14-dec-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class IWRegistry
{
	/**
	 *
	 */
	static private $_registry = array();

    /**
     * Singleton Pattern
     */
    private function __construct()
    {}

	/**
     * Registers a shared object.
     *
     * @param   string      $name The name for the object.
     * @param   object      $obj  The object to register.
     * @throws  IWException
     * @return  void
     */
    static public function register($name, $obj)
    {
        if (!is_string($name)) {
            throw new IWException('First argument $name must be a string.');
        }

        // don't register the same name twice
        if (array_key_exists($name, self::$_registry)) {
           throw new IWException("Object named '$name' already registered.  Did you mean to call registry()?");
        }

        // only objects may be stored in the registry
        if (!is_object($obj)) {
           throw new IWException("Only objects may be stored in the registry.");
        }

        $e = '';
        /*
        // an object can only be stored in the registry once
        foreach (self::$_registry as $dup=>$registeredObject) {
            if ($obj === $registeredObject) {
                $e = "Duplicate object handle \"$name\" already exists in the registry as \"$dup\".";
                break;
            }
        }
        */

        /**
         * @todo throwing exceptions inside foreach could cause leaks, use a workaround
         *       like this until a fix is available
         */
        if ($e) {
            throw new IWException($e);
        }

        self::$_registry[$name] = $obj;
    }

    /**
     * Retrieves a registered shared object, where $name is the
     * registered name of the object to retrieve.
     *
     * If the $name argument is NULL, an array will be returned where
         * the keys to the array are the names of the objects in the registry
         * and the values are the class names of those objects.
     *
     * @see     register()
     * @param   string      $name The name for the object.
     * @throws  IWException
     * @return  object      The registered object.
     */
    static public function registry($name=null)
    {
        if ($name === null) {
            $registry = array();
            foreach (self::$_registry as $name=>$obj) {
                $registry[$name] = get_class($obj);
            }
            return $registry;
        }

        if (!is_string($name)) {
            throw new IWException('First argument $name must be a string, or null to list registry.');
        }

        if (!array_key_exists($name, self::$_registry)) {
           throw new IWException("No object named \"$name\" is registered.");
        }

        return self::$_registry[$name];
    }


    /**
     * Returns TRUE if the $name is a named object in the
     * registry, or FALSE if $name was not found in the registry.
     *
     * @param  string $name
     * @return boolean
     */
    static public function isRegistered($name)
    {
        return isset(self::$_registry[$name]);
    }
}
?>
