<?php
/**
 * Singleton SessionHanlder
 * @author Marten Koedam <marten@basesoft.com>
 * @package I-Ware
 * @subpackage Core
 * @version $Id$
 */
class IWSessionHandler
{
	var $langIf = false;
	var $db = false;
	protected static $instance;

	var $locale_default = 'en_GB.UTF-8';

	/**
	 * DO NOT USE CONSTRUCTOR!!!
	 * This is a singleton.. Use the createSessionHandler() method
	 * @private
	 */
	function IWSessionHandler($db = false)
	{
		//define('SESSION_HANDLER_INSTANCE_MADE', true);

		$this->db = $db;
		$this->_container = new SessionContainerDB($this->db, 'TBL_SESSION_DATA');
		session_set_save_handler(
				array($this->_container, 'open'),
				array($this->_container, 'close'),
				array($this->_container, 'read'),
				array($this->_container, 'write'),
				array($this->_container, 'destroy'),
				array($this->_container, 'gc'));

		session_set_cookie_params(0, '/' . SITE_PATH);
		session_start();

		if (!isset ($_SESSION['session_user_id']))
			return $this->_setDefault();
		return $this->_validate();
	}
	/**
	 * The Singleton Factory method
	 */
	static function & createSessionHandler($db)
	{
		if (!isset (self :: $instance))
			self :: $instance = new IWSessionHandler($db);
		return self :: $instance;
	}
	/**
	 *
	 */
	function setLanguageInterface($langIf)
	{
		$this->langIf = $langIf;
	}
	/**
	 *
	 */
	function getText($txt)
	{
		return ($this->langIf) ? $this->langIf->getText($txt) : $txt;
	}
	/**
	 *
	 */
	function setSessionLocale()
	{
		$locales = array (LC_TIME, LC_NUMERIC);

		if (!isset ($_SESSION['locales']))
		{
			$db_locale = $this->_getLocale($this->db);
			debug('Setting session locales to ' . $db_locale);

			foreach ($locales as $locale)
				$_SESSION['locales'][$locale] = $db_locale;
		}

		foreach ($locales as $locale)
		{
			if (empty ($_SESSION['locales'][$locale]) || !setlocale($locale, $_SESSION['locales'][$locale]))
			{
				//debug(sprintf('Failed to setlocale [%s] to [%s]. Falling back to default [%s]', $locale, $_SESSION['locales'][$locale], $this->locale_default));
				setlocale($locale, $this->locale_default);
			}
		}

		//debug('LC_TIME: '.setlocale(LC_TIME, "0"));
		//debug('LC_NUMERIC: '.setlocale(LC_NUMERIC, "0"));
	}

	function _getLocale($db) {
		$sql = "SELECT l.LANGUAGE_LOCALE as LOCALE
			FROM TBL_INFO_PERSON ip
				INNER JOIN TBL_USER u
					ON u.INFO_PERSON_ID = ip.INFO_PERSON_ID
				INNER JOIN TBL_LANGUAGE l
					ON ip.LANGUAGE_ID = l.LANGUAGE_ID
				WHERE u.USER_ID = ?";

		$q = $db->query($sql);
		$q->bind_values($_SESSION['session_user_id']);

		if (!$q->execute() || !($r = $q->fetch()))
			return false;

		$locale = $r->LOCALE;

		return $locale;
	}

	/**
	 * login function
	 * @param string user Username
	 * @param string passwd Password for the user. Is empty string if we log in as a different user from the sysadmin or operator universe
	 * @param string thrownException
	 * @param integer login_type 0 = normal login, 1 = login from sysadmin/operator without password, 2 = login with token link from email
	 */
	function doLogin($user, $passwd, & $thrownException, $login_type = 0)
	{
		$log = getLog();

		$is_operator = false;

		$login_ok = false;

		if ($login_type == LOGIN_SWITCH)
		{
			$is_sysadmin = isset ($_SESSION['session_group_group_id']) && ($_SESSION['session_group_group_id'] == GROUP_SYSADMIN);

			switch($_SESSION['session_group_group_id']) {

				case GROUP_SYSADMIN:
					$login_ok = true;
					break;

				case GROUP_CRSO:

					$crso_id = $_SESSION['crso_relation_id'];
					$product_id = $_SESSION['selected_product_id'];

					// check if this crso has the country setting HasCrsoParticipantLogin enabled
					$rel = new Relation($this->db, $crso_id ,true);
					$country = new Country($this->db, $rel->getCountry(), true);
					if(!($country->has(HasCrsoParticipantLogin, $product_id))) {
						throw new IWException('Crso does not have HasCrsoParticipantLogin');
					}

					// check if the requested user is part of the crso _and_ has ALLOW_CRSO_LOGIN enabled
					$sql = "select * from TBL_USER u
					  INNER JOIN TBL_RELATION_USER ru on u.USER_ID = ru.USER_ID
					  INNER JOIN TBL_RELATION r on r.RELATION_ID = ru.RELATION_ID
					  INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT cpp
						ON cpp.PARTICIPANT_RELATION_ID = r.RELATION_ID
						   AND cpp.CRSO_RELATION_ID = ?
						   AND cpp.PRODUCT_ID = ?
					WHERE u.USER_NAME = ? AND u.ALLOW_CRSO_LOGIN = 1";

					$q = $this->db->query($sql);
					$q->bind_values($crso_id, $product_id, $user);
					$q->execute();

					if ($q->fetch()) {
						$login_ok = true;
					}

				default:
					break;

			}

			if (!$login_ok) {
				$log->alert('User Login `' . $user . '`. ' . $this->getText('Unauthorized to login as a different user'));
				header('HTTP/1.1 403 Forbidden');
				throw new IWUserException('Permission Denied');
			}

			$log->info('doLogin(): LOGIN_SWITCH: user_id ['.$_SESSION['session_user_id'].'] switching to user_name '.$user);
		}

		session_regenerate_id(false); // regenerate session ID but don't delete the old data
		$this->_setDefault(); // Initialize the values

		/// Get all errors
		$this->getText('Something went wrong executing the query');
		$this->getText('Incorrect username and/or password');
		$this->getText('This is not an active user');
		// End errors

		$sql = "SELECT USER_ID, USER_PASSWORD, USER_ACTIVE, INFO_PERSON_ID, USER_LAST_LOGIN, USER_SALT
					FROM TBL_USER
					WHERE BINARY USER_NAME = ?";
		$q = $this->db->query($sql);
		$q->bind_values($user);
		if (!$q->execute())
		{
			$thrownException .= $this->getText('Something went wrong executing the query');
			return false;
		}
		if (!($r = $q->fetch()))
		{
			$thrownException .= $this->getText('Incorrect username and/or password');
			$log->info('User Login `' . $user . '` failed... User does not exist');
			return false;
		}
		require_once (IWARE . 'UserRestriction.php');
		$ur = new UserRestriction($this->db);
		if (!$ur->validateUser($r->USER_ID))
		{
			$thrownException .= $this->getText('Insufficient Credentials');
			$log->info('User Login `' . $user . '` failed... Incorrect ip address for username');
			return false;
		}

		$skip_password_check = array (// array with the type of logins that don't need password
			LOGIN_SWITCH,
			LOGIN_TOKEN
		);
		if (!in_array($login_type, $skip_password_check))
		{
			if (CRYPT_BLOWFISH == 1)
			{
				$salt = $r->USER_SALT;
				$pass_check = crypt($passwd,$salt);
			}
			else
				$pass_check = sha1($passwd);

			if ($pass_check != $r->USER_PASSWORD) {
				$thrownException .= $this->getText('Incorrect username and/or password');
				$log->info('User Login `' . $user . '` failed... Incorrect password for username');
				return false;
			}
		}

		if (!$r->USER_ACTIVE)
		{
			$thrownException .= $this->getText('This is not an active user');
			$log->info('User Login `' . $user . '` failed... Not Active');
			return false;
		}

		// Start setting the session...
		$sql = "SELECT G.GROUP_NAME, G.GROUP_ID, GC.GROUP_COMPANY_ID,
						U.USER_ID, U.USER_NAME, U.USER_PASSWORD, U.USER_DEFAULT_PAGE, IP.LANGUAGE_ID,
						U.USER_LAST_LOGIN, U.LAST_PASSWORD_UPDATE,
						C.COMPANY_ID, C.COMPANY_NAME
					FROM TBL_USER as U
					INNER JOIN TBL_GROUP_USER as GU
						USING(USER_ID)
					INNER JOIN TBL_GROUP as G
						USING(GROUP_ID)
					INNER JOIN TBL_INFO_PERSON as IP
						ON U.INFO_PERSON_ID = IP.INFO_PERSON_ID
					INNER JOIN TBL_GROUP_COMPANY as GC
						ON IP.COMPANY_ID = GC.COMPANY_ID
							AND G.GROUP_ID = GC.GROUP_ID
					INNER JOIN TBL_COMPANY as C
						ON IP.COMPANY_ID = C.COMPANY_ID
					WHERE U.USER_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($r->USER_ID);
		if (!$q->execute()) {
			$thrownException .= $this->getText('Something went wrong executing the query');
			return false;
		}
		if (!($r = $q->fetch()))
		{
			$thrownException .= $this->getText('Incorrect username and/or password');
			$log->info('User Login `' . $user . '` failed... Could not get GROUP info for user. ');
			return false;
		}

		$_SESSION['session_group_group_id'] = $r->GROUP_ID;
		$_SESSION['session_group_company_id'] = $r->GROUP_COMPANY_ID;
		$_SESSION['session_group_group_name'] = $r->GROUP_NAME;
		$_SESSION['session_user_id'] = $r->USER_ID;
		$_SESSION['session_user_language_id'] = $r->LANGUAGE_ID;
		$_SESSION['session_user_name'] = $r->USER_NAME;
		//$_SESSION['session_user_company_id'] = $r->COMPANY_ID;//
		//$_SESSION['session_user_company_name'] = $r->COMPANY_NAME;//
		//$_SESSION['session_user_default_page'] = $r->USER_DEFAULT_PAGE;//
		$_SESSION['user_last_login'] = $r->USER_LAST_LOGIN;
		unset ($_SESSION['locales']); // set no locale so it will filled from the db again
		$this->setSessionLocale();
		$this->setTranslatorLanguages($r->USER_ID);

		$log->info('Login `' . $user . '` succesful...');
		return true;
	}

	/**
	 *
	 */
	function login($user, $passwd, & $thrownException)
	{
		return $this->doLogin($user, $passwd, $thrownException, LOGIN_NORMAL);
	}

	/**
	 * function that logs a sysadmin or operator as a different user, without needing to supply a password
	 */
	function loginAsUser($user, & $thrownException)
	{
		return $this->doLogin($user, '', $thrownException, LOGIN_SWITCH); // we don't provide a password
	}

	/**
	 * function that logs a user that clicks on the link that he received in the email, after registering or requesting a new password
	 */
	function loginFromEmail($token, & $thrownException)
	{
		$number_of_tokens = $this->expirePreviousUserTokens($token);
		if (!$number_of_tokens)
		{
			alert("Inexistent token (token=$token)");
			header('HTTP/1.1 403 Forbidden');
			throw new IWUserException("Invalid login link: Link doesn't exist");
		}

		// get the user using the $token information. we check if the token is still active or it's not expired yet
		$sql = "SELECT U.USER_NAME as USER_NAME, UOTL.ACTIVE as ACTIVE, DATEDIFF(CURDATE(), UOTL.GENERATED) as AGE
					FROM TBL_USER AS U INNER JOIN TBL_USER_ONE_TIME_LOGIN AS UOTL USING(USER_ID)
					WHERE UOTL.TOKEN = ?
					ORDER BY UOTL.GENERATED DESC LIMIT 1";
		$q = $this->db->query($sql);
		$q->bind_values($token);
		if (!$q->execute())
		{
			alert("Couldn't execute the one-time-login token query... (token=$token)");
			header('HTTP/1.1 403 Forbidden');
			throw new IWUserException('Permission Denied');
		}
		if (!($r = $q->fetch()))
		{
			alert("Inexistent token (token=$token)");
			header('HTTP/1.1 403 Forbidden');
			throw new IWUserException("Invalid login link: Link doesn't exist");
		}

		if ($r->AGE > LOGIN_TOKEN_LIFETIME)
		{
			alert("Token '$token' has expired " . ($r->AGE - LOGIN_TOKEN_LIFETIME) . " days ago.");
			header('HTTP/1.1 403 Forbidden');
			throw new IWUserException("Invalid login link: Link has expired");
		}
		if ($r->AGE < 0) // token generation date is in the future
		{
			alert("Token '$token' has the generation date in the future. It can be used in " . - $r->AGE . " days.");
			header('HTTP/1.1 403 Forbidden');
			throw new IWUserException("Invalid login link: Link doesn't exist");
		}
		if ($r->ACTIVE == 0) // token is not active. it it was not expired and inactive, it means it was used before
		{
			alert("Token '$token' has is not expired but is inactive. It was probably used before");
			header('HTTP/1.1 403 Forbidden');
			throw new IWUserException("The login link was opened twice. Check other browser tabs or request a new password.");
		}

		return $this->doLogin($r->USER_NAME, '', $thrownException, LOGIN_TOKEN);
	}

	/**
	 * function that checks if a user has more then one active tokens, and expires all of them, except the most recent one
	 * @param string $token The token that we use to check for what user we need to expire the old tokens
	 * @return bool The number of tokens that a user has
	 */
	private function expirePreviousUserTokens($token)
	{
		$sql_user_entries = "SELECT count(TOKEN) AS ENTRIES FROM TBL_USER_ONE_TIME_LOGIN
		   			WHERE USER_ID in (SELECT USER_ID FROM TBL_USER_ONE_TIME_LOGIN WHERE TOKEN = ?)";
		$q_user_entries = $this->db->query($sql_user_entries);
		$q_user_entries->bind_values($token);
		$q_user_entries->execute();
		if (!$q_user_entries->execute())
		{
			alert("Couldn't count how many tokens the user has... (token=$token)");
			header('HTTP/1.1 403 Forbidden');
			throw new IWUserException('Permission Denied');
		}
		$r_user_entries = $q_user_entries->fetch();
		if ($r_user_entries->ENTRIES > 1)
		{
			// they have more then one token, so we must invalidate the old ones. only the latest should be left unchanged
			// first, get the list of tokens that need to be marked as inactive
			$sql_list = "SELECT TOKEN FROM TBL_USER_ONE_TIME_LOGIN as UOT
			   				WHERE USER_ID IN (SELECT USER_ID FROM TBL_USER_ONE_TIME_LOGIN WHERE TOKEN = ?)
			     			AND GENERATED < (SELECT MAX(GENERATED) FROM TBL_USER_ONE_TIME_LOGIN WHERE USER_ID=UOT.USER_ID)";
			$q_list = $this->db->query($sql_list);
			$q_list->bind_values($token);
			$q_list->execute();
			$r_list = $q_list->fetchAll();
			$tokens = array();
			foreach ($r_list as $rec)
				$tokens[] = $rec->TOKEN;

			if(count($tokens)==0) {
				alert("Couldn't update the old tokens for the user with token=$token, most likely cause by double requesting the link.");
				header('HTTP/1.1 403 Forbidden');
				throw new IWUserException('Invalid login link: Multiple entries with the same timestamp found. Please request a new link.');
			}

			// mark them as inactive
			$sql_process = "UPDATE TBL_USER_ONE_TIME_LOGIN SET ACTIVE = 0 WHERE TOKEN IN ('" . implode("','",$tokens) . "')";
			$q_process = $this->db->query($sql_process);
			if (!$q_process->execute())
			{
				alert("Couldn't update the old tokens for the user with token=$token");
				header('HTTP/1.1 403 Forbidden');
				throw new IWUserException('Permission Denied');
			}
		}
		return $r_user_entries->ENTRIES;
	}

	/**
	 * invalidates the one-time-login token given as parameter
	 * @param string $token
	 */
	function invalidateLoginToken($token)
	{
		$sql = "UPDATE TBL_USER_ONE_TIME_LOGIN SET ACTIVE = 0 WHERE TOKEN = ?";
		$q = $this->db->query($sql);
		$q->bind_values($token);
		$q->execute();
	}

	private function setTranslatorLanguages($id)
	{
		require_once (IWARE . 'Translator.php');
		$t = new Translator($this->db);
		$rs = $t->getByUserId($id);
		foreach ($rs as $r)
			$_SESSION['user_translate_languages'][$r->LANGUAGE_ID] = $r->LANGUAGE_ID;
	}

	/**#@+
	 * @access protected
	 */
	/**
	 *
	 */
	function _validate()
	{
		if ($_SESSION['session_user_id'] == 0)
			return $this->_setDefault();
		$this->setSessionLocale();
	}
	/**
	 *
	 */
	function _setDefault()
	{
		$_SESSION = array();
		$_SESSION['session_group_group_id'] 				= 1;
		//$_SESSION['session_user_app_visible'] 				= 1;//
		$_SESSION['session_group_company_id'] 				= 1;
		$_SESSION['session_group_group_name']				= 'public';
		//$_SESSION['show_language_update'] 					= 'yes';//
		//$_SESSION['show_message_update'] 					= 'no';//Deprecated//
		//$_SESSION['user_style_color'] 						= '';//Deprecated //
		//$_SESSION['refreshTop'] 							= 'no';//Deprecated //
		$_SESSION['session_user_id'] 						= 1;
		$_SESSION['session_user_language_id'] 				= (isset($_SESSION['session_user_language_id']))?$_SESSION['session_user_language_id']:44;
		$_SESSION['session_user_name'] 						= 'public';
		//$_SESSION['session_user_application_id'] 			= APPLICATION_ID; //
		$_SESSION['session_user_company_id'] 				= 1;
		//$_SESSION['session_user_company_name']				= 'ELC'; // FIXME
		//$_SESSION['Session_Application_Server'] 			= SITE_SERVER; //
		//$_SESSION['Session_Application_Virtual_Root'] 		= SITE_HOMEDIR;//
		//$_SESSION['Session_Application_Path'] 				= SITE_PATH;//
		//$_SESSION['session_site_title']						= '';//
		//$_SESSION['session_encryption_key'] 				= ENCRYPTION_KEY; //
		//$_SESSION['session_no_access_page'] 				= defined('NO_ACCESS_PAGE') ? NO_ACCESS_PAGE : '';//
		//$_SESSION['session_logout_page']					= defined('LOGOUT_PAGE') ? LOGOUT_PAGE : '';//
		$_SESSION['session_user_default_page'] 				= false;
		// WRONG PLACE :S
		$_SESSION['selected_product_id'] = 1; // Defaults back to ELC->Lamps
		$_SESSION['browser_supported'] = CHECK_FOR_BROWSER; // Check for browser.
		$_SESSION['user_translate_languages'] = array ();
		$_SESSION['user_first_login'] = true;
		$_SESSION['user_password_expired'] = true;
		$_SESSION['login_type'] = LOGIN_FAILED; // used for displaying various messages when users log in
		$this->setSessionLocale();
	}
	/**#@-*/
}

define('CHECK_FOR_BROWSER', 0);
define('BROWSER_CHECK_APPROVED', 1);
define('BROWSER_CHECK_DENIED', 2);

class SessionContainerDB
{
	private $tbl;
	private $db;
	private $crc = false;
	private $expiry = false;

	public function __construct($db, $tbl = 'TBL_SESSION_DATA')
	{
		$this->db = $db;
		$this->tbl = $tbl;
	}

	public function open($save_path, $session_name)
	{
		return true;
	}

	public function close()
	{
		return true;
	}
	/**
	 * Read the data from the database
	 */
	public function read($id) {
		$sql = sprintf("SELECT SQL_NO_CACHE expiry, data FROM %s WHERE id = ? AND expiry >= ?", $this->tbl);
		$q = $this->db->query($sql);
		$q->bind_values(md5($id), time());
		if (!$q->execute())
			return false;

		if (!($r = $q->fetch()))
			return false;

		$this->expiry = $r->expiry;
		$this->crc = strlen($r->data) . crc32($r->data);
		return $r->data;
	}

	public function write($id, $data)
	{
		// Nothing has changed, dont update the BLOB
		if (($this->crc !== false) && ($this->crc === strlen($data) . crc32($data)))
		{
			if ($this->expiry !== false && $this->expiry - time() < 960) //if we expire within 16 mins
			{
				$sql = sprintf("UPDATE %s SET expiry = ? WHERE id = ?", $this->tbl);
				$q = $this->db->query($sql);
				$q->bind_values((time() + ini_get('session.gc_maxlifetime')), md5($id));
				if (!$q->execute())
					return false;
			}
			return true;
		}
		$sql = sprintf("SELECT SQL_NO_CACHE id FROM %s WHERE id = ?", $this->tbl);
		$q = $this->db->query($sql);
		$q->bind_values(md5($id));
		if (!$q->execute())
			return false;

		if ($r = $q->fetch())
		{
			$sql = sprintf("UPDATE %s SET expiry = ?, data = ? WHERE id = ?", $this->tbl);
			$q = $this->db->query($sql);
			$q->bind_values((time() + ini_get('session.gc_maxlifetime')), $data, md5($id));
			if (!$q->execute())
				return false;
			return true;
		}

		$sql = sprintf("INSERT INTO %s (id, expiry, data) VALUES (?,?,?)", $this->tbl);
		$q = $this->db->query($sql);
		$q->bind_values(md5($id), (time() + ini_get('session.gc_maxlifetime')), $data);
		if (!$q->execute())
			return false;

		return true;
	}

	public function destroy($id)
	{
		$sql = sprintf("DELETE FROM %s WHERE id = ?", $this->tbl);
		$q = $this->db->query($sql);
		$q->bind_values(md5($id));
		if (!$q->execute())
			return false;
		return true;
	}
	/**
	 * Garbage collector
	 */
	public function gc()
	{
		$sql = sprintf("DELETE FROM %s WHERE expiry < ?", $this->tbl);
		$q = $this->db->query($sql);
		$q->bind_values(time());
		if (!$q->execute())
			return false;

		$sql = sprintf("OPTIMIZE TABLE %s", $this->tbl);
		$q = $this->db->query($sql);
		if (!$q->execute())
			return false;

		return true;
	}
}
?>
