<?php
define('IWSYNC',SITE_HOMEDIR.SITE_PATH.'related/iwsync/');
/**
 * @author Marten Koedam <marten@dalines.net>
 * @copyright Copyright 2007, BaseSoft Solutions b.v.
 * @package IW
 * @version $Id$
 */
class IWSync
{
	private static function saveFile($content,$name)
	{
		$fp = fopen(IWSYNC.$name.'.php','w+');
		fwrite($fp,"<?php\n".$content."\n?>");
		fclose($fp);
	}
	/**#@+
	 * All export functions
	 */
	public static function exportAll()
	{
		self::exportScriptPrivs();
		self::exportInterfacePrivs();
		self::exportScripts();
		self::exportInterfaces();
	}
	public static function exportScriptPrivs()
	{
		self::exportTable('TBL_SCRIPT_GROUP',array('SCRIPT_ID','GROUP_COMPANY_ID','SCRIPT_GROUP_ACCESS'),'script.privs');
	}
	public static function exportInterfacePrivs()
	{
		self::exportTable('TBL_GROUP_COMPANY_PRIVILEGES',
			array('GROUP_COMPANY_ID','INTERFACE_ID','BLOCK_ID','ITEM_ID', 'PRIV'),
			'interface.privs',
			array('GROUP_COMPANY_ID','INTERFACE_ID','BLOCK_ID','ITEM_ID'));
	}
	public static function exportScripts()
	{
		self::exportTable('TBL_SCRIPT',
			array('SCRIPT_ID','SCRIPT_NAME','SCRIPT_PARENT_ID','SCRIPT_POSITION','SCRIPT_PATH','SCRIPT_MENUITEM','APPLICATION_ID','SCRIPT_UNIVERSE','INTERFACE_ID'),
			'script.scripts',
			array('SCRIPT_ID'));
	}
	public static function exportInterfaces()
	{
		self::exportTable('TBL_INTERFACE',array('INTERFACE_ID','HANDLE','CLASS','PATH'),'interface.interfaces',array('INTERFACE_ID'));
		self::exportTable('TBL_INTERFACE_BLOCK',
			array('BLOCK_ID','INTERFACE_ID','POSITION','TYPE','HANDLE','TITLE_ITEM'),
			'interface.blocks',
			array('INTERFACE_ID','BLOCK_ID'));
		self::exportTable('TBL_INTERFACE_ITEM',
			array('ITEM_ID','BLOCK_ID','POSITION','TYPE','CALLBACK','NAME','HANDLE','CLOSE_ROW','BEHAVIOUR','LABEL_DISPOSITION','HELP_ENABLED'),
			'interface.items',
			array('BLOCK_ID','ITEM_ID'));
	}
	private static function exportTable($tbl,$fields,$file,$order=null)
	{
		$order || $order = $fields;
		$sql = sprintf("SELECT ".implode(',',$fields)." FROM %s ORDER BY ".implode(',',$order),$tbl);
		$q = IWRegistry::registry('db')->query($sql);
		$q->execute();
		$rows = $q->fetchAll();
		if(!count($rows))
			throw new IWException(sprintf('IWSync - %s is empty!',$tbl));

		$rv = '';
		foreach($rows as $r)
		{
			$fieldValues = array();
			foreach($fields as $field)
				$fieldValues[] = sprintf("'%s'=>'%s'",$field,$r->$field);
			$rv .= sprintf("\$".$tbl."[] = array(%s);\n",implode(',',$fieldValues));
		}
		self::saveFile(sprintf("\$".$tbl." = array();\n%s",$rv),$file);
	}
	/**#@-*/
	/**#@+
	 * All import functions
	 */
	/**
	 *
	 */
	public static function importAll()
	{
		IWRegistry::registry('db')->begin();
		self::importScriptPrivs(false);
		self::importInterfacePrivs(false);
		self::importScripts(false);
		self::importInterfaces(false);

		IWRegistry::registry('db')->commit();
	}
	/**
	 *
	 */
	public static function importScriptPrivs($begin=true)
	{
		self::importTable('TBL_SCRIPT_GROUP','script.privs',$begin);
	}
	/**
	 *
	 */
	public static function importInterfacePrivs($begin=true)
	{
		self::importTable('TBL_GROUP_COMPANY_PRIVILEGES','interface.privs',$begin);
	}
	/**
	 *
	 */
	public static function importScripts($begin=true)
	{
		self::importTable('TBL_SCRIPT','script.scripts',$begin);
	}
	/**
	 *
	 */
	public static function importInterfaces($begin=true)
	{
		self::importTable('TBL_INTERFACE','interface.interfaces',$begin);
		self::importTable('TBL_INTERFACE_BLOCK','interface.blocks',$begin);
		self::importTable('TBL_INTERFACE_ITEM','interface.items',$begin);
	}
	/**#@-*/
	private static function importTable($tbl,$file,$begin=true)
	{
		$begin && IWRegistry::registry('db')->begin();
		$sql = sprintf("DELETE FROM %s",$tbl);
		$q = IWRegistry::registry('db')->query($sql);
		$q->execute();

		$$tbl = array();
		include(IWSYNC.$file.'.php');
		foreach($$tbl as $values)
			self::insertLine($tbl,$values);

		$begin && IWRegistry::registry('db')->commit();
	}
	private static function insertLine($tbl,$values)
	{
		$fields = array_keys($values);
		$sql = sprintf("INSERT INTO %s (".implode(',',$fields).") VALUES(".substr(str_repeat('?,',count($fields)),0,-1).")",$tbl);
		$q = IWRegistry::registry('db')->query($sql);
		$q->bind_values(array_values($values));
		$q->execute();
	}
}
?>
