<?php
/**
 * Syslog
 *
 * @package PACKAGE
 * @subpackage SUBPACKAGE
 * @since Dec 15, 2006
 * @author Martijn van den Broek <mjbroek@gmail.com>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */

class IWSyslog
{
	private static $instance;
	private $debug = true;

	function __construct()
	{
		$ident = $_SERVER['HTTP_HOST'];
		$ident = preg_replace('/\W/', '', $ident);

		if (defined('SYSLOG_IDENT'))
			$ident .= '-'.SYSLOG_IDENT;

		openlog($ident, LOG_ODELAY, LOG_LOCAL0);
	}

	function __destruct()
	{
		closelog();
	}

	/**
	 * sends the message to syslog
	 */
	private function message($level, $data)
	{
		$user_id = isset($_SESSION['session_user_id']) ? $_SESSION['session_user_id'] : '0';
		$remote_addr = isset($_SERVER['REMOTE_ADDR']) ? $_SERVER['REMOTE_ADDR'] : '-';

		// convert array to string
		if (is_array($data))
			$data = print_r($data, true);

		foreach (explode("\n", $data) as $line)
		{
			$line = sprintf('%s: uid %s: %s', $remote_addr, $user_id, $line);
			syslog($level, $line);
		}
	}

	/**
	 * to create a sigleton
	 */
	static function create()
	{
		if ( !isset(self::$instance) )
			self::$instance = new IWSyslog();

		return self::$instance;
	}

	/**
	 * debug-level message
	 */
	function debug($data)
	{
		$this->message(LOG_DEBUG, $data);
	}

	/**
	 * informational message
	 */
	function info($data)
	{
		$this->message(LOG_INFO, $data);
	}

	/**
	 * normal, but significant, condition
	 */
	function notice($data)
	{
		$this->message(LOG_ERR, $data);
	}

	/**
	 * warning conditions
	 */
	function warning($data)
	{
		$this->message(LOG_WARNING, $data);
	}

	/**
	 * error conditions
	 */
	function error($data)
	{
		$this->message(LOG_ERR, $data);
	}

	/**
	 * critical conditions
	 */
	function critical($data)
	{
		$this->message(LOG_CRIT, $data);
	}

	//backwards compat
	function crit($data)
	{
		$this->message(LOG_CRIT, $data);
	}

	/**
	 * action must be taken immediately
	 */
	function alert($data)
	{
		$this->message(LOG_ALERT, $data);
	}

	/**
	 * system is unusable
	 */
	function emerg($data)
	{
		$this->message(LOG_EMERG, $data);
	}
}

?>