<?php
require_once(IWARE.'IWFileWriter.php');
/**
 * @package I-Ware
 * @author Nick Loeve <nick@basesoft.com>
 * @since 20/10/2006
 * @copyright 2006 BaseSoft B.V.
 * $Id$
 *
 * This class is a file writer for xml - see CVSWriter also
 */
class IWXMLWriter implements IWFileWriter
{

	private $ds;
	private $hash_each_line_on_write;
	public function __construct($ds=null, $hash_each_line_on_write=false)
	{
		!empty($ds) && $this->setData($ds);
		$this->hash_each_line_on_write = $hash_each_line_on_write;
	}


	public function write($filename)
	{
		return file_put_contents($filename, $this->getFormattedData());
	}

	public function setData($dataset)
	{
		do
		{
			if ( is_array($dataset) )
			{
				if ( empty($dataset) || !is_array(current($dataset)) )
					break;//error
				$this->ds = $dataset;
			}
			elseif ( $dataset instanceof Query )
				while ( $row = $dataset->fetch() )
					$this->ds[] = (array) $row;
			else
				break;//error

			return true;
		}while(false);

		throw new IWXMLWriterException('Dataset is invalid format or type');
	}

	public function getFormattedData()
	{
		if(empty($this->ds) || is_null($this->ds))
			throw new IWXMLWriterException('No dataset available');
		$xmlWriter = xmlwriter_open_memory();
		xmlwriter_set_indent($xmlWriter, true);
		xmlwriter_start_document($xmlWriter, '1.0', 'UTF-8', 'no');
		xmlwriter_start_element($xmlWriter, 'export');

		foreach($this->ds as $aRow)
		{
			xmlwriter_start_element($xmlWriter, 'line');
			if($this->hash_each_line_on_write)
				$hash_value = '';
			foreach ($aRow as $key => $bitbucket)
			{
				if($this->hash_each_line_on_write)
					$hash_value .= $aRow[$key];
				$cleanTagName = str_replace(array(' ', '/'), '_', strtolower($key));
				if(false === @xmlwriter_start_element($xmlWriter, $cleanTagName))
					throw new IWXMLWriterException('Invalid tag name: '.$key);
				xmlwriter_text($xmlWriter, strlen($aRow[$key]) > 250 ? substr($aRow[$key], 0, 250) : $aRow[$key]);
				xmlwriter_end_element($xmlWriter);
			}
			if($this->hash_each_line_on_write)
			{
				xmlwriter_start_element($xmlWriter, 'hash_line');
				xmlwriter_text($xmlWriter, md5($hash_value));
				xmlwriter_end_element($xmlWriter);
			}
			xmlwriter_end_element($xmlWriter);
		}
		xmlwriter_end_element($xmlWriter);
		xmlwriter_end_document($xmlWriter);
		$return = xmlwriter_output_memory($xmlWriter);
		unset($xmlWriter);
		return $return;

	}
}

class IWXMLWriterException extends IWFileWriterException {}
?>
