<?php
require_once(IWARE.'AbstractObject.php');
require_once(IWARE.'InfoPerson.php');
/**
 * A collection of Info persons
 *
 * @author Marten Koedam <marten@basesoft.com>
 * @version $Id$
 * @since 09-07-2005
 * @package I-Ware
 */
class InfoGroup extends AbstractObject
{
	/**
	 * @var Array $info_people (Object InfoPerson)
	 */
	var $info_people = false;
	/**
	 * @var string $name
	 */
	var $name = '';
	/**
	 * @var Company $company
	 */
	var $company;
	/**
	 * @var int $id
	 */
	var $id;
	/**
	 * @param int $id INFO_GROUP_ID
	 */
	function set($id='')
	{
		$id = $id ? $id : $this->id;
		if ( !is_numeric($id) )
			return false;

		$sql = "SELECT NAME, COMPANY_ID
			FROM TBL_INFO_GROUP
			WHERE INFO_GROUP_ID = ? AND COMPANY_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values(array($id, $_SESSION['session_user_company_id']));
		$q->execute(true);
		if(!($r = $q->fetch()))
			return false;
		$this->id = $id;
		$this->name = $r->NAME;
		$this->company->id = $r->COMPANY_ID;

		$sql = "SELECT INFO_PERSON_ID
			FROM TBL_INFO_GROUP_INFO_PERSON
			WHERE INFO_GROUP_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values(array($id));
		$q->execute();
		$rs = $q->fetchAll();
		if(!count($rs))
			return false;

		$this->info_people = array();
		foreach($rs as $r)
			$this->info_people[$r->INFO_PERSON_ID] = new InfoPerson($this->db,$r->INFO_PERSON_ID,true);
	}
	/**
	 *
	 */
	function insert()
	{
		$id = false;
		$sql = "INSERT INTO TBL_INFO_GROUP (NAME,COMPANY_ID) VALUES (?,?)";
		$q = $this->db->query($sql);
		$q->bind_values(array($this->name,$_SESSION['session_user_company_id']));
		if(!$q->execute(true))
			return false;
		$id = $q->lastid();

		if($this->info_people)
			$this->insertPeople();

		return $id;
	}
	/**
	 *
	 */
	function selectInfoGroups($selected = '')
	{
		$rv = '';
		$sql = "SELECT INFO_GROUP_ID, NAME,
				IF((INFO_GROUP_ID = $selected),' selected=\"selected\"', '') as SELECTED
			FROM TBL_INFO_GROUP
			WHERE COMPANY_ID = ?
			ORDER BY NAME";
		$q = $this->db->query($sql);
		$q->bind_values(array($_SESSION['session_user_company_id']));
		$q->execute();
		while($r=$q->fetch())
			$rv .= '<option'.$r->SELECTED.' value="'.$r->INFO_GROUP_ID.'">'.$r->NAME.'</option>';
		return $rv;
	}
	/**
	 *
	 */
	function selectInfoPeople()
	{

	}
	/**
	 * Get a resultset for TableFactory to be displayed
	 */
	function getInfoPeopleDisplay()
	{
		$rv = array();
		foreach($this->info_people as $id => $info_person)
		{
			$line['id'] = $id;
			$line['name'] = $info_person->first_name.' '.$info_person->middle_name.' '.$info_person->last_name;
			$rv[] = $line;
		}
		return $rv;
	}
	/**
	 * Update the snizzleeeee
	 */
	function update()
	{
		$rv = parent::update();
		$sql = "DELETE FROM TBL_INFO_GROUP_INFO_PERSON
			WHERE INFO_GROUP_ID = ? AND INFO_PERSON_ID NOT IN(".implode(',',array_keys($this->info_people)).")";
		$q = $this->db->query($sql);
		$q->bind_values(array($this->id));
		$rv = $q->execute();

		$sql = "REPLACE INTO TBL_INFO_GROUP_INFO_PERSON (INFO_GROUP_ID,INFO_PERSON_ID) VALUES ";
		foreach($this->info_people as $id => $bitbucket)
			$sql .= "( ".$this->id.", ".$id." ),";
		$sql = substr($sql,0,-1);
		$q = $this->db->query($sql);
		$rv = $q->execute();
		return $rv;
	}
	/**#@+
	 * @access private
	 */
	/**
	 * Initialize objectmap(s)
	 *
	 * Called in the Constructor children must either override the Constructor or this method.
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_INFO_GROUP';
		$this->objectmap['primary']['id'] = 'INFO_GROUP_ID';
		$this->objectmap['properties']['name'] = 'NAME';
		$this->objectmap['properties']['company->id'] = 'COMPANY_ID';
	}
	/**#@-*/
}
/**

CREATE TABLE TBL_INFO_GROUP
(
	INFO_GROUP_ID INT(11) NOT NULL AUTO_INCREMENT,
	COMPANY_ID INT(11) NOT NULL,
	NAME VARCHAR(255) NOT NULL,

	PRIMARY KEY(INFO_GROUP_ID)
);

CREATE TABLE TBL_INFO_GROUP_INFO_PERSON
(
	INFO_GROUP_ID INT(11) NOT NULL,
	INFO_PERSON_ID INT(11) NOT NULL,

	PRIMARY KEY(INFO_GROUP_ID,INFO_PERSON_ID)
);

**/