<?php
	/**
	 * LanguageInterface
	 * @author Martijn van den Broek <mjbroek@xs4all.nl>
	 * @version $Id$
	 * @package I-Ware
	*/

	/**
	 * we need some classes
	 */
	require_once(IWARE . 'sql.php');
	require_once(IWARE . 'form.php');
	require_once(IWARE . 'form_select_db.php');
	require_once(IWARE . 'table_factory.php');

	require_once(IWARE.'group.php');

	# admin table constants
	define ('LE_TABLE_PAGEADMIN', 0);
	define ('LE_TABLE_ADMIN',     1);
	define ('LE_TABLE_SEARCH',    2);

	/**
	 * LanguageInterface
	 * @package I-Ware
	 */
	class LanguageInterface
	{
		private $language_id;
		private $translation_cache = array();
		private $simple_mode = false;
		private $print_tags = false;
		private $is_cacheable;

		# boolean to control the language interface
		private $_isAdmin = null; // null means unknown, let _isAdmin() take care of it

		// array with all handles used in the current page
		private $pageHandles = array();

		/**
		 * Constructor
		 * @param database $db (database as reference)
		 */
		function LanguageInterface(&$db_app, &$db_langif)
		{
			$this->db_app = $db_app;
			$this->db_langif = is_null($db_langif) ? $db_app : $db_langif;


			# set language id
			$_SESSION['session_user_language_id'] = $this->language_id = $this->_verifyLanguageId();

			# language
			$this->secondlanguage_id = (!empty($_SESSION['LanguageAdminSecondLanguage'])) ?
			                                   $_SESSION['LanguageAdminSecondLanguage'] : $this->_otherLanguage();

			# show all translations by default
			$this->showSetTranslation = (isset($_COOKIE['LanguageAdminShowSetTranslations'])) ?
			                                   (bool) $_COOKIE['LanguageAdminShowSetTranslations'] : 1;

			# shows handles if translation if empty, off by default and only for admin mode
			$this->showEmptyTranslations = (isset($_COOKIE['LanguageAdminShowEmptyTranslations'])) ?
			                                   (bool) $_COOKIE['LanguageAdminShowEmptyTranslations'] : true;


			$this->fallbackToEnglish = (isset($_SESSION['FallbackToEnglish'])) ?
			                                   (bool) $_SESSION['FallbackToEnglish'] : 1;


			$this->disabled = (isset($_COOKIE['LanguageAdminDisabled'])) ? $_COOKIE['LanguageAdminDisabled'] : 0;


			# counter to make unique tables
			$this->table_counter = 0;

			# max chars we show in translation preview
			$this->trmaxlen = 50;

			$this->is_cacheable = function_exists('apc_store');
		}

		/**
		 * Get/Set functions
		 */
		public function setPrintTags($print_tags=true)
		{
			$this->print_tags = $print_tags;
		}

		/**
		 * Changes the current language ID
		 */
		public function setLanguage($id)
		{
			$this->language_id = $id;
		}

		public function setSecondLanguage($id)
		{
			$this->secondlanguage_id = $id;
		}

		public function getLanguageId()
		{
			return $this->language_id;
		}

		/**
		 * Change the isAdmin boolean
		 */
		public function setAdmin($bool)
		{
			$this->_isAdmin = $bool;
		}

		/**
		 * gets a translation for the given handle. The handle is created if it does not exist.
		 * @param string $handle
		 * @param bool $print_tags If true a span with ID will be added around the translation, used by the language admin to find a translation position
		 * @param array replacements.. example: <code>$langIf->�('This building has {FLOORS} floors', null, 8)</code> will return 'This building has 8 floors'.
		 * @return string
		 */
		public function getText($handle, $print_tags=null, $replacements=array(), $language_id=null )
		{

			# see if language_id is overridden
			if ($language_id == null)
				$language_id = $this->language_id;

			if (empty($handle))
				return '';

			if ( strlen($handle) > 255)
			{
				trigger_error('getText(): Handle cannot be more than 255 characters, translation ignored', E_USER_WARNING);
				return $handle;
			}

			if (strip_tags($handle) != $handle)
			{
				trigger_error('getText(): Found HTML tag in translation.. translating translations? '.$handle, E_USER_NOTICE);
				return $handle;
			}

			// fallback group to public
			$group_id = isset($_SESSION['session_group_group_id']) ? $_SESSION['session_group_group_id'] : 1;
			$group_bit = Group::getGroupBit($group_id);

			$tr = false;
			if($this->is_cacheable)
			{
				$key = 'li_'.$handle.$group_bit;
				$apc_lookup = md5($key);
				if(($cached = apc_fetch($apc_lookup)) && isset($cached[$language_id]))
					$tr = $cached[$language_id];
			}
			if(!$tr)
			{
				$pdo = $this->db_langif;

				// #2621: code below used to be in a procedure p_iwGetText but this gave
				// mysql random memory read errors, so instead do it here
				//
				// start p_iwGetText

				$translation = '';
				$handle_id = null;

				# See if we can get a translation directly
				$stmt = $pdo->prepare('SELECT t.TRANSLATION, h.TRANSLATION_HANDLE_ID
					FROM TBL_TRANSLATION_HANDLES h
					LEFT JOIN TBL_TRANSLATIONS t
						ON h.TRANSLATION_HANDLE_ID=t.FK_TRANSLATION_HANDLE_ID
					WHERE
						TRANSLATION_HANDLE=?
						AND (FK_LANGUAGE_ID IS NULL OR FK_LANGUAGE_ID = ?)
					LIMIT 1');
				$stmt->bindValue(1, $handle);
				$stmt->bindValue(2, $language_id);
				$stmt->execute();
				$row = $stmt->fetchAssoc();

				if ($row) {
					$translation = $row['TRANSLATION'];
					$handle_id = $row['TRANSLATION_HANDLE_ID'];
				}

				// no translation found lets see if we need to make the handle
				if (!$translation) {

					$stmt = $pdo->prepare('SELECT TRANSLATION_HANDLE_ID
						FROM TBL_TRANSLATION_HANDLES h
						LEFT JOIN TBL_TRANSLATIONS t
							ON h.TRANSLATION_HANDLE_ID=t.FK_TRANSLATION_HANDLE_ID
						WHERE
							TRANSLATION_HANDLE=?
						LIMIT 1');

					$stmt->bindValue(1, $handle);
					$stmt->execute();
					$row = $stmt->fetchAssoc();

					if ($row) {
						$handle_id = $row['TRANSLATION_HANDLE_ID'];
					}

					if (!$handle_id) {

						$stmt = $pdo->prepare('INSERT INTO TBL_TRANSLATION_HANDLES (TRANSLATION_HANDLE) VALUES(?)');
						$stmt->bindValue(1, $handle);
						$stmt->execute();

						$stmt = $pdo->prepare('SELECT LAST_INSERT_ID() as LAST_INSERT_ID');
						$stmt->execute();
						$row = $stmt->fetchAssoc();
						$handle_id = $row['LAST_INSERT_ID'];
					}
				}

				assert(is_numeric($handle_id));
				$stmt = $pdo->prepare('UPDATE TBL_TRANSLATION_HANDLES SET `GROUPS` = `GROUPS` | ? WHERE TRANSLATION_HANDLE_ID = ?');
				$stmt->bindValue(1, $group_bit);
				$stmt->bindValue(2, $handle_id);
				$stmt->execute();

				// end p_iwGetText

				$tr = $translation;

				unset($stmt);

				if($this->is_cacheable)
				{
					if(!$cached)
						$cached = array();
					$cached[$language_id] = $tr;
					apc_store($apc_lookup, $cached);
				}
			}

			// cache this handle
			// TODO: only if admin
			if (!in_array($tr, $this->pageHandles))
				$this->pageHandles[] = $handle;

			/**
			 * if the translation is emptry and we are in admin mode, lets
			 * display the handle (instead of nuffin)
			 */
			if (empty($tr) && $this->_isAdmin() && $this->showEmptyTranslations && (!$this->fallbackToEnglish || $language_id == 44) )
				$tr = '<i>'.$handle.'</i>';

			/**
			 * Fix Replacements
			 */
			$tr = preg_replace_callback("/{(\w+)}/", function($m) use ($replacements) {
				if (isset($replacements[$m[1]]))
					return $replacements[$m[1]];
				return $m[0];
			}, $tr);

			/**
			 * Give the user an english translation if this current is empty
			 */
			if (empty($tr) && $language_id != 44 && $this->fallbackToEnglish )
				return $this->getText($handle, $print_tags, $replacements, 44);

			return $tr;
		}

		private function getCacheScriptName()
		{
			if (is_null($this->script_name))
				return $_SERVER['PHP_SELF'];
			else
				return $this->script_name;

		}

		/**
		 * This will return the best languageid for the user
		 * Check if user can modify, then all is good.. else see if status is 1 in db, else get default langid
		 */
		private function _verifyLanguageId()
		{
			if (!isset($_SESSION['session_user_language_id']))
				return 44;

			if ($this->_allowModify($_SESSION['session_user_language_id']))
				return $_SESSION['session_user_language_id'];
			else
			{
				$sql = "SELECT * FROM TBL_LANGUAGE WHERE LANGUAGE_ID=? AND LANGUAGE_STATUS & 1";
				$q = $this->db_app->query($sql);
				$q->bind_values($_SESSION['session_user_language_id']);
				$q->execute();

				if ($q->fetch())
					return $_SESSION['session_user_language_id'];
				else
					return 44; // TODO: make nice GCONF interface for default stuff
			}
		}

		private function _otherLanguage()
		{
			// @todo: default english

			$sql = "SELECT * FROM TBL_LANGUAGE WHERE LANGUAGE_ID != ?";
			$q = $this->db_app->query($sql);
			$q->bind_values($this->language_id);
			$q->execute();

			if ($row = $q->fetchAssoc())
				return $row['LANGUAGE_ID'];
			else
				return $this->language_id;

		}

		/**
		 * Checks if the current user is admin
		 */
		private function _isAdmin()
		{
			if (isset($this->_isAdmin))
				return $this->_isAdmin;
			if(isset($_SESSION['language_admin']))
				return $this->_isAdmin = $_SESSION['language_admin'];
			// check admin allowed ip's.. can be added in site_settings
			global $langIf_IPs;

			//
			$sql = "SELECT IP_ADDRESS FROM TBL_ADMIN_IP WHERE IP_ADDRESS != ''";
			$q = $this->db_app->query($sql);
			$q->execute();
			while ($r = $q->fetch())
				$langIf_IPs[] = $r->{'IP_ADDRESS'};

			if (isset($langIf_IPs) && is_array($langIf_IPs)) {
				foreach ($langIf_IPs as $allowed_ip)
				{
					$my_ip = $_SERVER['REMOTE_ADDR'];
					if (strpos($my_ip, $allowed_ip) !== false) {
						$_SESSION['language_admin'] = $this->_isAdmin = true;
						return true;
					}
				}
			}

			$_SESSION['language_admin'] = $this->_isAdmin = false;
			return false;
		}

		/**
		 * Checks if the current ip is allowed to modify or the current user is a translator
		 */
		private function _allowModify($languageId=null)
		{
			if (is_null($languageId))
				$languageId = $this->language_id;

			if ($this->_isAdmin())
				return true;

			if (!isset($_SESSION['user_translate_languages']))
				return false;

			if(!count($_SESSION['user_translate_languages']))
				return false;

			if(isset($_SESSION['user_translate_languages'][$languageId]))
				return true;

			return false;

		}

		private function clearApcCache($handle_id)
		{
			$sql = "SELECT TRANSLATION_HANDLE FROM TBL_TRANSLATION_HANDLES WHERE TRANSLATION_HANDLE_ID=?";
			$q = $this->db_langif->query($sql);
			$q->bind_values($handle_id);
			$q->execute();
			if (!($r = $q->fetch()))
				return false;


			$handle = $r->TRANSLATION_HANDLE;
			$auth = 'bGFuZ2FkbWluOkdVOFVodVN0OWhVYjJ1OHI=';
			$header = "\r\nAuthorization: Basic $auth";

			/**
			 * Lets delete the cache on all concurrent webservers by connecting
			 * to the internal ip and request the current virtualhost
			 */
			foreach(explode(';', WEB_SERVERS) as $connect)
			{
				$host = $_SERVER['HTTP_HOST'];
				$params = array(
				      'http' => array('header' =>'Host: '.$host.$header, 'user_agent' => 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.8.1.4) Gecko/20070515 Firefox/2.0.0.4')
				);

				$url = '/'.sprintf('weee_apc_delete.php?handle=%s', urlencode($handle));
				$contents = file_get_contents($connect.$url, false);
				if (!$contents)
				{
					trigger_error('langif::clearApcCache() - failed to connect to server '.$connect, E_USER_NOTICE);
					continue;
				}


				debug(sprintf('Connecting to [%s/%s] to delete apc_cache for handle [%s]', $connect, $host, $handle));
				if(strpos($contents, 'deleted keys for handle:'.$handle) === false)
					trigger_error('langif::clearApcCache() - failed to clear cache at '.$connect, E_USER_NOTICE);

			}
			return true;
		}

		/**
		 * Deletes a handle from the database. Also from related tables.
		 * @param int $handle_id translation_handle_id
		 */
		public function deleteById($handle_id)
		{
			if (!is_numeric($handle_id))
			    trigger_error('LanguageInterface::deleteById() - Supplied argument $handle_id is not numeric', E_USER_ERROR);

			if (!$this->_allowModify())
				trigger_error( 'User is NOT allowed to modify translations', E_USER_ERROR);

			# tbl_translation_handles
			$sql = new SQL_Delete('TBL_TRANSLATION_HANDLES');
			$sql->addWhere('translation_handle_id', $handle_id);
			$ex = $this->db_langif->query($sql->getText());
			$ex->execute();

			// tbl_translations
 			$sql = new SQL_Delete('TBL_TRANSLATIONS');
			$sql->addWhere('FK_TRANSLATION_HANDLE_ID', $handle_id);
			$ex = $this->db_langif->query($sql->getText());
			$ex->execute();

			info('LangIf: deleted handle '.$handle_id);

			$this->clearApcCache($handle_id);

			return true;
		}

		/**
		 * Creates a new entry in tbl_translation_handles and returs the new
		 * generated ID.
		 * @param string handle
		 * @return int translation_handle_id
		*/
		public function newHandle($handle)
		{
			if (!$this->_allowModify())
				trigger_error( 'User is NOT allowed to modify translations', E_USER_ERROR);

			if (empty($handle))
				trigger_error('LanguageInterface::newHandle() - $handle cant be empty', E_USER_ERROR);

			if ( $this->exists( $handle ) )
				trigger_error('LanguageInterface::updateHandle() - Cant update handle because the handle for this level already exists.', E_USER_ERROR);

			$sql = new SQL_Insert('TBL_TRANSLATION_HANDLES');
			$sql->add('TRANSLATION_HANDLE', $handle);
			$ex = $this->db_langif->query($sql->getText());

			if (!$ex->execute())
				trigger_error('Error executing query', E_USER_ERROR);

			return $ex->lastid();
		}

		/**
		 * Updates a handle (name) for the given handle_id
		 */
		public function updateHandle( $handle_id, $handle )
		{
			if (!$this->_allowModify())
				trigger_error( 'User is NOT allowed to modify translations', E_USER_ERROR);

			if (empty($handle)) {
				trigger_error('LanguageInterface::updateHandle() - Supplied argument $handle is empty.');
				return;
			}

			$found_id = $this->exists( $handle );
			if ( $found_id && ($found_id != $handle_id) )
				trigger_error('LanguageInterface::updateHandle() - Cant update handle because the handle for this level already exists.', E_USER_ERROR);

			$sql = new SQL_Update('TBL_TRANSLATION_HANDLES');
			$sql->add('TRANSLATION_HANDLE', $handle);
			$sql->addWhere('TRANSLATION_HANDLE_ID', $handle_id);

			$ex = $this->db_langif->query($sql->getText());
			$ex->execute();

			if (!$ex->execute())
				trigger_error('Error executing query', E_USER_ERROR);

			$this->clearApcCache($handle_id);

			return ($ex->affected_rows() == 1);
		}


		/**
		 * Sets (inserts or updates) a text for the given handle_id and language.
		 * If the translation string is empty the translation will be deleted. (thats a good thing)
		 * @param int $handle_id
		 * @param int $language_id
		 * @param string $text
		*/
		public function setTranslation($handle_id, $language_id, $text)
		{
			if (!$this->_allowModify())
				trigger_error( 'User is NOT allowed to modify translations', E_USER_ERROR);

			if (!is_numeric($handle_id))
				trigger_error('LanguageInterface::setTranslation - supplied argument $handle_id is not numeric', E_USER_ERROR);

			if (!is_numeric($language_id))
				trigger_error('LanguageInterface::setTranslation - supplied argument $language_id is not numeric', E_USER_ERROR);

			# check if the entry already exists, then update else make a new
			$sql = new SQL_Select('TBL_TRANSLATIONS');
			$sql->addWhere('FK_TRANSLATION_HANDLE_ID', $handle_id);
			$sql->addWhere('FK_LANGUAGE_ID', $language_id);
			$ex = $this->db_langif->query($sql->getText());

			if (!$ex->execute())
				trigger_error('Error executing query', E_USER_ERROR);

			if (!$ex->fetch())
			{
				$sql = new SQL_Insert('TBL_TRANSLATIONS'); # new translation
				$sql->add('TRANSLATION',              $text);
				$sql->add('FK_TRANSLATION_HANDLE_ID', $handle_id);
				$sql->add('FK_LANGUAGE_ID',           $language_id);
			}
			else
			{
				if (empty($text))
				{
					$sql = new SQL_Delete('TBL_TRANSLATIONS');
					$sql->addWhere('FK_TRANSLATION_HANDLE_ID', $handle_id);
					$sql->addWhere('FK_LANGUAGE_ID',           $language_id);
				}
				else
				{
					$sql = new SQL_Update('TBL_TRANSLATIONS'); # existing translation
					$sql->add('TRANSLATION',                   $text);
					$sql->addWhere('FK_TRANSLATION_HANDLE_ID', $handle_id);
					$sql->addWhere('FK_LANGUAGE_ID',           $language_id);
				}
			}

			$ex = $this->db_langif->query($sql->getText());
			$ex->execute();

			info(sprintf('LangIf::setTranslation() - Updated handle_id [%s] with language [%s] to text [%s]', $handle_id, $language_id, $text));

			$this->clearApcCache($handle_id);

			return true;
		}

		function setHandleGroups($handleId, $groups)
		{
			if (!$this->_allowModify())
				trigger_error( 'User is NOT allowed to modify translations', E_USER_ERROR);

			$sql = new SQL_Update('TBL_TRANSLATION_HANDLES');
			$sql->add('GROUPS', $groups);
			$sql->addWhere('TRANSLATION_HANDLE_ID', $handleId);

			$ex = $this->db_langif->query($sql->getText());
			$ex->execute();
			info(sprintf('LangIf::setHandleGroups() - Updated handle_id [%s] to groups [%s]', $handleId, $groups));

			$this->clearApcCache($handleId);
		}

		function markOutOfDate($handleId, $outofdate=true)
		{
			if (!$this->_allowModify())
				trigger_error( 'User is NOT allowed to modify translations', E_USER_ERROR);

			$sql = "UPDATE TBL_TRANSLATIONS SET OUTOFDATE=? WHERE FK_TRANSLATION_HANDLE_ID =? AND FK_LANGUAGE_ID != 44";
			$q = $this->db_langif->query($sql);
			$q->bind_values((int) $outofdate, $handleId);
			$q->execute();

			info(sprintf('LangIf::markOutOfDate() - Marked translations for handle [%s] to outofdate [%s]', $handleId, (int) $outofdate));
		}


		/**
		 * Returns the handle (string) from the given handle_id, false if not found
		 * @param int handle_id
		 * @return string handle
		 */
		function getHandleById($handle_id)
		{
			if (!is_numeric($handle_id))
			    trigger_error('LanguageInterface::getHandleById() - Supplied argument $handle_id is not numeric', E_USER_ERROR);

			$sql = new SQL_Select('TBL_TRANSLATION_HANDLES');
			$sql->addWhere('TRANSLATION_HANDLE_ID', $handle_id);
			$ex = $this->db_langif->query($sql->getText());
			$ex->execute();

			if (!($row = $ex->fetch()))
				return false;

			$row = $ex->fetch();
			return $row->{'TRANSLATION_HANDLE'};
		}

		/**
		 * Returns the handle (query object) from the given handle_id, false if not found
		 * @param int handle_id
		 * @return string handle
		 */
		function getHandleObjById($handle_id)
		{
			if (!is_numeric($handle_id))
			    trigger_error('LanguageInterface::getHandleObjById() - Supplied argument $handle_id is not numeric', E_USER_ERROR);

			$sql = new SQL_Select('TBL_TRANSLATION_HANDLES');
			$sql->addWhere('TRANSLATION_HANDLE_ID', $handle_id);
			$ex = $this->db_langif->query($sql->getText());
			$ex->execute();

			if (!($row = $ex->fetchAssoc()))
				return false;

			$row = $ex->fetchAssoc();
			return $row;
		}

		/**
		 * gets the language from the given id
		 * @param int language_id
		 * @return array resultset
		*/
		function languageById($language_id)
		{
			if (!is_numeric($language_id))
			    trigger_error('LanguageInterface::languageById() - Supplied argument $language_id is not numeric', E_USER_ERROR);

			$sql = new SQL_Select('TBL_LANGUAGE');
			$sql->addWhere('LANGUAGE_ID', $language_id);
			$ex = $this->db_app->query($sql->getText());

			if ( !$ex->execute() )
				trigger_error('LanguageInterface::languageById() - Error executing query: ', E_USER_ERROR);

			return $ex->fetchAssoc();
		}


		/**
		 * @param int/array translation_handle_id / translation_handle_id-range
		 * @return string xml
		 *
		 * Returns (for example):
		 * <code>
		 * 	<handle name="USER_NAME">
		 * 		<script_id>null</script_id>
		 * 		<script_name>my script</script_name>
		 * 		<application_id>10</application_id>
		 * 		<translations>
		 * 			<translation language_id="31">gebruikersnaam</translation>
		 * 			<translation language_id="44">user name</translation>
		 * 		</translations>
		 * 	</handle>
		 * </code>
		 */
		function exportHandle($param)
		{
			$sql = "SELECT *
			FROM TBL_TRANSLATION_HANDLES th
			INNER JOIN TBL_TRANSLATIONS t
				ON TRANSLATION_HANDLE_ID=FK_TRANSLATION_HANDLE_ID
			WHERE t.TRANSLATION <> '' ";
			if (is_array($param))
			{
				if (sizeof($param) != 2)
					trigger_error('Range needs to be two numbers', E_USER_ERROR);

				if ($param[0] > $param[1])
					trigger_error('range0 must be smaller than range1', E_USER_ERROR);

				$sql .= 'AND TRANSLATION_HANDLE_ID BETWEEN '.$param[0].' AND '.$param[1].' ';
			}
			else if (is_numeric($param))
				$sql .= 'AND TRANSLATION_HANDLE_ID='.$param.' ';
			else
				trigger_error('Invalid param ('.$param.')', E_USER_ERROR);

			$sql .= 'GROUP BY TRANSLATION_HANDLE_ID';

			$q = $this->db_langif->query($sql);
			$q->execute();

			$rv = '';
			while ($row = $q->fetch())
			{
				$rv .= xml_tag('handle', XML_TAG_NORMAL, array('name'=>$row->{'TRANSLATION_HANDLE'} ));

				$sql = 'SELECT * FROM TBL_TRANSLATIONS WHERE FK_TRANSLATION_HANDLE_ID=?';
				$q2 = $this->db_langif->query($sql);
				$q2->bind_values($row->{'TRANSLATION_HANDLE_ID'});
				$q2->execute();
				$rv .= xml_tag('translations');
				while ($row2 = $q2->fetch())
				{
					if (!empty($row2->{'TRANSLATION'}))
						$rv .= xml_quicktag('translation', $row2->{'TRANSLATION'}, array('language_id'=>$row2->{'FK_LANGUAGE_ID'}));
				}
				$rv .= xml_tag('translations', XML_TAG_CLOSED);

				$rv .= xml_tag('handle', XML_TAG_CLOSED);
			}

			return $rv;
		}


		function exportCsv($baseLang, $destLangs, $groups, $outofdate_or_empty)
		{
			if (!is_numeric($baseLang))
				trigger_error('BaseLang is not numeric', E_USER_ERROR);

			if (!is_array($destLangs))
				trigger_error('DestLangs is not an array', E_USER_ERROR);

			$sql = "SELECT
					h.TRANSLATION_HANDLE_ID as HANDLE,
					tb.TRANSLATION AS BASE_LANG ";

			// add selects
			foreach ($destLangs as $destLang)
				$sql .= ', t'.$destLang.'.TRANSLATION as DEST_LANG_'.$destLang.' ';

			$sql .= 'FROM TBL_TRANSLATION_HANDLES h
				INNER JOIN TBL_TRANSLATIONS tb
					ON h.TRANSLATION_HANDLE_ID=tb.FK_TRANSLATION_HANDLE_ID
					AND tb.FK_LANGUAGE_ID=? ';

			// this will contains an array of where statements
			$where = array();

			// add joins
			foreach ($destLangs as $destLang)
			{
				$sql .= 'LEFT JOIN TBL_TRANSLATIONS t'.$destLang.'
						ON h.TRANSLATION_HANDLE_ID=t'.$destLang.'.FK_TRANSLATION_HANDLE_ID
						AND t'.$destLang.'.FK_LANGUAGE_ID='.$destLang.' ';

				// if true we only select outofdate translations and new ones which will be added to the final where statement
				if ($outofdate_or_empty)
					$where[] = 'AND ( (t'.$destLang.'.OUTOFDATE=1) OR (TRIM(t'.$destLang.'.TRANSLATION) = "") ) ';
			}

			$sql .= 'WHERE TRIM(tb.TRANSLATION) != ""
				AND h.GROUPS & ? AND (h.GROUPS != 0 OR ? = 0) ';

			// process wheres
			foreach ($where as $w)
				$sql .= $w;

			$sql .= 'ORDER BY BASE_LANG';

			$q = $this->db_langif->query($sql);
			$q->bind_values($baseLang, $groups, $groups);
			$q->execute();

			$trs = array();
			while ($row = $q->fetch())
			{
				$tr = array(
					'HANDLE'=>$row->HANDLE,
					'BASE_LANG_'.$baseLang=>trim($row->BASE_LANG)
				);

				foreach ($destLangs as $destLang)
					$tr['DEST_LANG_'.$destLang] = trim($row->{'DEST_LANG_'.$destLang});

				$trs[] = $tr;
			}

			if (empty($trs))
				throw new IWException('ExportCSV: Nothing to export', E_USER_ERROR);

			// filter array
			// - loop trough the translations
			// - if current tr1 translation is same as previous,
			//   add id to previous row and delete this one
			$size = sizeof($trs);
			for ($i=1; $i<$size; $i++)
			{
				if ($trs[$i]['BASE_LANG_'.$baseLang] == $trs[$i-1]['BASE_LANG_'.$baseLang])
				{
					// we dont want empty translations in destination, so check
					// if previous row has empty and if current doesnt replace previous with current
					foreach ($destLangs as $destLang) {
						if (!empty($trs[$i-1]['DEST_LANG_'.$destLang]) && empty($trs[$i]['DEST_LANG_'.$destLang]))
							$trs[$i]['DEST_LANG_'.$destLang] = $trs[$i-1]['DEST_LANG_'.$destLang];
					}

					$trs[$i]['HANDLE'] .= ';'.$trs[$i-1]['HANDLE'];
					unset($trs[$i-1]);
				}
			}

			require_once(IWARE.'IWCSVWriter.php');
			$csv = new IWCSVWriter($trs);
			$csv->eol = "\n";
			$csv->show_header = true;
			return $csv;
		}


		function exportCsvFAQs($baseLang, $destLangs)
		{
			if (!is_numeric($baseLang))
				trigger_error('BaseLang is not numeric', E_USER_ERROR);

			if (!is_array($destLangs))
				trigger_error('DestLangs is not an array', E_USER_ERROR);

			$sql = "SELECT T1.FAQ_ID,
						   T1.QUESTION_TEXT AS Q_BASE_LANG,
						   T1.ANSWER_TEXT AS A_BASE_LANG ";
			foreach ($destLangs as $destLang){
				$sql .= ', T2'.$destLang.'.QUESTION_TEXT as Q_DEST_LANG_' . $destLang . ' ';
				$sql .= ', T2'.$destLang.'.ANSWER_TEXT as A_DEST_LANG_' . $destLang . ' ';
			}

			$sql .= 'FROM TBL_FAQ_LANG T1 ';
			foreach ($destLangs as $destLang)
				$sql .= 'LEFT JOIN TBL_FAQ_LANG T2'.$destLang.'
						ON T1.FAQ_ID=T2'.$destLang.'.FAQ_ID
						AND T2'.$destLang.'.LANGUAGE_ID='.$destLang.' ';
			$sql .= " WHERE T1.LANGUAGE_ID=? ORDER BY T1.FAQ_ID";
			$q = $this->db_app->query($sql);
			$q->bind_values($baseLang);
			$q->execute();

			$trs = array();
			while ($row = $q->fetch())
			{
				$contentBase = trim($row->A_BASE_LANG);
				$contentBase = strip_tags(br2nl(html_entity_decode($contentBase, ENT_QUOTES, 'UTF-8')), '<%>');

				$tr = array(
					'FAQ_ID'=>$row->FAQ_ID,
					'Q_BASE_LANG_'.$baseLang=>trim($row->Q_BASE_LANG),
					'A_BASE_LANG_'.$baseLang=>$contentBase
				);

				foreach ($destLangs as $destLang){

					$contentDest = trim($row->{'A_DEST_LANG_'.$destLang});
					$contentDest = strip_tags(br2nl(html_entity_decode($contentDest, ENT_QUOTES, 'UTF-8')), '<%>');

					$tr['Q_DEST_LANG_'.$destLang] = trim($row->{'Q_DEST_LANG_'.$destLang});
					$tr['A_DEST_LANG_'.$destLang] = $contentDest;
				}

				$trs[] = $tr;
			}

			if (empty($trs))
				throw new IWException('ExportCSV: Nothing to export', E_USER_ERROR);

			require_once(IWARE.'IWCSVWriter.php');
			$csv = new IWCSVWriter($trs);
			$csv->eol = "\n";
			$csv->show_header = true;
			return $csv;
		}

		function exportCsvEmails($baseLang, $destLangs)
		{
			if (!is_numeric($baseLang))
				trigger_error('BaseLang is not numeric', E_USER_ERROR);

			if (!is_array($destLangs))
				trigger_error('DestLangs is not an array', E_USER_ERROR);

			$sql = "SELECT T1.MESSAGE_ID,
						   T1.SUBJECT AS SUBJECT_BASE_LANG,
						   T3.CONTENT AS CONTENT_BASE_LANG ";
			foreach ($destLangs as $destLang){
				$sql .= ', T2'.$destLang.'.SUBJECT as SUBJECT_DEST_LANG_' . $destLang . ' ';
				$sql .= ', T3'.$destLang.'.CONTENT as CONTENT_DEST_LANG_' . $destLang . ' ';
			}

			$sql .= ' FROM TBL_MESSAGE_LANG T1
					INNER JOIN TBL_MESSAGE_CONTENT T3
					ON T1.MESSAGE_ID=T3.MESSAGE_ID
					AND T3.LANGUAGE_ID=T1.LANGUAGE_ID';
			foreach ($destLangs as $destLang){
				$sql .= ' LEFT JOIN TBL_MESSAGE_LANG T2'.$destLang.'
						ON T1.MESSAGE_ID=T2'.$destLang.'.MESSAGE_ID
						AND T2'.$destLang.'.LANGUAGE_ID='.$destLang.' ';
				$sql .= ' LEFT JOIN TBL_MESSAGE_CONTENT T3'.$destLang.'
						ON T1.MESSAGE_ID=T3'.$destLang.'.MESSAGE_ID
						AND T3'.$destLang.'.LANGUAGE_ID='.$destLang.' ';
			}
			$sql .= " WHERE T1.LANGUAGE_ID=? ORDER BY T1.MESSAGE_ID";
			$q = $this->db_app->query($sql);
			$q->bind_values($baseLang);
			$q->execute();

			$trs = array();
			while ($row = $q->fetch())
			{
				$contentBase = trim($row->CONTENT_BASE_LANG);
				$contentBase = br2nl(html_entity_decode($contentBase, ENT_QUOTES, 'UTF-8'));
				$contentBase = preg_replace ('/<[^%][^>]*>/', '', $contentBase); // strip tags except <%>

				$tr = array(
					'MESSAGE_ID'=>$row->MESSAGE_ID,
					'SUBJECT_BASE_LANG_'.$baseLang=>trim($row->SUBJECT_BASE_LANG),
					'CONTENT_BASE_LANG_'.$baseLang=>$contentBase
				);

				foreach ($destLangs as $destLang){

					$contentDest = trim($row->{'CONTENT_DEST_LANG_'.$destLang});
					$contentDest = br2nl(html_entity_decode($contentDest, ENT_QUOTES, 'UTF-8'));
					$contentDest = preg_replace ('/<[^%][^>]*>/', '', $contentDest); // strip tags except <%>

					$tr['SUBJECT_DEST_LANG_'.$destLang] = trim($row->{'SUBJECT_DEST_LANG_'.$destLang});
					$tr['CONTENT_DEST_LANG_'.$destLang] = $contentDest;
				}

				$trs[] = $tr;
			}

			if (empty($trs))
				throw new IWException('ExportCSV: Nothing to export', E_USER_ERROR);

			require_once(IWARE.'IWCSVWriter.php');
			$csv = new IWCSVWriter($trs);
			$csv->eol = "\n";
			$csv->show_header = true;
			return $csv;
		}

		function importCsvFAQs($fileName)
		{
			info('importCsvFAQs: starting import from '.$fileName);
			if($fileName == '')
				throw new IWUserException('No file uploaded');
			$fo = new SplFileObject($fileName);
			$head = true;
			$baseLang = null;
			$destLangs = array();
			$question = array();
			$answer = array();
			$affected = 0;

			$delimiter = ','; $enclosure = '"';
			$i = 0; while($fields = $fo->fgetcsv($delimiter, $enclosure))
			{
				if ($head)
				{
					info('importCsvFAQs: working on head row');
					if ($fields[0] != 'FAQ_ID')
						throw new IWUserException('Invalid file uploaded - FAQ_ID column not found');
					if (!@preg_match("/Q_BASE_LANG_(\d+)$/", $fields[1], $matches))
  						throw new IWUserException('Invalid file uploaded - Q_BASE_LANG column not found');
					if (!@preg_match("/A_BASE_LANG_(\d+)$/", $fields[2], $matches))
  						throw new IWUserException('Invalid file uploaded - A_BASE_LANG column not found');
  					$baseLang = $matches[1];
					for ($i=3; $i<sizeof($fields); $i++)
					{
						if (empty($fields[$i]))
							continue;
						if(($i % 2) != 0)
						{
							if (!@preg_match("/Q_DEST_LANG_(\d+)$/", $fields[$i], $matches))
								throw new IWUserException('Invalid file uploaded - Q_DEST_LANG column not found');
						}
						else
						{
							if (!@preg_match("/A_DEST_LANG_(\d+)$/", $fields[$i], $matches))
								throw new IWUserException('Invalid file uploaded - A_DEST_LANG column not found');
							debug('importCsvFAQs: adding dest lang '.$matches[1]);
							$destLangs[] = $matches[1];
						}
					}
					$head = false;
				}
				else
				{
					$this->db_app->begin();
					$faq_ids = explode(';', $fields[0]);
					foreach($faq_ids as $faq_id)
					{
						if (!$faq_id)
							continue;
						for ($i=0; $i<sizeof($destLangs); $i++)
						{
							foreach ($faq_ids as $faq_id)
							{
								$language_id = $destLangs[$i];
								if (!isset($fields[2*$i+3]))
								{
									trigger_error('No translation for question column '.(2*$i+3).' on line '.$i, E_USER_WARNING);
									continue;
								}
								if (!isset($fields[2*$i+4]))
								{
									trigger_error('No translation for answer column '.(2*$i+4).' on line '.$i, E_USER_WARNING);
									continue;
								}
								$question = $fields[2*$i+3];
								$answer = $fields[2*$i+4];
								$sql = "REPLACE INTO TBL_FAQ_LANG
									(FAQ_ID, LANGUAGE_ID, QUESTION_TEXT, ANSWER_TEXT)
									VALUES (?, ?, ?, ?)";
								$q = $this->db_app->query($sql);
								$q->bind_values($faq_id, $language_id, $question, $answer);
								$q->execute();
								$affected += $q->affected_rows();
							}
						}
					}
					$this->db_app->commit();
				}
				$i++;
			}
			info('importCsvFAQs: imported '.$affected.' FAQs');
			return $affected;
		}

		function importCsvFAQorEmail($fileName,$what)
		// $what can be "FAQ" or "Email"
		{
			if($what == "FAQ")
			{
				$field0 = "FAQ_ID";
				$import = "importCsvFAQs";
			}
			else
				if($what == "Email")
				{
					$field0 = "MESSAGE_ID";
					$import = "importCsvEmails";
				}
			else
				return;

			info($import.': starting import from '.$fileName);
			if($fileName == '')
				throw new IWUserException('No file uploaded');
			$fo = new SplFileObject($fileName);
			$head = true;
			$baseLang = null;
			$destLangs = array();
			$col1 = array();
			$col2 = array();
			$affected = 0;

			$delimiter = ','; $enclosure = '"';
			$i = 0; while($fields = $fo->fgetcsv($delimiter, $enclosure))
			{
				if ($head)
				{
					info($import.': working on head row');
					if ($fields[0] != $field0)
						throw new IWUserException('Invalid file uploaded - '. $field0 .' column not found');
					if($what == "FAQ")
					{
						if (!@preg_match("/Q_BASE_LANG_(\d+)$/", $fields[1], $matches))
  							throw new IWUserException('Invalid file uploaded - Q_BASE_LANG column not found');
						if (!@preg_match("/A_BASE_LANG_(\d+)$/", $fields[2], $matches))
  							throw new IWUserException('Invalid file uploaded - A_BASE_LANG column not found');
					}
					else
					{
						if (!@preg_match("/SUBJECT_BASE_LANG_(\d+)$/", $fields[1], $matches))
  							throw new IWUserException('Invalid file uploaded - SUBJECT_BASE_LANG column not found');
						if (!@preg_match("/CONTENT_BASE_LANG_(\d+)$/", $fields[2], $matches))
  							throw new IWUserException('Invalid file uploaded - CONTENT_BASE_LANG column not found');
					}
  					$baseLang = $matches[1];
					for ($i=3; $i<sizeof($fields); $i++)
					{
						if (empty($fields[$i]))
							continue;
						if(($i % 2) != 0)
						{
							if($what == "FAQ"){
							if (!@preg_match("/Q_DEST_LANG_(\d+)$/", $fields[$i], $matches))
								throw new IWUserException('Invalid file uploaded - Q_DEST_LANG column not found');
							}
							else{
							if (!@preg_match("/SUBJECT_DEST_LANG_(\d+)$/", $fields[$i], $matches))
								throw new IWUserException('Invalid file uploaded - SUBJECT_DEST_LANG column not found');
							}
						}
						else
						{
							if($what == "FAQ"){
							if (!@preg_match("/A_DEST_LANG_(\d+)$/", $fields[$i], $matches))
								throw new IWUserException('Invalid file uploaded - A_DEST_LANG column not found');
							}
							else{
							if (!@preg_match("/CONTENT_DEST_LANG_(\d+)$/", $fields[$i], $matches))
								throw new IWUserException('Invalid file uploaded - CONTENT_DEST_LANG column not found');
							}
							debug('importCsvFAQs: adding dest lang '.$matches[1]);
							$destLangs[] = $matches[1];
						}
					}
					$head = false;
				}
				else
				{
					$this->db_app->begin();
					$ids = explode(';', $fields[0]);
					foreach($ids as $id)
					{
						if (!$id)
							continue;
						for ($i=0; $i<sizeof($destLangs); $i++)
						{
							foreach ($ids as $id)
							{
								$cont1 = $cont2 = true;
								$language_id = $destLangs[$i];
								if (empty($fields[2*$i+3]))
								{
									trigger_error('No translation for column '.(2*$i+3).' langID='.$language_id.' for id '.$id, E_USER_WARNING);
									$cont1 = false;
								}
								if (empty($fields[2*$i+4]))
								{
									trigger_error('No translation for column '.(2*$i+4).' langID='.$language_id.' for id '.$id, E_USER_WARNING);
									$cont2 = false;
								}
								if($cont1 == false && $cont2 == false)
								{
									echo " => <b>Not importing translation for id $id  for langID $language_id </b><br /><br />";
									continue;
								}
								$col1 = $fields[2*$i+3];
								$col2 = $fields[2*$i+4];

								// save as html to db for tinymce editor (and html email)
								$col2 = nl2br($col2);

								if($what == "FAQ")
								{
									$sql = "REPLACE INTO TBL_FAQ_LANG
										(FAQ_ID, LANGUAGE_ID, QUESTION_TEXT, ANSWER_TEXT)
										VALUES (?, ?, ?, ?)";
									$q = $this->db_app->query($sql);
									$q->bind_values($id, $language_id, $col1, $col2);
									$q->execute();
									$affected += $q->affected_rows();
								}
								else
								{
									$base_lang = $fields[2];
									$content = $col2;

									$matches = array();
									preg_match_all("/(<%\w+%>)/", $base_lang, $matches);
									foreach ($matches[0] as $match) {
										if (strpos($content, $match) === false) {
											trigger_error(sprintf('missing replacement fields for [%s]: [%s]. "%s" => "%s"', $fields[0], $match, $base_lang, $content), E_USER_WARNING);
											continue;
										}
									}

								    $sql = "REPLACE INTO TBL_MESSAGE_LANG
										(MESSAGE_ID, LANGUAGE_ID, SUBJECT, TINY_CONTENT)
										VALUES (?, ?, ?, ?)";
								    $q = $this->db_app->query($sql);
								    $q->bind_values($id, $language_id, $col1, $col1);
									$q->execute();
									$affected += $q->affected_rows();
									$sql = "REPLACE INTO TBL_MESSAGE_CONTENT
										(MESSAGE_ID, LANGUAGE_ID, CONTENT_TYPE, CONTENT)
										VALUES (?, ?, ?, ?)";
									$q = $this->db_app->query($sql);
									$q->bind_values($id, $language_id, 'text/html', $col2);
									$q->execute();
								}
							}
						}
					}
					$this->db_app->commit();
				}
				$i++;
			}
			info($import.': imported '.$affected.' translations');
			return $affected;
		}

		function importCsv($fileName)
		{
			info('importCsv: starting import from '.$fileName);

			if($fileName == '')
				throw new IWUserException('No file uploaded');

			$fo = new SplFileObject($fileName);

			$head = true;
			$baseLang = null;
			$destLangs = array();
			$translations = array();
			$affected = 0;

			$delimiter = ';'; $enclosure = '"';
			$i = 0; while($fields = $fo->fgetcsv($delimiter, $enclosure))
			{
				// BUG? - strip a last \n from the last field if it exists (there might be 2)
				//if ( substr($fields[sizeof($fields)-1], -1) == "\n")
					//$fields[sizeof($fields)-1] = substr($fields[sizeof($fields)-1], 0, -1);

				if ($head)
				{
					info('importCsv: working on head row');

					// process head columns
					if ($fields[0] != 'HANDLE')
						//trigger_error('Invalid CSV: Handle column not found', E_USER_ERROR);
						throw new IWUserException('Invalid file uploaded - HANDLE column not found');

					// read baselang
					if (!@preg_match("/BASE_LANG_(\d+)$/", $fields[1], $matches))
						//trigger_error('Invalid CSV: BaseLang not found', E_USER_ERROR);
  						throw new IWUserException('Invalid file uploaded - BASE_LANG column not found');
					$baseLang = $matches[1];

					// read destination langs
					for ($i=2; $i<sizeof($fields); $i++)
					{
						if (empty($fields[$i])) // dunno why this can happen but it did
							continue;

						if (!@preg_match("/DEST_LANG_(\d+)$/", $fields[$i], $matches))
							throw new IWUserException('Invalid file uploaded - DEST_LANG column not found');

						debug('importCsv: adding dest lang '.$matches[1]);

						$destLangs[] = $matches[1];
					}

					$head = false;
				}
				else
				{
					// process normal row and insert into db
					// - get handle ids from first column
					// - fill translations from every column

					$this->db_langif->begin();

					$handle_ids = explode(';', $fields[0]);

					foreach($handle_ids as $handle_id)
					{
						if (!$handle_id)
							continue;

						for ($i=0; $i<sizeof($destLangs); $i++)
						{
							foreach ($handle_ids as $handle_id)
							{
								$base_lang = $fields[1];
								$language_id = $destLangs[$i];

								if (!isset($fields[$i+2]))
								{
									trigger_error('No translation for column '.($i+2).' on line '.$i, E_USER_WARNING);
									continue;
								}

								$translation = $fields[$i+2];

								//debug(sprintf('language_id: %s, $handle_id: %s, translation: %s, base_lang: %s', $language_id, $handle_id, $translation, $base_lang));

								$matches = array();
								preg_match_all("/({\w+})/", $base_lang, $matches);
								foreach ($matches[0] as $match) {
									if ($translation != '' && strpos($translation, $match) === false) {
										trigger_error(sprintf('missing replacement fields for handle(s) [%s]. "%s" => "%s"', $handle_ids, $base_lang, $translation), E_USER_ERROR);
									}
								}

								$sql = "REPLACE INTO TBL_TRANSLATIONS
									(FK_LANGUAGE_ID, FK_TRANSLATION_HANDLE_ID, TRANSLATION)
									VALUES (?, ?, ?)";
								$q = $this->db_langif->query($sql);
								$q->bind_values($language_id, $handle_id, $translation);


								$q->execute();

								$affected += $q->affected_rows();
							}
						}
					}

					$this->db_langif->commit();

				}
				$i++;
			}

			info('importCsv: imported '.$affected.' translations');

			return $affected;
		}


		/**
		 * Gets all handles with translations etc for the current script
		 * @param int $secondlanguage_id set ID of second language
		 * @see pageAdmin()
		 * @return array $rows
		 */
		function getPageAdminHandles($handles)
		{

			$bind_args = array(
				$this->language_id,
				$this->secondlanguage_id
			);

			$strHandles = '';
			foreach ($handles as $handle) {
				$strHandles .= "?,";
				$bind_args[] = $handle;
			}
			$strHandles = substr($strHandles, 0, -1);

			$sql = "SELECT
						TRANSLATION_HANDLE_ID,
						TRANSLATION_HANDLE,
						th.GROUPS as groups,
					    tr1.TRANSLATION as tr1_translation,
					    tr1.FK_LANGUAGE_ID as tr1_language_id,
						tr1.OUTOFDATE as tr1_outofdate,
					    tr2.TRANSLATION as tr2_translation,
					    tr2.FK_LANGUAGE_ID as tr2_language_id,
						tr2.OUTOFDATE as tr2_outofdate
					FROM
					    TBL_TRANSLATION_HANDLES as th
					LEFT JOIN TBL_TRANSLATIONS tr1
					    ON TRANSLATION_HANDLE_ID = tr1.FK_TRANSLATION_HANDLE_ID
					    AND tr1.FK_LANGUAGE_ID=?
					LEFT JOIN TBL_TRANSLATIONS tr2
					    ON TRANSLATION_HANDLE_ID = tr2.FK_TRANSLATION_HANDLE_ID
					    AND tr2.FK_LANGUAGE_ID=?
					WHERE th.TRANSLATION_HANDLE IN (".$strHandles.") ";

			# hide set translations if requested
			if (!$this->showSetTranslation)
				$sql .= ' AND ( tr1.TRANSLATION IS NULL OR tr2.TRANSLATION IS NULL) ';

			$ex = $this->db_langif->query($sql);

			call_user_func_array(array($ex, 'bind_values'), $bind_args);

			$ex->execute();

			return $this->processAdminQuery($ex);
		}

		/**
		 * Gets all handles with translations etc for the current script
		 * @param int $secondlanguage_id set ID of second language
		 * @see pageAdmin()
		 * @return array $rows
		 */
		function getAdminHandles($emptyOnly, $limitRows, $firstLanguage, $secondLanguage, $filter)
		{
			$sql = "SELECT TRANSLATION_HANDLE_ID, TRANSLATION_HANDLE,
						th.GROUPS as groups,
					    tr1.TRANSLATION as tr1_translation,
					    tr1.FK_LANGUAGE_ID as tr1_language_id,
						tr1.OUTOFDATE as tr1_outofdate,
					    tr2.TRANSLATION as tr2_translation,
					    tr2.FK_LANGUAGE_ID as tr2_language_id,
						tr2.OUTOFDATE as tr2_outofdate
					FROM
					    TBL_TRANSLATION_HANDLES as th
					LEFT JOIN TBL_TRANSLATIONS tr1
					    ON TRANSLATION_HANDLE_ID = tr1.FK_TRANSLATION_HANDLE_ID
					    AND tr1.FK_LANGUAGE_ID=?
					LEFT JOIN TBL_TRANSLATIONS tr2
					    ON TRANSLATION_HANDLE_ID = tr2.FK_TRANSLATION_HANDLE_ID
					    AND tr2.FK_LANGUAGE_ID=?
					WHERE th.GROUPS = GROUPS & ? AND (th.GROUPS != 0 OR ? = 0)
					";

			$ex = $this->db_langif->query($sql);
			$ex->bind_values( $firstLanguage,
			                  $secondLanguage,
			                  $filter['groups'], $filter['groups']);

			debug('groups: '.$filter['groups']);
			$ex->execute();

			return $this->processAdminQuery($ex, $filter, $limitRows, $emptyOnly);
		}

		/**
		 * used in above functions
		 */
		private function processAdminQuery($ex, $filter=array(), $limitRows=null, $emptyOnly=false)
		{
			# while fetching, filter the array
			$arr = array();
			$sort = array();

			while ($row = $ex->fetchAssoc())
			{
				# Apply filters
				if (!empty($filter['handle']) && !@preg_match($filter['handle'], $row['TRANSLATION_HANDLE']) )
					continue;

				if (!empty($filter['translation']) && !@preg_match($filter['translation'], $row['tr1_translation']) )
					continue;

				# empty only
				if ($emptyOnly && !empty($row['tr1_translation']))
					continue;

				# limit rows
				if (!is_null($limitRows) && sizeof($arr) > $limitRows)
					break;

				# add translation
				$arr[$row['TRANSLATION_HANDLE']] = $row;
				$sort[$row['TRANSLATION_HANDLE']] = $row['tr1_translation'];

				//debug('out of date: '.$row['tr2_outofdate']);
			}

			# sort it by tr1_translation
			array_multisort($sort, SORT_ASC, SORT_STRING, $arr);

			return $arr;

		}

		/**
		 * displays the page specifc admin form
		 * @see /scripts/bots/i-w/language_process.php where the form is submitted to.
		 */
		function pageAdmin( $new_win=false )
		{
			if (!$this->_allowModify()||!isset($_SESSION['session_user_id']))
				return;

			# see if we are a translator, then make the admin simple
			require_once(IWARE . 'Translator.php');
			$t = new Translator($this->db_app);
			$q = $t->getByUserId($_SESSION['session_user_id']);
			$editableLangs = array();
			while ($r = $q->fetch())
				$editableLangs[] = $r->LANGUAGE_ID;
			$this->simple_mode = (in_array($this->language_id, $editableLangs));

			# collect the output and later on put it in a string to return it
			ob_start();

			# javascript includes
			?>
				<script type="text/javascript" src="/<?= SITE_PATH; ?>includes/i-w/language_admin.js"></script>
				<script type="text/javascript">
					langAdmin = new LangAdmin('<?= SITE_PATH; ?>');
				</script>
			<?php

			# open in new win?
			if ($new_win)
			{
				if(is_array($new_win))
					echo '<div style="position: absolute; top: '.$new_win['top'].'; left: '.$new_win['left'].';">';
				$btn = new Form_Button('btnOpenWin', 'LangAdmin');
				$btn->onClick( sprintf("langAdmin.openLangWin( %s, %s, '%s', '%s' ); ", $this->application_id, $this->script_id, $this->script_name, (isset($_REQUEST['script_parent_id']) ? (int)$_REQUEST['script_parent_id'] : 0) ));
				echo $btn->getHtml();
				if(is_array($new_win))
					echo '</div>';
				return;
			}

			# normal page admin start
			?>
				<script type="text/javascript">

					// show language interface
					addEvent( window, 'load', function()
					{
						var show = getCookie('DisplayLangIf');
						displayNode(getElById('pageAdminFrame'), show);
						if (show)
							langAdmin.showPageAdminHandleTable(show);

						//
						var div = getElById('divTranslate');
						if (div)
						{
							div.innerHTML = 'Translate';
							addEvent(div, 'mouseover', function() {
								this.style.cursor = 'pointer';
							});

							addEvent(div, 'click', toggleTranslate);
						}
					});

					// toggles the langif table
					function toggleTranslate() {
						var show = toggleDisplay(getElById('pageAdminFrame'));
						setCookie('DisplayLangIf', show ? 1 : 0);
						if (show)
							langAdmin.showPageAdminHandleTable(show);
					}
				</script>

				<table id="pageAdminFrame" class="pageAdminFrame" style="display: none;">

					<caption>
						<?php
							if (function_exists('printHelp'))
								printHelp($this, 'Language Interface', array('title'=>'Language Interface', 'top'=>'0px') );
						?>
						Language Interface
					</caption>
					<tbody>
						<tr>
							<td>
								<div class="">
									<div style="float: left;">
										<?php
											$btn = new Form_Button('btnNew', 'New Handle');
											$btn->onclick("document.getElementById('frmNewHandle').style.display = 'block';");
											//if (!$this->simple_mode)
												//echo $btn->getHtml();
										?>
									</div>
									<div style="float: right;">
										<table>
											<tr>
												<td>
													&nbsp;
												</td>
												<td>
													<?php
														$chk = new Form_Checkbox('chkShowSetTranslations');
														$chk->setLabel('Hide set translations in list');
														$chk->setChecked(!$this->showSetTranslation);
														$chk->onclick('langAdmin.showSetTranslations_onchange(this);');
														echo $chk->getHtml();
													?>
												</td>
												<td>
													<?php
														# SELECT language
														$sql = new SQL_Select('TBL_LANGUAGE');
														$sql->addSelect('LANGUAGE_ID as VALUE');
														$sql->addSelect('LANGUAGE AS CAPTION');
														$ex = $this->db_app->query($sql->getText());
														$ex->execute();

														$sel = new Form_Select_DB('selLanguage', $ex);
														$sel->onChange("document.location.href='/".SITE_PATH."scripts/i-w/bots/language_process.php?Action=ChangeSecondLanguage&SecondLanguage='+this.value;");
														$sel->setSelectedValue($this->secondlanguage_id);
														$sel->setLabel('Second Language:');
														echo $sel->getHtml();
													?>
												</td>
											</tr>
											<tr>
												<td></td>
												<td>
													<?php
														$chk = new Form_Checkbox('chkShowEmptyTranslations');
														$chk->setLabel('Display handles for empty translations');
														$chk->setChecked($this->showEmptyTranslations);
														$chk->onclick('langAdmin.showEmptyTranslations_onchange(this);');
														echo $chk->getHtml();
													?>
												</td>
												<td align="right">
													<?php
														$btn = new Form_Button('btnAdmin', 'Language Admin');
														$btn->onclick("var win = new NewWindow(1000, 600); win.setSource('/".SITE_PATH."scripts/i-w/language_admin/?script_parent_id=".((isset($_REQUEST['script_parent_id'])) ? (int)$_REQUEST['script_parent_id'] : 0) ."'); win.open();");

														//if (!$this->simple_mode)
															//echo $btn->getHtml();
													?>
												</td>
											</tr>

											<tr>
												<td></td>
												<td>
													<?php
														$chk = new Form_Checkbox('chkFallbackToEnglish');
														$chk->setLabel('Fallback to english for empty translations');
														$chk->setChecked($this->fallbackToEnglish);
														$chk->onclick('langAdmin.fallBackToEnglish(this);');
														echo $chk->getHtml();
													?>
												</td>
											</tr>
										</table>



									</div>

									<div style="clear: both;"></div>
								</div>

								<? if ( !$this->disabled ): ?>

									<br />
									<!--
									<form method="post" action="/scripts/i-w/bots/language_process.php"
										    id="frmNewHandle" style="display: none;">
										<?php
											# hidden form fields
								            $h = new Form_Hidden('Action', 'Insert');
								            echo $h->getHtml();

											$tfData = array();

								            $txt = new Form_Text('handle');
								            $txt->setWidth(150);
											$row['Handle'] = $txt->getHtml();

								            $btn = new Form_Button('btnAdd', 'Add', 'submit');
											$row[''] = $btn->getHtml();
											$tfData[] = $row;

											$tf = new TableFactory($tfData);
											$tf->width = 'width="95%"';
											$tf->setCaption('<img src="/'.SITE_PATH.'images/block_arrow.png" /> New Handle', false);
											echo $tf->getXHTMLTable( array('table'=>'pageAdmin' ) );
										?>
										<br />
										<br />
									</form>
									-->
									<?php
										#  print the table
										//echo $this->adminHandleTable( $this->getPageAdminHandles(), 'Language Page-Admin');
									?>

									<form name="frmLangIf" method="post" action="/<?=SITE_PATH;?>scripts/i-w/bots/language_process.php">
									<?php
							            # hidden values
							            $in = new Form_Hidden('Action');
										echo $in->getHtml();

							            $in = new Form_Hidden('script_parent_id', (isset($_REQUEST['script_parent_id'])) ? (int)$_REQUEST['script_parent_id'] : 0);
										echo $in->getHtml();

							            $in = new Form_Hidden('firstlanguage_id', $this->language_id);
										echo $in->getHtml();

							            $in = new Form_Hidden('secondlanguage_id', $this->secondlanguage_id);
										echo $in->getHtml();


									?>

									<div id="divAdminHandles">Loading..</div>

									<div style="margin: 10px; float: left;">
										<?php
								            $btn = new Form_Button('btnSelectAll', 'Select All', 'button');
								            $btn->onClick("langAdmin.checkAll( document.getElementById('divAdminHandles') );");
								            echo $btn->getHtml() . '&nbsp;';

								            $btn = new Form_Button('btnDelete', 'Delete Handles', 'submit');
								            $btn->onClick("if (confirm('Are you sure you want to delete the selected handles?')) { document.frmLangIf.Action.value='Delete'; return true; } return false; ");
								            if (!$this->simple_mode)
								            	echo $btn->getHtml() . '&nbsp;';

								            $btn = new Form_Button('btnUpdate', 'Save Translations', 'button');
								            $btn->onClick("this.disabled = 'disabled'; langAdmin.updateHandles($$('frmLangIf'), function() { document.location.reload(); } );");
								            $btn->setAccessKey('S');
								            echo $btn->getHtml() . '&nbsp;';

								            $btn = new Form_Button('btnExport', 'Export Translations', 'submit');
								            $btn->onClick("document.frmLangIf.Action.value='Export';");
								            $btn->setAccessKey('E');
								            if (!$this->simple_mode)
								            	echo $btn->getHtml();
										?>

									</div>
									<div style="margin: 10px; float: right">
										Translations marked&nbsp;<span style="border: 1px dashed #ff0000; ">&nbsp;dashed red&nbsp;</span>&nbsp;are outofdate and can be exported seperatly.
									</div>
									<br style="clear: both" />

									</form>

									<script type="text/javascript">
										langAdmin.firstLanguageId   = <?=$this->language_id; ?>;
										langAdmin.secondLanguageId = <?=$this->secondlanguage_id; ?>;
										langAdmin.simple_mode = <?=$this->simple_mode ? 1 : 0; ?>;

										langAdmin.pageHandles = <?=exportArrayToJs($this->pageHandles); ?>;
										//alert(langAdmin.pageHandles);

										langAdmin.groups = <?=exportArrayToJs(Group::getGroups());?>;

									</script>

								<? else: ?>
									<p style="color: #f00;">I'm disabled</p>
								<? endif; // if $this->disabled ?>
							</td>
						</tr>
					</tbody>
				</table>
			<?php

			$html = ob_get_contents();
			ob_end_clean();

			return $html;
		}

	}
?>
