<?php
/**
 * @package I-Ware
 * @subpackage List
 * @since 12-jul-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class ListBuilder
{
	/**
	 * @var Database $db
	 */
	var $db;
	/**
	 * @var LanguageInterface $langIf
	 */
	var $langIf;
	/**
	 * @access private
	 * @var Array $_lists (ListObjects)
	 */
	var $_lists = array();
	/**
	 * Constructor
	 * @param Database &$db
	 * @param LanguageInterface $langIf
	 */
	function ListBuilder(&$db,$langIf=false)
	{
		$this->db =& $db;
		$this->langIf = $langIf;
	}
	/**
	 * Registers a list to the Builder that can be used
	 * @param %List &$list
	 */
	function registerList(&$list)
	{
		$this->_lists[$list->handle] =& $list;
	}
	/**
	 * Gets all the lists in the $_lists variable
	 * @return Array
	 */
	function &getLists()
	{
		return $this->_lists;
	}
	/**
	 * Gets a list with $handle from the $_lists variable
	 * @param string $handle
	 * @return %List &$list
	 */
	function &getList($handle)
	{
		return $this->_lists[$handle];
	}
	/**
	 * Creates a new instance of a list by its handle
	 * @param string $handle
	 * @return $handleList $list
	 */
	function &createList($handle,$type=LIST_HTML_OUTPUT)
	{
		$objName = $handle;
		if(file_exists(LISTS.APP_PATH.$objName.'.php'))
			require_once(LISTS.APP_PATH.$objName.'.php');
		elseif(file_exists(IWARE_LISTS.$objName.'.php'))
			require_once(IWARE_LISTS.$objName.'.php');
		else
		{
			trigger_error('Cannot locate List `'.$objName.'`',E_USER_WARNING);
			exit();
		}
		$list = new $objName($this->db, $this->langIf, $type);
		return $list;
	}
}
/**
 * @package I-Ware
 * @subpackage List
 * @since 12-jul-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class DefaultList
{
	/**
	 * @var Database $db
	 */
	var $db;
	/**
	 * @var string $handle
	 */
	var $handle = '';
	/**
	 * @var LanguageInterface $langIf
	 */
	var $langIf;
	/**#@+
	 * @access private
	 */
	/**
	 * @var string $_query
	 */
	var $_query;
	/**
	 * @var string $_orderBy
	 */
	var $_orderBy = '';
	/**
	 * @var string $_groupBy
	 */
	var $_groupBy = '';
	/**
	 * @var array $_groupByArray
	 */
	var $_groupByArray = array();
	/**
	 * @var array $_hiddenCols
	 */
	var $_hiddenCols = array();
	/**
	 * @var array $_where
	 */
	var $_where = array();
	/**
	 *
	 */
	var $_union = '';
	var $_showType = 0;

	/**
	 * override exportTitle to set a custom filename for the csv/xml export
	 */
	var $exportTitle = '';

	/**#@-*/
	/**
	 * @param Database &$db
	 * @param LanguageInterface &$langIf
	 */
	function DefaultList($db,$langIf,$showType=LIST_HTML_OUTPUT)
	{
		$this->db =& $db;
		$this->langIf =& $langIf;
		$this->_showType = $showType;
		$this->_setWhere();
		$this->setQuery();
	}

	function setShowType($showType)
	{
		$this->_showType = $showType;
	}

	function showExport()
	{
		require_once(IWARE.'table_factory.php');

		$ds = $this->getDataSet();
		$tf = new TableFactory($ds);
		$tf->hideColumns($this->getHiddenColumns());
		$processor = $this->getProcessColumns();
		foreach($processor as $callback=>$columns)
			$tf->setValueProcessor($columns,array($this,$callback));

		if ($this->exportTitle)
			$fileTitle = $this->exportTitle;
		else
			$fileTitle = $this->handle.'Export';

		$ds = $tf->getFilteredDS();
		switch($this->_showType)
		{
			case LIST_BUTTON_CSV_EXPORT:
				require_once(IWARE.'IWCSVWriter.php');
				$writer = new IWCSVWriter($ds);
				$rv = $writer->getString();
				$contentType = 'text/plain';
				$fileTitle .= '.txt';
			break;
			case LIST_BUTTON_XML_EXPORT:
				require_once(IWARE.'IWXMLWriter.php');
				$writer = new IWXMLWriter($ds);
				$rv = $writer->getFormattedData();
				$contentType = 'octet-stream';
				$fileTitle .= '.xml';
			break;
		}
		header("Cache-control: ");
	    header("Pragma: ");
	    header(sprintf("Content-Type: application/%s",$contentType));
	    header(sprintf("Content-Disposition: attachment; filename=\"%s\" ", $fileTitle));
	    header("Content-transfer-Encoding: binary\n");
	    echo $rv;
		exit();
	}

	function setQuery()
	{

	}
	/**
	 * Returns the translation of the handle if the langIf variable is a LanguageInterface
	 * @param string $handle
	 * @return string
	 */
	function getText($handle)
	{
		if(!$this->langIf)
			return $handle;
		return $this->langIf->getText($handle);
	}
	/**
	 * Gets the $selectedValue from the filters if applicable.
	 * Returns a string with all set filters, used when printing lists
	 * @return string $rv
	 */
	function getSelectedFilters()
	{
		if(!sizeof($this->_where))
			return '';
		$rv = '';

		foreach($this->_where as $where)
		{
			list($obj,$func) = $where->callback;

			if($selectedValue = $obj->$func('selectedValue'))
				$rv .= '<span style="float:left; padding-right: 10px;">&#187; '.$selectedValue.'</span>';
		}

		return $rv;
	}
	/**
	 * Gets all the filter functions and their displays (usually selects)
	 * @param string $callback, The function that needs to be called on the click of the reload button
	 * @return string $rv
	 */
	function getFilters($callback)
	{
		if(!sizeof($this->_where))
			return '';
		$rv = '';
		foreach($this->_where as $where)
		{
			list($obj,$func) = $where->callback;
			$rv .= '<span style="float:left; padding-right: 10px;">'.$obj->$func('filter').'</span>';
		}

		return $rv.'<span style="float:left;"><input type="button" id="id_ListRefresh" onclick="'.$callback.'();" value="'.$this->getText('Refresh Dataset').'" /></span><span style="clear:both;"></span>';
	}
	/**
	 * Get all the filterable fields
	 * @return array $rv
	 */
	function getFilterFields()
	{
		$rv = array();
		foreach($this->_where as $where => $bitbucket)
			$rv[] = $where;
		return $rv;
	}
	/**
	 * Gets the executed dataset of the list
	 * @return ResultSet $q
	 */
	function getDataSet()
	{
		// Check on required where statements, if they return false, dont continue;
		if(($where = $this->_getWhere())===false)
			return false;

		$sql = '('.$this->_filterQuery().' '.$where.' '.$this->_getGroupBy().' '.$this->_getHaving().') '.$this->_union.$this->_orderBy;
		$q = $this->db->query($sql);
		$q->execute();
		return $q;
	}
	function tfFooter()
	{
		return false;
	}
	function tfSortingHat()
	{
		return false;
	}
	function getButtonBarText()
	{
		return  false;
	}
	function tfHeaderProc()
	{
		return array();
	}
	/**
	 * Default Value Processor for the tableFactory
	 */
	function tfValueProcessor($args)
	{
		return $args['value'];
	}
	/**
	 * Gets the columns that have a value processor
	 */
	function getProcessColumns()
	{
		return array();
	}
	/**
	 *
	 */
	function setEmptyText()
	{
		return '';
	}
	/**
	 *
	 */
	function getPostScript()
	{
		return '';
	}

	function _getCSVButton()
	{
		return array('CSV Export'=>array('onclick'=>"exportView('csv');"));
	}
	function _getXMLButton()
	{
		return array('XML Export'=>array('onclick'=>"exportView('xml');"));
	}
	/**
	 *
	 */
	function getButtons()
	{
		return array();
	}
	/**
	 * Gets all the columns that should be hidden in the list
	 */
	function getHiddenColumns()
	{
		return $this->_hiddenCols;
	}
	/**
	 * Gets the columns and their modifier
	 */
	function getColumnModifier()
	{
		return array();
	}

        /**
         * Gets collapsed columns
         */
        function getCollapseColumns()
        {
            return array();
        }
	/**#@+
	 * @access private
	 */
	/**
	 * Abstract for the Query filter, override and change anything in the _query var to your likings
	 */
	function _filterQuery()
	{
		return $this->_query;

	}

	function _getHaving()
	{
		return '';
	}

	/**
	 * @return string $rv
	 */
	function _getGroupBy()
	{
		$rv = $this->_groupBy;
		if(!$this->_groupByArray)
			return $rv;
		$rv .= (!$rv)?"GROUP BY ":', ';
		$rv .= implode(', ',$this->_groupByArray);
		return $rv;
	}
	/**
	 * @return mixed
	 */
	function _getWhere()
	{
		if(!sizeof($this->_where))
			return '';
		$rv = '';
		foreach($this->_where as $col => $where)
		{
			list($obj,$func) = $where->callback;
			if(!method_exists($obj,$func))
				trigger_error('Undefined function '.$func.' on Object',E_USER_WARNING);

			$value = $obj->$func('setValue');
			if(!$where->useinquery)
			{
				if(empty($value)&& $where->required&& $value!==0)
					return false;
				continue;
			}

			if($where->completedStatement)
			{
				$rv .= $where->completedStatement.' AND  ';
			}
			elseif(!is_null($value))
			{
				if(empty($value)&& $where->required&& $value!==0)
					return false;
				if(empty($value)&& $value!==0)
					continue;
				$this->_addHiddenCol($col);
				$this->_addHiddenCol($where->related);
				$this->_removeHiddenCol($where->showCols);
				if($where->groupBy)
					$this->_addGroupBy($where->groupBy);
				$rv .= $where->field.' '.$where->operator.' \''.$value.'\' AND  ';
			}
			else
			{
				$rv .= 'ISNULL('.$where->field.') AND  ';
			}
		}
		if(!$rv)
			return $rv;
		return 'WHERE '.substr($rv,0,-5);
	}
	/**
	 * Override this function
	 */
	function _setWhere()
	{

	}
	/**
	 * Add some $add to &$arr
	 * @param mixed $add
	 * @param array &$arr
	 */
	function _add($add,&$arr)
	{
		$add = (array)$add;
		foreach($add as $val)
			$arr[$val] = $val;
	}
	/**
	 * Add extra group by fields to the group by array
	 * @param mixed $gb
	 */
	function _addGroupBy($gb)
	{
		$this->_add($gb, $this->_groupByArray);
	}
	/**
	 * Add extra hidden cols to the hidden cols array
	 * @param mixed $col
	 */
	function _addHiddenCol($col)
	{
		$this->_add($col, $this->_hiddenCols);
	}
	/**
	 * Remove hidden cols if applicable
	 * @param mixed $col
	 */
	function _removeHiddenCol($col)
	{
		if(!$col)
			return;
		$col = (array) $col;
		foreach($col as $name)
			unset($this->_hiddenCols[$name]);
	}
	/**
	 * Add a new where object to the where array
	 * @param string $index
	 * @param array $args
	 */
	function _addWhere($index,$args)
	{
		$this->_where[$index] = new ListWhere($args);
	}

	protected function getCurrentUrl() {
		$uri_parts = explode('?', $_SERVER['REQUEST_URI'], 2);
		return $uri_parts[0];
	}

	/**#@-*/
}
/**
 * @package I-Ware
 * @subpackage List
 * @since 12-jul-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class ListWhere
{
	/**
	 * The field to query in the where statement, this includes the table name
	 * @var string $field
	 */
	var $field = '';
	/**
	 * Callback function array('OBJECT','FUNCTION')
	 * @var array $callback
	 */
	var $callback = '';
	/**
	 * Is the where a required where, if not then the query can continue without
	 * @var bool $required
	 */
	var $required = false;
	/**
	 * Related fields to this field, those will be hidden if the where is selected
	 * @var mixed $related
	 */
	var $related = '';
	/**
	 * @var mixed $groupBy
	 */
	var $groupBy = false;
	/**
	 * Comparison operator (=,!=,etc.)
	 * @var string $operator
	 */
	var $operator = ' = ';
	var $completedStatement = false;
	/**
	 *
	 */
	var $showCols = false;
	var $useinquery = true;
	/**
	 * Constructor
	 * @param array $props
	 */
	function ListWhere($props)
	{
		foreach($props as $prop => $val)
			$this->$prop = $val;
	}
}
define('LIST_HTML_OUTPUT',0);
define('LIST_BUTTON_CSV_EXPORT',1);
define('LIST_BUTTON_XML_EXPORT',2);
?>
