<?php
/**
 *	MySQL_Query provides all of the MySQL-specific functionality for Query objects. It is accessed through the Database class API.
 *
 *	@package I-Ware
 *	@subpackage Database
 *	@author John Luxford <lux@simian.ca>
 *	@since 2001-05-29
 *	@version 1.0
 *	@copyright 2001, Simian Systems
 *	@license http://www.simian.ca/license.php
 */
class MySQL_Query extends Query
{
	/**
	 *	@var string $sql
	 */
	var $sql = "";
	/**
	 *	@var Result $result
	 */
	var $result = "";
	/**
	 *	@var string $field
	 */
	var $field = "";
	/**
	 *	@var Resource $connection
	 */
	var $connection = "";
	var $exception = false;
	var $noCommitMode = false;
	var $outputFile = "";

	/**
	 *	Constructor method.
	 *
	 *	@param string $sql
	 *	@param Resource $conn
	 */
	function MySQL_Query ($sql = "", &$db) {
		$this->exception = false;
		$this->Query ($sql, $db);
	}

	/**
	 * Sets {@link $this->exception} with a {@link DatabaseException}
	 * @param int $errorCode A MySQL Errorcode
	 * @param string $errorMsg Description of $errorCode
	 * @return bool Always false
	 */
	function throwException( $errorCode, $errorMsg )
	{
		require_once( IWARE."database_exception.php" );
		$errorcode_map = array(
			0 => DB_ERROR_NOSUCHTABLE,
            1004 => DB_ERROR_CANNOT_CREATE,
            1005 => DB_ERROR_CANNOT_CREATE,
            1006 => DB_ERROR_CANNOT_CREATE,
            1007 => DB_ERROR_ALREADY_EXISTS,
            1008 => DB_ERROR_CANNOT_DROP,
            1022 => DB_ERROR_ALREADY_EXISTS,
            1046 => DB_ERROR_NODBSELECTED,
            1048 => DB_ERROR_CONSTRAINT,
            1050 => DB_ERROR_ALREADY_EXISTS,
            1051 => DB_ERROR_NOSUCHTABLE,
            1054 => DB_ERROR_NOSUCHFIELD,
            1062 => DB_ERROR_ALREADY_EXISTS,
            1064 => DB_ERROR_SYNTAX,
            1100 => DB_ERROR_NOT_LOCKED,
            1136 => DB_ERROR_VALUE_COUNT_ON_ROW,
            1146 => DB_ERROR_NOSUCHTABLE,
            1216 => DB_ERROR_CONSTRAINT,
            1217 => DB_ERROR_CONSTRAINT
		);

		$e = isset($errorcode_map[$errorCode])?$errorcode_map[$errorCode]:DB_ERROR;
		$this->exception = new DatabaseException( $e, $errorCode, $errorMsg );
		return false;
	}

	# http://dev.mysql.com/doc/refman/5.0/en/error-messages-server.html
	# Add your own exceptions if needed!
	private $exceptionMap = array(
		'MysqlForeignKeyException' => array(1451, 1452)
	);

	/**
	 *	Executes the current SQL query.
	 */
	function execute ( $dontDieOnMe = false )
	{
		if( ! $this->result = mysql_query ($this->tmp_sql, $this->connection))
		{
			$error = mysql_error($this->connection);
			$errno = mysql_errno($this->connection);
			$this->rollback();

			if( $dontDieOnMe ) # this is deprecated
			{
				trigger_error('execute(true) is deprecated.. use exceptions', E_USER_NOTICE);
				return $this->throwException($errno, $error);
			}
			else
			{
				$errstr = $error.' ('.$errno.')'." : ".$this->tmp_sql . ' ; on '.$this->db->name." at " .  getenv("REQUEST_URI");

				# check exceptionMap for errno, else throw default exception
				foreach ($this->exceptionMap as $class => $error_codes)
					if (in_array($errno, $error_codes))
						throw new $class($errstr, $errno);

				throw new MysqlUnknownException($errstr, $errno);
			}
		}

		//BEGIN Query Profiler
		if ( $_SERVER['SERVER_NAME'] == 'nestorix' && defined('ERROR_NO_FRIENDLY') && defined('IWARE') && defined('PROFILE')
				&& (strpos($this->tmp_sql, 'EXPLAIN') !== 0)  //prevent endless loop
				&& ($selpos = strpos($this->tmp_sql, 'SELECT')) !== false //catch CREATE TEMPORARY TABLE
				&& (strpos($this->tmp_sql, 'WHERE') !== false
						|| strpos($this->tmp_sql, 'JOIN') !== false
					) //only if WHERE condition or JOINing
			)
		{
//			$q = $this->db->query('EXPLAIN '.substr($this->tmp_sql, $selpos));
			$q = new MySQL_Query('EXPLAIN '.substr($this->tmp_sql, $selpos), $this->db);
			$q->execute();
			while ( $r = $q->fetch() )
				if ( $r->type == 'ALL' ) //&& $r->rows > 40 ) //Only display (potentially slow joins)
				{
					require_once(IWARE.'table_factory.php');
					//require_once('/home/chris/public_html/iquest/classes/i-w/table_factory.php');
					$q->data_seek(); //reset
					$tf = new TableFactory($q);
					if ( function_exists('debug') )
					{
						notice("Profiler: Potentially slow query. Dumping explain...\n$q->tmp_sql\n".
						$tf->getPlainTextTable().
						"\n--------------------------------\n");
					}
					else
					{
						$tf->hot = true;
						echo "<hr /><pre>$q->tmp_sql</pre>";
						echo $tf->getXHTMLTable('Professional');
						echo "<hr />";
						flush();
					}
					break;
				}
		}
		//END Query Profiler

		if (isset($file))
		{
			fwrite($file, " : " . $this->affected_rows() . " \n");
			fclose($file);
		}

		return $this->result;
	}

	/**
	 *	Returns the name of the specified column in the table currently being queried.
	 *
	 *	@param integer $num
	 *	@return string
	 */
	function field( $num = 0 )
	{
		if( $this->result )
			return mysql_field_name ($this->result, $num);
		return 0;
	}

	/**
	 *	Returns the number of rows affected or found by the current query.
	 *
	 *	@return integer
	 */
	function rows()
	{
		if( $this->result )
			return mysql_num_rows( $this->result );
		return 0;
	}

	/**
	 *	Returns the number of rows affected or found by the current query.
	 *
	 *	@return integer
	 */
	function cols()
	{
		if( $this->result )
			return mysql_num_fields( $this->result );
		return 0;
	}

	/**
	 *	Returns the row ID generated by the database during the previous SQL insert query.
	 *
	 *	@return integer
	 */
	function lastid ()
	{
		if( $this->result )
			return mysql_insert_id ($this->connection);
		return 0;
	}

	function fetch ()
	{
		if ($this->result)
			return mysql_fetch_object ($this->result);
		else
			return 0;
	}

	function fetchAssoc()
	{
		if ($this->result)
			return mysql_fetch_assoc($this->result);
		else
			return 0;
	}
	/**
	 *	Tells the database system to let go of its data from the previous query.
	 *
	 *	@return bool
	 */
	function free()
	{
		if( $this->result )
			return mysql_free_result( $this->result );
	}

	/**
	 *
	 *	@param integer $rownum
	 *	@return bool
	 **/
	function data_seek($rownum = 0)
	{
		if( $this->result )
		{
			if( @mysql_data_seek( $this->result, $rownum ))
				return true;
			return false;
		}
	}

	/**
	 *	Issues a mysql "begin" statement, beginning a transaction.
	 */
	function begin()
	{
		mysql_query( "begin", $this->connection );
	}

	/**
	 *	Issues a mysql "commit" statement, committing the current transaction.
	 */
	function commit()
	{
		mysql_query( "commit", $this->connection );
	}

	/**
	 *	Issues a mysql "rollback" statement, rolling back the current transaction.
	 *
	 *	@return bool
	 */
	function rollback()
	{
		return mysql_query( "rollback", $this->connection );
	}

	/**
	 *	Return the number of affected rows.
	 *
	 *	@return integer
	 */
	function affected_rows()
	{
		return mysql_affected_rows( $this->connection );
	}

	/**
	 *	Gets the result set and returns it as an associated and numerical array
	 *
	 *	@return array
	 */
	function fetch_array()
	{
		if ($this->result)
			return mysql_fetch_array( $this->result );
		return 0;
	}
}

/**
 *	MySQL_Driver provides all of the MySQL-specific functionality for Database objects.  It is accessed through the Database class API.
 *
 *	@package I-Ware
 *	@subpackage Database
 *	@author John Luxford <lux@simian.ca>
 *	@since 2001-05-29
 *	@version 1.0
 *	@copyright 2001, Simian Systems
 *	@license http://www.simian.ca/license.php
 */
class MySQL_Driver
{
	/**
	 *	@var Resource $connection
	 */
	var $connection;
	/**
	 *	@var string $host
	 */
	var $host;
	/**
	 *	@var string $name
	 */
	var $name;
	/**
	 *	@var string $user
	 */
	var $user;
	/**
	 *	@var string $pass
	 */
	var $pass;
	/**
	 *	@var bool $persistent
	 */
	var $persistent;

	var $outputFile = "";  // See documentation @ database class

	function MySQL_Driver ($name = "", $host = "", $user = "", $pass = "", $persistent = 1) {
		$this->name = $name;
		$this->host = $host;
		$this->user = $user;
		$this->pass = $pass;
		$this->persistent = $persistent;
		$this->connect();
	}

	/**
	 *	Establishes a connection with the database
	 */
	function connect()
	{
		$connect = 'mysql_'.($this->persistent?'p':'').'connect';
		if ( !($this->connection = $connect($this->host, $this->user, $this->pass, true)) )
			trigger_error(sprintf("Could not connect to host [%s] with user [%s]", $this->host, $this->user), E_USER_ERROR);

		if ( !mysql_select_db($this->name, $this->connection) )
			trigger_error(sprintf("Could not select database [%s] on host [%s] with user [%s]", $this->name, $this->host, $this->user), E_USER_ERROR);
		if(!mysql_query('SET NAMES utf8'))
			trigger_error(sprintf("Could not set connection NAMES to utf8: database [%s] on host [%s] with user [%s]", $this->name, $this->host, $this->user), E_USER_ERROR);
	}

	/**
	 *	Creates and returns a Query object. Inherits full functionality from Database class
	 */
	function query( $sql = "" )
	{

	}
}

# Exceptions which can be throwed in this class
abstract class MysqlException extends Exception {}
class MysqlForeignKeyException extends MysqlException {}
class MysqlUnknownException extends MysqlException {}

?>
