<?php
 //Independency of mktime() limits.
define('SCHEDULE_CALCULATOR_ERROR', -1); //Throw would be nice....
/**
 * Calculate the next executiontime for cronlike masks.
 * @author Chris Wesseling <chris.wesseling@xs4all.nl>
 * @package I-Ware
 * @subpackage Task
 * @version $Id$
 */
class ScheduleCalculator
{
	var $year, $month, $day, $hour, $minute;

	/**
	 * @param String $dateformat a format string like used in strftime
	 */
	function ScheduleCalculator($dateformat='YmdHi')
	{
		$this->__df = $dateformat;
	}
	/**
	 * Get the next time the mask is met.
	 * Uses a cronlike mask with an extra place at the end.
	 * ie.
	 * minute hour dayofmonth month dayofweek year
	 *
	 * Like in cron asterisks, ranges and steps can be used.
	 * With the addition that * in year stands for 0-9999
	 *
	 * The string
	 * '*\/15 9-17/2 * * 1-5 2004,2006'
	 * stands for weekday's, odd officehours, every quarter of een hour in 2004 and 2006
	 * (Don't use the \ it's the because _ASTERISK__BACKSLASH_ ends the comment sequence)
	 *
	 * @see man 5 crontab
	 *
	 * @param String $mask Cronlike ScheduleMask with extra place at the end for year
	 * @param mixed $start Anything that goes into the PEAR::Date contructor like a UNIXTIME
	 * @return mixed Formatted Time or False if date doesn't occur in the future
	 */
	function getNextTime($mask, $start=null)
	{
		$start = $start?$start:currentDate();

		if(get_class($start) != 'IWDateTime')
			throw new IWException('You must use a IWDateObject or null in the ScheduleCalculater->getNextTime call');
		$this->year = $start->getYear();
		$this->month = $start->getMonth();
		$this->day = $start->getDay();
		$this->hour = $start->getHour();
		$this->minute = (int)$start->getMinute(); //we do this so tasks don't run too close together
		$this->__parseMask($mask);
		if ( $this->__filterImpossible() == SCHEDULE_CALCULATOR_ERROR )
			return false;
		if ( $this->calculateTime() == SCHEDULE_CALCULATOR_ERROR )
			return false;
		return $this->formatDate();
	}
	/**
	 * Format calculated Time
	 * @param String $format
	 * @return String
	 */
	function formatDate($format = null)
	{
		$format || $format = $this->__df;
		$date = new IWDateTime($this->year.
				str_pad($this->month, 2, '0', STR_PAD_LEFT).
				str_pad($this->day, 2, '0', STR_PAD_LEFT).
				str_pad($this->hour, 2, '0', STR_PAD_LEFT).
				str_pad($this->minute, 2, '0', STR_PAD_LEFT).
				'00'
				);
		return $date->format($format);
	}
	/**
	 * @return int SCHEDULE_CALCULATOR_ERROR on error
	 */
	function calculateTime()
	{
		if ( isset($this->__year_pos) )
		{
			if ( !in_array($this->year, $this->__year_pos) )
			{
				foreach ( $this->__year_pos as $year )
				{
					if ( $year < $this->year )
						continue;
					$this->year = $year;
					$this->__first('month');
					break;
				}
				if ( !in_array($this->year, $this->__year_pos) ) //still?
					return SCHEDULE_CALCULATOR_ERROR;
			}
			else //set the pos pointer
				while ( current($this->__year_pos) != $this->year )
					next($this->__year_pos);
		}
		else
			while ( $this->year % $this->__year_step !== 0 )
				$this->year++;


		$timeunits = array('month', 'day', 'hour', 'minute');
		foreach ( $timeunits as $i=>$timeunit )
		{
			reset($this->{'__'.$timeunit.'_pos'});
			if ( !in_array($this->$timeunit, $this->{'__'.$timeunit.'_pos'}) )
			{
				if ( $this->$timeunit > end($this->{'__'.$timeunit.'_pos'}) ) //now larger than last => first
				{
					$this->__first($timeunit);
					if ( isset($timeunits[$i-1]) )//iff exists a larger timeunit, increment it
					{
						if ( $this->__next($timeunits[$i-1]) == SCHEDULE_CALCULATOR_ERROR )
							return SCHEDULE_CALCULATOR_ERROR;
					}
					else
					{
						if ( $this->__nextYear() == SCHEDULE_CALCULATOR_ERROR ) //there's always a bigger fish
							return SCHEDULE_CALCULATOR_ERROR;
					}
				}
				elseif ( $this->$timeunit < reset($this->{'__'.$timeunit.'_pos'}) ) //now smaller then first => first
					$this->__first($timeunit);
				else
					while ( $next = next($this->{'__'.$timeunit.'_pos'}) ) //now somewhere between possibilities =>next pos
					{
						if ( $next > $this->$timeunit )
						{
							$this->$timeunit = $next;
							if ( isset($timeunits[$i+1]) )//iff exists a smaller timeunit, roll it to first
								$this->__first($timeunits[$i+1]);
							break;
						}
					}
			}
			else //set pos pointer to current
				while ( current($this->{'__'.$timeunit.'_pos'}) != $this->$timeunit )
					next($this->{'__'.$timeunit.'_pos'});
		}

		while ( !checkdate($this->month, $this->day, $this->year)
				|| !in_array(date('w', mktime($this->hour,
							$this->minute,
							0,
							$this->month,
							$this->day,
							$this->year)),
					$this->__wday_pos) )
			if ( $this->__next('day') == SCHEDULE_CALCULATOR_ERROR )
				return SCHEDULE_CALCULATOR_ERROR;
	}
	/**#@+
	 * @access protected
	 */
	function __nextYear()
	{
		if ( isset($this->__year_pos) )
		{
			if ( !$this->year = next($this->__year_pos) )
				return SCHEDULE_CALCULATOR_ERROR;
		}
		else
			while ( ++$this->year % $this->__year_step !== 0 )
				continue;
		if ( $this->year > $this->__year_max )
			return SCHEDULE_CALCULATOR_ERROR;
	}
	/**
	 * Set $this->$timeunit to the next posibility (and larger units if this unit rolls over to first)
	 * @param String $timeunit 'year'|'month'|'day'|'hour'|'minute'
	 */
	function __next($timeunit)
	{
		if ( $timeunit == 'year' )
			return $this->__nextYear();
		$timeunits = array('minute', 'hour', 'day', 'month');
		$entrycheck = true;
		foreach ( $timeunits as $unit )
		{
			if ( $entrycheck && $unit != $timeunit )
				continue;
			if ( !$this->$unit = next($this->{'__'.$unit.'_pos'}) )
			{
				$this->$unit = reset($this->{'__'.$unit.'_pos'});
				if ( $unit == 'month' )
					return $this->__nextYear();
			}
			else
				break;
			$entrycheck = false;
		}
	}
	/**
	 * Set $this->$timeunit (and smaller timeunits) to the first posibility
	 * @param String $timeunit 'month'|'day'|'hour'|'minute'
	 */
	function __first($timeunit)
	{
		$timeunits = array('month', 'day', 'hour', 'minute');
		$entrycheck = true;
		foreach ( $timeunits as $unit )
		{
			if ( $entrycheck && $unit != $timeunit )
				continue;
			$this->$unit = reset($this->{'__'.$unit.'_pos'});
			$entrycheck = false;
		}
	}
	function __parseMask($fullmask)
	{
		$parts = array('minute',
				'hour',
				'day',
				'month',
				'wday',
				'year');
		$fullmask = explode(' ', $fullmask);
		//foreach ( array_combine_custom($parts, split(' ',$fullmask)) as $part=>$mask )
		for ( $i = 0; $i < count($parts); $i++ )
		{
			list($part, $mask) = array($parts[$i], $fullmask[$i]);
			unset($this->{'__'.$part.'_pos'}); //clear posibility array
			if ( preg_match('%(\*)(?:/(\d+))?%', $mask, $match) )
			{
				if ( !empty($match[2]) )
					$this->{'__'.$part.'_step'} = $match[2];
				else
					$this->{'__'.$part.'_step'} = 1;
				if ( $part != 'year' )
					$this->{'__'.$part.'_pos'} = $this->__range($this->{'__'.$part.'_min'},
							$this->{'__'.$part.'_max'},
							$this->{'__'.$part.'_step'});
			}
			else
			{
				$this->{'__'.$part.'_pos'} = array();
				$members =& $this->{'__'.$part.'_pos'};

				preg_match_all('%(\d+)(?:-(\d+)(?:/(\d+))?)?%', $mask, $pieces, PREG_SET_ORDER);
				//1=start, 2=end, 3=step
				foreach ( $pieces as $piece )
					if ( empty($piece[2]) )
						$members[] = $piece[1];
					else
						$members = array_merge($members,
									$this->__range($piece[1],
										$piece[2],
										(isset($piece[3])?$piece[3]:1)));
			}
		}
	}
	function __range($from, $to, $step)
	{
		$range = range($from, $to);

		if ( $step == 1 )
			return $range;

		for ( $i=0; $i < count($range); $i+=$step )
				$rv[] = $range[$i];
		return $rv;
	}
	function __filterImpossible()
	{
		$daymax = null;
		$timeunits = array('month', 'day', 'hour', 'minute');
		foreach ( $timeunits as $unit )
		{
			$pos =& $this->{'__'.$unit.'_pos'};
			$min = $this->{'__'.$unit.'_min'};
			$max = ($unit != 'day')?$this->{'__'.$unit.'_max'}:$this->__determineDayMax();

			sort($pos, SORT_NUMERIC);
			while ( !empty($pos) && (end($pos) > $max) )
				array_pop($pos);
			while ( !empty($pos) && (reset($pos) < $min) )
				array_shift($pos);

			if ( empty($pos) )
				return SCHEDULE_CALCULATOR_ERROR;
		}
	}
	/**
	 * @return int
	 */
	function __determineDayMax()
	{
		$curmax = $max = 0;
		foreach ( $this->__month_pos as $m )
		{
			if ( in_array($m, array(4, 6, 9, 11)) )
				$max = 30; //30 dagen hebben april, juni, september en november
			elseif ( $m == 2 )
				$max = 29; //29 dagen heeft februari alleen
			else
				$max = 31; //en de rest 30 en 1

			if ( $curmax < $max )
				$curmax = $max;
			if ( $max == 31 )
				return 31;
		}
		return $curmax;
	}
	/**
	 * @var String DateFormat
	 */
	var $__df;
	var $__minute_min = 0;
	var $__minute_max = 59;
	var $__hour_min = 0;
	var $__hour_max = 23;
	var $__day_min = 1;
	var $__day_max = 31;
	var $__month_min = 1;
	var $__month_max = 12;
	var $__year_min = 0;
	//max of checkdate is actually 2^15-1
	var $__year_max = 9999; //the max of capital Y (and a lot sooner than 2^15)
	var $__wday_min = 0;
	var $__wday_max = 6;
	/**#@-*/
}
?>
