<?php
/**
 * @copyright 2006 BaseSoft B.V.
 * @author Marten Koedam <marten@basesoft.com>
 * @package I-Ware
 * @version $Id$
 */
class Snapshot
{
	/**
	 * @var string $db_user
	 */
	var $db_user;
	/**
	 * @var string $db_passwd
	 */
	var $db_passwd;
	/**
	 * @var string $db_name
	 */
	var $db_name;
	/**
	 * @var string $sql_server
	 */
	var $sql_server;
	/**
	 * @var string $path
	 */
	var $path;
	/**
	 * @param string $db_user;
	 * @param string $db_passwd;
	 */
	function Snapshot($db_user,$db_passwd,$db_name,$sql_server)
	{
		$this->db_user = $db_user;
		$this->db_passwd = $db_passwd;
		$this->db_name = $db_name;
		$this->sql_server = $sql_server;
		if(defined('SNAPSHOT_SAVE_PATH'))
			$this->path = SNAPSHOT_SAVE_PATH;
		else
			$this->path = '/home/clients/'.$_SESSION['session_user_company_name'].'/snapshots/';
		if(!is_dir($this->path))
		{
			// create directory structure
		}
	}
	/**
	 *
	 */
	function getList()
	{
		$rv = array();
		if($dh = opendir($this->path))
		{
			$i = 0;
			while(($file = readdir($dh)) !== false)
			{
				if($file == "." || $file == "..")
					continue;
				if(!preg_match('/\.SQL/',$file))
					continue;
				$rv[$i]['file'] = $file;
				$i++;
			}
			closedir($dh);
		}

		rsort($rv);

		return $rv;
	}
	/**
	 *
	 */
	function takeSnapshot($name)
	{
		if(!defined('TEST_SITE'))
			return false;

		$name = preg_replace('/[^\w\d]*/', '', $name);

		$dmp_name = $this->path.$name.'_'.date('YmdHis').'.SQL';
		$cmd = '(/usr/bin/mysqldump -u'.$this->db_user.' -p'.$this->db_passwd.' -h'.$this->sql_server.' --opt '.escapeshellarg($this->db_name).' > '.$dmp_name . ') 2>&1';

		$output = exec($cmd); // dont rely on exit code from mysql!
		if (!empty($output))
			throw new Exception(sprintf("Error creating snapshot. Command: '%s' Output: '%s'. ", $cmd, $output));

		info('succesful created snapshot ['.$dmp_name.']');

		return true;
	}
	/**
	 *
	 */
	function restoreSnapshot($fileName)
	{
		// Only if test site
		if(!defined('TEST_SITE'))
		{
			trigger_error('Cannot load snapshot: TEST_SITE is not defined', E_USER_ERROR);
			return false;
		}

		if(!file_exists($this->path.$fileName)) {
			trigger_error('Cannot load snapshot: cannot find file ('.$this->path.$fileName.')', E_USER_ERROR);
			return false;
		}

		$cmd = '(MYSQL_PWD='.$this->db_passwd.' /usr/bin/mysql -u'.$this->db_user.' '.$this->db_name.' -h'.$this->sql_server.' < '.$this->path.escapeshellarg($fileName).') 3>&1 1>&2 2>&3 ';
		debug('executing: '.'/usr/bin/mysql -u'.$this->db_user.' -p'.$this->db_passwd.' '.$this->db_name.' -h'.$this->sql_server.' < '.$this->path.escapeshellarg($fileName));

		$output = exec($cmd); // dont rely on exit code from mysqldump!
		if (!empty($output))
			throw new Exception(sprintf("Error restoring snapshot. Command: '%s' Output: '%s'. ", $cmd, $output));

		return true;
	}
	/**
	 *
	 */
	function deleteSnapshot($name)
	{
		// Only if test site
		if(!defined('TEST_SITE'))
			return false;

		if(!file_exists($this->path.$name))
			return false;
		return unlink($this->path.$name);
	}
}
?>
