<?php
if(file_exists(SITE_HOMEDIR . SITE_PATH.'classes/'.APP_PATH.CLASS_PREFIX.'SuckerfishMenu.php'))
	require_once(SITE_HOMEDIR . SITE_PATH.'classes/'.APP_PATH.CLASS_PREFIX.'SuckerfishMenu.php');
/**
 * SuckerFish
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright Copyright 2005, BaseSoft b.v.
 * @package I-Ware
 * @since 16-11-2005
 * @version $Id$
 */
class SuckerfishMenu
{
	/**
	 * @var Database $db IWARE Database
	 */
	var $db;
	/**
	 * @var Database $dbApp APPLICATION Database
	 */
	var $dbApp;
	/**
	 *
	 */
	var $langIf = false;
	/**
	 *
	 */
	var $menu = array();

	var $scripts = array();

	/**
	 * @param Database $db
	 * @param Database $dbApp
	 */
	function SuckerfishMenu($db,$dbApp,&$langIf,$menuItem=1)
	{
		$this->langIf =& $langIf;
		$this->db = $dbApp;
		$this->dbApp = $dbApp;

		$cachable = function_exists('apc_store');
		if($cachable)
		{
			$lookup = 'SFMenu'.$_SESSION['session_user_id'].'_'.
				(isset($_SESSION['selected_participant_id'])?$_SESSION['selected_participant_id']:0).'_'.
				(isset($_SESSION['selected_product_id'])?$_SESSION['selected_product_id']:0).APC_SUFFIX;
			if($menu = apc_fetch($lookup))
			{
				list($this->menu,$this->scripts) = $menu;
				return;
			}
		}

		$this->_init($this->_filter(),$menuItem);
		$this->_postFilter();
		if($cachable)
		{
			$menu = array($this->menu,$this->scripts);
			apc_store($lookup,$menu);
		}
	}

	static function create($db,$dbApp,$langIf,$menuItem=1)
	{
		$file = SITE_HOMEDIR . SITE_PATH.'classes/'.APP_PATH.CLASS_PREFIX.'SuckerfishMenu.php';
		if (file_exists($file))
		{
			$class = CLASS_PREFIX.'SuckerfishMenu';
			$that = new $class($db,$dbApp,$langIf,$menuItem);
			return $that;
		}

		$that = new SuckerfishMenu($db,$dbApp,$langIf,$menuItem);
		return $that;
	}

	function _postFilter()
	{
	}

	function hasAccess($script_parent_id, $script_id)
	{
		return isset($this->scripts[$script_parent_id][$script_id]);
	}

	function unsetScript($script_parent_id, $script_id)
	{
		if (isset($this->menu[$script_parent_id][$script_id]))
			unset($this->menu[$script_parent_id][$script_id]);

		if (isset($this->scripts[$script_parent_id][$script_id]))
			unset($this->scripts[$script_parent_id][$script_id]);

		if (isset($this->menu[$script_id]))
			unset($this->menu[$script_id]);

		if (isset($this->scripts[$script_id]))
			unset($this->scripts[$script_id]);
	}

	/**
	 *
	 */
	function getText($handle)
	{
		return $this->langIf?$this->langIf->getText($handle):$handle;
	}
	/**#@+
	 * @access protected
	 */
	/**
	 * @param Array $rights (SCRIPT_ID [,SCRIPT_ID ...])
	 */
	function _init($rights,$menuItem = 1)
	{
		if(sizeof($rights)==0)
			return false;
		$sql = "SELECT SCRIPT_ID, SCRIPT_NAME, SCRIPT_PARENT_ID, SCRIPT_PATH, SCRIPT_MENUITEM
			FROM TBL_SCRIPT
			WHERE APPLICATION_ID = ?
				AND SCRIPT_ID IN( ".implode(',',$rights)." )
			ORDER BY SCRIPT_PARENT_ID, SCRIPT_POSITION";
		$q = $this->db->query($sql);
		$q->bind_values(array(APPLICATION_ID));
		$q->execute();
		while($r = $q->fetch())
		{
			if ($r->SCRIPT_MENUITEM)
				$this->menu[$r->SCRIPT_PARENT_ID][$r->SCRIPT_ID] = array('SCRIPT_PATH'=>$r->SCRIPT_PATH, 'SCRIPT_NAME'=>$r->SCRIPT_NAME);

			$this->scripts[$r->SCRIPT_PARENT_ID][$r->SCRIPT_ID] = array('SCRIPT_PATH'=>$r->SCRIPT_PATH, 'SCRIPT_NAME'=>$r->SCRIPT_NAME);
		}
	}
	/**
	 *
	 */
	function _filter()
	{
		$sql = "SELECT SCRIPT_ID
			FROM TBL_SCRIPT_GROUP
			WHERE GROUP_COMPANY_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values(array($_SESSION['session_group_company_id']));
		$q->execute();
		$rights = array(0);
		while($r = $q->fetch())
			$rights[$r->SCRIPT_ID] = $r->SCRIPT_ID;
		return $rights;
	}
	/**
	 *
	 */
	function _build($current,$top=true)
	{
		$this->__build($current,$top);
	}

	private function __build($current,$top=true)
	{
		$rv = ($top?'':'<br />').'<ul'.($top?' id="nav" class="first"':'').'>' . "\n";
		$class = $top?$this->aClass:$this->secondClass;
		$devLink = '<a id="sfMenu%s" '.$class.' href="%s">%s</a>';

		foreach($current as $id => $script)
		{
			$path = $script['SCRIPT_PATH'];
			$path = $path?'/'.SITE_PATH.$path:'#';
			$name = $script['SCRIPT_NAME'];

			$rv .= '<li>';
			$tr = $this->getText('Menu ' . $name);
			if (empty($tr))
				$tr = 'Menu '.$this->getText('Menu'.$id);
			$rv .= sprintf($devLink,$id,$path,$tr);

			$rv .= (isset($this->menu[$id])?$this->__build($this->menu[$id],false):'');
			$rv .= '</li>' . "\n";

		}
		$rv .= '</ul>' . "\n";
		return $rv;
	}

	/**#@-*/
	/**
	 * 	<ul id="nav">
	 * 		<li><a href="#">Percoidei</a>
	 *			<ul>
	 *				<li><a href="#">Remoras</a></li>
	 *				<li><a href="#">Tilefishes</a></li>
	 * 				<li><a href="#">Bluefishes</a></li>
	 *				<li><a href="#">Tigerfishes</a></li>
	 *			</ul>
	 *		</li>
	 *		<li><a href="#">Anabantoidei</a>
	 *			<ul>
	 *				<li><a href="#">Climbing perches</a></li>
	 *				<li><a href="#">Labyrinthfishes</a></li>
	 *				<li><a href="#">Kissing gouramis</a></li>
	 *				<li><a href="#">Pike-heads</a></li>
	 *				<li><a href="#">Giant gouramis</a></li>
	 *			</ul>
	 *		</li>
	 *		<!-- etc. -->
	 *	</ul>
	 */
	function display($class='', $target='',$secondMenuClass='', $lookup='menu')
	{
		$this->aClass = $class;
		$this->aTarget = $target;
		$this->secondClass = $secondMenuClass?'class="'.$secondMenuClass.'"':'';

		$rv = '';
		if(sizeof($this->$lookup)==0)
			return $rv;
		$rv = ''.$this->__build($this->{$lookup}[0]).'';
		return $rv;
	}

	/**
	 * @todo make work with $this->scripts
	 */
	function getSiteMapBar($script_id)
	{
		if($script_id == 0)
			return false;
		$sql = "SELECT SCRIPT_PARENT_ID, SCRIPT_PATH, SCRIPT_NAME
			FROM TBL_SCRIPT
			WHERE SCRIPT_ID = ?
				AND APPLICATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values(array($script_id,APPLICATION_ID));
		$q->execute();
		$r = $q->fetch();

		$rv = $this->getSiteMapBar($r->SCRIPT_PARENT_ID);
		$rv[$script_id] = array('SCRIPT_PATH'=>$r->SCRIPT_PATH, 'SCRIPT_NAME'=>$r->SCRIPT_NAME);
		return $rv;
	}
}

?>
