<?php
//WE NEED THIS TO MAKE SURE THAT DOCUMENTS DON'T BORK ON PERMISSIONS
define('IN_TASK_RUNNER', 1);

/**
 * @package I-Ware
 * @subpackage Task
 * @author Chris Wesseling <chris@basesoft.com>
 * @author Marten Koedam <marten@dalines.net>
 * @copyright 2005 BaseSoft B.V.
 * @version $Id$
 */
class TaskRunner
{
	/**
	 * @access private
	 * @var Array $_tasks
	 */
	var $_tasks = array();
	var $_schedulemaks = array();
	var $_names = array();
	var $_taskLoadtime;
	/**
	 * @access private
	 * @var Log_mail $log
	 */
	var $log;
	/**
	 * @var Database $db IWARE Setting database
	 * @var bool $mail Whether to use mail log
	 */
	function TaskRunner(&$db,$mail=null,$load=true) // mail is deprecated
	{
		global $site_title;
		$this->log = getLog();
		$this->db =& $db;
		if($load)
			$this->loadTasks();
	}
	/**
	 *
	 */
	function loadTasks()
	{
		//$this->log->info("Loading tasks...\n");
		$this->_taskLoadtime = ($date = currentDate());
		$this->_tasks = array();
		$sql = "SELECT TASK_ID,TASK_NAME,TASK, SCHEDULE_MASK
			FROM TBL_TASK
			WHERE STATUS = ?
				AND NEXT_EXECUTE_TIME <= '".str_replace('T',' ',$date->format(DateTime::ISO8601))."'";
		$q = $this->db->query($sql);
		$q->bind_values(array(TASK_PENDING));
		$q->execute();
		while($r = $q->fetch())
		{
			$this->log->info('Loading task - ID: '.$r->TASK_ID.' NAME: '.$r->TASK_NAME);
			if ( ! class_exists($r->TASK_NAME) ) //FIXME find i-w tasks...
				require_once(TASKS.APP_PATH.$r->TASK_NAME.'.php');
			$this->_tasks[$r->TASK_ID] = unserialize($r->TASK);
			/** THIS IS TO STOP TASKS RUNNING ON OTHER DBS! **/
			$this->_tasks[$r->TASK_ID]->db = $this->db;
			$this->_tasks[$r->TASK_ID]->id = $r->TASK_ID;
			$this->_schedulemasks[$r->TASK_ID] = $r->SCHEDULE_MASK;
			$this->_names[$r->TASK_ID] = $r->TASK_NAME;
		}
	}
	/**
	 *
	 */
	function executeTasks()
	{
		foreach($this->_tasks as $id => $task)
		{
			$this->log->info("------------------------------");
			$this->log->info('Locking task - ID: '.$id);
			if(!$this->_lockTask($task))
			{
				$this->log->warning('Lock task failed - ID: '.$id);
				continue;
			}
			$this->log->info('Task execution started - ID:'.$id);
			$task->execute($this->log);
			if ( !$this->_reschedule($task, $this->_names[$id], $this->_schedulemasks[$id]) )
			{
				$this->log->info('Deleting task - ID:'.$id);
				$this->cancel($task);
			}
			$this->log->info("------------------------------\n");
		}
		//$this->log->info("TaskRunner finished!\n");
		//$this->log->close();
	}
	/**
	 * @param Task &$task
	 */
	function _lockTask(&$task)
	{
		$sql = "UPDATE TBL_TASK
				SET STATUS = ?
			WHERE TASK_ID = ?
				AND STATUS != ? -- might still be running
				AND NEXT_EXECUTE_TIME <= ? -- might have been rescheduled since loadtime by another process";
		$q = $this->db->query($sql);
		$q->bind_values(array(TASK_RUNNING, $task->id, TASK_RUNNING, str_replace('T',' ',$this->_taskLoadtime->format(DateTime::ISO8601))));
		$q->execute();
		return $q->affected_rows();
	}
	function _reschedule(&$task, $taskName, $schedule_mask)
	{
		$minnexttime = clone $this->_taskLoadtime;
		$minnexttime->modify('+1 minute');
		return $this->schedule($task, $taskName, $schedule_mask, $minnexttime);
	}
	/**
	 * @param Task $task
	 * @param string $taskName
	 * @param string $schedule_mask
	 * @param Date $start minimum schedule date
	 * @return mixed
	 */
	function schedule(&$task, $taskName, $schedule_mask, $start=null)
	{
		list($minute,$hour,$day_of_month,$month,$day_of_week,$year) = explode(' ',$schedule_mask);

		if(!($nextTime = $this->_nextTime($schedule_mask, $start)))
			return false;

		$sql = "REPLACE INTO TBL_TASK
			 (TASK_ID, TASK, TASK_NAME, SCHEDULE_MASK, STATUS, NEXT_EXECUTE_TIME)
			VALUES (?, ?, ?, ?, ?, ?)";
		$q = $this->db->query($sql);
		unset($task->db);
		$q->bind_values(array($task->id, serialize($task),$taskName,$schedule_mask, TASK_PENDING, $nextTime));
		$q->execute();
		$id = $q->lastid();
		$this->log->info('Scheduling task - ID: '.$id.' Next execution time: '.$nextTime);
		return $id;
	}

	function cancel($task)
	{
		$sql = "DELETE FROM TBL_TASK WHERE TASK_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($task->id);
		return $q->execute();
	}

	/**#@+
	 * @access protected
	 */
	/**
	 * @param string $schedule_mask
	 */
	function _nextTime($schedule_mask, $start=null)
	{
		require_once(IWARE.'ScheduleCalculator.php');
		$sc = new ScheduleCalculator('YmdHis');
		return $sc->getNextTime($schedule_mask, $start);
	}
	/**#@-*/

	public static function translateStatus(&$langIf,$status)
	{
		switch($status)
		{
			case TASK_PENDING:
				return array($langIf->getText('Task Pending'),'status_new');
			break;
			case TASK_RUNNING:
				return array($langIf->getText('Task Running'),'status_verified');
			break;
			case TASK_ERROR:
				return array($langIf->getText('Task Error'),'status_forfeit');
			break;
		}
	}

	function getIdByName($name)
	{
		$sql = "SELECT TASK_ID FROM TBL_TASK WHERE TASK_NAME = ?";
		$q = $this->db->query($sql);
		$q->bind_values($name);
		$q->execute();
		$r = $q->fetch();
		return $r->TASK_ID;
	}

	function setNextExecuteTime($id,$nextTime)
	{
		$sql = "UPDATE TBL_TASK SET NEXT_EXECUTE_TIME = ?
			WHERE TASK_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($nextTime,$id);
		$q->execute();
	}

	function getNextExecuteTime($id)
	{
		$sql = "SELECT NEXT_EXECUTE_TIME FROM TBL_TASK
			WHERE TASK_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		$r = $q->fetch();
		return $r->NEXT_EXECUTE_TIME;
	}
}

require_once(IWARE.'AbstractObject.php');

class DefaultTask extends AbstractObject
{
	var $id;
	var $task;
	var $task_name;
	var $schedule_mask;
	var $status;
	var $next_execute_time;

	function set($id=null)
	{
		$id || $id = $this->id;
		$sql = "SELECT TASK, TASK_NAME, SCHEDULE_MASK,
				`STATUS`, NEXT_EXECUTE_TIME
			FROM TBL_TASK
			WHERE TASK_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($id);
		$q->execute();
		if(!($r = $q->fetch()))
			return false;

		$this->id = $id;
		$this->task = $r->TASK;
		$this->task_name = $r->TASK_NAME;
		$this->schedule_mask = $r->SCHEDULE_MASK;
		$this->status = $r->STATUS;
		$this->next_execute_time = $r->NEXT_EXECUTE_TIME;
		return $this->complete = true;
	}

	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_TASK';
		$this->objectmap['primary']['id'] = 'TASK_ID';
		$this->objectmap['properties']['task'] = 'TASK';
		$this->objectmap['properties']['task_name'] = 'TASK_NAME';
		$this->objectmap['properties']['schedule_mask'] = 'SCHEDULE_MASK';
		$this->objectmap['properties']['status'] = 'STATUS';
		$this->objectmap['properties']['next_execute_time'] = 'NEXT_EXECUTE_TIME';
	}
}

/**#@+
 *
 */
define('TASK_PENDING',1);
define('TASK_RUNNING',2);
define('TASK_ERROR',3);
/**#@-*/

/*
*/
?>
