<?php
/**
 * @package
 * @subpackage
 * @since 26-jan-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class TranslationCache
{
	protected $db;
	protected $cachePath = '/tmp';
	protected $language_id;

	public function __construct($db,$language_id)
	{
		$this->db = $db;
		$this->language_id = $language_id;
	}

	public function setTmpCachePath($cachePath='/tmp')
	{
		$this->cachePath = $cachePath;
	}

	public function generate()
	{
		// For every script
		$sql = "SELECT DISTINCT SCRIPT_NAME FROM TBL_SCRIPT_HANDLES";
		$q = $this->db->query($sql);
		$q->execute();
		while($r = $q->fetch())
			$this->generateScriptCache($r->SCRIPT_NAME);
	}

	protected function generateScriptCache($script)
	{
		$q = $this->getTranslationsForScript($script);
		$rs = $q->fetchAll();
		if(!count($rs))
			return false;

		$content = "\$trans = array();\n";
		foreach($rs as $r)
			$content .= sprintf("\$trans['%s'] = '%s';\n",$r->TRANSLATION_HANDLE,addcslashes($r->TRANSLATION,"'"));

		$content = sprintf("<?php\n%s\n?>",$content);
		$fn = sprintf('trans.%s.%s.php',$this->language_id,md5($script));
		$fp = fopen(WRITABLE.$fn,'w+');
		fwrite($fp,$content);
		fclose($fp);
		exit;
	}

	protected function getTranslationsForScript($script)
	{
		$sql = "SELECT TRANSLATION, TRANSLATION_HANDLE FROM TBL_TRANSLATIONS as T
			INNER JOIN TBL_SCRIPT_HANDLES as SH
				ON T.FK_TRANSLATION_HANDLE_ID = SH.FK_TRANSLATION_HANDLE_ID
			INNER JOIN TBL_TRANSLATION_HANDLES as TH
				ON SH.FK_TRANSLATION_HANDLE_ID = TH.TRANSLATION_HANDLE_ID
			WHERE FK_LANGUAGE_ID = ?
				AND SH.SCRIPT_NAME = ?";
		$q = $this->db->query($sql);
		$q->bind_values($this->language_id,$script);
		$q->execute();
		return $q;
	}
}
?>
