<?php
require_once(IWARE.'AbstractObject.php');
/**
 * @author Marten Koedam <marten@dalines.net>
 * @copyright 2007, BaseSoft B.V.
 * @since 13th August 2007
 */
class UserRestriction extends AbstractObject
{
	public $user;
	public $ip;

	function delete($user_id=null,$ip=null)
	{
		$user_id || $user_id = $this->user->id;
		$ip || $ip = $this->ip;

		$sql = "DELETE FROM TBL_USER_RESTRICTION
			WHERE USER_ID = ?
				AND IP_ADDRESS = ?";
		$q = $this->db->query($sql);
		$q->bind_values($user_id,$ip);
		return $q->execute();
	}

	function validateUser($user_id=null,$ip=null)
	{
		$user_id || $user_id = $this->user->id;
		$ip || $ip = $_SERVER['REMOTE_ADDR'];

		$allowed_ips = $this->getAllowedIps($user_id);
		if(!count($allowed_ips)) // If not specified its a free to access account.
			return true;

		foreach($allowed_ips as $allow)
			if (strpos($ip, $allow->IP_ADDRESS) !== false)
				return true;

		info(sprintf('UserRestriction::validateUser():: IP [%s] tried loggin in to User: [%s]',$ip,$user_id));
		return false; // NO ACCESS
	}

	function getAllowedIps($user_id=null)
	{
		$sql = "SELECT IP_ADDRESS
				FROM TBL_USER_RESTRICTION
			WHERE USER_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($user_id);
		$q->execute();
		$r = $q->fetchAll();
		return $r;
	}

	function getOverview()
	{
		$sql = "SELECT u.USER_ID, u.USER_NAME, ip.INFO_PERSON_EMAIL, ur.IP_ADDRESS, '' as Remove
			FROM TBL_USER as u
			INNER JOIN TBL_INFO_PERSON as ip
				ON u.INFO_PERSON_ID = ip.INFO_PERSON_ID
			INNER JOIN TBL_USER_RESTRICTION as ur
				ON u.USER_ID = ur.USER_ID
			ORDER BY u.USER_ID, ur.IP_ADDRESS";
		$q = $this->db->query($sql);
		$q->execute();
		return $q;
	}

	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_USER_RESTRICTION';
		$this->objectmap['primary']['user->id'] = 'USER_ID';
		$this->objectmap['primary']['ip'] = 'IP_ADDRESS';
		$this->objectmap['inserttype'] = 'REPLACE';
	}
}
?>
