<?php
require_once(IWARE.'AbstractObject.php');
/**
 *
 * @author Martijn van den Broek
 * @package I-Ware
 * @version $Id$
 */
class VAT extends AbstractObject
{
	var $vat_no;
	var $status;

	function validatePending($validateUrl=null)
	{
		$sql = "SELECT * FROM TBL_VAT WHERE STATUS=?";
		$q = $this->db->query($sql);
		$q->bind_values(VAT_PENDING);
		$q->execute();

		require_once(IWARE . 'VATChecker.php');
		$vatChecker = new VATChecker($validateUrl);

		$new_status = VAT_PENDING;
		while ($row = $q->fetch())
		{
			$vat = $row->{'VAT_NO'};

			if ($vatChecker->check($vat))
				$new_status = VAT_VALID;
			else
			{
				$undecided = array();
				$undecided[] = 'Unable to connect to the VAT number validation site';
				$undecided[] = 'Internal error';
				$new_status = in_array($vatChecker->getLastError(), $undecided) ? VAT_PENDING : VAT_INVALID;
			}

			$sql = 'UPDATE TBL_VAT SET STATUS=? WHERE VAT_NO=?';
			$q2 = $this->db->query($sql);
			$q2->bind_values($new_status, $vat);
			$q2->execute();

			info(sprintf("VAT::validatePending() - Updated VAT status for VAT '%s' to '%s'", $vat, $new_status));
		}
	}

	function getVATDisplay($vat_no=null)
	{
		$vat_no || $vat_no = $this->vat_no;
		if(!$this->isComplete())
		{
			$this->vat_no = $vat_no;
			$this->set();
		}
		$this->status || $this->status = VAT_VALID;

		switch ($this->status)
		{
			case VAT_INVALID:
				$class='warning';
			break;
			case VAT_PENDING:
				$class='warning';
			break;
			case VAT_VALID:
			default:
				$class='info';
			break;
		}
		return '<span class="'.$class.'">'.$vat_no.'</span>';
	}

	/**#@+
	 * @access private
	 */
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_VAT';
		$this->objectmap['primary']['vat_no'] = 'VAT_NO';
		$this->objectmap['properties']['status'] = 'STATUS';

		$this->typemap['vat_no']= 'string';
		$this->typemap['status']= 'int';
	}
}

define('VAT_INVALID', 0);
define('VAT_PENDING', 1);
define('VAT_VALID', 2);

?>
