<?php
	/**
	 * @copyright Copyright 2004, BaseSoft b.v.
	 * @author Martijn van den Broek <martijn@basesoft.com>
	 * @version $Id$
	 * @package I-Ware
	 * @subpackage Classes
	 */

	require_once(IWARE . 'WizardPage.php');

	/**
	 * class Wizard
	 * @abstract
	 */
	class Wizard
	{
		var $_pages = array();
		var $_history = null;
		var $_roadmap = array();

		function Wizard($wizard_id, $title)
		{
			if (!is_numeric($wizard_id) || $wizard_id == 0)
				trigger_error('wizard_id is not numeric', E_USER_ERROR);

			global $langIf;
			$this->langIf = $langIf;

			$this->_wizard_id = $wizard_id;
			$this->_title = $langIf->getText($title);
			$this->_history = & $_SESSION['Wizard'][$this->_wizard_id]['History'];

			$this->registerPage('Init');
		}

		function _defaultJs()
		{
			if ($this->getWizVar('dont_send_js') )
				return;

			$this->setWizVar('dont_send_js', true);

			?>
				if (!window.checkEmpty)
				{
					window.checkEmpty = function(obj_name, msgErr)
					{
						var obj = document.getElementsByName(obj_name)[0];

						if (!obj)
						{
							alert('Cant find object: ' + obj_name);
							return false;
						}

						if (obj.value.trim() == '')
						{
							MessageBox.prototype.Critical("<?=jsescape($this->tr('Missing input')); ?>",
								msgErr,
								function() { obj.focus(); } );

							return false;
						}

						return true;
					}
				}
			<?php
		}


		/**
		 * Callback from JSRPC
		 * @return WizardPage
		 */
		function _getPage($params)
		{
			if (!isset($params['BUTTON']) || !is_numeric($params['BUTTON']) )
			{
				//-- reset wiz vars
				$this->_clearWizVars();

				//-- no button: return init page
				$this->_history = array();
				$this->_history[] = 'Init';

				//-- register all post data as wizvars
				if (isset($params['POST']) && is_array($params['POST']))
				{
					foreach ($params['POST'] as $key=>$value)
						$this->setWizVar($key, $value);
				}

				//-- set wiz title and preprocess this page
				$page = & $this->getPage('Init');

				$this->preProcess($page);
				$page->wizard_title = $this->_title;
				$page->roadmap = $this->_roadmap;
				return $page;
			}

			if (!isset($params['PageHandle']) || empty($params['PageHandle']) )
				trigger_error('No PageHandle set', E_USER_ERROR);

			$pageHandle = $params['PageHandle'];

			if ( !array_key_exists($pageHandle, $this->_pages))
				trigger_error('Unknown PageHandle', E_USER_ERROR);

			//-- default page, the switch below should set this to one in the _pages array
			$page = new WizardPage($this->_wizard_id , 'Default page, user should not see this');
			$page->disableAllButtons();

			switch ($params['BUTTON'])
			{
				case BT_PREV:
					$lastPageHandle = array_pop( $this->_history );

					//-- post process the previous page
					$this->postProcess( $this->getPage($pageHandle), $params);

					//-- pop from history
					$page = $this->_pages[$lastPageHandle];
				break;

				case BT_NEXT:
				case BT_FINISH:
				case BT_CLOSE:
					//-- push to history
					array_push($this->_history, $pageHandle);

					//-- post process the previous page
					$this->postProcess( $this->getPage($pageHandle), $params);

					//-- go to the next page by checking if the page has a next_page set.
					//   else try to get the next pageHandle in the collection
					//   else go back to Init.
					if (isset($this->_pages[$pageHandle]->next_page))
						$nextPageHandle = $this->_pages[$pageHandle]->next_page;
					else
						$nextPageHandle = $this->_getNextPage($pageHandle);

					if ($nextPageHandle)
						$page = &$this->getPage($nextPageHandle);

				break;

				case BT_CANCEL:
					//-- handle custom cancel actions
					$this->cancelProcess();

					//-- return empty page
					$page = new WizardPage($this->_wizard_id);
					$page->content = '';
					$page->disableAllButtons();

					return $page;
				break;
			}

			//-- enable back button if we got history (history always has 1 page)
			$page->buttons[BT_PREV][BA_DISABLED] = (sizeof($this->_history) > 1) ? 0 : 1;

			//-- pre process page
			$this->preProcess($page);

			//-- give the new page to javascript o//
			return $page;
		}

		/**
		 * @abstract
		 */
		function getTitle()
		{
			return 'Default Wizard Title';
		}

		/**
		 * @abstract
		 */
		function cancelProcess()
		{
			?>
				alert('Cancel action is not implemented');
			<?php
		}

		/**
		 * @static
		 * @return Wizard
		*/
		function getObjectById($wizard_id)
		{
			global $db_iware;
			$sql = "SELECT * FROM TBL_WIZARD WHERE WIZARD_ID=?";
			$q = $db_iware->query($sql);
			$q->bind_values($wizard_id);
			$q->execute();

			if (!($row = $q->fetchAssoc()))
				trigger_error('No such Wizard ('.$wizard_id.')', E_USER_ERROR);

			$class_name = $row['WIZARD_CLASSNAME'];
			$class_url = $row['WIZARD_CLASSURL'];

			//debug('Class: ' . $class_url);

			$file = WIZARDS . $class_url .'/'. $class_name . '.php';
			if ( !file_exists($file))
				trigger_error('Wizard class File does not exist', E_USER_ERROR);

			require_once($file);
			$wiz = new $class_name( $wizard_id );

			return $wiz;
		}

		/**
		 * Adss a page to the page collection
		 */
		function registerPage($handle, $wizPage=null)
		{
			if (is_null($wizPage))
				$wizPage = new WizardPage($this->_wizard_id);

			$wizPage->pageHandle = $handle;
			$this->_pages[$handle] = $wizPage;
		}

		function & getPage($pageHandle)
		{
			if (array_key_exists($pageHandle, $this->_pages))
				return $this->_pages[$pageHandle];
			else
				trigger_error('No such WizardPage ('.$pageHandle.')', E_USER_ERROR);
		}

		function clearPages()
		{
			$this->_pages = array();
			$this->registerPage('Init');
		}


		/**
		 * @abstract
		 */
		function preProcess(&$page)
		{
			debug('Preprocessing:' . $page->pageHandle);
		}

		/**
		 * @abstract
		 */
		function postProcess(&$page, $params)
		{
			debug('Postprocessing: ' . $page->pageHandle . ' Params: ' . print_r($params, true) );
		}

		function getWizVar($varname, $default=null)
		{
			if (isset($_SESSION['Wizard'][$this->_wizard_id]['Vars'][$varname]))
				return $_SESSION['Wizard'][$this->_wizard_id]['Vars'][$varname];
			elseif (!is_null($default))
				return $default;
			else
				trigger_error("Undefined WizVar '$varname'", E_USER_NOTICE);
		}

		function getWizVars()
		{
			return $_SESSION['Wizard'][$this->_wizard_id]['Vars'];
		}

		function setWizVar($varname, $value)
		{
			$_SESSION['Wizard'][$this->_wizard_id]['Vars'][$varname] = $value;
		}

		function _clearWizVars()
		{
			$_SESSION['Wizard'][$this->_wizard_id]['Vars'] = array();
		}

		function issetWizVar($varname)
		{
			return isset( $_SESSION['Wizard'][$this->_wizard_id]['Vars'][$varname]);
		}

		/**
		 * @access protected
		 */
		function setRoadMap($roadmap, $translate=true)
		{
			if (is_array($roadmap))
			{
				if ($translate)
					foreach ($roadmap as $key=>$road)
						$roadmap[$key] = $this->langIf->getText($road);

				$this->_roadmap = $roadmap;
			}
		}

		/**
		 * Returns the next available pageHandle based on the given pageHandle
		 */
		function _getNextPage($pageHandle)
		{
			while ($pageHandle != key($this->_pages))
				next($this->_pages);

			if (!next($this->_pages))
				return false;

			return key($this->_pages);
		}

		/**
		 * Gets a WizardID by Name, false if no Wizard is found
		 * @static
		 * @param string Name
		 * @return int WizardID
		 */
		function getWizardIdByName($name)
		{
			global $db_iware;
			$sql = "SELECT WIZARD_ID FROM TBL_WIZARD WHERE WIZARD_CLASSNAME=?";
			$q = $db_iware->query($sql);
			$q->bind_values($name);
			$q->execute();

			if (!($row = $q->fetchAssoc()))
				return false;


			return $row['WIZARD_ID'];
		}
	}

?>