<?php
	/**
	 * @copyright Copyright 2004, BaseSoft b.v.
	 * @author Martijn van den Broek <martijn@basesoft.com>
	 * @version $Id$
	 * @package I-Ware
	 * @subpackage Classes
	 */

	//-- button types
	define ('BT_PREV',   0);
	define ('BT_NEXT',   1);
	define ('BT_CANCEL', 2);
	define ('BT_FINISH', 3);
	define ('BT_CLOSE',  4);

	//-- button args
	define ('BA_DISABLED', 0);
	define ('BA_DISPLAY',  1);
	define ('BA_ONCLICK',  2);


	/**
	 * class WizardPage
	 */
	class WizardPage
	{
		var $content = '';
		var $buttons = array();
		var $title = '';
		var $pageHandle = '';
		var $wizard_title = '';
		var $wizard_id = 0;

		/**
		 * Constructor
		 * @param int WizardID
		 * @param string Content
		 */
		function WizardPage($wizard_id)
		{
			$this->wizard_id = $wizard_id;

			//-- Default show or hide
			$this->showButtons(BT_CANCEL, BT_PREV, BT_NEXT);
			$this->hideButtons(BT_FINISH);

			//-- Default disable the previous button.. Will be enabled by Wizard possible
			$this->buttons[BT_PREV][BA_DISABLED]  = 1;

			//-- Default click for cancel
			global $langIf;
			if (isset($langIf))
			{
				$str = jsescape($langIf->getText('Are you sure you want to cancel this Wizard?'), "'");
				$this->onButtonClick(BA_ONCLICK,
					"MessageBox.prototype.Question('Wizard', '".$str."',
						function()
						{
							wiz._loadPage(BT_CANCEL);
						}
					); return false;");
			}
		}

		/**
		 * Disables all buttons
		 */
		function disableAllButtons()
		{
			foreach ($this->buttons as $key=>$value)
				$this->buttons[$key][BA_DISABLED] = 1;
		}

		/**
		 * Enables the given buttons.
		 * <code>$page->enableButtons( BT_NEXT, BT_FINISH );</code>
		 */
		function enableButtons()
		{
			foreach (func_get_args() as $button)
				$this->buttons[$button][BA_DISABLED] = 0;
		}

		/**
		 * Enables the given buttons.
		 * <code>$page->disableButtons( BT_NEXT, BT_FINISH );</code>
		 */
		function disableButtons()
		{
			foreach (func_get_args() as $button)
				$this->buttons[$button][BA_DISABLED] = 1;
		}

		/**
		 * Hides the given buttons
		 * <code>$page->hideButtons( BT_NEXT, BT_FINISH );</code>
		 */
		function hideButtons()
		{
			foreach (func_get_args() as $button)
				$this->buttons[$button][BA_DISPLAY] = 0;
		}

		/**
		 * Shows the given buttons
		 * <code>$page->showButtons( BT_NEXT, BT_FINISH );</code>
		 */
		function showButtons()
		{
			foreach (func_get_args() as $button)
				$this->buttons[$button][BA_DISPLAY] = 1;
		}

		/**
		 * Sets a JavaScript action for a button.
		 * You may return false to prevent the page from submitting
		 */
		function onButtonClick($button, $action)
		{
			$this->buttons[$button][BA_ONCLICK] = $action;
		}
	}
?>