<?php
/**
 * @package I-Ware
 */
  class Company
  {
  	/*
	 |----------------------------------------------------------------------------
	 |Author : Mbassa DABO
	 |----------------------------------------------------------------------------
	 |Pupose : Class for managing companies.
	 |
	 |---------------------------------------------------------------------------
	 |Variables :
	 |
	 |		-  (String) $table: Main table for companies (TBL_COMPANY).
	 |		-  (String) $companyname : Name of a company.
	 |		-  (Integer) $id : Id assigned to  a company.
	 |		-  (String) $db
	 | 		-  (Integer) $info_person_id
	 |---------------------------------------------------------------------------
	 |Last Modified : 10/04/2002
	 |
	 |----------------------------------------------------------------------------
	 */
  	//var $name;
	//var $groupdefault_language;
/* 01/05/2002 Revised by Bas Schuiling .
*/

	var $table;
	var $companyname;
	var $company_email = "";
	var $comp_app_visible;
	var $com_app_options;
	var $comp_app_payed;
	var $application_id;
	var $id;
	var $db;

	function Company($db,$id=0,$companyname="")
	{
		//global $companyname;
		$this->clear();
		$this->table = "TBL_COMPANY";
		$this->id = $id;
		$this->companyname = $companyname;
		$this->db = $db;

	}

	function update($company_id)
	{
		$sql = "UPDATE TBL_COMPANY SET COMPANY_NAME = ? , COMPANY_EMAIL = ?
				WHERE COMPANY_ID = ? ";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($this->companyname, $this->company_email,$company_id));
		$ex->execute();
		return true;
	}

	/* 26/08/2004  Check if company name already exists */
	function checkCompanyName($company_name)
	{
		$sql = "SELECT * FROM TBL_COMPANY WHERE COMPANY_NAME = ?";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($company_name));
		$ex->execute();
		if($r = $ex->fetch())
			return true;
		else
			return false;
	}

	function Add()
	{
		/*
		 |-------------------------------
		 | Purpose : Add new company in the table TBL_COMPANY.
		 | Output :
		 |			- 1 : If the query is OK
		 |			- FALSE :  If an error occurs.
		 | Last modified : 28/04/2004
		 */
		$sql = "INSERT INTO TBL_COMPANY (COMPANY_NAME) VALUES (?)";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($this->companyname));

		if($ex->execute())
		{
			$company_id = $ex->lastId();
			$sql = "INSERT INTO TBL_COMP_APP VALUES(?,?,?,?) ";
			$ex = $this->db->query($sql);
			$ex->bind_values(array($company_id,$this->application_id,$this->comp_app_visible, $this->com_app_options));
			if ($ex->execute())
				return $company_id;
			else
				return false;
		}
		else
			return  FALSE;
	}


	function  GetAll($id=0)
	{
		/*
		 |-------------------------------------------------------------------------------------------
		 | Purpose : Extract and retun all company found in the table TBL_COMAPNY. If an id is
		 |			 given as parameter, the only the recordset of the company wich has the id $id
		 |			 is selected.
		 |------------------------------------------------------------------------------------------
		 | Inputs : (Integer) $id : Id of a company. NOT REQUIRED !
		 |------------------------------------------------------------------------------------------
		 | Output :
		 |			- (Objcet) $q_st : If the query a recorset is found
		 |			- FALSE :  If an error occurs.
		 |------------------------------------------------------------------------------------------
		 | Last modified : 10-04-2002
		 |------------------------------------------------------------------------------------------
		 */

		$sql = "SELECT *  FROM TBL_COMPANY";
		if($id)
		{
			$sql .= " WHERE COMPANY_ID=$id";
		}

		$q_st = $this->db->query($sql);
		if(!$q_st)
		{
				return FALSE;
		}
		if($ex = $q_st->execute())
		{
			return $q_st;
		}
		else
		{
			return  FALSE;
		}
	}

	function Delete($id)
	{
		/*
		 |-----------------------------------------------------------------------------------------------
		 | Purpose : Delete a sepecific company identified by the id $id from the table TBL_COMPANY
		 |----------------------------------------------------------------------------------------------
		 | Inputs : (Integer) $id : Id of a company. REQUIRED !!.
		 |------------------------------------------------------------------------------------------
		 | Output :
		 |			- (Integer) 1 : If the application is remoned successfully
		 |			- FALSE :  If an error occurs, or an empty id is given as parameter
		 |------------------------------------------------------------------------------------------
		 | Last modified : 10-04-2002
		 |------------------------------------------------------------------------------------------
		 */

		if(!$id)
		{
			return FALSE;
		}
		$sql = "DELETE FROM  ".$this->table. " WHERE COMPANY_ID=$id";
		$q_st = $this->db->query($sql);
		if(!$q_st)
		{
				return FALSE;
		}
		if($q_st->execute())
		{
			return 1;
		}
		else
		{
			return  FALSE;
		}
	}

	function GetallcompaniesAndApplication($company_id, $application_id)
	{
		/*
		 |-------------------------------------------------------------------------------------------
		 | Purpose : Extract and retun all recordset from the table TBL_COMP_APP which satisfied the
		 |			 where condition.
		 |------------------------------------------------------------------------------------------
		 | Inputs : (Integer) $id : Id of a company. NOT REQUIRED !
		 |------------------------------------------------------------------------------------------
		 | Output :
		 |			- (Objcet) $q_st : If the query a recorset is found
		 |			- FALSE :  If an error occurs.
		 |------------------------------------------------------------------------------------------
		 | Last modified : 28/04/2004
		 |------------------------------------------------------------------------------------------
		 */
		$sql = "SELECT * FROM TBL_COMP_APP WHERE COMPANY_ID=$company_id AND APPLICATION_ID= ?";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($application_id));
		if ($ex->execute())
			return $ex;
		else
			return  FALSE;
	}

	function EditApplicationSettings($companyid, $application_id,$comp_app_visible,$comp_app_payed)
	{
		/*
		 |-------------------------------------------------------------------------------------------
		 | Purpose : Edit an application setting identified by the companyid and applicationid.
		 |			 The table tbl_comp_app is used. Note that due to some problem with the Blob type
		 |			 management, the field comp_app_options is not taken in consideration within the
		 |			 update query. It Should be fixed !!!!!!
		 |------------------------------------------------------------------------------------------
		 | Inputs :
		 |			- $companyid : (integer) id of a company
		 |			- $application_id : (integer) id of an application
		 |			- $comp_app_visible : (boolean) 1 or 0. Should be crypted
		 |			- $comp_app_payed
		 |------------------------------------------------------------------------------------------
		 */

		//global $comp_app_visible,$com_app_options,$comp_app_payed;

		$sql = "UPDATE TBL_COMP_APP SET COMP_APP_VISIBLE='$comp_app_visible', COMP_APP_PAYED='$comp_app_payed'";
		$sql .= " WHERE COMPANY_ID=$companyid and APPLICATION_ID=$application_id";
		echo "$sql";
		$q_st = $this->db->query($sql);

		if(!$q_st)
		{
				return FALSE;
		}
		if($q_st->execute())
		{
			return 1;
		}
		else
		{
			return  FALSE;
		}
	}

	Function Clear()
	{
		$this->company_name = "";
		$this->company_email = "";
		$this->comp_app_visible = "";
		$this->com_app_options = "";
		$this->application_id = 0;
		$this->id = 0;
	}

	Function Set($company_id, $application_id =0)
	{
	/* Return -1 when recordset is empty */
		$sql = "SELECT * FROM TBL_COMPANY WHERE COMPANY_ID = $company_id";
	//	echo $sql;
		$ex = $this->db->query($sql);
		if ($ex->execute())
		{
			if ($tbl = $ex->fetch())
			{
				$this->companyname = $tbl->{'COMPANY_NAME'};
				$this->company_email = $tbl->{"COMPANY_EMAIL"};
				$this->id = $tbl->{'COMPANY_ID'};
			}
			else { return -1;}
		}
		else { return false; }

		if ( $application_id != '' || $application_id != 0)
		{
			$sql = "SELECT * FROM TBL_GROUP_COMPANY WHERE COMPANY_ID = $company_id AND";
			$sql .= " APPLICATION_ID = $application_id";

			$ex = $this->db->query($sql);
			if ($ex->execute())
			{
				if ($tbl = $ex->fetch())
				{
					$this->comp_app_visible = $tbl->{'COMP_APP_VISIBLE'};
					$this->com_app_options = $tbl->{'COMP_APP_OPTIONS'};
					$this->application_id = $tbl->{"APPLICATION_ID"};
				}
				else { return -1; }
			}
			else { return false; }
		}
	}

	function getApplicationId($company_id)
	{
		$sql = "SELECT * FROM TBL_COMP_APP WHERE COMPANY_ID = ?";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($company_id));

		if ($ex->execute())
		{
			$row = $ex->fetch();
			$application_id = $row->{"APPLICATION_ID"};
			return $application_id;
		}
		else
			return -1;


	}

	function getCompanyNameByPublication($publish_id, $mode = "Q", $db_edit = "" )
	{
		if( $db_edit == "" )
			$db_edit = $this->db;
		$sql = "SELECT COMPANY_ID FROM TBL_PUBLISH WHERE PUBLISH_ID = ?";
		$ex = $db_edit->query($sql);
		$ex->bind_values(array($publish_id));
		if ( !$ex->execute())
			return "";

		$row = $ex->fetch();
		$sql = "SELECT COMPANY_NAME FROM TBL_COMPANY WHERE COMPANY_ID = ?";
		$ex = $db_edit->query($sql);
		$ex->bind_values(array($row->{"COMPANY_ID"}));
		if ($ex->execute() && ($row = $ex->fetch()))
		{
			return $row->{"COMPANY_NAME"};
		}
		else
			return "";
	}

	function getUserCount($company_id)
	{
		$sql= "SELECT COUNT(USER_ID) AS CNT_USER FROM TBL_USER INNER JOIN TBL_INFO_PERSON ON
				TBL_USER.INFO_PERSON_ID = TBL_INFO_PERSON.INFO_PERSON_ID WHERE
				COMPANY_ID = ?";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($company_id));

		if ($ex->execute())
		{
			$row = $ex->fetch();
			return $row->{"CNT_USER"};
		}
		else
			return 0;

	}

	function getAvailablePredefStones()
	{
		$sql = "SELECT PREDEF_STONE_ID, PREDEF_STONE_NAME ";
		$sql .= "FROM TBL_COMPANY_PREDEF_STONE ";
		$sql .= "WHERE COMPANY_ID = ? ";
		$sql .= "ORDER BY PREDEF_STONE_NAME";
		$ex = $this->db->query( $sql );
		$ex->bind_values(array( $this->id) );

		$available = array();
		if(!$ex->execute())
			return $available;

		while( $row = $ex->fetch())
			$available[] = $row;

		return $available;
	}

  }
?>
