<?php
/**
 * DefaultReporter
 * @author Chris Wesseling <chris@basesoft.nl>
 * @version $Id: default_reporter.php 392 2005-06-03 11:00:31Z Martijn van den Broek $
 * @package I-Ware
 * @todo Test / Fix in Isurvey
 */

class DefaultReporter
{
	/**
	 * @var int PUBLISH_ID
	 */
	var $publish_id = 0;
	/**
	 * @var array of QUESTION_ID
	 */
	var $question_ids = array();
	/**
	 * @var int ELEMENT_ID
	 */
	var $element_id = 0;
	/**
	 * @var int LANGUAGE_ID
	 */
	var $language_id;

	/**
	 * @var int LANGUAGE_ID
	 */
	var $interclean_report = false;

	var $tbl_response_cache = false;

	/**
	 * Constructor
	 * @param int $publish_id PUBLISH_ID OR ARRAY (!)
	 * @param bool $stat Prepend a StatStone?
	 */
	function DefaultReporter($publish_id, $stat=false, $tpl_set = array(), $show_open_button = true  )
	{
		if (empty($tpl_set))
			$tpl_set = array('type'=>'isurvey', 'class_name'=>'IqTemplateStone', 'include'=> ISURVEY . 'iq_template_stone.php');

		$this->publish_id = $publish_id;
		$this->stat = $stat;
		$this->tpl_set = $tpl_set;

		if ($tpl_set['type'] == 'isurvey')
		{
			do
			{
				if ( !$this->_setSource() )
					break;
				if ( !$this->_prepCache() )
					break;
			}
			while ( $ok = false );

		}
		elseif ($tpl_set['type'] == 'interclean')
		{
			global $db;
			$this->src_db = $db;
		}
		elseif ($tpl_set['type'] == 'formoverview')
		{
			global $db;
			$this->src_db = $db;
		}

		$this->_setLanguage();

		require_once( $tpl_set['include'] );

		$this->template = new $tpl_set['class_name'] ($this->src_db, $this->language_id, $_SESSION['session_user_company_id']);
		$this->template->show_open_button = $show_open_button;

		if ( ! isset($ok) )
			return false;
	}


	/**
	 * @param bool $echo Echo and flush immediately and return ''?
	 * @todo Language_id for the stones
	 * @return string
	 */
	function displayReport($echo=false)
	{
		global $langIf;
		$rv = '<script type="text/javascript" src="http://'.SITE_SERVER.'/'.(SITE_PATH?SITE_PATH:'').'includes/i-w/language_interface.js"></script>';
		$rv .= '<script type="text/javascript">';// src="http://'.SITE_SERVER.'/'.(SITE_PATH?SITE_PATH:'').'includes/i-w/"></script>';
		$rv .= "langIf.addText('show', '".$langIf->getText('show')."');
		langIf.addText('hide', '".$langIf->getText('hide')."');
		function toggleDisplay(el)
		{
			if ( el.style.display == '' )
			{
				el.style.display = 'none';
				el.parentNode.focus();
				return false;
			}
			else
			{
				el.style.display = '';
				return true;
			}
		}
		function toggleInnerHTML(el, val, trueContent, falseContent, thruLangIf)
		{
			if ( val )
				el.innerHTML = (thruLangIf?langIf.getText(trueContent):trueContent);
			else
				el.innerHTML = (thruLangIf?langIf.getText(falseContent):falseContent);
		}</script>";

		if ($this->tpl_set['type'] == 'isurvey')
			$this->template->tbl_response = $this->src_tbl;
		if($this->tbl_response_cache)
			$this->template->tbl_response_cache = $this->tbl_response_cache;

		if ( empty($this->question_ids) )
			$this->question_ids = $this->template->getQuestionIds($this->publish_id);


		if ( $this->stat )
		{
			require_once(STONES.'stat_stone.php');

			$stat = new StatStone($this->src_db, $this->language_id, $_SESSION['session_user_company_id']);
			$stat->getData($this->publish_id);
			$rv .= $stat->display();
			if ( $echo )
			{
				echo $rv;
				$rv = '';
			}
		}

		foreach ( $this->question_ids as $qid )
		{

			$this->template->getData($qid, null); //null... we don't know the answertype (iq only)

			$rv .= $this->template->display();
			if ( $echo )
			{
				echo $rv;
				flush();
				$rv = '';
			}
			$this->template->clear();
		}
		return $rv;
	}


	/**#@+
	 * @access private
	 */
	/**
	 * @var ResponseCache $cache
	 */
	var $cache;
	/**
	 * @var Database $cache
	 */
	var $src_db;
	/**
	 * @var string $src_tbl 'TBL_RESPONSE' | 'TBL_RESPONSE_TEMP'
	 */
	var $src_tbl;

	/**
	 * Set the right source for data.
	 *
	 * Sets $this->src_db and $this->src_tbl to the rigth values;
	 * @todo move this all to ResponseCache
	 * @return bool
	 */
	function _setSource()
	{
		global $db_edit;
		$sql = "SELECT PUBLISH_STATUS, PUBLISH_FINAL_SUBMIT
			FROM TBL_PUBLISH
		   	INNER JOIN TBL_PUBLISH_SURVEY
				USING(PUBLISH_ID)
			WHERE TBL_PUBLISH.PUBLISH_ID = ?";
		$q = $db_edit->query($sql);
		$q->bind_values(array($this->publish_id));
		if ( $q->execute() )
		{
			$r = $q->fetch();
			switch ( $r->{'PUBLISH_STATUS'} )
			{
				case STATUS_ONLINE:
					global $db_publish;
					$this->src_db = $db_publish;
					$this->src_tbl = ($r->{'PUBLISH_FINAL_SUBMIT'}?'TBL_RESPONSE_TEMP':'TBL_RESPONSE');
				return true;
				case STATUS_FINISHED:
					global $db_history;
					$this->src_db = $db_history;
					$this->src_tbl = 'TBL_RESPONSE';
				return true;
			}
		}
		return false;
	}

	/**
	 * Sets the default language for the current report.
	 */
	function _setLanguage()
	{
		$sql = "SELECT QUESTIONNAIRE_DEF_LANGUAGE
			FROM TBL_QUESTIONNAIRE
			WHERE QUESTIONNAIRE_ID = ?";
		$ex = $this->src_db->query( $sql );
		$ex->bind_values( array( $this->element_id ));
		$ex->execute();
		if( $row = $ex->fetch())
			return $this->language_id = $row->{"QUESTIONNAIRE_DEF_LANGUAGE"};
		return $this->language_id = $_SESSION["session_user_language_id"];
	}

	/**
	 * Prepare the ResponseCache
	 * @todo move this all to ResponseCache
	 */
	function _prepCache()
	{
		global $db_edit;
		require_once(ISURVEY.'response_cache.php');

		$className = 'ResponseCache';
		if(@include_once(CUSTOM.$_SESSION['session_user_company_name'].'ResponseCache.php'))
			$className = $_SESSION['session_user_company_name'].$className;

		require_once(ISURVEY_PUBLISH.'publish.php');
		$publish = new Publish($db_edit, $_SESSION['session_user_company_id']);
		$publish->set($this->publish_id);
		$this->element_id = $publish->element_id;
		$this->cache = new $className($this->src_db);
		$this->cache->generateCache($publish, $this->src_tbl);
		if($this->cache->pre)
			$this->tbl_response_cache = $this->cache->pre.'TBL_RESPONSE_CACHE';
		return true;
	}
	/**#@-*/
}
?>
