<?php
/**
 * Language_Interface
 * @author Martijn van den Broek <mjbroek@xs4all.nl>
 * @version $Id: language_interface.php 41 2005-04-18 07:43:11Z Martijn van den Broek $
 * @package I-Ware
*/

/**
 * dependencies
 */
require_once(IWARE . 'form.php');
require_once(IWARE . 'sql.php');

require_once(IWARE.'IWException.php');

define('ERR_STR_FIX_ALL', 'Exiting! all errors should be fixed');

/**
 * class EventHandler
 * @abstract class to handle events
 */
class EventHandler
{
	private $observers = array();

	/**
	 * variable to control to spit out html or not
	 */
	public $htmlOut = true;

	/**
	 * @param database
	 */
	function EventHandler($db) { $this->db = $db; }

	/**
	 * Inserts a new ticket into the database
	 * @param string title
	 * @param int level
	 * @param string data
	 */
	function make_ticket($title, $level=0, $data='')
	{
		// create  a random int as ticket-nr
		$ticket_nr = uniqid();

		// body
		$sessionId = session_id();
		$ip_addr = $_SERVER['REMOTE_ADDR'];
		$hostname = gethostbyaddr($_SERVER['REMOTE_ADDR']);

		$session_data = print_r(isset($_SESSION) ? $_SESSION : array(), true);
		$post_data = print_r(isset($_POST) ? $_POST : array(), true);
		$get_data = print_r(isset($_GET) ? $_GET : array(), true);
		$server_data = print_r(isset($_SERVER) ? $_SERVER : array(), true);


// create body.. no indenting so it will display nice in the email (dont want html)
$body = "
title: $title\n
level: $level\n
ip_addr: $ip_addr\n
hostname: $hostname\n
sessionId: $sessionId\n
\n
error data: $data\n\n
get_data: $get_data\n\n
post_data: $post_data\n\n
session_data: $session_data\n\n
server_data: $server_data\n\n
";

		/**
		 * Mail object, do not get from registry, this can cause issues when an error happens while mailing.
		 */
		require_once(IWARE . 'phpmailer/class.phpmailer.php');
		require_once(IWARE . 'phpmailer/class.smtp.php');

		$mail = new PHPMailer();
		$mail->IsSMTP();
		$mail->SetLanguage("en", IWARE.'phpmailer/');

		$mail->Host = SMTP_SERVERS; // specify main and backup server (maybe add backup server)
		$mail->SMTPKeepAlive = true;

		// fill mail
		$host = isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : 'unknown';
		$mail->FromName = $mail->From = 'eventhandler@basesoft.com';
		$mail->AddAddress('servers@mail.clusternet');
		$mail->Subject = sprintf('[%s] incident %s', $host, $ticket_nr);
		$mail->IsHTML(false);
		$mail->WordWrap = 200;
		$mail->Body = $body;
		$mail->CharSet = 'utf-8';

		// send
		if (!$mail->Send())
			alert(sprintf('EventHandler: error: while sending ticket mail %s', $ticket_nr));

		return $ticket_nr;
	}

	/**
	 * makes a nice formatted string from an array or object
	 * @todo maybe make a recursion proof var_dump?
	 * @access protected
	 * @return string
	 */
	function _var_dump($arr)
	{
		return htmlspecialchars (print_r($arr, true));
	}

	/**
	 * same as _var_dump but replaces mewlines and html entities to 1 lined string
	 * @access protected
	 */
	function _html_var_dump($arr)
	{
		return str_replace(array("\n", "\r", " "), array('<br/>', '', '&nbsp;'), $this->_var_dump($arr));
	}

	function getEventOverview()
	{
		$sql = "SELECT EVENT_ID,
		        DATE_FORMAT(EVENT_DATETIME, '%d-%m-%Y %h:%i') AS DATE,
		        EVENT_TITLE,
		        CONCAT('<a href=\"event_details.php?EVENT_ID=', EVENT_ID, '\"><img src=\"../../../images/preview_small.gif\" style=\"border: 0;\"></a>') as EVENT_DETAILS

		        FROM TBL_EVENTS
		        ORDER BY EVENT_DATETIME DESC";

		$q = $this->db->query($sql);
		$q->execute();

		return $q;
	}

	function getEventDetails($event_id)
	{
		$sql = "SELECT *
		        FROM TBL_EVENTS
		        WHERE EVENT_ID = ?
		       ";
		$q = $this->db->query($sql);
		$q->bind_values($event_id);
		$q->execute();

		return $q->fetchAssoc();
	}

	function registerObserver($eventType, $callback)
	{
		$this->observers[$eventType][] = $callback;
	}

	function unregisterObserver($eventType, $callback)
	{
		// TODO
	}

	function notifyObservers($eventType)
	{
		if (isset($this->observers[$eventType]))
		{
			foreach ($this->observers[$eventType] as $observer)
			{
				if (is_array($observer))
					$observer[0]->$observer[1]($eventType);
				else
					$observer($eventType);
			}
		}
	}

	/**
	 * makes an html block for the given title and array
	 * @access private
	 * @return string html
	 */
	protected function _arr_details($title, $arr)
	{
		ob_start();
		$uniqueid = md5(uniqid(rand(), true));

		?>
			<b><?=$title?></b>
			<div style=" margin: 5px;  margin-left: 20px; padding: 2px;">
				<a href="#" onclick="document.getElementById('divArr_<?=$uniqueid; ?>').style.display = (this.innerHTML == '+') ? '' : 'none'; this.innerHTML = (this.innerHTML == '+') ? '-' : '+'; return false;" style="font-weight: bold; text-decoration: none; font-family: Monospace; ">+</a>
				size: <?=sizeof($arr);?>

				<div id="divArr_<?=$uniqueid; ?>"  style="display: none;">

					<div style="float: left; padding-left: 28px;">var dump:</div>
					<div style="float: left;">
						<pre style="margin: 0; padding: 0; padding-left: 10px; font-family: Monospace;"><?=$this->_html_var_dump( $arr ); ?></pre>
					</div>
					<div style="clear: both;"></div>
				</div>
			</div>
		<?php

		$retval = ob_get_contents();
		ob_end_clean();
		return $retval;
	}

	/**
	 * @var int Last Ticket nr
	 */
	protected $last_nr = false;

	/**
	 * Get last ticketnr
	 * @return int or False.. if no errors thrown.
	 */
	protected function getLastTicket()
	{
		return $this->last_nr;
	}

	protected function friendlyMessage($errstr, $errno, $ticket_no)
	{
		?>
			<!-- start friendly error message -->
			<div align="center">
				<div style="font-family: Verdana; font-size: 8pt; border: 1px solid #CCCCCC; width: 600px; margin: 20px; padding: 0 20px 20px 20px; background-color: #FFFFEE; text-align: left;">
					<h2><?=$errstr?></h2>
					<em><?=$this->getErrorValue($errno);?> occured.</em><br />
					Information about this exception has been sent to the application developer for further investigation.
					<strong>Incident #<?=$ticket_no?></strong>
				</div>
			</div>
			<!-- end friendly error message -->
		<?php
	}

	/**
	 * Makes a backtrace in nice formatted html
	 * @access private
	 * @return string
	 */
	protected function _html_backtrace($calls)
	{
		//return;

		ob_start();
		?>
				<b>Backtrace</b>
				<?php
					for ($i=sizeof($calls)-1; $i >= 0; $i--)
					{

						?>
							<div style=" margin: 5px;  margin-left: 20px; padding: 2px;">
								<div>
									<a href="#" onclick="document.getElementById('divBacktrace_<?=$this->uniqueid?>_<?=$i?>').style.display = (this.innerHTML == '+') ? '' : 'none'; this.innerHTML = (this.innerHTML == '+') ? '-' : '+'; return false;" style="font-weight: bold; text-decoration: none; font-family: Monospace; ">+</a>
									<?php
										if (isset($calls[$i]['function']))
											echo 'calling <span style="font-weight: bold;">' . $calls[$i]['function'] . '</span>() ';
									?>
									<?php
										if (isset($calls[$i]['line']))
											echo 'on line <span style="font-weight: bold;">' . $calls[$i]['line'] . '</span>';
									?>
									<?php
										if (isset($calls[$i]['file']))
											echo 'at <span style="font-weight: bold;">' . $calls[$i]['file'] . '</span>';
									?>
								</div>

								<?php
									if (isset($calls[$i]['args']))
									{
										?>
											<div id="divBacktrace_<?=$this->uniqueid; ?>_<?=$i?>" style="margin: 5px; display: none;">
												<div style="float: left; padding-left: 28px;">Arguments:<br></div>
												<div style="float: left;">
													<pre style="margin: 0; padding: 0; padding-left: 10px; font-family: Monospace;"><?=$this->_html_var_dump($calls[$i]['args']); ?></pre>
												</div>
												<div style="clear: both;"></div>
											</div>

										<?php
									}
								?>
							</div>
						<?php
					}
				?>
		<?php

		return ob_get_clean();;
	}

	/**
	 * sets the error reporter to friendly
	 * @param bool
	 */
	function setFriendly($bool=true)
	{
		$this->friendly = $bool;
	}

	function _html_handler($ticket_no, $errno, $errstr, $errfile, $errline, $errtrace)
	{
		if(isset($_REQUEST['script_parent_id'])&& $_REQUEST['script_parent_id']==350)
			return;
		$this->uniqueid = md5(uniqid(rand(), true));

		ob_start();

		if ($this->friendly)
			$this->friendlyMessage($errstr, $errno, $ticket_no);
		else
		{
			?>
				<!-- ERROR: <?=$this->getErrorValue($errno)?>: <?=$errstr?> in <?=$errfile?> on line <?=$errline?> -->
				<div style="Font-family: Verdana; font-size: 8pt; text-align: left;" id='id_exception'>
					<a href="#" onclick="document.getElementById('divDetails_<?=$this->uniqueid?>').style.display = (this.innerHTML == '+') ? '' : 'none'; this.innerHTML = (this.innerHTML == '+') ? '-' : '+'; return false;" style="font-weight: bold; text-decoration: none; font-family: Monospace; ">+</a>
					<b><?=$this->getErrorValue($errno); ?>:</b> <?=$errstr?>  in <b><?=$errfile?></b>  on line <b><?=$errline?></b>

					<div id="divDetails_<?=$this->uniqueid; ?>" style="display: none; text-align: left;">
						<ul style="padding: 0; margin: 5px; margin-left: 30px;">
							<li><?=$this->_html_backtrace($errtrace); ?></li>
							<li style="margin: 0; padding: 2px; padding-left: 20px;"><?=$this->_arr_details( '$_POST', $_POST ); ?></li>
							<li><?=$this->_arr_details( '$_GET', $_GET ); ?></li>
							<? if (isset($_SESSION)): ?>
								<li><?=$this->_arr_details( '$_SESSION', $_SESSION ); ?></li>
							<? endif; ?>
						</ul>
					</div>

				</div>
			<?php
		}

		echo "\n<!-- ERROR END -->\n";
		$retval = ob_get_contents();
		ob_end_clean();

		echo preg_replace('/\n?\r?/', '', $retval); //-- we dont wanna see newlines
	}

	function getErrorValue($errno)
	{
		if (isset($this->errValues[$errno]))
			return $this->errValues[$errno];
		else
			return 'Unknown errorcode '.$errno.' ';
	}

}

/**
 * class Errorhandler
 * This class handles error events. It receives the messages and makes a nice output with debug features.
 * On production servers the class can be set to friendly, where the user doesnt see all the frightning errors.
 *
 * Usage:
 * <code>
 * 	$errorHandler = new ErrorHandler($db, false); //-- set true for friendly messages
 * </code>
 *
 * To test simply trigger an error:
 * <code>
 * 	echo $uninitialised_var;
 * </code>
 */

# in case this constant is not defined
if (!defined('E_RECOVERABLE_ERROR'))
	define('E_RECOVERABLE_ERROR', 0);

class ErrorHandler extends EventHandler
{
	protected $errValues = array ( E_STRICT		   => 'Strict',
							 E_ERROR           => 'Fatal Error',
		                     E_WARNING         => 'Warning',
		                     E_PARSE           => 'Parse Error',
		                     E_NOTICE          => 'Notice',
		                     E_CORE_ERROR      => 'Core Error',
		                     E_CORE_WARNING    => 'Core Warning',
		                     E_COMPILE_ERROR   => 'Compile Error',
		                     E_COMPILE_WARNING => 'Compile Warning',
		                     E_USER_ERROR      => 'Fatal Error',
		                     E_USER_WARNING    => 'Warning',
		                     E_USER_NOTICE     => 'Notice',
		                     E_STRICT          => 'Strict',
		                     E_RECOVERABLE_ERROR => 'Recoverable Error');

	/**
	 * Constructor
	 * @param database
	 * @param bool if true, friendly messages will be given to the user. If false debug is enabled.
	 */
	function ErrorHandler($db, $friendly=true)
	{
		$this->friendly = $friendly;
		$this->setReporting(E_ALL ^ E_WARNING ^ E_USER_WARNING ^ E_NOTICE ^ E_USER_NOTICE); // default reporting
		parent::EventHandler($db);

		// set error handler to this class
		set_error_handler(array($this, '_error'));
	}

	/**
	 * Sets the level where errors are showed
	 * @param int The level where errors are shown to the developer or user
	 * @param int The level where errors are logged (free advice: keep this at E_ALL)
	 */
	function setReporting($show_level = E_ALL, $log_level = E_ALL)
	{
		$this->_show_level = $show_level;
		$this->_log_level = $log_level;

		error_reporting($log_level);
	}

	/**
	 * Receives all incomming errors.
	 */
	function _error($errno, $errstr, $errfile, $errline)
	{

		if ($errstr == ERR_STR_FIX_ALL) {
			info(ERR_STR_FIX_ALL);
			return;
		}

		if ( !error_reporting() ) //@? I've been expecting you old man. Now the circle is complete. When I last left you, I was but the learner. Now I am the master.
			return; //Only a master of evil Darth.

		/**
		 * Dont log any strict errors.. we could enable this someday and fix them.
		 */
		if ($errno == E_STRICT)
			return;

		if ($this->friendly) //-- only save errors if we are friendly
		{
			$this->last_nr = $ticket_nr = $this->make_ticket(sprintf('Error: (%s)', $errstr), $errno,
				sprintf("%s:\n %s \n in %s on line %s\nBacktrace: %s", $this->getErrorValue($errno),
					$errstr, $errfile, $errline, print_r(array_shift(debug_backtrace()), true)));
		}
		else {
			$this->last_nr = $ticket_nr = 0;
		}

		if( ( $this->htmlOut && $errno & $this->_show_level ) == $errno )
		{
			$backtrace = defined('ERROR_BACKTRACE') && ERROR_BACKTRACE ? debug_backtrace() : array();
			$errstr = ($this->friendly)?'Unexpected Error':$errstr;
			$this->_html_handler($ticket_nr, $errno, $errstr, $errfile, $errline, $backtrace );
		}

		//$log = getLog();
		//$log->alert('PHP Error: ('.$errstr.') - ' . $this->getErrorValue($errno) . " - " . $errstr . " in " . $errfile . ' on line ' . $errline . ' - ticket ' . $ticket_nr);
		alert(sprintf('PHP Error [%s]: %s in %s on line %s. Ticket: %s', $this->getErrorValue($errno), $errstr, $errfile, $errline, $ticket_nr));
		//alert('Backtrace for Error: '.print_r(array_reverse(debug_backtrace()), true), true);

		// when errors are visible are visible (non production), all errors are fatal
		if (defined('ERROR_NO_FRIENDLY') && ERROR_NO_FRIENDLY) {
			echo 'Exception: '.$errstr;
			throw new Exception(ERR_STR_FIX_ALL);
		}

		//-- exit if fatal error
		if ($errno == E_ERROR || $errno == E_PARSE || $errno == E_USER_ERROR)
		{
			parent::notifyObservers('E_FATAL');
			exit();
		}
	}
}


/**
 * ExceptionHandler
 */

class ExceptionHandler extends EventHandler
{
	protected $errValues = array ( 0 => 'Exception');

	function ExceptionHandler($db, $friendly=true)
	{
		$this->friendly = $friendly;
		parent::EventHandler($db);

		// set error handler to this class
		set_exception_handler(array($this, '_exception'));
	}

	function _exception($e)
	{
		if ($this->friendly) //-- only save errors if we are friendly
		{
			$this->last_nr = $ticket_nr = $this->make_ticket(sprintf('Exception: (%s)', $e->getMessage()), 0,
				sprintf("%s:\n %s \n in %s on line %s\nBacktrace: %s", $this->getErrorValue($e->getCode()),
					$e->getMessage(), $e->getFile(), $e->getLine(), htmlspecialchars (print_r(array_reverse($e->getTrace()), true))));
		}
		else
			$this->last_nr = $ticket_nr = 0;

		alert(sprintf('Uncaught Exception: (%s) - ticket %s', $e->getMessage(),  $ticket_nr));
		alert('Backtrace for Exception: '.print_r($e->getTrace(), true));

		if ($this->htmlOut)
		{
			$message = (defined('TEST_SITE')&&TEST_SITE) ? $e->getMessage() : 'An unexpected exception occured'; // only show more info when in test_site
			if(in_array(get_class($e),array('IWUserException', 'IWFileUploadException')))
				$message = $e->getMessage();
			$this->_html_handler($ticket_nr, $e->getCode(), $message, $e->getFile(), $e->getLine(), $e->getTrace());
		}
	}
}


?>
