<?php
/*
*	Central Survey export module
*
*	Used to export stuff from the application to various formats
*
*	@package I-Ware
*	@author Bas schuiling
*	@copyright (2003) Basesoft
* 	@since 21-08-2003
*	@version 2.1
*/

define("EXPORT_PDF",1);
define("EXPORT_XLS",2);
define("EXPORT_SPSS",3);
define("EXPORT_XML",4);
define("EXPORT_TXT",5);


/*
*	Class Export
*
*	Exports the stuff
*/
Class Export
{
var $db;
var $grid;
var $application_path;
var $base_dir;

var $element_id = 0;
var $element_type = 0;
var $language_id = 0;
var $language_name = "";
var $company_name = "";

var $export_options = array();

var $export_type = EXPORT_XLS;

var $raw_output = "";
var $html_output = "";

var $other_info = "";
var $destination_file = "";

/*
* Contructor
*
* Constructor for class Export
*
*	@param Object $db
*	@param Integer $element_id
*	@param Integer $element_type
*/
function Export($db, $element_id, $element_type)
{
	$this->db = $db;
	$this->element_id = $element_id;
	$this->element_type = $element_type;
}

/*
* Builds the export output
*
* Parses all give settings and stats working on a output.
*
*/
function buildExport()
{
	switch($this->export_type)
	{
		case EXPORT_XLS:
		case EXPORT_TXT:
			$this->replaceLoginByParticipant($this->grid);
			$this->buildXLS();
		break;
		case EXPORT_PDF:
			$this->buildHtml();
			$this->buildPDF();
		break;
	}
}

function buildPDF()
{
	if (isset($this->export_options["password"]))
		$password = $this->export_options["password"];
	else
		$password = "";
	$nu = date( "YmdHis" );
	$file_path = $this->application_path;
	$convertedHTML = new parser( $file_path."report".$this->language_id."-0.html", $this->base_dir, $this->element_id, $this->company_name );
	$convertedHTML->db = $this->db;
	$convertedHTML->convertHTML();
	$file_name = $file_path."report".$this->language_id."-0.html";

	if ($password == "")
		$options = "--fontsize 9 --right 19mm --left 19mm --no-title --headfootsize 8 --header '...' --footer 't./' --no-jpeg";
	elseif ($password != "" && !empty($password) )
		$options = "--fontsize 9 --right 19mm --left 19mm --no-title --headfootsize 8 --header '...' --footer 't./' --no-jpeg --encryption --user-password '".addslashes(stripslashes($password))."'";
	$destination_file = $file_path."pdf".$this->language_id."_".$nu.".pdf";
	$this->destination_file = $destination_file;
	$pdf = new topdf( $file_name, $options, $destination_file );
	$pdf->createPDFFile();

	//global $st;

	//exit( "eep: ".(m()-$st) );

	$this->other_info = $nu;
}

function buildXLS()
{
	$grid = $this->grid;
	$output = "";

	if ( isset($this->export_options["delimiter"]))
		$delimiter = $this->export_options["delimiter"];
	 else
		$delimiter = ",";

	$header_row = array();
	$row = $grid->getNextRow();

	$question_row = array();
	$subquestion_row = array();
	$question_row[0] = "";
	$subquestion_row[0] = "";

	foreach($row as $row_item)
	{
		//addEvent(__LINE__,"rowItem",$row_item);
		preg_match("/(Q[0-9]+)/si",$row_item,$qmatches);

		if (sizeOf($qmatches) > 1)
			$question_row[] = $qmatches[1];

		preg_match("/(S[0-9]+)/si",$row_item,$matches);
		if (sizeOf($matches) > 1)
			//$subquestion_row[] = $qmatches[1] . "_" . $matches[1];
			$subquestion_row[] = $matches[1];
		else
			$subquestion_row[] = "";

		$header_row[] = $grid->getElementText($row_item);
	}

	$questionOut    =  array("");
	$subquestionOut  = array("");
	$old_q = "";
	$old_s = "";

	//addEvent(__LINE__,"questioNRwo",$question_row);
	//addEvent(__LINE__,"headahrow",$subquestion_row);
	//global $debug;
	//echo $debug->outputDebugLIst();

	if(sizeOf($header_row) != sizeOf($question_row))
		exit("INVALID SIZE FATAL ERROR");

	//exit();
	foreach($header_row as $answer_row)
	{
		$q = array_shift($question_row);
		if ($q != $old_q)
		{
			$questionOut[] = $grid->getElementText($q);
			$old_q = $q;
		}	else $questionOut[] = " ";

		$s = array_shift($subquestion_row);
		if ($s != $old_s)
		{
			$subquestionOut[] =  $grid->getElementText($s);
			$old_s = $s;
		}	else $subquestionOut[] = " ";
	}

	array_shift($questionOut);
	array_shift($subquestionOut);
	$output .= implode("$delimiter",$questionOut);
	$output .= "\n";
	$output .= implode("$delimiter",$subquestionOut);
	$output .= "\n";
	$output .= implode("$delimiter",$header_row);
	$output .= "\n";
	while($row = $grid->getNextRow())
	{
		$row = str_replace("\n"," ",str_replace("\r"," ",$row));
		$output .= implode("$delimiter",$row);
		$output .= "\n";

	}

//	global $debug;
//	echo $debug->outputDebugList();
	$this->html_output .= $output;
}

function buildHTML()
{
	$directory_path = $this->application_path;

	$dir = new Dir();
	$dir->CreateDirectoryStructure($directory_path);

	if (! isset($page_num)) $page_num = 0;

	$file_path = "report" . $this->language_id . "-" . $page_num . ".html";
	$file = fopen($directory_path . $file_path,"w");
	fwrite($file,$this->html_output);
	fclose($file);
}

/*
* Builds HTML header or footers
*
*	Builds for PDF ( and other reports ) the HTML headers and footers needed
*
*	@param String $type
*/
function buildHTMLheaders($type = "HEADER")
{
	if ($type == "HEADER")
	{
			$html = "<center>
					<table align=center cellspacing=\"1\" cellpadding=\"0\" width=\"95%\" border=\"0\" style=\"background: #000000;\">
					<tr>
					<td style=\"background: #FFFFFF;\"><br>
						<center>
							<table width=\"95%\" align=center cellpadding=\"0\" cellspacing=\"0\" border=\"0\">
								<tr><td>";
	}
	elseif ($type == "FOOTER")
	{
			$html =  "				</td></tr>
								</table>
							</td></tr>
							</table>
						</center>
						</body>
					</html>";
	}
		$html = "";
		$this->html_output .= $html;
}

function BuildXmlStyle()
{
	$languageName = $this->language_name;
	$xmlStyleDoc = new Xmlsearch($this->base_dir."include/style.xml");

	$includeDir = "include/";
	$docLoaded = false;
	if (isset($report_id))
	{
		if (file_exists($this->application_path ."stylesheet.xml"))
		{
			$xmlDataDoc = new Xmlsearch($this->application_path."stylesheet.xml");
			$docLoaded = true;
		}
		else
		{
			$xmlDataDoc = new Xmlsearch($this->base_dir.str_replace("\\", "/", $includeDir)."def_stylesheet.xml");
			$docLoaded = true;
		}
	}
	else
	{
		$docLoaded = false;
	}
	$css_code = "";
	//Collection of editable stylesheet javascript names
	$xmlstring = "style";
	$xmlStyles = $xmlStyleDoc->selectNodes($xmlstring);
	//Collection of style classes
	$xmlstring = "class";
	$xmlStyleClasses = $xmlStyleDoc->selectNodes($xmlstring);
	$xmlArray = array();
	for ($i = 0; $i < sizeof($xmlStyleClasses->xmlnodesArray); $i++)
	{
		$xmlClassClassname = $xmlStyleClasses->selectSingleNode("classname", $xmlStyleClasses->xmlnodesArray[$i]);
		$xmlClassName = $xmlStyleClasses->selectSingleNode($languageName."/name", $xmlStyleClasses->xmlnodesArray[$i]);
		$xmlClassDesc = $xmlStyleClasses->selectSingleNode($languageName."/description", $xmlStyleClasses->xmlnodesArray[$i]);
		$css_code .= ".".$xmlClassClassname."{\n";
		for ($j = 0; $j < sizeof($xmlStyles->xmlnodesArray); $j++)
		{
			$xmlStyleJs = $xmlStyles->selectSingleNode("jsName", $xmlStyles->xmlnodesArray[$j]);
			$xmlStyleCss = $xmlStyles->selectSingleNode("cssName", $xmlStyles->xmlnodesArray[$j]);
			$xmlStyleDef = $xmlStyles->selectSingleNode("default", $xmlStyles->xmlnodesArray[$j]);
			if ($docLoaded == true)
				$xmlStyleDef = $xmlDataDoc->getContent($xmlDataDoc->selectSingleNode($xmlClassClassname."/".$xmlStyleJs));
			if( $xmlStyleCss == "background" )
			{
				$xmlArray["$xmlClassClassname"] = $xmlStyleDef;
			}
			$css_code .= $xmlStyleCss.":".$xmlStyleDef.";\n";
		}
		$css_code .= "}\n";
	}
	return $xmlArray;
}

/*
*	Output / shows export
*
*	Function to output or show the exported results. This varies for the different export types.
* 	XLS / XML / SPSS will open a file dialog while PDF export loads the PDF file into the browser
*/
function show()
{
	//ob_end_clean();
	//ob_start();
	if ($this->export_type == EXPORT_XLS)
	{
		header("Content-Disposition: Attachment; filename=export.xls");
		header("Content-type: application/ms-excel");
		echo $this->html_output;
		exit();
	}
	elseif ($this->export_type == EXPORT_PDF)
	{
		if( file_exists($this->application_path ."pdf".$this->language_id."_".$this->other_info.".pdf") )
		{	echo "<script language=\"JavaScript\">\nwindow.open(\"../../../klant/".$this->company_name."/R".$this->element_id."/pdf".$this->language_id."_".$this->other_info.".pdf\",'blank');\n
				  </script>\n";
		}
		global $script_parent_id;
		echo "<script language=\"javascript\">\nlocation.href='../../i-w/stone/preview_stone.php?element_id=" . $this->element_id . "&element_type=".$this->element_type . "&language_id=". $this->language_id . "&script_parent_id=" . $script_parent_id . ( isset( $this->login_id ) ? "&login_id=".$this->login_id : "" )."';\n
			  </script>\n";
		exit();
	}
	elseif($this->export_type == EXPORT_TXT)
	{
		header("Content-Disposition: Attachment; filename=export.txt");
		header("Content-type: plain/text");
		echo $this->html_output;
		exit();
	}
}

function replaceLoginByParticipant($grid)
{
	$grid->removeSpecialRows();
	$grid->rotateGrid();
	$login_array = $grid->grid[0];

	array_shift($login_array);

	$new_array = array();
	foreach($login_array as $login_key => $login_id)
	{
		$sql = "SELECT RELATION_NAME FROM TBL_RELATION
				INNER JOIN TBL_PARTICIPANT ON TBL_PARTICIPANT.RELATION_ID =
				TBL_RELATION.RELATION_ID INNER JOIN TBL_LOGIN ON TBL_LOGIN.USER_ID =
				TBL_PARTICIPANT.PARTICIPANT_ID WHERE LOGIN_ID = ?";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($login_id));
		if($ex->execute())
		{
			$row = $ex->fetch();
			if ($row->{"RELATION_NAME"} == "")
				$new_array[$login_key] = $login_id;
			else
				$new_array[$login_key]= $row->{"RELATION_NAME"};

		}
		else
			$new_array[$login_key] = $login_id;

	}


	if (sizeOf($new_array) != sizeOf($login_array))
		return;

	array_unshift($new_array,"LOGIN_ID");
	array_shift($grid->grid[0]);
	$grid->grid[0] = $new_array;
	$this->grid = $grid;
	$this->grid->rotateRealGrid();
	$this->grid->rotateGrid();
}


}



?>
