<?php

/**
 * Form Element
 * @author Martijn van den Broek <mjbroek@xs4all.nl>
 * @copyright Copyright � 2005, Martijn van den Broek
 * @version $Id$
 * @package I-Ware
 * @subpackage Form
*/

/**
 * Abstract HTML class
 * Handles low level html stuff
 * @abstract
 * @package I-Ware
 * @subpackage Form
 */
class Html
{

	/**#@+
	 * @access private
	 */
    var $arrAttribs = array();
    /**#@-*/

	function Html()
	{
	}

	/**
	 * @access protected
	 */
	function param($name, $value)
	{
		return $name.'="'.htmlentities($value,ENT_COMPAT,'UTF-8').'" ';
	}

	/**
	 * @access protected
	 */
	function beginTag($name)
	{
		return '<'.$name.' ';
	}

	/**
	 * @access protected
	 * @param mixed $name String or Bool.. Ture => Close tag with />
	 */
	function endTag($name = '')
	{
		if ($name == '')
			return '>';
		elseif ( $name === true )
			return ' />';
		else
			return '</'.$name.'>';
	}

	/**
	 * @access protected
	 */
    function addAttrib($name, $value)
    {
    	//debug(sprintf('setting attrib [%s] to [%s]', $name, $value));
		$this->arrAttribs[$name] = $value;
    }

    function getAttrib($name)
    {
    	return isset($this->arrAttribs[$name]) ? $this->arrAttribs[$name] : false;
		//return $this->arrAttribs[$name];
    }

	/**
	 * @access protected
	 */
    function getAttribs()
    {
    	if (!$this->getAttribValue('id'))
			$this->setID('id_' . md5(uniqid(rand(), true)));

        $retval = '';
        foreach ($this->arrAttribs as $key=>$value)
            $retval .= $this->param($key, $value);

		if ( isset($this->isDisabled) && $this->isDisabled )
			$retval .= ' DISABLED ';

		if (isset($this->isReadonly) && $this->isReadonly )
			$retval .= ' READONLY ';

        return $retval;
    }

	/**
	 * returns the value of an attribute set in the class, false if not found
	 * @access protected
	 * @param string $name
	 * @return string value
	 */
    function getAttribValue($name)
    {
        foreach ($this->arrAttribs as $key=>$value)
		{
			if ($key == $name)
				return $value;
        }
        return false;
    }

	/**
	 * css class of the element
	 * @param string @class
	 */
    function setClass($class) { $this->addAttrib('class', $class);  }

	/**
	 * @access protected
	 */
    function getStyle()
    {
        $retval = '';

        if (isset($this->width))
        {
			if (is_numeric($this->width))
				$retval .= 'width: ' . $this->width.  'px; ';
			else
				$retval .= 'width: ' . $this->width . '; ';
		}

        if (isset($this->height))
        {
			if (is_numeric($this->height))
				$retval .= 'height: ' . $this->height.  'px; ';
			else
				$retval .= 'height: ' . $this->height. '; ';
		}

        $retval .= (isset($this->style_verticalAlign)) ? 'vertical-align: ' . $this->style_verticalAlign .'; ' : '';
        $retval .= (isset($this->style_display)) ? 'display: ' . $this->style_display .'; ' : '';

        return $retval;
    }

	/**
	 * @access public
	 * @param int $width width of the element
	 * @param int $height height of the element
	 */
    function setSize($width, $height)
    {
        $this->setWidth($width);
        $this->setHeight($height);
    }

	/**
	 * @access public
	 * @param int $width width of the element
	 */
    function setWidth($width)
    {
        $this->width = $width;
    }

	/**
	 * @access public
	 * @param int $height height of the element
	 */
    function setHeight($height)
    {
        $this->height = $height;
    }

	/**
	 * set behaviour attribute
	 * @param string @class
	 */
    function setBehaviour($behaviour) { $this->addAttrib('behaviour', $behaviour);  }
	function getBehaviour() { return $this->getAttrib('behaviour'); }
}

/**
 * Abstract Form_Element class
 * Provides basic methods to form elements
 * @abstract
 * @package I-Ware
 * @subpackage Form
 */
class Form_Element extends Html
{
	/**#@+
	 * @access private
	 */
    var $name = '';
    /**#@-*/

	/**
	 * @param string @name
	 */
    function Form_Element($name = '')
    {
        $this->name = $name;
    }

	/**
	 * HTML name of the element
	 * @param string @name
	 */
    function setName($name)   { $this->addAttrib('name', $name);    }

	/**
	 * HTML value of the element
	 * @param string @value
	 */
    function setValue($value) { $this->addAttrib('value', $value);  }

	/**
	 * HTML ID of the element
	 * @param string @name
	 */
    function setID($id)       { $this->addAttrib('id', $id); }

	/**
	 * Disables the button (makes it grey and unclickable)
	 * @param bool $disabled
	 */
	function setDisabled($disabled = true)
	{
		$this->isDisabled = $disabled;
	}

	/**
	 * Sets the element to readonly. Only applies to text (,password) and textarea.
	 * @param bool $readonly
	 */
	function setIsReadonly($readonly = true)
	{
		$this->isReadonly = $readonly;
	}

	/**
	 * @see setIsReadonly
	 */
	function setReadonly($readonly = true)
	{
		$this->setIsReadonly($readonly);
	}

	/**
	 * Sets the tabIndex for the control
	 * @param int $index
	 */
	function setTabIndex($index)
	{
		$this->addAttrib('tabindex', $index);
	}

	/**
	 * Sets the tabIndex for the control
	 * @param int $index
	 */
	function setAccessKey($key)
	{
		$this->addAttrib('accesskey', $key);
	}

	/**
	 * Set a label for the element, the user can click on it to (de)check it.
	 * Currently only supported by Form_Checkbox and Form_Select.
	 * @param string $label
	 */
    function setLabel($label)
    {
    	$this->label = $label;
    }

	/**#@+
	 * javascript code for event
	 * @param string @action javascript
	 */
    function onClick($action)     { $this->addAttrib('onclick' ,    $this->jsEscape($action)); }
    function onDblClick($action)  { $this->addAttrib('ondblclick' , $this->jsEscape($action)); }
    function onChange($action)    { $this->addAttrib('onchange',    $this->jsEscape($action)); }
    function onSelect($action)    { $this->addAttrib('onselect' ,   $this->jsEscape($action)); }
    function onKeyUp($action)     { $this->addAttrib('onkeyup',     $this->jsEscape($action)); }
    function onKeyDown($action)   { $this->addAttrib('onkeydown',   $this->jsEscape($action)); }
    function onKeyPress($action)  { $this->addAttrib('onkeypress',  $this->jsEscape($action)); }
    function onFocus($action)     { $this->addAttrib('onfocus',     $this->jsEscape($action)); }
    function onBlur($action)      { $this->addAttrib('onblur' ,     $this->jsEscape($action)); }
    function onMouseOver($action) { $this->addAttrib('onmouseover', $this->jsEscape($action)); }
    function onMouseOut($action)  { $this->addAttrib('onmouseout',  $this->jsEscape($action)); }


	/**#@-*/

	/**
	 * escapes newlines in a string for javascript
	 * @access private
	 */
	function jsEscape($js)
	{
	    $js = str_replace("\n", '\n', $js);
	    $js = str_replace("\r", '\r', $js);
	    return $js;
	}

	/**
	 * @return string HTML ID
	 */
	function getID() { return $this->getAttribValue('id'); }

	/**
	 * @return string HTML Name
	 */
	function getName() { return $this->getAttribValue('name'); }

}

/**
 * Form Button
 *
 * To create a form button:
 * <code>
 * $btn = new Form_Button('btn1', 'My Button');
 * $btn->onClick("alert('hello')");
 * $btn->setWidth(80);
 * echo $btn->getHtml();
 * </code>
 *
 *
 * @package I-Ware
 * @subpackage Form
*/
class Form_Button extends Form_Element
{

	/**#@+
	 * @access private
	 */
	var $type = 'button'; //-- or submit or reset
	/**#@-*/

	/**
	 * Button Constructor
	 * @param string $name HTML name (form element name)
	 * @param string $value HTML value (caption of the button)
	 * @param string $type button type, 'button' or 'submit'
	 */
	function Form_Button($name='btn', $value='button', $type='button')
	{
		$this->setName($name);
		$this->setValue($value);
		$this->setType($type);

		$this->setClass('button');
	}

	/**
	 * changes the type of the button ('button', 'submit' or 'reset')
	 * @param string $type
	 */
	function setType($type)
	{
		if ( $type == 'button' || $type == 'submit' || $type == 'reset')
			$this->type = $type;
		else
			trigger_error('Form_Button::setType() - Invalid type supplied.', E_USER_ERROR);
	}

	/**
	 * @return string returns the button HTML
	 */
	function getHtml()
	{
		$retval = $this->beginTag('input');
		$this->addAttrib('type', $this->type);
		$retval .= $this->getAttribs();
		$retval .= $this->param('style', $this->getStyle());
		$retval .= $this->endTag(true);

		return $retval;
	}

}

/**
 * Form Checkbox
 *
 * Example:
 * <code>
 * $btn = new Form_Checkbox('chk1', 'on', true);
 * $btn->setLabel('Click me');
 * $btn->onChange("alert(this.checked);");
 * echo $btn->getHtml();
 * </code>
 *
 * @package I-Ware
 * @subpackage Form
*/
class Form_Checkbox extends Form_Element
{
	/**#@+
	 * @access private
	 */
    var $checked = false;
	var $label = NULL;
	/**#@-*/

	/**
	 * @param string $name HTML name
	 * @param string $value HTML value
	 * @param bool $checked initial status of the checkbox (checked or not)
	 */
    function Form_Checkbox($name, $value='on', $checked=false)
    {
        $this->setName($name);
        $this->setValue($value);
        $this->setClass('Checkbox');
        $this->setChecked($checked);
    }

	/**
	 * change the status of the checkbox
	 * @param bool $checked
	 */
    function setChecked($checked)
    {
        $this->checked = $checked;
    }

    function getHtml()
    {
        $retval  = $this->beginTag('input');
        $retval .= $this->param('type', 'checkbox');
        $retval .= $this->getAttribs();
        $this->style_verticalAlign = 'middle';
        $retval .= $this->param('style', $this->getStyle());
        if ($this->checked)
            $retval .= ' CHECKED';
        $retval .= $this->endTag(true);

        if (isset($this->label))
        {
            $lb = new Form_Label($this->label, $this->getAttribValue('id') );
            $retval = sprintf('<div style="white-space: nowrap;">%s&nbsp;%s</div>', $retval, $lb->getHtml());
        }

        return $retval;
    }

}


/**
 * Form DateTime
 * @abstract
 * @package I-Ware
 * @subpackage Form
*/
class IWFormDateTime
{
    var $timestamp;
    var $name;

    function __construct($name)
    {
        $this->name = $name;
        $this->setToday();
    }

    function setTimestamp($timestamp)
    {
        $this->timestamp = $timestamp;
    }

    function setToday()
    {
        $this->timestamp = mktime ( date('H'), date('i'), 0, date('m'), date('d'), date('Y'));
    }

    function setDateTime($year, $month, $day, $hour, $minute, $second=0)
    {
        $this->timestamp = mktime ( $hour, $minute, $second, $month, $day, $year);
    }

    function setDate($year, $month, $day)
    {
        $this->timestamp = mktime ( 0, 0, 0, $month, $day, $year);
    }

    function setTime($hour, $minute, $second=0)
    {
        $this->timestamp = mktime ( $hour, $minute, $second, 1, 1, 2000);
    }

    function setMySQLDateTime($datetime)
    {
        if ($datetime == '')
            $this->setDateTime(2000, 1, 1, 0, 0, 0);
        else
        {
            $arrDateTime = explode(' ', $datetime);
            $arrDate = explode('-', $arrDateTime[0]);
            $arrTime = explode(':', $arrDateTime[1]);
            $this->timestamp = mktime($arrTime[0], $arrTime[1], $arrTime[2], $arrDate[1], $arrDate[2], $arrDate[0]);
        }
    }

    function setMySQLDate($date)
    {
        if ($date == '')
            $this->setDate(2000, 1, 1);
        else
        {
            $arrDate = explode("-", $date);
            $this->timestamp = mktime(0, 0, 0, $arrDate[1], $arrDate[2], $arrDate[0]);
        }
    }

    function setMySQLTime($time)
    {
        if ($time == '')
            $this->setTime(0,0,0);
        else
        {
            $arrTime = explode(":", $time);
            $this->timestamp = mktime($arrTime[0], $arrTime[1], $arrTime[2], 1, 1, 2000);
        }
    }


    function getMonthName($month_nr)
    {
        return strftime ('%B', mktime (0,0,0,$month_nr,1,2005));
    }

	function onchange($js)
	{
		$this->onchange = $js;
	}


    function getDateHtml()
    {
        $sel_day = new Form_Select($this->name.'_day');

        if (isset($this->onchange))
        	$sel_day->onchange($this->onchange);

        for ($i=1; $i<=31; $i++)
            $sel_day->addItem($i, $i, ($i == date('d', $this->timestamp)));


        $sel_month = new Form_Select($this->name.'_month');

        if (isset($this->onchange))
        	$sel_month->onchange($this->onchange);

        for ($i=1; $i<=12; $i++)
            $sel_month->addItem($this->getMonthName($i), $i, ($i == date('m', $this->timestamp)));


        $sel_year = new Form_Select($this->name.'_year');

        if (isset($this->onchange))
        	$sel_year->onchange($this->onchange);

        for ($i=1900; $i<=2050; $i++)
            $sel_year->addItem($i, $i, ($i == date('Y', $this->timestamp)));

        return '<span style="white-space: nowrap;">' . $sel_day->getHtml().'-'.$sel_month->getHtml().'-'.$sel_year->getHtml() .'</span>';
    }

    function getTimeHtml()
    {
        $sel_hour = new Form_Select($this->name.'_hour');

        if (isset($this->onchange))
        	$sel_hour->onchange($this->onchange);

        for ($i=0; $i<=23; $i++)
            $sel_hour->addItem(substr('0'.$i, -2, 2), $i, ($i == date('H', $this->timestamp)));

        $sel_minute = new Form_Select($this->name.'_minute');

        if (isset($this->onchange))
        	$sel_minute->onchange($this->onchange);

        for ($i=0; $i<=59; $i++)
            $sel_minute->addItem(substr('0'.$i, -2, 2), $i, ($i == date('i', $this->timestamp)));

        $sel_second = new Form_Select($this->name.'_second');

        if (isset($this->onchange))
        	$sel_second->onchange($this->onchange);

        for ($i=0; $i<=59; $i++)
            $sel_second->addItem(substr('0'.$i, -2, 2), $i, ($i == date('s', $this->timestamp)));

        return '<span style="white-space: nowrap;">' . $sel_hour->getHtml().':'.$sel_minute->getHtml().':'.$sel_second->getHtml() . '</span>';
    }


    //--

	/**
	 * Makes a string from a submitted Form_Date
	 * @param string name of the Form_Date
	 * @return string mySQLDate
	 */
	function mysqlDate($name)
	{
	    global $_REQUEST;
	    return $_REQUEST[$name.'_year'].'-'.$_REQUEST[$name.'_month'].'-'.$_REQUEST[$name.'_day'];
	}

	/**
	 * Makes a string from a submitted Form_Time
	 * @param string name of the Form_Time
	 * @return string mySQLTime
	 */
	function mysqlTime($name)
	{
	    global $_REQUEST;
	    return $_REQUEST[$name.'_hour'].':'.$_REQUEST[$name.'_minute'].':'.$_REQUEST[$name.'_second'];
	}

	/**
	 * Makes a string from a submitted Form_DateTime
	 * @param string name of the Form_DateTime
	 * @return string mySQLDateTime
	 */
	function mysqlDateTime($name)
	{
	    return $this->mysqlDate($name).' '.$this->mysqlTime($name);
	}

	/**
	 * Creats a mySQL DateTime string from the given arguments
	 * @param int day
	 * @param int month
	 * @param int year
	 * @param int hour
	 * @param int minute
	 * @param int second
	 * @return string mySQLDateTime
	 */
	function dbDateTime($day, $month, $year, $hour, $minute, $second)
	{
	    return $this->mysqlDate($day, $month, $year).' '.mysqlTime($hour, $minute, $second);
	}

	/**
	 * Converts a mysqlDate to timestamp
	 * @param string mySQLDate
	 * @return int timestamp
	 */
	public static function mysqlDateToTimestamp($date)
	{
	    $arrDate = explode("-", $date);

	    if (sizeof($arrDate) != 3)
			return '';

	    return mktime(0, 0, 0, $arrDate[1], $arrDate[2], $arrDate[0]);
	}

	/**
	 * Converts a mysqlDateTime to timestamp
	 * @param string mySQLDateTime
	 * @return int timestamp
	 */
	public static function mysqlDateTimeToTimestamp($datetime)
	{
	    $arrDateTime = explode(' ', $datetime);
	    $arrDate = explode('-', $arrDateTime[0]);
	    $arrTime = explode(':', $arrDateTime[1]);
	    return mktime($arrTime[0], $arrTime[1], $arrTime[2], $arrDate[1], $arrDate[2], $arrDate[0]);
	}

	public static function makePretty( $date='', $short=true )
	{
		if (empty($date))
			return '';

		if (is_numeric($date))
			$timestamp = $date;
		else
			$timestamp = IWFormDateTime::mysqlDateToTimestamp($date);

		if (empty($timestamp))
			$timestamp = mktime();

		return strftime('%e %B %Y', $timestamp);
	}

	public static function makePrettyDateTime($date='')
	{
		if (is_numeric($date))
			$timestamp = $date;
		else
			$timestamp = IWFormDateTime::mysqlDateTimeToTimestamp($date);

		return strftime('%e %B %Y %H:%M', $timestamp);
	}

	public static function makePrettyPeriod( $timestamp1='', $timestamp2 )
	{
		if (empty($timestamp1))
			$timestamp1 = mktime();

		if (empty($timestamp2))
			$timestamp2 = mktime();

		return strftime('%e %B', $timestamp1) . ' - ' . strftime('%e %B %Y', $timestamp2);
	}

}

/**
 * Form Form_Date
 * Creates a date selection box that can be used in HTML forms
 * Example:
 * <code>
 * $dt = new Form_Date('mydate');
 * $dt->setDate(2005, 3, 25);
 * echo $dt->getHtml();
 * </code>
 *
 * after submitting the date can be value can be read with:
 * <code>echo DateTime::mysqlDate('mydate'); //-- will return: "2005-3-25" </code>
 * @package I-Ware
 * @subpackage Form
*/

class Form_Date extends IWFormDateTime
{
    function Form_Date($name)
    {
        parent::__construct($name);
    }

    function getHtml()
    {
        return $this->getDateHtml();
    }
}

/**
 * Form Form_Time
 * Creates a time selection box that can be used in HTML forms
 * Example:
 * <code>
 * $dt = new Form_Time('mytime');
 * $dt->setTime(11, 33);
 * echo $dt->getHtml();
 * </code>
 *
 * after submitting the date can be value can be read with:
 * <code>echo mysqlDate('mytime'); //-- will return: "11:33" </code>
 * @package I-Ware
 * @subpackage Form
*/

class Form_Time extends IWFormDateTime
{
    function Form_Time($name)
    {
        parent::__construct($name);
    }
    function getHtml()
    {
        return $this->getTimeHtml();
    }
}

/**
 * Form DateTime
 * Creates a datetime selection box that can be used in HTML forms
 * Example:
 * <code>
 * $dt = new Form_DateTime('mydatetime');
 * $dt->setDateTime(2005, 3, 25, 11, 33);
 * echo $dt->getHtml();
 * </code>
 *
 * after submitting the date can be value can be read with:
 * <code>echo mysqlDate('mydate'); //-- will return: "2005-3-25 11:33" </code>
 * @package I-Ware
 * @subpackage Form
*/
class Form_DateTime extends IWFormDateTime
{
    function Form_DateTime($name)
    {
        parent::__construct($name);
    }

    function getHtml()
    {
        return $this->getDateHtml().'&nbsp;&nbsp;'.$this->getTimeHtml();
    }
}

/**
 * Form Form_Year
 * Creates a year selection box that can be used in HTML forms
 * Example:
 * <code>
 * $dt = new Form_Year('mydatetime', 2005);
 * echo $dt->getHtml();
 * </code>
 * @package I-Ware
 * @subpackage Form
*/
class Form_Year
{
    function Form_Year($name, $value=2003)
    {
        $this->name = $name;
        $this->value = $value;
        $this->setRange(1900, 2050);
    }

    function setRange($start, $end)
    {
        $this->start_year = $start;
        $this->end_year = $end;
    }

    function getHtml()
    {
        $sel_year = new Form_Select($this->name);
        for ($i=$this->start_year; $i<=$this->end_year; $i++)
            $sel_year->addItem($i, $i);

        $sel_year->setSelectedValue($this->value);

        return $sel_year->getHtml();;
    }
}


/**
 * Form File
 *
 * Creates a file selection input to upload files
 * <code>
 * $text = new Form_File('text1');
 * echo $text->getHtml();
 * </code>
 *
 * @package I-Ware
 * @subpackage Form
*/

class Form_File extends Form_Element
{
    function Form_File($name)
    {
        $this->setName($name);
        $this->setClass('text');
    }

    function getHtml()
    {
        $retval  = $this->beginTag('input');
        $this->addAttrib('type', 'file');
        $retval .= $this->getAttribs();
        $retval .= $this->param('style', $this->getStyle());
        $retval .= $this->endTag(true);

        return $retval;
    }

}


/**
 * Form Hidden
 *
 * Creates a hidden form element
 * <code>
 * $hidden = new Form_Hidden('MyHidden', '1235');
 * echo $hidden->getHtml();
 * </code>
 *
 * @package I-Ware
 * @subpackage Form
 */
class Form_Hidden extends Form_Element
{
	/**
	 * Constructor
	 * @param string $name
	 * @param string $value
	 */

	function Form_Hidden($name, $value='')
	{
		$this->setName($name);
		$this->setValue($value);
	}

	function getHtml()
	{
		$retval  = $this->beginTag('input');
		$this->addAttrib('type', 'hidden');
		$retval .= $this->getAttribs();
		$retval .= $this->endTag(true);

		return $retval;
	}
}

/**
 * Form Label
 *
 * Creates a label
 * <code>
 * $label = new Form_Label('Click me', $sel->getID());
 * echo $hidden->getHtml();
 * </code>
 *
 * @package I-Ware
 * @subpackage Form
 */
class Form_Label extends Form_Element
{
	/**
	 * Constructor
	 * @param string $name
	 * @param string $value
	 */
	function Form_Label($value, $for=null)
	{
		$this->value = $value;
		$this->for = $for;
	}

	function getHtml()
	{
		$retval  = $this->beginTag('label');

		if (isset($this->for))
			$retval .= $this->param('for', $this->for);

		$retval .= $this->getAttribs();
		$retval .= $this->endTag();
		$retval .= $this->value;
		$retval .= $this->endTag('label');

		return $retval;
	}

}

/**
 * Form Radio
 *
 * Create a serie radio boxes:
 * <code>
 * $rd = new Form_Radio('MyRadio', '10', 'Option 1', false);
 * echo $rd->getHtml();
 * $rd = new Form_Radio('MyRadio', '11', 'Option 2', true);
 * echo $rd->getHtml();
 * $rd = new Form_Radio('MyRadio', '12', 'Option 3', false);
 * echo $rd->getHtml();
 * </code>
 *
 * @package I-Ware
 * @subpackage Form
 */

class Form_Radio extends Form_Element
{
	/**
	 * Constructor
	 * @param string $name
	 * @param string $value
	 * @param string $caption label attached to the radio option
	 * @param bool $checked make this option selected
	 */
	function Form_Radio($name, $value, $caption=null, $checked=false)
	{
		$this->setName($name);
		$this->setValue($value);
		$this->setChecked($checked);
		$this->setClass('Checkbox');

		if ($caption != null)
			$this->setLabel($caption);
	}

	/**
	 * set the caption for this radio
	 * @param string label
	 */

    function setLabel($label)
    {
    	$this->label = $label;
    }

	/**
	 * @param bool $checked
	 */
	function setChecked($checked)
	{
		$this->checked = $checked;
	}

	function getHtml()
	{
		$retval  = $this->beginTag('input');
		$retval .= $this->param('type', 'radio');
		$retval .= $this->getAttribs();
        $this->style_verticalAlign = 'middle';
        $retval .= $this->param('style', $this->getStyle());
		$retval .= ($this->checked) ? ' CHECKED ' : '';
		$retval .= $this->endTag(true);

        if (isset($this->label))
        {
        	$retval .= $this->beginTag('label');
        	$retval .= $this->param('for', $this->getAttribValue('id'));
	        $retval .= $this->endTag();
	        $retval .= '&nbsp;'.$this->label;
        	$retval .= $this->endTag('label');

        }

		return $retval;
	}

}

/**
 * Form Select
 * Creats a select input box.
 *
 * Example1. Create a drop down selection input with three values:
 * <code>
 * $sel = new Form_Select('sel1');
 * $sel->addOption('option 1', 'value1');
 * $sel->addOption('option 2', 'value2');
 * $sel->addOption('option 3', 'value3');
 * $sel->setSelectedValue('value2');
 * echo $sel->getHtml();
 * </code>
 *
 * Example 2. Creats a listbox with 10 options and multiple selection
 *
 * <code>
 * $sel = new Form_Select('sel2');
 * $sel->setMultiple();
 * $sel->setSelectSize(5);
 * for ($i=0; $i<10; $i++)
 *     $sel->addOption('option '.$i, 'value_'.$i);
 * echo $sel->getHtml();
 * </code>
 *
 * @package I-Ware
 * @subpackage Form
 */

class Form_Select extends Form_Element
{
	/**#@+
	 * @access private
	 */
	var $isMultiple = false;
	var $items = array();
	var $in_optgroup = false;
	/**#@-*/

	/**
	 * Constructor
	 * @param string $name
	 */
	function Form_Select($name)
	{
		$this->setName($name);
		$this->setClass('select');
	}

	/**
	 * Sets the size of the select, if > 1 the select will become a listbox
	 * @param int $size
	 */
	function setSelectSize($size)
	{
		$this->addAttrib('size', $size);
	}

	/**
	 * If $multiple is set to true the select can have multiple selected values (only for listbox)
	 * @param bool $multiple
	 */
	function setMultiple($multiple = true)
	{
		$this->isMultiple = $multiple;
	}

	/**
	 * @see addItem
	 * @deprecated for backwards compatibility
	 */
	function addItem($caption, $value='', $selected=false, $add_to_top=true)
	{
	    $this->addOption ($caption, $value, $selected, $add_to_top);
	}

	/**
	 * add an option to the select
	 * @param string $caption caption of an option
	 * @param string $value value of the option
	 * @param bool $selected makes the option selected (can have multiple selected if select is set to multiple)
	 * @param bool $add_to_top adds the option to the top of the list
	 */
	function addOption($caption, $value=false, $selected=false, $add_to_bottom=true,$opts=array())
	{
		$item = array_merge($opts,array('type'=>'option', 'caption'=>$caption,
			                       'value'=>$value = ($value===false) ? $caption : $value,
			                       'selected'=>$selected));

		if ($add_to_bottom)
			$this->items[] = &$item;
		else
			array_unshift ($this->items, $item);
	}

	function addOptGroup($label)
	{
		$this->items[] = array('type'=>'optgroup', 'label' => $label );
	}

	/**
	 * Sets the selected option by index
	 * @param int $index
	 */
	function setItemIndex($index)
	{
		foreach ($this->items as $item)
			$item['selected'] = false;

		$this->items[$index]['selected'] = true;
	}

	/**
	 * Sets the selected option by value
	 */
	function setSelectedValue($value)
	{
		foreach($this->items as $key => $item)
		{
			if ($item['value'] == $value)
				$this->items[$key]['selected'] = true;
		}
	}

	/**
	 * @access private
	 */
	function getOptionsHtml()
	{
		$retval = '';

		foreach ($this->items as $item)
		{
			if ($item['type'] == 'option')
			{
				$retval .= $this->beginTag('option');
				foreach($item as $index=>$value)
				{
					if(!in_array($index,array('selected','caption','type')))
						$retval .= $this->param($index, $value);
				}
				$retval .= ($item['selected']) ? 'selected="selected"' : '';
				$retval .= $this->endTag();
				$retval .= $item['caption'];
				$retval .= $this->endTag('option');
			}
			else if ( $item['type'] == 'optgroup')
			{
				if ($this->in_optgroup)
					$retval .= $this->endTag('optgroup');

				$retval .= $this->beginTag('optgroup');
				$retval .= $this->param('label', $item['label']);
				$retval .= $this->endTag();

				$this->in_optgroup = true;
			}
		}

		if ($this->in_optgroup)
			$retval .= $this->endTag('optgroup');

		return $retval;
	}

	function getHtml()
	{
		$retval = '';

        if (isset($this->label))
        {
			$lb = new Form_Label($this->label, $this->getAttribValue('id') );
			$retval .= $lb->getHtml();
			$retval .= '&nbsp;';
        }

		$retval .= $this->beginTag('select');
		$retval .= $this->getAttribs();

		$retval .= $this->param('style', $this->getStyle());
		$retval .= ($this->isMultiple) ? ' multiple' : '';
		$retval .= $this->endTag();

		$retval .= $this->getOptionsHtml();
		$retval .= $this->endTag('select');

		return $retval;
	}

}

/**
 * Form Text
 *
 * Creates a textbox
 * <code>
 * $txt = new Form_Text('txtBox', 'hello there');
 * $txt->setMaxLength(20);
 * echo $txt->getHtml();
 * </code>
 *
 * @package I-Ware
 * @subpackage Form
 */
class Form_Text extends Form_Element
{
	/**#@+
	 * @access private
	 */
    var $isPassword = false;
	/**#@-*/

	/**
	 * Constructor
	 * @param string $name
	 * @param string $value
	 */
	function Form_Text($name, $value='')
    {
        $this->setName($name);
        $this->setValue($value);
        $this->setClass('text');
    }

    /**
	 * Sets the max character length for the textfield
	 * @param int $length
	 */
    function setMaxLength($length)
    {
        $this->addAttrib('maxlength', $length);
    }

    /**
	 * Sets the size of the textbox
	 * @deprecated better use CSS techniques
	 * @see setWidth
	 */
    function setTextSize($nr)
    {
        $this->addAttrib('size', $nr);
    }

    /**
	 * Changes the textfield to a password input
	 * @param bool $isPassword
	 */
    function setIsPassword($isPassword=true)
    {
        $this->isPassword = $isPassword;
    }

    function getHtml()
    {
        $retval  = $this->beginTag('input');

        if ($this->isPassword)
            $this->addAttrib('type', 'password');
        else
            $this->addAttrib('type', 'text');

        $retval .= $this->getAttribs();
        $retval .= $this->param('style', $this->getStyle());
        $retval .= $this->endTag(true);

        return $retval;
    }

}

/**
 * Form Form_Textarea
 *
 * Creates a textarea
 * <code>
 * $textarea = new Form_Textarea('txtArea', 'hello there');
 * $textarea->setSize(300, 100);
 * echo $textarea->getHtml();
 * </code>
 *
 * @package I-Ware
 * @subpackage Form
 */

class Form_Textarea extends Form_Element
{
	/**#@+
	 * @access private
	 */
	var $value;
	/**#@-*/

	/**
	 * Constructor sets the name and value
	 * @param string $name
	 * @param string $value
	 */
	function Form_Textarea($name, $value='')
	{
		$this->setName($name);
		$this->value = $value;
		$this->setClass('textarea');
	}

	function getHtml()
	{
		$retval  = $this->beginTag('textarea');
		$retval .= $this->addAttrib('style', $this->getStyle());
		$retval .= $this->getAttribs();
		$retval .= $this->endTag();
		$retval .= $this->value;
		$retval .= $this->endTag('textarea');

		return $retval;
	}

}


?>
