<?php
/**
 * @package I-Ware
 */
class Group
{
	// * 10/04/2003 Bas Schuiling :: complete rebuild class group
	var $db;
	var $group_id;
	var $group_name;
	var $group_company_id;
	var $user_id;

	function group($db)
	{
		$this->db = $db;
	}

	// group general
	function set($group_id)
	{
		$sql = "SELECT * FROM TBL_GROUP WHERE GROUP_ID = ?";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($group_id));
		if (! $ex->execute() || !($row = $ex->fetch()))
			return false;
		$this->group_id = $row->{"GROUP_ID"};
		$this->group_name = $row->{"GROUP_NAME"};
	}

	function getGroupCompany($group_id,$company_id)
	{
		$sql = "SELECT * FROM TBL_GROUP_COMPANY WHERE GROUP_ID = ?
				AND COMPANY_ID = ?";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($group_id,$company_id));
		$ex->execute();
		$row= $ex->fetch();
		$group_company_id = $row->{"GROUP_COMPANY_ID"};
		return $group_company_id;
	}

	function add()
	{
		return $this->addGroup();
	}

	function addGroup()
	{
		$sql = "INSERT INTO TBL_GROUP(GROUP_NAME) VALUES (?)";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($this->group_name));
		$ex->execute();
		return $ex->lastId();
	}

	function updateGroup($group_id)
	{
		$sql = "UPDATE TBL_GROUP SET GROUP_NAME = ? WHERE GROUP_ID = ?";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($this->group_name,$this->group_id));
		$ex->execute();
	}

	function removeGroup($group_id)
	{
		$sql = "DELETE FROM TBL_GROUP WHERE GROUP_ID = ?";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($group_id));
		$ex->execute();

		$sql = "DELETE FROM TBL_GROUP_USER WHERE GROUP_ID = ?";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($group_id));
		$ex->execute();

		$sql = "DELETE FROM TBL_GROUP_COMPANY WHERE GROUP_ID = ?";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($group_id));
		$ex->execute();
	}

	function fetchGroup($company_id = 0)
	{
		$sql = "SELECT * FROM TBL_GROUP";
		if ($company_id > 0)
		{	$sql .= " INNER JOIN TBL_GROUP_COMPANY ON
					  TBL_GROUP.GROUP_ID = TBL_GROUP_COMPANY.GROUP_ID
					  WHERE COMPANY_ID = ?";
		}
		$ex = $this->db->query($sql);
		if ($company_id > 0)
			$ex->bind_values(array($company_id));
		$ex->execute();
		return $ex;

	}

	// group user
	function addUserTogroup($group_id, $user_id)
	{
		$sql = "INSERT INTO TBL_GROUP_USER(GROUP_ID,USER_ID) VALUES (?,?)";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($group_id,$user_id));
		$ex->execute();
	}

	function removeUserFromGroup($group_id,$user_id)
	{
		$sql = "DELETE FROM TBL_GROUP_USER WHERE GROUP_ID = ? AND USER_ID = ?";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($group_id,$user_id));
		$ex->execute();

	}

	function removeAllgroupfromUser($user_id)
	{
		$sql = "DELETE FROM TBL_GROUP_USER WHERE USER_ID = ?";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($user_id));
		$ex->execute();
	}

	function checkUserInGroup($group_id,$user_id)
	{
		$sql = "SELECT * FROM TBL_GROUP_USER WHERE USER_ID = ? AND GROUP_ID = ?";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($user_id,$group_id));
		if ($ex->execute())
		{
			if ($row = $ex->fetch())
			{
				$gid = $row->{"GROUP_ID"};
				$uid = $row->{"USER_ID"};

				if ($gid == $group_id && $uid == $user_id)
					return true;
			}
			else return false;
		}
		else return false;
	}

	function fetchGroupRights($group_id, $company_id)
	{
		$sql = "SELECT * FROM TBL_SCRIPT_GROUP INNER JOIN TBL_GROUP_COMPANY ON
				TBL_SCRIPT_GROUP.GROUP_COMPANY_ID = TBL_GROUP_COMPANY.GROUP_COMPANY_ID AND COMPANY_ID = ?
				AND GROUP_ID = ? ORDER BY SCRIPT_ID";
		$ex = $this->db->query($sql);
		$ex->bind_values(array($company_id,$group_id));
		$ex->execute();
		return $ex;
	}

	function addOrUpdateRightForGroup($group_company_id, $script_id, $group_company_access = 0)
	{
		$db_privileges =& $GLOBALS['db_privileges'];
		$sql= "SELECT SCRIPT_ID FROM TBL_SCRIPT_GROUP
				WHERE GROUP_COMPANY_ID = ? AND SCRIPT_ID = ? ";
		$ex = $db_privileges->query($sql);
		$ex->bind_values(array($group_company_id,$script_id));
		$ex->execute();
		$rs = $ex->fetchAll();
		if (!count($rs))
		{
			$sql = "INSERT INTO TBL_SCRIPT_GROUP (SCRIPT_ID,GROUP_COMPANY_ID, SCRIPT_GROUP_ACCESS)
					VALUES ( ?,?,?)";
			$ex = $db_privileges->query($sql);
			$ex->bind_values(array($script_id,$group_company_id,$group_company_access));

		}
		else
		{
			$sql = "UPDATE TBL_SCRIPT_GROUP SET SCRIPT_GROUP_ACCESS = ?
					WHERE SCRIPT_ID = ? AND GROUP_COMPANY_ID = ? ";
			$ex = $db_privileges->query($sql);
			$ex->bind_values(array($group_company_access, $script_id, $group_company_id));

		}
		$ex->execute();
		return true;
	}

	function removeGroupRight($group_company_id, $script_id)
	{
		$db_privileges =& $GLOBALS['db_privileges'];
		$sql = "DELETE FROM TBL_SCRIPT_GROUP WHERE GROUP_COMPANY_ID = ?
				AND SCRIPT_ID = ?";
		$ex = $db_privileges->query($sql);
		$ex->bind_values(array($group_company_id,$script_id));
		$ex->execute();
	}

	function addGroupCompany( $group_id, $company_id )
	{
		$sql = "INSERT INTO TBL_GROUP_COMPANY ( GROUP_ID, COMPANY_ID ) VALUES ( ?, ? )";
		$ex = $this->db->query( $sql );
		$ex->bind_values( array( $group_id, $company_id ));
		$ex->execute();
		return $ex->lastId();
	}

	/**
	 * TODO: i see a pattern
	 */
	static public function getGroupBit($groupId)
	{
		switch ($groupId)
		{
			case GROUP_PUBLIC:
				return GROUP_BIT_PUBLIC;

			case GROUP_ACCOUNTANT:
				return GROUP_BIT_ACCOUNTANT;

			case GROUP_CRSO:
				return GROUP_BIT_CRSO;

			case GROUP_PARTICIPANT:
				return GROUP_BIT_PARTICIPANT;

			case GROUP_SYSADMIN:
				return GROUP_BIT_SYSADMIN;

			default:
				trigger_error('Group does not exist: '.$groupId, E_USER_ERROR);
		}
	}

	/**
	 * fixme: elc specific
	 * @return array of all possible groups
	 */
	static public function getGroups()
	{
		return array(
			GROUP_BIT_PUBLIC=>'Public',
			GROUP_BIT_PARTICIPANT=>'Participant',
			GROUP_BIT_CRSO=>'CRSO',
			GROUP_BIT_ACCOUNTANT=>'Accountant',
			GROUP_BIT_SYSADMIN=>'Sysadmin');
	}
}

?>
