<?php

/**
 * LanguageExport
 *
 * Exports the translatable texts from $GetLanguage with $ShowLanguage(s) for $applicationId
 * Can also export the IntercleanMasterfile texts.
 *
 * @package I-Ware
 * @subpackage LanguageInterface
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @since 17-02-2005
 * @version 0.2
 */
class LanguageExport
{
	var $db;
	var $app_id;

	/**
	 * Constructor
	 *
	 * @param Database $db
	 * @param integer $app_id default 0
	 */
	function LanguageExport( $db, $app_id = 0 )
	{
		$this->db = $db;
		$this->app_id = $app_id;
	}

	/**
	 * Gets all the non-empty translatable fields for all the scripts in a certain $app_id
	 *
	 * @param integer $primaryLanguage
	 * @param array $languages
	 */
	function generateExport( $primaryLanguage, $languages = array())
	{
		/**
		 * Create the headers with the language names.
		 */
		$header = array( "", "" );
		$header[] = "'".$this->_getLangName( $primaryLanguage, $primaryLanguage )."'";
		if( sizeof( $languages ) > 0 )
			foreach( $languages as $langId )
				$header[] = "'".$this->_getLangName( $langId, $primaryLanguage )."'";
		$header = implode( ",", $header )."\r\n";

		/**
		 * Query the text in the primary language.
		 */
		$sql = "SELECT LANGIF_ID, SCRIPT_ID, SCRIPT_NAME, LANGUAGE_ID, LANG_ARRAY
			FROM TBL_LANGUAGE_INTERFACE
			WHERE APPLICATION_ID = ? AND LANGUAGE_ID = ?
			ORDER BY LANGIF_ID";
		$ex = $this->db->query( $sql );
		$ex->bind_values( array( $this->app_id, $primaryLanguage ));
		$ex->execute();
		$langArray = array();
		$allLang = array();
		while( $row = $ex->fetch())
		{
			$current = unserialize( $row->{"LANG_ARRAY"} );
			$langArray = array_merge( $langArray, $current );

			foreach( $current as $name => $value )
			{
				if( ! empty( $value ))
					$allLang[$name] = array("id" => $row->{"LANGIF_ID"}, "name" => $name, $row->{"LANGUAGE_ID"} => $value );
			}
			if( sizeof( $languages ) > 0 )
			{
				/**
				 * If there are 1 or more secundairy languages query them.
				 */
				$sql = "SELECT LANGIF_ID, LANGUAGE_ID, LANG_ARRAY FROM TBL_LANGUAGE_INTERFACE
					WHERE APPLICATION_ID = ?
						AND LANGUAGE_ID IN ( ".implode( ",", $languages )." )
						AND SCRIPT_ID = ? AND SCRIPT_NAME = ?";
				$exe = $this->db->query( $sql );
				$exe->bind_values( array( $this->app_id, $row->{"SCRIPT_ID"}, $row->{"SCRIPT_NAME"} ));
				$exe->execute();
				while( $langRow = $exe->fetch())
				{
					$current = unserialize( $langRow->{"LANG_ARRAY"} );
					foreach( $current as $name => $value )
					{
						if( ! isset( $allLang[$name] ))
						{
							if( ! empty( $value ))
							{
								$langArray[$name] = "";
								$allLang[$name] = array("id" => $langRow->{"LANGIF_ID"}, "name" => $name, $row->{"LANGUAGE_ID"} => "", $langRow->{"LANGUAGE_ID"} => $value );
							}
						}
						else
							$allLang[$name][$langRow->{"LANGUAGE_ID"}] = $value;
					}
				}
			}
		}
		asort( $langArray );

		return $langArray;



		$output = "".$header;

		/**
		 * Implode the language array so it is comma seperated.
		 */
		foreach( $langArray as $name => $notUsed )
		{
			if( ! is_numeric( $name ) && isset( $allLang[$name] ))
			{
				$tmp = preg_replace( "/[,|\\\n|\\\r]/i", " ", $allLang[$name] );
				$output .= implode( ",", $tmp )."\r\n";
			}
		}
		/**
		 * Send the headers for a CSV output. File will ask to download.
		 */
		header("Content-Disposition: Attachment; filename=export".$primaryLanguage.".csv");
		header("Content-type: text/plain");
		echo $output;
		exit();
	}

	/**
	 *
	 */
	function importApplicationLanguage( $tmpName, $name, $language_id, $remove = true )
	{
		$path = SITE_HOMEDIR.SITE_PATH."klant/temp/";
		move_uploaded_file( $tmpName, $path.basename($name) );
		$stream = "";

		if( ! is_file( $path.$name ))
			exit( $path.$name." ---- ".$tmpName );
		$fp = fopen( $path.$name, "r" );

		while( ! feof( $fp ))
			$stream .= fread( $fp, 8192 );

		$import = $this->_generateArray( $stream );
		$sql = "SELECT SCRIPT_ID, SCRIPT_NAME
			FROM TBL_LANGUAGE_INTERFACE
			WHERE LANGIF_ID = ?";
		$ex = $this->db->query( $sql );
		$query = array();
		$tmp = array();
		$deletionName = array();
		foreach( $import as $langifID => $language )
		{
			foreach( $language as $name => $memo )
			{
				$deletionName[] = $name;
				$query[] = "( '".substr( $name, 0, 1 )."', '".$language_id."', '".
					str_replace( "\r", "", $memo )."', '".$name."', '".$this->app_id."' )";
			}
		}
		$sql = "DELETE FROM TBL_PURE_LANGUAGE
			WHERE PURE_TEXT IN ( '".implode( "', '", $deletionName)."' )
				AND APPLICATION_ID = ? AND LANGUAGE_ID = ?";
		$ex = $this->db->query( $sql );
		$ex->bind_values( array( $this->app_id, $language_id ));
		$ex->execute();

		$sql = "INSERT INTO TBL_PURE_LANGUAGE
			( FIRST_LETTER, LANGUAGE_ID, `TEXT`, PURE_TEXT, APPLICATION_ID ) VALUES ";
		$sql .= implode( ", ", $query );
		$ex = $this->db->query( $sql );
		$ex->execute();

		$sql = "DELETE FROM TBL_LANGUAGE_INTERFACE WHERE APPLICATION_ID = ? AND LANGUAGE_ID = ?";
		$ex = $this->db->query( $sql );
		$ex->bind_values( array( $this->app_id, $language_id ));
		$ex->execute();

//		print_r( $query );
	}
	/**
	 *
	 */
	function _____importApplicationLanguage( $tmpName, $name, $language_id, $remove = true )
	{
		$path = SITE_HOMEDIR.SITE_PATH."klant/temp/";
		move_uploaded_file( $tmpName, $path.basename($name) );
		$stream = "";

		if( ! is_file( $path.$name ))
			exit( $path.$name." ---- ".$tmpName );
		$fp = fopen( $path.$name, "r" );

		while( ! feof( $fp ))
			$stream .= fread( $fp, 8192 );

		$import = $this->_generateArray( $stream );
		$sql = "SELECT SCRIPT_ID, SCRIPT_NAME
			FROM TBL_LANGUAGE_INTERFACE
			WHERE LANGIF_ID = ?";
		$ex = $this->db->query( $sql );
		$query = array();
		$tmp = array();
		foreach( $import as $langifID => $language )
		{
			$ex->bind_values( array( $langifID ));
			$ex->execute();
			if( $row = $ex->fetch())
			{
				if( isset( $tmp[$row->{"SCRIPT_ID"}][$row->{"SCRIPT_NAME"}] ))
					$tmp[$row->{"SCRIPT_ID"}][$row->{"SCRIPT_NAME"}] = array_merge( $tmp[$row->{"SCRIPT_ID"}][$row->{"SCRIPT_NAME"}], $language );
				else
					$tmp[$row->{"SCRIPT_ID"}][$row->{"SCRIPT_NAME"}] = $language;
			}
			else
				echo $langifID." NOT FOUND<br>";
		}

		foreach( $tmp as $script_id => $vals )
		{
			foreach( $vals as $script_name => $language )
			{
				$currentArray = array();
				if( ! $remove )
				{
					$sql = "SELECT * FROM TB";
				}

				$sql = "DELETE FROM TBL_LANGUAGE_INTERFACE
					WHERE SCRIPT_ID = ? AND SCRIPT_NAME = ?
					AND LANGUAGE_ID = ? AND APPLICATION_ID = ?";
				$exe = $this->db->query( $sql );
				$exe->bind_values( array( $script_id, $script_name, $language_id, $this->app_id ));
				$exe->execute();

				$langArray = serialize( array_merge( $currentArray, $language ));

				$query[] = "( '".$script_id."', '".
					$script_name."', '".$language_id."', '".$langArray."', '".$this->app_id."' )";
			}
		}

		$sql = "INSERT INTO TBL_LANGUAGE_INTERFACE
			( SCRIPT_ID, SCRIPT_NAME, LANGUAGE_ID, LANG_ARRAY, APPLICATION_ID ) VALUES ";
		$sql .= implode( ",", $query );
		$ex = $this->db->query( $sql );
		$ex->execute();
		echo $sql;
	}
	/**
	 *
	 */
	function _generateArray( $stream )
	{
		$rows = explode( "\n", $stream );
		array_shift( $rows );
		$importArray = array();
		foreach( $rows as $row )
		{
			$cols = explode("\t",$row);
			if( sizeof( $cols ) == 3 )
				$importArray[$cols[0]][$cols[1]] = addslashes( stripslashes( $cols[2] ));
		}
		return $importArray;
	}

	/**
	 * Gets the name of the languages from the iware Db
	 *
	 * @param integer $language_id
	 * @param integer $view_language_id
	 */
	function _getLangName( $language_id, $view_language_id )
	{
		global $db_iware;

		$sql = "SELECT LANG_NAME_NAME FROM TBL_LANG_NAME WHERE LANGUAGE_ID =
			? AND LANG_NAME_DISPLAY_ID = ?";
		$ex = $db_iware->query($sql);
		$ex->bind_values(array($language_id, $view_language_id));
		$ex->execute();
		if( $row = $ex->fetch())
			return $row->{"LANG_NAME_NAME"};
		return "LANG ".$language_id;
	}

	/**
	 * Exports the masterfile $masterfile_id to be translated.
	 *
	 * @param integer $masterfile_id
	 * @param integer $primaryLanguage
	 * @param array $languages
	 */
	function masterfileExport( $masterfile_id, $primaryLanguage, $languages = array())
	{
		global $iware;

		/**
		 * Create the headers with the language names.
		 */
		$header = array( "" );
		$header[] = "'".$this->_getLangName( $primaryLanguage, $primaryLanguage )."'";
		if( sizeof( $languages ) > 0 )
			foreach( $languages as $langId )
				$header[] = "'".$this->_getLangName( $langId, $primaryLanguage )."'";
		$header = implode( ",", $header )."\r\n";

		$output = "";

		/**
		 * Get all the SPACE CATEGORIES in all languages chosen
		 */
		$sql = "SELECT SPACE_CATEGORY_ID, LANG_NAME, LANGUAGE_ID
			FROM TBL_SPACE_CATEGORY_NAME
			WHERE MASTER_FILE_ID = ? ";
		if( sizeof( $languages ) > 0 )
			$sql .= "AND LANGUAGE_ID IN ( ".$primaryLanguage.", ".implode(", ", $languages)." )";
		else
			$sql .= "AND LANGUAGE_ID = '".$primaryLanguage."'";
		$ex = $this->db->query( $sql );
		$ex->bind_values( array( $masterfile_id ));
		$ex->execute();

		$langArray = array();
		$orderArray = array();

		while( $row = $ex->fetch())
		{
			if( ! isset($langArray[$row->{"SPACE_CATEGORY_ID"}]))
			{
				$orderArray[$row->{"SPACE_CATEGORY_ID"}] = $row->{"LANG_NAME"};
				$langArray[$row->{"SPACE_CATEGORY_ID"}] = array( "id" => $row->{"SPACE_CATEGORY_ID"}, $row->{"LANGUAGE_ID"} => $row->{"LANG_NAME"} );
			}
			else
				$langArray[$row->{"SPACE_CATEGORY_ID"}][$row->{"LANGUAGE_ID"}] = $row->{"LANG_NAME"};
		}

		$category = "CATEGORY\r\n";
		$category .= $header;
		asort( $orderArray );
		foreach( $orderArray as $id => $notUsed )
		{
			if( isset( $langArray[$id] ))
			{
				$tmp = preg_replace( "/[,|\\\n|\\\r]/i", " ", $langArray[$id] );
				$category .= implode( ",", $tmp )."\r\n";
			}
		}
		$output .= $category."\r\n";


		/**
		 * Get all the SPACE TYPES/GROUPS in all languages chosen
		 */
		$sql = "SELECT SPACE_TYPE_ID, LANG_NAME, LANGUAGE_ID
			FROM TBL_SPACE_TYPE_NAME
			WHERE MASTER_FILE_ID = ? ";
		if( sizeof( $languages ) > 0 )
			$sql .= "AND LANGUAGE_ID IN ( ".$primaryLanguage.", ".implode(", ", $languages)." )";
		else
			$sql .= "AND LANGUAGE_ID = '".$primaryLanguage."'";
		$ex = $this->db->query( $sql );
		$ex->bind_values( array( $masterfile_id ));
		$ex->execute();

		$langArray = array();
		$orderArray = array();

		while( $row = $ex->fetch())
		{
			if( ! isset($langArray[$row->{"SPACE_TYPE_ID"}]))
			{
				$orderArray[$row->{"SPACE_TYPE_ID"}] = $row->{"LANG_NAME"};
				$langArray[$row->{"SPACE_TYPE_ID"}] = array( "id" => $row->{"SPACE_TYPE_ID"}, $row->{"LANGUAGE_ID"} => $row->{"LANG_NAME"} );
			}
			else
				$langArray[$row->{"SPACE_TYPE_ID"}][$row->{"LANGUAGE_ID"}] = $row->{"LANG_NAME"};
		}

		$type = "SPACE TYPE\r\n".$header;
		asort( $orderArray );
		foreach( $orderArray as $id => $notUsed )
		{
			if( isset( $langArray[$id] ))
			{
				$tmp = preg_replace( "/[,|\\\n|\\\r]/i", " ", $langArray[$id] );
				$type .= implode( ",", $tmp )."\r\n";
			}
		}
		$output .= $type."\r\n";

		/**
		 * Get all the ELEMENTS in all languages chosen
		 */
		$sql = "SELECT ELEMENT_ID, LANG_NAME, LANGUAGE_ID
			FROM TBL_ELEMENT_NAME
			WHERE MASTER_FILE_ID = ? ";
		if( sizeof( $languages ) > 0 )
			$sql .= "AND LANGUAGE_ID IN ( ".$primaryLanguage.", ".implode(", ", $languages)." )";
		else
			$sql .= "AND LANGUAGE_ID = '".$primaryLanguage."'";
		$ex = $this->db->query( $sql );
		$ex->bind_values( array( $masterfile_id ));
		$ex->execute();

		$langArray = array();
		$orderArray = array();

		while( $row = $ex->fetch())
		{
			if( ! isset($langArray[$row->{"ELEMENT_ID"}]))
			{
				$orderArray[$row->{"ELEMENT_ID"}] = $row->{"LANG_NAME"};
				$langArray[$row->{"ELEMENT_ID"}] = array( "id" => $row->{"ELEMENT_ID"}, $row->{"LANGUAGE_ID"} => $row->{"LANG_NAME"} );
			}
			else
				$langArray[$row->{"ELEMENT_ID"}][$row->{"LANGUAGE_ID"}] = $row->{"LANG_NAME"};
		}

		$element = "ELEMENT\r\n".$header;
		asort( $orderArray );
		foreach( $orderArray as $id => $notUsed )
		{
			if( isset( $langArray[$id] ))
			{
				$tmp = preg_replace( "/[,|\\\n|\\\r]/i", " ", $langArray[$id] );
				$element .= implode( ",", $tmp )."\r\n";
			}
		}
		$output .= $element."\r\n";
		/**
		 * Send the headers for a CSV output. File will ask to download.
		 */
		header("Content-Disposition: Attachment; filename=masterfile_export".$primaryLanguage.".csv");
		header("Content-type: text/plain");
		echo $output;
		exit();
	}
}

?>
