<?php
/**
 * Message Handling classes
 * @author Chris Wesseling <chris@basesoft.com>
 * @version $Id$
 * @package I-Ware
*/
require_once(IWARE.'AbstractObject.php');
/**
 * Message altering replacement prefix
 * @see _alterMessage()
 */
define('REPL_PREFIX', '<%');
/**
 * Message altering replacement postfix
 * @see _alterMessage()
 */
define('REPL_POSTFIX', '%>');

/**#@+
 * Message type
 */
define('MESSAGE_EMAIL', 1);
define('MESSAGE_SMS', 2);
define('MESSAGE_FAX', 3);
/**#@-*/
/**
 *  Handles Message Events
 */
class MessageHandler
{
	/**
	 * @var Database
	 */
	var $db;

	/**
	 * @param Database ApplicationDB (contains eventlisteners and messages)
	 */
	function MessageHandler($db=null, $db_edit=null, $db_iware=null)
	{
		//global $GLOBALS; //Should be a super global, but with obelix you never know
		$this->db = $db?$db:IWRegistry::registry('db');
		$this->db_edit = $db_edit?$db_edit:$this->db;
		$this->_setOrigin();
		$events = $this->getMessageEvents();
		foreach ($events as $event)
			if ( ! defined($event->handle) )
				define($event->handle, $event->id);
	}

	/**
	 * Add a new EvenListener
	 * @param MessageEvent $me Event to listen to
	 * @param int $meid MessageId of Message to send
	 * @param int $mtype MessageType (MESSAGE_EMAIL|MESSAGE_SMS)
	 * @param string $cb Callback function
	 */
	function addEventListener($me, $mid, $mtype=MESSAGE_EMAIL, $cb='_defaultCallback')
	{
		$sql = "REPLACE INTO TBL_MESSAGE_EVENT_LISTENER
				(MESSAGE_EVENT_ID,
				 MESSAGE_ID,
				 MESSAGE_TYPE,
				 CALLBACK)
				VALUES(?,?,?,?)";
		$q = $this->db->query($sql);
		$q->bind_values(array($me->id, $mid, $mtype, $cb));
		return $q->execute();
	}

	function getListenersByEvent($event)
	{
		if ( $this->_checkListeners($event) )
			return $event->listeners;
		return array();
	}

	/**
	 * @return Array of MessageEvents
	 */
	function getMessageEvents()
	{
		$rv = array();
		$sql = "SELECT *
			FROM TBL_MESSAGE_EVENT";
		$q = $this->db->query($sql);
		if ( $q->execute(true) ) //don't die... Tbl could be absent
			while ( $r = $q->fetch() )
				$rv[] = new MessageEvent($r->MESSAGE_EVENT_ID, $r->DESCRIPTION_HANDLE, $this);
		return $rv;
	}
	/**
	 * @return array type_id=>handle
	 */
	function getMessageTypes()
	{
		$rv = array();
		$rv[MESSAGE_EMAIL] = 'MESSAGE_EMAIL';
		$rv[MESSAGE_SMS] = 'MESSAGE_SMS';
		$rv[MESSAGE_FAX] = 'MESSAGE_FAX';
		return $rv;
	}

	/**
	 * @param MessageEvent $event
	 */
	function handleEvent(&$event, $dry_run)
	{
		$this->_dry_run = $dry_run;

		if ( !$event->id )
			return !trigger_error('MessageHandler::handleEvent() : No event id set', E_USER_NOTICE); //Don't die, log and return false

		if ( $this->_checkListeners($event) == 0 ) {
			info('MessageEvent::handleEvent() - No listeners for event [' . $event->id.']');
			return true;
		}
		$this->_prepareMessageQueue($event);
		$this->_sendMessages($event);

	}
	/**#@+
	 * @access private
	 */
	/**
	 * @var string Legal 'From:' and 'Sender:' header value
	 */
	var $_sender;
	/**
	 * @var string Legal 'Return-Path:' header value
	 */
	var $_return_path;
	/**
	 * @var bool dry_run
	 */
	//var $_dry_run = true;

	/**
	 * Set Sender, From and possibly Return-Path
	 * Called from contructor.
	 * Override this in child class if you want to change the defaults.
	 * But do end with a return parent::_setOrigin(), so this function can be expanded in the future with other defaults!
	 * It will always leave child settings in tact.
	 */
	function _setOrigin()
	{
		empty($this->_sender) && $this->_sender = 'message-bot@basesoft.com';
		empty($this->_return_path) && $this->_return_path = false; //defaults to apache user
		return true;
	}

	/**
	 * Right now this sends messages from $event immediatly
	 * This could be changed to some smart queue mechanism.
	 * @param MessageEvent $event
	 */
	function _sendMessages(&$event)
	{
		if (sizeof($event->messages) == 0)
			debug ('MessageEvent::_sendMessages() - No messages to send');

		// array Messages Container (Type=>array(Destination=>array(MESSAGE_ID=>msg, msg...)))
		foreach ( $event->messages as $type=>$bitbucket ) {
			foreach ( $event->messages[$type] as $destination=>$messages ) {
				foreach ( $messages as $mid=>$msg )
				{
					if ( isset($event->destination_info[$destination]->language_id) )
						$language_id = $event->destination_info[$destination]->language_id;
					else
					{
						debug('message_handler::_sendMessages(): No language_id set on ('
									.get_class($event->destination_info[$destination])
									.') destination_info['
									.$destination.'] Sending message in default language.');
						$language_id = $msg->default_language_id;
					}
					switch ( $type )
					{
						case MESSAGE_EMAIL:

							$mail = IWRegistry::registry('mail');
							$mail->ClearAllRecipients();
							$mail->ClearAttachments();
							$mail->ClearCustomHeaders();
							$mail->ClearReplyTos();

							// check if we passed any attachments in the params
							if (array_key_exists('attachments', $event->params)) {


								foreach ( $event->params['attachments'] as $attachment ) {

									// check if $attachment is a string or an array that has keys (path, name, etc)
									if (is_array($attachment)) {
										$att_path = $attachment['path'];
										$att_name = "";
										if (array_key_exists('name', $attachment))
											$att_name = $attachment['name'];
										$att_encoding = "base64";
										if (array_key_exists('encoding', $attachment))
											$att_encoding = $attachment['encoding'];
										$att_type = "application/octet-stream";
										if (array_key_exists('type', $attachment))
											$att_type = $attachment['type'];

										if (!$mail->AddAttachment($att_path, $att_name, $att_encoding, $att_type))
											info("_sendMessages:: could not attach file '" . $att_path . ' name='.$att_name);

									} elseif (is_string($attachment)) {
										// just a string, so it's only the path of the file we want to attach
										if (!$mail->AddAttachment($attachment))
											info("_sendMessages:: could not attach file '" . $attachment);
									} else {
										info("_sendMessages:: could not attach '" . $attachment. '. Unknown data.');
									}
								}
							}

							$mail->From = $this->_sender;
							$this->_senderName && $mail->FromName = $this->_senderName;
							$mail->AddAddress($destination);
							$mail->Subject = $msg->getSubject($language_id);

							$mail->IsHTML(true);
							$mail->WordWrap = 50; // set word wrap to 50 characters

							$mail->Body    = $msg->getHTML($language_id);
							$mail->AltBody = $msg->html2txt($msg->getHTML($language_id));
							$mail->CharSet = 'utf-8';

							if (!$this->_dry_run)
							{
								if(mb_strlen($mail->Subject) == 0 || mb_strlen($mail->Body) == 0)
								{
									info('Not sending email. Subject or Body is empty. Message ID: '.$msg->id);
									break;
								}

								if(!$mail->Send())
								{
									if (SMTP_SERVERS == '') {
										info('not sending mail, SMTP_SERVERS is empty');
										return;
									}

								   trigger_error("Email message could not be sent: ".$mail->ErrorInfo, E_USER_NOTICE);
								   //no we gotta try the other smtp server... whatever that is
								   $mail->SmtpClose(); //kill all existing SMPT connections
								   $mail->Host = 'localhost'; // this should always be available on our servers
								   if(!$mail->Send())
								   {
										//we have tried two different ways and we still cannot send
										trigger_error("Email message could not be sent second try: ".$mail->ErrorInfo, E_USER_WARNING);
								   }
								   else
									info("Second try: Sent email with subject '".$msg->getSubject($language_id)."' to ".$destination);

								}
								else
									info("Sent email with subject '".$msg->getSubject($language_id)."' to ".$destination);
							}
							else
								info(sprintf("Dry-run enabled! Not sending message to '%s' with subject %s", $destination, $msg->getHeader('Subject')));
						break;
					}
				}
			}
		}
	}
	/**
	 * Load listeners to $event
	 * Sets $event->listeners
	 * @param MessageEvent $event
	 * @return int Number of listeners found
	 */
	function _checkListeners(&$event)
	{
		$sql = "SELECT MESSAGE_ID, MESSAGE_TYPE, CALLBACK
			FROM TBL_MESSAGE_EVENT_LISTENER
			WHERE MESSAGE_EVENT_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values(array($event->id));
		if ( !$q->execute(true) )
			trigger_error('MessageHandler::_checkListeners() : '.
					$q->exception->toString(), E_USER_NOTICE);
		else
			while ( $r = $q->fetch() )
				$event->listeners[] = (array) $r;
		return count($event->listeners);
	}
	/**
	 * Prepare messages by calling the callback from the listeners.
	 *
	 * @param MessageEvent $event
	 */
	function _prepareMessageQueue(&$event)
	{
		foreach ( $event->listeners as $listener )
			$this->$listener['CALLBACK']($event, $listener['MESSAGE_ID'], $listener['MESSAGE_TYPE']);

	}
	/**
	 * This is the defaultCallback action
	 *
	 * A Callback should fill $event->messages.
	 *
	 * This Callback splits this task up in several steps..
	 * So you can easily extend this class and overload only the steps you want different behaviour for.
	 * Or add another Callback and make use of some of these steps.
	 * <ol>
	 * 	<li>_getDestinations()</li>
	 * 	<li>_filterDestinations()</li>
	 * 	<li>_getMessage()</li>
	 * 	<li>_alterMessage()</li>
	 * </ol>
	 *
	 * @param MessageEvent $event
	 * @param int MESSAGE_ID
	 * @param int MESSAGE_TYPE
	 */
	function _defaultCallback(&$event, $message_id, $message_type)
	{
		$dests = $this->_getDestinations($event, $message_id, $message_type);
		if ( !count($dests) ) { //no need to go on
			debug('MessageHandler::_defaultCallback() - No destinations found');
			return true;
		}
		$this->_setInitiator($event, $message_type);
		$dests = $this->_filterDestinations($dests, $event, $message_id, $message_type);
		if ( !count($dests) ) { //no need to go on
			debug('MessageHandler::_defaultCallback() - No destinations found');
			return true;
		}
		$this->_getMessage($dests, $event, $message_id, $message_type);
		$this->_alterMessage($dests, $event, $message_id, $message_type);
	}

	/**
	 * Gets destinations, being email addresses, cellphone numbers, faxnumbers...
	 * Anything, depending on type
	 * Returns it in an array AND leaves them in $event->messages
	 * @param MessageEvent $event
	 * @param int MESSAGE_ID
	 * @param int MESSAGE_TYPE
	 * @returns array
	 */
	function _getDestinations(&$event, $message_id, $message_type)
	{
		$rv = array();
		$sql = "SELECT DESTINATION
				FROM TBL_MESSAGE_EVENT_DESTINATION
				WHERE MESSAGE_EVENT_ID = ?
					AND MESSAGE_ID = ?
					AND MESSAGE_TYPE = ?";
		$q = $this->db->query($sql);
		$q->bind_values(array($event->id, $message_id, $message_type));
		$q->execute();
		while ( $r = $q->fetch() )
		{
			$event->messages[$message_type][$r->DESTINATION][$message_id] = '';
			$rv[] = $r->DESTINATION;
		}
		return $rv;
	}
	/**
	 * Set the initiator
	 *
	 * @param MessageEvent $event
	 * @param int MESSAGE_TYPE
	 */
	function _setInitiator(&$event, $message_type)
	{
		//Set the field to get from db
		switch ( $message_type )
		{
			case MESSAGE_EMAIL:
				$field = 'INFO_PERSON_EMAIL';
			break;
			case MESSAGE_SMS:
				$field = 'INFO_PERSON_MOBILE';
			break;
		}
		$sql = "SELECT ip.$field as DESTINATION
			FROM TBL_INFO_PERSON as ip
			INNER JOIN TBL_USER as u
				USING(INFO_PERSON_ID)
			WHERE u.USER_ID = ?";
		$q = $this->db_edit->query($sql);
		$q->bind_values(array(isset($_SESSION['session_user_id']) ? $_SESSION['session_user_id'] : 1));
		$q->execute();
		$r = $q->fetch();
		$event->initiator[$message_type] = $r->DESTINATION;
	}
	/**
	 * Filter the initiator (session_user) from the destinations
	 *
	 * @param array $dests Destinations to filter
	 * @param MessageEvent $event
	 * @param int MESSAGE_ID
	 * @param int MESSAGE_TYPE
	 * @returns array
	 */
	function _filterDestinations($dests, &$event, $message_id, $message_type)
	{
		$filter = array();
		$filter[] = $init = $event->initiator[$message_type];
		unset($event->messages[$message_type][$init]);
		return array_diff($dests, $filter);
	}
	/**
	 * Fetch message content from application db
	 * Fills $event->messages further.
	 * @param array $dests infoTypes
	 * @param MessageEvent $event
	 * @param int $message_id MESSAGE_ID
	 * @param int $message_type MESSAGE_TYPE
	 * @return array of Messages
	 */
	function _getMessage($dests, &$event, $message_id, $message_type)
	{
		$lang_ids = $rv = array();
		$msg = new Message($this->db);
		foreach ( $dests  as $dest )
		{
			$event->destination_info[$dest]->isComplete() || $event->destination_info[$dest]->set();
			if ( isset($event->destination_info[$dest]->language_id) )
				$lang_ids[] = $event->destination_info[$dest]->language_id;
			$rv[$message_id] = $event->messages[$message_type][$dest][$message_id] =& $msg;
		}
		$msg->set($message_id, array_unique($lang_ids));

		return $rv;
	}
	/**
	 * Alter message in $event->messages
	 * @param array $dests
	 * @param MessageEvent $event
	 * @param int $message_id MESSAGE_ID
	 * @param int $message_type MESSAGE_TYPE
	 * @return array of Messages
	 */
	function _alterMessage($dests, &$event, $message_id, $message_type)
	{
		$rv = array();
		foreach ( $dests as $dest )
		{
			$contents = implode('', $event->messages[$message_type][$dest][$message_id]->content);
			$contents .= implode('', $event->messages[$message_type][$dest][$message_id]->html_content);
			$contents .= implode('', $event->messages[$message_type][$dest][$message_id]->tiny_content);

			if ( !empty($event->destination_info) && preg_match_all('/'.REPL_PREFIX.'RECIPIENT_(\w*?)'.REPL_POSTFIX.'/', $contents , $info_fields) )
				foreach ( array_unique($info_fields[1]) as $field )
					$event->messages[$message_type][$dest][$message_id]->alterContent(array(REPL_PREFIX.'RECIPIENT_'.$field.REPL_POSTFIX=>$event->destination_info[$dest]->{strtolower($field)}));

			//Other replacements
			if ( !empty($event->params['replacements']) )
				$event->messages[$message_type][$dest][$message_id]->alterContent($event->params['replacements']);
			$rv[$message_id] = $event->messages[$message_type][$dest][$message_id];
		}
		return $rv;
	}
	/**#@-*/
}

/**
 * An event that could initiate Message sending
 */
class MessageEvent
{
	/**
	 * @var MessageHandler
	 */
	var $handler = false;
	/**
	 * @var array Listener Container
	 */
	var $listeners = array();
	/**
	 * @var array Initiator Destinations Containter (Type=>destination)
	 */
	var $initiator = array();
	/**
	 * @var array Messages Container (Type=>array(Destination=>array(MESSAGE_ID=>msg, msg...)))
	 */
	var $messages = array();
	/**
	 * @var array Parameter Container (name=>value)
	 */
	var $params = array();
	/**
	 * @var array (string)Destination=>(InfoPerson) Destination
	 */
	var $destination_info = array();
	/**
	 * @param int $id MESSAGE_EVENT_ID
	 * @param string $handle DESCRIPTION_HANDLE
	 * @param MessageHandler $handler
	 * @param array $params parameters
	 *     params can contain key 'attachments', which can be an array of strings or an array of arrays,
	 *     each containing keys path, name, encoding and/or type. the values of these keys are used as the
	 *     parameters of the AddAttachment method for the mail object
	 */
	function MessageEvent($id, $handle=null, $handler=null, $params=array())
	{
		$this->id = $id;
		$this->handler || ( $this->handler = $handler?$handler:new MessageHandler() );
		$this->handle = $handle;
		$this->params = (array) $params;
	}

	function addReplacement($placeholder, $value)
	{
		if ( empty($value) )
			$value = '';
		$this->params['replacements'][REPL_PREFIX.$placeholder.REPL_POSTFIX] = $value;
	}

	/**
	 * Throw Event
	 * Can be called statically for ease of use.
	 * <code>
	 * MessageEvent::throw(INTERCLEAN_COMPLAINT_LOGGED, array('project_id'=>3))
	 * </code>
	 * @static
	 * @param int $id MESSAGE_EVENT_ID
	 * @param array $params parameters
	 * @param string $handlerclass Classname of handler
	 */
	public static function send($id=null, $params=array(), $handlerclass='MessageHandler', $dry_run=false)
	{
		$event = new MessageEvent($id, '', new $handlerclass(), $params);
		$event->handler->handleEvent($event, $dry_run);
	}

}

/**
 * A Message
 */
class Message extends AbstractObject
{
	/**
	 * @var int MESSAGE_ID
	 */
	var $id;
	/**
	 * @var string MESSAGE_HANDLE
	 */
	var $handle;
	/**
	 * @var int COMPANY_ID
	 */
	var $company_id;
	/**
	 * @var int DEFAULT_LANGUAGE_ID
	 */
	var $default_language_id;
	/**
	 * @var string Message content
	 */
	var $content = array();
	/**
	 * @var string Message content
	 */
	var $html_content = array();
	/**
	 * @var string Tiny message content.. 160 chars max;
	 */
	var $tiny_content = array();
	/**
	 * @var string
	 */
	var $subject = array();

	/**
	 * @param int $id
	 * @param mixes $language_id int or array of int
	 * @param int $company_id
	 */
	function set($id=null, $language_id=null, $company_id=null)
	{
		$id = $id ? $id : $this->id;
		$company_id = $company_id ? $company_id : $this->company_id;
		$company_id = $company_id ? $company_id : isset($_SESSION['session_user_company_id']) ? $_SESSION['session_user_company_id'] : 1;
		$language_id = (array) $language_id;
		$values = $language_id;

		$sql = "SELECT m.MESSAGE_ID as id, m.COMPANY_ID as company_id,
				m.DEFAULT_LANGUAGE_ID as default_language_id,
				l.LANGUAGE_ID as language_id,
				l.SUBJECT as subject,
				l.TINY_CONTENT as tiny_content,
				c.CONTENT_TYPE,
				c.CONTENT
			FROM TBL_MESSAGE as m
			LEFT JOIN TBL_MESSAGE_LANG as l
				ON l.MESSAGE_ID = m.MESSAGE_ID
					AND l.LANGUAGE_ID IN ( 	m.DEFAULT_LANGUAGE_ID".str_repeat(',?', count($language_id)).")
			LEFT JOIN TBL_MESSAGE_CONTENT as c
				ON c.MESSAGE_ID = m.MESSAGE_ID
					AND c.LANGUAGE_ID = l.LANGUAGE_ID
			WHERE m.MESSAGE_ID = ? AND m.COMPANY_ID = ?";
		$values[] = $id;
		$values[] = $company_id;
		$q = $this->db->query($sql);
		$q->bind_values($values);
		$this->lastQuery =& $q;
		if ( $q->execute(true) )
		{
			if ( ! ($r = $q->fetch()) )
				return false;
			do
			{
				$this->id = $r->id;
				$this->company_id = $r->company_id;
				$this->default_language_id = $r->default_language_id;
				if ( $r->language_id )
				{
					$this->subject[$r->language_id] = $r->subject;
					$this->tiny_content[$r->language_id] = $r->subject;
					switch ( $r->CONTENT_TYPE )
					{
						case 'text/plain':
							$this->content[$r->language_id] = $r->CONTENT;
						break;
						case 'text/html':
							$this->html_content[$r->language_id] = $r->CONTENT;
						break;
					}
				}
			}while ( $r = $q->fetch() );
			return true;
		}
		return false;
	}

	/**
	 * Use update() or insert() to really write to database
	 * @param int $languageId
	 * @param array $content (a subset of) (subject=>'', plain=>'', html=>'', tiny=>'')
	 */
	function addLanguage($languageId, $content)
	{
		isset($content['subject']) && $this->subject[$languageId] = $content['subject'];
		isset($content['plain']) && $this->content[$languageId] = $content['plain'];
		isset($content['html']) && $this->html_content[$languageId] = $content['html'];
		isset($content['tiny']) && $this->tiny_content[$languageId] = $content['tiny'];
	}

	function getSubject($languageId, $fallback=true)
	{
		$rv = $this->_get('subject', (int)$languageId);
                if(empty($rv) && $fallback)
                    return $this->_get('subject', 44);
                return $rv;
	}
	function getHTML($languageId, $fallback=true)
	{
		$rv = $this->_get('html_content', (int)$languageId);

		if (empty($rv) && $fallback)
			return $this->_get('html_content', 44);

		return $rv;
	}

	function getPlain($languageId, $fallback=true)
	{
		$rv = $this->_get('content', (int)$languageId);

		if (empty($rv) && $fallback)
			return $this->_get('content', 44);

		return $rv;
	}

	/**
	 * Add a header to an email
	 * like:
	 * <code>
	 * $msg->addHeader('From', 'kwak@braatsel.net');
	 * </code>
	 *
	 * See RFC 822 for more options.
	 * @param string $h Header
	 * @param string $v Value
	 */
	function addHeader($h, $v)
	{
		$this->headers[$h] = $v;
	}

	/**
	 * retrieves a header
	 */
	function getHeader($h)
	{
		if (isset($this->headers[$h]))
			return $this->headers[$h];
		else
			return false;
	}

	/**
	 * Alter message content
	 * Replaces placeholders with real content
	 * @param array $replacements
	 */
	function alterContent($replacements=array())
	{
		foreach ( $this->_getLanguages() as $languageId )
		{
			$this->_checkLanguage($languageId); //init
			list($this->content[$languageId], $this->html_content[$languageId], $this->tiny_content[$languageId]) =
				str_replace(array_keys($replacements), array_values($replacements), array($this->content[$languageId],
					$this->html_content[$languageId], $this->tiny_content[$languageId]));
		}
	}

	/**
	 * @return int MESSAGE_ID or false on error
	 */
	function insert()
	{
		$this->company_id = $this->company_id ? $this->company_id : $_SESSION['session_user_company_id'];
		if ( !parent::insert() )
			return false;
		return $this->_insertContent()?$this->id:false;
	}
	/**
	 * @param Database $db
	 * @return bool
	 */
	function update()
	{
		parent::update();
		return $this->_insertContent()?$this->id:false;
	}
	/**#@+
	 * @access private
	 */
	function _get($src, $lid)
	{
		return isset($this->{$src}[$lid])?$this->{$src}[$lid]:'';
	}
	function _insertContent()
	{
		$languageIds = array();
		foreach ($this->langprops as $prop)
			foreach ( array_keys($this->$prop) as $lid )
				$languageIds[] = $lid;

		$languageIds = array_unique($languageIds);

		if ( empty($languageIds) )
			return true; //done... that was easy

		$sql = "REPLACE INTO TBL_MESSAGE_CONTENT (MESSAGE_ID, LANGUAGE_ID, CONTENT_TYPE, CONTENT) VALUES ";
		$langsql = "REPLACE INTO TBL_MESSAGE_LANG (MESSAGE_ID, LANGUAGE_ID, SUBJECT, TINY_CONTENT) VALUES ";

		$values = array();
		foreach ($languageIds as $languageId)
		{
			if ( !empty($this->content[$languageId]) )
			{
				$sql .= "(?, ?, 'text/plain', ?), ";
				$values[] = $this->id;
				$values[] = $languageId;
				$values[] = $this->content[$languageId];
			}
			if ( !empty($this->html_content[$languageId]) )
			{
				$sql .= "(?, ?, 'text/html', ?), ";
				$values[] = $this->id;
				$values[] = $languageId;
				$values[] = $this->html_content[$languageId];
			}
			if ( !empty($this->subject[$languageId]) || !empty($this->tiny_content[$languageId]) )
			{
				$langsql .= "(?, ?, ?, ?), ";
				$langvalues[] = $this->id;
				$langvalues[] = $languageId;
				$langvalues[] = empty($this->subject[$languageId])?null:$this->subject[$languageId];
				$langvalues[] = empty($this->tiny_content[$languageId])?null:$this->tiny_content[$languageId];

			}

			if ( empty($values) )
				return $this->id;
		}
		//replace content
		foreach ( array('', 'lang') as $pfix )
		{
			$sql = substr(${$pfix.'sql'}, 0, -2);
			$this->lastQuery = $this->db->query($sql);
			$this->lastQuery->bind_values(${$pfix.'values'});
			if ( !$this->lastQuery->execute(false) )
				return false;
		}
		return true;
	}

	/**
	 * Simple parsing of html
	 * Changes Entities, strips tags and does some nl converting (<br>, <p>, <table>, <tr>)
	 *
	 * We could use lynx -stdin -dump for this....
	 *
	 * @param string $html
	 * @return string
	 */
	static function html2txt($html)
	{

		/*
		$html = escapeshellarg($html);
		return `echo $html|links -dump`;
		return `echo $html|lynks -stdin -dump`;
		*/
		//fix links
		$html = preg_replace('/<a[^>]*href="([^"]*)"[^>]*>([^<]*)<\/a>/i', '\\2 ( \\1 )', $html);
		return strip_tags(preg_replace(array('/<br\s*\/?\s*>/i',
									'/<\/?p[^<>]*>/i',
									'/<\/tr>/i',
									'/<\/h\d>/i',
									'/<\/?pre>/i',
									'/<\/div>/i',
									'/<\/?table>/i'
									), "\n",
								str_replace(array("\n", "\r"), '',
									html_entity_decode($html)
								)//str
							)//preg
					);
	}

	function truncate76($txt)
	{
		while(preg_match('/^.{73,}$/m',$txt))
			$txt  = preg_replace("/^(.{0,72})[ \n]/m","\\1\n", $txt);
		return $txt;
	}
	function _initObjectMaps()
	{
		$this->objectmap['dbtable'] = 'TBL_MESSAGE';
		$this->objectmap['primary']['id'] = 'MESSAGE_ID';
		$this->objectmap['properties']['company_id'] = 'COMPANY_ID';
		$this->objectmap['properties']['handle'] = 'MESSAGE_HANDLE';
		$this->objectmap['properties']['default_language_id'] = 'DEFAULT_LANGUAGE_ID';
	}

	/**
	 * @param int
	 */
	function _checkLanguage($languageId)
	{
		foreach ($this->langprops as $arr)
			if ( !isset($this->{$arr}[$languageId]) )
				$this->{$arr}[$languageId] = isset($this->{$arr}[$this->default_language_id])
					?$this->{$arr}[$this->default_language_id]
					:'';

		return $languageId;
	}
	/**
	 * Get all set languages;
	 * @return array array(LANGUAGE_ID, LANGUAGE_ID, ...);
	 */
	function _getLanguages()
	{
		$rv = array();
		foreach ($this->langprops as $arr)
			$rv = array_merge($rv, array_keys($this->$arr));
		return array_unique($rv);
	}

	/**
	 * @var array
	 */
	var $langprops = array('content', 'html_content', 'tiny_content', 'subject');
	/**
	 * @var Mail_mime
	 */
	var $mime = false;
	/**
	 * @var array ( 'filepath' => 'mime_type' )
	 */
	var $attachments = array();
	/**
	 * @var array
	 */
	var $headers = array();
	/*#@-*/
}
?>
