<?php
/**
 * TableFactory
 * @author Chris Wesseling <chris.wesseling@xs4all.nl>
 * @version $Id: table_factory.php 456 2005-07-01 13:02:47Z martijn van den broek $
 * @package I-Ware
 * @subpackage Core
 * @todo refactor getXHTMLTable, getRotatedXHTMLTable and getDimmPDAJSTable... Some stuff doesn't work on the latter 2.
 */
class TableFactory
{
	/**
	 * @var boolean Don't show header see {@link decapitate()}
	 */
	var $noHeader = false;
	/**
	 * @var boolean Directly print instead of returning a string. This can make large amounts of data more responsive to the end-user.
	 */
	var $hot = false;
	/**
	 * @var int How many tabs to indent output. To make HTML humanoid-readable.
	 */
	var $indentlevel = 0;
	/**
	 * @var int {@link getDimmPDAJSTable()} skips this much rows from dataset.
	 */
	var $skipPDA = 0;
	/**
	 * @var string This is added in the table tag. Known to be used for other things then width.
	 */
	var $width = " width=\"100%\"";
	/**#@+
	 * @access private
	 */
	/**
	 * @var string Text that is shown if dataset is empty.
	 */
	var $_emptyText = '';
	/**
	 * @var array Dataset. This id filled by {@link setData()}
	 */
	var $ds;
	/**
	 * @var Language_Interface The header translator.
	 */
	var $langIf;
	/**
	 * @var array head=>mixed Processor See {@link setValueProcessor}
	 */
	var $valproc;
	/**
	 * @var array head=>(string) Processor See {@link setCellHeaderProcessor}
	 */
	var $cheadproc;
	/**
	 * @var array (string)cellheaders if the cheadproc retval contains one of these, the td gets the extra 'selected' class.
	 */
	var $highlitHeaders = array();
	/**
	 * @var string See {@link setCaption}
	 */
	var $caption;
	/**
	 * @var string See {@link setCaptionAttributeString}
	 */
	var $captionAttr = '';
	/**
	 * @var boolean See {@link setResizeColumns}
	 */
	var $resizableCols;
	/**
	 * @var array See {@link setSortingFunctions}
	 */
	var $sortingFuncs = array();
	/**
	 * @var array heads that shouldn't end up in the output
	 */
	var $hiddenCols = array();
	/**
	 * @var array head=>(mixed) Processor. Works in conjunction with $evalCols. (Return)values get added inside the td-tag
	 */
	var $modifyCols = array();
	/**
	 * @var boolean Should the modifyCols be eval'ed?
	 */
	var $evalCols = false;
	/**
	 * @var array head=>(mixed) Processor. Returnvalues get added inside the th-tag
	 */
	var $thproc = array();
	/**
	 * @var array [(obj)Object, (string)functionname]  Gets called for each row.
	 */
	var $rowproc = array(); // contains rowproc callback
	/**
	 * @var bool Is dataset columnized?
	 */
	var $columnized = false;
	var $footproc = false;
	/**#@-*/

	/**
	 * Constructor
	 * @param mixed $dataset 2d array or {@link Query} object
	 * @param Language_Interface $langIf If set used to translate Headers and Captions.
	 */
	function TableFactory($dataset='', $langIf='')
	{
		$this->setData($dataset);
		$langIf && $this->setHeaderTranslator($langIf);
	}

	function getFilteredDS()
	{
		$rv = array();
		$visibleColumnCount = $i = 0;
		$i = 1;
		$footer = '';
		$th = true;

		$colindex =& $i;
		foreach($this->ds as $row)
		{
			$j = 0;
			$rowindex =& $j;
			foreach($row as $orighead => $value)
			{
				$head = $this->_decolumnize($orighead);
				if (is_array($this->valproc) && (isset($this->valproc[$head]) || array_key_exists(TF_ALL, $this->valproc))) //execute ValueProcessor
				{
					$proc = (isset($this->valproc[$head])) ? $head : TF_ALL;

					if (is_array($this->valproc[$proc]))
					{
						$obj = &$this->valproc[$proc][0]; // need a reference so cant use list()
						$func = $this->valproc[$proc][1];

						if (method_exists($obj, $func))
						{
							$args = array();
							$args['i'] = $i;
							$args['j'] = $j;
							$args['value'] = $value;
							$args['row'] = &$row;
							$args['head'] = $head;
							$args['extra'] = '';
							$args['visibleColumnCount'] = $visibleColumnCount;
							$args['visibleColumns'] = $this->getVisibleColumns();
							$value = $obj->{$func}($args);
						}
						else
							trigger_error('Error executing valueprocessor callback ['.$func.']', E_USER_ERROR);
					}
					else
						$value = eval($this->valproc[$proc]);
				}
				if ( !in_array($head, $this->hiddenCols) )
				{
					if($th)
						$rv[0][$orighead] = $orighead;
					$rv[$i][$orighead] = $value;
					$j++;
				}
			}
			$th = false;
			$i++;
		}
		return $rv;
	}

	/**
	 * Create a XHTML table
	 *
	 * NB Returns just the caption and footer if the dataset is empty.. This is wanted behaviour and should not be changed!
	 * <code>
	 * $factory = new TableFactory(array(array('head1'=>'row1value1', 'head2'=>'row1value2'), array('head1'=>'row2value1', 'head2'=>'row2value2')));
	 * echo $factory->getXHTMLTable(array('td'=>'antwoordform', 'thead'=>'kopform'));
	 * </code>
	 * @param mixed (array) $classes XHTML-tag => classname OR (string) table class
	 * @param int $modulus If set The row/colnumber % modulus is appended to tdclass (css) row/col depends on direction
	 * @param string $direction 'row'|'col' (if row the mod is appended to the trclass too)
	 * @param boolean $splitResult if true return an array, else return string
	 * @return mixed (string) XHTML table with thead, tfoot and tbody (if any) OR (array) [(string) table, (string) javascript]
	 */
	function getXHTMLTable($classes=array('td'=>'antwoordform', 'thead'=>'kopform', 'th'=>'kopform', 'caption'=>'kopform'), $modulus='', $direction='row',$splitResult=false)
	{
		//smarter css
		$visibleColumnCount = 0;
		$footer = $hatscript = $jsscript = '';
		if ( is_string($classes) )
			$classes = array('table'=>$classes, 'tr'=>'tr');
		//Ensure id if needed.
		if ( !empty($this->sortingFuncs) || $this->resizableCols)
		{
			empty($this->id) && $this->id = uniqid('t');
			if(!$splitResult)
				$jsscript = '<script type="text/javascript"><!-- ';
		}

		if ( $this->resizableCols )
			$jsscript .= "\nColumnResizer.prototype.init(document.getElementById('$this->id'));";

		if ( !empty($this->sortingFuncs) )
			$hatscript = "\n".'hat'.$this->id.' = new SortingHat(\''.$this->id.'\', {';


		$indents = str_repeat("\t", $this->indentlevel);
		$rv = $indents.'<table '.$this->width.' '.(isset($this->id)?' id="'.$this->id.'"':'').(isset($classes['table'])?' class="'.$classes['table'].'"':'').">\n";

		if (sizeof($this->ds) == 0 && !empty($this->_emptyText))
			$rv .= '<tr><td>'.$this->_emptyText.'</td></tr>';

		$th = $tb = null;
		$i = 0;
		$colindex =& $i;
		isset($this->caption) && $rv .= "$indents\t<caption".$this->captionAttr.(isset($classes['caption'])?' class="'.$classes['caption'].'"':'').'>'.$this->caption."</caption>\n";
		foreach ( $this->ds as $row)
		{
			if(!is_array($row))
				continue;
			$j = 0;
			$rowindex =& $j;

			/**
			 * Execute row processor
			 */
			$rowargs = array();
			$rowargs['attr'] = array();
			if(isset($classes['tr']))
				$rowargs['attr']['class'] = $classes['tr'].((is_numeric($modulus)&& $direction=='row')?$i%$modulus:'');
			if ($this->rowproc)
			{
				$obj = &$this->rowproc[0]; // need a reference so cant use list()
				$func = $this->rowproc[1];

				if (method_exists($obj, $func))
				{
					$rowargs['row'] = &$row;
					// if callback returns false lets skip this iteration
					if ($obj->{$func}($rowargs) === false)
						continue;
				}
				else
					trigger_error('Error executing thead processor callback', E_USER_ERROR);
			}

			foreach ( $row as $orighead=>$value )
			{
				$head = $this->_decolumnize($orighead);
				$onmouseover = $onclick = $cellheaders = $extra = "";
				if ( !isset($hdone) && !in_array($head, $this->hiddenCols) )
				{
					$thExtra = $headerText = '';
					if ( array_key_exists($orighead, $this->sortingFuncs) )
					{
						$onclick = ' onclick="hat'.$this->id.'.sort('.$j.',this, \''.$orighead.'\');"';
						$hatscript .= $j.':'.$this->sortingFuncs[$orighead].',';
						$onmouseover = 'onmouseover="this.style.cursor=\'Pointer\';"';
						$thExtra .= ' style="text-decoration: underline;"';
					}
					// execute th processor

					if (is_array($this->thproc) && (isset($this->thproc[$head]) || array_key_exists(TF_ALL, $this->thproc)))
					{
						$proc = (isset($this->thproc[$head])) ? $head : TF_ALL;

						if (is_array($this->thproc[$proc]))
						{
							$obj = &$this->thproc[$proc][0]; // need a reference so cant use list()
							$func = $this->thproc[$proc][1];

							if (method_exists($obj, $func))
							{
								$args = array();
								$args['head'] = &$head;
								$args['i'] = $i;
								$args['j'] = $j;
								$args['headerText'] = &$headerText;
								$thExtra .= $obj->{$func}($args);
							}
							else
								trigger_error('Error executing thead processor callback', E_USER_ERROR);
						}
						else if (isset($this->thproc[$head]))
							$thExtra .= eval($this->thproc[$head]);
					}
					$visibleColumnCount++;
					$th .= "$indents\t\t\t<th ".$thExtra.' '.(isset($classes['th'])?' class="'.$classes['th'].'"':'').$onclick.' '.$onmouseover.'>'.$this->_getHeaderText($head,$headerText)."</th>\n";
				}
				$tdclass = (isset($classes['td'])?$classes['td']:'');
				if ( is_numeric($modulus) )
				{
					if ( $direction == 'row' )
						$tdclass && $tdclass .= $i%$modulus;
					else
						$tdclass && $tdclass .= $j%$modulus;
				}

				if (is_array($this->valproc) && (isset($this->valproc[$head]) || array_key_exists(TF_ALL, $this->valproc))) //execute ValueProcessor
				{
					$proc = (isset($this->valproc[$head])) ? $head : TF_ALL;

					if (is_array($this->valproc[$proc]))
					{
						$obj = &$this->valproc[$proc][0]; // need a reference so cant use list()
						$func = $this->valproc[$proc][1];

						if (method_exists($obj, $func))
						{
							$args = array();
							$args['i'] = $i;
							$args['j'] = $j;
							$args['value'] = $value;
							$args['row'] = &$row;
							$args['head'] = $head;
							$args['extra'] = &$extra;
							$args['visibleColumnCount'] = $visibleColumnCount;
							$args['visibleColumns'] = $this->getVisibleColumns();
                                                        isset($previousrow) && $args['previousrow'] =& $previousrow;
                                                        isset($orighead) && $args['orighead'] =& $orighead;
                                                        isset($tdclass) && $args['tdclass'] =& $tdclass;

							$value = $obj->{$func}($args);
						}
						else
							trigger_error('Error executing valueprocessor callback ['.$func.']', E_USER_ERROR);
					}
					else
						$value = eval($this->valproc[$proc]);
				}

				if ( isset($this->cheadproc[$head]) )
				{
					$cellheaders = ' headers="'.eval($this->cheadproc[$head]).'"';
					if ( array_intersect($this->highlitHeaders, explode(' ', eval($this->cheadproc[$head]))) )
						$tdclass .= ' selected';
				}
				if( array_key_exists( $head, $this->modifyCols )&& $this->evalCols)
					$extra .= eval( $this->modifyCols[$head] );
				elseif( array_key_exists( $head, $this->modifyCols ))
					$extra .= $this->modifyCols[$head];
				if( isset( $this->colidproc[$head] ))
					$extra .= ' id="'.eval($this->colidproc[$head]).'"';

				if ( !in_array($head, $this->hiddenCols) )
				{

					if (is_array($value))
						$value = print_r($value, true);

					$tb .= "$indents\t\t\t<td ".(!empty($tdclass)?'class="'.$tdclass.'"':'').$extra.$cellheaders.">$value</td>\n";
					$j++;
				}
			}

			if ( isset($th) )
			{
				if ( !$this->noHeader )
					$rv .= "$indents\t<thead".(isset($classes['thead'])?' class="'.$classes['thead'].'"':'').">\n\t\t<tr>\n$th\t\t</tr>\n\t</thead>\n";
				$rv .= "$indents\t<tbody>\n";
				unset($th);
				$hdone = true;
			}
			if(!empty($this->ds[0]))
			{
				$attributes = '';
				foreach($rowargs['attr'] as $attr=>$value)
					$attributes .= sprintf(' %s="%s"',$attr,$value);
				$rv .= "$indents\t\t<tr".$attributes.">\n$tb\t\t</tr>\n";
			}
			$tb = null;
			$previousrow = $row;
			$i++;
			if ( $this->hot )
			{
				echo $rv;//no flush().. Flush has a bad effect on Maximum possible server load.
				$rv = '';
			}
		}
		if(is_array($this->footproc))
		{
			$obj = &$this->footproc[0];
			$func = $this->footproc[1];

			$this->footer = $obj->$func();
		}

		if ( isset($this->footer) )
		{
			$footer = "$indents\t<tfoot".(isset($classes['tfoot'])?' class="'.$classes['tfoot'].'"':'').">\n\t\t";
			if ( is_array($this->footer) )
			{
				$footerrows = $this->_multiLnFooter?$this->footer:array($this->footer);
				while ( list(,$footerrow) = each($footerrows) )
				{
					$footer .= '<tr>';
					foreach ( $footerrow as $value )
					{
						if ( is_array($value) )
							$value = eval($value[0]);
						$footer .= "$indents\t\t\t<td ".(isset($classes['tf'])?' class="'.$classes['tf'].'"':'').'>'.$value."</td>\n";
					}
					$footer .= '</tr>';
				}
			}
			else
				$footer .= "$indents\t\t\t".'<td colspan="'.$visibleColumnCount.'" '.(isset($classes['tf'])?' class="'.$classes['tf'].'"':'').'>'.$this->footer."</td>\n";

			$footer .= "$indents\t\t</tr>\n</tfoot>\n";
		}
		$rv .= (!empty($this->ds[0])?"$indents\t</tbody>\n":'').$footer."$indents</table>\n";
		if ( $hatscript && ($hatscript[strlen($hatscript)-1] == ',') )
			$hatscript = substr($hatscript, 0, -1);
		if ( $jsscript && !$splitResult )
			$rv .= $jsscript.$hatscript.'});//--></script>';
		elseif( $splitResult )
			return array($rv,$jsscript.$hatscript.'});');
		return $rv;
	}
	/**
	 * Renders TH on the left followed by the TD with the value.
	 * Be careful some of the cooler features don't work here.
	 * Regular oldskool eval'ed valueprocessors should do just fine, though.
	 * @param array $classes [(string)XHTMLTag=>(string)Class]
	 * @param int $rowspercol Number of rows in the tables
	 * @return string XHTML
	 */
	function getRotatedXHTMLTable( $classes = array('table'=>'headertable', 'th'=>'headertdlabel', 'td'=>'headertd'), $rowspercol = 2 )
	{
		global $langIf;
		$cnt = 0;
		$currentColIndex = -1;

		$columns = array();
		$processCols = array();

		foreach( $this->ds[0] as $label => $data )
		{
			if( $cnt % $rowspercol == 0 )
				$currentColIndex++;
			if( !in_array($label, $this->hiddenCols ))
			{
				$columns[$currentColIndex][] = $label;
				$cnt++;
			}
			else
				$processCols[] = $label;
		}
		//set classes
		$headertable = (isset($classes['table'])?$classes['table']:'');
		$headertdlabel = (isset($classes['th'])?$classes['th']:'');
		$headertd = (isset($classes['td'])?$classes['td']:'');

		$output = "<table ".$this->width." class=\"$headertable\" cellspacing=\"0\" cellpadding=\"2\">";
		foreach( $this->ds as $row )
		{
			for( $i = 0; $i < $rowspercol; $i++ )
			{
				$output .= "<tr>";
				foreach( $columns as $index => $labels )
				{
					if( isset( $labels[$i] ))
					{
						$value = $row[$labels[$i]];
						$value = (isset($this->valproc[$labels[$i]])?eval($this->valproc[$labels[$i]]):$row[$labels[$i]]);
						$output .= "<th width=\"12%\" nowrap=\"nowrap\"	class=\"$headertdlabel\">".((isset($langIf)) ? $langIf->getText( $labels[$i] ) : $labels[$i]) ."</th><td width=\"13%\" nowrap=\"nowrap\" class=\"$headertd\" ".($index<=(count($columns)-$rowspercol)?"style=\"border-right: 1px solid #999999;\">":'>').$value."</td>";
					}
					else
						$output .= "<td class=\"$headertdlabel\">&nbsp;</td><td class=\"$headertd\">&nbsp;</td>";
				}
				$output .= "</tr>";
			}
			if( is_array( $processCols ))
			{
				foreach($processCols as $label )
				{
					$value = $row[$label];
					$value = (isset($this->valproc[$label])?eval($this->valproc[$label]):$row[$label]);
				}
			}
			$previousrow = $row;
		}
		$output .= "</table>";

		return $output;

//		$output = "<table></table>";
	}
	/**
	 * Simple plaintexttable
	 * Doesn't do heads... Doesn't use value processors... Be my guest...
	 * @param string $r Row end
	 * @param string $c Columns seperator
	 * @return string
	 */
	function getPlainTextTable($r="\n", $c="\t")
	{
		$rv = '';
		foreach ( $this->ds as $row )
		{
			foreach ( $row as $col )
				$rv .= $col.$c;
			$rv .= $r;
		}
		return $rv;
	}
	/**
	 * Returns a script instead of a table. To enable a gain of approx. 75% in datasize.
	 *
	 * Strips numeric part of 'class','value','name' & 'href' properties and all CDATA of rows.
	 * Replaces them with placeholder variables.
	 * Creates a javascript function that document.writes a row.
	 * Creates a function call for each row.
	 *
	 * Skips $this->skipPDA rows.
	 * ie. Set $this->skipPDA = 1, if the first row does not follow the structure of the rest of the table
	 *
	 * May choke on advanced combi's of ValueProcessors and ColumnModifiers (aka as Fatalities ;)
	 *
	 * @param array $classes (tagname=>class, ...)
	 * @param int $modulus ${direction}Count%$modulus is appended to the styles
	 * @param string $direction ('row'|'col')
	 * @return string
	 */
	function getDimmPDAJSTable($classes=array('td'=>'antwoordform', 'thead'=>'kopform', 'th'=>'kopform', 'caption'=>'kopform'), $modulus='', $direction='row')
	{
		// Matches all class|value|name|href properties within tags, that contain a number.
		$regexpPROPS = '/(.*?(?:class|value|name|href)=[\'"])([^0-9"]*\d[^"]*)([\'"])/s';
		// Matches all numbers and prefix
		$regexpNUMBERS = '/(\D*)(\d+)/';
		// Matches all CDATA
		$regexpCDATA = "/([^>]*>)([^<]+)/";
		$rv = '<table '.$this->width.' '.(isset($classes['table'])?' class="'.$classes['table'].'"':'').">\n";
		$th = $tb = null;
		$i = 0;
		isset($this->caption) && $rv .= "<caption".(isset($classes['caption'])?' class="'.$classes['caption'].'"':'').'>'.$this->caption."</caption>\n";
		foreach ( $this->ds as $row)
		{
			$j = 0;
			foreach ( $row as $orighead=>$value )
			{
				$head = $this->_decolumnize($orighead);
				$cellheaders = $extra = "";

				if( array_key_exists( $head, $this->modifyCols )&&!$this->evalCols)
					$extra = $this->modifyCols[$head];
				if ( !isset($hdone) && !in_array($head, $this->hiddenCols) )
					$th .= "<th".(isset($classes['th'])?' class="'.$classes['th'].'"':'').$extra.'>'.$this->_getHeaderText($head)."</th>\n";
				$tdclass = (isset($classes['td'])?$classes['td']:'');
				if ( is_numeric($modulus) )
				{
					if ( $direction == 'row' )
						$tdclass .= $i%$modulus;
					else
						$tdclass .= $j%$modulus;
				}
				$value = (isset($this->valproc[$head])?eval($this->valproc[$head]):$value);

				if ( isset($this->cheadproc[$head]) )
				{
					$cellheaders = ' headers="'.eval($this->cheadproc[$head]).'"';
					if ( array_intersect($this->highlitHeaders, explode(' ', eval($this->cheadproc[$head]))) )
						$tdclass .= ' selected';
				}
				if( array_key_exists( $head, $this->modifyCols )&& $this->evalCols)
					$extra = eval( $this->modifyCols[$head] );
				elseif( array_key_exists( $head, $this->modifyCols ))
					$extra = $this->modifyCols[$head];
				if( isset( $this->colidproc[$head] ))
					$extra .= ' id="'.eval($this->colidproc[$head]).'"';

				if ( !in_array($head, $this->hiddenCols) )
					$tb .= "<td ".(!empty($tdclass)?'class="'.$tdclass.'"':'').$extra.$cellheaders.">$value</td>\n";
				$j++;
			}

			if ( isset($th) )
			{
				if ( !$this->noHeader )
					$rv .= "<thead".(isset($classes['thead'])?' class="'.$classes['thead'].'"':'').">\n\t\t<tr>\n$th\t\t</tr>\n\t</thead>\n";
				if ( isset($this->footer) )
				{
					$rv .= "<tfoot".(isset($classes['tfoot'])?' class="'.$classes['tfoot'].'"':'').">\n\t\t<tr>";
					if ( is_array($this->footer) )
						foreach ( $this->footer as $value )
						{
							if ( is_array($value) )
								$value = eval($value[0]);
							$rv .= "<td ".(isset($classes['tf'])?' class="'.$classes['tf'].'"':'').'>'.$value."</td>\n";
						}
					else
						$rv .= '<td colspan="'.count($row).'" '.(isset($classes['tf'])?' class="'.$classes['tf'].'"':'').'>'.$this->footer."</td>\n";

					$rv .= "</tr>\n</tfoot>\n";
				}
				$rv .= "<tbody>\n";
				unset($th);
				$hdone = true;
			}

			do
			{
				if ( $i < $this->skipPDA )
				{
					$rv .= "<tr".(isset($classes['tr'])?' class="'.$classes['tr'].((is_numeric($modulus)&& $direction=='row')?$i%$modulus:'').'"':'').">\n$tb\t\t</tr>\n";
					if( count( $this->ds ) == ($i-1) )
						$closeTable = $rv .= '</tbody></table>';
					break;
				}

				// Org == <tr.* stripped of tabs and newlines
				$org = preg_replace('/\t?\n?\r?/', '', "<tr".(isset($classes['tr'])?' class="'.$classes['tr'].((is_numeric($modulus)&& $direction=='row')?$i%$modulus:'').'"':'').">\n$tb\t\t</tr>\n");
				if ( ! isset($template) ) // Build the template javascript function.
				{
					// Matches the properties.
					preg_match_all($regexpPROPS, $org, $matches);
					$cI = $pI = -1;
					$template = 'var t = \\\'';
					foreach ( $matches[1] as $index => $value ) // Loop all the props.
					{
						if ( preg_match_all($regexpCDATA, $value, $lighters) ) // Look for CDATA
							foreach ( $lighters[0] as $li => $lv )
								$template .= str_replace($lv, $lighters[1][$li].'\\\'+c['.++$cI.']+\\\'', $value); // Replace the fullmatch with prefix plus variable part
						else
							$template .= $value; // No CDATA
						preg_match_all($regexpNUMBERS, $matches[2][$index], $flinstones); // Check for numbers
						$playdoh = $matches[2][$index];
						foreach ( $flinstones[0] as $li => $lv )
							$playdoh = str_replace($lv, $flinstones[1][$li].'\\\'+p['.++$pI.']+\\\'', $playdoh); // Replace the fullmatch prefix plus variable part
						$template .= $playdoh.$matches[3][$index];
					}
					$value = str_replace(implode('',$matches[0]), '', $org); // Strip full row of all props that have been matched before
					if ( preg_match_all($regexpCDATA, $value, $lighters) ) // Look for CDATA in last part of the row
						foreach ( $lighters[0] as $li => $lv )
							$template .= str_replace($lv, $lighters[1][$li].'\\\'+c['.++$cI.']+\\\'', $value); // Replace the fullmatch with prefix plus variable part
					else
						$template .= $value; // No CDATA

					$template .= '\\\';';

					$template = preg_replace('/[\\\]{2}/', '', preg_replace('/\'/', '\\\'', $template));
					$rv = "<script type=\"text/javascript\">\nvar tS = false;\nfunction gT(c, p)\n{\n
								if ( !tS ) { document.write('".str_replace(array("\t","\n","\r","'"), array_pad(array("\'"), -4, ''), $rv)."');\n tS=true;}\n";
					$rv .= $template;
					$rv .= "document.write(t);
					}\n";
				}
				// Build the function calls
				preg_match_all($regexpPROPS, $org, $p);
				preg_match_all($regexpCDATA, $org, $c);
				$props = '';
				foreach( $p[0] as $index => $value )
				{
					preg_match_all($regexpNUMBERS, $p[2][$index], $flinstones);
					$props .= '\''.implode('\', \'',$flinstones[2]).'\',';
				}
				$props = substr($props, 0, -1);

				$rv .= 'gT(new Array(\''.implode('\',\'', $c[2]).'\'), new Array('.$props.'));'."\n";
			} while (false);

			$tb = null;
			$previousrow = $row;
			$i++;
			if ( $this->hot )
			{
				echo $rv;//no flush().. Flush has a bad effect on Maximum possible server load.
				$rv = '';
			}
		}
		return $rv.(!isset($closeTable)?((!empty($this->ds)?"document.write('</tbody>');\n":'')."document.write('</table>');\n</script>"):"");
	}

	/**
	 * set noHeader to true ;)
	 */
	function decapitate()
	{
		$this->noHeader = true;
	}
	/**
	 * Set Data
	 *
	 * Sets new Data. Unsets caption and footer, but leaves Language_Interface and Processors intact.
	 * @param mixed $dataset 2d array or {@link Query} object
	 * @param bool $show_empty Show emty head on a empty resultset.
	 * @todo strtolower(substr(get_class($dataset), -5)) == 'query' is a hack, should be is_a($dataset, 'query') but needs newer PHP
	 */
	function setData($dataset, $show_empty=false)
	{
		$this->id = null;
		$this->unsetCaption();
		unset($this->footer);
		$this->noHeader = false;
		$this->captionAttr = '';
		$this->sortingFuncs = $this->ds = array();
		if ( is_array($dataset) )
			$this->ds = $dataset;
		elseif (is_object($dataset) && strtolower(substr(get_class($dataset), -5)) == 'query' )
		{
			while ( $row = $dataset->fetch() )
				$this->ds[] = (array) $row;
			if ( empty($this->ds) && $show_empty && strtolower(get_class($dataset)) == 'mysql_query' )
				while ( $field = mysql_fetch_field($dataset->result) )
					$this->ds[0][$field->name] = '';
		}
		else
			$this->ds = (array) $dataset;

		$this->columnized = false;
		$this->resizableCols = false;
	}
	/**
	 *
	 */
	function getVisibleColumns()
	{
		if(!isset($this->ds[0]))
			return false;
		$cols = $this->ds[0];
		foreach($this->hiddenCols as $colname)
			unset($cols[$colname]);
		if(is_array($cols))
			return array_keys($cols);
		return false;
	}

	/**
	 * @param Language_Interface $langIf
	 */
	function setHeaderTranslator(&$langIf)
	{
		$this->langIf = &$langIf;
	}
	/**
	 * Add some code to be executed on a Column.
	 * <code>
	 * $tf->setValueProcessor('timestamp', 'return strftime(\'%x\', $value);')
	 * $tf->setValueProcessor('NAME', 'if ( isset($previousrow) && $value == $previousrow[$orighead] ){ $tdclass .= \'-collapse\'; return \'\';}else return $value;'); // but this is shortcutted in $tablefactory->collapseColumns('NAME');
	 * </code>
	 * $code can be an array consisting of an object and a method name.
	 * The method should return the TD CData and should take one parameter 'args' which is filled accordingly:
	 * $args['i'] = $i;  The numeric row index
	 * $args['j'] = $j; The numeric column index
	 * $args['value'] = $value; The original cellvalue in the dataset
	 * $args['row'] = &$row; A reference to the current row
	 * $args['head'] = $head; The orinal columnname (NB $row[$head] could be != $value, if columnized!!)
	 * $args['extra'] = &$extra; The string added in the td-tag
	 *
	 * @param mixed $head string Headname | array Headnames
	 * @param mixed string $code OR (array)[(object)Object, (string)Method] Code gets eval'ed ($tdclass = Class of the cel, $value = the Celvalue, $orighead = realColumnHeadname $previousrow = Duh...)
	 */
	function setValueProcessor($head, $code)
	{
		$heads = (array) $head;
		foreach ( $heads as $head )
			$this->valproc[$head] = $code;
	}

	function setFooterProcessor($code,$multiline=false)
	{
		$this->footproc = $code;
		$this->_multiLnFooter = $multiline;
	}

	/**
	 * If no $head is given, unsets all ValueProcessors.
	 * @param mixed $head string Headname | array Headnames
	 */
	function unsetValueProcessor($head='')
	{
		if ( $head )
		{
			$heads = (array) $head;
			foreach ( $heads as $head )
				unset($this->valproc[$head]);
		}
		else
			unset($this->valproc);
	}
	/**
	 *
	 */
	function setTHProcessor($head, $code)
	{
		$heads = (array) $head;
		foreach ( $heads as $head ) {
			$this->thproc[$head] = $code;
		}
	}
	/**
	 * Add some code which returns the string for the (td headers="..")
	 * @param mixed $head string Headname | array Headnames
	 * @param string $code Code that gets eval'ed ($tdclass = Class of the cel, $value = the Celvalue, $previousrow = Duh...)
	 */
	function setCellHeaderProcessor($head, $code)
	{
		$heads = (array) $head;
		foreach ( $heads as $head )
			$this->cheadproc[$head] = $code;
	}

	/**
	 * Add some ID to cell.
	 * $code should return the id for the td.
	 * @param mixed $head string Headname | array Headnames
	 * @param string $code Code that gets eval'ed ($tdclass = Class of the cel, $value = the Celvalue, $previousrow = Duh...)
	 */
	function setCellIdProcessor($head, $code)
	{
		$heads = (array) $head;
		foreach ( $heads as $head )
			$this->colidproc[$head] = $code;
	}

	/**
	 * If no $head is given, unsets all CellHeaderProcessors.
	 * @param mixed $head string Headname | array Headnames
	 */
	function unsetCellHeaderProcessor($head='')
	{
		if ( $head )
		{
			$heads = (array) $head;
			foreach ( $heads as $head )
				unset($this->cheadproc[$head]);
		}
		else
			unset($this->cheadproc);
	}

	/**
	 * Executes given callback for all rows.
	 * @param array $callback [(object) object, (string) method]
	 */
	function setRowProcessor($callback)
	{
		$this->rowproc = $callback;
	}

	/**
	 * @param string $caption
	 * @param bool $translate Run thru Language_Interface
	 * @param array $args Array with caption properties, can enable help and or start wizard from here.
	 */
	function setCaption($caption, $translate=true, $args=null)
	{
		$caption = (($translate && isset($this->langIf))?$this->langIf->getText($caption):$caption);

		$rspan = '';
		if (is_array($args))
		{
			if (isset($args['wizard']))
				$rspan .= '&nbsp;<a href="'.$args['wizard'].'"><img src="'.REL_IMAGES.'i-w/TableFactory/caption_wizard.gif" style="border: 0;" title="Start Wizard" /></a>';

			if (isset($args['help']))
			{
				$handle = (empty($args['help'])) ? 'pageHelp' : $args['help'];
				$tr = '';
				if (isset($this->langIf)) {
					$langIf = $this->langIf;
					$tr = $this->langIf->getText($handle);
				}
				else {
					global $langIf;
					$tr = $langIf->getText($handle);
				}

				if (!$tr == '') {
					$rspan .= '&nbsp;<img id="img_'.$handle.'" src="'.REL_IMAGES.'i-w/TableFactory/caption_help.gif" title="Context Help" />';
					$rspan .= "<script type=\"text/javascript\"> if (typeof help != 'undefined')
					           	help.registerById('img_".$handle."', '".$handle."');
					           </script>";
				}
			}
		}

		if ($rspan != '')
		{
			$html  = '<span style="float: left;">'.$caption.'</span>';
			$html .= '<span style="float: right;">'.$rspan.'</span>';
		}
		else
			$html = $caption;

		$this->caption = $html;


	}
	/**
	 * If $foot is an array and the value in the array is an array to, the first value of that array will be eval'ed.
	 *
	 * This will render a 2 cell footer Total: TOTAL value from the *first* row in the body.
	 * (thead and tfoot are generated during the first row)
	 * <code>
	 * $tablefactory->setFooter(array('Total:', array('return $row['TOTAL';'));
	 * </code>
	 * You can add valueProcessors to set your own $variables in the TableFactory
	 * scope and create your own aggregate functions on the body cells.
	 *
	 * @param mixed $foot [string|array] If string footer gets a colspan over the width.
	 * @param bool $multiline $foot is multi dimensional like dataset.. (3rd dimension array of $foot triggers eval!)
	 */
	function setFooter($foot, $multiline=false)
	{
		$this->footer = $foot;
		$this->_multiLnFooter = $multiline;
	}
	/**
	 * @param string $caption
	 */
	function unsetCaption()
	{
		unset($this->caption);
	}
	/**
	 * Adds a string in the caption tag.
	 * @param string $str Like 'onclick="alert(\'clickey\');"'
	 */
	function setCaptionAttributeString($str)
	{
		$this->captionAttr = $str;
	}
	/**
	 * Don't show Columns from dataset in output.
	 * Prevents the mentioned columns from ending up in the table.
	 * @param mixed Columnnames
	 */
	function hideColumns($cols=array())
	{
		$cols = (array) $cols;
		$this->hiddenCols = $cols;
	}
	/**
	 * Add a single Column to the already hidden columns.
	 * @param string $colname
	 */
	function hideColumn($colname)
	{
		$this->hiddenCols = array_unique($this->hiddenCols + (array) $colname);
	}
	/**
	 * Remove a Column from the hidden colmnlist.
	 * @param string $colname
	 */
	function unhideColumn($colname)
	{
		$this->hiddenCols = array_diff($this->hiddenCols, (array) $colname);
	}
	/**
	 * Shortcut for a commonly used ValueProcessor.
	 * If the cell above this cell contains exactly the same value, blank this cell and add '-collapse' to the class.
	 * Used heavily in Interclean amongst other places to keep a clean table free from obvious redundancy.
	 * @param mixed string Columname | array of string Columnnames
	 */
	function collapseColumns($cols='')
	{
		$cols = (array) $cols;
		foreach ( $cols as $col	)
			$this->setValueProcessor($col, 'if ( isset($previousrow) && $value == $previousrow[$orighead] ){ $tdclass && $tdclass .= \'-collapse\'; return \'\';}else return $value;');
	}
	/**
	 * Define which <td headers="..."> should be highlit in advance.
	 * Highlit, means having the 'selected' cssclass added to the cell.
	 * @param array of string CellHeaderNames
	 */
	function highlightCellHeaders($headers='')
	{
		$this->highlitHeaders = (array) $headers;
	}
	/**
	 * Modifies the columns.
	 * Could be a string that is added in each td-tag in the column.
	 * Could be evalcode, of which the retval is added in the td-tag.
	 * @param array $cols Columnnames => modifier
	 * @param boolean $eval
	 */
	function columnModifier($cols=array(), $eval = false)
	{
		$this->modifyCols = $cols;
		$this->evalCols = $eval;
	}
	/**
	 * Add Javascript SortingHatFunctions to thead. If you want to sort a locale'ed quantity you can use a valueprocessor
	 * to append a sort_val='123' to the td.
	 * <code>
	 * 	$tf->setSortingFunctions(array('DEB_NUM'=>"'Int'",
	 * 		'Babalaty'=>"function(a, b){ return (Math.rand()*2-1);}")
	 * 		);
	 * </code>
	 * NB setData() resets these.
	 * @param array (string) Columnname => (string) JavaScript function
	 */
	function setSortingFunctions($b)
	{
		$this->sortingFuncs = $b;
	}
	/**
	 * Make columns resizable.
	 * NB setData() resets this to false.
	 * @param bool $b
	 */
	function setResizeColumns($b=true)
	{
		$this->resizableCols = (bool) $b;
	}

	/**
	 * Devides dataset in n columns
	 *
	 * This probably can't be used in conjunction with {@link setCellHeaderProcessor()}.
	 * I you need it... Use the source, Luke!
	 * I think I fixed this.. But haven't used nor tested it.. Else see {@link _decolumnize()}
	 * @param int $n Columncount
	 */
	function columnize($n=4)
	{
		$temp_ds = array(); // intermediate
		$rowc = count($this->ds)/$n; //rows per col
		$rowi = $coli = 0; // init rownum & colnum aaii
		foreach ( $this->ds as $r ) //cycle dataset
		{
			if ( $rowi >= $rowc )
			{
				$coli++;
				$rowi = 0;
			}
			foreach ( $r as $head=>$val ) //cycle row
				$temp_ds[$rowi][$coli.'columnized'.$head] = $val;
			$rowi++;
		}
		$this->ds = $temp_ds;
		$this->columnized = true;
	}

	/**
	 * Sets a text for the table which will be displayed if the table is empty
	 */
	function setEmptyText($str)
	{
		$this->_emptyText = $str;
	}

	/**#@+
	 * @access private
	 */

	/**
	 * @param string $head da 'ead aaii
	 * @return string
	 */
	function _getHeaderText($head,$extra=false)
	{
		if($extra)
			return $extra;
		return (isset($this->langIf)?$this->langIf->getText($head):$head);
	}
	/**
	 * Translate a colmunized header to it's original value.
	 * @param string $head da 'ead aaii
	 * @return string
	 */
	function _decolumnize($head)
	{
		return $this->columnized?preg_replace('/\d+columnized/', '', $head):$head;
	}
	/**#@-*/
}


define('TF_ALL', '__TF_ALL__'); // can be used to apply a valueprocessor to all columns

?>
