<?php
require_once(INTERFACE_OBJ."InterfaceObject.php");
require_once(INTERFACE_OBJ."InterfaceItem.php");
/**
 * interfaceBlock
 *
 * Creates interfaceBlocks from items.
 *
 * @author Marten Koedam <marten@basesoft.com>
 * @author Chris Wesseling <chris.wesseling@xs4all.nl>
 * @copyright (2005) BaseSoft B.V.
 * @since 13-05-2005
 * @version $Id$
 * @package I-Ware
 * @subpackage Core
 */
class InterfaceBlock extends InterfaceObject
{
	/**
	 * Add your new block types here.
	 * @return array ( 'BLOCK_TYPE', # )
	 */
	function getTypes()
	{
		return array('IF_BLOCK_DEFAULT'=>1,
			   'IF_BLOCK_ICONMENU'=>2,
			   'IF_BLOCK_TOGGLE'=>3,
			   'IF_BLOCK_NO_TITLE'=>4
			   );
	}
	/**
	 * @var string $itemClass
	 */
	var $itemClass = 'interfaceItem';
	/**
	 * @var string $title Translated (or not) $title
	 */
	var $title = '';
	/**
	 * @var string $handle Used for Help
	 */
	var $handle = '';
	/**
	 * @var int $type_id
	 */
	var $type_id = '';
	/**
	 * @var int $postition
	 */
	var $position = '';
	/**
	 * @var int $access
	 */
	var $access = 15;
	/**
	 * @var Array $items (ITEM_ID=>(HANDLE,ITEM_CONTENT=>())
	 */
	var $items = array();
	/**
	 * Constructor
	 * @param Database $db IWARE database
	 * @param Database $db_app DB_EDIT database
	 * @param LanguageInterface $langIf
	 * @param int $id BLOCK_ID
	 * @param array $blockmap
	 */
	function InterfaceBlock(&$db,&$db_app,&$langIf,$id=0,$blockmap=array())
	{
		$this->_defineTypes();
		$this->id = $id;
		parent::InterfaceObject($db, $db_app, $langIf);
		count($blockmap) && $this->processMap($blockmap);
	}
	function load()
	{
		$ifmap = $this->parent->_filter($this->_selectBlock($this->id));
		if ( !isset($ifmap[$this->parent->id][$this->id]['PRIV']) )
			$ifmap[$this->parent->id][$this->id]['PRIV'] = $ifmap[$this->parent->id]['PRIV'];

		$ifmap[$this->parent->id][$this->id]['PARENT'] =& $this->parent;
		$this->processMap($ifmap[$this->parent->id][$this->id]);
	}

	function processMap($map)
	{
		$this->priv = $map['PRIV'];
		$this->type_id = $map['TYPE'];
		$this->setTitle($map['HANDLE']);
		$this->open = !isset($map['CLOSED']);
		$this->parent = &$map['PARENT'];
		$this->if_obj = &$this->parent;

		foreach ( $map as $key=>$val )
		{
			if ( $key && is_numeric($key) )
			{
				$val['PRIV'] = isset($val['PRIV'])?$val['PRIV']:$map['PRIV'];
				$val['PARENT'] = &$this;
				$this->items[$key] = $this->interfaceItem($key, $val);

				/**
				 * save a reference of the IF_ITEM_INTERFACEs so we can call processInput on them
				 */
				if ($this->items[$key]->type_id == IF_ITEM_INTERFACE)
					$this->if_obj->nestedItemInterfaces[] = &$this->items[$key];
			}
		}

		// execute callbacks which came from $settings (see interfacefactory constructor)
		if (isset($this->if_obj->block_callbacks[$this->id])) {
			list ($obj, $func) = $this->if_obj->block_callbacks[$this->id];
			if (method_exists($obj, $func))
				$obj->{$func}($this);
		}
	}
	/**
	 * Sets the title of the Block
	 * @param string $title
	 */
	function setTitle($title)
	{
		$this->handle = $title;
		$this->title = ($title && $this->langIf) ? $this->langIf->getText( $title ) : $title;
	}
	/**
	 * Creates a new instance of an interface item
	 * @param string $title
	 * @param Array $item (ITEM_TYPE,CONTENT=>())
	 */
	function interfaceItem($id, $item)
	{
		if( ! self::$checked && file_exists( INTERFACES.$this->parent->appPath.$this->parent->classPrefix.'.php' ))
			require_once(INTERFACES.$this->parent->appPath.$this->parent->classPrefix.'.php');
		self::$checked = true;
		$className = class_exists($this->parent->classPrefix.'Item')?$this->parent->classPrefix.'Item':'InterfaceItem';
		return new $className($this->db,$this->db_app,$this->langIf,$id, $item);
	}

	public static $checked = false;
	/**
	 * @param array $classes CSS classes
	 * @param bool $splitjs Return 1 string or array of 2 string (html, scripts)
	 * @return mixed $rv
	 */
	function getXHTMLBlock($classes=array(), $splitjs=false)
	{
		return $this->_displayBlock($classes, $splitjs);
	}
	/**
	 * Saves all content of a block
	 * @param string $thrownException
	 * @return bool
	 */
	function saveBlock(&$thrownException)
	{
		if ( !is_numeric($this->position) )
		{
			if ( is_numeric($this->id) && $this->id )
			{
				$field = 'POSITION';
				$key = 'BLOCK';
				$values[] = $this->id;
			}
			else
			{
				$field = 'IFNULL(MAX(POSITION),0)+1';
				$key = 'INTERFACE';
				$values[] = $this->parent_id;
			}

			$sql = "SELECT $field as pos
				FROM TBL_INTERFACE_BLOCK
				WHERE ${key}_ID = ?";
			$q = $this->db->query($sql);
			$q->bind_values($values);
			if ( $q->execute() )
			{
				$r = $q->fetch();
				$this->position = $r->pos;
			}
			else
			{
				$thrownException .= 'Failed to determine block position';
				return false;
			}
		}

		$sql = "REPLACE INTO TBL_INTERFACE_BLOCK (INTERFACE_ID, TYPE, POSITION, BLOCK_ID, HANDLE)
			VALUES (?, ?, ?, ?, ?)";
		$q = $this->db->query($sql);
		$q->bind_values(array($this->parent_id, $this->type_id, $this->position, $this->id, $this->handle));
		if ( $q->execute() )
			return (is_numeric($this->id) || ($this->id = $q->lastid())); //== true

		//oops;
		$thrownException .= $q->exception->toString();
		$thrownException .= 'Failed to save block';
		return false;
	}
	/**
	 * Deletes a block
	 * @param string $thrownException
	 * @return bool
	 */
	function deleteBlock(&$thrownException)
	{

	}
	/**
	 *
	 */
	function removeChild($item_id)
	{
		if(!isset($this->items[$item_id]))
			return;
		$this->parent->fb->unregister($this->items[$item_id]->name);
		unset($this->items[$item_id]);
	}
	/**
	 *
	 */
	function hide()
	{
		$this->_hidden = true;
	}

	/**#@+
	 * @access private
	 */
	/**
	 * @var interfaceFactory $if
	 */
	var $if = false;

	/**
	 * @var bool $open State (can be closed)
	 */
	var $open = true;

	/**
	 * @var bool $hidden Display (can be hidden)
	 */
	var $_hidden = false;

	/**
	 * Sets the interfaceFactory variable
	 */
	function _setInterfaceFactory()
	{
		require_once(INTERFACE_OBJ."InterfaceFactory.php");
		$this->if = new InterfaceFactory($this->db,$this->langIf);
	}
	/**
	 * Display a single block with all its items
	 */
	function _displayBlock($classes, $split)
	{
		// SCRIPTS = function declarations for onchange, etc. events
		// POSTSCRIPT = code to be executed after block load.
		$postscript = $script = '';
		$toggle = false;
		$rv = '<a name="'.'a_ifB_'.$this->id.'"></a>';
		//info('classes: '.print_r($classes, true));
		$rv .= $split?'':'<div class="Professional '. (is_array($classes) ? implode(" ",$classes) : $classes) .(($this->_hidden) ? ' hidden' : '').'">';
		switch( $this->type_id )
		{
			case IF_BLOCK_TOGGLE:
				$toggle = true;
				if ( !$split && !$this->open )
				{
					require_once(IWARE.'table_factory.php');

					$tf = new TableFactory();
					$tf->setData(array(array()),true);
					$help = ($this->handle != '') ? getHelp($this->langIf, $this->handle, array('title'=>$this->handle, 'top'=>'0px')) : '';
					$tf->setCaption('<img src="/'.SITE_PATH.'images/block_arrow.png" /> <span style="float: left;">'.$this->title .'</span><span style="float: right;">'.$help.'</span>');
					$tf->setCaptionAttributeString(' onclick="ifHandler.toggleBlock('.$this->id.');" ');
					$tf->id = 'ifB_'.$this->id;
					$rv .= $tf->getXHTMLTable('Professional');
					$script .= "ifHandler.staleBlocks[".$this->id."] = '&interface_id=".$this->parent->id.$this->parent->staleblockparams."';";
					break;
				}
			case IF_BLOCK_NO_TITLE:
			case IF_BLOCK_DEFAULT:
				require_once(IWARE.'table_factory.php');

				$width = isset($this->parent->FirstColWidth)?$this->parent->FirstColWidth:40;
				$remainWidth = 100-$width;

				$tf = new TableFactory();

				$show_label_row = $rf = false; //Row Factory
				$decapitate = true;
				$label_row = $spannedrows = $row = $data = array();
				foreach( $this->items as $item )
				{
					if(isset($item->width))
						$width = $item->width;
					$row = array_merge($row, (array) $item->getXHTMLItem($classes));
					if(($item->label_disposition==DISPOSITION_TOP||$item->label_disposition==DISPOSITION_BOTTOM))
					{
						$show_label_row = ($show_label_row==-1)?-1:$item->label_disposition;
						$label_row[] = $item->title;
					}
					else
						$label_row[] = '<!-- ITEM['.$item->id.'] -->';
					if ( isset($item->script) )
						$script .= str_replace("\n", "\n\t\t", $item->script);
					if ( isset($item->postscript) )
						$postscript .= str_replace("\n", "\n\t\t", $item->postscript);
					if ( $item->close_row )
					{
						if ( $rf )
						{
							$ds = array();
							if($show_label_row===DISPOSITION_TOP)
							{
								$ds[] = array_combine_custom($label_row,$row);
								$decapitate = false;
							}
							else
								$ds[] = $row;
							/**
							 * @todo FIXME (DISPOSITION_BOTTOM)
							 */
							if($show_label_row===DISPOSITION_BOTTOM)
								$ds[] = $label_row;
							$rf->setData($ds);
							$decapitate&& $rf->decapitate();
							$spannedrows[] = count($data);
							if ( $item->type_id == IF_ITEM_ROW_CONTROL )
							{
								if (!$this->_getHideInput())
									$rf->setFooter('<input type="text" value="1" size="1" ondblclick="this.value=1; this.style.display=\'none\'" style="border-width:0px; display: none;" /><input type="button" id="rowControlButtonAddLine_'.$this->id.'" class="button" onclick="if( ifRowControlClicked ){this.previousSibling.style.display = \'\';} ifRowControlClicked=1; ifHandler.duplicateParentsLastRow(this, this.previousSibling.value.toString()); ;window.setTimeout(\'ifRowControlClicked=0;\', 750);" value="'.(isset($item->addRowText)?$item->addRowText:'+').'" ondblclick="this.previousSibling.style.display = \'\';" />');
								//$rf->columnModifier(array_pad(array((count($row)-1)=>' width="5%" align="center"'), -1*($c=count($row)), ' width="'.floor(95/$c).'%"'));
								if(!$decapitate)
									$rf->columnModifier(array(array_pop($label_row)=>'class="if_row_control"'));
								else
									$rf->columnModifier(array((count($row)-1)=>'class="if_row_control"'));
							}
							else
							{
								if(!$decapitate)
								{
									$rf->setValueProcessor(array_shift($label_row),'$extra = \' width="'.$width.'";\'; return $value;');
									$rf->setValueProcessor($label_row,'$extra = \' width="'.floor($remainWidth/($c-1)).'";\'; return $value;');
								}
								else
									$rf->columnModifier(array_pad(array(0=>' width="'.$width.'%"'), $c=count($row), ' width="'.floor($remainWidth/($c-1)).'%"'));
							}
							$data[] = (array) $rf->getXHTMLTable(array('tr'=>'tr', 'tfoot'=>'if_row_control'), 2);
							$rf = false;
							$decapitate = true;
						}
						else
						{
							if($show_label_row===DISPOSITION_TOP)
								$data[] = $label_row;
							$data[] = $row;
							if($show_label_row===DISPOSITION_BOTTOM)
								$data[] = $label_row;
						}

						$label_row = $row = array();
						$show_label_row = -1;
					}
					else
						$rf || $rf = new TableFactory();
				}
				if ( !empty($row) ) //last item with close_row == false
				{
					$rf->setData(array($row));
					$rf->decapitate();
					$rf->indentlevel = 1;
					$spannedrows[] = count($data);
					if ( $item->type_id == IF_ITEM_ROW_CONTROL )
					{
						$rf->setFooter('<input type="button" class="button" onclick="ifHandler.duplicateParentsLastRow(this);" value="+" />');
						$rf->columnModifier(array_pad(array((count($row)-1)=>' width="5%" align="center"'), -1*($c=((count($row)-1)?(count($row)-1):1)), ' width="'.floor(95/$c).'%"'));
					}
					else
						$rf->columnModifier(array_pad(array(0=>' width="'.$width.'%"'), $c=((count($row)-1)?(count($row)-1):1), ' width="'.floor($remainWidth/$c).'%"'));
					$data[] = (array) $rf->getXHTMLTable('Container');
				}

				//Spannedrows? Do the spans!
				//The local variablenames in this ValueProcessor are prefixed with 'VP_' to prevent scope namespace collisions.
				!empty($spannedrows) && $tf->setValueProcessor(0, 'if ( in_array($i, $VP_spanned = explode(\',\', \''.implode(',', $spannedrows).'\')) )
							{
								//This row is spanned. So $value contains a prefab table built by $rf
								$tdclass = "Container"; //Use a clear container class
								$extra = " colspan=\"2\""; //Set the span
								$this->hideColumn(1); //Hide the last column
								if ( isset($this->modifyCols[$head]) )
								{
									$VP_modifier = $this->modifyCols[$head];
									unset($this->modifyCols[$head]);
								}
							}
							elseif ( in_array($i-1, $VP_spanned) )
							{
								//the previous row had a span
								$this->unhideColumn(1); //show the last column again
								isset($VP_modifier) && $this->modifyCols[$head] = $VP_modifier;
							}
							return $value;');

				$tf->setData($data);
				$tf->decapitate();
				for ( $i=0; $i < count($data); $i++ )
					if ( !in_array($i, $spannedrows) )
					{
						$tf->columnModifier(array_pad(array(0=>' width="'.$width.'%"'), $c=((count($data[0])-1)?(count($data[0])-1):1), ' width="'.floor($remainWidth/$c).'%"'));
						break;
					}
				if($this->type_id != IF_BLOCK_NO_TITLE)
				{
					$help = ($this->handle != '') ? getHelp($this->langIf, $this->handle, array('title'=>$this->handle, 'top'=>'0px')) : '';
					$tf->setCaption($help.'<img src="/'.SITE_PATH.'images/block_arrow.png" /> '.$this->title);
				}
				$tf->indentlevel = 1;

				empty($classes) && $classes = $toggle?'Professional':'Professional Header';
				$tf->id = 'ifB_'.$this->id;
				if($toggle)
					$tf->setCaptionAttributeString(' onclick="ifHandler.toggleBlock('.$this->id.');" ');
				$rv .= $tf->getXHTMLTable($classes,2);
				//$toggle && (!$split) && $rv .= '</div>';
			break;
			case IF_BLOCK_ICONMENU: // Crossbrowser iconmenu support
				require_once(IWARE."table_factory.php");

				$data = '';
				foreach( $this->items as $item )
					$data .= $item->getXHTMLItem($classes);

				$tf = new TableFactory(array(array($data)));
				$tf->decapitate();
				$tf->setCaption($this->title);
				$rv = $tf->getXHTMLTable();
			break;
			default:
				foreach( $this->items as $item )
					$rv .= $item->getXHTMLItem($classes);
		}

		$rv .= $this->_parseHiddenFields();

		if ( !$split )
			return "\n\t".($script?'<script type="text/javascript"><!--'."\n".$script."\n\t--></script>\n":'').$rv.($postscript?'<script type="text/javascript"><!--'."\n".$postscript."\n\t--></script>\n":'').'</div>';

		return array($rv, $script, $postscript);
	}
	function _defineTypes()
	{
		foreach ( $this->getTypes() as $name=>$val )
		{
			if ( defined($name) )
				return;
			define($name, $val);
		}
	}
	function _getHideInput()
	{
		if(isset($this->_hideInputs))
			return $this->_hideInputs;
		if(isset($this->parent->_hideInputs))
			return $this->parent->_hideInputs;
		return false;
	}
	var $_hideInputs = null;
	/**#@-*/
}
?>
