<?php
require_once(INTERFACE_OBJ."InterfaceObject.php");
require_once(INTERFACE_OBJ."InterfaceBlock.php");
/**
 * interfaceFactory
 *
 * Creates interfaces from Blocks.
 *
 * @author Marten Koedam <marten@basesoft.com>
 * @author Chris Wesseling <chris.wesseling@xs4all.nl>
 * @copyright (2005) BaseSoft B.V.
 * @since 13-05-2005
 * @version $Id$
 * @package I-Ware
 * @subpackage Core
 */
class InterfaceFactory extends InterfaceObject
{
	/**
	 *
	 */
	var $jsSplit = false;
	/**
	 *
	 */
	var $fb;
	/**
	 * @var  string $process
	 */
	var $process = 'save';
	/**
	 * @var  string $handle
	 */
	var $handle = '';
	/**
	 *
	 */
	var $appPath = 'i-w/';
	/**
	 * @var string $itemClass
	 */
	var $classPrefix = 'Interface';
	/**
	 * @var string $delimiter
	 */
	var $delimiter = '';
	/**
	 * @var string $staleblockparams Default params used by javascript ifHandler.loadBlock
	 */
	var $staleblockparams = '';
	/**
	 * @var Array $blocks
	 */
	var $blocks = array();
	/**
	 *
	 */
	var $_setValues = false;
	/**
	 *
	 */
	var $_hideInputs = false;
	/**
	 *
	 */
	var $_IfTitle = false;
	/**
	 * @var string $script JavaScript printed in header
	 */
	var $script = '';
	 /**
	  * var Array $nestedItemInterfaces Array containing InterfaceItems of type IF_ITEM_INTERFACE
	  */
	var $nestedItemInterfaces = array();

	/**
	 * Constructor
	 * @param Database $db IWARE database
	 * @param LanguageInterface $langIf
	 * @param Array $blocks (BLOCK_ID=>(TITLE,ITEMS))
	 * @todo remove blocks array and replace with settings
	 */
	function InterfaceFactory(&$db_iware,&$db_app,&$langIf,$settings=array())
	{
		parent::InterfaceObject($db_iware, $db_app, $langIf);

		require_once(IWARE.'FormBehaviour.php');
		$this->fb = FormBehaviour::createFormBehaviour($langIf, $db_app);

		// extract pre-callbacks from settings
		$this->pre_callbacks = isset($settings['pre_callbacks']) ? $settings['pre_callbacks'] : array();
		unset($settings['pre_callbacks']);

		// extract callbacks from settings
		$this->callbacks = isset($settings['callbacks']) ? $settings['callbacks'] : array();
		unset($settings['callbacks']);

		// extract block_callbacks from settings
		$this->block_callbacks = isset($settings['block_callbacks']) ? $settings['block_callbacks'] : array();
		unset($settings['block_callbacks']);

		// unset vars we dont need
		unset($settings['classPrefix'], $settings['appPath']);

		// save all settings as local variables
		// if string, eval it :O
		foreach($settings as $key => $value)
		{
			if (is_string($value) && !is_numeric($value) && !is_object($value))
				$this->$key = eval($value);
			else
				$this->$key = $value;
		}
	}

	/**
	 * FactoryMethod
	 * Instantiatiates and initializes a fresh new InterfaceFactory Object
	 * <code>
	 *		$if =& InterfaceFactory::createInterfaceFactory(1, $db_iware_settings, $db_app, $langIf);
	 * </code>
	 * @access static
	 */
	public static function createInterfaceFactory($interface_id, $db_iware, $db_app, $langIf, $settings=array())
	{
		if(!is_numeric($interface_id))
		{
			$db_if = IWRegistry::registry('db');
			$sql = "SELECT INTERFACE_ID FROM TBL_INTERFACE
				WHERE `HANDLE` = ?";
			$q = $db_if->query($sql);
			$q->bind_values($interface_id);
			$q->execute();

			if(!($r = $q->fetch())||($r2 = $q->fetch()))
				throw new IWException('None or multiple instances found');
			$interface_id = $r->INTERFACE_ID;
		}

		$if = new InterfaceFactory($db_iware, $db_app, $langIf, $settings);
		$if->loadClasses($interface_id);

		$classPrefix = isset($settings['classPrefix']) ? $settings['classPrefix'] : $if->classPrefix;
		$appPath = isset($settings['appPath']) ? $settings['appPath'] : $if->appPath;

		$file = INTERFACES.$appPath.$classPrefix.'.php';

		if(is_file($file))
		{
			require_once($file);
			$if = new $classPrefix($db_iware, $db_app, $langIf, $settings);
		}
		else
			info(sprintf('createInterfaceFactory() - cannot init class, file (%s) not found.', $file));


		$if->init($interface_id);
		return $if;
	}
	/**
	 * Loads the class for the blocks and items
	 */
	function loadClasses($interface_id)
	{
		$sql = "SELECT CLASS, PATH FROM TBL_INTERFACE WHERE INTERFACE_ID = ?";
		$q = $this->db_if->query($sql);
		$q->bind_values(array($interface_id));
		$q->execute();
		if( $r = $q->fetch())
		{
			$this->classPrefix = $r->{'CLASS'};
			$this->appPath = $r->PATH;
		}
	}
	/**
	 * @param int $interface_id INTERFACE_ID
	 */
	function init($interface_id)
	{
		$this->id = $interface_id;

		$q = $this->_selectInterface($interface_id);
		$this->loadClasses($interface_id);
		$ifmap = $this->_filter($q);
		if ( !isset($ifmap[$interface_id]) )
			return false;

		$this->handle = $ifmap[$interface_id]['HANDLE'];

		foreach ( $ifmap[$interface_id] as $bkey=>$bval )
		{
			if ( $bkey && is_numeric($bkey) )
			{
				$bval['PRIV'] = isset($bval['PRIV'])?$bval['PRIV']:$ifmap[$interface_id]['PRIV'];
				$bval['PARENT'] = &$this;
				$this->blocks[$bkey] = $this->interfaceBlock($bkey, $bval);
			}
		}
		return true;

	}
	/**
	 * Sets the delimiter between blocks
	 * @param string $delimiter;
	 */
	function setDelimiter($delimiter='')
	{
		$this->delimiter = $delimiter;
	}
	/**
	 * Set the interface blocks for processing.
	 * @param Array $blocks (BLOCK_ID=>(TITLE,ITEMS))
	 */
	function setInterfaceBlocks($blocks)
	{
		foreach($blocks as $id => $content)
			$this->addBlock($id,$content);
	}
	/**
	 * Adds a block to the interface
	 * @param int $blockId
	 * @param Array $content (TITLE,ITEMS)
	 */
	function addBlock($blockId,$content)
	{
		list($title, $type_id, $items) = $content;
		//$this->blocks[$blockId] = $this->interfaceBlock($title,$type_id,$items);
	}
	/**
	 *
	 */
	function addIconBlock($title, $interfaceIcons)
	{
		$this->addBlock(0,array($title, IF_BLOCK_ICONMENU, array()));
		foreach( $interfaceIcons as $i => $iconContent )
		{
			$this->addItemToBlock(0,$i,array($iconContent['title'],
				array(
				'type_id'=>IF_ITEM_ICON,
				'icon'=>'/'.SITE_PATH.'images/'.$iconContent['image'],
				'disabled'=>(!$iconContent['enabled']),
				'onClick'=>$iconContent['function'])));
		}
	}
	/**
	 * Adds an item to a block
	 * @param int $blockId
	 * @param int $itemId
	 * @param Array $item
	 */
	function addItemToBlock($blockId,$itemId,$item)
	{
		$this->blocks[$blockId]->addItem($itemId,$item);
	}
	/**
	 * Creates a new instance of an interface block
	 * @param int $bid BLOCK_ID
	 * @param Array $blockmap (HANDLE, PRIV, TYPE, array(ITEM_ID=>itemmap))
	 * @return interfaceBlock $block
	 */
	function interfaceBlock($bid, $blockmap=array())
	{
		if( ! self::$checked && file_exists( INTERFACES.$this->appPath.$this->classPrefix.'.php' ))
			require_once(INTERFACES.$this->appPath.$this->classPrefix.'.php');
		self::$checked = true;
		$className = class_exists($this->classPrefix.'Block')?$this->classPrefix.'Block':'InterfaceBlock';
		return new $className($this->db,$this->db_app,$this->langIf,$bid,$blockmap);
	}
	public static $checked = false;

	/**
	 * Creates an XHTML Interface
	 * @param Array $classes (ITEM_TYPE=>CLASS_NAME)
	 * @return string $rv
	 */
	function getXHTMLInterface($classes=array(), $nested=false)
	{
		if(isset($this->_process)&& $this->_process=='print')
			return $this->getPrintInterface($classes);

		$rv = $nested?'':$this->getInterfaceBehaviour().$this->getInterfaceHeader();
		$postscript = $script = '';
		foreach( $this->blocks as $block )
		{
			if($this->jsSplit)
			{
				list($content,$tmpscript,$tmppostscript) = $block->getXHTMLBlock($classes,true);
				$rv .= $content.$this->delimiter."\n";
				$script .= $tmpscript;
				$postscript .= $tmppostscript;
			}
			else
				$rv .= $block->getXHTMLBlock($classes).$this->delimiter."\n";
		}
		$rv .= $this->hiddenFieldsHtml();
		$rv .= $nested?'':$this->getInterfaceFooter();
		if(!$this->jsSplit)
			return $rv;

		return array($rv,$script,$postscript);
	}
	/**
	 *
	 */
	function setJsSplit($split=false)
	{
		$this->jsSplit = $split;
	}
	/**
	 *
	 */
	function getInterfaceBehaviour()
	{
		return '<script type="text/javascript" src="'.REL_IWARE_INCLUDES.'FormBehaviour.js"></script>
			<script type="text/javascript"><!--

				var fb = new FormBehaviour();
				addEvent(window,\'load\', function(){
						addEvent(getElById(\'interface_form\'),\'submit\',function() {

							try {

								if (typeof tinyMCE != \'undefined\')
									tinyMCE.triggerSave();

								if(fb.validate(this)) {

									this.submit();

									// after submitting disable all inputs, in a timeout else chrome doesnt submit them :s
									setTimeout(function() {
									 	var inputs = document.getElementsByTagName(\'input\');
									 	for (var i in inputs) {
										 	inputs[i].disabled = true;
									 	}
									}, 1);

								}
							} catch (e) {
								alert(e);
							}
						})
					});
			--></script>
			';
	}
	/**
	 *
	 */
	function getPrintInterface($classes=array())
	{
		$this->_setValues = true;
		$this->_hideInputs = true;

		$rv = $this->getInterfaceHeader();
		foreach( $this->blocks as $block )
			$rv .= $block->getXHTMLBlock($classes).$this->delimiter."\n";
		$rv .= $this->hiddenFieldsHtml();
		$rv .= $this->getInterfaceFooter();
		return $rv;
	}
	function hiddenFieldsHtml()
	{
		return $this->_parseHiddenFields();
	}
	/**
	 * @abstract
	 */
	function getInterfaceHeader()
	{
		$rv = "<script type=\"text/javascript\"><!--\n".$this->fb->getJavaScriptFailures()."--></script>";
		if (!empty($this->script))
			$rv .= "<script type=\"text/javascript\"><!--\n".$this->script."\n--></script>";
		$rv .= ($this->_IfTitle?'<div style="font-weight: bold;width:100%;margin:2px auto;text-align: left;">&#187; '.$this->_IfTitle.'</div>':'').
			'<form style="margin: 0px; padding: 0px;" action="'.str_replace('?'.$_SERVER['QUERY_STRING'],'',$_SERVER["REQUEST_URI"]).'" method="post" enctype="multipart/form-data" id="interface_form" onsubmit="return false;">';

		return $rv;
	}
	var $showRequiredString = true;
	/**
	 * @abstract
	 */
	function getInterfaceFooter()
	{
		$rv = (isset($this->requiredItems)&& $this->showRequiredString)?'<p><span class="required">*</span> '.$this->langIf->getText('Marked Fields are required').'</p>':'';
		$rv .= "\t"."\n".
			'<input id="interface_form_process" type="hidden" name="process" value="'.$this->process.'" />';
		$rv .= '</form>';
		return $rv;
	}
	/**
	 *
	 */
	function removeChild($block_id)
	{
		if (!isset($this->blocks[$block_id]))
			trigger_error(sprintf('InterfaceFactory::removeChild() - cannot remove block [%s] because does not exist', $block_id), E_USER_WARNING);

		foreach($this->blocks[$block_id]->items as $item_id=>$bitbucket)
			$this->blocks[$block_id]->removeChild($item_id);

		unset($this->blocks[$block_id]);
	}

	/**
	 * Saves the complete interface.
	 * @param string $thrownException
	 */
	function processInput(&$thrownException)
	{
	}

	/**
	 * Calls processInput() for all nested interfaces. They are registered in block
	 */
	function processNestedInterfaces(&$thrownException)
	{
		foreach ($this->nestedItemInterfaces as $index=>$bitbucket)
		{
			$item = $this->nestedItemInterfaces[$index];

			if (isset($item->settings))
				$settings = &$item->settings;
			else
				$settings = array();

			$if = &InterfaceFactory::createInterfaceFactory($item->name,
				$this->db, $this->db_app, $this->langIf, $settings);

			if (!$if->processInput($thrownException))
				return false;
		}

		return true;
	}
}

?>
