<?php
/**
 * @package Interface
 * @subpackage InterfaceHandler
 * @since 19-mrt-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class InterfaceHandler
{
	/**
	 * Interface object
	 * @var InterfaceObject $ifObject
	 */
	public $ifObject;
	/**
	 * @var string $universe
	 */
	public $universe;
	/**#@+
	 * @access protected
	 */
	/**
	 * @var int|bool $script_id
	 */
	protected $script_id = false;
	/**
	 * @var string $extraHeader
	 */
	protected $extraHeader = '';
	/**
	 * @var string $extraFooter
	 */
	protected $extraFooter = '';
	/**
	 * @var string $script_header
	 * @todo make this prettier (register scripts in an array or something)
	 */
	protected $script_header = '';
	/**
	 * @var array $extraStyle (nameOfStyleSheet,nameOfStyleSheet,...)
	 */
	protected $extraStyle = array();
	/**
	 * @var string $thrownException
	 */
	protected $thrownException;
	/**#@-*/
	/**
	 * @access private
	 * @static
	 * @var InterfaceHandler $instance
	 */
	private static $instance;
	/**
	 * @access private
	 * @static
	 * @var array $mapping
	 */
	private static $mapping = array('i-w'=>array('IW',IWARE),'elc'=>array('WEEE',ELC_CLASSES));
	public static $outputCall = 'getXHTMLInterface';
	public $skipChecks = array();
	/**
	 * Protected constructor, can only be created from this object (static) or from its children
	 * @param string $universe
	 */
	protected function __construct($universe)
	{
		$this->universe = $universe;
	}
	/**
	 * Intializes the interface handler
	 * @static
	 */
	public static function init()
	{
		if(self::$instance)
			throw new IWException('InterfaceHandler already initialized');

		// Load the universe Handler.
		preg_match('/\/scripts\/(.*)\/(.*)\/(Interface[\w]+)\.php/', $_SERVER['REQUEST_URI'],$matches);

		if(!count($matches))
			if(!count($matches))
				throw new IWException('Incorrect request URI');

		$application = $matches[1];
		$universe = $matches[2];
		$interface = $matches[3];

		if(!array_key_exists($application, self::$mapping)) {
			require_once(IWARE_SCRIPTS.'PermissionDenied.php');
			exit();
		}

		$instanceName = self::$mapping[$application][0].ucfirst($universe).'InterfaceHandler';
		if(!is_file(self::$mapping[$application][1].$instanceName.'.php'))
		{
			require_once(IWARE_SCRIPTS.'PermissionDenied.php');
			exit();
		}
		require_once(self::$mapping[$application][1].$instanceName.'.php');

		self::$instance = new $instanceName($universe);
		self::$instance->{$interface.'Action'}(); // caught by __call if the function doesnt exist
	}
	/**
	 * Renders the interface and returns the content.
	 * @static
	 * @todo make this prettier. (no more output buffering)
	 */
	public static function display()
	{
		if(!self::$instance) // There is no InterfaceHandler initialized
			throw new IWException('Display called when not initialized');

		if(!($script_id = self::$instance->getScriptId())) // We dont have a script id, something must have fucked up. Please check TBL_SCRIPT
			throw new IWException(sprintf('No Script could be found for the requested interface `%s` in universe `%s`',self::$instance->ifObject->handle,self::$instance->universe));

		ob_start(); // start output buffering

		global $page_title_postfix; // FIXME

		// Validate Script Privileges;

		$site_title = SITE_TITLE;
		$import_scope = self::$instance->exportScopeVars();
		foreach($import_scope as $name => $val)
		{
			${$name} = $val;
		}
		if(self::validateAccess($script_id))
		{
			require_once(XHTML_HEADER);
			echo self::$instance->getHeader();
			echo self::$instance->ifObject->{self::$outputCall}();
			echo self::$instance->getFooter();
			require_once(XHTML_FOOTER);
		}
		else
			throw new InterfacePermissionException(sprintf('Unauthorized access to script [%s]',$script_id));

		$rv = ob_get_clean(); // Get the content to return.
		return $rv;
	}

	public static function validateAccess($script_id)
	{
		$script_parent_id = self::getScriptParentId($script_id);
		$Remoras = SuckerfishMenu::create(IWRegistry::registry('db'), IWRegistry::registry('db'), IWRegistry::registry('langIf'));
		$rv = $Remoras->hasAccess($script_parent_id, $script_id);
		return $rv;

	}

	public static function getScriptParentId($script_id)
	{
		$scriptDb = IWRegistry::registry('db');
		$sql = "SELECT SCRIPT_PARENT_ID, SCRIPT_NAME FROM TBL_SCRIPT WHERE SCRIPT_ID=?";
		$q = $scriptDb->query($sql);
		$q->bind_values($script_id);
		$q->execute();
		if ($row = $q->fetch())
		{

			do
			{
				//$script_name = $script_name?$script_name:$row->SCRIPT_NAME;
				//if(!($script_parent_id == 0 && isset($_SESSION["session_group_company_id"])))
					//break;
				$script_parent_id = $row->{'SCRIPT_PARENT_ID'};
				$sql = "SELECT SCRIPT_ID FROM TBL_SCRIPT_GROUP WHERE SCRIPT_ID=? AND GROUP_COMPANY_ID=? AND SCRIPT_GROUP_ACCESS > 0";
				$q2 = IWRegistry::registry('db')->query($sql);
				$q2->bind_values($script_parent_id, $_SESSION["session_group_company_id"]);
				$q2->execute();
				if ($r2 = $q2->fetch()) { //
					return $row->{'SCRIPT_PARENT_ID'};
				}
			}while ($row = $q->fetch());
		}
		return false;
	}

	/**#@+
	 * Getters & Setters
	 */
	public function getHeader()
	{
		return $this->extraHeader;
	}
	public function getFooter()
	{
		return $this->extraFooter;
	}
	public function getScriptHeader()
	{
		return $this->script_header;
	}
	public function getScriptId()
	{
		if($this->script_id)
			return $this->script_id;

		$ifId = self::$instance->ifObject->id;
		$sql = "SELECT SCRIPT_ID FROM TBL_SCRIPT
			WHERE SCRIPT_UNIVERSE = ?
				AND INTERFACE_ID = ?";
		$q = IWRegistry::registry('db')->query($sql);
		$q->bind_values($this->universe,$ifId);

		$q->execute();
		if((!($r = $q->fetch()))||($r2 = $q->fetch())) // There must be ONE and ONE only
			return false;
		return $r->SCRIPT_ID;
	}
	public function getExtraStyle()
	{
		return $this->extraStyle;
	}
	public function getThrownException()
	{
		return $this->thrownException;
	}

	protected function setHeader($str)
	{
		$this->extraHeader = $str;
	}
	protected function setFooter($str)
	{
		$this->extraFooter = $str;
	}
	protected function setScriptHeader($str)
	{
		$this->script_header = $str;
	}
	protected function setScriptId($id)
	{
		$this->script_id = $id;
	}
	protected function setExtraStyle($style)
	{
		$this->extraStyle[] = $style;
	}
	public function setThrownException($str)
	{
		$this->thrownException = $str;
	}
	/**#@-*/
	/**
	 *
	 */
    protected function instantiateInterfaceFactory($ifName, $settings=array())
    {

		/**
		 * The variable $this->func_input_filter can be overruled by setting
		 * this to a specific callback filter (like if you need to allow html
		 * data). By default we just sanitize like init normally does.
		 *
		 * See also WEEESysadminInterfaceHandler::InterfaceEmailTranslatorAction
		 *
		 */
    	if (!isset($this->func_input_filter))
    	{
			$_GET = sanitize_array_recursive($_GET);
			$_POST = sanitize_array_recursive($_POST);
			$_REQUEST = sanitize_array_recursive($_REQUEST);
    	}
    	else
			call_user_func($this->func_input_filter);

		/**
		 * Now just continue...
		 */
		try {
			self::$instance->universeChecks(self::$instance->skipChecks);

			require_once(INTERFACE_OBJ.'InterfaceFactory.php');
		self::$instance->ifObject = InterfaceFactory::createInterfaceFactory($ifName, IWRegistry::registry('db'), IWRegistry::registry('db'), IWRegistry::registry('langIf'), $settings);
			$str = '';
			self::$instance->ifObject->processInput($str);

			$this->setThrownException($str);
		}
		catch(InterfacePermissionException $e)
		{
			header('HTTP/1.0 403 Forbidden');
			alert('Permission Exception: '.$e->getMessage());
			throw new IWUserException('You don\'t have permission to access the requested resource');
		}
		// All other exceptions are caught by the EventHandler.
	}

	/**
	 *
	 */
	public function __call($func,$args=array())
	{
		if(substr($func,0,9)=='Interface')
		{
			$func = substr($func,0,-6);
    		return $this->instantiateInterfaceFactory($func);
		}
		throw new IWException(sprintf('Call to undefined function `%s`',$func));
	}

	public function exportScopeVars()
	{
		return array(
			'script_header' => $this->getScriptHeader(),
			'show_style' => $this->getExtraStyle(),
			'thrownException' => $this->getThrownException()
		);
	}

	public static function universeChecks($skipChecks)
	{
		require_once(INTERFACE_OBJ.'InterfacePermission.php');
		$perm = new InterfacePermissions(self::$instance->universe,$skipChecks);
		$perm->validate();

	}
}
?>
