<?php
require_once(INTERFACE_OBJ."InterfaceObject.php");
require_once(IWARE . 'form.php');
/**
 * interfaceItem
 *
 * Creates interfaceItems.
 *
 * @author Marten Koedam <marten@basesoft.com>
 * @author Chris Wesseling <chris.wesseling@xs4all.nl>
 * @copyright (2005) BaseSoft B.V.
 * @since 13-05-2005
 * @version $Id$
 * @package I-Ware
 * @subpackage Core
 */
class InterfaceItem extends InterfaceObject
{

	public $text = '';
	public $script = '';
	public $postscript = '';

	/**
	 * Add your new block types here.
	 * @return array ( 'BLOCK_TYPE', # )
	 */
	function getTypes()
	{
		return array('IF_ITEM_DROPDOWN'=>1,
			'IF_ITEM_TEXTFIELD'=>2,
			'IF_ITEM_DATE'=>3,
			'IF_ITEM_CHECKBOX'=>4,
			'IF_ITEM_SUBMIT'=>5,
			'IF_ITEM_ICON'=>6,
			'IF_ITEM_FANCY_SELECT'=>7,
			'IF_ITEM_LINK'=>8,
			'IF_ITEM_TEXT'=>9,
			'IF_ITEM_ROW_CONTROL'=>10,
			'IF_ITEM_MULTI_SELECT'=>11,
			'IF_ITEM_PASSWORD'=>12,
			'IF_ITEM_BUTTON_BAR'=>13,
			'IF_ITEM_RADIO'=>14,
			'IF_ITEM_TEXTAREA'=>15,
			'IF_ITEM_INTERFACE'=>16,
			'IF_ITEM_FILE'=>17,
			'IF_ITEM_TABLE_FACTORY'=>18
			);
	}
	/**
	 * @var string $title Translated (or not) $title
	 */
	var $title = "";
	/**
	 * @var string $handle Used for Help
	 */
	var $handle = "";
	/**
	 * @var string $name Used for <(input|select) name>
	 */
	var $name = "";
	/**
	 * @var array $properties HTML properties of the <input>/<select> $key="$value"
	 */
	var $properties = array();
	/**
	 * @var int $postition
	 */
	var $position = '';
	/**
	 * @var string $behaviour
	 */
	var $behaviour = '';
	/**
	 *
	 */
	var $label_disposition = 4;
	/**
	 * @var int $priv
	 */
	var $priv = 15;
	/**
	 * @var bool $no_priv
	 */
	var $no_priv = false;
	/**
	 * @var InterfaceBlock $parent
	 */
	var $parent = false;
	/**
	 *
	 */
	var $_hideInput = null;
	/**
	 * Constructor
	 * @param Database $db IWARE database
	 */
	function InterfaceItem($db,$db_app,$langIf,$id=0,$itemmap=array())
	{
		$this->_defineTypes();
		$this->id = $id;
		parent::InterfaceObject($db, $db_app, $langIf);
		$this->_setDefaultValues();
		$itemmap && $this->processMap($itemmap);
		$behaviour = (isset($this->properties['behaviour'])?$this->properties['behaviour'].';':'').$this->behaviour;

		//if (isset($this->type_id) && $this->type_id == IF_ITEM_FILE)
			//$behaviour .= ';document';

		if ($this->parent != false && $behaviour && strlen($behaviour))
			$this->parent->parent->fb->register($this->name,$behaviour,$this->handle);
		else
		{
			$registeredBehaviour = isset($this->parent->parent->fb) ? $this->parent->parent->fb->getBehaviour($this->name) : '';
			$this->behaviour = $registeredBehaviour;
		}


	}
	/**
	 * @param Array $map
	 */
	function processMap($map)
	{
		$this->priv = $map['PRIV'];
		$this->type_id = $map['TYPE'];
		$this->setTitle($map['HANDLE']);
		$this->callback = $map['CALLBACK'];
		$this->name = $map['NAME'];
		$this->close_row = $map['CLOSE_ROW'];
		$this->behaviour = $map['BEHAVIOUR'];
		$this->help_enabled = $map['HELP_ENABLED'];
		$this->label_disposition = (int)$map['LABEL_DISPOSITION'];
		$this->parent = &$map['PARENT'];
		$this->if_obj = &$this->parent->parent;

		// pre-callbacks, get executed before the item callback
		if (isset($this->if_obj->pre_callbacks[$this->name])) {
			list ($obj, $func) = $this->if_obj->pre_callbacks[$this->name];
			if (method_exists($obj, $func))
				$obj->{$func}($this);
		}

		// execute item callback
		if ( method_exists($this, $this->callback) )
			$this->{$this->callback}();

		// execute extra callbacks which came from $settings (see interfacefactory constructor)
		if (isset($this->if_obj->callbacks[$this->name])) {
			list ($obj, $func) = $this->if_obj->callbacks[$this->name];
			if (method_exists($obj, $func))
				$obj->{$func}($this);
		}
	}
	/**
	 * Saves Item
	 * @param string $thrownException
	 */
	function saveItem(&$thrownException)
	{
		if ( !is_numeric($this->position) )
		{
			if ( is_numeric($this->id) && $this->id )
			{
				$field = 'POSITION';
				$key = 'ITEM';
				$values[] = $this->id;
			}
			else
			{
				$field = 'IFNULL(MAX(POSITION),0)+1';
				$key = 'BLOCK';
				$values[] = $this->parent_id;
			}

			$sql = "SELECT $field as pos
				FROM TBL_INTERFACE_ITEM
				WHERE ${key}_ID = ?";
			$q = $this->db->query($sql);
			$q->bind_values($values);
			if ( $q->execute(true) )
			{
				$r = $q->fetch();
				$this->position = $r->pos;
			}
			else
			{
				$thrownException .= 'Failed to determine item position';
				return false;
			}
		}

		$sql = "REPLACE INTO TBL_INTERFACE_ITEM (ITEM_ID, TYPE, POSITION, BLOCK_ID, HANDLE, CALLBACK, NAME, CLOSE_ROW, BEHAVIOUR, LABEL_DISPOSITION, HELP_ENABLED)
			VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
		$q = $this->db->query($sql);
		$q->bind_values(array($this->id, $this->type_id, $this->position, $this->parent_id, $this->handle, $this->callback, $this->name, $this->close_row, $this->behaviour, $this->label_disposition, $this->help_enabled));
		if ( $q->execute(true) )
			return (is_numeric($this->id) || $this->id = $q->lastid()); //== true

		//oops;
		$thrownException .= $q->exception->toString();
		$thrownException .= 'Failed to save Item';
		return false;
	}
	/**
	 * Deletes Item
	 * @param string $thrownException
	 */
	function deleteItem(&$thrownException)
	{

	}
	/**
	 * Sets the title of the Block
	 * @param string $title
	 */
	function setTitle($title)
	{
		$this->handle = $title;
		$this->title = ($title && $this->langIf) ? $this->langIf->getText( $title ) : $title;
	}
	/**
	 * @param Array $classes
	 * @return string $rv
	 */
	function getXHTMLItem($classes=array())
	{
		$this->_setAccessibility();
		$rv = $this->_displayItem($classes);
		return $rv;
	}
	/**#@+
	 * @priv private
	 */
	/**
	 * @var interfaceFactory $if
	 */
	var $if = false;
	/**
	 * Sets the interfaceFactory variable
	 */
	function _setInterfaceFactory()
	{
		require_once(INTERFACE_OBJ."InterfaceFactory.php");
		$this->if = new InterfaceFactory($this->db,$this->langIf);
	}
	/**
	 * Sets the default values of an item
	 */
	function _setDefaultValues()
	{
	}
	/**
	 * Sets the privibility
	 */
	function _setAccessibility()
	{
		if( !( READ_ACCESS & $this->priv ))
			return $this->no_priv = true;
		if( READ_ACCESS & $this->priv &&
			!( WRITE_ACCESS & $this->priv ) &&
			!( CHANGE_ACCESS & $this->priv ))
			return $this->properties['disabled'] = 'disabled';
	}
	/**
	 * Displays a single item
	 */
	function _displayItem($classes=array())
	{
		empty($this->properties['id']) && $this->properties['id'] = 'ifI_'.$this->id;
		if( $this->no_priv )
			return array();

		$hidden = $rv = '';
		if( ! empty( $this->hiddenFields ))
			$hidden = $this->_parseHiddenFields();

		// Bring back values from the post data to automagically refill the form
		$selects = array(IF_ITEM_MULTI_SELECT,IF_ITEM_DROPDOWN,IF_ITEM_FANCY_SELECT);
		if($this->parent->parent->_setValues&&!in_array($this->type_id,$selects))
		{
			$value = inputStrToArr($this->name, $_POST);

			if($this->type_id == IF_ITEM_CHECKBOX)
			{
				if( !empty($value) )
					$this->properties['checked'] = 'checked';
				else
					unset($this->properties['checked']);
			}
			elseif($this->type_id == IF_ITEM_RADIO)
			{
				foreach ( $this->options as $opti=>$optmap )
					if ( $this->options[$opti]['value'] == $value )
						$this->options[$opti]['checked'] = 'checked';
				unset($value);
			}
			if($this->type_id != IF_ITEM_CHECKBOX)
				isset($value) && $this->properties['value'] = $value;
			elseif($this->_getHideInput())
				isset($value) && $this->properties['value'] = $value;
		}

		if($this->_getHideInput()&& $this->type_id!=IF_ITEM_BUTTON_BAR)
			return $this->_displayItemNoInput($classes,$hidden);
		if($this->behaviour)
		{
			$this->properties['behaviour'] = (isset($this->properties['behaviour'])?$this->properties['behaviour'].';':'').$this->behaviour;
			$this->properties['displayname'] = $this->handle;
			if(strpos($this->properties['behaviour'],'required')!==false)
			{
				$this->parent->parent->requiredItems = true;
				$this->title .= ' <span class="required">*</span>';
			}
		}
		switch( $this->type_id )
		{
			case IF_ITEM_MULTI_SELECT:
				$this->properties['multiple'] = 'multiple';
			case IF_ITEM_DROPDOWN:
				$rv = array();
				if($this->label_disposition==DISPOSITION_LEFT)
					$rv[] = $this->title;
				$rv[] = $hidden.'<select'.$this->_parseProps().' name="'.$this->name.'">'.$this->_getOptions().'</select>'.$this->_parseHelp();
				if($this->label_disposition==DISPOSITION_RIGHT)
					$rv[] = $this->title;
			break;
			case IF_ITEM_RADIO:
				unset($this->properties['id']);
				$rv = array();
				$rv[0] = $this->title;
				$rv[1] = $hidden;
				$radios = array();
				foreach ( $this->options as $optmap )
				{
					$radio = '<input'.$this->_parseProps().' type="radio" name="'.$this->name.'"';
					$value = $id = null;
					foreach ( $optmap as $prop=>$pval )
					{
						if ( $prop == 'value' )
							$value = $pval;
						if ( $prop == 'id' )
							$id = $pval;

						if ($prop == 'checked')
							$radio .= $pval ? ' checked="checked"' : '';
						else if ( $prop == 'text' )
							$text = $pval;
						else
							$radio .= ' '.$prop.'="'.$pval.'"';
					}

					if ( is_null($id) )
					{
						$id = 'ifRadio'.$this->name.$value;
						$radio .= ' id="'.$id.'"';
					}

					$radio .= ' />';
					$label = '<label for="'.$id.'">'.$text.'</label>';
					$radios[] = array($radio, $label);
				}
				if($help=$this->_parseHelp())
					$radios[(sizeof($radios)-1)][1] .= $help;
				$tf = new TableFactory($radios);
				$tf->decapitate();
				$tf->columnize(!empty($this->columncount)?$this->columncount:4);
				$rv[1] .= $tf->getXHTMLTable(array('table'=>'if_container'));
			break;
			case IF_ITEM_FILE:

				// try to get the document, that could throw an exception
				try {
					$document = $this->_getDocument();
				}
				catch (IWFileUploadException $e) {
					$document = null;
				}

				$html = '';

				// Ask user if he wants to keep current or new
				if ($document)
				{
					// set upload to disabled
					$this->properties['disabled'] = true;

					//we need to clear behaviour and only add it when they gonna submit a new file
					$behaviours = isset($this->properties['behaviour']) ? $this->properties['behaviour'] : '';
					unset($this->properties['behaviour']);
					$behaviours = explode(';', $behaviours);

					$rd = new Form_Radio('Documents['.$this->id.']' , $document->id);
					$rd->setChecked(true);
					$rd->setLabel($this->langIf->getText('Keep current file') . ' ('.$document->name.')');
					$rd->setId('rdKeepFileYes'.$this->id);
					$onclick = "var el = $('ifI_".$this->id."'); el.disabled=true; el.value='';$('old_doc_".$this->id."').disabled = false;";
					//removeBehaviours
					foreach($behaviours as $behaviour)
						$onclick .= 'fb.removeBehaviour($(\'ifI_'.$this->id.'\'), \''.$behaviour.'\');';
					$rd->onclick($onclick);

					$hd = new Form_Hidden($this->name, $document->id);
					$hd->setID('old_doc_'.$this->id);
					$html .= $rd->getHtml() . $hd->getHtml(). '<br />';

					$rd = new Form_Radio('Documents['.$this->id.']' , '');
					$rd->setId('rdKeepFileNo'.$this->id);
					$rd->setLabel($this->langIf->getText('Upload new file'));
					$onclick = "$('ifI_".$this->id."').disabled = false; $('old_doc_".$this->id."').disabled = true;";
					//add behaviours
					foreach($behaviours as $behaviour)
						$onclick .= 'fb.setBehaviour($(\'ifI_'.$this->id.'\'), \''.$behaviour.'\');';

					$rd->onclick($onclick);
					$html .= $rd->getHtml();
				}

				$html .= $hidden.'<input'.$this->_parseProps().' type="file" name="'.$this->name.'" />'.$this->_parseHelp();
				$html .= '<br /><div><a href="javascript:alert(\''.$this->langIf->getText("Each file to upload must be 500KB or lower in size. When scanning a document, please scan to a JPEG file, with as lower resolution as possible.").'\');">'.$this->langIf->getText('Max Filesize 500KB').'</a></div>';
				$rv[0] = $this->title;
				$rv[1] = $html;
			break;
			case IF_ITEM_PASSWORD:
				if (!isset($this->properties['autocomplete']))
					$this->properties['autocomplete'] = 'off';
				$type = isset($type)?$type:'password';
			case IF_ITEM_DATE:
			case IF_ITEM_TEXTFIELD:
				$type = isset($type)?$type:'text';
				$rv = array();
				if($this->label_disposition==DISPOSITION_LEFT)
					$rv[] = $this->title;
				$rv[] = $hidden.'<input'.$this->_parseProps().' type="'.$type.'" name="'.$this->name.'" />'.$this->_parseHelp();
				if($this->label_disposition==DISPOSITION_RIGHT)
					$rv[] = $this->title;
			break;
			case IF_ITEM_TEXTAREA:

				$rv = array();
				if($this->label_disposition==DISPOSITION_LEFT)
					$rv[] = $this->title;
				unset($this->properties['value']); // Never fill value in properties
				$html = $hidden.'<textarea'.$this->_parseProps().' name="'.$this->name.'">'.(isset($this->value)?$this->value:'').'</textarea>';

				/**
				 * TinyMCE
				 */
				if (isset($this->is_tiny_mce) && $this->is_tiny_mce)
				{
					// default settings
					$settings_default['theme_advanced_toolbar_location'] = 'top';
					$settings_default['theme_advanced_toolbar_align'] = 'left';

					// enter thingy.. blegh
					$settings_default['force_br_newlines'] = 'true';
					$settings_default['force_p_newlines'] = 'false';

					$settings_default['theme_advanced_buttons1'] = "separator,insertdate,inserttime,preview,zoom,separator,forecolor,backcolor";
					$settings_default['theme_advanced_buttons2'] = "bullist,numlist,separator,outdent,indent,separator,undo,redo,separator,link,unlink,anchor,image,cleanup,help,code";
					$settings_default['theme_advanced_buttons3'] = "hr,removeformat,visualaid,separator,sub,sup,separator,charmap";

					$settings_default['language'] = 'lang'; // include lang.js, which does our own translation

					// merge default with item settings
					$this->tiny_mce_settings = array_merge($settings_default,
						isset($this->tiny_mce_settings) ? $this->tiny_mce_settings : array());

					// FIXME multiple TINYMCE on one page with different settings
					$this->script = "

							if(typeof document.initedTinyMCE == 'undefined')
							{
								tinyMCE.init(".exportArrayToJsObject($this->tiny_mce_settings).");
								document.initedTinyMCE = true;
							}

							eventHandler.addEvent('unloadblock', function() {
								ifHandler.removeTinyMCEForItem(".$this->id.");
							});
							";

					$js = "
  						setTimeout(function(){ifHandler.addTinyMCEForItem(".$this->id.")},1);
						//addEvent(window,'load', function(){ifHandler.addTinyMCEForItem(".$this->id.")});
					";

					if(isset($this->postscript) && $this->postscript)
						$js .= $this->postscript;
					$this->postscript = $js;
				}

				$rv[] = $html;
				if($this->label_disposition==DISPOSITION_RIGHT)
					$rv[] = $this->title;
			break;
			case IF_ITEM_CHECKBOX:
				$rv = array();
				$rv[] = $this->title;
				if(empty($this->properties['value']))
					$this->properties['value'] = '1';
				$rv[] = $hidden.'<input'.$this->_parseProps().' type="checkbox" name="'.$this->name.'" /> ';
				if($help=$this->_parseHelp())
					$rv[(sizeof($rv)-1)] .= $help;
			break;
			case IF_ITEM_SUBMIT:
				$rv = array();
				if($this->label_disposition==DISPOSITION_LEFT)
					$rv[] = $this->title;
				$rv[] = $hidden.'<input'.$this->_parseProps().' type="submit" />';
				if($this->label_disposition==DISPOSITION_LEFT)
					$rv[] = $this->title;
			break;
			case IF_ITEM_ICON:
				// style="filter: gray();"
				$rv .= $hidden.'<button'.$this->_parseProps().' class="icon" onclick="'.
					$this->item['onClick'].'"><img src="'.$this->item['icon'].'" /></button>';
			break;
			case IF_ITEM_FANCY_SELECT:
				/*
				 * available buttons: up, down, left, right, leftAll, rightAll
				 * for $this->fancybuttons = array(button=>properties);
				 */
				$rv = array();
				unset($this->properties['id']);
				$rv[0] = $hidden.'<select'.$this->_parseProps().' multiple="multiple" id="'.$this->name.'_options">'.$this->_getOptions('options').'</select>';
				$rv[1] = '';
				foreach ( $this->fancybuttons as $button=>$props )
				{
					if ( in_array($button, array('up', 'down')) ) //FIXME not yet in FancySelectController
						$rv[1] .= '<input id="IF_I'.$this->id.'_FSB'.$button.'" type="button"'.$this->_parseProps($props).' style="width:17em; margin: 1px;" onclick="ifHandler.move'.ucfirst($button).'(\''.$this->name.'\');" value="'.(isset($this->langIf)?$this->langIf->getText('Button:'.$button):$button).'" /><br />';
					else
						$rv[1] .= '<input id="IF_I'.$this->id.'_FSB'.$button.'" type="button"'.$this->_parseProps($props).' style="width:17em; margin: 1px;" onclick="FancySelectController.prototype.getController(\''.$this->name.'\').move'.ucfirst($button).'();" value="'.(isset($this->langIf)?$this->langIf->getText('Button:'.$button):$button).'" /><br />';
				}
				$rv[2] = '<select'.$this->_parseProps().' multiple="multiple" id="'.$this->name.'_selected">'.$this->_getOptions('selectedoptions').'</select>';
				$rv = isset($this->fancyHeaders)?array($this->fancyHeaders,$rv):array($rv);
				$tf = new TableFactory($rv);
				$tf->decapitate();
				$rv = $tf->getXHTMLTable(array('table'=>'if_container'));
				//init the hidden inputhandler (it's an observer that gets created in the contructor)
				$this->postscript = 'FancySelectController.prototype.getController(\''.$this->name.'\');
				addEvent(document.getElementById(\''.$this->name.'_options\'), \'dblclick\', function(e){FancySelectController.prototype.getController(\''.$this->name.'\').moveRight()});
				addEvent(document.getElementById(\''.$this->name.'_selected\'), \'dblclick\', function(e){FancySelectController.prototype.getController(\''.$this->name.'\').moveLeft()});';
			break;
			case IF_ITEM_LINK:
				$rv .= $hidden.'<a'.$this->_parseProps().'>'.$this->title.'</a>';
			break;
			case IF_ITEM_ROW_CONTROL:
				//$rv .= '<input type="button" class="button" onclick="ifHandler.duplicateParentsLastRow(this);" value="+" />';
				//this could go in the footer of prevcontainer table;
				$rv .= '<input type="button" name="ifI_'.$this->id.'_RC_RB[0]" class="button" onclick="ifHandler.removeParentRow(this);" value="'.((isset($this->removeText)) ? $this->removeText : '-') .'" />';
			break;
			case IF_ITEM_INTERFACE:
				$rv = $hidden.(isset($this->text)?$this->text:'');

				if (isset($this->showInterface) && !$this->showInterface )
					break;

				if (!is_numeric($this->name))
					trigger_error('Not a valid interface for IF_ITEM_INTERFACE ('.$this->id.')', E_USER_ERROR);
				else
				{
					if (isset($this->settings))
						$settings = &$this->settings;
					else
						$settings = array();

					$if = InterfaceFactory::createInterfaceFactory($this->name, $this->db, $this->db_app, $this->langIf, $settings);
					$rv .= '<input type="hidden" name="nestedIf[]" value="'.$if->id.'" />'.$if->getXHTMLInterface('Nested', true);
				}
			break;
			case IF_ITEM_TEXT:
				if(isset($this->twoCols))
					$rv = array($hidden.(isset($this->text)?$this->text:''),'');
				elseif(isset($this->rightCol))
					$rv = array($this->rightCol,$hidden.(isset($this->text)?$this->text:''));
				else
					$rv .= $hidden.(isset($this->text)?$this->text:'');
			break;
			case IF_ITEM_BUTTON_BAR:
				$this->text = $hidden.(isset($this->text)?$this->text:'');
				if(isset($this->twoCols))
					$rv = array($this->text,$this->_parseButtonBar());
				else
					$rv .= $this->text.$this->_parseButtonBar();
			break;
			// TODO: FIXME
			case IF_ITEM_TABLE_FACTORY:
				$rv = $hidden.'';
			break;
			default:
				$rv .= $this->title;
		}
		return $rv;
	}
	/**
	 * Creates a date selection interface (Day,Month,Year)
	 * @param int $firstYear First year to show up in the selectionbox compared to current year
	 * @param int $lastYear Last year to show up in the selectionbox compared to current year
	 * @return string $rv
	 */
	function _parseDateSelector($firstYear=-3,$lastYear=4)
	{
		$rv = '';
		// Day selection first from 1 to 31
		if(!isset($this->hideDay))
		{
			$rv = '<select name="'.$this->name.'[day]"';
			foreach ( $this->properties  as $prop=>$val )
				$rv .= " $prop=\"$val\"";
			$rv .='>';
			$this->options = array( array('value'=>'','text'=>'-'));
			for( $i = 1; $i < 32; $i++ )
			{
				$this->options[] = array('value'=>$i,'text'=>$i);
				if( isset( $this->selectedDay ) && $this->selectedDay == $i )
					$this->options[count($this->options)-1]['selected'] = 'selected';
			}
			$name = $this->name;
			$this->name = $this->name.'[day]';
			$rv .= $this->_getOptions().'</select> ';
			$this->name = $name;
		}

		// Month selection from Jan to Dec
		$rv .= '<select name="'.$this->name.'[month]"';
		foreach ( $this->properties  as $prop=>$val )
			$rv .= " $prop=\"$val\"";
		$rv .='>';
		$this->options = array( array('value'=>'','text'=>'-'));
		for( $i = 1; $i < 13; $i++ )
		{
			$text = date( 'F', mktime(0,0,0,$i,1));
			$text = $this->langIf?$this->langIf->getText($text):$text;
			$this->options[] = array('value'=>$i,'text'=>$text);
			if( isset( $this->selectedMonth ) && $this->selectedMonth == $i )
				$this->options[count($this->options)-1]['selected'] = 'selected';
		}
		$name = $this->name;
		$this->name = $this->name.'[month]';
		$rv .= $this->_getOptions().'</select> ';
		$this->name = $name;

		// Year selection
		if(!isset($this->hideYear))
		{
			$rv .= '<select name="'.$this->name.'[year]"';
			foreach ( $this->properties  as $prop=>$val )
				$rv .= " $prop=\"$val\"";
			$rv .='>';
			$this->options = array( array('value'=>'','text'=>'-'));
			for( $i = $firstYear; $i < $lastYear; $i++ )
			{
				$year = date( 'Y' )+$i;
				$this->options[] = array('value'=>$year,'text'=>$year);
				if( isset( $this->selectedYear ) && $this->selectedYear == $year )
					$this->options[count($this->options)-1]['selected'] = 'selected';
			}
			$name = $this->name;
			$this->name = $this->name.'[year]';
			$rv .= $this->_getOptions().'</select> ';
			$this->name = $name;
		}

		if( ! empty( $this->showHours )) // Appends hour/minute selection
			$rv .= $this->_parseHourSelector();
		if( ! empty( $this->showCalendar )) // Appends calendar icon
			$rv .= $this->_parseCalendar();
		return $rv;
	}
	/**
	 * @var bool $showHours
	 */
	var $showHours;
	/**
	 * @var bool $showCalendar
	 */
	var $showCalendar;
	/**
	 * Creates a small icon for a calendar.
	 */
	function _parseCalendar()
	{
		$icon = defined('CALENDAR_ICON')?CALENDAR_ICON:'icon_cal.gif';
		return '<img src="'.REL_IMAGES.$icon.'" align="absmiddle" hspace="2" border="0" />';
	}
	/**
	 * Creates the hour and minute selection boxes for a date-selector
	 */
	function _parseHourSelector()
	{
		$rv = '';
		// Hour selection
		$rv .= '<select name="'.$this->name.'[hour]"';
		foreach ( $this->properties  as $prop=>$val )
			$rv .= " $prop=\"$val\"";
		$rv .='>';
		$this->options = array( array('value'=>'','text'=>'-'));
		for( $i = 0; $i < 24; $i++ )
		{
			$this->options[] = array('value'=>$i,'text'=>str_pad($i, 2, '0', STR_PAD_LEFT));
			if( isset( $this->selectedHour ) && $this->selectedHour == $i )
				$this->options[count($this->options)-1]['selected'] = 'selected';
		}
		$name = $this->name;
		$this->name = $this->name.'[hour]';
		$rv .= $this->_getOptions().'</select> ';
		$this->name = $name;

		// Minute selection
		$rv .= '<select name="'.$this->name.'[minute]"';
		foreach ( $this->properties  as $prop=>$val )
			$rv .= " $prop=\"$val\"";
		$rv .= '>';
		$this->options = array( array('value'=>'','text'=>'-'));
		for( $i = 0; $i < 60; $i+=15 )
		{
			$this->options[] = array('value'=>$i,'text'=>str_pad($i, 2, '0', STR_PAD_LEFT));
			if( isset( $this->selectedMinute ) && $this->selectedMinute <= $i )
				$this->options[count($this->options)-1]['selected'] = 'selected';
		}
		$name = $this->name;
		$this->name = $this->name.'[hour]';
		$rv .= $this->_getOptions().'</select> ';
		$this->name = $name;

		return $rv;
	}
	/**
	 * Create <option>'s
	 * @return string '<option></option>'...
	 */
	function _getOptions($opt='options', $xtrprops=array())
	{
		$selectedValue = array();
		if($this->parent->parent->_setValues)
		{
			if(false===strpos($this->name,'[')&&isset($_POST[$this->name]))
				$selectedValue[] = $_POST[$this->name];
			elseif(false!==strpos($this->name,'[]'))
			{
				// This is non-exploitable eval code.
				$postVar = "['".implode("']['", explode('[',str_replace(']','',str_replace('[]','',$this->name))))."']";
				$test = '';
				eval('$test = isset($_POST'.$postVar.')?$_POST'.$postVar.':"";');
				if($test)
					$selectedValue = $test;
			}
			else
			{
				// The name is an array, eval it and check Post data
				// This is non-exploitable eval code.
				$postVar = "['".implode("']['", explode('[',str_replace(']','',$this->name)))."']";
				$test = '';
				eval('$test = isset($_POST'.$postVar.')?$_POST'.$postVar.':"";');
				if($test)
					$selectedValue[] = $test;
			}
		}

		$rv = '';
		if ( $doopts = (isset($this->optgroups)&&count($this->optgroups)))
		{
			if( isset( $this->firstOption ))
			{
				$rv .= '<option';
				foreach ( $this->firstOption as $prop=>$pval )
				{
					if ( $prop == 'text' )
					{
						$text = $pval;
						continue;
					}
					elseif($prop == 'value' && in_array($pval,$selectedValue))
						$rv .= ' selected="selected"';
					elseif($prop == 'selected')
					{
						if($pval)
							$rv .= ' selected="selected"';
						continue;
					}

					$rv .= ' '.$prop.'="'.$pval.'"';
				}
				$rv .= '>'.$text.'</option>';
			}
			list($label, $options) = array_shift($this->optgroups);
		}
		else
		{
			$options = isset($this->$opt)?$this->$opt:array();
			$this->optgroups = array();
		}

		do
		{
			if ( $doopts )
				$rv .= '<optgroup label="'.$label.'">';
			foreach ( $options as $map )
			{
				$text = '';
				$rv .= '<option';
				foreach ( array_merge($map, $xtrprops) as $prop=>$pval )
				{
					if ( $prop == 'text' )
					{
						$text = $pval;
						continue;
					}
					elseif($prop == 'value' && in_array($pval,$selectedValue))
						$rv .= ' selected="selected"';
					elseif($prop == 'selected')
					{
						if($pval)
							$rv .= ' selected="selected"';
						continue;
					}

					$rv .= ' '.$prop.'="'.$pval.'"';
				}
				$rv .= '>'.$text.'</option>';
			}
			if ( $doopts )
				$rv .= '</optgroup>';
		}while ( list($label, $options) = array_shift($this->optgroups) );
		return $rv;
	}
	/**
	 * Defines the different types of items
	 */
	function _defineTypes()
	{
		foreach ( $this->getTypes() as $name=>$val )
		{
			if ( defined($name) )
				return;
			define($name, $val);
		}
	}
	/**
	 *
	 */
	var $_BarButtons = 7;
	/**
	 *
	 */
	var $_CustomButtons = array();
	/**
	 *
	 */
	var $_ButtonOptions = array();
	/**
	 *
	 */
	function _parseButtonBar()
	{
		$rv = '<div style="width: 100%; text-align: right;">';
		foreach($this->_CustomButtons as $button)
			$rv .= $button;
		foreach ( array(BUTTON_SAVE=>'Save', BUTTON_CANCEL=>'Cancel', BUTTON_DELETE=>'Delete') as $button=>$text )
		{
			if ( $this->_BarButtons & $button )
			{
				$this->_ButtonOptions[$button]['type'] = $button == BUTTON_SAVE?'submit':(isset($this->_ButtonOptions[$button]['type'])?$this->_ButtonOptions[$button]['type']:'button');
				$this->_ButtonOptions[$button]['value'] = $this->langIf->getText($text);
				!isset($this->_ButtonOptions[$button]['id']) &&	$this->_ButtonOptions[$button]['id']= 'ifI_'.$this->id.'_DB_'.$button; //DefaultButton
				$rv .= '<input style="min-width: 100px;" '.$this->_parseProps($this->_ButtonOptions[$button]).' /> ';

			}
		}
		return $rv.'</div>';
	}
	/**
	 * @param string $value
	 * @param string $type
	 * @param Array $props
	 * <code></code>
	 */
	function _addCustomButton($value,$type='submit',$props=array())
	{
		empty($props['id']) && $props['id'] = 'ifI_'.$this->id.'_CB_'.count($this->_CustomButtons); //CustomButton
		$this->_CustomButtons[] = '<input style="min-width: 100px;" type="'.$type.'" value="'.$value.'"'.$this->_parseProps($props).' /> ';
	}
	/**
	 *
	 */
	function _displayItemNoInput($classes=array(),$hidden)
	{
		$rv = '';
		$this->hiddenFields = array();

		switch($this->type_id)
		{
			case IF_ITEM_FANCY_SELECT:

				$selectedValue = array();
				$val = '';
				if($this->parent->parent->_setValues)
				{
					//_data
					$names = str_replace('[]','_data',$this->name);
					if(isset($_POST[$names]))
					{
						$val .= '<ul>';
						foreach($_POST[$names] as $value)
						{
							if(!in_array($value, $selectedValue))
							{
								$this->hiddenFields[] = array('name'=>$names.'[]','value'=>$value);
								$val .= '<li><span>'.$value.'</span></li>';
								$selectedValue[] = $value;
							}
						}
						$val .= '</ul>';
					}
				} else {
					$val .= '<ul>';
					foreach($this->properties['value'] as $value)
					{
						if(!in_array($value, $selectedValue))
						{
							$val .= '<li><span>'.$value.'</span></li>';
							$selectedValue[] = $value;
						}
					}
					$val .= '</ul>';
				}
				$rv[] = $hidden.$this->_parseHiddenFields().$val;
			break;
			case IF_ITEM_MULTI_SELECT:
			case IF_ITEM_DROPDOWN:
			case IF_ITEM_RADIO:

				$selectedValue = array();
				if($this->parent->parent->_setValues)
				{
					if(false===strpos($this->name,'[')&&isset($_POST[$this->name]))
						$selectedValue[] = $_POST[$this->name];
					elseif(false!==strpos($this->name,'[]'))
					{
						$postVar = "['".implode("']['", explode('[',str_replace(']','',str_replace('[]','',$this->name))))."']";
						$test = '';
						// This is non-exploitable eval code.
						eval('$test = isset($_POST'.$postVar.')?$_POST'.$postVar.':"";');
						if($test)
							$selectedValue = $test;
					}
					else
					{
						// The name is an array, eval it and check Post data
						$postVar = "['".implode("']['", explode('[',str_replace(']','',$this->name)))."']";
						// This is non-exploitable eval code.
						$test = '';
						eval('$test = isset($_POST'.$postVar.')?$_POST'.$postVar.':"";');
						if($test)
							$selectedValue[] = $test;
					}
				}
				switch ( $this->label_disposition )
				{
					case DISPOSITION_LEFT:
						$titpos = 0;
						$selpos = 1;
					break;
					case DISPOSITION_RIGHT:
						$titpos = 1;
					default:
						$selpos = 0;
					break;
				}
				isset($titpos) && $rv[$titpos] = $this->title;
				$rv[$selpos] = '';

				// if optgroups, merge all optgroups to $this->options
				if (isset($this->optgroups))
				{
					$this->options = array();
					foreach ($this->optgroups as $optgroups)
					{
						list($label, $options) = $optgroups;
						$this->options = array_merge($this->options, $options);
					}
				}

				if(isset($this->options)&&sizeof($this->options))
				{
					foreach($this->options as $map)
					{
						if(in_array($map['value'],$selectedValue)||((!$this->parent->parent->_setValues)&&(isset($map['selected'])&& $map['selected'])))
						{
							$rv[$selpos] .= '<span>'.$map['text'].'</span>';
							$this->hiddenFields[] = array('name'=>$this->name,'value'=>$map['value']);
						}
					}
				}
				$this->script = exportArrayToJS(isset($this->options) ? $this->options : array(), str_replace(array('[',']'), '_', $this->name.'Map'));
				$rv[$selpos] = $hidden.$this->_parseHiddenFields().$rv[$selpos];
			break;
			case IF_ITEM_PASSWORD:
				$this->hiddenFields[] = array('name'=>$this->name,'value'=>$this->properties['value']);
				$rv = array();
				$rv[] = $this->title;
				$rv[] = $hidden.$this->_parseHiddenFields().str_repeat('*',strlen($this->properties['value']));
			break;
			case IF_ITEM_CHECKBOX:
				$this->hiddenFields[] = array('name'=>$this->name,'value'=>$this->properties['value']);
				$rv = array();
				if($this->label_disposition==DISPOSITION_LEFT)
					$rv[] = $this->title;
				$rv[] = $hidden.$this->_parseHiddenFields().'<img src="/'.SITE_PATH.'images/i-w/'.($this->properties['value']?'true':'false').'.gif" />';
				if($this->label_disposition==DISPOSITION_RIGHT)
					$rv[] = $this->title;
			break;
			case IF_ITEM_ROW_CONTROL:
				$rv = array();
			break;
			case IF_ITEM_FILE:
				require_once(IWARE . 'Document.php');
				$d = $this->_getDocument();

				$show = '';
				if($d)
					$show = sprintf('<a href="/'.SITE_PATH.'scripts/i-w/document/download.php?Document[id]=%s">%s</a>', $d->id, $d->name);

				$this->hiddenFields[] = array('name'=>"Documents[".$this->id."]", 'value'=>($d?$d->id:''));

				$rv = array();
				$rv[] = $hidden.$this->_parseHiddenFields().$this->title;
				$rv[] = $show;

			break;

			case IF_ITEM_TEXT:
				if(isset($this->twoCols))
					$rv = array($hidden.(isset($this->text)?$this->text:''),'');
				elseif(isset($this->rightCol))
					$rv = array($this->rightCol,$hidden.(isset($this->text)?$this->text:''));
				else
					$rv .= $hidden.(isset($this->text)?$this->text:'');
				//$rv = array();
				//$rv[] = (isset($this->text)?$this->text:'');
			break;

			default:
	 			$this->properties['value'] = isset($this->properties['value'])?$this->properties['value']:'';
				if (!isset($this->no_hidden))
					$this->hiddenFields[] = array('name'=>$this->name,'value'=>$this->properties['value']);
				$rv = array();
				if($this->label_disposition==DISPOSITION_LEFT)
					$rv[] = $this->title;
				// @Mkprtty
				$rv[] = $hidden.$this->_parseHiddenFields().'<span>'.$this->properties['value'].'</span>';
				if($this->label_disposition==DISPOSITION_RIGHT)
					$rv[] = $this->title;
			break;

		}
		return $rv;
	}

	/**
	 * gets some help
	 */
	function _parseHelp()
	{
		if(!$this->help_enabled)
			return '';

		list($img,$script) = getInterfaceHelp($this->langIf, $this->handle, array('top'=>'2', 'right'=>'0', 'float'=>'', 'vertical-align'=>'-25%') );
		$this->postscript = $script;
		return $img;
	}
	function getDocument($name,$id=false,$actuallyMoveIt=true)
	{
		$this->name = $name;
		$this->id = $id;
		return $this->_getDocument($actuallyMoveIt);
	}
	/**
	 * @return Document
	 */
	function _getDocument($actuallyStoreIt=true)
	{
		// prevent warnings, these variables are created in eval()
		$name = $type = $tmp_name = $size = null;
		//need this in case we are dealing with an existing file
		$error = UPLOAD_ERR_OK;

		//this is friggin nasty... need to clean this up one day
		// Create the variables name, type, tmp_name, error, and size
		foreach( array('name', 'type', 'tmp_name', 'error', 'size') as $item)
		{
			if(false===($bracketpos = strpos($this->name,'['))) {
				$$item = isset($_FILES[$this->name])?$_FILES[$this->name][$item]:'';
			}
			else
			{
				$temp = substr($this->name,0,$bracketpos).'['.$item.']'.substr($this->name,$bracketpos);
				$postVar = "['".implode("']['", explode('[',str_replace(']','',$temp)))."']";
				// FIXME: possible exploit
				eval('$'.$item.' = isset($_FILES'.$postVar.')?$_FILES'.$postVar.':"";');
			}
		}

		/**
		 * Check for upload errors
		 */

		// cache translations
		$this->langIf->getText('The file exceeded the maximum filesize');
		$this->langIf->getText('The file was partially uploaded');
		$this->langIf->getText('No file was uploaded');
		$this->langIf->getText('Error while uploading file');

		switch ($error)
		{
			case UPLOAD_ERR_INI_SIZE:
			case UPLOAD_ERR_FORM_SIZE:
				throw new IWFileUploadException('The file exceeded the maximum filesize');
			case UPLOAD_ERR_PARTIAL:
				throw new IWFileUploadException('The file was partially uploaded');
			case UPLOAD_ERR_NO_FILE:
				if(strpos($this->behaviour, 'required') != false)
				    throw new IWFileUploadException('No file was uploaded');

			case UPLOAD_ERR_OK:
				break; // pass

			case UPLOAD_ERR_NO_TMP_DIR:
			case UPLOAD_ERR_CANT_WRITE:
			case UPLOAD_ERR_EXTENSION:
			default:
				trigger_error(sprintf('Cannot save uploaded file: Error [%s]', $error), E_USER_NOTICE); // make ticket
				throw new IWFileUploadException('Error while uploading file');
		}
		/*
		if($size > 500000)
			throw new IWFileUploadException('The file exceeded the maximum filesize');
		if($type == "application/x-msdos-program")
		    throw new IWFileUploadException('File type error.');
		*/
		//also check the that the file on the file system is the same one uploaded
		if($tmp_name && !is_uploaded_file($tmp_name))
		{
			trigger_error('Cannot save uploaded file: possible tampering with uploaded file', E_USER_NOTICE); // make ticket
			throw new IWFileUploadException('Error while uploading file');
		}

		//check file type
		if($tmp_name)
		{
			//$finfo = finfo_open(FILEINFO_MIME);
			//$mimetype = finfo_file($finfo, $tmp_name);
			//we allow:
			// PDF
			// IMAGES
			// PLAIN TEXT
			//if($mimetype != 'application/octet-stream' && $mimetype != 'application/pdf' && (strncmp($mimetype, 'image/', 6) !== 0) && (strncmp($mimetype, 'text/plain', 10) !== 0))
			//	throw new IWFileUploadException(sprintf('File type %s is not allowed.', $mimetype));
			if(!preg_match('/^.*\.(jpg|jpeg|png|gif|pdf|txt|csv)$/i',$name))
				throw new IWFileUploadException('Supplied file type is not allowed: '.$name);
		}

		/**
		 * If we have a value set for $_FILES, present the user with the option to select a new file.
		 * By default, disable the upload control and show two radio's with the first selected.
		 */
		require_once(IWARE . 'Document.php');
		$document = false;

		// See if a file is uploaded
		if (!empty($name))
		{
			// TODO: set to temp
			$document = Document::createDocument($this->db_app);
			$document->temp_name = $tmp_name;
			$document->name = basename($name);
			$document->temp_session_id = session_id();
			if ($actuallyStoreIt && !$document->insert())
				trigger_error('Error while inserting document to database', E_USER_ERROR);
		}
		else if (!empty($_REQUEST['Documents'][$this->id]))// See if we have a file id in the radio
		{
			//$document = new Document($this->db_app);
			$document = Document::createDocument($this->db_app);
			$document->set($_REQUEST['Documents'][$this->id]);
		}

		return $document;

	}
	function _getHideInput()
	{
		//debug('checkediecheck _getHideInput(): '.$this->if_obj->check);

		if(isset($this->_hideInput))
			return $this->_hideInput;
		if(isset($this->parent->_hideInputs))
			return $this->parent->_hideInputs;
		if(isset($this->if_obj->_hideInputs))
			return $this->if_obj->_hideInputs;
		return false;
	}
	/**#@-*/
}

/**
 * Exceptions
 */
class IWFileUploadException extends IWException {}

/**#@+
 * Default button defines
 */
define('BUTTON_SAVE',1);
define('BUTTON_CANCEL',2);
define('BUTTON_DELETE',4);
/**#@-*/
/**#@+
 * Label Disposition Defines
 */
define('DISPOSITION_TOP',1);
define('DISPOSITION_RIGHT',2);
define('DISPOSITION_BOTTOM',3);
define('DISPOSITION_LEFT',4);
/**#@-*/
?>
