<?php
/**
 * interfaceObject
 *
 * @author Marten Koedam <marten@basesoft.com>
 * @author Chris Wesseling <chris.wesseling@xs4all.nl>
 * @copyright (2005) BaseSoft B.V.
 * @since 17-05-2005
 * @version $Id$
 * @package I-Ware
 * @subpackage Core
 */
class InterfaceObject
{
	/**
	 * @var Database $db IWARE Database
	 */
	var $db;
	/**
	 * @var Database $db_app APPLICATION Database
	 */
	var $db_app;
	/**
	 * @var LanguageInterface $langIf
	 */
	var $langIf = '';
	/**
	 * @var Array $hiddenFields
	 */
	var $hiddenFields =  array();
	/**
	 * Constructor
	 * @param Database $db IWARE Database
	 * @param Database $db_app APPLICATION Database
	 */
	function InterfaceObject($db,$db_app,$langIf)
	{
		$this->db = $this->db_if = $db;
		$this->db_app = $db_app;
		$langIf && $this->setItemTranslator($langIf);
		$this->uid = $_SESSION['session_user_id'];
		$this->gcid = $_SESSION['session_group_company_id'];
	}
	/**
	 * Sets the active user
	 * Used for filtering and privilege level etc.
	 * @param int $uid USER_ID
	 */
	function setUserId($uid)
	{
		$this->uid = $uid;
	}
	/**
	 * Updates the default settings of a user for a block
	 * @param int $bid BLOCK_ID
	 * @param int $open
	 */
	function toggleBlock($bid, $open)
	{
		if ( $open )
			$sql = "DELETE FROM TBL_USER_CLOSED_BLOCKS
				WHERE USER_ID = ? AND BLOCK_ID = ?";
		else
			$sql = "REPLACE INTO TBL_USER_CLOSED_BLOCKS (USER_ID, BLOCK_ID)
				VALUES (?, ?)";
		$q = $this->db_app->query($sql);
		$q->bind_values(array($this->uid, $bid));
		$q->execute();
	}
	/**
	 * Sets the active GroupCompany
	 * Used for filtering and privilege level etc.
	 * @param int $gcid GROUP_COMPANY_ID
	 */
	function setGroupCompanyId($gcid)
	{
		$this->gcid = $gcid;
	}
	/**
	 * Sets object properties from array/object
	 * @param array $props ('property'=>value)
	 */
	function preset($props)
	{
		foreach ( $props as $prop=>$val )
			$this->$prop = $val;
	}
	/**
	 * Sets the translator for the items of the interface
	 * @param LanguageInterface $langIf
	 */
	function setItemTranslator(&$langIf)
	{
		$this->langIf =& $langIf;
	}
	/**
	 * Save/Update Interface
	 * @param array $if InterfaceMap (handle=>'', id=>(0|INTERFACE_ID)
	 * @return int INTERFACE_ID
	 */
	function saveInterface($if)
	{
		$if['id'] = isset($if['id'])?$if['id']:null;

		$sql = "REPLACE INTO TBL_INTERFACE (INTERFACE_ID, HANDLE, CLASS, PATH)
			VALUES (?, ?, ?, ?)";
		$q = $this->db->query($sql);
		$q->bind_values(array($if['id'], $if['handle'],$if['class'],$if['path']));
		if ( $q->execute(true) )
			return ($if['id']?$if['id']:$q->lastid());
		return false;
	}
	/**
	 * Delete Interface
	 * @param array $if InterfaceMap (handle=>'', id=>(0|INTERFACE_ID)
	 * @return bool
	 */
	function deleteInterface($if)
	{
		// Delete blocks
		$sql = "DELETE FROM TBL_INTERFACE WHERE INTERFACE_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values(array($if['id']));
		if ( $q->execute(true) )
			return true;
		return false;
	}
	/**#@+
	 * @access private
	 */
	/**
	 * @var int $uid Active USER_ID
	 */
	var $uid = 0;
	/**
	 * @var int $gcid Active GROUP_COMPANY_ID
	 */
	var $gcid = 0;
	/**
	 * Filter the selected interface on Rights and Opened interfaceBlocks
	 * @param Query $q
	 * @see function::_select()
	 */
	function _filter($q)
	{
		$rv = array();
		$blocks = array(0=>0);
		$items = array(0=>0);
		while ( $r = $q->fetch() )
		{
			$rv[$r->INTERFACE_ID]['HANDLE'] = $r->INTERFACE_HANDLE;
			$rv[$r->INTERFACE_ID][$r->BLOCK_ID]['HANDLE'] = $r->BLOCK_HANDLE;
			$rv[$r->INTERFACE_ID][$r->BLOCK_ID]['TYPE'] = $r->BLOCK_TYPE;
			$rv[$r->INTERFACE_ID][$r->BLOCK_ID][$r->ITEM_ID] = array('NAME'=>$r->ITEM_NAME,
					'HANDLE'=>$r->ITEM_HANDLE,
					'TYPE'=>$r->ITEM_TYPE,
					'CALLBACK'=>$r->CALLBACK,
					'CLOSE_ROW'=>$r->CLOSE_ROW,
					'BEHAVIOUR'=>$r->BEHAVIOUR,
					'LABEL_DISPOSITION'=>$r->LABEL_DISPOSITION,
					'HELP_ENABLED'=>$r->HELP_ENABLED);
			$blocks[$r->BLOCK_ID] = $r->INTERFACE_ID ;
			$items[$r->ITEM_ID] = null;
		}
		$sql = "SELECT gcp.INTERFACE_ID, gcp.BLOCK_ID, gcp.ITEM_ID, gcp.PRIV
			FROM TBL_GROUP_COMPANY_PRIVILEGES as gcp
			WHERE gcp.GROUP_COMPANY_ID = ?
				AND gcp.INTERFACE_ID IN (".implode(',', array_keys($rv)).")
				AND gcp.BLOCK_ID IN (".implode(',', array_keys($blocks)).")
				AND gcp.ITEM_ID IN (".implode(',', array_keys($items)).")
			ORDER BY gcp.INTERFACE_ID, gcp.BLOCK_ID, gcp.ITEM_ID";
		$q = $this->db->query($sql);
		$q->bind_values(array($this->gcid));
		$q->execute();
		$rs = $q->fetchAll();
		if ( !count($rs))
			return array();
		foreach($rs as $r)
		{
			if ( $r->ITEM_ID && isset($rv[$r->INTERFACE_ID][$r->BLOCK_ID][$r->ITEM_ID]) )
			{
				if ( !$r->PRIV )
					unset($rv[$r->INTERFACE_ID][$r->BLOCK_ID][$r->ITEM_ID]);
				else
					$rv[$r->INTERFACE_ID][$r->BLOCK_ID][$r->ITEM_ID]['PRIV'] = $r->PRIV;
			}
			elseif ( $r->BLOCK_ID && isset($rv[$r->INTERFACE_ID][$r->BLOCK_ID]) )
			{
				if ( !$r->PRIV )
					unset($rv[$r->INTERFACE_ID][$r->BLOCK_ID]);
				else
					$rv[$r->INTERFACE_ID][$r->BLOCK_ID]['PRIV'] = $r->PRIV;
			}
			else
			{
				if ( !$r->PRIV )
					unset($rv[$r->INTERFACE_ID]);
				else
					$rv[$r->INTERFACE_ID]['PRIV'] = $r->PRIV;
			}
		}
		$sql = "SELECT ucb.BLOCK_ID
			FROM TBL_USER_CLOSED_BLOCKS as ucb
			WHERE ucb.USER_ID = ? AND BLOCK_ID IN (".implode(', ', array_keys($blocks)).")";
		$q = $this->db_app->query($sql);
		$q->bind_values(array($this->uid));
		$q->execute();
		while ( $r = $q->fetch() )
			$rv[$blocks[$r->BLOCK_ID]][$r->BLOCK_ID]['CLOSED'] = true;

		return $rv;
	}
	/**
	 * Selects all information of the interface and it's blocks
	 * @param int $id INTERFACE_ID
	 * @return Query $q
	 */
	function _selectInterface($id)
	{
		return $this->_select($id);
	}
	/**
	 * Selects all information of the block and it's items
	 * @param int $id BLOCK_ID
	 * @return Query $q
	 */
	function _selectBlock($id)
	{
		return $this->_select($id, "IB.BLOCK");
	}
	/**
	 * Selects all information of the item
	 * @param int $id ITEM_ID
	 * @return Query $q
	 */
	function _selectItem($id)
	{
		return $this->_select($id, "II.ITEM");
	}
	/**
	 * Selects all the necessary parts of the interfaces
	 * @param int $id
	 * @param string $type
	 * @return Query $q
	 */
	function _select($id, $type='I.INTERFACE')
	{
		$sql = 'SELECT I.INTERFACE_ID, I.HANDLE as INTERFACE_HANDLE,
				IB.BLOCK_ID, IB.HANDLE as BLOCK_HANDLE, IB.TYPE as BLOCK_TYPE,
				II.ITEM_ID, II.HANDLE as ITEM_HANDLE, II.TYPE as ITEM_TYPE,
				II.NAME as ITEM_NAME, II.CALLBACK, II.CLOSE_ROW,
				II.BEHAVIOUR, II.LABEL_DISPOSITION, II.HELP_ENABLED
			FROM TBL_INTERFACE as I
			INNER JOIN TBL_INTERFACE_BLOCK as IB
				USING( INTERFACE_ID )
			INNER JOIN TBL_INTERFACE_ITEM as II
				USING( BLOCK_ID )
			WHERE '.$type.'_ID = ?
			ORDER BY IB.POSITION, II.POSITION';
		$q = $this->db_if->query($sql);
		$q->bind_values( array( $id ));
		$q->execute();
		return $q;
	}
	/**#@-*/
	/**
	 *
	 */
	function generateClassFile($interface_id,$show_extra=false)
	{
		$rv = "<?php\n".
			"require_once(INTERFACE_OBJ.'InterfaceFactory.php');\n".
			"/**\n * \n".
			" * @package \n".
			" * @subpackage Interface\n".
			" * @since ".date('d-m-Y')."\n".
			" * @copyright ".date('Y')." BaseSoft B.V.\n".
			" * @version \$Id\$\n".
			" */\n";
		$q = $this->_select($interface_id);

		$extra = $settings = $callbacks = array();
		$i = 0;
		while($r = $q->fetch())
		{
			if($i == 0)
			{
				$rv .= "class ".$r->INTERFACE_HANDLE." extends InterfaceFactory\n{\n".
					"\t/**\n".
					"\t * interface[id] = ".$r->INTERFACE_ID."\n".
					"\t * @param \$db Database (I_WARE_SETTINGS)\n".
					"\t * @param \$db_app Database (APPLICATION DATABASE)\n".
					"\t * @param \$langIf LanguageInterface\n".
					"\t */\n".
					"\tfunction ".$r->INTERFACE_HANDLE."(&\$db, &\$db_app, &\$langIf)\n".
					"\t{\n".
					"\t\tparent::InterfaceFactory(\$db, \$db_app, \$langIf);\n".
					"\t\t\$this->FirstColWidth = 25;\n".
					"\t\t\$this->delimiter = '<br />';\n".
					"\t}\n".
					"\t/**\n".
					"\t * @param string &\$thrownException\n".
					"\t */\n".
					"\tfunction processInput(&\$thrownException)\n\t{\n\t\t\n\t}\n}\n".
					"/**\n".
					" * @package \n".
					" * @subpackage Interface\n".
					" * @since ".date('d-m-Y')."\n".
					" * @copyright ".date('Y')." BaseSoft B.V.\n".
					" * @version \$Id\$\n".
					" */\n".
					"class ".$r->INTERFACE_HANDLE."Item extends InterfaceItem\n{\n".
					"\n".
					"";
			}
			if(empty($r->CALLBACK))
				continue;
			if(!in_array($r->CALLBACK,$callbacks))
			{
				$rv .=
					"\t/**\n".
					"\t * %callback".$r->CALLBACK."%\n".
					"\t */\n".
					"\tfunction ".$r->CALLBACK."()\n\t{\n".
					"\t}\n\n";
				$callbacks[] = $r->CALLBACK;
			}
			$settings[$r->CALLBACK][$r->ITEM_ID] = $r->ITEM_ID."] = ".$r->ITEM_NAME;
			$extra[$r->CALLBACK][$r->ITEM_ID] = " * item[".$r->ITEM_ID."] = {\n".
				" * \t[handle] = ".$r->ITEM_HANDLE."\n".
				" * \t[behaviour] = ".$r->BEHAVIOUR."\n".
				" * \t[help] = ".$r->HELP_ENABLED."\n".
				" * \t[close_row] = ".$r->CLOSE_ROW."\n".
				" * \t[label_pos] = ".$r->LABEL_DISPOSITION."\n".
				" * \n";
			$i++;
		}

		foreach($settings as $callback => $items)
			$rv = str_replace('%callback'.$callback.'%', 'item['.implode("\n\t * item[",$items), $rv);

		$rv .= "}";
		if(true)
		{
			$rv .= "\n\n/**\n";
			foreach($extra as $callback => $items)
				$rv .= " * CALLBACK[".$callback."]\n".implode("",$items);
			$rv .= " */";
		}
		$rv .= "\n?>";
		return highlight_string($rv, true);
	}

	/**
	 * Parses hidden fields
	 * @return string $rv
	 */
	function _parseHiddenFields()
	{
		$rv = '';
		if( empty( $this->hiddenFields ) || ! is_array( $this->hiddenFields ))
			return $rv;
		foreach( $this->hiddenFields as $params )
			$rv .= '<input type="hidden" '.$this->_parseProps($params).' />'."\n";
		return $rv;
	}

	/**
	 * Parse $this->properties array
	 * @param Array $properties (PROPERTY=>VALUE)
	 * @return string ' $key="$value"' ...
	 */
	function _parseProps( $properties = '' )
	{
		$properties || $properties = $this->properties;
		$rv = '';
		foreach ( $properties as $prop=>$val )
		{
			switch($prop) {
				case 'selected':
					if ($val)
						$rv .= ' selected="selected"';
					break;
				case 'checked':
					if ($val)
						$rv .= ' checked="checked"';
					break;
				default:
					$rv .= ' '.$prop.'="'.htmlentities($val,ENT_COMPAT,'UTF-8').'"';
			}
		}
		return $rv;
	}

}

/* IWARE DATABASE

CREATE TABLE `TBL_INTERFACE`
(
	`INTERFACE_ID` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
	`HANDLE` VARCHAR(255) NOT NULL,

	PRIMARY KEY( `INTERFACE_ID` )
);

CREATE TABLE `TBL_INTERFACE_BLOCK`
(
	`BLOCK_ID` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
	`INTERFACE_ID` INT(11) UNSIGNED NOT NULL,
	`POSITION` TINYINT(3) UNSIGNED NOT NULL,
	`TYPE` TINYINT(3) UNSIGNED NOT NULL DEFAULT 1,
	`HANDLE` VARCHAR(255) NOT NULL,

	PRIMARY KEY( `BLOCK_ID` )
);

CREATE TABLE `TBL_INTERFACE_ITEM`
(
	`ITEM_ID` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
	`BLOCK_ID` INT(11) UNSIGNED NOT NULL,
	`POSITION` TINYINT(3) UNSIGNED NOT NULL,
	`TYPE` TINYINT(3) UNSIGNED NOT NULL,
	`CALLBACK` VARCHAR(255) NOT NULL,
	`NAME` VARCHAR(255) NOT NULL,
	`HANDLE` VARCHAR(255) NOT NULL,

	PRIMARY KEY( `ITEM_ID` )
);

*/

/* APPLICATION DATABASE

CREATE TABLE TBL_USER_CLOSED_BLOCKS
(
	USER_ID INT(11) UNSIGNED NOT NULL,
	BLOCK_ID INT(11) UNSIGNED NOT NULL,

	PRIMARY KEY( USER_ID, BLOCK_ID )
);

CREATE TABLE TBL_GROUP_COMPANY_PRIVILEGES
(
	GROUP_COMPANY_ID INT(11) UNSIGNED NOT NULL,
	INTERFACE_ID INT(11) UNSIGNED NOT NULL,
	BLOCK_ID INT(11) UNSIGNED NOT NULL,
	ITEM_ID INT(11) UNSIGNED NOT NULL,
	PRIV TINYINT(3) UNSIGNED NOT NULL DEFAULT 1,

	PRIMARY KEY (GROUP_COMPANY_ID, INTERFACE_ID, BLOCK_ID, ITEM_ID)
);

*/

?>
