<?php
/**
 * @package Interface
 * @subpackage InterfacePermissions
 * @since 17-jul-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class InterfacePermissions
{
	protected $skipChecks = array();
	protected $universe = 'public';

	function __construct($universe,$skipChecks)
	{
		$this->skipChecks = $skipChecks;
		$this->universe = ucfirst($universe);

		foreach($_GET as $key => $value)
		{
			if(isset($_POST[$key]) && $_POST[$key] !== $value)
				throw new InterfacePermissionException(sprintf('Attempted Spoofing of GET/POST value for [%s] ', $key));
		}

		foreach($_POST as $key => $value)
		{
			if(isset($_GET[$key]) && $_GET[$key] !== $value)
				throw new InterfacePermissionException(sprintf('Attempted Spoofing of GET/POST value for [%s] ', $key));
		}
	}
	function validate()
	{
		$func = 'get'.$this->universe.'Permissions';
		$obj = 'Interface'.$this->universe.'PermissionCheck';
		$obj = new $obj();
		foreach($this->$func() as $perm)
		{
			if(in_array($perm,$this->skipChecks))
				continue;
			$perm = ucfirst($perm);
			$obj->$perm();
		}
	}

	function getSysadminPermissions()
	{
		return array();
	}
	/**
	 * @deprecated
	 */
	function getOperatorPermissions()
	{
		return array();
	}
	function getAccountantPermissions() {
		return array(
			'crsoid',
			'participantid'
		);
	}
	function getCrsoPermissions()
	{
		return array(
			'crsoid',
			'participantid',
			'trustid',
			'subrogatorid',
			'trusteeid',
			'userid');
	}
	function getParticipantPermissions()
	{
		return array(
			'participantid',
			'refundid',
			'popid',
			'refundexportid',
			'declarationid',
			'subrogatorid',
			'trusteeid',
			'trustid',
			'crsoid',
			'userid');
	}
	function getPublicPermissions()
	{
		return array();
	}
	function getDefaultPermissions()
	{
		return array();
	}
}
/**
 *
 */
class InterfaceDefaultPermissionCheck
{
	function __call($name,$args)
	{
		if(method_exists($this,'_validate'.$name))
			$this->{'_validate'.$name}();
		else
			throw new InterfacePermissionException(sprintf('Undefined function for permission check [%s]',$name));
	}
}
/**
 *
 */
class InterfacePublicPermissionCheck extends InterfaceDefaultPermissionCheck {}
/**
 *
 */
class InterfaceSysadminPermissionCheck extends InterfaceDefaultPermissionCheck {}
/**
 * @deprecated Operator is no longer part of the Black Box, v2.0
 */
class InterfaceOperatorPermissionCheck extends InterfaceDefaultPermissionCheck
{
}
class InterfaceAccountantPermissionCheck extends InterfaceDefaultPermissionCheck {

	function _validateCrsoid() {

		$crso_id = array();
		if(!empty($_REQUEST['crso']['id']))
			$crso_id[] = $_REQUEST['crso']['id'];
		if(!empty($_REQUEST['crso_id']))
			$crso_id[] = $_REQUEST['crso_id'];
		if(!empty($_REQUEST['crso']['relation']['id']))
			$crso_id[] = $_REQUEST['crso']['relation']['id'];

		if(!$crso_id)
			return;

		$crso_id = array_unique($crso_id);
		$sql = "SELECT CRSO_RELATION_ID FROM TBL_ACCOUNTANT_CRSO WHERE ACCOUNTANT_RELATION_ID = ? AND CRSO_RELATION_ID IN (".implode(',', $crso_id).")";
		$q = IWRegistry::registry('db')->query($sql);
		$q->bind_values($_SESSION['accountant_relation_id']);
		$q->execute();

		$row = $q->fetchAll();
		if(!count($row)||(count($row)!=count($crso_id)))
			throw new InterfacePermissionException(sprintf('Accountant trying to access unauthorized CRSO id(s) [%s]',implode(',',$crso_id)));
	}
	function _validateParticipantid()
	{
		$id = array();

		empty($_REQUEST['participant']['id']) || $id[] = $_REQUEST['participant']['id'];
		empty($_REQUEST['participant_id']) || $id[] = $_REQUEST['participant_id'];
		if(!$id)
			return;

		$id = array_unique($id);

		$sql = "SELECT DISTINCT PARTICIPANT_RELATION_ID
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CPP
			INNER JOIN TBL_ACCOUNTANT_CRSO as AC
				ON CPP.CRSO_RELATION_ID = AC.CRSO_RELATION_ID
			WHERE PARTICIPANT_RELATION_ID IN (".implode(',',$id).")
				AND ACCOUNTANT_RELATION_ID = ?";
		$q = IWRegistry::registry('db')->query($sql);
		$q->bind_values($_SESSION['accountant_relation_id']);
		$q->execute();

		$row = $q->fetchAll();
		if(!count($row)||(count($row)!=count($id)))
			throw new InterfacePermissionException(sprintf('Accountant trying to access unauthorized Participant id(s) [%s]',implode(',',$id)));
	}

}
/**
 *
 */
class InterfaceCrsoPermissionCheck extends InterfaceDefaultPermissionCheck
{
	function _validateCrsoid()
	{
		if(!empty($_REQUEST['crso']['id']))
			if($_REQUEST['crso']['id']!=$_SESSION['crso_relation_id'])
				throw new InterfacePermissionException(sprintf('CRSO trying to access unauthorized CRSO id [%s] (1)',$_REQUEST['crso']['id']));
		if(!empty($_REQUEST['crso_id']))
			if($_REQUEST['crso_id']!=$_SESSION['crso_relation_id'])
				throw new InterfacePermissionException(sprintf('CRSO trying to access unauthorized CRSO id [%s] (2)',$_REQUEST['crso_id']));
		if(!empty($_REQUEST['crso']['relation']['id']))
			if($_REQUEST['crso']['relation']['id']!=$_SESSION['crso_relation_id'])
				throw new InterfacePermissionException(sprintf('CRSO trying to access unauthorized CRSO id [%s] (3)',$_REQUEST['crso_id']));
	}
	function _validateParticipantid($id = array())
	{
		$id || $id = array();

		empty($_REQUEST['participant']['id']) || $id[] = $_REQUEST['participant']['id'];
		empty($_REQUEST['participant_id']) || $id[] = $_REQUEST['participant_id'];
		if(!$id)
			return;

		$id = array_unique($id);

		$sql = "SELECT DISTINCT PARTICIPANT_RELATION_ID
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CPP
			WHERE PARTICIPANT_RELATION_ID IN (".implode(',',$id).") AND CRSO_RELATION_ID = ?";
		$q = IWRegistry::registry('db')->query($sql);
		$q->bind_values($_SESSION['crso_relation_id']);
		$q->execute();

		$row = $q->fetchAll();
		if(!count($row)||(count($row)!=count($id)))
			throw new InterfacePermissionException(sprintf('CRSO trying to access unauthorized Participant id(s) [%s]',implode(',',$id)));
	}
	function _validateUserid()
	{
		$id = array();
		empty($_REQUEST['user']['id']) || $id[] = (int) $_REQUEST['user']['id'];
		empty($_REQUEST['user_id']) || $id[] = (int) $_REQUEST['user_id'];

		if(!$id)
			return;

		$id = array_unique($id);
		$correct = array();
		// Get the CRSO users
		$sql = "SELECT DISTINCT USER_ID FROM TBL_RELATION_USER
			WHERE RELATION_ID = ?";
		$q = IWRegistry::registry('db')->query($sql);
		$q->bind_values($_SESSION['crso_relation_id']);
		$q->execute();
		$row = $q->fetchAll();
		if(count($row))
			foreach($row as $r)
				$correct[] = $r->USER_ID;

		// Get the CRSO-Participant Users
		$sql = "SELECT DISTINCT USER_ID
			FROM TBL_RELATION_USER as RU
			INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT as CPP
				ON RU.RELATION_ID = CPP.PARTICIPANT_RELATION_ID
			WHERE CRSO_RELATION_ID = ?";
		$q = IWRegistry::registry('db')->query($sql);
		$q->bind_values($_SESSION['crso_relation_id']);
		$q->execute();
		$row = $q->fetchAll();
		if(count($row))
			foreach($row as $r)
				$correct[] = $r->USER_ID;

		foreach($id as $check)
			if(!in_array($check,$correct))
				throw new InterfacePermissionException(sprintf('CRSO trying to acces unauthorized User id [%s]',$check));
	}
	function _validateTrustid()
	{
		$id = array();
		empty($_REQUEST['trust']['id']) || $id[] = (int)$_REQUEST['trust']['id'];
		empty($_REQUEST['trust_id']) || $id[] = (int)$_REQUEST['trust_id'];
		if(!$id)
			return;

		$id = array_unique($id);

		$sql = "SELECT DISTINCT TRUST_ID, SUBROGATOR_RELATION_ID
			FROM TBL_TRUST
			WHERE TRUST_ID IN(".implode(',',$id).") AND CRSO_RELATION_ID = ?";
		$q = IWRegistry::registry('db')->query($sql);
		$q->bind_values($_SESSION['crso_relation_id']);
		$q->execute();
		$row = $q->fetchAll();
		if(!count($row)||(count($row)!=count($id)))
			throw new InterfacePermissionException(sprintf('CRSO trying to access unauthorized Trust id(s) [%s] (1)',implode(',',$id)));
		if(!isset($_REQUEST['subrogator']['id']))
			return;
		foreach($row as $r)
			if($r->SUBROGATOR_RELATION_ID != $_REQUEST['subrogator']['id'])
				throw new InterfacePermissionException(sprintf('CRSO trying to access unauthorized Trust id(s) [%s] (2)',implode(',',$id)));
	}
	function _validateSubrogatorid()
	{
		$id = array();
		empty($_REQUEST['subrogator']['id']) || $id[] = (int)$_REQUEST['subrogator']['id'];
		empty($_REQUEST['subrogator_id']) || $id[] = (int)$_REQUEST['subrogator_id'];
		$this->_validateParticipantid($id);
	}
	function _validateTrusteeid()
	{
		$id = array();
		empty($_REQUEST['trustee']['id']) || $id[] = (int)$_REQUEST['trustee']['id'];
		empty($_REQUEST['trustee_id']) || $id[] = (int)$_REQUEST['trustee_id'];
		$this->_validateParticipantid($id);
	}
}
/**
 *
 */
class InterfaceParticipantPermissionCheck extends InterfaceDefaultPermissionCheck
{
	function _validateParticipantid()
	{
		$ids = array();
		empty($_REQUEST['participant']['id']) || $ids[] = (int) $_REQUEST['participant']['id'];
		empty($_REQUEST['participant_id']) || $ids[] = (int) $_REQUEST['participant_id'];
		empty($_REQUEST['participant']['relation']['id']) || $ids[] = (int) $_REQUEST['participant']['relation']['id'];
		if(!$ids)
			return;

		$ids = array_unique($ids);

		// Check if there is an active/pending remote viewer request
		if(IWRegistry::isRegistered('remote_viewer')) {
			$remote = IWRegistry::registry('remote_viewer');
			// Make sure they dont access any stuff thats related to a CRSO other than the remote viewer
			if((count($ids)!=1) || ($remote->participant->relation->id!=$ids[0])) {
				throw new InterfacePermissionException(sprintf('Participant trying to access unauthorized Participant ids [%s] while in Remote Viewer mode',implode(',',$ids)));
			}
		}
		foreach($ids as $id) {
			if(!isset($_SESSION['participant_ids'][$id]))
				throw new InterfacePermissionException(sprintf('Participant trying to access unauthorized Participant ids [%s] (1)', implode(',', $ids)));
		}
	}
	function _validateCrsoid()
	{
		$id = array();
		// Check whether or not any of this users participant has access to the crsoid from the request.
		empty($_REQUEST['crso']['id']) || $id[] = (int)$_REQUEST['crso']['id'];
		empty($_REQUEST['crso_id']) || $id[]= (int)$_REQUEST['crso_id'];
		empty($_REQUEST['crso']['relation']['id']) || $id[] = (int)$_REQUEST['crso']['relation']['id'];
		if(!$id)
			return;

		$id = array_unique($id);

		// Check if there is an active/pending remote viewer request
		if(IWRegistry::isRegistered('remote_viewer')) {
			$remote = IWRegistry::registry('remote_viewer');
			// Make sure they dont access any stuff thats related to a CRSO other than the remote viewer
			if((count($id)!=1) || ($remote->crso->relation->id!=$id[0])) {
				throw new InterfacePermissionException(sprintf('Participant trying to access unauthorized CRSO ids [%s] while in Remote Viewer mode',implode(',',$id)));
			}
		}


		$sql = "SELECT CRSO_RELATION_ID
			FROM TBL_CRSO_PARTICIPANT
			WHERE PARTICIPANT_RELATION_ID IN (".implode(',',$_SESSION['participant_ids']).")
				AND CRSO_RELATION_ID IN (".implode(',',$id).")";
		$q = IWRegistry::registry('db')->query($sql);
		$q->execute();
		$row = $q->fetchAll();
		if(!count($row))
			throw new InterfacePermissionException(sprintf('Participant trying to access unauthorized CRSO ids [%s] (1)',implode(',',$id)));
	}
	function _validateUserid()
	{
		$id = array();
		empty($_REQUEST['user']['id']) || $id[] = (int) $_REQUEST['user']['id'];
		empty($_REQUEST['user_id']) || $id[] = (int) $_REQUEST['user_id'];

		if(!$id)
			return;

		$id = array_unique($id);

		$participant_ids = $_SESSION['participant_ids'];
		if(IWRegistry::isRegistered('remote_viewer')) {
			$remote = IWRegistry::registry('remote_viewer');
			$participant_ids = array($remote->participant->relation->id);
		}

		$sql = "SELECT DISTINCT USER_ID FROM TBL_RELATION_USER
			WHERE RELATION_ID IN(".implode(',',$participant_ids).")
				AND USER_ID IN (".implode(',',$id).")";
		$q = IWRegistry::registry('db')->query($sql);
		$q->execute();
		$row = $q->fetchAll();
		if(!count($row)||(count($row)!=count($id)))
			throw new InterfacePermissionException(sprintf('Participant trying to access unauthorized User id(s) [%s]',implode(',',$id)));
	}

	function _validateRefundid()
	{
		$rr_id = array();
		empty($_REQUEST['refund']['id']) || $rr_id[] = (int)$_REQUEST['refund']['id'];
		empty($_REQUEST['refund_request']['id']) || $rr_id[] = (int)$_REQUEST['refund_request']['id'];
		empty($_REQUEST['rr']['id']) || $rr_id[] = (int)$_REQUEST['rr']['id'];
		if(!$rr_id)
			return;

		$rr_id = array_unique($rr_id);

		$participant_ids = $_SESSION['participant_ids'];
		if(IWRegistry::isRegistered('remote_viewer')) {
			$remote = IWRegistry::registry('remote_viewer');
			$participant_ids = array($remote->participant->relation->id);
		}

		$sql = "SELECT PARTICIPANT_RELATION_ID FROM TBL_REFUND_REQUEST
			WHERE REFUND_REQUEST_ID IN (".implode(',',$rr_id).")
				AND PARTICIPANT_RELATION_ID IN(".implode(',', $participant_ids).")";
		$q = IWRegistry::registry('db')->query($sql);
		$q->execute();
		$row = $q->fetchAll();
		if(count($row)!=count($rr_id))
			throw new InterfacePermissionException(sprintf('Participant trying to access unauthorized Refund id(s) [%s]',implode(',',$rr_id)));
	}

	function _validateDeclarationid()
	{
		$id = array();
		empty($_REQUEST['declaration']['id']) || $id[] = (int)$_REQUEST['declaration']['id'];
		empty($_REQUEST['declaration_id']) || $id[] = (int)$_REQUEST['declaration_id'];
		empty($_REQUEST['decl']['id']) || $id[] = (int)$_REQUEST['decl']['id'];
		if(!$id)
			return;

		$id = array_unique($id);

		$participant_ids = $_SESSION['participant_ids'];
		if(IWRegistry::isRegistered('remote_viewer')) {
			$remote = IWRegistry::registry('remote_viewer');
			$participant_ids = array($remote->participant->relation->id);
		}

		$sql = "SELECT PARTICIPANT_RELATION_ID FROM TBL_DECLARATION
			WHERE DECLARATION_ID IN (".implode(',',$id).")
				AND PARTICIPANT_RELATION_ID IN (".implode(',', $participant_ids).")";
		$q = IWRegistry::registry('db')->query($sql);
		$q->execute();
		$row = $q->fetchAll();
		if(count($row)!=count($id))
			throw new InterfacePermissionException(sprintf('Participant trying to access unauthorized Declaration id(s) [%s]',implode(',',$id)));
	}

	function _validateRefundexportid()
	{
		// Not sure if this one is used or not
		$id = array();
		empty($_REQUEST['re']['id']) || $id[] = (int)$_REQUEST['re']['id'];
		empty($_REQUEST['refund_export']['id']) || $id[] = (int)$_REQUEST['refund_export']['id'];
		if(!$id)
			return;

		$id = array_unique($id);

		$participant_ids = $_SESSION['participant_ids'];
		if(IWRegistry::isRegistered('remote_viewer')) {
			$remote = IWRegistry::registry('remote_viewer');
			$participant_ids = array($remote->participant->relation->id);
		}

		$sql = "SELECT PARTICIPANT_RELATION_ID FROM TBL_REFUND_EXPORT
			WHERE REFUND_EXPORT_ID IN (".implode(',',$id).")
				AND PARTICIPANT_RELATION_ID IN (".implode(',', $participant_ids).")";
		$q = IWRegistry::registry('db')->query($sql);
		$q->execute();
		$row = $q->fetchAll();
		if(count($row)!=count($id))
			throw new InterfacePermissionException(sprintf('Participant trying to access unauthorized RefundExport id(s) [%s]',implode(',',$id)));
	}

	function _validatePopid()
	{
		$id = array();
		empty($_REQUEST['pop']['id']) || $id[] = (int)$_REQUEST['pop']['id'];
		empty($_REQUEST['pop_id']) || $id[] = (int)$_REQUEST['pop_id'];
		if(!$id)
			return;

		$id = array_unique($id);

		$participant_ids = $_SESSION['participant_ids'];
		if(IWRegistry::isRegistered('remote_viewer')) {
			$remote = IWRegistry::registry('remote_viewer');
			$participant_ids = array($remote->participant->relation->id);
		}

		$sql = "SELECT PARTICIPANT_RELATION_ID FROM TBL_PROOF_OF_PAYMENT
			WHERE PROOF_OF_PAYMENT_ID IN (".implode(',',$id).")
				AND PARTICIPANT_RELATION_ID IN (".implode(',', $participant_ids).")";
		$q = IWRegistry::registry('db')->query($sql);
		$q->execute();
		$row = $q->fetchAll();
		if(count($row)!=count($id))
			throw new InterfacePermissionException(sprintf('Participant trying to access unauthorized PoP id(s) [%s]',implode(',',$id)));
	}

	function _validateTrustid()
	{
		$id = array();
		empty($_REQUEST['trust']['id']) || $id[] = (int)$_REQUEST['trust']['id'];
		empty($_REQUEST['trust_id']) || $id[] = (int)$_REQUEST['trust_id'];
		if(!$id)
			return;

		$subrogator = empty($_REQUEST['subrogator']['id'])?false:(int)$_REQUEST['subrogator']['id'];
		$trustee = empty($_REQUEST['trustee']['id'])?false:(int)$_REQUEST['trustee']['id'];


		$sql = "SELECT SUBROGATOR_RELATION_ID, TRUSTEE_RELATION_ID
			FROM TBL_TRUST WHERE TRUST_ID IN(".implode(',',$id).")";
		$q = IWRegistry::registry('db')->query($sql);
		$q->execute();
		$row = $q->fetchAll();
		if(!count($row))
			throw new InterfacePermissionException(sprintf('Participant trying to access unauthorized Trust id(s) [%s] (1)',implode(',',$id)));

		foreach($row as $r)
		{
			if(!isset($_SESSION['participant_ids'][$r->SUBROGATOR_RELATION_ID])&&!isset($_SESSION['participant_ids'][$r->TRUSTEE_RELATION_ID]))
				throw new InterfacePermissionException(sprintf('Participant trying to access unauthorized Trust id(s) [%s] (2)',implode(',',$id)));
			if($trustee&&($trustee!=$r->TRUSTEE_RELATION_ID)&&!isset($_SESSION['participant_ids'][$r->TRUSTEE_RELATION_ID]))
				throw new InterfacePermissionException(sprintf('Participant trying to access unauthorized Trust id(s) [%s] (3)',implode(',',$id)));
			if($subrogator&&($subrogator!=$r->SUBROGATOR_RELATION_ID)&&!isset($_SESSION['participant_ids'][$r->SUBROGATOR_RELATION_ID]))
				throw new InterfacePermissionException(sprintf('Participant trying to access unauthorized Trust id(s) [%s] (4)',implode(',',$id)));
		}
	}

	function _validateSubrogatorid()
	{
		$id = array();
		empty($_REQUEST['subrogator']['id']) || $id[] = (int)$_REQUEST['subrogator']['id'];
		empty($_REQUEST['subrogator_id']) || $id[] = (int)$_REQUEST['subrogator_id'];

		if(!$id)
			return;

		foreach($id as $check)
			if(!isset($_SESSION['participant_ids'][$check]))
				throw new InterfacePermissionException(sprintf('Participant trying to access unauthorized Subrogator id(s) [%s]',implode(',',$id)));
	}

	function _validateTrusteeid()
	{
		$id = array();
		empty($_REQUEST['trustee']['id']) || $id[] = (int)$_REQUEST['trustee']['id'];
		empty($_REQUEST['trustee_id']) || $id[] = (int)$_REQUEST['trustee_id'];

		if(!$id)
			return;

		foreach($id as $check)
			if(!isset($_SESSION['participant_ids'][$check]))
				throw new InterfacePermissionException(sprintf('Participant trying to access unauthorized Trustee id(s) [%s]',implode(',',$id)));

	}
}
/**
 *
 */
class InterfacePermissionException extends IWException {}
?>
