<?php
/**
 * FileDescription
 *
 * @package PACKAGE
 * @subpackage SUBPACKAGE
 * @since Dec 15, 2006
 * @author Martijn van den Broek <mjbroek@gmail.com>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */

require_once(IWARE."phpchartdir.php");

class DashboardHandler
{
	static function saveLayout($data)
	{
		debug($data);

		$sql = 'UPDATE TBL_USER SET DASHBOARD = ? WHERE USER_ID=?';
		$q = IWRegistry::registry('db')->query($sql);
		$q->bind_values(serialize($data), $_SESSION['session_user_id']);
		$q->execute();
	}

	static function loadLayout()
	{
		$sql = 'SELECT DASHBOARD FROM TBL_USER WHERE USER_ID = ?';
		$q = IWRegistry::registry('db')->query($sql);
		$q->bind_values($_SESSION['session_user_id']);
		$q->execute();

		$r = $q->fetch();
		return $r->DASHBOARD ? unserialize($r->DASHBOARD) : array();
	}

	static function getCRSOs()
	{
		require_once(ELC_CLASSES.'CRSO.php');
		$c = new CRSO(IWRegistry::registry('db'));
		$q = $c->selectCRSOs();
		$q->execute();

		$rv = array();
		while ($r = $q->fetch())
			$rv[$r->CRSO_RELATION_ID] = $r->NAME;

		return $rv;
	}

	static function getProducts()
	{
		require_once(ELC_CLASSES.'Product.php');
		$c = new Product(IWRegistry::registry('db'));
		$q = $c->getProducts();
		$q->execute();

		$rv = array();
		while ($r = $q->fetch())
			$rv[$r->PRODUCT_ID] = $r->PRODUCT_NAME;

		return $rv;
	}

	static function getLastMonths()
	{
		$rv = array();

		for ($i=0; $i<12; $i++)
		{
			$timestamp = strtotime(sprintf("-%s month", $i));
			$rv[date('m.Y', $timestamp)] = date('F Y', $timestamp);
		}

		return $rv;
	}

	static function LastLogins()
	{
		// FIXME: security

		$db = IWRegistry::registry('db');

		$sql = '
			SELECT
				USER_NAME,
				g.GROUP_NAME as UNIVERSE,
				UNIX_TIMESTAMP(USER_LAST_LOGIN) as LOGGED_IN_DATE,
				NOT ISNULL(FROM_UNIXTIME(sd.expiry)) as LOGGED_IN
			FROM TBL_USER u
			INNER JOIN TBL_GROUP_USER gu ON u.USER_ID = gu.USER_ID
			INNER JOIN TBL_GROUP g ON gu.GROUP_ID = g.GROUP_ID
			LEFT JOIN TBL_SESSION_DATA as sd
				ON sd.data LIKE BINARY CONCAT(\'%session_user_name|s:\', LENGTH(USER_NAME), \':"\', USER_NAME ,\'"%\')
			WHERE NOT ISNULL(USER_LAST_LOGIN)
			ORDER BY USER_LAST_LOGIN DESC
			LIMIT 5;';

		$q = $db->query($sql);
		$q->execute();

		$rv = array();
		while ($r = $q->fetchAssoc())
		{
			$r['LOGGED_IN_DATE'] = ago($r['LOGGED_IN_DATE']);
			$rv[] = $r;
		}
		return $rv;
	}

	/**
	 *  @param status of the widget
	 */
	static function getNotifications($params)
	{
		require_once(ELC_CLASSES.'Notification.php');

		$limit = (int) $params['limit'];

		$status = 0;
		if (isset($params['show_read']) && $params['show_read'])
			$status += NOTIFICATION_STATUS_READ;

		if (isset($params['show_unread']) && $params['show_unread'])
			$status += NOTIFICATION_STATUS_UNREAD;

		// get my relations
		$relation_ids = array();
        if($_SESSION['session_group_group_id'] == GROUP_PARTICIPANT)
        	$relation_ids[] = $_SESSION['selected_participant_id'];
        else
    		foreach($_SESSION['session_user_relations'] as $relation)
    			$relation_ids[] = $relation['id'];

		$n = new Notification(IWRegistry::registry('db'));
		$q = $n->select($_SESSION['session_user_id'], $relation_ids, $status, $limit);

		$rv = array();
		while ($r = $q->fetch())
		{
			$n = array();
			$sender = new Relation(IWRegistry::registry('db'), $r->SENDER_ID);
			$sender->set();
			$n['SENDER'] = $sender->name;
			$n['SUBJECT'] = $r->SUBJECT;
			$n['STATUS'] = $r->STATUS;
			$n['NOTIFICATION_ID'] = $r->NOTIFICATION_ID;
			$n['SENT_DATE'] = Form_Date::makePrettyDateTime($r->{'SENT_DATE'});

			$rv[] = $n;
		}

		return $rv;

	}

}

?>