<?php
/**
 * FileDescription
 *
 * @package PACKAGE
 * @subpackage SUBPACKAGE
 * @since Dec 15, 2006
 * @author Martijn van den Broek <mjbroek@gmail.com>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */

# http://phpxmlrpc.sourceforge.net/doc-2/ch07s03.html
require_once(IWARE. 'xmlrpc/xmlrpc.inc');

class DataPilotHandler
{
	private static function _sources()
	{
		// FIXME: security:

		global $dba_user, $dba_password, $sql_server, $dba_name;
		$base_url = sprintf('mysql://%s:%s@%s:3306/%s', $dba_user, $dba_password, $sql_server, $dba_name);

		$sources = array(
			'provisional_declaration' => $base_url . sprintf('?query=%s',
				urlencode('select * from VIEW_PROVISIONAL_DECLARATION'))
		);

		return $sources;
	}

	static function sources()
	{
		$sources = self::_sources();

		// we only return keys and key value as title
		// fixme: langif
		$rv = array();
		foreach (array_keys($sources) as $source)
			$rv[$source] = $source;

		return $rv;
	}

	static function sourceFields($source)
	{
		$sources = self::_sources();

		if (!isset($sources[$source]))
			throw new IWException('Invalid source: '.$source);

		$client = new xmlrpc_client(DATAPILOT_SERVER_URL);
		$msg = new xmlrpcmsg("fields", array(new xmlrpcval($sources[$source], "string")));
		$resp = $client->send($msg);

		if ($resp->faultCode())
			throw new IWException('DataPilotServer Exception:'. $resp->faultString());

		$val = $resp->value();
		if (!$val->kindOf() == 'array')
			throw new IWException('Error while parsing result.. expected array');

		return php_xmlrpc_decode($val);
	}

	static function result($params)
	{
		$source = $params['source'];
		$selection = $params['selection'];

		$sources = self::_sources();
		if (!isset($sources[$source]))
			throw new IWException('Invalid source: '.$source);
		$source = $sources[$source];

		$client = new xmlrpc_client(DATAPILOT_SERVER_URL);

		$msg = new xmlrpcmsg("result_as_table");
		$msg->addParam(new xmlrpcval($source, "string"));
		$msg->addParam(php_xmlrpc_encode($selection));

		$resp = $client->send($msg);

		if ($resp->faultCode())
			throw new IWException('DataPilotServer Exception:'.$resp->faultString());

		// extract values out of response
		$val = $resp->value();
		$html = $val->scalarVal();

		return $html;
	}

	static function fieldValues($params)
	{
		$source = $params['source'];
		$field = $params['field'];

		$sources = self::_sources();
		if (!isset($sources[$source]))
			throw new IWException('Invalid source: '.$source);
		$source = $sources[$source];

		$client = new xmlrpc_client(DATAPILOT_SERVER_URL);
		$msg = new xmlrpcmsg("field_values", array(
			new xmlrpcval($source, "string"),
			new xmlrpcval($field, "string"),
			)
		);
		$resp = $client->send($msg);

		if ($resp->faultCode())
		{
			alert($resp->faultString()); // log error to syslog
			throw new IWException('DataPilotServer Exception:'.$resp->faultString());
		}

		$val = $resp->value();
		return php_xmlrpc_decode($val);
	}
}

?>