<?php
/**
 * @package
 * @subpackage
 * @since 13-okt-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */

function getVirtualStock(&$broker)
{
	$params = $broker->params;
	empty($params['producer_id']) && $params['producer_id'] = null;
	if(!$params['quantity'])
		return array('status'=>'success','line_index'=>$params['line_index']);
	global $db, $langIf;

	require_once(ELC_CLASSES.'RefundExport.php');
	$re = new RefundExport($db);
	// Get the current virtual stock.

	if(!isset($_SESSION['participant_ids'][$params['participant_id']]))
		return array();

	$vstock = $re->getVirtualStock($params['crso_id'],$params['participant_id'],$params['producer_id'],$_SESSION['selected_product_id']);
	// Substract any Confirmed Exports for participant/crso/producer
	$vstock = getPendingExports($params['crso_id'],$params['participant_id'],$params['producer_id'],$vstock);

	// validate if this is the larger then the amount provided

	debug('('.$vstock.' >= '.$params['quantity'].') == '.$params['line_index']);

	if($vstock>=$params['quantity'])
		return array('status'=>'success','line_index'=>$params['line_index']);
	return array('status'=>'failure','line_index'=>$params['line_index']);
}

function getPendingExports($crso_id,$participant_id,$producer_id,$vstock)
{
	global $db;
	$sql = sprintf("
			SELECT
				RR.REFUND_REQUEST_ID,
				SUM(RRL.QUANTITY) AS QUANTITY,
				RRL.PRODUCER_RELATION_ID
			FROM
				TBL_REFUND_REQUEST RR
			INNER JOIN TBL_REFUND_REQUEST_LINE RRL USING(REFUND_REQUEST_ID)
			WHERE ISNULL(RR.REFUND_EXPORT_ID)
			AND RR.STATUS>=?
			AND RR.CRSO_RELATION_ID = ?
			AND RR.PARTICIPANT_RELATION_ID = ? %s
			GROUP BY RR.REFUND_REQUEST_ID, RRL.PRODUCER_RELATION_ID
			ORDER BY RR.CREATED, RR.REFUND_REQUEST_ID
			",
			($producer_id?' AND RRL.PRODUCER_RELATION_ID = '.$producer_id.' ':'')
			);
	$q = $db->query($sql);
	$q->bind_values(REFUND_REQUEST_APPROVED, $crso_id, $participant_id);
	if(!$q->execute())
		return false;
	$approvedRefund = $tempRRId = 0;
	$invalidProducers = array();

	while($r = $q->fetch())
	{
		if(((int)$r->{'QUANTITY'}) <= $vstock)
			$vstock -= (int)$r->{'QUANTITY'};
		else
			return $vstock;
	}
	return $vstock;
}

function refundImporterMarkAsResolved(&$broker)
{
	$params = $broker->params;

	// TODO: this should be a CRSO/Accountant action?
	if($_SESSION['session_group_group_id']!=GROUP_SYSADMIN)
		throw new IWException('Permission Denied');

	require_once(ELC_CLASSES.'RefundRequest.php');

	$rr = new RefundRequest(IWRegistry::registry('db'),$params['rr_id'],true);
	if(!$rr->isComplete())
		throw new IWException('Incorrect use of Importer Resolve (1)');

	if($rr->importer_status != REFUND_IMPORTER_STATUS_REJECTED)
		throw new IWException('Incorrect use of Importer Resolve (2)');

	$rr->importer_status = REFUND_IMPORTER_STATUS_RESOLVED;
	$rr->update();
	return array();
}

?>
