<?php
/**
 * @package
 * @subpackage
 * @since 10-nov-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
function requestAnnualConfirmationChange(&$broker)
{
	$params = $broker->params;
	global $db, $langIf;

	require_once(ELC_CLASSES.'AnnualConfirmation.php');
	require_once(ELC_CLASSES.'CRSO.php');
	require_once(ELC_CLASSES.'Participant.php');
	require_once(ELC_CLASSES.'Relation.php');
	require_once(ELC_CLASSES.'Product.php');
	require_once(ELC_CLASSES.'User.php');

	$participant = new Participant($db, array('relation'=>new Relation($db, $_SESSION['selected_participant_id'], true)));
	$crso = new CRSO($db, array('relation'=>new Relation($db, $params['crso_id'], true)));
	$product = new Product($db,$_SESSION['selected_product_id']);

	$ac = new AnnualConfirmation($db, array('crso'=>$crso, 'participant'=>$participant, 'year'=>$params['year'], 'product'=>$product));
	if(!$ac->setByParticipantCRSOAndYear())
		return array();

	// Dont SPAM the CRSO
	if($ac->change_request_status==ANNUAL_CONFIRMATION_CHANGE_REQUESTED)
		return;

	$ac->change_request_status = ANNUAL_CONFIRMATION_CHANGE_REQUESTED;
	$ac->update();
	// Log all the data pls.
	info(sprintf('Participant [%s] requested an Annual Confirmation change at CRSO [%s] for year [%s]',
			$participant->relation->id, $crso->relation->id,$params['year']));

        log_elc_event(sprintf('Participant requested Annual Confirmation change for year [%s]', $params['year']) ,
            $_SESSION['session_user_id'], $product->id, $crso->relation->id, $participant->relation->id
        );

	require_once(ELC_CLASSES.'Notification.php');
	Notification::send($db,
	array('subject'=>$langIf->getText('Notification: Request Change for Annual Confirmation'),
		'sender'=>$participant->relation,
		'message'=>$langIf->getText('Notification: Participant `{PARTICIPANT}` requests change for Annual Conf {YEAR}',
					null,  array('PARTICIPANT'=>$participant->relation->name,'YEAR'=>$params['year']))),
	array(
		array('recipient'=>$crso->relation,
			'callback'=>sprintf("go('/".SITE_PATH."scripts/elc/crso/InterfaceAnnualConfirmationList.php?year=%s')", $params['year'])
	)));
}

function approveAnnualConfirmationChange(&$broker)
{
	$params = $broker->params;
	global $db, $langIf;

	require_once(ELC_CLASSES.'AnnualConfirmation.php');
	$ac = new AnnualConfirmation($db,$params['ac_id'],true);

	if((!isset($_SESSION['crso_relation_id']))||($_SESSION['crso_relation_id']!=$ac->crso->relation->id))
		return array();

	$ac->change_request_status = ANNUAL_CONFIRMATION_CHANGE_APPROVED;
	$ac->status = ANNUAL_CONFIRMATION_REOPENED;
	$ac->update();

	info(sprintf('CRSO [%s] approved change Annual Confirmation for Participant [%s] for year [%s]',
		$ac->crso->relation->id,$ac->participant->relation->id,$ac->year));

	$ac->crso->relation->set(); // For the notification

        log_elc_event(sprintf('CRSO authorized Annual Confirmation change for year [%s]', $ac->year) ,
            $_SESSION['session_user_id'], $ac->product->id, $ac->crso->relation->id, $ac->participant->relation->id
        );


	require_once(ELC_CLASSES.'Notification.php');
	Notification::send($db,
	array('subject'=>$langIf->getText('Notification: Change for Annual Confirmation Approved'),
		'sender'=>$ac->crso->relation,
		'message'=>$langIf->getText('Notification: CRSO `{CRSO}` has approved change for Annual Conf {YEAR}',
					null,  array('CRSO'=>$ac->crso->relation->name,'YEAR'=>$ac->year))),
	array(
		array('recipient'=>$ac->participant->relation,
			'callback'=>sprintf("go('/".SITE_PATH."scripts/elc/participant/InterfaceAnnualConfirmation.php?crso[id]=%s&year=%s')",
				$ac->crso->relation->id,$ac->year)
	)));
}

function rejectAnnualConfirmationChange(&$broker)
{
	$params = $broker->params;
	global $db, $langIf;

	require_once(ELC_CLASSES.'AnnualConfirmation.php');
	$ac = new AnnualConfirmation($db,$params['ac_id'],true);

	if((!isset($_SESSION['crso_relation_id']))||($_SESSION['crso_relation_id']!=$ac->crso->relation->id))
		return array();

	$ac->change_request_status = 0;
	//$ac->status = ANNUAL_CONFIRMATION_REOPENED;
	$ac->update();

	info(sprintf('CRSO [%s] rejected change Annual Confirmation for Participant [%s] for year [%s]',
		$ac->crso->relation->id,$ac->participant->relation->id,$ac->year));

	$ac->crso->relation->set(); // For the notification

        log_elc_event(sprintf('CRSO rejected Annual Confirmation change for year [%s]', $ac->year) ,
            $_SESSION['session_user_id'], $ac->product->id, $ac->crso->relation->id, $ac->participant->relation->id
        );

	require_once(ELC_CLASSES.'Notification.php');
	Notification::send($db,
	array('subject'=>$langIf->getText('Notification: Change for Annual Confirmation Rejected'),
		'sender'=>$ac->crso->relation,
		'message'=>$langIf->getText('Notification: `{CRSO}` has rejected change Annual Conf {YEAR}, reason:',
					null,  array('CRSO'=>$ac->crso->relation->name,'YEAR'=>$ac->year)).'<br />'.$params['reject_msg']),
	array(
		array('recipient'=>$ac->participant->relation
	)));
}

?>
