<?php
/**
 * @package ELC
 * @subpackage Registration
 * @author Nick Loeve <nick@basesoft.com>
 * @since 26/10/2006
 * @copyright 2006 BaseSoft B.V.
 * $Id$
 */
function requestChangeParticipantType(&$broker)
{
	$params = $broker->params;
	global $db, $langIf;

	require_once(ELC_CLASSES.'CRSOParticipantProduct.php');

	if(!isset($_SESSION['participant_ids'][$params['participant_id']]))
		return array();

	$participant = new Participant($db, array('relation'=>new Relation($db, $params['participant_id'], true)));
	$crso = new CRSO($db, array('relation'=>new Relation($db, $params['crso_id'], true)));
	$product = new Product($db,$_SESSION['selected_product_id']);

	$cp = new CRSOParticipantProduct($db, array('crso'=>$crso, 'participant'=>$participant, 'product'=>$product));

	if(!$cp->set())
		return array();

	$sendNotification = false;
	switch((int)$params['status'])
	{
		case 1:
			if($cp->producer_exporter ^ PARTICIPANT_EXPORTER)
			{
				$sendNotification = true;
				$cp->participant_type_change_request = PARTICIPANT_TYPE_CHANGE_EXPORTER;
			}
			break;
		case 2:
			if($cp->producer_exporter ^ PARTICIPANT_PRODUCER)
			{
				$sendNotification = true;
				$cp->participant_type_change_request = PARTICIPANT_TYPE_CHANGE_PRODUCER;
			}
			break;
		default:
			break;
	}
	if($sendNotification)
	{
		$cp->update();
		// Log all the data pls.
		info(sprintf('Participant [%s] requested a participant type change at CRSO [%s], request type [%s]',
				$participant->relation->id, $crso->relation->id, $cp->participant_type_change_request));

		require_once(ELC_CLASSES.'Notification.php');
		Notification::send($db,
		array('subject'=>$langIf->getText('Notification: Request participant type change'),
			'sender'=>$participant->relation,
			'message'=>$langIf->getText('Notification: Participant `{PARTICIPANT}` requests to become {PARTICIPANT_TYPE}',
						null,  array(
										'PARTICIPANT'=>$participant->relation->name,
										'PARTICIPANT_TYPE' =>
												$cp->participant_type_change_request == 1 ?
												$langIf->getText('exporter'): $langIf->getText('producer')))),
		array(
			array('recipient'=>$crso->relation,
				'callback'=>sprintf("go('/".SITE_PATH."scripts/elc/crso/InterfaceRegistrantDetails.php?participant[id]=%s#participantchangetype')", $participant->relation->id)
		)));

                log_elc_event(sprintf('Participant requested participant type change as [%s]',
                    $cp->participant_type_change_request == PARTICIPANT_TYPE_CHANGE_PRODUCER ? 'producer' : 'exporter'),
                    $_SESSION['session_user_id'], $product->id, $crso->relation->id, $participant->relation->id
                );
	}
	else
	{
		info(sprintf('Participant [%s] requested a participant type change at CRSO [%s], request type [%s] - NOT PROCESSED',
				$participant->relation->id, $crso->relation->id, $params['status']));
	}
}

function authorizeChangeParticipantType(&$broker)
{
	$params = $broker->params;
	global $db, $langIf;

	if((!isset($_SESSION['crso_relation_id']))||($_SESSION['crso_relation_id']!=$params['crso_id']))
		return array();

	require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
	require_once(ELC_CLASSES.'Relation.php');
	require_once(ELC_CLASSES.'User.php');

	$participant = new Participant($db, array('relation'=>new Relation($db, $params['participant_id'], true)));
	$crso = new CRSO($db, array('relation'=>new Relation($db, $params['crso_id'], true)));
	$product = new Product($db,$_SESSION['selected_product_id']);

	$cp = new CRSOParticipantProduct($db, array('crso'=>$crso, 'participant'=>$participant, 'product'=>$product),true);

	if($cp->participant_type_change_request == PARTICIPANT_TYPE_CHANGE_PRODUCER)
	{
		$cp->participant_type_change_request = 4;
		$cp->producer_exporter = PARTICIPANT_EXPORTER + PARTICIPANT_PRODUCER;
		$cp->update();
		info(sprintf('CRSO [%s] authorized a participant type change request that was Requested by participant [%s], status now [%s], request status [%s]',
			$crso->relation->id,$participant->relation->id, $cp->producer_exporter, $cp->participant_type_change_request));

                log_elc_event('CRSO authorized participant type change as [producer]',
                    $_SESSION['session_user_id'], $product->id, $crso->relation->id, $participant->relation->id
                );
	}
	else if($cp->participant_type_change_request == PARTICIPANT_TYPE_CHANGE_EXPORTER)
	{
		$cp->participant_type_change_request = null;
		$cp->producer_exporter = PARTICIPANT_EXPORTER + PARTICIPANT_PRODUCER;
		$cp->update();
		info(sprintf('CRSO [%s] authorized a participant type change request that was Requested by participant [%s], status now [%s], request status [%s]',
			$crso->relation->id,$participant->relation->id, $cp->producer_exporter, $cp->participant_type_change_request));
                log_elc_event('CRSO authorized participant type change as [exporter]',
                    $_SESSION['session_user_id'], $product->id, $crso->relation->id, $participant->relation->id
                );
	}

	// Send notification with a callback to the right interface.
	require_once(ELC_CLASSES.'Notification.php');
	Notification::send($db,
	array('subject'=>$langIf->getText('Notification: Approved Request Change for Participant Type'),
		'sender'=>$crso->relation,
		'message'=>$langIf->getText('Notification: Request has been approved')),
	array(
		array('recipient'=>$participant->relation,
			'callback'=>sprintf("go('/".SITE_PATH."scripts/elc/participant/InterfaceRegistrationOverview.php')")
	)));
}

function rejectChangeParticipantType(&$broker)
{
	$params = $broker->params;
	global $db, $langIf;

	if((!isset($_SESSION['crso_relation_id']))||($_SESSION['crso_relation_id']!=$params['crso_id']))
		return array();

	require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
	require_once(ELC_CLASSES.'Relation.php');
	require_once(ELC_CLASSES.'User.php');

	$participant = new Participant($db, array('relation'=>new Relation($db, $params['participant_id'], true)));
	$crso = new CRSO($db, array('relation'=>new Relation($db, $params['crso_id'], true)));
	$product = new Product($db,$_SESSION['selected_product_id']);

	$cp = new CRSOParticipantProduct($db, array('crso'=>$crso, 'participant'=>$participant, 'product'=>$product),true);


	$cp->participant_type_change_request = DB_NULL;
	$cp->update();
	info(sprintf('CRSO [%s] rejected a participant type change request that was Requested by participant [%s], status now [%s], request status [%s]',
		$crso->relation->id,$participant->relation->id, $cp->producer_exporter, $cp->participant_type_change_request));

        log_elc_event('CRSO rejected participant type change',
            $_SESSION['session_user_id'], $product->id, $crso->relation->id, $participant->relation->id
        );

	// Send notification with a callback to the right interface.
	require_once(ELC_CLASSES.'Notification.php');
	Notification::send($db,
	array('subject'=>$langIf->getText('Notification: Rejected Request Change Participant Type'),
		'sender'=>$crso->relation,
		'message'=>$langIf->getText('Notification: Request has been rejected').'<br />'.$langIf->getText('Notification: Reason of Rejection').'<br />'.$params['reject_msg']),
	array(
		array('recipient'=>$participant->relation
	)));
}
?>
