<?php
/**
 * @package ELC
 * @subpackage Declaration
 * @since 28-aug-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
 function requestChangeProvisional(&$broker)
{
	$params = $broker->params;
	global $db, $langIf;

	require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
	require_once(ELC_CLASSES.'Relation.php');
	require_once(ELC_CLASSES.'User.php');

	if(!isset($_SESSION['participant_ids'][$params['participant_id']]))
		return array();

	$participant = new Participant($db, array('relation'=>new Relation($db, $params['participant_id'], true)));
	$crso = new CRSO($db, array('relation'=>new Relation($db, $params['crso_id'], true)));
	$product = new Product($db, (isset($params['product_id']) ? $params['product_id'] : 1), true);

	$cp = new CRSOParticipantProduct($db, array('crso'=>$crso, 'participant'=>$participant, 'product'=>$product));

	if(!$cp->set())
		return array();

	// Add status PROVISIONAL_CHANGE_REQUEST_PENDING
	$cp->provisional_change_request_status = PROVISIONAL_CHANGE_REQUEST_PENDING;
	$cp->update();
	// Log all the data pls.
	info(sprintf('Participant [%s] requested a provisional change at CRSO [%s] with product [%s]',
			$participant->relation->id, $crso->relation->id, $product->id));

	require_once(ELC_CLASSES.'Notification.php');
	Notification::send($db,
	array('subject'=>$langIf->getText('Notification: Request Change for Provisional Declaration'),
		'sender'=>$participant->relation,
		'message'=>$langIf->getText('Notification: Participant `{PARTICIPANT}` requests change for Provisional Decl',
					null,  array('PARTICIPANT'=>$participant->relation->name))),
	array(
		array('recipient'=>$crso->relation,
			'callback'=>sprintf("go('/".SITE_PATH."scripts/elc/crso/InterfaceRegistrantDetails.php?participant[id]=%s#provchange')", $participant->relation->id)
	)));

        log_elc_event('Participant requested provisional change',
            $_SESSION['session_user_id'], $product->id, $crso->relation->id, $participant->relation->id
        );
}
/**
 *
 */
function authorizeChangeProvisional(&$broker)
{
	$params = $broker->params;
	global $db, $langIf;

	if((!isset($_SESSION['crso_relation_id']))||($_SESSION['crso_relation_id']!=$params['crso_id']))
		return array();

	require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
	require_once(ELC_CLASSES.'Relation.php');
	require_once(ELC_CLASSES.'User.php');

	$participant = new Participant($db, array('relation'=>new Relation($db, $params['participant_id'], true)));
	$crso = new CRSO($db, array('relation'=>new Relation($db, $params['crso_id'], true)));
	$product = new Product($db, $params['product_id'], true);

	$cp = new CRSOParticipantProduct($db, array('crso'=>$crso, 'participant'=>$participant, 'product'=>$product),true);
	if($cp->provisional_change_request_status!=PROVISIONAL_CHANGE_REQUEST_PENDING)
	{
		info(sprintf('CRSO [%s] tried to authorize a provisional change request that was not requested by participant [%s] for product [%s]',
			$crso->relation->id,$participant->relation->id, $product->id));
		return;
	}
	$cp->provisional_change_request_status = PROVISIONAL_CHANGE_REQUEST_AUTHORIZED;
	$cp->update();
	// Log plssss.
	info(sprintf('CRSO [%s] authorized a provisional change request that was requested by participant [%s] for product [%s]',
			$crso->relation->id,$participant->relation->id, $product->id));

	// Send notification with a callback to the right interface.
	require_once(ELC_CLASSES.'Notification.php');
	Notification::send($db,
	array('subject'=>$langIf->getText('Notification: Approved Request Change for Prov Decl'),
		'sender'=>$crso->relation,
		'message'=>$langIf->getText('Notification: Request has been approved')),
	array(
		array('recipient'=>$participant->relation,
			'callback'=>sprintf("go('/".SITE_PATH."scripts/elc/participant/InterfaceProvisionalForm.php?crso[id]=%s&participant[id]=%s&product[id]=%s')", $crso->relation->id,$participant->relation->id, $product->id)
	)));

        log_elc_event('CRSO authorized provisional change',
            $_SESSION['session_user_id'], $product->id, $crso->relation->id, $participant->relation->id
        );
}
/**
 *
 */
function rejectChangeProvisional(&$broker)
{
	$params = $broker->params;
	global $db, $langIf;

	if((!isset($_SESSION['crso_relation_id']))||($_SESSION['crso_relation_id']!=$params['crso_id']))
		return array();

	require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
	require_once(ELC_CLASSES.'Relation.php');
	require_once(ELC_CLASSES.'User.php');

	$participant = new Participant($db, array('relation'=>new Relation($db, $params['participant_id'], true)));
	$crso = new CRSO($db, array('relation'=>new Relation($db, $params['crso_id'], true)));
	$product = new Product($db,$_SESSION['selected_product_id']);

	$cp = new CRSOParticipantProduct($db, array('crso'=>$crso, 'participant'=>$participant, 'product'=>$product),true);
	if($cp->provisional_change_request_status!=PROVISIONAL_CHANGE_REQUEST_PENDING)
	{
		info(sprintf('CRSO [%s] tried to reject a provisional change request that was not Requested by participant [%s]',
			$crso->relation->id,$participant->relation->id));
		return;
	}
	// Set it back to Open so it can be requested again.
	$cp->provisional_change_request_status = PROVISIONAL_CHANGE_REQUEST_OPEN;
	$cp->update();
	// Log this as well
	info(sprintf('CRSO [%s] rejected a provisional change request that was Requested by participant [%s] with message: %s',
			$crso->relation->id,$participant->relation->id,$params['reject_msg']));
	// Send notification with reject message and without a callback
	require_once(ELC_CLASSES.'Notification.php');
	Notification::send($db,
	array('subject'=>$langIf->getText('Notification: Rejected Request Change for Prov Decl'),
		'sender'=>$crso->relation,
		'message'=>$langIf->getText('Notification: Request has been rejected').'<br />'.$langIf->getText('Notification: Reason of Rejection').'<br />'.$params['reject_msg']),
	array(
		array('recipient'=>$participant->relation
	)));

        log_elc_event('CRSO rejected provisional change',
            $_SESSION['session_user_id'], $product->id, $crso->relation->id, $participant->relation->id
        );
}

?>
