<?php
require_once(ELC_CLASSES.'Trust.php');
/**
 * For the trustee/subrogator.
 *
 * @package ELC
 * @subpackage Subrogation
 * @since 18-sep-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
function requestTrustTermination(&$broker)
{
	global $db, $langIf;

	$params = $broker->params;
	$trust = new Trust($db,$params['trust_id'],true);
	// Check if the $params['type']['id'] is part of this user relations
	$type = $params['type'];
	if($trust->{$type}->relation->id != $params['type_id'])
	{
		// Throw Crit?
		return;
	}
	if(!isset($_SESSION['participant_ids'][$params['type_id']]))
	{
		// Throw Crit?
		return;
	}
	$status = $trust->terminate_request_status?$trust->terminate_request_status:0;
	// This whole process has been done before. Dont spam the CRSO.
	if($status == (TRUST_TERMINATION_SUBROGATOR_APPROVED+TRUST_TERMINATION_TRUSTEE_APPROVED))
		return;
	$status |= ($type=='subrogator')?
		TRUST_TERMINATION_SUBROGATOR_APPROVED:TRUST_TERMINATION_TRUSTEE_APPROVED;
	$trust->terminate_request_status = $status;
	$trust->update();

	$trust->subrogator->relation->set();
	$trust->trustee->relation->set();
	$callback = "MessageBox.prototype.Question('%s','%s',function(){
			var params = new Array();
			params['type'] = '%s';
			params['trust_id'] = '%d';
			params['type_id'] = '%d';
			jsbroker.call('%s',params,function(){
				MessageBox.prototype.Information('%s','%s');
			});
		});";
	$confirmationMessage = 'The Request for Termination has been sent';
	$function = 'requestTrustTermination';

	// Sent a message to the other party
	if(!($status & TRUST_TERMINATION_SUBROGATOR_APPROVED))
	{
		$receiver = $recipient = $trust->subrogator->relation;
		$sender = $trust->trustee->relation;

                log_elc_event(sprintf('Trust termination request to subrogator [%s] by trustee [%s]',
                            $trust->subrogator->relation->name, $trust->trustee->relation->name),
                    $_SESSION['session_user_id'], $_SESSION['selected_product_id'],
                    $trust->crso->relation->id, $trust->trustee->relation->id
                );

	}
	elseif(!($status & TRUST_TERMINATION_TRUSTEE_APPROVED))
	{
		$receiver = $recipient = $trust->trustee->relation;
		$sender = $trust->subrogator->relation;

                log_elc_event(sprintf('Trust termination request to trustee [%s] by subrogator [%s]',
                            $trust->trustee->relation->name, $trust->subrogator->relation->name),
                    $_SESSION['session_user_id'], $_SESSION['selected_product_id'],
                    $trust->crso->relation->id, $trust->subrogator->relation->id
                );
	}
	else
	{
		// Sent to CRSO
		$recipient = $trust->crso->relation;
		$sender = $trust->{$type}->relation;
		$receiver = $trust->{(($type=='subrogator')?'trustee':'subrogator')}->relation;
		$confirmationMessage = 'The Trust has been terminated';
		$function = 'approveTrustTermination';

                log_elc_event(sprintf('Approval of Trust termination request (trustee [%s] subrogator [%s]) sent to CRSO',
                            $trust->trustee->relation->name, $trust->subrogator->relation->name),
                    $_SESSION['session_user_id'], $_SESSION['selected_product_id'],
                    $trust->crso->relation->id, null
                );
	}
	$message = '{SENDER} requested a termination for trust with {RECEIVER}';
	$replacements = array('SENDER'=>$sender->name,
		'RECEIVER'=>$receiver->name);

	$callback = sprintf($callback,
		$langIf->getText('MessageBox: Title Termination Request'),
		$langIf->getText($message,null,$replacements),
		(($type=='subrogator')?'trustee':'subrogator'),
		$trust->id,
		$recipient->id,
		$function,
		$langIf->getText('MessageBox: Title Termination Request'),
		$langIf->getText($confirmationMessage)
		);

	require_once(ELC_CLASSES.'Notification.php');
	Notification::send($db,
	array('subject'=>$langIf->getText('Notification: Trust Termination'),
		'sender'=>$sender,
		'message'=>$langIf->getText($message,null,$replacements)),
	array(
		array('recipient'=>$recipient,
			'callback'=>$callback
	)));
}
/**
 * For the CRSO
 */
function approveTrustTermination(&$broker)
{
	global $db, $langIf;
	$db = IWRegistry::registry('db');

	$params = $broker->params;
	$trust = new Trust($db,$params['trust_id'],true);
	if(($trust->terminate_request_status!=(TRUST_TERMINATION_SUBROGATOR_APPROVED+TRUST_TERMINATION_TRUSTEE_APPROVED))
		&& $trust->approved == TRUST_APPROVED)
	{
		// crit?
		return;
	}
	$db->begin();
	// Set termination status to 7 but only if termination status = 3.
	$trust->terminate_request_status = TRUST_TERMINATION_APPROVED;
	// Update the trust status to 0 (disabled)
	$trust->approved = TRUST_DISABLED;
	$trust->update();
	require_once(ELC_CLASSES.'ProvisionalDeclaration.php');

	$pd = new ProvisionalDeclaration($db);
	// Put the provisional back to the subrogator.
	if(!$pd->removeSubrogation($trust)) // Remaining months only (see 1.2.4.10)
	{
		// crit
		$db->rollback();
		return;
	}
	require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
	// Set no declaration back to 0.
	$cp = new CRSOParticipantProduct($db);
	$cp->participant = $trust->subrogator;
	$cp->crso = $trust->crso;
	$cp->product = $trust->product;
	$cp->set();
	$cp->no_declaration = 0;
	$cp->update();

	$user_id = isset($_SESSION['session_user_id']) ? $_SESSION['session_user_id'] : null;
	$product_id = isset($_SESSION['selected_product_id']) ? $_SESSION['selected_product_id'] : null;

	log_elc_event(sprintf('CRSO approved Trust termination request (trustee [%s] subrogator [%s])',
				$trust->trustee->relation->name, $trust->subrogator->relation->name),
		$user_id, $product_id,
		$trust->crso->relation->id, null
	);

	$db->commit();
}
?>
