<?php
/**
 * @package ELC
 * @subpackage rpc
 */
	/**
	 * Test function
	 */
	function getElcInspectorChilds(&$broker)
	{
		$params = $broker->params;
		//debug('params: '.print_r($params, true));

		global $db;
		$inspector = new ELCInspector($db);

		$rv = array();
		$params['type'] = isset($params['type']) ? $params['type'] : '';
		switch ($params['type'])
		{
			// this is the start node, check who we are and give the appropriate childs
			case 'root':
				$group_company = $_SESSION['session_group_group_name'];
				switch ($group_company)
				{
					// we are sysadmin, lets give all countries
					case 'sysadmin':
						return $inspector->getCountries();
						break;
					case 'accountant':
					case 'participant':
					case 'crso':
					default:
				}
			break;

			// make two links to crso's and operators when opening a country
			case 'country':
				$n = new InspectorNode();
				$n->setCallParam('country_id', $params['country_id']);

				$n->set('title', 'CRSOs');
				$n->setCallParam('type', 'crsos');
				$rv[] = $n->toArray();

				/*$n->set('title', 'Accountants');
				$n->setCallParam('type', 'accountants');
				$rv[] = $n->toArray();*/

				return $rv;
			break;

			case 'crsos':
				return $inspector->getCRSOs($params['country_id']);
			case 'crso':
				return $inspector->getCrso($params['relation_id']);
			case 'accountants':
				return $inspector->getAccountants($params['country_id']);
			case 'accountant':
				return $inspector->getAccountant($params['relation_id']);
			case 'participants':
				return $inspector->getParticipants($params['relation_id']);
			case 'participant':
				return $inspector->getParticipant($params['relation_id'], $params['crso_id']);
			case 'users':
				return $inspector->getUsers($params['relation_id'], isset($params['crso_id']) ? $params['crso_id'] : 0);
			case 'trust':
				return $inspector->getTrust($params['relation_id'], $params['crso_id']);
			case 'subrogators':
				return $inspector->getTrustParticipantsAsSubrogator($params['relation_id'], $params['crso_id']);
			case 'trustees':
				return $inspector->getTrustParticipantsAsTrustee($params['relation_id'], $params['crso_id']);
			default:
				$rv = array();
		}

		return $rv;
	}

	class ELCInspector
	{
		function ELCInspector($db_app)
		{
			$this->db_app = $db_app;
		}

		function getCountries()
		{
			require_once(ELC_CLASSES.'Country.php');
			$c = new Country($this->db_app);
			$q = $c->selectForInspection();
			$rv = array();
			while ($row = $q->fetch())
			{
				$node = new InspectorNode();
				$node->set('title', $row->{'NAME'} );
				$node->setCallParam('type', 'country');
				$node->setCallParam('country_id', $row->{'COUNTRY_ID'});
				$rv[] = $node->toArray();
			}

			return $rv;
		}

		function getAccountants($country_id)
		{
			$sql = "SELECT R.NAME, R.RELATION_ID
					FROM TBL_RELATION R
					INNER JOIN TBL_ACCOUNTANT_CRSO AC ON AC.ACCOUNTANT_RELATION_ID = R.RELATION_ID
					INNER JOIN TBL_RELATION RR ON RR.RELATION_ID = AC.CRSO_RELATION_ID
					INNER JOIN TBL_ADDRESS AD ON AD.ADDRESS_ID = RR.ADDRESS_ID
					INNER JOIN TBL_COUNTRY C ON AD.COUNTRY_ID = C.COUNTRY_ID
					WHERE C.COUNTRY_ID=?";
			$q = $this->db_app->query($sql);
			$q->bind_values($country_id);
			$q->execute();
			$rv = array();
			while ($row = $q->fetch())
			{
				$node = new InspectorNode();
				$node->set('title', $row->{'NAME'} );
				$node->setCallParam('type', 'accountant');
				$node->setCallParam('relation_id', $row->{'RELATION_ID'});
				$rv[] = $node->toArray();
			}
			return $rv;
		}

		function getAccountant($accountant_relation_id)
		{
			//We comment this out so operators cannot see the other operator users

			//$n = new InspectorNode();
			//$n->setCallParam('relation_id', $operator_relation_id);

			//$n->set('title', 'Users');
			//$n->setCallParam('type', 'users');
			//$rv[] = $n->toArray();

			$rv = array();

			return $rv;
		}

		function getCRSOs($country_id)
		{
			require_once(ELC_CLASSES . 'CRSO.php');
			$o = new CRSO($this->db_app);
			$q = $o->selectCRSOs('',  // ignore sla_accepted
				$country_id);

			$rv = array();
			while ($row = $q->fetch())
			{
				$node = new InspectorNode();
				$node->set('title', $row->{'NAME'} );
				$node->setCallParam('type', 'crso');
				$node->setCallParam('relation_id', $row->{'CRSO_RELATION_ID'});
				$rv[] = $node->toArray();
			}
			return $rv;
		}

		function getCrso($crso_relation_id)
		{
			$n = new InspectorNode();
			$n->setCallParam('relation_id', $crso_relation_id);

			$n->set('title', 'Users');
			$n->setCallParam('type', 'users');
			$rv[] = $n->toArray();

			$n->set('title', 'Participants');
			$n->setCallParam('type', 'participants');
			$rv[] = $n->toArray();

			$n->set('title', 'Settings');
			$n->setCallParam('type', 'crsosettings');
			$rv[] = $n->toArray();

			$n->set('title', 'Product Categories');
			$n->setCallParam('type', 'crsoproductcategories');
			$rv[] = $n->toArray();

			$n->set('title', 'Product Fee');
			$n->setCallParam('type', 'crsoproductfee');
			$rv[] = $n->toArray();

			$n->set('title', 'Blocked VATs');
			$n->setCallParam('type', 'crsoblockedvats');
			$rv[] = $n->toArray();

			return $rv;
		}

		function getParticipants($crso_relation_id)
		{
			require_once(ELC_CLASSES . 'Participant.php');
			$p = new Participant($this->db_app);
			$q = $p->selectByCRSO($crso_relation_id);

			$rv = array();
			while ($row = $q->fetch())
			{
				$node = new InspectorNode();
				$node->set('title', $row->{'PARTICIPANT_NAME'} );
				$node->setCallParam('type', 'participant');
				$node->setCallParam('relation_id', $row->{'PARTICIPANT_ID'});
				$node->setCallParam('crso_id', (string)$crso_relation_id);
				$rv[] = $node->toArray();
			}
			return $rv;
		}

		function getParticipant($participant_relation_id, $crso_relation_id)
		{
			$n = new InspectorNode();
			$n->setCallParam('relation_id', $participant_relation_id);
			$n->setCallParam('crso_id', (string)$crso_relation_id);

			$n->set('title', 'Users');
			$n->setCallParam('type', 'users');
			$rv[] = $n->toArray();

			$n->set('title', 'Provisional Declaration');
			$n->setCallParam('type', 'provdec');
			$rv[] = $n->toArray();

			$n->set('title', 'Trust');
			$n->setCallParam('type', 'trust');
			$rv[] = $n->toArray();

			return $rv;
		}

		function getTrust($participant_relation_id, $crso_relation_id)
		{
			$rv = array();
			$n = new InspectorNode();
			$n->setCallParam('crso_id', $crso_relation_id);
			$n->setCallParam('relation_id', $participant_relation_id);

			$n->set('title', 'As Subrogator');
			$n->setCallParam('type', 'subrogators');
			$rv[] = $n->toArray();

			$n->set('title', 'As Trustee');
			$n->setCallParam('type', 'trustees');
			$rv[] = $n->toArray();

			return $rv;
		}

		function getTrustParticipantsAsSubrogator($participant_relation_id, $crso_relation_id)
		{
			require_once(ELC_CLASSES . 'Trust.php');
			$t = new Trust($this->db_app);

			$rv = array();
			$n = new InspectorNode();
			$n->setCallParam('crso_id', $crso_relation_id);
			$n->setCallParam('relation_id', $participant_relation_id);
			$n->setCallParam('type', 'subrogator');

			$q = $t->selectSubrogations($participant_relation_id, TRUST_APPROVED, $crso_relation_id);
			while($r = $q->fetch())
			{
				$n->set('title', $r->{'NAME'});
				$n->setCallParam('trust_id', $r->{'TRUST_ID'});
				$rv[] = $n->toArray();
			}
			return $rv;
		}

		function getTrustParticipantsAsTrustee($participant_relation_id, $crso_relation_id)
		{
			require_once(ELC_CLASSES . 'Trust.php');
			$t = new Trust($this->db_app);

			$rv = array();
			$n = new InspectorNode();
			$n->setCallParam('crso_id', $crso_relation_id);
			$n->setCallParam('relation_id', $participant_relation_id);
			$n->setCallParam('type', 'trustee');

			$q = $t->selectTrusts($participant_relation_id, TRUST_APPROVED, $crso_relation_id, $_SESSION['selected_product_id']);
			while($r = $q->fetch())
			{
				$n->set('title', $r->{'NAME'});
				$n->setCallParam('trust_id', $r->{'TRUST_ID'});
				$rv[] = $n->toArray();
			}
			return $rv;
		}

		function getUsers($relation_id, $crso_id)
		{
			require_once(ELC_CLASSES.'User.php');
			$u = new ELCUser($this->db_app);
			$q = $u->selectByRelation($relation_id);
			$rv = array();
			while ($row = $q->fetch())
			{
				$node = new InspectorNode();
				$node->set('title', empty($row->{'USER_NAME'}) ? '<i>username unavailable</i>' : $row->{'USER_NAME'});
				$node->setCallParam('type', 'user');
				$node->setCallParam('user_id', $row->{'USER_ID'});
				if($crso_id)
					$node->setCallParam('crso_id', $crso_id);
				$node->setCallParam('participant_id', $relation_id);
				$rv[] = $node->toArray();
			}
			return $rv;
		}
	}

	class InspectorNode
	{
		var $_title = 'InspectorNode';
		var $_iconClass = '';
		var $_callMethod = 'getElcInspectorChilds';
		var $_callParams = array();

		function InspectorNode($params=array())
		{
			foreach ($params as $key=>$value)
				$this->set($key, $value);
		}

		function set($name, $value)
		{
			$this->{'_'.$name} = $value;
		}

		function setCallParam($name, $value)
		{
			$this->_callParams[$name] = $value;
		}

		function toArray()
		{
			return array (
				'title'=>$this->_title,
				'iconClass'=>$this->_iconClass,
				'callMethod'=>$this->_callMethod,
				'callParams'=>$this->_callParams );
		}
	}

?>
