/**
 * IWARE.AbstractWidget
 *
 * @package IWARE
 * @subpackage Dashboard
 * @since May 10, 2007
 * @author Martijn van den Broek <mjbroek@gmail.com>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */

/**

 */
IWARE.AbstractWidget = function()
{
}

IWARE.AbstractWidget.prototype =
{
	debug: false,

	/**
	 * Widget Title displayed in the NewWidgets dialog and used by setTitle() as a default widget title.
	 * This will be sent to the serverside langIf for translation
	 * @protected
	 */
	title: 'Abstract Widget Title',

	/**
	 * See title
	 * @protected
	 */
	description: 'This is a Abstract Widget which can be extended',

	/**
	 * Array of strings that will be included before calling setBody and setTitle
	 * @protected
	 */
	includes: [],

	/**
	 * Point this to a Settings class to enable settings, you probably want IWARE.WidgetSettingsSettings
	 * @protected
	 */
	settingsClass: null,

	/**
	 * Override this to change the width of the widget
	 * @protected
	 */
	width: 350,

	/**
	 * Array of translation handles which will be translated before init is called
	 * @access protected
	 */
	lang_handles: [],

	/**
	 * @access private
	 */
	settings: null,

	/**
	 * Builds the widget, this is called from dashboard.
	 * @private
	 */
	init: function()
	{
		// container
		this.container = document.createElement('DIV');
		this.container.id = 'widget_'+IWARE.Dashboard.widgetCounter++;
		this.container.className = 'widget';
		this.container.style.width = this.width + 'px';

		// head
		var titlebar = document.createElement('DIV');
		titlebar.className = 'widget_head';
		this.container.appendChild(titlebar);

		// topleft div
		var divTopLeft = document.createElement('DIV');
		titlebar.appendChild(divTopLeft);
		divTopLeft.className = 'topleft';

		// table to align stuff
		var table = document.createElement('TABLE');
		divTopLeft.appendChild(table);
		var tbody = document.createElement('TBODY');
		table.appendChild(tbody);
		var tr = document.createElement('TR');
		tbody.appendChild(tr);

		// title
		this.tdTitle = document.createElement('TD');
		this.tdTitle.className = 'title';
		tr.appendChild(this.tdTitle);

		var tdButtons = document.createElement('TD');
		tdButtons.className = 'buttons';
		tr.appendChild(tdButtons);

		// prepare a tr for settings
		var tr = document.createElement('TR');
		tbody.appendChild(tr);

		var tdSettings = document.createElement('TD');
		tdSettings.colSpan = 2;
		tr.appendChild(tdSettings);

		// settings
		if(this.settingsClass)
		{
			this.debug && dumpln('AbstractWidget: initialising settings');

			this.settings = new this.settingsClass();
			this.settings.init();

			// prepare the panel, we add it later
			//var tdSettings = document.createElement('DIV');
			tdSettings.id = this.container.id + '_tdSettings';
			tdSettings.style.display = 'none';
			tdSettings.className = 'settings';
			tdSettings.appendChild(this.settings.getSettingsForm());

			// add the button
			var btn = document.createElement('INPUT');
			btn.type = 'button';
			btn.value = '>';
			tdButtons.appendChild(btn);

			btn.onclick = function()
			{
				var div = tdSettings;
				div.style.display = (div.style.display == 'none') ? '' : 'none';
				btn.value = (btn.value == '>') ? 'v' : '>';
			}
		}

		// close widget button
		var cBtn = document.createElement('INPUT');
		cBtn.type = 'button';
		cBtn.value = 'x';
		tdButtons.appendChild(cBtn);

		// fixme: use events
		var container_id = this.container.id;
		addEvent(cBtn, 'click', function()
		{
			WEEE.dashboard.removeWidget(container_id);
		});

	},

	/**
	 * Returns the node of the container. Called by dashboard to add the widget to the html body
	 * @final
	 */
	getNode: function()
	{
		if (!this.container)
			throw 'no container'

		return this.container;
	},

	/**
	 * Override this function to change the title of the widget. By default it uses the langIf to translate the widget title
	 * @protected
	 */
	setTitle: function()
	{
		this.tdTitle.innerHTML = langIf.getText(this.title);
	},

	/**
	 * Shows the body, called by Dashboard after this.init
	 * @final
	 */
	showBody: function()
	{
		var body = document.createElement('DIV');
		body.className = 'widget_body';
		body.id = this.container.id + "_body"
		this.getNode().appendChild(body);
		this.setBody(body);
	},

	/**
	 * Override this function to change the body of the widget
	 * @protected
	 */
	setBody: function(nNode)
	{
		nNode.innerHTML = 'This is an Abstract Widget. You can extend widgets in your own class';
	}
}

