
IWARE.DataPilotWidget = function()
{
	IWARE.DataPilotWidget.superclass.constructor.call(this);
}

YAHOO.extend(IWARE.DataPilotWidget, IWARE.AbstractWidget,
{
	title: 'IWARE.DataPilotWidget',
	description: 'Abstract DataPilot Widget',

	settingsClass: IWARE.WidgetSettings,

	includes: [
		'/includes/i-w/IWARE.WizardDialog.js',
		'/includes/i-w/IWARE.DataPilotWizard.js',
		'/includes/i-w/IWARE.DataPilotSelector.js',
		'/includes/i-w/Dashboard/css/datapilot.css'

	],

	init: function()
	{
		IWARE.DataPilotWidget.superclass.init.call(this);

		// settings
		var tdSettings = this.settings.addRow().cells[0]

		// wizard button
		var oBtn = new YAHOO.widget.Button({
			type: "button",
			label: langIf.getText("DataPilot Wizard"),
			container: tdSettings
		});
		oBtn.addListener("click", hitch(this, '_showWizard'));

		// page fields
		this.dvPageFields = document.createElement('div');
		tdSettings.appendChild(this.dvPageFields);

		// prepare the wizard
		this._wizard = new IWARE.DataPilotWizard();
		this._wizard.render();

		// this is called whenever the wizard receives a result
		this._wizard.onFinish.subscribe(function(type, args, me)
		{
			me._makeResult();
			me._makePageFields();
		}, this);
	},

	_makePageFields: function()
	{
		var selection = this._wizard.getSelection();
		var source = this._wizard.getSource();

		// fill settings with page-fields
		removeChildNodes(this.dvPageFields)
		var tbl = document.createElement('table')
		this.dvPageFields.appendChild(tbl)

		tbody = document.createElement('tbody')
		tbl.appendChild(tbody)

		selection['page_fields'].forEach(function(field_values, field, a)
		{
			var tr = document.createElement('tr')
			tbody.appendChild(tr)

			var td = document.createElement('td')
			tr.appendChild(td)
			td.innerHTML = field

			var td = document.createElement('td')
			tr.appendChild(td)

			var sel = document.createElement('select')
			sel.name = field
			td.appendChild(sel)

			// now we have the select we fill it with its values
			params = []
			params['source'] = source
			params['field'] = field

			// fill the select with values from the server
			jsbroker.call('DataPilotHandler.fieldValues', params, function(rv) {
				rv.forEach(function(sOpt, i, a) {
					sel.options.add(new Option(sOpt, sOpt));
				});
			});

			// reload the datapilot onchange
			sel.onchange = hitch(this, '_makeResult')

		}, this);
	},

	_makeResult: function()
	{
		var selection = this._wizard.getSelection();
		var source = this._wizard.getSource();

		// add any page_fields to the selection
		selection['page_fields'].forEach(function(field_values, field, a)
		{
			dumpln(field)

			// values are always stored in a select
			var frm = this.settings.getSettingsForm();
			var sel = frm[field]

			if (!sel)
				return true; // continue

			values = []
			for (var i=0; i<sel.options.length; i++) {
				if (sel.options[i].selected)
					values.push(sel.options[i].value)
			}

			selection['page_fields'][field] = values
		}, this);

		// prepare rpc arguments
		params = []
		params['source'] = source
		params['selection'] = selection

		// call the datapilot server to fetch the result
		jsbroker.call('DataPilotHandler.result', params, function(result)
		{
			// fill the body with the result
			this.bodyNode.innerHTML = result

			// scale the body so that the table fits in it.. offset of 40
			this.container.style.width = (this.bodyNode.childNodes[0].offsetWidth + 40) + 'px'

		}, this);
	},

	_showWizard: function()
	{
		this._wizard.show();
	},

	setBody: function(nNode)
	{
		this.bodyNode = nNode;
		nNode.innerHTML = 'Start the wizard!';
	}

});




