
/**
 * Basic RPC functionality.
 *
 * Extend this class if you want to have a widget that updates automaticly
 */

IWARE.RpcWidget = function()
{
	IWARE.RpcWidget.superclass.constructor.call(this);
}

YAHOO.extend(IWARE.RpcWidget, IWARE.AbstractWidget,
{
	title: 'Abstract RPC Widget',
	description: 'Extend this widget to use basic rpc behaviour',


	/**
	 * Call where to get the data from. This rpc call has to return a sql-alike array
	 * @protected
	 */
	rpcCall: '',

	/**
	 * Interval (msec) how often the refresh is called
	 */
	interval: 1000 * 60, // 1 minute default

	/**
	 * when setBody is called the widget is ready for data display. then the first fetchData. You might want to override this and call super
	 * @protected
	 */
	setBody: function()
	{
		this.fetchData();
	},

	/**
	 * makes the RPC call and calls this.refresh() when done with its data
	 * @private
	 * @final
	 */
	fetchData: function()
	{
		if (!this.rpcCall)
			throw 'RpcWidget::fetchData: you must specify an rpc call';

		jsbroker.call(this.rpcCall, this.getRpcParams(), function(data)
		{
			// schedule a new fetchData
			setTimeout(hitch(this, 'fetchData'), this.interval);

			// call refresh
			this.refresh(data);

		}, this);
	},

	/**
	 * @abstract
	 */
	refresh: function(data)
	{
	},

	/**
	 * Returns the RPC params to make the RPC call. By default it checks if
	 * you have settings defined and posts the form values. Else an empty array.
	 * Override this method if you need other params to be passed aswel.
	 * @access protected
	 */
	getRpcParams: function()
	{
		return (this.settings) ? this.settings.getFormValues() : [];
	}

});