/**
 * WEEE.TableWidget
 * A Baseclass for Widgets displaying Table Data
 *
 * @package IWARE
 * @subpackage Dashboard
 * @since May 10, 2007
 * @author Martijn van den Broek <mjbroek@gmail.com>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */

IWARE.TableWidget = function()
{
	IWARE.TableWidget.superclass.constructor.call(this);
}

YAHOO.extend(IWARE.TableWidget, IWARE.RpcWidget,
{
	title: 'Abstract Table Widget',
	description: 'A base class for displaying table data',

	/**
	 * reference to the table tbody
	 * @access private
	 */
	tbody: null,
	trhead: null,

	/**
	 * Associative object where the key is the column and value is a function. The
	 * data is passed as the first argument and you can return a new value or node.
	 * @access protected
	 */
	valueProcessors: {},

	/**
	 * fixme: doc
	 */
	rowProcessors: [],

	hideColumns: [],


	tableCssClass: '',

	/**
	 * @acces protected
	 * @final
	 */
	setBody: function(nNode)
	{
		this.table = document.createElement('TABLE');
		nNode.appendChild(this.table);

		this.table.style.width = '100%';
		this.table.className = 'Profesional ' + this.tableCssClass;

		var thead = document.createElement('THEAD');
		this.table.appendChild(thead);

		this.trhead = document.createElement('TR');
		thead.appendChild(this.trhead);

		this.tbody = document.createElement('TBODY');
		this.table.appendChild(this.tbody);

		IWARE.TableWidget.superclass.setBody.call(this);
	},

	/**
	 * Updates the table data. Set an Interval on this with an RPC call to keep updating data.
	 * @final
	 */
	refresh: function(data)
	{
		/**
		 * Check if have an empty THEAD, then get the first row and use the keys as
		 * the column headers. Also use the clientside langif to translate the
		 * column headers and make it print the translation in the node.
		 */
		if (data.count() && this.trhead && this.trhead.childNodes.length == 0)
		{
			var columns = data[0].array_keys();
			var translations = [];

			columns.forEach(function(data, i, a)
			{
				// dont add column if header is in this.hideColumns
				if (this.hideColumns.in_array(data))
					return false; // continue

				var th = document.createElement('TH');
				this.trhead.appendChild(th);

				//dumpln('translation: '+data);
				translations[data] = th;

			}, this);

			langIf.translateNodes(translations);
		}

		// remove any existing data
		removeChildNodes(this.tbody);

		if (!(data instanceof Array))
			throw 'refresh() - data is not an Array';

		// create the rows (again)
		data.forEach(function(row, i, a)
		{
			var tr = document.createElement('TR');
			tr.className = 'tr'+((parseInt(i)+1) % 2);
			this.tbody.appendChild(tr);

			if (!(row instanceof Array))
				throw 'refresh() - data row['+i+'] is not an Array';

			row.forEach(function(data, j, a)
			{
				// dont add cell if header is in this.hideColumns
				if (this.hideColumns.in_array(j))
					return false; // continue

				var td = document.createElement('TD');
				tr.appendChild(td);

				// apply valueprocessor
				if (typeof this.valueProcessors[j] != 'undefined')
					data = this.valueProcessors[j](data, row);

				if (typeof data == 'string')
					td.innerHTML = data;
				else
					td.appendChild(data);

			}, this);

			// apply rowprocessor after the cells have been created
			this.rowProcessors.forEach( function(func, k, a) {
				func.apply(this, [tr, row]);
			}, this);

		}, this);
	}

} );

