/**
 * @package IW
 * @subpackage QueryBuilder
 * @author Marten Koedam <marten AT dalines DOT org>
 */
function IWCustomQueryFactory(properties)
{
	this.properties = properties;
	this.modes = {
          text: [
            {text: "contains", value: "~" },
            {text: "doesn't contain", value: "!~" },
            {text: "begins with", value: "^" },
            {text: "ends with", value: "$" },
            {text: "is", value: "" },
            {text: "is not", value: "!" }
            ],
          number: [
            {text: "is (or)", value: "~" },
            {text: "is not (or)", value: "!~" },
            {text: "larger than", value: "^" },
            {text: "smaller than", value: "$" },
            {text: "is", value: "" },
            {text: "is not", value: "!" }
            ],
          select: [
            {text: "is", value: "" },
            {text: "is not", value: "!" }
            ],
          period: [
          	{text: "is", value: "" },
            {text: "is not", value: "!" },
            {text: "larger than", value: "^" },
            {text: "smaller than", value: "$" },
            {text: "between", value: "~" },
            {text: "not between", value: "!~" }
          	]
        };
	this.ids = [];
	this.filterIndex = [];
	this.idIndex = 0;
	this.modeSelect = {
		text: 'or',
		number: 'or',
		select: 'or',
		period: 'and'
		}
}

IWCustomQueryFactory.prototype = {

	rebuildFilters: function(filters,groupby,descending)
	{
		filters = filters?filters:false;
		groupby = groupby?groupby:false;
		descending = descending?descending:false;

		if(filters)
		{
			filters.forEach(function(element,index,array)
			{
				this.addFilter(false,element['handle'],element['value'],element['mode']);
			},this);
		}

		if(groupby)
		{
			Select_setSelectedValue($$('groupby'),groupby);
			$$('descending').disabled = '';
		}

		if(descending)
			$$('descending').checked = 'checked';
	},

	addFilter: function(sel,val,selected,selectedMode)
	{
		if(sel)
			val = sel.options[sel.selectedIndex].value;
		else
			val = val;

		props = this.properties[val];
		if(typeof(props) != 'undefined')
		{
			var newFilter = ((typeof this.filterIndex[val] == 'undefined')||(this.filterIndex[val].sizeof()==0))?true:false;
			var nNode = this.renderFilter(val+'[]',props,selected);

			nTh = document.createElement('th');
			if(newFilter)
				nTh.innerHTML = props['label'];

			nTd1 = document.createElement('td');
			nTd1.className = 'alignright';
			if(newFilter && this.modes[props['type']] &&(nModes = this.renderFilter(val+'_mode',{type: 'select', options: this.modes[props['type']]},selectedMode)))
				nTd1.appendChild(nModes); // Create the modes
			else if( this.modes[props['type']] )
				nTd1.innerHTML = this.modeSelect[props['type']];
			nTd2 = document.createElement('td');
			nTd2.appendChild(nNode);

			nInputRemove = document.createElement('input');
			nInputRemove.type = 'button';
			nInputRemove.value = '-';
			addEvent(nInputRemove,'click',function(){CQF.removeFilter(this)});
			nTdRemove = document.createElement('td');
			nTdRemove.className = 'remove';
			nTdRemove.appendChild(nInputRemove);

			nTr = document.createElement('tr');
			nTr.appendChild(nTh);
			nTr.appendChild(nTd1);
			nTr.appendChild(nTd2);
			nTr.appendChild(nTdRemove);

			el = $('filtertable').tBodies[0];
			if((typeof this.filterIndex[val] != 'undefined')&&(this.filterIndex[val].sizeof()>0))
			{
				lookupId = this.filterIndex[val].array_keys().pop(); // Add after the last
				if($(lookupId).nextSibling)
					el.insertBefore(nTr,$(lookupId).nextSibling);
				else
					el.appendChild(nTr);
			}
			else
				el.appendChild(nTr);
			nTr.id = this.addId(val);
		}
		if(sel)
			sel.selectedIndex = null;
	},
	addId: function(val)
	{
		rv = 'filterTr'+this.idIndex;
		this.ids[rv] = val;
		if(typeof(this.filterIndex[val]) == 'undefined')
			this.filterIndex[val] = new Array();
		this.filterIndex[val][rv] = this.idIndex;

		this.idIndex++;
		return rv;
	},
	renderFilter: function(handle,props,selected)
	{
		var nFilterInput = false;
		var selected = selected?selected:false;

		switch(props['type'])
		{
			case 'period':
				nFilterInput = document.createElement('input');
				nFilterInput.type='text';
				nFilterInput.name=handle;
				if(selected)
					nFilterInput.value = selected;
				else
					nFilterInput.value = 'YYYY-MM-DD';
			break;
			case 'text':
			case 'number':
				nFilterInput = document.createElement('input');
				nFilterInput.type='text';
				nFilterInput.name=handle;
				if(selected)
					nFilterInput.value = selected;
			break;
			case 'select':
				nFilterInput = document.createElement('select');
				nFilterInput.name = handle;

				if(props['options'])
				{
					props['options'].forEach( function(element, index, array)
					{
						var opt = new Option(element['text'],element['value']);
						opt.innerHTML = element['text'];
						if(selected && selected == element['value'])
							opt.selected = 'selected';
						nFilterInput.appendChild(opt);
					}, nFilterInput);
				}
			break;
			case 'radio':
				nFilterInput = document.createElement('div');
				if(props['options'])
				{
					//alert(print_r(props['options']));
					props['options'].forEach( function(element, index, array)
					{
						var opt = document.createElement('input');
						opt.type = 'checkbox';
						opt.value = element['value'];
						opt.id = handle+element['value'];
						opt.className = 'checkboxspacer';
						var lbl = document.createElement('label');
						lbl.setAttribute('for',handle+element['value']);
						lbl.innerHTML = element['text'];
						opt.name = handle;
						if(selected && selected[element['value']])
							opt.checked = 'checked';
						nFilterInput.appendChild(opt);
						nFilterInput.appendChild(lbl);
					}, nFilterInput);
				}
				// disable the option in the select.
				elSelect = $$('filter');
				opt = SelectController.prototype.getController(elSelect).getOptionByValue(handle.replace('[]',''));
				opt.disabled = 'disabled';
			break;
		}
		return nFilterInput;
	},
	setGroupBy: function(sel)
	{
		if(sel.value)
			$$('descending').disabled = '';
		else
			$$('descending').disabled = 'disabled';
	},
	removeFilter: function(rowMember)
	{
		if ( rowMember.nodeType != 1 || rowMember.tagName.toLowerCase() != 'tr' ) //rowMember == rowElement
			return IWCustomQueryFactory.prototype.removeFilter(rowMember.parentNode);

		rv = rowMember.id;
		handle = CQF.ids[rv];
		delete CQF.ids[rv];
		delete CQF.filterIndex[handle][rv];
		elSelect = $$('filter');
		opt = SelectController.prototype.getController(elSelect).getOptionByValue(handle);
		opt.disabled = '';

		var tableBodyElement = rowMember.parentNode;
		if ( rowMember.parentNode.rows.length > 1 )
			rowMember.parentNode.removeChild(rowMember); //harakiri
	},
	gotoPage: function(label,page,field,direction)
	{
		fEl = $('custom_query_form');

		el = document.createElement('input');
		el.name = 'page['+label+']';
		el.value = page;
		el.type = 'hidden';
		fEl.appendChild(el);

		field = field?field:false;
		direction = direction?direction:false;
		if(field)
			return this.orderBy(field,direction);
		fEl.submit();
	},
	setPreset: function(sel)
	{
		fEl = $('custom_query_form');
		fEl.submit();
	},
	savePreset: function(presetName,isDefault,isPublic)
	{
		fEl = $('custom_query_form');
		if(!presetName)
			return false;

		opts = {save_query: true, is_public: isPublic,is_default: isDefault,preset_name: presetName};
		for(i in opts)
		{
			if(!opts[i])
				continue;

			el = document.createElement('input');
			el.name = i;
			el.value = opts[i];
			el.type = 'hidden';
			fEl.appendChild(el);
		}

		fEl.submit();
	},
	orderBy: function(field,direction,label,page)
	{
		fEl = $('custom_query_form');

		el = document.createElement('input');
		el.name = 'orderBy';
		el.value = field;
		el.type = 'hidden';

		fEl.appendChild(el);

		el = document.createElement('input');
		el.name = 'direction';
		el.value = direction;
		el.type = 'hidden';

		fEl.appendChild(el);

		label = label?label:false;
		page = page?page:false;
		if(label)
			return this.gotoPage(label,page);

		fEl.submit();
	},
	doExport: function(eType,group)
	{
		fEl = $('custom_query_form');

		el1 = document.createElement('input');
		el1.name = 'export[format]';
		el1.value = eType;
		el1.type = 'hidden';

		fEl.appendChild(el1);

		el2 = document.createElement('input');
		el2.name = 'export[group]';
		el2.value = group;
		el2.type = 'hidden';

		fEl.appendChild(el2);

		fEl.submit();

		// Remove childs again.
		fEl.removeChild(el1);
		fEl.removeChild(el2);
	}
}