/**
 * JS Broker Object
 * JS side of the JS/PHP Ajax Broker pair
 * @author Marten Koedam <marten@basesoft.com>
 * @author Chris Wesseling <chris@basesoft.com>
 * @copyright Copyright 2005, BaseSoft b.v.
 * @package I-Ware
 * @subpackage Includes
 * @since 25 july 2005
 * @version $Id$
 */
/**
 * Contructor, initialized calls, nextCall and ajax object.
 */
function JsBroker(path,debug)
{
	this.path = path?path:'/';
	this.nextCall = 0;
	this.calls = new Array();
	//this.ajax = new AjaxObject(phpbroker);
	this.debug = debug?debug:false;
	this.windebug = false;
	this.debugI = 1;

	this.msgqueue = new Array();
	this.phpbroker  = '';
}

JsBroker.prototype.addPhpBroker = function(phpbroker)
{
	this.phpbroker = phpbroker;
}

/**
 * Add a call to the broker instance.
 * @param string method
 * @param mixed params
 * @param object (optional)
 */
JsBroker.prototype.call = function(method, params, callback, obj, doserialize)
{
	doserialize = (typeof doserialize != 'undefined') ? doserialize : true;
/*
	dumpln('RPC Call: '+method);
	dumpln(' - Callback: '+callback);
	dumpln(' - Params: '+print_r(params));
*/

	callback = callback?callback:function(){};
	document.body.style.cursor = 'progress';

	var ajax = new AjaxObject( this.phpbroker );

	if (typeof obj == 'undefined')
		this.calls[this.nextCall] = callback;
	else
		this.calls[this.nextCall] = new Array(callback, obj);
	if(doserialize) {
		var postParams = new Array();
		postParams['method'] = method;
		postParams['callId'] = this.nextCall;
		postParams['params'] = params;
		ajax.callPage(postParams);
	} else {
		var formValues = new Array();
		formValues.push(['method', method]);
		formValues.push(['callId', this.nextCall]);
		ajax.callPage(params.URLEncode(), formValues, false);
	}
	return this.nextCall++;
}
/**
 * Add a call to the broker instance, and also attach vars to the request.
 * @param array requestVars An array with each element being [name of var, value of var]
 * @param string method
 * @param mixed params
 * @param object (optional)
 */
JsBroker.prototype.callAndAttachRequestVars = function(requestVars, method, params, callback, obj)
{
	dumpln('RPC Call: '+method);
	dumpln(' - Callback: '+callback);
	dumpln(' - Params: '+print_r(params));
	callback = callback?callback:function(){};
	document.body.style.cursor = 'progress';

	var ajax = new AjaxObject( this.phpbroker );

	if (typeof obj == 'undefined')
		this.calls[this.nextCall] = callback;
	else
		this.calls[this.nextCall] = new Array(callback, obj);

	var postParams = new Array();
	postParams['method'] = method;
	postParams['callId'] = this.nextCall;
	postParams['params'] = params;

	ajax.callPage(postParams, requestVars);
	return this.nextCall++;
}

/**
 * Add a call to the broker instance, and also submit a form from the page.
 * @param string Form element Id
 * @param string method
 * @param mixed params
 * @param object (optional)
 */
JsBroker.prototype.callAndSubmitForm = function(formId, method, params, callback, obj)
{
	dumpln('RPC Call: '+method);
	dumpln(' - Callback: '+callback);
	dumpln(' - Params: '+print_r(params));
	callback = callback?callback:function(){};
	document.body.style.cursor = 'progress';

	var ajax = new AjaxObject( this.phpbroker );

	if (typeof obj == 'undefined')
		this.calls[this.nextCall] = callback;
	else
		this.calls[this.nextCall] = new Array(callback, obj);

	var postParams = new Array();
	postParams['method'] = method;
	postParams['callId'] = this.nextCall;
	postParams['params'] = params;

	var formValues = new FormBehaviourSubmission(formId).foldForm();
	ajax.callPage(postParams, formValues);
	return this.nextCall++;
}

/**
 * Cancel a call
 * @param int callId
 */
JsBroker.prototype.cancel = function(callId)
{
	if (this.calls[callId])
		delete this.calls[callId];
}
/**
 * Executes the callback function for callId
 * @param int callId
 * @param string params // Serialized
 */
JsBroker.prototype.execute = function(callId, params)
{
	if(!this.calls[callId])
		return true;
	params = unserialize(params);

	var cb = this.calls[callId];

	if (cb instanceof Array)
	{
		var m = cb[0];
		var obj = cb[1];
		m.call(obj, params);
	}
	else
		cb(params);

	delete this.calls[callId];

	if (this.calls.sizeof() == 0)
		document.body.style.cursor = 'auto';
}

/**
 * Callback function from the debug window which shows the msgs from the queue
 */
JsBroker.prototype._windebugOnLoad = function()
{
	while (msg = this.msgqueue.pop())
		this.windebug.showMessage( E_ERROR, msg['title'], msg['body'] );
}

JsBroker.prototype.handleError = function(e)
{
	throw e;
}
