/**
 * @copyright Copyright 2005, BaseSoft b.v.
 * @package I-Ware
 * @subpackage Includes
 * @version $Id$
 */

/**
 * LanguageInterface
 * Check if there is such a object.. cache translations, else make a dummy
 */
if (typeof langIf == 'undefined')
	langIf = { getText: function(arg) { return arg }, translateNodes: function() {}, translateHandles: function() {} };
else {
	addEvent(window, 'load', function() {
		langIf.translateHandles( new Array('MsgBox OK', 'MsgBox Cancel', 'MsgBox Yes', 'MsgBox No') );
	});
}

MessageBox = function() {};
MessageBox.prototype._counter = 0;
/**
 * @access private
 */
MessageBox.prototype._show = function(header, options_override)
{
	// default options for all messageboxes
	var options = {
		width: "300px",
		fixedcenter: true,
		visible: false,
		raggable: true,
		close: true,
		constraintoviewport: true,
		modal:true
	}

	// merge arguments
	for (option_name in options_override)
		options[option_name] = options_override[option_name];

	// show dialog
	var msgbox = new YAHOO.widget.SimpleDialog("MessageBox_"+this._counter, options);
	msgbox.setHeader(header);
	msgbox.render(document.body);
	msgbox.show();

	// add ID's to buttons
	var buttons = msgbox.cfg.getProperty('buttons');
	for (i=0; i<buttons.length; i++)
		buttons[i].htmlButton.id = 'msgBtn_'+this._counter+'_'+i;

	this._counter++;
}

/**
 * Static methods
 * @param string caption
 * @param string text
 * @param function optional callback function called when clicking the OK button
 */
MessageBox.prototype.Information = function(header, txt, callback )
{
	if (typeof callback == 'undefined')
		callback = function() { return false };

	var options = {
		text: txt,
		icon: 'information',
		buttons: [
			{ text:langIf.getText('MsgBox OK'), handler: function() { callback.call(); this.hide() } , isDefault:true }
		]
	}

	this._show(header, options);
}

/**
 * Shows a warning
 * @see MessageBox.prototype.Information for usages
 * <code>
 * 	MessageBox.prototype.Warning('Hello', 'But why', function() { alert('aargh!')} );
 * </code>
 */
MessageBox.prototype.Warning = function(header, txt, callback )
{
	if (typeof callback == 'undefined')
		callback = function() { return false };

	var options = {
		text: txt,
		icon: 'warning',
		buttons: [
			{ text:langIf.getText('MsgBox OK'), handler: function() { callback.call(); this.hide() } , isDefault:true }
		]
	}

	this._show(header, options);
}

/**
 * Shows a critical message
 * @see MessageBox.prototype.Information for usage
 */
MessageBox.prototype.Critical = function(header, txt, callback )
{
	if (typeof callback == 'undefined')
		callback = function() { return false };

	var options = {
		text: txt,
		icon: 'critical',
		buttons: [
			{ text:langIf.getText('MsgBox OK'), handler: function() { callback.call(); this.hide() } , isDefault: true }
		]
	}

	this._show(header, options);
}


/**
 * Displays a question, callbacks can be used to get the result
 *
 * Example:
 * <code>
 * 	MessageBox.prototype.Question('Question', 'Hello?',
 * 		function() {
 * 			alert('you pressed YES!')
 * 		},
 * 		function() {
 * 			alert('you pressed NO!')
 * 		}
 * 	);
 * </code>
 */
MessageBox.prototype.Question = function(header, txt, callbackYes, callbackNo )
{
	if (typeof callbackNo == 'undefined')
		callbackNo = function() { return false };

	var options = {
		text: txt,
		icon: 'question',
		buttons: [
			{ text:langIf.getText('MsgBox Yes'), handler: function() { callbackYes.call(); this.hide() } , isDefault:true },
			{ text:langIf.getText('MsgBox No'),  handler: function() { callbackNo.call(); this.hide() } }
		]
	}

	this._show(header, options);
}

MessageBox.prototype.Wait = function(header, body)
{
	var waitbox = new YAHOO.widget.Panel("wait",
		{
			width:"300px",
			fixedcenter:true,
			close:false,
			draggable:false,
			modal:true,
			visible:false,
			effect:{effect:YAHOO.widget.ContainerEffect.FADE, duration:0.5}
		}
	);

	body = '<div style="text-align: center" style="margin: 10px;"><p>'+body+' </p><img src="'+REL_IMAGES+'i-w/rel_interstitial_loading.gif" /></div>';

	waitbox.setHeader(header);
	waitbox.setBody(body);
	waitbox.render(document.body);
	waitbox.show();

	return waitbox;
}

MessageBox.prototype.Progress = function(header, body)
{
	var waitbox = new YAHOO.widget.Panel("progress",
		{
			width:"300px",
			fixedcenter:true,
			close:false,
			draggable:false,
			modal:true,
			visible:false,
			effect:{effect:YAHOO.widget.ContainerEffect.FADE, duration:0.5}
		}
	);

	var body = '\
		<div style="text-align: center" style="margin: 10px;"> \
			<p>'+body+' </p> \
			<div id="progress_container" style="border: 1px inset black;"> \
				<div id="progress_bar" style="background-color: rgb(61,119,203); width: 0px;">&nbsp;</div> \
			</div> \
		</div>';

	waitbox.setHeader(header);
	waitbox.setBody(body);
	waitbox.render(document.body);
	waitbox.show();

	waitbox.setProgress = function(progress)
	{
		//console.debug('progress: ' + progress);
		var container = document.getElementById('progress_container');
		var bar = document.getElementById('progress_bar');

		bar.style.width = parseInt( (container.offsetWidth / 100) * progress) + 'px'
	}

	return waitbox;
}



/*
AddEvent(window, 'load', function()
{
	var msg = MessageBox.prototype.Information( 'caption', 'message' );
	//setTimeout( function() { msg.close(); }, 5000);
});
*/

/**
 * Another class: MessageBoxProgress
 */



