/**
 * @author Chris Wesseling <chris.wesseling@xs4all.nl>
 * @author Marten Koedam <marten@basesoft.com>
 * @package IWare
 * @subpackage includes
 * @version $Id$
 */
/**
 * Tree
 *
 * interface Displayable
 * {
 *		// hide your stuff from the view.
 * 		void hide();
 *		// display stuff from the view
 * 		// param DOMNode nTarget
 * 		void display(nTarget);
 * }
 *
 * interface NavNodeContainer
 * {
 * 		// Iterate all elements in the Container by calling fCallback with Element as it's first parameter
 *		// thisObject should be 'this' in the fCallback scope.
 * 		void forEach(fCallback, thisObject);
 *
 *		//Any true, false value will do. Like Number
 *		//May return true if unsure ;) Ajax call anyone??
 * 		Boolean hasElements();
 * }
 *
 * @implements Displayable
 * @param String title 'Leave name'
 * @param String iconClass 'CSS-Class for icon display'
 * @param Boolean bOpen Start the tree node in an "expanded" state?
 * @param NavNodeContainer aChildren Displayables which are children of this node.
 */
NavigationNode = function(title, iconClass, bOpen, aChildren)
{
	this.title = title.toString();
	this.iconClass = iconClass?iconClass.toString():this.iconClass;
	this.status = bOpen?bOpen:false;
	if ( aChildren != UNDEFINED
			&& aChildren.hasElements != UNDEFINED
			&& aChildren.forEach != UNDEFINED ) //if aChildren implements NavNodeContainer
		this.children = aChildren;
}
NavigationNode.prototype =
{
	status: '',
	title: '',
	iconClass: '',
	children: [],
	nextSibling: false,
	previousSibling: false,
	lastChild: false,
	parentNavNode: null, //parent in the tree
	_node: null, //Main container Element
	_control: null, //expand/collapse control Element
	_title: null, //Link Element
	negerinBehaviour: function(){ this.toggle();}, //ggetserae strategy method
	display: function(nTarget)
	{
		//if ( this._node )
			//return removeCssClass(this._node, 'hidden');

		this._node = document.createElement('DIV');
		this._control = document.createElement('DIV');
		this._title = document.createElement('A');

		this._node.id = this.generateId(nTarget, this.title);
		this._node.className = 'NavNode';
		if ( this.nextSibling )
			addCssClass(this._node, 'hasNext');

		this._control.className = 'Control';

		this._title.className = 'Title'+(this.iconClass?' '+this.iconClass:'');
		this._title.href = ''; //to make it draggable.
		this._title.innerHTML = this.title;

		this._node.appendChild(this._control);
		this._node.appendChild(this._title);
		nTarget.appendChild(this._node);

		if ( this.children.hasElements() )
		{
			addCssClass(this._control, 'collapsed')
			addEvent(this._control, 'click', hitch(this, 'toggle'));
		}

		addEvent(this._title, 'click', hitch(this, 'perform'));
		if ( this.status )
			this.expand();
	},
	expand: function()
	{
		if ( this.status == 'expanded' )
			return;
		removeCssClass(this._control, 'collapsed');
		if ( this.children.hasElements() )
		{
			delete this.lastChild;
			this.children.forEach(
					function(element, ix, a) //ix & a, are here only for speed reasons iff container is an Array
					{
						element.parent = this;
						if ( this.lastChild )
						{
							element.previousSibling = this.lastChild;
							this.lastChild.nextSibling = element;
							this.lastChild.display(this._node); //lastChilds view of his family is complete now..
						}
						this.lastChild = element;
					}, this);
			this.lastChildAction = 'display';
			this.handleLastChild();

			addCssClass(this._control, 'expanded'); //set control expanded iff hasChildren
		}
		this.status = 'expanded';
	},
	handleLastChild: function()
	{
		if ( this.lastChild && this.lastChildAction )
			this.lastChild[this.lastChildAction](this._node);
	},
	collapse: function()
	{
		if ( this.status == 'collapsed' )
			return;
		removeCssClass(this._control, 'expanded');
		if ( this.children.hasElements() )
		{
			addCssClass(this._control, 'collapsed'); //set control iff hasChildren
			this.children.forEach(this.cbHide);
			this.lastChildAction = 'hide';
		}
		this.status = 'collapsed';
	},
	cbHide: function(element, ix, a) //callback used for hiding children
	{
		element.hide();
	},
	hide: function()
	{
		if ( !this._node )
			return;

		removeEvent(this._control, 'click', hitch(this, 'toggle'));
		removeEvent(this._title, 'click', hitch(this, 'perform'));

		['lastChild', '_title', '_control', '_node'].forEach(
				function (node, i, a)
				{
					removeNode(this[node]);
					delete this[node];
				}, this);
	},
	toggle: function()
	{
		if ( !this.status || (this.status == 'collapsed') )
			 return this.expand();
		 return this.collapse();
	},
	perform: function(e)
	{
		e.preventDefault?e.preventDefault():null;
		e.returnValue = false;
		this.negerinBehaviour(); //strategy hook
	},

	generateId: function(parent, title)
	{
		var id = title.replace(/\W/g, "");
		if(parent && parent.className && parent.className.match(/NavNode/))
			id = parent.id + '_' + id;
		else
			id = 'NavNode_'+ id;
		return id;
	},

	getTitlePath: function()
	{
		path = [this.title];
		p = this.parent;
		while(p)
		{
			path.push(p.title);
			p = p.parent;
		}
		return path.reverse();
	}
}

/**
 * @implements NavNodeContainer
 */
AsyncContainer = function(sCallMethod, aCallParams)
{
	this.__callMethod = sCallMethod;
	this.__callParams = aCallParams;
	this.__nodeCache = [];
	this.__loadingNode = new NavigationNode('Loading...', 'loadingIcon', false, []);
}

AsyncContainer.prototype =
{
	__callId: -1,//JsBroker callId
	__callMethod: '', //JsBroker callmethod
	__callParams: [], //JsBroker callParams
	__callback: function(){},//callback passed to forEach
	__parentNavNode: {}, //'this' object in callback.. The 'parentNode'
	__nodeCache: [], //cache remembering current nodes.
	__loadingNode: {},
	hasElements: function()
	{
		return true;
	},
	forEach: function(fCallback, oThis)
	{
		//remember callback && thisObject
		this.__callback = fCallback;
		if ( oThis )
			this.__parentNavNode = oThis;

		if (  this.__parentNavNode
				&& (fCallback === this.__parentNavNode.cbHide) ) //hide??
			return this.__nodeCache.forEach(fCallback, oThis); //use realnodes !!

		//else display... get new nodes
		//this.__callback.call(this.__parentNavNode, this.__loadingNode); //this should be the same as the next dirty line
		//todo FIXME
		if(!this.__loadingNode._node)
			this.__loadingNode.display(this.__parentNavNode._node);
		//Cancel current call
		if ( this.__callId >= 0 )
			jsbroker.cancel(this.__callId);
		//Do new Ajax Call
		this.__callId = jsbroker.call(this.__callMethod,
				this.__callParams,
				this.receiveElements,
				this);
	},
	receiveElements: function(aElements)
	{
		this.__loadingNode.hide();
		aElements.forEach(function(el, i, a)
				{
					var node = this.createNewNode(el);
					this.__nodeCache.push(node);
					if ( this.__parentNavNode )
						this.__callback.call(this.__parentNavNode, node, i, a);
					else
						this.__callback(node, i, a);
				}, this);
		if ( this.__parentNavNode )
			this.__parentNavNode.handleLastChild();
		if ( aElements.count() == 0 )
			this.__parentNavNode.collapse();
	},
	createNewNode: function(aParams) //strategy factory method
	{
		var nav = new NavigationNode(aParams['title'],
			aParams['iconClass'], false, new AsyncContainer(aParams['callMethod'], aParams['callParams']));
		return nav;
	}
}
