selectControllers = {};
function SelectController(nModel)
{
	//Poor man's private
	if ( SelectController.caller )
	{
		if ( SelectController.caller !== SelectController.prototype.getController )
			throw 'SelectController() is private. Use SelectController.prototype.getController()';
	}

	//don't notifyObservers.
	this._silent = false;
	this._observers = [];
	this._model = nModel;
}

SelectController.prototype =
{
	//static factory method
	getController: function(nModel)
	{
		var c;
		if ( c = selectControllers[nModel.id+nModel.name] )
		{
			c.checkModel(nModel);
			return c;
		}
		return window.selectControllers[nModel.id+nModel.name] = new SelectController(nModel);
	},
	checkModel: function(nModel)
	{
		if ( this._model !== nModel )
		{
			this._model = nModel;
			this.notifyObservers();
		}
	},
	destroy: function()
	{
		delete selectControllers[this._model.id+this._model.name];
	},
	getOptions: function()
	{
		var rv = [];
		for ( var i=0; i < this._model.options.length; i++ )
			rv.push(this._model.options[i]);
		return rv;
	},
	getSelectedOptions: function()
	{
		var rv = [];
		this.getOptions().forEach(function(opt, idx, options)
				{
					if ( opt.selected )
						rv.push(opt);
				});
		return rv;
	},
	addOption: function(nOption)
	{
		if ( nOption.getAttribute('text') && !nOption.innerHTML )
			nOption.innerHTML = nOption.getAttribute('text');
		nOption.innerHTML = nOption.innerHTML.toString().trim();
		if ( !nOption.innerHTML )
			return;
		//check if already exist
		var add = true;
		this.getOptions().forEach(function(o, i, a)
				{
					if ( o.innerHTML == nOption.innerHTML )
						add = false;
				}, this);
		if ( !add )
			return;
		//if not exists
		if ( Browser.isMSIE )
			this._model.options.appendChild(nOption, this._model);
		else
			this._model.options.add(nOption);

		this.notifyObservers();
	},
	addOptions: function(aOptions)
	{
		if ( !aOptions.count() )
			return;
		this._silent = true;
		aOptions.forEach(this.addOption, this);
		this._silent = false;
		this.notifyObservers();
	},
	replaceOptions: function(aOptions)
	{
		this._silent = true;
		this.removeAllOptions();
		this._silent = false;
		this.addOptions(aOptions);
	},
	removeOption: function(nOption)
	{
		var rv = false;
		this.getOptions().forEach(function (opt, index, options)
				{
					if ( opt === nOption )
						rv = opt.parentNode.removeChild(opt);
				});
		if ( rv )
			this.notifyObservers();
		return rv;
	},
	removeSelectedOptions: function()
	{
		var rv = [];
		this.getOptions().forEach(function(opt, idx, options)
				{
					if ( opt.selected )
						rv.push(opt.parentNode.removeChild(opt));
				});
		if ( rv.count() )
			this.notifyObservers();
		return rv;
	},
	removeAllOptions: function()
	{
		var rv = [];
		this.getOptions().forEach(function(opt, idx, options)
				{
					rv.push(opt.parentNode.removeChild(opt));
				});
		if ( rv.count() )
			this.notifyObservers();
		return rv;
	},
	getOptionByValue: function(sValue)
	{
		var rv = false;
		this.getOptions().forEach(function (opt, index, options)
				{
					if ( opt.value == sValue )
						rv = opt;
				});
		return rv;
	},
	setSelectedByValue: function(aValues) //can be sValue like Select_setSelectedValue() in global.js
	{
		aValues = (getObjectType(aValues) != 'Array')?[aValues]:aValues; //cast
		this.getOptions().forEach(function(nOption, i, a)
				{
					nOption.selected = aValues.in_array(nOption.value);
				});
		//do notifications??? or leave that to onchange??
	},
	removeOptionByValue: function(sValue)
	{
		var rv = this.getOptionByValue(sValue)
		if ( rv )
			rv.parentNode.removeChild(rv);
		this.notifyObservers();
		return rv;
	},
	registerObserver: function(oObserver)
	{
		if ( ! oObserver.notifyChanged )
			throw 'Observer doesn\'t implement Observer interface';
		this._observers.push(oObserver);
	},
	removeObserver: function(oObserver)
	{
		this._observers.forEach(function(observer, ix, observers)
				{
					if ( observer === oObserver )
						delete observers[ix];
				});
	},
	notifyObservers: function()
	{
		if ( this._silent )
			return;
		this._observers.forEach(function(observer, ix, observers)
				{
					observer.notifyChanged(this);
				}, this);
	},
	getModel: function()
	{
		return this._model;
	}
}

fancySelectControllers = {}; //this could be a prototype attribute... They're shared over all instances...
function FancySelectController(sName)
{
	//Poor man's private
	if ( FancySelectController.caller )
	{
		if ( FancySelectController.caller !== FancySelectController.prototype.getController )
			throw 'FancySelectController() is private. Use FancySelectController.prototype.getController()';
	}

	this._silent = false;
	this._queue = 0;
	this._observers = [];
	this._name = sName;
	this._hiddenBuilder = {
				_name : (this._name.match(/\[\]$/)?this._name:this._name+'[]'),
				_hiddens : $$s(this._name), //TODO does this leak memory?
				notifyChanged: function(selCont)
				{
					this._hiddens.forEach(removeNode);
					//rebuild
					selCont.getOptions().forEach(function(opt, i, a)
						{
							var hidden = document.createElement('input');
							hidden.type = 'hidden';
							hidden.value = opt.value;
							hidden.name = this._name;
							this._hiddens.push(selCont.getModel().parentNode.appendChild(hidden));
						}, this);
				}
			};
	this._optionModel = SelectController.prototype.getController($(this._name+'_options'));
	this._selectedModel = SelectController.prototype.getController($(this._name+'_selected'));
	this._selectedModel.registerObserver(this._hiddenBuilder);
	this._selectedModel.registerObserver(this);
	this._hiddenBuilder.notifyChanged(this._selectedModel); //initialize could be done on submit......

}

FancySelectController.prototype =
{
	//static factory method
	getController: function(sName)
	{
		var c;
		if ( c = fancySelectControllers[sName] )
		{
			c.checkModels();
			return c;
		}
		return window.fancySelectControllers[sName] = new FancySelectController(sName);
	},
	checkModels: function()
	{
		this._selectedModel = SelectController.prototype.getController($(this._name+'_selected'));
		this._optionModel = SelectController.prototype.getController($(this._name+'_options'));
		this._hiddenBuilder.notifyChanged(this._selectedModel); //initialize could be done on submit......
	},
	moveRight: function()
	{
		this._silent = true;
		this._selectedModel.addOptions(this._optionModel.removeSelectedOptions());
		this._silent = false;
		this.sendQueue();
	},
	moveAllRight: function()
	{
		this._silent = true;
		this._selectedModel.addOptions(this._optionModel.removeAllOptions());
		this._silent = false;
		this.sendQueue();
	},
	moveLeft: function()
	{
		this._silent = true;
		this._optionModel.addOptions(this._selectedModel.removeSelectedOptions());
		this._silent = false;
		this.sendQueue();
	},
	moveAllLeft: function()
	{
		this._silent = true;
		this._optionModel.addOptions(this._selectedModel.removeAllOptions());
		this._silent = false;
		this.sendQueue();
	},
	destroy: function()
	{
		[this._selectedModel, this._optionModel].forEach(function(m, i, a)
				{
					m.destroy();
				});
		delete fancySelectControllers[this._name];
	},
	getOptions: function()
	{
		var rv = [];
		[this._selectedModel, this._optionModel].forEach(function(m, i, a)
				{
					rv = rv.concat(m.getOptions());
				});
		return rv;
	},
	getSelectedOptions: function()
	{
		return this._selectedModel.getOptions();
	},
	addOption: function(nOption)
	{
		if ( !nOption.innerHTML && nOption.getAttribute('text') )
			nOption.innerHTML = nOption.getAttribute('text');

		nOption.innerHTML = nOption.innerHTML.toString().trim();

		var add = true;
		this.getOptions().forEach(function(o, i, a)
				{
					if ( o.innerHTML == nOption.innerHTML )
					{
						var n = this._optionModel.removeOption(o);
						if ( n )
							this._selectedModel.addOption(n);
						add = false;
					}
				}, this);
		if ( add )
			this._selectedModel.addOption(nOption);
	},
	addOptions: function(aOptions)
	{
		this._selectedModel.addOptions(aOptions);
	},
	removeOption: function(nOption)
	{
		var rv = false;
		[this._selectedModel, this._optionModel].forEach(function(m, i, a)
				{
					if ( m.removeOption(nOption) )
						rv = nOption;
				});
		return rv;
	},
	removeOptionByValue: function(sValue)
	{
		var rv = false;
		[this._selectedModel, this._optionModel].forEach(function(m, i, a)
				{
					if ( opt = m.removeOptionByValue(sValue) )
						rv = opt;
				});
		return rv;
	},
	registerObserver: function()
	{
		SelectController.prototype.registerObserver.apply(this, arguments);
	},
	notifyChanged: function()
	{
		if ( this._silent )
			this._queue++;
		else
			this.notifyObservers();
	},
	sendQueue: function()
	{
		if ( this._queue > 0 )
			this.notifyObservers();
		this._queue = 0;
	},
	notifyObservers: function()
	{
		SelectController.prototype.notifyObservers.apply(this);
	},
	removeObserver: function(oObserver)
	{
		SelectController.prototype.removeObserver.apply(this, arguments);
	}
}
