/**
 * SortingHat
 * Sorts tables
 * @param mixed table (string) TableId || (HTMLTableElement) table
 * @param object/array functionmap ( (int)colnum=>(function)compare function )
 */

function SortingHat(table, functionmap)
{
	if ( typeof table == 'string' )
		this.table = document.getElementById(table);
	else
		this.table = table;

	if (typeof functionmap != 'undefined')
		this.functionmap = functionmap;

	this.directionmap = new Array();
	this.sorting = false; //lock to 1 sort at a time;
	this.imgId = false;
	this.imgDirection = 'up';
}

/**
 * Sets the functionmap
 * @param object/array functionmap ( (int)colnum=>(function)compare function )
 */
SortingHat.prototype.setFunctionMap = function(functionmap)
{
	this.functionmap = functionmap;
}

/**
 * Sort a column
 * @param int col, colnum
 */
SortingHat.prototype.sort = function(col,th,thIdentifier)
{
	th = th?th:false;
	thIdentifier = thIdentifier?thIdentifier:false;
	if ( this.table.tBodies[0].rows.length == 1 )
		return true;
	if ( this.sorting )
		return false;
//	var msg = MessageBox.prototype.Wait('Sorting...','Please wait while doing nothing...');

	if (typeof this.functionmap == 'undefined')
		throw 'this.functionmap not set';

	if ( typeof this.functionmap[col] != 'function' )
		this.functionmap[col] = this['cmp'+this.functionmap[col]];
	if ( typeof this.functionmap[col] != 'function' )
		return false;

	if(th)
	{
		var tmpId = 'sortImg'+thIdentifier;
		if(this.imgId && tmpId==this.imgId)
		{
			el = $(this.imgId);
			this.imgDirection = (this.imgDirection=='up')?'down':'up';
			el.src = '/'+SITE_PATH+'images/i-w/if_arrow_'+this.imgDirection+'.gif';
		}
		else
		{
			if(el = $(this.imgId))
				el.parentNode.removeChild(el);
			th.innerHTML = th.innerHTML+' <img src="/'+SITE_PATH+'images/i-w/if_arrow_down.gif" id="'+tmpId+'" />';
			this.imgDirection = 'down';
		}
		this.imgId = tmpId;
	}

	var rv = this.sorting = true;
	this.directionmap[col] = this.directionmap[col]?this.directionmap[col]:1;
	try
	{
		var rows = this.stqsort(this.table.tBodies[0].rows, col);
		this.directionmap[col] = -1 * this.directionmap[col];

		this.table.replaceChild(document.createElement('tbody'), this.table.tBodies[0]);
		var i = 0;
		var index = 0;
		while ( row = rows.shift() )
		{
			if ( (index = row.className.search(/\d+$/)) > -1 )
			{
				row.className = row.className.substring(0, index)+i;
				i = i?0:1;
			}

			this.table.tBodies[0].appendChild(row);
		}
	}
	catch(e)
	{
		//Catch or all sorting will automagicly break with this error
		var rv = false;
	}
	finally
	{
		this.sorting = false;
		document.body.style.cursor = 'auto';
	}
	return rv;
//	msg.close();
}
/**
 * Stable Quicksort
 * @param array rows
 * @param int col
 */
SortingHat.prototype.stqsort = function(rows, col)
{
	var lt = new Array();
	var eq = new Array();
	var gt = new Array();

	var cnt = rows.length;

	if ( cnt <= 1 )
		return rows;

	var pV = rows[Math.round(cnt/2)].cells[col];
	for ( var i=0 ; i < cnt ; i++ )
	{
		var cmp = this.functionmap[col](rows[i].cells[col], pV);
		switch ( cmp )
		{
			case this.directionmap[col] * 1:
				gt.push(rows[i]);
			break;
			case 0:
				eq.push(rows[i]);
			break;
			case this.directionmap[col] * -1:
				lt.push(rows[i]);
			break;
		}
	}
	return this.stqsort(lt, col).concat(eq, this.stqsort(gt, col));
}

/**
 * Simple compare function
 * Does a > b, a==b nothing more.
 * @param mixed a
 * @param mixed b
 * @return int -1|0|1
 */
SortingHat.prototype._cmp = function(a, b)
{
	if ( a > b )
		return 1;
	else if ( a < b )
		return -1;
	else
		return 0;
}

/**#@+
 * Compare function
 * @param HTMLTableCellElement a
 * @param HTMLTableCellElement b
 * @return int -1|0|1
 */

/**
 * Compares parseInt(innerHTML)
 * if parseInt isNaN it compares innerHTML
 */
SortingHat.prototype.cmpInt = function(ca, cb)
{
	var a = parseInt(ca.innerHTML, 10);
	var b = parseInt(cb.innerHTML, 10);
	a = isNaN(a)?ca.innerHTML:a;
	b = isNaN(b)?cb.innerHTML:b;
	return SortingHat.prototype._cmp(a,b);
}
/**
 * Compares parseFloat(innerHTML)
 * if parseFloat isNaN it compares innerHTML
 */
SortingHat.prototype.cmpFloat = function(ca, cb)
{
	var a = parseFloat(ca.innerHTML);
	var b = parseFloat(cb.innerHTML);
	if(isNaN(a))
		return 0;
	if(isNaN(b))
		return 1;
	return SortingHat.prototype._cmp(a,b);
}
/**
 * Compares Int(striptags(innerHTML))
 */
SortingHat.prototype.cmpIntNoTags = function(ca, cb)
{
	var re = /\<[^<]*>/g;
	var a = ca.innerHTML.replace(re, '').toLowerCase();
	var b = cb.innerHTML.replace(re, '').toLowerCase();

	a = parseInt(a, 10);
	b = parseInt(b, 10);
	a = isNaN(a)?ca.innerHTML.replace(re, '').toLowerCase():a;
	b = isNaN(b)?cb.innerHTML.replace(re, '').toLowerCase():b;

	return SortingHat.prototype._cmp(a,b);
}
/**
 * Compares date(innerHTML)
 * dd mm yyyy with any non digit delimiter
 */
SortingHat.prototype.cmpDate = function(ca, cb)
{

	var re = /(\d+)\D*(\d+)\D*(\d*)/;
	var a = re.exec(ca.innerHTML);
	var b = re.exec(cb.innerHTML);
	a = a?new Date(a[3], a[2]-1, a[1]).valueOf():new Date(0);
	b = b?new Date(b[3], b[2]-1, b[1]).valueOf():new Date(0);
	return SortingHat.prototype._cmp(a,b);
}
/**
 * Compares String(innerHTML)
 */
SortingHat.prototype.cmpString = function(ca, cb)
{

	var a = ca.innerHTML;
	var b = cb.innerHTML;
	return SortingHat.prototype._cmp(a.toLowerCase(),b.toLowerCase());
}
/**
 * Compares String(striptags(innerHTML))
 */
SortingHat.prototype.cmpStringNoTags = function(ca, cb)
{

	var re = /\<[^<]*>/g;
	var a = ca.innerHTML.replace(re, '').toLowerCase();
	var b = cb.innerHTML.replace(re, '').toLowerCase();
	return SortingHat.prototype._cmp(a,b);
}
/**
 * Shuffles ;)
 */
SortingHat.prototype.cmpShuffle = function(ca, cb)
{
	return Math.round((Math.random()*2)-1);
}

/**
 * Compares by the sort_val attribute in a TD
 * Useful when you want to display another value in the innerHTML and sort by this value
 * for example, full date and sort by timestamp or localed quanity and sort by real value
 */
SortingHat.prototype.cmpSortVal = function(nA, nB)
{
	var a = parseInt(nA.getAttribute('sort_val'), 10);
	var b = parseInt(nB.getAttribute('sort_val'), 10);
	a = isNaN(a)?0:a;
	b = isNaN(b)?0:b;
	return SortingHat.prototype._cmp(a,b);
}

//**#@-*/
