/**
 * Checks if a value exists in an array
 * @param mixed value
 * @param bool strict (default false) Use value === element
 */
Array.prototype.in_array = function(value, strict)
{
	strict = strict?strict:false;
	var container = {strict:strict, value:value, rv:false}

	this.forEach(
		function(element,index,array)
		{
			if ( this.strict && element === this.value )
				return this.rv = true;
			else if ( !this.strict && element == this.value )
				return this.rv = true;
		}
	, container);
	return container.rv;
}
/**
 * @deprecated use in_array
 */
Array.prototype.inArray = Array.prototype.in_array;

Array.prototype.append = function()
{
	var c,o,i,rv=[];
	this.forEach(function(el, id, arr)
	{
		rv[id] = el;
	});

	if (typeof arguments == 'Array' || typeof arguments == 'Object')
		throw "Expected object or array argument";

	for ( c = 0; c < arguments.length ; c++ )
	{
		o = arguments[c];
		for ( i in o )
			rv[i] = o[i];
	}
	return rv;
}

/**
 * Do a callback on every Array member
 * "Return true" in your callback, like you would use a "break" in php foreach
 *
 * @param function callback(element,index,array) Even if you don't use index or array in your callback, declaring them will make it run faster. For some reason.
 * @param object thisObject Object to use as this when executing callback
 *
 *
 * example:
 * <code>
 *	//create the Victim
 * 	var arr = new Array();
 * 	for ( var i=0; i<1000 ; i++ )
 * 		arr.push(1);
 * 	var container = {sum:0, rv=false}; //prepare a container for hitching stuff into the callback scope
 *	//This will find out if the sum of elements is at least 300; In a weird way ;)
 * 	arr.forEach( function(element, index, array)
 * 					{
 * 						this.sum += element;
 * 						if ( this.sum >= 300 )
 * 							return this.rv = true;
 * 					}//end callback function
 * 				,container); //see the last param!
 * 	//container.rv is now the (bool) you're looking for.
 * </code>
 *
 * Now we can also give the second argument thisObject. The 'this' in example below will refer
 * to the the object instead of the callback function. Example:
 * <code>
 * 	Wizard.prototype.updateRoadMap = function() {
 * 		roadMap.forEach( function(element, index, array)
 * 		{
 * 			var li = document.createElement('LI');
 * 			li.innerHTML = element;
 * 			this._roadMapList.appendChild(li);
 * 		}, this);
 * 	}
 * </code>
 */
//if ( typeof Array.prototype.forEach == 'undefined' ) //this is implemented in JS 1.5
Array.prototype.forEach = function(callback, thisObject)
{
	for(var index in this)
	{
		if ( !(index in Array.prototype) )
		{
			var element = this[index];
			if (typeof thisObject == 'undefined')
			{
				if ( callback(element, index, this) )
					return;
			}
			else
			{
				if ( callback.call(thisObject, element, index, this) )
					return;
			}
		}
	}
};

/**
 * Filters elements of an array using a callback function.
 * array_filter returns a new Array with all elements for which callback(element) returns true;
 * @param Function callback
 * @return Array
 */
Array.prototype.array_filter = function(callback)
{
	var container = { rv:new Array(), callback: callback };
	this.forEach(function(el, i, a)
			{
				if ( this.callback(el) )
					this.rv[i] = el;
			}
			, container);
	return container.rv;
}
/**
 * Counts all occurences of the elementvalues
 * WARNING! This is not a save function
 * eg. Array(1,2,1,'count','count','length','length', 'toString').array_count_values()
 * Will *NOT* work as expected. I leave the mess as a practice to the reader.
 * @return Array
 */
Array.prototype.array_count_values = function()
{
	var container = { rv:new Array()};
	this.forEach(
		function(element,index,array)
		{
			if(typeof element == 'string' || typeof element == 'number')
			{
				if(this.rv[element])
					this.rv[element]+=1;
				else
					this.rv[element]=1;
			}
		}, container
	);
	return container.rv;
}
/**
 * Shuffle an array
 */
Array.prototype.shuffle = function()
{
	var rv = new Array();
	var keys = this.array_keys();
	var i = 0;
	var total = multiplier = keys.sizeof();
	while(total)
	{
		i = Math.floor(Math.random()*multiplier);
		if(!rv[keys[i]])
		{
			rv[keys[i]] = this[keys[i]];
			total--;
		}
	}
	return rv;
}
/**
 * Pad array to the specified length with a value
 */
Array.prototype.array_pad = function(size,value)
{
	funct = (size<0)?'unshift':'push';
	size = (size<0)?(size*-1):size;
	if(size<=this.sizeof())
		return;
	while(size>this.sizeof())
	{
		if(funct=='unshift')
			this.unshift(value);
		else
			this.push(value);
	}
	return this;
}
/**
 * Computes the intersection of arrays using keys for comparison
 * Returns an array containing all the values of $this that have keys that are present in $arr
 */
Array.prototype.array_intersect_key = function()
{
	var c = { rv:[], diff:false, key:true, cmp:[], m:'array_keys' };
	this.__parseDiffArgs(arguments, c);
	this.forEach(this._diff, c);

	return c.rv;
}
/**
 * Computes the intersection of arrays
 * Returns an array containing all the values of $this that are present in $arr
 */
Array.prototype.array_intersect = function()
{
	var c = { rv:[], diff:false, cmp:[], m:'array_values' };
	this.__parseDiffArgs(arguments, c);
	this.forEach(this._diff, c);

	return c.rv;
}
/**
 * Computes the difference of arrays using keys for comparison
 * Returns an array containing all the values of $this that have keys that are not present in $arr
 */
Array.prototype.array_diff_key = function()
{
	var c = { rv:[], diff:true, key:true, cmp:[], m:'array_keys' };
	this.__parseDiffArgs(arguments, c);
	this.forEach(this._diff, c);

	return c.rv;
}
/**
 * Computes the difference of arrays
 * Returns an array containing all the values of $this that are not present in $arr
 */
Array.prototype.array_diff = function()
{
	var c = { rv:[], diff:true, cmp:[], m:'array_values' };
	this.__parseDiffArgs(arguments, c);
	this.forEach(this._diff, c);
	return c.rv;
}
Array.prototype.__parseDiffArgs = function(args, c)
{
	for ( var i=0; i<args.length ; i++ )
		c.cmp = c.cmp.concat(args[i][c.m]());
}
/**
 * Returns an array with all string keys lowercased or uppercased
 * @param string toCase (upper|lower)
 */
Array.prototype.array_change_key_case = function(toCase)
{
	toCase=toCase?toCase:'lower';
	var c = {rv:[],
		m:'to'+toCase.slice(0,1).toUpperCase()+toCase.slice(1).toLowerCase()+'Case' //method
	};
	this.forEach(
		function(element,index,array)
		{
			if(typeof index != 'string')
				this.rv[index] = element;
			else
				this.rv[index[this.m]()] = element;
		}, c
	);
	return c.rv;
}
/**
 * Calculate the sum of values in an array
 */
Array.prototype.array_sum = function()
{
	var c = {rv:0};
	this.forEach(
		function(element,index,array)
		{
			if(typeof element == 'number')
				this.rv += element;
		}, c
	);
	return c.rv;
}
/**
 * Count elements in an array, or properties in an object
 * @return Number
 */
Array.prototype.count = function()
{
	var size = 0;
	for(var index in this)
		if( !(index in Array.prototype) )
			size++;
	return size;
}
/**
 * Alias for count()
 * @see Array.prototype.count()
 */
Array.prototype.sizeof = Array.prototype.count;
/**
 * Is here to implement the NavNodeContainer interface.
 * It should return a boolean, but that just makes it slower.
 */
Array.prototype.hasElements = Array.prototype.count;
/**
 * Join array elements with a string
 * @param String glue
 * @param String pieces
 */
Array.prototype.implode = function(glue)
{
	var c = {glue:glue.toString(), rv:''};
	this.forEach( function(element,index,array)
		{
			if ( this.rv )
				this.rv += this.glue;
			this.rv += element;
		}, c);
	return c.rv;
}
/**
 * Checks if the given key or index exists in the array
 * <code>
 * new Array().array_key_exists(/^b/); //checks if there's a key starting with b
 * </code>
 * @param mixed key Number, String or Regexp
 * @return bool
 */
Array.prototype.array_key_exists = function(key)
{

	if ( key instanceof RegExp )
	{
		var c = {rv:false, re:key};
		this.forEach( function (element, index, array)
		{
			if ( index.match(this.re) )
				return this.rv = true;
		}, c);
		return c.rv;
	}
	else
		return ( isset(this[key]) && !isset(Array.prototype[key]) )
}
/**
 * Checks if any of the keys of an array converted into a RegExp matches given String
 * @param String str
 * @return bool
 */
Array.prototype.array_key_match = function(str)
{
	var c = {rv:false, str:str};
	this.forEach( function(element, index, array)
	{
		if ( this.str.match(new RegExp(index)) )
			return this.rv = true;
	}, c);
	return c.rv;
}
/**
 * Return all the keys of an array
 * @param mixed sv (Search Value)
 * @param bool strict (== or ===)
 * @return Array rv
 */
Array.prototype.array_keys = function(sv,strict)
{
	strict = strict?strict:false;
	var c = {rv: new Array(), sv:sv, strict:strict};
	this.forEach(this.__array_keys, c);
	return c.rv;
}
/**
 * Callback for array_keys
 * @param mixed element
 * @param mixed index
 * @param Array array
 * @return False
 */
Array.prototype.__array_keys = function(element,index,array)
{
	if ( !isset(this.sv) )
	{
		this.rv.push(index);
		return;
	}
	if ( isset(this.sv) && !this.strict && this.sv == element)
	{
		this.rv.push(index);
		return;
	}
	if ( isset(this.sv) && this.strict && this.sv === element)
		this.rv.push(index);
}
/**
 * Return all the values of an array
 */
Array.prototype.array_values = function()
{
	var c = {rv:new Array()};
	this.forEach(
		function(element,index,array)
		{
			this.rv.push(element);
		}, c
	);
	return c.rv;
}
/**
 * Fill THIS array with values
 * @param Number si StartIndex
 * @param Number num number
 * @param mixed value
 * @return void
 */
Array.prototype.array_fill = function(si, num, value)
{
	for(var i=si;i<(si+num);i++)
		this[i] = value;
}
/**
 * Exchanges all keys with their associated values in an array
 */
Array.prototype.array_flip = function()
{
	var c = {rv:[]};

	this.forEach(
		function(element,index,array)
		{
			if ( ! (element.toString in Array.prototype) )
				this.rv[element] = index;
		}, c
	);

	return c.rv;
}
/**
 * Pick one or more random entries out of an array
 */
Array.prototype.array_rand = function(num)
{
	num=num?num:1;
	var tmp = this.shuffle();

	var c = { rv:[], num:num };
	tmp.forEach(
		function(element,index,array)
		{
			if(this.num)
			{
				this.rv[index] = element;
				this.num--;
			}
		}, c
	);

	return c.rv;
}
/**
 * Extract variables by index from this Array and returns them
 * Returns the deleted element(s) as an array
 * @param mixed index
 */
Array.prototype.extract = function(index)
{
	rv = new Array();
	if(typeof index == 'Array')
		index = new Array(new String(index));

	for(var i=0; i < index.sizeof(); i++ )
	{
		if(this[index[i]])
		{
			rv[index[i]] = (this[index[i]]);
			delete this[index[i]];
		}
	}
	return rv;
}
/**
 * Deletes variables by index from this
 * Returns the number of elements deleted
 * @param mixed index
 * @return int
 */
Array.prototype.array_delete = function(index)
{
	return this.extract(index).sizeof();
}
/**#@+
 * @access protected
 */
/**
 * @param mixed element
 * @param mixed index
 * @param Array array
 */
Array.prototype._diff = function(element,index,array)
{
	if ( !this.key )
	{
		if ( this.diff && !this.cmp.in_array(element) )
			this.rv[index] = element;
		if ( !this.diff && this.cmp.in_array(element) )
			this.rv[index] = element;
	}
	else
	{
		if ( this.diff && !this.cmp.in_array(index) )
			this.rv[index] = element;
		if ( !this.diff && this.cmp.in_array(index) )
			this.rv[index] = element;
	}
}
/**#@-*/
/**
 *
 */
function ArrayParams()
{
	this.params = new Array();
}
/**
 * Sets a local ArrayParams parameter
 * @param mixed param
 *
 */
ArrayParams.prototype.set = function(param,value)
{
	this.params[param] = value;
}
/**
 *
 */
ArrayParams.prototype.get = function(param)
{
	return this.params[param];
}
var ArrayP = new ArrayParams();
