/**
 * AutoComplete
 *
 * @version $Id: global.js 1270 2005-11-22 10:01:36Z Martijn van den Broek $
 * @package I-Ware
 *
 * <code>
 * 	var ac = new AutoComplete( document.getElementById('txtMyText'), 'dictionary' );
 * 	ac.maxWords = 15; // change from max 20 to 15
 * </code>
 */

/**
 * Constructor
 * @param text element
 * @param source call (optional)
 */
function AutoComplete(el, src)
{
	this.timeout = 200; // time after which the rpc is called
	this._el = el; /// dom element
	this._el.autocomplete = 'off'; /// disable browsers own autocomplete
	this._tm_id = false; // timeout id
	this._selectedIndex = -1; /// selectedIndex (using arrows and mouse)
	this.maxWords = 20; // max words to return

	if (typeof src != 'undefined')
		this.setSource(src);

	addEvent(el, 'keypress', hitch(this, 'onKeyPress'));
	addEvent(el, 'blur', hitch(this, '_showBox', false));
}

/**
 * @param string sourceCall
 */
AutoComplete.prototype.setSource = function(src)
{
	this._acSource = src;
}

/**#@+
 * @access private
 */

AutoComplete.prototype.onKeyPress = function(ev)
{
	//window.status = ev.keyCode;

	var skipKeys = new Array(37, 39); // key left, key right,
	if (skipKeys.in_array(ev.keyCode))
		return;

	if (ev.keyCode == 13) // ENTER
	{
		if (this._selectedIndex >= 0)
		{
			this._el.value = this._box.childNodes[this._selectedIndex].innerHTML;
			this._showBox(false);

			ev.preventDefault();
			return false;
		}
	}

	// KPUP
	if (ev.keyCode == 38)
	{
		this._showBox(true);
		this._selectPrev();
		ev.preventDefault();
		return;
	}

	// KPDOWN
	if (ev.keyCode == 40)
	{
		this._showBox(true);
		this._selectNext();
		ev.preventDefault();
		return;
	}

	// BACKSPACE, maybe build a cache someday?
	if (ev.keyCode == 8)
		this._showBox(false);


	// hide box if no value
	if (this._el.value == '')
		this._showBox(false);

	// Normal procedure..
	// cancel any previous calls
	if (this._tm_id)
		clearTimeout(this._tm_id);

	// load some new words
	this._tm_id = setTimeout(hitch(this, '_loadWords'), this.timeout);
}

AutoComplete.prototype._selectPrev = function()
{
	if (this._selectedIndex > 0)
		this._select(this._selectedIndex-1);
}

AutoComplete.prototype._selectNext = function()
{
	if (this._selectedIndex < this._box.childNodes.length-1)
		this._select(this._selectedIndex+1);
}

AutoComplete.prototype._select = function(selectedIndex, select)
{
	if (typeof select == 'undefined')
		select = true;

	// de-select old and select new
	if (this._selectedIndex != -1)
		this._box.childNodes[this._selectedIndex].className = 'Word';

	if (selectedIndex != -1 && this._box.childNodes[selectedIndex])
		this._box.childNodes[selectedIndex].className = 'Word selected';

	// save new selected
	this._selectedIndex = selectedIndex;
}

AutoComplete.prototype._loadWords = function()
{
	if (this._acSource == 'undefined')
	{
		alert('AutoComplete - no source defined.. use setSource();');
		return;
	}

	var params = new Array();
	params['source'] = this._acSource;
	params['word'] = this._el.value;
	params['maxWords'] = this.maxWords;
	this._callid = jsbroker.call( 'AutoComplete', params, hitch(this, 'cbAutoComplete') );
}

AutoComplete.prototype._showBox = function(display)
{
	if (typeof display == 'undefined')
		display = true;

	if (!this._box)
	{
		if (Browser.isMSIE)
		{
			var br = document.createElement('BR');
			br = insertAfter(br, this._el);

			var box = document.createElement('DIV');
			this._box = insertAfter(box, br);

			// TODO: we probably want iframe crap here too

		}
		else
		{
			var box = document.createElement('DIV');
			this._box = insertAfter(box, this._el);
		}
		this._box.className = 'AutoComplete';
	}

	this._box.style.display = display ? '' : 'none';
	this._box.style.width = this._el.offsetWidth -10 + 'px'; // padding
}

AutoComplete.prototype._clearBox = function()
{
	if (this._box);
		removeChildNodes(this._box);

	this._selectedIndex = -1;
}

AutoComplete.prototype.cbAutoComplete = function(words)
{
	if (words.length == 0)
	{
		this._showBox(false);
		return;
	}

	this._clearBox();
	this._showBox();

	words.forEach( function(element, index, array)
	{
		var div = document.createElement('DIV');
		div.className = 'Word';
		this._box.appendChild( div );
		div.innerHTML = element;
		addEvent(div, 'mouseover', hitch(this, '_select', index));
		addEvent(div, 'mouseover', function() { this.style.cursor = 'default'; } );

		addEvent(div, 'mousedown', hitch(this, 'wordClick', element));
	}, this);
}

AutoComplete.prototype.wordClick = function(word)
{
	this._el.value = word;
	this._showBox(false);
}

/**#@-*/