/**
 * class FormValidator
 * @version $Id: formvalidator.js 35 2005-04-13 07:18:43Z Martijn van den Broek $
*/
/*

	FormValidator

	EXAMPLE:

	function submit_click()
	{
		if ( fv.validate() )
			document.frm.submit();
	}

	var fv;

	function body_onload()
	{
		var fv = new FormValidator();
		fv.addElement( 'Bedrijf',          document.getElementById('PressCustomerCompanyName')                );
		fv.addElement( 'Opdrachtgever',    document.getElementById('PressCustomerClientName')                 );
		fv.addElement( 'Naam',             document.getElementById('PressCustomerName'),       'name'         );
		fv.addElement( 'Adres',            document.getElementById('PressCustomerAddress')                    );
		fv.addElement( 'Postcode',         document.getElementById('PressCustomerPostalCode')                 );
		fv.addElement( 'Plaats',           document.getElementById('PressCustomerCity'),       'name'         );
		fv.addElement( 'Telefoon',         document.getElementById('PressCustomerPhone'),      'phone'        );
		fv.addElement( 'Fax',              document.getElementById('PressCustomerFax'),        'phone', false );
		fv.addElement( 'Mobiele telefoon', document.getElementById('PressCustomerCell'),       'phone', false );
		fv.addElement( 'E-mail',           document.getElementById('PressCustomerEmail'),      'email'        );
		fv.addElement( 'Website',          document.getElementById('PressCustomerWebsite'),    'url',   false );
		fv.addElement( 'Opmerkingen',      document.getElementById('PressCustomerRemarks'),    null,    false );
	}

*/

function FormValidator()
{
	this.elements = new Array();

	this.regex = new Array();
	this.regex['alphanumeric'] = /^([a-zA-Z0-9\s])+$/;             //-- match lower and uppercase chars, numbers and whitespace
	this.regex['name']         = /^([a-zA-Z\s])+$/;                //-- match lower and uppercase chars and whitespace
	this.regex['phone']        = /^([0-9])+([0-9\-\s])*([0-9])+$/; //-- match numbers with in the middle one or more minusses (-)
	this.regex['email']        = /^([a-zA-Z0-9_\.\-])+\@(([a-zA-Z0-9\-])+\.)+([a-zA-Z0-9]{2,4})+$/;
	this.regex['url']          = /^([\d\w\W])+$/

}

FormValidator.prototype.addElement = function(caption, element, checktype, mandatory)
{
	checktype = checktype || 'alphanumeric';
	mandatory = (mandatory == false) ? false : true; //-- not as above cuz underined false wouldnt come trough either :/

	if ( !(checktype == 'alphanumeric' || checktype == 'name' ||
	       checktype=='email' || checktype=='url'    || checktype=='phone') )
	{
		alert('FormValidator::addElement - Type is not a valid type ('+checktype+')');
	}

	if (!element) {
		alert('FormValidator::addElement() - Supplied argument is not a valid element.');
		return;
	}

	var arr = new Array();
	arr['caption']   = caption;
	arr['element']   = element;
	arr['checktype'] = checktype;
	arr['mandatory'] = mandatory;


	this.elements.push( arr );
}

FormValidator.prototype.markElement = function(element, bool)
{
	if (bool)
	{
		element.style.borderColor = '#F00';
		element.style.backgroundColor = '#FDF0F0';
	}
	else
	{
		element.style.borderColor = '#CCC';
		element.style.backgroundColor = '#F9F9F9';
	}
}

FormValidator.prototype.validate = function()
{
	valid = true;
	var emsg = 'Please correct the folowing errors: \n'; //-- error message

	for (var i=0; i<this.elements.length; i++)
	{
		var caption   = this.elements[i]['caption'];
		var element   = this.elements[i]['element'];
		var checktype = this.elements[i]['checktype'];
		var mandatory = this.elements[i]['mandatory'];

		if (!mandatory && element.value == '') //-- not mandatory and empty always is valid
		{
			this.markElement(element, false);
		}
		else if (mandatory && element.value == '') //-- mandatory and emtpy never is valid
		{
			this.markElement(element, true);
			emsg += ' - ' + caption + ' is empty \n';
			valid = false;
		}
		else //-- mandatory and not empty: go check if the content is valid
		{
			if (this.regex[checktype])
			{
				var filter = this.regex[checktype];
				if (!filter.test(element.value))
				{
					this.markElement(element, true);
					emsg += ' - ' + caption + ' is not valid \n';
					valid = false;
				}
				else
					this.markElement(element, false);
			}
		}
	}

	if (!valid)
		MessageBox.prototype.Critical( '', emsg.nl2br());

	return valid;
}
