<?php
/**
 * Functions for Site Wide Global use.
 *
 * @package I-Ware
 * @subpackage includes
 * @version $Id: functions.php 476 2005-07-11 13:44:58Z martijn van den broek $
 */

/**
 * Displays a transparent pixel of $width x $height
 *
 * @param integer $width
 * @param integer $height
 * @return string <img />
 */
function mkDot($width=1, $height=1)
{
	return '<img src="/'.SITE_PATH.'images/0.gif" height="'.$height.'" width="'.$width.'" />';
}

/**
 * Add the drag and drop functionality to the $output var
 *
 * @param string $output Full $dragInfo output with '{draganddrop}' placeholder
 * @param string $title StoneLabel
 * @param integer $id $dragInfo ID
 * @param string $dragInfo ['Stone'|'Question'|'SubQuestion'|'Answer'...]
 * @return string Param $output with '{draganddrop}' replaced (depending on $GLOBALS['source'])
 */
function parseDragDrop( $output, $title, $id, $dragInfo = "Stone" )
{
	global $source;
	if ($source != S_NORMAL)
		return str_replace( "{draganddrop}", "", $output );

	$replace = " ondrop=\"drop(this)\" ondragover=\"overDrag()\" ondragenter=\"enterDrag()\" myLabel=\"'".$title."\" dragInfo=\"".$dragInfo."\" dragId=\"".$id."\"";
	$output = str_replace( "{draganddrop}", $replace, $output );

	$return = "<a href=\"#\" name=\"".$id.$dragInfo."\" myLabel=\"".$title."\" dragInfo=\"".$dragInfo."\" dragId=\"".$id."\" ondragstart=\"startDrag()\" ondragend=\"endDrag()\">";
	$return .= $output.= "</a>";

	return $return;
}

/**
 *
 **/
function parsePaging( $total, $limit, $current, $defLink, $size='95%', $fields=6, $pre_pagenr='[', $post_pagenr=']', $separator='' )
{
	$pages = ceil( $total / $limit );
	if( $pages <= 1 )
		return "";
	$i = 0;
	$content = "<table width=\"".$size."\" cellpadding=\"0\" cellspacing=\"0\">
		<tr><td class=\"paging\" align=\"right\">";

	if( $pages > 10 )
	{
		$show_first = false;
		$show_last = false;
		if(( $current - ($fields) ) < 0 )
			$start = 0;
		else
		{
			$start = ( $current - $fields+1 );
			$show_first = true;
		}
		if(( $current + $fields ) > $pages )
			$end = $pages;
		else
		{
			$end = ( $current + $fields );
			$show_last = true;
		}

		if( $show_first )
			$content .= $pre_pagenr."<a href=\"".$defLink."&page=0\">1</a>".$post_pagenr." ... ";

		for( $i = $start; $i < $end; $i++ )
		{
			if( $current != $i )
				$content .= $pre_pagenr."<a href=\"".$defLink."&page=".$i."\">".( $i + 1 )."</a>".$post_pagenr;
			else
				$content .= $pre_pagenr."<b>".( $i + 1 )."</b>".$post_pagenr;

			$content .= $separator;
		}

		$content = substr($content, 0, strlen($content) - strlen($separator) );

		if( $show_last )
			$content .= " ... ".$pre_pagenr."<a href=\"".$defLink."&page=".( $pages - 1 )."\">".$pages."</a>".$post_pagenr;
	}
	else
	{
		$i = 0;
		while( $i < $pages )
		{
			if( $current != $i )
				$content .= $pre_pagenr."<a href=\"".$defLink."&page=".$i."\">".( $i + 1 )."</a>".$post_pagenr;
			else
				$content .= $pre_pagenr."<b>".( $i + 1 )."</b>".$post_pagenr;
			$i++;
		}
	}
	return $content."</td></tr></table>";
}
/**
 * Export a php Array (or any other thing) to a JavaScript Array (or any other thing)
 * @param array $a php Array to export
 * @param string $n Legal name for JavaScript var
 * @return string
 */
function exportArrayToJS($a, $n='')
{
	return ($n?'var '.$n.' = ':'').'unserialize(\''.jsescape(mb_serialize($a), '\'').'\')'.($n?";\n":'');
}

function exportArrayToJsObject($a)
{
	$rv = '{';
	foreach ($a as $key=>$value)
	{
		if (is_object($value))
			$value = (array) $value;

		if (is_array($value))
			$rv .= jsescape($key) .': '.exportArrayToJsObject($value, false);
		else
			$rv .= jsescape($key).':\''.jsescape($value).'\'';

		$rv .= ',';
	}

	$rv = substr($rv, 0, -1);

	$rv .= "}";

	return $rv;
}

/**
 * Escapes a string for javascript
 * <code>alert('<?=jsescape($mystring);?>');</code>
 */
function jsescape($js, $escape='"')
{
	$search = array('\\',"\n","\r","\t",$escape);
	$replace = array('\\\\','\n','\r','\t','\\'.$escape);
	return str_replace($search, $replace, $js);
}

/**
 * A MultiByte safe serialize
 * Safe for usage with JavaScript unserialize
 * @param mixed $var thing to serialize
 * @return string serialized $var
 */
function mb_serialize($var)
{
	$firsttry = serialize($var);
	if ( mb_strlen($firsttry, 'utf-8') == strlen($firsttry) )
		return $firsttry;
	$strings = '';
	__mb_serialize_getReplacements($var, $strings);
	return str_replace(array_keys($strings), array_values($strings), $firsttry);
}

function __mb_serialize_getReplacements($var, &$rv)
{
	$rv || $rv = array();
	$normalshit = array('integer', 'boolean', 'double', 'NULL');

	if ( in_array(gettype($var), $normalshit) )
		return 0;

	if ( gettype($var) == 'string' )
		if ( ($mblen = mb_strlen($var, 'utf-8')) != ($len = strlen($var)) )
			return $rv[serialize($var)] = preg_replace('/^s:'.$len.':/', 's:'.$mblen.':', serialize($var));
		else
			return true;

	//array or object
	if ( gettype($var) == 'object')
		$var = get_object_vars($var);

	foreach ( $var as $key=>$value )
	{
		__mb_serialize_getReplacements($key, $rv);
		__mb_serialize_getReplacements($value, $rv);
	}
	return true;
}

function mb_unserialize($serial_str) {

	//$out = preg_replace('!s:(\d+):"(.*?)";!se', "'s:'.strlen('$2').':\"$2\";'", $serial_str );
	$out = preg_replace_callback('!s:(\d+):"(.*?)";!s', function($m) {

		return 's:'.strlen($m[2]).':"'.$m[2].'";';

	}, $serial_str);

	return unserialize($out);
}

/**
 * Replaces tag characters with html entities so a html can be shown as string
 */
function htmlescape($html)
{
	return preg_replace( array('/</', '/>/'), array('&lt;', '&gt;'), $html);
}

/**
 * JavaScript Translation
 * Returns a translation prepared for javascript strings
 */
function jstr($str, $langIf=null)
{
	if (is_null($langIf))
		$langIf = IWRegistry::registry('langIf');

	return jsescape($langIf->getText($str));
}

/**
 * Short log functions
 */
function debug($str)
{
	if(!defined('TEST_SITE'))
		return;

	$log = getLog();
	$log->debug($str);
}
function info($str)
{
	$log = getLog();
	$log->info($str);
}
function alert($str)
{
	$log = getLog();
	$log->alert($str);
}
function notice($str)
{
	$log = getLog();
	$log->notice($str);
}


/**
 * Some xml helper functions
 */
define('XML_TAG_NORMAL', 0);
define('XML_TAG_CLOSED', 1);
define('XML_TAG_SINGLE', 2);

function xmlentities($string)
{
   //return str_replace ( array ( '&', '"', "'", '<', '>' ), array ( '&amp;' , '&quot;', '&apos;' , '&lt;' , '&gt;' ), $string );

	$from = array ( '&', '"', "'", '<', '>' );
	$to = array ( '&amp;' , '&quot;', '&apos;' , '&lt;' , '&gt;' );

	$rv = $string;

	for ($i=0; $i<count($from); $i++)
		$rv = mb_ereg_replace($from[$i], $to[$i], $rv);

	return $rv;
}

/**
 * @param tagname
 * @param tagtype XML_TAG_NORMAL|XML_TAG_CLOSED|XML_TAG_SINGLE
 */
function xml_tag($name, $type=XML_TAG_NORMAL, $args=array())
{
	$retval = '';

	$retval .= '<';
	if ($type == XML_TAG_CLOSED)
		$retval .= '/';
	$retval .= $name;

	foreach ($args as $key=>$value)
		$retval .= ' ' . $key . '="'.xmlentities($value).'"';

	if ($type == XML_TAG_SINGLE)
		$retval .= '/';
	$retval .= '>';

	if ($type == XML_TAG_CLOSED)
		$retval .= "\n";

	return $retval;
}

/**
 * Quick tag
 * @param string tagname
 * @param string value
 * @param array arguments
 */
function xml_quicktag($name, $value, $args=array())
{
	$retval = xml_tag($name, XML_TAG_NORMAL, $args);
	$retval .= xmlentities($value);
	$retval .= xml_tag($name, XML_TAG_CLOSED);
	return $retval;
}
/**
 *
 */
function multi_unique($array) {
   foreach ($array as $k=>$na)
       $new[$k] = serialize($na);
   $uniq = array_unique($new);
   foreach($uniq as $k=>$ser)
       $new1[$k] = unserialize($ser);
   return ($new1);
}

/**
* Converts bytes to a human readable string
* @param int $bytes Number of bytes
* @param int $precision Number of decimal places to include in return string
* @param array $names Custom usage strings
* @return string formatted string rounded to $precision
*/
function prettyBytes($sizeInBytes, $precision = 2)
{
        return ($sizeInBytes < 1024)?"$sizeInBytes bytes":round(($sizeInBytes / pow(1024,floor(log($sizeInBytes,1024)))),$precision)." ".substr(" KMGT",log($sizeInBytes,1024),1)."b";
}

/**
 * from http://nl2.php.net/manual/en/function.ini-get.php
 */
function return_bytes($val) {
   $val = trim($val);
   $last = strtolower($val{strlen($val)-1});
   switch($last) {
       // The 'G' modifier is available since PHP 5.1.0
       case 'g':
           $val *= 1024;
       case 'm':
           $val *= 1024;
       case 'k':
           $val *= 1024;
   }

   return $val;
}

/**
 * @see php.net/manual/en/function.array-combine.php
 */
if (!function_exists('array_combine_custom')) {
   function array_combine_custom($a, $b) {
       $c = array();
       if (is_array($a) && is_array($b))
           while (list(, $va) = each($a))
               if (list(, $vb) = each($b))
                   $c[$va] = $vb;
               else
                   break 1;
       return $c;
   }
}

/**
 * @param string help_postfix
 * @param array props
 */
function printHelp($langIf, $handle, $props=array(),$return=false)
{
	if (!isset($props['top']))
		$props['top'] = '5px';

	if (!isset($props['right']))
		$props['right'] = '5px';

	if (!isset($props['image']))
		$props['image'] = REL_IMAGES . 'i-w/help.gif';

	if (!isset($props['float']))
		$props['float'] = 'right';

	if (!isset($props['vertical-align']))
		$props['vertical-align'] = '0';

	// langIf can now be null too
	if (!is_null($langIf))
		$translation = trim($langIf->getText('HELP '.$handle));
	else
		$translation = $handle;

	// only show help if translation set
	if (!empty($translation)) {

		$title = (isset($props['title'])) ? $langIf->getText('HELP Title: ' . $props['title']) : '';
		$js  = (empty($title)) ? '' : '<strong>'.$title.'</strong><br/><br/>';
		$js .= $translation;

		$id = uniqid('imgHelp_');


		$img = ' <img src="'.$props['image'].'" id="'.$id.'" style="position: relative; display: none; ';# float: '.$props['float'].'; top: '.$props['top'].'; right: '.$props['right'].'; vertical-align: '.$props['vertical-align'].';display: none;" />';
		foreach( array('float', 'top', 'right', 'vertical-align') as $prop )
			if ( !empty($props[$prop]) )
			{
				if ( in_array($prop, array('top', 'right')) && is_numeric($props[$prop]) )
					$props[$prop] .= 'px';
				$img .= $prop.': '.$props[$prop].'; ';
			}
		$img .= '" />';
		$script = "if (typeof help != 'undefined') {
						help.registerById('".$id."', \"".jsescape($js)."\");
						document.getElementById('".$id."').style.display = '';
					}
				";
		if($return)
			return array($img,$script);

		echo $img.'
			<script type="text/javascript"><!--
			addEvent(window, \'load\', function()
				{
					'.$script.'
				});
			//--></script>';
	}
}

function getHelp($langIf, $handle, $props=array())
{
	ob_start();
	printHelp($langIf, $handle, $props);
	$html = ob_get_contents();
	ob_clean();
	return $html;
}

function getInterfaceHelp($langIf,$handle,$props=array())
{
	return printHelp($langIf,$handle,$props,true);
}

/**
 * Locale Number Formatter
 */
function locale_nf($number,$dec=0)
{
	if($number==='') return '';
	$locale = localeconv();
	//PHP is retarded and returns garbish for latvian thousands sep
        //so we need to check for sane values
        $t_sep = $locale['thousands_sep'];
        if(isset($t_sep[0]) && $t_sep[0] == chr(194)) //194 == UTF-8 non-breaking space
                $t_sep = ' ';

        return number_format($number,
		$dec,
		$locale['decimal_point'],
		$t_sep);
}

/**
 *
 */
function getLog()
{
	require_once(IWARE.'IWSyslog.php');
	return IWSyslog::create();
}

/**
 * adds a string to page title, should make an xhtml object for this kinda stuff someday
 */
function setPageTitlePostfix($postfix)
{
	global $page_title_postfix;
	$page_title_postfix = $postfix;
}

define('MSG_BOX_QUEST', 0);
define('MSG_BOX_INFO',  1);
define('MSG_BOX_WARN',  2);
define('MSG_BOX_CRIT',  3);
/**
 * @param int $type MSG_BOX_QUEST | MSG_BOX_INFO | MSG_BOX_WARN | MSG_BOX_CRIT
 * @param string $title
 * @param strign $body
 * @param string callback1
 * @param string callback2
 */
function jsMessageBox($type, $title, $body, $callb1='', $callb2='')
{
	$types[MSG_BOX_QUEST] = 'Question';
	$types[MSG_BOX_INFO]  = 'Information';
	$types[MSG_BOX_WARN]  = 'Warning';
	$types[MSG_BOX_CRIT]  = 'Critical';

	return "MessageBox.prototype.".$types[$type]."('".jsescape($title, "'")."', '".jsescape($body, "'")."'". (!empty($callb1)?", ".$callb1:''). ((!empty($callb2)) ? ", ".$callb2:'')  .");";
}

/**
 * Used for testing, modify the date to your likings.
 *
 * Used for testing, modify the date to your likings. Requires PEAR/Date
 * @return Date &$date
 */
function &currentDate($datetime=false)
{
	global $__dt;
	if($datetime&&defined('DATETIME_FILE')&&defined('TEST_SITE')&&file_exists(DATETIME_FILE))
	{
		$__dt = $datetime;
		file_put_contents(DATETIME_FILE, $datetime);
	}
	elseif(!isset($__dt) || !$__dt)
	{
		$__dt = null;
		if(defined('DATETIME_FILE')&&defined('TEST_SITE')&&file_exists(DATETIME_FILE))
		{
			$__dt = file_get_contents(DATETIME_FILE);
		}
	}

	$date = new IWDateTime($__dt);
	return $date;
}

/**
 * This sets the ELC virtual date
 */
function setVirtualDate($date)
{
	$log = getLog();
	$log->info("New virtual date set to " . $date);

	$fp = fopen(DATETIME_FILE,'w+');
	fwrite($fp, $date.' 00:15:00');
	fclose($fp);

}

/**
 * Returns a PEAR::Date for the given value and format
 * @todo make compatible with YY
 * @todo add optional time stuff
 */
function createDate($value, $format="DD-MM-YYYY")
{
	$f_pieces = array_flip(explode('-', $format));
	$v_pieces = explode('-', trim($value));
	if(count($v_pieces)!=3)
		return false;

	$year = $v_pieces[ $f_pieces['YYYY'] ];
	$month = $v_pieces[ $f_pieces['MM'] ];
	$day = $v_pieces[ $f_pieces['DD'] ];

	$date = new IWDateTime();
	$date->setDate($year, $month, $day);

	return $date;
}

/**
 * Formats a date from a mask to a mask
 * @param string subject
 * @param string srcmask (DD-MM-YY)
 * @param string srcmask (YY-MM-DD)
 * @return string
 */
function reindex_date($subject, $srcmask='DD-MM-YYYY', $destmask='YYYY-MM-DD')
{
	if (empty($subject))
		return '';

	$subject_pieces = explode('-', $subject);
	$srcmask_pieces = explode('-', $srcmask);
	$destmask_pieces = explode('-', $destmask);

	$map = array();
	foreach ($subject_pieces as $key=>$value)
		$map[$srcmask_pieces[$key]] = $subject_pieces[$key];

	$destmap = array();
	foreach ($destmask_pieces as $dest_piece)
		$destmap[] = $map[$dest_piece];

	return implode('-', $destmap);
}

/**
 * headers to message.php with the requested message_handle
 * messages should be defined in that file.
 * @param string msg_handle
 */
function header_message($msg_handle)
{
	// never header to message if its message itself,
	// or phpbroker for translations
	$script = basename($_SERVER['SCRIPT_NAME']);

	if ($script == 'message.php' || $script == 'PhpBroker.php' || $script == 'XmlRpcServerEvent.php' || $script == 'SoapServer.php')
		return;

	header('location: /'.SITE_PATH.'scripts/i-w/default/message.php?msg='.$msg_handle);
	die();
}


/**
 * Returns the value for the given string in array
 * <code>
 * $str = 'baard[aap]';
 * echo inputStrToArr($str, array('baard'=>array('aap'=>'kwak')) ); // prints kwak
 * </code>
 * @param string $str
 * @param Array $arr
 */
function inputStrToArr($str, $arr)
{
	// The name is not an array, check Post data
	if(false===strpos($str,'['))
		$value = isset($arr[$str])?$arr[$str]:false;
	else
	{
		// The name is an array, eval it and check Post data
		$postVar = "['".implode("']['", explode('[',str_replace(']','',$str)))."']";
		$cmd = '$value = isset($arr'.$postVar.')?$arr'.$postVar.':false;';
		eval($cmd);
	}

	return $value;
}

/**
 * This function will validate the POST size against the maximum allowed.
 */
function validatePOSTLength()
{

	if ( $_SERVER['REQUEST_METHOD'] == 'POST' && empty($_POST) && empty($_FILES) && $_SERVER['CONTENT_LENGTH'] > 0 )
	{
		$error = 'Posted data is too large. '.$_SERVER['CONTENT_LENGTH'].' bytes exceeds the maximum size of '.ini_get('post_max_size').' bytes.';
		throw new IWException($error);
	}
}

/**
 *  todo: translate
 */
function ago($timestamp)
{
   $difference = time() - $timestamp;
   $periods = array("second", "minute", "hour", "day", "week", "month", "years", "decade");
   $lengths = array("60","60","24","7","4.35","12","10");
   for($j = 0; $difference >= $lengths[$j]; $j++)
   $difference /= $lengths[$j];
   $difference = round($difference);
   if($difference != 1) $periods[$j].= "s";
   $text = "$difference $periods[$j] ago";
   return $text;
}

/**
 * This function checks the input for valid input. It will throw an exception if
 * input is filtered.
 * @param mixed $user_input The array with values that need to be checked
 * @return array The array with the input (unchanged)
 */
function sanitize_array_recursive($user_input)
{
	if(is_array($user_input)) {
		foreach($user_input as $key => $value) {
			$user_input[$key] = sanitize_array_recursive($user_input[$key]);
		}
	} else {
		$original = $user_input;
		$filtered = filter_var($original, FILTER_SANITIZE_STRING, FILTER_FLAG_NO_ENCODE_QUOTES);

		if ($original != $filtered) {
			// send a forbidden code
			header('HTTP/1.1 403 Forbidden');
			// make a ticket with which var is modified
			alert(sprintf('Input data filtered: %s', $original));
			// present the user with a message
			throw new IWException('Security Exception: Input is filtered. The system administrator has been notified.');
		}

		$user_input = $filtered;
	}

	return $user_input;
}


/**
 * This function enables logging of important user visible events for ELC
 * All params are null by default allowing you to log all combinations of the
 * associated params
 *
 * It is assumed that you will not use this function call until init.php has run
 * and the db class has been required and db connection has been made.
 */
function log_elc_event($msg, $user_id = null, $prod_id=null, $crso_id=null, $part_id=null)
{
    $db = IWRegistry::registry('db');

    $sql  = "
        INSERT INTO
            TBL_ACTIVITY_LOG(
                USER_ID, PRODUCT_ID, CRSO_RELATION_ID, PARTICIPANT_RELATION_ID, MESSAGE, CREATED
            )VALUES(?, ?, ?, ?, ?, NOW())
    ";
    $query = $db->query($sql);
    $query->bind_values($user_id, $prod_id, $crso_id, $part_id, $msg);
    try
    {
        $query->execute();
    }
    catch(Exception $e)
    {
        //pass - we do not want integrity constraint exceptions to go anywhere
    }
}

function br2nl($string){
  $return=eregi_replace('<br[[:space:]]*/?'.
    '[[:space:]]*>',chr(13).chr(10),$string);
  return $return;
}


function cracklibCheck($password, &$message)
{
	if(!defined('CRACKLIB_PATH')) {
		define('CRACKLIB_PATH', '/usr/sbin/');
	}

	// Clean up password
	$password=str_replace("\r", "", $password);
	$password=str_replace("\n", "", $password);

	// Run password through cracklib-check
	exec("echo ".escapeshellarg($password)." | ".CRACKLIB_PATH."cracklib-check 2>/dev/null", $output, $return_var);

	// Check it ran properly
	if($return_var==0) {
		if(preg_match("/^.*\: ([^:]+)$/", $output[0], $matches))  {
			// Check response
			if(strtoupper($matches[1])=="OK")  {
                // Password is strong
                $message="";
				return(true);
			} else {
				// Cracklib doesn't like it
				$message=$matches[1];
				return(false);
			}
        } else  {
			// Badly formatted response from cracklib-check.
			throw new Exception("Didn't understand cracklib-check response.");
		}
	} else  {
		// Some sort of execution error
		throw new Exception("Failed to run cracklib-check.");
	}
}

?>
