/**
 * JS Help class
 *
 * Example:
 * <code>
 * 		AddEvent(window, 'load', function()
 * 		{
 * 			//-- add the translation 'help_freeVar' to a TD with id 'tdFreeVar'
 * 			help.registerById( 'tdFreeVar', 'help_freeVar' );
 *
 * 			//-- register some elements in the document.
 * 			//   translations in langif will have 'help_' pre-added to it
 * 			help.registerById( 'fsReportSingle' );
 * 			help.registerById( 'fsReportAdvanced' );
 * 			help.registerById( 'fsSelectSampleType' );
 * 			help.registerById( 'fsSelectBuilding' );
 * 			help.registerById( 'fsSelectSpaces' );
 * 			help.registerById( 'fsSelectSpaceCategories' );
 * 			help.registerById( 'fsSelectSpaceTypes' );
 * 			help.registerById( 'fsSelectPeriods' );
 *
 * 			//-- start the help.
 * 			help.init();
 * 		});
 */

function Help()
{
	this.alwaysOn = false;
    this.enabled = false;

    this.elements     = new Array();
    this.translations = new Array();
    this.admin_mode = false;

    addEvent(window, 'load', hitch(this, 'init'));;
}

Help.prototype.init = function()
{
	if (this.alwaysOn)
		this.enabled = true;

    //-- create the help dialog, dont show it yet
    var div = document.createElement('DIV');
    div.id = 'divHelp';
    div.style.backgroundColor = '#ffffee';
    div.style.border = '1px solid black';
    div.style.padding = '5px';
    div.style.position = 'absolute';
    div.style.zIndex = 100000;
    div.style.display = 'none';
    div.style.overflow = 'visible';
    div.style.textAlign = 'left';
    div.style.fontSize = '8pt';

	div.onmouseover = function () { this.style.cursor = 'pointer'; }
	addEvent(div, 'click', hitch(this, 'close'));


    document.body.insertBefore(div, document.body.childNodes[document.body.childNodes.length-1]);

    //-- create the iframe behind the dialog (IE - <select> fix)
    if (Browser.isMSIE)
    {
        var ifr = document.createElement('IFRAME');
        ifr.id  = 'ifrHelp';
        ifr.zIndex = 5;
        ifr.style.position = 'absolute';
        ifr.style.display = 'none';
        ifr.style.padding = 0;
        ifr.style.margin = 0;
        ifr.style.filter = 'alpha(opacity=20)';
        ifr.src = '/' + SITE_PATH + 'scripts/i-w/blank.html';

        document.body.insertBefore(ifr, document.body.childNodes[document.body.childNodes.length-1]);
        //document.frames['ifrHelp'].document.location.href = 'about:blank';
    }

    //-- add the event that the help dialog will close when clicked in nowhere (body)
    //AddEvent(document.body, 'click', function() { help.close(); } );
}

Help.prototype.registerById = function (id, translation_handle )
{
	translation_handle = translation_handle || 'help_' + id;

    if(id)
	{
        var el = document.getElementById(id);
        if (el)
        {
            el.onmouseover = function()
            {
                if (help && help.enabled )
                {
                    this.style.cursor = 'help';

                    //-- since we gonna override the current onclick event,
                    //   lets store the old one and restore it later (after click)
                    //if (this.onclick)
                        //help.temp_click = this.onclick;

                    //-- now set the new onclick
                    this.onclick = function(e)
                    {
                        if (help.enabled)
                        {
                            //-- show the help dialog
                            help._showHelp(this.id, e);

                            //-- restore the old onclick (if any)
                            /*
                            if (help.temp_click)
                            {
                                this.onclick = help.temp_click;
                                help.temp_click = null;
                            }
                            */

							if ( !help.alwaysOn )
                            	help.enabled = false;
                        }

                        //-- IE only: this will make sure the <body> event onclick wont
                        //   be called (else it will instant close the help dialog)
                        if (document.all)
                            window.event.cancelBubble = true;
                    }
                }
                else
                    this.style.cursor = 'default';
            }
        }

		this.elements[id] = translation_handle;
    }
}

Help.prototype.enableHelp = function( bool )
{
	if (typeof bool == 'undefined')
		bool = true;

    this.enabled = bool;
}

//-- private function show called by onclick on element
Help.prototype._showHelp = function(id, event)
{
    //-- make sure the dialog is closed before we open it
    this.close();

	var theTop = 0;
	if (document.documentElement && document.documentElement.scrollTop)
		theTop = document.documentElement.scrollTop;
	else if (document.body)
		theTop = document.body.scrollTop

    //-- save the x and y mouse for later use
    this.x = (Browser.isMSIE) ?
    	window.event.clientX :
    	window.pageXOffset + event.clientX;

    this.y = (Browser.isMSIE) ?
    	theTop + window.event.clientY :
    	theTop + event.clientY;

	var body_width = document.body.offsetWidth;

	var translation_handle = this.elements[id];

   	//-- Let AJAX get the translations, and start the animation
	/*
	jsbroker.call('LangIfRPC.getText', translation_handle, function(translation)
	{
	    //-- before we animate we need to know the actual size so lets draw it now,
	    //   animate() will then resize and open it.
	    var div = document.getElementById('divHelp');
	    var x = help.x;
	    var y = help.y - div.offsetHeight;

	    div.style.left = '1px';
	    div.style.top  = '1px';

	    div.innerHTML = translation;

	    div.style.overflow = 'visible';
	    div.style.display = 'inline';

	    help.animate();
	});
	*/

    var div = document.getElementById('divHelp');
    var x = help.x;
    var y = help.y - div.offsetHeight;

    div.style.left = '1px';
    div.style.top  = '1px';

    div.innerHTML = translation_handle;

    div.style.overflow = 'visible';
    div.style.display = 'inline';

    help.animate();
}

//-- close function called when clicking on help dialog or <body>
Help.prototype.close = function()
{
    if (this.admin_mode)
        return false;

    var div = document.getElementById('divHelp');

    if (!div)
        alert('Help::close() - divHelp not found. make sure help is initialised.');

    div.style.display = 'none';
    div.innerHTML = '';
    div.style.width = null;
    div.style.height = null;

    if (document.all)
    {
        var ifr = document.getElementById('ifrHelp');
        ifr.style.display = 'none';
    }
}

/*
    animate() and the timeout method animate_tm() will fancy open the dialog
*/
Help.prototype.animate = function()
{
    var div = document.getElementById('divHelp');

	// max width
	if (div.offsetWidth > 500)
		div.style.width = 500 + 'px';

    this.width  = div.offsetWidth;
    this.height = div.offsetHeight - 10;

	//-- see if we fit to the left, else go right
	var animateRight = (this.x < document.body.offsetWidth - this.width - 50) ||
		(div.offsetWidth > document.body.offsetWidth);

	// see if we fit right else move x
	if (this.x - this.width < 0)
		this.x = this.width + 10;

    div.style.overflow = 'hidden';
    div.style.width = 0;
    div.style.height = 0;

    this.steps = 10;
    this.step  = 0;

    //-- start the timeout
    this.animate_tm(animateRight);
}

Help.prototype.animate_tm = function(animateRight)
{
	//alert(animateRight);

	if (this.step <= this.steps)
    {
        var div = document.getElementById('divHelp');

        var x = this.x;
        var y = this.y + 15;

        var width  = (this.step * (this.width / this.steps));
        var height = (this.step * (this.height / this.steps));

        if (!animateRight)
        	x -= width;

        if (y < 5) y = 5;

        div.style.left = x + 'px';
        div.style.top  = y + 'px';

        div.style.width  = width  + 'px';
        div.style.height = height + 'px';

        //-- the iframe
        if (document.all)
        {
            var ifr = document.getElementById('ifrHelp');
            ifr.style.display = 'inline';
            ifr.style.left    = x + 'px';
            ifr.style.top     = y + 'px';
            ifr.style.width   = width + 'px';
            ifr.style.height  = height + 12 + 'px';
        }

        // transparency?
        /*
        var opacity = this.step * (100 / this.steps);
        if (document.all)
            div.style.filter = 'alpha(opacity=' + opacity + ')';
        else
            div.style.MozOpacity = opacity + '%';
        */

        window.setTimeout('help.animate_tm( '+animateRight+' );', 10);
        this.step++;
    }
}

Help.prototype.activate = function()
{
	this.enabled = true;
}
