/**
 * LangAdmin
 * @author Martijn van den Broek
 * @version $Id: language_admin.js 39 2005-04-15 13:07:29Z Martijn van den Broek $
*/

function LangAdmin(path)
{
	this.tableHandles = new Array();
	this.path = path;
}

LangAdmin.prototype.labelClick = function(handle_id) { }

/**
 * updateHandles saves all open handles ot the database
 * @param form HtmlForm where to search for handles / checkboxes etc
 * @param callcack Function executed when finished
 */
LangAdmin.prototype.updateHandles = function(form, callback)
{
	var progressbox = MessageBox.prototype.Progress('Langif', 'Saving translations..');

	var calls = 0;

	var checks = form.getElementsByTagName('input');
	for (var i=0; i<checks.length; i++)
	{
		// we are only looking for checkboxes
		if (checks[i].name != 'chkHandleId')
			continue;

		if (!checks[i].checked)
			continue;

		var handle_id = checks[i].value;

		// build the group bit
		var groups = 0; this.groups.forEach(function(bitname, bit, a) {
			var group_check = document.getElementById('group_'+handle_id+'_'+bit);
			if (group_check.checked) {
				groups |= bit
			}
		}, this);

		// save the groups bit
		var params = []; params['handle_id'] = handle_id; params['groups'] = groups;
		calls++; jsbroker.call('LangIfRPC.setHandleGroups', params, function() { calls--; } );

		// save out of date..
		var el = document.getElementById('outofdate_'+handle_id);
		if (!el.disabled) {
			var params = []; params['handle_id'] = handle_id;
			params['outofdate'] = el.checked;
			calls++; jsbroker.call('LangIfRPC.markOutOfDate', params, function() { calls--; } );
		}

		// save the first translation field
		params = [];
		params['handle_id'] = handle_id
		params['language_id'] = this.firstLanguageId
		params['translation'] = document.getElementById('txt_tr1_'+handle_id).value
		calls++; jsbroker.call('LangIfRPC.setTranslation', params, function() { calls--; } );

		// and the second
		if (document.getElementById('txt_tr2_'+handle_id)) {
			params = [];
			params['handle_id'] = handle_id
			params['language_id'] = this.secondLanguageId
			params['translation'] = document.getElementById('txt_tr2_'+handle_id).value
			calls++; jsbroker.call('LangIfRPC.setTranslation', params, function() { calls--; } );
		}
	}

	total_calls = calls

	// show a popup progress bar that reloads the page when all calls are finished
	var interval = setInterval(function() {

		progressbox.setProgress( ((calls / total_calls  * 100) * -1) + 100 );

		if (calls <= 0) {
			progressbox.hide();
			callback.call();
			clearInterval(interval);
		}
	}, 1000);
}

// this switches (open or closes) a translation box, supply the checkbox of the row and hte handle_id
LangAdmin.prototype.switchTranslation = function(handleId, showTxt)
{
	var chkEl = document.getElementById('chkHandle_' + handleId);

	if (typeof(showTxt) == 'undefined')
		showTxt = !chkEl.checked;
	chkEl.checked  = showTxt;

	if (typeof this.firstLanguageId == 'undefined' || this.firstLanguageId == '' )
		throw 'this.firstLanguageId is not defined or empty. This is a bug!';

	if (typeof this.secondLanguageId == 'undefined' || this.secondLanguageId == '' )
		throw 'this.secondLanguageId is not defined or empty. This is a bug!';

	var nTr = document.getElementById('langIfTr_' + handleId);
	[1, 2].forEach( function(i)
	{
		if (nTr.aTrRow['tr'+i+'_translation'] == null)
			nTr.aTrRow['tr'+i+'_translation'] = '';

		// save translation if modified before we trash it
		if (!showTxt) {
			// textarea
			var nTxt = getElById('txt_tr'+i+'_' + handleId);
			nTr.aTrRow[ 'tr'+i+'_translation' ] = nTxt.value;
		}

		var nTd = document.getElementById('langIfTdTr'+i+'_'+handleId);
		removeChildNodes(nTd);

		// dont show textfield in simple mode
		if (i == 2 && langAdmin.simple_mode)
			showTxt = false;

		if (showTxt) {
			var nTxt = document.createElement('TEXTAREA');
			nTxt.name = 'translation'+i+'['+handleId+']['+ ((i == 1) ?
				langAdmin.firstLanguageId : langAdmin.secondLanguageId) +']';
			nTxt.id = 'txt_tr'+i+'_' + handleId;
			nTxt.style.width = '100%';

			// base the height on the non editable div (somehow this doesnt work)
			var divHeight = $('langIfTdTr'+i+'_'+handleId).offsetHeight;
			nTxt.style.height = ((divHeight > 70) ? divHeight : 70) + 'px';

			nTxt.value = nTr.aTrRow['tr'+i+'_translation'];
			nTd.appendChild(nTxt);
			addEvent(nTxt, 'click', function(e)
			{
				if (e.stopPropagation)
					e.stopPropagation();
				else
					e.cancelBubble = true;
			});
		}
		else {
			var nP = document.createElement('P');
			nP.innerHTML = nTr.aTrRow[ 'tr'+i+'_translation' ];
			nTd.appendChild(nP);
		}
	});

	// select and img
	try {
		document.getElementById('divHandleSettings_' + handleId).style.display = (showTxt) ? '' : 'none';
	} catch (e) { }


	// give a border to the gettext span
	if (el = document.getElementById('langif_' + handleId))
		el.style.border = (chkEl.checked) ? '1px solid #f00' : '';

}

// deletes a script from the collection
LangAdmin.prototype.deleteScript = function(handle_id, script_name)
{
	if (confirm('Are you sure you want to delete this script from this handle?'))
		document.location.href='/'+this.path+'scripts/i-w/bots/language_process.php?Action=DeleteScriptHandle&handle_id='+handle_id+'&script_name='+script_name;
}

// to change a level
LangAdmin.prototype.lvlOnChange = function(obj, translation_handle_id)
{
	if (typeof lang_script_name == 'undefined')
		throw new Error('lang_script_name unknown');

	if (typeof script_id == 'undefined')
		throw new Error('script_id unknown');

	if (typeof script_parent_id == 'undefined')
		throw new Error('script_name unknown');


	obj.dont_store_index = true;
	if (confirm('You are about to change the level of the selected handle.\nUnsaved translations will be lost, are you sure?'))
	{
		obj.old_index = obj.selectedIndex;
		document.location.href='/'+this.path+'scripts/i-w/bots/language_process.php?Action=ChangeLevel&handle_id='+translation_handle_id+'&script_name='+lang_script_name+'&NewLevel='+obj.value + '&script_id='+script_id+'&script_parent_id='+script_parent_id;
	}
	else
		obj.options[obj.old_index].selected = true;

	obj.dont_store_index=false;
}


// opens or closes a translation table
LangAdmin.prototype.switchTable = function(table_id)
{
	var table = document.getElementById('adminTable_' + table_id);
	var img  = document.getElementById('img_' + table_id);

	if (table)
	{
		table._collapsed = !table._collapsed;
		collapse = table._collapsed;

		table.tBodies[0].style.display = (collapse) ? 'none' : '';
		table.tFoot.style.display = (collapse)      ? 'none' : '';

		if (document.all) // IE fix
			table.style.borderWidth = (collapse) ? 0 : 1;
	}
	else
		alert('LangAdmin::switchTable() - Cant switch table ('+table_id+')');

	if (img)
		img.src = (collapse) ? '/'+this.path+'images/i-w/language_interface/plus.png' : '/'+this.path+'images/i-w/language_interface/minus.png';

}

LangAdmin.prototype.checkAll = function(table)
{
	var checks = table.getElementsByTagName('INPUT');
	for (var i=0; i<checks.length; i++)
	{
		var chk = checks[i];
		if (chk.id.substr(0,10) != 'chkHandle_')
			continue;

		this.switchTranslation(chk.value, true);
	}
}

LangAdmin.prototype.moveHandle = function(handle_id)
{
	document.location.href = 'editHandle.php?handle_id='+handle_id;
}

LangAdmin.prototype.showSetTranslations_onchange = function(el)
{
	document.location.href='/'+this.path+'scripts/i-w/bots/language_process.php?Action=ShowSetTranslations&value=' + ((el.checked) ? '0' : '1');
}

LangAdmin.prototype.showEmptyTranslations_onchange = function(el)
{
	document.location.href='/'+this.path+'scripts/i-w/bots/language_process.php?Action=ShowEmptyTranslations&value=' + ((el.checked) ? '1' : '0');
}

LangAdmin.prototype.fallBackToEnglish = function(el)
{
	document.location.href='/'+this.path+'scripts/i-w/bots/language_process.php?Action=FallbackToEnglish&value=' + ((el.checked) ? '1' : '0');
}

LangAdmin.prototype.switchFrame = function()
{
	var cookiename = 'langadmin_pageframe';
	var cookievalue = getCookie(cookiename);

	var show = (cookievalue != '1') ? '1': '0';
	setCookie(cookiename, show);

	document.getElementById('pageAdminFrame').tBodies[0].style.display = (show == '1') ? '' : 'none';
	document.getElementById('imgFrameSwitch').src = '/'+this.path+'images/i-w/language_interface/' + ( (show == '1') ? 'minus.png' : 'plus.png' );

	if (show == '1')
		this.showAdminHandleTable(true);
	else
	{
		var div = document.getElementById('divAdminHandles');
		removeChildNodes(div);
		div.innerHTML = 'Loading...';
	}
}

LangAdmin.prototype.editRich = function(handle_id)
{
	var nwin = new NewWindow ( 900, 500 );
	nwin.setSource( '/'+this.path+'scripts/i-w/bots/LanguageInterfaceEditRich.html' );
	var win = nwin.open();

	win.handle_id = handle_id;
}

LangAdmin.prototype.disable = function(el)
{
	document.location.href='/'+this.path+'scripts/i-w/bots/language_process.php?Action=Disable&value=' + ( el.checked ? '1' : '0');
}

/**
 * Shows the admin handle table for the current script
 */
LangAdmin.prototype.showPageAdminHandleTable = function()
{
	if (typeof this.pageHandles == 'undefined')
		alert('cannot make admin table: this.pageHandles not set');

	var params = new Array();
	params['pageHandles'] = this.pageHandles;

	msgProgress = MessageBox.prototype.Wait('Loading langif', 'Please wait while loading the Language Interface');
	//msgProgress.setProgressText('Receiving '+this.pageHandles.length+' translations...');

	jsbroker.call( 'LangIfRPC.getPageAdminHandles', params,
		function(params)
		{
			var handles = params;
			var div = document.getElementById('divAdminHandles');

			langAdmin.createAdminTable(div, handles, msgProgress)
		}
	  );
}

LangAdmin.prototype.createAdminTable = function (target, handles, msgProgress)
{
	var start_execution_time = new Date().getTime();
	var div = target;

	var firstLanguage = null;
	var secondLanguage = null;

	var table = new TableFactory( handles );
	table.setSortingMap( [].append( {'tr1_translation':'String', 'tr2_translation':'String'} ) );
	table.setCaption('LangIf Page Admin');
	table.setClassName('Professional Header');

	var arrHide = [ 'TRANSLATION_HANDLE_ID', 'tr1_language_id', 'tr2_language_id', 'groups', 'tr1_outofdate', 'tr2_outofdate'   ];

	if (this.simple_mode)
		arrHide.push('scriptcount');

	table.hideColumns( arrHide );

	// get groups if set, else leave empty
	var groups = (typeof this.groups != 'undefined') ? this.groups : new Array();

	table.addValueProcessor('TRANSLATION_HANDLE', function(nTd, aRow)
	{
		nTd.style.verticalAlign = 'top';

		// clear td
		removeChildNodes(nTd);

		var handleId   = aRow['TRANSLATION_HANDLE_ID'];
		var handleName = aRow['TRANSLATION_HANDLE'];
		var dbgroups   = aRow['groups'];
		var tr1_language_id = aRow['tr1_language_id'];

		// handle checkbox
		var nChk = document.createElement('INPUT');
		nChk.style.verticalAlign = 'middle';
		nChk.name = 'chkHandleId';
		nChk.type = 'checkbox';
		nChk.value = handleId;
		nChk.id = 'chkHandle_' + handleId;
		nTd.appendChild(nChk);

		addEvent(nChk, 'click', function()
		{
			langAdmin.switchTranslation(handleId, this.checked);
		});

		// linky
		if (!langAdmin.simple_mode)
		{
			var nA = document.createElement('A');
			nA.innerHTML = handleName;
			nA.href = '#';
			addEvent(nA, 'click', function(ev)
			{
				ev.returnValue = false;
				if (ev.preventDefault)
					ev.preventDefault();

				langAdmin.switchTranslation(handleId);
			});
			nTd.appendChild(nA);

			// br
			nTd.appendChild( document.createElement('BR') );

			// Settings Container
			var divSettings = document.createElement('DIV');
			divSettings.style.display = 'none';
			divSettings.id = 'divHandleSettings_'+handleId;
			divSettings.style.marginLeft = '20px';
			nTd.appendChild(divSettings);

			// Groups: Add a group container to settings
			var divGroups = document.createElement('DIV');
			divGroups.style.marginBottom = '8px';
			divGroups.id = 'divGroups_'+handleId;
			divSettings.appendChild(divGroups);

			var span = document.createElement('SPAN');
			span.innerHTML = '<b>Groups:</b> <br />'
			divGroups.appendChild(span);

			// var groups should be set from outside as an associative array
			// where the key is a bitwise value and value the description string
			var j = 0; groups.forEach(function(el, i, a)
			{
				var chk = document.createElement('INPUT');
				chk.type = 'checkbox';
				chk.name = chk.id = 'group_'+handleId+'_'+i;
				chk.checked = parseInt(i) & parseInt(dbgroups);
				divGroups.appendChild(chk);

				var lb = document.createElement('LABEL');
				lb.style.width = '80px';
				lb.htmlFor = chk.name;
				lb.innerHTML = el+'&nbsp; ';
				divGroups.appendChild(lb);

				// lets add a br after every third entry
				if ((j+1) % 2 == 0) {
					divGroups.appendChild(document.createElement('BR'));
				}

				j++;
			});

			// outofdate checkbox
			var divOutOfDate = document.createElement('DIV');
			divSettings.appendChild(divOutOfDate);

			var chk = document.createElement('INPUT');
			chk.type = 'checkbox';
			chk.name = chk.id = 'outofdate_'+handleId;
			chk.checked = (tr1_language_id == 44);
			divOutOfDate.appendChild(chk);
			chk.disabled = (tr1_language_id != 44);

			var lbl = document.createElement('LABEL');
			lbl.htmlFor = chk.name;
			lbl.innerHTML = 'Mark non-en outofdate';
			divOutOfDate.appendChild(lbl);


			// rich edit (disabled till this is working again)
			/*
			var a = document.createElement('A');
			a.id = 'aHandleRichEdit_' + handleId;
			a.style.display = 'none';
			a.href = '#';
			a.title = 'Rich Edit';
			addEvent(a, 'click', function(ev)
			{
				langAdmin.editRich(handleId);

				ev.returnValue = false;
				if (ev.preventDefault)
					ev.preventDefault();
			});

			var img = document.createElement('IMG');
			img.style.border = '0';
			img.style.marginLeft = '4px';
			img.style.verticalAlign = 'middle';
			img.src = '/' + SITE_PATH + 'images/i-w/insert_text.gif';
			a.appendChild(img);

			nTd.appendChild(a);
			*/
		}
	});

	table.addValueProcessor('tr1_translation', function(nTd, aRow)
	{
		var handleId   = aRow['TRANSLATION_HANDLE_ID'];

		nTd.id = 'langIfTdTr1_' + handleId;
		nTd.style.verticalAlign = 'top';

		if (parseInt(aRow['tr1_outofdate']))
			nTd.style.border = '1px dashed #ff0000';

		addEvent(nTd, 'click', hitch(langAdmin, 'switchTranslation', handleId));

		if (!Browser.isMSIE)
			addEvent(nTd, 'click', function() { document.getElementById('txt_tr1_' + handleId).focus(); } );

		if (firstLanguage == null)
			firstLanguage = aRow['tr1_language_id'];
	});

	table.addValueProcessor('tr2_translation', function(nTd, aRow)
	{
		var handleId   = aRow['TRANSLATION_HANDLE_ID'];
		nTd.id = 'langIfTdTr2_' + handleId;
		nTd.style.verticalAlign = 'top';

		if (parseInt(aRow['tr2_outofdate']))
			nTd.style.border = '1px dashed #ff0000';

		addEvent(nTd, 'click', hitch(langAdmin, 'switchTranslation', handleId));

		if (!Browser.isMSIE)
			addEvent(nTd, 'click', function() { document.getElementById('txt_tr2_' + handleId).focus(); } );

		if (secondLanguage == null && aRow['tr2_language_id'] > 0 )
			secondLanguage = aRow['tr2_language_id'];

	});

	table.addRowProcessor(function(nTr, aRow)
	{
		var handleId   = aRow['TRANSLATION_HANDLE_ID'];

		nTr.id = 'langIfTr_' + handleId;
		nTr.aTrRow = aRow; // save the full translation info in the row for later reference

		addEvent(nTr, 'mouseover', function() { this.style.cursor = 'default'; });
	});

	removeChildNodes(div);

	table.insertAt(div);

	setTimeout(hitch(msgProgress, 'hide'), 1000);
	//msgProgress.setProgressText('Done.');

	// try this, some cols might not be shown in simple mode
	try {
		table.renameColumn('TRANSLATION_HANDLE', 'Handle');
		table.renameColumn('tr1_translation', 'First Language ('+firstLanguage+')');
		table.renameColumn('tr2_translation', 'Second Language ('+secondLanguage+')');
		table.renameColumn('scriptcount', 'Scripts');
	} catch (e) { }

	time = new Date().getTime() - start_execution_time;
	dumpln('table time: ' + time);
}

