<?php
/**
 * @package I-Ware
 * @subpackage rpc
 */

/**
 * Gets the available handles for the current script
 */

class LangIfRPC extends IWAbstractRPC
{
	static function sanitize_params($method, $params)
	{
		if ($method == 'setTranslation')
			return $params;
		else
			return parent::sanitize_params($method, $params);
	}

	/**
	 * RPC method to get translation(s)
	 * @param mixed string or array with handles
	 * @return mixed string translation or array with key as handle, translation as value
	 */
	static function getText($params)
	{
		$langIf = IWRegistry::registry('langIf');

		if(!$langIf)
			return false;

		if (is_array($params))
		{
			$rv = array();
			foreach ($params as $handle)
				$rv[$handle] = $langIf->getText($handle);

			return $rv;
		}
		elseif ( is_string($params) )
		{
			return $langIf->getText($params, false);
		}

		return '';
	}

	/**
	 * Admin functions
	 */
	static function getPageAdminHandles($params)
	{
		$langIf = IWRegistry::registry('langIf');
		return $langIf->getPageAdminHandles($params['pageHandles']);
	}

	static function getAdminHandles($params)
	{
		$langIf = IWRegistry::registry('langIf');

		$arr = $langIf->getAdminHandles(
			(bool) $params['emptyOnly'],
			(int) $params['limitRows'],
			(int) $params['firstLanguage'],
			(int) $params['secondLanguage'],
			(array) $params['filters']
		);

		return $arr;
	}

	static function setHandleGroups($params)
	{
		$langIf = IWRegistry::registry('langIf');
		$langIf->setHandleGroups($params['handle_id'], $params['groups']);
	}

	static function markOutOfDate($params)
	{
		$langIf = IWRegistry::registry('langIf');
		$langIf->markOutOfDate($params['handle_id'], $params['outofdate']);
	}

	static function setTranslation($params)
	{
		$langIf = IWRegistry::registry('langIf');
		$langIf->setTranslation($params['handle_id'], $params['language_id'], $params['translation']);
	}
}

?>
