/**
 * @copyright Copyright 2004, BaseSoft b.v.
 * @version $Id$
 * @package I-Ware
 * @subpackage JavaScript
 */

//-- Button types
var BT_PREV   = 0;
var BT_NEXT   = 1;
var BT_CANCEL = 2;
var BT_FINISH = 3;
var BT_CLOSE  = 4;

//-- Button arguments
var BA_DISABLED = 0;
var BA_DISPLAY  = 1;
var BA_ONCLICK  = 2;

/**
 * Constructor of the Wizard Class
 * <code>
 * 	var wiz = new Wizard();
 * 	document.write( wiz.getHtml() );
 * 	wiz.init();
 *
 * </code>
 */
function Wizard()
{
	//-- private variables
	this._buttons = new Array();
	this._wizPage = new wizardpage();
}

/**
 * getHtml() returns the body of the wizard, should be printed somewhere in the document
 * @return string HTML
 */
Wizard.prototype.getHtml = function()
{
	return ' \
		<table class="wizard"> \
			<tr> \
				<td class="title" colspan="2"><span id="sWizTitle"></span> - <span id="sWizSubTitle"></span></td> \
			</tr> \
			<tr> \
				<td class="roadMap"> \
					<ul id="ulRoadMapList"></ul> \
				</td> \
				<td class="tdWizContent"> \
					<div id="wizContent"></div> \
				</td> \
			</tr> \
			<tr> \
				<td class="buttons" colspan="2"> \
					<input type="button" id="btnCancel" class="button" style="width: 80px; display: none;" /> \
					&nbsp; \
					<input type="button" id="btnPrevious" class="button" style="width: 80px; display: none;"  /> \
					<input type="submit" id="btnNext" class="button" style="width: 80px; display: none;" /> \
					&nbsp; \
					<input type="button" id="btnFinish" class="button" style="width: 80px; display: none;" /> \
					<input type="button" id="btnClose" class="button" style="width: 80px; display: none;"/> \
				</td> \
			</tr> \
		</table> \
	';
}

/**
 * Initialises the wizard by registering and translating the buttons, fields and other elements.
 * Should be called after the wizard is printed using getHtml
 */
Wizard.prototype.init = function()
{
	//-- register elements as private
	this._content = document.getElementById('wizContent');
	this._title = document.getElementById('sWizTitle');
	this._subtitle = document.getElementById('sWizSubTitle');
	this._roadMapList = document.getElementById('ulRoadMapList');

	//-- Init buttons. keep the index the same as defines or rewrite code below
	var buttons = new Array( 'btnPrevious', 'btnNext', 'btnCancel', 'btnFinish', 'btnClose');
	buttons.forEach( function( element, index, array )
	{
		this._buttons[index] = document.getElementById(element);
		this._buttons[index].button_id = index;

		var b = this._buttons[index];
		b.style.display = 'none';
		b.disabled = true;
		addEvent(b, 'click', hitch(this, '_buttonClick') );

	}, this);

	//-- Make some translations
	var handles = new Array('Wizard Previous', 'Wizard Next', 'Wizard Cancel',
		'Wizard Finish', 'Wizard Close');

	jsbroker.call( 'LangIfRPC.getText', handles, function(tr)
	{
		this._buttons[BT_PREV].value   = tr['Wizard Previous'];
		this._buttons[BT_NEXT].value   = tr['Wizard Next'];
		this._buttons[BT_CANCEL].value = tr['Wizard Cancel'];
		this._buttons[BT_FINISH].value = tr['Wizard Finish'];
		this._buttons[BT_CLOSE].value  = tr['Wizard Close'];
	}, this );
}

/**
 * This loads a new Wizard, can be run multiple times after loading init()
 * @param int WizardID
 */
Wizard.prototype.load = function(WizardID, post)
{
	if (typeof post == 'undefined')
		post = new Array();

	//-- Load the first page, no buttons supplied so it will return the Init page
	var params = new Array();
	params['WizardID'] = WizardID;
	params['POST'] = post;

	jsbroker.call( 'Wizard_LoadPage', params, hitch(this, '_updatePage')  );
}

/**
 * @access private
 */
Wizard.prototype._buttonClick = function(e)
{
	var btn = (typeof e != 'undefined' && e.target) ? e.target : window.event.srcElement;

	//-- check for button callbacks, if they return false dont process any further
	if ( this._wizPage.buttons[btn.button_id][BA_ONCLICK] )
	{
		eval('retval = function() {' + this._wizPage.buttons[ btn.button_id ][BA_ONCLICK] + '}');

		if (retval() === false)
		{
			btn.disabled = false;
			return false;
		}
	}

	this._loadPage(btn.button_id);
}

/**
 * @access private
 */
Wizard.prototype._loadPage = function(btn_id)
{
	this.disableAllButtons(true);

	//-- save the button pressed
	var params = new Array();
	params['BUTTON'] = btn_id;
	params['POST'] = this._extractForm( this._content.firstChild );
	params['WizardID'] = this._wizPage.wizard_id;
	params['PageHandle'] = this._wizPage.pageHandle;

	//-- send and receive data
	jsbroker.call( 'Wizard_LoadPage', params, hitch(this, '_updatePage') );
}

/**
 * This method recursively extracts all form values from a node collection.
 * @access private
 * @param node
 * @param array optional
 * @return array
 */
Wizard.prototype._extractForm = function( node, arr )
{
	if (typeof arr == 'undefined')
		arr = new Array();

    do
    {
        if ( node.nodeType != 1 )
            continue;

		switch (node.tagName)
		{
			case 'INPUT':
				if ( node.name == '' )
					break;

				//-- ignore all radios and checkboxes that are not checked
				if ((node.type == 'radio' || node.type == 'checkbox') && !node.checked)
					break;

				re = /^(.+)\[(.*)\]$/;
				if ( found = node.name.match(re) )
				{
					//-- its an array, lets save it :D
					var x = found[2];
					if (typeof arr[found[1]] == 'undefined')
						arr[found[1]] = new Array();

					if ( x != '')
						arr[found[1]][x] = node.value;
					else
						arr[found[1]].push(node.value);

				}
				else //-- can add it normally
					arr[node.name] = node.value;
			break;

			case 'SELECT':

				if ( node.name != '' )
				{
					var name = node.name;
					var subkey = false;
					//-- check if the name is array, then make it
					//   so we can push the option values
					re = /^(.+)\[(.*)\]$/;
					if (found = node.name.match(re))
					{
						name = found[1];
						if(!arr[name])
							arr[name] = new Array();
						if(found[2])
						{
							subkey = found[2]
							arr[name][subkey] = '';
						}
					}


					//-- loop trough the options and add/set if selected
					for (var i=0; i<node.options.length; i++)
					{
						var opt = node.options[i];
						if (opt.selected)
						{
							var value = (opt.value == '') ? node.name : opt.value;
							if ( arr[name] instanceof Array)
							{
								if(subkey)
									arr[name][subkey] = opt.value;
								else
									arr[name].push(opt.value);
							}
							else
								arr[name] = opt.value;
						}
					}
				}
			break;

            default:
                if ( node.hasChildNodes() )
                    this._extractForm(node.firstChild, arr);

		} //end switch tagName
    }
    while ( node = node.nextSibling  );

    return arr;
}

/**
 * @access private
 */
Wizard.prototype._rebuildRoadMap = function(pageHandle)
{
	var roadMap = this._roadMap;

	//-- trash it and build
	removeChildNodes(this._roadMapList);

	roadMap.forEach( function(element, index, array)
	{
		var li = document.createElement('LI');
		li._index = index;
		li.innerHTML = element;
		this._roadMapList.appendChild(li);
	}, this);
}

/**
 * @access private
 */
Wizard.prototype._updateRoadMap = function( pageHandle )
{
	if (this._roadMap.array_key_match(pageHandle))
	{
		var mark_past=true;
		for (var i=0; i<this._roadMapList.childNodes.length; i++)
		{
			var childNode = this._roadMapList.childNodes[i];
			if (pageHandle.match(new RegExp(childNode._index)))
			{
				childNode.className = 'Now';
				this._subtitle.innerHTML = this._roadMap[pageHandle];

				mark_past = false;
			}
			else
				childNode.className = mark_past ? 'Past' : 'Future';
		}
	}
}

/**
 * Dissables or enables all buttons
 * @param bool disabled
 */
Wizard.prototype.disableAllButtons = function(disable)
{
	if (typeof disable == 'undefined')
		disable = true;

	this._buttons.forEach( function(element, index, array)
	{
		element.disabled = disable;
	});
}

/**
 * This method recursively extracts all form values from a node collection.
 * @access private
 * @param node
 * @param array optional
 * @return array
 */
Wizard.prototype._updatePage = function(wizPage)
{
	//-- save the current page
	this._wizPage = wizPage;

	//-- set title
	if (typeof this._wizPage.wizard_title != 'undefined' && this._wizPage.wizard_title != '')
		this._title.innerHTML = this._wizPage.wizard_title;

	//-- update road map if set
	if (typeof this._wizPage.roadmap != 'undefined')
	{
		this._roadMap = this._wizPage.roadmap;
		this._rebuildRoadMap(this._wizPage.pageHandle);
	}
	//-- update roadmap in a seperate thread
	setTimeout( hitch(this, '_updateRoadMap', this._wizPage.pageHandle), 1);

	//-- show / hide buttons
	var btn_args = this._wizPage.buttons;
	this._buttons.forEach( function( el, i, ar)
	{
		if ( btn_args[el.button_id] )
		{
			//-- display or not, here i come.. u cant hide.. lalalalaa
			el.style.display = (btn_args[el.button_id][BA_DISPLAY]) ? '' : 'none';

			//-- disabled or not
			el.disabled = (btn_args[el.button_id][BA_DISABLED]) ? true : false;
		}
	});

	removeChildNodes(this._content);

	var html = '<div>';
	if (typeof this._wizPage.title != 'undefined')
		html += '<h1>'+this._wizPage.title+'</h1>';
	html += this._wizPage.content;
	html += '</div>';

	this._content.innerHTML = html;

	//-- Workaround for MSIE, that wont show stuff after the <h1>
	//   FIX (M)IE
	if (Browser.isMSIE)
	{
		AddEvent(this._content, 'focus', function() { this.innerHTML = this.innerHTML; });
		this._content.focus();
	}
}

/**
 * Class WizardPage, filled from unserialize in JSRPC.. dont touch this! tadadadaaa
 */
function wizardpage() { }
