/* Import plugin specific language pack */
//tinyMCE.importPluginLanguagePack('field_references', 'en'); // <- Add a comma separated list of all supported languages

/****
 * Steps for creating a plugin from this field_references:
 *
 * 1. Change all "template" to the name of your plugin.
 * 2. Remove all the callbacks in this file that you don't need.
 * 3. Remove the popup.htm file if you don't need any popups.
 * 4. Add your custom logic to the callbacks you needed.
 * 5. Write documentation in a readme.txt file on how to use the plugin.
 * 6. Upload it under the "Plugins" section at sourceforge.
 *
 ****/

/**
 * Gets executed when a editor instance is initialized
 */
function TinyMCE_field_references_initInstance(inst)
{
	//-- nuffin here
}

/**
 * Gets executed when a editor needs to generate a button.
 *
		field_references: new Array
		(
			{ title: 'Recipient', fields: [].append({
					'First Name': 'RECIPIENT_FIRST_NAME',
					'Last Name': 'RECIPIENT_LAST_NAME'
			})},

	    	{ title: 'Company', fields: [].append({
	    		'Name': 'COMPANY_NAME',
	    		'VAT': 'COMPANY_VAT'
	    	})}
		)
 *
 */
function TinyMCE_field_references_getControlHTML(control_name)
{
	var refs = tinyMCE.getParam("field_references", new Array() );
	refs = [].append(refs);

	switch (control_name)
	{
		case "field_references":

			var html = '<select class="mceSelectList" id="{$editor_id}_field_references" \
			                    onchange="tinyMCE.execInstanceCommand(\'{$editor_id}\',\'mce_field_references_insert\', false, this);" \>';
			html += '<option>-- Insert Field --</option>';

			//-- optgroups
			refs.forEach( function(fields, opt_title, array)
			{
				html += '<optgroup label="'+ opt_title +'">';

				//-- the fields
				var fields = [].append(fields);
				fields.forEach( function(fieldCaption, fieldHandle, array)
				{
					html += '<option value="'+fieldHandle+'">';
					html += fieldCaption;
					html += '</option>';
				});

				html += '</optgroup>';
			});

			html += '</select>';
			return html;
	}

	return "";
}

/**
 * Gets executed when a command is called.
 */
function TinyMCE_field_references_execCommand(editor_id, element, command, user_interface, value)
{

	var sel = value;

	// Handle commands
	switch (command)
	{
		// Remember to have the "mce" prefix for commands so they don't intersect with built in ones in the browser.

		case 'mce_field_references_insert':

			var _value = sel.options[sel.selectedIndex].innerHTML;
			var _name  = sel.options[sel.selectedIndex].value;

			var html = TinyMCE_field_references_createFieldReference( _name, _value );
			var inst = tinyMCE.getInstanceById(editor_id);
			inst.execCommand('mceInsertContent', false, html);

			sel.selectedIndex = 0;

			return true;

		break;

		case "mcefield_references":
			return true;
	}

	// Pass to next handler in chain
	return false;
}


/**
 * Gets executed when contents is inserted / retrived.
 */
function TinyMCE_field_references_cleanup(type, content)
{
	switch (type) {
		case "get_from_editor":

			var reg = /<input.*?name="(\w+)"[^>]*>/
			while (content.match(reg))
				content = content.replace( reg, '<%$1%>');

			break;

		case "insert_to_editor":
			//alert("[TO] Value HTML string: " + content);

			var refs = tinyMCE.getParam("field_references", new Array() );
			refs = [].append(refs);

			refs.forEach( function(fields, opt_title, array)
			{
				fields = [].append(fields);
				fields.forEach( function(fieldCaption, fieldHandle, array)
				{
					var reg = '<%' + fieldHandle + '%>';
					while (content.match(reg))
						content = content.replace( reg,
							TinyMCE_field_references_createFieldReference(fieldHandle, fieldCaption) );

				});
			});

			// alert(content);
			break;

		case "get_from_editor_dom":
			//alert("[FROM] Value DOM Element " + content.innerHTML);

			// Do custom cleanup code here

			break;

		case "insert_to_editor_dom":
			//alert("[TO] Value DOM Element: " + content.innerHTML);

			// Do custom cleanup code here

			break;
	}

	return content;
}

/* returns a nice field input */
function TinyMCE_field_references_createFieldReference( _name, _value )
{
	return ' <input type="text" class="text" name="'+_name+'" value="'+_value+'" style="border: 0; text-style: italic; background-color: #eee; border: 1px solid #ccc;" disabled="disabled" /> ';
}
