<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'Country.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @since 09-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceAuthorisationOverviewCommon extends InterfaceFactory
{
	var $authorize_type = array();
	var $overview_method = 'selectAuthorizeOverview';

	var $_process;

	var $allow_authorize = false;

    /**
     * interface[id] = 125
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceAuthorisationOverviewCommon(&$db, &$db_app, &$langIf)
    {
        // This is an Abstract class. Before this point the decendant classes
        // should already have initialised a $this->model['object'].
        if (get_class($this) == strtolower('InterfaceAuthorisationOverviewCommon'))
        	trigger_error('Cannot init: this is an Abstract class', E_USER_ERROR);

        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        validatePOSTLength();

		// fill by request
		if (isset($_REQUEST['crso']['id']) && !empty($_REQUEST['crso']['id']))
			$this->model['crso'] = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, (int)$_REQUEST['crso']['id'])));

		if (isset($_REQUEST['participant']['id']) && !empty($_REQUEST['participant']['id']))
			$this->model['participant'] = new Participant($this->db_app, array('relation'=>new Relation($this->db_app, (int)$_REQUEST['participant']['id'])));

		// date filter
    	$date =& currentDate();

		if (isset($_REQUEST['month']) && !empty($_REQUEST['month']))
        	$this->model['month'] = (int)$_REQUEST['month'];
        else
        	$this->model['month'] = $date->getMonth();

        $this->model['year'] = $date->getYear();

        $this->model['participants'] = array();

		/**
		 * This Interface is used in different universes, check the group_company and fill the select
		 */
		require_once(IWARE . 'group.php');
		switch ($_SESSION['session_group_group_id'])
		{
			case GROUP_ACCOUNTANT:
			case GROUP_SYSADMIN:
				$this->model['crsos'] = array();
				// fill available crsos
				if($_SESSION['session_group_group_id'] == GROUP_ACCOUNTANT){
					require_once(ELC_CLASSES.'AccountantCRSO.php');
					$accountant_crso = new AccountantCRSO($this->db_app);
					$q = $accountant_crso->getAccountantCRSOs($_SESSION['accountant_relation_id']);
				}
				else {
					require_once(ELC_CLASSES.'CRSO.php');
					$crso = new CRSO($this->db);
					$q = $crso->selectCRSOs(1,null,$_SESSION['selected_product_id']);
				}
				while ($r = $q->fetch())
					$this->model['crsos'][] = array($r->CRSO_RELATION_ID, $r->NAME);

				if(count($this->model['crsos']) == 1){
					list ($id, $name) = $this->model['crsos'][0];
					$this->model['crso'] = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, $id)));
				}

				$q = $this->model['object']->selectParticipants(isset($this->model['crso']) ? $this->model['crso']->relation->id : false,$_SESSION['selected_product_id']);
				while ($r = $q->fetch())
					$this->model['participants'][$r->PARTICIPANT_RELATION_ID] = array($r->PARTICIPANT_RELATION_ID, $r->NAME);

				$this->authorize_type[] = REFUND_AUTHORIZE_OPERATOR;

				break;


			case GROUP_CRSO:

				$q = $this->model['object']->selectParticipants($_SESSION['crso_relation_id'],$_SESSION['selected_product_id']);
				while ($r = $q->fetch())
					$this->model['participants'][$r->PARTICIPANT_RELATION_ID] = array($r->PARTICIPANT_RELATION_ID, $r->NAME);

				$this->authorize_type[] = REFUND_AUTHORIZE_STATUTORY_AUDITOR;
				$this->authorize_type[] = REFUND_AUTHORIZE_OPERATOR;
				$this->authorize_type[] = REFUND_AUTHORIZE_GENERAL_MANAGER;

				$this->hiddenFields = array(array('name'=>'crso[id]','value'=> $_SESSION['crso_relation_id']));
				$this->model['crso'] = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app, $_SESSION['crso_relation_id'])));

				break;

			case GROUP_PARTICIPANT:

				/**
				 * Fill the participant select with all participant exporters
				 */
				$p = new Participant($this->db_app);
				// Only select the participants that actually have stuff.
				$q = $this->getParticipants();
				//$q = $p->selectParticipantsByUser($_SESSION['session_user_id'], PARTICIPANT_EXPORTER);
				while ($r = $q->fetch())
					$this->model['participants'][$r->PARTICIPANT_ID] = array($r->PARTICIPANT_ID, $r->PARTICIPANT_NAME);

				/**
				 * and the same with crso
				 */
				$p = new CRSO($this->db_app);
				$q = $p->selectCRSOsByParticipantUser($_SESSION['session_user_id'], PARTICIPANT_EXPORTER);
				while ($r = $q->fetch())
					$this->model['crsos'][] = array($r->CRSO_ID, $r->CRSO_NAME);


				if(count($this->model['crsos'])==1)
				{
					list($crso_id,) = current($this->model['crsos']);
					$this->model['crso'] = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app, (int)$crso_id)));
				}
				if(count($this->model['participants'])==1)
				{
					list($participant_id,) = current($this->model['participants']);
					$this->model['participant'] = new Participant($this->db_app,array('relation'=>new Relation($this->db_app, (int)$participant_id)));
				}

				if (!isset($this->model['participant']) || !isset($this->model['crso']))
					break; // cannot continue because the user has to select a crso and participant first

				// RelationUser
				require_once(ELC_CLASSES.'RelationUser.php');
				require_once(ELC_CLASSES.'Product.php');
				$ru = new RelationUser($this->db_app);
				$ru->set($_SESSION['session_user_id'], $this->model['participant']->relation->id);

				// CRSOParticipant
				$cp = new CRSOParticipantProduct($this->db_app, array('participant'=>$this->model['participant'], 'crso'=>$this->model['crso'],'product'=>new Product($this->db_app,$_SESSION['selected_product_id'])), true);

				// check if Rui is admin (hihi) so we can authorize by statitory auditor
				if ($ru->isAdmin()) {
					foreach ($this->authorize_property as $prop)
					{
						if ($cp->{$prop} == REFUND_AUTHORIZE_STATUTORY_AUDITOR) // participant setting is auth-by-auditor
							$this->authorize_type[] = REFUND_AUTHORIZE_STATUTORY_AUDITOR;
					}
				}

				// also check if we can authorize by general manager
				if (isset($cp->general_manager->id) && $cp->general_manager->id == $_SESSION['session_user_id'] // this user is gm
					&& $cp->general_manager_authorized == GENERAL_MANAGER_AUTHORIZED) // gm is authorized
				{
					foreach ($this->authorize_property as $prop)
						if ($cp->{$prop} == REFUND_AUTHORIZE_GENERAL_MANAGER) // participant setting is auth-by-auditor
							$this->authorize_type[] = REFUND_AUTHORIZE_GENERAL_MANAGER;
				}

			break;
			default:
				alert(sprintf('Access denied by user [%s] to InterfacePaymentOverview', $_SESSION['session_user_id']));
				throw new InterfacePermissionException(sprintf('Access denied by user [%s] to InterfacePaymentOverview', $_SESSION['session_user_id']));
		}

		if(isset($this->model['crso'])) {
			$this->model['country'] = new Country($this->db_app, $this->model['crso']->relation->getCountry(),true);
		}

		if (isset($_REQUEST['process']))
			$this->_process = $_REQUEST['process'];

		// TODO: check rights!
    	if (count($this->authorize_type))
    		$this->allow_authorize = true;

    }

    function getParticipants()
    {
    	switch ($_SESSION['session_group_group_id'])
    	{
    		case GROUP_ACCOUNTANT:
    			return $this->model['object']->selectParticipants(isset($this->model['crso']) ? $this->model['crso']->relation->id : false,$_SESSION['selected_product_id']);
    		break;
    		case GROUP_SYSADMIN: // ?
			case GROUP_PARTICIPANT:
				$p = new Participant($this->db_app);
				return $p->selectParticipantsByUser($_SESSION['session_user_id'], PARTICIPANT_EXPORTER);
			break;
    	}
    }

	function _filter($q)
	{
		$rv = parent::_filter($q);

    	// hide button block if we are not allowed to authorize
    	if (!$this->allow_authorize)
			unset($rv[$this->id][306]);
		else
    	    // unset document signature item [945] in block [306] which is only used InterfaceConfirmationOverviewExport
    		unset($rv[$this->id][306][945]);

		//kill month select see #481
		unset($rv[$this->id][304][1107]);

    	return $rv;
    }

    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
    	if (isset($_REQUEST['process']))
    	{
    		$this->db->begin();
	        $this->_processCheckboxes();
    		$this->db->commit();

	        preg_match('/\/scripts\/(.*)\/(.*)\/(Interface[\w]+)\.php/', $_SERVER['REQUEST_URI'],$matches);

	        header(sprintf('location: %s?participant[id]=%s&crso[id]=%s',
	        	$matches[0], (int)$_REQUEST['participant']['id'], (int)$_REQUEST['crso']['id']));

	        die();
    	}
    }

    /**
     * Saves the object from $_REQUEST data
     * @access protected
     */
    function _processCheckboxes()
    {
    	if (!isset($_REQUEST['checkboxes']) || empty($_REQUEST['checkboxes']))
    		return; // no checkboxes - do nothing

        $body = array();
    	foreach ($_REQUEST['checkboxes'] as $id)
        {
        	$obj = new $this->objectClass($this->db_app, (int)$id);

        	if ($_REQUEST['process'] == 'authorize') {
        		$obj->status = $this->status_approved;
        		$obj->authorized_by = new ElcUser($this->db_app, $_SESSION['session_user_id']);
        		$obj->authorized_by_type = $this->authorize_type[0]; // save how we authorized (refund request only)
        	}

        	else if ($_REQUEST['process'] == 'deny')
        		$obj->status = $this->status_denied;
        	else
        		trigger_error(sprintf('Unknown process [%s]', $_REQUEST['process']));

        	info(sprintf('%s [%s] to status [%s] by user [%s]', $this->objectClass, $obj->id,
        		$obj->status, $_SESSION['session_user_id']));

        	$obj->update();

        	if($this->objectClass == 'ProofOfPayment' && $_REQUEST['process'] == 'deny')
        		$body[] = "<b>".$this->langIf->getText('PoP ID:')."</b> ".$id .
			          " &nbsp;&nbsp;(<a href='../participant/InterfacePaymentForm.php?pop[id]=$id '> "
			          . $this->langIf->getText('view details') ." </a> ) <br />";

        	//if we are an operator AND this is a POP we trigger a document review
        	if($_SESSION['session_group_group_id'] == GROUP_ACCOUNTANT && $this->objectClass == 'ProofOfPayment')
        		$this->_triggerDocumentReview($obj);
		}
    	//send notification in case POP is rejected
		if($_SESSION['session_group_group_id'] == GROUP_ACCOUNTANT && $this->objectClass == 'ProofOfPayment' && $_REQUEST['process'] == 'deny')
        {
        	require_once(ELC_CLASSES.'Notification.php');
        	$final_body = '';
        	foreach($body as $b)
        		$final_body .= $b;
       	 	if(isset($_REQUEST['reason']) && $_REQUEST['reason'] != '')
			{
				$final_body .= "<br /><b>".$this->langIf->getText('Reason:')."</b><br />";
				if($_REQUEST['reason'] == "Other")
					$final_body .= str_replace("\n","<br />",$_REQUEST['other']);
				else
					$final_body .= $this->langIf->getText($_REQUEST['reason']);
			}
        	$message = array('subject'=> $this->langIf->getText('PoP Rejected'),
					'user'=> new ELCUser($this->db_app, $_SESSION['session_user_id']),
					'message'=> $final_body,
					'product'=> new Product($this->db_app, $_SESSION['selected_product_id'])
				);
        	Notification::send(
				$this->db_app,
				$message,
				array(array('recipient'=> new Relation($this->db_app, $this->model['participant']->relation->id)))
			);
        }
	}

	function _triggerDocumentReview(&$obj)
	{
		try
		{
			if(!$obj->isComplete())
				$obj->set();
			if(!isset($obj->document) || !$obj->document || !isset($obj->document->id))
				return;
			require_once(ELC_CLASSES.'CRSOAdminFee.php');
			$caf = new CRSOAdminFee($this->db_app);
			$fee = $caf->getCurrentFee($obj->crso->relation->id, CRSO_ADMIN_FEE_POP_REVIEW);

			require_once(ELC_CLASSES.'RefundExportDocumentReview.php');
			$redr = new RefundExportDocumentReview($this->db_app);
			$redr->document =&  $obj->document;
			$redr->review_cost =  $fee ? $fee : 0;
			$redr->crso =& $obj->crso;
			$redr->participant =& $obj->participant;
			$redr->product = &$obj->product;
			if(!$redr->insert())
				trigger_error(sprintf('Error inserting document review: CRSO [%s] Document [%s]', $obj->crso->relation->id, $obj->document->id));
		}
		catch(Exception $e)
		{
			//we already reviewed this doc
		}
	}
}

/**
 * @package ELC
 * @subpackage Interface
 * @since 09-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceAuthorisationOverviewCommonBlock extends InterfaceBlock
{
}

/**
 * @package ELC
 * @subpackage Interface
 * @since 09-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceAuthorisationOverviewCommonItem extends InterfaceItem
{
	var $_hideColumns = array('REFUND_REQUEST_CONFIRMATION_ID');
	var $script = '';
	var $_shownColumns = array();
	var $_totalFields = array();
	var $_headType = array();

    /**
     * item[932] =
     */
    function getParticipantSelect()
    {
    	$crso_id = isset($this->if_obj->model['crso']) ? $this->if_obj->model['crso']->relation->id : '';
    	if($_SESSION['session_group_group_id']==GROUP_CRSO) {
    		$crso_id = $_SESSION['crso_relation_id'];
    	}

    	$participant_id = isset($this->if_obj->model['participant']) ? $this->if_obj->model['participant']->relation->id : '';

		if(!$crso_id)
		{
			$this->options[] = array('text'=>'- '.$this->langIf->getText('Select CRSO First').' -', 'value'=>'');
			return;
		}
    	$this->options[] = array('text'=>'- '.$this->langIf->getText('Select Participant').' -', 'value'=>'');
    	if(isset($this->if_obj->model['participants']))
    	{
    		foreach ($this->if_obj->model['participants'] as $participant) {
    			list ($id, $name) = $participant;
    			$this->options[] = array('text'=>((mb_strlen($name)>35)?mb_substr($name,0,35).'...':$name), 'value'=>$id, 'selected'=>( $id == $participant_id ));
    		}
    	}

    	preg_match('/\/scripts\/(.*)\/(.*)\/(Interface[\w]+)\.php/', $_SERVER['REQUEST_URI'],$matches);

    	$this->properties['onchange'] = sprintf("go('%s?crso[id]=%s&participant[id]='+this.value);",
    		$matches[0], $crso_id);

    	//$this->properties['onchange'] = "_LoadOverview();"; // GRR AJAX DOESNT WORK WITH DESCENDENT INTERFACES

		$this->postscript = "
        function _LoadOverview()
        {
			var params = new Array();

			params['crso'] = new Array();
			params['crso']['id'] = getElByName('crso[id]').value;

			params['participant'] = new Array();
			params['participant']['id'] = getElByName('participant[id]').value;;

			params['interface_id'] = ".$this->if_obj->id.";

			ifHandler.loadBlock( new Array(new String('305')), params );
		}";
    }

    /**
     * item[934] =
     */
    function getCRSOSelect()
    {
	$rv = null;
		// Check for any active Remote Viewer connections.
		if(IWRegistry::isRegistered('remote_viewer')) {
			$rv = IWRegistry::registry('remote_viewer');
		}
    	$crso_id = isset($this->if_obj->model['crso']) ? $this->if_obj->model['crso']->relation->id : '';
    	$participant_id = isset($this->if_obj->model['participant']) ? $this->if_obj->model['participant']->relation->id : '';

    	$this->options[] = array('text'=>'- '.$this->langIf->getText('Select CRSO').' -', 'value'=>'');
    	foreach ($this->if_obj->model['crsos'] as $crso) {
    		list ($id, $name) = $crso;
		if($rv && $rv->crso->relation->id!=$id)
			continue;
    		$this->options[] = array('text'=>((mb_strlen($name)>35)?mb_substr($name,0,35).'...':$name), 'value'=>$id, 'selected'=>($id == $crso_id));
    	}
    	preg_match('/\/scripts\/(.*)\/(.*)\/(Interface[\w]+)\.php/', $_SERVER['REQUEST_URI'],$matches);

    	$this->properties['onchange'] = sprintf("go('%s?participant[id]=%s&crso[id]='+this.value);",
			$matches[0], '');

    }

    //Not used
	//Being filtered out.... see #481
    function getSelectMonth()
    {
    	$crso_id = isset($this->if_obj->model['crso']) ? $this->if_obj->model['crso']->relation->id : '';
    	$participant_id = isset($this->if_obj->model['participant']) ? $this->if_obj->model['participant']->relation->id : '';
    	$month = $this->if_obj->model['month'];

    	$this->options = array();

    	for($i=1;$i<=12;$i++)
    	{
    		$this->options[$i-1] = array('text'=>strftime('%B', mktime(1, 1, 1, $i, 1)), 'value'=>$i);
    		if($i == $month)
    			$this->options[$i-1]['selected'] = 'selected';
    	}
    	preg_match('/\/scripts\/(.*)\/(.*)\/(Interface[\w]+)\.php/', $_SERVER['REQUEST_URI'],$matches);
    	$this->properties['onchange'] = sprintf("go('%s?participant[id]=%s&crso[id]=%s&month='+this.value);",
    		$matches[0], $participant_id, $crso_id);

    }

    //Not used
    function getOtherInput()
    {
    	$this->properties['style'] = "visibility:hidden;";
    }

    /**
     * item[933] =
     */
    function getOverview()
    {
    	$tr[0] = $this->langIf->getText('Please select a CRSO and Participant.');
    	$tr[1] = $this->langIf->getText('Not authorized to authorize');

    	if (!isset($this->if_obj->model['crso']) || !isset($this->if_obj->model['participant']) ) {
    		$this->text = $tr[0];
    		return ;
    	}

    	if (!$this->if_obj->allow_authorize) {
    		$this->text = $tr[1];
    		return ;
    	}

		$crso_id = $this->if_obj->model['crso']->relation->id;
		$participant_id = isset($this->if_obj->model['participant']) ? $this->if_obj->model['participant']->relation->id : array_keys($this->if_obj->model['participants']);

    	if(is_array($participant_id) && !count($participant_id))
    	{
    		$this->text = $tr[0];
    		return ;
    	}

		$q = $this->if_obj->model['object']->{$this->if_obj->overview_method}($this->if_obj->authorize_type,
			$this->if_obj->authorisation_filter, $crso_id, $participant_id,$_SESSION['selected_product_id']);

    	//$rv = (bool) $q->rows();

		require_once(IWARE.'TableFactoryColumnPlugin.php');
		$tf = new TableFactoryColumnPlugin(false,$this->langIf, $this->db, $this->db_app);
		$tf->setData($q, $this->if_obj->objectClass.'Overview',false,true);
		$tf->setEmptyText($this->langIf->getText('No registrations found') );

		$hideCols = array_merge($this->_hideColumns, array('AUTHORISATION_BY_TYPE'));

		// hide some extra columns in multiline - ticket #2035
	   	if ($this->if_obj->model['country']->has(HasProofOfPaymentMultiline, $_SESSION['selected_product_id']))
	   		$hideCols = array_merge($hideCols, array('INVOICE_ID', 'DOCUMENT', 'FEE', 'PAYMENT_DATE', 'INVOICE_DATE'));

	   	// Hide authorize for universes based on Country Setting #2137
	   	if($_SESSION['session_group_group_id']==GROUP_CRSO && !$this->if_obj->model['country']->has(HasCRSORefundConfirmations, $_SESSION['selected_product_id'])) {
			$hideCols[] = 'AUTHORIZE_CHECKBOX';
		} elseif($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT && $this->if_obj->model['country']->has(HasCRSORefundConfirmations, $_SESSION['selected_product_id'])) {
			$hideCols[] = 'AUTHORIZE_CHECKBOX';
		}

		$tf->hideColumns($hideCols);

		$tf->setValueProcessor(
			array('AUTHORIZE_CHECKBOX', 'STATUS', 'AMOUNT',
				'PROOF_OF_PAYMENT_ID', 'DOCUMENT', // payment
				'REFUND_REQUEST_ID', 'PROOF_OF_EXPORT',
				'REFUND_EXPORT_DATE', 'QUANTITY', 'FEE', // refund request
				'IMPORTER_VAT_NO'
			),
			array($this, '_tfValueProcessor')
		);

		$tf->setRowProcessor(array($this, '_tfRowProcessor'));
		$tf->columnModifier(array('QUANTITY'=>'style="text-align:right;"','FEE'=>'style="text-align:right;"','AMOUNT'=>'style="text-align:right;"'));

		$tf->setFooterProcessor(array($this,'_tfFooterProcessor'));

		$this->_shownColumns = $tf->getVisibleColumns();

		$this->text = isset($this->text)?$this->text:'';
		$this->text .= $tf->getXHTMLTable('Professional',2) . '<br/>';
		return true;
		//return $rv;
    }

    function _tfValueProcessor($args)
    {
    	switch ($args['head'])
    	{
    		case 'AUTHORIZE_CHECKBOX':
    			$chk = new Form_Checkbox('checkboxes[]', $args['row'][$this->primary_key], false);
    			return $chk->getHtml();
    		break;

    		case $this->primary_key:
    			return sprintf('<a href="%s">%s</a>', $this->getDetailsUrl($args['row'][$this->primary_key]), $args['row'][$this->primary_key]);
    		break;
    		case 'QUANTITY':
    			isset($this->_totalFields[$args['head']]) || $this->_totalFields[$args['head']] = 0;
    			$this->_headType[$args['head']] = 0;
    			$this->_totalFields[$args['head']] += $args['value'];
    			return locale_nf($args['value'], 0);
    		break;

			case 'STATUS':
				list($title, $class) = $this->if_obj->model['object']->translateStatus($this->langIf, $args['value']);
				return sprintf('<span class="%s">%s</span>', $class, $title);
			break;
    	}

    }

    /**
     * rowprocessor is empty here, used in some child classes
     */
    function _tfRowProcessor($row)
    {
    }

    /**
     * item[935] =
     */
    function getButtons()
    {
    	$this->_BarButtons = 0;

    	// dont show buttons if we dont have a crso/part selected
    	if (!isset($this->if_obj->model['crso']) || !isset($this->if_obj->model['participant']) )
    		return;

    	if($_SESSION['session_group_group_id']==GROUP_CRSO && !$this->if_obj->model['country']->has(HasCRSORefundConfirmations, $_SESSION['selected_product_id'])) {
			return;
		} elseif($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT && $this->if_obj->model['country']->has(HasCRSORefundConfirmations, $_SESSION['selected_product_id'])) {
			return;
		}


		$this->script .= sprintf("
			function has_selected_checkboxes()
			{
				// check if we selected atleast one checkbox and give an error
				var checks = document.getElementsByName('checkboxes[]');
				var err = true;
				for (var i=0; i<checks.length; i++)
				{
					if (checks[i].checked) {
						err = false;
						break;
					}
				}

				if (err) {
					".jsMessageBox(MSG_BOX_CRIT, '', $this->langIf->getText('Select atleast one '.$this->if_obj->objectClass))."
					return false
				}

				return true
			}
		");

		// authorize buttons
    	$this->_addCustomButton($this->langIf->getText('Approve '.$this->if_obj->objectClass), 'button',
    		array('onclick'=>jsMessageBox(MSG_BOX_QUEST,
    				$this->langIf->getText('Authorize selected'),
    				$this->langIf->getText('Are you sure you want to authorize the selected?'),
					"function() { $('interface_form').process.value='authorize'; if (has_selected_checkboxes() && fb.validate($('interface_form'))) { $('interface_form').submit();}  }")
				)
		);
    	$this->_addCustomButton($this->langIf->getText('Reject '.$this->if_obj->objectClass), 'button',
    		array('onclick'=>jsMessageBox(MSG_BOX_QUEST,
    				$this->langIf->getText('Deny Selected'),
    				$this->langIf->getText('Are you sure you want to deny the selected?'),
					"function() { $('interface_form').process.value='deny'; $('interface_form').submit(); }")
				)
    	);
    }

    function _tfFooterProcessor()
    {
    	if(!(bool) sizeof($this->_totalFields))
    	   	return false;

    	$cols = array_flip($this->_shownColumns);
		$rv = explode(',',str_repeat(',',(sizeof($this->_shownColumns)-1)));
		foreach($this->_totalFields as $head => $val)
		{
			$pos = $cols[$head];
			$round = $this->_headType[$head];
			$rv[$pos] = '<span style="float:right; font-weight: bold;">'.locale_nf($val,$round).'</span><span style="clear:both;"></span>';
		}
		return $rv;
    }
}

/**
 * CALLBACK[getParticipantSelect]
 * item[932] = {
 *     [handle] = Item: Participant
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getCRSOSelect]
 * item[934] = {
 *     [handle] = Item: CRSO
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getOverview]
 * item[933] = {
 *     [handle] = Item: ProofOfPaymentOverview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
