<?php

require_once(INTERFACES.'elc/common/InterfaceAuthorisationOverviewCommon.php');
require_once(ELC_CLASSES.'RefundRequest.php');

/**
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @version $Id$
 */
class InterfaceAuthorisationOverviewExport extends InterfaceAuthorisationOverviewCommon
{
	var $objectClass = 'RefundRequest';

	/**
	 * New status after authorize / deny
	 */
	var $status_approved = REFUND_REQUEST_APPROVED;
	var $status_denied = REFUND_REQUEST_REJECTED;

	/**
	 * CRSO/Participant settings column(s) to use
	 */
	var $authorize_property = array('authorize_refund_request_eu', 'authorize_refund_request_non_eu');

	/**
	 * Overview Filter
	 */
	var $authorisation_filter = array(REFUND_REQUEST_PENDING, REFUND_REQUEST_REJECTED);

	/**
	 * Constructor
	 */
	function InterfaceAuthorisationOverviewExport(&$db, &$db_app, &$langIf)
	{
		parent::InterfaceFactory($db, $db_app, $langIf);

		$rr = new RefundRequest($this->db_app);
		$this->model['object'] = &$rr;

		parent::InterfaceAuthorisationOverviewCommon($db, $db_app, $langIf);
	}
	function _filter($q)
	{
		$rv = parent::_filter($q);
		unset($rv[125][306][1403]);
		return $rv;
	}
	/**
	 * Loads the class for the blocks and items
	 */
	function loadClasses($interface_id)
	{
		$this->id = $interface_id;
		$this->classPrefix = 'InterfaceAuthorisationOverviewExport';
		$this->appPath = 'elc/common/';
	}

}

/**
 * @package ELC
 * @subpackage Interface
 * @since 09-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceAuthorisationOverviewExportBlock extends InterfaceAuthorisationOverviewCommonBlock
{

}

/**
 * @package ELC
 * @subpackage Interface
 * @since 09-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceAuthorisationOverviewExportItem extends InterfaceAuthorisationOverviewCommonItem
{
	var $primary_key = 'REFUND_REQUEST_ID';

	function getDetailsUrl($refund_id)
	{
		return sprintf('InterfaceExportForm.php?refund[id]=%s', $refund_id);
	}

	function _tfValueProcessor($args)
	{
    	switch ($args['head'])
    	{
			case 'AUTHORIZE_CHECKBOX':
    			$chk = new Form_Checkbox('checkboxes[]', $args['row'][$this->primary_key], false);
    			$chk->setID('authorize_'.$args['i']);
    			return $chk->getHtml();
    		break;

			case 'PROOF_OF_EXPORT':
				if(!$args['value'])
					return '';

				require_once(ELC_CLASSES.'Document.php');
				$doc = new ELCDocument($this->db_app,$args['value'],true); // for validation.

				if ($this->if_obj->_process == 'print')
					return sprintf('<img src="%s" />', REL_IMAGES.'i-w/true.gif');
				else
					return '<a href="/'.SITE_PATH.'scripts/i-w/document/download.php?Document[id]='.$doc->id.'">'.$this->langIf->getText('Export Document').'</a>';
			break;

			case 'REFUND_EXPORT_DATE':
				$pieces = explode('-',$args['value']);
				if(sizeof($pieces)!=3)
					return '';
				return $pieces[2].'-'.$pieces[1].'-'.$pieces[0];
			break;
			case 'IMPORTER_VAT_NO':
				require_once(IWARE.'VAT.php');

				$vat = new VAT($this->db_app,array('vat_no'=>$args['value']),true);
				return $vat->getVATDisplay();
			break;
    	}

    	return parent::_tfValueProcessor($args);
	}
}

?>
