<?php
require_once(INTERFACES.'elc/common/InterfaceAuthorisationOverviewCommon.php');

require_once(ELC_CLASSES.'ProofOfPayment.php');

/**
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @version $Id$
 */
class InterfaceAuthorisationOverviewPayment extends InterfaceAuthorisationOverviewCommon
{
	var $objectClass = 'ProofOfPayment';

	/**
	 * New status after authorize / deny
	 */
	var $status_approved = PROOF_OF_PAYMENT_APPROVED;
	var $status_denied = PROOF_OF_PAYMENT_REJECTED;

	/**
	 * CRSO/Participant settings column(s) to use
	 */
	var $authorize_property = array('authorize_proof_of_payment');

	/**
	 * Define the types which we want to see in the overview
	 */
	var $authorisation_filter = array(PROOF_OF_PAYMENT_PENDING, PROOF_OF_PAYMENT_REJECTED);

	function InterfaceAuthorisationOverviewPayment(&$db, &$db_app, &$langIf)
	{
		parent::InterfaceFactory($db, $db_app, $langIf);

		$pop = new ProofOfPayment($db_app);
		$this->model['object'] = &$pop;

		parent::InterfaceAuthorisationOverviewCommon($db, $db_app, $langIf);
	}

	/**
	 * Loads the class for the blocks and items
	 */
	function loadClasses($interface_id)
	{
		$this->id = $interface_id;
		$this->classPrefix = 'InterfaceAuthorisationOverviewPayment';
		$this->appPath = 'elc/common/';
	}

	function _filter($q)
	{
		$rv = parent::_filter($q);
		// Hide the authorization block depending on universe and country setting.
	    if($_SESSION['session_group_group_id']==GROUP_CRSO && !$this->model['country']->has(HasCRSORefundConfirmations, $_SESSION['selected_product_id'])) {
			unset($rv[$this->id][306]);
		} elseif($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT && (!isset($this->model['country']) || $this->model['country']->has(HasCRSORefundConfirmations, $_SESSION['selected_product_id']))) {
			unset($rv[$this->id][306]);
		}
		return $rv;
	}
}

/**
 * @package ELC
 * @subpackage Interface
 * @since 09-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceAuthorisationOverviewPaymentBlock extends InterfaceAuthorisationOverviewCommonBlock
{
}

/**
 * @package ELC
 * @subpackage Interface
 * @since 09-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceAuthorisationOverviewPaymentItem extends InterfaceAuthorisationOverviewCommonItem
{
	var $primary_key = 'PROOF_OF_PAYMENT_ID';

	function getDetailsUrl($refund_id)
	{
		return sprintf('InterfacePaymentForm.php?pop[id]=%s', $refund_id);
	}


	function getReasonSelect()
    {
    	$this->script = "
			var messageText = '';
			function set_message()
			{
				if(document.getElementById('id_o') == undefined)
				{
					var elem = document.createElement('input');
					elem.type = 'hidden';
					elem.name = 'other';
					elem.id = 'id_o';
					elem.value = messageText;
					document.getElementById('interface_form').appendChild(elem);
				}
				else document.getElementById('id_o').value = messageText;
			}
			function view_popup()
			{
				var el = document.getElementById('ifI_1403').value;
				if(el == 'Other')
				{".
					jsMessageBox(MSG_BOX_INFO,
    				$this->langIf->getText('Type in your reason'),
    				'<div align=center style=\'padding-top:10px;\'><textarea onblur="messageText = this.value;" id="id_other" name=other cols=45 rows=3></textarea></div>',
    				'function(){set_message()}').
    			"
				}
    		}";

    	$reasons = array(
    			'The uploaded document is illegible or does not contain the required data',
    			'The date given on portal does not correspond with the uploaded invoice',
    			'The quantity given on portal does not correspond with the uploaded invoice',
    			'The invoice number given on portal does not correspond with the uploaded invoice',
    			'The date, the quantity and the invoice number given on portal do not correspond with the uploaded invoice',
    			'The document has not been issued by the supplier identified',
    			'The uploaded invoice has an unknown format',
    			'Other',
    			);
    	$this->options[] = array('text'=>'- '.$this->langIf->getText('Select Reason').' -', 'value'=>'');
    	foreach($reasons as $reason)
    		$this->options[] = array('text'=>$reason, 'value'=>$reason);
    	$this->properties['onchange'] = "view_popup()";

    }

	function _tfValueProcessor($args)
	{
    	switch ($args['head'])
    	{
			case 'AUTHORIZE_CHECKBOX':
				if($_SESSION['session_group_group_id']==GROUP_CRSO && $args['row']['AUTHORISATION_BY_TYPE']!=REFUND_AUTHORIZE_OPERATOR) {
					return '-';
				}

    			$chk = new Form_Checkbox('checkboxes[]', $args['row'][$this->primary_key], false);
    			$chk->setID('authorize_'.$args['i']);
    			return $chk->getHtml();
    		break;

			case 'FEE':
				return locale_nf($args['value'], 2);
			break;

    		case 'AMOUNT':
    			isset($this->_totalFields[$args['head']]) || $this->_totalFields[$args['head']] = 0;
    			$this->_headType[$args['head']] = 2;
    			$this->_totalFields[$args['head']] += $args['value'];
    			return locale_nf($args['row']['QUANTITY'] * $args['row']['FEE'], 2);
    		break;

    		case 'DOCUMENT':
				if(!$args['value'])
					return '';

				require_once(ELC_CLASSES.'Document.php');
				$doc = new ELCDocument($this->db_app,$args['value'],true); // for validation.

				return '<a href="/'.SITE_PATH.'scripts/i-w/document/download.php?Document[id]='.$doc->id.'">'.$this->langIf->getText('PoP Document').'</a>';
    		break;
    	}

    	return parent::_tfValueProcessor($args);
	}
}


?>
