<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @since 18-07-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceCRSODateSettings extends InterfaceFactory
{
	/**
	 * array with fields that are build in the getSettings() item callback
	 * @access private
	 */
	var $_fields = array();

	var $currentValues;

	var $fieldStartDate;
	var $frmSelType;
	var $frmValue;

    /**
     * interface[id] = 139
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceCRSODateSettings(&$db, &$db_app, &$langIf)
    {
    	if (get_class($this) == strtolower('InterfaceCRSODateSettings'))
    		trigger_error('Abstract class', E_USER_ERROR);

        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 40;
        $this->delimiter = '<br />';

		// register common field
    	$txt = new Form_Text($this->fieldStartDate); // fieldStartDate must be defined in derived class
    	$txt->setClass('');
    	$txt->setBehaviour(sprintf('required(%s, true);date;elc_not_past', $this->frmSelType->getName()));
		$this->frmStartDate = &$txt;

		require_once(ELC_CLASSES.'FormBehaviourPlugins.php');
		$this->fb->addPlugin('elc_not_past', 'FormBehaviourPluginElcNotPast');

		/**
		 * Check if the field isset in POST and has a value and register it in formbehaviour with the fields bevaviour
		 */
		foreach (array('frmSelType', 'frmValue', 'frmStartDate') as $field)
		{
			if (!is_object($this->$field))
				trigger_error('not an object in class '.get_class($this).': $this->'.$field);

			$this->fb->register($this->$field->getName(), $this->$field->getBehaviour(), $field);
		}
    }

    function processInput(&$thrownException)
    {
        $invalid = $this->langIf->getText('Please correct the form errors');

        if(!$this->fb->validate())
        {
        	$thrownException .= $invalid;
        	return false;
        }

        return true;
    }
}
/**
 * @package
 * @subpackage Interface
 * @since 18-07-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */

class InterfaceCRSODateSettingsBlock extends InterfaceBlock {}

class InterfaceCRSODateSettingsItem extends InterfaceItem
{
    /**
     * item[1019] =
     */
    function getCRSOSettings()
    {
		$this->if_obj->frmSelType->addOption('- '.$this->langIf->getText('Select').' - ', '', false, false);

    	$ds = &$this->if_obj->_fields;
    	$ds[] = array (
			'setting'=>$this->if_obj->frmSelType->getHtml(),
			'date'=>$this->if_obj->frmStartDate->getHtml(),
			'value'=>$this->if_obj->frmValue->getHtml()
		);

    	$t = new TableFactory($ds, $this->if_obj->langIf);
    	$t->setValueProcessor(array('date', 'value'), array($this, '_tfValueProcessor'));
    	$t->collapseColumns(array('setting'));
    	$t->columnModifier(array('setting'=>' width="40%"'));
    	$t->hideColumns('typeId');
    	$this->text = $t->getXHTMLTable('Professional', 2);
    }

    function _tfValueProcessor($args)
    {
    	switch ($args['head'])
    	{
    		case 'date':
    			// this should exclude the form_text
    			$pieces = explode('-', $args['value']);
    			if (sizeof($pieces) == 3)
    				return reindex_date($args['value'], 'YYYY-MM-DD', 'DD-MM-YYYY');
    			else
    				return $args['value'];

    		case 'value':

    			if (isset($args['row']['typeId']) && $args['value'] == $this->if_obj->currentValues[$args['row']['typeId']])
    			{
    				// overview is ordered by date and we dont want to see a bold twice
    				$this->if_obj->currentValues[$args['row']['typeId']] = -1;
    				return '<b>'.$args['value'].'</b>';
    			}
    			return $args['value'];

    	}
    }

}

/**
 * CALLBACK[getCRSOSettings]
 * item[1019] = {
 *     [handle] = Item: CRSOSettings
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 1
 *
 */
?>
