<?php
require_once(INTERFACE_OBJ."InterfaceFactory.php");
require_once(IWARE.'group.php');
require_once(IWARE."address.php");
require_once(ELC_CLASSES.'FormBehaviourPlugins.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'CRSOParticipant.php');
require_once(ELC_CLASSES.'RelationInvoice.php');
require_once(ELC_CLASSES.'Country.php');
require_once(ELC_CLASSES.'uk/UKRegistrationData.php');
/**
 * InterfaceCompanyDetails
 *
 * Common Interface to be included in many other interface, that allows for viewing/editing participant company details.
 *
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @author Nick Loeve <nick@basesoft.com>
 * @copyright (2006) BaseSoft B.V.
 * @version $Id$
 */
class InterfaceCompanyDetails extends InterfaceFactory
{
	/**
	 * @var boolean Whether to include invoice information in rendering/processing
	 */
	var $hideInvoiceDetails = true;
	var $alwayshide = array('relation[vat_no]');

	/**
	 * @var boolean Whether this interface is being used in an ajax context
	 */
	var $ajaxUsage = false;

	function InterfaceCompanyDetails(&$db, &$db_app, &$langIf, $settings=array())
	{


		$this->FirstColWidth = 25;

		$this->model['target'] = 'ifI_841';
		if(isset($settings['targetId']))
		{
			$this->model['target'] = eval($settings['targetId']);
			unset($settings['targetId']);
		}
		$this->model['hideButtons'] = false;
		if(isset($settings['hideButtons']))
		{
			$this->model['hideButtons'] = (bool)$settings['hideButtons'];
			unset($settings['hideButtons']);
		}

		if (!$settings && isset($_REQUEST['participant']['id']) )
		{
			$this->ajaxUsage = isset($_REQUEST['ajaxUsage']) && !(int)$_REQUEST['ajaxUsage'] ? false : true;
			$this->hideInvoiceDetails = isset($_REQUEST['hideInvoiceDetails']) && (int)$_REQUEST['hideInvoiceDetails'] ? true : false;
			switch ($_SESSION['session_group_group_id'])
			{
				case GROUP_CRSO:
					$_REQUEST['crso']['id'] = $_SESSION['crso_relation_id'];
				case GROUP_SYSADMIN:
				case GROUP_ACCOUNTANT:
				case GROUP_PARTICIPANT:

					if(isset($_REQUEST['edit']) && $_REQUEST['edit'])
						$settings['_hideInputs'] = 'return false;';
					else
						$settings['_hideInputs'] = 'return true;';
					break;
				default:
					$settings['_hideInputs'] = 'return true;';
					break;
			}
			$settings['rel'] = '$relation = new Relation($this->db_app,'.(int) $_REQUEST['participant']['id'].'); return $relation;';
			$settings['participant'] = 'return new Participant($this->db_app,array(\'relation\'=> new Relation($this->db_app,'.((int)$_REQUEST['participant']['id']).')));';

			if(!$this->hideInvoiceDetails) {
				$settings['invoice'] = 'return new RelationInvoice($this->db_app, array(\'relation\'=> new Relation($this->db_app,' . ((int)$_REQUEST['participant']['id']) . ')));';
			}

			if(isset($_REQUEST['crso']['id'])) {
				$settings['cp'] = 'return new CRSOParticipantProduct($this->db_app);';
			}
		} else {

			// for #2652: check if we're crso and participant cannot edit company details.. then set the invoice object
			if ($_SESSION['session_group_group_id'] == GROUP_CRSO) {
				$crso = new CRSO($db_app, array('relation'=>new Relation($db_app, $_SESSION['crso_relation_id'])));
				$country = new Country($db_app, $crso->getCountry(), true);
				if( $country->has(HasParticipantCannotEditCompany, $_SESSION['selected_product_id'])) {
					$settings['invoice'] = 'return new RelationInvoice($this->db_app, array(\'relation\'=> new Relation($this->db_app,' . ((int)$_REQUEST['participant']['id']) . ')));';
				}
			}

		}

		parent::InterfaceFactory($db, $db_app, $langIf, $settings);

		//this is required as the item has this behaviour by default
		$this->fb->addPlugin('elc_vat_unique', 'FormBehaviourPluginElcVatUnique');


	}

	function init($id)
	{
		$this->id = $id;
		parent::init($id);
		// Rights shit

		if(isset($this->overrideRights))
			foreach($this->overrideRights as $key => $value)
				$this->$key = eval($value);
	}

	function processInput(&$thrownException)
	{
		if($_SESSION['session_group_group_id'] == GROUP_PUBLIC)
			return;



		if($_SESSION['session_group_group_id'] == GROUP_PARTICIPANT) {

			// raise exception if trying to edit company details when not allowed
			$country = new Country($this->db_app, $this->cp->crso->getCountry(), true);
			if ($country->has(HasParticipantCannotEditCompany, $_SESSION['selected_product_id'])) {
				throw new IWException('permission denied');
			}

		}


		/**
		 *
		 * @todo this has been done especially for ajax form submission, may need more work to make it generic
		 */
		if(isset($_REQUEST['process']) && $_REQUEST['process'] == 'ajaxsave')
		{
			// validate default request params
			// this is done through ajax, interface rights are read for everyone. please check

			require_once(INTERFACE_OBJ.'InterfacePermission.php');
			$ip = new InterfacePermissions($_SESSION['session_group_group_name'],array());
			try {
				$ip->validate();
			} catch(InterfacePermissionException $e) {
				throw $e;
			}

			$invalid = $this->langIf->getText('Please correct the form errors');
			$this->fb->unregister('relation[vat_no]');


			foreach($this->alwayshide as $field)
			{
				if(false===strpos($field,'['))
					unset($_REQUEST[$field],$_POST[$field]);
				else
				{
					// The name is an array, eval it and check Post data
					$postVar = "['".implode("']['", explode('[',str_replace(']','',$field)))."']";
					eval('unset($_REQUEST'.$postVar.',$_POST'.$postVar.');');
				}
			}

			//accountant can only save bank account related info
			if($_SESSION['session_group_group_id'] == GROUP_ACCOUNTANT){
				$rel = new Relation($this->db_app, (int)$_REQUEST['participant']['id']);
				$rel->bank_account_number = $_REQUEST['relation']['bank_account_number'];
				$rel->iban_no = $_REQUEST['relation']['iban_no'];
				$rel->swift_bic_code = $_REQUEST['relation']['swift_bic_code'];
				$rel->dd_bank_account = $_REQUEST['relation']['dd_bank_account'];
				$rel->dd_iban_no = $_REQUEST['relation']['dd_iban_no'];
				$rel->dd_swift_bic_code = $_REQUEST['relation']['dd_swift_bic_code'];

				//refresh the interface items
				$this->rel =& $rel;

				log_elc_event('Company Bank Account Information has been updated',
					$_SESSION['session_user_id'], null,
					isset($_REQUEST['crso']['id']) ? (int)$_REQUEST['crso']['id'] : null,
					isset($_REQUEST['participant']['id']) ? (int)$_REQUEST['participant']['id'] : null
				);
				$this->db_app->begin();
				$rel->update();
				//commit
				$this->db_app->commit();
				$this->init($this->id);
				return;
			}

			//now we instantiate new abstract objects and update them
	        $rel = new Relation($this->db_app, (int)$_REQUEST['participant']['id']);
	        $rel->set();
			$rel->customer_contact->set();
			$rel->address->set();
			$rel->customer_contact->setProperties($_REQUEST['contact']);
			$rel->address->setProperties($_REQUEST['address']);
			$rel->invoice_language->id = (int)$_REQUEST['invoice_language']['id'];



			unset($_REQUEST['relation']['vat_no']);
			$rel->setProperties($_REQUEST['relation']);

			$part = new Participant($this->db_app, array('relation' => $rel));
			$part->set();
			 //Participant->business_start_date should not be edited via this interface
			unset($_REQUEST['participant']['business_start_date']);
			$part->setProperties($_REQUEST['participant']);

	        if(!$this->hideInvoiceDetails)
	        {
		        $inv = new RelationInvoice($this->db_app, array('relation' => $rel));
		        $inv->set();
		        $inv->info_person->set();
		        $inv->address->set();
		        $inv->info_person->setProperties($_REQUEST['invoice_person']);
		        if (isset($_REQUEST['IsSameAsCompany']))
		        {
					if($inv->address->address_id && $inv->address->address_id != $rel->address->address_id)
						$old_invoice_address = $inv->address->address_id;
					$inv->address =& $rel->address;
					$inv->name = $rel->name;
		        }
				else
				{
					$inv->setProperties($_REQUEST['relation_invoice']);
					if($inv->address->address_id == $rel->address->address_id)
						$inv->address = new Address($this->db_app);
					$inv->address->setProperties($_REQUEST['invoice_address']);
				}
	        }


	        //refresh the interface items
	        $this->rel =& $rel;
	        $this->participant =& $part;
	        if(!$this->hideInvoiceDetails)
	        	$this->invoice =& $inv;

			if(!$this->fb->validate())
			{
	        	$this->init($this->id);
	        	return $thrownException .= $invalid;
			}

	        // start transaction
			$this->db_app->begin();
			//update
			if(isset($_REQUEST['crso']['id']) && isset($_REQUEST['cp']['producer_reg_no']))
			{
			 	$country = new Country($this->db_app,$this->cp->crso->getCountry(),true);
			 	if(!$country->has(HasRegNumberAsProducer,$_SESSION['selected_product_id']))
			 		throw new InterfacePermissionException('Illegal use of producer reg no');
				$cp = new CRSOParticipant($this->db_app,array('participant'=>$this->cp->participant, 'crso'=>$this->cp->crso));
				$cp->producer_reg_no = trim($_REQUEST['cp']['producer_reg_no']);
				$cp->update();
				//update status for EA if country is UK
				if($this->cp->crso->getCountry() == '826')
				{
					$rd = new UKRegistrationData($this->db_app);
					$rd->set($this->cp->crso->relation->id,$this->cp->participant->relation->id);
					$rd->updateWEEEStatusPRNProvided();
				}
			}
			$rel->customer_contact->update();
	        $rel->address->update();
	        $rel->update();
	        $part->update();
	        if(!$this->hideInvoiceDetails)
	        {
	        	$inv->info_person->update();
	        	if(!$inv->address->address_id)
	        		$inv->address->insert();
	        	else
	        		$inv->address->update();
	        	$inv->update();
	        	if(isset($old_invoice_address))
	        		$inv->address->delete($old_invoice_address);
	        }

                log_elc_event('Company Information has been updated',
                    $_SESSION['session_user_id'], null,
                    isset($_REQUEST['crso']['id']) ? (int)$_REQUEST['crso']['id'] : null,
                    isset($_REQUEST['participant']['id']) ? (int)$_REQUEST['participant']['id'] : null
                );

	        //commit
	        $this->db_app->commit();
	        $this->init($this->id);
		}
	}

	function _filter($q)
	{
		$rv = parent::_filter($q);

		$items = array (
			594, // Item: BankAccount
			595, // Item: IBANNumber
			690, // IBANSwiftBicCode
			598, // DdBankAccountNumber
			599, // IBANNumberDd
			692, // DDSwiftBicCode
			600, // POBoxNumber
			691, // SpacerSwiftBic
			693, // SpacerDDSwift
			596, // DirectDebit
			602,  // BusinessStartDate
		);

		$skip = true;
		switch ($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
			case GROUP_ACCOUNTANT:
				$items = array();
				$relation = new Relation($this->db_app,$_REQUEST['crso']['id']);
			break;

			case GROUP_PARTICIPANT:
				require_once(ELC_CLASSES.'RelationUser.php');
				$ru = new RelationUser($this->db_app);
				$q = $ru->getUserRelations($_SESSION['session_user_id']);
				while ($row = $q->fetch())
					if ($row->{'RELATION_ID'} == $this->rel->id)
						$items = array();
				$this->alwayshide[] = 'address[country_id]';
				$this->alwayshide[] = 'relation[bank_account_number]';
				$this->alwayshide[] = 'relation[iban_no]';
				$this->alwayshide[] = 'relation[swift_bic_code]';
				$this->alwayshide[] = 'relation[dd_bank_account]';
				$this->alwayshide[] = 'relation[dd_iban_no]';
				$this->alwayshide[] = 'relation[dd_swift_bic_code]';
				$this->alwayshide[] = 'hasDirectDebit';

			break;

			case GROUP_CRSO:
				$items = array();


				$this->alwayshide[] = 'relation[bank_account_number]';
				$this->alwayshide[] = 'relation[iban_no]';
				$this->alwayshide[] = 'relation[swift_bic_code]';
				$this->alwayshide[] = 'relation[dd_bank_account]';
				$this->alwayshide[] = 'relation[dd_iban_no]';
				$this->alwayshide[] = 'relation[dd_swift_bic_code]';
				$this->alwayshide[] = 'hasDirectDebit';

				// #2652: if HasParticipantCannotEditCompany is on, then the crso can edit fields
				$crso = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, $_SESSION['crso_relation_id'])));
				$country = new Country($this->db_app, $crso->getCountry(), true);
				if( $country->has(HasParticipantCannotEditCompany, $_SESSION['selected_product_id'])) {

					// let crso also edit invoice details
					$this->hideInvoiceDetails = false;

				} else {
					// normal behaviour: participant can edit company details
					// but only accountant can edit invoice data
					$this->hideInvoiceDetails = true;
					$this->alwayshide[] = 'relation[email]';

				}

			break;

			default:
				// do nothing
		}

		//nobody can see Producer Reg No Item if the Country doesn't have this setting
		$crso_id = (isset($_SESSION['crso_relation_id']))?$_SESSION['crso_relation_id']:((isset($this->cp)&&isset($this->cp->crso))?$this->cp->crso->relation->id:(isset($_REQUEST['crso']['id'])?$_REQUEST['crso']['id']:false));
		if($crso_id)
			$relation = new Relation($this->db_app,$crso_id);
		else
			$relation = new Relation($this->db_app,$_REQUEST['participant']['id']);
		require_once(ELC_CLASSES.'Country.php');
		$country = new Country($this->db_app,$relation->getCountry(),true);
		if(!$country->has(HasRegNumberAsProducer,$_SESSION['selected_product_id']))
		{
			$items[] = ITEM_REG_NO;
			$skip = false;
		}

		if ($skip&&isset($rv[$this->id][BLOCK_COMPANY]))
			unset($rv[$this->id][BLOCK_COMPANY][ITEM_REG_SPACER]);

		foreach ($items as $item)
			unset($rv[$this->id][146][$item]);

		if($this->hideInvoiceDetails)
			unset($rv[$this->id][320]);

		//edit buttons
		if($this->_hideInputs && !$this->ajaxUsage)
			unset($rv[$this->id][321]);

		return $rv;
	}

	function getDynamicJSBehaviours()
	{
		ob_start();
		?>
				ifHandler.watchFormOnChange("interface_form", function(){ $("ajaxstatus").innerHTML = "<?php echo $this->langIf->getText('form details have been changed but are not saved...'); ?>";})

				/**
				 * Invoice
				 */
				function toggleInvoice(nNode)
				{
					var invoice = new Array('relation_invoice[name]','invoice_address[country_id]',
						'invoice_address[address_street_name]','invoice_address[city_name]',
						'invoice_address[address_zipcode]');

					var check = nNode;

					invoice.forEach(function(element, index, array)
					{
						el = document.getElementsByName(element)[0];
						el.disabled = check.checked?'disabled':'';

						if(check.checked)
						{
							addCssClass(el,'disabled');
							el.value = '';
						}
						else
							removeCssClass(el,'disabled');
					})
				}

				if($('IsSameAsCompany'))
					addEvent($('IsSameAsCompany'), 'click', function(){ toggleInvoice($('IsSameAsCompany')); });

		<?php
		return ob_get_clean();
	}


}

class InterfaceCompanyDetailsItem extends InterfaceItem
{
	/**#@+
	 * Callback functions made from InterfaceItem
	 */
	/**
	 *
	 */
	function getItemValue()
	{


		$available_to_accountant = array('relation[bank_account_number]', 'relation[iban_no]', 'relation[swift_bic_code]',
								'relation[dd_bank_account]', 'relation[dd_iban_no]', 'relation[dd_swift_bic_code]');
		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT && !in_array($this->name, $available_to_accountant))
			$this->_hideInput = true;
		$this->properties['value'] = '';
		$rel =& $this->parent->parent->rel;
		$part =& $this->if_obj->participant;
		$cp = isset($this->if_obj->cp)?$this->if_obj->cp:array();

		if (!$rel)
			return;

		if(!$rel->complete)
		{
			$rel->set();
			$rel->customer_contact->set();
			$rel->address->set();
			$part->set();

			if (!is_array($cp)) {
				$cp->participant =& $part;
				if (isset($_REQUEST['crso']['id'])) // we can only show the producer reg no if we have a crso
					$cp->set((int)$_REQUEST['crso']['id'], null, $_SESSION['selected_product_id']);
			}

		}
		$relation = (array) $rel;
		$contact = (array) $rel->customer_contact;
		$address = (array) $rel->address;
		$participant = (array) $part;
		$cp = (array) $cp; // Added for ProducerRegNo
		$name = str_replace('[','[\'',str_replace(']','\']',$this->name));
		if ($name)
			eval('$this->properties[\'value\'] = isset($'.$name.')?$'.$name.':\'\';');

		/**
		 * @todo VERIFY THIS: we never allow editing of VAT in an interface?
		 */
		if(in_array($this->name,$this->if_obj->alwayshide))
			$this->_hideInput = true;

		if($this->name == 'cp[producer_reg_no]' &&isset($this->if_obj->cp))
		{
			require_once(ELC_CLASSES.'CRSOParticipant.php');
			$cp = new CRSOParticipant($this->db_app);
			$cp->participant = $this->if_obj->participant;
			$cp->crso = $this->if_obj->cp->crso;
			$cp->set();
			$cp->crso->set();
			$cp->crso->relation->set();
			$this->properties['value'] = $cp->producer_reg_no;
			$handle = $this->handle.'_'.$cp->crso->relation->name;
			// we need bytes but we dont want to break the multibyte string
			$handle = (strlen($handle)>254)?mb_strcut($handle,0,254):$handle;
			$crsoTrans = IWRegistry::registry('langIf')->getText($handle);
			$this->title = empty($crsoTrans)?$this->title:$crsoTrans;
		}
	}

	function getItemValueInvoice()
	{
		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT)
			$this->_hideInput = true;
		$this->properties['value'] = '';
		$ip =& $this->if_obj->invoice->info_person;
		$add =& $this->if_obj->invoice->address;
		$inv =& $this->if_obj->invoice;
		$invoice_address = (array) $add;
		$invoice_person = (array) $ip;
		$relation_invoice = (array)$inv;

		switch($this->name)
		{
			case 'relation_invoice[name]':
			case 'invoice_address[address_street_name]':
			case 'invoice_address[address_zipcode]':
			case 'invoice_address[city_name]':
				$addressSame = $this->_isAddressSame();
				$relationSame = $this->_isRelationSame();
				if(isset($_POST['IsSameAsCompany']) || ($addressSame && $relationSame))
				{
					$this->properties['disabled'] = 'disabled';
					$this->properties['class'] = 'disabled';
					break;
				}
			default:
				$name = str_replace('[','[\'',str_replace(']','\']',$this->name));
				eval('$this->properties[\'value\'] = isset($'.$name.')?$'.$name.':\'\';');
				break;
		}
		if(in_array($this->name,$this->if_obj->alwayshide))
			$this->_hideInput = true;
	}
	/**
	 *
	 */
	function getCountrySelect()
	{
		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT)
			$this->_hideInput = true;
		require_once(ELC_CLASSES.'Country.php');
		$country = new Country($this->db_app);
		$q = $country->select();
		$this->options = array(array('text'=>'- '.$this->langIf->getText('Select Country').' -','value'=>''));

		$i = 1;
		while($r = $q->fetch())
		{
			$this->options[$i] = array('text'=>$r->NAME,'value'=>$r->COUNTRY_ID);
			if (isset($this->parent->parent->rel->address->country_id) && $this->parent->parent->rel->address->country_id == $r->COUNTRY_ID)
				$this->options[$i]['selected'] = 'selected';
			$i++;
		}
		if(in_array($this->name,$this->if_obj->alwayshide))
			$this->_hideInput = true;
	}
	/**
	 *
	 */
	function getLanguageSelect()
	{
		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT)
			$this->_hideInput = true;
		require_once(IWARE.'nLanguage.php');
		$lang = new nLanguage($this->db_app);
		$q = $lang->get();
		$this->options = array(array('value'=>'','text'=>'- '.$this->langIf->getText('Select language').' -'));
		$i = 1;
		while($r = $q->fetch())
		{
			$this->options[$i] = array('text'=>$r->LANGUAGE,'value'=>$r->LANGUAGE_ID);
			if(isset($this->parent->parent->rel->invoice_language->id)&& $this->parent->parent->rel->invoice_language->id==$r->LANGUAGE_ID)
				$this->options[$i]['selected'] = 'selected';
			$i++;
		}
		if(in_array($this->name,$this->if_obj->alwayshide))
			$this->_hideInput = true;
	}
	/**
	 *
	 */
	function getSpacer()
	{
		$this->properties['value'] = '';
		$this->twoCols = true;
	}

	function get2colTxt()
	{
		$this->getSpacer();
	}
	/**
	 *
	 */
	function getDirectDebit()
	{
		$this->options = array();
		$this->options[1] = array('value'=>1,'text'=>$this->langIf->getText('Yes'));
		$this->options[0] = array('value'=>0,'text'=>$this->langIf->getText('No'));

		if ($this->if_obj->rel->dd_bank_account)
		{
			$this->options[1]['checked'] = 'checked';
			$this->options[1]['selected'] = 'selected';
		}
		else
		{
			$this->options[0]['checked'] = 'checked';
			$this->options[0]['selected'] = 'selected';
		}
		if(in_array($this->name,$this->if_obj->alwayshide))
			$this->_hideInput = true;
	}

	function getBusinessStartDate()
	{
		$this->_hideInput = true;
		if (isset($this->if_obj->cp))
		{
			if(!$this->if_obj->cp->complete && isset($_REQUEST['participant']['id']) && isset($_REQUEST['crso']['id']))
				$this->if_obj->cp->set((int)$_REQUEST['crso']['id'], (int)$_REQUEST['participant']['id']);
			elseif(!$this->if_obj->cp->complete)
				$this->if_obj->cp->set();
			$this->properties['value'] = Form_Date::makePretty($this->if_obj->cp->obligation_start_date);
		}
		else
			$this->properties['value'] = '-';

	}

	/**
     * item[946] =
     */
    function getButtons()
    {
    	$this->_BarButtons = 0;
    	if($this->if_obj->model['hideButtons'])
    		return;

    	switch ($_SESSION['session_group_group_id'])
		{
			case GROUP_PARTICIPANT:

				// if HasParticipantCannotEditCompany is on, don't show any buttons if we're a participant, instead they should contact the CRSO
				$country = new Country($this->db_app,$this->if_obj->cp->crso->getCountry(), true);
				if($country->has(HasParticipantCannotEditCompany, $_SESSION['selected_product_id'])) {
					break;
				}

			case GROUP_SYSADMIN:
			case GROUP_ACCOUNTANT:
			case GROUP_CRSO:
				if(!$this->if_obj->_hideInputs)
				{
					$this->_CustomButtons[] = '<div id="ajaxstatus" style="color: red; display: inline;"></div>&nbsp;&nbsp;';
					$this->_addCustomButton($this->langIf->getText("View"), "button", array(
							'onclick' => "\$('interface_form_process').value = 'ajaxedit'; var params = []; params['ifId'] = ".$this->if_obj->id."; params['targetId'] = '".$this->if_obj->model['target']."'; params['settings'] = []; params['settings']['targetId'] = 1; jsbroker.callAndAttachRequestVars([['edit', '0'], ['participant[id]', '".((int)$_REQUEST['participant']['id'])."']".(isset($_REQUEST['crso']['id']) ? ",['crso[id]', '".((int)$_REQUEST['crso']['id'])."']" : "")."], 'loadInterface', params, ifHandler.buildInterface);"
							));
					$this->_addCustomButton($this->langIf->getText("Save"), "button", array(
							'onclick' => " \$('ajaxstatus').innerHTML = '".$this->langIf->getText('saving form details...')."'; \$('interface_form_process').value = 'ajaxsave'; var params = []; params['ifId'] = ".$this->if_obj->id."; params['targetId'] = '".$this->if_obj->model['target']."'; params['settings'] = []; params['settings']['targetId'] = 1; jsbroker.callAndSubmitForm('interface_form', 'loadInterface', params, ifHandler.buildInterface);"
							));
					$this->_addCustomButton((int)$_REQUEST['participant']['id'], "hidden", array(
							'name' => 'participant[id]'
							));
					$this->_addCustomButton(1, "hidden", array(
							'name' => 'edit'
							));
					if(isset($_REQUEST['crso']['id']))
						$this->_addCustomButton((int)$_REQUEST['crso']['id'], "hidden", array('name' => 'crso[id]'));
				}
				else
				{

					$this->_addCustomButton($this->langIf->getText("Edit"), "button", array(
						'onclick' => "\$('interface_form_process').value = 'ajaxedit'; var params = []; params['ifId'] = ".$this->if_obj->id."; params['targetId'] = '".$this->if_obj->model['target']."'; params['settings'] = []; params['settings']['targetId'] = 1; jsbroker.callAndAttachRequestVars([['edit', '1'], ['participant[id]', '".((int)$_REQUEST['participant']['id'])."']".(isset($_REQUEST['crso']['id']) ? ",['crso[id]', '".((int)$_REQUEST['crso']['id'])."']" : "")."], 'loadInterface', params, ifHandler.buildInterface);"
						));
					if(isset($_REQUEST['crso']['id']))
						$this->_addCustomButton((int)$_REQUEST['crso']['id'], "hidden", array('name' => 'crso[id]'));
					if(isset($_REQUEST['participant']['id']))
						$this->_addCustomButton((int)$_REQUEST['participant']['id'], "hidden", array('name' => 'participant[id]'));

					$this->_addCustomButton($this->langIf->getText("Export masterdata"), "button", array(
							'onclick' => "$('interface_form').process.value='export'; $('interface_form').submit(); "
							));
				}
				break;
		}
    }

	function getIsSameAsCompany()
	{
		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT)
			$this->_hideInput = true;
		if(!$this->if_obj->invoice->complete)
		{
			$this->if_obj->invoice->set();
			$this->if_obj->invoice->info_person->set();
			$this->if_obj->invoice->address->set();
			$this->if_obj->invoice->relation->set();
		}

		$addressSame = $this->_isAddressSame();
		$relationSame = $this->_isRelationSame();

		$this->properties['id'] = 'IsSameAsCompany';
		//$this->properties['value'] = '1';

		if(isset($_POST['IsSameAsCompany']) || ($addressSame && $relationSame))
			$this->properties['checked'] = 'checked' && $this->properties['value'] = '1';
		else
			$this->properties['value'] = '0';

	}

	function getRightColText()
	{
		//$this->twoCols = true;
		$this->text = $this->langIf->getText('Invoice User Information');
	}

	function getCountrySelectInvoice()
	{
		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT)
			$this->_hideInput = true;
		$disabled = false;
		$addressSame = $this->_isAddressSame();
		$relationSame = $this->_isRelationSame();
		if(isset($_POST['IsSameAsCompany']) || ($addressSame && $relationSame))
		{
			$disabled = true;
			$this->properties['disabled'] = 'disabled';
			$this->properties['class'] = 'disabled';
		}
		require_once(ELC_CLASSES.'Country.php');
		$country = new Country($this->db_app);
		$q = $country->select();
		$this->options = array(array('text'=>'- '.$this->langIf->getText('Select Country').' -','value'=>''));

		$i = 1;
		while($r = $q->fetch())
		{
			$this->options[$i] = array('text'=>$r->NAME,'value'=>$r->COUNTRY_ID);
			if ($this->parent->parent->invoice->address->country_id == $r->COUNTRY_ID && !$disabled)
				$this->options[$i]['selected'] = 'selected';
			$i++;
		}
	}

	function _compareAddress($address1, $address2)
	{
		return (
					$address1->address_id == $address2->address_id ||
					(
						$address1->address_street_name == $address2->address_street_name &&
						$address1->address_street_number == $address2->address_street_number &&
						$address1->address_zipcode == $address2->address_zipcode &&
						$address1->city_name == $address2->city_name &&
						$address1->country_id == $address2->country_id
					)
				);
	}

	function _isAddressSame()
	{
		return $this->_compareAddress($this->if_obj->invoice->address, $this->if_obj->rel->address);
	}

	function _isRelationSame()
	{
		return ($this->if_obj->invoice->name == $this->if_obj->rel->name ||
				$this->_isAddressSame() && $this->if_obj->invoice->name == '');
	}

	function getCompanyEditText() {

		// bit of a hack, call getItemValue() once to initialise the $this->if_obj->cp
		$this->getItemValue();

		$warning_text = $this->langIf->getText('Warning only CRSO can edit company information');

		if(isset($_REQUEST['crso']['id'])) {

			$country = new Country($this->db_app, $this->if_obj->cp->crso->getCountry(), true);
			if($country->has(HasParticipantCannotEditCompany, $_SESSION['selected_product_id'])) {
				$this->text = '<span class="warning">'.$warning_text.'</span>';
			}
		}
	}

	/**#@-*/
}
/**#@+
 *
 */
define('BLOCK_COMPANY',146);
define('ITEM_REG_NO',592);
define('ITEM_REG_SPACER',593);
/**#@-*/
