<?php

require_once(INTERFACES.'elc/common/InterfaceAuthorisationOverviewExport.php');
require_once(ELC_CLASSES.'RefundRequest.php');
require_once(ELC_CLASSES.'RefundRequestConfirmation.php');



/**
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @version $Id$
 */
class InterfaceConfirmationOverviewExport extends InterfaceAuthorisationOverviewExport
{
	/**
	 * When we validate the refund request will get the status confirmed
	 * This still has approved in it due to backwards compatability.
	 * Removed the APPROVAL STEP FROM REFUND for ticket #1071
	 */
	var $authorisation_filter = array(REFUND_REQUEST_PENDING, REFUND_REQUEST_APPROVED, REFUND_REQUEST_CONFIRMED);

	/**
	 * status after approving / denying
	 */
	var $status_approved = REFUND_REQUEST_CONFIRMED;
	var $status_denied = REFUND_REQUEST_REJECTED;
	var $overview_method = 'selectConfirmationOverview';

	function InterfaceConfirmationOverviewExport(&$db, &$db_app, &$langIf)
	{
		parent::InterfaceAuthorisationOverviewExport($db, $db_app, $langIf);

		$rr =& $this->model['object'];

		$rrc = new RefundRequestConfirmation($this->db_app);
		$this->model['rrc'] = &$rrc;

		$d = &currentDate();
		$time = $d->getTime();

		// set title to last month if we are before export else current month
		//$month = ($d->getDay() < 11) ? 1 : 0; // FIXME: replace 11 with RefundExportDay
		//setPageTitlePostfix(sprintf('(%s %s)', strftime('%B', strtotime('-'.$month.' month', $time)), strftime('%Y', strtotime('-'.$month.' month', $time))));
		//if(!isset($this->authorize_type[0]))
			//exit('No Refund Settings');

		if (isset($this->model['crso']) && isset($this->model['participant']) && isset($this->authorize_type[0]))
		{
			if ($rr_c_id = $rr->getRefundRequestConfirmationId($this->model['crso']->relation->id, $this->model['participant']->relation->id,$_SESSION['selected_product_id'],$this->authorize_type[0]))
			{
				//debug('setting RefundRequestConfirmation to '.$rr_c_id);
				$rrc->set($rr_c_id);
				$this->hiddenFields[] = array('name'=>'rrc[id]', 'value'=>$rr_c_id);
			}
			//else
				//debug('new RefundRequestConfirmation');
		}
	}

	/**
	 * Loads the class for the blocks and items
	 */
	function loadClasses($interface_id)
	{
		$this->id = $interface_id;
		$this->classPrefix = 'InterfaceConfirmationOverviewExport';
		$this->appPath = 'elc/common/';
	}

	/**
	 * This makes sure the parent filter isnt called so the signature document isnt hidden.
	 * If anyone has a better fix for this..
	 */
	function _filter($q)
	{
		$rv = InterfaceFactory::_filter($q);

    	if (!isset($this->model['crso']) || !isset($this->model['participant']) || ($this->_process == 'print'))
    		unset($rv[$this->id][306][945]);

		// hide button block if we are not allowed to authorize
    	if (!$this->allow_authorize)
			unset($rv[$this->id][306]);


		// kill month select, see #481
		unset($rv[$this->id][304][1107]);
		unset($rv[$this->id][306][1403]);
		// and make participant select close the row
		$rv[$this->id][304][932]['CLOSE_ROW'] = 1;

		// hide crso/part selection if we gonna print
		if ($this->_process == 'print')
			unset($rv[$this->id][304]);

		return $rv;
    }

    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
    	if (isset($_REQUEST['process']))
    	{
    		// we are not gonna process anything in print
    		if ($_REQUEST['process'] == 'print')
    			return;

    		if ($_REQUEST['process'] == 'authorize')
    		{
	    		$this->db->begin();

	    		$rc = new RefundRequestConfirmation($this->db_app);
				$this->model['rrc'] =& $rc;

	        	if (isset($_REQUEST['rrc']['id']))
	        	{
	        		// TODO: check rights
	        		if(!$rc->set((int)$_REQUEST['rrc']['id']))
	        		{
	        			die('Permission Denied!');
	        		}
	        	}
	        	else
	        	{
	        		$rc->crso =& $this->model['crso'];
	        		$rc->participant =& $this->model['participant'];
	        		$rc->product = new Product($this->db_app,$_SESSION['selected_product_id']);
	        	}
				$d = &currentDate();
				$time = $d->getTime();

	    		$rc->month = date('m', strtotime('-1 month', $time));
	    		$rc->year = date('Y', strtotime('-1 month', $time));

	    		/**
	    		 * @todo FIXME If we do not clean up old documents...
	    		 */
	    		$item = new InterfaceItem($this->db,$this->db_app,$this->langIf);
	    		if ($document = $item->getDocument('document')) {
	        		$rc->document =& $document;
	        		$rc->document->makePermanent();
	    		}

	        	//mark the document unreviewed
	        	$rc->review_status = RR_CONFIRMATION_UNREVIEWED;
	        	$rc->auth_by_type = $this->authorize_type[0];

	        	if (isset($_REQUEST['rrc']['id']))
	        		$rc->update();
	        	else
	    			$rc->insert();

				$this->_processCheckboxes();
	    		$this->db->commit();

				// Send a notification to the operator that this participant has confirmed exports
				// Get the operators:
				require_once(ELC_CLASSES.'Notification.php');
                $recipients = array();

                $country = new Country($this->db_app, $this->model['crso']->relation->getCountry(), true);

                if($country->has(HasCRSORefundConfirmations, $_SESSION['selected_product_id'])) {
                	$recipient = $this->model['crso']->relation->id;
                	$universe = 'crso';
                } else {
                	require_once(ELC_CLASSES.'AccountantCRSO.php');
                	$ac = new AccountantCRSO($this->db_app);
                	$acc = $ac->getCRSOAccountants($this->model['crso']->relation->id,$_SESSION['selected_product_id']);
                	$r = $acc->fetch();
                	$recipient = new Relation($this->db_app, $r->ACCOUNTANT_RELATION_ID);
                	$universe = 'accountant';
                }

				$recipients[] = array('recipient'=> $recipient,
							'callback'=>"go('/".SITE_PATH."scripts/elc/".$universe."/InterfaceReviewRefundExports.php?refund_export_overview[type]=0&participant[id]=".$this->model['participant']->relation->id."&crso[id]=".$this->model['crso']->relation->id."');");

				$map = array(
					'sender'  => &$this->model['participant']->relation,
					'type'    => NOTIFICATION_TYPE_INFO,
					'subject' => $this->langIf->getText('Refund Request Confirmation'),
					'message' => $this->langIf->getText('Participant {PARTICIPANT} has uploaded a new export confirmation document',null,
						array('PARTICIPANT'=>$this->model['participant']->relation->name))
					);
				if($recipients)
					Notification::send($this->db_app,$map,$recipients);

				preg_match('/\/scripts\/(.*)\/(.*)\/(Interface[\w]+)\.php/', $_SERVER['REQUEST_URI'],$matches);

		        header(sprintf('location: %s?participant[id]=%s&crso[id]=%s&checkstock=true',
		        	$matches[0], (int)$_REQUEST['participant']['id'], (int)$_REQUEST['crso']['id']));

		        die();
    		}
    		else {
    			// process deny (and other stuff?) in parent class
	    		parent::processInput($thrownException);
    		}
    	}
    }

    function _processCheckboxes()
    {
		$rr_confirm_id = $this->model['rrc']->id;
    	$rr = new RefundRequest($this->db_app);

		if ($_REQUEST['process'] == 'authorize')
		{
	    	// first rollback any confirmed requests with rr_confirm_id to approved
	    	$rr->rollbackConfirmedRequests($rr_confirm_id);

	    	// second reset all refund requests with rr_confirm_id to NULL so it can be updated
	    	// (we dont know which checkboxes are not checked)
			$rr->nullifyRequestConfirmationId($rr_confirm_id);

	        foreach ($_REQUEST['checkboxes'] as $id)
	        {
	        	$rr->set($id);

	     		// In case of the refund request confirmation overview a new RefundRequestConfirmation object can be created.
	     		// Here we attach the foreign key.
	     		//debug(sprintf("updating rr [%s] to rr conf id [%s]", $id, $rr_confirm_id));
	     		$rr->refund_request_confirmation_id = $rr_confirm_id;
	     		$rr->update();
	        }
		}
		elseif ($_REQUEST['process'] == 'deny')
		{
	        foreach ($_REQUEST['checkboxes'] as $id)
	        {
	        	// if we reject a RefundRequest, make sure refund_request_confirmation_id is NULL
	        	$rr->set($id);
	     		$rr->refund_request_confirmation_id = DB_NULL;
	     		$rr->update();
	        }
		}


        // change status in parent
        parent::_processCheckboxes();
    }

	/**#@-*/
	/**
	 * Get the information table on top of the Declaration Form
	 * @todo mkpretty
	 */
	function getInterfaceHeader()
	{
		$rv = parent::getInterfaceHeader();

		if ($this->_process != 'print')
			return $rv;

		$p =& $this->model['crso']->relation;
		$p->set();
		$p->address->set();

		$c =& $this->model['participant']->relation;
		$c->set();
		$c->address->set();

		require_once(ELC_CLASSES.'Country.php');
		$country = new Country($this->db_app);
		$country->set($p->address->country_id);
		$p_country = $country->name;
		$country->set($c->address->country_id);
		$c_country = $country->name;

		$d = &currentDate();
		$time = $d->getTime();

		$rv .= '
			<h1>'.$this->langIf->getText('Refund Request Confirmation Print').'</h1>
			<table class="Professional" width="100%">
				<tr>
					<td width="25%">'.$this->langIf->getText('PERIOD').'</td>
					<td width="25%">
						<strong style="font-size: 10pt !important">'.
							strftime('%B',strtotime('-1 month', $time)).' '.strftime('%Y', strtotime('-1 month', $time)).
						'</strong></td>
					<td width="50%">
						'.$this->langIf->getText('Document Checksum').':
						<strong style="font-size: 10pt !important">'.RefundRequest::generateConfirmProofOfExportChecksum($_REQUEST['checkboxes']).'</strong>
					</td>
				</tr>
				<tr>
					<td>'.$this->langIf->getText('Print date').'</td>
					<td>
						'.$d->format('d-m-Y').'
					</td>
				</tr>
				<tr>
					<td colspan="3"><br /></td>
				</tr>
				<tr>
					<td colspan="2">'.$p->name.'</td>
					<td>'.$c->name.'</td>
				</tr>
				<tr>
					<td colspan="2">'.$p->address->address_street_name.'</td>
					<td>'.$c->address->address_street_name.'</td>
				</tr>
				<tr>
					<td colspan="2">'.$p->address->city_name.', '.$p_country.'</td>
					<td>'.$c->address->city_name.', '.$c_country.'</td>
				</tr>
				<tr>
					<td colspan="2"></td>
					<td>'.$c->address->address_zipcode .', '.$c->email.'</td>
				</tr>
				<tr>
					<td>'.$this->langIf->getText('VAT Number').'</td>
					<td>'.$p->vat_no.'</td>
					<td></td>
				</tr>
				<tr>
					<td>'.$this->langIf->getText('Bank Account Number').'</td>
					<td>'.$p->bank_account_number.'</td>
					<td></td>
				</tr>
			</table>
			<br /><br />
			';

		return $rv;
	}

	function getInterfaceFooter()
	{
		$rv = parent::getInterfaceFooter();

		if ($this->_process != 'print')
			return $rv;

		$user = new ELCUser($this->db_app, $_SESSION['session_user_id'], true);
		$user->info_person->set();

		$lines = array();
		$lines[] = $user->info_person->getFullName();
		$lines[] = $user->info_person->title;
		$lines[] = $user->info_person->position;
		$lines[] = $user->info_person->department;
		$lines[] = $user->info_person->telhouse;
		$lines[] = $user->info_person->fax;
		$lines[] = $user->info_person->email;

		$rv .= '<table class="Professional">';

		foreach ($lines as $line)
		{
			if (empty($line))
				continue;

			$rv .= '<tr>
						<td>'.$line.'</td>
					</tr>';
		}

		// signature box
		$rv .= '	<tr>
						<td>
							<div style="border: 1px solid black; margin: 10px; width: 200px; height: 50px;"></div>
						</td>
					</tr>
				</table>
		';

		return $rv;
	}

}

/**
 * @package ELC
 * @subpackage Interface
 * @since 09-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceConfirmationOverviewExportBlock extends InterfaceAuthorisationOverviewExportBlock
{
	/**
	 * change title for one block
	 */
	function setTitle($title)
	{
		if ($this->id == 305)
			parent::setTitle('Confirmation Overview Export Block');
		else
			parent::setTitle($title);
	}

	function _displayBlock($classes, $split)
	{
		if ($this->id == 306)
			$classes[] = 'no_print';

		return parent::_displayBlock($classes, $split);
	}
}

/**
 * @package ELC
 * @subpackage Interface
 * @since 09-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceConfirmationOverviewExportItem extends InterfaceAuthorisationOverviewExportItem
{
	/**
	 * item[868] = refund[export_document]
	 */
	function getDocumentSignature()
	{
		//We now always force a new document as the hash will change when selecting different refund requests
		//if (isset($this->if_obj->model['rrc']->document->id))
			//$_REQUEST['Documents'][$this->id] = (int) $this->if_obj->model['rrc']->document->id;
	}

	function getCutOffDate()    {
		if(!isset($this->if_obj->model['crso'])|| !$this->if_obj->model['crso'])
			$this->text = '';
		else {
			require_once(ELC_CLASSES.'CRSOProduct.php');
			$cp = new CRSOProduct($this->db_app);
			$cp->set($this->if_obj->model['crso']->relation->id, $_SESSION['selected_product_id']);
			$r = $cp->getNextCutOffDateRefund();
			if(!$r)
				$this->text = '';
			else
				$this->text = "<span class='cut-off-date'>".$this->langIf->getText('Upcoming export date').": ".$r."</span>";
		}
	}

	function getOverview()
    {
    	// we dont want to see checkbox column in print overview
    	if ($this->if_obj->_process == 'print')
    	{
			$this->_hideColumns[] = 'AUTHORIZE_CHECKBOX';
			$this->_hideColumns[] = 'STATUS';
    	}

		if(isset($this->if_obj->model['crso']) && $this->if_obj->model['crso']) {
			$country = new Country($this->db_app, $this->if_obj->model['crso']->relation->getCountry(), true);
			if (!$country->has(HasDeclByCategory, $_SESSION['selected_product_id'])) {
				$this->_hideColumns[] = 'PRODUCT_CATEGORY';
			}
		}

		if(!parent::getOverview())
			return;

		// Dont show if we're gonna print it
		if(isset($_REQUEST['process'])&& $_REQUEST['process'] == 'print')
			return;


		$this->text .= sprintf('<input id="selectAllCheckboxes" type="button" value="%s" onclick="rr_selectAll();" />
				<input id="deselectAllCheckboxes" type="button" value="%s" onclick="rr_deselectAll();" />',
				$this->langIf->getText('Select All'),
				$this->langIf->getText('Deselect All'));

		$this->script = "
			function rr_selectAll()
			{
				els = \$\$s('checkboxes[]');
				for(var i = 0;i<els.length;i++)
				{
					els[i].checked = 'checked';
				}
			}
			function rr_deselectAll()
			{
				els = \$\$s('checkboxes[]');
				for(var i = 0;i<els.length;i++)
				{
					els[i].checked = false;
				}
			}
			";

		if($this->if_obj->model['rrc']->id)
		{
			$this->if_obj->model['rrc']->set();
			$this->if_obj->model['rrc']->document->set();

			list($trans,$class ) = $this->if_obj->model['rrc']->translateStatus($this->langIf,$this->if_obj->model['rrc']->review_status);

			$this->text .= sprintf('<br /><br />%s<br /><a href="/%sscripts/i-w/document/download.php?Document[id]=%d">%s</a> - <span class="%s">%s</span><br />',
				$this->langIf->getText('Your Document has been filed as '),
				SITE_PATH,
				$this->if_obj->model['rrc']->document->id,
				$this->if_obj->model['rrc']->document->name,
				$class,$trans);
		}
    }

    function _tfValueProcessor($args)
    {
    	switch ($args['head'])
    	{
    		// Override checkbox html. Checkbox is selected if a REFUND_REQUEST_CONFIRMATION_ID isset
    		case 'AUTHORIZE_CHECKBOX':

    			$selected = !empty($args['row']['REFUND_REQUEST_CONFIRMATION_ID']) ||
    				(isset($_REQUEST['process']) && isset($_REQUEST['checkboxes']) && $this->if_obj->_process == 'print' &&
    					in_array($args['row'][$this->primary_key], $_REQUEST['checkboxes']));

    			$chk = new Form_Checkbox('checkboxes[]', $args['row'][$this->primary_key], $selected);
    			$chk->setID('authorize_'.$args['i']);
    			$chk->setBehaviour('required');
    			return $chk->getHtml();
    		break;

    		case 'REFUND_REQUEST_ID':
    			if ($this->if_obj->_process == 'print')
    				return $args['value'];
    		break;
    	}

    	return parent::_tfValueProcessor($args);
    }

    function _tfRowProcessor($args)
    {
    	/**
    	 * Hide this row if we are in print preview and the current checkbox is set in $_POST
    	 */
    	$row = &$args['row'];
    	if ($this->if_obj->_process == 'print' && (!isset($_REQUEST['checkboxes']) || !in_array($row[$this->primary_key], $_REQUEST['checkboxes'])))
    		return false;
    }

    function getButtons()
    {
    	// dont show buttons if no crso/part are selected
    	if (!isset($this->if_obj->model['crso']) || !isset($this->if_obj->model['participant'])) {
    		$this->_BarButtons = 0;
    		return;
    	}

		// print button if in print preview
		if ($this->if_obj->_process == 'print')
		{
			$this->_BarButtons = 0;
			$this->_addCustomButton($this->langIf->getText('Print'), 'button', array('onclick'=>"print();"));
			$this->_addCustomButton($this->langIf->getText('Close Window'), 'button', array('onclick'=>"window.close();"));
			return;
		}

    	//we need this to get some javascript, but we will overwite the custom buttons
    	parent::getButtons();
    	$this->_CustomButtons = array();

    	preg_match('/\/scripts\/(.*)\/(.*)\/(Interface[\w]+)\.php/', $_SERVER['REQUEST_URI'],$matches);

    	// normal buttons (print preview and defau)
    	$this->_addCustomButton($this->langIf->getText('Print Preview'), 'button', array('onclick'=>"print_preview();"));
    	$this->script .= "
			function print_preview()
			{
				if (!has_selected_checkboxes())
					return;

				var nwin = new NewWindow(800, 600);
				nwin.setSource('about:blank');
				nwin.setName('winPrint');
				nwin.setScrollbars(true);
				nwin.center_me = false
				var win = nwin.open(); // returns a window object

				var frm = $('interface_form');
				frm.target='winPrint';
				frm.process.value='print';
				frm.submit();

				// restore
				frm.target='_self';
				frm.process.value='save';
			}
		";

    	// authorize buttons
    	$this->_addCustomButton($this->langIf->getText('Confirm '.$this->if_obj->objectClass), 'button',
    		array('onclick'=>jsMessageBox(MSG_BOX_QUEST,
    				$this->langIf->getText('Confirm selected'),
    				$this->langIf->getText('Are you sure you want to confirm the selected?'),
					"function() { $('interface_form').process.value='authorize'; if (has_selected_checkboxes() && fb.validate($('interface_form'))) { $('interface_form').submit();}  }")
				)
		);
    	$this->_addCustomButton($this->langIf->getText('Reject '.$this->if_obj->objectClass), 'button',
    		array('onclick'=>jsMessageBox(MSG_BOX_QUEST,
    				$this->langIf->getText('Deny Selected'),
    				$this->langIf->getText('Are you sure you want to deny the selected?'),
					"function() { $('interface_form').process.value='deny'; if (has_selected_checkboxes()) $('interface_form').submit(); }")
				)
    	);


    	//here we attach form watch behaviour to force new document id generation if checkboxes change
    	$this->script .= '

			function doCheckboxChange()
			{
				$("ifI_935_CB_1").disabled = "disabled";
				addEvent($("ifI_935_CB_0"), "click", function(){
					$("ifI_935_CB_1").disabled = "";
				});
			}

			addEvent(window, "load", function(){
				checkEls = document.getElementsByName("checkboxes[]");
				checkIds = []
				for (var i=0; i <checkEls.length; i++)
				{
					checkIds.push(checkEls[i].id);
				}

				ifHandler.watchFormOnChange("interface_form",
				doCheckboxChange,
				checkIds);
			});

		';
    }
}



?>
