<?php
/**
 * Created on Jan 7, 2008
 *
 * @author Nicoleta Militaru <nicoleta@basesoft.com>
 **/

require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'uk/UKCRSOData.php');
require_once(ELC_CLASSES.'uk/UKCRSODataLine.php');
require_once(ELC_CLASSES.'uk/UKCRSOSupplier.php');

define('ReuseBlock',496);
define('Reg32Block',557);
define('Reg39Block',558);

class InterfaceCrsoEAData extends InterfaceFactory
{

 /** interface[id] = 183
 * @param $db Database (I_WARE_SETTINGS)
 * @param $db_app Database (APPLICATION DATABASE)
 * @param $langIf LanguageInterface
 */
	function InterfaceCrsoEAData(&$db, &$db_app, &$langIf)
	{
		parent::InterfaceFactory($db, $db_app, $langIf);
		$this->FirstColWidth = 25;
		$this->delimiter = '<br />';

		$this->hide = false;

		$this->tonnes = array();

		$this->model['data'] = new UKCRSOData($this->db);
		$this->model['data_line'] = new UKCRSODataLine($this->db);
		$this->model['supplier'] = new UKCRSOSupplier($this->db);
		$this->suppliers = array();

		$this->model['years'] = array();
		$this->model['quarters'] = array();
		$this->model['crso'] = false;
		$this->model['year'] = false;
		$this->model['quarter'] = false;

		if(($_SESSION['session_group_group_id'] == GROUP_ACCOUNTANT || $_SESSION['session_group_group_id'] == GROUP_SYSADMIN) &&
			isset($_REQUEST['crso']['id']))
			$this->model['crso'] = new CRSO($this->db,array('relation'=>new Relation($this->db,$_REQUEST['crso']['id'])));
		if($_SESSION['session_group_group_id'] == GROUP_CRSO)
			$this->model['crso'] = new CRSO($this->db,array('relation'=>new Relation($this->db,$_SESSION['crso_relation_id'])));

		$this->new_schema = true;
		$this->has_all_suppliers = true;
		if($this->model['crso'])
		{
			$q = $this->model['supplier']->selectOverview($this->model['crso']->relation->id, false);
			while($r = $q->fetch())
				$this->suppliers[$r->TYPE][] = array('supplier_id'=>$r->SUPPLIER_ID, 'approval_no'=>$r->APPROVAL_NO,
													 'name'=>$r->NAME, 'active' => $r->ACTIVE);

			if(count($this->suppliers) == 0 || !isset($this->suppliers[AE]) || !isset($this->suppliers[ATF]))
		 		$this->has_all_suppliers = false;

			$this->model['years'] = array();
			$this->model['years'] = $this->model['data']->getYears($this->model['crso']->relation->id);

			$this->model['year'] = (isset($_REQUEST['year'])&&in_array($_REQUEST['year'],$this->model['years']))?$_REQUEST['year']:
							((count($this->model['years'])==1)?current($this->model['years']):false);
			if($this->model['year'])
			{
				if($this->model['year'] < 2010)
					$this->new_schema = false;
				$this->model['quarters'] = array();
				$this->model['quarters'] = $this->model['data']->getQuarters($this->model['crso']->relation->id,$this->model['year']);

	 			$this->model['quarter'] = (isset($_REQUEST['quarter'])&&in_array($_REQUEST['quarter'],$this->model['quarters']))?$_REQUEST['quarter']:
							((count($this->model['quarters'])==1)?current($this->model['quarters']):false);
				if($this->model['quarter'])
	 			{
	 				if($this->model['year'] == 2010 && $this->model['quarter'] != 4)
						$this->new_schema = false;

					try {
						$this->model['data']->set($this->model['crso']->relation->id, $this->model['year'], $this->model['quarter']);
						$this->_getLines($this->model['data']->id);
					} catch (UKNoDataException $e) {
						// pass
					}

				}
			}
		}

		if(!$this->model['crso'] || !$this->model['year'] || !$this->model['quarter'])
			$this->hide = true;

		$this->script = "
			function reloadOverview(sel)
			{";
		if($_SESSION['session_group_group_id'] == GROUP_CRSO)
			$this->script .= "go('InterfaceCrsoEAData.php?year='+sel.options[sel.selectedIndex].value);
								}";
		else
			$this->script .= "
					if($('ifI_1436') == undefined)
						selectedCRSO = $$('crso[id]').value;
					else
					{
						crso = $('ifI_1436');
						selectedCRSO = crso.options[crso.selectedIndex].value;
					}
					if(selectedCRSO)
						go('InterfaceCrsoEAData.php?crso[id]='+selectedCRSO+'&year='+sel.options[sel.selectedIndex].value);
					}
					";
		$this->script .= "
			function reloadOverview2(sel)
			{
				if($('ifI_1434') == undefined)
					selectedYear = $$('year').value;
				else
				{
					year = $('ifI_1434');
					selectedYear = year.options[year.selectedIndex].value;
				}
			";
		if($_SESSION['session_group_group_id'] == GROUP_CRSO)
			$this->script .= "go('InterfaceCrsoEAData.php?year='+selectedYear+'&quarter='+sel.options[sel.selectedIndex].value);
								}";
		else
			$this->script .= "
				if($('ifI_1436') == undefined)
					selectedCRSO = $$('crso[id]').value;
				else
				{
					crso = $('ifI_1436');
					selectedCRSO = crso.options[crso.selectedIndex].value;
				}
				if(selectedCRSO)
					go('InterfaceCrsoEAData.php?crso[id]='+selectedCRSO+'&year='+selectedYear+'&quarter='+sel.options[sel.selectedIndex].value);
				}";
		$this->script .= "
			function doSave()
			{
				if(fb.validate($('interface_form')))
				{
					$('id_mod').style.visibility = 'hidden';
					$$('process').value = 'save';
					$('interface_form').submit();
				}
			}
			function disablePublish()
			{
				$('ifI_1433_CB_3').disabled = true;
				$('id_mod').style.visibility = 'visible';
			}";
		$this->script .= sprintf("
			function doPublish()
			{
				MessageBox.prototype.Question('%s','%s',
												function()
												{
												  $$('process').value = 'publish';
												  $('interface_form').submit();
												}
											);
			}",
		$this->langIf->getText('Publish Data'),
		$this->langIf->getText('This information will be included in the next Actuals Report and you will not have the possibility to changed it anymore. Is all data correct?'));
		if($this->new_schema && !$this->has_all_suppliers)
			$this->script .= sprintf("MessageBox.prototype.Question('%s','%s',
												function()
												{
												  window.location = '/scripts/elc/crso/InterfaceCrsoEASuppliers.php';
												}
											);",
			$this->langIf->getText('No suppliers set for ATF and/or AE data'),
			$this->langIf->getText('You cannot save recycle data without having those suppliers. Do you wish to set suppliers?'));
 	}

	function _getLines($data_id)
	{
		$q = $this->model['data_line']->selectOverview($data_id);

		while($r = $q->fetch())
		{
			$sid = 'ALL';
			if($r->SUPPLIER_ID)
				$sid = 	$r->SUPPLIER_ID;
			$this->tonnes[$r->TYPE][$sid][$r->CATEGORY_NAME][$r->OBLIGATION_TYPE] = $r->TONNES;
		}
	}

	function processInput(&$thrownException)
	{
		if(isset($_POST['process']))
		{
			if($_SESSION['session_group_group_id'] != GROUP_CRSO)
				return;
			if($_POST['process'] == 'save')
			{
				if(!$this->fb->validate())
					return $thrownException .= $this->langIf->getText('Please correct the form errors');

				try {
					$this->model['data']->set($this->model['crso']->relation->id,$this->model['year'],$this->model['quarter']);
					$this->model['data_line']->updateLines($this->model['data']->id,$_POST['tonnes']);
				} catch (UKNoDataException $e) {

					$this->model['data']->crso_id = $this->model['crso']->relation->id;
					$this->model['data']->year = $this->model['year'];
					$this->model['data']->quarter = $this->model['quarter'];
					$this->model['data']->insert();

					$this->model['data']->set($this->model['crso']->relation->id,
								$this->model['year'],
								$this->model['quarter']);
					$this->model['data_line']->insertLines($this->model['data']->id,$_POST['tonnes']);
				}
			}
			$this->_getLines($this->model['data']->id);
			$this->init(183);
 		}
 	}
	function init($if_id)
	{
	  	parent::init($if_id);

		// schema changed from v1.3.1 to v2.0 starting from 2010, Q4
		// so blocks depend on that

	  	if($this->new_schema)
			unset($this->blocks[ReuseBlock]);
		else {
		   	unset($this->blocks[Reg32Block]);
		   	unset($this->blocks[Reg39Block]);
		}
	}
 }

 /** * @package
  * @subpackage Interface
  * @since 07-01-2008
  *  @copyright 2008 BaseSoft B.V. *
  * @version $Id$
  */

  class InterfaceCrsoEADataItem extends InterfaceItem
  {

  function getSimpleOverview($type)
   {

	   $tonnes = array();

		if(count($this->if_obj->tonnes)>0) {
			$tonnes = $this->if_obj->tonnes;
		}

		for ($category_i=1; $category_i<=UK_EA_CATEGORY_LEN; $category_i++) {

			for ($oblig_i = 1; $oblig_i<=UK_EA_OBLIG_LEN; $oblig_i++) {

				if (!isset($tonnes[$type]['ALL'][$category_i][$oblig_i]))
					$tonnes[$type]['ALL'][$category_i][$oblig_i] = ($this->if_obj->hide) ? '' : '0.000';

			}
		}

		$rv = '<table width="85%">';

		$rv .= '
				<tr>
					<td>&nbsp;</td><td align="right"><b>'.$this->if_obj->model['data_line']->translateObligationType(1).'</b></td>';

	   // #2644: dont show b2b for these
	   if ($type != REG32 && $type != REG39) {
		   $rv .= '
					<td align="right"><b>' . $this->if_obj->model['data_line']->translateObligationType(2) . '</b></td><td>&nbsp</td>';
	   }

	   $rv .='
				</tr>';

		for ($category_i=1; $category_i<=UK_EA_CATEGORY_LEN; $category_i++) {

			$rv .= '<tr>
						<td width="40%">' . $this->if_obj->model['data_line']->translateCategoryName($category_i).'</td>';

			for ($oblig_i = 1; $oblig_i<=UK_EA_OBLIG_LEN; $oblig_i++) {

				// #2644: dont show b2b for these
				if ( ($type == REG32 || $type == REG39) && $oblig_i == 2 )
					continue;

				$rv .= '<td width="20%" align="right">';

				$rv .= (!$this->if_obj->hide) ? '<input disabled="disabled" type=text name="tonnes['.$type.'][ALL]['.$category_i.']['.$oblig_i.']" behaviour="required;decimal;"
					onChange="disablePublish();" value="' . $tonnes[$type]['ALL'][$category_i][$oblig_i] . '" style="text-align:right">' :
					$tonnes[$type]['ALL'][$category_i][$oblig_i];
				$rv .= '</td>';
			}

			$rv .= '<td align="center">Tonnes</td></tr>';
			$rv .='</tr>';

	   }

		$rv .= '</table>';
   		return $rv;
   }

   function getComplexOverview($type)
   {

   		$tonnes = array();
   		if(count($this->if_obj->tonnes)>0){

   			$tonnes = $this->if_obj->tonnes;

			// new supplier might have been added
			if($this->if_obj->new_schema && isset($this->if_obj->suppliers[$type]) && count($this->if_obj->suppliers[$type]) > count($tonnes[$type])) {

				foreach($this->if_obj->suppliers[$type] as $v) {

					if (!isset($tonnes[$type][$v['supplier_id']])) {
						for ($category_i=1; $category_i<=UK_EA_CATEGORY_LEN; $category_i++) {
							for ($oblig_i = 1; $oblig_i<=UK_EA_OBLIG_LEN; $oblig_i++) {
								$tonnes[$type][$v['supplier_id']][$category_i][$oblig_i] = (!$this->if_obj->hide) ? '0.000' : ' ';
							}
						}

					}
				}
   			}
   		}
   		else
   		{
   			$supl = array(array('supplier_id'=>'ALL'));
   			if(isset($this->if_obj->suppliers[$type]))
   				$supl = $this->if_obj->suppliers[$type];
   			foreach($supl as $v){

				for ($category_i=1; $category_i<=UK_EA_CATEGORY_LEN; $category_i++) {
					for ($oblig_i = 1; $oblig_i <= UK_EA_OBLIG_LEN; $oblig_i++) {
						$tonnes[$type][$v['supplier_id']][$category_i][$oblig_i] = (!$this->if_obj->hide)?'0.000':' ';
					}
				}

   			}
   		}

   		if(!isset($tonnes[$type]))
   			return;

   		$supplier_info = array();
   		if(isset($this->if_obj->suppliers[$type])){
   			foreach($this->if_obj->suppliers[$type] as $v)
   				$supplier_info[$v['supplier_id']] = $v;
   		}

		$rv = '<table width="85%">
				<tr>
					<td>&nbsp;</td><td align="right"><b>'.$this->if_obj->model['data_line']->translateObligationType(1).'</b></td>
					<td align="right"><b>'.$this->if_obj->model['data_line']->translateObligationType(2).'</b></td><td>&nbsp</td>
				</tr>';

		foreach($tonnes[$type] as $supplier=>$v){


			// by default show supplier, unless the supplier is not active anymore _or_ has data (previous reports while he was still active)
			$show_supplier = true;

			if (!$supplier_info[$supplier]['active']) {

				$show_supplier = false;

				for ($category_i=1; $category_i<=UK_EA_CATEGORY_LEN; $category_i++) {
					for ($oblig_i = 1; $oblig_i<=UK_EA_OBLIG_LEN; $oblig_i++) {
						if (isset($tonnes[$type][$supplier][$category_i][$oblig_i])) {
							$value = $tonnes[$type][$supplier][$category_i][$oblig_i];
							if ((float) $value > 0)
								$show_supplier = true;
						}
					}
				}
			}

			if (!$show_supplier)
				continue;

			$rv .='
				<tr>
					<td style="font-weight:bold; font-style:italic; padding-left:0px;">'.$supplier_info[$supplier]['name'].' ('.$supplier_info[$supplier]['approval_no'].'):</td>
					<td></td>
					<td></td>
					<td></td>
				</tr>
			';

			for ($category_i=1; $category_i<=UK_EA_CATEGORY_LEN; $category_i++) {

				$rv .= '<tr>
							<td width="40%">' . $this->if_obj->model['data_line']->translateCategoryName($category_i) . '</td>';

				for ($oblig_i = 1; $oblig_i<=UK_EA_OBLIG_LEN; $oblig_i++) {

					$value = isset($tonnes[$type][$supplier][$category_i][$oblig_i]) ? $tonnes[$type][$supplier][$category_i][$oblig_i] : '0.000';

					$rv .= ' <td width="20%" align="right">';
					$rv .= (!$this->if_obj->hide) ? '<input disabled="disabled" type=text name="tonnes[' . $type . '][' . $supplier . '][' . $category_i . ']['.$oblig_i.']" behaviour="required;decimal;"
							onChange="disablePublish();" value="' . $value . '" style="text-align:right">' :
						$value;
					$rv .= '</td>';
				}

				$rv .= '<td align="center">Tonnes</td></tr>';

			}
		}

		$rv .= '</table>';
		return $rv;
   }

  /**
   *  item[1416] = DCF
   *  item[1417] = ATF
   *  item[1418] = AE
   *  item[1419] = Reuse
   */
   function getOverviewDCF()
   {
   		$this->text = $this->getSimpleOverview(DCF);
   }
   function getOverviewATF()
   {
   		$this->text = $this->getComplexOverview(ATF);
   }
   function getOverviewAE()
   {
   		$this->text = $this->getComplexOverview(AE);
   }
   function getOverviewReuse()
   {
		$this->text = $this->getSimpleOverview(REUSE);
   }
  function getOverviewReg32()
   {
		$this->text = $this->getSimpleOverview(REG32);
   }
  function getOverviewReg39()
   {
		$this->text = $this->getSimpleOverview(REG39);
   }
   function getCRSOSelect()
    {
    	switch($_SESSION['session_group_group_id'])
		{
			case GROUP_CRSO:
				$filter = '';
				$selectedValue = '';
				$setValue = $_SESSION['crso_relation_id'];
				$rs = array();
				break;
			case GROUP_ACCOUNTANT:
				$sql = "SELECT R.RELATION_ID as CRSO_RELATION_ID, R.`NAME`
					FROM TBL_ACCOUNTANT_CRSO as AC
					INNER JOIN TBL_RELATION as R
						ON AC.CRSO_RELATION_ID = R.RELATION_ID
					INNER JOIN TBL_ADDRESS as A
						USING(ADDRESS_ID)
					WHERE A.COUNTRY_ID = 826
					AND AC.ACCOUNTANT_RELATION_ID = ".$_SESSION['accountant_relation_id']."
					ORDER BY NAME";
				$q = $this->db->query($sql);
				$q->execute();
				$rs = $q->fetchAll();
			break;
			case GROUP_SYSADMIN:
				$sql = "SELECT R.RELATION_ID as CRSO_RELATION_ID, R.`NAME`
					FROM TBL_CRSO as C
					INNER JOIN TBL_RELATION as R
						ON C.CRSO_RELATION_ID = R.RELATION_ID
					INNER JOIN TBL_ADDRESS as A
						USING(ADDRESS_ID)
					WHERE A.COUNTRY_ID = 826
					ORDER BY NAME";
				$q = $this->db->query($sql);
				$q->execute();
				$rs = $q->fetchAll();
			break;
		}

		$this->options = array(array('value'=>'','text'=>'- '.$this->langIf->getText('Select CRSO').' -'));
		if(count($rs) == 1)
		{
			$this->if_obj->model['crso'] = $rs[0]->CRSO_RELATION_ID;
			$name = $rs[0]->NAME;
			$name = (mb_strlen($name)>35)?mb_substr($name,0,35).'...':$name;
			$this->options[] = array('value'=>$rs[0]->CRSO_RELATION_ID,'text'=>$name,
				'selected'=>'selected');
			$this->if_obj->model['years'] = $this->if_obj->model['data']->getYears($rs[0]->CRSO_RELATION_ID);
		}
		else
		foreach($rs as $r)
    	{
    		$name = $r->NAME;
    		$name = (mb_strlen($name)>35)?mb_substr($name,0,35).'...':$name;
    		$this->options[] = array('value'=>$r->CRSO_RELATION_ID,'text'=>$name,
				'selected'=>(($this->if_obj->model['crso']&& $this->if_obj->model['crso']->relation->id==$r->CRSO_RELATION_ID)?'selected':false));
			$this->properties['onchange'] = "if(this.options[this.selectedIndex].value) go('InterfaceCrsoEAData.php?crso[id]='+this.options[this.selectedIndex].value);";
    	}
    }
   function getYearSelect()
    {
    	$this->options = array();
    	if(!$this->if_obj->model['crso'])
    	{
    		$this->options[] = array('value'=>'','text'=>'- '.$this->langIf->getText('Select CRSO First').' -');
    		return;
    	}

    	if(count($this->if_obj->model['years'])==1)
    	{
    		$this->_hideInput = true;
    		$this->options[] = array('value'=>$this->if_obj->model['year'],'text'=>$this->if_obj->model['year'],'selected'=>'selected');
    		return;
    	}
    	$this->options[] = array('value'=>'','text'=>'- '.$this->langIf->getText('Select Year').' -');
    	foreach($this->if_obj->model['years'] as $year)
    		$this->options[] = array('value'=>$year,'text'=>$year,'selected'=>(($this->if_obj->model['year']==$year)?'selected':false));

    	$this->properties['onchange'] = 'reloadOverview(this);';
    }
    function getQuarterSelect()
    {
    	$this->options = array();
    	if(!$this->if_obj->model['crso'])
    	{
    		$this->options[] = array('value'=>'','text'=>'- '.$this->langIf->getText('Select CRSO First').' -');
    		return;
    	}

    	if(!$this->if_obj->model['year'])
    	{
    		$this->options[] = array('value'=>'','text'=>'- '.$this->langIf->getText('Select Year First').' -');
    		return;
    	}

    	if(count($this->if_obj->model['quarters'])==1)
    	{
    		$this->_hideInput = true;
		$this->options[] = array('value'=>$this->if_obj->model['quarter'],'text'=>'Quarter '.$this->if_obj->model['quarter'],'selected'=>'selected');
		return;
	}
	$this->options[] = array('value'=>'','text'=>'- '.$this->langIf->getText('Select Quarter').' -');
	foreach($this->if_obj->model['quarters'] as $quarter)
		$this->options[] = array('value'=>$quarter,'text'=>"Quarter ".$quarter,'selected'=>(($this->if_obj->model['quarter']==$quarter)?'selected':false));

	$this->properties['onchange'] = 'reloadOverview2(this);';
    }

   function getButtons()
   {
		$this->_BarButtons = 0;
		$has_suppliers = true;
		if(count($this->if_obj->suppliers) == 0 || !isset($this->if_obj->suppliers[AE]) ||
		 !isset($this->if_obj->suppliers[ATF]))
		 	$has_suppliers = false;
		if(!$this->if_obj->hide && ($this->if_obj->has_all_suppliers || !$this->if_obj->new_schema))
		{
			$this->_CustomButtons[] = '<div style="float:left"><span class="error">* '.$this->langIf->getText('All fields are required').'</span></div>';
			$this->_CustomButtons[] = '<span id="id_mod" class="error" style="visibility:hidden;">'.
						$this->langIf->getText('Data was modified. You need to save it.').'...&nbsp;</span>';
			if($_SESSION['session_group_group_id'] == GROUP_CRSO)
			{
				$this->_addCustomButton($this->langIf->getText('Save'),'button',array('onclick'=>'doSave()', 'disabled'=>'disabled'));
			}
		}
   }

	  function getEnableEdit() {
		  $this->text = '<input type="button" id="btnEnableEdit" value=" '.$this->langIf->getText('Enable edit', false).' " onclick="enableEdit(this)" />

			<script type="text/javascript">
				window.enableEdit = function(button) {

					$("ifI_1433_CB_2").disabled = false;

					var inputs = document.getElementsByTagName("input");
					for (var i in inputs ) {

						var input = inputs[i];
						if (!input.getAttribute) {
							continue;
						}

						var name = input.getAttribute("name");
						if (!name || !name.match(/^tonnes\[/)) {
							continue;
						}

						input.disabled = false;
						button.disabled = true;

					}

				}
			</script>

		  ';
	  }
	  function getLastUpdated() {
		  //$this->twoCols = true;
		  $this->text = $this->langIf->getText('Last updated'). ': '. $this->if_obj->model['data']->last_update;
	  }
 }

 /** * CALLBACK[getOverview]
  *  item[1416] = {
  *      [handle] = Item: CollectedFromDCF
  *      [behaviour] =
  *      [help] = 0
  *      [close_row] = 1
  *      [label_pos] = 4
  *
  *  item[1417] = {

  *      [handle] = Item: DeliveredToATF
  *      [behaviour] =
  *      [help] = 0
  *      [close_row] = 1
  *      [label_pos] = 4
  *
  *  item[1418] = {

  *      [handle] = Item DeliveredToAE
  *      [behaviour] =
  *      [help] = 0
  *      [close_row] = 1
  *      [label_pos] = 4
  *
  *   item[1419] = {

  *      [handle] = Item: ForReuse
  *      [behaviour] =
  *      [help] = 0
  *      [close_row] = 1
  *      [label_pos] = 4
  *   */
?>
