<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'Country.php');
require_once(ELC_CLASSES.'Declaration.php');
require_once(ELC_CLASSES.'Relation.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @since 07-04-2006
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceDeclarationStatus extends InterfaceFactory
{
	var $participant = array('id'=>false);
	var $period = false;
	var $crso = array('id'=>false);
    /**
     * interface[id] = 93
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceDeclarationStatus(&$db, &$db_app, &$langIf, $settings=array())
    {
    	// check for rights
		require_once(IWARE . 'group.php');
		switch ($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
				// allow all
				break;

			// TODO: add access for Accountant?

			case GROUP_CRSO:
				// check session_crso_id == this->crso_id
				if ($settings['crso']['id'] == $_SESSION['crso_relation_id'])
					break;

			case GROUP_PARTICIPANT:
				// check crso in crso_participant
				// check participant in session, dont allow all participants
				//break;

			default:
				alert(sprintf('Access denied by user [%s] to view CRSO Declaration status', $_SESSION['session_user_id']));
				throw new InterfacePermissionException(sprintf('Access denied by user [%s] to view CRSO Declaration status', $_SESSION['session_user_id']));
		}

        parent::InterfaceFactory($db, $db_app, $langIf, $settings);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
    }
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 07-04-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceDeclarationStatusItem extends InterfaceItem
{
    /**
     * item[797] =
     */
    function getOverview()
    {
		$makeSelection = $this->langIf->getText('Please make a selection first');

		/*
		echo 'participant: ' . $this->if_obj->participant['id'] . ' <br />';
		echo 'crso: ' . $this->if_obj->crso['id'] . ' <br />';
		echo 'period: ' . $this->if_obj->period . ' <br />';
		*/

		$valueProcessor = 'tfValueProcessorBoolean';
		if(!$this->if_obj->period&&!$this->if_obj->participant['id']) {
			$valueProcessor = 'tfValueProcessorNumber';
			list ($title, $ds) = $this->_getParticipantOverview();
		}
		else if($this->if_obj->period && !$this->if_obj->participant['id'] )
			list($title, $ds) = $this->_getPeriodOverview();
		else if(!$this->if_obj->period && $this->if_obj->participant['id'] )
			list($title, $ds) = $this->_getParticipantOverview(
				array('D.PARTICIPANT_RELATION_ID'=>$this->if_obj->participant['id']));
		else
			list($title, $ds) = $this->_getParticipantOverview(
				array('D.PARTICIPANT_RELATION_ID'=>$this->if_obj->participant['id'],
					'D.`MONTH`'=>$this->if_obj->period));

		//$hideColumns = array('AGGREGATED_STATUS');

		// set Country Settings for CRSO.
		$crso = new Relation($this->db_app,$this->if_obj->crso['id']);
		$country = new Country($this->db_app,$crso->getCountry(),true);

		$hideColumns = array();
		if(!$country->has(HasCrsoDeclarationDetails,$_SESSION['selected_product_id'])) {
			$hideColumns[] = 'AGGREGATED_QUANTITY';
    	} elseif($_SESSION['session_group_group_id']==GROUP_CRSO) {
			// Check for CRSO universe. Set the Relation User and do a clearance level check.
    		require_once(ELC_CLASSES.'CRSO.php');
			require_once(ELC_CLASSES.'RelationUser.php');
			$ru = new RelationUser($this->db_app);
			if(!$ru->hasClearance(CRSOClearanceDeclarationDetails, $_SESSION['session_user_id'], $_SESSION['crso_relation_id'])) {
				$hideColumns[] = 'AGGREGATED_QUANTITY';
			}
		}

		require_once(IWARE.'TableFactoryColumnPlugin.php');
		$tf = new TableFactoryColumnPlugin(false,$this->langIf,$this->db,$this->db_app);

		$tf->setEmptyText($this->langIf->getText('No declarations found for set filter') );
		$tf->setData($ds, 'DeclarationStatus',false,true);

		$tf->setSortingFunctions(array(
			'PARTICIPANT_NAME'=>"'String'",
			'AGGREGATED_QUANTITY'=>"'SortVal'",
			'DECLARATION_STARTED'=>"'String'",
			'DECLARATION_CLOSED'=>"'String'",
			'DECLARATION_RESUBMIT'=>"'String'",
			'DECLARATION_FORFEIT'=>"'String'",
			'DECLARATION_LATE'=>"'String'"
			));

		$help = ($this->parent->title != '') ? getHelp($this->langIf, $this->parent->handle, array('title'=>$this->parent->handle, 'top'=>'0px')) : '';
		$tf->setCaption($help.'<img src="/'.SITE_PATH.'images/block_arrow.png" /> '.$this->langIf->getText('Declaration Status Overview').' - '.$title, false);
		$tf->setValueProcessor( array('DECLARATION_STARTED', 'DECLARATION_CLOSED', 'DECLARATION_RESUBMIT',
			'DECLARATION_FORFEIT', 'DECLARATION_LATE'), array($this, $valueProcessor));

		$tf->setValueProcessor('AGGREGATED_QUANTITY', array($this, 'tfValueProcessorQuantity'));
		$tf->columnModifier(array('AGGREGATED_QUANTITY'=>'style="text-align:right;"'));

		$tf->setValueProcessor('MONTH', array($this, 'tfMonth'));

		$tf->hideColumns($hideColumns);
		$this->text = $tf->getXHTMLTable('Professional',2);
    }

    function tfValueProcessorBoolean($args)
    {
    	$img = ($args['value']) ? 'true.gif' : 'false.gif';
    	return '<img src="/'.SITE_PATH.'images/i-w/'.$img.'" />';
    }

    function tfValueProcessorNumber($args)
    {
    	if ($args['value'] < 3)
    		return $this->tfValueProcessorBoolean($args);

    	return $args['value'];
    }

    function tfValueProcessorQuantity($args)
    {
    	$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
    	return locale_nf($args['value'], 2);
    }

	function tfMonth($args)
	{
		return sprintf('<a href="InterfaceCRSODeclarationStatus.php?period=%s">%s</a>',
			$args['value'], strftime('%B', mktime(1, 1, 1, $args['value'], 1) ));
	}


	/**@#+
	 * @access private
	 */
	/**
	 * @return array $rv
	 * @todo move SQL to some Object
	 * @todo trustee
	 */
	function _execQuery($params=array())
	{
		$date =& currentDate();
		$year = $date->getYear();

		/*
		define('DECLARATION_OPEN',1);
		define('DECLARATION_NORMAL',2);
		define('DECLARATION_FORFEIT',4);
		define('DECLARATION_LATE',8);
		define('DECLARATION_DECLARED',16);
		define('DECLARATION_RESUBMIT',32);
		*/

		$sql = "SELECT
				D.`MONTH`,
				D.PARTICIPANT_RELATION_ID,
				R.NAME AS PARTICIPANT_NAME,
				IF(BIT_OR(DL.STATUS) & (".DECLARATION_DECLARED." | ".DECLARATION_NORMAL." | ".DECLARATION_LATE." | ".DECLARATION_RESUBMIT.") AND (SUM(DL.QUANTITY) > 0), 1, 0) as DECLARATION_STARTED,
				IF(BIT_OR(DL.STATUS) & (".DECLARATION_NORMAL." | ".DECLARATION_LATE." | ".DECLARATION_RESUBMIT.") AND (SUM(DL.QUANTITY) > 0), 1, 0) as DECLARATION_CLOSED,
				IF(BIT_OR(DL.STATUS) & ".DECLARATION_RESUBMIT." = ".DECLARATION_RESUBMIT.", 1, 0) as DECLARATION_RESUBMIT,
				IF(BIT_OR(DL.STATUS) & ".DECLARATION_FORFEIT." = ".DECLARATION_FORFEIT.", 1, 0) as DECLARATION_FORFEIT,
				IF(BIT_OR(DL.STATUS) & ".DECLARATION_LATE." = ".DECLARATION_LATE.", 1, 0) as DECLARATION_LATE
				,SUM(DL.QUANTITY) as AGGREGATED_QUANTITY
				-- visible fee invoiced: normal + resubmit + late
				-- forfeit invoiced: sum(forfeit)
				-- settled: visble + forfeit
				FROM TBL_DECLARATION as D
				INNER JOIN TBL_DECLARATION_LINE as DL
					ON D.DECLARATION_ID=DL.DECLARATION_ID
				INNER JOIN TBL_RELATION R
					ON D.PARTICIPANT_RELATION_ID=R.RELATION_ID
				WHERE D.CRSO_RELATION_ID = ?
				";

		$values = array($this->if_obj->crso['id']);
		foreach ($params as $key=>$value)
		{
			$sql .= " AND ".$key."=? ";
			$values[] = $value;
		}

		$sql.= "GROUP BY D.`DECLARATION_ID`
			ORDER BY AGGREGATED_QUANTITY DESC
		";

		$q = $this->db_app->query($sql);
		$q->bind_values($values);
		$q->execute();

		return $q;
	}

	function _getParticipantOverview($arrWhere=array())
	{

		// participant selected, show all months
		$q = $this->_execQuery($arrWhere);
		$ds = array();

		while ($row = $q->fetch())
		{
			foreach ($row as $column=>$value)
			{
				if ($column == 'MONTH')
					$ds[ $row->{'MONTH'} ][$column] = $value;

				if (in_array($column, array('PARTICIPANT_NAME', 'PARTICIPANT_RELATION_ID', 'MONTH')))
					continue;

				if (!isset($ds[ $row->{'MONTH'} ][$column]))
					$ds[ $row->{'MONTH'} ][$column] = 0;

				$ds[ $row->{'MONTH'} ][$column] += $value;
			}
		}

		$name = $this->langIf->getText('All participants');
		if (isset($arrWhere['D.PARTICIPANT_RELATION_ID']))
		{
			$p = new Relation($this->db_app);
			$p->set($arrWhere['D.PARTICIPANT_RELATION_ID']);
			$name = $p->name;
		}

		return array($name, array_values($ds));
	}

	function _getPeriodOverview()
	{
		// month selected, show all participants
		$q = $this->_execQuery(array('D.`MONTH`'=>$this->if_obj->period));
		$ds = array();

		while ($row = $q->fetch())
		{
			foreach ($row as $column=>$value)
			{
				if ($column == 'PARTICIPANT_NAME')
					$ds[ $row->{'PARTICIPANT_RELATION_ID'} ][$column] = $value;

				if (in_array($column, array('PARTICIPANT_NAME', 'PARTICIPANT_RELATION_ID', 'MONTH')))
					continue;

				if (!isset($ds[ $row->{'PARTICIPANT_RELATION_ID'} ][$column]))
					$ds[ $row->{'PARTICIPANT_RELATION_ID'} ][$column] = 0;

				$ds[ $row->{'PARTICIPANT_RELATION_ID'} ][$column] += $value;
			}
		}

		return array(strftime('%B', mktime(1, 1, 1, $this->if_obj->period, 1) ), array_values($ds));
	}

}

/**
 * CALLBACK[getOverview]
 * item[797] = {
 *     [handle] = Item: DeclarationStatusOverview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
