<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'CRSOProductDimension.php');
/**
 *
 * @package
 * @subpackage Interface
 * @since 11-04-2007
 * @copyright 2007 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceDimensionManagement extends InterfaceFactory
{
    /**
     * interface[id] = 169
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceDimensionManagement(&$db, &$db_app, &$langIf)
    {


        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

		$this->model['crso'] = new CRSO($this->db_app);

		if($_SESSION['session_group_group_id']==GROUP_SYSADMIN)
		{

			if(isset($_REQUEST['crso']['id']))
			{
				$this->model['crso']->set($_REQUEST['crso']['id']);
				$this->model['cpd'] = new CRSOProductDimension($this->db_app,
					array('product'=> new Product($this->db_app,$_SESSION['selected_product_id']),
						'crso'=>$this->model['crso']));
			}

		} else if($_SESSION['session_group_group_id']==GROUP_CRSO) {

			$this->model['crso']->set($_SESSION['crso_relation_id']);

			$this->model['cpd'] = new CRSOProductDimension($this->db_app,
				array('product'=> new Product($this->db_app,$_SESSION['selected_product_id']),
					'crso'=>$this->model['crso']));
		} else {
			throw new Exception('Permission denied');
		}


		if(isset($_REQUEST['cpd']['id']))
		{
			$this->model['cpd']->set($_REQUEST['cpd']['id']);
			if($this->model['cpd']->crso->relation->id!=$_REQUEST['crso']['id'])
				exit('CRSO-Dimension Mismatch');
		}


        $this->script = sprintf("
			function confirmDimensionDelete()
			{
				MessageBox.prototype.Question('%s','%s',function(){
					$$('process').value = 'delete';
					$('interface_form').submit();
					//alert('te');
				});
			}
			",
			$this->langIf->getText('MessageBox: Confirm Delete'),
			$this->langIf->getText('MessageBox Content: Confirm Delete Dimension')
			);

    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        if(!isset($_REQUEST['process']))
        	return;

		if(isset($_REQUEST['process']) && $_REQUEST['process'] == 'disable')
		{
			$cpd = new CRSOProductDimension($this->db_app);
			$cpd->product = new Product($this->db_app, $_SESSION['selected_product_id']);
			$cpd->crso = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, $this->model['cpd']->crso->relation->id)));
			$cpd->disableDimension($_REQUEST['cpd']['id']);

			header(sprintf('location: InterfaceDimensionManagement.php'));
			exit();
		}

        if($_POST['process']=='save')
        {
	        $this->model['cpd']->name = $_POST['dimension']['name'];
	        $this->model['cpd']->insert();
	        header('Location: InterfaceDimensionManagement.php?crso[id]='.$this->model['crso']->relation->id);
	        exit();
	    }
	    elseif($_POST['process']=='delete')
	    {
	    	exit('This command is not implemented.');
	    }
    }
}
/**
 * @package
 * @subpackage Interface
 * @since 11-04-2007
 * @copyright 2007 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceDimensionManagementItem extends InterfaceItem
{

    /**
     * item[1317] = crso[id]
     */
    function getCRSOSelect()
    {

    	// get all the CRSOs for the selected_product

    	$this->options = array(array('text'=>'- '.$this->langIf->getText('Select CRSO').' -','value'=>''));
    	$crsos = $this->if_obj->model['crso']->selectCRSOs(1,'',$_SESSION['selected_product_id']);
    	while($r = $crsos->fetch())
    	{
    		$selected = $this->if_obj->model['crso']->isComplete()&&($r->CRSO_RELATION_ID == $this->if_obj->model['crso']->relation->id);
    		$this->options[] = array('text'=>$r->NAME,'value'=>$r->CRSO_RELATION_ID,'selected'=>$selected);
    	}

        $this->properties['onchange'] = "_LoadOverview();";
	    $this->postscript = "
		    function _LoadOverview()
		    {
		        var params = new Array();
		        params['crso'] = new Array();
				params['crso']['id'] = getElByName('crso[id]').value;
		        params['interface_id'] = ".$this->if_obj->id."; // you MUST supply this
		        ifHandler.loadBlock( new Array(445,446,447), params );
		    }";
    }

    /**
     * item[1318] =
     */
    function getDimensionOverview()
    {
    	if(!$this->if_obj->model['crso']->isComplete())
    		return;
    	$ds = $this->if_obj->model['cpd']->selectOverview($_SESSION['selected_product_id'],$this->if_obj->model['crso']->relation->id);
    	$tf = new TableFactory($ds,IWRegistry::registry('langIf'));
    	$tf->setValueProcessor(TF_ALL,array($this,'tfValueProcessor'));

		if($_SESSION['session_group_group_id']==GROUP_CRSO)
			$tf->hideColumns(array('DISABLE_DIMENSION'));

    	$this->text = $tf->getXHTMLTable('Professional',2);

    }

	function tfValueProcessor($args)
	{

		// crso is view only, so just return the values as they are (no links).. #2538
		if($_SESSION['session_group_group_id']==GROUP_CRSO)
			return $args['value'];

		switch($args['head'])
		{
			case 'CPD_ID':
			case 'DIMENSION_NAME':
			case 'DIMENSION_NAME':

				return sprintf('<a href="InterfaceDimensionManagement.php?crso[id]=%s&cpd[id]=%s">%s</a>',
					$this->if_obj->model['crso']->relation->id,
					$args['row']['CPD_ID'],
					$args['value']
					);
			break;
			case 'DISABLE_DIMENSION':

				$end_year = $args['value'];

				$confirm_msg = htmlspecialchars($this->langIf->getText('Are you sure you want to disable the dimension? This will set the end year to the next declaration year and will remove this dimension from any provisional declaration for that year.'));

				if ($end_year) {
					return $this->langIf->getText('Disabled');
				} else {
					$url = '?crso[id]='.$this->if_obj->model['crso']->relation->id.'&cpd[id]='.$args['row']['CPD_ID'].'&process=disable';
					return sprintf('<a href="%s" onclick="return confirm(\'%s\')">%s</a>', $url, $confirm_msg,  $this->langIf->getText('Disable'));
				}

				break;
		}
		return $args['value'];
	}

    /**
     * item[1319] = dimension[name]
     */
    function getDimensionName()
    {
    	if(!$this->if_obj->model['crso']->isComplete())
    	{
    		$this->properties['disabled'] = 'disabled';
    		return;
    	}
    	if($this->if_obj->model['cpd']->isComplete())
    	{
    		$this->hiddenFields = array(array('name'=>'cpd[id]','value'=>$this->if_obj->model['cpd']->id));
    		$this->properties['value'] = $this->if_obj->model['cpd']->name;
    	}
    }

    /**
     * item[1320] =
     */
    function getButtons()
    {
    	if(!$this->if_obj->model['crso']->isComplete())
    		return $this->_BarButtons = 0;
    	if($this->if_obj->model['cpd']->isComplete())
    		$this->_BarButtons = BUTTON_SAVE+BUTTON_DELETE;
    	else
    		$this->_BarButtons = BUTTON_SAVE;

    	$this->_ButtonOptions[BUTTON_DELETE] = array('onclick'=>'confirmDimensionDelete();');
    }

}

/**
 * CALLBACK[getCRSOSelect]
 * item[1317] = {
 *     [handle] = Item: CRSO Selection
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getDimensionOverview]
 * item[1318] = {
 *     [handle] = Item: DimensionOverview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getDimensionName]
 * item[1319] = {
 *     [handle] = Item: DimensionName
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[1320] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>