<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'ELCMessageHandler.php');
require_once(ELC_CLASSES.'PwcImporter.php');

/**
 *
 * @package WEEEImporter
 * @subpackage Interface
 * @since 05-12-2007
 * @author Marten Koedam <marten@dalines.net>
 * @copyright 2007 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceImporter extends InterfaceFactory
{
	public $model = array();
    /**
     * interface[id] = 182
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceImporter(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        //these are the classes that will be used for import
        $this->model['imports'] = array(
		'PwcImporter'=>'Decl. and Refund Inv. and Cr. Notes',
		'WEEEImporterReminder'=>'Reminder Notes',
		'WEEEImporterGuaranteeAndLate'=>'Guarantees and Lates',
                'WEEEImporterAgeing'=>'Ageing Information'
		);

        $this->script = "
		function reloadBlocks()
		    {
		        var params = new Array();
		        params['import'] = new Array();
				params['import']['type'] = getElByName('import[type]').value;
		        params['interface_id'] = 182; // you MUST supply this
		        ifHandler.loadBlock( new Array(491,492), params );
		    }";

		$this->model['lock'] = $this->model['WEEEImporter'] = false;
		if(isset($_POST['import']['type'])&& array_key_exists($_POST['import']['type'],$this->model['imports']))
		{
			$this->model['selectedImport'] = $_POST['import']['type'];
			$this->model['WEEEImporter'] = new $this->model['selectedImport']($this->db_app);
			$this->model['feedback'] = '';
			$this->model['dryrun'] = empty($_POST['validate']);
		}
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        if(!isset($_POST['process']))
			return;

        if(!$this->model['WEEEImporter'])
			return;

        $process = $_POST['process'];

        $fixErrorsFirst = IWRegistry::registry('langIf')->getText('Please correct import errors first');

        if(!$this->model['dryrun'])
        {
			// Dry run again first, if it has errors, do not continue;
			if(!$this->model['WEEEImporter']->import())
			{
				$this->model['feedback'] = $this->parseLog();
				$this->init($this->id);
				return $thrownException .= $fixErrorsFirst;
			}
			$this->model['WEEEImporter']->import(false);
                        $this->model['feedback'] = $this->parseLog();
        }
        else
        {
			$this->model['WEEEImporter']->import();
			$this->model['feedback'] = $this->parseLog();
        }
        $this->init($this->id);
    }

    private function parseLog()
    {
		$rv = '';
		$failure = $this->model['WEEEImporter']->getFailures();
		$log = $this->model['WEEEImporter']->getLog();
		foreach($log as $filename => $values)
		{
			$error = in_array($filename,$failure)?'warning':'info';
			$rv .= sprintf('<span class="%s">%s</span><br />%s<br /><br />',
				$error,
				$filename,implode('<br />',$values));
		}
		return $rv;
    }
}
/**
 * @package
 * @subpackage Interface
 * @since 05-12-2007
 * @copyright 2007 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceImporterItem extends InterfaceItem
{

    /**
     * item[1409] = import[type]
     */
    function getImportType()
    {
		$this->options = array(array('text'=>'- '.IWRegistry::registry('langIf')->getText('Select Import').' -','value'=>''));

		foreach($this->if_obj->model['imports'] as $class => $title)
		{
			$selected = (isset($this->if_obj->model['selectedImport'])&&($this->if_obj->model['selectedImport'] == $class))?'selected':'';
			$this->options[] = array('text'=>IWRegistry::registry('langIf')->getText($title),'value'=>$class,'selected'=>$selected);
		}

		$this->properties['onchange'] = 'reloadBlocks()';
    }

    /**
     * item[1410] =
     */
    function getFeedback()
    {
		if(!$this->if_obj->model['WEEEImporter'])
			return;

		if(isset($this->if_obj->model['locked']))
			return $this->text = IWRegistry::registry('langIf')->getText('The Imports of this type are Locked');

		$this->text = '';

		$dryrun = $this->if_obj->model['dryrun']?'(Test Mode)':'';
		$message = '<span class="%s" style="font-size: 140%%">%s %s</span><br /><br />';

		if(!$this->if_obj->model['WEEEImporter']->importSuccessful())
			$this->text = sprintf($message,'warning',$dryrun, IWRegistry::registry('langIf')->getText('Please correct the errors'));
		else
			$this->text = sprintf($message,'info',$dryrun, IWRegistry::registry('langIf')->getText('All files passed succesful'));

		return $this->text .= $this->if_obj->model['feedback'];
    }

    /**
     * item[1411] = validate
     */
    function getValidationCheckbox()
    {
		$this->properties['value'] = false;

		if(!$this->if_obj->model['WEEEImporter'])
			return $this->_hideInput = true;

		if(isset($this->if_obj->model['locked']))
			$this->properties['disabled'] = 'disabled';
		if(!$this->if_obj->model['WEEEImporter']->importSuccessful())
			$this->properties['disabled'] = 'disabled';

		$this->properties['onclick'] = '';
    }

    /**
     * item[1412] =
     */
    function getButtons()
    {
		$this->_BarButtons = 0;
			$this->_addCustomButton('Import','submit');
    }

}

/**
 * CALLBACK[getImportType]
 * item[1409] = {
 *     [handle] = Item: Select Import Type
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getFeedback]
 * item[1410] = {
 *     [handle] = Item: Feedback
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getValidationCheckbox]
 * item[1411] = {
 *     [handle] = Item: ValidatedFeedback
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[1412] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
