<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @author Nick Loeve <nick@basesoft.com>
 * @since 09-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceNotificationDetails extends InterfaceFactory
{
    /**
     * interface[id] = 102
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceNotificationDetails(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 50;
        $this->delimiter = '<br />';

        //pre-load the notification for the interface items
        require_once(ELC_CLASSES.'Notification.php');
        require_once(ELC_CLASSES.'Product.php');
    	$this->notification = new Notification($this->db_app, array('product' => new Product($this->db_app, $_SESSION['selected_product_id'])));
    	$this->nr = new NotificationRecipient($this->db_app);

    	if(!isset($_SESSION['session_user_relations']))
    		throw new InterfacePermissionException('Unauthorized access to notification details');

    	$this->user_relation_ids = array();
    	foreach($_SESSION['session_user_relations'] as $relation)
    		$this->user_relation_ids[] = $relation['id'];

    	//check we have a notification to play with
    	if(!isset($_REQUEST['notification']['id']) || !is_numeric($_REQUEST['notification']['id']) ||
    		!$this->notification->set((int)$_REQUEST['notification']['id']) ||
    		!$this->nr->set($this->notification->id, $_SESSION['session_user_id'], $this->user_relation_ids))
    	{
    		header('Location: '.($_SESSION['session_user_default_page']?$_SESSION['session_user_default_page']:'/'.SITE_PATH));
    		exit();
    	}

    	//mark the notification read
    	$this->nr->markRead($this->notification->id, $_SESSION['session_user_id'], $this->user_relation_ids);

    	//need this if replying
    	$this->hiddenFields[] = array('name' => 'notification[id]', 'value' => $this->notification->id);
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        if(isset($_POST['process']))
		{
			if($_POST['process'] == 'save')
			{
				//is this from a relation or just individual?
				$body = $_REQUEST['reply']['message']. '<br /><br /><p style="color: #0000CD;">'.$this->langIf->getText('In reply to').
		   				':<br /><br /> '.$this->_setConversationIndent($this->notification->message).'</p>';
				$message = array('subject'=> strip_tags($_REQUEST['reply']['subject']),
						'user'=> new ELCUser($this->db_app, $_SESSION['session_user_id']),
						'message'=> $body,
						'product'=> new Product($this->db_app, $_SESSION['selected_product_id'])
					);
				if(isset($_REQUEST['reply']['relation']) && $_REQUEST['reply']['relation'] != 0)
					$message['sender'] = new Relation($this->db_app, (int)$_REQUEST['reply']['relation']);

				Notification::send(
					$this->db_app,
					$message,
					array(array('user'=> new ELCUser($this->db_app, $this->notification->sender_user_id)))
				);

				//redirect to inbox
				header('Location: InterfaceNotificationInbox.php?notification_sent=1');
				exit();
			}
			else
			if($_POST['process'] == 'forward')
			{
				//is this from a relation or just individual?
				$body = $_REQUEST['forward']['message']. $this->_setConversationIndentFW($this->notification->message);
				$message = array('subject'=> strip_tags($_REQUEST['forward']['subject']),
						'user'=> new ELCUser($this->db_app, $_SESSION['session_user_id']),
						'message'=> $body,
						'product'=> new Product($this->db_app, $_SESSION['selected_product_id'])
					);
				if(isset($_REQUEST['reply']['relation']) && $_REQUEST['reply']['relation'] != 0)
					$message['sender'] = new Relation($this->db_app, (int)$_REQUEST['reply']['relation']);

				Notification::send(
					$this->db_app,
					$message,
					array(array('recipient'=> new Relation($this->db_app, (int)$_REQUEST['forward']['relation'])))
				);

				//redirect to inbox
				header('Location: InterfaceNotificationInbox.php?notification_sent=1');
				exit();
			}
		}
    }
	function _setConversationIndent($message)
    {
    	return preg_replace('@(<br>|<br />)@', '${1}&gt;', '&gt;'.$message);
    }
	function _setConversationIndentFW($message)
    {
    	$head = $this->langIf->getText('From').": ".$this->_getSender()."<br />".
    			$this->langIf->getText('On').": ".
    				Form_Date::makePrettyDateTime($this->notification->sent_date);
    	$ret = '<br /><br /><p style="color: #0000CD;"> --- '.$this->langIf->getText('Original message')." --- ".
		'<br /> '.$head."<br /><br />".$this->notification->message.'</p>';
		return $ret;
    }
	function _getSender()
    {
    	$sender = 'System';
		if (!empty($this->notification->sender_id))
		{
    		require_once(ELC_CLASSES . 'Relation.php');
    		$rel = new Relation($this->db_app);
    		$rel->set($this->notification->sender_id);
    		$sender = $rel->name;
    	}

    	if (!empty($this->notification->sender_user_id))
    	{
    		require_once(ELC_CLASSES . 'User.php');
    		$user = new ELCUser($this->db_app);
    		$user->set($this->notification->sender_user_id);
    		$user->info_person->set();
    		$name = $user->info_person->getFullName();
    		if (!empty($name))
    			$sender != 'System' ? $sender .= ' ('.$name.')' : $sender = $name;
    	}
    	return $sender;
    }
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 09-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceNotificationDetailsItem extends InterfaceItem
{

    /**
     * item[925] =
     */
    function getSubject()
    {
    	$this->text = '<div style="padding: 8px;">';
    	switch($this->if_obj->notification->type)
    	{
    		case NOTIFICATION_TYPE_MESSAGE:
    			$this->text .=
					'<img align="left" src="/'.SITE_PATH.'images/i-w/tango/16x16/status/dialog-information.gif" border="0" /> ';
    			break;

    		case NOTIFICATION_TYPE_INFO:
    			$this->text .=
					'<img align="left" src="/'.SITE_PATH.'images/i-w/tango/16x16/status/dialog-information.gif" border="0" /> ';
    			break;

    		case NOTIFICATION_TYPE_WARNING:
    			$this->text .=
					'<img align="left" src="/'.SITE_PATH.'images/i-w/tango/16x16/status/dialog-warning.png" border="0" /> ';
    			break;

    		case NOTIFICATION_TYPE_ERROR:
    			$this->text .=
					'<img align="left" src="/'.SITE_PATH.'images/i-w/tango/16x16/status/dialog-error.png" border="0" /> ';
    			break;

    	}
    	$this->text .= '&nbsp;&nbsp;
						<b>'.$this->if_obj->notification->subject.'</b></div>';
    }

    /**
     * item[926] =
     */
    function getSender()
    {
    	$sender = 'System';
		if (!empty($this->if_obj->notification->sender_id))
		{
    		require_once(ELC_CLASSES . 'Relation.php');
    		$rel = new Relation($this->db_app);
    		$rel->set($this->if_obj->notification->sender_id);
    		$sender = $rel->name;
    	}

    	if (!empty($this->if_obj->notification->sender_user_id))
    	{
    		require_once(ELC_CLASSES . 'User.php');
    		$user = new ELCUser($this->db_app);
    		$user->set($this->if_obj->notification->sender_user_id);
    		$user->info_person->set();
    		$name = $user->info_person->getFullName();
    		if (!empty($name))
    			$sender != 'System' ? $sender .= ' ('.$name.')' : $sender = $name;
    	}
    	$this->text = '<div style="display: block; padding: 8px;">'.
    		$this->langIf->getText('Sender').': '.$sender.'</div>';
    }

    /**
     * item[927] =
     */
    function getSentDate()
    {
    	$this->text = '<div style="padding: 8px;display: relative; float: right; top: 5px; left: 5px;"><i>'.
    		Form_Date::makePrettyDateTime($this->if_obj->notification->sent_date).'</i></div>';
    }

    /**
     * item[928] =
     */
    function getNotification()
    {
    	$this->text = $this->text = '<div id="ifI_'.$this->id.'" style="display: block; padding: 8px;">'.
    		$this->if_obj->notification->message;
    	if($this->if_obj->nr->callback)
    	{
    		$mouse = 'onmouseover="addCssClass(this, \'hover\');" onmouseout="removeCssClass(this, \'hover\');"';
    		$this->text .= '<div style="padding: 10px 0px 10px 0px;"><a href="#" onclick="'.
    			$this->if_obj->nr->callback.'" '.$mouse.' id="more_information_'.$this->if_obj->notification->id.'">'.$this->langIf->getText('More Information').'</a></div>';

    	}
    	$this->text .= '</div>';
    }

    /**
     * item[929] = reply[subject]
     */
    function getSubjectReply()
    {
    	$this->properties['size'] = 100;
    	$this->properties['value'] = $this->_setSubjectIndent($this->if_obj->notification->subject);
    }

	function getSubjectForward()
    {
    	$this->properties['size'] = 100;
    	$this->properties['value'] = $this->_setSubjectIndentFW($this->if_obj->notification->subject);
    }

    /**
     * item[930] = reply[message]
     */
    function getNotificationReply()
    {
    	$this->properties['style'] = 'width: 100%; height: 200px;';
		$this->is_tiny_mce = true;
		$this->tiny_mce_settings['theme_advanced_buttons1'] = "code,bold,italic,underline,separator,bullist,numlist,separator,undo,redo,link,unlink";
		$this->tiny_mce_settings['theme_advanced_buttons2'] = '';
		$this->tiny_mce_settings['theme_advanced_buttons3'] = '';

		//this will trigger an error(because of the input sanitize)
        //-> the "In reply to....." is concatenated to the message when sending it
		// FIXME - ^ this is not such a nice thing (do it for the forward part too)

		//$this->value = '<br /><br /><p style="color: #0000CD;">'.$this->langIf->getText('In reply to').
		//	':<br /><br /> '.$this->_setConversationIndent($this->if_obj->notification->message).'</p>';

		/**
		 * @TODO Need to block replies when sender is system
		 */
		if(!empty($this->if_obj->notification->sender_id) || !empty($this->if_obj->notification->sender_user_id))
		{
			$this->postscript = '

				addEvent(window,\'load\', function(){
					ifHandler.toggleBlock('.$this->parent->id.');

					function setUpReplyField()
					{
						ifHandler.toggleBlock('.$this->parent->id.');
						removeEvent(getElById("ifB_'.$this->parent->id.'"), "click", setUpReplyField);
					}

					addEvent(getElById("ifB_'.$this->parent->id.'"), "click", setUpReplyField);

				});
				';
		}
		else
		{
			$this->parent->hide();
		}
    }

	function getNotificationForward()
    {
    	$this->properties['style'] = 'width: 100%; height: 200px;';
		$this->is_tiny_mce = true;
		$this->tiny_mce_settings['theme_advanced_buttons1'] = "code,bold,italic,underline,separator,bullist,numlist,separator,undo,redo,link,unlink";
		$this->tiny_mce_settings['theme_advanced_buttons2'] = '';
		$this->tiny_mce_settings['theme_advanced_buttons3'] = '';

		/**
		 * @TODO Need to block replies when sender is system
		 */
		if(!empty($this->if_obj->notification->sender_id) || !empty($this->if_obj->notification->sender_user_id))
		{
			$this->postscript = '

				addEvent(window,\'load\', function(){
					ifHandler.toggleBlock('.$this->parent->id.');

					function setUpReplyField()
					{
						ifHandler.toggleBlock('.$this->parent->id.');
						removeEvent(getElById("ifB_'.$this->parent->id.'"), "click", setUpReplyField);
					}

					addEvent(getElById("ifB_'.$this->parent->id.'"), "click", setUpReplyField);

				});
				';
		}
		else
		{
			$this->parent->hide();
		}
    }

    /**
     * item[931] =
     */
    function getButtonsReply()
    {
    	$this->_BarButtons = 0;
    	$text = "<div align=left>".$this->langIf->getText('The original message will be automaticlly attached')."</div>";
    	$sel = new Form_Select('reply[relation]');
		$sel->addOption($this->langIf->getText('Yourself'), '0');
		foreach($_SESSION['session_user_relations'] as $relation)
			$sel->addOption($relation['name'], $relation['id']);
		if(!empty($this->if_obj->nr->recipient_id))
			$sel->setSelectedValue($this->if_obj->nr->recipient_id);
		$this->_CustomButtons[] = $text . $this->langIf->getText('Send as').': '.$sel->getHtml().'&nbsp;';
		$this->_addCustomButton($this->langIf->getText('Send'), 'button',
    		array('onclick'=>"\$('interface_form').process.value='save'; \$('interface_form').submit();"));
    }

	function getButtonsForward()
    {
    	$this->_BarButtons = 0;
    	$text = "<div align=left>".$this->langIf->getText('The original message will be automaticlly attached')."</div>";
    	$sel = new Form_Select('reply[relation]');
		$sel->addOption($this->langIf->getText('Yourself'), '0');
		foreach($_SESSION['session_user_relations'] as $relation)
			$sel->addOption($relation['name'], $relation['id']);
		if(!empty($this->if_obj->nr->recipient_id))
			$sel->setSelectedValue($this->if_obj->nr->recipient_id);
		$this->_CustomButtons[] = $text.$this->langIf->getText('Send as').': '.$sel->getHtml().'&nbsp;';

		$possibleRecipients = array();
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_CRSO:
				require_once(ELC_CLASSES.'AccountantCRSO.php');
				$ac = new AccountantCRSO($this->db_app);
				$q = $ac->getCRSOAccountants($_SESSION['crso_relation_id']);
				while($r = $q->fetch())
				{
					$possibleRecipients[] = array('name' => $r->NAME, 'id' => $r->ACCOUNTANT_RELATION_ID);
				}
				break;
			case GROUP_PARTICIPANT:
				require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
				$cp = new CRSOParticipantProduct($this->db_app);
				// Only for the selected participant. See #787
				//get the CRSO(s) for this participant
				//foreach($_SESSION['participant_ids'] as $id)
				//{
					$q = $cp->getParticipantCRSOs($_SESSION['selected_participant_id'], 'q', false, false, $_SESSION['selected_product_id']);
					while($r = $q->fetch())
					{
						$possibleRecipients[] = array('name' => $r->{'CRSO_NAME'}, 'id' => $r->{'CRSO_RELATION_ID'});
					}
				//}
				break;
		}
		$sel = new Form_Select('forward[relation]');
		for($i=0;$i<count($possibleRecipients);$i++)
		{
			$sel->addOption($possibleRecipients[$i]['name'], $possibleRecipients[$i]['id']);
		}
		$this->_CustomButtons[] = $this->langIf->getText('to').': '.$sel->getHtml().'&nbsp;';
		$this->_addCustomButton($this->langIf->getText('Send'), 'button',
    		array('onclick'=>"\$('interface_form').process.value='forward'; \$('interface_form').submit();"));
    }


    function _setConversationIndent($message)
    {
    	return preg_replace('@(<br>|<br />)@', '${1}&gt;', '&gt;'.$message);
    }

    function _setSubjectIndent($subject)
    {
    	return preg_match('@^\s*RE:@', $subject) ? $subject : 'RE: '.$subject;
    }

	function _setSubjectIndentFW($subject)
    {
    	return preg_match('@^\s*Fwd:@', $subject) ? $subject : 'Fwd: '.$subject;
    }

}

/**
 * CALLBACK[getSubject]
 * item[925] = {
 *     [handle] = Item: NotificationDetailsSubject
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getSender]
 * item[926] = {
 *     [handle] = Item: NotificationDetailsSender
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getSentDate]
 * item[927] = {
 *     [handle] = Item: NotificationDetailsSentDate
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getNotification]
 * item[928] = {
 *     [handle] = Item: NotificationDetailsNotification
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getSubjectReply]
 * item[929] = {
 *     [handle] = Item: NotificationDetailsSubjectReply
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getNotificationReply]
 * item[930] = {
 *     [handle] = Item: NotificationDetailsReply
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtonsReply]
 * item[931] = {
 *     [handle] = Item: NotificationDetailsReplyButtons
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
