<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'User.php');
require_once(ELC_CLASSES.'Relation.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @author Nick Loeve <nick@basesoft.com>
 * @since 16-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceNotificationHelpdesk extends InterfaceFactory
{
    /**
     * interface[id] = 126
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceNotificationHelpdesk(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        if(!isset($_REQUEST['helpdesk']['relation']))
        {
        	header('Location: InterfaceNotificationInbox.php?script_parent_id=0');
        	exit();
        }

        //get current users fullname
    	require_once(ELC_CLASSES.'User.php');
    	$this->user = new ELCUser($this->db_app);
    	$this->user->set($_SESSION['session_user_id']);

    	//get the relation name we are sending to
    	require_once(ELC_CLASSES.'Relation.php');
    	$this->rel = new Relation($this->db_app);
    	$this->rel->set((int)$_REQUEST['helpdesk']['relation']);

    	//check that we should be able to send to this relation
    	require_once(IWARE.'group.php'); // this defines group types
    	$possibleRecipients = array();

		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_CRSO:
				//get the accountant(s) for this CRSO
				require_once(ELC_CLASSES.'AccountantCRSO.php');
				$ac = new AccountantCRSO($this->db_app);
				$q = $ac->getCRSOAccountants($_SESSION['crso_relation_id']);
				while($r = $q->fetch())
				{
					$possibleRecipients[] = array('name' => $r->NAME, 'id' => $r->ACCOUNTANT_RELATION_ID);
				}
				break;
			case GROUP_PARTICIPANT:
				require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
				$cp = new CRSOParticipantProduct($this->db_app);
				// SELECTED Participant Only. See trac ticket #787
				//get the CRSO(s) for this participant
				//foreach($_SESSION['participant_ids'] as $id)
				//{
					$q = $cp->getParticipantCRSOs($_SESSION['selected_participant_id'], 'q', false, false, $_SESSION['selected_product_id']);
					while($r = $q->fetch())
					{
						$possibleRecipients[] = array('name' => $r->{'CRSO_NAME'}, 'id' => $r->{'CRSO_RELATION_ID'});
					}
				//}
				break;
		}

		if(!count($possibleRecipients) ||
			false === array_search(array('name' => $this->rel->name, 'id' => $this->rel->id), $possibleRecipients))
		{
			header('Location: InterfaceNotificationInbox.php?script_parent_id=0');
			exit();
		}

		$this->hiddenFields[] = array('name' => 'helpdesk[relation]', 'value' => (int)$_REQUEST['helpdesk']['relation']);
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
    	$notSendingEmptyMessage = $this->langIf->getText('Cannot sent an empty message');
    	if(isset($_REQUEST['process']) && $_REQUEST['process'] == 'save')
    	{
    		if(empty($_REQUEST['helpdesk']['message']))
    			return $thrownException .= $notSendingEmptyMessage;
    		require_once(ELC_CLASSES.'Notification.php');
			$message = array('subject'=> $_REQUEST['helpdesk']['subject'],
					'user'=> new ELCUser($this->db_app, $_SESSION['session_user_id']),
					'message'=> $_REQUEST['helpdesk']['message'],
					'product'=> new Product($this->db_app, $_SESSION['selected_product_id'])
				);

			if(isset($_REQUEST['helpdesk']['sender']) && $_REQUEST['helpdesk']['sender'] != 0)
					$message['sender'] = new Relation($this->db_app, (int)$_REQUEST['helpdesk']['sender']);

			Notification::send(
				$this->db_app,
				$message,
				array(array('recipient'=> new Relation($this->db_app, (int)$_REQUEST['helpdesk']['relation'])))
			);

			//redirect to inbox
			header('Location: InterfaceNotificationInbox.php?helpdesk_sent=1');
			exit();
    	}
    }
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 16-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceNotificationHelpdeskItem extends InterfaceItem
{

    /**
     * item[942] =
     */
    function getSubject()
    {

    	$this->if_obj->user->info_person->set();
    	$subject = $this->langIf->getText(
				'Helpdesk Question: from {PERSON_FULLNAME} to {RELATION_NAME}', null,
				array(
						'PERSON_FULLNAME' => $this->if_obj->user->info_person->getFullName(),
						'RELATION_NAME' => $this->if_obj->rel->name
				)
    	);
    	$this->if_obj->hiddenFields[] = array('name' => 'helpdesk[subject]', 'value' => $subject);
    	$this->text = '<div style="padding: 8px;"><b>'.$subject.'</b></div>';
    }

    /**
     * item[943] = helpdesk[message]
     */
    function getMessage()
    {
    	$this->properties['style'] = 'width: 100%; height: 200px;';
		$this->is_tiny_mce = true;
		$this->tiny_mce_settings['theme_advanced_buttons1'] = "bold,italic,underline,separator,bullist,numlist,separator,undo,redo,link,unlink";
		$this->tiny_mce_settings['theme_advanced_buttons2'] = '';
		$this->tiny_mce_settings['theme_advanced_buttons3'] = '';
		$this->value = '';
    }

    /**
     * item[944] =
     */
    function getButtons()
    {
    	$this->_BarButtons = 0;
    	$sel = new Form_Select('helpdesk[sender]');
		$sel->addOption($this->langIf->getText('Yourself'), '0');
		if($_SESSION['session_group_group_id']==GROUP_PARTICIPANT)
		{
			$relation = new Relation($this->db_app,$_SESSION['selected_participant_id'],true);
			$sel->addOption($relation->name, $relation->id);
		}
		else
		{
			foreach($_SESSION['session_user_relations'] as $relation)
				$sel->addOption($relation['name'], $relation['id']);
		}
		$this->_CustomButtons[] = $this->langIf->getText('Send as').': '.$sel->getHtml().'&nbsp;';
    	$this->_addCustomButton($this->langIf->getText('Send'));
    }

}

/**
 * CALLBACK[getSubject]
 * item[942] = {
 *     [handle] = Item: NotificationHelpdeskSubject
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getMessage]
 * item[943] = {
 *     [handle] = Item: NotificationHelpdeskMessage
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[944] = {
 *     [handle] = Item: NotificationHelpdeskButtons
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
