<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(IWARE.'group.php');
require_once(ELC_CLASSES.'AccountantCRSO.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @author Nick Loeve <nick@basesoft.com>
 * @since 06-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceNotificationInbox extends InterfaceFactory
{
    /**
     * interface[id] = 100
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceNotificationInbox(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);

        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        require_once(ELC_CLASSES.'Notification.php');
        require_once(ELC_CLASSES.'Product.php');
    	$this->noti = new Notification($this->db_app, array('product'=>new Product($this->db_app, $_SESSION['selected_product_id'])));

        if(!isset($_SESSION['session_user_relations']))
        	throw new InterfacePermissionException('Unauthorized access to notification details');

        //pagination stuff
        $this->user_relation_ids = array();
        if($_SESSION['session_group_group_id'] == GROUP_PARTICIPANT)
        {
        	$this->user_relation_ids[] = $_SESSION['selected_participant_id'];
        }
        else
    	{
    		foreach($_SESSION['session_user_relations'] as $relation)
    			$this->user_relation_ids[] = $relation['id'];
    	}
        $this->unread_page = isset($_REQUEST['unread_page']) ? (int)$_REQUEST['unread_page'] : 1;
        $this->read_page = isset($_REQUEST['read_page']) ? (int)$_REQUEST['read_page'] : 1;
    	$this->unread_count = $this->noti->count(
    									$_SESSION['session_user_id'],
    									$this->user_relation_ids,
    									NOTIFICATION_STATUS_UNREAD);
        $this->read_count = $this->noti->count(
    									$_SESSION['session_user_id'],
    									$this->user_relation_ids,
    									NOTIFICATION_STATUS_READ);

    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        //have we just sent a normal notification?
        if(isset($_REQUEST['notification_sent']) && $_REQUEST['notification_sent'])
        {
        	$this->script = '
						addEvent(window, \'load\', function(){
						MessageBox.prototype.Information(\''.$this->langIf->getText("Notification Sent").'\', \''.
						$this->langIf->getText("Your notification was sent").'!\');
						});
					';
    	}

    	//have we just sent a helpdesk notification?
    	if(isset($_REQUEST['helpdesk_sent']) && $_REQUEST['helpdesk_sent'])
        {
        	$this->script = '
						addEvent(window, \'load\', function(){
						MessageBox.prototype.Information(\''.$this->langIf->getText("Helpdesk Notification Sent").'\', \''.
						$this->langIf->getText("Your enquiry was sent").'!\');
						});
					';
    	}
    }
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 06-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceNotificationInboxItem extends InterfaceItem
{

    /**
     * item[816] =
     * item[818] =
     */
    function getButtons()
    {
    	$this->_BarButtons = 0;

    	require_once(IWARE.'group.php'); // this defines group types

    	//if this is the bottm bar then mqake a compose widget
    	if($this->id == 818)
    		$this->_buildCompositionButtons();
    	//if we are a crso or participant, and the this is the top button bar
    	//then build the help desk widget
    	if(($_SESSION['session_group_group_id'] == GROUP_CRSO ||
			$_SESSION['session_group_group_id'] == GROUP_PARTICIPANT) && $this->id == 816)
		{
			$this->_buildHelpDeskButtons();
		}
    }

    /**
     * item[817] =
     * item[893] =
     */
    function getNotifications()
    {
    	$classes = array(
    		NOTIFICATION_TYPE_INFO=>'info',
    		NOTIFICATION_TYPE_WARNING=>'warning',
    		NOTIFICATION_TYPE_ERROR=>'error',
    		NOTIFICATION_TYPE_MESSAGE=>'message');

    	$html = '<ul class="notifications">';
    	$sw = 1;

		require_once(ELC_CLASSES.'RelationUser.php');
		$ru = new RelationUser($this->db_app);

    	$notifications = 0;
    	$notification_status = NOTIFICATION_STATUS_UNREAD;
    	$notification_count = $this->if_obj->unread_count;
    	$notification_page = $this->if_obj->unread_page;
    	$notification_item = 'unread_page';

  		//adjust notification status that we look for based on block/item
    	switch($this->id)
    	{
    		case 893: //READ ITEMS
    			$notification_status = NOTIFICATION_STATUS_READ;
    			$notification_count = $this->if_obj->read_count;
    			$notification_page = $this->if_obj->read_page;
    			$notification_item = 'read_page';
    			break;
    	}

    	//work out paging here
    	$notification_offset = ($notification_page - 1) * NOTIFICATION_INBOX_PAGE_LIMIT;

    	$q = $this->if_obj->noti->select($_SESSION['session_user_id'],
    										$this->if_obj->user_relation_ids,
    										$notification_status,
    										NOTIFICATION_INBOX_PAGE_LIMIT,
    										$notification_offset);

    	while ($r = $q->fetch())
    	{
    		// check rights
    		if ($r->{'RECIPIENT_ID'}) {
    			if (!$ru->set($_SESSION['session_user_id'], $r->{'RECIPIENT_ID'}) && !$ru->status)
    				continue;
    			//check for participant and then clearance level
    			if(	isset($_SESSION['participant_ids']) &&
    				in_array($r->{'RECIPIENT_ID'}, $_SESSION['participant_ids']) &&
    				$r->{'CLEARANCE_LEVEL'} &&
    				!$ru->hasClearance($r->{'CLEARANCE_LEVEL'},$_SESSION['session_user_id'],$r->{'RECIPIENT_ID'}))
    				{
    					continue;
    				}
    		}



    		$class = $classes[ $r->{'TYPE'} ];

    		$mouse = (!empty($r->{'CALLBACK'})) ?
    			'onmouseover="addCssClass(this, \'hover\');" '.
    			'onmouseout="removeCssClass(this, \'hover\');"' : '';

    		$sender = 'System';
    		if (!empty($r->{'SENDER_ID'}))
    		{
	    		require_once(ELC_CLASSES.'Relation.php');
	    		$rel = new Relation($this->db_app);
	    		$rel->set($r->{'SENDER_ID'});
	    		$sender = $rel->name;
	    	}

	    	if (!empty($r->{'SENDER_USER_ID'}))
	    	{
	    		require_once(ELC_CLASSES . 'User.php');
	    		$user = new ELCUser($this->db_app);
	    		$user->set($r->{'SENDER_USER_ID'});
	    		$user->info_person->set();
	    		$name = $user->info_person->getFullName();
	    		if (!empty($name))
    				$sender != 'System' ? $sender .= ' ('.$name.')' : $sender = $name;
	    	}

    		$onclick = 'onclick="go(\'/'.SITE_PATH.'scripts/elc/public/InterfaceNotificationDetails.php?notification[id]='.$r->{'NOTIFICATION_ID'}.'\');"';

    		// hmm this can be done prettier
    		$html .= '<li id="li_'.$r->{'NOTIFICATION_ID'}.'" class="'.($notification_status == NOTIFICATION_STATUS_UNREAD ? $class : '').' switch'.$sw.'" '.$onclick.' '.$mouse.'>'.
	    				'<b><a href="#" '.$onclick.' id="notification_'.$r->{'NOTIFICATION_ID'}.'">'.$r->{'SUBJECT'}.'</a></b>'.
	    				'<div style="display: relative; float: right; top: 5px; left: 5px;"><i>'.Form_Date::makePrettyDateTime($r->{'SENT_DATE'}).'</i></div>'.
	    				'<div style="display: block;">'.$this->langIf->getText('Sender').': '.$sender.'</div>'.
    				'</li>';

    		$sw = ($sw) ? 0 : 1;
    		$notifications++;
    	}


    	if (!$notifications)
    		$html .= '<li class="switch0">'.$this->langIf->getText('No recent notifications').'</li>';


		$html .= '</ul>';

		//add pages display
		if($notification_count > NOTIFICATION_INBOX_PAGE_LIMIT)
		{
			$html .= '<div style="display: block">Pages: ';
			$pages = ($notification_count / NOTIFICATION_INBOX_PAGE_LIMIT) +
						(($notification_count % NOTIFICATION_INBOX_PAGE_LIMIT) > 0 ? 1 : 0);
			for($i = 1; $i <= $pages; $i++)
			{
				$i != $notification_page
					?
						$html .= '<a href="#" onclick="go(\'/'.SITE_PATH.'scripts/elc/public/InterfaceNotificationInbox.php'.$this->_buildPagedQueryString($notification_item, $i).'\');">'.$i.'</a> &nbsp;'
					:
						$html .= $i.' &nbsp;'
					;
			}
			$html .= '</div>';
		}
		$this->text = $html;
    }

	/**
	 * @param $item string The set of messages (based on status) getting built into pages
	 * @param $index int The page number being linked to
	 */
	function _buildPagedQueryString($item, $index)
	{
		$query_string = '?';
		foreach($_GET as $key => $value)
		{
			if($key != $item)
				$query_string .= $key.'='.$value.'&';
		}
		return $query_string .= $item.'='.$index;
	}


	function _buildHelpDeskButtons()
	{
		$possibleRecipients = array();

		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_CRSO:
				$ac = new AccountantCRSO($this->db_app);
				$q = $ac->getCRSOAccountants($_SESSION['crso_relation_id']);
				while($r = $q->fetch())
				{
					$possibleRecipients[] = array('name' => $r->NAME, 'id' => $r->ACCOUNTANT_RELATION_ID);
				}
				break;
			case GROUP_PARTICIPANT:
				require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
				$cp = new CRSOParticipantProduct($this->db_app);
				// Only for the selected participant. See #787
				//get the CRSO(s) for this participant
				//foreach($_SESSION['participant_ids'] as $id)
				//{
					$q = $cp->getParticipantCRSOs($_SESSION['selected_participant_id'], 'q', false, false, $_SESSION['selected_product_id']);
					while($r = $q->fetch())
					{
						$possibleRecipients[] = array('name' => $r->{'CRSO_NAME'}, 'id' => $r->{'CRSO_RELATION_ID'});
					}
				//}
				break;
		}

		/**
		 * @todo build helpdesk widget and associated scripts
		 */
		if(count($possibleRecipients))
		{
			//build the helpdesk widget
			$onclick = "go('/".SITE_PATH."scripts/elc/public/InterfaceNotificationHelpdesk.php?helpdesk[relation]=' + \$('helpdeskrelation').options[\$('helpdeskrelation').selectedIndex].value);";
			$sel = new Form_Select('helpdesk[relation]');
			$sel->setID("helpdeskrelation");
			foreach($possibleRecipients as $relation)
				$sel->addOption($relation['name'], $relation['id']);
			$this->_CustomButtons[] = $this->langIf->getText('Helpdesk').': '.$sel->getHtml().'&nbsp;';
			$this->_addCustomButton($this->langIf->getText('Enquire'), 'button', array('onclick' => $onclick));
		}
	}

	/**
	 * This method will build functionality to send notifications to
	 * all universes lower in privilege than the current user, and where
	 * a relationship exists
	 */
	function _buildCompositionButtons()
	{
		$this->script =
        "
			function show_checkbox_sys(val1,val2)
			{
				var cc = document.getElementById('composerelation').value
				var el1 = document.getElementById('op')
				var el2 = document.getElementById('sys')
				var arr1 = val1.split(';')
				var arr2 = val2.split(';')
				var bool = 0;
				for(i = 0; i < arr1.count(); i++)
					if(cc == arr1[i]) bool = 1
				for(i = 0; i < arr2.count(); i++)
					if(cc == arr2[i]) bool = 2
				if(bool == 1){
					el1.style.visibility='visible'
					el2.style.visibility='hidden'
				}
				else if(bool == 2){
					el2.style.visibility='visible'
					el1.style.visibility='visible'
				}
				else{
					el2.style.visibility='hidden'
					el1.style.visibility='hidden'
				}
			}";

		$possibleRecipients = array('PARTICIPANT' => array(), 'CRSO' => array(), 'ACCOUNTANT' => array(),'EVERYDODY' => array());
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_PARTICIPANT:
				//can send to noone
				break;
			case GROUP_CRSO:
				//can send to their participants
				$this->_getRecipientsForCRSO($possibleRecipients, $_SESSION['crso_relation_id']);
				break;
			case GROUP_ACCOUNTANT:
				//can send to all it's crsos and partiaipanta that belong to thiose crsos
				$this->_getRecipientsForAccountant($possibleRecipients, $_SESSION['accountant_relation_id']);
				break;
			case GROUP_SYSADMIN:
				//can send to everyone
				$this->_getRecipientsForSysadmin($possibleRecipients);
				break;
		}
		if($possibleRecipients['PARTICIPANT'] || $possibleRecipients['CRSO'] || $possibleRecipients['ACCOUNTANT'])
		{
			//build the compose widget
			$onclick = "go('/".SITE_PATH."scripts/elc/public/InterfaceNotificationCompose.php?compose[relation]=' + \$('composerelation').options[\$('composerelation').selectedIndex].value";
			$sel = new Form_Select('compose[relation]');
			$sel->setID("composerelation");
			foreach($possibleRecipients as $key => $group)
			{
				count($group) && $sel->addOptGroup($key);
				foreach($group as $id => $rel)
					$sel->addOption($rel, $id);
			}
			if($_SESSION['session_group_group_id'] == GROUP_SYSADMIN)
			{
				$keys1 = array();
				foreach(array_keys($possibleRecipients['CRSO']) as $k)
					if(strpos($k, ";") === false) $keys1[] = $k;
				$val1 = implode(";",$keys1);
				$keys2 = array();
				foreach(array_keys($possibleRecipients['ACCOUNTANT']) as $k)
					if(strpos($k, ";") === false) $keys2[] = $k;
				$val2 = implode(";",$keys2);
				$sel->onChange("show_checkbox_sys('$val1','$val2')");

				$check = new Form_Checkbox("all_part");
				$check->setID("ap");

				$this->_CustomButtons[] = "<div id='op' style=\"visibility: hidden;float: right;padding-right: 100px;padding-top:5px;\">&nbsp;" . $check->getHtml()."&nbsp;".$this->langIf->getText('To all its participants')."&nbsp;</div>";
				$onclick .= "+'&all_part=' + \$\$('all_part').checked";

				$check = new Form_Checkbox("all_crso");
				$check->setID("ac");

				$this->_CustomButtons[] = "<div id='sys' style=\"visibility: hidden;float: right;padding-right: 10px;padding-top:5px;\">&nbsp;" . $check->getHtml()."&nbsp;".$this->langIf->getText('To all its CRSOs')."&nbsp;</div>";
				$onclick .= "+'&all_crso=' + \$\$('all_crso').checked";
			}
			$this->_CustomButtons[] = $this->langIf->getText('Compose').': '.$sel->getHtml().'&nbsp;';
			$onclick .= ");";
			$this->_addCustomButton($this->langIf->getText('Go'), 'button', array('onclick' => $onclick));
		}
	}

	function _getRecipientsForCRSO(&$possibleRecipients, $cId, $no_all = false)
	{
		require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
		$cp = new CRSOParticipantProduct($this->db_app);
		$q = $cp->getCRSOParticipants($cId, 'q', false, $_SESSION['selected_product_id']);
		while($r = $q->fetch())
		{
			$possibleRecipients['PARTICIPANT'][(string)$r->{'PARTICIPANT_RELATION_ID'}] = $this->tfLongNames($r->{'PARTICIPANT_NAME'});
		}
		if(!$no_all && count($possibleRecipients['PARTICIPANT'])>1)
		{
			$all = implode(";",array_keys($possibleRecipients['PARTICIPANT']));
			$possibleRecipients['PARTICIPANT']['allp'] = "ALL PARTICIPANTS";
		}
	}

	function _getRecipientsForAccountant(&$possibleRecipients, $accountant_id, $no_all = false)
	{
		$ac = new AccountantCRSO($this->db_app);
		$q = $ac->getAccountantCRSOs($accountant_id);
		while($r = $q->fetch())
		{
			if($_SESSION['session_group_group_id'] == GROUP_SYSADMIN)
				$this->_getRecipientsForCRSO($possibleRecipients, $r->CRSO_RELATION_ID,true);
			$possibleRecipients['CRSO'][(string)$r->CRSO_RELATION_ID] = $this->tfLongNames($r->NAME);
		}

		if(!$no_all && count($possibleRecipients['PARTICIPANT'])>0)
		{
			$all = implode(";",array_keys($possibleRecipients['PARTICIPANT'])) . ";" . implode(";",array_keys($possibleRecipients['CRSO']));
			$possibleRecipients['EVERYBODY']['everybody'] = "EVERYBODY";

		}
		if(!$no_all && count($possibleRecipients['PARTICIPANT'])>1)
		{
			$all1 = implode(";",array_keys($possibleRecipients['PARTICIPANT']));
			$possibleRecipients['PARTICIPANT']['allp'] = "ALL PARTICIPANTS";
		}
		if(!$no_all && count($possibleRecipients['CRSO'])>1)
		{
			$all2 = implode(";",array_keys($possibleRecipients['CRSO']));
			$possibleRecipients['CRSO']['allc'] = "ALL CRSOs";
		}
	}

	function _getRecipientsForSysadmin(&$possibleRecipients)
	{
		$sql = "SELECT A.RELATION_ID, R.NAME FROM TBL_ACCOUNTANT A
				INNER JOIN TBL_RELATION as R
					ON A.RELATION_ID = R.RELATION_ID";
		$q = $this->db->query($sql);
		$q->execute();

		while($r = $q->fetch())
		{
			$this->_getRecipientsForAccountant($possibleRecipients, $r->RELATION_ID, true);
			$possibleRecipients['ACCOUNTANT'][(string)$r->RELATION_ID] = $this->tfLongNames($r->NAME);
		}

		if(count($possibleRecipients['PARTICIPANT'])>0)
		{
			$all = implode(";",array_keys($possibleRecipients['PARTICIPANT'])) .
					";" .
					implode(";",array_keys($possibleRecipients['CRSO'])) .
					";" .
					implode(";",array_keys($possibleRecipients['ACCOUNTANT']));
			$possibleRecipients['EVERYBODY']['everybody'] = "EVERYBODY";

		}
		if(count($possibleRecipients['PARTICIPANT'])>1)
		{
			$all1 = implode(";",array_keys($possibleRecipients['PARTICIPANT']));
			$possibleRecipients['PARTICIPANT']['allp'] = "ALL PARTICIPANTS";
		}
		if(count($possibleRecipients['CRSO'])>1)
		{
			$all2 = implode(";",array_keys($possibleRecipients['CRSO']));
			$possibleRecipients['CRSO']['allc'] = "ALL CRSOs";
		}
		if(count($possibleRecipients['ACCOUNTANT'])>1)
		{
			$all3 = implode(";",array_keys($possibleRecipients['ACCOUNTANT']));
			$possibleRecipients['ACCOUNTANT']['allo'] = "ALL ACCOUNTANTS";
		}
	}
	function tfLongNames($value)
	{
		return (mb_strlen($value)>35)?mb_substr($value,0,35).'...':$value;
	}
}

/**
 * CALLBACK[getButtons]
 * item[816] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[818] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getNotifications]
 * item[817] = {
 *     [handle] = Item: NotificationInbox
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[893] = {
 *     [handle] = Item: NotificationInboxReadItems
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
