<?php
/**
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @since 20-01-2005
 * @version $Id$
 */

require_once(INTERFACE_OBJ."InterfaceFactory.php");

require_once(ELC_CLASSES."Brand.php");
require_once(ELC_CLASSES.'Country.php');
require_once(ELC_CLASSES."ParticipantBrand.php");
require_once(ELC_CLASSES."Participant.php");
require_once(ELC_CLASSES."CRSOParticipantProduct.php");
require_once(ELC_CLASSES."CRSOParticipant.php");
require_once(ELC_CLASSES."CRSO.php");
require_once(ELC_CLASSES."CRSOProduct.php");
require_once(ELC_CLASSES."RelationInvoice.php");
require_once(ELC_CLASSES."RelationUser.php");
require_once(ELC_CLASSES."User.php");
require_once(ELC_CLASSES."Product.php");
require_once(ELC_CLASSES."ProductCategory.php");
require_once(ELC_CLASSES.'ProvisionalDeclaration.php');
require_once(ELC_CLASSES.'FormBehaviourPlugins.php');
require_once(ELC_CLASSES.'CRSOProductDimension.php');

require_once(IWARE."InfoPerson.php");
require_once(IWARE."GroupUser.php");
require_once(IWARE."address.php");
require_once(IWARE."RelationType.php");
require_once(IWARE."group.php");
require_once(IWARE.'Document.php');

define('BLOCK_ADMISSION', 97);
define('BLOCK_COMPANY',   98);
define('BLOCK_USER',      101);
define('BLOCK_PROV1',     102);
define('BLOCK_PROV2',     103);
define('BLOCK_BRANDS',    100);
define('BLOCK_INVOICE',   127);
define('BLOCK_DOCUMENTS', 129);
define('BLOCK_CONFIRM',   144);
define('BLOCK_COUNTRY_SPECIFIC_TEXT', 414);

define('CONFIRM_PRODUCER', 0);
define('CONFIRM_EXPORTER', 1);

define('ITEM_DD_BANK_ACCOUNT', 427);
define('ITEM_DD_IBAN_NO',      429);
define('ITEM_DD_SWIFT_BIC_CODE',      688);
define('ITEM_BUSINESS_START_DATE', 430);
define('ITEM_DOCUMENT_BANK', 545);

class InterfaceParticipantRegistrationCommon extends InterfaceFactory
{
	var $thrownException = '';
	var $exporter = true;

	function InterfaceParticipantRegistrationCommon($db, $db_app, $langIf)
	{
		// see if the product id is passed in the request
		if (isset($_REQUEST['product']['id']))
		{
			//check it is actually a product
			$is_prod = new Product($db_app, (int)$_REQUEST['product']['id']);
			if(!$is_prod->set())
			{
				throw new InterfacePermissionException('Unable to set product');
			}
			$_SESSION['selected_product_id'] = (int)$_REQUEST['product']['id'];
		}

		// check the confirm page
		if (isset($_POST['process']))
		{
			$this->showRequiredString = false;
			if(($_POST['process'] == 'check')&&isset($_POST['admission'])) // Producer || Exporter
			{
				if (($_POST['admission'] & PARTICIPANT_PRODUCER) == PARTICIPANT_PRODUCER)
					$this->confirm_page = CONFIRM_PRODUCER;
				else
					$this->confirm_page = CONFIRM_EXPORTER;
			}
			elseif($_POST['process'] == 'secondcheck') // Exporter
				$this->confirm_page = CONFIRM_EXPORTER;
		}

		$this->FirstColWidth = 25;
		$this->delimiter = '<br />';

		if(isset($_POST['process']))
			$this->_setValues = true;

		parent::InterfaceFactory($db, $db_app, $langIf);

		$this->fb->addPlugin('elc_vat_unique', 'FormBehaviourPluginElcVatUnique');
		$this->fb->addPlugin('elc_reg_months', 'FormBehaviourPluginElcRegMonths');
		$this->fb->addPlugin('elc_reg_totalsum', 'FormBehaviourPluginElcTotalSum');
		$this->fb->addPlugin('elc_reg_provdouble', 'FormBehaviourPluginElcProvDouble');
		$this->fb->addPlugin('elc_reg_startdate', 'FormBehaviourPluginElcStartdate');
		$this->fb->addPlugin('elc_reg_participant_crso', 'FormBehaviourPluginElcParticipantCrso');
		$this->fb->addPlugin('elc_vat_block', 'FormBehaviourPluginElcVATBlock');

		$crso = new CRSO($this->db_app);
		$crso->set($this->crso_id);
		$crso->relation->set();
		$crso->relation->address->set();

		$country = new Country($this->db_app);
		$country->initSettingsItemIdMap();
		$country->set($crso->relation->address->country_id);
		$this->model['crso'] =& $crso;
		$this->model['country'] =& $country;
		if(!$country->has(HasRefundRequest,$_SESSION['selected_product_id']))
			$this->exporter = false;

		$this->hiddenFields[] = array('name'=>'crso[id]', 'value'=>$this->crso_id);
		$this->hiddenFields[] = array('name'=>'product[id]', 'value'=>$_SESSION['selected_product_id']);
	}

	function init($if_id)
	{
		parent::init($if_id);

		// Alter interface item privileges for CRSO Country etc...
		if(!count($this->blocks))
			return false;

		$country = $this->model['country'];

		$hasSettings = false;
		foreach($country->settingsArray as $setting)
		{
			if(!$country->has(constant($setting),$_SESSION['selected_product_id'])) {
				$this->blocks[BLOCK_PROV2]->removeChild($country->itemmap[constant($setting)]);
			} elseif(isset($this->blocks[BLOCK_PROV2]->items[$country->itemmap[constant($setting)]])) {
				$hasSettings = true;
			}
		}
		if(!$country->has(HasRegNumberAsProducer,$_SESSION['selected_product_id']))
			$this->blocks[BLOCK_COMPANY]->removeChild($country->itemmap[HasRegNumberAsProducer]);
		else
			$this->blocks[BLOCK_COMPANY]->removeChild(579);
		if(!$hasSettings)
			$this->removeChild(BLOCK_PROV2);
		if(!$country->has(HasDocumentUpload,$_SESSION['selected_product_id']))
			$this->removeChild(BLOCK_DOCUMENTS);

		if(($this->crso_id != 163 || $_SESSION['selected_product_id'] != 1) && $this->crso_id != 221)
		{
			$this->removeChild(BLOCK_COUNTRY_SPECIFIC_TEXT);
		}



		// What the hell are we doing!
		if(!empty($_POST['process']))
		{
			$this->_processModifiers();
			if( !($this->validated = (int)$this->fb->validate()) )
				$this->thrownException .= $this->langIf->getText('Please correct the form errors');

			if ( !$this->validated ) {
				$_POST['process'] = '';
				$this->init($if_id);
			}
		}

		$this->country =& $country;
	}

	function _processModifiers()
	{
		/**
		 * This adds a formbehaviour check for vat_block
		 */
		require_once(ELC_CLASSES.'CRSOVATBlock.php');

		if(isset($_POST['admission']) && $_POST['admission'] & PARTICIPANT_PRODUCER)
			$this->fb->appendBehaviour('relation[vat_no]', sprintf('elc_vat_block(%s, %s)',$this->crso_id, CRSO_VAT_BLOCK_TYPE_NEW_PRODUCER));

		if(isset($_POST['admission']) && $_POST['admission'] & PARTICIPANT_EXPORTER)
			$this->fb->appendBehaviour('relation[vat_no]', sprintf('elc_vat_block(%s, %s)',$this->crso_id, CRSO_VAT_BLOCK_TYPE_NEW_EXPORTER));
	}

	/**
	 *
	 */
	function processInput(&$thrownException)
	{
		$this->process = 'check';
		if(isset($_POST['process']))
		{
			if( isset($this->validated) && !$this->validated )
				return $thrownException .= $this->thrownException;

			if($_POST['process'] == 'check')
			{
				if(($_POST['admission'] & PARTICIPANT_PRODUCER)
					&& ($_POST['admission'] & PARTICIPANT_EXPORTER))
					$this->process = 'secondcheck';
				else
					$this->process = 'save';
				$this->_hideInputs = true;
			}
			elseif($_POST['process'] == 'secondcheck')
			{
				$this->process = 'save';
				$this->_hideInputs = true;
			}
			elseif($_POST['process'] == 'save')
				$this->_saveInput();
		}
	}

	/**
	 */
	function _saveInput()
	{
		// start transaction
		$this->db_app->begin();

		$relation = new Relation($this->db_app, $_POST['relation']);
		$relation->relation_type = new RelationType($this->db_app, RELATION_PARTICIPANT);

		$relation->address = new Address($this->db_app, $_POST['address']);
		$relation->address->insert();

		$relation->customer_contact = new InfoPerson($this->db_app, $_POST['contact']);
		$relation->customer_contact->insert();

		$relation->invoice_language = new nLanguage($this->db_app, $_POST['invoice_language']);

		$participant = new Participant($this->db_app, $_POST['participant']);

		$participant->relation =& $relation;
		$participant->relation->insert();

		$participant->insert();

		// Invoice
		$ri = new RelationInvoice($this->db_app);
		$ri->relation =& $relation;
		if (isset($_POST['IsSameAsCompany']))
		{
			$ri->address =& $relation->address;
			$ri->name = $relation->name;
		}
		else
		{
			$ri->name =  $_POST['relation_invoice']['name'];
			$ri->address = new Address($this->db_app, $_POST['invoice_address']);
			$ri->address->insert();
		}

		$ri->info_person = new InfoPerson($this->db_app, $_POST['invoice_person']);
		$ri->info_person->insert();
		$ri->insert();

		// User
		$user =& $this->_saveUser();

		// relation user
		$ru = new RelationUser($this->db_app);
		$ru->relation =& $relation;
		$ru->user =& $user;
		$ru->is_admin = 1;
		$ru->status = 0;
		$ru->default_product = new Product($this->db_app,$_SESSION['selected_product_id']);
		$ru->insert();

		// Attach Participant to CRSO
		$crso_p = new CRSOParticipantProduct($this->db_app,
			array('crso'=>new CRSO($this->db_app, array('relation'=> new Relation($this->db_app,$this->crso_id)))));

		$cp = new CRSOParticipant($this->db_app);
		$cp->crso =& $crso_p->crso;
		$cp->participant =& $participant;

		$crso_p->producer_exporter = (int) $_POST['admission'];
		require_once(ELC_CLASSES.'Product.php');
		$crso_p->product = new Product($this->db_app,$_SESSION['selected_product_id']);
		if(isset($_POST['cp']['producer_reg_no']))
			$cp->producer_reg_no = $_POST['cp']['producer_reg_no'];
		$cp->insert();
		$crso_p->participant =& $participant;
		// dd-mm-yyyy -> yyyy-mm-dd
		if (isset($_POST['participant']['business_start_date']))
		{
			$pieces = explode('-', $_POST['participant']['business_start_date']);

			if (sizeof($pieces) == 3)
				$crso_p->setObligationStartDate($pieces[2].'-'.$pieces[1].'-'.$pieces[0]);
		}
		$crso_p->insert();


		$pb =& $this->_saveBrands($participant,$crso_p->crso);
		$this->_saveDocument($participant,$crso_p->crso);
		$this->_saveProvisional($participant,$crso_p->crso,$pb);
		// End Provisional

		require_once(ELC_CLASSES.'Notification.php');
  		Notification::send($this->db_app,
 			array('subject'=>$this->langIf->getText('Notification: New Partipant'),
 				'message'=>$this->langIf->getText("Notification: New Participant has registred")),
 			array(array(
 				'recipient'=>$crso_p->crso->relation,
 				'callback'=>"go('/".SITE_PATH."scripts/elc/crso/InterfaceRegistrantDetails.php?participant[id]=".$crso_p->participant->relation->id."')"
 			)));

		require_once(ELC_CLASSES.'ELCMessageHandler.php');
		MessageEvent::send(PARTICIPANT_REGISTRATION,
			array('user'=>$user, 'relationuser'=>$ru, 'crso'=>$crso_p->crso,
				'participant'=>$crso_p->participant), 'ELCMessageHandler');

		$this->db_app->commit();

                log_elc_event('Participant Registration Complete',
                    null, $_SESSION['selected_product_id'],
                    $crso_p->crso->relation->id, $crso_p->participant->relation->id
                );

		// goto finish page
		if (!headers_sent())
			header('location: /'.SITE_PATH.'scripts/elc/public/RegistrationFinish.php');

		die();
	}

	function &_saveBrands(&$participant,&$crso)
	{
		// If Brand doesnt exist for the current CRSO, create it;
		$pb = new ParticipantBrand($this->db_app,array('participant'=>$participant));
		$pb->brand = new Brand($this->db_app,array('crso'=>$crso));

		if (isset($_POST['brands']))
			$brandNames = array_unique($_POST['brands']);
		else
			$brandNames = array();

		foreach($brandNames as $brandName)
		{
			if(!$brandName)
				continue;
			if(!is_numeric($brandName))
			{
				$brandName = substr($brandName,4); #strip 'new_'
				if(!$pb->brand->setByNameByCRSO($brandName,null,$_SESSION['selected_product_id']));
				{
					unset($pb->brand->id);
					$pb->brand->name = $brandName;
					$pb->brand->active = 1;
					$pb->brand->product = new Product($this->db_app,$_SESSION['selected_product_id']);
					$pb->brand->insert();
				}
			}
			else
				$pb->brand->id = $brandName;
			if($pb->brand->id)
				$pb->insert();
		}
		return $pb;
		// End Brands
	}

	function &_saveUser()
	{
		$user = new ELCUser($this->db_app);
		$user->application_id = APPLICATION_ID;

		$user->info_person = new InfoPerson($this->db_app, $_POST['user']);
		$user->info_person->company_id = $_SESSION['session_user_company_id'];
		$address = new Address($this->db_app, $_POST['user_address']);
		$user->info_person->address_id = $address->insert();
		$user->info_person->insert();

		$user->insert();

		$gu = new GroupUser($this->db_app);
		$gu->user =& $user;
		$gu->group_id = GROUP_PARTICIPANT;
		$gu->insert();

		return $user;
	}

	function _saveDocument(&$participant,&$crso)
	{
		if ($this->country->settings & HasDocumentUpload)
		{
			require_once(ELC_CLASSES . 'CRSOParticipantDocument.php');
			$doc = new Document($this->db_app);
			$doc->set($_REQUEST['Documents'][ITEM_DOCUMENT_BANK]);
			$doc->temp_session_id = DB_NULL;
			$doc->update(); // set temp_session_id to NULL so it wont be trashed later

			$crso_p_doc = new CRSOParticipantDocument($this->db_app,
				array('crso'=>$crso, 'participant'=>$participant, 'document'=>$doc));

			$crso_p_doc->insert();
		}
	}

	function _saveProvisional(&$participant,&$crso,&$pb)
	{
		$pd = new ProvisionalDeclaration($this->db_app,array(
			'crso'=>$crso,
			'participant'=>$participant));
		$pd->product = new Product($this->db_app,$_SESSION['selected_product_id']);

		$date =& currentDate();
		if (($_POST['admission'] & PARTICIPANT_PRODUCER) == PARTICIPANT_PRODUCER)
		{
			if(!isset($_POST['provisional'])) // Only one year quantity was set
			{
				$ds = array('year'=>$date->getYear(),
					'quantity'=>(int)$_POST['TotalQuantity']
					);
				$pd->addLine($ds);
				$pd->insert(false);
			}
			else // Check wether or not a key is valid (month-quantity-brand etc.)
			{
				foreach($_POST['provisional'] as $index => $content)
				{
					$content['year'] = $date->getYear();
					if(isset($content['product_category'])) // Create a new product category
						$content['product_category'] = new ProductCategory($this->db,$content['product_category']);
					if(isset($content['dimension']))
						$content['dimension'] = new CRSOProductDimension($this->db,$content['dimension']);
					if(isset($content['brand'])) // Add brand if not exists
					{
						// Override participant brand for PHP5 compatibility
						$pb = new ParticipantBrand($this->db_app,array('participant'=>$participant));
						$pb->brand = new Brand($this->db_app,array('crso'=>$crso));
						if(!is_numeric($content['brand']))
						{
							$brandName = substr($content['brand'],4);
							if(!$pb->brand->setByNameByCRSO($brandName,null,$_SESSION['selected_product_id']))
							{
								$pb->brand->id = '';
								$pb->brand->name = $brandName;
								$pb->brand->active = 1;
								$pb->brand->product = new Product($this->db_app,$_SESSION['selected_product_id']);
								$pb->brand->insert();
							}
						}
						else
							$pb->brand->id = $content['brand'];
						$content['brand'] =& $pb->brand;
					}

					$pd->addLine($content);
				}
				$pd->insert(false);
			}
		}
	}

	function getInterfaceBehaviour()
	{
		ob_start();

		?>
			<script type="text/javascript" src="<?=REL_IWARE_INCLUDES?>FormBehaviour.js"></script>
			<script type="text/javascript">

				var BLOCK_PROV1 = <?=BLOCK_PROV1?>;
				var BLOCK_PROV2 = <?=BLOCK_PROV2?>;
				var BLOCK_BRANDS = <?=BLOCK_BRANDS?>;
				var BLOCK_DOCUMENTS = <?=BLOCK_DOCUMENTS?>;

				var PARTICIPANT_PRODUCER = <?=PARTICIPANT_PRODUCER?>;
				var PARTICIPANT_EXPORTER = <?=PARTICIPANT_EXPORTER?>;

				var ITEM_BUSINESS_START_DATE = <?=ITEM_BUSINESS_START_DATE; ?>;

				/**
				 * Form Behaviour
				 */
				var fb = new FormBehaviour();

				/*
				fb._addPlugin('elc_reg_months', FormBehaviourAbstractPlugin.prototype.mkPlugin(
				{
					_message: 'Please enter all required months',
					check: function()
					{
						 // - get start date, convert to date object
						 // - check of this year, else use 1-1-thisyear
						 // - make an array of months to check, loop through the elements
						 //   and delete from array if exist
						 // - only valid if array is empty
						 //

						this._status = FB_STATUS_INVALID;

						var nStartDate = getElById('ifI_' + ITEM_BUSINESS_START_DATE);

						if (!fb.nodeIsValid(nStartDate)) {
							return;
						}

						var pieces = nStartDate.value.split('-'); // dd-mm-yyyy
						if (pieces.length != 3) {
							return;
						}

						// set year
						var current_year = new Date().getFullYear();
						start_date = new Date(pieces[2], pieces[1]-1, pieces[0]);
						if (start_date.getFullYear() != current_year)
							start_date = new Date(current_year, 1, 1);

						// fill required months
						var months = new Array();
						for (var i=start_date.getMonth()-1; i<12; i++)
							months.push(i);

						// is month required, else we cant check and this is valid
						try {
							$$('provisional[0][month]');
						} catch (e) {
							this._status = FB_STATUS_VALID;
							return;
						}

						// loop elements and delete from array if set
						for (var i=0; i<100; i++)
						{
								//var sel = $$('provisional['+i+'][month]');
							var sel = document.getElementsByName('provisional['+i+'][month]')[0];

							if (!sel)
								continue;

							months.forEach(function(monthId, k, array)
							{
								if (months[k] == sel.value-1)
									delete months[k];
							}, this);
						}

						if (months.count() != 0)
							return; // return invalid

						this._status = FB_STATUS_VALID;
					}
				}));

				fb._addPlugin('elc_reg_totalsum', FormBehaviourAbstractPlugin.prototype.mkPlugin(
				{
					_message: 'The total sum does not match the quantity',
					check: function()
					{
						var total = 0;

						for (var i=0; i<100; i++)
						{
							try {
								var sel = $$('provisional['+i+'][quantity]');
							} catch (e) {
								continue;
							}

							dumpln('sel: ' + sel.name);

							total += parseInt(sel.value);
						}

						dumpln('total: ' + total);
						this._status = (total == parseInt(this.getNode().value)) ? FB_STATUS_VALID : FB_STATUS_INVALID;
					}
				}));
				*/

				/**
				 * Window onLoad
				 */
				addEvent(window,'load', function()
				{
					addEvent(getElById('interface_form'),'submit',function()
					{
						if(fb.validate(this))
						{
							this.submit();
						}
					});

					<? if(empty($_POST['process'])): ?>

						// Invoice
						toggleInvoice(document.getElementsByName('IsSameAsCompany')[0]);
						enable = (Radio_getSelectedValue('HasDirectDebit')=='yes');
						toggleDD(!enable);

						// Radio
						if (adm = Radio_getSelectedValue('admission'))
							onAdmissionClick(adm);

					<? endif; ?>
				});


				/**
				 * Registration Functions
				 */

				function onAdmissionClick(nNodeValue)
				{
					var tables = [getElById('ifB_'+BLOCK_PROV1),  // prov
						getElById('ifB_'+BLOCK_PROV2),  // prov 2
						getElById('ifB_'+BLOCK_DOCUMENTS),
						getElById('ifB_'+BLOCK_BRANDS) ]; // docs

					var show = ((nNodeValue & PARTICIPANT_PRODUCER) == PARTICIPANT_PRODUCER);

					for (var i=0; i<tables.length; i++)
					{
						try {
							disableFormNodes(tables[i], !show);
							displayNode(tables[i], show);
						} catch (e) { }
					}
				}

				function toggleDD(enable)
				{
					var dd = new Array('relation[dd_bank_account]','relation[dd_iban_no]', 'relation[dd_swift_bic_code]');

					var check = enable;

					dd.forEach(function(element, index, array)
					{
						el = document.getElementsByName(element)[0];
						el.disabled = check?'disabled':'';

						if(check)
						{
							addCssClass(el,'disabled');
							el.value = '';
						}
						else
							removeCssClass(el,'disabled');
					});
				}

				/**
				 * Invoice
				 */
				function toggleInvoice(nNode)
				{
					var invoice = new Array('relation_invoice[name]','invoice_address[country_id]',
						'invoice_address[address_street_name]','invoice_address[city_name]',
						'invoice_address[address_zipcode]');

					var check = nNode;

					invoice.forEach(function(element, index, array)
					{
						el = document.getElementsByName(element)[0];
						el.disabled = check.checked?'disabled':'';

						if(check.checked)
						{
							addCssClass(el,'disabled');
							el.value = '';
						}
						else
							removeCssClass(el,'disabled');
					})
				}

				/**
				 * Brands
				 */
				function checkProvisionalBrands()
				{
					brandSel = getElById('brands[]_selected');
					brands = new Array();

					for(var i = 0;i<brandSel.options.length;i++)
					{
						if(brandSel.options[i].selected)
							brands.push(brandSel.options[i].value);
					}
					block = getElById('ifB_'+BLOCK_PROV2);
					sels = block.getElementsByTagName('SELECT');
					for(var i=0;i<sels.length;i++)
					{
						sel = sels[i];
						if(sel.name.match(/provisional\[\d+\]\[brand\]/))
						{
							if(brands.inArray(sel.value))
							{
								MessageBox.prototype.Warning(
									'<?=jsescape($this->langIf->getText('Brands Delete Warning')); ?>',
									'<?=jsescape($this->langIf->getText('Cannot remove brand, in use by Provisional')); ?>');

								return false;
							}
						}
					}
					return true;
				}

				function addBrand(el)
				{
                    var o = document.createElement('option');
                    o.innerHTML = el.value;
                    o.value = 'new_'+el.value;
					FancySelectController.prototype.getController('brands[]').addOption(o);
				}

			</script>

		<?php
		return ob_get_clean();
	}
}

class InterfaceParticipantRegistrationCommonBlock extends InterfaceBlock
{
	function setTitle($title)
	{
		parent::setTitle($title);

		$date =& currentDate();
		if ($this->id == BLOCK_PROV1)
			$this->title .= ' ('.$date->getYear().')';
	}
}


class InterfaceParticipantRegistrationCommonItem extends InterfaceItem
{
	function getItemValue()
	{
		if($this->name == 'relation[website]')
			$this->properties['value'] = 'http://';

		if($this->name == 'cp[producer_reg_no]')
		{
			$handle = $this->handle.'_'.$this->if_obj->model['crso']->relation->name;
			// we need bytes but we dont want to break the multibyte string
			$handle = (strlen($handle)>254)?mb_strcut($handle,0,254):$handle;
			$crsoTrans = IWRegistry::registry('langIf')->getText($handle);
			$this->title = empty($crsoTrans)?$this->title:$crsoTrans;
		}
	}
	function getAdmissionSelection()
	{
		$this->options[0] = array('onclick'=>'onAdmissionClick(this.value)', 'value'=>PARTICIPANT_PRODUCER, 'text'=>$this->langIf->getText('Admission as Producer'));
		if($this->if_obj->exporter)
		{
			$this->options[1] = array('onclick'=>'onAdmissionClick(this.value)', 'value'=>PARTICIPANT_EXPORTER, 'text'=>$this->langIf->getText('Admission as Exporter'));
			$this->options[2] = array('onclick'=>'onAdmissionClick(this.value)', 'value'=>PARTICIPANT_PRODUCER + PARTICIPANT_EXPORTER, 'text'=>$this->langIf->getText('Admission as Producer and Exporter'));
		}
		else
			$this->options[0]['checked'] = 'checked';
		$this->columncount = 1;
	}

	function getCrsoInformation()
	{
		$this->text = '';
	}

	function getDirectDebit()
	{
		$this->options = array(
			0=>array('value'=>'yes', 'text'=>$this->langIf->getText('Yes'), 'onclick'=>'toggleDD(false);'),
			1=>array('value'=>'no', 'text'=>$this->langIf->getText('No'),'onclick'=>'toggleDD(true);'),
		);
		if(isset($_POST['HasDirectDebit']))
		{
			if($_POST['HasDirectDebit']=='no')
				$this->options[1]['checked'] = 'checked';
			else
				$this->options[0]['checked'] = 'checked';
		}
		else
			$this->options[1]['checked'] = 'checked';
	}
	/**
	 *
	 */
	function getCountrySelect()
	{
		$crso = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app,$this->if_obj->crso_id)));
		$crso->relation->set();
		$crso->relation->address->set();

		require_once(ELC_CLASSES.'Country.php');
		$country = new Country($this->db_app);

		// by default, select crso country, or check if the participant had a country selected (like in confirmation)
		$selected_id = $crso->relation->address->country_id;
		if (isset($this->parent->parent->address['country_id']))
			$selected_id = $this->parent->parent->address['country_id'];

		$country->fillInterfaceItem($this, $selected_id);
	}
	/**
	 *
	 */
	function getLanguageSelect()
	{
		require_once(IWARE.'nLanguage.php');
		$lang = new nLanguage($this->db_app);
		$q = $lang->get();
		$this->options = array(array('value'=>'','text'=>'- '.$this->langIf->getText('Select language').' -'));
		$i = 1;
		while($r = $q->fetch())
		{
			$this->options[$i] = array('text'=>$r->LANGUAGE,'value'=>$r->LANGUAGE_ID);
			if(!isset($this->parent->parent->userInfo))
			{
				if($r->LANGUAGE_ID == $_SESSION['session_user_language_id'])
					$this->options[$i]['selected'] = 'selected';
			}
			elseif(isset($this->parent->parent->person['language_id'])&& $this->parent->parent->person['language_id']==$r->LANGUAGE_ID)
				$this->options[$i]['selected'] = 'selected';
			$i++;
		}
	}

	function getRowControlValues()
	{
		switch ($this->name)
		{

			case 'addBrands':
				$this->removeText = $this->langIf->getText('Remove Brands');
				$this->addRowText = $this->langIf->getText('Add More Brands');
			break;

			case 'addDeclarationLine':
				$this->removeText = $this->langIf->getText('Remove Declaration Line');
				$this->addRowText = $this->langIf->getText('Add More Provisional Lines');
			break;
		}
		if(!isset($this->provisional)&&isset($_POST['provisional']))
		{
			$this->provisional = $_POST['provisional'];

			if(!empty($_POST['process']))
			{

				$this->postscript = exportArrayToJS($this->parent->parent->brandNames, 'brands').
					exportArrayToJS($this->provisional, 'prov').'

					addEvent(window,\'load\',function(){

						var intlinenumber = 0;
						prov.forEach(function(line,linenumber,lines)
						{
							line.forEach(function(item,itemname,a)
							{
								el = document.getElementsByName(\'provisional[\'+intlinenumber+\'][\'+itemname+\']\')[0];
								el.value = item;

								if ( itemname == \'product_category\' )
								{
									var text = "";
									provisional_0__product_category_Map.forEach(function(el1, i1, l1)
									{
										if (el1["value"] == item) {
											text = el1["text"];
											return false;
										}
									});
									getNextElementSibling(el).innerHTML = text;
								}
								else if ( itemname == \'dimension\' )
								{
									var text = "";
									provisional_0__dimension_Map.forEach(function(el1, i1, l1)
									{
										if (el1["value"] == item) {
											text = el1["text"];
											return false;
										}
									});
									getNextElementSibling(el).innerHTML = text;
								}
								else if ( itemname == \'month\' )
									getNextElementSibling(el).innerHTML = provisional_0__month_Map[item-1][\'text\']; // month index 0 is januari but value is 1 so we do -1
								else if(itemname == \'brand\')
								{
									if(parseInt(item)==item)
										getNextElementSibling(el).innerHTML = brands[item];
									else
										getNextElementSibling(el).innerHTML = item.substr(4);
								}
								else
									getNextElementSibling(el).innerHTML = item;
							});
							ifHandler.duplicateParentsLastRow(el, 1);
							intlinenumber++;
						});
						el = document.getElementsByName(\'provisional[\'+intlinenumber+\'][quantity]\')[0];
						ifHandler.removeParentRow(el, false);
					});
					';
			}
			else
			{ // FIXME replace [\'+intlinenumber+\'][Quantity] with the ROWCONTROL BUTTON!
				$this->postscript = exportArrayToJS($this->provisional, 'prov').'
					var intlinenumber = 0;
					var el;
					prov.forEach(function(line,linenumber,lines)
					{
						line.forEach(function(item,itemname,a)
						{
							if(itemname==\'product_category\'||itemname==\'brand\'||itemname==\'dimension\')
							{
								el = document.getElementsByName(\'provisional[\'+intlinenumber+\'][\'+itemname+\']\')[0];
								Select_setSelectedValue( el, item );
							}
							else
							{
								el = document.getElementsByName(\'provisional[\'+intlinenumber+\'][\'+itemname+\']\')[0];
								el.value = item;
							}
						});
						ifHandler.duplicateParentsLastRow(el, 1);
						intlinenumber++;
					});
					el = document.getElementsByName(\'provisional[\'+intlinenumber+\'][quantity]\')[0];
					ifHandler.removeParentRow(el, false);
					';
			}
		}
	}

	function get2colTxt()
	{
		$this->twoCols = true;
	}

	function getButtons()
	{
		if (!empty($_POST['process'])&& isset($_POST['admission']))
		{
			// confirm stuff
			if ($_POST['process'] == 'check' || $_POST['process'] == 'secondcheck')
			{
				// always hide admission
				$this->parent->parent->blocks[BLOCK_ADMISSION]->hide();

				// buttons
				$this->_BarButtons = 0;
				$this->text = $this->langIf->getText('Are above settings correct?').'<br />';
				$this->_addCustomButton($this->langIf->getText('No, I would like to make changes'),'button',
					array('onclick'=>'document.getElementsByName(\'process\')[0].value=\'\'; document.getElementById(\'interface_form\').submit();',
						'name'=>'signup'));
				$this->_addCustomButton($this->langIf->getText('Yes, I am ready to continue'),'submit',
					array('onsubmit'=>'','name'=>'signup', 'disabled'=>'disabled', 'id'=>'btnSignup'));

				// custom confirm stuff
				if ($this->parent->parent->confirm_page == CONFIRM_PRODUCER)
				{
					global $page_title;
					$page_title = 'Participant Registration Confirmation';

				}
				elseif ($this->parent->parent->confirm_page == CONFIRM_EXPORTER)
				{
					global $page_title;
					$page_title = 'Refund Requester Confirmation';

					// hide producer blocks
					foreach (array(BLOCK_PROV1, BLOCK_PROV2, BLOCK_BRANDS, BLOCK_DOCUMENTS) as $blockId)
						if (isset($this->parent->parent->blocks[$blockId]))
							$this->parent->parent->blocks[$blockId]->hide();
				}
			}
		}
		else // normal registration form
		{
			//$this->_BarButtons = BUTTON_SAVE+BUTTON_CANCEL;
			$this->_BarButtons = 0;

			$this->_addCustomButton($this->langIf->getText('Go to confirmation page'), 'button',

				array('onclick'=>'if (fb.validate($(\'interface_form\'))){ '.
					jsMessageBox(MSG_BOX_INFO, '',
						$this->langIf->getText("Please confirm your application at the next page"),
						'function() {$(\'interface_form\').submit(); }'
					).
				'}')
			);

			$this->parent->parent->blocks[BLOCK_CONFIRM]->hide();
		}
	}

	function getQuantity()
	{
		$this->properties['class'] = 'quantity';
		if(isset($_POST['provisional']))
			$_POST['provisional'] = array_values($_POST['provisional']);
	}
	function getRightColText()
	{
		$this->twoCols = true;
		$this->text = $this->langIf->getText('Invoice User Information');
	}
	function getPhoneFaxValue()
	{
		$this->getItemValue();
		if(!isset($this->properties['value'])||$this->properties['value'] == '')
			$this->properties['value'] = '+';
	}
	/**
	 *
	 */
	function getProductCategorySelect()
	{
		$d = currentDate();
		$pc = new ProductCategory($this->db_app);
		$q = $pc->selectOverview(array($_SESSION['selected_product_id']), $this->if_obj->crso_id, $d->getYear());

		$this->properties['style'] = "width: 40ex";

		$this->options = array(array('text'=>'- '.$this->langIf->getText('Select Category').' -','value'=>''));
		$i = 1;
		while($r = $q->fetch())
		{
			$this->options[$i] = array('text'=>$r->CATEGORY_NAME, 'value'=>$r->PRODUCT_CATEGORY_ID);
			$i++;
		}
	}
	function getDimensionSelect()
	{
		$d = currentDate();
		$cpd = new CRSOProductDimension($this->db_app);
		$q = $cpd->selectOverview(array($_SESSION['selected_product_id']), $this->if_obj->crso_id, $d->getYear());

		//$this->properties['style'] = 'width: 40ex';
		$this->options = array(array('text'=>'- '.$this->langIf->getText('Select Dimension').' -','value'=>''));
		$i = 1;
		while($r = $q->fetch())
		{
			$this->options[$i] = array('text'=>$r->DIMENSION_NAME, 'value'=>$r->CPD_ID);
			$i++;
		}
	}

	function getIntroText()
	{
		$this->text = $this->langIf->getText('Please select your type of business for registration at the ELC Clearinghouse');
	}

	function getBusinessStartDate()
	{
		$crso = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app,$this->if_obj->crso_id)),true);
		$crso_product = new CRSOProduct($this->db_app,array('crso'=>$crso));
		$crso_product->product = new Product($this->db_app,$_SESSION['selected_product_id']);
		$crso_product->set();
		$needles = explode('-',$crso_product->minimal_obligation_date);
		$this->properties['value'] = $needles[2].'-'.$needles[1].'-'.$needles[0];
		$this->_hideInput = true;
	}


	function getConfirmRegistration()
	{
		// FIXME ugly fix
		$this->_BarButtons = 0;

		$chk = new Form_Checkbox('');
		if (isset($this->parent->parent->confirm_page))
			$chk->setLabel( ($this->parent->parent->confirm_page == CONFIRM_PRODUCER) ?
				 $this->langIf->getText('Confirmation Checkbox Participant') :
				 $this->langIf->getText('Confirmation Checkbox Refund Requester') );

		$chk->setID('confirmCheckbox');

		$chk->onclick("getElById('btnSignup').disabled = !this.checked; ");
		$this->text = '<div align="center" style="margin: 15px;">'.$chk->getHtml().'</div>';
	}

	function getIsSameAsCompany()
	{
		$this->properties = array('onclick'=>"toggleInvoice(this);");
		$this->properties['value'] = '1';
		if(isset($_POST['IsSameAsCompany']))
			$this->properties['checked'] = 'checked';
	}

	function getMonthSelect()
	{
		for ($i=1; $i<=12; $i++)
			$this->options[] = array('text'=>strftime('%B', mktime(0, 0, 0, $i, 1)), 'value'=>$i);

	}

	/**
	 * Warning: no $this->_displayItemNoInput for fancy select is overridden in this class
	 */
	function getBrandSelect()
	{
		$this->fancyHeaders = array(0=>$this->langIf->getText('Available Brands'),1=>'',2=>$this->langIf->getText('Your Selected Brands'));

		$props = array('class'=>'button');
		$this->fancybuttons = array('right'=>$props, 'left'=>$props, 'allRight'=>$props, 'allLeft'=>$props);//, 'up'=>$props, 'down'=>$props);

		$this->properties['size'] = 8;
		$this->properties['style'] = 'width: 40ex;';

		$names = $this->options = $this->selectedoptions = array();

		$used = array();

		$b = new Brand($this->db_app);
		$q = $b->getByCRSO($this->if_obj->crso_id,$_SESSION['selected_product_id']);

		while($r = $q->fetch()) // TODO: REMOVE USED ITEMS
		{
			$names[$r->BRAND_ID] = $r->NAME;
			if(isset($_REQUEST['brands'])&&in_array($r->BRAND_ID,$_REQUEST['brands']))
				continue;
			$this->options[] = array('text'=>$r->NAME,'value'=>$r->BRAND_ID);
		}

		if(isset($_REQUEST['brands']))
		{
			foreach($_REQUEST['brands'] as $value)
			{
				if(is_numeric($value))
				{
					$this->selectedoptions[] = array('text'=>$names[$value],'value'=>$value);
					$used[] = $value;
				}
				else
					$this->selectedoptions[] = array('text'=>substr($value,4),'value'=>$value);
			}
		}

		$this->parent->parent->brandNames = $names;
	}
	/**
	 *
	 */
	function getProvisionalBrand()
	{

		$b = new Brand($this->db_app);
		$q = $b->getByCRSO($this->if_obj->crso_id,$_SESSION['selected_product_id']);

		while($r = $q->fetch())
		{
			$names[$r->BRAND_ID] = $r->NAME;
			$this->options[] = array('text'=>$r->NAME,'value'=>$r->BRAND_ID);
		}

		$descriptive_option = '';
		$this->properties = array('onclick'=>'handleProvisionalBrandOberserver(this);');

		$this->options = array(array('text'=>'- '.$this->langIf->getText('Select Brand').' -', 'value'=>$descriptive_option ));
		if(isset($_POST['brands']))
		{
			$brands = array_unique($_POST['brands']);
			foreach($brands as $value)
			{
				if(is_numeric($value))
					$this->options[] = array('text'=>$names[$value],'value'=>$value);
				else
					$this->options[] = array('text'=>substr($value,4),'value'=>$value);
			}
		}
		$this->postscript =
			'function handleProvisionalBrandOberserver(nNode)
			{
				var sNodeName = nNode.name;
			    var ob = SelectController.prototype.getController(nNode);
				if ( ob.notifyChanged )
					return; //just once

				ob.notifyChanged = function(c)
						   {
							   try {
									if ( Browser.isMSIE )
										var select = $(sNodeName);
									else
										var select = $$(sNodeName);
									if ( select == UNDEFINED )
										throw "model gone";
								   select = SelectController.prototype.getController(select);
							   } catch(e) {
									c.removeObserver(this);
									return;
							   }
							   var dopt = select.getOptionByValue(\''.$descriptive_option.'\');
							   var selected = select.getSelectedOptions()[0];
							   var found = (dopt === selected);
							   var clones = [dopt];
							   c.getSelectedOptions().forEach(function(o,i,a)
								   {
									  found = found?found:( o.value == selected.value );
									  clones.push(o.cloneNode(true));
								   });
							   if ( !found )
							   {
								   MessageBox.prototype.Warning(\''.$this->langIf->getText('WarningTitle: Brand in use.').'\',\''.
										   $this->langIf->getText('Warning: Brand is in use.').'\');
								   c.addOption(selected.cloneNode(true));
								   return;
							   }

							   select.replaceOptions(clones);
							   select.setSelectedByValue(selected.value);
						   };
				var f = FancySelectController.prototype.getController(\'brands[]\');
				f.registerObserver(ob);
				ob.notifyChanged(f);
			}';

	}
	/**
	 *
	 */
	function getNewBrand()
	{
		if(empty($_POST['process']))
		{
			$this->text = '<div style="text-align: right; padding-right: 55px;">'.$this->langIf->getText('Enter Brand Name').'
					<input type="text" style="width: 50ex;" id="newbrandinput" /><br /><br />
					<input type="button" class="button" id="addBrandButton" onclick="addBrand(document.getElementById(\'newbrandinput\'));" value="'.$this->langIf->getText('Add New Brand').'" />
				</div>';
		}
	}

	/**
	 * OVERWRITE of parent
	 */
	function _displayItemNoInput($classes=array(),$hidden)
	{
		if($this->type_id != IF_ITEM_FANCY_SELECT)
			return parent::_displayItemNoInput($classes,$hidden);


		$rv = '';
		$this->hiddenFields = array();

		// Brand Fancy Select

		$selectedValue = array();
		$val = '';
		if($this->parent->parent->_setValues)
		{
			//_data
			$names = str_replace('[]','',$this->name);
			if(isset($_POST[$names]))
			{
				$val .= '<ul>';
				foreach($_POST[$names] as $value)
				{
					if(!in_array($value, $selectedValue))
					{
						$this->hiddenFields[] = array('name'=>$names.'[]','value'=>$value);
						if(is_numeric($value))
						{
							// Set brand
							$b = new Brand($this->db_app);
							$b->set($value);
							$val .= '<li><span>'.$b->name.'</span></li>';
						}
						else
							$val .= '<li><span>'.substr($value,4).'</span></li>';

						$selectedValue[] = $value;
					}
				}
				$val .= '</ul>';
			}
		}
		$rv[] = $hidden.$this->_parseHiddenFields().$val;
		return $rv;
	}

	function getCountrySpecificText()
	{
		switch($this->if_obj->crso_id)
		{
			case 163:
				$this->text = "Estimated by converting the units into weight using industry average figures the conversion factor is: 100 units = 13.1 kg If you know the weight of WEEE Gas discharge lamps accurately, please enter that data instead [under Reported Weight]";
				break;
			case 221:
				$this->text = '

<style>

						table.Professional tr td .hungary-text td {
							background-color: #EEE !important;
						}

						.hungary-text td {
							border: 1px solid #ccc;
							border-collapse: collapse;

						}

						.hungary-text tr:nth-child(1) td {
							font-weight: bold !important;
						}

						table.Professional tr td .hungary-text tr:nth-child(even) td {
							background-color: #e5e5e5 !important;
						}


					</style>

					<table class="hungary-text">


						<TR>
							<TD    VALIGN=MIDDLE>E+E berendezés csoport kódja</TD>
							<TD   VALIGN=MIDDLE>E+E berendezés csoport rövid leírása:</TD>
							<TD   VALIGN=MIDDLE>Az E+E berendezéscsoporthoz tartozó Vámtarifaszámok:</TD>
							<TD   VALIGN=MIDDLE>A Vámtarifaszám megnevezése:</TD>
						</TR>
						<TR>
							<TD    VALIGN=MIDDLE>TL lámpák</TD>
							<TD   VALIGN=MIDDLE>Fénycső,Két végén fejelt bármilyen formájú</TD>
							<TD   VALIGN=MIDDLE SDVAL="85393110" SDNUM="1033;">85393110</TD>
							<TD   VALIGN=MIDDLE>Fénycső, izzókatóddal, Két végén fejelt bármilyen formájú</TD>
						</TR>
						<TR>
							<TD    VALIGN=MIDDLE>CFL lámpák</TD>
							<TD   VALIGN=MIDDLE>Fénycső, Egy végén fejelt: Energiatakarékos lámpa (kompakt fénycső)</TD>
							<TD   VALIGN=MIDDLE SDVAL="85393190" SDNUM="1033;">85393190</TD>
							<TD   VALIGN=MIDDLE>Fénycső, izzókatóddal, Más: Egy végén fejelt: Energiatakarékos lámpa (kompakt fénycső)</TD>
						</TR>
						<TR>
							<TD  ROWSPAN=6   VALIGN=MIDDLE>Speciális lámpák</TD>
							<TD  ROWSPAN=6  VALIGN=MIDDLE>Speciális, kisüléses elven működő lámpák</TD>
							<TD   VALIGN=MIDDLE SDVAL="85393210" SDNUM="1033;">85393210</TD>
							<TD   VALIGN=MIDDLE>Higanylámpa: Higany-, és kevertfényű lámpák</TD>
						</TR>
						<TR>
							<TD   VALIGN=MIDDLE SDVAL="85393250" SDNUM="1033;">85393250</TD>
							<TD   VALIGN=MIDDLE>Nátriumlámpa: Nagynyomású és kisnyomású nátriumlámpák</TD>
						</TR>
						<TR>
							<TD   VALIGN=MIDDLE SDVAL="85393290" SDNUM="1033;">85393290</TD>
							<TD   VALIGN=MIDDLE>Fémhalogén lámpa</TD>
						</TR>
						<TR>
							<TD   VALIGN=MIDDLE SDVAL="85393900" SDNUM="1033;">85393900</TD>
							<TD   VALIGN=MIDDLE>Más: hidegkatódos fénycsövek (CCFL) vagy külső elektródos fénycsövek (EEFL)</TD>
						</TR>
						<TR>
							<TD   VALIGN=MIDDLE SDVAL="85394910" SDNUM="1033;">85394910</TD>
							<TD   VALIGN=MIDDLE>Más kisülési cső, ultraibolya lámpa</TD>
						</TR>
						<TR>
							<TD   VALIGN=MIDDLE SDVAL="85437050" SDNUM="1033;">85437050</TD>
							<TD   VALIGN=MIDDLE>Napozólámpák, amelyek gázkisüléses elven működőnek</TD>
						</TR>
						<TR>
							<TD  ROWSPAN=4   VALIGN=MIDDLE>LED lámpák</TD>
							<TD  ROWSPAN=4  VALIGN=MIDDLE>LED-ek, LED-es retrofit (a hagyományos izzókkal és a Hg tartalmú lámpákkal csereszabatos és fejeléssel ellátott) lámpák, és az úgynevezett LED modulok: olyan LED-es termékek amelyeket a végfelhasználó a lámpatestben cserélni tud</TD>
							<TD   VALIGN=MIDDLE SDVAL="85414010" SDNUM="1033;">85414010</TD>
							<TD   VALIGN=MIDDLE>Fénykibocsátó dióda, LED-es termék</TD>
						</TR>
						<TR>
							<TD   VALIGN=MIDDLE SDVAL="85437090" SDNUM="1033;">85437090</TD>
							<TD   VALIGN=MIDDLE>&quot;Másutt nem említett elektromos gép és készülék egyedi feladatokra, Más.&quot; Olyan LED-es termék melyet ez alatt a vámtarifaszámon hoztak be Magyarországra.</TD>
						</TR>
						<TR>
							<TD   VALIGN=MIDDLE SDVAL="9405" SDNUM="1033;">9405</TD>
							<TD   VALIGN=MIDDLE>&quot;Lámpatest és világítóberendezés&quot;: olyan LED-es termék melyet ez alatt a vámtarifaszámon hoztak be Magyarországra.</TD>
						</TR>
						<TR>
							<TD   VALIGN=MIDDLE >85391000</TD>
							<TD   VALIGN=MIDDLE>Zárt betétes fényszóró egység (lámpa), angolul: sealed beam lamp unit. Olyan LED-es termék melyet ez alatt a vámtarifaszámon hoztak be Magyarországra.</TD>
						</TR>

					</table>


							';
				break;
			default:
				$this->text = "";
		}
	}
}




?>
