<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'ELCChart.php');

define('BLOCK_LISTS',345);
define('BLOCK_PUT_ON_MARKET',346);
define('BLOCK_VISIBLE_FEE',347);
define('BLOCK_OUTSTANDING_RECEIVABLES',348);
define('BLOCK_COLLECTED_RECEIVABLES',349);
define('BLOCK_CRSO_SELECT',352);

/**
 *
 * @package
 * @subpackage Interface
 * @since 02-08-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceParticipantReport extends InterfaceFactory
{
	var $_hiddenBlocks = array();
    /**
     * interface[id] = 143
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceParticipantReport(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);

		require_once(IWARE . 'group.php');
		$this->model['crso'] = false;
		switch ($_SESSION['session_group_group_id'])
		{
			case GROUP_CRSO:
				$this->model['crso'] = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, $_SESSION['crso_relation_id'], true)));

				require_once(ELC_CLASSES.'RelationUser.php');
				$ru = new RelationUser($this->db_app);
				$ru->set($_SESSION['session_user_id'], $_SESSION['crso_relation_id']);

				$this->model['hasEnoughParticipantsCheck'] = $ru->hasClearance(CRSOClearanceDeclarationDetails) ? 'FALSE' : 'TRUE';
				$this->model['hasEnoughParticipantsCheckRefund'] = $ru->hasClearance(CRSOClearanceRefundDetails) ? 'FALSE': 'TRUE';
			break;
			case GROUP_ACCOUNTANT:
			case GROUP_SYSADMIN:
				if(!empty($_REQUEST['crso']['id'])&&is_numeric($_REQUEST['crso']['id']))
					$this->model['crso'] = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, (int)$_REQUEST['crso']['id'], true)));
				$country_ids = isset($country_ids)?$country_ids:'';
				$this->model['crsos'] = array();
				// fill available crsos
				if($_SESSION['session_group_group_id'] == GROUP_SYSADMIN){
					$crso = new CRSO($this->db_app);
					$q = $crso->selectCRSOs(1,$country_ids,$_SESSION['selected_product_id']);
				}
				else {
					require_once(ELC_CLASSES.'AccountantCRSO.php');
					$accountant_crso = new AccountantCRSO($this->db_app);
					$q = $accountant_crso->getAccountantCRSOs($_SESSION['accountant_relation_id'],$_SESSION['selected_product_id']);
				}
				$rs = $q->fetchAll();
				if(count($rs) == 1){
					$_REQUEST['crso']['id'] = $rs[0]->CRSO_RELATION_ID;
					$this->model['crso'] = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, (int)$_REQUEST['crso']['id'], true)));
				}
				foreach ($rs as $r){
					$this->model['crsos'][] = array($r->CRSO_RELATION_ID, $r->NAME);
					$id = $r->CRSO_RELATION_ID;
				}

				$this->model['hasEnoughParticipantsCheck'] = 'FALSE';
				$this->model['hasEnoughParticipantsCheckRefund'] = 'FALSE';
			break;
			default:
				alert('Access denied to InterfaceParticipantReport');
				throw new InterfacePermissionException('Incorrect group to access Management Report');
		}

		$this->_hiddenBlocks = array(BLOCK_OUTSTANDING_RECEIVABLES,BLOCK_COLLECTED_RECEIVABLES);

		if($this->model['crso'])
			setPageTitlePostfix(sprintf('(%s)', $this->model['crso']->relation->name));
		else
			$this->_hideEverythingButCRSOSelect();


        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        $date =& currentDate();

        $m = isset($_REQUEST['month'])?(int)$_REQUEST['month']:$date->getMonth();
        //$m = ($m==0)?12:$m;
        $this->model['year'] = isset($_REQUEST['year']) ? (int)$_REQUEST['year']:$date->getYear();

        $this->model['current_month'] = $m+1;
        $this->model['month'] = $m;
        $this->model['current_month_name'] = strftime('%B', mktime(0, 0, 0,  $this->model['current_month'], 1));
        $this->model['month_name'] = strftime('%B', mktime(0, 0, 0, $this->model['month'], 1));
        $this->model['last_month_name'] = strftime('%B', mktime(0, 0, 0, ($this->model['month']-1), 1));
        $this->model['january'] = strftime('%B', mktime(0, 0, 0, 1, 1));
        $this->model['other_months'] = $this->langIf->getText('other months');
    }

    function _hideEverythingButCRSOSelect()
    {
		$this->_hiddenBlocks = array(
			BLOCK_LISTS,
			BLOCK_PUT_ON_MARKET,
			BLOCK_VISIBLE_FEE,
			BLOCK_OUTSTANDING_RECEIVABLES,
			BLOCK_COLLECTED_RECEIVABLES
			);
    }

	function _filter($q)
	{
		$rv = parent::_filter($q);

    	foreach($this->_hiddenBlocks as $blockId)
			unset($rv[$this->id][$blockId]);

    	return $rv;
    }

    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {

    }

    function getInterfaceHeader()
    {
    	$rv = parent::getInterfaceHeader();

    	// FIXME: make dynamic
    	$month_prev_inv = $this->model['month_name'];
    	$month_prev_decl = $this->model['last_month_name'];
    	$month_curr_inv = $this->model['current_month_name'];
    	$month_curr_decl = $this->model['month_name'];

		$rv .= '
			<table class="formHeader" width="50%">
				<tr>
					<td width="50%">&nbsp;</td>
					<th width="25%">'.$this->langIf->getText('Declaration').'</th>
					<th width="25%">'.$this->langIf->getText('Invoice').'</th>
				</tr>
				<tr>
					<td>'.$this->langIf->getText('Previous Month').'</td>
					<td>'.$month_prev_decl.'</td>
					<td>'.$month_prev_inv.'</td>
				</tr>
				<tr>
					<td>'.$this->langIf->getText('Current Month').'</td>
					<td>'.$month_curr_decl.'</td>
					<td>'.$month_curr_inv.'</td>
				</tr>
			</table>
			<br />
		';

		return $rv.$this->langIf->getText(
				'ManagementReport Introduction Text {PRESENT_MONTH} {PREVIOUS_MONTH}',
				false,
				array('PRESENT_MONTH' => $this->model['current_month_name'],
				'PREVIOUS_MONTH' => $this->model['month_name'])
		).'<br /><br />';
    }
}

require_once(ELC_CLASSES.'ELCReportQuantityTable.php');
class ELCReportBreakdownStatisticalDeclaration extends ELCReportQuantityTable
{
	function getData()
	{
		$this->setTitles('Breakdown by Statistical Period (Declared)','In Units');
		$this->_numtype = 'Int';

		$dbh = IWRegistry::registry('db');
		$stmt = $dbh->prepare(sprintf("CALL sp_elcReportBreakdownStatisticalDeclaration(?,?,?,%s,?)",$this->getProperty('hasEnoughParticipantsCheck')));
		$stmt->bind_values(array($this->getProperty('crso')->relation->id,$this->getProperty('month'),$this->getProperty('year'),$_SESSION['selected_product_id']));
		$stmt->execute();
		$breakBefore = array('Total Declared','Total declared in the month','Total declared in later periods','impact late declarations');

		$this->setTotalRow(3);
		$this->setTotalRow(10);
		$this->setTotalRow(12);

		return $this->getDataSet($stmt,'last_month_name','month_name',$breakBefore,array('Breakdown','credit','normal','late','resubmit'));
	}
}

class ELCReportBreakdownInvoicedDeclaration extends ELCReportQuantityTable
{
	function getData()
	{
		$this->setTitles('Breakdown by Invoiced Period','In Units');
		$this->_numtype = 'Int';
		$this->hidePrevOn = 13;
		$this->hideCurrOn = 12;

		$dbh = IWRegistry::registry('db');
		$stmt = $dbh->prepare(sprintf("CALL sp_elcReportBreakdownInvoicedDeclaration(?,?,?,%s,?)",$this->getProperty('hasEnoughParticipantsCheck')));
		$stmt->bind_values(array($this->getProperty('crso')->relation->id,$this->getProperty('month'),$this->getProperty('year'),$_SESSION['selected_product_id']));
		$stmt->execute();
		$breakBefore = array('Total Invoiced','Invoiced in the month (total for period)','Corrections for previous periods');

		$this->setTotalRow(3);
		$this->setTotalRow(9);

		return $this->getDataSet($stmt,'month_name','current_month_name',$breakBefore,array('Total in month'));
	}
}

class ELCReportVisibleFeeDeclaration extends ELCReportQuantityTable
{
	function getData()
	{
		$this->setTitles('Visible Fee - Put on Market','All amounts in Local Currency');
		$this->_numtype = 'Float';
		$this->hidePrevOn = 13;
		$this->hideCurrOn = 12;

		$dbh = IWRegistry::registry('db');
		$stmt = $dbh->prepare(sprintf("CALL sp_elcReportVisibleFeeDeclaration(?,?,?,%s,?)",$this->getProperty('hasEnoughParticipantsCheck')));
		$stmt->bind_values(array($this->getProperty('crso')->relation->id,$this->getProperty('month'),$this->getProperty('year'),$_SESSION['selected_product_id']));
		$stmt->execute();
		$this->setTotalRow(0);

		return $this->getDataSet($stmt,'month_name','current_month_name');
	}
}

class ELCReportVarianceAnalysisStatisticalPeriod extends ELCReportQuantityTable
{
	function getData()
	{
		$this->setTitles('Variance analysis by statistical period','In Units');
		$this->_numtype = 'Int';

		$dbh = IWRegistry::registry('db');
		$stmt = $dbh->prepare("CALL sp_elcReportVarianceAnalysisStatisticalPeriod(?,?,?,?)");
		$stmt->bind_values(array($this->getProperty('crso')->relation->id,$this->getProperty('month'),$this->getProperty('year'),$_SESSION['selected_product_id']));
		$stmt->execute();
		$breakBefore = array('In percentage');

		$this->setTotalRow(2);
		$this->setTotalRow(4);

		return $this->getDataSet($stmt,'last_month_name','month_name',$breakBefore);
	}
}

class ELCReportVarianceAnalysisInvoicedPeriod extends ELCReportQuantityTable
{
	function getData()
	{
		$this->setTitles('Variance analysis by invoiced period','All amounts in Local Currency');
		$this->_numtype = 'Float';
		$this->hidePrevOn = 13;
		$this->hideCurrOn = 12;

		$dbh = IWRegistry::registry('db');
		$stmt = $dbh->prepare("CALL sp_elcReportVarianceAnalysisInvoicedPeriod(?,?,?,?)");
		$stmt->bind_values(array($this->getProperty('crso')->relation->id,$this->getProperty('month'),$this->getProperty('year'),$_SESSION['selected_product_id']));
		$stmt->execute();
		$breakBefore = array('In percentage');

		$this->setTotalRow(2);
		$this->setTotalRow(4);

		return $this->getDataSet($stmt,'month_name','current_month_name',$breakBefore);
	}
}

class ELCReportReconciliationDeclarationVsInvoicedPeriod extends ELCReportQuantityTable
{
	function getData()
	{
		$this->setTitles('Reconciliation declaration vs invoiced period','In Units');
		$this->_numtype = 'Int';
		$this->hidePrevOn = 14;
		$this->hideCurrOn = 13;

		$dbh = IWRegistry::registry('db');
		$stmt = $dbh->prepare("CALL sp_elcReportReconciliationDeclarationVsInvoicedPeriod(?,?,?)");
		$stmt->bind_values(array($this->getProperty('crso')->relation->id,$this->getProperty('year'),$_SESSION['selected_product_id']));
		$stmt->execute();

		$this->setTotalRow(0);

		return $this->getDataSet($stmt,'january','other_months');
	}
}

class ELCReportBreakdownStatisticalRefund extends ELCReportQuantityTable
{
	function getData()
	{
		$this->setTitles('Breakdown by Statistical Period (Requested)','In Units');
		$this->_numtype = 'Int';

		//$dbh = IWRegistry::registry('db');
		//$stmt = $dbh->prepare(sprintf("CALL sp_elcReportBreakdownStatisticalRefund(?,?,?,%s,?)",$this->getProperty('hasEnoughParticipantsCheckRefund')));
		//$stmt->bind_values(array($this->getProperty('crso')->relation->id,$this->getProperty('month'),$this->getProperty('year'),$_SESSION['selected_product_id']));
		//$stmt->execute();
		//FIXME this sp is borked
		//return $this->getDataSet($stmt,'last_month_name','month_name');
		return array();
	}
}

class ELCReportBreakdownInvoicedRefund extends ELCReportQuantityTable
{
	function getData()
	{
		$this->setTitles('Refund Request - Breakdown by Invoiced Period','In Units');
		$this->_numtype = 'Int';
		$this->hidePrevOn = 13;
		$this->hideCurrOn = 12;

		$dbh = IWRegistry::registry('db');
		$stmt = $dbh->prepare(sprintf("CALL sp_elcReportBreakdownInvoicedRefund(?,?,?,%s,?)",$this->getProperty('hasEnoughParticipantsCheckRefund')));
		$stmt->bind_values(array($this->getProperty('crso')->relation->id,$this->getProperty('month'),$this->getProperty('year'),$_SESSION['selected_product_id']));
		$stmt->execute();

		return $this->getDataSet($stmt,'month_name','current_month_name');
	}
}

class ELCReportVisibleFeeRefund extends ELCReportQuantityTable
{
	function getData()
	{
		$this->setTitles('Refund Request - Visible Fee','All amounts in Local Currency');
		$this->_numtype = 'Float';
		$this->hidePrevOn = 13;
		$this->hideCurrOn = 12;

		$dbh = IWRegistry::registry('db');
		$stmt = $dbh->prepare(sprintf("CALL sp_elcReportVisibleFeeRefund(?,?,?,%s,?)",$this->getProperty('hasEnoughParticipantsCheckRefund')));
		$stmt->bind_values(array($this->getProperty('crso')->relation->id,$this->getProperty('month'),$this->getProperty('year'),$_SESSION['selected_product_id']));
		$stmt->execute();
		$this->setTotalRow(4);

		return $this->getDataSet($stmt,'month_name','current_month_name',array('Total'));
	}
}

/**
 * @package
 * @subpackage Interface
 * @since 02-08-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceParticipantReportItem extends InterfaceItem
{
	public static $trans;
	/**
     * item[1065] =
     */
    function getLists()
    {

    	// set up general variables for all lists
    	ELCReportQuantityTable::setUp($this->langIf, $this->db_app,
    		array('month'=>$this->if_obj->model['month'],
    			'hasEnoughParticipantsCheck'=>$this->if_obj->model['hasEnoughParticipantsCheck'],
    			'hasEnoughParticipantsCheckRefund'=>$this->if_obj->model['hasEnoughParticipantsCheckRefund'],
    			'year'=>$this->if_obj->model['year'],
				'crso'=>$this->if_obj->model['crso'],
				'month_name'=>$this->if_obj->model['month_name'],
				'current_month_name'=>$this->if_obj->model['current_month_name'],
				'january'=>$this->if_obj->model['january'],
				'other_months'=>$this->if_obj->model['other_months'],
				'last_month_name'=>$this->if_obj->model['last_month_name'])); // properties

    	$tfdata = array();
		// New Setup:

		// Declaration:
		// ROW1
		$row = array();
		// Breakdown by Statistical Period (DECLARED)
		$list = new ELCReportBreakdownStatisticalDeclaration();
		$row[] = $list->getHtml();
		// Breakdown by Invoiced Period
		$list = new ELCReportBreakdownInvoicedDeclaration();
		$row[] = $list->getHtml();
		$tfdata[] = $row;

		$row = array('');
		$listAmount = new ELCReportVisibleFeeDeclaration();
		$row[] = $listAmount->getHtml();
		$tfdata[] = $row;

		// ROW2
		// Declarations
		$row = array();
		// Variance Analysis by Statistical Period
		$list = new ELCReportVarianceAnalysisStatisticalPeriod();
		$row[] = $list->getHtml();
		// Variance Analysis by Invoiced Period
		$list = new ELCReportVarianceAnalysisInvoicedPeriod();
		$row[] = $list->getHtml();
		$tfdata[] = $row;

		// ROW3
		// Declarations
		$row = array('');
		// Reconciliation declaration vs invoiced period
		$list = new ELCReportReconciliationDeclarationVsInvoicedPeriod();
		$row[] = $list->getHtml(array($this,'thReconProc'));
		$tfdata[] = $row;

	$t = new TableFactory($tfdata);
	$t->decapitate();
	$t->setCaption(IWRegistry::registry('langIf')->getText('Put on the Market'));
	$t->setValueProcessor(TF_ALL, array($this, '_tfValueProcessor'));
		$this->text .= $t->getXHTMLTable('');

		$tfdata = array();

		// ROW4
		// Refund Request:
		$row = array('');
		// Breakdown by Invoided Period
		$list = new ELCReportBreakdownInvoicedRefund();
		$row[] = $list->getHtml();
		$tfdata[] = $row;


		// ROW5
		$row = array('');
		// Visible Fee - Refund Requests
		$list = new ELCReportVisibleFeeRefund();
		$row[] = $list->getHtml();
		$tfdata[] = $row;

		// give lists
    	$t = new TableFactory($tfdata);
    	$t->decapitate();
	$t->setCaption(IWRegistry::registry('langIf')->getText('Refund Requests'));
    	$t->setValueProcessor(TF_ALL, array($this, '_tfValueProcessor'));
	$this->text .= $t->getXHTMLTable('');
    }

	function thReconProc(&$args)
	{
		if($args['head'] == 'YTD')
			$args['headerText'] = IWRegistry::registry('langIf')->getText('Total for Year');
		if ($args['j'] == 0)
			return 'style="visibility: hidden;"';
		else
			return 'style="text-align: right; width: 100px;"';
	}

	function _tfValueProcessor($args)
	{
		$args['extra'] = ' width="50%" ';
		return $args['value'];
	}

    function getCRSOSelect()
    {
    	$crso_id = ($this->if_obj->model['crso'] instanceof CRSO) ? $this->if_obj->model['crso']->relation->id : '';
    	$month = $this->if_obj->model['month'];

    	$this->options[] = array('text'=>'- '.$this->langIf->getText('Select CRSO').' -', 'value'=>'');
    	foreach ($this->if_obj->model['crsos'] as $crso) {
    		list ($id, $name) = $crso;
    		$name = (mb_strlen($name)>35)?mb_substr($name,0,35).'...':$name;
    		$this->options[] = array('text'=>$name, 'value'=>$id, 'selected'=>($id == $crso_id));
    	}

    	//$this->properties['onchange'] = "_LoadOverview();";
    	if(preg_match('/\/scripts\/(.*)\/(.*)\/([\w]+)\.php/', $_SERVER['REQUEST_URI'],$matches))
    		$this->properties['onchange'] = sprintf("go('%s?month=%s&year=%s&crso[id]='+this.value);", $matches[0], $month, $this->if_obj->model['year']);

    }

    function getMonthSelect()
    {
    	$crso_id = ($this->if_obj->model['crso'] instanceof CRSO) ? $this->if_obj->model['crso']->relation->id : '';
    	if(!$crso_id)
    		$this->properties['disabled'] = 'disabled';
    	$month = $this->if_obj->model['month'];

    	$this->options = array();

    	for($i=1;$i<=12;$i++)
    	{
    		$this->options[$i-1] = array('text'=>strftime('%B', mktime(1, 1, 1, $i, 1)), 'value'=>$i);
    		if($i == $month)
    			$this->options[$i-1]['selected'] = 'selected';
    	}
    	preg_match('/\/scripts\/(.*)\/(.*)\/([\w]+)\.php/', $_SERVER['REQUEST_URI'],$matches);
    	$this->properties['onchange'] = sprintf("go('%s?crso[id]=%s&month='+this.value+'&year=%s');", $matches[0],$crso_id,$this->if_obj->model['year']);
    }

    /**
     * item[1116] =
     */
    function get2colTxt()
    {
    	$this->twoCols = true;
    }

    /**
     * item[1117] =
     */
    function getYearSelect()
    {
    	$crso_id = ($this->if_obj->model['crso'] instanceof CRSO) ? $this->if_obj->model['crso']->relation->id : '';

    	$date = currentDate();
		$year = (int)$date->getYear();

    	if($year == WEEE_APPLICATION_START_YEAR)
    		$this->_hideInput = true;

    	$this->options = array();

    	for($i=WEEE_APPLICATION_START_YEAR; $i<=$year; $i++)
			$this->options[] = array('value'=>$i,'text'=>$i,'selected'=>($i==$this->if_obj->model['year']?'selected':false));

		preg_match('/\/scripts\/(.*)\/(.*)\/([\w]+)\.php/', $_SERVER['REQUEST_URI'],$matches);
		$this->properties['onchange'] = sprintf("go('%s?crso[id]=%s&month=%s&year='+this.value);", $matches[0],$crso_id,$this->if_obj->model['month']);
    }

    /**
     * item[1066] =
     * item[1067] =
     * item[1068] =
     * item[1069] =
     */
    function getChart()
    {
    	if(!isset(self::$trans))
    		self::$trans = ELCChart::generateTranslations($this->langIf);

    	$html = $this->_getChartImage();
    	$html .= $this->_getChartDescription();
    	$this->text = $html;
    }

	function _getChartImage()
	{
		$type = 0;
		switch ($this->name)
		{
			case 'collectedReceivables':
				return 'Currently not available';
				$type = ELC_CHART_TYPE_COLLECTED_RECEIVABLES; // fixme
			break;
			case 'outstandingReceivables':
				return 'Currently not available';
				$type = ELC_CHART_TYPE_OUTSTANDING_RECEIVABLES; // fixme
			break;
			case 'putOnMarket':
				$type = ELC_CHART_TYPE_PUT_ON_MARKET;
			break;
			case 'visbleFee':
				$type = ELC_CHART_TYPE_VISIBLE_FEE;
			break;
		}

		return sprintf('<img src="/'.SITE_PATH.'scripts/elc/bots/ChartFactory.php?chart_type=%s&crso[id]=%s&year=%s" />',
			$type, $this->if_obj->model['crso']->relation->id, $this->if_obj->model['year']);
	}

	function _getChartDescription()
	{
		return sprintf('<p>%s</p>', $this->langIf->getText('Chart Description: '.$this->name));
	}

}

/**
 * CALLBACK[getLists]
 * item[1065] = {
 *     [handle] = Item: Lists
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getPutOnMarket]
 * item[1066] = {
 *     [handle] = Item: PutOnMarket
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getVisbleFee]
 * item[1067] = {
 *     [handle] = Item: VisibleFee
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getOutstandingReceivables]
 * item[1068] = {
 *     [handle] = Item: OutstandingReceivables
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getCollectedReceivables]
 * item[1069] = {
 *     [handle] = Item: CollectedReceivables
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */

?>
