<?php
require_once(INTERFACE_OBJ."InterfaceFactory.php");
require_once(ELC_CLASSES.'Country.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'Document.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'ProofOfPayment.php');
require_once(ELC_CLASSES.'RelationUser.php');
require_once(ELC_CLASSES.'RelationDocument.php');
require_once(ELC_CLASSES.'FormBehaviourPlugins.php');
require_once(ELC_CLASSES.'Product.php');
require_once(IWARE.'group.php');

define('ProofOfPaymentBlock',167);
define('ProofOfPaymentMultilineBlock', 510);
define('ProofOfPaymentMultilineLinesBlock', 511);


define('VATInputItem',859);
define('VATInputItemMultiple', 1488);

define('ExportDocumentItem',860);

define('SupplyChainBlock1', 416);
define('SupplyChainBlock2', 417);

define('ItemSelectProducer', 864);
define('ItemSelectProducerMultiline', 1491);

define('ItemCompany', 885);
define('ItemCompanyMultiple', 1494);

/**
 *
 * @package ELC
 * @subpackage Interface
 * @since 12-05-2006
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfacePaymentForm extends InterfaceFactory
{
	var $availableCRSOParticipants = array();
	/**
	 * interface[id] = 63
	 * @param $db Database (I_WARE_SETTINGS)
	 * @param $db_app Database (APPLICATION DATABASE)
	 * @param $langIf LanguageInterface
	 */
	function InterfacePaymentForm(&$db, &$db_app, &$langIf)
	{
		parent::InterfaceFactory($db, $db_app, $langIf);
		$this->FirstColWidth = 25;
		$this->delimiter = '<br />';
		$this->hiddenFields = array();

		validatePOSTLength();

		// see if we can get the environment from request
		$pop_id = (isset($_REQUEST['pop']['id'])) ? (int)$_REQUEST['pop']['id'] : null;

		// Specific interface settings for different groups
		$participant_id = $crso_id = $product_id = null;
		switch ($_SESSION['session_group_group_id'])
		{
			case GROUP_PARTICIPANT:
				$participant_id = $_SESSION['selected_participant_id'];
				$product_id = $_SESSION['selected_product_id'];
				$crso_id = 	(isset($_REQUEST['crso']['id'])) ? (int)$_REQUEST['crso']['id'] : null;

				// if we have a pop we should be able to get the crso
				if ($pop_id)
				{
					$pop = new ProofOfPayment($this->db_app, (int)$_REQUEST['pop']['id'], true);
					$crso_id = $pop->crso->relation->id;
					if(!isset($_SESSION['participant_ids'][$pop->participant->relation->id]))
						throw new InterfacePermissionException('Linked PoP is not of an authorized Participant');
				}
				else
				{
					if(!isset($_REQUEST['participant']['id'])||!isset($_REQUEST['crso']['id']))
						throw new InterfacePermissionException('Invalid Request');
				}
			break;
			case GROUP_CRSO:

				$this->_hideInputs = true;

				if ($pop_id)
				{
					$pop = new ProofOfPayment($this->db_app, (int)$_REQUEST['pop']['id'], true);
					$crso_id = $pop->crso->relation->id;
					if($_SESSION['crso_relation_id']!=$crso_id) {
						throw new InterfacePermissionException('No permission');
					}
					$participant_id = $pop->participant->relation->id;
				}

				break;
			case GROUP_ACCOUNTANT:
				$this->_hideInputs = true;

				if ($pop_id)
				{
					require_once(ELC_CLASSES.'AccountantCRSO.php');

					$ac = new AccountantCRSO($this->db_app);
					$q = $ac->getAccountantCRSOs($_SESSION['accountant_relation_id'], $_SESSION['selected_product_id']);
					$crsos = array();
					while($r = $q->fetch())
						$crsos[] = $r->CRSO_RELATION_ID;
					$pop = new ProofOfPayment($this->db_app, (int)$_REQUEST['pop']['id'], true);
					$crso_id = $pop->crso->relation->id;
					if(!in_array($crso_id, $crsos)) {
						throw new InterfacePermissionException('No permission');
					}
					$participant_id = $pop->participant->relation->id;
				}

				break;
			default:
				throw new InterfacePermissionException('No permission');
		}

		// build a cpp object so we can pass it to the controller
		$cpp = new CRSOParticipantProduct(IWRegistry::registry('db'));
		if(!$cpp->set($crso_id, $participant_id, $_SESSION['selected_product_id']))
			throw new InterfacePermissionException('Incorrect values for CRSO-Participant-Product');

		// init controller
		require_once(ELC_CLASSES.'ProofOfPaymentController.php');
		$this->model['controller'] = ProofOfPaymentController::instance($cpp, new ELCUser($this->db_app, $_SESSION['session_user_id']), $this->fb);

		if($pop_id)
		{
			$this->hiddenFields[] = array('name'=>'pop[id]','value'=>$pop_id);
			$pop = $this->model['controller']->getProofOfPayment($pop_id);

			// if the status is not editable, hide the inputs
			if ($pop->status != PROOF_OF_PAYMENT_PENDING && $pop->status != PROOF_OF_PAYMENT_REJECTED)
				$this->_hideInputs = true;
		}
		else
		{
			 // make a new empty pop with one line
			$pop = new ProofOfPayment($this->db_app);
			$pop->participant = new Participant($this->db_app,array('relation'=>new Relation($this->db_app, $participant_id)));
			$pop->crso = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app, $crso_id)));

			$pop->lines[1] = new ProofOfPaymentLine($this->db, array('proof_of_payment'=>$pop,'line_index'=>1));

		}

		// register some objects
		$this->model['cpp'] = $this->model['controller']->getCpp();
  		$this->model['pop'] = $pop;

		// Country settings
	   	$relation = new Relation($this->db_app,$this->model['pop']->crso->relation->id,true);
	   	$this->model['country'] = new Country($this->db_app,$relation->getCountry(),true);
	   	$this->model['country']->initSettingsItemIdMap();

		// init pop
  		$pop->participant->relation->set();
  		$pop->participant->relation->address->set();
  		$pop->participant->relation->address->country = new Country($this->db_app,$pop->participant->relation->address->country_id,true);

  		// fill producer_relation so that we can access names and vat
  		if ($pop->producer_relation)
  			$pop->producer_relation->set();

  		// fill crso
  		$pop->crso->relation->set();
		$pop->crso->relation->address->set();
  		$pop->crso->relation->address->country = new Country($this->db_app,$pop->crso->relation->address->country_id,true);

		// what is this again?
		if(isset($_POST['process']) && $_POST['process']=='save')
			$this->_setValues = true;

		// save the current _process request
		if (isset($_REQUEST['process']))
			$this->_process = $_REQUEST['process'];
		else
			$this->_process = '';



	}

	function init($if_id)
	{
	  	$this->hiddenFields[] = array('name'=>'crso[id]','value'=>$this->model['pop']->crso->relation->id);
	  	$this->hiddenFields[] = array('name'=>'participant[id]','value'=>$this->model['pop']->participant->relation->id);

		parent::init($if_id);

	   	// PoP Multiline refactor (#2021)
	   	if (!$this->model['country']->has(HasProofOfPaymentMultiline, $_SESSION['selected_product_id']))
	   	{
	   		// hide multiline blocks if country in single line pop
	   		unset($this->blocks[ProofOfPaymentMultilineBlock]);
	   		unset($this->blocks[ProofOfPaymentMultilineLinesBlock]);

			// check if the pop only has 1 line else something is wrong
			if (count($this->model['pop']->lines) !== 1)
				trigger_error(sprintf('Pop [%s] doesnt contain 1 line',$this->model['pop']->id), E_USER_ERROR);

	   	}
	   	else {
	   		// hide singleline block for when multiline enabled
	   		unset($this->blocks[ProofOfPaymentBlock]);
	   	}

		if(!$this->model['country']->has(HasRefundInternalVAT,$_SESSION['selected_product_id']))
	   	{
	   		// if country doesnt have HasRefundInternalVAT hide the selects

	   		if (isset($this->blocks[ProofOfPaymentBlock]))
	   			$this->blocks[ProofOfPaymentBlock]->removeChild(ItemSelectProducer);

	   		if (isset($this->blocks[ProofOfPaymentMultilineLinesBlock]))
	   			$this->blocks[ProofOfPaymentMultilineBlock]->removeChild(ItemSelectProducerMultiline);
		}
	   	else
	   	{
	   		//  HasRefundInternalVAT enabled.. remove the open fields

	   		if (isset($this->blocks[ProofOfPaymentBlock])) {
				$this->blocks[ProofOfPaymentBlock]->removeChild(ItemCompany);
		   		$this->blocks[ProofOfPaymentBlock]->removeChild(VATInputItem);
	   		}

	   		// if multiline we only remove the company name field. the vat input field will
	   		// be used to show the vat from the select
		   	if (isset($this->blocks[ProofOfPaymentMultilineLinesBlock])) {
		   		$this->blocks[ProofOfPaymentMultilineBlock]->removeChild(ItemCompanyMultiple);
		   	}

	   	}

	   	// check supply chain block and hide if the the country does not have the setting
	   	if ($this->model['country']->has(HasPopSupplyChain, $_SESSION['selected_product_id']))
	   	{
	   		// check if the the participant has bought directly from the producer
	   		if (count($this->model['pop']->supply_chain) == 1)
	   		{
	   			$supply_chain = $this->model['pop']->supply_chain[1];
	   			if ($supply_chain->vat_no == $this->model['pop']->producer_relation->vat_no && $supply_chain->invoice_no == $this->model['pop']->lines[1]->invoice_id )
	   			{
	   				$this->blocks[SupplyChainBlock2]->items[1200]->properties['value'] = true;
	   				$this->blocks[SupplyChainBlock2]->items[1200]->properties['checked'] = true;
	   				$this->script = sprintf('window.onload = function() { $(\'%s\').style.display=\'none\'; };', 'ifB_'.SupplyChainBlock1);
	   			}
	   		}
	   	}
	   	else
	   	{
	   		// hide supply chain if not in country setting
	   		$this->removeChild(SupplyChainBlock1);
	   		$this->removeChild(SupplyChainBlock2);
	   	}

	   	// err and whats this? doesnt seem to exist anymore.. commented!
	   	//$this->blocks[ProofOfPaymentBlock]->removeChild(638);
	   	//$this->blocks[ProofOfPaymentBlock]->removeChild(857);
	}

	/**
	 * @param string $thrownException
	 */
	function processInput(&$thrownException)
	{
		$langCorrectForm = $this->langIf->getText('Please correct the form errors');
		$noDocument = $this->langIf->getText('No Documents attached');

		if(in_array($_SESSION['session_group_group_id'], array(GROUP_CRSO, GROUP_ACCOUNTANT))) {
			return;
		}

		if(isset($_POST['process']))
		{
			if($_POST['process']=='save' || $_POST['process'] == 'save_and_next')
			{
				$this->_setValues = true;

				$supply_chain = array();
				if ($this->model['country']->has(HasPopSupplyChain, $_SESSION['selected_product_id']))
				{
					$supply_chain_tmp = array();
					if (isset($_POST['supply_chain_directly_producer']) && $_POST['supply_chain_directly_producer'])
					{
						// POP line refactor: supply chain currently doesnt make much sense anymore when multiline is on
						// because here below we just take the first available producer.

						$r = new Relation($this->db_app, Relation::getByVAT($_POST['producer_relation_vat_no'], $this->db_app));
						if (!$r->set())
							trigger_error('Could not get build a relation object from '.$_POST['producer_relation_vat_no'], E_USER_ERROR);

						$supply_chain_tmp[] = array(
							'company_name' => $r->name,
							'vat_no' => $r->vat_no,
							'invoice_no' => $_POST['line'][0]['invoice_id'],
							'comment' => ''
						);
					}
					else
						$supply_chain_tmp = $_POST['supply_chain'];

					// now convert the string array to real objects
					foreach ($supply_chain_tmp as $sup_tmp)
						$supply_chain[] = new ProofOfPaymentSupplyChain($this->db_app, $sup_tmp);
				}

				// save document if uploaded
				$item = new InterfaceItem($this->db,$this->db_app,$this->langIf);
				$document = $item->getDocument('document[name]',ExportDocumentItem);
				if($document)
				{
					//we are updating a proof of payment and need to make a new copy of the document
					//for review, but only if the document stays the same
					if(!empty($_POST['pop']['id']) && !isset($_FILES['document']) && $document)
					{
						$document2 = Document::createDocument($this->db_app);
						$document2->name = $document->name;
						$document2->file_name = $document->file_name;
						$document2->size = $document->size;
						$document2->temp_name = DOCUMENT_REPOSITORY.$document->_getFilenameFromId($document->id);
						$document = $document2;
						$document->insert();
					}

				}

				try
				{
					try
					{
						// loop post lines and build
						$lines = array();
						foreach ($_POST['line'] as $postLine) {

							$popLine = new ProofOfPaymentLine($this->db_app);
							$popLine->fee = $postLine['fee'];
							$popLine->quantity = $postLine['quantity'];
							$popLine->invoice_id = $postLine['invoice_id'];
							$popLine->payment_date = $this->_formatDate($postLine['payment_date']);

							$lines[] = $popLine;
						}

						if ($document == "")
							$document = null;

						$producer = new Relation($this->db_app,array('vat_no'=>$_POST['producer_relation_vat_no']));
						if (!empty($_POST['producer_relation_name'])) {
							$producer->name = $_POST['producer_relation_name'];
						}

						// send stuff to controller
						$pop = $this->model['controller']->proofOfPayment(
							empty($_POST['pop']['id']) ? null : $_POST['pop']['id'], // insert or update
							$producer,
							$lines,
							$document,
							$supply_chain);


					}
					catch(FormBehaviourException $e)
					{
						// We caught a formbehaviour exception. That means it contains failures,
						// add those to the Interface-FB so it will display the errors in JS
						$this->fb->setFailures($e->getFailures());

						throw new WEEEException($langCorrectForm);
					}
				}
				catch (WEEEException $e)
				{
					$this->init($this->id);

					// tell the user the exception message
					$thrownException = $e->getMessage();
					return;
				}

				log_elc_event(sprintf('Participant added/edited PoP [%s]', empty($_POST['pop']['id']) ? '' : $_POST['pop']['id']),
					$_SESSION['session_user_id'], $_SESSION['selected_product_id'],
					$pop->crso->relation->id, $pop->participant->relation->id
				);

				if ($_REQUEST['process'] == 'save_and_next')
					header(sprintf('Location: InterfacePaymentForm.php?participant[id]=%s&crso[id]=%s',
						$pop->participant->relation->id,
						$pop->crso->relation->id));
				else
					header(sprintf('Location: InterfacePaymentOverview.php?participant[id]=%s', $this->model['pop']->participant->relation->id));

				die();
			}
			elseif($_POST['process']=='delete')
			{
				// FIXME: use controller

				$this->model['pop']->delete();
				header('Location: InterfacePaymentOverview.php?participant[id]='.$this->model['pop']->participant->relation->id);
				exit();
			}
		}
	}

	function _formatDate($date)
	{
		$pieces = explode('-',$date);
		if(sizeof($pieces)==3)
			return $pieces[2].'-'.$pieces[1].'-'.$pieces[0];
		return '';
	}

	function getInterfaceHeader()
	{
		$rv = parent::getInterfaceHeader();

		$rv .= '
			<table class="formHeader" width="100%">
				<tr>
					<td width="25%">'.$this->langIf->getText('DATE').'</td>
					<td width="25%"><strong style="font-size: 10pt !important">'.Form_Date::makePretty(date('Y-m-d')).'</strong></td>
					<td width="50%"></td>
				</tr>
				<tr>
					<td colspan="3"><br /></td>
				</tr>
				<tr>
					<td colspan="2">'.$this->model['pop']->participant->relation->name.'</td>
					<td>'.$this->model['pop']->crso->relation->name.'</td>
				</tr>
				<tr>
					<td colspan="2">'.$this->model['pop']->participant->relation->address->address_street_name.'</td>
					<td>'.$this->model['pop']->crso->relation->address->address_street_name.'</td>
				</tr>
				<tr>
					<td colspan="2">'.$this->model['pop']->participant->relation->address->city_name.', '.$this->model['pop']->participant->relation->address->country->name.'</td>
					<td>'.$this->model['pop']->crso->relation->address->city_name.', '.$this->model['pop']->crso->relation->address->country->name.'</td>
				</tr>
				<tr>
					<td colspan="2"></td>
					<td>'.$this->model['pop']->crso->relation->address->address_zipcode.', '.$this->model['pop']->crso->relation->email.'</td>
				</tr>
				<tr>
					<td>'.$this->langIf->getText('VAT Number').'</td>
					<td>'.$this->model['pop']->participant->relation->vat_no.'</td>
					<td></td>
				</tr>
				<tr>
					<td>'.$this->langIf->getText('Bank Account Number').'</td>
					<td>'.$this->model['pop']->participant->relation->bank_account_number.'</td>
					<td></td>
				</tr>
			</table>
			<br /><br />
			';

		// show warning in single-line mode
		if (!$this->model['country']->has(HasProofOfPaymentMultiline, $_SESSION['selected_product_id']))
			$rv .= '<div style="float: left"><span class="status_forfeit">'.$this->langIf->getText('Document upload required, PoP will be rejected otherwise').'</span></div>';

		require_once(ELC_CLASSES.'CRSOProduct.php');
		$cp = new CRSOProduct($this->db_app);
		$cp->set($this->model['pop']->crso->relation->id, $_SESSION['selected_product_id']);
		$rv .= "<div style='text-align: right'><span class='cut-off-date'>".$this->langIf->getText('Upcoming export date').
				": ".$cp->getNextCutOffDateRefund()."</span></div>";

		return $rv;
	}

	function getInterfaceFooter()
	{
		$rv = parent::getInterfaceFooter();

		if ($this->_process != 'print')
			return $rv;

		$user = new ELCUser($this->db_app, $_SESSION['session_user_id'], true);
		$user->info_person->set();

		$lines = array();
		$lines[] = $user->info_person->getFullName();
		$lines[] = $user->info_person->title;
		$lines[] = $user->info_person->position;
		$lines[] = $user->info_person->department;
		$lines[] = $user->info_person->telhouse;
		$lines[] = $user->info_person->fax;
		$lines[] = $user->info_person->email;

		$rv .= '<table class="Professional">';

		foreach ($lines as $line)
		{
			if (empty($line))
				continue;

			$rv .= '<tr>
						<td>'.$line.'</td>
					</tr>';
		}

		// signature box
		$rv .= '	<tr>
						<td>
							<div style="border: 1px solid black; margin: 10px; width: 200px; height: 50px;"></div>
						</td>
					</tr>
				</table>
		';

		return $rv;
	}

}
/**
 * @package ELC
 * @subpackage Interface
 * @since 12-05-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfacePaymentFormItem extends InterfaceItem
{

	function getSpacer()
	{
		$this->properties['value'] = '';
		$this->twoCols = true;
	}


	function getVatField()
	{
		$this->rightCol = $this->langIf->getText('Item: VAT Number');
		$this->text = '<span id="vatNumber">'.(isset($_REQUEST['pop']['producer_relation_vat_no'])?$_REQUEST['pop']['producer_relation_vat_no']:'').'</span>';
	}

	/**
	 * item[864] = pop[vat_no]
	 */
	function getProducerSelect()
	{
		$country = $this->if_obj->model['country'];

		$this->options = array(array('value'=>'','text'=>'- '.$this->langIf->getText('Select Producer').' -'));
		$cp = new CRSOParticipantProduct($this->db_app);
		$q = $cp->getCRSOParticipants($this->if_obj->model['pop']->crso->relation->id,'q',PARTICIPANT_PRODUCER,$_SESSION['selected_product_id']);
		$vat = '';

		if(!empty($this->if_obj->model['pop']->producer_relation->id))
		{
			$this->if_obj->model['pop']->producer_relation->set();
			$vat = $this->if_obj->model['pop']->producer_relation->vat_no;
		}

		while($r = $q->fetch())
		{
			if($r->PARTICIPANT_RELATION_ID==$this->if_obj->model['pop']->participant->relation->id)
				continue;
			$part = (mb_strlen($r->PARTICIPANT_NAME)>35)?mb_substr($r->PARTICIPANT_NAME,0,35).'...':$r->PARTICIPANT_NAME;
			$this->options[] = array('value'=>$r->VAT_NO,'text'=>$part,'selected'=>($vat==$r->VAT_NO)?'selected':false);
		}


		if (!$country->has(HasProofOfPaymentMultiline, $_SESSION['selected_product_id'])) {

			// single line is on

			$this->script = "
				function showVat(sel)
				{
					$('vatNumber').innerHTML = sel.value;
				}
				";

		} else {

			// multiline is on

			// showVat is called onchange of the select. Then we wanna set the vat value in the input-text next to this
			// select. The input in which we wanna show the vat has the same name as the select.. just find the
			// second one in the document with the same name and set its value.
			$this->script = "
				function showVat(sel)
				{
					var input = document.getElementsByName(sel.name)[1];
					input.value = sel.value;
				}
				";

			// register a postscript for this item which updates all select inputs on the page.. this is to show the
			// vat value after loading a pop
			$this->postscript = "
				YAHOO.util.Event.addListener(window, 'load', function() {
				var selects = document.getElementsByTagName('select');
					for (var i=0; i<selects.length; i++) {
						if (/^line\[/.test(selects[i].name)) {
							showVat(selects[i]);
						}
					}
				});

			";


		}

		$this->properties['onchange'] = 'showVat(this);';

	}

	/**
	 * item[858] = pop[invoice_id]
	 * item[862] = pop[quantity]
	 * item[863] = pop[fee]
	 */
	function getItemValue()
	{
		$country = $this->if_obj->model['country'];

		if($this->name == 'producer_relation_name')
		{
			if(empty($this->if_obj->model['pop']->producer_relation->id))
				return $this->properties['value'] = '';
			$this->if_obj->model['pop']->producer_relation->set();
			return $this->properties['value'] = $this->if_obj->model['pop']->producer_relation->name;
		}

		// from here it doesnt make sense to continiue use when we are in multiline mode
		// because the values get set by the rowcontrol which sets them using JS.. so in that case just return
		if ($country->has(HasProofOfPaymentMultiline, $_SESSION['selected_product_id']))
			return;

		foreach (array('invoice_id', 'quantity', 'fee','payment_date') as $item)
		{
			if($item=='payment_date')
				return $this->properties['value'] = $this->if_obj->_formatDate($this->if_obj->model['pop']->lines[1]->$item);

			if ($this->name == 'line[0]['.$item.']') {
				if ($this->if_obj->_hideInputs && $item=='quantity')
					$this->properties['value'] = locale_nf($this->if_obj->model['pop']->lines[1]->$item, 0);
				else
					$this->properties['value'] = $this->if_obj->model['pop']->lines[1]->$item;
				return;
			}
		}
	}

	/**
	 * item[859] = pop[vat_no]
	 */
	function getVatNo()
	{
		$country = $this->if_obj->model['country'];

		// when we are in multiline and have internalVat the user has to select a producer and the vat is shown in the
		// vat input so the user cant edit it.. set it readonly
		if ($country->has(HasProofOfPaymentMultiline, $_SESSION['selected_product_id']) &&
			$country->has(HasRefundInternalVAT, $_SESSION['selected_product_id']))
		{
			$this->properties['readonly'] = 'readonly';
		}

		if(empty($this->if_obj->model['pop']->producer_relation->id))
			return '';

		$this->if_obj->model['pop']->producer_relation->set();
		$this->properties['value'] = $this->if_obj->model['pop']->producer_relation->vat_no;
		return $this->if_obj->model['pop']->producer_relation->vat_no;
	}

	/**
	 * item[860] = document[name]
	 */
	function getInvoiceDocument()
	{
		if ($this->if_obj->model['pop']->document)
			$_REQUEST['Documents'][$this->id] = (int) $this->if_obj->model['pop']->document->id;
	}

	/**
	 * item[861] =
	 */
	function get2colTxt()
	{
		$this->twoCols = true;
	}


	function getBoughtDirectlyFromProducer()
	{
		$this->properties['onclick'] = "\$('ifB_".SupplyChainBlock1."').style.display = this.checked ? 'none' : '' ";

		$this->properties['value'] = false;
	}

	// multiline rowcontrol
	function getMultilineRowControlValues() {

		if (isset($_POST['line']))
			$lines = $_POST['line'];
		else
		{
			$lines = array();
			$i = 0;
			foreach ($this->if_obj->model['pop']->lines as $pop_line)
			{
				$fields = array('invoice_id', 'payment_date', 'quantity', 'fee');

				foreach ($fields as $field) {

					if ($field == 'payment_date') {
						$lines[$i][$field] = $this->if_obj->_formatDate($pop_line->payment_date);
					}
					else
						$lines[$i][$field] = $pop_line->$field;
				}
				$i++;
			}
		}

		if (!empty($lines))
		{
			if ($this->if_obj->_hideInputs)
			{
				$this->postscript = exportArrayToJS($lines, 'pop_lines').'

					var intlinenumber = 0;
					pop_lines.forEach(function(line, linenumber, lines)
					{
						line.forEach(function(item, itemname, a)
						{
							el = $$(\'line[\'+intlinenumber+\'][\'+itemname+\']\');
							var span = el.parentNode.getElementsByTagName("span")[0];
							span.innerHTML = item;
						});
						ifHandler.duplicateParentsLastRow(el, 1);
						intlinenumber++;
					});
					// remove last row
					el = $$(\'line[\'+intlinenumber+\'][invoice_id]\');
					ifHandler.removeParentRow(el, false);
				';
			}
			else {
				$this->postscript = exportArrayToJS($lines, 'pop_lines').'

					var intlinenumber = 0;
					pop_lines.forEach(function(line, linenumber, lines)
					{
						line.forEach(function(item, itemname, a)
						{
							el = $$(\'line[\'+intlinenumber+\'][\'+itemname+\']\');
							el.value = item || "";

							// in print mode the value is shown in a span
							var span = el.parentNode.getElementsByTagName("span")[0];
							if (span) {

								var value = item;

								if (itemname == "quantity") {
									value = number_format(item, 0, ",", ".");
									el.parentNode.style.textAlign = "right";
								}

								if (itemname == "fee") {
									value = number_format(item, 8, ",", ".");
									el.parentNode.style.textAlign = "right";
								}

								span.innerHTML = value;
							}

						});

						ifHandler.duplicateParentsLastRow(el, 1);
						intlinenumber++;
					});

					// remove last row
					el = $$(\'line[\'+intlinenumber+\'][invoice_id]\');
					ifHandler.removeParentRow(el, false);
				';
			}

		}
	}

	// supply chain
	function getRowControlValues()
	{
		//restore_error_handler();

		if (isset($_POST['supply_chain']))
			$supply_chain = $_POST['supply_chain'];
		else
		{
			$supply_chain = array();
			$i = 0;
			foreach ($this->if_obj->model['pop']->supply_chain as $pop_supply_chain)
			{
				foreach (array('company_name', 'vat_no', 'invoice_no', 'comment') as $field)
					$supply_chain[$i][$field] = $pop_supply_chain->$field;
				$i++;
			}
		}

		if (!empty($supply_chain))
		{
			if ($this->if_obj->_hideInputs)
			{
				$this->postscript = exportArrayToJS($supply_chain, 'supply_chain').'

					var intlinenumber = 0;
					supply_chain.forEach(function(line, linenumber, lines)
					{
						line.forEach(function(item, itemname, a)
						{
							el = $$(\'supply_chain[\'+intlinenumber+\'][\'+itemname+\']\');
							el.parentNode.innerHTML = item;
						});
					});
				';
			}
			else
				$this->postscript = exportArrayToJS($supply_chain, 'supply_chain').'

					var intlinenumber = 0;
					supply_chain.forEach(function(line, linenumber, lines)
					{
						line.forEach(function(item, itemname, a)
						{
							el = $$(\'supply_chain[\'+intlinenumber+\'][\'+itemname+\']\');
							el.value = item;
						});

						ifHandler.duplicateParentsLastRow(el, 1);
						intlinenumber++;
					});

					// remove last row
					el = $$(\'supply_chain[\'+intlinenumber+\'][company_name]\');
					ifHandler.removeParentRow(el, false);
				';

		}
	}

	function getSupplyChainDescription()
	{
		$this->twoCols = true;
		$this->text = $this->langIf->getText('SupplyChainDescription');
	}

	/**
	 * item[639] =
	 */
	function getButtons()
	{
		if(in_array($_SESSION['session_group_group_id'], array(GROUP_CRSO, GROUP_ACCOUNTANT))) {
			$this->_BarButtons = 0;
			return;
		}
		if (!$this->if_obj->_hideInputs)
		{
			if(!$this->if_obj->model['pop']->id||!$this->if_obj->model['pop']->isDeletable())
				$this->_BarButtons = BUTTON_SAVE+BUTTON_CANCEL;

			$this->_addCustomButton($this->langIf->getText('Save and Next Registration'), 'submit',
				array('onclick'=>"$$('process').value='save_and_next';"));

			$this->_ButtonOptions[BUTTON_DELETE] = array('onclick'=>jsMessageBox(MSG_BOX_QUEST,$this->langIf->getText('Delete Warning'),$this->langIf->getText('Delete ProofOfPayment Confirmation'),"function(){\$\$('process').value='delete';\$('interface_form').submit();}"));
		}
		else
			$this->_BarButtons = BUTTON_CANCEL;

		$this->_ButtonOptions[BUTTON_CANCEL] = array('onclick'=>"go('InterfacePaymentOverview.php');");
	}


}
/**
 * CALLBACK[getCRSOSelect]
 * item[638] = {
 *	 [handle] = Item: CRSO Selection
 *	 [behaviour] =
 *	 [help] = 0
 *	 [close_row] = 0
 *	 [label_pos] = 4
 *
 * CALLBACK[getParticipantSelect]
 * item[857] = {
 *	 [handle] = Item: Participant Selection
 *	 [behaviour] =
 *	 [help] = 0
 *	 [close_row] = 1
 *	 [label_pos] = 1
 *
 * CALLBACK[getItemValue]
 * item[858] = {
 *	 [handle] = Item: Invoice Number
 *	 [behaviour] = integer
 *	 [help] = 0
 *	 [close_row] = 0
 *	 [label_pos] = 4
 *
 * item[862] = {
 *	 [handle] = Item: Quantity Requested
 *	 [behaviour] = integer
 *	 [help] = 0
 *	 [close_row] = 0
 *	 [label_pos] = 4
 *
 * item[863] = {
 *	 [handle] = Item: Product Fee
 *	 [behaviour] = numeric
 *	 [help] = 0
 *	 [close_row] = 1
 *	 [label_pos] = 4
 *
 * CALLBACK[getVatNo]
 * item[859] = {
 *	 [handle] = Item: VAT Number
 *	 [behaviour] = vat
 *	 [help] = 0
 *	 [close_row] = 1
 *	 [label_pos] = 4
 *
 * CALLBACK[getInvoiceDocument]
 * item[860] = {
 *	 [handle] = Item: Invoice Document
 *	 [behaviour] = getInvoiceDocument
 *	 [help] = 0
 *	 [close_row] = 0
 *	 [label_pos] = 1
 *
 * CALLBACK[get2colTxt]
 * item[861] = {
 *	 [handle] = Item: Dummy
 *	 [behaviour] =
 *	 [help] = 0
 *	 [close_row] = 1
 *	 [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[639] = {
 *	 [handle] = Item: ButtonBar
 *	 [behaviour] =
 *	 [help] = 0
 *	 [close_row] = 1
 *	 [label_pos] = 4
 *
 */
?>
