<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'Country.php');
require_once(ELC_CLASSES.'ProductCategory.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @since 30-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceProductCategoryOverview extends InterfaceFactory
{
	/**
	 * @var boolean Whether this interface is being used in an ajax context
	 */
	var $ajaxUsage = false;

	/**
	 * interface[id] = 135
	 * @param $db Database (I_WARE_SETTINGS)
	 * @param $db_app Database (APPLICATION DATABASE)
	 * @param $langIf LanguageInterface
	 */
	function InterfaceProductCategoryOverview(&$db, &$db_app, &$langIf)
	{
		//for CRSO only users with CRSOClearanceMaintenance can access this interface
		if($_SESSION['session_group_group_id'] == GROUP_CRSO)
		{
			$ru = new RelationUser($db_app);
			if(!$ru->hasClearance(CRSOClearanceMaintenance, $_SESSION['session_user_id'],$_SESSION['crso_relation_id']))
				throw new InterfacePermissionException('['.$_SESSION['session_user_id'].'] tried to access crso admin interface without correct clearance: CRSOClearanceMaintenance');
		}

		parent::InterfaceFactory($db, $db_app, $langIf);
		$this->FirstColWidth = 25;
		$this->delimiter = '<br />';

		$this->ajaxUsage = isset($_REQUEST['ajaxUsage']) && (int)$_REQUEST['ajaxUsage'] ? true : false;


		require_once(IWARE.'group.php');

		$pc = new ProductCategory($this->db_app);
		$this->model['pc'] = $pc;

		switch ($_SESSION['session_group_company_id'])
		{
			case GROUP_SYSADMIN:
				if (isset($_REQUEST['crso']['id']))
					$pc->crso = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, (int)$_REQUEST['crso']['id'])));
			break;

			case GROUP_CRSO:
				$pc->crso = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, (int)$_SESSION['crso_relation_id'])));
			break;

			default:
				throw new InterfacePermissionException('Unauthorized access to Product Category Overview');
		}

		if(isset($pc->crso))
			$this->model['country'] = new Country($this->db_app,$pc->crso->relation->getCountry(),true);

		if (isset($_REQUEST['product_category']['id']))
		{
			if(!$pc->set((int)$_REQUEST['product_category']['id']))
				throw new IWException('ProductCategory does not exist');
			if(!$pc->is_for_crso($_SESSION['crso_relation_id']))
			   throw new IWUserException('Permission denied');
			$this->hiddenFields[] = array('name'=>'product_category[id]', 'value'=>$pc->id);
		}

	}

	function init($id)
	{
		$this->id = $id;
		parent::init($id);


		if ($this->ajaxUsage && isset($this->blocks[BLOCK_CRSO_SELECT]))
			unset($this->blocks[BLOCK_CRSO_SELECT]);
		if(isset($this->model['country'])&&(!$this->model['country']->has(HasDeclByCategory,$_SESSION['selected_product_id'])))
			unset($this->blocks[BLOCK_BUTTON_BAR]);
	}

	/**
	 * @param string $thrownException
	 */
	function processInput(&$thrownException)
	{

		if(isset($_REQUEST['process']) && $_REQUEST['process'] == 'disable')
		{
			$pc = new ProductCategory($this->db_app);
			$pc->product = new Product($this->db_app, $_SESSION['selected_product_id']);
			$pc->crso = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, $this->model['pc']->crso->relation->id)));
			$pc->disableCategory($_REQUEST['product_category']['id']);

			header(sprintf('location: InterfaceProductCategoryOverview.php'));
			exit();
		}

		else if(isset($_REQUEST['process']) && $_REQUEST['process'] == 'save')
		{
			$pc = new ProductCategory($this->db_app, $_REQUEST['product_category']);
			$pc->product = new Product($this->db_app, $_SESSION['selected_product_id']);
			$pc->crso = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, $this->model['pc']->crso->relation->id)));

			if (isset($_REQUEST['product_category']['id']))
				$pc->update();
			else {
				// for new categories, set category start year to current year
				$pc->start_year = currentDate()->getYear();
				$pc->insert();
			}

			if(!$this->ajaxUsage)
			{
				//header(sprintf('location: %s?crso[id]=%s', $_SERVER['PHP_SELF'], $pc->crso->relation->id));
				header(sprintf('location: InterfaceProductCategoryOverview.php'));
				exit();
			}
			else
			{
				unset($this->model['pc']);
				$this->model['pc'] =& $pc;
				$this->init($this->id);
			}
		}
	}
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 30-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceProductCategoryOverviewItem extends InterfaceItem
{

	/**
	 * item[992] = crso[id]
	 */
	function getCrsoSelect()
	{
		$crso = new CRSO($this->db_app);
		$q = $crso->selectCRSOs();

		$this->options[] = array('text'=>'- '.$this->langIf->getText('Select CRSO').' -', 'value'=>'');

		while ($row = $q->fetch())
		{
			$name = $row->NAME;
			$name = (mb_strlen($name)>35)?mb_substr($name,0,35).'...':$name;
			$this->options[] = array('text'=>$name,
				'value'=>$row->CRSO_RELATION_ID,
				'selected'=>(bool) (isset($this->if_obj->model['pc']->crso) && isset($this->if_obj->model['pc']->crso->relation->id) && $row->CRSO_RELATION_ID == $this->if_obj->model['pc']->crso->relation->id));
		}
		$this->properties['onchange'] = "_LoadOverview();";
		$this->postscript = "
		function _LoadOverview()
		{
			var params = new Array();
			params['crso'] = new Array();
			params['crso']['id'] = getElByName('crso[id]').value;
			params['interface_id'] = ".$this->if_obj->id."; // you MUST supply this
			ifHandler.loadBlock( new Array(new String('323')), params );
		}";
	}

	/**
	 * item[993] =
	 */
	function getOverview()
	{
		$countrySettingUnavailable = $this->langIf->getText('Product Categories not enabled for this crso/country');
		if (isset($this->if_obj->model['pc']->crso) && isset($this->if_obj->model['pc']->crso->relation->id))
		{

			if(!$this->if_obj->model['country']->has(HasDeclByCategory,$_SESSION['selected_product_id']))
				return $this->text = $countrySettingUnavailable;

			$pc = new ProductCategory($this->db_app);
			$q = $pc->selectOverview(array($_SESSION['selected_product_id']), $this->if_obj->model['pc']->crso->relation->id); // FIXME: product_id

			$t = new TableFactory($q, $this->langIf);
			$t->setValueProcessor(array('PRODUCT_CATEGORY_ID', 'DISABLE_CATEGORY', 'CURRENT_FEE'),
				array($this, '_tfValueProcessor'));

			$t->setEmptyText($this->langIf->getText('No product categories found'));
			$this->text = $t->getXHTMLTable('Professional', 2);


			// We check if the country product setting has Weight as Amount Calculation.
			$id = $this->if_obj->model['pc']->crso->getCountry($this->if_obj->model['pc']->crso->relation->id);
			$c = new WEEECountry($this->db_app, array('id'=>$id), true);
			if($c->has(HasWeightAsBasisForAmount, $_SESSION['selected_product_id'])) {
				$this->text .= $this->langIf->getText('Note that amounts are based on fee times weight for this product.');
			}

		}
		else
			$this->text = $this->langIf->getText('Please select a CRSO');
	}

	function _tfValueProcessor($args)
	{
		switch ($args['head'])
		{
			case 'PRODUCT_CATEGORY_ID':
				if(!$this->if_obj->ajaxUsage)
				{
					return sprintf('<a href="%s?product_category[id]=%s">%s</a>',
					'InterfaceProductCategoryOverview.php' , $args['value'], $args['value']);
				}
				else
				{
					return sprintf('<a href="#" onclick="%s">%s</a>',
					"callParams = []; callParams['targetId'] = 'ifI_841'; ifHandler.loadInterface(135, callParams, [['crso[id]', '".$this->if_obj->model['pc']->crso->relation->id."'], ['product_category[id]', '".$args['value']."'], ['ajaxUsage', '1']]);",
					$args['value'], $args['value']);
				}
			break;

			case 'DISABLE_CATEGORY':

				$end_year = $args['value'];

				$confirm_msg = htmlspecialchars($this->langIf->getText('Are you sure you want to disable the category? This will set the category end year to the closest possible year and will remove this category from any provisional declaration for that year.'));

				if ($end_year) {
					return $this->langIf->getText('Disabled');

				} else {
					$url = '?product_category[id]='.$args['row']['PRODUCT_CATEGORY_ID'].'&process=disable';
					return sprintf('<a href="%s" onclick="return confirm(\'%s\')">%s</a>', $url, $confirm_msg,  $this->langIf->getText('Disable'));
				}

			break;

			case 'CURRENT_FEE':
				require_once(ELC_CLASSES.'ProductFee.php');
				$pc_fee = new ProductCategoryFee($this->db_app);
				$fee = $pc_fee->getFee($args['row']['PRODUCT_CATEGORY_ID'], $this->if_obj->model['pc']->crso->relation->id);

				if (!$fee)
					return $this->langIf->getText('No Product Fee set');

				return $fee;
				break;

		}
	}

	/**
	 * item[994] = product_category[id]
	 */
	function getProductCategory()
	{
		$this->properties['value'] = $this->if_obj->model['pc']->name;
	}

	/**
	 * item[1610] = product_category[description]
	 */
	function getProductCategoryDescription() {
		$this->properties['value'] = $this->if_obj->model['pc']->description;
	}

	/**
	 * item[995] =
	 */
	function getButtons()
	{
		$this->_BarButtons = 0;

		if(!$this->if_obj->ajaxUsage)
		{
			if (isset($this->if_obj->model['pc']->id))
			{
				$this->_addCustomButton($this->langIf->getText('Cancel'), 'button',
					array('onclick'=>"go('InterfaceProductCategoryOverview.php');"));
				$this->_addCustomButton($this->langIf->getText('Update'), 'submit');
			}
			else
				$this->_addCustomButton($this->langIf->getText('Add'), 'submit');
		}
		else
		{
			if (isset($this->if_obj->model['pc']->id))
			{
				$this->_addCustomButton($this->if_obj->model['pc']->id, "hidden", array('name' => 'product_category[id]'));
				$this->_addCustomButton($this->langIf->getText('Cancel'), 'button',
					array('onclick'=>"callParams = []; callParams['targetId'] = 'ifI_841'; ifHandler.loadInterface(135, callParams, [['crso[id]', '".$this->if_obj->model['pc']->crso->relation->id."'], ['ajaxUsage', '1']]);"));
				$this->_addCustomButton($this->langIf->getText('Update'), 'button',
					array('onclick'=> " \$('ajaxstatus').innerHTML = '".$this->langIf->getText('saving form details...')."'; \$('interface_form_process').value = 'save'; var params = []; params['ifId'] = ".$this->if_obj->id."; params['targetId'] = 'ifI_841'; jsbroker.callAndSubmitForm('interface_form', 'loadInterface', params, ifHandler.buildInterface);"));
			}
			else
			{
				$this->_addCustomButton($this->langIf->getText('Add'), 'button',
				array('onclick'=> " \$('ajaxstatus').innerHTML = '".$this->langIf->getText('saving form details...')."'; \$('interface_form_process').value = 'save'; var params = []; params['ifId'] = ".$this->if_obj->id."; params['targetId'] = 'ifI_841'; jsbroker.callAndSubmitForm('interface_form', 'loadInterface', params, ifHandler.buildInterface);"));
			}
			$this->_CustomButtons[] = '<div id="ajaxstatus" style="color: red; display: inline;"></div>&nbsp;&nbsp;';
			$this->_addCustomButton($_REQUEST['crso']['id'], "hidden", array('name' => 'crso[id]'));
			$this->_addCustomButton($_REQUEST['ajaxUsage'], "hidden", array('name' => 'ajaxUsage'));
		}
	}
}

/**
 * CALLBACK[getCrsoSelect]
 * item[992] = {
 *	 [handle] = Item: SelectCRSO
 *	 [behaviour] = required
 *	 [help] = 0
 *	 [close_row] = 1
 *	 [label_pos] = 4
 *
 * CALLBACK[getOverview]
 * item[993] = {
 *	 [handle] = Item: ProductCategoryOverview
 *	 [behaviour] =
 *	 [help] = 0
 *	 [close_row] = 1
 *	 [label_pos] = 4
 *
 * CALLBACK[getProductCategory]
 * item[994] = {
 *	 [handle] = Item: ProductCategory
 *	 [behaviour] = required
 *	 [help] = 0
 *	 [close_row] = 0
 *	 [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[995] = {
 *	 [handle] = Item: ButtonBar
 *	 [behaviour] =
 *	 [help] = 0
 *	 [close_row] = 1
 *	 [label_pos] = 4
 *
 */

 /**#@+
 *
 */
define('BLOCK_CRSO_SELECT',322);
define('BLOCK_BUTTON_BAR',324);
/**#@-*/

?>
