<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @since 14-09-2006
 * @author Marten Koedam <marten@dalines.net>
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceProvDeclOverview extends InterfaceFactory
{
    /**
     * interface[id] = 147
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceProvDeclOverview(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        $date = currentDate();
        $this->model['year'] = (int)$date->getYear();
        if(!empty($_REQUEST['prov']['year']))
        	$this->model['year'] = (int)$_REQUEST['prov']['year'];

        $this->model['crso'] = false;
        if(!empty($_REQUEST['crso']['id']))
        	$this->model['crso'] = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app,$_REQUEST['crso']['id']),true));

        $this->model['participant'] = new Participant($this->db_app,array('relation'=>new Relation($this->db_app,$_SESSION['selected_participant_id'])));

        $this->script = "
		    function _LoadOverview()
		    {
		        var params = new Array();
		        params['crso'] = new Array();
				params['crso']['id'] = getElByName('crso[id]').value;
				params['prov'] = new Array();
				params['prov']['year'] = getElByName('prov[year]').value;
		        params['interface_id'] = 147; // you MUST supply this
		        ifHandler.loadBlock( new Array(new String('363'), new String('376'), new String('364')), params );
		    }";
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {

    }

}
/**
 * @package ELC
 * @subpackage Interface
 * @since 14-09-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceProvDeclOverviewItem extends InterfaceItem
{
	private static $index = 0;

	function getYearSelect()
    {
    	$crso_id = ($this->if_obj->model['crso'] instanceof CRSO) ? $this->if_obj->model['crso']->relation->id : '';

    	$date = currentDate();
		$year = (int)$date->getYear();

    	if($year == WEEE_APPLICATION_START_YEAR)
    		$this->_hideInput = true;

    	$this->options = array();

    	for($i=WEEE_APPLICATION_START_YEAR; $i<=$year; $i++)
			$this->options[] = array('value'=>$i,'text'=>$i,'selected'=>($i==$this->if_obj->model['year']?'selected':false));
		$this->properties['onchange'] = "_LoadOverview();";
    }

	function getCRSOSelect()
	{
		$rv = null;
		// Check for any active Remote Viewer connections.
		if(IWRegistry::isRegistered('remote_viewer')) {
			$rv = IWRegistry::registry('remote_viewer');
		}
		// Only select the CRSO for the selected participant

		$cp = new CRSOParticipantProduct($this->db_app);
		$q = $cp->getParticipantCRSOs($this->if_obj->model['participant']->relation->id,'q',false,false,$_SESSION['selected_product_id']);
		$rs = $q->fetchAll();
		if(count($rs)==1)
		{
			$r = $rs[0];
			if($rv && $rv->crso->relation->id!=$r->CRSO_RELATION_ID)
				return;
			$this->_hideInput = true;
			$name = $r->CRSO_NAME;
			$this->options = array(array('text'=>((mb_strlen($name)>35)?mb_substr($name,35).'...':$name),'value'=>$r->CRSO_RELATION_ID,'selected'=>true));
        	$this->if_obj->model['crso'] = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app,$r->CRSO_RELATION_ID),true));
			return;
		}

		$this->options = array(array('text'=>'- '.$this->langIf->getText('Select CRSO').' -','value'=>''));

		foreach($rs as $r)
		{
			if($rv && $rv->crso->relation->id!=$r->CRSO_RELATION_ID)
				continue;
			$name = $r->CRSO_NAME;
			$selected = false;
			if($this->if_obj->model['crso']&&($this->if_obj->model['crso']->relation->id==$r->CRSO_RELATION_ID))
				$selected = 'selected';
			$this->options[] = array('text'=>((mb_strlen($name)>35)?mb_substr($name,35).'...':$name),'value'=>$r->CRSO_RELATION_ID,'selected'=>$selected);
		}

		$this->properties['onchange'] = "_LoadOverview();";

	}
    /**
     * item[1093] =
     */
    function getProvDeclOverviews()
    {
    	$this->langIf->getText('Please Select a CRSO First');
    	if(!$this->if_obj->model['crso'])
    	{
    		$this->text = $this->langIf->getText('Please Select a CRSO First');
    		return;
    	}

    	// First Load Own Provisional
    	list($tbl,$script) = $this->_getOverview($this->if_obj->model['participant'],null,'{PARTICIPANT}');
    	$this->text .= $tbl;
    	$this->postscript .= $script;

   		$this->postscript .= "($('thProvMonth')) && hatProvisionalDeclarationDetails1.sort(0, \$('thProvMonth'), 'MONTH');";

    	require_once(ELC_CLASSES.'Trust.php');
    	$trust = new Trust($this->db_app);
    	// Load the Provisional of all Subrogated Prov.
    	$q = $trust->selectSubrogations($this->if_obj->model['participant']->relation->id,TRUST_APPROVED,$this->if_obj->model['crso']->relation->id,$_SESSION['selected_product_id']);
    	while($r = $q->fetch())
    	{
    		$trustee = new Participant($this->db_app,array('relation'=>new Relation($this->db_app,$r->TRUSTEE_RELATION_ID)));
    		list($tbl,$script) = $this->_getOverview($trustee,$r->TRUST_ID,'Subrogated To {PARTICIPANT}');
    		$this->text .= $tbl;
    		$this->postscript .= $script;
    	}

    	// Load the Provisional of all Recevied (Trust) Prov.
    	$q = $trust->selectTrusts($this->if_obj->model['participant']->relation->id,TRUST_APPROVED,$this->if_obj->model['crso']->relation->id,$_SESSION['selected_product_id']);
    	while($r = $q->fetch())
    	{
    		$subrogator = new Participant($this->db_app,array('relation'=>new Relation($this->db_app,$r->SUBROGATOR_RELATION_ID,true)));
    		list($tbl,$script) = $this->_getOverview($this->if_obj->model['participant'],$r->TRUST_ID,'Trusted from {PARTICIPANT}',$subrogator);
    		$this->text .= $tbl;
    		$this->postscript .= $script;
    	}
    }

    function _getOverview($participant,$trust=null,$txt=false,$replace=null)
    {
    	self::$index++;
    	$dateObj = currentDate();
    	$year = $this->if_obj->model['year'] ? $this->if_obj->model['year'] : (int)$dateObj->getYear();
    	$replace || $replace = $participant;

    	$this->width = '100%';
    	if(!$this->if_obj->model['crso']->complete)
    	{
    		$this->if_obj->model['crso']->set();
    		$this->if_obj->model['crso']->relation->set();
    	}
    	$participant->set();
    	$participant->relation->set();

    	require_once(ELC_CLASSES.'Country.php');
		require_once(ELC_CLASSES.'ProvisionalDeclaration.php');
		$country = new Country($this->db_app);
		$country->set($this->if_obj->model['crso']->getCountry());

    	$ds = array();
		$prov = new ProvisionalDeclaration($this->db_app);
		$isAggregate = false;


		if ($country->settings & HasProvisionalPerMonth)
		{
			$prov->set(
						$this->if_obj->model['crso']->relation->id,
						$participant->relation->id,
						$year,
						$trust,
						$_SESSION['selected_product_id']);
			$lines = $prov->getLines();
		}
		else
		{
			$isAggregate = true;
			$lines =& $prov->getAggregateLinesByYear(
						$this->if_obj->model['crso']->relation->id,
						$participant->relation->id,
						$year,
						$trust,
						$_SESSION['selected_product_id']);
		}

		foreach($lines as $i => $content)
		{
			if(!$isAggregate)
				$ds[$i]['MONTH'] = $content->month;
			if($country->has(HasDeclByCategory,$_SESSION['selected_product_id']))
			{
				$content->product_category->set();
				$ds[$i]['PRODUCT_CATEGORY'] = $content->product_category->name;
			}
			if($country->has(HasDeclarationDimension,$_SESSION['selected_product_id']))
			{
				$content->dimension->set();
				$ds[$i]['CPD_ID'] = $content->dimension->name;
			}
			if($country->has(HasDeclByBrand,$_SESSION['selected_product_id']))
			{
				$content->brand->set();
				$ds[$i]['BRAND'] = $content->brand->name;
			}
			$ds[$i]['QUANTITY'] = $content->quantity;
			if($country->has(HasDeclOfWeight,$_SESSION['selected_product_id']))
				$ds[$i]['WEIGHT'] = $content->weight;
			if($country->has(HasDeclOfWeightWithDecimals,$_SESSION['selected_product_id']))
				$ds[$i]['WEIGHT_DEC'] = $content->weight_dec;
		}

		$this->_totals = 0;
		$this->_cols = array();
		$this->_hiddenCols = array();


		$tf = new TableFactory($ds,$this->langIf);
		$tf->id = "ProvisionalDeclarationDetails".self::$index;
		$tf->setEmptyText($this->langIf->getText('No declaration lines found'));
		$tf->setCaption($this->langIf->getText('Details for {YEAR}', null, array('YEAR' => $year)).
			($txt?' ('.$this->langIf->getText($txt,null,array('PARTICIPANT'=>$replace->relation->name)).')':''), false);
		$tf->setValueProcessor(array('QUANTITY','WEIGHT','WEIGHT_DEC','MONTH'), array($this, 'tfValueProcessor'));
		$tf->setTHProcessor(array('MONTH', 'QUANTITY','WEIGHT','WEIGHT_DEC'), array($this,'tfTHProcessor'));
		$tf->columnModifier(array('QUANTITY'=>'style="text-align:right;"','WEIGHT'=>'style="text-align:right;"','WEIGHT_DEC'=>'style="text-align:right;"'));

		$tf->setSortingFunctions(array(
			'PRODUCT_CATEGORY'=>"'String'",
			'CPD_ID'=>"'String'",
			'BRAND'=>"'String'",
			'WEIGHT'=>"'SortVal'",
			'WEIGHT_DEC'=>"'SortVal'",
			'QUANTITY'=>"'SortVal'",
			'MONTH'=>"'SortVal'"
			));

		$tf->setFooterProcessor(array($this,'tfFooter'));

		return $tf->getXHTMLTable('Professional',2,'row',true);
    }

	function tfFooter() // getClass
	{
		if(!$this->_cols||in_array('QUANTITY',$this->_hiddenCols))
			return false;
		$rv = explode(',',str_repeat(',',(sizeof($this->_cols)-1)));

		$pos = array_flip($this->_cols);
		$rv[$pos['QUANTITY']] = '<span style="float:left; font-weight: bold;">'.$this->langIf->getText('Total').'</span><span style="float:right; font-weight: bold;">'.locale_nf($this->_totals,0).'</span><span style="clear:both;"></span>';
		return $rv;
	}

    function tfTHProcessor($params)
    {
    	switch ($params['head'])
    	{
    		case 'MONTH':
    			return ' id="thProvMonth"';
    		break;
    		case 'WEIGHT_DEC':
    		case 'WEIGHT':
    		case 'QUANTITY':
    			return ' class="th-right"';
    		break;
    	}
    }

    function tfValueProcessor($params)
    {
    	switch ($params['head'])
    	{
    		case 'WEIGHT_DEC':
    			$params['extra'] .= sprintf(' sort_val="%s" ', $params['value']);
    			return locale_nf($params['value'], 3);
    		break;
    		case 'WEIGHT':
    			$params['extra'] .= sprintf(' sort_val="%s" ', $params['value']);
    			return locale_nf($params['value']);
    		break;
    		case 'QUANTITY':
    			$this->_totals += $params['value'];
    			$this->_cols = $params['visibleColumns'];
    			$params['extra'] .= sprintf(' sort_val="%s" ', $params['value']);
    			return locale_nf($params['value']);
    		break;
    		case 'MONTH':
    			$params['extra'] = sprintf(' sort_val="%s" ', $params['value']);
				return strftime('%B', mktime(1, 1, 1, $params['value'], 1) );
			break;
    	}
    }
}

/**
 * CALLBACK[getProvDeclOverviews]
 * item[1093] = {
 *     [handle] = Item: ProvDeclOverview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
