<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'CRSO.php');

define('BLOCK_RR',273);
define('ITEM_RR_EU',887);
define('ITEM_RR_NON_EU',888);
define('ITEM_RR_SETTINGS', 1010);

/**
 *
 * @package ELC
 * @subpackage Interface
 * @since 06-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceRefundSettings extends InterfaceFactory
{
    /**
     * interface[id] = 109
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceRefundSettings(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';


		/**
		 * This Interface is used in different universes, check the
		 * group_company and set the right properties
		 */
		require_once(IWARE . 'group.php');
		switch ($_SESSION['session_group_group_id'])
		{
			case GROUP_PARTICIPANT:

				if (empty($_SESSION['selected_participant_id']))
					trigger_error('No participant selected', E_USER_ERROR);

		        //mark if we have selected a pid or crso
		        $this->model['currentPID'] = $_SESSION['selected_participant_id'];
		        $this->model['currentCID'] = isset($_REQUEST['crso']['id']) && is_numeric($_REQUEST['crso']['id']) ? (int)$_REQUEST['crso']['id'] : false;
		        $this->model['currentCP'] = false;

		        //now loop and match to crsos
		        $this->model['cps'] = array();
		        $queryObj = new CRSOParticipantProduct($this->db_app);

		    	$pid = $this->model['currentPID'];
		    	$cids = $queryObj->getParticipantCRSOs($pid, 'array', PARTICIPANT_EXPORTER,false,$_SESSION['selected_product_id']);
		    	foreach($cids as $cid)
		    	{
		    		$cp = new CRSOParticipantProduct($this->db_app);
		    		$cp->set($cid,$pid,$_SESSION['selected_product_id']);
		    		$cp->participant->relation->set();
		    		$cp->crso->relation->set();
		    		$this->model['cps'][] = clone $cp;

		        	if($this->model['currentPID'] == $pid && $this->model['currentCID'] == $cid)
						$this->model['currentCP'] = clone $cp;
					$cp = null;
		    	}

		        //if we have not chosen a cid or cp, but have only one possible CRSOParticipant then use it
		        if(!$this->model['currentCP'] && count($this->model['cps']) == 1)
		        {
		        	$cp = current($this->model['cps']);
		        	$this->model['currentCID'] = $cp->crso->relation->id;
		        	$this->model['currentCP'] = $cp;
		        }

			break;

			case GROUP_CRSO:
				if (empty($_REQUEST['participant']['id']))
					trigger_error('No participant selected', E_USER_ERROR);

		        $this->model['currentPID'] = $_REQUEST['participant']['id'];
		        $this->model['currentCID'] = $_SESSION['crso_relation_id'];

	    		$cp = new CRSOParticipantProduct($this->db_app);
	    		$cp->set($this->model['currentCID'],$this->model['currentPID'],$_SESSION['selected_product_id']);
	    		if (!$cp->crso->relation->set())
	    			trigger_error('Failed setting CRSOParticipantProduct', E_USER_ERROR);

		        $this->model['currentCP'] = $cp;

		        $this->model['cps'] = array();
		        $this->model['cps'][] = clone $cp;
			break;

			default:
				trigger_error('Interface not available for group', E_USER_ERROR);
		}

        // see if GM is authorized, approved, woteva
        $this->model['hasApprovedGM'] = false;
        $this->model['operator_translation'] = '';
        if($this->model['currentCP'])
        {
        	if($this->model['currentCP']->general_manager&&($this->model['currentCP']->general_manager_authorized==GENERAL_MANAGER_AUTHORIZED))
				$this->model['hasApprovedGM'] = true;

			$country = new Country($this->db_app, $this->model['currentCP']->crso->relation->getCountry(),true);
			if($country->has(HasCRSORefundConfirmations, $_SESSION['selected_product_id'])) {
				$this->model['operator_translation'] = $this->langIf->getText('CRSO');
			} else {
				$this->model['operator_translation'] = $this->langIf->getText('Accountant');
			}
        }
    }

    function _filter($q)
    {
    	$rv = parent::_filter($q);
    	if(!$this->model['currentPID'] || !$this->model['currentCID'])
    	{
    		unset($rv[$this->id][BLOCK_RR][ITEM_RR_EU],$rv[$this->id][BLOCK_RR][ITEM_RR_NON_EU]);
    	}
    	else
    	{
    		unset($rv[$this->id][BLOCK_RR][ITEM_RR_SETTINGS]);
    	}
    	return $rv;
    }

    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        if(isset($_POST['process']))
        {
        	if($_POST['process']=='save')
        	{
        		if(($_SESSION['session_group_group_id']==GROUP_PARTICIPANT)&& $this->model['currentCP']->hasRefundSettings())
        		{
        			header(sprintf('Location: InterfaceRefundSettings.php?crso[id]=%s&participant[id]=%s&settingsSaved=1', $this->model['currentCP']->crso->relation->id, $this->model['currentCP']->participant->relation->id));
					die();
        		}
        		foreach (array('authorize_refund_request_eu', 'authorize_refund_request_non_eu') as $type)
        		{
					// check if the option is submitted, it could be disabled
					if (isset($_REQUEST['cp'][$type]))
						$this->model['currentCP']->$type = $_REQUEST['cp'][$type];
				}
				if($_SESSION['session_group_group_id'] != GROUP_PARTICIPANT)
				  	if (isset($_REQUEST['cp']['authorize_proof_of_payment']))
							$this->model['currentCP']->authorize_proof_of_payment = $_REQUEST['cp']['authorize_proof_of_payment'];

				if(isset($_REQUEST['cp']))
				{
					info(sprintf('Refund settings for CRSOParticipant [%s][%s] changed to [%s] by user [%s]',
						$this->model['currentCP']->crso->relation->id,
						$this->model['currentCP']->participant->relation->id,
						str_replace("\n", " ", print_r($_REQUEST['cp'], true)),
						$_SESSION['session_user_id']
					));

                                        log_elc_event('Participant refund settings changed',
                                            $_SESSION['session_user_id'], $this->model['currentCP']->product->id,
                                            $this->model['currentCP']->crso->relation->id,
                                            $this->model['currentCP']->participant->relation->id
                                        );
				}
				$this->model['currentCP']->update();

				header(sprintf('Location: InterfaceRefundSettings.php?crso[id]=%s&participant[id]=%s&settingsSaved=1', $this->model['currentCP']->crso->relation->id, $this->model['currentCP']->participant->relation->id));
				die();
        	}
        }

        if(isset($_REQUEST['settingsSaved']) && $_REQUEST['settingsSaved'])
        	$this->script = '
						addEvent(window, \'load\', function(){
						MessageBox.prototype.Information(\''.$this->langIf->getText("TITLE_Refund Settings").'\', \''.
						$this->langIf->getText("Refund Settings Saved!").'\');
						});
					';
    }
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 06-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceRefundSettingsItem extends InterfaceItem
{

    /**
     * item[891] = participant[id]
     */
    function getParticipantSelect()
    {
		$this->_hideInput = true;
		$p = new Participant($this->db_app, array('relation'=>new Relation($this->db_app,$this->if_obj->model['currentPID'])));
		$p->relation->set();
		$this->options = array(array('text'=>$p->relation->name,
			'value'=>$p->relation->id, 'selected'=>true));
    }

    /**
     * item[892] = crso[id]
     */
    function getCRSOSelect()
    {
		if(sizeof($this->if_obj->model['cps']) == 1)
			$this->_hideInput = true;
		else
			$this->options[] = array('text'=>'- '.$this->langIf->getText('Select CRSO').' -', 'value'=>'');

		foreach ($this->if_obj->model['cps'] as $crsoP)
		{
			if(	$this->if_obj->model['currentPID'] && $this->if_obj->model['currentPID'] != $crsoP->participant->relation->id)
				continue;
			$this->options[] = array('text'=>$crsoP->crso->relation->name,
				'value'=>$crsoP->crso->relation->id,
				'selected'=> $this->if_obj->model['currentCID'] && ($crsoP->crso->relation->id == $this->if_obj->model['currentCID']));
		}

		//$this->properties['onchange'] = "_loadBlocks();";
        $this->postscript = "
		addEvent(window, 'load', function(){

			ifHandler.watchFormOnChange('interface_form', function(){
	            var crso_id = \$\$s('".$this->name."')[0].value;
				var participantId = '".$this->if_obj->model['currentPID']."';
				var interface_id=".$this->if_obj->id.";
	            var url = '&interface_id='+interface_id+'&crso[id]='+crso_id+'&participant[id]=' + participantId;
				if(crso_id && participantId)
					ifHandler.loadBlock( new Array(273,274), url );
				else
					document.location.href='InterfaceRefundSettings.php?' + (crso_id ? 'crso[id]=' + crso_id : '') + (participantId ? 'participant[id]=' + participantId : '')
	        }, ['ifI_892', 'ifI_891']);

		});";
    }

    /**
     * item[887] = cp[authorize_refund_request_eu]
     * item[888] = cp[authorize_refund_request_non_eu]
     */

    function getItemValue()
    {
    	$this->options = array();
    	$this->columncount = 1;
       	switch($this->name)
    	{
/*
    		case 'cp[authorize_refund_request_eu]':
    			$this->options[] = array('text'=>$this->langIf->getText('Importer through email'),
				'value'=>REFUND_AUTHORIZE_EMAIL,
				'checked'=>$this->_getChecked($this->name, REFUND_AUTHORIZE_EMAIL));
*/
    		case 'cp[authorize_proof_of_payment]':
    			($this->name== 'cp[authorize_proof_of_payment]') && $this->options[] = array('text'=>$this->if_obj->model['operator_translation'],
    			'value'=>REFUND_AUTHORIZE_OPERATOR,
    			'checked'=>$this->_getChecked($this->name, REFUND_AUTHORIZE_OPERATOR));
    		case 'cp[authorize_refund_request_non_eu]':
    		default:

	   			$a = array('text'=>$this->langIf->getText('General Manager'),
						'value'=>REFUND_AUTHORIZE_GENERAL_MANAGER,
						'checked'=>$this->_getChecked($this->name, REFUND_AUTHORIZE_GENERAL_MANAGER),
						);
				if (!$this->if_obj->model['hasApprovedGM'])
					$a['disabled'] = 'disabled';

				$this->options[] = $a;

    			$this->options[] = array('text'=>$this->langIf->getText('Statutory Auditor'),
						'value'=>REFUND_AUTHORIZE_STATUTORY_AUDITOR,
    					'checked'=>$this->_getChecked($this->name, REFUND_AUTHORIZE_STATUTORY_AUDITOR));
    		break;
    	}

    	/**
		 * The participant is only allowed to set preferences the first
		 * time, after that he has to make a request to the CRSO
		 * https://trac.basesoft.com/iqelc/ticket/90
		 */
		 if ($_SESSION['session_group_group_id'] == GROUP_PARTICIPANT)
		 {
		 	foreach($this->options as $option)
		 	{
		 		if ($option['checked'])
		 			$this->properties['disabled'] = 'disabled';
		 	}
		 }
    }

    function getRefundSettings()
    {
    	$data = array();
    	foreach($this->if_obj->model['cps'] as $cp)
    	{
    		$row = array('Item: Participant' => $cp->participant->relation->name, 'Item: CRSO' => $cp->crso->relation->name);
    		$this->if_obj->model['currentCP'] =& $cp;
    		$row['Item: RefundRequest EU'] = array(
//				array($this->langIf->getText('Importer through email'),
//					$this->_getChecked('cp[authorize_refund_request_eu]', REFUND_AUTHORIZE_EMAIL)),
				array($this->langIf->getText('General Manager'),
					$this->_getChecked('cp[authorize_refund_request_eu]', REFUND_AUTHORIZE_GENERAL_MANAGER)),
				array($this->langIf->getText('Statutory Auditor'),
					$this->_getChecked('cp[authorize_refund_request_eu]', REFUND_AUTHORIZE_STATUTORY_AUDITOR))
    						);
    		$row['Item: RefundRequest Non-EU'] = array(
//				array($this->langIf->getText('Importer through email'),
//					$this->_getChecked('cp[authorize_refund_request_non_eu]', REFUND_AUTHORIZE_EMAIL)),
				array($this->langIf->getText('General Manager'),
					$this->_getChecked('cp[authorize_refund_request_non_eu]', REFUND_AUTHORIZE_GENERAL_MANAGER)),
				array($this->langIf->getText('Statutory Auditor'),
					$this->_getChecked('cp[authorize_refund_request_non_eu]', REFUND_AUTHORIZE_STATUTORY_AUDITOR))
    						);
			$row['Item: RefundRequest ProofOfPayment'] = array(
				array($this->if_obj->model['operator_translation'],
					$this->_getChecked('cp[authorize_proof_of_payment]', REFUND_AUTHORIZE_OPERATOR)),
				array($this->langIf->getText('General Manager'),
					$this->_getChecked('cp[authorize_proof_of_payment]', REFUND_AUTHORIZE_GENERAL_MANAGER)),
				array($this->langIf->getText('Statutory Auditor'),
					$this->_getChecked('cp[authorize_proof_of_payment]', REFUND_AUTHORIZE_STATUTORY_AUDITOR))
    						);
    		$data[] = $row;
    	}
    	//put it in a table
    	require_once(IWARE.'table_factory.php');
    	$tf = new TableFactory($data, $this->langIf);
		$tf->setValueProcessor(
			array(
				'Item: RefundRequest EU',
				'Item: RefundRequest Non-EU',
				'Item: RefundRequest ProofOfPayment'
			), array($this, '_tfValueProcessor'));
    	$this->text = $tf->getXHTMLTable('Professional', 2);

    }

    function _tfValueProcessor($args)
    {
    	//debug(print_r($args['value'], true));
    	$rv = '<table border="0">';
    	foreach($args['value'] as $item)
    		$rv .= '<tr><td>'.$item[0].'</td><td><img src="/'.SITE_PATH.'images/i-w/'.($item[1]?'true':'false').'.gif" /></td></tr>';
    	$rv .= '</table>';
    	return $rv;
    }

    function _getChecked($item_name, $authorize_by)
    {
		if($this->if_obj->model['currentCP'])
		{
			$property = substr($item_name, 3, -1); // cp[authorize_refund_request_eu] -> authorize_refund_request_eu
			return $this->if_obj->model['currentCP']->$property == $authorize_by;
		}
		return false;
    }
    function getGeneralManagerMessage()
    {
    	$this->twoCols = true;
 		$this->text = '';

    	$message = $this->langIf->getText('There is no approved General Manager');
    	$this->text .= $this->if_obj->model['hasApprovedGM']?'':'<p class="warning">'.$message.'</p>';

    	// add a warning for the participant
    	$message = $this->langIf->getText('Warning: Refund settings can be edited once');
		if ($_SESSION['session_group_group_id'] == GROUP_PARTICIPANT)
			$this->text .= '<p class="warning">'.$message.'</span>';

    }
    function get2colTxt()
    {
    	$this->twoCols = true;
    }

    /**
     * item[889] =
     */
    function getButtons()
    {
    	if(!$this->if_obj->model['currentCP'])
    	{
    		$this->_BarButtons = 0;
    		return;
    	}
        if(($_SESSION['session_group_group_id']==GROUP_PARTICIPANT)&&
        	$this->if_obj->model['currentCP']->hasRefundSettings(array('authorize_refund_request_eu','authorize_refund_request_non_eu')))
        {
        	$this->_BarButtons = 0;
    		return;
        }
    	$this->_BarButtons = BUTTON_SAVE;
     }
}

/**
 * CALLBACK[getItemValue]
 * item[886] = {
 *     [handle] = Item: ProofOfPayment
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[887] = {
 *     [handle] = Item: RefundRequest EU
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * item[888] = {
 *     [handle] = Item: RefundRequest Non-EU
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[889] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
