<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
/**
 *
 * @package
 * @subpackage Interface
 * @since 04-03-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceTicketDetails extends InterfaceFactory
{
    /**
     * interface[id] = 186
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceTicketDetails(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        // Validate if the User has a linked trac account.
        $user = new ELCUser($this->db_app, $_SESSION['session_user_id'], true);
        if(!defined('TRAC_URL')) {
        	throw new InterfacePermissionException('No trac instance linked');
        }
        if(!$user->trac_user) {
        	throw new InterfacePermissionException('Permission denied');
        }

        $user->trac_user->set();
        $this->model['trac'] = new TracClient(TRAC_URL, $user->trac_user->user_name, $user->trac_user->password);

        // Validate if the ticket requested is indeed one of the current users' tickets.
        if(!$this->model['trac']->isReporter($_REQUEST['ticket']['id'])) {
        	throw new InterfacePermissionException('You are not the reporter of this ticket');
        }
        $this->model['ticket'] = $this->model['trac']->getTicketDetails($_REQUEST['ticket']['id']);

        // Store the ticket id in a hidden field for post processing.
        $this->hiddenFields = array(
			array('name'=>'ticket[id]','value'=>$_REQUEST['ticket']['id'])
    	);

    	if(isset($_GET['message'])) {

    		switch($_GET['message']) {
    			case 'comment':
    				$mtitle = 'Comment saved';
    				$mcontent = 'Your comment was succesfully saved.';
    			break;
    			case 'reopened':
    				$mtitle = 'Ticket reopened';
    				$mcontent = 'The ticket has been reopened for evaluation.';
    			break;
    			// this is when the user arrived from the ticket form interface.
    			case 'newticket':
    				$mtitle = 'Ticket filed';
    				$mcontent = 'Your ticket has been filed succesfully.';
    			break;
    			// They supplied a message that was not added by the system, they are messing around.
    			default:
    				throw new InterfacePermissionException('Incorrect message supplied.');
    		}

    		$this->script = "MessageBox.prototype.Information(
				'".jsescape($langIf->getText($mtitle))."',
				'".jsescape($langIf->getText($mcontent))."',
				function() {
					this.close();
				}
			);";

    	}
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
    	if(isset($_POST['process'])) {
			$extra = '';
			// If we are saving, make sure the comment field is not empty.
			if($_POST['process'] == 'save') {
				if(!empty($_POST['ticket']['comment'])) {
					$this->model['trac']->addComment($_POST['ticket']['id'], htmlentities($_POST['ticket']['comment']));
					$extra = '&message=comment';
				}
			} elseif($_POST['process'] == 'reopen') {
				$this->model['trac']->reOpen($_POST['ticket']['id'], htmlentities($_POST['ticket']['comment']));
				$extra = '&message=reopened';
			}
			header('Location: InterfaceTicketDetails.php?ticket[id]='.$_POST['ticket']['id'].$extra);
			exit();
		}
    }

    /**
     * Get the interface header, this is basically the same layout as a real trac ticket has.
     */
   	function getInterfaceHeader() {
   		$rv = parent::getInterfaceHeader();

   		// Similar as DeclarationForm we add some header stuff here.
   		$rv .= '
   			<h2 class="ticket">'.$this->langIf->getText('Ticket').' #'.$_REQUEST['ticket']['id'].' - '.$this->model['ticket']['Summary'].'</h2>
   			<div class="ticket">
   			<table class="ticket" width="100%">
   				<tbody>';

   		$close = false;
   		foreach($this->model['ticket']['ticket'] as $field => $value) {
   			if(!$close) {
   				$rv .= '<tr>';
   			}
   			$rv .= sprintf('<th>%s</th><td>%s</td>',
   				$this->langIf->getText($field),
   				$value
   			);
   			if($close) {
   				$rv .= '</tr>';
   				$close = false;
   			} else {
   				$close = true;
   			}
   		}
   		$rv .= '<tr>
   				<th colspan="4" class="description">'.$this->langIf->getText('Description').'</th>
   			</tr>
   			<tr>
   				<td colspan="4" class="description">'.$this->model['trac']->format($this->model['ticket']['Description']).'</td>
   			</tr>';

   		$rv .= '</tbody>
   			</table>
   			</div>';

   		if(count($this->model['ticket']['changelog'])===0) {
   			return $rv;
   		}

   		$rv .= '<h2 class="ticket">'.$this->langIf->getText('Change History').'</h2>
   			<div class="ticketchangecontainer">
   		';
   		foreach($this->model['ticket']['changelog'] as $dt => $values) {
   			$rv .= '<div class="ticketchange">
   				<h3>'.$this->langIf->getText('by {reporter} at {time}', null, array('reporter'=> $values['reporter'], 'time'=> $this->model['trac']->convertDateTime($dt))).'</h3>';
   			if(count($values['changes']) > 0) {
   				$rv .= '<ul>';
   				foreach($values['changes'] as $field => $changes) {
   					// We changed a value
   					if($changes['old']&& $changes['new']) {
	   					$rv .= '<li>'.$this->langIf->getText('{field} changed from {from} to {to}', null, array(
	   						'field'=>sprintf('<strong>%s</strong>', $this->langIf->getText($field)),
	   						'from'=>'<em>'.$changes['old'].'</em>',
	   						'to'=>'<em>'.$changes['new'].'</em>',
	   					)).'</li>';
	   				// We set a new value to an empty field.
   					} elseif($changes['new']) {
   						$rv .= '<li>'.$this->langIf->getText('{field} set to {to}', null, array(
	   						'field'=>sprintf('<strong>%s</strong>', $this->langIf->getText($field)),
	   						'to'=>'<em>'.$changes['new'].'</em>',
	   					)).'</li>';
	   				// We deleted the value.
   					} else {
   						$rv .= '<li>'.$this->langIf->getText('{field} {from} deleted', null, array(
	   						'field'=>sprintf('<strong>%s</strong>', $this->langIf->getText($field)),
	   						'from'=>'<em>'.$changes['old'].'</em>',
	   					)).'</li>';
   					}
   				}
   				$rv .= '</ul>';
   			}
   			// If there was a comment made, show it.
   			if(array_key_exists('comment', $values)) {
   				$rv .= sprintf('<div>%s</div>', $this->model['trac']->format($values['comment']));
   			}
   			$rv .= '</div>';
   		}
   		$rv .= '</div>';

   		return $rv;
   	}
}
/**
 * @package
 * @subpackage Interface
 * @since 04-03-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceTicketDetailsItem extends InterfaceItem
{
    /**
     * item[1500] = ticket[comment]
     */
    function getCommentField()
    {
    	//$this->twoCols = true;
    	$this->properties['style'] = 'width: 100%; height: 200px;';
    }
    /**
     * item[1501] =
     */
    function getButtons()
    {
    	$this->_BarButtons = BUTTON_SAVE+BUTTON_CANCEL;
    	$this->_ButtonOptions[BUTTON_CANCEL] = array('onclick'=>"go('InterfaceTicketOverview.php');");
    	// The ticket was previously closed, but the user wants to reopen it.
    	if($this->if_obj->model['ticket']['ticket']['Status'] == 'closed') {
    		$this->_addCustomButton($this->langIf->getText('Re-open ticket'),'button', array(
    			'onclick'=>"$$('process').value = 'reopen';$('interface_form').submit();"));
    	}
    }

}

/**
 * CALLBACK[getCommentField]
 * item[1500] = {
 *     [handle] = Item: Ticket comment
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 1
 *
 * CALLBACK[getButtons]
 * item[1501] = {
 *     [handle] = Item: Button bar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
