<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'Relation.php');
/**
 *
 * @package
 * @subpackage Interface
 * @since 05-03-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceTicketForm extends InterfaceFactory
{
    /**
     * interface[id] = 187
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceTicketForm(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        // Validate if the User has a linked trac account.
        $user = new ELCUser($this->db_app, $_SESSION['session_user_id'], true);
        if(!defined('TRAC_URL')) {
        	throw new InterfacePermissionException('No trac instance linked');
        }
        if(!$user->trac_user) {
        	throw new InterfacePermissionException('Permission denied');
        }
        $user->trac_user->set();
        $this->model['trac'] = new TracClient(TRAC_URL, $user->trac_user->user_name, $user->trac_user->password, $user->info_person);
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
    	if(isset($_POST['process'])) {
			if($_POST['process'] == 'save') {

				if(empty($_POST['ticket']['summary'])||empty($_POST['ticket']['description'])) {
					header('Location: InterfaceTicketForm.php');
					exit();
				}

				$crso = new Relation($this->db_app, $_SESSION['crso_relation_id'], true);
				$name = (strlen($crso->name) > 30) ? substr($crso->name, 0, 10).'...'.substr($crso->name, -10) : $crso->name;
				$summary = sprintf('[%s] %s', $name, $_POST['ticket']['summary']);

				$id = $this->model['trac']->createTicket(
					$summary,
					$_POST['ticket']['description'],
					$_POST['ticket']['priority'],
					$_POST['ticket']['type']
				);

				header('Location: InterfaceTicketDetails.php?ticket[id]='.$id.'&message=newticket');
				exit();
			}
		}
    }
}
/**
 * @package
 * @subpackage Interface
 * @since 05-03-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceTicketFormItem extends InterfaceItem
{

    /**
     * item[1502] = ticket[summary]
     */
    function getSummary()
    {
    	$this->properties['style'] = 'width: 100%;';
    }

    /**
     * item[1503] = ticket[description]
     */
    function getDescription()
    {
    	$this->twoCols = true;
    	$this->properties['style'] = 'width: 100%; height: 200px;';
    }

    /**
     * item[1504] = ticket[priority]
     */
    function getPrioritySelect()
    {
    	$default = 'major';
    	$this->options = array();
    	$priorities = $this->if_obj->model['trac']->getPriorities();
    	foreach($priorities as $priority) {
    		$opt = array('text'=>$this->langIf->getText($priority), 'value'=>$priority);
    		if($priority == $default) {
    			$opt['selected'] = 'selected';
    		}
    		$this->options[] = $opt;
    	}
    }

    /**
     * item[1505] = ticket[type]
     */
    function getTypeSelect()
    {
    	$this->options = array();
    	$types = $this->if_obj->model['trac']->getTypes();
    	foreach($types as $type) {
    		$this->options[] = array('text'=>$this->langIf->getText($type), 'value'=>$type);
    	}
    }

    /**
     * item[1506] =
     */
    function getButtons()
    {
    	$this->_BarButtons = BUTTON_SAVE+BUTTON_CANCEL;
    	$this->_ButtonOptions[BUTTON_CANCEL] = array('onclick'=>"go('InterfaceTicketOverview.php');");
    }

}

/**
 * CALLBACK[getSummary]
 * item[1502] = {
 *     [handle] = Item: Ticket Summary
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getDescription]
 * item[1503] = {
 *     [handle] = Item: Ticket Description
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getPrioritySelect]
 * item[1504] = {
 *     [handle] = Item: Ticket Priority
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getTypeSelect]
 * item[1505] = {
 *     [handle] = Item: Ticket Type
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[1506] = {
 *     [handle] = Item: Button bar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
