<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
/**
 *
 * @package Trac
 * @subpackage Interface
 * @since 04-03-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceTicketOverview extends InterfaceFactory
{
    /**
     * interface[id] = 185
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceTicketOverview(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        // Validate if the User has a linked trac account.
        $user = new ELCUser($this->db_app, $_SESSION['session_user_id'], true);
        if(!defined('TRAC_URL')) {
        	throw new InterfacePermissionException('No trac instance linked');
        }
        if(!$user->trac_user) {
        	throw new InterfacePermissionException('Permission denied');
        }
        $user->trac_user->set();
        $this->model['trac'] = new TracClient(TRAC_URL, $user->trac_user->user_name, $user->trac_user->password);
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {

    }
}
/**
 * @package
 * @subpackage Interface
 * @since 04-03-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceTicketOverviewItem extends InterfaceItem
{
	private $rowsPerPage = 25;

    /**
     * item[1497] =
     */
    function getTicketOverview()
    {
    	$trac =& $this->if_obj->model['trac'];

    	$cp = isset($_REQUEST['page']) ? $_REQUEST['page'] : 0;
    	$offset = $cp*$this->rowsPerPage;
    	list($overview, $cnt) = $trac->getTicketOverview($offset, $this->rowsPerPage);

    	require_once(IWARE.'TableFactoryColumnPlugin.php');
		$tf = new TableFactoryColumnPlugin(false,$this->langIf, $this->db, $this->db_app);
		$tf->setData($overview, 'TicketOverview',false,true);
		$tf->setEmptyText($this->langIf->getText('No tickets found'));

		$tf->setSortingFunctions(array(
			'Ticket Summary'=>"'String'",
			'Ticket Type'=>"'String'",
			'Ticket Status'=>"'String'",
			'Ticket Priority'=>"'String'",
			'Ticket Created'=>"'String'",
			'Ticket Last Updated'=>"'String'"
		));

		$tf->setValueProcessor(TF_ALL, array($this, 'tfValueProcessor'));

		$this->text = $this->_renderPaging($cnt).$tf->getXHTMLTable('Professional',2).$this->_renderPaging($cnt);
    }

    function _renderPaging($rows) {
    	$rv = '<div style="text-align: right;">[ ';

    	$cp = isset($_REQUEST['page']) ? $_REQUEST['page'] : 0;
    	$pages = ceil($rows/$this->rowsPerPage);
    	if($pages==1){
    		return '';
    	}

    	for($i = 1;$i<=$pages; $i++)
			$rv .= (($i-1)==$cp)?'<strong>'.$i.'</strong> ':sprintf('<a href="/%sscripts/elc/crso/InterfaceTicketOverview.php?page=%s">%s</a> ',SITE_PATH, ($i-1),$i);
		return $rv.' ]</div>';
    }

    function tfValueProcessor($args) {
    	switch($args['head']) {
    		case 'Ticket Summary':
    			return sprintf('<a href="InterfaceTicketDetails.php?ticket[id]=%s">%s</a>',
    				$args['row']['ID'],
    				$args['value']
    			);
    		break;
    		case 'Ticket Type':
    		case 'Ticket Priority':
    		case 'Ticket Status':
    			return $this->langIf->getText($args['value']);
    		break;
    	}
    	return $args['value'];
    }

    /**
     * item[1496] = create
     */
    function getButtons()
    {
    	$this->_BarButtons = 0;
    	$this->_addCustomButton($this->langIf->getText('Create new ticket'),'button', array(
    			'onclick'=>"go('InterfaceTicketForm.php');"));
    }


}

/**
 * CALLBACK[getTicketOverview]
 * item[1497] = {
 *     [handle] = Item: Ticket overview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
