<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'RelationUser.php');
require_once(ELC_CLASSES.'Trust.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'ProvisionalDeclaration.php');
require_once(IWARE.'group.php');

define('BLOCK_SUBROGATION_APPROVAL',229);
define('ITEM_APPROVAL_DOCUMENT',1095);


/**
 *
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @since 15-03-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceTrustDetail extends InterfaceFactory
{

    /**
	 * @var boolean Whether this interface is being used in an ajax context
	 */
	var $ajaxUsage = false;

    /**
     * interface[id] = 90
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
	 * @todo make work for CRSO as well, HIDE the details of the provisional
     */
    function InterfaceTrustDetail(&$db, &$db_app, &$langIf)
    {
        $this->_setAjaxUsage();

		$trust = new Trust($db_app,$_REQUEST['trust'],true);
		$this->model['trust'] =& $trust;

		if($trust->approved==TRUST_DISABLED)
			exit('This Trust has been cancelled');


		parent::InterfaceFactory($db, $db_app, $langIf);
    	$this->FirstColWidth = 25;
		$this->delimiter = '<br />';

		$pd = new ProvisionalDeclaration($this->db_app);
		$this->model['validation_key'] = $key = $pd->getValidationKeyForTrust($trust->id);
		$this->hiddenFields = array(
			array('name'=>'validation_key','value'=>$key)
			);

		$this->_setDocumentRights();

		if(($_SESSION['session_group_group_id']==GROUP_PARTICIPANT)&&($trust->approved==TRUST_APPROVED))
		{
			$this->script = sprintf("
				function terminationRequest()
				{
					MessageBox.prototype.Question('%s','%s',function(){
						var params = new Array();
						params['type'] = '%s';
						params['trust_id'] = '%d';
						params['type_id'] = '%d';
						jsbroker.call('requestTrustTermination',params,function(){
							MessageBox.prototype.Information('%s','%s');
						});
					});
				}
				",
				$this->langIf->getText('MessageBox: Title Termination Request'),
				$this->langIf->getText('MessageBox: Confirm Termination Request'),
				(isset($_REQUEST['subrogator']['id'])?'subrogator':'trustee'),
				$_REQUEST['trust']['id'],
				(isset($_REQUEST['subrogator']['id'])?$_REQUEST['subrogator']['id']:$_REQUEST['trustee']['id']),
				$this->langIf->getText('MessageBox: Title Termination Request'),
				$this->langIf->getText('MessageBox: Termination Request has been sent')
				);
		}
		elseif(($_SESSION['session_group_group_id']==GROUP_CRSO)&&($trust->approved!=TRUST_APPROVED))
		{
			$this->script = sprintf("
				function terminateTrust()
				{
					MessageBox.prototype.Question('%s','%s',function(){
						var params = new Array();
						params['trust_id'] = '%d';
						jsbroker.call('approveTrustTermination',params,function(){
							MessageBox.prototype.Information('%s','%s',function(){
								go('/%s');
							});
						});
					});
				}
				",
				$this->langIf->getText('MessageBox: Title Terminate Trust'),
				$this->langIf->getText('MessageBox: Confirm Termination of Trust'),
				$_REQUEST['trust']['id'],
				$this->langIf->getText('MessageBox: Title Terminate Trust'),
				$this->langIf->getText('MessageBox: The Trust has been terminated'),
				SITE_PATH
				);
		}
    }
    function init($interface_id)
    {
    	parent::init($interface_id);
    	if(!$this->ajaxUsage)
		{
			$this->model['trust']->{$this->relation_type}->relation->set();
			setPageTitlePostfix('('.$this->model['trust']->{$this->relation_type}->relation->name.')');
		}
    }

    function _filter($q)
    {
    	$rv = parent::_filter($q);

    	$blocks = array(
    						228, //Block: SubrogationDetails
    						BLOCK_SUBROGATION_APPROVAL, //Block: SubrogationApproval
    						230  //Block: ButtonBar
    					);

    	switch ($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
				//this interface only really used in ajax calls
				//may need to check $this->ajaxUsage in future but for now remove buttons and approval widget
				$blocks = array(BLOCK_SUBROGATION_APPROVAL, 230);

				//we need a type
				$this->relation_type = isset($_REQUEST['subrogator']['id'])?'subrogator':'trustee';
				$this->relation_type_id = $_REQUEST[$this->relation_type]['id'];
				break;

			case GROUP_PARTICIPANT:
				$relation_id = isset($_REQUEST['subrogator']['id'])?$_REQUEST['subrogator']['id']:$_REQUEST['trustee']['id'];
				$this->relation_type = isset($_REQUEST['subrogator']['id'])?'subrogator':'trustee';
				$ru = new RelationUser($this->db_app);
				$ru->set($_SESSION['session_user_id'],$relation_id);
				if(!$ru->hasClearance(ClearanceFeeRegistration))
				{
					alert('['.$_SESSION['session_user_id'].'] no permission for subrogation details for trust ['.$this->model['trust']->id.']');
					throw new InterfacePermissionException('['.$_SESSION['session_user_id'].'] no permission for subrogation details for trust ['.$this->model['trust']->id.']');
				}
				$clearance = array($this->model['trust']->trustee->relation->id,$this->model['trust']->subrogator->relation->id,$this->model['trust']->crso->relation->id);
				if(!in_array($relation_id,$clearance))
				{
					alert('['.$_SESSION['session_user_id'].'] no permission for Trust ['.$this->model['trust']->id.']');
					throw new InterfacePermissionException('['.$_SESSION['session_user_id'].'] no permission for Trust ['.$this->model['trust']->id.']');
				}
				$this->relation_type_id = $_REQUEST[$this->relation_type]['id'];
				$blocks = array();

				$remote = null;
				// Check for any active Remote Viewer connections.
				if(IWRegistry::isRegistered('remote_viewer')) {
					$remote = IWRegistry::registry('remote_viewer');
				}
				if($remote &&  $remote->participant->relation->id!=$this->model['trust']->trustee->relation->id) {
					alert('['.$_SESSION['session_user_id'].'] no permission for Trust details due to Remote Viewer for ['.$this->model['subrogator']->relation->id.']');
					throw new InterfacePermissionException('['.$_SESSION['session_user_id'].'] no permission for Trust for ['.$this->model['subrogator']->relation->id.']');
				}


				break;

			case GROUP_CRSO:
				if ($this->model['trust']->crso->relation->id != $_SESSION['crso_relation_id'])
	    		{
					alert('['.$_SESSION['session_user_id'].'] no permission for subrogation details for trust ['.$trust->id.']');
					throw new InterfacePermissionException('['.$_SESSION['session_user_id'].'] no permission for subrogation details for trust ['.$trust->id.']');
	    		}
	    		$this->relation_type = 'crso';
	    		$this->relation_type_id = $_SESSION['crso_relation_id'];
	    		$blocks = array();
				break;

			default:
				break;
		}

		if(!$this->model['doc'])
			unset($rv[$this->id][BLOCK_SUBROGATION_APPROVAL][ITEM_APPROVAL_DOCUMENT]);

		foreach ($blocks as $block)
			unset($rv[$this->id][$block]);

		debug('relation_type: '.$this->relation_type);
		debug('relation_type_id: '.$this->relation_type_id);

		return $rv;
    }

    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
    	// translation cache
    	foreach (array(
    		'Notification: Trust Request',
    		'Notification: {SUBROGATOR} requests a trust with {TRUSTEE} for CRSO {CRSO}',
    		'Notification: Trust Request',
    		'Notification: Participant {SUBROGATOR} requests a trust with Participant {TRUSTEE}',
    		'Notification: Trust Accepted',
    		'Notification: Trust between {SUBROGATOR} and {TRUSTEE} has been accepted by CRSO {CRSO}',
    		'Notification: Empty provisional',
    		'Notification: Empty provisional message'
    		) as $tr) {
    			$this->langIf->getText($tr);
    	}

        if(isset($_POST['process']))
		{
			if($_POST['process'] == 'save')
			{
				if($_REQUEST['validation_key']!=$this->model['validation_key'])
				{
					$thrownException .= $this->langIf->getText('Subrogator Changed Provisional While Reviewing, please revalidate');
					return;
				}

				$this->model['trust']->subrogator->relation->set();
				$this->model['trust']->trustee->relation->set();
				$this->model['trust']->crso->relation->set();

				require_once(ELC_CLASSES . 'Notification.php');

				$recipients = $map = array();
				$map['sender'] = $this->model['trust']->{$this->relation_type}->relation;
				$map['type'] = NOTIFICATION_TYPE_INFO;

				$header_spid = 0;
				$send = false;

				$this->db->begin();

				switch($this->relation_type)
				{
					case 'subrogator': // Is this even possible?
						//trigger_error('Invalid CASE, subrogators should use InterfaceSubrogationApplication', E_USER_ERROR);
					break;
					case 'trustee':
						$map['subject'] = $this->langIf->getText('Notification: Trust Request');
						$map['message'] = $this->langIf->getText('Notification: Participant {SUBROGATOR} requests a trust with Participant {TRUSTEE}',null,
							array('SUBROGATOR'=>$this->model['trust']->subrogator->relation->name,
								'TRUSTEE'=>$this->model['trust']->trustee->relation->name));

						$recipients[] = array('recipient'=>$this->model['trust']->crso->relation,
							'callback'=>"go('/".SITE_PATH."scripts/elc/crso/InterfaceTrustDetail.php?trust[id]=".$this->model['trust']->id."');");

						$header_spid = 62420;
						$send = isset($_POST['trust']['approved'])?($_POST['trust']['approved'] == TRUST_TRUSTEE_APPROVED):false;

                                                if($send)
                                                {
                                                    log_elc_event(sprintf('Trust between subrogator [%s] and trustee [%s] approved by trustee',
                                                            $this->model['trust']->subrogator->relation->name,
                                                            $this->model['trust']->trustee->relation->name),
                                                        $_SESSION['session_user_id'], $_SESSION['selected_product_id'],
                                                        $this->model['trust']->crso->relation->id, $this->model['trust']->trustee->relation->id
                                                    );
                                                }
                                                else
                                                {
                                                    log_elc_event(sprintf('Trust between subrogator [%s] and trustee [%s] rejected by trustee',
                                                            $this->model['trust']->subrogator->relation->name,
                                                            $this->model['trust']->trustee->relation->name),
                                                        $_SESSION['session_user_id'], $_SESSION['selected_product_id'],
                                                        $this->model['trust']->crso->relation->id, $this->model['trust']->trustee->relation->id
                                                    );
                                                }

					break;
					case 'crso':
						$map['subject'] = $this->langIf->getText('Notification: Trust Accepted');
						$map['message'] = $this->langIf->getText('Notification: Trust between {SUBROGATOR} and {TRUSTEE} has been accepted by CRSO {CRSO}',null,
							array('SUBROGATOR'=>$this->model['trust']->subrogator->relation->name,
								'TRUSTEE'=>$this->model['trust']->trustee->relation->name,
								'CRSO'=>$this->model['trust']->crso->relation->name));

						$recipients[] = array('recipient'=>$this->model['trust']->trustee->relation,
							'callback'=>"go('/".SITE_PATH."scripts/elc/participant/InterfaceTrustDetail.php?trust[id]=".$this->model['trust']->id."&trustee[id]=".$this->model['trust']->trustee->relation->id."');");
						$recipients[] = array('recipient'=>$this->model['trust']->subrogator->relation,
							'callback'=>"go('/".SITE_PATH."scripts/elc/participant/InterfaceTrustDetail.php?trust[id]=".$this->model['trust']->id."&subrogator[id]=".$this->model['trust']->subrogator->relation->id."');");

						// Send a notice to the the subrogator if he can stop doing declarations
						$date =& currentDate();
						require_once(ELC_CLASSES.'ProvisionalDeclaration.php');
						$pd = new ProvisionalDeclaration($this->db_app,
								array('crso'=>$this->model['trust']->crso,
									'participant'=>$this->model['trust']->subrogator,
									'year'=>$date->getYear(),
									'product'=>new Product($this->db_app,$_SESSION['selected_product_id'])));
						if($pd->validateNoDeclaration())
						{
							Notification::send($this->db_app,
								array('subject'=>$this->langIf->getText('Notification: Empty provisional'),
									'message'=>$this->langIf->getText('Notification: Empty provisional message')),
								array(array('recipient'=>$this->model['trust']->subrogator->relation)));
						}

						// transfer the provisional from the subrogator to the trustee
						$pd = new ProvisionalDeclaration($this->db_app,
								array('crso'=>$this->model['trust']->crso,
									'participant'=>$this->model['trust']->subrogator,
									'year'=>$date->getYear(),
									'trust'=>$this->model['trust'],
									'product'=>new Product($this->db_app,$_SESSION['selected_product_id'])));
						$pd->changeParticipantForTrust($this->model['trust']->trustee->relation->id);

						// make sure the trustee always has no_declarations on
						$cp = new CRSOParticipantProduct($this->db_app,
							array('crso'=>$this->model['trust']->crso,
								'participant'=>$this->model['trust']->trustee,
								'product'=>new Product($this->db_app,$_SESSION['selected_product_id'])), true);
						$cp->no_declaration = 0;
						$cp->update();

						$header_spid = 61200;
						$send = ($_POST['trust']['approved'] == TRUST_CRSO_APPROVED);
                                                if($send)
                                                {
                                                    log_elc_event(sprintf('Trust between subrogator [%s] and trustee [%s] approved by CRSO',
                                                            $this->model['trust']->subrogator->relation->name,
                                                            $this->model['trust']->trustee->relation->name),
                                                        $_SESSION['session_user_id'], $_SESSION['selected_product_id'],
                                                        $this->model['trust']->crso->relation->id, null
                                                    );
                                                }
                                                else
                                                {
                                                    log_elc_event(sprintf('Trust between subrogator [%s] and trustee [%s] rejected by CRSO',
                                                            $this->model['trust']->subrogator->relation->name,
                                                            $this->model['trust']->trustee->relation->name),
                                                        $_SESSION['session_user_id'], $_SESSION['selected_product_id'],
                                                        $this->model['trust']->crso->relation->id, null
                                                    );
                                                }
					break;
				}

				if ($send)
					Notification::send($this->db_app, $map, $recipients);

				if($this->model['doc']&& $this->model['doc']['upload'])
					$this->_processDocument();

				if(isset($_POST['trust']['approved']))
					$this->model['trust']->approved |= $_POST['trust']['approved'];
				$this->model['trust']->update();

				$this->db->commit();
			}

			header('location: Interface'.(($this->relation_type=='subrogator')?'Subrogation':'Trust').'Overview.php?'.(($this->relation_type=='subrogator')?'subrogator':'trustee').'[id]='.$this->model['trust']->{(($this->relation_type=='subrogator')?'subrogator':'trustee')}->relation->id);
			die();
		}
    }

	function _setAjaxUsage()
	{
		if(!isset($_REQUEST['ajaxUsage']) || !$_REQUEST['ajaxUsage'])
			return $this->ajaxUsage = false;
		return $this->ajaxUsage = true;
	}

	function _processDocument()
	{
		if(empty($_FILES['document']['name']['name']))
			return;
		require_once(ELC_CLASSES.'RelationDocument.php');
		$rd = new RelationDocument($this->db_app);

		$item = new InterfaceItem($this->db,$this->db_app,$this->langIf);
        $document = $item->getDocument('document[name]',ITEM_APPROVAL_DOCUMENT);
        $document->makePermanent();

        $rd->relation =& $this->model['trust']->subrogator->relation;
        $rd->document =& $document;
        $rd->objectmap['inserttype'] = 'REPLACE';
       	$rd->insert();

       	$rd->relation =& $this->model['trust']->trustee->relation;
        $rd->document =& $document;
        $rd->objectmap['inserttype'] = 'REPLACE';
       	$rd->insert();

       	$this->model['trust']->document = $document;
	}

	function _setDocumentRights()
	{
		if($_SESSION['session_group_group_id']!=GROUP_PARTICIPANT)
			return $this->model['doc'] = false;

		// If the document exists and either trustee/subrogator, possible download
		$doc = array('upload'=>false,'download'=>true);

		$rt = isset($_REQUEST['subrogator']['id'])?'subrogator':'trustee';
		switch($rt)
		{
			case 'trustee':
				// If the document is not there yet, and the participant is the trustee, possible upload
				if(!$this->model['trust']->document)
					$doc['upload'] = true;
			break;
			case 'subrogator':
				// If the document does exists and the participant is the subrogator, possible upload
				$doc['upload'] = true;
			break;
		}
		if($this->model['trust']->document)
			$_REQUEST['Documents'][ITEM_APPROVAL_DOCUMENT] = $this->model['trust']->document->id;

    	$this->model['doc'] = $doc;
	}
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 15-03-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceTrustDetailItem extends InterfaceItem
{

    /**
     * item[773] = 54
     */
    function getSubrogatorCompanyDetails()
    {
		$this->settings = array();
		$this->settings['_hideInputs'] = 'return true;';
		$this->settings['rel'] = 'return new Relation($this->db_app,'.$this->if_obj->model['trust']->subrogator->relation->id.');';
		$this->settings['participant'] = 'return new Participant($this->db_app,array(\'relation\'=> new Relation($this->db_app,'.$this->if_obj->model['trust']->subrogator->relation->id.')));';
		$this->settings['cp'] = new CRSOParticipantProduct($this->db_app, array('crso'=>$this->if_obj->model['trust']->crso, 'participant'=>$this->if_obj->model['trust']->subrogator,'product'=>new Product($this->db_app,$_SESSION['selected_product_id'])), true);

		$this->settings['block_callbacks'][146] = array($this, 'getSubrogatorCompanyDetailsTitle');
    }
	function getSubrogatorCompanyDetailsTitle(&$block)
	{
		$block->setTitle('Company Information Subrogator');
	}

    /**
     * item[774] = 54
     */
    function getTrusteeCompanyDetails()
    {
    	$this->parent->setTitle('Company Information Trustee');

		$this->settings = array();
		$this->settings['_hideInputs'] = 'return true;';
		$this->settings['rel'] = 'return new Relation($this->db_app,'.$this->if_obj->model['trust']->trustee->relation->id.');';
		$this->settings['participant'] = 'return new Participant($this->db_app,array(\'relation\'=> new Relation($this->db_app,'.$this->if_obj->model['trust']->trustee->relation->id.')));';
		$this->settings['cp'] = new CRSOParticipantProduct($this->db_app, array('crso'=>$this->if_obj->model['trust']->crso, 'participant'=>$this->if_obj->model['trust']->trustee,'product'=>new Product($this->db_app,$_SESSION['selected_product_id'])), true);;

		$this->settings['block_callbacks'][146] = array($this, 'getTrusteeCompanyDetailsTitle');
    }
	function getTrusteeCompanyDetailsTitle(&$block)
	{
		$block->setTitle('Company Information Trustee');
	}

    /**
     * item[775] =
     */
    function getProvisionalDeclaration()
    {
		require_once(ELC_CLASSES.'Country.php');
		require_once(ELC_CLASSES.'ProvisionalDeclaration.php');
		$country = new Country($this->db_app);
		$country->set($this->if_obj->model['trust']->crso->getCountry());

		// return if crso is not allowed to view prov details
		require_once(IWARE . 'group.php');
    	if ($_SESSION['session_group_group_id'] == GROUP_CRSO) {

    		require_once(ELC_CLASSES.'RelationUser.php');
    		$ru = new RelationUser($this->db_app);
    		// Check for clearance level.
    		if(!$country->has(HasCrsoProvisionalDetails,$_SESSION['selected_product_id'])||!$ru->hasClearance(CRSOClearanceProvisionalDeclaration, $_SESSION['session_user_id'], $_SESSION['crso_relation_id'])) {
				$this->text = '';
				return;
    		}
		}

		// the owner of the provisional remains the subrogator till the trust is approved
		if ($this->if_obj->model['trust']->approved == TRUST_APPROVED)
			$relation_id = $this->if_obj->model['trust']->trustee->relation->id;
		else
			$relation_id = $this->if_obj->model['trust']->subrogator->relation->id;

		$date = currentDate();
		$ds = array();
		$prov = new ProvisionalDeclaration($this->db_app);
		$prov->set(
			$this->if_obj->model['trust']->crso->relation->id,
			$relation_id,
			$date->getYear(),
			$this->if_obj->model['trust']->id,
			$this->if_obj->model['trust']->product->id);
		if($country->has(HasProvisionalPerMonth,$_SESSION['selected_product_id']))
			$lines = $prov->getLines();
		else
			$lines = $prov->getAggregateLinesByYear();


		foreach($lines as $i => $content)
		{
			if($country->has(HasProvisionalPerMonth,$_SESSION['selected_product_id']))
				$ds[$i]['MONTH'] = strftime('%B',mktime(0,0,0,$content->month,1));
			$ds[$i]['QUANTITY'] = $content->quantity;
			if($country->has(HasDeclOfWeight,$_SESSION['selected_product_id']))
				$ds[$i]['WEIGHT'] = $content->weight;
			if($country->has(HasDeclByCategory,$_SESSION['selected_product_id']))
			{
				$content->product_category->set();
				$ds[$i]['PRODUCT_CATEGORY'] = $content->product_category->name;
			}
			if($country->has(HasDeclarationDimension,$_SESSION['selected_product_id']))
			{
				$content->dimension->set();
				$ds[$i]['CPD_ID'] = $content->dimension->name;
			}
			if($country->has(HasDeclByBrand,$_SESSION['selected_product_id']))
			{
				$content->brand->set();
				$ds[$i]['BRAND'] = $content->brand->name;
			}
		}

		$tf = new TableFactory($ds,$this->langIf);
		$tf->setEmptyText($this->langIf->getText('No declaration lines found'));
		$help = ($this->parent->title != '') ? getHelp($this->langIf, 'Trust Provisional Overview', array('title'=>'Trust Provisional Overview', 'top'=>'0px')) : '';
		$tf->setCaption($help.'<img src="/'.SITE_PATH.'images/block_arrow.png" /> '.$this->langIf->getText('Trust Provisional Overview'), false);
		$tf->setValueProcessor(array('QUANTITY','WEIGHT'), array($this, 'tfValueProcessor'));
		$tf->columnModifier(array('QUANTITY'=>'style="text-align:right;"','WEIGHT'=>'style="text-align:right;"'));

		$this->text = $tf->getXHTMLTable('Professional',2);
    }

    function tfValueProcessor($params)
    {
    	switch ($params['head'])
    	{
    		case 'WEIGHT':
    			return locale_nf($params['value'], 0);
    		case 'QUANTITY':
    			return locale_nf($params['value'], 0);
    	}
    }

    /**
     * item[776] = trust[approved]
     */
    function getApproval()
    {
		$status = TRUST_APPROVED;
		switch($this->if_obj->relation_type)
		{
			case 'subrogator':
				$status = TRUST_SUBROGATOR_APPROVED;
			break;
			case 'trustee':
				$status = TRUST_TRUSTEE_APPROVED;
			break;
			case 'crso':
				$status = TRUST_CRSO_APPROVED;
			break;
		}
		$this->properties['value'] = $status;
		if(($this->if_obj->model['trust']->approved & $status) == $status)
		{
			$this->properties['checked'] = 'checked';
			$this->properties['disabled'] = 'disabled';
			$this->if_obj->model['disableSave'] = true;
		}
		elseif($this->if_obj->relation_type=='crso'&&!($this->if_obj->model['trust']->approved & TRUST_TRUSTEE_APPROVED))
		{
			$this->properties['disabled'] = 'disabled';
			$this->if_obj->model['disableSave'] = true;
		}
		if($this->if_obj->model['doc']&& $this->if_obj->model['doc']['upload'])
			unset($this->if_obj->model['disableSave']);
    }

    /**
     * item[777] =
     */
    function getButtons()
    {
		$this->hiddenFields = array(
			array(
				'name'=>$this->if_obj->relation_type.'[id]',
				'value'=>$this->if_obj->relation_type_id
				),
			array(
				'name'=>'trust[id]',
				'value'=>$_REQUEST['trust']['id']
				)
			);

		$this->_BarButtons = BUTTON_SAVE+BUTTON_CANCEL;
		if(isset($this->if_obj->model['disableSave']))
			$this->_ButtonOptions[BUTTON_SAVE] =array('disabled'=>"disabled");

		$this->_ButtonOptions[BUTTON_CANCEL] =array('onclick'=>"go('Interface".(($this->if_obj->relation_type=='subrogator')?'Subrogation':'Trust')."Overview.php?".$this->if_obj->relation_type."[id]=".$this->if_obj->relation_type_id."')");
		if(($_SESSION['session_group_group_id']==GROUP_PARTICIPANT)&&($this->if_obj->model['trust']->approved==TRUST_APPROVED))
			$this->_addCustomButton($this->langIf->getText('Termination Request'),'button',array('onclick'=>'terminationRequest();'));
		if ($this->if_obj->model['trust']->approved != TRUST_APPROVED&&($_SESSION['session_group_group_id']==GROUP_CRSO))
			$this->_addCustomButton($this->langIf->getText('Terminate Trust'),'button',array('id'=>'terminate_trust','onclick'=>'terminateTrust();'));
    }
    /**
     * item[1095]
     */
    function getDocumentUpload()
    {
    	if(!$this->if_obj->model['doc']['upload'])
    		$this->_hideInput = true;
    }

}

/**
 * CALLBACK[getSubrogatorCompanyDetails]
 * item[773] = {
 *     [handle] = Item: SubrogatorCompany
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getTrusteeCompanyDetails]
 * item[774] = {
 *     [handle] = Item: TrusteeCompany
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getProvisionalDeclaration]
 * item[775] = {
 *     [handle] = Item: ProvisionalDeclaration
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getApproval]
 * item[776] = {
 *     [handle] = Item: SubrogationApproval
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[777] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
