<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
/**
 * For more information on the UK Management report and it's exceptions to normal WEEE
 * Black Box behaviour, please check the wiki page as found at https://trac-pwc.basesoft.com/wiki/UK_EA/update_2009
 * 
 * @package 
 * @subpackage Interface
 * @since 06-06-2009
 * @copyright 2009 BaseSoft B.V.
 * @version $Id$
 */

define('UK_MANAGEMENT_FILTER_BLOCK', 500);
define('UK_MANAGEMENT_PARTICIPANT_ITEM', 1438); 

class InterfaceUKManagementReport extends InterfaceFactory
{
    /**
     * interface[id] = 184
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceUKManagementReport(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';
        
        //$date =& currentDate();
        //$year = (int)$date->getYear();
        // TODO: management report doesn't work with the changes made for 2010, so only enable it for year<=2009
        $year = 2009;
        $this->model['year'] = isset($_REQUEST['year']) ? (int)$_REQUEST['year']:$year;
        
        // Make sure the CRSO cannot go back before 2009.
    	require_once(ELC_CLASSES.'uk/uk_define.php');
    	// Even though the CRSO now has access to details (ticket #2064) we still dont allow them acces to before 2009
    	$this->model['minyear'] = ($_SESSION['session_group_group_id'] == GROUP_SYSADMIN)?START_YEAR:2009;
    	if($this->model['year'] < $this->model['minyear'])
    		$this->model['year'] = $this->model['minyear'];
        
        // TODO: fix this
    	$this->model['crso_id'] = 163; // HARD CODED UK CRSO ID
       	$this->model['participant_id'] = !empty($_REQUEST['participant']['id']) ? (int)$_REQUEST['participant']['id']:null;
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        
    }

	function getInterfaceHeader()
    {
    	$rv = parent::getInterfaceHeader();
    	
		$rv .= '<div style="font-size: 120%;">
			'.$this->langIf->getText('UK Management Report Introduction Text'). '
			<br /><br />
			<span style="font-weight: bold;">'.$this->langIf->getText('Exception Product Lamps and Luminaires').'</span><br /><br />
			'.$this->langIf->getText('Exception Lamps and Luminaries Description').'
			</div><br />
		';
		
		return $rv;
    }
}

/**
 * @package 
 * @subpackage Interface
 * @since 06-06-2009
 * @copyright 2009 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceUKManagementReportItem extends InterfaceItem
{
	var $__totalsLabel = 'Total EA WEEE Actuals';
	var $__removeUnitHeaders = false;
    /**
     * item[1438] = participant[id]
     */
    function getParticipantSelect()
    {
    	$this->options = array(array('value'=>'', 'text'=>'- '.$this->langIf->getText('Select Participant').' -'));

		// Get all producers
    	require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
		$cpp = new CRSOParticipantProduct($this->db);
		$q = $cpp->getCRSOParticipants($this->if_obj->model['crso_id'],'q',PARTICIPANT_PRODUCER,$_SESSION['selected_product_id']);
    	while($r = $q->fetch()) {
    		$selected = ($this->if_obj->model['participant_id'] == $r->PARTICIPANT_RELATION_ID)?'selected':'';
    		$this->options[] = array(
				'value'=>$r->PARTICIPANT_RELATION_ID,
				'text'=>$r->PARTICIPANT_NAME,
				'selected'=>$selected);
    	}
    	
    	preg_match('/\/scripts\/(.*)\/(.*)\/([\w]+)\.php/', $_SERVER['REQUEST_URI'],$matches);
    	$this->properties['onchange'] = sprintf("go('%s?year=%s&participant[id]='+this.value);", $matches[0], $this->if_obj->model['year']);
    }

    /**
     * item[1439] = year
     */
    function getYearSelect()
    {
    	//$date = currentDate();
		//$year = (int)$date->getYear();
		// TODO: management report doesn't work with the changes made for 2010, so only enable it for year<=2009
		$year = 2009;
    	
    	if($year == $this->if_obj->model['minyear'])
    		$this->_hideInput = true;
    	
    	$this->options = array();
    	
    	for($i=$this->if_obj->model['minyear']; $i<=$year; $i++)
			$this->options[] = array('value'=>$i,'text'=>$i,'selected'=>($i==$this->if_obj->model['year']?'selected':false));
		
		preg_match('/\/scripts\/(.*)\/(.*)\/([\w]+)\.php/', $_SERVER['REQUEST_URI'],$matches);
		$this->properties['onchange'] = sprintf("go('%s?year='+this.value+'&participant[id]=%s');", $matches[0], $this->if_obj->model['participant_id']);
    }

    /**
     * This will fill the management report tables with data by calling stored procedures on the database.
     * item[1440] = 
     */
    function getResults() {
    	
    	$this->text = sprintf('<div style="font-weight: bold;">%s</div><br />', $this->langIf->getText('EA WEEE Actuals breakdown'));

    	$dbh = IWRegistry::registry('db');
    	$stmt = $dbh->prepare("CALL sp_elcUKReportedToEA(?,?,?,?)");
    	$stmt->bind_values($this->if_obj->model['participant_id'], $this->if_obj->model['crso_id'], $this->if_obj->model['year'],$_SESSION['selected_product_id']);
    	$stmt->execute();
    	
    	$this->__totals = array();
    	$this->__removeUnitHeaders = true;
    	$t = new TableFactory($stmt);
    	$t->setThProcessor(TF_ALL, array($this, '_thProcessor'));
    	$t->setValueProcessor(TF_ALL, array($this, '_tfValueProcessor'));
    	$t->setCaption('Reported to the EA by Invoiced Period');
    	$t->setFooterProcessor(array($this,'tfFooter'));
    	$this->text .= $t->getXHTMLTable('').'<br />';

		
		$stmt = $dbh->prepare("CALL sp_elcUKReconcoliationVsEA(?,?,?,?)");
    	$stmt->bind_values($this->if_obj->model['participant_id'], $this->if_obj->model['crso_id'], $this->if_obj->model['year'],$_SESSION['selected_product_id']);
    	$stmt->execute();
    	
    	$this->__removeUnitHeaders = false;
    	$this->__totals = array();
    	$this->__totalsLabel = 'Total in Quarter';
    	$t = new TableFactory($stmt);
    	$t->setThProcessor(TF_ALL, array($this, '_thProcessor'));
    	$t->setValueProcessor(TF_ALL, array($this, '_tfValueProcessor'));
    	$t->setCaption('Reconciliation declarations made on WEEE BB by invoiced period vs EA WEEE Actuals reported');
    	$t->setRowProcessor(array($this, '_rowProcessor'));
    	$t->setFooterProcessor(array($this,'tfFooter'));
    	$this->text .= $t->getXHTMLTable('').'<br />';

    	if($this->if_obj->model['year']<=2009) {
			$stmt = $dbh->prepare("CALL sp_elcUKReconcoliationVsInvoiced(?,?,?,?)");
	    	$stmt->bind_values($this->if_obj->model['participant_id'], $this->if_obj->model['crso_id'], $this->if_obj->model['year'],$_SESSION['selected_product_id']);
	    	$stmt->execute();
	
			$this->__totalsLabel = 'Total Reconciliation';
	    	$t = new TableFactory($stmt);
	    	$t->decapitate();
	    	$t->setThProcessor(TF_ALL, array($this, '_thProcessor'));
	    	$t->setValueProcessor(TF_ALL, array($this, '_tfValueProcessor'));
	    	$t->setCaption('Reconciliation declarations made on WEEE BB by declaration period vs invoiced period');
	    	$t->setFooterProcessor(array($this,'tfFooter'));
	    	$this->text .= $t->getXHTMLTable(array('caption'=>'no-background')).'<br />';
    	}
    }
    
    function _rowProcessor($args) {
		
    }

	function _thProcessor($args) {
		
		if($this->__removeUnitHeaders) {
			$args['head'] = (substr($args['head'],-5)=='Units')? '': $args['head'];
		}
		
		if ($args['j'] == 0)
			return 'style="visibility: hidden;"';  
		else
			return 'style="text-align: right; width: 75px;"';
	}
	
	function _tfValueProcessor($args) {
		$v = $args['value'];

		if ($args['j'] != 0) {
			if($v==0)
				$v = null;
			$this->__totals[$args['head']] = isset($this->__totals[$args['head']])?$this->__totals[$args['head']]+$v:$v;
			$args['extra'] = 'style="text-align: right; width: 75px;" ';


			if(is_numeric($v) && ((int)$v==$v))
				$v = locale_nf($v);
			elseif(is_float($v)||is_numeric($v))
				$v = locale_nf($v, 3);
		} else {
			$v = $this->langIf->getText($v);
		}

		return $v;
	}
	
	function tfFooter()
	{
		$rv = array(''=>sprintf('<span style="font-weight: bold;">%s</span>', $this->langIf->getText($this->__totalsLabel)));
		for($i = 1; $i < 5; $i++) {
			$rv['Q'.$i.' Units']=isset($this->__totals['Q'.$i.' Units'])?'<div style="text-align: right; font-weight: bold;">'.locale_nf($this->__totals['Q'.$i.' Units'],0).'</div>':'';
			$rv['Q'.$i.' Weight']=isset($this->__totals['Q'.$i.' Weight'])?'<div style="text-align: right; font-weight: bold;">'.locale_nf($this->__totals['Q'.$i.' Weight'],3).'</div>':'';
		}
		$rv['Total Units']=isset($this->__totals['Total Units'])?'<div style="text-align: right; font-weight: bold;">'.locale_nf($this->__totals['Total Units'],0).'</div>':'';
		$rv['Total Weight']=isset($this->__totals['Total Weight'])?'<div style="text-align: right; font-weight: bold;">'.locale_nf($this->__totals['Total Weight'],3).'</div>':'';
		return $rv;
	}
}

/**
 * CALLBACK[getParticipantSelect]
 * item[1438] = {
 *     [handle] = Item: Participant Select
 *     [behaviour] = 
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 * 
 * CALLBACK[getYearSelect]
 * item[1439] = {
 *     [handle] = Item: Select Year
 *     [behaviour] = 
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 * 
 * CALLBACK[getResults]
 * item[1440] = {
 *     [handle] = Item: Yearly Results
 *     [behaviour] = 
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 * 
 */
?>