<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');

require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'AccountantCRSO.php');
require_once(ELC_CLASSES.'AccountantCRSOUser.php');
require_once(ELC_CLASSES.'RelationUser.php');
/**
 * Interface used to distribute clearance levels for CRSO or AccountCRSO objects.
 *
 * @package
 * @subpackage Interface
 * @since 18-06-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceUserClearanceLevels extends InterfaceFactory
{
    /**
     * interface[id] = 195
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceUserClearanceLevels(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        // Either a RelationUser (for CRSO) or an AccountantCRSOUser
        $this->model['target_object'] = null;
        // Either a CRSO object or an AccountantCRSO object.
        $this->model['source_object'] = null;
        // Array with Clearance Level strings (to be translated and to get the constant).
        $this->model['clearance_map'] = null;
        $this->target_info = '';

        $this->hiddenFields = array(array('name'=>'user_id','value'=>$_REQUEST['user_id']));

        switch($_SESSION['session_group_group_id']) {

        	case GROUP_CRSO:

        		$ru = new RelationUser($this->db_app);
        		$ru->set($_SESSION['session_user_id'], $_SESSION['crso_relation_id']);
        		if(!$ru->isComplete() || !$ru->hasClearance(CRSOClearanceUserManagement)) {
        			// This CRSO user has NO rights to be here!
        			throw new InterfacePermissionException('No permission to view this page.');
        		}
        		if($_REQUEST['user_id']==$_SESSION['session_user_id']) {
        			// Cant change your OWN clearance levels.
        			throw new InterfacePermissionException('No permission to view this page.');
        		}

        		$this->model['target_object'] = new RelationUser($this->db_app);
        		$this->model['target_object']->set($_REQUEST['user_id'], $_SESSION['crso_relation_id']);
        		if(!$this->model['target_object']->isComplete()) {
        			// This crso user relation does not exist!
        			throw new InterfacePermissionException('Incorrect CRSO User relation supplied');
        		}

        		// Check if the target user is the admin user, if so, this interface shouldn't be available.
        		if($this->model['target_object']->isAdmin()) {
        			throw new InterfacePermissionException('Admin users do not have clearance levels');
        		}
			$this->model['target_object']->user->set();

        		$this->model['source_object'] = new CRSO($this->db_app);
        		$this->model['source_object']->set($_SESSION['crso_relation_id']);

        		$this->model['clearance_map'] = CRSO::getClearanceLevels();
        		$this->model['header_location'] = '/scripts/elc/crso/InterfaceCRSOUserOverview.php';

        		$this->target_info = sprintf('CRSO Relation [%s]', $_SESSION['crso_relation_id']);

			setPageTitlePostfix(sprintf(' (%s)', $this->model['target_object']->user->name));

        	break;
        	case GROUP_ACCOUNTANT:

        		$this->model['source_object'] = new AccountantCRSO($this->db_app);
        		// TODO: This should already be filtered on InterfacePermission
        		$this->model['source_object']->setByAccountantAndCRSO($_SESSION['accountant_relation_id'], $_REQUEST['crso_id']);
        		if(!$this->model['source_object']->isComplete()) {
        			// This accountant crso relation does not exist!
        			throw new InterfacePermissionException('Incorrect Accountant CRSO relation supplied');
        		}

        		$acu = new AccountantCRSOUser($this->db_app);
        		$acu->set($this->model['source_object']->id, $_SESSION['session_user_id']);
        		if(!$acu->isComplete() || !$acu->has(AccountantClearanceUserManagement)) {
				// This Accountant CRSO user has NO rights to be here!
				throw new InterfacePermissionException('No permission to view this page.');
			}

			$this->model['target_object'] = new AccountantCRSOUser($this->db_app);
			$this->model['target_object']->set($this->model['source_object']->id, $_REQUEST['user_id']);
        		if(!$this->model['target_object']->isComplete()) {
        			// This accountant crso user relation does not exist!
        			throw new InterfacePermissionException('Incorrect Accountant CRSO User relation supplied');
        		}

        		$this->model['clearance_map'] = Accountant::getClearanceLevels();
        		$this->model['header_location'] = '/scripts/elc/accountant/InterfaceUserOverview.php';

        		$this->target_info = sprintf('Accountant CRSO [%s]', $this->model['target_object']->accountant_crso->id);

        		$this->hiddenFields[] = array('name'=>'crso_id', 'value' => $_REQUEST['crso_id']);

			setPageTitlePostfix(sprintf(' (%s)', $this->model['target_object']->user->name));

        	break;
        	case GROUP_SYSADMIN:
				// Check if accountant_id isset
				if(isset($_REQUEST['accountant_id'])) {
					// We are dealing with an accountant crso user.
					$this->model['source_object'] = new AccountantCRSO($this->db_app);
				$this->model['source_object']->setByAccountantAndCRSO($_REQUEST['accountant_id'], $_REQUEST['crso_id']);
				if(!$this->model['source_object']->isComplete()) {
					// This accountant crso relation does not exist!
					throw new InterfacePermissionException('Incorrect Accountant CRSO relation supplied');
				}
				$user = new ELCUser($this->db_app, $_REQUEST['user_id'], true);
				$this->model['target_object'] = new AccountantCRSOUser($this->db_app);
				$this->model['target_object']->accountant_crso = $this->model['source_object'];
				$this->model['target_object']->user = $user;
				$this->model['target_object']->set();

					$this->model['clearance_map'] = Accountant::getClearanceLevels();
					$this->model['header_location'] = sprintf('%s/scripts/elc/sysadmin/InterfaceAccountantUserOverview.php?accountant[id]=%s',
						SITE_PATH,
						$_REQUEST['accountant_id']
					);

					$this->hiddenFields[] = array(
						'name'=>'accountant_id', 'value' => $_REQUEST['accountant_id']
					);
					$this->hiddenFields[] = array(
						'name'=>'crso_id', 'value' => $_REQUEST['crso_id']
					);
					setPageTitlePostfix(sprintf(' (%s)', $user->name));
					break;
				}

				// We are dealing with a CRSO user.
        		$this->model['target_object'] = new RelationUser($this->db_app);
        		$this->model['target_object']->set($_REQUEST['user_id'], $_REQUEST['crso_id']);
        		if(!$this->model['target_object']->isComplete()) {
        			// This crso user relation does not exist!
        			throw new InterfacePermissionException('Incorrect CRSO User relation supplied');
        		}
        		// Check if the target user is the admin user, if so, this interface shouldn't be available.
        		if($this->model['target_object']->isAdmin()) {
        			throw new InterfacePermissionException('Admin users do not have clearance levels');
        		}

        		$this->model['source_object'] = new CRSO($this->db_app);
        		$this->model['source_object']->set($_REQUEST['crso_id']);

        		$this->model['clearance_map'] = CRSO::getClearanceLevels();
        		$this->model['header_location'] = sprintf('/scripts/elc/operator/InterfaceCRSOUserOverview.php?crso[id]=%s',
        			$_REQUEST['crso_id']);

        		$this->target_info = sprintf('CRSO Relation [%s]', $_REQUEST['crso_id']);
        		$this->hiddenFields[] = array('name'=>'crso_id', 'value' => $_REQUEST['crso_id']);

        	break;
        	default:
        		throw new InterfacePermissionException('No permission to view this page.');
        }
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        if(isset($_REQUEST['process']) && $_REQUEST['process'] == 'save') {
		// Default to zero;
		$new_clearance = 0;
		if(isset($_REQUEST['clearance'])) {
			foreach($_REQUEST['clearance'] as $c) {
				$c = (int) $c;
					// Make sure the provided clearance levels are part of the source.
				if(!(($this->model['source_object']->available_clearance_levels & $c) == $c)) {
					info('Trying to set an unavailable clearance level.');
					throw new InterfacePermissionException('Trying to set an unavailable clearance level.');
				}
			}

			$new_clearance = array_sum($_REQUEST['clearance']);
        	}

        	info(sprintf('Changing clearance levels for user [%s] %s from %s to %s',
        		$this->model['target_object']->user->id,
        		$this->target_info,
        		$this->model['target_object']->clearance_level,
        		$new_clearance
        	));

        	$this->model['target_object']->clearance_level = $new_clearance;
        	$this->model['target_object']->update();
        	header(sprintf('Location: %s', $this->model['header_location']));
        	exit();
        }
    }
}
/**
 * @package
 * @subpackage Interface
 * @since 18-06-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceUserClearanceLevelsItem extends InterfaceItem
{

    /**
     * item[1543] =
     */
    function getClearanceLevels()
    {
    	// Get all available clearance levels. Make checkboxes for those.
    	$clearance_map = $this->if_obj->model['clearance_map'];
    	$target = $this->if_obj->model['target_object'];
    	$source = $this->if_obj->model['source_object'];

    	$ds = array();

    	// Loop the clearance map.
    	foreach($clearance_map as $clearance) {
	   		$c = constant($clearance);

	   		// Only show the clearance levels that are available based on the source.
	   		if(!(($source->available_clearance_levels & $c) == $c)) {
	   			continue;
	   		}
	   		// If the target object has clearance level set, mark as checked.
	   		$checked = (($target->clearance_level & $c) == $c) ? ' checked="checked"' : '';

       		// Make a checkbox input.
       		$ds[] = array('Clearance level'=>
       			sprintf('<input type="checkbox" name="clearance[]" value="%s" %s /> %s',
       				$c,
       				$checked,
       				$this->langIf->getText($clearance)
       			),
       			'Additional Information'=>nl2br($this->langIf->getText('Information for '.$clearance)));
    	}

    	// Create a tablefactory object to show the inputs neatly.
    	$tf = new TableFactory($ds, $this->langIf);
    	$tf->decapitate();
    	$this->text = nl2br($this->langIf->getText('Clearance Level introduction text')).'<br />'.$tf->getXHTMLTable('Professional', 2);
    }

    /**
     * item[1544] =
     */
    function getButtons()
    {
    	$this->_BarButtons = BUTTON_SAVE + BUTTON_CANCEL;

    	$this->_ButtonOptions[BUTTON_CANCEL] = array('onclick'=>"document.location.href='".$this->if_obj->model['header_location']."'");
    }

}

/**
 * CALLBACK[getClearanceLevels]
 * item[1543] = {
 *     [handle] = Item: Clearance Levels
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[1544] = {
 *     [handle] = Item: Button Bar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
