<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'CRSOVATBlock.php');

/**
 *
 * @package
 * @subpackage Interface
 * @since 04-09-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceVATBlock extends InterfaceFactory
{
    /**
     * interface[id] = 146
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceVATBlock(&$db, &$db_app, &$langIf)
    {
        //for CRSO only users with CRSOClearanceBlockVat can access this interface
    	if($_SESSION['session_group_group_id'] == GROUP_CRSO)
        {
        	$ru = new RelationUser($db_app);
        	if(!$ru->hasClearance(CRSOClearanceBlockVat, $_SESSION['session_user_id'],$_SESSION['crso_relation_id']))
        		throw new InterfacePermissionException('['.$_SESSION['session_user_id'].'] tried to access crso admin interface without correct clearance: CRSOClearanceBlockVat');
        }

    	parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        $this->ajaxUsage = isset($_REQUEST['ajaxUsage']) && (int)$_REQUEST['ajaxUsage'] ? true : false;

		require_once(IWARE.'group.php');
        switch ($_SESSION['session_group_group_id'])
		{
			case GROUP_CRSO:
				$this->model['crso'] = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, $_SESSION['crso_relation_id'])));
			break;

		    case GROUP_SYSADMIN:
				$this->model['crso'] = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, $_REQUEST['crso']['id'])));
				$this->hiddenFields[] = array('name'=>'crso[id]', 'value'=>$_REQUEST['crso']['id']);
		    break;

		    default:
				info('User ['.$_SESSION['session_user_id'].'] has no privileges for Interface');
		    	throw new InterfacePermissionException('User ['.$_SESSION['session_user_id'].'] has no privileges for Interface (VATBlock)');
		}

    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        if(isset($_REQUEST['process']))
        {
    		$v = new CRSOVATBlock($this->db_app);
    		$v->vat_no = str_replace(' ', '', $_REQUEST['vat_block']['vat_no']);
    		$v->crso = $this->model['crso'];

        	if($_REQUEST['process']=='save' || $_REQUEST['process'] == 'ajaxusersave')
        	{
	 			if(!$this->fb->validate())
	        		return $thrownException .= $this->langIf->getText('Please correct the form errors');

        		$v->types_blocked = isset($_REQUEST['vat_block']['types_blocked']) ? array_sum($_REQUEST['vat_block']['types_blocked']) : 0;
        		$v->insert();
        	}

        	if ($_REQUEST['process'] == 'delete')
        		$v->delete();

	        $this->init($this->id);
        }
    }
}
/**
 * @package
 * @subpackage Interface
 * @since 04-09-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceVATBlockItem extends InterfaceItem
{

    /**
     * item[1088] =
     */
    function getOverview()
    {
    	$v = new CRSOVATBlock($this->db_app);
    	$ds = $v->selectOverview($this->if_obj->model['crso']->relation->id);

    	$tf = new TableFactory($ds, $this->langIf);
    	$tf->setEmptyText($this->langIf->getText('No VAT Blocks found'));
    	$tf->setValueProcessor(array('REMOVE_BLOCK', 'TYPES_BLOCKED'), array($this, '_tfValueProcessor'));
    	$this->text = $tf->getXHTMLTable('Professional');
    }

    function _tfValueProcessor($args)
    {
    	switch ($args['head'])
    	{
    		case 'TYPES_BLOCKED':
    			$html = '';
    			foreach (CRSOVATBlock::getTypes() as $value=>$caption)
    			{
    				if ($value & (int) $args['value'])
    					$html .= $this->langIf->getText($caption) .'<br/>';
    			}
    			return $html;
    		break;

    		case 'REMOVE_BLOCK':
    			$args['row']['VAT_NO'] = str_replace("'","\\'",$args['row']['VAT_NO']);
           		$args['row']['VAT_NO'] = str_replace('"','\\"',$args['row']['VAT_NO']);
           		$args['row']['VAT_NO'] = str_replace("&#39;","\\&#39;",$args['row']['VAT_NO']);
           		$args['row']['VAT_NO'] = str_replace('&#34;','\\&#34;',$args['row']['VAT_NO']);
           		if ($this->if_obj->ajaxUsage)
	    			$cb = "function()
					{
						callParams = [];
						callParams['targetId'] = 'ifI_841';
						ifHandler.loadInterface(".$this->if_obj->id.", callParams,
							[['process', 'delete'],
							['vat_block[vat_no]', '".jsescape($args['row']['VAT_NO'], "'")."'],
							['crso[id]', '".$this->if_obj->model['crso']->relation->id."'],
							['ajaxUsage', '1']]);
	    			}";

	    		else
	    			$cb = 'function() { go(\'InterfaceVATBlock.php?process=delete&vat_block[vat_no]='.$args['row']['VAT_NO'].'\'); }';

	    		$onclick = jsMessageBox(MSG_BOX_QUEST, '',
	    			$this->langIf->getText('Are you sure you want to remove this vat block?'),
					$cb);
	    		return sprintf('<a href="#", onclick="%s return false;">%s</a>', $onclick, $this->langIf->getText('Remove'));
    		break;
    	}
    }

    /**
     * item[1089] = vatblock[vat_no]
     */
    function getVATNo()
    {
    	if (!isset($_REQUEST['process']) && isset($_REQUEST['vat_block']['vat_no']))
    		$this->properties['value'] = $_REQUEST['vat_block']['vat_no'];

    }

    /**
     * item[1090] =
     */
    function getVatBlockTypes()
    {
    	$this->text = '';
    	foreach (CRSOVATBlock::getTypes() as $value=>$caption)
    	{
    		$chk = new Form_Checkbox('vat_block[types_blocked][]', $value);
    		$chk->setLabel($this->langIf->getText($caption));

    		// preset from request var
    		if (!isset($_REQUEST['process']) && isset($_REQUEST['vat_block']['types_blocked']) && ($value & (int) $_REQUEST['vat_block']['types_blocked']))
    			$chk->setChecked(true);

    		$this->text .= $chk->getHtml() .'<br />';
    	}
    }

    /**
     * item[1091] =
     */
    function getVatBlockAdd()
    {
    	$this->properties['value'] = $this->langIf->getText('Add VAT Block');
    	$this->properties['type'] = 'button';

    	// Only submit if any of the checkboxes are checked.

    	$this->properties['onclick'] = "validateCheckboxes(); return false;";
    	$this->script = sprintf("
			function validateCheckboxes()
			{
				var el = document.getElementsByName('vat_block[types_blocked][]');
				var cont = false;
				for(var i=0;i<el.length;i++) {
					if(el[i].checked)
						cont = true;
				}

				if(!cont)
					MessageBox.prototype.Information('%s','%s');
				else
				{
					var nForm = \$('interface_form');
					if(fb.validate(nForm))
						nForm.submit();
				}
			}",
			$this->langIf->getText('Form Errors'),
			$this->langIf->getText('Please select at least one Block Type')
			);


    	if ($this->if_obj->ajaxUsage)
    	{
    		$this->properties['onclick'] = "
				// \$('ajaxuserstatus').innerHTML = '".$this->langIf->getText('saving form details...')."';
				\$('interface_form_process').value = 'ajaxusersave';
				var params = [];
				params['ifId'] = ".$this->if_obj->id.";
				params['targetId'] = 'ifI_841';
				jsbroker.callAndSubmitForm('interface_form', 'loadInterface', params, ifHandler.buildInterface);
				return false;
			";
    	}
    }

}

/**
 * CALLBACK[getOverview]
 * item[1088] = {
 *     [handle] = Item: VATBlockOverview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getVATNo]
 * item[1089] = {
 *     [handle] = Item: VATNo
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 1
 *
 * CALLBACK[getVatBlockTypes]
 * item[1090] = {
 *     [handle] = Item: VatBlockTypes
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 1
 *
 */
?>
