<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'ProductCategory.php');
require_once(ELC_CLASSES.'ProductFee.php');
/**
 *
 * @package
 * @subpackage Interface
 * @since 31-05-2007
 * @author Marten Koedam <marten@dalines.net>
 * @copyright 2007 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceProductCategoryFee extends InterfaceFactory
{
    /**
     * interface[id] = 171
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceProductCategoryFee(&$db, &$db_app, &$langIf)
    {
    	//for CRSO only users with CRSOClearanceMaintenance can access this interface
    	if($_SESSION['session_group_group_id'] == GROUP_CRSO)
        {
        	$ru = new RelationUser($db_app);
        	if(!$ru->hasClearance(CRSOClearanceMaintenance, $_SESSION['session_user_id'],$_SESSION['crso_relation_id']))
        		throw new InterfacePermissionException('['.$_SESSION['session_user_id'].'] tried to access crso admin interface without correct clearance: CRSOClearanceMaintenance');
        }

    	parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        // TODO: Extend for operator usage?
        switch($_SESSION['session_group_group_id'])
        {
        	case GROUP_CRSO:
        		$this->model['crso'] = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app,$_SESSION['crso_relation_id'])));

        	break;
        	default:
        		throw new InterfacePermissionException('Unauthorized Access to Product Category Fee');
        }

        $this->model['pc'] = new ProductCategory($this->db_app);
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        if(!isset($_POST['process']))
        	return;

        if($_POST['process']=='save')
        {
        	//check the product id
			$pc = new ProductCategory($this->db_app, (int)$_POST['product_category']['id']);
			if(!$pc->set((int)$_POST['product_category']['id']))
        		throw new IWException('ProductCategory does not exist');
			if(!$pc->is_for_crso($_SESSION['crso_relation_id']))
				throw new IWUserException('Permission denied');

        	// Check the date.
        	if(!$this->fb->validate())
				return $thrownException .= $this->langIf->getText('Please correct the form errors');

			$date = currentDate();

			$month = (int)$_POST['start_month'];
			$year = (int)$_POST['start_year'];


			if(ProductCategoryFee::hasFeeAlready($this->db_app, $_POST['product_category']['id'], $_SESSION['selected_product_id'], $this->model['crso']->relation->id, $year.'-'.$month.'-01'))
				return $thrownException .= $this->langIf->getText('There is already a fee set for that period.');

			$q = $this->db_app->query("SELECT IFNULL(f_elcProductFee(?, ?, ?, NOW()),'-') AS FEE");
			$q->bind_values($_SESSION['selected_product_id'],$this->model['crso']->relation->id,$_POST['product_category']['id']);
			$q->execute();
			$r = $q->fetch();
			if(is_numeric($r->FEE)&&(($year==$date->getYear())&&($month<$date->getMonth())))
				return $thrownException .= $this->langIf->getText('Incorrect Start Date supplied, must be in the future.');
			elseif($r->FEE == '-')
				$start_date = '2005-01-01';
			else
				$start_date = $year.'-'.$month.'-01';

			$pcf = new ProductCategoryFee($this->db_app);
			$pcf->product_category = new ProductCategory($this->db_app,$_POST['product_category']['id']);
			$pcf->crso = $this->model['crso'];
			$pcf->start_date = $start_date;
			$pcf->fee = $_POST['product_category']['fee'];
			$pcf->insert();
			$pcf->product_category->set($_POST['product_category']['id']);

			//send notification to CRSO's accountants
			require_once(ELC_CLASSES.'AccountantCRSO.php');
			$ac = new AccountantCRSO($this->db_app);
			$q = $ac->getCRSOAccountants($this->model['crso']->relation->id, $_SESSION['selected_product_id']);

			$recs = array();
			while($r = $q->fetch())
				$recs[] = array('recipient'=> new Relation($this->db_app, $r->ACCOUNTANT_RELATION_ID));

			require_once(ELC_CLASSES.'Notification.php');
			$body = $this->langIf->getText('Product category:') . $pcf->product_category->name.
					"<br />" . $this->langIf->getText('Start date: ') . $pcf->start_date.
			        "<br />" . $this->langIf->getText('New fee:') . $_POST['product_category']['fee'];
			$message = array('subject'=> $this->langIf->getText('Product Category Fee changed'),
					'user'=> new ELCUser($this->db_app, $this->model['crso']->relation->id),
					'message'=> $body,
					'product'=> new Product($this->db_app, $_SESSION['selected_product_id'])
				);
			Notification::send(
				$this->db_app,
				$message,
				$recs
				);
			header('Location: InterfaceProductCategoryFee.php');
			exit();
        }
    }
}
/**
 * @package
 * @subpackage Interface
 * @since 31-05-2007
 * @copyright 2007 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceProductCategoryFeeItem extends InterfaceItem
{

    /**
     * item[1328] = product_category[id]
     */
    function getCategorySelect()
    {
    	$this->options = array(array('text'=>'- '.$this->langIf->getText('Category Select').' -','value'=>''));
    	$q = $this->if_obj->model['pc']->selectOverview($_SESSION['selected_product_id'],$this->if_obj->model['crso']->relation->id);
    	$selectedCat = isset($_POST['product_category']['id'])?$_POST['product_category']['id']:'';
    	while($r = $q->fetch())
    		$this->options[] = array('value'=>$r->PRODUCT_CATEGORY_ID,'text'=>$r->CATEGORY_NAME,'selected'=>(($r->PRODUCT_CATEGORY_ID==$selectedCat)?'selected':false));
    }

    /**
     * item[1329] = start_month
     */
    function getStartMonth()
    {
    	$this->options = array();
		$date = currentDate();
		$selectedMonth = isset($_POST['start_month'])?$_POST['start_month']:$date->getMonth();
		for($i = 1; $i < 13; $i++)
			$this->options[] = array('value'=>$i,'text'=>strftime('%B', mktime(1, 1, 1, $i, 1) ),'selected'=>(($i==$selectedMonth)?'selected':false));
    }

    /**
     * item[1330] = start_year
     */
    function getStartYear()
    {
    	$this->options = array();
    	$date = currentDate();
    	$startyear = 2006; // not back in time
		$selectedYear = isset($_POST['start_year'])?$_POST['start_year']:$date->getYear();
		for($i = $startyear; $i <= ($date->getYear()+1); $i++)
			$this->options[] = array('value'=>$i,'text'=>$i,'selected'=>(($i==$selectedYear)?'selected':false));
    }

    /**
     * item[1331] =
     */
    function getCurrentFees()
    {
    	$ds = $this->if_obj->model['pc']->selectOverview($_SESSION['selected_product_id'],$this->if_obj->model['crso']->relation->id);
    	$tf = new TableFactory($ds,$this->langIf);
    	$tf->id = 'ProductCatCurrentFee';
    	$tf->setValueProcessor(TF_ALL,array($this,'tfValueProcessor'));
		$tf->hideColumns(array('PRODUCT_CATEGORY_ID', 'DISABLE_CATEGORY'));
    	$this->text = $tf->getXHTMLTable('Professional',2);
    }

    function tfValueProcessor($args)
    {
    	switch($args['head'])
    	{
    		case 'CURRENT_FEE':
    			$d = currentDate();
    			$sql = "SELECT f_elcProductFee(?,?,?,?) as FEE";
    			$q = $this->db_app->query($sql);
    			$q->bind_values($_SESSION['selected_product_id'],
    				$this->if_obj->model['crso']->relation->id,
    				$args['row']['PRODUCT_CATEGORY_ID'],
    				$d->getYear().'-'.$d->getMonth().'-01 00:00:00');
    			$q->execute();
    			$r = $q->fetch();
    			return $r->FEE?'<span class="error">'.$r->FEE.'</span>':$this->langIf->getText('No Fee Available');
    		break;
    	}
    	return $args['value'];
    }

	/**
     * item[1332] =
     */
    function getCategoryFeeOverview()
    {
    	$pcf = new ProductCategoryFee($this->db_app,array('crso'=>$this->if_obj->model['crso']));
    	$ds = $pcf->getFeeOverview($_SESSION['selected_product_id']);

    	$tf = new TableFactory($ds,$this->langIf);
    	$tf->id = 'ProductCatFeeOverview';
    	$tf->setValueProcessor(TF_ALL,array($this,'tfValProc'));
    	$tf->hideColumns(array('Month','Year'));
    	$this->text = $tf->getXHTMLTable('Professional',2);
    }

	function tfValProc($args)
	{
		switch($args['head'])
		{
			case 'StartDate':

				return strftime('%B', mktime(1, 1, 1, $args['row']['Month'], 1)).' '.$args['row']['Year'];
			break;
		}
		return $args['value'];
	}

	function getButtons()
	{
		$this->_BarButtons = BUTTON_SAVE;
	}

	function getMessage() {

    	// We check if the country product setting has Weight as Amount Calculation.
    	$id = $this->if_obj->model['crso']->getCountry($this->if_obj->model['crso']->relation->id);
    	$c = new WEEECountry($this->db_app, array('id'=>$id), true);
    	if($c->has(HasWeightAsBasisForAmount, $_SESSION['selected_product_id'])) {
    		$this->text = $this->langIf->getText('Note that amounts are based on fee times weight for this product.');
    	}

	}
}

/**
 * CALLBACK[getCategorySelect]
 * item[1328] = {
 *     [handle] = Item: ProductCategory
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getStartMonth]
 * item[1329] = {
 *     [handle] = Item: StartMonth
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getStartYear]
 * item[1330] = {
 *     [handle] = Item: StartYear
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getCurrentFees]
 * item[1331] = {
 *     [handle] = Item: CurrentCategoryFee
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getCategoryFeeOverview]
 * item[1332] = {
 *     [handle] = Item: CategoryFeeOverview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
