<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'Product.php');
require_once(ELC_CLASSES.'ProductFee.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @since 14-02-2006
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceProductFeeRegister extends InterfaceFactory
{
	/**
	 * @var boolean Whether this interface is being used in an ajax context
	 */
	var $ajaxUsage = false;
	/**
	 * interface[id] = 68
	 * @param $db Database (I_WARE_SETTINGS)
	 * @param $db_app Database (APPLICATION DATABASE)
	 * @param $langIf LanguageInterface
	 */
	function InterfaceProductFeeRegister($db,$db_app,$langIf)
	{
		//for CRSO only admin users can access this interface
    	if($_SESSION['session_group_group_id'] == GROUP_CRSO)
        {
        	$ru = new RelationUser($db_app);
        	$ru->set($_SESSION['session_user_id'],$_SESSION['crso_relation_id']);
        	if(!$ru->hasClearance(CRSOClearanceMaintenance))
        		throw new InterfacePermissionException('['.$_SESSION['session_user_id'].'] tried to access crso admin interface without correct clearance CRSOClearanceMaintenance');
        }

		$this->ajaxUsage = isset($_REQUEST['ajaxUsage']) && (int)$_REQUEST['ajaxUsage'] ? true : false;

		parent::InterfaceFactory($db, $db_app,$langIf);
		$this->FirstColWidth = 25;
		$this->delimiter = '<br />';

		$this->model['crso_id'] = isset($_SESSION['crso_relation_id']) ? $_SESSION['crso_relation_id'] :
				(isset($_REQUEST['crso']['id']) ? $_REQUEST['crso']['id']:
						0);
	}

	function init($id)
	{
		$this->id = $id;
		parent::init($id);


		if ($this->ajaxUsage && isset($this->blocks[BLOCK_FEE_REGISTER]))
			unset($this->blocks[BLOCK_FEE_REGISTER]);
		if ($this->ajaxUsage && isset($this->blocks[BLOCK_FEE_REGISTER_BUTTONS]))
			unset($this->blocks[BLOCK_FEE_REGISTER_BUTTONS]);
	}

	/**
	 * @param string $thrownException
	 * @todo Multiple Product support!
	 */
	function processInput(&$thrownException)
	{
		if(isset($_POST['process']))
		{
			if($_POST['process'] == 'save')
			{
				if(!$this->fb->validate())
					return $thrownException .= $this->langIf->getText('Please correct the form errors');
				//$pieces = explode('-',$_POST['product_fee']['start_date']);
				$date = currentDate();

				$month = (int)$_POST['product_fee']['start_month'];
				$year = (int)$_POST['product_fee']['start_year'];

				if(ProductFee::hasFeeAlready($this->db_app, $_SESSION['selected_product_id'], $this->model['crso_id'], $year.'-'.$month.'-01'))
					return $thrownException .= $this->langIf->getText('There is already a fee set for that period.');
				// Check if there is NO current Fee, if there is, apply check for future dates only, else it can be in the past.
				// See ticket #1185

				$q = $this->db_app->query("SELECT IFNULL(f_elcProductFee(?, ?, NULL, NOW()),'-') AS FEE");
				$q->bind_values($_SESSION['selected_product_id'],$this->model['crso_id']);
				$q->execute();
				$r = $q->fetch();
				if(is_numeric($r->FEE)&&(($year==$date->getYear())&&($month<$date->getMonth())))
					return $thrownException .= $this->langIf->getText('Incorrect Start Date supplied, must be in the future.');
				elseif($r->FEE == '-')
					$_POST['product_fee']['start_date'] = '2005-01-01';
				else
					$_POST['product_fee']['start_date'] = $year.'-'.$month.'-01';
				$pf = new ProductFee($this->db_app,$_POST['product_fee']);
				$pf->crso = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app,$this->model['crso_id'])));
				$pf->product = new Product($this->db_app,array('id'=>$_SESSION['selected_product_id']));
				$pf->insert();

				//send notification to CRSO's accountants
				require_once(ELC_CLASSES.'AccountantCRSO.php');
				$ac = new AccountantCRSO($this->db_app);
				$q = $ac->getCRSOAccountants($this->model['crso_id'], $_SESSION['selected_product_id']);

				$recs = array();
				while($r = $q->fetch())
					$recs[] = array('recipient'=> new Relation($this->db_app, $r->ACCOUNTANT_RELATION_ID));

				require_once(ELC_CLASSES.'Notification.php');
				$body = $this->langIf->getText('Start date:').$_POST['product_fee']['start_date'].
				        "<br />".$this->langIf->getText('New fee:').$_POST['product_fee']['fee'];
				$message = array('subject'=> $this->langIf->getText('Product Fee changed'),
						'user'=> new ELCUser($this->db_app, $this->model['crso_id']),
						'message'=> $body,
						'product'=> new Product($this->db_app, $_SESSION['selected_product_id'])
					);
				Notification::send(
					$this->db_app,
					$message,
					$recs
					);
			}
			$this->init($this->id);
		}
	}
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 14-02-2006
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceProductFeeRegisterItem extends InterfaceItem
{

	function getCurrentFee()
	{
		$p = new ProductFee($this->db_app);
		$this->twoCols = true;
		$this->text = '<span class="warning">'.$this->langIf->getText('Current fee').': &nbsp;&nbsp;'. $p->getFee($_SESSION['selected_product_id'], $this->if_obj->model['crso_id']) . '</span>';
	}

	/**
	 * item[648] =
	 */
	function getItemValue()
	{
		if(isset($_POST['product_fee']['fee']))
			$this->properties['value'] = $_POST['product_fee']['fee'];
	}
	/**
	 * Hide the item input if it is the first time for this CRSO.
	 */
	function getDateInput()
	{
		$pf = new ProductFee($this->db_app);
		$pf->crso = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app,$this->if_obj->model['crso_id'])));
		$this->options = array();
		/**if(!$pf->getFee($_SESSION['selected_product_id']))
		{
			$this->properties['value'] = '01-01-2005';
			$this->_hideInput = true;
		}**/
		$date = currentDate();
		if($this->name=='product_fee[start_month]')
		{
			$selectedMonth = isset($_POST['product_fee']['start_month'])?$_POST['product_fee']['start_month']:$date->getMonth();
			for($i = 1; $i < 13; $i++) // set fee for period. so if January is selected, start date is februari
				$this->options[] = array('value'=>$i,'text'=>strftime('%B', mktime(1, 1, 1, $i, 1) ),'selected'=>(($i==$selectedMonth)?'selected':false));
			return;
		}
		$startyear = 2006; // not back in time
		$selectedYear = isset($_POST['product_fee']['start_year'])?$_POST['product_fee']['start_year']:$date->getYear();
		for($i = $startyear; $i < ($date->getYear() + 2); $i++)
			$this->options[] = array('value'=>$i,'text'=>$i,'selected'=>(($i==$selectedYear)?'selected':false));
	}

	function getFeeOverview()
	{
		$pf = new ProductFee($this->db_app);
		//FIXME: product id
		$ds = $pf->getAllFees($_SESSION['selected_product_id'], $this->if_obj->model['crso_id']);

		require_once(IWARE.'table_factory.php');
		$tf = new TableFactory($ds, $this->langIf);
		$tf->setValueProcessor(array('FEE'),array($this,'tfFee'));
		$tf->setValueProcessor(array('START_DATE'),array($this,'tfStartDate'));
		$tf->setSortingFunctions(array('START_DATE' => "'SortVal'",
										'FEE' => "'SortVal'"));


		$this->text = $tf->getXHTMLTable('Professional',2);
	}

	function tfFee($args)
	{
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
    	return locale_nf($args['value'], 8);
	}

    function get2colTxt()
    {
		$this->twoCols = true;
    }

    function getMessage() {

    	$this->twoCols = true;

    	// We check if the country product setting has Weight as Amount Calculation.
    	$r = new Relation($this->db_app);
    	$id = $r->getCountry($this->if_obj->model['crso_id']);
    	$c = new WEEECountry($this->db_app, array('id'=>$id), true);
    	if($c->has(HasWeightAsBasisForAmount, $_SESSION['selected_product_id'])) {
    		$this->text = $this->langIf->getText('Note that amounts are based on fee times weight for this product.');
    	}

    }

	function tfStartDate($args)
	{
		$tmp = substr($args['value'], 0, 10); //get the date only... for some reason this is a DateTime
		$args['extra'] .= sprintf(' sort_val="%s" ', str_replace('-', '', $tmp));
		$parts = explode('-', $tmp);
		return strftime('%x', mktime(1, 1, 1, $parts[1], $parts[2], $parts[0]) );
	}

	/**
	 * item[649] =
	 */
	function getButtons()
	{
		$this->_BarButtons = BUTTON_SAVE;
	}

}

define('BLOCK_FEE_REGISTER', 176);
define('BLOCK_FEE_REGISTER_BUTTONS', 177);

?>
