<?php
require_once(INTERFACE_OBJ."InterfaceFactory.php");

require_once(ELC_CLASSES.'Relation.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'RelationUser.php');

require_once(ELC_CLASSES.'Country.php');
require_once(ELC_CLASSES.'ProvisionalDeclaration.php');
require_once(ELC_CLASSES.'Brand.php');
require_once(ELC_CLASSES.'ProductCategory.php');

require_once(ELC_CLASSES.'Notification.php');

require_once(IWARE.'encryption.php');
/**
 * InterfaceRegistrantDetails
 *
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright (2006) BaseSoft B.V.
 * @version $Id$
 */
class InterfaceRegistrantDetails extends InterfaceFactory
{
	function InterfaceRegistrantDetails($db, $db_app, $langIf)
	{
		// Is the participant selected part of the current CRSO?
		// Make sure they dont chnage IDs in the request string.
		$this->cp = new CRSOParticipantProduct($db_app);
		if(!$this->cp->set($_SESSION['crso_relation_id'],(int)$_REQUEST['participant']['id'],$_SESSION['selected_product_id']))
			throw new InterfacePermissionException('Incorrect combination for CRSO-Participant-Product');

		$this->cp->participant->relation->set();
		setPageTitlePostfix('('.$this->cp->participant->relation->name.')');

		$this->FirstColWidth = 25;
		$this->delimiter = '<br />';
		parent::InterfaceFactory($db, $db_app, $langIf);
	}

	function getInterfaceHeader()
	{
		$head = parent::getInterfaceHeader();
		return $head .
		"<script>

			function statusChange(el)
			{
				try
				{
					if (el.checked)
					{
						switch (el.value)
						{
							case '".PARTICIPANT_STATUS_ACTIVE."':
								$('status_confirmed').checked = el.checked;

							case '".PARTICIPANT_STATUS_CONFIRMED."':
								$('status_authorized').checked = el.checked;

							case '".PARTICIPANT_STATUS_AUTHORIZED."':
								$('status_verified').checked = el.checked;
						}
					}
					else
					{
						switch (el.value)
						{
							case '".PARTICIPANT_STATUS_VERIFIED."':
								$('status_authorized').checked = el.checked;

							case '".PARTICIPANT_STATUS_AUTHORIZED."':
								$('status_confirmed').checked = el.checked;

							case '".PARTICIPANT_STATUS_CONFIRMED."':
								$('status_active').checked = el.checked;
						}

					}
				}
				catch (e)
				{
					// ignore errors, it could be that the participant is confirmed
					// and then there are no checkboxes
				}

			}
		</script>";
	}

	/**
	 * @param &$thrownException
	 */
	function processInput(&$thrownException)
	{
		if(isset($_POST['process']))
		{
			if($_POST['process'] == 'save')
			{
				$this->db_app->begin();
				$status = PARTICIPANT_STATUS_NEW;

				if(isset($_POST['crso_participant']['status'])) {
					foreach($_POST['crso_participant']['status'] as $type => $value)
					{
						debug(sprintf('comparing %s and %s', $status, $value));
						$status = ($status<$value)?$value:$status;
					}
				}
				else if (isset($_REQUEST['crso_participant']['active']))
					$status = (int)$_REQUEST['crso_participant']['status'] = PARTICIPANT_STATUS_CONFIRMED;

				$cp = new CRSOParticipantProduct($this->db_app);
				$cp->set($_SESSION['crso_relation_id'],(int)$_REQUEST['participant']['id'],$_SESSION['selected_product_id']);
				$originalActive = $cp->active;
				$cp->active = (int) isset($_POST['crso_participant']['active']);
				if($cp->status<PARTICIPANT_STATUS_CONFIRMED)
					$cp->status = $status;
				//
				$this->langIf->getText('Notification: Registration Confirmed');
				$this->langIf->getText('Notification: CRSO `{CRSO}` confirmed registration participant `{PARTICIPANT}`');

				// REWRITE - 2007-10-29 TICKET #1784
				// The participant is being activated
				if($cp->active&&!$originalActive)
				{
					// Active also means CONFIRMED if we werent that already
					if($cp->status < PARTICIPANT_STATUS_CONFIRMED)
						$cp->status = PARTICIPANT_STATUS_CONFIRMED;

					// We only send a notification if the participant is not ready (we can never go back in status)
					$sendNotification = ($cp->status!=PARTICIPANT_STATUS_PARTICIPANT_READY);

					require_once(ELC_CLASSES.'RelationUser.php');
					require_once(ELC_CLASSES.'Country.php');
					$callback = '';

					// Check CRSO country settings, if they have No Subrogation, do this:
					// See Ticket #35
					$country = new Country($this->db_app,$cp->crso->getCountry(),true);
					// We dont have subrogation:
					// TICKET #411 OR we got Reactivated
					if((!$country->has(HasSubrogation,$_SESSION['selected_product_id']))||
						($cp->status==PARTICIPANT_STATUS_PARTICIPANT_READY))
					{
						// Force the status to Participant Ready
						$cp->status = PARTICIPANT_STATUS_PARTICIPANT_READY;
						$cp->update(); // Update, cause otherwise new Declarations wont be inserted.
						require_once(ELC_CLASSES.'Declaration.php');
						$d = new Declaration($this->db_app);
						// insert new declarations for current participant
						$d->insertNewDeclarations(array('participant'=>$cp->participant,'crso'=>$cp->crso),$_SESSION['selected_product_id']);
					}
					// We have to add a callback to the Declaration Settings page.
					elseif($cp->producer_exporter & PARTICIPANT_PRODUCER)
						$callback = "go('/".SITE_PATH."scripts/elc/participant/InterfaceDeclarationSettings.php?product[id]=".$_SESSION['selected_product_id']."&participant[id]=".$cp->participant->relation->id."&crso[id]=".$cp->crso->relation->id."')";

					if($sendNotification)
					{
						$ru = new RelationUser($this->db_app);
						$cp->crso->relation->set();
						$cp->participant->relation->set();

						Notification::send($this->db_app,
							array('subject'=>$this->langIf->getText('Notification: Registration Confirmed'),
								'sender'=>$cp->crso->relation,
								'message'=>$this->langIf->getText("Notification: CRSO `{CRSO}` has confirmed the registration to participant `{PARTICIPANT}`", null,
									array('CRSO'=>$cp->crso->relation->name, 'PARTICIPANT'=>$cp->participant->relation->name))),
							array(array('recipient'=>$cp->participant->relation,
								'user'=>$ru->getRelationAdmin($cp->participant->relation->id),
								'callback'=>$callback
							)));
					}
					// Activate Participant brands.
					$brand = new Brand($this->db_app);
					$brand->activateByParticipant((int)$_REQUEST['participant']['id'],$_SESSION['crso_relation_id'],$_SESSION['selected_product_id']);

					// User stuff.
					$ru = new RelationUser($this->db_app);
					$user =& $ru->getRelationAdmin((int)$_REQUEST['participant']['id']);
					$user->set();
					$user->info_person->set();
					// Set the Admin user as active
					$ru->set($user->id, (int)$_REQUEST['participant']['id']);
					$ru->status = 1;
					$ru->update();

					// The user was not yet created, do it now.
					if(!$user->name)
					{
						srand((float)microtime() * 1000000);
						$user->active = 1;
						$i = 1; do {
							$name = str_replace(' ', '', $user->info_person->last_name);
							$name = mb_substr($name, 0, 6);
							$name = $name . str_pad($i, 4, '0', STR_PAD_LEFT);
							$i++;
						} while(!$user->isUniqueUserName($name));

						$user->name = $name;
						$user->setPassword($user->generatePasswd());
						$user->setPasswordInfoAndToken();
						$user->update();

						require_once(ELC_CLASSES.'ELCMessageHandler.php');
						MessageEvent::send(PARTICIPANT_CONFIRMATION, array('user'=>$user, 'relationuser'=>$ru, 'crso'=>$cp->crso, 'participant'=>$cp->participant), 'ELCMessageHandler');
						//MessageEvent::send(PARTICIPANT_CONFIRMATION_DAY_AFTER, array('user'=>$user, 'relationuser'=>$ru, 'crso'=>$cp->crso, 'participant'=>$cp->participant), 'ELCMessageHandler');
					}

                                        log_elc_event('Participant Confirmed by CRSO',
                                            $user->id, $_SESSION['selected_product_id'],
                                            $_SESSION['crso_relation_id'],
                                            (int)$_REQUEST['participant']['id']
                                        );

				}
				// Update the CRSOParticipantProduct
				$cp->update();
				$this->db_app->commit();
				header('Location: InterfaceRegistrationsOverview.php');
				exit();
			}
			if($_REQUEST['process'] == 'export')
			{
				$ids = array((int)$_REQUEST['participant']['id']);
				$part = new Participant($this->db_app);
				$csv = $part->export_masterdata($ids,$_SESSION['crso_relation_id']);
				if (!headers_sent())
				header('Content-Disposition: attachment; filename="Masterdata_'.((int)$_REQUEST['participant']['id']).'.csv"');
		echo $csv->getString();
		die();
			}
			$this->init($this->id);
		}
	}
}

class InterfaceRegistrantDetailsItem extends InterfaceItem
{
	/**#@+
	 * Callback functions made from InterfaceItem
	 */
	function getStatusValue()
	{
		$cp =& $this->parent->parent->cp;

		$this->properties['onchange'] = 'statusChange(this);';
		$this->_hideInput = (int) ($cp->status >= PARTICIPANT_STATUS_CONFIRMED);

		switch($this->name)
		{
			case 'crso_participant[status][new]':
				$this->properties['id'] = 'status_new';
				$this->properties['disabled'] = 'disabled';
				$this->properties['checked'] = 'checked';
				$this->properties['value'] = PARTICIPANT_STATUS_NEW;
			break;
			case 'crso_participant[status][verified]':
				$this->properties['id'] = 'status_verified';
				if($cp->status >= PARTICIPANT_STATUS_VERIFIED)
					$this->properties['checked'] = 'checked';
				$this->properties['value'] = PARTICIPANT_STATUS_VERIFIED;
			break;
			case 'crso_participant[status][authorized]':
				$this->properties['id'] = 'status_authorized';
				if($cp->status >= PARTICIPANT_STATUS_AUTHORIZED)
					$this->properties['checked'] = 'checked';
				$this->properties['value'] = PARTICIPANT_STATUS_AUTHORIZED;
			break;
			case 'crso_participant[status][confirmed]':
				$this->properties['id'] = 'status_confirmed';
				if($cp->status >= PARTICIPANT_STATUS_CONFIRMED)
					$this->properties['checked'] = 'checked';
				$this->properties['value'] = PARTICIPANT_STATUS_CONFIRMED;
			break;
		}
	}

	/**
	 *
	 */
	function getCompanyInterface()
	{
		$this->settings = array('targetId'=>'return "ifB_145";');
		$this->settings['hideInvoiceDetails'] = 'return true;';
		$this->settings['_hideInputs'] = 'return true;';
		$this->settings['ajaxUsage'] = 'return true;';
		$this->settings['rel'] = '$relation = new Relation($this->db_app,'.$_REQUEST['participant']['id'].'); return $relation;';
		$this->settings['participant'] = 'return new Participant($this->db_app,array(\'relation\'=> new Relation($this->db_app,'.$_REQUEST['participant']['id'].')));';
		$this->settings['cp'] = &$this->if_obj->cp;
		$this->settings['callbacks']['relation[vat_no]'] = array($this, 'getVatValue');

		require_once(ELC_CLASSES.'CRSO.php');
		require_once(ELC_CLASSES.'RelationUser.php');
		$ru = new RelationUser($this->db);

		// Check for Clearance level.
		if(!$ru->hasClearance(CRSOClearanceRegistrationDetails, $_SESSION['session_user_id'], $_SESSION['crso_relation_id'])) {
			$this->settings['hideButtons'] = true;
		}
	}

	function getVatValue(&$obj)
	{
		require_once(IWARE . 'VAT.php');

		$vat_no = $obj->properties['value'];
		$vat = new VAT($this->db_app, array('vat_no'=>$vat_no), true);

		$txt_valid   = '<span class="info">'. $this->langIf->getText('VAT is valid').'</span>';
		$txt_invalid = '<span class="warning">'.$this->langIf->getText('VAT is invalid').'</span>';
		$txt_pending = '<span class="warning">'.$this->langIf->getText('VAT check is pending').'</span>';

		$text = $txt_valid;
		if ($vat->isComplete())
		{
			switch ($vat->status)
			{
				case VAT_INVALID:
					$text = $txt_invalid;
				break;
				case VAT_PENDING:
					$text = $txt_pending;
				break;
			}
		}
		//$obj->type_id = IF_ITEM_TEXT;
		$obj->properties['value'] = $vat_no.'<br />'.$text;
		$obj->no_hidden = true;
	}

	function getProvisionalDeclaration()
	{
		$crso = $this->parent->parent->cp->crso;

		$html = '';
		$date = currentDate();
		$prov = new ProvisionalDeclaration($this->db_app,array(
					'crso'=>$crso,
					'participant'=>$this->parent->parent->cp->participant,
					'year'=>$date->getYear(),
					'product'=>new Product($this->db, $_SESSION['selected_product_id'])
					));
		$prov->set();

		// TODO: Show information if participant is a trustee

		// show provisional is set
		//$lines = $prov->getLines();
		// See ticket #1150
		$lines = $prov->getAllProvisional();
		$img = $r = $lines->fetch() ? 'true.gif' : 'false.gif';
		$html .= '<p>'.$this->langIf->getText('Participant has Provisional').' <img src="'.REL_IMAGES.'i-w/'.$img.'" style="vertical-align: middle;"></p>';

		// see if this crso in this country is allowed to see provisional details
		$country = new Country($this->db_app, $crso->getCountry(), true);
		if ($country->has(HasCrsoProvisionalDetails, $_SESSION['selected_product_id']))
		{
			require_once(ELC_CLASSES.'RelationUser.php');
    		$ru = new RelationUser($this->db_app);
    		// Check for clearance.
    		if($ru->hasClearance(CRSOClearanceProvisionalDeclaration, $_SESSION['session_user_id'], $crso->relation->id)){

				$t = new TableFactory($prov->getOverview($country->settings), $this->langIf);
				$t->setValueProcessor('month', 'return strftime(\'%B\',mktime(0,0,0,$value, 1));');
				$t->setValueProcessor('product_category', 'return $value->name;');
				$t->setValueProcessor('dimension', 'return $value->name;');
				$t->setValueProcessor('brand', 'return $value->name;');
				$html .= $t->getXhtmlTable('Professional', 2);
    		}
		}

		// Check if the participant has requested a change of his/her provisional
		// We have a CP at the interface, and it has been set.
		if($this->if_obj->cp->provisional_change_request_status == PROVISIONAL_CHANGE_REQUEST_PENDING)
		{
			// Possibility to Approve, or reject.
			$this->script = sprintf("
				function AuthorizeProvChange()
				{
					ChangeProvChangeStatus('authorizeChangeProvisional');
				}
				function RejectProvChange()
				{
					MessageBox.prototype.Information('%s', '%s', function(){TriggerRejectProvChange();});
				}
				function TriggerRejectProvChange()
				{
					var params = new Array();
					params['reject_msg'] = $('prov_reject_msg').value;
					ChangeProvChangeStatus('rejectChangeProvisional',params);
				}
				function ChangeProvChangeStatus(callmethod,params)
				{
					DisableButtons();
					params = params?params:new Array();
					params['participant_id'] = %s;
					params['crso_id'] = %s;
					params['product_id'] = %s;

					// see includes/elc/rpc/RequestChangeProvisional.php
					jsbroker.call(callmethod, params, function(p)
					{
						MessageBox.prototype.Information('', '%s');
					});
				}

				function DisableButtons()
				{
					\$('authorize_prov_change').disabled = 'disabled';
					\$('reject_prov_change').disabled = 'disabled';
				}
				",
				$this->langIf->getText('Reject Provisional Change Request'),
				$this->langIf->getText('Please provide a reason of rejection').'<br /><textarea id="prov_reject_msg" cols="55" rows="6"></textarea>',
				$this->if_obj->cp->participant->relation->id,
				$this->if_obj->cp->crso->relation->id,
				$_SESSION['selected_product_id'],
				$this->langIf->getText('Message has been sent to Participant'));


			$rv = '<a name="provchange"><span class="forfeit">'.
				$this->langIf->getText('Participant has requested a change for their Provisional').
				'</span>
				<br />
				<input type="button" id="authorize_prov_change" value="'.$this->langIf->getText('Authorize Change').'" onclick="AuthorizeProvChange();" />
				<input type="button" id="reject_prov_change" value="'.$this->langIf->getText('Reject Change').'" onclick="RejectProvChange();" /></a>';

			$html = $rv.$html;
			// On REJECT the CRSO can supply a reason
		}

		$this->text = $html;
	}

	function getAdditionalDocuments()
	{
		require_once(ELC_CLASSES . 'CRSOParticipantDocument.php');
		$d = new CRSOParticipantDocument($this->db_app,
			array(
				'participant'=>$this->parent->parent->cp->participant,
				'crso'=>$this->parent->parent->cp->crso));
		$docs = $d->getDocuments();

		$tdata = array();
		foreach ($docs as $doc)
		{
			$row = array();
			$row['DOCUMENT_NAME'] = '<a href="/'.SITE_PATH.'scripts/i-w/document/download.php?Document[id]='.$doc->id.'">'.$doc->name.'</a>';
			$row['DOCUMENT_MIME'] = $doc->getMimeType();
			$row['DOCUMENT_SIZE'] = prettyBytes($doc->size);
			$row['DOCUMENT_DATE'] = Form_Date::makePretty($doc->created);

			$tdata[] = $row;
		}

		$t = new TableFactory($tdata);
		$this->text = $t->getXhtmlTable('Professional', 2);
	}

	function getCrsoParticipantActive()
	{
		$this->properties['id'] = 'status_active';
		$this->properties['checked'] = (int) $this->if_obj->cp->active;
		$this->properties['value'] = PARTICIPANT_STATUS_ACTIVE;

		$title = $this->langIf->getText('Crso confirm active title');
		$body = $this->langIf->getText('Crso confirm active body');

		$this->properties['onclick'] =
			"if (!this.checked){ return; };" .
				jsMessageBox(MSG_BOX_QUEST, $title,  $body,
					"function() { $('status_active').checked = true; statusChange($('status_active')); }",
					"function() { $('status_active').checked = false; statusChange($('status_active')); } ");

		$this->properties['onchange'] = 'statusChange(this)';
	}

	function getParticipantRequestTypeChange()
	{
		$this->_BarButtons = 0;
		$cp = $this->if_obj->cp;
		$participantHasRequestProducer = $this->langIf->getText('Participant has requested to become a producer');
		$participantHasRequestExporter = $this->langIf->getText('Participant has requested to become a export');
		$participantRequestProducer = $this->langIf->getText('Authorize participant request to become a producer');
		$participantRequestProducerReject = $this->langIf->getText('Reject participant request to become a producer');
		$participantRequestExporter = $this->langIf->getText('Authorize participant request to become an exporter');
		$participantRequestExporterReject = $this->langIf->getText('Reject participant request to become an exporter');
		$rejectReason = $this->langIf->getText('Please provide a reason of rejection');
		$messageSent = $this->langIf->getText('Authoriztion has been sent to Participant');

		if($cp->participant_type_change_request && $cp->participant_type_change_request != PARTICIPANT_TYPE_CHANGE_PRODUCER_AUTHORIZED)
		{
			$this->script = sprintf("
				function AuthorizeTypeChange()
				{
					ChangeParticipantType('authorizeChangeParticipantType');
				}
				function RejectTypeChange()
				{
					MessageBox.prototype.Information('%s', '%s', function(){TriggerRejectChangeParticipantType();});
				}
				function TriggerRejectChangeParticipantType()
				{
					var params = new Array();
					params['reject_msg'] = $('change_type_reject_msg').value;
					ChangeParticipantType('rejectChangeParticipantType',params);
				}
				function ChangeParticipantType(callmethod,params)
				{
					DisableButtons();
					params = params?params:new Array();
					params['participant_id'] = %s;
					params['crso_id'] = %s;
					params['status'] = %s;

					// see includes/elc/rpc/RequestChangeParticipantType.php
					jsbroker.call(callmethod, params, function(p)
					{
						MessageBox.prototype.Information('', '%s');
					});
				}

				function DisableButtons()
				{
					\$('authorize_type_change').disabled = 'disabled';
					\$('reject_type_change').disabled = 'disabled';
				}
				",
				$cp->participant_type_change_request == 1 ? $participantRequestExporterReject : $participantRequestProducerReject,
				$rejectReason.'<br /><textarea id="change_type_reject_msg" cols="55" rows="6"></textarea>',
				$this->if_obj->cp->participant->relation->id,
				$this->if_obj->cp->crso->relation->id,
				$cp->participant_type_change_request,
				$messageSent);


			$this->text = '<a name="participantchangetype"><span class="forfeit">'.
				($cp->participant_type_change_request == 1 ? $participantHasRequestExporter : $participantHasRequestProducer).
				'</span>
				<br /><br />
				<input type="button" id="authorize_type_change" value="'.
					($cp->participant_type_change_request == 1 ? $participantRequestExporter : $participantRequestProducer).
					'" onclick="AuthorizeTypeChange();" />
				<input type="button" id="reject_type_change" value="'.
					($cp->participant_type_change_request == 1 ? $participantRequestExporterReject : $participantRequestProducerReject).
				'" onclick="RejectTypeChange();" /></a>';
		}
	}

	function getParticipantReady()
	{
		$cp =& $this->parent->parent->cp;
		$this->_hideInput = true; // always hide input, the participant has to set this
		$this->properties['value'] = ($cp->status >= PARTICIPANT_STATUS_PARTICIPANT_READY) ? PARTICIPANT_STATUS_PARTICIPANT_READY : 0;
	}

	/**
	 *
	 */
	function getButtons()
	{
		$this->hiddenFields = array(array('name'=>'participant[id]','value'=>$_REQUEST['participant']['id']));
		$this->_BarButtons = BUTTON_SAVE + BUTTON_CANCEL;
		$this->_ButtonOptions[BUTTON_CANCEL] = array('onclick'=>"document.location.href='InterfaceRegistrationsOverview.php';");
		$this->_addCustomButton($this->langIf->getText("Export masterdata"), "button", array(
							'onclick' => "$('interface_form').process.value='export'; $('interface_form').submit(); "
							));
	}
	/**#@-*/
}
