<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'AnnualConfirmation.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'AccountantCRSO.php');
require_once(ELC_CLASSES.'Country.php');
/**
 *
 * @package WEEE
 * @subpackage Interface
 * @since 03-01-2007
 * @copyright 2007 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceACOverview extends InterfaceFactory
{
    /**
     * interface[id] = 160
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceACOverview(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

       	$this->model['years'] = array();
       	$this->model['crso'] = false;
       	$this->model['reviewType'] = isset($_REQUEST['review_status']['type']) && $_REQUEST['review_status']['type'] !== '' ? $_REQUEST['review_status']['type'] : false;
        $this->model['ac'] = new AnnualConfirmation($this->db_app);
        if(isset($_REQUEST['crso']['id']))
        {
        	$r = new Relation($this->db);
			$country = new Country($this->db,$r->getCountry($_REQUEST['crso']['id']),true);
			if($country->has(HasAnnualConfirmationDisabled, $_SESSION['selected_product_id']))
				throw new IWAJAXPermissionException("User ".$_SESSION['session_user_id'].
				" accessing ACOverview with AC disabled by country setting for CRSO ".$_REQUEST['crso']['id']);

        	$this->model['ac']->crso = $this->model['crso'] = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app,$_REQUEST['crso']['id'])));
        	$this->model['years'] = $this->model['ac']->getCRSOYears(null,'array',false,$_SESSION['selected_product_id']);
        }
        $this->model['year'] = (isset($_REQUEST['year'])&&in_array($_REQUEST['year'],$this->model['years']))?$_REQUEST['year']:
			((count($this->model['years'])==1)?current($this->model['years']):false);

        $this->script = sprintf("
			function reloadOverview(sel)
			{
				crso = $('ifI_1174');
				selectedCRSO = crso.options[crso.selectedIndex].value;
				if(selectedCRSO)
					go('InterfaceAnnualConfirmationOverview.php?crso[id]='+selectedCRSO+'&year='+sel.options[sel.selectedIndex].value);
			}

			function approveAC()
			{
				MessageBox.prototype.Question('%s','%s',function(){reviewAC('approve')});
			}

			function reviewAC(type)
			{
				$$('process').value = type;
				$('interface_form').submit();
			}

			function rejectAC()
			{
				MessageBox.prototype.Question('%s','%s',function(){reviewAC('reject')});
			}
			",
			$this->langIf->getText('MessageBoxTitle: Approve Annual Confirmations'),
			$this->langIf->getText('MessageBox: Approve SELECTED Annual Confirmations?'),
			$this->langIf->getText('MessageBoxTitle: Reject Annual Confirmations'),
			$this->langIf->getText('MessageBox: Reject SELECTED Annual Confirmations?'));
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        if(!isset($_POST['process']))
        	return;

        $subject = $this->langIf->getText('Annual Confirmation Rejected');
        $body = 'AC for {PARTICIPANT} and {YEAR} at {CRSO} rejected';
        $this->langIf->getText($body);
        $bodies = $recipients = array();
        $review_status = AC_REVIEW_REJECTED;
        $status = ANNUAL_CONFIRMATION_REOPENED; // Reopen if rejected
        $this->db_app->begin();
        switch($_POST['process'])
        {
        	case 'approve':
        		$review_status = AC_REVIEW_APPROVED;
        		$status = ANNUAL_CONFIRMATION_CLOSED;
        	case 'reject':
        		if(!isset($_POST['acid']))
	        		break;
	        	foreach($_POST['acid'] as $id)
	        	{
	        		$ac = new AnnualConfirmation($this->db_app,$id);
	        		$ac->review_status = $review_status;
	        		$ac->status = $status;
	        		$ac->update();
	        		if($review_status==AC_REVIEW_REJECTED)
	        		{
	        			$ac->set();
	        			$ac->participant->relation->set();
	        			$ac->crso->relation->set();
	        			$recipients[$ac->id] = array('recipient'=>$ac->participant->relation,
							'callback'=>"go('/".SITE_PATH."scripts/elc/participant/InterfaceAnnualConfirmation.php')");
						$bodies[$ac->id] = $this->langIf->getText($body,null,
							array('PARTICIPANT'=>$ac->participant->relation->name,
								'YEAR'=>$ac->year,
								'CRSO'=>$ac->crso->relation->name));
	        		}
	        	}
        	break;
        }
        if($recipients)
        	$this->sendNotifications($recipients,$subject,$bodies);

        $this->db_app->commit();

        header(sprintf('Location: InterfaceAnnualConfirmationOverview.php?crso[id]=%s&year=%s&review_status[type]=%s',$this->model['crso']->relation->id,$this->model['year'],$this->model['reviewType']));
        exit;
    }

    function sendNotifications($recipients, $subject, $bodies)
    {
    	foreach($recipients as $id => $rcpt)
    	{
    		require_once(ELC_CLASSES.'Notification.php');
			Notification::send($this->db_app,
				array('subject'=>$subject,
					'sender'=>new Relation($this->db_app,1),
					'message'=>$bodies[$id]
				),
				array($rcpt)
			);
    	}
    }
}
/**
 * @package
 * @subpackage Interface
 * @since 03-01-2007
 * @copyright 2007 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceACOverviewItem extends InterfaceItem
{

	public $totalHash = '';


    /**
     * item[1174] = crso[id]
     */
    function getCRSOSelect()
    {
    	$this->options = array(array('value'=>'','text'=>'- '.$this->langIf->getText('Select CRSO').' -'));
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
				$crso = new CRSO($this->db_app);
				$q = $crso->selectCRSOs();
				break;
			case GROUP_ACCOUNTANT:
		    	$accountant_crso = new AccountantCRSO($this->db_app);
				$q = $accountant_crso->getAccountantCRSOs($_SESSION['accountant_relation_id'],$_SESSION['selected_product_id']);
				break;
		}
		$rs = $q->fetchAll();

    	// only get CRSOs which do not have AC disabled as country setting
		$crsos = array();
		foreach($rs as $r)
		{
			$rel = new Relation($this->db);
			$country = new Country($this->db,$rel->getCountry($r->CRSO_RELATION_ID),true);
			if(!$country->has(HasAnnualConfirmationDisabled, $_SESSION['selected_product_id']))
				$crsos[] = array('id' => $r->CRSO_RELATION_ID, 'name' => $r->NAME);
		}

		if(count($crsos) == 1){
			$this->if_obj->model['ac'] = new AnnualConfirmation($this->if_obj->db_app);
			$this->if_obj->model['ac']->crso =
			$this->if_obj->model['crso'] =
				new CRSO($this->if_obj->db_app,array('relation'=>new Relation($this->if_obj->db_app, $crsos[0]['id'])));
			$this->if_obj->model['years'] = $this->if_obj->model['ac']->getCRSOYears(null,'array',false,$_SESSION['selected_product_id']);
		}
    	foreach($crsos as $r)
    	{
    		$name = $r['name'];
    		$name = (mb_strlen($name)>35)?mb_substr($name,0,35).'...':$name;
    		$this->options[] = array('value'=>$r['id'],'text'=>$name,
				'selected'=>(($this->if_obj->model['crso']&& $this->if_obj->model['crso']->relation->id==$r['id'])?'selected':false));
    	}

    	$this->properties['onchange'] = "if(this.options[this.selectedIndex].value) go('InterfaceAnnualConfirmationOverview.php?crso[id]='+this.options[this.selectedIndex].value);";
    }

    /**
     * item[1175] = year
     */
    function getYearSelect()
    {
    	$this->options = array();
    	if(!$this->if_obj->model['crso'])
    	{
    		$this->options[] = array('value'=>'','text'=>'- '.$this->langIf->getText('Select CRSO First').' -');
    		return;
    	}

    	if(count($this->if_obj->model['years'])==1)
    	{
    		$this->_hideInput = true;
    		$this->options[] = array('value'=>$this->if_obj->model['year'],'text'=>$this->if_obj->model['year'],'selected'=>'selected');
    		return;
    	}
    	$this->options[] = array('value'=>'','text'=>'- '.$this->langIf->getText('Select Year').' -');
    	foreach($this->if_obj->model['years'] as $year)
    		$this->options[] = array('value'=>$year,'text'=>$year,'selected'=>(($this->if_obj->model['year']==$year)?'selected':false));

    	$this->properties['onchange'] = 'reloadOverview(this);';
    }

	function getTypeDropdown()
    {
    	$crso_id = $this->if_obj->model['crso']?$this->if_obj->model['crso']->relation->id:false;
    	//possible statuses of confirmations
    	//they don't need to see rejected ones :)
    	$review_types = array(AC_REVIEW_OPEN, AC_REVIEW_APPROVED);

    	$this->options = array(array('value'=>'','text'=>'- '.$this->langIf->getText('Select').' -'));
    	$i = 1;
    	foreach($review_types as $type)
    	{
    		$trans = $this->if_obj->model['ac']->translateReviewStatus($this->langIf, $type);
    		$this->options[$i] = array('value' => $type, 'text' => $trans[0], //);
				'selected'=>(($this->if_obj->model['reviewType']!== false && $this->if_obj->model['reviewType'] == $type)?'selected':false) );
    		//if($this->if_obj->model['reviewType']!== false && $this->if_obj->model['reviewType'] == $type)
    			//$this->options[$i]['selected'] = 'selected';
    		$i++;
    	}

    	$this->properties['onchange'] = sprintf("go('%s?review_status[type]='+this.value+'&crso[id]=%s');",
    		'InterfaceAnnualConfirmationOverview.php', $crso_id);
    }

    /**
     * item[1176] =
     */
    function getACOverview()
    {
    	$makeSelectionFirst = $this->langIf->getText('Make Selection First');
    	if(!$this->if_obj->model['year']||!$this->if_obj->model['crso'])
    		return $this->text = $makeSelectionFirst;

    	$ds = $this->if_obj->model['ac']->getCRSOOverview(null,$_SESSION['selected_product_id'],$this->if_obj->model['year'],ANNUAL_CONFIRMATION_CLOSED,$this->if_obj->model['reviewType']);
    	$tf = new TableFactory($ds,$this->langIf);
    	$tf->setValueProcessor(TF_ALL,array($this,'tfValueProcessor'));
    	$tf->hideColumns(array('PARTICIPANT_RELATION_ID','CRSO_RELATION_ID'));
    	$this->text = $tf->getXHTMLTable('Professional',2);
    }

	function tfValueProcessor($args)
	{
		$trans = $style = false;
		switch($args['head'])
		{
			case 'CHECKBOX':
				if($args['row']['STATUS']!=ANNUAL_CONFIRMATION_CLOSED)
					return '';
				return sprintf('<input type="checkbox" name="acid[]" value="%s" />',$args['value']);
			break;
			case 'REQUESTED_CHANGE':
				$img = ($args['value']) ? 'true.gif' : 'false.gif';
    			return '<img src="/'.SITE_PATH.'images/i-w/'.$img.'" />';
			break;
			case 'STATUS':
				list($trans,$style) = AnnualConfirmation::translateStatus($this->langIf,$args['value']);
			case 'REVIEW_STATUS':
				if(!$trans)
					list($trans,$style) = AnnualConfirmation::translateReviewStatus($this->langIf,$args['value']);
				return sprintf('<span class="%s">%s</span>',$style,$trans);
			break;
			case 'AC_DOCUMENT':
    			if(!$args['value'])
    				return '<img src="/'.SITE_PATH.'images/i-w/false.gif" />';
				require_once(ELC_CLASSES.'Document.php');
				$doc = new ELCDocument($this->db_app,$args['value'],true);
				$name = $this->langIf->getText('Download');
				return sprintf('<a href="/%sscripts/i-w/document/download.php?Document[id]=%d">%s</a>',SITE_PATH,$doc->id,$name);
			break;
			case 'HASH':
				$this->_cols = $args['visibleColumns'];
				$this->totalHash .= $hash = md5($args['value'].
    			$this->if_obj->model['year'].
    			str_pad($args['row']['PARTICIPANT_RELATION_ID'],8,'0',STR_PAD_LEFT).
    			str_pad($args['row']['CRSO_RELATION_ID'],8,'0',STR_PAD_LEFT));
    			return $hash;
    			break;
			default:
				return $args['value'];
		}
	}

    /**
     * item[1177] =
     */
    function getButtons()
    {
    	$this->_BarButtons = 0;
    	$this->_addCustomButton($this->langIf->getText('Approve ACs'),'button',array('onclick'=>'approveAC();'));
    	$this->_addCustomButton($this->langIf->getText('Reject ACs'),'button',array('onclick'=>'rejectAC();'));
    }
}

/**
 * CALLBACK[getCRSOSelect]
 * item[1174] = {
 *     [handle] = Item: CRSO Selection
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getYearSelect]
 * item[1175] = {
 *     [handle] = Item: Year Selection
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getACOverview]
 * item[1176] = {
 *     [handle] = Item: ACOverview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
