<?php

require_once(INTERFACES.'elc/common/InterfaceCRSODateSettings.php');

/**
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @version $Id$
 */
class InterfaceCRSOAdminFee extends InterfaceCRSODateSettings
{
	/**
	 * Constructor
	 */
	function InterfaceCRSOAdminFee(&$db, &$db_app, &$langIf)
	{
		$this->InterfaceFactory($db, $db_app, $langIf);

		require_once(ELC_CLASSES.'CRSOAdminFee.php');
		$af = new CRSOAdminFee($db_app);
		foreach (CRSOAdminFee::getTypes() as $caption=>$adminFeeType)
		{
			$q = $af->selectCrsoTypeFee($_REQUEST['crso']['id'], $adminFeeType);
			$noset = $this->langIf->getText('No admin fee set');
			if (!($row = $q->fetch()))
			{
				$this->_fields[] = array(
					'setting'=>$this->langIf->getText($caption),
					'date'=>'-',
					'value'=>$noset,
					'typeId'=>$adminFeeType);
			}
			else
			{

				do
				{
					$value = locale_nf($row->{'FEE'}, 2);
					$this->_fields[] = array(
						'setting'=>$this->langIf->getText($caption),
						'date'=>$row->{'START_DATE'},
						'value'=>$value,
						'typeId'=>$adminFeeType
					);
				}while ($row = $q->fetch());
			}

			$this->currentValues[$adminFeeType] = $af->getCurrentFee($_REQUEST['crso']['id'], $adminFeeType);
		}

		/**
		 * Fields
		 */
		// select type
    	$sel = new Form_Select('adminFee[admin_fee_type]');
    	$sel->setBehaviour('required(adminFee[fee], true)');
    	foreach (CRSOAdminFee::getTypes() as $caption=>$adminFeeType)
    		$sel->addOption($this->langIf->getText($caption), $adminFeeType);
		$this->frmSelType = &$sel;

		// value
    	$txt = new Form_Text('adminFee[fee]');
    	$txt->setBehaviour('required(adminFee[admin_fee_type],true);decimal');
    	$txt->setClass('');
		$this->frmValue = &$txt;

		// start date
		$this->fieldStartDate = 'adminFee[start_date]';

		// call parent
		$this->InterfaceCRSODateSettings($db, $db_app, $langIf);
	}

	function loadClasses()
	{
		$this->classPrefix = 'InterfaceCRSOAdminFee';
		$this->appPath = 'elc/operator/';
	}

    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
    	if ($_REQUEST['process'] == 'save' && isset($_REQUEST['adminFee']) && is_array($_REQUEST['adminFee']))
    	{
    		if (!parent::processInput($thrownException))
    			return false;

    		$arr = $_REQUEST['adminFee'];
    		$arr['start_date'] = reindex_date($arr['start_date'], 'DD-MM-YYYY', 'YYYY-MM-DD');

    		$di = new CRSOAdminFee($this->db_app, $arr);
    		$di->crso = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, $_REQUEST['crso']['id'])));
    		$di->insert();
    	}

    	return true;
    }

}

/**
 * @package ELC
 * @subpackage Interface
 * @since 09-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceCRSOAdminFeeBlock extends InterfaceCRSODateSettingsBlock {}

class InterfaceCRSOAdminFeeItem extends InterfaceCRSODateSettingsItem {}

?>
