<?php

require_once(INTERFACES.'elc/common/InterfaceCRSODateSettings.php');
require_once(ELC_CLASSES.'DeclarationInterest.php');

/**
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @version $Id$
 */
class InterfaceCRSODeclarationInterest extends InterfaceCRSODateSettings
{
	/**
	 * Constructor
	 */
	function InterfaceCRSODeclarationInterest(&$db, &$db_app, &$langIf)
	{
		$this->InterfaceFactory($db, $db_app, $langIf);

		require_once(ELC_CLASSES.'DeclarationInterest.php');
		$di = new DeclarationInterest($db_app);
		foreach (DeclarationInterest::getTypes() as $caption=>$declIntType)
		{
			$q = $di->selectCrsoDeclIntType($_REQUEST['crso']['id'], $declIntType,$_SESSION['selected_product_id']);
			$noset = $this->langIf->getText('No declaration interest rate set');
			if (!($row = $q->fetch()))
			{
				$this->_fields[] = array(
					'setting'=>$this->langIf->getText($caption),
					'date'=>'-',
					'value'=>$noset,
					'typeId'=>$declIntType);
			}
			else
			{
				do
				{
					$value = $row->{'INTEREST_RATE'};
					$this->_fields[] = array(
						'setting'=>$this->langIf->getText($caption),
						'date'=>$row->{'START_DATE'},
						'value'=>$value,
						'typeId'=>$declIntType
					);
				}while ($row = $q->fetch());
			}

			$this->currentValues[$declIntType] = $di->getCurrentRate($_REQUEST['crso']['id'], $declIntType,$_SESSION['selected_product_id']);
		}

		/**
		 * Fields
		 */
		// select type
    	$sel = new Form_Select('interestRate[declaration_interest_type]');
    	$sel->setBehaviour('required(interestRate[interest_rate], true)');
    	foreach (DeclarationInterest::getTypes() as $caption=>$adminFeeType)
    		$sel->addOption($this->langIf->getText($caption), $adminFeeType);
		$this->frmSelType = &$sel;

		// value
    	$txt = new Form_Text('interestRate[interest_rate]');
    	$txt->setBehaviour('required(interestRate[declaration_interest_type], true);integer');
    	$txt->setClass('');
		$this->frmValue = &$txt;

		// start date
		$this->fieldStartDate = 'interestRate[start_date]';
		$this->InterfaceCRSODateSettings($db, $db_app, $langIf);
	}

	function processInput(&$thrownException)
	{
     	if (!parent::processInput($thrownException))
     	{
     		debug('not saving interest');
     		return false;
     	}
 		else
 			debug('saving interest');


    	if ($_REQUEST['process'] == 'save' && isset($_REQUEST['interestRate']) && is_array($_REQUEST['interestRate']))
    	{
    		$arr = $_REQUEST['interestRate'];
    		$arr['start_date'] = reindex_date($arr['start_date'], 'DD-MM-YYYY', 'YYYY-MM-DD');

    		$di = new DeclarationInterest($this->db_app, $arr);
    		$di->crso = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, $_REQUEST['crso']['id'])));
    		$di->product = new Product($this->db_app, $_SESSION['selected_product_id']);
    		$di->insert();
    	}

    	return true;
	}

	function loadClasses()
	{
		$this->classPrefix = 'InterfaceCRSODeclarationInterest';
		$this->appPath = 'elc/operator/';
	}
}

/**
 * @package ELC
 * @subpackage Interface
 * @since 09-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceCRSODeclarationInterestBlock extends InterfaceCRSODateSettingsBlock
{
}

/**
 * @package ELC
 * @subpackage Interface
 * @since 09-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceCRSODeclarationInterestItem extends InterfaceCRSODateSettingsItem
{
    /**
     * todo: move to child
     */
    function getFormSelectType()
    {
    	return 'good';
    }

    function getFormValue()
    {
    	return 'good';
    }

}

?>
