<?php
require_once(INTERFACE_OBJ."InterfaceFactory.php");
/**
 * List of currently created CRSO's
 *
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @since 20-01-2005
 * @version $Id$
 */
class InterfaceCRSOOverview extends InterfaceFactory
{
	function InterfaceCRSOOverview($db, $db_app, $langIf)
	{
		$this->FirstColWidth = 25;
		$this->delimiter = '<br />';
		parent::InterfaceFactory($db, $db_app, $langIf);

		require_once(ELC_CLASSES.'CRSO.php');
	}

	function processInput(&$thrownException)
	{
		if(isset($_POST['process']))
		{
			if($_POST['process'] == 'delete')
			{
				/**
				 * CRSO deletion policy:
				 * - No USERs can be defined for this CRSO
				 * - No declarations can be made for this CRSO
				 * - No Brands can be attached to this CRSO
				 * - No participants can be registered for this CRSO
				 */

			}
			elseif($_POST['process'] == 'save')
			{
				foreach($_POST['accepted_sla'] as $crso_id => $status)
				{
					$crso = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app,$crso_id)));
					$crso->accepted_sla = $status;
					$crso->update();
				}
				$this->init($this->id);
			}
		}
	}
}
/**
 * @package ELC
 * @subpackage Interface
 */
class InterfaceCRSOOverviewItem extends InterfaceItem
{
	/**
	 * @todo Fix them ValueProcessors to be one callback function
	 */
	function getOverview()
	{
		require_once(IWARE.'TableFactoryColumnPlugin.php');

		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
				$crso = new CRSO($this->db_app);
				$ds = $crso->selectOverview(null,$_SESSION['selected_product_id']);
				break;
			case GROUP_ACCOUNTANT:
				$sql = "SELECT '' as CHECKBOX, AC.CRSO_RELATION_ID, '' as CRSO_SETTINGS,
						R.`NAME` as RELATION_NAME, COUNTRY.`NAME` as COUNTRY_NAME,
						R.`EMAIL`, C.ACCEPTED_SLA, COUNT(DISTINCT U.`USER_ID`) as USERS,
						'' as WEBSITE_LINK, COUNT(DISTINCT CP.PRODUCT_ID) as PRODUCTS
					FROM TBL_ACCOUNTANT_CRSO AC
					INNER JOIN TBL_CRSO C ON C.CRSO_RELATION_ID = AC.CRSO_RELATION_ID
					INNER JOIN TBL_CRSO_PRODUCT as CP
						ON C.CRSO_RELATION_ID = CP.CRSO_RELATION_ID
					INNER JOIN TBL_RELATION as R
						ON C.CRSO_RELATION_ID = R.RELATION_ID
					INNER JOIN TBL_ADDRESS as A
						USING(ADDRESS_ID)
					INNER JOIN TBL_COUNTRY as COUNTRY
						USING(COUNTRY_ID)
					LEFT JOIN TBL_RELATION_USER as RU
						ON R.RELATION_ID = RU.RELATION_ID
					LEFT JOIN TBL_USER as U
						ON RU.USER_ID = U.USER_ID
					WHERE AC.ACCOUNTANT_RELATION_ID = ?
					GROUP BY R.RELATION_ID";
				$ds = $this->db->query($sql);
				$ds->bind_values($_SESSION['accountant_relation_id']);
				$ds->execute();
				break;
		}

		$tf = new TableFactoryColumnPlugin(false,$this->langIf,$this->db,$this->db_app);
		$tf->setEmptyText($this->langIf->getText('No CRSOs found') );
		$tf->setData($ds, 'CRSOOverview',false,true);
		$tf->setSortingFunctions(
			array('CRSO_RELATION_ID'=>"'Int'",
				'COUNTRY_NAME'=>"'String'",
				'RELATION_NAME'=>"'String'",
				'EMAIL'=>"'String'",
				'USERS'=>"'String'",
				'ACCEPTED_SLA', "'String'"
				));

		$help = ($this->parent->title != '') ? getHelp($this->langIf, $this->parent->handle, array('title'=>$this->parent->handle, 'top'=>'0px')) : '';
		$tf->setCaption($help.'<img src="/'.SITE_PATH.'images/block_arrow.png" /> '.$this->langIf->getText('CRSO Overview'), false);

		$tf->setValueProcessor('CRSO_RELATION_ID','return \'<a href="InterfaceCRSORegistration.php?crso[id]=\'.$row[\'CRSO_RELATION_ID\'].\'">\'.$value.\'</a>\';');
		$tf->setValueProcessor('CRSO_SETTINGS','return \'<a id="crso_settings_\'.$row[\'CRSO_RELATION_ID\'].\'" href="InterfaceCRSOSettings.php?crso[id]=\'.$row[\'CRSO_RELATION_ID\'].\'">\'.$this->langIf->getText(\'Edit\').\'</a>\';');
		$tf->setValueProcessor('USERS','return \'<a id="crso_users_\'.$row[\'CRSO_RELATION_ID\'].\'" href="InterfaceCRSOUserOverview.php?crso[id]=\'.$row[\'CRSO_RELATION_ID\'].\'">\'.$value.\'</a>\';');

		$tf->setValueProcessor('CHECKBOX', 'return \'<input type="checkbox" disabled="disabled" name="crso_relation[]" value="\'.$row[\'CRSO_RELATION_ID\'].\'" />\';');
		$tf->setValueProcessor('WEBSITE_LINK', 'return \'<a href="#" onclick="showLink(\\\'\'.$row[\'RELATION_NAME\'].\'\\\', \'.$row[\'CRSO_RELATION_ID\'].\');" />'.$this->langIf->getText('Website Link').'</a>\';');
		$tf->setValueProcessor('ACCEPTED_SLA','return \'<input type="hidden" name="accepted_sla[\'.$row[\'CRSO_RELATION_ID\'].\']" value="\'.$value.\'" />\'.
			($value?\'<img src="/\'.SITE_PATH.\'images/i-w/true.gif" />\':\'<img src="/\'.SITE_PATH.\'images/i-w/false.gif" />\');');

		$tf->setValueProcessor('PRODUCTS',array($this,'tfProducts'));

		$tf->hideColumns( array('VAT_NO') );

		$this->text = '<span class="error" id="ChangeMsg"></span>
			<script type="text/javascript"><!--
				var changed = false;
				function changeImg(el,crsoId)
				{
					if(!changed)
					{
						changed = true;
						getElById(\'ChangeMsg\').innerHTML = \''.$this->langIf->getText('Changes were made but not saved').'\';
					}
					iEl = document.getElementsByName(\'accepted_sla[\'+crsoId+\']\')[0];
					if(iEl.value==1)
						el.src=\'/'.SITE_PATH.'images/i-w/false.gif\';
					else
						el.src=\'/'.SITE_PATH.'images/i-w/true.gif\';
					iEl.value = (iEl.value==1)?0:1;
				}

				function showLink(crsoName, crsoId)
				{
					MessageBox.prototype.Information(\''.$this->langIf->getText('CRSO Website Link').' (\'+crsoName+\')\', \''.$this->langIf->getText('This CRSO should place a link on their website').'<br /><textarea rows="2" cols="50" readonly="readonly">https://'.SITE_SERVER.SITE_PATH.'crsoindex.php?crso[id]=\'+crsoId+\'&product[id]='.$_SESSION['selected_product_id'].'</textarea>\');
				}
			--></script>
			'.$tf->getXHTMLTable('Professional',2);
	}
	function tfProducts($args)
	{
		return sprintf('<a id="crso_products_%s" href="InterfaceCRSOProductManagement.php?crso[id]=%s">%s</a>',$args['row']['CRSO_RELATION_ID'], $args['row']['CRSO_RELATION_ID'],$args['value']);
	}
	/**
	 * Displays the New and Delete buttons
	 */
	function getButtons()
	{
		$this->text = "
			<script type=\"text/javascript\"><!--
			function deleteCRSOs()
			{
				MessageBox.prototype.Question('".$this->langIf->getText('Warning')."', '".$this->langIf->getText('Are you sure you want to delete the selected CRSOs')."',
					function(){
						document.getElementsByName('process')[0].value='delete';
						document.getElementById('interface_form').submit();
					}, function(){});
			}
			--></script>
			";
		$this->_addCustomButton($this->langIf->getText('Add new CRSO'),'button',array('onclick'=>"document.location.href='InterfaceCRSORegistration.php';"));
		$this->_BarButtons = BUTTON_DELETE;
		//$this->_addCustomButton($this->langIf->getText('Remove selected CRSOs'),'button', array('onclick'=>"deleteCRSOs();"));
	}
}

?>
