<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'Product.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'CRSOProduct.php');
/**
 *
 * @package
 * @subpackage Interface
 * @since 23-11-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceCRSOProductManagement extends InterfaceFactory
{
    /**
     * interface[id] = 157
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceCRSOProductManagement(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        if(!isset($_REQUEST['crso']['id']))
        	throw new InterfacePermissionException('Incorrect Number of arguments supplied');

        $this->model['crso'] = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app,$_REQUEST['crso']['id'], true)));
        $this->model['cp'] = new CRSOProduct($this->db_app,array('crso'=>$this->model['crso']));
        $this->model['saveDisabled'] = true;
        $this->hiddenFields = array(array('name'=>'crso[id]','value'=>$this->model['crso']->relation->id));
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        if(!isset($_POST['process']))
        	return;

        if($_POST['process'] == 'save')
        {
        	foreach($_POST['product'] as $id)
        	{
        		$this->model['cp']->product = new Product($this->db_app,$id);
        		$this->model['cp']->insert();
        	}
        	header(sprintf('Location: InterfaceCRSOProductManagement.php?crso[id]=%s',$this->model['crso']->relation->id));
        	exit();
        }
    }
}
/**
 * @package
 * @subpackage Interface
 * @since 23-11-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceCRSOProductManagementItem extends InterfaceItem
{

    /**
     * item[1154] =
     */
    function getProductOverview()
    {
    	$ds = $this->if_obj->model['cp']->getCRSOProductOverview();
    	$tf = new TableFactory($ds,$this->langIf);
    	$tf->setCaption(sprintf($this->langIf->getText('Product Overview').' (%s)',$this->if_obj->model['crso']->relation->name),false);
    	$tf->hideColumn('PRODUCT_ID');
    	$tf->setValueProcessor('CHECKBOX',array($this,'tfProductSelected'));
    	$this->text = $tf->getXHTMLTable('Professional',2);
    }

	function tfProductSelected($args)
	{
		if($args['value'])
			return sprintf('<img src="/%simages/i-w/true.gif" id="product_img_%s" />',SITE_PATH, htmlentities($args['row']['PRODUCT_NAME'], ENT_QUOTES, 'UTF-8'));

		$this->if_obj->model['saveDisabled'] = false;
		return sprintf('<input id="product_%s" type="checkbox" name="product[]" value="%s" />',htmlentities($args['row']['PRODUCT_NAME'], ENT_QUOTES, 'UTF-8'), $args['row']['PRODUCT_ID']);
	}

    /**
     * item[1155] =
     */
    function getButtons()
    {
    	$this->_BarButtons = BUTTON_CANCEL;

    	$this->_ButtonOptions[BUTTON_CANCEL]['onclick'] = "go('InterfaceCRSOOverview.php');";
    	// Add a custom button for save with MessageBox warning

    	$props = array('onclick'=>'productWarning();');
    	if($this->if_obj->model['saveDisabled'])
    		$props['disabled'] = 'disabled';
    	$this->_addCustomButton($this->langIf->getText('Save'),'button',$props);

    	$this->postscript = sprintf("
			function productWarning()
			{
				MessageBox.prototype.Question('%s','%s',function(){
					$('interface_form').submit();
				});
			}
			",
			$this->langIf->getText('Warning Title'),
			$this->langIf->getText('Warning, this is an irriversable action, continue?'));
    }

}

/**
 * CALLBACK[getProductOverview]
 * item[1154] = {
 *     [handle] = Item: ProductOverview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[1155] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>