<?php
require_once(INTERFACE_OBJ."InterfaceFactory.php");
require_once(ELC_CLASSES."FormBehaviourPlugins.php");
require_once(IWARE.'group.php');
/**
 * InterfaceCRSORegistration
 *
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @version $Id$
 */
class InterfaceCRSORegistration extends InterfaceFactory
{
	/**
	 * @var boolean Whether this interface is being used in an ajax context
	 */
	var $ajaxUsage = false;

	function InterfaceCRSORegistration($db, $db_app, $langIf)
	{
		//check ajax usage
		$this->ajaxUsage = isset($_REQUEST['ajaxUsage']) && (int)$_REQUEST['ajaxUsage'] ? true : false;

		$this->FirstColWidth = 25;
		$this->delimiter = '<br />';
		// If we are not processing, there is no need to set values from POST data
		if(isset($_POST['process']))
			$this->_setValues = true;
		parent::InterfaceFactory($db, $db_app, $langIf);

		require_once(IWARE.'address.php');
		require_once(IWARE.'InfoPerson.php');
		require_once(IWARE.'nLanguage.php');
		require_once(IWARE.'RelationType.php');
		require_once(ELC_CLASSES.'CRSO.php');
		require_once(ELC_CLASSES.'RelationInvoice.php');

		if($this->ajaxUsage && !isset($_REQUEST['edit']))
			$this->_hideInputs = true;
		if(isset($_REQUEST['crso']['id']))
		{
			$cr = new CRSO($this->db);
			$cr->set((int)$_REQUEST['crso']['id']);
			$country_id = $cr->getCountry();
			if($country_id == '826')
				$this->fb->register('address[address_zipcode]', 'required;ukpostcode');
			else
				$this->fb->register('address[address_zipcode]', 'required;zipcode');
		}
	}
	/**
	 * Processes the input
	 * @param string $thrownException
	 */
	function processInput(&$thrownException)
	{
		$this->process = 'check'; // Defaults to check.
		if(isset($_POST['process']))
		{
			if(!$this->fb->validate())
			{
				$_POST['process'] = '';
				return $thrownException .= $this->langIf->getText('Please correct the form errors');
			}
			$update = isset($_REQUEST['crso']['id']);
			// CHECK UNIQUENESS OF VAT-NUMBER
			if(!$update&&!Relation::isUniqueVAT($_POST['relation']['vat_no'],$this->db_app))
			{
				$_POST['process'] = '';
				return $thrownException = $this->langIf->getText('An application has already been made for the requested VAT');
			}

			if($_POST['process'] == 'check')
			{
				$this->process = 'save'; // Next process is save.
				$this->_hideInputs = true; // Dont show the inputs but a confirmation page.

				global $page_title;
				$page_title = 'CRSO Registration Confirmation';

			}
			elseif($_POST['process'] == 'save')
			{
				$this->db_app->begin();

// if the CRSO is already set, we're gonna update

				$crso = new CRSO($this->db_app);
				if($update)
				{
					$crso->set($_POST['crso']['id']);
					$old_country_id = $crso->getCountry(); // get old country, from before the submit
					$crso->relation->set(); // get the old VAT, from before the submit
					$old_vat_no = $crso->relation->vat_no;
				}
				// Checkboxes dont have a value by the default of InterfaceItem...
				$crso->accepted_sla = empty($_POST['crso']['accepted_sla'])?0:1;
				$crso->default_language_id = $_POST['crso_language']['id'];

				$crso->reg_no = $_POST['crso']['reg_no'];
				$crso->misc_1 = $_POST['crso']['misc_1'];
				// Insert a new entry to the database CRSO_PRODUCT for the crso + selected product;

				$ri = new RelationInvoice($this->db_app);

				$relation = $_POST['relation'];
				// Add a contact
				// Not sure what this will do yet, field is listed as NAME,
				// maybe save in INFO_PERSON_LAST_NAME?
				$contact = $_POST['contact'];
				$contact['company_id'] = $_SESSION['session_user_company_id']; // DefCompany
				$contact['language_id'] = $_SESSION['session_user_language_id']; // DefLanguage
				$relation['customer_contact'] = new InfoPerson($this->db_app,$contact);
				if($update)
					$relation['customer_contact']->update();
				else
					$relation['customer_contact']->insert();
				// invoice_language
				// Defaults to the current language, it isnt used yet, but might be used in the future
				$relation['invoice_language'] = new nLanguage($this->db_app,$_SESSION['session_user_language_id']);
				// Add the address settings
				// Need I say more?

				// test if some fields can be changed or not
				if ($update) {
					if ((int)$_POST['address']['country_id'] != $old_country_id) {
						$_POST['address']['country_id'] = $old_country_id;
						info('Someone tried to change the CRSO country by enabling the Country field.');
					}
					if ($relation['vat_no'] != $old_vat_no) {
						$relation['vat_no'] = $old_vat_no;
						info('Someone tried to change the CRSO VAT Number by making the field writable.');
					}
				}
				// end of country change test

				$relation['address'] = new Address($this->db_app,$_POST['address']);
				if($update)
					$relation['address']->update($relation['address']->address_id);
				else
					$relation['address']->insert();
				// Invoice settings
				// If the invoice is the same as CRSO address, dont do anything with it.
				if(isset($_POST['IsSameAsCRSO']))
					$ri->address =& $relation['address'];
				else
				{
					// Different addresss for invoicing is used.
					$ri->address = new Address($this->db_app,$_POST['invoice_address']);
					if($update)
						$ri->address->update($_POST['invoice_address']['address_id']);
					else
						$ri->address->insert();
				}
				$contact = $_POST['invoice_person'];
				if(!$update)
				{
					$contact['company_id'] = $_SESSION['session_user_company_id'];
					$contact['language_id'] = $_SESSION['session_user_language_id'];
					$ri->info_person = new InfoPerson($this->db_app,$contact);
					$ri->info_person->insert();
				}
				else
				{
					$ri->info_person = new InfoPerson($this->db_app,$contact);
					$ri->info_person->update();
				}
				// Add the relation type
				if(!$update)
				{
					$relation['relation_type'] = new RelationType($this->db_app,RELATION_CRSO);
					$relation = new Relation($this->db_app,$relation);
					$crso->relation =& $relation;
					$ri->relation =& $relation;

					$crso->relation->insert();
					$crso->insert();
					$ri->insert();

					require_once(ELC_CLASSES.'CRSOProduct.php');
					require_once(ELC_CLASSES.'Product.php');
					$crso_product = new CRSOProduct($this->db_app);
					$crso_product->crso =& $crso;
					$crso_product->product = new Product($this->db_app,$_SESSION['selected_product_id']);
					$crso_product->insert();

					require_once(ELC_CLASSES.'Notification.php');
			  		Notification::send($this->db_app,
			 			array('subject'=>$this->langIf->getText('Notification: No visible Fee'),
			 				'message'=>$this->langIf->getText("Notification: Please set a visible fee")),
			 			array(array(
			 				'recipient'=>$crso->relation,
			 				'callback'=>"go('/".SITE_PATH."scripts/elc/crso/InterfaceProductFeeOverview.php')"
			 			)));

				}
				else
				{
					$relation = new Relation($this->db_app,$relation);
					$crso->relation =& $relation;
					$ri->relation =& $relation;
					$crso->relation->update();
					$crso->update();
					$ri->update();
				}

                                log_elc_event(sprintf('CRSO Registration %s', $update ? 'updated': 'completed'),
                                    $_SESSION['session_user_id'], $_SESSION['selected_product_id'],
                                    $crso->relation->id, null
                                );

				$this->db_app->commit();
				if(!$this->ajaxUsage)
				{
					header('Location: InterfaceCRSOOverview.php');
					exit();
				}
				else
				{
					$this->init($this->id);
				}
			}
		}
	}

	function getInterfaceBehaviour()
	{
		ob_start();
?>
			<script type="text/javascript" src="<?=REL_IWARE_INCLUDES?>FormBehaviour.js"></script>
			<script type="text/javascript">
				/**
				 * Form Behaviour
				 */
				var fb = new FormBehaviour();
				/**
				 * Window onLoad
				 */
				addEvent(window,'load', function()
				{
					addEvent(getElById('interface_form'),'submit',function()
					{
						if(fb.validate(this))
						{
							this.submit();
						}
					});
					<?if(empty($_POST['process'])){?>
					toggleFields();
					enable = (Radio_getSelectedValue('HasDirectDebit')=='yes');
					toggleDD(!enable);
					<?}?>
				});

				function toggleDD(enable)
				{
					var dd = new Array('relation[dd_bank_account]','relation[dd_iban_no]', 'relation[dd_swift_bic_code]');

					var check = enable;

					dd.forEach(function(element, index, array)
					{
						el = document.getElementsByName(element)[0];
						el.disabled = check?'disabled':'';

						if(check)
						{
							addCssClass(el,'disabled');
							el.value = '';
						}
						else
							removeCssClass(el,'disabled');
					});
				}
				function toggleFields()
				{
					var invoice = new Array('invoice[name]','invoice_address[country_id]',
						'invoice_address[address_street_name]','invoice_address[city_name]',
						'invoice_address[address_zipcode]');
					var check = document.getElementsByName('IsSameAsCRSO')[0];

					invoice.forEach(function(element, index, array)
						{
							el = document.getElementsByName(element)[0];
							el.disabled = check.checked?'disabled':'';

							if (check.checked)
								FormBehaviour.prototype.removeBehaviour(el, 'required');
							else
								FormBehaviour.prototype.setBehaviour(el, 'required');

							if(check.checked)
								addCssClass(el,'disabled');
							else
								removeCssClass(el,'disabled');
							if(check.checked)
								el.value = '';
						})
				}
			</script>
<?php
		return ob_get_clean();
	}

	function getDynamicJSBehaviours()
	{
		ob_start();
		?>
				ifHandler.watchFormOnChange("interface_form", function(){ $("ajaxstatus").innerHTML = "<?php echo $this->langIf->getText('form details have been changed but are not saved...'); ?>";})

				/**
				 * Invoice
				 */
				function toggleFields()
				{
					var invoice = new Array('invoice[name]','invoice_address[country_id]',
						'invoice_address[address_street_name]','invoice_address[city_name]',
						'invoice_address[address_zipcode]');
					var check = document.getElementsByName('IsSameAsCRSO')[0];

					invoice.forEach(function(element, index, array)
						{
							el = document.getElementsByName(element)[0];
							el.disabled = check.checked?'disabled':'';

							if (check.checked)
								FormBehaviour.prototype.removeBehaviour(el, 'required');
							else
								FormBehaviour.prototype.setBehaviour(el, 'required');

							if(check.checked)
								addCssClass(el,'disabled');
							else
								removeCssClass(el,'disabled');
							if(check.checked)
								el.value = '';
						})
				}

				<?php
				if($this->IsSameAsCRSO)
				{
				?>
				if($('IsSameAsCRSO'))
					toggleFields();
				<?php
				}
				?>

		<?php
		return ob_get_clean();
	}

}

class InterfaceCRSORegistrationItem extends InterfaceItem
{
	/**#@+
	 * Callback functions made from InterfaceItem
	 */
	/**
	 *
	 */
	function getItemValue()
	{
		if(isset($_REQUEST['crso']['id']))
		{
			if(!isset($this->parent->parent->userInfo))
			{
				$this->hiddenFields = array(array('name'=>'crso[id]','value'=>$_REQUEST['crso']['id']));

				$crso = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app,$_REQUEST['crso']['id'])));
				$crso->set();
				$crso->relation->set();
				$crso->relation->address->set();
				$crso->relation->customer_contact->set();
				$ri = new RelationInvoice($this->db_app,array('relation'=>new Relation($this->db_app,$_REQUEST['crso']['id'])));
				$ri->set();
				$ri->address->set();
				$ri->info_person->set();

				$invoiceAddr = '';
				if($crso->relation->address->address_id==$ri->address->address_id)
					$this->parent->parent->IsSameAsCRSO = 1;
				else
					$invoiceAddr = $ri->address;
				$this->hiddenFields[] = array('name'=>'address[address_id]','value'=>$crso->relation->address->address_id);
				$this->parent->parent->address = (array) $crso->relation->address;
				$this->hiddenFields[] = array('name'=>'contact[id]','value'=>$crso->relation->customer_contact->id);
				$this->parent->parent->contact = (array) $crso->relation->customer_contact;
				$this->hiddenFields[] = array('name'=>'relation[id]','value'=>$crso->relation->id);
				$this->parent->parent->relation = (array) $crso->relation;
				$this->parent->parent->crso = (array) $crso;
				$this->hiddenFields[] = array('name'=>'invoice_address[address_id]','value'=>$ri->address->address_id);
				$this->parent->parent->invoice_address = (array) $invoiceAddr;
				$this->hiddenFields[] = array('name'=>'invoice_person[id]','value'=>$ri->info_person->id);
				$this->parent->parent->invoice_person = (array) $ri->info_person;

				setPageTitlePostfix('('.$crso->relation->name.')');

				$this->parent->parent->userInfo = true;
			}
			$name = str_replace('[','[\'',str_replace(']','\']',$this->name));
			eval('$this->properties[\'value\'] = isset($this->parent->parent->'.$name.')?$this->parent->parent->'.$name.':\'\';');
			if($this->type_id==IF_ITEM_CHECKBOX&& $this->properties['value'])
				$this->properties['checked'] = 'checked';
			if($this->name == 'relation[vat_no]' && isset($this->parent->parent->relation['vat_no']))
			{
				$this->properties['readonly'] = 'readonly';
				$this->properties['class'] = 'disabled';
			}
		}
	}
	function getPhoneFaxValue()
	{
		$this->getItemValue();
		if(!isset($this->properties['value'])||$this->properties['value'] == '')
			$this->properties['value'] = '+';
	}
	function getIsSameAsCRSO()
	{
		$this->getItemValue();
		$this->properties['id'] = 'IsSameAsCRSO';
		$this->properties['onclick'] = 'toggleFields();';
	}
	/**
	 *
	 */
	function getCountrySelect()
	{
		require_once(ELC_CLASSES.'Country.php');
		$country = new Country($this->db_app);
		$checkVar = ($this->name=='invoice_address[country_id]')?'invoice_address':'address';
		$q = $country->select();
		$this->options = array(array('text'=>'- '.$this->langIf->getText('Make Country Selection').' -','value'=>''));

		if($this->name=='address[country_id]'&&isset($this->parent->parent->address['country_id']))
		{
			$country->set($this->parent->parent->address['country_id']);
			if(!$country->validateUpdate())
			{
				$this->properties['disabled'] = 'disabled';
				$this->properties['class'] = 'disabled';
				$this->hiddenFields[] = array('name'=>'address[country_id]','value'=>$this->parent->parent->address['country_id']);
			}
		}

		$i = 1;
		while($r = $q->fetch())
		{
			$this->options[$i] = array('text'=>$r->NAME,'value'=>$r->COUNTRY_ID);
			if(isset($this->parent->parent->{$checkVar}['country_id']) && ($this->parent->parent->{$checkVar}['country_id']==$r->COUNTRY_ID))
				$this->options[$i]['selected'] = 'selected';
			$i++;
		}
	}
	/**
	 *
	 */
	function getButtons()
	{
		if(!$this->if_obj->ajaxUsage)
		{
			return $this->_getButtonsNoAjax();
		}

		$this->_addCustomButton(1, "hidden", array(
							'name' => 'ajaxUsage'
							));

		$this->_BarButtons = 0;


		switch ($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
				if(!$this->if_obj->_hideInputs)
				{
					$this->_CustomButtons[] = '<div id="ajaxstatus" style="color: red; display: inline;"></div>&nbsp;&nbsp;';
					$this->_addCustomButton($this->langIf->getText("View"), "button", array(
							'onclick' => "\$('interface_form_process').value = 'ajaxedit'; var params = []; params['ifId'] = ".$this->if_obj->id."; params['targetId'] = 'ifI_841'; jsbroker.callAndAttachRequestVars([['ajaxUsage', '1']".(isset($_REQUEST['crso']['id']) ? ",['crso[id]', '".$_REQUEST['crso']['id']."']" : "")."], 'loadInterface', params, ifHandler.buildInterface);"
							));
					$this->_addCustomButton($this->langIf->getText("Save"), "button", array(
							'onclick' => " \$('ajaxstatus').innerHTML = '".$this->langIf->getText('saving form details...')."'; \$('interface_form_process').value = 'save'; var params = []; params['ifId'] = ".$this->if_obj->id."; params['targetId'] = 'ifI_841'; jsbroker.callAndSubmitForm('interface_form', 'loadInterface', params, ifHandler.buildInterface);"
							));
					$this->_addCustomButton(1, "hidden", array(
							'name' => 'edit'
							));
					if(isset($_REQUEST['crso']['id']))
						$this->_addCustomButton($_REQUEST['crso']['id'], "hidden", array('name' => 'crso[id]'));
				}
				else
				{
					$this->_addCustomButton($this->langIf->getText("Edit"), "button", array(
						'onclick' => "\$('interface_form_process').value = 'ajaxedit'; var params = []; params['ifId'] = ".$this->if_obj->id."; params['targetId'] = 'ifI_841'; jsbroker.callAndAttachRequestVars([['ajaxUsage', '1'], ['edit', '1']".(isset($_REQUEST['crso']['id']) ? ",['crso[id]', '".$_REQUEST['crso']['id']."']" : "")."], 'loadInterface', params, ifHandler.buildInterface);"
						));
					if(isset($_REQUEST['crso']['id']))
						$this->_addCustomButton($_REQUEST['crso']['id'], "hidden", array('name' => 'crso[id]'));
				}
				break;
		}

	}

	function _getButtonsNoAjax()
	{
		$this->text = "";
			if(isset($_POST['process'])&& $_POST['process'] == 'check')
			{
				$this->_BarButtons = 0;
				$this->text = $this->langIf->getText('Are above settings correct?').'<br />';
				$this->_addCustomButton($this->langIf->getText('No, I would like to make changes'),'button', array('onclick'=>'document.getElementsByName(\'process\')[0].value=\'\'; document.getElementById(\'interface_form\').submit();','name'=>'signup'));
				$this->_addCustomButton($this->langIf->getText('Yes, I am ready to continue'),'submit', array('onsubmit'=>'','name'=>'signup'));
			}
			else{
				$this->_BarButtons = BUTTON_SAVE+BUTTON_CANCEL;
				$this->_ButtonOptions[BUTTON_CANCEL] = array('onclick'=>"document.location.href='InterfaceCRSOOverview.php';");
			}
	}

	/**
	 *
	 */
	function getLanguageSelect()
	{
		require_once(IWARE.'nLanguage.php');
		$lang = new nLanguage($this->db_app);
		$q = $lang->get();
		$this->options = array(array('value'=>'','text'=>'- '.$this->langIf->getText('Select language').' -'));
		$i = 1;
		while($r = $q->fetch())
		{
			$this->options[$i] = array('text'=>$r->LANGUAGE,'value'=>$r->LANGUAGE_ID);
			if(!isset($this->parent->parent->userInfo))
			{
				if($r->LANGUAGE_ID == $_SESSION['session_user_language_id'])
					$this->options[$i]['selected'] = 'selected';
			}
			elseif(isset($this->parent->parent->crso['default_language_id'])&& $this->parent->parent->crso['default_language_id']==$r->LANGUAGE_ID)
				$this->options[$i]['selected'] = 'selected';
			$i++;
		}
	}

	function getDirectDebit()
	{
		$d = isset($this->parent->parent->relation)?$this->parent->parent->relation:false;


		$this->options = array(
			0=>array('value'=>'yes', 'text'=>$this->langIf->getText('Yes'), 'onclick'=>'toggleDD(false);'),
			1=>array('value'=>'no', 'text'=>$this->langIf->getText('No'),'onclick'=>'toggleDD(true);'),
		);
		if(isset($_POST['HasDirectDebit']))
		{
			if($_POST['HasDirectDebit']=='no')
				$this->options[1]['checked'] = 'checked';
			else
				$this->options[0]['checked'] = 'checked';
		}
		elseif($d&&(empty($d->dd_bank_account)&&empty($d->dd_iban_no)))
			$this->options[1]['checked'] = 'checked';
		else
			$this->options[0]['checked'] = 'checked';
	}

	function getSpacer()
	{
		$this->twoCols = true;
	}
	function getRightColText()
	{
		$this->twoCols = true;
		$this->text = $this->langIf->getText('Invoice User Information');
	}

	/**#@-*/
}
