<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'Product.php');

define('BLOCK_CRSO_PRODUCT_CATEGORIES',390);

/**
 *
 * @package ELC
 * @subpackage Interface
 * @since 18-04-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceCRSOSettings extends InterfaceFactory
{
	/**
	 * @var boolean Whether this interface is being used in an ajax context
	 */
	var $ajaxUsage = false;

    /**
     * interface[id] = 103
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceCRSOSettings(&$db, &$db_app, &$langIf, &$settings)
    {
    	//check ajax usage
		$this->ajaxUsage = isset($_REQUEST['ajaxUsage']) && (int)$_REQUEST['ajaxUsage'] ? true : false;
        $this->InterfaceFactory($db, $db_app, $langIf, $settings);
        $this->FirstColWidth = 40;
        $this->delimiter = '<br />';

		require_once(ELC_CLASSES.'CRSO.php');
		require_once(ELC_CLASSES.'CRSOProduct.php');
		$this->model_crso = new CRSO($this->db_app);
		$this->model_crso->set((int)$_REQUEST['crso']['id']);
		$this->model_crso->relation->set();
		$this->model['crsoproduct'] = new CRSOProduct($this->db_app,array('crso'=>$this->model_crso));
		$this->model['crsoproduct']->product = new Product($this->db_app,$_SESSION['selected_product_id']);
		if(!$this->model['crsoproduct']->set())
		{
			// Log this shit
			throw InterfacePermissionException('This CRSO is not registered for the selected product.');
		}


		require_once(ELC_CLASSES.'Country.php');
		$this->model['country'] = new Country($this->db_app,$this->model_crso->getCountry(),true);

		$this->staleblockparams = '&crso[id]='.$this->model_crso->relation->id;
		$this->fb->addPlugin('elc_forfeit_start_date', 'FormBehaviourPluginElcForfeitStartDate');
		$this->fb->addPlugin('elc_refund_start_date', 'FormBehaviourPluginElcRefundStartDate');

		setPageTitlePostfix('('.$this->model_crso->relation->name.')');
    }

    function init($ifId)
    {
    	$rv = parent::init($ifId);

    	 if(!$this->model['country']->has(HasDeclByCategory,$_SESSION['selected_product_id']))
    	 	unset($this->blocks[BLOCK_CRSO_PRODUCT_CATEGORIES]);

    	return $rv;
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
		if ( isset($_POST['process']) )
		{
			if($_POST['process'] == 'product_category')
			{
				require_once(ELC_CLASSES.'ProductCategory.php');
				$pc = new ProductCategory($this->db_app, $_REQUEST['product_category']);
				$pc->product = new Product($this->db_app, $_SESSION['selected_product_id']); // FIXME: product_id
				$pc->crso = $this->model_crso;

				if (isset($_REQUEST['product_category']['id']))
					$pc->update();
				else
					$pc->insert();

                                log_elc_event(sprintf('CRSO product category [%s] %s',
                                        $pc->name,
                                        isset($_REQUEST['product_category']['id']) ? 'updated' : 'created'),
                                    $_SESSION['session_user_id'],
                                    $pc->product->id,
                                    $pc->crso->relation->id,
                                    null
                                );
				header(sprintf('location: %s?crso[id]=%s', 'InterfaceCRSOSettings.php', $pc->crso->relation->id));
				exit();
				return;
			}
			// Set the clearance level on the CRSO object
			if(isset($_POST['clearance']))
				$this->model_crso->setCRSOAvailableClearance($_POST['clearance']);

			// We need to remove refund_start_date from fb checks if the same as in db
			// This post variable is NOT set if the checkbox for Refund Start Date same as Obligation Date is checked.
			if(isset($_POST['crso']['refund_start_date'])&&
				reindex_date($_POST['crso']['refund_start_date'], 'DD-MM-YYYY', 'YYYY-MM-DD') == $this->model['crsoproduct']->refund_start_date)
			{
				$this->fb->unregister('crso[refund_start_date]');
			}

			$this->processNestedInterfaces($thrownException);

	        $invalid = $this->langIf->getText('Please correct the form errors');
	        if(!$this->fb->validate())
	        {
	        	//debug('main interface false form');
				$this->init($this->id);
	        	$thrownException .= $invalid;
	        	return;
	        }

			// start db transaction
			$this->db_app->begin();

			if ( isset($_POST['glaccounts']) )
			{
				//HANDLE G/L Account No.'s
				require_once(ELC_CLASSES.'DeclarationType.php');
				if ( $_POST['process'] == 'save' )
				{
					foreach ( $_POST['glaccounts'] as $typeid=>$account )
					{
						$this->model_crso->setGLAccountForDeclarationType(
								$account, new DeclarationType($this->db_app, constant($typeid))
						);
					}
				}
			}

			// Get the country setting for this CRSO Product
			$this->model['crsoproduct']->forfeit_day = $_POST['crso']['forfeit_day'];
			if($this->model['country']->has(HasCrsoOneExportDay,$_SESSION['selected_product_id']))
				$this->model['crsoproduct']->updateForfeitDayForAllProducts($_POST['crso']['forfeit_day'],$this->model['country']);

			// check if the crso[forfeit_start_date] is allowed to be submited (if it's manually enabled it will be submited)
			$strdate = $this->model['crsoproduct']->forfeit_start_date;
			if ($strdate == '' || $strdate == DB_NULL) { // date can be empty
				// it was allowed to be submited, so now we check it
				if (isset($_POST['crso']['forfeit_start_date'])) // item can be disabled
				{
					if (empty($_POST['crso']['forfeit_start_date'])) // item can be empty
						$this->model['crsoproduct']->forfeit_start_date = DB_NULL;
					else {
						$this->model['crsoproduct']->forfeit_start_date = reindex_date($_POST['crso']['forfeit_start_date'], 'DD-MM-YYYY', 'YYYY-MM-DD');
					}
				}
			} else {
				if (isset($_POST['crso']['forfeit_start_date'])) {
					// this field shouldn't have been submited, but it was... so that means someone tried to
					// enable it to change it's value ? Maybe log the attempt or something ?
					info('Somebody tried to change CRSO forfeit start date');
				}
			}
			// end of forfeit_start_day check

			// minimal obligation date
			$this->model['crsoproduct']->minimal_obligation_date = reindex_date($_POST['crso']['minimal_obligation_date'], 'DD-MM-YYYY', 'YYYY-MM-DD');


			//refund start date
			if (isset($_POST['crso']['refund_start_date'])) // item can be disabled
			{
				if (empty($_POST['crso']['refund_start_date'])) // item can be empty
					$this->model['crsoproduct']->refund_start_date = DB_NULL;
				else {
					$this->model['crsoproduct']->refund_start_date = reindex_date($_POST['crso']['refund_start_date'], 'DD-MM-YYYY', 'YYYY-MM-DD');
				}
			}

			if(isset($_POST['crso']['refund_start_date_same_as_obligation']))
			{
				$this->model['crsoproduct']->refund_start_date = reindex_date($_POST['crso']['minimal_obligation_date'], 'DD-MM-YYYY', 'YYYY-MM-DD');
			}

			$this->model['crsoproduct']->refund_export_day = $_POST['crso']['refund_export_day'];
			$this->model['crsoproduct']->declaration_reminder_day = $_POST['crso']['declaration_reminder_day'];

			$this->model_crso->update();
			$this->model['crsoproduct']->update();

                        log_elc_event('CRSO settings updated',
                            $_SESSION['session_user_id'],
                            $this->model['crsoproduct']->product->id,
                            $this->model['crsoproduct']->crso->relation->id,
                            null
                        );

			// commit transaction
			$this->db_app->commit();

			if(!$this->ajaxUsage)
			{
				header(sprintf('Location: %s?crso[id]=%s', 'InterfaceCRSOSettings.php', $this->model_crso->relation->id));
				exit();
			}
			else
			{
				unset($this->model_glaccounts);
				$this->init($this->id);
			}
		}
    }

	function getDynamicJSBehaviours()
	{
		ob_start();
		?>
				toggleRefund = function()
				{
					refundStartCheck = $('refund_start_date_same_as_obligation');
					if(refundStartCheck.checked)
					{
						$("refund_start_date").value = "";
						$("refund_start_date").disabled = "disabled";
					}
					else
					{
						$("refund_start_date").disabled = "";
					}

				};

				$('refund_start_date_same_as_obligation').onclick = toggleRefund;

				ifHandler.watchFormOnChange("interface_form", function(){ $("ajaxstatus").innerHTML = "<?php echo $this->langIf->getText('form details have been changed but are not saved...'); ?>";})
		<?php
		return ob_get_clean();
	}

}
/**
 * @package ELC
 * @subpackage Interface
 * @since 18-04-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceCRSOSettingsItem extends InterfaceItem
{

	function setCRSOId()
	{
		$this->text = '<input type="hidden" name="crso[id]" value="'.$this->if_obj->model_crso->relation->id.'" />';
	}
    /**
     * item[829] = glaccounts[DT_MONTHLY_DECLARATION]
     * item[830] = glaccounts[DT_RESUBMISSION]
     * item[831] = glaccounts[DT_NEGATIVE_DECLARATION]
     * item[832] = glaccounts[DT_FORFEIT_DECLARATION]
     * item[833] = glaccounts[DT_LATE_DECLARATION]
     * item[834] = glaccounts[DT_NEGATIVE_FORFEIT_DECLARATION]
     * item[835] = glaccounts[DT_GUARANTEE_DOCUMENT_150]
     * item[836] = glaccounts[DT_GUARANTEE_DOCUMENT_200]
     * item[837] = glaccounts[DT_GUARANTEE_REIMBURSEMENT_DOCUMENT]
     * item[1072] = glaccounts[DT_REFUND_REQUEST]
     */
    function getGLAccounts()
    {
		if ( ! isset($this->if_obj->model_glaccounts) ) //do once and only if GLAccounts block is open
		{
			require_once(ELC_CLASSES.'DeclarationType.php');
			DeclarationType::defineConstants($this->db_app);
			$this->if_obj->model_glaccounts = $this->if_obj->model_crso->getGLAccount();
		}

		preg_match('/glaccounts\[([^\]]+)]/', $this->name, $match);
		if ( isset($this->if_obj->model_glaccounts[constant($match[1])]) )
			$this->properties['value'] = $this->if_obj->model_glaccounts[constant($match[1])];
    }

    /**
     * item[1017] = 139
     */
    function getAdminFee()
    {
    	$this->settings['classPrefix'] = 'InterfaceCRSOAdminFee';
    	$this->settings['appPath'] = 'elc/operator/';
    }

    /**
     * item[1018] = 139
     */
    function getDeclarationInterest()
    {
    	$this->settings['classPrefix'] = 'InterfaceCRSODeclarationInterest';
    	$this->settings['appPath'] = 'elc/operator/';
    }

	/**
	 * Declaration Settings
	 */
	function getForfeitDay()
	{
		if (isset($this->if_obj->model['crsoproduct']->forfeit_day))
			$this->properties['value'] = $this->if_obj->model['crsoproduct']->forfeit_day;
		else
			$this->properties['value'] = 11;
	}

	function getForfeitStartDate()
	{
		if (isset($this->if_obj->model['crsoproduct']))
		{
			$strdate = $this->if_obj->model['crsoproduct']->forfeit_start_date;

			if ($strdate == '' || $strdate == DB_NULL) // date can be empty
				return;
			$date = createDate($strdate, 'YYYY-MM-DD');

		     //return int 0 if the dates are equal, -1 if d1 is before d2, 1 if d1 is after d2
			if (IWDateTime::compare(currentDate(), $date))
				$this->properties['disabled'] = 'disabled';

			$this->properties['value'] = reindex_date($strdate, 'YYYY-MM-DD', 'DD-MM-YYYY');
		}
	}

	function getRefundStartDate()
	{
		if (isset($this->if_obj->model['crsoproduct']))
		{
			$this->properties['id'] = "refund_start_date";

			$strdate = $this->if_obj->model['crsoproduct']->refund_start_date;
			if ($strdate == '' || $strdate == DB_NULL) // date can be empty
				return;

			$date = createDate($strdate, 'YYYY-MM-DD');

		     //return int 0 if the dates are equal, -1 if d1 is before d2, 1 if d1 is after d2
			if (IWDateTime::compare(createDate($this->if_obj->model['crsoproduct']->minimal_obligation_date, 'YYYY-MM-DD'), $date) == 0)
			{
				$this->properties['disabled'] = 'disabled';
				$this->properties['value'] = '';
			}
			else
			{
				$this->properties['value'] = reindex_date($strdate, 'YYYY-MM-DD', 'DD-MM-YYYY');
			}

		}
	}

	function getRefundStartSameAsObligation()
	{
		if (isset($this->if_obj->model['crsoproduct']))
		{
			$hasMinObligation = $this->if_obj->model['crsoproduct']->minimal_obligation_date != "" &&
								$this->if_obj->model['crsoproduct']->minimal_obligation_date != DB_NULL;
			$hasRefundStart = $this->if_obj->model['crsoproduct']->refund_start_date != "" &&
								$this->if_obj->model['crsoproduct']->refund_start_date != DB_NULL;

			if ($hasMinObligation &&
				$hasRefundStart &&
				IWDateTime::compare(
				createDate($this->if_obj->model['crsoproduct']->minimal_obligation_date, 'YYYY-MM-DD'),
				createDate($this->if_obj->model['crsoproduct']->refund_start_date, 'YYYY-MM-DD')) == 0)
			{
				$this->properties['checked'] = 'checked';
			}
			if(!$this->if_obj->ajaxUsage)
				$this->properties['onclick'] = 'toggleRefund(this);';
			else
				$this->properties['id'] = 'refund_start_date_same_as_obligation';
			$this->properties['value'] = 1;

			if(!$this->if_obj->ajaxUsage)
			{
				$this->script = '
							function toggleRefund(refundStartCheck)
							{
								if(refundStartCheck.checked)
								{
									$("refund_start_date").value = "";
									$("refund_start_date").disabled = "disabled";
								}
								else
								{
									$("refund_start_date").disabled = "";
								}

							}
						';
			}
		}
	}

	function getRefundExportDay()
	{
		if (!empty($this->if_obj->model['crsoproduct']->refund_export_day))
			$this->properties['value'] = $this->if_obj->model['crsoproduct']->refund_export_day;
		else
			$this->properties['value'] = '';
	}

	function getMinimalObligationDate()
	{
		if (isset($this->if_obj->model['crsoproduct']))
		{
			$strdate = $this->if_obj->model['crsoproduct']->minimal_obligation_date;
			if ($strdate == '' || $strdate == DB_NULL) // date can be empty
				return;

			$this->properties['value'] = reindex_date($strdate, 'YYYY-MM-DD', 'DD-MM-YYYY');
		}
	}

    function getButtons()
    {
    	if($this->if_obj->ajaxUsage)
    	{


    		$this->_addCustomButton(1, "hidden", array(
							'name' => 'ajaxUsage'
							));

			$this->_BarButtons = 0;
			$this->_CustomButtons[] = '<div id="ajaxstatus" style="color: red; display: inline;"></div>&nbsp;&nbsp;';
			$this->_addCustomButton($this->langIf->getText("Save"), "button", array(
							'onclick' => " \$('ajaxstatus').innerHTML = '".$this->langIf->getText('saving form details...')."'; \$('interface_form_process').value = 'save'; var params = []; params['ifId'] = ".$this->if_obj->id."; params['targetId'] = 'ifI_841'; jsbroker.callAndSubmitForm('interface_form', 'loadInterface', params, ifHandler.buildInterface);"
							));
    	}
        else{
    		$this->_BarButtons = BUTTON_CANCEL+BUTTON_SAVE;
    		$this->_ButtonOptions[BUTTON_CANCEL] = array('onclick'=>"document.location.href='InterfaceCRSOOverview.php';");
        }
    }

    function getDecReminderDay()
    {
    	if (!empty($this->if_obj->model['crsoproduct']->declaration_reminder_day))
			$this->properties['value'] = $this->if_obj->model['crsoproduct']->declaration_reminder_day;
		else
			$this->properties['value'] = '';
    }

    function getFiller()
    {
    	$this->twoCols = 2;
    	$this->text = '&nbsp;';
    }

    /**
     * item[1147] =
     */
    function getProductCategoryOverview()
    {
    	require_once(ELC_CLASSES.'ProductCategory.php');
	    if(!$this->if_obj->model['country']->has(HasDeclByCategory,$_SESSION['selected_product_id']))
			return $this->text = '';

		$pc = new ProductCategory($this->db_app);
		$q = $pc->selectOverview(array($_SESSION['selected_product_id']), $this->if_obj->model_crso->relation->id); // FIXME: product_id

		$t = new TableFactory($q, $this->langIf);
		$t->setValueProcessor(array('PRODUCT_CATEGORY_ID', 'CURRENT_FEE'),
			array($this, '_tfProductCategoryValueProcessor'));

		$t->setEmptyText($this->langIf->getText('No product categories found'));
		$this->text = $t->getXHTMLTable('Professional', 2);
    }

	function _tfProductCategoryValueProcessor($args)
	{
		switch ($args['head'])
		{
			case 'PRODUCT_CATEGORY_ID':
				// FIXME for editing.
				//return $args['value'];
				return sprintf('<a href="%s?crso[id]=%s&product_category[id]=%s">%s</a>',
					'InterfaceCRSOSettings.php' , $this->if_obj->model_crso->relation->id, $args['value'], $args['value']);
			break;

			case 'CURRENT_FEE':
				require_once(ELC_CLASSES.'ProductFee.php');
				$pc_fee = new ProductCategoryFee($this->db_app);
				$fee = $pc_fee->getFee($args['row']['PRODUCT_CATEGORY_ID'], $this->if_obj->model_crso->relation->id);

				if (!$fee)
					return $this->langIf->getText('No Product Fee set');

				return $fee;
			break;
		}
	}

    /**
     * item[1148] = product_category[name]
     */
    function getProductCategory()
    {
    	if(!isset($_REQUEST['product_category']['id']))
    		return;

    	$pc = new ProductCategory($this->db_app,$_REQUEST['product_category']['id'],true);

    	$this->hiddenFields = array(array('name'=>'product_category[id]','value'=>$_REQUEST['product_category']['id']));
    	$this->properties['value'] = $pc->name;
    }
    /**
     * Option to set available clearance levels on the CRSO object.
     */
    function getClearanceLevels() {

    	$clearance_map = CRSO::getClearanceLevels();
    	$ds = array();

    	// Loop the clearance map.
    	foreach($clearance_map as $clearance) {
	   		$c = constant($clearance);

	   		// If the target object has clearance level set, mark as checked.
	   		$checked = (($this->if_obj->model_crso->available_clearance_levels & $c) == $c) ? ' checked="checked"' : '';

       		// Make a checkbox input.
       		$ds[] = array('Clearance level'=>
			sprintf('<input id="clearance_%s" type="checkbox" name="clearance[]" value="%s" %s /> %s',
				$c,
       				$c,
       				$checked,
       				$this->langIf->getText($clearance)
       			),
       			'Additional Information'=>nl2br($this->langIf->getText('Information for '.$clearance)));
    	}

    	// Create a tablefactory object to show the inputs neatly.
    	$tf = new TableFactory($ds, $this->langIf);
    	$tf->decapitate();
    	$this->text = nl2br($this->langIf->getText('Clearance Level introduction text')).'<br />'.$tf->getXHTMLTable('Professional', 2);
    }

    /**
     * item[1149] =
     */
    function getProductCategoryButtons()
    {
    	$this->_BarButtons = 0;
    	$props = array('onclick'=>'submitProductCategory();');
    	$addProductCat = $this->langIf->getText('Add Product Category');
    	$updateProductCat =$this->langIf->getText('Update Product Category');
    	$txt = isset($_REQUEST['product_category']['id'])?$updateProductCat:$addProductCat;

    	$this->_addCustomButton($txt,'button',$props);

    	$this->script = sprintf("
			function submitProductCategory()
			{
				// Question box, cause of submit, other things might be lost.
				MessageBox.prototype.Question('%s','%s',function(){
					$$('process').value = 'product_category';
					$('interface_form').submit();
				})
			}
			",
			$this->langIf->getText('MessageBox: Warning'),
			$this->langIf->getText('MessageBox Content: Other changes except Category might be undone')
			);
    }


/**
 * CALLBACK[setCRSOId]
 * item[839] = {
 *     [handle] = Item: CRSO Id
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getGLAccounts]
 * item[829] = {
 *     [handle] = DT_MONTHLY_DECLARATION
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[830] = {
 *     [handle] = DT_RESUBMISSION
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[831] = {
 *     [handle] = DT_NEGATIVE_DECLARATION
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[832] = {
 *     [handle] = DT_FORFEIT_DECLARATION
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[833] = {
 *     [handle] = DT_LATE_DECLARATION
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[834] = {
 *     [handle] = DT_NEGATIVE_FORFEIT_DECLARATION
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[835] = {
 *     [handle] = DT_GUARANTEE_DOCUMENT_150
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[836] = {
 *     [handle] = DT_GUARANTEE_DOCUMENT_200
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[837] = {
 *     [handle] = DT_GUARANTEE_REIMBURSEMENT_DOCUMENT
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getAdminFee]
 * item[1017] = {
 *     [handle] = Item: AdminFee
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getDeclarationInterest]
 * item[1018] = {
 *     [handle] = Item: DeclarationInterest
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[838] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */

}

