<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'CRSOSupportDocument.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'AccountantCRSO.php');
require_once(IWARE.'Document.php');
/**
 *
 * @package
 * @subpackage Interface
 * @since 02-07-2007
 * @author Marten Koedam <marten@dalines.net>
 * @copyright 2007 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceCRSOSupportDocuments extends InterfaceFactory
{
    /**
     * interface[id] = 173
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceCRSOSupportDocuments(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        $this->model['csd'] = new CRSOSupportDocument($this->db_app);

    	switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
				$crso = new CRSO($this->db_app);
				$q = $crso->selectCRSOs();
				break;
			case GROUP_ACCOUNTANT:
		    	$accountant_crso = new AccountantCRSO($this->db_app);
				$q = $accountant_crso->getAccountantCRSOs($_SESSION['accountant_relation_id'],$_SESSION['selected_product_id']);
				break;
			default:
				throw new InterfacePermissionException('No Permission');
		}
		$this->model['crsos'] = array();
		while($r = $q->fetch()) {
			$this->model['crsos'][$r->CRSO_RELATION_ID] = $r->NAME;
		}


        $this->script = "
			function saveDocuments()
			{
				$$('process').value = 'saveAll';
				$('interface_form').submit();
			}

			function deleteDocuments()
			{
				$$('process').value = 'delete';
				$('interface_form').submit();
			}
			";
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        if(!isset($_POST['process']))
        	return;

        if($_POST['process'] == 'save')
        {
        	if(!$this->fb->validate())
				return $thrownException .= IWRegistry::registry('langIf')->getText('Please correct the form errors');
        	$this->model['csd']->setProperties($_POST['csd']);
        	$this->model['csd']->crso = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app,$_POST['crso']['id'])));

			$item = new InterfaceItem($this->db,$this->db_app,$this->langIf);
		  	$this->model['csd']->document = $item->getDocument('csd[document][id]',1339,true);
		  	$this->model['csd']->document->makePermanent();

			$this->model['csd']->insert();
			header('Location: InterfaceCRSOSupportDocuments.php');
			exit();
        }

        if($_POST['process'] == 'saveAll')
        {
        	foreach($_POST['csds'] as $crsoid => $document)
        	{
        		$this->model['csd']->crso = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app,$crsoid)));
        		foreach($document as $docid => $title)
        		{
        			$this->model['csd']->document = new Document($this->db_app,$docid);
        			$this->model['csd']->title = $title;
        			$this->model['csd']->update();
        		}
        	}
        	header('Location: InterfaceCRSOSupportDocuments.php');
			exit();
        }

        if($_POST['process'] == 'delete')
        {
        	foreach($_POST['remove'] as $crsoid => $document)
        		foreach($document as $docid)
        			$this->model['csd']->delete($crsoid,$docid);
        	header('Location: InterfaceCRSOSupportDocuments.php');
			exit();
        }
    }
}
/**
 * @package
 * @subpackage Interface
 * @since 02-07-2007
 * @copyright 2007 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceCRSOSupportDocumentsItem extends InterfaceItem
{

    /**
     * item[1337] = crso[id]
     */
    function getCRSOSelect()
    {
    	if(count($this->if_obj->model['crsos']) == 1){
    		$ids = array_keys($this->if_obj->model['crsos']);
			$_POST['crso']['id'] = $ids[0];
		}

		$this->options = array(array('text'=>'- '.$this->langIf->getText('CRSO Select').' -','value'=>''));
		$crso_id = isset($_POST['crso']['id'])?$_POST['crso']['id']:false;
		foreach($this->if_obj->model['crsos'] as $id => $name)
		{
			$selected = ($id == $crso_id)?'selected':false;
			$this->options[] = array('text'=>$name,'value'=>$id,'selected'=>$selected);
		}

//		$this->properties['onchange'] = "";
    }

    /**
     * item[1338] = csd[title]
     */
    function getDocumentTitle()
    {

    }

    /**
     * item[1339] = csd[document][id]
     */
    function getDocument()
    {

    }

    /**
     * item[1340] =
     */
    function getButtons()
    {
    	$this->_BarButtons = BUTTON_SAVE;
    }

    /**
     * item[1341] =
     */
    function getSupportDocumentOverview()
    {
    	$ds = $this->if_obj->model['csd']->getOverview(array_keys($this->if_obj->model['crsos']));
    	$tf = new TableFactory($ds,IWRegistry::registry('langIf'));
    	$tf->setCaption('CRSO Support Documents');
    	$tf->setValueProcessor(TF_ALL,array($this,'tfValueProcessor'));
    	$tf->hideColumns(array('CRSO_RELATION_ID','DOCUMENT_ID'));
    	$this->text = $tf->getXHTMLTable('Professional');
    }

    function tfValueProcessor($args)
    {
    	switch($args['head'])
    	{
    		case 'Remove':
    			return sprintf('<input type="checkbox" name="remove[%s][%s]" value="%s" />',
    				$args['row']['CRSO_RELATION_ID'],
    				$args['row']['DOCUMENT_ID'],
    				$args['row']['DOCUMENT_ID']
    				);
    		break;
    		case 'DocumentTitle':
    			return sprintf('<input type="text" size="60" behaviour="comment" name="csds[%s][%s]" value="%s" />',
    				$args['row']['CRSO_RELATION_ID'],
    				$args['row']['DOCUMENT_ID'],
    				$args['value']);
    		break;
    	}
    	return $args['value'];
    }

    /**
     * item[1344] =
     */
    function getDeleteButtons()
    {
    	$this->_BarButtons = BUTTON_DELETE;

    	$this->_ButtonOptions[BUTTON_DELETE]['onclick']='deleteDocuments()';
    	$this->_addCustomButton(IWRegistry::registry('langIf')->getText('Save'),'button',
    		array('onclick'=>'saveDocuments()'));
    }
}

/**
 * CALLBACK[getCRSOSelect]
 * item[1337] = {
 *     [handle] = Item: CRSO Selection
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getDocumentTitle]
 * item[1338] = {
 *     [handle] = Item: Document Title
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getDocument]
 * item[1339] = {
 *     [handle] = Item: Support Document
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[1340] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getSupportDocumentOverview]
 * item[1341] = {
 *     [handle] = Item: Support Document Overview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>