<?php
/**
 *
 */
require_once(INTERFACE_OBJ."InterfaceFactory.php");
/**
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @since 22-01-2005
 * @version $Id$
 */
class InterfaceCRSOUserOverview extends InterfaceFactory
{
	function InterfaceCRSOUserOverview($db, $db_app, $langIf)
	{
		require_once(ELC_CLASSES.'CRSO.php');
		// CHECK IF THIS CRSO HAS ACCEPTED SLA, otherwise they cannot add new users for this CRSO
		// Sysadmin can __always__ add new Users to a CRSO even if the SLA hasnt been accepted yet
		// Only CRSO with right Clearance or Sysadmin can add new Users.
		if($_SESSION['session_group_group_id']==GROUP_CRSO)
		{
			if(!isset($_SESSION['crso_relation_id']))
				throw new InterfacePermissionException('Not authorized to view interface (1)');

			require_once(ELC_CLASSES.'RelationUser.php');
			$ru = new RelationUser($db_app);
			$ru->set($_SESSION['session_user_id'], $_SESSION['crso_relation_id']);
			if(!$ru->hasClearance(CRSOClearanceUserManagement))
				throw new InterfacePermissionException('Not authorized to view interface (2)');
			$crso_id = $_SESSION['crso_relation_id'];
		} else {
			$crso_id = $_REQUEST['crso']['id'];
		}

		$this->model['crso'] = new CRSO($db_app, array('relation'=>new Relation($db_app, $crso_id)));

		$this->model['crso']->relation->set();
		setPageTitlePostfix('('.$this->model['crso']->relation->name.')');

		$this->FirstColWidth = 25;
		$this->delimiter = '<br />';
		parent::InterfaceFactory($db, $db_app, $langIf);
	}

	function processInput(&$thrownException)
	{
		if(isset($_POST['process']))
		{
			if($_POST['process']=='delete')
			{
				require_once(ELC_CLASSES.'User.php');
				if(isset($_POST['crso_user']))
				{
					// CRSOUsers can always be deleted ??
					foreach($_POST['crso_user'] as $user_id)
					{
						if($user_id == $_SESSION['session_user_id'])
						{
							$thrownException .= $this->langIf->getText('Cannot delete yourself');
							$this->init($this->id);
							return false;
						}
						$user = new ELCUser($this->db_app,$user_id);
						$user->delete();
					}
				}
			}
			$this->init($this->id);
		}
	}
}
/**
 * @package ELC
 * @subpackage Interface
 */
class InterfaceCRSOUserOverviewItem extends InterfaceItem
{
	function getOverview()
	{
		// This is not an Sysadmin who should be able to view this page.
		// Check to see wether or not it is a CRSO
		$hideCols = array('CHECKBOX');
		if($_SESSION['session_group_group_id']==GROUP_CRSO)
			$hideCols[] = 'TRANSLATE';

		require_once(IWARE.'TableFactoryColumnPlugin.php');
		$ds = $this->if_obj->model['crso']->selectUserOverview($this->if_obj->model['crso']->relation->id);

		$tf = new TableFactoryColumnPlugin(false,$this->langIf,$this->db,$this->db_app);
		$tf->setData($ds, 'CRSOUserOverview',false,true);
		$tf->setSortingFunctions(
			array(
				'USER_ID'=>"'Int'",
				'USER_ACTIVE'=>"'String'",
				'IS_ADMIN'=>"'String'",
				'CONTACT_NAME'=>"'String'",
				'USER_NAME'=>"'String'",
				'TRANSLATE'=>"'String'"
			));
		$help = ($this->parent->title != '') ? getHelp($this->langIf, $this->parent->handle, array('title'=>$this->parent->handle, 'top'=>'0px')) : '';
		$tf->setCaption($help.'<img src="/'.SITE_PATH.'images/block_arrow.png" /> '.$this->langIf->getText('CRSO User Overview'), false);
		$tf->hideColumns($hideCols);
		$tf->setValueProcessor(TF_ALL, array($this, 'tfValueProcessor'));
		$tf->setEmptyText($this->langIf->getText('No users found') );

		$this->text = $tf->getXHTMLTable('ProfessionalWhite',2);
	}

	function tfValueProcessor($args) {
		switch($args['head']) {
			case 'CLEARANCE_LEVELS':
				// Admin users already have all clearance!
				if($args['row']['IS_ADMIN']) {
					return '';
				}
				// Cant change own clearance.
				if($args['row']['USER_ID']==$_SESSION['session_user_id'])
					return '';
				$params = $path = '';
				if($_SESSION['session_group_group_id']==GROUP_SYSADMIN) {
					$params = sprintf('&crso_id=%s', $this->if_obj->model['crso']->relation->id);
				}

				return sprintf('<a href="InterfaceUserClearanceLevels.php?user_id=%s%s">%s</a>',
					$args['row']['USER_ID'],
					$params,
					$this->langIf->getText('Edit clearance')
				);
			break;
			case 'TRANSLATE':
				return sprintf(
					'<a id="user_languages_%s" href="InterfaceUserTranslator.php?user[id]=%s">%s</a>',
					$args['row']['USER_ID'],
					$args['row']['USER_ID'],
					$args['value']
				);
			break;
			case 'CHECKBOX':
				// Can never delete self!
				if($args['row']['USER_ID']==$_SESSION['session_user_id'])
					return;
				return sprintf(
					'<input type="checkbox" name="crso_user[]" value="%s" />',
					$args['row']['USER_ID']
				);
			break;
			case 'USER_ID':
				return sprintf(
					'<a href="InterfaceCRSOUserRegistration.php?crso[id]=%s&user[id]=%s">%s</a>',
					$this->if_obj->model['crso']->relation->id,
					$args['row']['USER_ID'],
					$args['value']
				);
			break;
			case 'USER_ACTIVE':
			case 'IS_ADMIN':
				$img = $args['value'] ? 'true' : 'false';
				return sprintf(
					'<img src="/%simages/i-w/%s.gif" />',
					SITE_PATH,
					$img
				);
			break;
		}

		return $args['value'];
	}

	function getButtons()
	{
		$this->hiddenFields = array(array('name'=>'crso[id]','value'=>$this->if_obj->model['crso']->relation->id));

		$this->_BarButtons = 0;
		$this->_addCustomButton($this->langIf->getText('Add new CRSO User'),'button',array('onclick'=>"document.location.href='InterfaceCRSOUserRegistration.php?crso[id]=".$this->if_obj->model['crso']->relation->id."';"));
	}
}

?>