<?php
require_once(INTERFACE_OBJ."InterfaceFactory.php");
/**
 *
 *
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @version $Id$
 */
class InterfaceCRSOUserRegistration extends InterfaceFactory
{
	function InterfaceCRSOUserRegistration($db, $db_app, $langIf)
	{
		// CHECK IF THIS CRSO HAS ACCEPTED SLA, otherwise they cannot add new users for this CRSO
		// Sysadmin can __always__ add new Users to a CRSO even if the SLA hasnt been accepted yet
		// Only CRSO with correct Clearance or Sysadmin can add new Users.
		if($_SESSION['session_group_group_id']==GROUP_CRSO)
		{
			if(!isset($_SESSION['crso_relation_id']))
				throw new InterfacePermissionException('Not authorized to view interface (1)');
			require_once(ELC_CLASSES.'RelationUser.php');
			$ru = new RelationUser($db_app);
			$ru->set($_SESSION['session_user_id'], $_SESSION['crso_relation_id']);
			if(!$ru->hasClearance(CRSOClearanceUserManagement))
				throw new InterfacePermissionException('Not authorized to view interface (2)');
		}
		$this->FirstColWidth = 25;
		$this->delimiter = '<br />';
		if(isset($_POST['process']))
			$this->_setValues = true;
		parent::InterfaceFactory($db, $db_app, $langIf);

		require_once(IWARE.'InfoPerson.php');
		require_once(IWARE.'group.php');
		require_once(IWARE.'GroupUser.php');
		require_once(ELC_CLASSES.'CRSO.php');
		require_once(ELC_CLASSES.'Product.php');
		require_once(ELC_CLASSES.'User.php');
		require_once(ELC_CLASSES.'Trac.php');
		require_once(ELC_CLASSES.'RelationUser.php');

		$this->user = new ELCUser($this->db_app);
		if(isset($_REQUEST['user']['id']))
		{
			$this->user->set($_REQUEST['user']['id']);
			$this->user->info_person->set();
			if($this->user->trac_user != null)
				$this->user->trac_user->set();
		}

		$crso = new Relation($db_app, $_REQUEST['crso']['id']);
		$crso->set();
		setPageTitlePostfix('('.(isset($this->user->name)?$this->user->name.' / ':'').$crso->name.')');
	}
	/**
	 *
	 */
	function processInput(&$thrownException)
	{
		// LOAD translations::
		$noTwoAdmins = $this->langIf->getText('This user cannot be the admin');
		$duplicateUser = $this->langIf->getText('Invalid UserName, User already exists');
		$noTracUser = $this->langIf->getText('No permission to set trac user');

		$this->process = 'check';
		if(isset($_POST['process']))
		{
			$ru = new RelationUser($this->db_app);
			$uid = isset($_REQUEST['user']['id'])?$_REQUEST['user']['id']:'';
			if(!empty($_POST['relation_user']['is_admin'])&&!$ru->validateAdmin($uid,$_REQUEST['crso']['id']))
			{
				$_POST['process'] = '';
				return $thrownException .= $noTwoAdmins;
			}

			// check if someone changed the username by making the field writable
			if(isset($_REQUEST['user']['id'])) {
				$old_user_info = new ELCUser($this->db_app);
				$old_user_info->set($_REQUEST['user']['id']);
				$old_username = $old_user_info->name;
				if ($_POST['user']['name'] != $old_username){
					$_POST['user']['name'] = $old_username;
					info('Someone tried to change the username of a CRSO user.');
				}
			}
			// end of username check

			if($_POST['process'] == 'check')
			{
				$this->process = 'save';
				$this->_hideInputs = true;

				global $page_title;
				$page_title = 'CRSO User Registration Confirmation';

			}
			elseif($_POST['process'] == 'save')
			{

				$this->db_app->begin();

				$update = isset($_REQUEST['user']['id']);
				$orgUserName = '';
				if($update)
				{
					$user = new ELCUser($this->db_app,$_REQUEST['user']['id']);
					$user->set();
					$orgUserName = $user->name;
				}

				$user = $_POST['user'];
				if (CRYPT_BLOWFISH == 1)
				{
					$u1 = sha1(uniqid());
					$salt = "\$2a\$07\$" . $u1 . "\$";
					$pass = crypt(ELCUser::generatePasswd(PASSWORD_LENGTH_CRSO),$salt);
					$user['salt'] = $salt;
				}
				else
					$pass = sha1(ELCUser::generatePasswd(PASSWORD_LENGTH_CRSO));
				$user['password'] = $pass; // generate random password. users will get a link with a login token anyway
				$user['active'] = (int) !empty($_POST['user']['active']);

				$user = new ELCUser($this->db_app,$user);
				$user->info_person = new InfoPerson($this->db_app,$_POST['person']);
				$user->info_person->company_id = $_SESSION['session_user_company_id'];

				if($orgUserName!=$user->name && (!$user->isUniqueUserName()))
				{
					$this->db_app->rollback();
					return $thrownException .= $duplicateUser;
				}

				$ru_vals = isset($_POST['relation_user']) ? $_POST['relation_user'] : array();
				// Add the user to right CRSO
				$ru = new RelationUser($this->db_app,$ru_vals);
				$ru->user =& $user;
				$ru->relation = new Relation($this->db_app,$_REQUEST['crso']['id']);
				$ru->is_admin = (int) !empty($_POST['relation_user']['is_admin']);
				$ru->status = 1; // shouldnt we need defines for this

				if($ru->is_admin&&!$ru->validateAdmin())
				{
					$this->db_app->rollback();
					return $thrownException .= $noTwoAdmins;
				}

				$loc = ($_SESSION['session_group_group_id']==GROUP_CRSO) ? 'crso' : 'operator';
				$header = '/'.SITE_PATH.'scripts/elc/'.$loc.'/InterfaceCRSOUserOverview.php?crso[id]='.$_REQUEST['crso']['id'];

				if(!$update)
				{
					// Add the user to the right group
					$gu = new GroupUser($this->db_app);
					$gu->group_id = GROUP_CRSO;
					$gu->user =& $user;

					$user->info_person->insert();
					if(isset($_POST['trac_user']) && $_POST['trac_user']['user_name'] != ''){
						if($_SESSION['session_group_group_id'] != GROUP_SYSADMIN)
							return $thrownException .= $noTracUser;
						$user->trac_user = new TracUser($this->db_app, $_POST['trac_user']);
						$user->trac_user->insert();
					}
					$user->insert();
					$gu->insert();
					$ru->default_product = new Product($this->db_app,$_SESSION['selected_product_id']);
					$ru->insert();
					$this->db_app->commit();

					require_once(ELC_CLASSES.'ELCMessageHandler.php');
					$user->setPasswordInfoAndToken();
					MessageEvent::send(NEW_USER, array('user'=>$user, 'relationuser'=>$ru), 'ELCMessageHandler');
					$user->login_token = ''; // we don't want to keep the login token after it was sent
					// header to clearance level interface.
					if(!$ru->is_admin) {
						$params = '';
						if($_SESSION['session_group_group_id']==GROUP_SYSADMIN) {
							$params = sprintf('&crso_id=%s', $_REQUEST['crso']['id']);
						}
						$header = sprintf('/%sscripts/elc/%s/InterfaceUserClearanceLevels.php?user_id=%s%s',
							SITE_PATH,
							$loc,
							$user->id,
							$params
						);
					}
				}
				else
				{
					if($ru->is_admin) {
						// If it is the admin, they shouldnt have clearance levels.
						$ru->clearance_level = 0;
					}
					$user->info_person->update();
					$ru->update();// Doenst work yet if the user is isnt set to admin ... EEEP!

					if(isset($_POST['trac_user']) && !empty($_POST['trac_user']['user_name'])){
						if($_SESSION['session_group_group_id'] != GROUP_SYSADMIN)
							return $thrownException .= $noTracUser;
						$user->trac_user = new TracUser($this->db_app, $_POST['trac_user']);
						if(isset($_POST['trac_user']['id']))
							$user->trac_user->update();
						else
							$user->trac_user->insert();
					}
					$user->update();
					$this->db_app->commit();
				}

                log_elc_event(sprintf('CRSO user [%s] %s',
                						$user->name,
                        				$update ? 'updated' : 'created'),
                        	$_SESSION['session_user_id'], $_SESSION['selected_product_id'],
                        	(int)$_REQUEST['crso']['id'], null
                );

				header(sprintf('Location: %s', $header));
				exit();
			}
		}
	}
}

class InterfaceCRSOUserRegistrationItem extends InterfaceItem
{
	/**#@+
	 * Callback functions made from InterfaceItem
	 */

	function get2colTxt()
	{
		$this->twoCols = true;
	}

	/**
	 *
	 */
	function getItemValue()
	{
		if(isset($_REQUEST['user']['id']))
		{
			if(!isset($this->parent->parent->userInfo))
			{
				$this->hiddenFields = array(array('name'=>'user[id]','value'=>$_REQUEST['user']['id']));
				$user = $this->parent->parent->user;

				$this->hiddenFields[] = array('name'=>'person[id]','value'=>$user->info_person->id);
				if($user->trac_user)
					$this->hiddenFields[] = array('name'=>'trac_user[id]','value'=>$user->trac_user->id);

				$ru = new RelationUser($this->db_app,array('user'=>$user, 'relation'=>new Relation($this->db_app,$_REQUEST['crso']['id'])));
				$ru->set();

				$this->parent->parent->person = (array) $user->info_person;
				$this->parent->parent->trac_user = (array) $user->trac_user;
				$this->parent->parent->user = (array) $user;
				$this->parent->parent->relation_user = (array) $ru;
				$this->parent->parent->userInfo = true;
			}
			if($this->name == 'user[name]' || ($this->name == 'trac_user[user_name]' && $this->parent->parent->trac_user != null))
			{
				$this->properties['style'] = 'border: 0px; background-color: transparent;';
				$this->properties['readonly'] = 'readonly';
			}
			$name = str_replace('[','[\'',str_replace(']','\']',$this->name));
			eval('$this->properties[\'value\'] = isset($this->parent->parent->'.$name.')?$this->parent->parent->'.$name.':\'\';');
			if($this->type_id==IF_ITEM_CHECKBOX&& $this->properties['value'])
				$this->properties['checked'] = 'checked';
			if($this->type_id==IF_ITEM_CHECKBOX)
				$this->properties['value']=1;
		}
	}

	function getPassword()
	{
		// DEPRECATED ! WE STORE PASSWORDS AS A HASH
		//$usr = (array) $this->if_obj->user;
		//$passwd = $usr['password'];
		//$crypter = new Crypter(ENCRYPTION_KEY);
		$this->properties['value'] = ''; //$crypter->decrypt( $passwd ); // FIXME . we don't show passwords anymore
		info('Calling deprecated getPassword(). (from InterfaceCRSOUserRegistration.php)');
		//throw new IWUserException('Permission Denied');
	}
	function getPhoneFaxValue()
	{
		$this->getItemValue();
		if(!isset($this->properties['value'])||$this->properties['value'] == '')
			$this->properties['value'] = '+';
	}
	/**
	 *
	 */
	function getLanguageSelect()
	{
		require_once(IWARE.'nLanguage.php');
		$lang = new nLanguage($this->db_app);
		$q = $lang->get();
		$this->options = array(array('value'=>'','text'=>'- '.$this->langIf->getText('Select language').' -'));
		$i = 1;
		while($r = $q->fetch())
		{
			$this->options[$i] = array('text'=>$r->LANGUAGE,'value'=>$r->LANGUAGE_ID);
			if(!isset($this->parent->parent->userInfo))
			{
				if($r->LANGUAGE_ID == $_SESSION['session_user_language_id'])
					$this->options[$i]['selected'] = 'selected';
			}
			elseif(isset($this->parent->parent->person['language_id'])&& $this->parent->parent->person['language_id']==$r->LANGUAGE_ID)
				$this->options[$i]['selected'] = 'selected';
			$i++;
		}
	}
	/**
	 *
	 */
	function getSpacer()
	{
		$this->twoCols = true;
	}
	/**
	 *
	 */
	function getButtons()
	{
		$this->hiddenFields = array(array('name'=>'crso[id]','value'=>$_REQUEST['crso']['id']));
		if(isset($_POST['process'])&& $_POST['process'] == 'check')
		{
			$this->_BarButtons = 0;
			$this->text = $this->langIf->getText('Are above settings correct?').'<br />';
			$this->_addCustomButton($this->langIf->getText('No, I would like to make changes'),'button', array('onclick'=>'document.getElementsByName(\'process\')[0].value=\'\'; document.getElementById(\'interface_form\').submit();','name'=>'signup'));
			$this->_addCustomButton($this->langIf->getText('Yes, I am ready to continue'),'submit', array('onsubmit'=>'','name'=>'signup'));
		}
		else
			$this->_ButtonOptions[BUTTON_CANCEL] = array('onclick'=>"document.location.href='InterfaceCRSOUserOverview.php?crso[id]=".$_REQUEST['crso']['id']."';");
	}
	/**#@-*/
}
