<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'ELCTaskRunner.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @since 15-09-2006
 * @author Marten Koedam <marten@dalines.net>
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceExportDownload extends InterfaceFactory
{
    /**
     * interface[id] = 148
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceExportDownload(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';
        $this->process = 'download';

        $this->script = sprintf("
			function triggerMasterdataExport()
			{
				MessageBox.prototype.Question('%s','%s',function(){
					$$('process').value = 'masterdata';
					$('interface_form').submit();
				});
			}
			",
			$this->langIf->getText('MessageBox: MasterData Export Trigger'),
			$this->langIf->getText('MessageBox Content: MasterData Export in about 20 mins')
			);
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        if(isset($_POST['process']))
        {
        	if($_POST['process'] == 'download')
        	{
        		if(!(empty($_REQUEST['crso']['id'])||empty($_REQUEST['export']['type'])||empty($_REQUEST['export']['date'])))
        		{
        			$fd = fopen(ELC_EXPORT_PATH.$_REQUEST['crso']['id'].'/'.$_REQUEST['export']['date'],'r');
				    header("Cache-control: ");
				    header("Pragma: ");
				    header("Content-Type: application/octet-stream");
				    header("Content-Length: ".filesize(ELC_EXPORT_PATH.$_REQUEST['crso']['id'].'/'.$_REQUEST['export']['date']));
				    header(sprintf("Content-Disposition: attachment; filename=\"%s\" ", $_REQUEST['export']['date']));
				    header("Content-transfer-Encoding: binary\n");

    			    while(!feof($fd))
    					echo fread($fd, 8192);

					fclose ($fd);
        			exit();
        		}
        	}
        	elseif($_POST['process'] == 'masterdata')
        	{
        		// get the task ID of Masterdata Export

        		$tr = new ELCTaskRunner($this->db_app, false);

        		if(!($id = $tr->getIdByName('TaskExportMasterFile')))
        			exit('No Masterdata Task found');

				$d = currentDate();
				$schedule = $d->getYear().'-'.$d->getMonth().'-'.$d->getDay().' 00:05:00';
				$tr->setNextExecuteTime($id,$schedule);

        		header('Location: InterfaceExportDownload.php');
        		exit();
        	}
        }
    }
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 15-09-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceExportDownloadItem extends InterfaceItem
{

    /**
     * item[1096] = export[type]
     */
    function getExportTypeSelect()
    {
    	$types = array(1=>'Declaration Export',2=>'Refund Export',3=>'Masterdata Export');
    	$set = isset($_REQUEST['export']['type'])?$_REQUEST['export']['type']:false;
    	$this->options = array(array('text'=>'- '.$this->langIf->getText('Select Export Type').' -','value'=>''));
    	foreach($types as $i => $type)
    		$this->options[] = array('text'=>$this->langIf->getText($type),'value'=>$i,'selected'=>($set==$i));

    	$this->properties['onchange'] = "_LoadOverview();";

    	$this->postscript = "
		    function _LoadOverview()
		    {
		        var params = new Array();
		        params['export'] = new Array();
				params['crso'] = new Array();
				params['export']['type'] = getElByName('export[type]').value;
				params['export']['date'] = getElByName('export[date]').value;
				params['crso']['id'] = getElByName('crso[id]').value;
		        params['interface_id'] = ".$this->if_obj->id."; // you MUST supply this
		        ifHandler.loadBlock( new Array(365,366), params );
		    }";
    }

    /**
     * item[1097] = export[date]
     */
    function getDateSelection()
    {
    	if(empty($_REQUEST['export']['type'])||empty($_REQUEST['crso']['id']))
    	{
    		$this->options = array(array('text'=>'- '.$this->langIf->getText('Select Type And CRSO First').' -','value'=>''));
    		return;
    	}
    	$nameMap = array(1=>'_TRASALES(?!_RR)',2=>'_TRASALES_RR',3=>'_MVEND\.(xml|txt)(?!\.diff)');
    	$name = $nameMap[$_REQUEST['export']['type']];
    	$text = array();
    	$set = isset($_REQUEST['export']['date'])?$_REQUEST['export']['date']:false;
    	$this->options = array();// array(array('text'=>'- '.$this->langIf->getText('Select Date of Export').' -','value'=>''));
    	$d = @opendir(ELC_EXPORT_PATH.$_REQUEST['crso']['id']);
		if($d)
		{
			while (($file = readdir($d)) !== false )
			if( $file != "." && $file != ".." && preg_match('/(\d+)'.$name.'/',$file, $match))
			{
				$date = $match[1];
				$text[] = $date;
				$this->options[] = array('text'=>substr($date,6,2).'-'.substr($date,4,2).'-'.substr($date,0,4),'value'=>$file,'selected'=>($set==$file));
			}


	    	array_multisort($text,SORT_DESC,$this->options);
	    	array_unshift($this->options,array('text'=>'- '.$this->langIf->getText('Select Date of Export').' -','value'=>''));


	    	closedir($d);
		}

    	$noExports = $this->langIf->getText('No exports available');
    	if(count($this->options) == 0)
    		$this->options[] = array('text' => $noExports, 'value' => '');

    	$this->properties['onchange'] = "_LoadOverview();";
    }
    /**
     * item[1099]
     */
    function getCRSOSelect()
    {
    	switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
				require_once(ELC_CLASSES.'CRSO.php');
				$crso = new CRSO($this->db_app);
				$q = $crso->selectCRSOs();
				break;
			case GROUP_ACCOUNTANT:
				require_once(ELC_CLASSES.'AccountantCRSO.php');
		    	$accountant_crso = new AccountantCRSO($this->db_app);
				$q = $accountant_crso->getAccountantCRSOs($_SESSION['accountant_relation_id'],$_SESSION['selected_product_id']);
				break;
		}

		$rs = $q->fetchAll();
    	if(count($rs) == 1){
			$_REQUEST['crso']['id'] = $rs[0]->CRSO_RELATION_ID;
		}

		$set = (!empty($_REQUEST['crso']['id']))?$_REQUEST['crso']['id']:false;

		$this->options = array(array('text'=>'- '.$this->langIf->getText('Select CRSO').' -','value'=>''));
		foreach($rs as $r)
			$this->options[] = array('text'=>((mb_strlen($r->NAME)>35)?mb_substr($r->NAME,0,35).'...':$r->NAME),'value'=>$r->CRSO_RELATION_ID,'selected'=>($set==$r->CRSO_RELATION_ID));
		$this->properties['onchange'] = "_LoadOverview();";
    }

    /**
     * item[1098] =
     */
    function getButtons()
    {
    	$this->_BarButtons = 0;

    	$disabled = (empty($_REQUEST['crso']['id'])||empty($_REQUEST['export']['type'])||empty($_REQUEST['export']['date']));
    	$opts = array();
    	$disabled && $opts['disabled'] = 'disabled';

    	$this->_addCustomButton($this->langIf->getText('Download'),'submit',$opts);
    }
	function get2colTxt()
	{
		$this->twoCols = true;
	}

	function getMasterDataButtons()
	{
		$this->_BarButtons = 0;

		// if the task isnt scheduled for the future: disable the button
		$tr = new ELCTaskRunner($this->db_app, false);

       	if(!($id = $tr->getIdByName('TaskExportMasterFile')))
			return;

		$opts = array('onclick'=>'triggerMasterdataExport();');
		$nextTime = $tr->getNextExecuteTime($id);
		if(substr($nextTime,0,4)!='3000')
			$opts['disabled'] = 'disabled';

		$this->_addCustomButton($this->langIf->getText('Trigger MasterData Export'),'button',$opts);
	}

	function getExportOverview()
	{
		/*$ds = array();
		require_once(ELC_CLASSES.'AccountantCRSO.php');
		$crso = new AccountantCRSO($this->db_app);
		$q = $crso->getAccountantCRSOs($_SESSION['accountant_relation_id'], $_SESSION['selected_product_id']);
		while($r = $q->fetch())
		{
			$path = ELC_EXPORT_PATH.$r->CRSO_RELATION_ID.'/';
			$d = @opendir(ELC_EXPORT_PATH.$_REQUEST['crso']['id']);
			if($d)
			{
				while (($file = readdir($d)) !== false )
				if( $file != "." && $file != ".." && preg_match('//',$file, $match))
		}*/
	}
}

/**
 * CALLBACK[getExportTypeSelect]
 * item[1096] = {
 *     [handle] = Item: Export Type
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getDateSelection]
 * item[1097] = {
 *     [handle] = Item: Date Selection
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[1098] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>