<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'ProofOfPaymentConfirmation.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'AccountantCRSO.php');
/**
 *
 * @package
 * @subpackage Interface
 * @since 10-01-2007
 * @copyright 2007 BaseSoft B.V.
 * @version $Id$
 */
class InterfacePoPConfirmationOverview extends InterfaceFactory
{
	public $review_status = false;
    /**
     * interface[id] = 164
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfacePoPConfirmationOverview(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        if($_SESSION['session_group_group_id']==GROUP_CRSO) {
        	require_once(ELC_CLASSES.'RelationUser.php');
        	$ru = new RelationUser($this->db_app);
        	if(!$ru->hasClearance(CRSOClearanceRefundDetails, $_SESSION['session_user_id'], $_SESSION['crso_relation_id'])) {
        		throw new InterfacePermissionException('No permission to view this page.');
        	}
        	$_REQUEST['crso']['id'] = $_SESSION['crso_relation_id'];
        	$this->hiddenFields = array(array('name'=>'crso[id]','value'=> $_SESSION['crso_relation_id']));

        }
        $this->model['confirmation'] = true;
        $country = null;
    	switch($_SESSION['session_group_group_id']) {
    		case GROUP_ACCOUNTANT:
    			$invalidOn = true;
    		case GROUP_CRSO:
    			if(!isset($_REQUEST['crso']['id']))
    				break;
    			$crso_id = $_REQUEST['crso']['id'];
    			require_once(ELC_CLASSES.'Country.php');
    			require_once(ELC_CLASSES.'Relation.php');
    			$rel = new Relation($this->db_app, $crso_id);
    			$country = new Country($this->db_app, $rel->getCountry(), true);
    			$invalidOn = isset($invalidOn) ? $invalidOn : false;
			break;
    	}
    	if($country && $country->has(HasCRSORefundConfirmations, $_SESSION['selected_product_id'])===$invalidOn) {
    		$this->model['confirmation'] = false;
    	}

        if(!isset($_REQUEST['popc']['status']))
        	$this->review_status = POP_CONFIRMATION_UNREVIEWED;
        else
        	$this->review_status = $_REQUEST['popc']['status'];

        $this->model['popc'] = new ProofOfPaymentConfirmation($this->db_app);

        $this->script = "
			function changeSelection()
			{
				var params = new Array();
     			params['crso'] = new Array();
				params['crso']['id'] = getElByName('crso[id]').value;
        		params['popc'] = new Array();
				params['popc']['status'] = getElByName('popc[status]').value;
        		params['interface_id'] = 164; // you MUST supply this
        		ifHandler.loadBlock( new Array(new String('412'),new String('413')), params );
			}

			function approveConfirmations()
			{
				if(!has_selected_checkboxes())
					return;

				".jsMessageBox(MSG_BOX_QUEST,
						$this->langIf->getText('MessageBoxTitle: Approve Selected PoPConfirmations'),
						$this->langIf->getText('MessageBoxBody: Approve Selected PoPConfirmations'), "function(){var frm = $('interface_form');
					frm.submit();}")."
			}

			function rejectConfirmations()
			{
				if(!has_selected_checkboxes())
					return;

				".jsMessageBox(MSG_BOX_QUEST,
						$this->langIf->getText('MessageBoxTitle: Reject Selected PoPConfirmations'),
						$this->langIf->getText('MessageBoxBody: Reject Selected PoPConfirmations'), "function(){var frm = $('interface_form');
					frm.process.value='reject';
					frm.submit();}")."
			}

			function has_selected_checkboxes()
			{
				// check if we selected atleast one checkbox and give an error
				var checks = document.getElementsByName('popcids[]');
				var err = true;
				for (var i=0; i<checks.length; i++)
				{
					if (checks[i].checked) {
						err = false;
						break;
					}
				}

				if (err) {
					".jsMessageBox(MSG_BOX_CRIT, '', $this->langIf->getText('Select atleast one PoP Confirmation'))."
					return false;
				}

				return true;
			}
			";
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        if(!isset($_REQUEST['process']))
        	return;
        if(!$this->model['confirmation']) {
        	return;
        }
        if($_REQUEST['process']=='save')
        {
        	foreach($_POST['popcids'] as $popcId)
        	{
        		$popc = new ProofOfPaymentConfirmation($this->db_app,$popcId,true);
        		$popc->review_status = POP_CONFIRMATION_APPROVED;
        		$popc->update();
        		$this->_triggerDocumentReview($popc);
        	}
        }
        elseif($_REQUEST['process']=='reject')
        {
        	foreach($_POST['popcids'] as $popcId)
        	{
        		$popc = new ProofOfPaymentConfirmation($this->db_app,$popcId,true);
        		$popc->review_status = POP_CONFIRMATION_REJECTED;
        		$popc->update();
        		$this->_triggerDocumentReview($popc);
    			$authorizeUser = $popc->getAuthorizedById();
    			$recipientsRelation = array(array('recipient'=>$popc->participant->relation));
    			$recipientsLink = array(array('recipient'=>$popc->participant->relation,
    				'user'=>new ELCUser($this->db_app,$authorizeUser),
					'callback'=>"go('/".SITE_PATH."scripts/elc/participant/InterfacePaymentConfirmation.php?crso[id]=".$popc->crso->relation->id."&popc[id]=".$popcId."')"));
				$this->sendNotifications($popc,$recipientsRelation,$recipientsLink);
        	}
        }
        $crsoId = isset($_REQUEST['crso']['id'])?$_REQUEST['crso']['id']:false;
        header('Location: InterfacePoPConfirmations.php?crso[id]='.$crsoId.'&popc[status]='.$this->review_status);
        exit();
    }

    function sendNotifications($popc,$relation,$link)
    {
    	$notificationSubject = $this->langIf->getText('NotificationSubject: PoP Confirmation Rejected');
    	$notificationRelationContentDesc = $this->langIf->getText('NotificationContent: PoPConfRejectedRelation {PARTICIPANT} {CRSO}');
    	$notificationUserContentDesc = $this->langIf->getText('NotificationContent: PoPConfRejectedUser {PARTICIPANT} {CRSO}');;

    	$popc->crso->relation->set();
    	$popc->participant->relation->set();

    	require_once(ELC_CLASSES.'Notification.php');
		// This one goes to everyone of the relation.
		Notification::send($this->db_app,
			array('subject'=>$notificationSubject,
				'sender'=>$popc->crso->relation,
				'message'=>
					$this->langIf->getText($notificationRelationContentDesc,
						null,
						array ('PARTICIPANT'=>$popc->participant->relation->name,
							'CRSO'=>$popc->crso->relation->name))
			),
			$relation
		);

		// This one goes to the user that originally confirmed the document.
		Notification::send($this->db_app,
			array('subject'=>$notificationSubject,
				'sender'=>$popc->crso->relation,
				'message'=>
					$this->langIf->getText($notificationUserContentDesc,
						null,
						array ('PARTICIPANT'=>$popc->participant->relation->name,
							'CRSO'=>$popc->crso->relation->name))
			),
			$link
		);
    }

    protected function _triggerDocumentReview($obj)
	{
		try
		{
			if(!isset($obj->document) || !$obj->document || !isset($obj->document->id))
				return;

			require_once(ELC_CLASSES.'CRSOAdminFee.php');
			$caf = new CRSOAdminFee($this->db_app);
			$crso = $obj->crso->relation->id;
			$fee = $caf->getCurrentFee($crso, CRSO_ADMIN_FEE_POP_CONFIRMATION);

			require_once(ELC_CLASSES.'RefundExportDocumentReview.php');
			$redr = new RefundExportDocumentReview($this->db_app);
			$redr->document =&  $obj->document;
			$redr->review_cost =  $fee ? $fee : 0;
			$redr->product = $obj->product;
			$redr->crso = $obj->crso;
			$redr->participant = $obj->participant;
			if(!$redr->insert())
				trigger_error(sprintf('Error inserting document review: CRSO [%s] Document [%s]', $obj->crso->relation->id, $obj->document->id));
		}
		catch(Exception $e)
		{
			//we already reviewed this doc
		}
	}

	function _filter($q) {
		$rv = parent::_filter($q);

		// TODO: Hide CRSO selection in case user is an Accountant with just one CRSO.




		return $rv;

	}
}
/**
 * @package
 * @subpackage Interface
 * @since 10-01-2007
 * @copyright 2007 BaseSoft B.V.
 * @version $Id$
 */
class InterfacePoPConfirmationOverviewItem extends InterfaceItem
{

    /**
     * item[1189] = crso[id]
     */
    function getCRSOSelect()
    {
		if($_SESSION['session_group_group_id']==GROUP_CRSO) {
			return;

		}

		// This is for sysadmin:
    	$this->options = array(array('value'=>'','text'=>'- '.$this->langIf->getText('All CRSOs').' -'));
    	$crsoId = isset($_REQUEST['crso']['id'])?$_REQUEST['crso']['id']:false;

    	switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
				$crso = new CRSO($this->db_app);
				$q = $crso->selectCRSOs();
				break;
			case GROUP_ACCOUNTANT:
		    	$accountant_crso = new AccountantCRSO($this->db_app);
				$q = $accountant_crso->getAccountantCRSOs($_SESSION['accountant_relation_id'],$_SESSION['selected_product_id']);
				break;
		}
		$rs = $q->fetchAll();
    	if(count($rs) == 1){
			$crsoId = $rs[0]->CRSO_RELATION_ID;
			$_REQUEST['crso']['id'] = $rs[0]->CRSO_RELATION_ID;
		}
    	foreach($rs as $r)
    	{
    		$selected = ($r->CRSO_RELATION_ID==$crsoId)?'selected':false;
    		$name = $r->NAME;
    		$name = (mb_strlen($name)>35)?mb_substr($name,0,35).'...':$name;
    		$this->options[] = array('value'=>$r->CRSO_RELATION_ID,'text'=>$name,
				'selected'=>$selected);
    	}
    	$this->properties['onchange'] = 'changeSelection();';
    }

    /**
     * item[1190] = popc[status]
     */
    function getStatusSelect()
    {
    	// Pending - default?
    	// Approved and Rejected
    	$this->options = array();
    	foreach(array(POP_CONFIRMATION_UNREVIEWED,POP_CONFIRMATION_APPROVED,POP_CONFIRMATION_REJECTED) as $status)
    	{
    		$selected = ($status==$this->if_obj->review_status)?'selected':false;
    		list($text,$bitbucket) = ProofOfPaymentConfirmation::translateStatus($this->langIf,$status);
    		$this->options[] = array('text'=>$text,'value'=>$status,'selected'=>$selected);
    	}
    	$this->properties['onchange'] = 'changeSelection();';
    }

    /**
     * item[1191] =
     */
    function getConfirmationOverview()
    {
    	if(!isset($_REQUEST['crso']['id'])) {

    		$this->text = $this->langIf->getText('Please select a CRSO first');
    		return;
    	}
    	$hiddenCols = array('PROOF_OF_PAYMENT_CONFIRMATION_ID');
    	if($this->if_obj->review_status!=POP_CONFIRMATION_OPEN || !$this->if_obj->model['confirmation']) {
    		$hiddenCols[] = 'CHECKBOX';
    	}

    	$ds = $this->if_obj->model['popc']->getOperatorOverview($this->if_obj->review_status,$_REQUEST['crso']['id']);
    	$tf = new TableFactory($ds,$this->langIf);
    	$tf->setEmptyText($this->langIf->getText('No confirmations found for current selection'));
    	$tf->hideColumns($hiddenCols);
    	$tf->columnModifier(array('QUANTITY'=>'style="text-align:right;"'));
    	$tf->setValueProcessor(TF_ALL,array($this,'tfValueProcessor'));
    	$tf->setTHProcessor(array('FEE','QUANTITY','AMOUNT'),'return \' class="th-right"\';');
    	$this->text = $tf->getXHTMLTable('Professional',2);
    }

	function tfValueProcessor($args)
	{
		switch($args['head'])
		{
			case 'CHECKBOX':
				return sprintf('<input type="checkbox" name="popcids[]" value="%s" />',$args['value']);
			break;
			case 'CHECKSUM':
				$popids = $this->if_obj->model['popc']->getPopsByConfirmation($args['row']['PROOF_OF_PAYMENT_CONFIRMATION_ID'],'array');
				return ProofOfPaymentConfirmation::generateConfirmProofOfPaymentChecksum($popids);
			break;
			case 'QUANTITY':
				$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
    			return locale_nf($args['value'], 0);
			break;
			case 'PoPCofirmationDocument':
				require_once(ELC_CLASSES.'Document.php');
				$doc = new ELCDocument($this->db_app,$args['value'],true);
				$name = $this->langIf->getText('Download');
				return sprintf('<a href="/%sscripts/i-w/document/download.php?Document[id]=%d">%s</a>',SITE_PATH,$doc->id,$name);
			break;
		}
		return $args['value'];
	}

    /**
     * item[1192] =
     */
    function getButtons()
    {
    	$this->_BarButtons = 0;
		if(!$this->if_obj->model['confirmation']) {
			return;
		}

    	if($this->if_obj->review_status!=POP_CONFIRMATION_UNREVIEWED)
    		return $this->text = $this->langIf->getText('No Actions Possible');

		$this->_addCustomButton($this->langIf->getText('Approve Confirmations'),'button',array('onclick'=>'approveConfirmations();'));
		$this->_addCustomButton($this->langIf->getText('Reject Confirmations'),'button',array('onclick'=>'rejectConfirmations();'));
    }

}

/**
 * CALLBACK[getCRSOSelect]
 * item[1189] = {
 *     [handle] = Item: CRSO Selection
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getStatusSelect]
 * item[1190] = {
 *     [handle] = Item: ReviewStatus
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getConfirmationOverview]
 * item[1191] = {
 *     [handle] = Item: ConfirmationOverview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[1192] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
