<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'AccountantCRSO.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'RequestedInvoice.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@dalines.net>
 * @since 31-08-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceRequestDebitCreditId extends InterfaceFactory
{
    /**
     * interface[id] = 145
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceRequestDebitCreditId(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        require_once(IWARE . 'group.php');
		$this->model['type'] = $this->model['crso'] = false;
		switch ($_SESSION['session_group_group_id'])
		{
			case GROUP_ACCOUNTANT:
				$crso = new AccountantCRSO($this->db_app);
				// fill available crsos
				$q = $crso->getAccountantCRSOs($_SESSION['accountant_relation_id']);
			break;
			case GROUP_SYSADMIN:
				$crso = new CRSO($this->db_app);
				$q = $crso->selectCRSOs();
			break;
			default: // isnt this already checked by SCRIPT Privileges?
				throw new InterfacePermissionException('Unauthorized Access to Interface');
		}
		$this->model['type'] = isset($_REQUEST['type'])?$_REQUEST['type']:false;
		if(!empty($_REQUEST['crso']['id']))
			$this->model['crso'] = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, $_REQUEST['crso']['id'], true)));
		$this->model['crsos'] = array();
		while ($r = $q->fetch())
			$this->model['crsos'][] = array($r->CRSO_RELATION_ID, $r->NAME);

		if($this->model['type']&& $this->model['crso'])
		{
			switch($this->model['type'])
    		{
    			case TYPE_DECLARATION_INVOICE:
    				$this->model['display'] = 'invoice';
    				$this->model['type_name'] = 'debit';
    				$this->model['pe'] = PARTICIPANT_PRODUCER;
    				$class = 'Declaration';
    			break;
    			case TYPE_DECLARATION_CREDIT:
    				$this->model['display'] = 'credit';
    				$this->model['type_name'] = 'credit';
    				$this->model['pe'] = PARTICIPANT_PRODUCER;
    				$class = 'Declaration';
    			break;
    			case TYPE_REFUND_INVOICE:
    				$this->model['display'] = 'invoice';
    				$this->model['type_name'] = 'debit';
    				$this->model['pe'] = PARTICIPANT_EXPORTER;
    				$class = 'RefundExport';
    			break;
    			case TYPE_REFUND_CREDIT:
    				$this->model['display'] = 'credit';
    				$this->model['type_name'] = 'credit';
    				$this->model['pe'] = PARTICIPANT_EXPORTER;
    				$class = 'RefundExport';
    			break;
    		}

    		require_once(ELC_CLASSES.$class.'.php');
    		$this->model['obj'] = new $class($this->db_app,array('crso'=>$this->model['crso']));
		}
 		$date =& currentDate();
        $this->model['year'] = isset($_REQUEST['year']) ? $_REQUEST['year']:$date->getYear();

		$this->model['ri'] = new RequestedInvoice($this->db_app);
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
    	$invalidSelection = $this->langIf->getText('Invalid Selection');
    	$participantRequired = $this->langIf->getText('No Participant Selected');
    	$periodRequired = $this->langIf->getText('No Period Selected');
        if(isset($_POST['process']))
        {
        	if($_POST['process']=='save')
        	{

        		if(!$this->model['crso']||!$this->model['type'])
        			return $thrownException .= $invalidSelection;
        		if(empty($_POST['participant']['id']))
        			return $thrownException .= $participantRequired;
        		if(empty($_POST['month']))
        			return $thrownException .= $periodRequired;

				// Everything seems alright, lets save this snizzle!
				$this->db_app->begin();

				$d =& currentDate();

				$this->model['ri']->month = $_REQUEST['month'];
				$this->model['ri']->year = $_REQUEST['year'];
				if(!$this->model['ri']->addId($this->model['obj'],$this->model['type_name'],$_REQUEST['participant']['id']))
				{
					$this->db_app->rollback();
					alert('Something went wrong for participant ['.$_REQUEST['participant']['id'].'] when requesting new '.$this->model['type_name'].' ID');
					return $thrownException .= "Something went wrong";
				}
				$this->db_app->commit();
				header(sprintf('Location: InterfaceRequestDebitCreditId.php?crso[id]=%s&type=%s',$this->model['crso']->relation->id,$this->model['type']));
				exit;
        	}
        }
    }

    function getInterfaceHeader()
    {
		$this->script = "
			function _LoadOverview()
			{
		        var params = new Array();
				params['crso'] = new Array();
		        params['crso']['id'] = getElByName('crso[id]').value;
				params['participant'] = new Array();
		        params['participant']['id'] = getElByName('participant[id]').value;
		        params['type'] = getElByName('type').value;
				params['month'] = getElByName('month').value;
				params['year'] = getElByName('year').value;
		        params['interface_id'] = ".$this->id."; // you MUST supply this
		        ifHandler.loadBlock( new Array('356','357','358'), params );
			}
			";
		$this->script .= sprintf("
			function RequestId()
			{
				if($$('participant[id]').value=='')
					return MessageBox.prototype.Warning('%s','%s');
				MessageBox.prototype.Question('%s','%s', function(){
					$('interface_form').submit();
				});
			}
			",
			$this->langIf->getText('MessageBox: Warning'), $this->langIf->getText('MessageBox: Selection of participant is required'),
			$this->langIf->getText('MessageBox: Request of Invoice ID'),$this->langIf->getText('MessageBox: Request Invoice ID, process cant be undone, continue?')
			);
    	return parent::getInterfaceHeader();
    }
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 31-08-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceRequestDebitCreditIdItem extends InterfaceItem
{

    /**
     * item[1082] = crso[id]
     */
    function getCRSOSelect()
    {
    	$crso_id = $this->if_obj->model['crso'] ? $this->if_obj->model['crso']->relation->id : '';

    	if(count($this->if_obj->model['crsos']) == 1){
			list ($id, $name) = $this->if_obj->model['crsos'][0];
    		$crso_id = $id;
			$_REQUEST['crso']['id'] = $id;
			$this->if_obj->model['crso'] = new CRSO($this->if_obj->db_app, array('relation'=>new Relation($this->if_obj->db_app, $id)));
		}

    	$this->options[] = array('text'=>'- '.$this->langIf->getText('Select CRSO').' -', 'value'=>'');
    	foreach ($this->if_obj->model['crsos'] as $crso) {
    		list ($id, $name) = $crso;
    		$this->options[] = array('text'=>((mb_strlen($name)>30)?mb_substr($name,0,30).'...':$name), 'value'=>$id, 'selected'=>($id == $crso_id));
    	}

    	$this->properties['onchange'] = "_LoadOverview();";
    	//$this->properties['onchange'] = sprintf("go('%s?type=%s&crso[id]='+this.value);", $_SERVER['PHP_SELF'], $month);
    }

    /**
     * item[1083] = type
     */
    function getTypeSelection()
    {
    	$type = $this->if_obj->model['type'] ? $this->if_obj->model['type'] : '';
    	$types = array(
    		TYPE_DECLARATION_INVOICE=>'Declaration Invoice ID',
			TYPE_DECLARATION_CREDIT=>'Declaration Credit ID',
			TYPE_REFUND_INVOICE=>'Refund Invoice ID',
			TYPE_REFUND_CREDIT=>'Refund Credit ID',
			);
		$this->options = array(array('value'=>'','text'=>'- '.$this->langIf->getText('Select ID Type').' -'));
		foreach($types as $id => $trans)
			$this->options[] = array('text'=>$this->langIf->getText($trans), 'value'=>$id, 'selected'=>($id == $type));
		$this->properties['onchange'] = "_LoadOverview();";
    }

    /**
     * item[1084] =
     */
    function getOverview()
    {
    	if(!$this->if_obj->model['crso']||!$this->if_obj->model['type'])
    		return;

    	require_once(IWARE.'table_factory.php');
    	$crso_id = $this->if_obj->model['crso']->relation->id;

    	$obj =& $this->if_obj->model['ri'];
		$q = $obj->{'get'.ucfirst(get_class($this->if_obj->model['obj'])).'Overview'}(strtoupper($this->if_obj->model['type_name']),$crso_id);

		$tf = new TableFactory($q,$this->langIf);
		$tf->setEmptyText($this->langIf->getText('No '.$this->if_obj->model['display'].' IDs have been requested for '.get_class($this->if_obj->model['obj']).'s at CRSO'));
		$tf->hideColumns(array('REQUESTED_INVOICE_ID'));
		$tf->setValueProcessor(array('MONTH'),array($this,'tfValueProcessor'));
		$tf->setSortingFunctions(
			array(
				'REFUND_REQUEST_DEBIT_ID'=>"'Int'",
				'PARTICIPANT_NAME'=>"'String'",
				'REQUEST_DATETIME'=>"'String'",
				'MONTH'=>"'SortVal'",
				'YEAR'=>"'Int'"
			));
    	$this->text = $tf->getXHTMLTable('Professional',2);
    }

	function tfValueProcessor($args)
	{
		switch($args['head'])
		{
			case 'MONTH':
				$args['extra'] = sprintf(' sort_val="%s" ', $args['value']);
				return strftime('%B', mktime(1, 1, 1, $args['value'], 1) );
			break;
		}
	}

    /**
     * item[1085] =
     */
    function getButtons()
    {
    	$this->_BarButtons = 0;
    	if(!$this->if_obj->model['crso']||!$this->if_obj->model['type'])
    		return;

    	// Add participant Selection!
    	// $this->text = '<span style="float:left;">'.$this->_getParticipantSelect($this->if_obj->model['crso']).'</span>';
    	$this->_addCustomButton($this->langIf->getText('Request new '.$this->if_obj->model['display'].' ID'),'button',array('onclick'=>'RequestId();'));
    }

    function getParticipantSelect()
    {
    	$this->options = array(array('text'=>' - '.$this->langIf->getText('Select CRSO and Type First').' - ','value'=>''));
    	if(!$this->if_obj->model['crso']||!$this->if_obj->model['type'])
    		return;

    	$crso = &$this->if_obj->model['crso'];
    	$p = new CRSOParticipantProduct($this->db_app);

    	$pid = isset($_REQUEST['participant']['id'])?$_REQUEST['participant']['id']:-1;
    	$q = $p->getCRSOParticipants($crso->relation->id,'q',$this->if_obj->model['pe'],$_SESSION['selected_product_id']);
    	$this->options = array(array('text'=>' - '.$this->langIf->getText('Participant Select').' - ','value'=>''));
    	while($r = $q->fetch())
    	{
    		$name = $r->PARTICIPANT_NAME;
    		$this->options[] = array('text'=>((mb_strlen($name)>30)?mb_substr($name,0,30).'...':$name),'value'=>$r->PARTICIPANT_RELATION_ID,'selected'=>($pid==$r->PARTICIPANT_RELATION_ID));
    	}
    	$this->properties['onchange'] = "_LoadOverview();";
    }

    function getPeriodSelect()
    {
    	// Trac Ticket #428:
    	//  * Filter on period depending on Obligation Date and/or Start Date of Participant/Refund Requester

    	if(!$this->if_obj->model['crso']||empty($_REQUEST['participant']['id']))
    	{
    		$this->options = array(array('text'=>' - '.$this->langIf->getText('Select CRSO And Participant First').' - ','value'=>''));
    		return;
    	}

    	$cp = new CRSOParticipantProduct($this->db_app);
    	$cp->set($this->if_obj->model['crso']->relation->id,$_REQUEST['participant']['id'],$_SESSION['selected_product_id']);

    	$start = (int) substr($cp->obligation_start_date, 5, 2);
    	$year = (int) substr($cp->obligation_start_date, 0, 4);
    	if($this->if_obj->model['year'] > $year) {
    		$start = 1;
    	}
    	$period = (isset($_REQUEST['month'])&& $_REQUEST['month'] >= $start) ? $_REQUEST['month'] :-1;
    	$this->options = array(array('text'=>'- '.$this->langIf->getText('Select Period').' -'));
    	for($i=$start;$i<=12;$i++)
    	{
    		$this->options[] = array('text'=>strftime('%B', mktime(1, 1, 1, $i, 1)),'value'=>$i,'selected'=>($i==$period));
    	}
    }

    /**
     * item[1118] =
     */
    function get2colTxt()
    {
    	$this->twoCols = true;
    }

    /**
     * item[1119] =
     */
    function getYearSelect()
    {
    	$date = currentDate();
		$year = (int)$date->getYear();

    	if($year == WEEE_APPLICATION_START_YEAR)
    		$this->_hideInput = true;

    	$this->options = array();

    	for($i=WEEE_APPLICATION_START_YEAR; $i<=$year; $i++)
			$this->options[] = array('value'=>$i,'text'=>$i,'selected'=>($i==$this->if_obj->model['year']?'selected':false));
		$this->properties['onchange'] = "_LoadOverview();";
    }
}

define('TYPE_DECLARATION_INVOICE',1);
define('TYPE_DECLARATION_CREDIT',2);
define('TYPE_REFUND_INVOICE',3);
define('TYPE_REFUND_CREDIT',4);

/**
 * CALLBACK[getCRSOSelect]
 * item[1082] = {
 *     [handle] = Item: CRSO Selection
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getTypeSelection]
 * item[1083] = {
 *     [handle] = Item: DeclOrRRTypeSelection
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getOverview]
 * item[1084] = {
 *     [handle] = Item: Overview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[1085] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>