<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'RefundRequestConfirmation.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'AccountantCRSO.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(IWARE.'group.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @author Nick Loeve <nick@basesoft.com>
 * @since 20-07-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceReviewRefundExports extends InterfaceFactory
{
    /**
     * interface[id] = 142
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceReviewRefundExports(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        $this->model['reviewType'] = isset($_REQUEST['refund_export_overview']['type']) && $_REQUEST['refund_export_overview']['type'] !== '' ? $_REQUEST['refund_export_overview']['type'] : false;
        $this->model['rrc'] = new RefundRequestConfirmation($this->db_app);

		// date filter
    	$date =& currentDate();

		if (isset($_REQUEST['month']) && !empty($_REQUEST['month']))
        	$this->model['month'] = $_REQUEST['month'];
        else
        	$this->model['month'] = $date->getMonth();

        if (isset($_REQUEST['year']) && !empty($_REQUEST['year']))
        	$this->model['year'] = $_REQUEST['year'];
        else
        	$this->model['year'] = $date->getYear();

        // fill available crsos
    	switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
				$crso = new CRSO($this->db_app);
				$q = $crso->selectCRSOs();
				break;
			case GROUP_ACCOUNTANT:
		    	$accountant_crso = new AccountantCRSO($this->db_app);
				$q = $accountant_crso->getAccountantCRSOs($_SESSION['accountant_relation_id']);
				break;
		}
        $this->model['crso'] = false;
		if($_SESSION['session_group_group_id']!=GROUP_CRSO) {
			while ($r = $q->fetch())
				$this->model['crsos'][] = array($r->CRSO_RELATION_ID, $r->NAME);

			if (isset($_REQUEST['crso']['id']) && !empty($_REQUEST['crso']['id']))
				$this->model['crso'] = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, $_REQUEST['crso']['id'])));
			else
				if(count($this->model['crsos']) == 1){
					list ($id, $name) = $this->model['crsos'][0];
					$this->model['crso'] = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, $id)));
				}

		} else {
			$this->model['crso'] = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, $_SESSION['crso_relation_id'])));
		}

		if(isset($this->model['crso']) && $this->model['crso']) {
			$this->model['country'] = new Country($this->db_app, $this->model['crso']->relation->getCountry(),true);
		}

    }

    function _filter($q)
	{
		$rv = parent::_filter($q);

		unset($rv[$this->id][343][1150]);
		unset($rv[$this->id][343][1111]);

		return $rv;
	}

    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        if(isset($_REQUEST['process']) && isset($_REQUEST['checkboxes']))
        {
        	foreach($_REQUEST['checkboxes'] as $rrcId)
			{
        		$recipientsRelation = array();
        		$recipientsLink = array();
				$rrc = new RefundRequestConfirmation($this->db_app, $rrcId);
				$rrc->set();
				switch($_REQUEST['process'])
	        	{
	        		case 'authorize':
	        			$rrc->review_status = RR_CONFIRMATION_APPROVED;
	        			break;
	        		case 'deny':
	        			$rrc->review_status = RR_CONFIRMATION_REJECTED;
	        			// Send an notification with the correct link to the user
	        			// who authorized it in the first place.
	        			$authorizeUser = $rrc->getAuthorizedById();
	        			$recipientsRelation = array(array('recipient'=>$rrc->participant->relation));
	        			$recipientsLink = array(array('recipient'=>$rrc->participant->relation,
	        				'user'=>new ELCUser($this->db_app,$authorizeUser),
							'callback'=>"go('/".SITE_PATH."scripts/elc/participant/InterfaceRefundConfirmation.php?rrc[id]=".$rrcId."')"));
	        			break;
	        	}
				$rrc->update();
				if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT)
					$this->_triggerDocumentReview($rrc);
				if($recipientsRelation)
					$this->sendNotifications($rrc,$recipientsRelation,$recipientsLink);
			}
        }

        $this->init($this->id);
    }

    function sendNotifications($rrc,$relation,$link)
    {
    	$notificationSubject = $this->langIf->getText('NotificationSubject: Refund Confirmation Rejected');
    	$notificationRelationContentDesc = $this->langIf->getText('NotificationContent: RefundConfRejectedRelation {PARTICIPANT} {CRSO}');
    	$notificationUserContentDesc = $this->langIf->getText('NotificationContent: RefundConfRejectedUser {PARTICIPANT} {CRSO}');;

    	$this->model['crso']->relation->set();
    	$rrc->participant->relation->set();

    	require_once(ELC_CLASSES.'Notification.php');
		// This one goes to everyone of the relation.
		Notification::send($this->db_app,
			array('subject'=>$notificationSubject,
				'sender'=>$this->model['crso']->relation,
				'message'=>
					$this->langIf->getText($notificationRelationContentDesc,
						null,
						array ('PARTICIPANT'=>$rrc->participant->relation->name,
							'CRSO'=>$this->model['crso']->relation->name))
			),
			$relation
		);

		// This one goes to the user that originally confirmed the document.
		Notification::send($this->db_app,
			array('subject'=>$notificationSubject,
				'sender'=>$this->model['crso']->relation,
				'message'=>
					$this->langIf->getText($notificationUserContentDesc,
						null,
						array ('PARTICIPANT'=>$rrc->participant->relation->name,
							'CRSO'=>$this->model['crso']->relation->name))
			),
			$link
		);
    }

    function _triggerDocumentReview(&$obj)
	{
		try
		{
			if(!isset($obj->document) || !$obj->document || !isset($obj->document->id))
				return;

			require_once(ELC_CLASSES.'CRSOAdminFee.php');
			$caf = new CRSOAdminFee($this->db_app);
			$crso = $obj->getCRSOForDocumentReview();
			$fee = $caf->getCurrentFee($crso, CRSO_ADMIN_FEE_REFUND_REVIEW);

			require_once(ELC_CLASSES.'RefundExportDocumentReview.php');
			$redr = new RefundExportDocumentReview($this->db_app);
			$redr->document =&  $obj->document;
			$redr->review_cost =  $fee ? $fee : 0;
			require_once(ELC_CLASSES.'Product.php');
			$redr->product = new Product($this->db_app, $_SESSION['selected_product_id']);

			//get crso and participant
			$sql = "
					SELECT
						RR.PARTICIPANT_RELATION_ID,
						RR.CRSO_RELATION_ID
					FROM TBL_REFUND_REQUEST_CONFIRMATION RRC
					INNER JOIN TBL_REFUND_REQUEST RR USING (REFUND_REQUEST_CONFIRMATION_ID)
					WHERE RRC.REFUND_REQUEST_CONFIRMATION_ID=?
					LIMIT 1
						";
			$q = $this->db_app->query($sql);
			$q->bind_values($obj->id);
			if(!$q->execute() || !($r = $q->fetch()))
				trigger_error(sprintf('Error inserting document review: CRSO [%s] Document [%s]', $obj->crso->relation->id, $obj->document->id));

			$redr->crso = new CRSO($this->db_app, array('relation' => new Relation($this->db_app, array('id' => $r->{'CRSO_RELATION_ID'}))));
			$redr->participant = new Participant($this->db_app, array('relation' => new Relation($this->db_app, array('id' => $r->{'PARTICIPANT_RELATION_ID'}))));
			if(!$redr->insert())
				trigger_error(sprintf('Error inserting document review: CRSO [%s] Document [%s]', $obj->crso->relation->id, $obj->document->id));
		}
		catch(Exception $e)
		{
			//we already reviewed this doc
		}
	}
}
/**
 * @package ELC
 * @subpackage Interface
 * @author Nick Loeve <nick@basesoft.com>
 * @since 20-07-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceReviewRefundExportsItem extends InterfaceItem
{
	var $_shownColumns = array();
	var $_totalFields = array();
	var $_headType = array();

    function getCRSOSelect()
    {
    	$crso_id = $this->if_obj->model['crso'] ? $this->if_obj->model['crso']->relation->id : '';

    	$this->options[] = array('text'=>'- '.$this->langIf->getText('Select CRSO').' -', 'value'=>'');
    	foreach ($this->if_obj->model['crsos'] as $crso) {
    		list ($id, $name) = $crso;
    		$name = (mb_strlen($name)>35)?mb_substr($name,0,35).'...':$name;
    		$this->options[] = array('text'=>$name, 'value'=>$id, 'selected'=>($id == $crso_id));
    	}

    	//$this->properties['onchange'] = "_LoadOverview();";
    	$this->properties['onchange'] = sprintf("go('%s?refund_export_overview[type]=%s&crso[id]='+this.value);",
    		'InterfaceReviewRefundExports.php', $this->if_obj->model['reviewType']);

    }

    /**
     * item[1062] = refund_export_overview[type]
     */
    function getTypeDropdown()
    {
    	$crso_id = $this->if_obj->model['crso']?$this->if_obj->model['crso']->relation->id:false;
    	//possible statuses of export confirmations
    	//they probably don't need to see approved ones :)
    	$review_types = array(RR_CONFIRMATION_UNREVIEWED, RR_CONFIRMATION_REJECTED, RR_CONFIRMATION_APPROVED);

    	$this->options = array(array('value'=>'','text'=>'- '.$this->langIf->getText('Select').' -'));
    	$i = 1;
    	foreach($review_types as $type)
    	{
    		$trans = $this->if_obj->model['rrc']->translateStatus($this->langIf, $type);
    		$this->options[$i] = array('value' => $type, 'text' => $trans[0]);
    		if($this->if_obj->model['reviewType']!== false && $this->if_obj->model['reviewType'] == $type)
    			$this->options[$i]['selected'] = 'selected';
    		$i++;
    	}

    	$this->properties['onchange'] = sprintf("go('%s?refund_export_overview[type]='+this.value+'&crso[id]=%s');",
    		'InterfaceReviewRefundExports.php', $crso_id);
    }

	//Not used
	//Being filtered out.... see #481
    function getSelectMonth()
    {
    	$crso_id = $this->if_obj->model['crso'] ? $this->if_obj->model['crso']->relation->id : '';
    	$month = $this->if_obj->model['month'];
    	$year = $this->if_obj->model['year'];

    	$this->options = array();

    	for($i=1;$i<=12;$i++)
    	{
    		$this->options[$i-1] = array('text'=>strftime('%B', mktime(1, 1, 1, $i, 1)), 'value'=>$i);
    		if($i == $month)
    			$this->options[$i-1]['selected'] = 'selected';
    	}

    	$this->properties['onchange'] = sprintf("go('%s?refund_export_overview[type]=%s&crso[id]=%s&year=%s&month='+this.value);",
    		'InterfaceReviewRefundExports.php', $this->if_obj->model['reviewType'], $crso_id, $year );

    }

    //Not used
	//Being filtered out.... see #481
    function getYearSelect()
    {
    	$crso_id = $this->if_obj->model['crso'] ? $this->if_obj->model['crso']->relation->id : '';
    	$month = $this->if_obj->model['month'];
    	$year = $this->if_obj->model['year'];

    	$this->options = array();

    	$date = currentDate();
    	$currentYear = $date->getYear();

    	for($i=WEEE_APPLICATION_START_YEAR;$i<=$currentYear;$i++)
    	{
    		$this->options[] = array('text'=>$i, 'value'=>$i);
    		if($i == $year)
    			$this->options[count($this->options) - 1]['selected'] = 'selected';
    	}

    	$this->properties['onchange'] = sprintf("go('%s?refund_export_overview[type]=%s&crso[id]=%s&month=%s&year='+this.value);",
    		'InterfaceReviewRefundExports.php', $this->if_obj->model['reviewType'], $crso_id, $month );

    }


    /**
     * item[1061] =
     */
    function getOverview()
    {
    	if($this->if_obj->model['reviewType'] === false)
    	{
    		$this->text = $this->langIf->getText('Please choose a review status');
    		return;
    	}
    	if(!$this->if_obj->model['crso'])
    		return $this->text = $this->langIf->getText('Please select a CRSO');


    	$q = $this->if_obj->model['rrc']->getOverviewForOperator($this->if_obj->model['reviewType'],$this->if_obj->model['crso']->relation->id);

    	require_once(IWARE.'table_factory.php');
    	$tf = new TableFactory($q, $this->langIf);
    	$tf->id = "RefundRequestExportOverview";
    	$tf->setEmptyText($this->langIf->getText('No RefundRequestConfirmations found.'));
    	$hideColumns = array('REFUND_REQUEST_CONFIRMATION_ID');
    	if($this->if_obj->model['reviewType'] == RR_CONFIRMATION_APPROVED)
    		$hideColumns[] = 'CHECKBOX';
    	if($_SESSION['session_group_group_id']==GROUP_CRSO)
    		$hideColumns[] = 'CRSO_NAME';

    	// Hide authorize for universes based on Country Setting #2137
	   	if($_SESSION['session_group_group_id']==GROUP_CRSO && !$this->if_obj->model['country']->has(HasCRSORefundConfirmations, $_SESSION['selected_product_id'])) {
			$hideColumns[] = 'CHECKBOX';
		} elseif($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT && $this->if_obj->model['country']->has(HasCRSORefundConfirmations, $_SESSION['selected_product_id'])) {
			$hideColumns[] = 'CHECKBOX';
		}

    	$tf->hideColumns($hideColumns);

    	//processors
    	$tf->setValueProcessor('CHECKBOX', array($this, '_tfValueProcessor'));
    	$tf->setValueProcessor('EXPORT_CONFIRMATION_DOCUMENT', array($this, '_tfValueProcessor'));
    	$tf->setValueProcessor('CHECKSUM', array($this, '_tfValueProcessor'));
    	$tf->setValueProcessor('QUANTITY', array($this, '_tfValueProcessorQuantityNoDecimals'));
    	$tf->setValueProcessor('MONTH', array($this, '_tfMonth'));

    	$tf->columnModifier(array('QUANTITY'=>'style="text-align:right;"'));
    	$tf->setFooterProcessor(array($this,'_tfFooterProcessor'));

    	$tf->setSortingFunctions(
    		array(
			'CHECKSUM'=>"'String'",
			'MONTH'=>"'SortVal'",
			'Year'=>"'Int'",
			'PARTICIPANT_NAME'=>"'String'",
			'CRSO_NAME'=>"'String'",
			'QUANTITY'=>"'SortVal'"	)
    		);

    	$this->_shownColumns = $tf->getVisibleColumns();


    	$this->text = $tf->getXHTMLTable(array('Professional'), 2);
    }

    /**
     * item[1063] =
     */
    function getButtonsReply()
    {
    	$this->_BarButtons = 0;
    	if($this->if_obj->model['reviewType'] == RR_CONFIRMATION_APPROVED)
    		return;

    	// dont show buttons if we dont have a crso/part selected
    	if ($this->if_obj->model['reviewType'] === false)
    		return;

    	if($_SESSION['session_group_group_id']==GROUP_CRSO && !$this->if_obj->model['country']->has(HasCRSORefundConfirmations, $_SESSION['selected_product_id'])) {
			return;
		} elseif($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT && $this->if_obj->model['country']->has(HasCRSORefundConfirmations, $_SESSION['selected_product_id'])) {
			return;
		}

		$this->script = sprintf("
			function has_selected_checkboxes()
			{
				// check if we selected atleast one checkbox and give an error
				var checks = document.getElementsByName('checkboxes[]');
				var err = true;
				for (var i=0; i<checks.length; i++)
				{
					if (checks[i].checked) {
						err = false;
						break;
					}
				}

				if (err) {
					".jsMessageBox(MSG_BOX_CRIT, '', $this->langIf->getText('Select at least one export confirmation'))."
					return false
				}

				return true
			}
		");

		// authorize buttons
    	$this->_addCustomButton($this->langIf->getText('Approve Confirmation Export'), 'button',
    		array('onclick'=>jsMessageBox(MSG_BOX_QUEST,
    				$this->langIf->getText('Authorize selected'),
    				$this->langIf->getText('Are you sure you want to authorize the selected?'),
					"function() { $('interface_form').process.value='authorize'; if (has_selected_checkboxes()) { $('interface_form').submit();}  }")
				)
		);
    	$this->_addCustomButton($this->langIf->getText('Reject Confirmation Export'), 'button',
    		array('onclick'=>jsMessageBox(MSG_BOX_QUEST,
    				$this->langIf->getText('Deny Selected'),
    				$this->langIf->getText('Are you sure you want to deny the selected?'),
					"function() { $('interface_form').process.value='deny'; if (has_selected_checkboxes())$('interface_form').submit(); }")
				)
    	);
    }

	function _tfValueProcessor($args)
	{
		switch($args['head'])
		{
			case 'CHECKBOX':
				return '<input type="checkbox" behaviour="required" name="checkboxes[]" value="'.$args['row']['REFUND_REQUEST_CONFIRMATION_ID'].'" />';
				break;
			case 'EXPORT_CONFIRMATION_DOCUMENT':
				return '<a href="/'.SITE_PATH.'scripts/i-w/document/download.php?Document[id]='.$args['value'].'">'.$this->langIf->getText('Export Confirmation Document').'</a>';
				break;
			case 'CHECKSUM':
				require_once(ELC_CLASSES.'RefundRequest.php');
				$rr =& $this->if_obj->model['rrc']->getRefundRequestsForConfirmation($args['row']['REFUND_REQUEST_CONFIRMATION_ID']);
				debug(print_r($rr, true));
				return RefundRequest::generateConfirmProofOfExportChecksum($rr);
				break;
		}
	}

	/**
     * Number formats a quanitity with no decimals
     */
    function _tfValueProcessorQuantityNoDecimals($args)
    {
    	isset($this->_totalFields[$args['head']]) || $this->_totalFields[$args['head']] = 0;
		$this->_headType[$args['head']] = 0;
		$this->_totalFields[$args['head']] += $args['value'];
    	$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
    	return locale_nf($args['value'], 0);
    }
    /**
     * Gets the month name thru the LOCALE
     */
    function _tfMonth($args)
	{
		return strftime('%B', mktime(1, 1, 1, $args['value'], 1) );
	}

	function _tfFooterProcessor()
    {
    	if(!(bool) sizeof($this->_totalFields))
    	   	return false;

    	$cols = array_flip($this->_shownColumns);
		$rv = explode(',',str_repeat(',',(sizeof($this->_shownColumns)-1)));
		foreach($this->_totalFields as $head => $val)
		{
			$pos = $cols[$head];
			$round = $this->_headType[$head];
			$rv[$pos] = '<span style="float:right; font-weight: bold;">'.locale_nf($val,$round).'</span><span style="clear:both;"></span>';
		}
		return $rv;
    }
}

/**
 * CALLBACK[getTypeDropdown]
 * item[1062] = {
 *     [handle] = Item: RefundExportOverviewType
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getOverview]
 * item[1061] = {
 *     [handle] = Item: RefundExportOverview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtonsReply]
 * item[1063] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
