<?php
require_once(INTERFACE_OBJ."InterfaceFactory.php");
require_once(ELC_CLASSES.'Declaration.php');
require_once(ELC_CLASSES.'Country.php');
require_once(ELC_CLASSES.'ProductFee.php');
require_once(ELC_CLASSES.'Brand.php');
require_once(ELC_CLASSES.'ProductCategory.php');

define('COMPONENT_DETAILS_BLOCK', 400);

/**
 *
 *
 * @package ELC
 * @subpackage Interface
 * @author
 * @copyright (2005) BaseSoft B.V.
 * @version $Id$
 */
class InterfaceDeclarationDetail extends InterfaceFactory
{
	function InterfaceDeclarationDetail($db, $db_app, $langIf)
	{
		if(!isset($_SESSION['participant_ids']))
			exit('You Dont Have Permission To View This Page');
		$this->d = new Declaration($db_app,$_REQUEST['declaration']);
		$this->d->set();
		$this->d->participant->relation->set();
		$rv = null;
		// Check for any active Remote Viewer connections.
		if(IWRegistry::isRegistered('remote_viewer')) {
			$rv = IWRegistry::registry('remote_viewer');
			if($rv->crso->relation->id!=$this->d->crso->relation->id || $rv->participant->relation->id!=$this->d->participant->relation->id) {
				throw new InterfacePermissionException('No permission to view this page');
			}
		}
		// This is validated by the InterfaceParticipantPermissionCheck();
		/*if(!in_array($this->d->participant->relation->id, $_SESSION['participant_ids']))
		{
			$log = getLog();
			$log->alert('User `'.$_SESSION['session_user_name'].'` no permission on declaration ['.$this->d->id.']');
			exit('You Dont Have Permission To View This Page');
		}*/
		$this->FirstColWidth = 25;
		$this->delimiter = '<br />';
		parent::InterfaceFactory($db, $db_app, $langIf);
	}
	/**
	 *
	 */
	function getInterfaceHeader()
	{
		$rv = parent::getInterfaceHeader();
		//$this->d->crso->set();
		$this->d->participant->relation->set();
		$ds = array('INVOICE_NR'=>$this->d->getInvoiceID(),
			'PERIOD'=>strftime('%B', mktime(0, 0, 0, $this->d->month, 1)).' '.$this->d->year,
			'COMPANY_NAME'=>$this->d->participant->relation->name,
			'VAT_NUMBER'=>$this->d->participant->relation->vat_no);

		$tf = new TableFactory(array($ds),$this->langIf);
		$rv .= $tf->getRotatedXHTMLTable(array('table'=>'overview_head')).'<br />';
		return $rv;
	}

	function _filter($q)
	{
		$rv = parent::_filter($q);
		if(!$this->d->isComplete())
			$this->d->set();
		$country_id = $this->d->crso->getCountry();
		$c = new Country($this->db_app,$country_id,true);
		if(!$c->has(HasDeclarationComponent, $this->d->product->id))
			unset($rv[$this->id][COMPONENT_DETAILS_BLOCK]);
		return $rv;
	}
}

class InterfaceDeclarationDetailItem extends InterfaceItem
{
	/**#@+
	 * Callback functions made from InterfaceItem
	 */
	/**
	 *
	 */
	function getDetails()
	{
		$d = $this->parent->parent->d;
		//$declarations = $d->getDeclarationLifeCycle();

		$pcf = new ProductCategoryFee($this->db_app);
		$error = '';
		if(!$pcf->getFee(null,$d->crso->relation->id))
			$error = 'Your CRSO has not entered the FEE yet';

		$d->set();
		$declarations = array($d);

		$this->text = ($error?'<span class="error">'.$this->langIf->getText($error).'</span>':'');
		$ds = array();
		foreach($declarations as $decl)
			$ds[] = $this->_getDetails($decl);
		$this->text .= implode('<br />', $ds);
	}
	/**
	 * @access private
	 * @param Declaration $d
	 * @todo TRUSTme
	 */
	function _getDetails(&$d)
	{
		$d->set();
		$d->participant->relation->set();

		$country_id = $d->crso->getCountry();
		$c = new Country($this->db_app,$country_id,true);
		$ds = array();
		foreach($d->lines as $line_index => $line)
		{
			$line->set();
			if((($c->settings & HasDeclByCategory)==HasDeclByCategory)&& $line->product_category)
				$line->product_category->set();
			if((($c->settings & HasDeclarationDimension)==HasDeclarationDimension)&& $line->dimension)
				$line->dimension->set();
			if((($c->settings & HasDeclByBrand)==HasDeclByBrand)&& $line->brand)
				$line->brand->set();
			$ds[] = $this->_getLine($line);
		}
		if(!sizeof($d->lines))
			$ds[] = $this->_getLine();
		$hideCols = array('CHECKBOX', 'TRUST', 'PRODUCT_CATEGORY_ID');
		if(!$c->has(HasDeclByCategory,$_SESSION['selected_product_id'])) {
			$hideCols[] = 'PRODUCT_CATEGORY';
			$hideCols[] = 'PRODUCT_CATEGORY_DESCRIPTION';
		}
		if(!$c->has(HasDeclarationDimension,$_SESSION['selected_product_id']))
			$hideCols[] = 'CPD_ID';
		if(!$c->has(HasDeclByBrand,$_SESSION['selected_product_id']))
			$hideCols[] = 'BRAND_NAME';
		if(!$c->has(HasDeclOfWeight,$_SESSION['selected_product_id']))
			$hideCols[] = 'WEIGHT';
		if(!$c->has(HasDeclOfWeightWithDecimals,$_SESSION['selected_product_id'])) {
			$hideCols[] = 'WEIGHT_DEC';
			// In case other CRSOs ask for this feature we can add a separate country setting.
			$hideCols[] = 'PRODUCT_CATEGORY_DESCRIPTION';
		}

		$date = currentDate();
		$datestring = $d->year.'-'.$d->month.'-01 00:00:00';

		require_once(IWARE.'TableFactoryColumnPlugin.php');
		$tf = new TableFactoryColumnPlugin(false,$this->langIf,$this->db,$this->db_app);
		$tf->dateTime = new IWDateTime($d->year.'-'.$d->month.'-01 00:00:00');
		$tf->pcf = new ProductCategoryFee($this->db_app);

		$tf->setData($ds, 'DeclarationDetail',false,true);
		$tf->setValueProcessor('COMPANY_NAME', array($this, 'tfValueProcessor'));
		$tf->setValueProcessor('VAT_NO', array($this, 'tfValueProcessor'));

		$tf->setValueProcessor('TRANSACTION_STATUS', array($this,'tfValueProcessor'));
		$tf->setValueProcessor('QUANTITY','return is_numeric($value)?locale_nf($value):$value;');
		$tf->setValueProcessor('FEE','return locale_nf($this->pcf->getFee($row[\'PRODUCT_CATEGORY_ID\'],'.$d->crso->relation->id.', $this->dateTime->format(\'Y-m-d h:i:s\')),2);');
		$tf->setTHProcessor(array('FEE','QUANTITY', 'WEIGHT', 'WEIGHT_DEC'),'return \'class="th-right"\';');
		$tf->columnModifier(array('QUANTITY'=>'style="text-align:right;"','FEE'=>'style="text-align:right;"','WEIGHT'=>'style="text-align:right;"','WEIGHT_DEC'=>'style="text-align:right;"'));
		$tf->hideColumns($hideCols);
		$rv = $tf->getXHTMLTable('Professional',2);
		return $rv;
	}

	function tfValueProcessor(&$params)
	{
		switch ($params['head'])
		{
			case 'TRANSACTION_STATUS':
				list($trans, $class) = Declaration::translateStatus($this->langIf,$params['value']);
				if(!$this->if_obj->d->crso->relation->isComplete())
					$this->if_obj->d->crso->relation->set();
				$handle = 'Declaration Forfeit_'.$this->if_obj->d->crso->relation->name;
				// we need bytes but we dont want to break the multibyte string
				$handle = (strlen($handle)>254)?mb_strcut($handle,0,254):$handle;
				$crsoTrans = ($params['value']!=DECLARATION_FORFEIT)?'':$this->langIf->getText($handle);
				$trans = empty($crsoTrans)?$trans:$crsoTrans;

				return sprintf('<span class="%s">%s</span>',$class,$trans);
			break;
			case 'COMPANY_NAME':
			case 'VAT_NO':

				if ($params['row']['TRUST'])
				{
					$trust = $params['row']['TRUST'];
					$trust->set();
					$p = $trust->subrogator->relation;
					$p->set();

					return ($params['head'] == 'VAT_NO') ?  $p->vat_no : $p->name;
				}
				else {
					return ($params['head'] == 'VAT_NO') ?
						$this->if_obj->d->participant->relation->vat_no :
						$this->if_obj->d->participant->relation->name;
				}
			break;
		}
	}

	/**
	 * @access protected
	 */
	function _getLine($line='')
	{
		$defLine = array('TRANSACTION_ID' => '',
				'TRUST'=>'',
				'COMPANY_NAME'=>'',
				'VAT_NO' =>'',
				'QUANTITY' => '',
				'FEE' => '',
				'BRAND_NAME' => '',
				'PRODUCT_CATEGORY'=>'',
				'PRODUCT_CATEGORY_DESCRIPTION'=>'',
				'CPD_ID'=>'',
				'PRODUCT_CATEGORY_ID'=>'',
				'WEIGHT'=>'',
				'WEIGHT_DEC'=>'',
				'TRANSACTION_STATUS' => DECLARATION_OPEN,
				'CHECKBOX'=>'false');

		if(!$line)
			return $defLine;

		return array('TRANSACTION_ID' => $line->line_index,
				'TRUST'=>$line->trust,
				'COMPANY_NAME' => '',
				'VAT_NO' => '',
				'QUANTITY' => $line->quantity,
				'FEE' => '',
				'BRAND_NAME' => ($line->brand instanceof Brand)?$line->brand->name:'',
				'PRODUCT_CATEGORY'=> ($line->product_category instanceof ProductCategory)?$line->product_category->name:'',
				'PRODUCT_CATEGORY_DESCRIPTION'=> ($line->product_category instanceof ProductCategory)?$line->product_category->description:'',
				'CPD_ID'=> ($line->dimension instanceof CRSOProductDimension)?$line->dimension->name:'',
				'PRODUCT_CATEGORY_ID'=>($line->product_category instanceof ProductCategory)?$line->product_category->id:null,
				'WEIGHT'=> $line->weight,
				'WEIGHT_DEC'=> $line->weight_dec,
				'TRANSACTION_STATUS' => $line->status,
				'CHECKBOX'=>'');
	}

	function getComponentDetails()
	{
		//$this->twoCols = false;
		require_once(ELC_CLASSES.'DeclarationComponent.php');
		$comp = new DeclarationComponent($this->db_app);
		$comp->set($this->if_obj->d->id);

		$lines = $comp->getFormLines();

		$tf = new TableFactoryColumnPlugin($lines,$this->langIf,$this->db,$this->db_app);
		$tf->setValueProcessor('component', array($this, 'tfComponentProcessor'));
		$tf->hideColumns(array('weight'));
		$tf->setTHProcessor(array('quantity'),'return \'class="th-right"\';');
		$tf->columnModifier(array('quantity'=>'style="text-align:right;"'));
		$this->text = $tf->getXHTMLTable('Professional',2);
	}

	function tfComponentProcessor(&$params)
	{
		//debug(print_r($params, true));
		return '<span id="product_component_'.$params['value']->id.'">'.$params['value']->name.'</span>';
	}

	function getFiller()
	{
		return '&nbsp;';
	}

	/**
	 *
	 */
	function getButtonBar()
	{
		$this->_BarButtons = BUTTON_CANCEL;
		$this->_ButtonOptions[BUTTON_CANCEL] = array('onclick'=>"document.location.href='InterfaceDeclarationOverview.php';");
	}
	/**#@-*/
}
