<?php
require_once(INTERFACE_OBJ."InterfaceFactory.php");
require_once(ELC_CLASSES.'AnnualConfirmation.php');
require_once(ELC_CLASSES.'Declaration.php');
require_once(ELC_CLASSES.'Country.php');
require_once(ELC_CLASSES.'ProductFee.php');
require_once(ELC_CLASSES.'User.php');
require_once(ELC_CLASSES.'ParticipantBrand.php');
require_once(ELC_CLASSES.'ProductCategory.php');
require_once(ELC_CLASSES.'Trust.php');
require_once(ELC_CLASSES.'Product.php');
require_once(ELC_CLASSES.'DeclarationComponent.php');
require_once(ELC_CLASSES.'CRSOProductDimension.php');
require_once(ELC_CLASSES.'CRSOProduct.php');

define('BLOCK_COMPONENT_OVERVIEW',399);
define('BLOCK_COUNTRY_SPECIFIC_TEXT',415);

/**
 * The one interface what it is really all about, Declarations
 *
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @version $Id$
 */
class InterfaceDeclarationForm extends InterfaceFactory
{
	public $processType = 'normal';
	function InterfaceDeclarationForm($db, $db_app, $langIf)
	{
		$this->FirstColWidth = 25;
		$this->delimiter = '<br />';

		parent::InterfaceFactory($db, $db_app, $langIf);
		$this->model['declaration'] = new Declaration($this->db_app);
		if(isset($_REQUEST['declaration']['id']))
			$this->__validateDeclaration($_REQUEST['declaration']['id']);
		elseif(isset($_REQUEST['month'])&&isset($_REQUEST['year'])&&isset($_REQUEST['crso']['id']))
			$this->__validateManualDeclaration($_REQUEST['crso']['id'],$_REQUEST['month'],$_REQUEST['year']);
		else
			throw new InterfacePermissionException('Invalid Arguments Supplied');

		$ac = new AnnualConfirmation($this->db_app);
		$ac->setByParticipantCRSOAndYear($this->model['declaration']->participant->relation->id,
			$this->model['declaration']->crso->relation->id,$this->model['declaration']->year,
			$_SESSION['selected_product_id']);
		if($ac->status == ANNUAL_CONFIRMATION_CLOSED)
			throw new IWUserException('The Declarations for this Year Are Closed');

		$this->model['declaration']->participant->relation->set();
		$this->model['declaration']->crso->relation->set();

		$this->model['country'] = new Country($this->db_app,$this->model['declaration']->crso->getCountry(),true);
		if($this->model['country']->has(HasDeclarationComponent, $_SESSION['selected_product_id']))
			$this->model['dc'] = new DeclarationComponent($db_app);
		$this->model['pc'] = new ProductCategory($this->db_app,array('crso'=>$this->model['declaration']->crso,
			'product'=>$this->model['declaration']->product));
		$this->model['cpd'] = new CRSOProductDimension($this->db_app,array('crso'=>$this->model['declaration']->crso,
			'product'=>$this->model['declaration']->product));
		$this->model['pb'] = new ParticipantBrand($this->db_app,array('participant'=>$this->model['declaration']->participant));

		$this->script = $this->__getJS();
	}

	private function __validateManualDeclaration($crso_id,$month,$year)
	{
		$months = $this->model['declaration']->hasManualDeclarations($_SESSION['selected_participant_id'],$crso_id,$_SESSION['selected_product_id'],$year);
		if(!isset($months[$month]))
		{
			alert(sprintf('DeclarationForm::Declaration - Participant [%s] trying to add month [%s] for year [%s] at crso [%s]',$_SESSION['selected_participant_id'],$month,$year,$crso_id));
			throw new IWUserException('You Cannot Add This Month Manually');
		}

		$this->hiddenFields = array(
			array('name'=>'crso[id]','value'=>$crso_id),
			array('name'=>'month','value'=>$month),
			array('name'=>'year','value'=>$year)
			);

		$this->model['declaration']->participant = new Participant($this->db_app,array('relation'=>new Relation($this->db_app,$_SESSION['selected_participant_id'])));
		$this->model['declaration']->crso = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app,$crso_id)));
		$this->model['declaration']->product = new Product($this->db_app,$_SESSION['selected_product_id']);
		$this->model['declaration']->year = $year;
		$this->model['declaration']->month = $month;
		$this->model['declaration']->aggregated_status = DECLARATION_OPEN;
		$this->model['declaration']->is_exported = 0;

		$this->processType = 'manual';

		setPageTitlePostfix($this->langIf->getText('Manual Declaration ({MONTH} {YEAR})',false,
			array('YEAR'=>$year,'MONTH'=>strftime('%B', mktime(0, 0, 0, $month, 1)))));
	}

	private function __validateDeclaration($id)
	{
		$this->model['declaration']->set($id);
		$this->hiddenFields = array(array('name'=>'declaration[id]','value'=>$id));
		$exit = false;
		$logmsg = '';
		if(!$exit&&($this->model['declaration']->participant->relation->id!=$_SESSION['selected_participant_id']))
			$exit = $logmsg = ' No Permission On Declaration.';
		if(!$exit&& $this->model['declaration']->isCredit())
			$exit = $logmsg = ' CREDITs cannot be altered.';
		if(!$exit&&!$this->model['declaration']->isNewestForMonth())
			$exit = $logmsg = ' Only the latest declaration for a month can be altered.';
		if(!$exit&&($this->model['declaration']->product->id!=$_SESSION['selected_product_id']))
			$exit = $logmsg = ' The Selected Product is not the product of this declaration.';
		if($exit)
		{
			alert(sprintf('DeclarationForm::Declaration [%s] - %s',$id,$logmsg));
			throw new InterfacePermissionException(sprintf('DeclarationForm::Declaration [%s] - %s',$id,$logmsg));
		}
		setPageTitlePostfix('('.Declaration::getButtonAction($this->langIf,$this->model['declaration']->aggregated_status,$this->model['declaration']->is_exported).')');
	}


	function init($ifId)
	{
		$rv = parent::init($ifId);

		if(!$this->model['country']->has(HasDeclarationComponent,$_SESSION['selected_product_id']))
			unset($this->blocks[BLOCK_COMPONENT_OVERVIEW]);

		// Denmark and HU
		if(($this->model['declaration']->crso->relation->id != 3) && ($this->model['declaration']->crso->relation->id != 221))
			unset($this->blocks[BLOCK_COUNTRY_SPECIFIC_TEXT]);


		return $rv;
	}

	/**
	 * Processes the POST data
	 */
	function processInput(&$thrownException)
	{
		if(isset($_POST['process']))
		{
			$log = getLog(); // Log everything but the kitchen sink
			if($_POST['process'] == 'save')
			{

				// this adds the duplicate line check
				require_once(ELC_CLASSES.'FormBehaviourPlugins.php');
				$this->fb->addPlugin('elc_declaration_line_unique', 'FormBehaviourPluginElcDeclarationLineUnique');
				$this->fb->addPlugin('elc_declaration_weight', 'FormBehaviourPluginElcDeclarationWeight');
				$this->fb->addPlugin('elc_declaration_weight_dec', 'FormBehaviourPluginElcDeclarationWeightDec');
				$this->fb->register('line', 'elc_declaration_line_unique');

				// add formbehaviour to all added lines

				foreach ($_POST['line'] as $index => $line)
				{
					// the last entry is always an empty line
					if ($index === count($_POST['line'])-1)
						continue;

					$this->fb->register(sprintf('line[%s][quantity]', $index), $this->getBehaviour('quantity', $index));

					if ($this->model['country']->has(HasDeclOfWeight, $_SESSION['selected_product_id']))
						$this->fb->register(sprintf('line[%s][weight]', $index), $this->getBehaviour('weight', $index));

					if ($this->model['country']->has(HasDeclOfWeightWithDecimals, $_SESSION['selected_product_id']))
						$this->fb->register(sprintf('line[%s][weight_dec]', $index), $this->getBehaviour('weight_dec', $index));

					if ($this->model['country']->has(HasDeclByBrand, $_SESSION['selected_product_id']))
						$this->fb->register(sprintf('line[%s][brand][id]', $index), $this->getBehaviour('brand', $index));

					if ($this->model['country']->has(HasDeclByCategory, $_SESSION['selected_product_id']))
						$this->fb->register(sprintf('line[%s][product_category][id]', $index), $this->getBehaviour('product_category', $index));

					if ($this->model['country']->has(HasDeclarationDimension, $_SESSION['selected_product_id']))
						$this->fb->register(sprintf('line[%s][dimension][id]', $index), $this->getBehaviour('dimension', $index));

					if (isset($line[$index]['trust']['id']))
						$this->fb->register(sprintf('line[%s][trust][id]', $index), $this->getBehaviour('trust', $index));

				}

				if(!$this->fb->validate()) // If not validated, let the user know
				{
					$_POST['process'] = '';
					return $thrownException .= $this->langIf->getText('Please correct the form errors');
				}

				foreach ($_POST['line'] as $index=>$line)
				{
					if (is_numeric($line['quantity']) && $line['quantity'] < 0)
						trigger_error('InterfaceDeclarationForm::processInput() - Quantity cannot be negative', E_USER_ERROR);
				}

				if($this->processType == 'manual')
				{
					$this->db_app->begin();

					$pass = true;
					if($this->model['country']->has(HasDeclarationComponent,$_SESSION['selected_product_id']))
						$pass = $this->model['declaration']->insertDeclarationLines(array_values($_POST['line']),$_POST['dc'],true);
					else
						$pass = $this->model['declaration']->insertDeclarationLines(array_values($_POST['line']),false,true);

					$this->__triggerAnnualConfirmation();
					if(!$pass)
					{
						$this->db_app->rollback();
						header('Location: InterfaceDeclarationOverview.php');
						exit();
					}
					info(sprintf('Manual Declaration for Month [%s] and Year [%s] added for Participant [%s] with Product [%s] at CRSO [%s]',
						$_REQUEST['month'],$_REQUEST['year'],$_SESSION['selected_participant_id'],
						$_SESSION['selected_product_id'],$_REQUEST['crso']['id']));
					$this->db_app->commit();

                                        log_elc_event(
                                            sprintf('Participant did manual declaration for month [%s], year [%s]',
                                                $_REQUEST['month'], $_REQUEST['year']),
                                            $_SESSION['session_user_id'], $_SESSION['selected_product_id'], (int)$_REQUEST['crso']['id'],
                                            $_SESSION['selected_participant_id']
                                        );

					header('Location: InterfaceDeclarationOverview.php');
					exit();
				}

				// Start a query, begin a transaction, eat some peanuts and wait
				$this->db_app->begin();
				// In case the participant registered in the previous year and got activated after the january exports.
				// See also ticket #2056
				$this->__triggerAnnualConfirmation();

				if($this->model['declaration']->validateUpdate())
				{
					// The update was rightfully done, commit it
					if($this->_updateDeclaration($thrownException)) {
					    $this->db_app->commit();
						log_elc_event(
							sprintf('Participant filed declaration for month [%s], year [%s]',
								$this->model['declaration']->month, $this->model['declaration']->year),
							$_SESSION['session_user_id'], $_SESSION['selected_product_id'],
							$this->model['declaration']->crso->relation->id,
							$_SESSION['selected_participant_id']
						);
                    }
				}
				elseif($this->_insertCreditDeclaration()) {
				    $this->db_app->commit(); // The insert was okee, commit it
					log_elc_event(
						sprintf('Participant resubmitted declaration for month [%s], year [%s]',
							$this->model['declaration']->month, $this->model['declaration']->year),
						$_SESSION['session_user_id'], $_SESSION['selected_product_id'],
						$this->model['declaration']->crso->relation->id,
						$_SESSION['selected_participant_id']
					);
				}

				// Go back to the overview so they can see what they've done
				header('Location: InterfaceDeclarationOverview.php');
				exit();
			}
		}
	}

	function __triggerAnnualConfirmation()
	{
		$date = currentDate();
		$currentYear = $date->getYear();
		$currentMonth = $date->getMonth();
		$currentDay = $date->getDay();

		if($this->model['declaration']->year == $currentYear)
			return;

		$this->model['declaration']->crso->set();

		$cp = new CRSOProduct($this->db_app,array('crso'=>$this->model['declaration']->crso,'product'=>$this->model['declaration']->product),true);

		$correctDay = ($cp->forfeit_day>$cp->refund_export_day)?($currentDay < $cp->forfeit_day):($currentDay < $cp->refund_export_day);

		if($correctDay &&
			(($this->model['declaration']->year+1) == $currentYear)&&($currentMonth==1))
			return;

		// This means the year was in the past and the AC's for that year have already been triggered.
		$ac = new AnnualConfirmation($this->db_app,array(
			'year'=>$this->model['declaration']->year,
			'crso'=>$this->model['declaration']->crso,
			'participant'=>$this->model['declaration']->participant,
			'product'=>$this->model['declaration']->product
			));
		// Lets check if this crso-participant-product already has one:
		if($ac->setByParticipantCRSOAndYear())
			return;
		$log = getLog();
		// We have to trigger the AC in order to let this participant be able to sign it
		$ac->triggerNewAnnualConfirmations(null,$log,null,$this->model['declaration']->year,
			$this->model['declaration']->participant->relation->id);
	}

	/**#@+
	 * @access private
	 */
	/**
	 * Updates the Declaration
	 * @param string $thrownException
	 */
	function _updateDeclaration(&$thrownException)
	{
		$this->model['declaration']->updateDeclarationLines(array_values($_POST['line']));
		if($this->model['country']->has(HasDeclarationComponent,$_SESSION['selected_product_id']))
			$this->model['dc']->updateDeclarationComponents($this->model['declaration'],$_POST['dc']);
		info('Declaration ['.$this->model['declaration']->id.'] updated by user ['.$_SESSION['session_user_id'].']');
		return true;
	}
	/**
	 * Insert a Credited Declaration (Late or Resubmit)
	 * @return bool
	 */
	function _insertCreditDeclaration()
	{
		$pass = true;
		if($this->model['country']->has(HasDeclarationComponent,$_SESSION['selected_product_id']))
			$pass = $this->model['declaration']->insertDeclarationLines(array_values($_POST['line']),$_POST['dc']);
		else
			$pass = $this->model['declaration']->insertDeclarationLines(array_values($_POST['line']));
		if(!$pass)
		{
			header('Location: InterfaceDeclarationOverview.php');
			exit();
		}
		info('Declaration ['.$this->model['declaration']->id.'] added by user ['.$_SESSION['session_user_id'].']');
		return true;
	}
	/**#@-*/
	/**
	 * Get the information table on top of the Declaration Form
	 * @todo mkpretty
	 */
	function getInterfaceHeader()
	{
		//removeLastChildBehaviour;
		$rv = parent::getInterfaceHeader();
		$d =& $this->model['declaration'];
		$p =& $d->participant->relation;
		$p->address->set();
		$c =& $d->crso->relation;
		$c->address->set();
		$country = new Country($this->db_app);
		$country->set($p->address->country_id);
		$p_country = $country->name;
		$country->set($c->address->country_id);
		$c_country = $country->name;

		$rv .= '
			<table class="formHeader" width="100%">
				<tr>
					<td width="25%">'.$this->langIf->getText('INVOICE_NR').'</td>
					<td width="25%"><strong style="font-size: 10pt !important">'.(($this->processType=='manual')?$this->langIf->getText('Manual Registration'):$d->getInvoiceID()).'&nbsp;</strong></td>
					<td width="50%"></td>
				</tr>
				<tr>
					<td width="25%">'.$this->langIf->getText('PERIOD').'</td>
					<td width="25%"><strong style="font-size: 10pt !important">'.strftime('%B', mktime(0, 0, 0, $d->month, 1)).' '.$d->year.'</strong></td>
					<td width="50%"></td>
				</tr>
				<tr>
					<td width="25%">'.$this->langIf->getText('LAST_UPDATED').'</td>
					<td width="25%"><strong style="font-size: 10pt !important">'.$d->getLastUpdated().'</strong></td>
					<td width="50%"></td>
				</tr>
				<tr>
					<td colspan="3"><br /></td>
				</tr>
				<tr>
					<td colspan="2">'.$p->name.'</td>
					<td>'.$c->name.'</td>
				</tr>
				<tr>
					<td colspan="2">'.$p->address->address_street_name.'</td>
					<td>'.$c->address->address_street_name.'</td>
				</tr>
				<tr>
					<td colspan="2">'.$p->address->city_name.', '.$p_country.'</td>
					<td>'.$c->address->city_name.', '.$c_country.'</td>
				</tr>
				<tr>
					<td colspan="2"></td>
					<td>'.$c->address->address_zipcode .', '.$c->email.'</td>
				</tr>
				<tr>
					<td>'.$this->langIf->getText('VAT Number').'</td>
					<td>'.$p->vat_no.'</td>
					<td></td>
				</tr>
				<tr>
					<td>'.$this->langIf->getText('Bank Account Number').'</td>
					<td>'.$p->bank_account_number.'</td>
					<td></td>
				</tr>
			</table>
			<br /><br />
			';


		return $rv;
	}

	private function __getJS()
	{
		$d =& $this->model['declaration'];
		$crso_id = $d->crso->relation->id;
		$period = $d->year.'-'.$d->month.'-01 00:00:00';

		return "
			function continueSaveProcess()
			{
				var nForm = \$('interface_form');
				var tableElement = getTableElement(\$\$('line[0][quantity]'));
				var lastChild = tableElement.tBodies[0].lastChild;
				while ( lastChild.nodeType != 1 ) //moz lastChild is a textNode
					lastChild = lastChild.previousSibling;
				// All elements in the last row that have a behaviour, should be cleared.
				['input', 'select'].forEach(function(sTag, iIndex, aArray)
				{
					var nodes = lastChild.getElementsByTagName(sTag);
					for (var i=0; i<nodes.length; i++)
					{
						nodes[i].setAttribute('behaviour','');
					}

				});

				if(fb.validate(nForm))
					nForm.submit();
			}

			function postProcessNewLine()
			{
				var tableElement = getTableElement(\$\$('line[0][quantity]'));
				var lastChild = tableElement.tBodies[0].lastChild.previousSibling;
				if(lastChild.nodeType!= 1)
					lastChild = lastChild.previousSibling;
				lastChild.style.display = '';
			}

			function getTableElement(tableMember)
			{
				if ( tableMember.nodeType != 1 || tableMember.tagName.toLowerCase() != 'table' )
					return getTableElement(tableMember.parentNode);
				return tableMember;
			}
			function setSubrogatorVAT(sel, lineIndex)
			{
				$('vat_no['+lineIndex+']').innerHTML = Trusts[sel.value];
			}
			function updatePCFee(sel, lineIndex) {

				var feeSpan = null;
				var descSpan = null;
				var rowNode = sel.parentNode.parentNode;
				var spansF = document.getElementsByName('declaration_pc_fee');
				var spansD = document.getElementsByName('declaration_pc_desc');
				for(var i in spansF) {
					if(spansF[i].parentNode.parentNode != rowNode) continue;
					feeSpan = spansF[i];
					break;
				}
				for(i in spansD) {
					if(spansD[i].parentNode.parentNode != rowNode) continue;
					descSpan = spansD[i];
					break;
				}

				// We do an RPC call here to retrieve the correct FEE for the selected product category
				var params = new Array();
				params['categoryid'] = sel.value;
				params['crsoid'] = '".$crso_id."';
				params['period'] = '".$period."';
				params['category_description'] = true;
				jsbroker.call('getProductCategoryFee', params, function(result) {
					var fee = result[0];
					var desc = result[1];
					if(feeSpan) feeSpan.innerHTML = number_format(fee, 2);
					if(descSpan) descSpan.innerHTML = desc;

				});
			}

		";
	}

	function getBehaviour($element, $index=null)
	{
		$required = ($this->model['declaration']->aggregated_status & DECLARATION_OPEN) ? '' : 'required';

		switch($element)
		{
			case 'quantity':
				return 'integer(0,null);'.$required;

			case 'weight':
				return sprintf('integer;required(line[%s][quantity],true);elc_declaration_weight', $index);

			case 'weight_dec':
				return sprintf('float;required(line[%s][quantity],true);elc_declaration_weight_dec', $index);

			case 'brand':
				return sprintf('required(line[%s][quantity],true)', $index);

			case 'product_category':
				return sprintf('required(line[%s][quantity],true)', $index);

			case 'dimension':
				return sprintf('required(line[%s][quantity],true)', $index);

			case 'trust':
				return sprintf('required(line[%s][quantity],true)', $index);

			default:
				trigger_error('no behaviour defined for element '.$element, E_USER_ERROR);
		}
	}
}
/**
 *
 *
 */
class InterfaceDeclarationFormItem extends InterfaceItem
{
	private function __getDeclarationLines($pcf)
	{
		if($this->if_obj->model['declaration']->id)
			return $this->if_obj->model['declaration']->getFormLines();

		if(isset($_POST['line']))
		{
			$rv = array();
			$defLine = $this->__getDefaultLine($pcf);
			foreach($_POST['line'] as $index => $values)
			{
				$rv[$index] = $defLine;
				$rv[$index]['TRANSACTION_ID'] = $index+1;
				$rv[$index]['QUANTITY'] = $values['quantity'];
				$rv[$index]['BRAND_NAME'] = isset($values['brand'])?'select':'';
				$rv[$index]['PRODUCT_CATEGORY'] = isset($values['product_category'])?'select':'';
				$rv[$index]['PRODUCT_CATEGORY_DESCRIPTION'] = '';
				$rv[$index]['CPD_ID'] = isset($values['dimension'])?'select':'';
				$rv[$index]['WEIGHT'] = isset($values['weight'])?$values['weight']:'';
				$rv[$index]['WEIGHT_DEC'] = isset($values['weight_dec'])?$values['weight_dec']:'';
				$rv[$index]['CHECKBOX'] = $index?true:false;
			}
			return $rv;
		}

		return array($this->__getDefaultLine($pcf));
	}

	private function __getDefaultLine($pcf)
	{
		return array('ALTER'=>true,
				'TRANSACTION_ID'=> 1,
				'SUBROGATOR'=> '',
				'TRUST_ID'=> '',
				'VAT_NO'=> '',
				'QUANTITY'=> '',
				'FEE'=> $pcf->getFee(0,$this->if_obj->model['declaration']->crso->relation->id),
				'BRAND_NAME'=> 'select',
				'PRODUCT_CATEGORY'=> 'select',
				'PRODUCT_CATEGORY_DESCRIPTION'=> '',
				'CPD_ID'=> 'select',
				'WEIGHT'=> '',
				'WEIGHT_DEC'=> '',
				'TRANSACTION_STATUS'=> DECLARATION_OPEN,
				'CHECKBOX'=>false);
	}

	/**#@+
	 * Callback functions made from InterfaceItem
	 */
	/**
	 * @todo New line support for new declarations, re-submits
	 */
	function getDetails()
	{
		$multiLine = false;

		$country = $this->if_obj->model['country'];
		$d = $this->if_obj->model['declaration'];
		$pcf = new ProductCategoryFee($this->db_app);

		$error = '';
		if(!$pcf->getFee(0,$d->crso->relation->id))
			$error = 'Your CRSO has not entered the FEE yet';

		// If there are ANY country settings, enable multiline
		if($country->settings > 0)
			$multiLine = true;

		$hiddenCols = array('TRANSACTION_ID','ALTER','TRUST_ID');
		$trust = new Trust($this->db_app);
		$trusts = $trust->selectTrusts($d->participant->relation->id,TRUST_APPROVED,$d->crso->relation->id,$_SESSION['selected_product_id']);
		$rs = $trusts->fetchAll();
		if (!count($rs))
			$hiddenCols[] = 'SUBROGATOR';
		else
		{
			$multiLine = true;
			$hiddenCols[] = 'VAT_NO';
		}

		$trustVAT = "var Trusts = new Array();\n";
		$trustVAT .= "Trusts[0] = ''\n";
		foreach($rs as $r)
			$trustVAT .= "Trusts[".$r->TRUST_ID."] = '".$r->VAT_NO."';\n";

		// Country settings:
		if(!$country->has(HasDeclByCategory,$_SESSION['selected_product_id'])) {
			$hiddenCols[] = 'PRODUCT_CATEGORY';
			$hiddenCols[] = 'PRODUCT_CATEGORY_DESCRIPTION';
		}
		if(!$country->has(HasDeclarationDimension,$_SESSION['selected_product_id']))
			$hiddenCols[] = 'CPD_ID';
		if(!$country->has(HasDeclByBrand,$_SESSION['selected_product_id']))
			$hiddenCols[] = 'BRAND_NAME';
		if(!$country->has(HasDeclOfWeight,$_SESSION['selected_product_id']))
			$hiddenCols[] = 'WEIGHT';
		if(!$country->has(HasDeclOfWeightWithDecimals,$_SESSION['selected_product_id'])) {
			$hiddenCols[] = 'WEIGHT_DEC';
			// In case other CRSOs ask for this feature we can add a separate country setting.
			$hiddenCols[] = 'PRODUCT_CATEGORY_DESCRIPTION';
		}

		$ds = $this->__getDeclarationLines($pcf);
		$originalNumberOfLines = count($ds);

		if($multiLine)
			$ds[] = $this->_generateMultiLineInterface($ds);
		$tf = new TableFactory($ds,$this->langIf);

		$tf->setValueProcessor(TF_ALL,array($this,'tfValueProcessor'));
		$tf->setRowProcessor(array($this,'tfRowProcessor'));
		$tf->setTHProcessor(array('FEE','QUANTITY','WEIGHT','WEIGHT_DEC'),'return \'class="th-right"\';');

		$tf->hideColumns($hiddenCols);
		$tf->columnModifier(array('FEE'=>'style="text-align:right;"','QUANTITY'=>'style="text-align:right;"','WEIGHT'=>'style="text-align:right;"','WEIGHT_DEC'=>'style="text-align:right;"'));

		$cp = new CRSOProduct($this->db_app);
		$cp->set($d->crso->relation->id, $_SESSION['selected_product_id']);

		$help = ($this->parent->title != '') ? getHelp($this->langIf, $this->parent->title, array('title'=>$this->parent->title, 'top'=>'0px')) : '';
		$forf = "<div style='float:left'><img src=\"/".SITE_PATH."images/block_arrow.png\" /> ".
				$this->langIf->getText('Declaration Detail').
				"</div><div style='text-align:right'><span class='cut-off-date'>".
				$this->langIf->getText("Upcoming cut-off date").": ".$cp->getNextCutOffDateDeclaration()."</span>".$help."</div>";
		$tf->setCaption($forf, false);

		$this->text = ($error?sprintf('<span class="error">%s</span>',$this->langIf->getText($error)):'').
			$tf->getXHTMLTable('Professional',2);
		if($d->validateNewLine()&& $multiLine)
			$this->text .= '<div style="text-align: right; padding-top: 2px;"><input onclick="ifHandler.duplicateParentsLastRow(document.getElementsByName(\'line[0][quantity]\')[0]);postProcessNewLine();" type="button" value="'.$this->langIf->getText('Add Line').'" id="declaration_add_line" /></div>';

		$emptyLinesWarningTitle = $this->langIf->getText('Warning, empty Lines');
		$emptyLinesWarningContent = $this->langIf->getText('Empty lines will trigger forfeits, continue?');

		$this->postscript = $trustVAT."
			function processSave()
			{
				var cont = true;

				for(var j=0;j<$originalNumberOfLines;j++)
				{
					if(\$\$('line['+j+'][quantity]').value == '')
						cont = false;
				}
				if(!cont)
				{
					";
		if($this->if_obj->processType == 'manual')
				$this->postscript .= "continueSaveProcess();\n";
		else
			$this->postscript .= "\nMessageBox.prototype.Question('$emptyLinesWarningTitle','$emptyLinesWarningContent',function(){
						continueSaveProcess();
					});\n";
		$this->postscript .= "
				}
				else
					continueSaveProcess();
			}";
	}

	function tfValueProcessor(&$params)
	{
		//$required = ($this->if_obj->model['declaration']->aggregated_status & DECLARATION_OPEN) ? '' : 'required';
		$alter = $params['row']['ALTER'];
		switch ($params['head'])
		{
			case 'QUANTITY':
				$value = $params['value'];
				if(isset($_POST['line'])&&isset($_POST['line'][$params['i']]))
					$value = (isset($_POST['line'][$params['i']]['quantity']))?$_POST['line'][$params['i']]['quantity']:$value;

				return $alter?
					sprintf('<input behaviour="%s" class="quantity" type="text" value="%s" name="line[%s][quantity]" />',$this->if_obj->getBehaviour('quantity', $params['i']), $value,$params['i']):
					sprintf('<input type="hidden" value="%s" name="line[%s][quantity]" />%s',$params['value'],$params['i'],locale_nf($params['value']));
			break;
			case 'WEIGHT':
				$value = $params['value'];
				if(isset($_POST['line'])&&isset($_POST['line'][$params['i']]))
					$value = (isset($_POST['line'][$params['i']]['weight']))?$_POST['line'][$params['i']]['weight']:$value;
				return $alter?
					sprintf('<input behaviour="%s" class="quantity" type="text" value="%s" name="line[%s][weight]" />',
						$this->if_obj->getBehaviour('weight', $params['i']),$value,$params['i']):
					sprintf('<input type="hidden" value="%s" name="line[%s][weight]" />%s',$params['value'],$params['i'],$params['value']);
			break;
			case 'WEIGHT_DEC':
				$value = $params['value'];
				if(isset($_POST['line'])&&isset($_POST['line'][$params['i']]))
					$value = (isset($_POST['line'][$params['i']]['weight_dec']))?$_POST['line'][$params['i']]['weight_dec']:$value;
				return $alter?
					sprintf('<input behaviour="%s" class="quantity" type="text" value="%s" name="line[%s][weight_dec]" />',
						$this->if_obj->getBehaviour('weight_dec', $params['i']),$value,$params['i']):
					sprintf('<input type="hidden" value="%s" name="line[%s][weight_dec]" />%s',$params['value'],$params['i'],$params['value']);
			break;
			case 'TRANSACTION_STATUS':
				if(($params['value'] & DECLARATION_LATE)==DECLARATION_LATE)
					$params['value'] = DECLARATION_LATE;
				list($trans, $class) = Declaration::translateStatus($this->langIf,$params['value']);

				$handle = 'Declaration Forfeit_'.$this->if_obj->model['declaration']->crso->relation->name;
				// we need bytes but we dont want to break the multibyte string
				$handle = (strlen($handle)>254)?mb_strcut($handle,0,254):$handle;

				$crsoTrans = ($params['value']!=DECLARATION_FORFEIT)?'':$this->langIf->getText($handle);
				$trans = empty($crsoTrans)?$trans:$crsoTrans;
				return sprintf('<span class="%s">%s</span>',$class,$trans);
			break;
			case 'BRAND_NAME':
				$pb = $this->if_obj->model['pb'];
				if($params['value']=="select")
				{
					$value = '';
					if(isset($_POST['line'])&&isset($_POST['line'][$params['i']]))
						$value = (isset($_POST['line'][$params['i']]['brand']))?$_POST['line'][$params['i']]['brand']['id']:'';
					return
						sprintf('<select behaviour="%s" name="line[%s][brand][id]" style="width: 120px;">'.
							'<option value="">- %s -</option>%s</select>',
							$this->if_obj->getBehaviour('brand', $params['i']),
							$params['i'],$this->langIf->getText('Select Brand'),
							$pb->select($this->if_obj->model['declaration']->crso->relation->id,null,$value));

				}
				if(!$params['value'])
					return '';

				$brand = new Brand($this->db_app,$params['value'],true);
				$brand_name = (mb_strlen($brand->name) > 30) ? mb_substr($brand->name, 0, 30)."..." : $brand->name;
				$rv = sprintf('<input type="hidden" name="line[%s][brand][id]" value="%s" />',
						$params['i'],$params['value']);
				return $rv.$brand_name;
			break;
			case 'PRODUCT_CATEGORY':
				$pc = $this->if_obj->model['pc'];
				// on change reload the fee & description for this product category
				if($params['value']=="select")
				{
					$value = '';
					if(isset($_POST['line'])&&isset($_POST['line'][$params['i']]))
						$value = (isset($_POST['line'][$params['i']]['product_category']))?$_POST['line'][$params['i']]['product_category']['id']:'';
					return
						sprintf('<select behaviour="%s" name="line[%s][product_category][id]" style="width: 120px;" onchange="updatePCFee(this, \'%s\');">'.
							'<option value="">- %s -</option>%s</select>',
							$this->if_obj->getBehaviour('product_category', $params['i']),
							$params['i'],$params['i'],$this->langIf->getText('Select Product Category'),$pc->select(null,$value, '', $this->if_obj->model['declaration']->year));
				}
				if(!$params['value'])
					return '';

				$pc->set($params['value']);
				$prod_name = (mb_strlen($pc->name) > 30) ? mb_substr($pc->name, 0, 30)."..." : $pc->name;
				$rv = sprintf('<input type="hidden" name="line[%s][product_category][id]" value="%s" />',
						$params['i'],$params['value']);

				return $rv.$prod_name;
			break;
			case 'CPD_ID':
				$pc = $this->if_obj->model['cpd'];
				if($params['value']=="select")
				{
					$value = '';
					if(isset($_POST['line'])&&isset($_POST['line'][$params['i']]))
						$value = (isset($_POST['line'][$params['i']]['dimension']))?$_POST['line'][$params['i']]['dimension']['id']:'';
					return
						sprintf('<select behaviour="%s" name="line[%s][dimension][id]">'.
							'<option value="">- %s -</option>%s</select>',
							$this->if_obj->getBehaviour('dimension', $params['i']),
							$params['i'],$this->langIf->getText('Select Dimension'),$pc->select(null,$value, '', $this->if_obj->model['declaration']->year));
				}
				if(!$params['value'])
					return '';

				$pc->set($params['value']);
				$prod_name = (mb_strlen($pc->name) > 30) ? mb_substr($pc->name, 0, 30)."..." : $pc->name;
				$rv = sprintf('<input type="hidden" name="line[%s][dimension][id]" value="%s" />',
						$params['i'],$params['value']);

				return $rv.$prod_name;
			break;
			case 'PRODUCT_CATEGORY_DESCRIPTION':
				return '<span name="declaration_pc_desc">'.$params['value'].'</span>';
			case 'FEE':
				return '<span name="declaration_pc_fee">'.locale_nf($params['value'],2).'</span>';

			case 'CHECKBOX':
				if (!$params['value'])
					return '';

				$btn = new Form_Button('', $this->langIf->getText('Remove Line'));
				$btn->onclick('removeNode(this.parentNode.parentNode)');
				return $btn->getHtml();

			case 'SUBROGATOR':
			case 'VAT_NO':

				if($params['row']['SUBROGATOR']=='select')
				{
					if($params['head']=='VAT_NO')
						return '<span id="vat_no['.$params['i'].']"></span>';

					$trust = new Trust($this->db_app);
					$q = $trust->selectTrusts($this->if_obj->model['declaration']->participant->relation->id,TRUST_APPROVED,$this->if_obj->model['declaration']->crso->relation->id,$_SESSION['selected_product_id']);
					$opts = '<option value="0">- '.$this->langIf->getText('No Subrogator, for own company').' -</option>';
					while($r = $q->fetch())
						$opts .= '<option value="'.$r->TRUST_ID.'">'.$r->NAME.' ('.$r->VAT_NO.')</option>';
					//return '<select onchange="setSubrogatorVAT(this,\''.$params['i'].'\');" name="line['.$params['i'].'][trust]">'.$opts.'</select>';
					return '<select name="line['.$params['i'].'][trust]" style="width: 120px;" behaviour="'.$this->if_obj->getBehaviour('trust', $params['i']).'">'.$opts.'</select>';
				}

				if ($params['row']['SUBROGATOR'])
				{
					$p = new Relation($this->db_app, $params['row']['SUBROGATOR']);
					$p->set();
					return ($params['head'] == 'VAT_NO') ?
						$p->vat_no :
						'<input type="hidden" name="line['.$params['i'].'][trust]" value="'.$params['row']['TRUST_ID'].'" />'.$p->name.' ('.$p->vat_no.')';
				}
				else
					return ($params['head'] == 'VAT_NO') ?
						$this->if_obj->model['declaration']->participant->relation->vat_no :
						'<input type="hidden" name="line['.$params['i'].'][trust]" value="'.$params['row']['TRUST_ID'].'" />'.$this->if_obj->model['declaration']->participant->relation->name;

			break;
		}
		return $params['value'];
	}
	/**
	 * @param Array $ds
	 * @return Array $ds
	 */
	function _generateMultiLineInterface($ds)
	{
		$newline = array();
		$headers = array_keys($ds[0]);
		foreach($headers as $title)
			$newline[$title] = $this->_processNewLine($title);
		return $newline;
	}
	/**
	 *
	 */
	function _processNewLine($title)
	{
		switch($title)
		{
			case 'ALTER':
				return 'hiderow';
			break;
			case 'VAT_NO':
			case 'PRODUCT_CATEGORY_DESCRIPTION':
				return '';
			break;
			case 'SUBROGATOR':
			case 'BRAND_NAME':
			case 'PRODUCT_CATEGORY':
			case 'CPD_ID':
				return 'select';
			break;
			case 'FEE':
				$pcf = new ProductCategoryFee($this->db_app);
				$error = '';
				return $pcf->getFee(null,$this->if_obj->model['declaration']->crso->relation->id, $this->if_obj->model['declaration']->year.'-'.$this->if_obj->model['declaration']->month.'-01 00:00:00');

			break;
			case 'CHECKBOX':
				return true;
			break;
			case 'TRANSACTION_STATUS':
				return DECLARATION_OPEN;
			break;
		}
		return '';
	}
	function tfRowProcessor(&$params)
	{
		if($params['row']['ALTER'] === 'hiderow')
			$params['attr']['style'] = 'display:none;';
		return true;
	}

	/**
	 * Component Declaration does not support WEIGHT_DEC
	 */
	function getComponentForm()
	{
		// Get the components of this declaration.
		$product = new Product($this->db_app,$_SESSION['selected_product_id'],true);

		$dc = new DeclarationComponent($this->db_app);
		$dc->set($this->if_obj->model['declaration']->id); // set by declaration
		$ds = $dc->getFormLines();

		$tf = new TableFactory($ds,$this->langIf);
		$tf->setCaption(sprintf('%s (%s)',$this->langIf->getText('Component Declaration'),$product->name),false);
		$tf->setValueProcessor(array('component','quantity','weight'),array($this,'tfComponentProcessor'));
		$tf->hideColumn('weight');
		$tf->columnModifier(array('weight'=>'style="text-align:right;"','quantity'=>'style="text-align:right;"'));

		$this->text = $tf->getXHTMLTable('Professional',2);
	}

	function tfComponentProcessor($args)
	{
		$rv = $args['value'];
		switch($args['head'])
		{
			case 'component':
				$rv = $args['value']->name;
			break;
			case 'quantity':
			case 'weight':
				$rv = sprintf('<input type="text" behaviour="integer(0,null)" class="quantity" name="dc[%s][%s]" value="%s"/>',$args['row']['component']->id,$args['head'],$args['value']);
			break;
		}

		return $rv;
	}

	function getCountrySpecificText()
	{

		switch($this->if_obj->model['declaration']->crso->relation->id)
		{

			case 221:

				// hungary specific country text
				// table copy paste from trac source at http://trac.weeeblackbox.com/ticket/592
				// can test this with user Domoko0001

				$this->text = '



<style>

						table.Professional .hungary-text td {
							background-color: #EEE !important;
						}

						.hungary-text td {
							border: 1px solid #ccc;
							border-collapse: collapse;
							font-size: 0.7rem;
						}

						table.Professional tr td .hungary-text tr:nth-child(even) {
							background-color: #e5e5e5 !important;
						}


					</style>

					<table class="hungary-text" width="100%" cellpadding="1" cellspacing="0" style="page-break-before: always">
						<col width="12%">
						<col width="26%">
						<col width="12%">
						<col width="50%">
						<tbody>
							<tr valign="top">
								<td width="87" height="69" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in"><b>E+E
									berendezés csoport kódja: </b>
									</p>
								</td>
								<td width="189" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" align="center" style="margin-right: 0.04in"><b>E+E
									berendezés csoport rövid leírása: </b>
									</p>
								</td>
								<td width="88" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in; margin-bottom: 0in"><b>Az
									E+E </b>
									</p>
									<p class="western" style="margin-left: 0in; margin-bottom: 0.03in">
									<b>berendezés csoporthoz
									tartozó </b>
									</p>
									<p class="western" style="margin-left: 0in"><b>vámtarifa-számok:
									</b>
									</p>
								</td>
								<td width="328" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western"><b>A
									Vámtarifaszám megnevezése: </b>
									</p>
								</td>
							</tr>
						</tbody>
						<tbody>
							<tr valign="top">
								<td width="87" height="32" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in"><b>TL
									lámpák </b>
									</p>
								</td>
								<td width="189" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" align="justify">Fénycső,
									Két végén fejelt bármilyen formájú 
									</p>
								</td>
								<td width="88" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in">8539
									3110 
									</p>
								</td>
								<td width="328" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" align="justify">Fénycső,
									izzókatóddal, Két végén fejelt bármilyen formájú 
									</p>
								</td>
							</tr>
						</tbody>
						<tbody>
							<tr valign="top">
								<td width="87" height="48" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in"><b>CFL
									lámpák </b>
									</p>
								</td>
								<td width="189" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" align="justify">Fénycső,
									Egy végén fejelt: Energiatakarékos lámpa (kompakt fénycső) 
									</p>
								</td>
								<td width="88" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in">8539
									3190 
									</p>
								</td>
								<td width="328" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" align="justify" style="margin-right: 0.04in">Fénycső,
									izzókatóddal, Egy végén fejelt: Energiatakarékos lámpa
									(kompakt fénycső) 
									</p>
								</td>
							</tr>
						</tbody>
						<tbody>
							<tr valign="top">
								<td rowspan="7" width="87" height="17" style="border-top: 1px solid #CCC;; border-bottom: none; border-left: 1px solid #CCC; border-right: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in"><b>Speciális
									lámpák </b>
									</p>
								</td>
								<td rowspan="7" width="189" style="border-top: 1px solid #CCC;; border-bottom: none; border-left: 1px solid #CCC; border-right: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" align="justify">Speciális,
									kisüléses elven működő lámpák 
									</p>
								</td>
								<td width="88" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in">8539
									3220 
									</p>
								</td>
								<td width="328" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western">Higany-,
									nátrium- és kevertfényű lámpák 
									</p>
								</td>
							</tr>
							<tr valign="top">
								<td width="88" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in">8539
									3290</p>
								</td>
								<td width="328" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western">Fémhalogén
									lámpa</p>
								</td>
							</tr>
							<tr valign="top">
								<td width="88" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in">8539
									3920</p>
								</td>
								<td width="328" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western">Hidegkatódos
									fénycsövek (CCFL) 
									</p>
								</td>
							</tr>
							<tr valign="top">
								<td width="88" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in">8539
									3980</p>
								</td>
								<td width="328" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-right: 0.04in">Kisülési lámpák</p>
								</td>
							</tr>
							<tr valign="top">
								<td width="88" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in">8539
									4100 
									</p>
								</td>
								<td width="328" style="border-top: 1px solid #CCC;; border-bottom: 1px solid #CCC; border-left: 1px solid #CCC; border-right: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western">Ívkisülésű
									lámpák 
									</p>
								</td>
							</tr>
							<tr valign="top">
								<td width="88" style="border-top: 1px solid #CCC;; border-bottom: 1px solid #CCC; border-left: 1px solid #CCC; border-right: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in">8539
									4900</p>
								</td>
								<td width="328" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western">UV-lámpák
									és infravörös fényt kibocsátó lámpák</p>
								</td>
							</tr>
							<tr valign="top">
								<td width="88" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in">8543
									7050 
									</p>
								</td>
								<td width="328" style="border-top: 1px solid #CCC; border-bottom: 1px solid #CCC; border-left: 1px solid #CCC; border-right: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" align="justify">Napágyak,
									napozólámpák, 	amelyek gázkisüléses elven működőnek, és
									egyéb napozási elven barnító eszközök</p>
								</td>
							</tr>
						</tbody>
						<tbody>
							<tr valign="top">
								<td rowspan="5" width="87" height="16" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in"><b>LED
									lámpák </b>
									</p>
								</td>
								<td rowspan="5" width="189" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" align="justify">LED-es
									retrofit (a hagyományos izzókkal és az energiatakarékos
									lámpákkal csereszabatos és fejeléssel ellátott) lámpák.
									Ide tartoznak a nem szabványos fejeléssel (csatlakozóval)
									ellátott termékek is, melyeket a végfelhasználó a
									lámpatestben cserélni tud. 
									</p>
								</td>
								<td width="88" style="border-top: 1px solid #CCC;; border-bottom: 1px solid #CCC; border-left: 1px solid #CCC; border-right: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in">8539
									5000</p>
								</td>
								<td width="328" style="border-top: 1px solid #CCC;; border-bottom: 1px solid #CCC; border-left: 1px solid #CCC; border-right: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western">LED lámpák</p>
								</td>
							</tr>
							<tr valign="top">
								<td width="88" style="border-top: 1px solid #CCC;; border-bottom: 1px solid #CCC; border-left: 1px solid #CCC; border-right: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in">8541
									4010</p>
								</td>
								<td width="328" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" align="justify">Fénykibocsátó
									dióda, LED-es termék, melynek fejelése: B15, B22, E10, E14,
									E27, E40, G4, GY6.35, G9, GU4, GU5.3, GU10, GZ10, G5, G13, S14s,
									S14d, S15s) és/ vagy ezek kiterjesztései pl.B15d vagy GY6.35-15
									stb.</p>
								</td>
							</tr>
							<tr valign="top">
								<td width="88" style="border-top: 1px solid #CCC;; border-bottom: 1px solid #CCC; border-left: 1px solid #CCC; border-right: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in">8543
									7090    <b>1.
									csoport</b></p>
								</td>
								<td width="328" style="border-top: 1px solid #CCC; border-bottom: 1px solid #CCC; border-left: 1px solid #CCC; border-right: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" align="justify" style="margin-right: 0.04in; margin-bottom: 0in">
									&quot;Másutt nem említett
									elektromos gép és készülék egyedi feladatokra,&quot; 
									</p>
									<p class="western" align="justify" style="margin-right: 0.04in">Olyan
									LED-es termék melynek fejelése: B15, B22, E10, E14, E27, E40,
									G4, GY6.35, G9, GU4, GU5.3, GU10, GZ10, G5, G13, S14s, S14d,
									S15s) és/ vagy ezek kiterjesztései pl. B15d vagy GY6.35-15
									stb.. 
									</p>
								</td>
							</tr>
							<tr valign="top">
								<td width="88" style="border-top: 1px solid #CCC;; border-bottom: 1px solid #CCC; border-left: 1px solid #CCC; border-right: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in">9405
											  <b> 1.
									csoport</b></p>
								</td>
								<td width="328" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" align="justify" style="margin-right: 0.04in; margin-bottom: 0in">
									&quot;Lámpatest és
									világítóberendezés&quot;: 
									</p>
									<p class="western" align="justify" style="margin-right: 0.04in">Olyan
									LED-es termék melynek fejelése: B15, B22, E10, E14, E27, E40,
									G4, GY6.35, G9, GU4, GU5.3, GU10, GZ10, G5, G13, S14s, S14d,
									S15s) és/ vagy ezek kiterjesztései pl. B15d vagy GY6.35-15 stb.</p>
								</td>
							</tr>
							<tr valign="top">
								<td width="88" style="border-top: 1px solid #CCC;; border-bottom: 1px solid #CCC; border-left: 1px solid #CCC; border-right: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" style="margin-left: 0in">8539
									1000 
									</p>
								</td>
								<td width="328" style="border: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" align="justify" style="margin-right: 0.04in; margin-bottom: 0in">
									Zárt betétes fényszóró
									egység (lámpa), angolul: sealed beam lamp unit. 
									</p>
									<p class="western" align="justify" style="margin-right: 0.04in">Olyan
									LED-es termék melynek fejelése: B15, B22, E10, E14, E27, E40,
									G4, GY6.35, G9, GU4, GU5.3, GU10, GZ10, G5, G13, S14s, S14d,
									S15s) és/ vagy ezek kiterjesztései pl. B15d vagy GY6.35-15 stb.</p>
								</td>
							</tr>
						</tbody>
						<tbody>
							<tr>
								<td colspan="4" width="699" height="18" valign="top" style="border-top: 1px solid #CCC; border-bottom: 1px solid #CCC; border-left: 1px solid #CCC; border-right: 1px solid #CCC; padding-top: 0in; padding-bottom: 0in; padding-left: 0.08in; padding-right: 0.03in">
									<p class="western" align="justify"><b>Kivételek</b>
									az elektromos és elektronikai berendezések
									hulladékgazdálkodásáról szóló 197/2014. (VIII.1.)
									Kormányrendelet hatálya alól: izzólámpák, halogén lámpák
									 
									</p>
								</td>
							</tr>
						</tbody>
					</table>
				';

				break;

			case 3:
				$this->text = "S&#229;fremt du har pligt til at rapportere andre kategorier end anf&#248;rt ovenfor, bedes du klikke p&#229; \"tilf&#248;j linie\", v&#230;lge kategori og rapportere volumenen der";
				break;
			default:
				$this->text = "";
		}
	}


	/**
	 *
	 */
	function getButtons()
	{
		$this->_BarButtons = BUTTON_CANCEL;
		$this->_ButtonOptions[BUTTON_CANCEL] = array('onclick'=>"go('InterfaceDeclarationOverview.php');");
		$this->_addCustomButton('Save','button', array('onclick'=>'processSave();'));
	}
	/**#@-*/
}
