<?php
require_once(INTERFACE_OBJ."InterfaceFactory.php");

require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'Declaration.php');
require_once(ELC_CLASSES.'Trust.php');
require_once(ELC_CLASSES.'AnnualConfirmation.php');

define('BLOCK_FILTER_SELECTION',385);

/**
 * The overview of the Declaration
 *
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @version $Id$
 */
class InterfaceDeclarationOverview extends InterfaceFactory
{
	function InterfaceDeclarationOverview($db, $db_app, $langIf)
	{
		if(!isset($_SESSION['selected_participant_id']))
			throw new InterfacePermissionException('Unauthorized Access to Interface');
		// Validate Clearance Level?

		$this->FirstColWidth = 25;
		$this->delimiter = '<br />';
		parent::InterfaceFactory($db, $db_app, $langIf);

		$this->model['year'] = isset($_REQUEST['year'])?$_REQUEST['year']:$this->getYear();
		$this->model['manual'] = array();

		$this->script = sprintf("
			var crsoMonths = new Array();
			function setMonths(sel)
			{
				if(!sel.value)
				{
					$('ifI_628_CB_0').disabled = 'disabled';
					$('monthselect').style.display = 'none';
					return;
				}
				$('monthselect').style.display = '';
				fillMonths(sel.value);
			}

			function changeMonth(sel)
			{
				if(!sel.value)
					$('ifI_628_CB_0').disabled = 'disabled';
				else
					$('ifI_628_CB_0').disabled = false;
			}

			function fillMonths(crso_id)
			{
				sel = $('monthselect');
				nNodes = sel.options;
				for(var i=(nNodes.length-1);i>0;i--)
					sel.removeChild(nNodes[i]);
				crsoMonths[crso_id].forEach( function(element, index, array)
 					{
	 					opt = new Option(element,index);
						opt.innerHTML = element;
						sel.appendChild(opt);
  					},sel);
			}

			function manualDeclaration()
			{
				crsoId = $('crsoselect').value;
				if(!crsoId)
					return false;
				month = $('monthselect').value;
				if(!month)
					return false;

				document.location.href = 'InterfaceDeclarationForm.php?crso[id]='+crsoId+'&month='+month+'&year=%s';
			}
			",$this->model['year']);
	}

	function getYear()
	{
		$date =& currentDate();
		$rv = (int) $date->getYear();

		$participant_id = $_SESSION['selected_participant_id'];
		$rel = new Relation($this->db_app,$participant_id,true);
		$cp = new CRSOParticipantProduct($this->db_app);
		$crsos = $cp->getParticipantCRSOs($participant_id,'array',PARTICIPANT_PRODUCER,false,$_SESSION['selected_product_id']);

		$ac = new AnnualConfirmation($this->db_app);
		foreach($crsos as $crso_id)
		{
			list($allGood,) = $ac->hasNoUnfiledMonths($participant_id,$crso_id,($rv-1),null,$_SESSION['selected_product_id']);
			if(!$allGood)
				return ($rv-1);
		}
		return $rv;
	}

	function init($ifId)
	{
		$rv = parent::init($ifId);

		$date = currentDate();
		$year = (int)$date->getYear();

    	if($year == WEEE_APPLICATION_START_YEAR)
    		unset($this->blocks[BLOCK_FILTER_SELECTION]);

		return $rv;
	}
}

class InterfaceDeclarationOverviewItem extends InterfaceItem
{
	/**#@+
	 * Callback functions made from InterfaceItem
	 */
	/**
	 * Gets the overview
	 */
	function getOverview()
	{
		$rv = null;
		// Check for any active Remote Viewer connections.
		if(IWRegistry::isRegistered('remote_viewer')) {
			$rv = IWRegistry::registry('remote_viewer');
		}
		$participant_id = $_SESSION['selected_participant_id'];
		$rel = new Relation($this->db_app,$participant_id,true);
		setPageTitlePostfix('('.$rel->name.')');
		// Check to see if the selected participant has more then one CRSO
		$cpp = new CRSOParticipantProduct($this->db_app);
		$crsos = $cpp->getParticipantCRSOs($participant_id,'array',PARTICIPANT_PRODUCER,false,$_SESSION['selected_product_id']);

		$d = new Declaration($this->db_app);
		$ds = array();
		// Loop all the CRSO for the selected participant.

		require_once(ELC_CLASSES.'CRSOProduct.php');
		$this->cp = new CRSOProduct($this->db_app);

		foreach($crsos as $crso_id)
		{
			if($rv && $rv->crso->relation->id!=$crso_id) {
				continue;
			}
			$ac = new AnnualConfirmation($this->db_app);
			$ac->setByParticipantCRSOAndYear($participant_id,$crso_id,$this->if_obj->model['year'],$_SESSION['selected_product_id']);
			$ds[] = $this->_getOverview($participant_id,$crso_id, $ac->status);
			if($ac->status != ANNUAL_CONFIRMATION_CLOSED)
			{
				if($md = $d->hasManualDeclarations($_SESSION['selected_participant_id'],$crso_id,$_SESSION['selected_product_id'],$this->if_obj->model['year']))
					$this->if_obj->model['manual'][$crso_id] = $md;
			}
		}
		$this->text = implode('<br />',$ds);
	}
	/**
	 * @access private
	 * @param int $participant_id PARTICIPANT_RELATION_ID
	 * @param int $crso_id CRSO_RELATION_ID
	 * @return string
	 */
	function _getOverview($participant_id,$crso_id,$ac_status)
	{
		// check the annual confirmation if Button column should be hidden.
		$d = new Declaration($this->db_app);
		// Select the overview query for the Participant and CRSO
		$hideCols = array('DECLARATION_NUMBER','IS_EXPORTED','INVOICE_YEAR','CRSO_RELATION_ID');
		$ds = $d->getOverview($participant_id,$crso_id, $this->if_obj->model['year'],$_SESSION['selected_product_id']);
		// GET TRUSTS
		$trust = new Trust($this->db_app);
		$q = $trust->selectTrusts($participant_id,TRUST_APPROVED,null,$_SESSION['selected_product_id']);
		if(!($r = $q->fetch()))
			$hideCols[] = 'SUBROGATOR';

		$crso = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app,$crso_id)));
		$crso->relation->set();
		$this->crso =& $crso;

		require_once(IWARE.'TableFactoryColumnPlugin.php');
		require_once(ELC_CLASSES.'Document.php');
		if($ac_status == ANNUAL_CONFIRMATION_CLOSED)
			$hideCols[] = 'DECLARATION_ACTION';

		$tf = new TableFactoryColumnPlugin(false,$this->langIf,$this->db,$this->db_app);
		$tf->setEmptyText($this->langIf->getText('No declarations found') );

		$tf->setData($ds, 'DeclarationOverview',false,true);
		$tf->db = $this->db_app;
		$tf->declaration =& $d;
		$tf->setSortingFunctions(array(
			'DECLARATION_NUMBER'=>"'String'",
			'CRSO_NAME'=>"'String'"
			));

		$help = ($this->parent->title != '') ? getHelp($this->langIf, $this->parent->handle, array('title'=>$this->parent->handle, 'top'=>'0px')) : '';
		$caption = '<img src="/'.SITE_PATH.'images/block_arrow.png" /> '.$this->langIf->getText('Declaration Overview').' ('.$crso->relation->name.')';
		if($ac_status == ANNUAL_CONFIRMATION_CLOSED)
			$caption .= ' &nbsp;&nbsp;<span class="status_forfeit">'.$this->langIf->getText('Declarations for {YEAR} are closed, unless...', null, array('YEAR' => $this->if_obj->model['year'])).'</span>';


		$this->cp->set($crso_id, $_SESSION['selected_product_id']);
		$forf = "<div style='float:left'>".$caption.
				"</div><div style='text-align:right'><span class='cut-off-date'>".
				$this->langIf->getText("Upcoming cut-off date").": ".$this->cp->getNextCutOffDateDeclaration()."</span>".$help."</div>";

		//$tf->setCaption($caption, false);
		$tf->setCaption($forf, false);

		$tf->hideColumns($hideCols);

		$tf->setValueProcessor(TF_ALL,array($this,'tfValueProcessor'));
		$tf->setTHProcessor('QUANTITY','return \'class="th-right"\';');
		$tf->columnModifier(array('QUANTITY'=>'style="text-align:right;"'));
		return $tf->getXHTMLTable('Professional',2);
	}

	function tfValueProcessor($args)
	{
		// we need this so we can check if user has access to certain scripts
		// (declaration details and declaration form)
		$Remoras = SuckerfishMenu::create(IWRegistry::registry('db'), IWRegistry::registry('db'), IWRegistry::registry('langIf'));
		switch ($args['head'])
		{
			case 'PERIOD':
				return strftime('%B', mktime(0, 0, 0, $args['value'], 1));
			break;
			case 'DECLARATION_ID':
				// show the link only in case the user has access to the declaration details script
				if($Remoras->hasAccess(62110, 62111)) {
					return sprintf('<a href="InterfaceDeclarationDetail.php?declaration[id]=%s">%s</a>',
						$args['value'],
						str_pad($args['value'],10,'0',STR_PAD_LEFT)
						);
				}
				else {
					return str_pad($args['value'],10,'0',STR_PAD_LEFT);
				}
			break;
			case 'DECLARATION_STATUS':
				$val = $args['row']['DECLARATION_STATUS'] = Declaration::getDisplayStatus($args['value']);
				list($trans,$class) = Declaration::translateStatus($this->langIf,$val);

				$handle = 'Declaration Forfeit_'.$this->crso->relation->name;
				// we need bytes but we dont want to break the multibyte string
				$handle = (strlen($handle)>254)?mb_strcut($handle,0,254):$handle;

				$crsoTrans = ($val!=DECLARATION_FORFEIT)?'':$this->langIf->getText($handle);
				$trans = empty($crsoTrans)?$trans:$crsoTrans;
				return sprintf('<span id="declaration_status_%s" class="%s">%s</span>',
					$args['i'],$class,$trans);
			break;
			case 'DECLARATION_ACTION':
				// show the action button only in case the user has access to the declaration form script
				if($Remoras->hasAccess(62110, 62112)) {
					if(substr($args['row']['DECLARATION_NUMBER'],0,1)=='C') // Credits dont have an action
						return '';
					$d = new Declaration($this->db_app);
					if(!$d->isNewestForMonth($args['row']['DECLARATION_ID'])) // Only the last of the month has an action
						return '';

					$buttonAction = Declaration::getButtonAction($this->langIf,$args['row']['DECLARATION_STATUS'],$args['row']['IS_EXPORTED']);

					return sprintf('<input type="button" id="declaration_%s" style="width: 120px;" onclick="%s" value="%s" />',
						$args['i'],
						sprintf("document.location.href='InterfaceDeclarationForm.php?declaration[id]=%s';",$args['row']['DECLARATION_ID']),
						$buttonAction
						);
				}
				else {
					return ' - ';
				}
			break;
			case 'QUANTITY':
				return is_numeric($args['value'])?locale_nf($args['value']):$args['value'];
			case 'SUBROGATOR':
				return ($args['value']) ? $args['value'] : '-';
			case 'DOCUMENT':

				if (!$args['value'])
					return;

				$document = new ELCDocument($this->db_app);
				$document->set($args['value']);
				// TODO: make this SELECT f_elcBuildInvoiceNoForInterfaces()
				$type = (substr($args['row']['DECLARATION_NUMBER'], 0, 1) == 'C' ) ? 'CRC' : 'INVI';
				$y = substr($args['row']['INVOICE_YEAR'],2);
				$invnr = $type.$args['row']['CRSO_RELATION_ID'].$y.substr($args['row']['DECLARATION_NUMBER'],1);
				return '<a href="/'.SITE_PATH.'scripts/i-w/document/download.php?Document[id]='.$args['value'].'">'.$invnr.'</a>';;
			break;
		}
		return $args['value'];
	}

    function getYearSelect()
    {
    	$date = currentDate();
		$year = (int)$date->getYear();

    	if($year == WEEE_APPLICATION_START_YEAR)
    		$this->_hideInput = true;

    	$this->options = array();

    	for($i=WEEE_APPLICATION_START_YEAR; $i<=$year; $i++)
			$this->options[] = array('value'=>$i,'text'=>$i,'selected'=>($i==$this->if_obj->model['year']?'selected':false));
		$this->properties['onchange'] = sprintf("go('%s?year='+this.value);", 'InterfaceDeclarationOverview.php');
    }

	/**
	 *
	 */
	function getButtons()
	{
		$this->_BarButtons = 0;
		// we need this so we can check if user has access to certain scripts (declaration form)
		$Remoras = SuckerfishMenu::create(IWRegistry::registry('db'), IWRegistry::registry('db'), IWRegistry::registry('langIf'));
		// show the 'New' button only in case the user has access to the declaration form script
		if($Remoras->hasAccess(62110, 62112)) {
			$crsos = $this->if_obj->model['manual'];
			if(count($crsos)>0)
			{
				$txt = sprintf('<option value="">- %s -</option>',$this->langIf->getText('Select CRSO'));
				$js = '';
				$rel = new Relation($this->db_app);
				foreach($crsos as $crso_id => $months)
				{
					$rel->set($crso_id);
					$txt .= sprintf('<option value="%s">%s</option>',$crso_id,$rel->name);
					$js .= "crsoMonths[".$crso_id."] = new Array();\n";
					foreach($months as $month)
						$js .= sprintf("crsoMonths[%s][%s] = '%s';\n",$crso_id,$month,strftime('%B', mktime(0, 0, 0, $month, 1)));
				}
				$this->script = $js;
				$options = sprintf('<select style="display:none;" id="monthselect" name="month" onchange="changeMonth(this);"><option value="">- %s -</option></select>',$this->langIf->getText('Select Month'));
				$this->text = sprintf('<span style="float: left;"><select id="crsoselect" name="crso[id]" onchange="setMonths(this);">%s</select>&nbsp;%s</span>',$txt,$options);
				$this->_addCustomButton($this->langIf->getText('New Declaration'),'button', array('onclick'=>"manualDeclaration();",'disabled'=>'disabled'));
			}
		}

		$this->_addCustomButton($this->langIf->getText('Declaration Index'),'button', array('onclick'=>"go('InterfaceDeclarationIndex.php');"));
	}
	/**#@-*/
}
