<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'RelationUser.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @since 14-04-2006
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceDeclarationSettings extends InterfaceFactory
{
    /**
     * interface[id] = 101
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceDeclarationSettings(&$db, &$db_app, &$langIf)
    {
		if(!isset($_REQUEST['participant']['id'])||!isset($_REQUEST['crso']['id'])||!isset($_REQUEST['product']['id']))
		{
			info('User ['.$_SESSION['session_user_id'].'] tried accessing DeclarationSettings without a participant or crso');
			throw new InterfacePermissionException('User ['.$_SESSION['session_user_id'].'] tried accessing DeclarationSettings without a participant or crso');
		}
        $ru = new RelationUser($db_app);
		if(!$ru->isAdmin($_SESSION['session_user_id'],(int)$_REQUEST['participant']['id']))
		{
			info('User ['.$_SESSION['session_user_id'].'] is not the admin user for Participant ['.((int)$_REQUEST['participant']['id']).']');
			throw new InterfacePermissionException('User ['.$_SESSION['session_user_id'].
                            '] is not the admin user for Participant ['.((int)$_REQUEST['participant']['id']).']');
		}

		parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

		$crso = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app,(int)$_REQUEST['crso']['id'])));
		$participant = new Participant($this->db_app,array('relation'=>new Relation($this->db_app,(int)$_REQUEST['participant']['id'])));
		$product = new Product($this->db_app, (int)$_REQUEST['product']['id']);

		$this->model['cp'] = new CRSOParticipantProduct($this->db_app,array('crso'=>$crso,'participant'=>$participant, 'product'=>$product));
		if(!$this->model['cp']->set())
			throw new InterfacePermissionException('Unauthorized Access to Interface');

		$this->model['cp']->participant->relation->set();
		$this->model['cp']->crso->relation->set();
		$this->model['cp']->product->set();
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
		$illegalOperation = $this->langIf->getText('You are not allowed to change this property');
        if(isset($_POST['process']))
		{
			if($_POST['process'] == 'save')
			{
				$this->db_app->begin();


// 				Save nodeclaration if allowed
				$date =& currentDate();
				require_once(ELC_CLASSES.'ProvisionalDeclaration.php');
				$pr = new ProvisionalDeclaration($this->db_app);

				if($pr->validateNoDeclaration($this->model['cp']->crso->relation->id,
					$this->model['cp']->participant->relation->id, $date->getYear(), $this->model['cp']->product->id))
				{
					if (isset($_REQUEST['crsoparticipant']['no_declaration']))
						$this->model['cp']->no_declaration = 1;
					else
						$this->model['cp']->no_declaration = 0;
				}
				else
					$this->model['cp']->no_declaration = 0;

				$this->model['cp']->update();

				// check if participant is READY
				// only allow ready for participant if prev status is confirmed
				if(isset($_POST['crsoparticipant']['status'])&& $this->model['cp']->status == PARTICIPANT_STATUS_CONFIRMED
					&& $_POST['crsoparticipant']['status']==PARTICIPANT_STATUS_PARTICIPANT_READY)
				{
					require_once(ELC_CLASSES.'Trust.php');
			    	$trust = new Trust($this->db_app,array(
						'subrogator'=>$this->model['cp']->participant,
						'crso'=>$this->model['cp']->crso,
						'product'=>$this->model['cp']->product));

					if($trust->countPendingSubrogations())
                                        {
						$this->db_app->rollback();
                                                throw new Exception('Something went wrong on countPendingSubrogations');
                                        }

					// set new status
					$this->model['cp']->status = PARTICIPANT_STATUS_PARTICIPANT_READY;
					$this->model['cp']->update(); // update is needed before insertNewDeclarations()

					// copied originally from crso/InterfaceRegistrantDetails
					$cp =& $this->model['cp'];
					$cp->crso->relation->set();
					$cp->participant->relation->set();

					// Insert new declarations !!
					require_once(ELC_CLASSES.'Declaration.php');
					$d = new Declaration($this->db_app);
					$d->insertNewDeclarations(array('participant'=>$cp->participant,'crso'=>$cp->crso), $this->model['cp']->product->id);
					// Also insert the new declarations for all the trusts for this subrogator.
					require_once(ELC_CLASSES.'Trust.php');
					$t = new Trust($this->db_app);
					$q = $t->selectSubrogations($cp->participant->relation->id,TRUST_APPROVED,$cp->crso->relation->id, $cp->product->id);
					while($r = $q->fetch())
					{
						info('Verifying new Declarations for Trust ['.$r->TRUST_ID.']');
						$p = new Participant($this->db_app,array('relation'=>new Relation($this->db,$r->TRUSTEE_RELATION_ID)));
						$d->insertNewDeclarations(array('participant'=>$p,'crso'=>$cp->crso), $cp->product->id);
					}

					// Send notification to crso that participant is PARTICIPANT_STATUS_PARTICIPANT_READY
					$this->model['cp']->participant->relation->set();
					require_once(ELC_CLASSES.'Notification.php');
					Notification::send($this->db_app,
							array('subject'=>$this->langIf->getText('Notification: Participant Ready'),
								'sender'=>$this->model['cp']->participant->relation,
								'message'=>$this->langIf->getText("Notification: Participant `{PARTICIPANT}` is ready for the Declaration process", null,
									array('PARTICIPANT'=>$this->model['cp']->participant->relation->name))),
							array(array('recipient'=>$this->model['cp']->crso->relation,
								'callback'=>"go('/".SITE_PATH."scripts/elc/crso/InterfaceRegistrantDetails.php?participant[id]=".$this->model['cp']->participant->relation->id."')"
					)));

                                        log_elc_event('Particpant set ready to declare',
					    $_SESSION['session_user_id'],
                                            $this->model['cp']->product->id,
                                            $this->model['cp']->crso->relation->id,
                                            $this->model['cp']->participant->relation->id
                                        );
				}

				$this->db_app->commit();

				if (!headers_sent())
					header('Location: InterfaceDeclarationSettings.php?product[id]='.$_REQUEST['product']['id'].'&participant[id]='.$this->model['cp']->participant->relation->id.'&crso[id]='.$this->model['cp']->crso->relation->id);
				die();
			}
		}
    }
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 14-04-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceDeclarationSettingsItem extends InterfaceItem
{

   /**
     * item[852] = crso
     * item[853] = participant
     */
    function getItemValue()
    {
    	switch ($this->name)
    	{
    		case 'crso':
    			$this->text = $this->if_obj->model['cp']->crso->relation->name;;
    		break;
    		case 'participant':
    			$this->text = $this->if_obj->model['cp']->participant->relation->name;
    		break;
    		case 'product':
    			$this->text = $this->if_obj->model['cp']->product->name;
    		break;
    	}
    }

    /**
     * item[819] =
     * item[821] =
     */
    function getText()
    {
    	$this->if_obj->FirstColWidth = 75;
		$this->text = $this->title;
    }

	function getTextTwoCols()
	{
    	$this->if_obj->FirstColWidth = 75;
		$this->text = $this->title;
		$this->twoCols = true;
	}

    /**
     * item[820] = crsoparticipant[status]
     */
    function getActivation()
    {
    	$this->if_obj->FirstColWidth = 25;

    	// Check if this participant has any outstanding non-validated trust for the current CRSO
    	//
    	require_once(ELC_CLASSES.'Trust.php');
    	$trust = new Trust($this->db_app,array(
			'subrogator'=>$this->if_obj->model['cp']->participant,
			'crso'=>$this->if_obj->model['cp']->crso,
			'product'=>$this->if_obj->model['cp']->product));

		if($trust->countPendingSubrogations())
			$this->properties['disabled'] = 'disabled';

		$this->properties['value'] = PARTICIPANT_STATUS_PARTICIPANT_READY;

		$this->properties['onchange'] = jsMessageBox(MSG_BOX_QUEST, '',
			$this->langIf->getText('Are you sure you want to activate the declaration proces?'),
				'function() {$(\'ifI_'.$this->id.'\').checked=true; } ',
				'function() {$(\'ifI_'.$this->id.'\').checked=false; } ');

		// The participant can be authorized for this CRSO if the status
		// of the CRSOParticipant = PARTICIPANT_STATUS_CONFIRMED
		if($this->if_obj->model['cp']->status != PARTICIPANT_STATUS_CONFIRMED)
		{
			if($this->if_obj->model['cp']->status == PARTICIPANT_STATUS_PARTICIPANT_READY)
				$this->properties['value'] = PARTICIPANT_STATUS_PARTICIPANT_READY;
			$this->_hideInput = true;
			return;
		}
    }

    /**
     * item[822] = crsoparticipant[no_declaration]
     */
    function getNoDeclaration()
    {
    	$this->if_obj->FirstColWidth = 70;

		$this->properties['value'] = '';
		$cp =& $this->if_obj->model['cp'];

		$this->properties['checked'] = $cp->no_declaration ? 'checked' : '';
		$this->properties['value'] = 1;

		$date =& currentDate();
		require_once(ELC_CLASSES.'ProvisionalDeclaration.php');
		$pr = new ProvisionalDeclaration($this->db_app);

		// Disable the option if we are not allowed to change
		//  - not a subrogator
		//  - some provisional is left
		if(!$pr->validateNoDeclaration($cp->crso->relation->id,$cp->participant->relation->id,$date->getYear(), $cp->product->id))
			$this->properties['disabled'] = 'disabled';
    }

	function getButtons()
	{
		$this->hiddenFields = array();
		$this->hiddenFields[] = array('name'=>'participant[id]','value'=>$_REQUEST['participant']['id']);
		$this->hiddenFields[] = array('name'=>'crso[id]','value'=>$_REQUEST['crso']['id']);
		$this->hiddenFields[] = array('name'=>'product[id]','value'=>$_REQUEST['product']['id']);
		$this->_BarButtons = BUTTON_SAVE;

		/**
		 * Disabled until finished
		 * Check for PROVISIONAL_EDIT_STATUS in TBL_CRSO_PARTICIPANT
		 *
		$this->script = sprintf("
			function RequestProvisionalChange()
			{
					var params = new Array();
					params['participant_id'] = %s;
					params['crso_id'] = %s;

					// see includes/elc/rpc/RequestChangeProvisional.php
					jsbroker.call('requestChangeProvisional', params, function(p)
					{
						MessageBox.prototype.Information('', '%s');
						$('ifI_823_CB_0').disabled = true;
					});
			}
			",
			$_REQUEST['participant']['id'],
			$_REQUEST['crso']['id'],
			jsescape($this->langIf->getText('Request for Change on Provisional sent'), "'")
			);

		if($this->if_obj->model['cp']->status == PARTICIPANT_STATUS_PARTICIPANT_READY)
		{
			$props = $this->if_obj->model['cp']->provisional_change_request_status?array('disabled'=>'disabled'):array('onclick'=>'RequestProvisionalChange()');
			$this->_addCustomButton($this->langIf->getText('Request Change for Provisional'),'button', $props);
		}
		*/
	}
}

/**
 * CALLBACK[getText]
 * item[819] = {
 *     [handle] = Item: ActivationExplaination
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[821] = {
 *     [handle] = Item: NoDeclarationText
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getActivation]
 * item[820] = {
 *     [handle] = Item: ParticipantActivation
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getNoDeclaration]
 * item[822] = {
 *     [handle] = Item: NoDeclaration
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
