<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'DeclarationController.php');

/**
 *
 * @package
 * @subpackage Interface
 * @since 26-10-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceDeclarationXml extends InterfaceFactory
{
	/**
	 * interface[id] = 201
	 * @param $db Database (I_WARE_SETTINGS)
	 * @param $db_app Database (APPLICATION DATABASE)
	 * @param $langIf LanguageInterface
	 */
	function InterfaceDeclarationXml(&$db, &$db_app, &$langIf)
	{
		parent::InterfaceFactory($db, $db_app, $langIf);
		$this->FirstColWidth = 25;
		$this->delimiter = '<br />';

		$user = new ELCUser($db_app, $_SESSION['session_user_id']);
		$cpp = new CRSOParticipantProduct($db_app);

		$this->model = array();
		$this->model['dryrun'] = isset($_POST['process']) ? isset($_POST['dryrun']) : true;

		$this->model['controller'] = new DeclarationController($db_app, $cpp, $user);
		$this->model['failures'] = array();
		$this->model['xml_import_result'] = array();

		$this->lang = array(
			'invalid_file' => $this->langIf->getText('Invalid file uploaded'),
			'xml_failures' => $this->langIf->getText('Please correct the XML failures'),
			'xml_imported' => $this->langIf->getText('The overview below shows all declarations that have been imported. '),
			'dry_run_warning' => $this->langIf->getText('WARNING: Dry-run checkbox is active. Uncheck the dry-run checkbox and upload the XML file to commit declarations.'),
		);
	}

	/**
	 * @param string $thrownException
	 */
	function processInput(&$thrownException)
	{
		if(isset($_POST['process']))
		{
			$log = getLog();

			if($_POST['process'] == 'export')
			{
				info('InterfaceDeclarationXml: exporting declarations');

				$xml = $this->model['controller']->prepareOpenDeclarationXML();

				header("Cache-control: ");
				header("Pragma: ");
				header("Content-Type: application/octet-stream");
				header("Content-Disposition: attachment; filename=\"declarations.xml\" ");
				header("Content-transfer-Encoding: binary\n");
				print $xml;
				exit;
			}

			if($_POST['process'] == 'import')
			{
				info('InterfaceDeclarationXml: importing declaration file');

				if (!is_uploaded_file($_FILES['xmlfile']['tmp_name'])) {
					$this->fb->setFailures(array('xmlfile'=>$this->lang['invalid_file']));
					return;
				}

				try {
					$xml = file_get_contents($_FILES['xmlfile']['tmp_name']);
					$this->model['xml_import_result'] = $this->model['controller']->declareXml($xml, $this->model['dryrun']);

				} catch (DeclarationXmlValidationException $e) {
					$err = $e->getMessage();
					$this->fb->setFailures(array('xmlfile'=>strip_tags($err)));
					return;

				} catch (DeclarationXmlFailuresException $e) {
					$this->fb->setFailures(array('xmlfile'=>$this->lang['xml_failures']));
					$this->model['failures'] = $e->failures;
				}
			}

			// re initialise the interface factory so items are re-rendered with failure data
			$this->init($this->id);

		}
	}
}

/**
 * @package
 * @subpackage Interface
 * @since 26-10-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceDeclarationXmlItem extends InterfaceItem
{

	/**
	 * item[1584] =
	 */
	function getIntroductionText()
	{

		$this->text = '
        	<style> ul.intro-list li { list-style: disc; } </style>
    		<p>'.$this->langIf->getText('Declaration XML Intro Text').'</p>
    		<ul class="intro-list">
	    		<li style="">'.$this->langIf->getText('Download the {XSD} validation document',
		null, array('XSD'=>'<a href="/scripts/elc/participant/xsd/DeclarationSubmit.xsd" target="_blank">DeclarationSubmit.xsd</a>')).'</li>
	    	</ul>';

		$this->postscript .= '

    		function decl_xml_frm_submit(action) {

    			var frm = document.forms["interface_form"];
    			frm.process.value=action;

    			if (fb.validate(frm)) {
    				frm.submit();
    			}

    		}

    		function toggleMappingDisplay(element) {
    			element.style.display = element.style.display == "none" ? "" : "none";
    		}
    	';
	}

	/* mappings */

	function getMappingsText() {

		$html = sprintf('<p>%s</p>', $this->langIf->getText('Click on the types below to open the list of mappings.'));

		$mappings = $this->if_obj->model['controller']->getMappings();
		foreach ($mappings as $map_type=>$obj) {

			$title = $this->langIf->getText($map_type);

			if (!sizeof($obj))
			continue;

			$html .= sprintf('
				<fieldset style="margin: 5px;">
				<legend style="cursor: pointer;" onclick="toggleMappingDisplay(document.getElementById(\'tb_'.$map_type.'\'));">%s</legend>
				<table style="display: none;" id="tb_'.$map_type.'">', $title);

			foreach ($obj as $id=>$name) {
				$html .= sprintf('
					<tr>
						<td style="width: 50px;">%s</td>
						<td>%s</td>
					</tr>', $id, $name);
			}

			$html .= '</table></fieldset>';
		}

		$this->text = $html;
	}

	/* export */

	function getExportXmlText() {
		$this->text = '
    		<p>'.$this->langIf->getText('Declaration XML Export Text').'</p>
    	';
	}

	function getExportButtons() {
		$this->_BarButtons = 0;
		$this->_addCustomButton($this->langIf->getText("Export XML"), "button", array('onclick' => "decl_xml_frm_submit('export'); "));

	}

	/* import */


	function getImportXmlText() {
		$this->twoCols = true;
		$this->text = '<p>'.$this->langIf->getText('Declaration XML Import Text').'</p>';
	}

	function getDryRun() {
		$this->properties['checked'] = $this->if_obj->model['dryrun'];
	}

	function getUploadXml() {
		$this->twoCols = true;
		$this->text = '<input type="file" name="xmlfile">';

	}

	function getResultText() {

		$html = '';

		$failures = $this->if_obj->model['failures'];
		if (sizeof($failures)) {

			foreach ($failures as $failure) {
				$row = array(
					'Context' => sprintf('<pre style="font-family: monospace; font-size: 8pt; height: 200px; overflow-y: auto;">%s</pre>', print_r($failure['context'], true)),
					'Message' => $failure['message']
				);
				$ds[] = $row;
			}

			$tf = new TableFactory($ds, $this->langIf);
			$tf->setCaption('XML parsing failures');
			$this->text = $tf->getXHTMLTable('professional', 2);
			return;
		}

		$xmlResult = $this->if_obj->model['xml_import_result'];
		if (isset($xmlResult['declarations'])) {
			$declarations = $xmlResult['declarations'];

			$html = '
				<p>'.$this->if_obj->lang['xml_imported'].'</p>';

			if ($this->if_obj->model['dryrun'])
				$html .= sprintf('<p class="warning">%s</p>', $this->if_obj->lang['dry_run_warning']);


			foreach ($declarations as $declaration) {

				$declaration->crso->relation->set();
				$declaration->participant->relation->set();
				$declaration->product->set();

				$html .= '

					<div style="margin-bottom: 20px; ">
					<div style="background-color: #97AFD3; color: #fff; padding: 2px; ">'.$this->langIf->getText('Declaration').'</div>

					<table style="width: 50%; margin-bottom: 10px;">
						<tr>
							<td>'.$this->langIf->getText('year').'</td>
							<td>'.$declaration->year.'</td>
						</tr>
						<tr>
							<td>'.$this->langIf->getText('month').'</td>
							<td>'.$declaration->month.'</td>
						</tr>
						<tr>
							<td>'.$this->langIf->getText('crso').'</td>
							<td>'.$declaration->crso->relation->name.'</td>
						</tr>
						<tr>
							<td>'.$this->langIf->getText('participant').'</td>
							<td>'.$declaration->participant->relation->name.'</td>
						</tr>
						<tr>
							<td>'.$this->langIf->getText('product').'</td>
							<td>'.$declaration->product->name.'</td>
						</tr>
					</table>
				';

				$lineDs = array();
				foreach ($declaration->lines as $line) {
					$row = array();

					if ($line->brand) {
						$line->brand->set();
						$row['brand'] = $line->brand->name;
					}

					if ($line->product_category) {
						$line->product_category->set();
						$row['product_category'] = $line->product_category->name;
					}

					if ($line->dimension) {
						$line->dimension->set();
						$row['dimensions'] = $line->dimension->name;
					}

					if ($line->weight || ($line->weight===0)) {
						$row['weight'] = $line->weight;
					}

					$row['quantity'] = $line->quantity;

					$lineDs[] = $row;
				}

				$lineTf = new TableFactory($lineDs, $this->langIf);
				$lineTf->width = ' width="70%"';
				$html .= $lineTf->getXHTMLTable('professional', 2);


				if (isset($xmlResult['components'][$declaration->id])) {
					$dc = $xmlResult['components'][$declaration->id];

					$componentDs = array();
					$lines = $dc->getLines();

					foreach ($lines as $line) {
						$row = array(
							'component' => $line->component->name,
							'quantity' => $line->quantity,
							'weight' => $line->weight
						);
						$componentDs[] = $row;

					}

					$componentTf = new TableFactory($componentDs, $this->langIf);
					$componentTf->width = ' width="70%"';
					$html .= $componentTf->getXHTMLTable('professional', 2);

				}


				$html .= '</div>';


			}

			$this->text = $html;
			return;
		}
	}


	function getImportButtons() {
		$this->_BarButtons = 0;
		$this->twoCols = true;
		$this->_addCustomButton($this->langIf->getText("Import XML"), "button", array('onclick' => "decl_xml_frm_submit('import');"));

	}


}

/**
 * CALLBACK[getIntroductionText]
 * item[1584] = {
 *     [handle] = Item: Introduction text
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
