<?php
require_once(INTERFACE_OBJ."InterfaceFactory.php");
require_once(ELC_CLASSES.'RefundRequest.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'RelationUser.php');

require_once(IWARE.'TableFactoryColumnPlugin.php');

define('SHOW_ALL_POES', 1);
define('SHOW_APPROVED_POES', 2);
define('SHOW_PENDING_POES', 3);
define('SHOW_REJECTED_POES', 4);

/**
 * @package ELC
 * @subpackage Interface
 * @author
 * @copyright (2005) BaseSoft B.V.
 * @version $Id$
 */
class InterfaceExportOverview extends InterfaceFactory
{
	function InterfaceExportOverview($db, $db_app, $langIf)
	{
		$this->FirstColWidth = 25;
		$this->delimiter = '<br />';
		parent::InterfaceFactory($db, $db_app, $langIf);
		$participant_id = $_SESSION['selected_participant_id']; // FIXME: use filter

        $ru = new RelationUser($this->db_app);
        if(!$ru->hasClearance(ClearanceProofOfPayment,$_SESSION['session_user_id'],$participant_id))
       	{
			info('User ['.$_SESSION['session_user_id'].'] has no privileges for Proof of Payment for Participant ['.(isset($_REQUEST['participant'])?$_REQUEST['participant']['id']:'-').']');
			throw new InterfacePermissionException('User ['.$_SESSION['session_user_id'].'] has no privileges for Proof of Payment for Participant ['.(isset($_REQUEST['participant'])?$_REQUEST['participant']['id']:'-').']');
       	}
		$this->model['filter'] = isset($_REQUEST['f']) ? (int)$_REQUEST['f'] : SHOW_PENDING_POES;
		$this->model['year'] = isset($_REQUEST['y']) ? (int)$_REQUEST['y'] : date('Y');
		$this->model['participant'] = new Participant($this->db_app, $participant_id);
		$this->model['cp'] = new CRSOParticipantProduct($this->db_app);
		$this->model['crsos'] = $this->model['cp']->getParticipantCRSOs($this->model['participant']->id, 'array', PARTICIPANT_EXPORTER,false,$_SESSION['selected_product_id']);
	}
}

class InterfaceExportOverviewItem extends InterfaceItem
{
	/**#@+
	 * Callback functions made from InterfaceItem
	 */
	/**
	 *
	 */
	function getOverview()
	{
		$rv = null;
		// Check for any active Remote Viewer connections.
		if(IWRegistry::isRegistered('remote_viewer')) {
			$rv = IWRegistry::registry('remote_viewer');
		}
		$rr = new RefundRequest($this->db_app);

		$filter = '';
		switch($this->if_obj->model['filter'])
		{
			case SHOW_PENDING_POES:
				$filter = ' AND STATUS = '.REFUND_REQUEST_PENDING;
				break;
			case SHOW_APPROVED_POES:
				$filter = ' AND STATUS = '.REFUND_REQUEST_CONFIRMED;
				break;
			case SHOW_REJECTED_POES:
				$filter = ' AND STATUS = '.REFUND_REQUEST_REJECTED;
				break;
		}
		if($this->if_obj->model['year'] != 0)
			$filter .= ' AND YEAR(R.CREATED) = '.((int) $this->if_obj->model['year']);

		require_once(ELC_CLASSES.'CRSOProduct.php');
		$cp = new CRSOProduct($this->db_app);

		$this->text = '';
		foreach ($this->if_obj->model['crsos'] as $crso_id)
		{
			if($rv && $rv->crso->relation->id!=$crso_id)
				continue;
			$ds = $rr->getOverview($this->if_obj->model['participant']->id, $crso_id,$_SESSION['selected_product_id'], $filter);
			$rel = new Relation($this->db_app, $crso_id, true);
			$this->model['rel'] = &$rel;

			$tf = new TableFactoryColumnPlugin(false,$this->langIf,$this->db,$this->db_app);
			$tf->setData($ds, 'RefundRequestOverview',false,true);
			$tf->setEmptyText($this->langIf->getText('No proof of exports found') );
			$help = ($this->parent->title != '') ? getHelp($this->langIf, $this->parent->handle, array('title'=>$this->parent->handle, 'top'=>'0px')) : '';

			$capt = '<img src="/'.SITE_PATH.'images/block_arrow.png" /> '.$this->langIf->getText('Proof of Export Overview').' ('.$rel->name.')';
			$cp->set($crso_id, $_SESSION['selected_product_id']);
			$capt = "<div style='float:left'>".$capt.
				"</div><div style='text-align:right'><span class='cut-off-date'>".
				$this->langIf->getText("Upcoming export date").": ".$cp->getNextCutOffDateRefund()."</span>".$help."</div>";

			$tf->setCaption($capt, false);

			$tf->setValueProcessor('STATUS','list($trans,$class) = RefundRequest::translateStatus($this->langIf, $value); return \'<span class="\'.$class.\'">\'.$trans.\'</span>\';');
			$tf->setValueProcessor(array('QUANTITY'),'return is_numeric($value)?locale_nf($value):\'\';');
			$tf->columnModifier(array('QUANTITY'=>'style="text-align:right;"','AMOUNT'=>'style="text-align:right;"'));
			$tf->setTHProcessor(array('QUANTITY','AMOUNT'),'return \' class="th-right"\';');
			$tf->setValueProcessor('AMOUNT','return locale_nf($value, 2);');
			$tf->setValueProcessor(array('REFUND_REQUEST_ID'), array($this, '_fixId'));
			$tf->setValueProcessor('PROOF_OF_EXPORT', array($this, '_getDocumentName'));
			$tf->setValueProcessor('REFUND_EXPORT_DATE', array($this, '_formatDate'));
			$tf->setValueProcessor('IMPORTER_VAT_NO', array($this, '_vatNo'));
			$tf->setValueProcessor('IS_EXPORTED', array($this, 'tfBooleanIcon'));
			$tf->setValueProcessor('CREATED',array($this,'_formatDateTime'));
			$tf->setSortingFunctions(array(
				'REFUND_REQUEST_ID'=>"'SortVal'",
				'CREATED'=>"'String'",
				'REFUND_EXPORT_DATE'=>"'String'",
				'STATUS'=>"'String'",
				'QUANTITY'=>"'String'",
				'IMPORTER_VAT_NO'=>"'SortVal'",
				'COUNTRY_NAME'=>"'String'",
				'IS_EXPORTED'=>"'String'"
				));
			$this->text .= $tf->getXHTMLTable('Professional',2) . '<br/>';
		}
	}

	function _getDocumentName($args)
	{
		if(!$args['value'])
			return '';
		require_once(ELC_CLASSES.'Document.php');
		$doc = new ELCDocument($this->db_app,$args['value'],true); // for validation.

		return '<a id="proofofexport_'.$doc->id.'" href="/'.SITE_PATH.'scripts/i-w/document/download.php?Document[id]='.$doc->id.'">'.$this->langIf->getText('Export Document').'</a>';
	}

	function tfBooleanIcon($args)
	{
		$img = ($args['value']) ? 'true.gif' : 'false.gif';
    	return '<img src="/'.SITE_PATH.'images/i-w/'.$img.'" />';
	}

	function _vatNo($args)
	{
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		require_once(IWARE.'VAT.php');
		$vat = new VAT($this->db_app);
		return $vat->getVATDisplay($args['value']);
	}

	function _formatDate($args)
	{
		$pieces = explode('-',$args['value']);
		if(sizeof($pieces)!=3)
			return '';
		return $pieces[2].'-'.$pieces[1].'-'.$pieces[0];
	}

	function _formatDateTime($args)
	{
		$dt = explode(' ',$args['value']);
		$date_pieces = explode('-',$dt[0]);
		$time = substr($dt[1],0,strlen($dt[1])-3);
		if(sizeof($date_pieces)!=3)
			return '';
		return $date_pieces[2].'-'.$date_pieces[1].'-'.$date_pieces[0].' '.$time;
	}

	function _fee($args)
	{
		require_once(ELC_CLASSES.'ProductFee.php');
		$pf = new ProductFee($this->db_app);
		$fee = $pf->getFee($_SESSION['selected_product_id'], $this->model['rel']->id, $args['row']['REFUND_EXPORT_DATE']); // FIXME: product_id;
		$args['row']['FEE'] = $fee;
		return locale_nf($fee, 2);
	}

	function _fixId($args)
	{
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		return '<a id="refund_'.$args['value'].'" href="InterfaceExportForm.php?refund[id]='.$args['value'].'">'.$args['value'].'</a>';
	}

	function getButtons()
	{
		$rv = null;
		// Check for any active Remote Viewer connections.
		if(IWRegistry::isRegistered('remote_viewer')) {
			$rv = IWRegistry::registry('remote_viewer');
		}
		$hid = new Form_Hidden('participant[id]', $this->if_obj->model['participant']->id);
		$this->text = $hid->getHtml();

		if (!$rv && sizeof($this->if_obj->model['crsos']) > 1)
		{
			$sel = new Form_Select('crso[id]');
			$sel->addOption(" - ".$this->langIf->getText('Select CRSO')." - ", '');
			require_once(ELC_CLASSES.'CRSO.php');
			foreach ($this->if_obj->model['crsos'] as $crso_id)
			{
				$crso = new CRSO($this->db_app, array('relation'=>new Relation($this->db_app, $crso_id)));
				$crso->relation->set();
				$sel->addOption($crso->relation->name, $crso->relation->id);
			}
			$this->text .= '<span style="float: left">'.$sel->getHtml().'</span>';
		}
		else
		{
			reset($this->if_obj->model['crsos']);
			$crso_id = $rv ? $rv->crso->relation->id : current($this->if_obj->model['crsos']);
			$hid = new Form_Hidden('crso[id]', $crso_id);
			$this->text .= '<span style="float: left">'.$hid->getHtml().'</span>';
		}

		$this->_BarButtons = 0;
		$this->_addCustomButton($this->langIf->getText('New Proof of Export'),'button',
			array('onclick'=>"if($$('crso[id]').value == '') { ".jsMessageBox(MSG_BOX_INFO, $this->langIf->getText("Select CRSO"), $this->langIf->getText("Please select a CRSO"))." } else { go('InterfaceExportForm.php?crso[id]='+$$('crso[id]').value+'&participant[id]='+$$('participant[id]').value); }"));
	}

	function getFilterSelection()
    {
    	$this->script = "
			function refreshWithFilter()
			{
				document.location.href='InterfaceExportOverview.php?f=' + $('poeFilter').options[$('poeFilter').selectedIndex].value + '&y=' + $('poeYearFilter').options[$('poeYearFilter').selectedIndex].value;
			}
		";
    	$select = new Form_Select('f');
    	$select->setID('poeFilter');
		$select->addOption($this->langIf->getText('Registration Approved'), SHOW_APPROVED_POES);
		$select->addOption($this->langIf->getText('Registration Pending'), SHOW_PENDING_POES);
		$select->addOption($this->langIf->getText('Registration Rejected'), SHOW_REJECTED_POES);
		$select->addOption($this->langIf->getText('All'), SHOW_ALL_POES);
		$select->setSelectedValue($this->if_obj->model['filter']);

		// Show a year select here as showing all of them can cause timeouts
		$yearSelect = new Form_Select('y');
		$yearSelect->setID('poeYearFilter');
		for($year = 2006; $year <= date('Y'); $year++) {
			$yearSelect->addOption($year, $year);
		}
		$yearSelect->addOption($this->langIf->getText('All years'), 0);
		$yearSelect->setSelectedValue($this->if_obj->model['year']);

		$this->text = '<span style="float:left; padding-right: 10px;">'.$select->getHtml().'</span>';
		$this->text .= '<span style="float:left; padding-right: 10px;">'.$yearSelect->getHtml().'</span>';

    	$this->text .= '<span style="float:left;"><input type="button" id="refresh_filter" onclick="refreshWithFilter();" value="'.$this->langIf->getText('Refresh Dataset').'" /></span><span style="clear:both;"></span>';
    }
	/**#@-*/
}
