<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'Relation.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @since 07-04-2006
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright 2006 BaseSoft B.V.
 * @version $Id: InterfaceOperatorDeclarationStatus.php 3265 2006-07-06 15:48:49Z Chris Wesseling $
 */
class InterfaceOperatorDeclarationStatus extends InterfaceFactory
{
    /**
     * interface[id] = 95
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceOperatorDeclarationStatus(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 17;
        $this->delimiter = '<br />';

		if(isset($_REQUEST['crso']['id']))
		{
			$crso = new Relation($this->db_app,$_REQUEST['crso']['id'],true);
			setPageTitlePostfix('('.$crso->name.')');
		}
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {

    }
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 07-04-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id: InterfaceOperatorDeclarationStatus.php 3265 2006-07-06 15:48:49Z Chris Wesseling $
 */
class InterfaceOperatorDeclarationStatusItem extends InterfaceItem
{

    /**
     * item[802] = crso[id]
     */
    function getCRSOSelect()
    {
    	switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
				require_once(ELC_CLASSES.'CRSO.php');
				$crso = new CRSO($this->db_app);
				$q = $crso->selectCRSOs();
				break;
			case GROUP_ACCOUNTANT:
				require_once(ELC_CLASSES.'AccountantCRSO.php');
		    	$accountant_crso = new AccountantCRSO($this->db_app);
				$q = $accountant_crso->getAccountantCRSOs($_SESSION['accountant_relation_id']);
				break;
		}
		$this->properties['onchange'] = "changeCRSO(this);";
		$this->options = array(array('text'=>'- '.$this->langIf->getText('Select CRSO').' -','value'=>''));
		while($r = $q->fetch())
			$this->options[] = array('text'=>$r->NAME,'value'=>$r->CRSO_RELATION_ID, 'selected'=>(isset($_REQUEST['crso']['id'])&&($_REQUEST['crso']['id']==$r->CRSO_RELATION_ID)));

		$this->script = "

			function changeCRSO(sel)
			{
				crsoID = sel.value;
				participantID = $$('participant[id]').value;
				period = $$('period').value;

				if(crsoID)
					go('InterfaceOperatorDeclarationStatus.php?crso[id]='+crsoID+'&participant[id]='+participantID+'&period='+period);
			}
			";

    }

    /**
     * item[803] = participant[id]
     */
    function getParticipantSelect()
    {
		if(!isset($_REQUEST['crso']['id']))
			return $this->options = array(array('text'=>'- '.$this->langIf->getText('Select a CRSO first').' -','value'=>''));
		require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
		require_once(ELC_CLASSES.'Participant.php');
		$p = new Participant($this->db_app);
		$q = $p->selectOverview($_REQUEST['crso']['id']);
		$this->options = array(array('text'=>'- '.$this->langIf->getText('Select participant').' -','value'=>''));
		while($r = $q->fetch())
			$this->options[] = array('text'=>$r->PARTICIPANT_NAME,'value'=>$r->PARTICIPANT_RELATION_ID, 'selected'=>(isset($_REQUEST['participant']['id'])&&($_REQUEST['participant']['id'] == $r->PARTICIPANT_RELATION_ID)));
    }

    /**
     * item[804] = period
     */
    function getPeriodSelect()
    {
		$this->options = array(array('text'=>'- '.$this->langIf->getText('Select period').' -','value'=>''));
		for($i=1;$i<=12;$i++)
			$this->options[] = array('text'=>strftime('%B', mktime(0, 0, 0, $i, 1)),'value'=>$i, 'selected'=>(isset($_REQUEST['period'])&&($_REQUEST['period']==$i)));
    }

    /**
     * item[805] =
     */
    function getButtons()
    {
		$this->_BarButtons = 0;
		$this->_addCustomButton($this->langIf->getText('Apply Filter'),'submit');
    }

    /**
     * item[806] = 93
     */
    function getDeclarationStatusInterface()
    {
		if(isset($_REQUEST['crso']['id']))
			$this->settings['crso']['id'] = $_REQUEST['crso']['id'];
		if(isset($_REQUEST['participant']['id']))
			$this->settings['participant'] = $_REQUEST['participant'];
		if(isset($_REQUEST['period']))
			$this->settings['period'] = $_REQUEST['period'];
    }

}

/**
 * CALLBACK[getCRSOSelect]
 * item[802] = {
 *     [handle] = Item: CRSO Selection
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getParticipantSelect]
 * item[803] = {
 *     [handle] = Item: ParticipantSelect
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getPeriodSelect]
 * item[804] = {
 *     [handle] = Item: PeriodSelect
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[805] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getDeclarationStatusInterface]
 * item[806] = {
 *     [handle] = Item: DeclarationStatusOverview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
