<?php
require_once(INTERFACES.'elc/common/InterfaceParticipantRegistrationCommon.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'Product.php');
/**
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @author Martijn van den Broek <martijn@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @since 20-01-2005
 * @version $Id$
 */
class InterfaceParticipantNewCRSORegistration extends InterfaceParticipantRegistrationCommon
{
	var $thrownException = '';
	function InterfaceParticipantNewCRSORegistration($db, $db_app, $langIf)
	{
		$ru = new RelationUser($db_app);
		if(!$ru->hasClearance(ClearanceApplyAtCRSO,$_SESSION['session_user_id'],$_REQUEST['participant']['id']))
		{
			alert('User ['.$_SESSION['session_user_id'].'] has no permission new CRSO for participant ['.$_REQUEST['participant']['id'].']');
			throw new InterfacePermissionException('User ['.$_SESSION['session_user_id'].'] has no permission new CRSO for participant ['.$_REQUEST['participant']['id'].']');
		}

		$cp = new CRSOParticipantProduct($db_app);
		if($cp->set($_REQUEST['crso']['id'],$_REQUEST['participant']['id'],$_REQUEST['product']['id'] ))
		{
			alert('User ['.$_SESSION['session_user_id'].'] tried to apply with participant ['.$_REQUEST['participant']['id'].'] at CRSO ['.$_REQUEST['crso']['id'].']');
			throw new IWUserException('You Already Are Registered At This CRSO');
		}


		$this->crso_id = $_REQUEST['crso']['id'];

		$this->hiddenFields = array(
			array('name'=>'participant[id]','value'=>$_REQUEST['participant']['id']),
			array('name'=>'crso[id]','value'=>$_REQUEST['crso']['id'])
		);

		parent::InterfaceParticipantRegistrationCommon($db, $db_app, $langIf);

		$this->model['multi_product'] = false;
		$q = $cp->getProductsForCRSOParticipant($_REQUEST['participant']['id'],$_REQUEST['crso']['id']);
		if($r = $q->fetch())
		{
			$GLOBALS['page_title'] = 'Apply at Different Product';
			$this->model['multi_product'] = true;
		}

	}
	function _processModifiers()
	{
		$this->fb->unregister('relation[vat_no]','elc_vat_unique');
	}

	/**
	 * Loads the class for the blocks and items
	 */
	function loadClasses($interface_id)
	{
		$this->id = $interface_id;
		$this->classPrefix = 'InterfaceParticipantNewCRSORegistration';
		$this->appPath = 'elc/participant/';
	}

	function _saveInput()
	{
		$this->db_app->begin();

		$participant = new Participant($this->db_app,array('relation'=>new Relation($this->db_app,$_REQUEST['participant']['id'])));
		$crso_p = new CRSOParticipantProduct($this->db_app,
			array('crso'=>new CRSO($this->db_app, array('relation'=> new Relation($this->db_app,$_REQUEST['crso']['id'])))));
		$crso_p->producer_exporter = (int) $_POST['admission'];
		$crso_p->participant =& $participant;
		$crso_p->product = new Product($this->db_app,$_SESSION['selected_product_id']);
		// dd-mm-yyyy -> yyyy-mm-dd
		if (isset($_POST['participant']['business_start_date']))
		{
			$pieces = explode('-', $_POST['participant']['business_start_date']);

			if (sizeof($pieces) == 3)
				$crso_p->setObligationStartDate($pieces[2].'-'.$pieces[1].'-'.$pieces[0]);
		}
		$crso_p->insert();

		$cp = new CRSOParticipant($this->db_app);
		$cp->crso =& $crso_p->crso;
		$cp->participant =& $participant;
		$cp->set();
		if(isset($_POST['cp']['producer_reg_no']))
			$cp->producer_reg_no = $_POST['cp']['producer_reg_no'];
		if($cp->isComplete())
			$cp->update();
		else
			$cp->insert();

		$pb =& $this->_saveBrands($participant,$crso_p->crso);
		$this->_saveDocument($participant,$crso_p->crso);
		$this->_saveProvisional($participant,$crso_p->crso,$pb);

		$this->db_app->commit();

		$ru = new RelationUser($this->db_app,array('relation'=>$participant->relation));
		$user =& $ru->getRelationAdmin();

		require_once(ELC_CLASSES.'ELCMessageHandler.php');
		require_once(ELC_CLASSES.'Notification.php');
		MessageEvent::send(PARTICIPANT_REGISTRATION, array('user'=>$user, 'relationuser'=>$ru, 'crso'=>$crso_p->crso, 'participant'=>$crso_p->participant), 'ELCMessageHandler');
		Notification::send($this->db_app,
 			array('subject'=>$this->langIf->getText('Notification: New Partipant'),
 				'message'=>$this->langIf->getText("Notification: New Participant has registred")),
 			array(array(
 				'recipient'=>$crso_p->crso->relation,
 				'callback'=>"go('/".SITE_PATH."scripts/elc/crso/InterfaceRegistrantDetails.php?participant[id]=".$crso_p->participant->relation->id."')"
 			)));
		if (!headers_sent())
			header('location: /'.SITE_PATH.'scripts/elc/public/RegistrationFinish.php');
		die();
	}
}
/**
 * @package ELC
 * @subpackage Interface
 */
class InterfaceParticipantNewCRSORegistrationBlock extends InterfaceParticipantRegistrationCommonBlock
{
	function InterfaceParticipantNewCRSORegistrationBlock(&$db,&$db_app,&$langIf,$id=0,$blockmap=array(), $itemclass='')
	{
		if(empty($_POST['process']))
		{
			$this->_hideInputs = null;
			if($id==BLOCK_COMPANY||$id==BLOCK_USER||$id==BLOCK_INVOICE)
				$this->_hideInputs = true;
		}
		parent::InterfaceParticipantRegistrationCommonBlock($db,$db_app,$langIf,$id,$blockmap, $itemclass);
	}
}
/**
 * @package ELC
 * @subpackage Interface
 */
class InterfaceParticipantNewCRSORegistrationItem extends InterfaceParticipantRegistrationCommonItem
{
	function getItemValue()
	{
		$this->properties['value'] = '';
		if(!isset($this->parent->parent->rel))
		{
			$rel = new Relation($this->db_app,$_REQUEST['participant']['id']);
			$this->parent->parent->rel =& $rel;
			$participant = new Participant($this->db_app,array('relation'=>$rel));
			$ru = new RelationUser($this->db_app,array('relation'=>$rel));
			$user = $ru->getRelationAdmin();

			$user->set();
			$user->info_person->set();
			$rel->set();
			$rel->customer_contact->set();
			$rel->address->set();
			$participant->set();
			$ri = new RelationInvoice($this->db_app,array('relation'=>$rel));
			$ri->set();
			$ri->info_person->set();
			$ri->address->set();
			$this->if_obj->invoice_address = (array) $ri->address;
			$this->if_obj->isSameAsCompany = ($ri->address->address_id == $rel->address->address_id);
			$this->if_obj->relation_invoice = $this->if_obj->isSameAsCompany?(array) $rel:(array) $ri;
			$this->if_obj->invoice_person = (array) $ri->info_person;

			$this->if_obj->relation = (array) $rel;
			$this->if_obj->contact = (array) $rel->customer_contact;
			$this->if_obj->address = (array) $rel->address;
			$this->if_obj->participant = (array) $participant;

			$this->if_obj->participant['business_start_date'] = '';
			$this->if_obj->user = (array) $user->info_person;
		}
		if($this->name == 'cp[producer_reg_no]')
			return $this->getProducerRegNo();

		$name = str_replace('[','[\'',str_replace(']','\']',$this->name));
		eval('$this->properties[\'value\'] = isset($this->parent->parent->'.$name.')?$this->parent->parent->'.$name.':\'\';');
	}

	function getBusinessStartDate()
	{
		if(!isset($_POST['process'])||$_POST['process'] != 'check')
			$this->_hideInput = false;
	}

	function getProducerRegNo()
	{
		$crso = new CRSO($this->db_app);
		$crso->set($this->if_obj->crso_id);
		$c = new Country($this->db_app,$crso->getCountry(),true);
		if($c->has(HasRegNumberAsProducer,$_SESSION['selected_product_id'])&&((!isset($_POST['process'])||$_POST['process'] != 'check')))
		{
			if($this->if_obj->model['multi_product'])
			{
				require_once(ELC_CLASSES.'CRSOParticipant.php');
				$cp = new CRSOParticipant($this->db_app);
				$cp->set($this->if_obj->crso_id,$_REQUEST['participant']['id']);
				$this->properties['value'] = $cp->producer_reg_no;
				$this->properties['readonly'] = 'readonly';
			}
			else
				$this->_hideInput = false; // But only if Country Setting and not confirmationb page
		}
	}

	function getIsSameAsCompany()
	{
		$this->properties['value'] = '';
		if($this->if_obj->isSameAsCompany)
		{
			$this->properties['value'] = 1;
			$this->properties['checked'] = 'checked';
		}
	}
}
?>
