<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'RelationUser.php');
require_once(ELC_CLASSES.'User.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @since 08-03-2006
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceParticipantUserOverview extends InterfaceFactory
{
    /**
     * interface[id] = 84
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceParticipantUserOverview(&$db, &$db_app, &$langIf)
    {
		$ru = new RelationUser($db_app);
		if(!$ru->isAdmin($_SESSION['session_user_id']))
		{
			alert('['.$_SESSION['session_user_name'].'] Permission denied for non-Admin User at ParticipantUserOverview');
			throw new InterfacePermissionException('['.$_SESSION['session_user_name'].'] Permission denied for non-Admin User at ParticipantUserOverview');
		}
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';


        $txt = $this->langIf->getText("You can not delete an administrator user.");
        if(isset($_REQUEST['hasWarnedOnAdmin']) && $_REQUEST['hasWarnedOnAdmin'])
        	$this->script .= "
						addEvent(window, 'load', function() {
					".jsMessageBox( MSG_BOX_WARN, $this->langIf->getText("Warning"), $txt )."
							});
							";

		$txt = $this->langIf->getText("You cannot delete the user.");
        if(isset($_REQUEST['delUserException']) && $_REQUEST['delUserException'])
        	$this->script .= "
						addEvent(window, 'load', function() {
					".jsMessageBox( MSG_BOX_WARN, $this->langIf->getText("Error"), $txt )."
							});
							";
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        if(isset($_POST['process']))
		{
			$hasWarnedOnAdmin = false;
			$delUserException = false;

			if(isset($_POST['deleteUser']) && $_POST['process'] == 'delete')
			{
				$ru = new RelationUser($this->db_app);
				$user = new ELCUser($this->db_app);
				foreach($_POST['deleteUser'] as $participantuser)
				{
					list($uid,$participant_id) = explode(';',$participantuser);
					if($ru->isAdmin($uid, $participant_id))
					{
						if(!$hasWarnedOnAdmin)
							$hasWarnedOnAdmin = true;

						continue;
					}
					$q = $ru->getUserRelations($uid);

					try
					{
						$this->db->begin();
						$rs = $q->fetchAll();
						if(count($rs) > 1) // If more then one participant, leave the user be
							$ru->deleteByUserRelation($uid,$participant_id);
						else // Trash the user if it is the last scion
							$user->delete($uid);

						$this->db->commit();
					}
					catch (IWQueryException $e)
					{
						$delUserException = true;
						$this->db->rollback();
					}


				}
			}
			header('Location: InterfaceParticipantUserOverview.php?hasWarnedOnAdmin='.((int)$hasWarnedOnAdmin).'&delUserException='.(int)$delUserException);
			exit();
		}
    }
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 08-03-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceParticipantUserOverviewItem extends InterfaceItem
{

    /**
     * item[742] =
     */
    function getOverview()
    {
		require_once(IWARE.'TableFactoryColumnPlugin.php');
		$ru = new RelationUser($this->db_app);
		$ds = $ru->selectUserOverviewForAdmin($_SESSION['session_user_id']);

		$tf = new TableFactoryColumnPlugin(false,$this->langIf,$this->db,$this->db_app);
		$tf->setEmptyText($this->langIf->getText('No users found') );
		$tf->setData($ds, 'ParticipantUserOverview',false,true);

		$tf->setRowProcessor(array($this, '_tfRowProcessor'));
		$help = ($this->parent->title != '') ? getHelp($this->langIf, $this->parent->handle, array('title'=>$this->parent->handle, 'top'=>'0px')) : '';
		$tf->setCaption($help.'<img src="/'.SITE_PATH.'images/block_arrow.png" /> '.$this->langIf->getText('Participant User Overview'), false);
		$tf->setValueProcessor('CHECKBOX', 'return \'<input type="checkbox" name="deleteUser[]" value="\'.$row[\'USER_ID\'].\';\'.$row[\'PARTICIPANT_ID\'].\'" />\';');
		$tf->setValueProcessor('STATUS','return $value?\'<img id="useractive_\'.$row[\'USER_ID\'].\'_\'.$row[\'PARTICIPANT_ID\'].\'" src="/\'.SITE_PATH.\'images/i-w/true.gif" />\':\'<img id="userinactive_\'.$row[\'USER_ID\'].\'_\'.$row[\'PARTICIPANT_ID\'].\'" src="/\'.SITE_PATH.\'images/i-w/false.gif" />\';');
		$tf->setValueProcessor('USER_ID','return \'<a href="InterfaceParticipantUserRegistration.php?user[id]=\'.$value.\'&participant[id]=\'.$row[\'PARTICIPANT_ID\'].\'">\'.$value.\'</a>\';');
		$tf->setValueProcessor('IS_ADMIN','return $value?\'<img src="/\'.SITE_PATH.\'images/i-w/true.gif" />\':\'<img src="/\'.SITE_PATH.\'images/i-w/false.gif" />\';');
		$tf->setValueProcessor('SETTINGS', 'return \'<a id="settings_edit_\'.$row[\'USER_ID\'].\'" href="InterfaceParticipantUserSettings.php?user[id]=\'.$row[\'USER_ID\'].\'&participant[id]=\'.$row[\'PARTICIPANT_ID\'].\'">\'.$this->langIf->getText(\'Edit\').\'</a>\';');
		$tf->setValueProcessor('GENERAL_MANAGER_AUTHORIZED', array($this, '_tfValueProcessor'));
		$tf->hideColumns(array('PARTICIPANT_ID', 'CLEARANCE_LEVEL'));
		//$tf->collapseColumns(array('PARTICIPANT_NAME','PARTICIPANT_VAT'));

		$tf->id = 'ParticipantUserOverview';
		$this->text = $tf->getXHTMLTable('Professional',2);
    }

    function _tfRowProcessor($args) {

	$rv = null;
		// Check for any active Remote Viewer connections.
		if(IWRegistry::isRegistered('remote_viewer')) {
			$rv = IWRegistry::registry('remote_viewer');
		} else {
			return;
		}
		if($rv->participant->relation->id!=$args['row']['PARTICIPANT_ID'])
			return false;

    }

	function _tfValueProcessor($args)
	{
		switch ($args['head'])
		{
			case 'GENERAL_MANAGER_AUTHORIZED':
				if (!$args['value'])
					return '<img src="/'.SITE_PATH.'images/i-w/false.gif" />';

				list ($status, $class) = CRSOParticipantProduct::translateGmStatus($this->langIf, $args['value']);
				return sprintf('<span class="%s">%s</span>', $class, $status);

			break;
		}
	}

    /**
     * item[743] =
     */
    function getButtons()
    {
		$this->_BarButtons = BUTTON_DELETE;
		$this->script = '

			var participant = \'\';

			function createNewUser()
			{
				if(!(val = $(\'relationSelect\').value))
					return;
				document.location.href=\'InterfaceParticipantUserRegistration.php?participant[id]=\'+val;
			}
			';
	$rv = null;
		// Check for any active Remote Viewer connections.
		if(IWRegistry::isRegistered('remote_viewer')) {
			$rv = IWRegistry::registry('remote_viewer');
			$rv->participant->relation->set();
			$relations = array(array('id'=>$rv->participant->relation->id, 'name'=>$rv->participant->relation->name));
		} else {
			$relations = $_SESSION['session_user_relations'];
		}

		if(count($relations)>1)
		{
			$this->text = '<div style="float: left;"><select id="relationSelect">
				<option value="">- '.$this->langIf->getText('Select Participant').' -</option>';
			foreach($relations as $relation)
				$this->text .= '<option value="'.$relation['id'].'">'.$relation['name'].'</option>'."\n";
			$this->text .= '</select></div>';
		}
		else
		{
			$this->text = '<input type="hidden" id="relationSelect" value="'.$relations[0]['id'].'" />';
		}

		$this->_addCustomButton($this->langIf->getText('Add new User'),'button',array('onclick'=>"createNewUser();"));
		$this->_ButtonOptions[BUTTON_DELETE] = array('onclick'=>"$$('process').value='delete';$('interface_form').submit();");
    }

}

/**
 * CALLBACK[getOverview]
 * item[742] = {
 *     [handle] = Item: UserOverview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[743] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
