<?php
require_once(INTERFACE_OBJ."InterfaceFactory.php");

require_once(ELC_CLASSES.'ProofOfPayment.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'Relation.php');
require_once(ELC_CLASSES.'Document.php');

define('SHOW_ALL_POPS', 1);
define('SHOW_APPROVED_POPS', 2);
define('SHOW_PENDING_POPS', 3);
define('SHOW_REJECTED_POPS', 4);

/**
 *
 *
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright (2006) BaseSoft B.V.
 * @version $Id$
 */
class InterfacePaymentOverview extends InterfaceFactory
{
	/**
	 * @todo Add check if the current Participant is a REFUNDREQUESTER
	 */
	function InterfacePaymentOverview($db, $db_app, $langIf)
	{
		$this->FirstColWidth = 25;
		$this->delimiter = '<br />';
		parent::InterfaceFactory($db, $db_app, $langIf);
		$this->model['year'] = isset($_REQUEST['y']) ? (int)$_REQUEST['y'] : date('Y');
		$this->model['filter'] = isset($_REQUEST['f']) ? (int)$_REQUEST['f'] : SHOW_PENDING_POPS;
		$this->model['participant'] = new Participant($this->db_app,array('relation'=>new Relation($this->db_app,$_SESSION['selected_participant_id'])));
		$this->model['participant']->relation->set();
		setPageTitlePostfix('('.$this->model['participant']->relation->name.')');
	}
}
/**
 * @package ELC
 * @subpackage Interface
 * @copyright (2006) BaseSoft B.V.
 * @author Marten Koedam <marten@basesoft.com>
 */
class InterfacePaymentOverviewItem extends InterfaceItem
{
	/**#@+
	 * Callback functions made from InterfaceItem
	 */
	/**
	 * @todo Multiple participant option
	 */
	function getOverview()
	{
		$remote = null;
		// Check for any active Remote Viewer connections.
		if(IWRegistry::isRegistered('remote_viewer')) {
			$remote = IWRegistry::registry('remote_viewer');
		}
		$filter = '';
		switch($this->if_obj->model['filter'])
		{
			case SHOW_PENDING_POPS:
				$filter = ' AND STATUS = '.PROOF_OF_PAYMENT_PENDING;
				break;
			case SHOW_APPROVED_POPS:
				$filter = ' AND STATUS = '.PROOF_OF_PAYMENT_APPROVED;
				break;
			case SHOW_REJECTED_POPS:
				$filter = ' AND STATUS = '.PROOF_OF_PAYMENT_REJECTED;
				break;
		}
		if($this->if_obj->model['year'] != 0)
			$filter .= ' AND YEAR(PoP.CREATED) = '.((int) $this->if_obj->model['year']);

		$participant_id = $this->if_obj->model['participant']->relation->id;

		// Loop all CRSO's
		$cp = new CRSOParticipantProduct($this->db_app);
		$crsos = $cp->getParticipantCRSOs($participant_id,'array',PARTICIPANT_EXPORTER,false,$_SESSION['selected_product_id']);
		// user is not an exporter for specified product (so no access to this script)
		if(empty($crsos))
		  return;
		$p = new ProofOfPayment($this->db_app);
		$rv = '';

		require_once(ELC_CLASSES.'CRSOProduct.php');
		$this->cp = new CRSOProduct($this->db_app);

		require_once(IWARE.'TableFactoryColumnPlugin.php');
		foreach($crsos as $crso_id) {
			if($remote && $remote->crso->relation->id!=$crso_id)
				continue;
			$rv .= $this->_getOverview($p->getOverview($participant_id,$crso_id,$_SESSION['selected_product_id'], $filter),$crso_id).'<br />';
		}
		$this->text = $rv;
	}
	/**@#+
	 * @access protected
	 */
	/**
	 * @param Resultset $ds
	 */
	function _getOverview($ds,$crso_id)
	{
		$rel = new Relation($this->db_app,$crso_id,true);
		$tf = new TableFactoryColumnPlugin(false,$this->langIf,$this->db,$this->db_app);
		$tf->setEmptyText($this->langIf->getText('No payments found') );

		$tf->setData($ds, 'ProofOfPaymentOverview',false,true);

		$tf->setSortingFunctions(array(
			'INVOICE_ID'=>"'Int'",
			'COMPANY_NAME'=>"'String'",
			'VAT_NO'=>"'String'",
			'INVOICE_DATE'=>"'String'",
			'QUANTITY'=>"'String'",
			'FEE'=>"'String'",
			'AMOUNT'=>"'String'",
			'REMARK'=>"'String'",
			'STATUS'=>"'String'",
			'CREATED'=>"'String'",
			'PROOF_OF_PAYMENT_ID'=>"'SortVal'"
			));

		$help = ($this->parent->title != '') ? getHelp($this->langIf, $this->parent->handle, array('title'=>$this->parent->handle, 'top'=>'0px')) : '';
		$capt = '<img src="/'.SITE_PATH.'images/block_arrow.png" /> '.$this->langIf->getText('Proof of Payment Overview').' ('.$rel->name.')';
		$this->cp->set($crso_id, $_SESSION['selected_product_id']);
		$capt = "<div style='float:left'>".$capt.
				"</div><div style='text-align:right'><span class='cut-off-date'>".
				$this->langIf->getText("Upcoming export date").": ".$this->cp->getNextCutOffDateRefund()."</span>".$help."</div>";
		$tf->setCaption($capt, false);

		$tf->setValueProcessor('STATUS','list($trans,$class) = ProofOfPayment::translateStatus($this->langIf, $value); return \'<span class="\'.$class.\'">\'.$trans.\'</span>\';');
		$tf->setValueProcessor(array('QUANTITY'),'return is_numeric($value)?locale_nf($value):\'\';');
		$tf->setValueProcessor(array('AMOUNT'),'return is_numeric($value)?locale_nf($value,2):\'\';');
		$tf->setValueProcessor('DOCUMENT',array($this,'_getDocumentName'));
		$tf->setValueProcessor('INVOICE_DATE',array($this,'_formatDate'));
		$tf->setValueProcessor('CREATED',array($this,'_formatDateTime'));
		$tf->setValueProcessor('VAT_NO',array($this,'_getVAT'));
		$tf->setValueProcessor('COMPANY_NAME',array($this,'_getCompanyName'));
		$tf->setValueProcessor('PROOF_OF_PAYMENT_ID',array($this,'_fixID'));

		// default hide cols
		$hide = array('IS_EXPORTED','PRODUCER_RELATION_ID');

		// hide some extra columns in multiline - ticket #2034
	   	$relation = new Relation($this->db_app,$crso_id,true);
	   	$country = new Country($this->db_app,$relation->getCountry(),true);
	   	$country->initSettingsItemIdMap();
	   	if ($country->has(HasProofOfPaymentMultiline, $_SESSION['selected_product_id']))
	   		$hide = array_merge($hide, array('INVOICE_ID', 'DOCUMENT', 'FEE', 'INVOICE_DATE'));

	   	// apply hidden cols
	   	$tf->hideColumns($hide);

		$tf->columnModifier(array('FEE'=>'style="text-align:right;"','QUANTITY'=>'style="text-align:right;"','AMOUNT'=>'style="text-align:right;"'));
		$tf->setTHProcessor(array('FEE','QUANTITY','AMOUNT'),'return \' class="th-right"\';');

		return $tf->getXHTMLTable('Professional',2);
	}
	function _fixID($args)
	{
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		return '<a href ="InterfacePaymentForm.php?pop[id]='.$args['value'].'">'.$args['value'].'</a>';
	}
	function _getVAT($args)
	{
		require_once(IWARE.'VAT.php');
		$rel = new Relation($this->db_app,$args['row']['PRODUCER_RELATION_ID'],true);
		$vat = new VAT($this->db_app);

		return $vat->getVATDisplay($rel->vat_no);
	}
	function _getCompanyName($args)
	{
		$rel = new Relation($this->db_app,$args['row']['PRODUCER_RELATION_ID'],true);
		return $rel->name;
	}
	function _formatDate($args)
	{
		$pieces = explode('-',$args['value']);
		if(sizeof($pieces)!=3)
			return '';
		return $pieces[2].'-'.$pieces[1].'-'.$pieces[0];
	}

	function _formatDateTime($args)
	{
		$dt = explode(' ',$args['value']);
		$date_pieces = explode('-',$dt[0]);
		$time = substr($dt[1],0,strlen($dt[1])-3);
		if(sizeof($date_pieces)!=3)
			return '';
		return $date_pieces[2].'-'.$date_pieces[1].'-'.$date_pieces[0].' '.$time;
	}

	function _getDocumentName($args)
	{
		if(!$args['value'])
			return '';

		$doc = new ELCDocument($this->db_app,$args['value'],true); // for validation.

		return '<a id="invoice_document_'.$doc->id.'"href="/'.SITE_PATH.'scripts/i-w/document/download.php?Document[id]='.$doc->id.'">'.$this->langIf->getText('PoP Document').'</a>';
	}
	/**#@-*/
	/**
	 * In Case this current participant is registered at multiple CRSO, show a select box infront of the new item
	 */
	function getButtons()
	{
		$remote = null;
		// Check for any active Remote Viewer connections.
		if(IWRegistry::isRegistered('remote_viewer')) {
			$remote = IWRegistry::registry('remote_viewer');
		}
		$disabled = false;
		$rel = new Relation($this->db_app);
		$cp = new CRSOParticipantProduct($this->db_app);
		$crsos = $cp->getParticipantCRSOs($this->if_obj->model['participant']->relation->id,'array',PARTICIPANT_EXPORTER,false,$_SESSION['selected_product_id']);

		if($remote) {
			$this->text = '<input value="'.$remote->crso->relation->id.'" type="hidden" id="crso[id]" />';
		} elseif(sizeof($crsos)>1) {
			$disabled = true;

			$this->text = '<span style="float:left">
					<select id="crso[id]" onchange="activateButton(this);">
						<option value="">- '.$this->langIf->getText('Select CRSO').' -</option>';
			foreach($crsos as $crso_id)
			{
				$rel->set($crso_id);
				$this->text .= '<option value="'.$crso_id.'">'.$rel->name.'</option>';
			}
			$this->text .= '</select>
				</span>';
		}
		else
			$this->text = '<input value="'.current($crsos).'" type="hidden" id="crso[id]" />';

		$this->script = "
			function activateButton(sel)
			{
				if(sel.value)
					$('popButton').disabled = '';
				else
					$('popButton').disabled = 'disabled';
			}

			function newPoP()
			{
				var crso_id = $('crso[id]').value;
				document.location.href='InterfacePaymentForm.php?participant[id]=".$this->if_obj->model['participant']->relation->id."&crso[id]='+crso_id;
			}
			";
		$this->_BarButtons = 0;
		$props = array('id'=>'popButton','onclick'=>'newPoP();');
		if($disabled)
			$props['disabled'] = 'disabled';
		$this->_addCustomButton($this->langIf->getText('New Proof of Payment'),'button', $props);
	}

    /**
     * item[1064] =
     */
    function getRefundTotals()
    {
    	// FIXME: remove this deprecated function

    	$participant_id = $this->if_obj->model['participant']->relation->id;

		// Loop all CRSO's
		$cp = new CRSOParticipantProduct($this->db_app);
		$crsos = $cp->getParticipantCRSOs($participant_id,'array',PARTICIPANT_EXPORTER,false,$_SESSION['selected_product_id']);

    	// user is not an exporter for specified product (so no access to this script)
    	if(empty($crsos))
    	   return;

    	$sql = "
			SELECT 'Total Amount Approved Proof Of Payments' as TITLE,
				SUM(QUANTITY) as 'Total Quantity'
			FROM
				TBL_PROOF_OF_PAYMENT pop
			INNER JOIN TBL_PROOF_OF_PAYMENT_LINE pop_line
					ON pop.PROOF_OF_PAYMENT_ID = pop_line.PROOF_OF_PAYMENT_ID
			WHERE
				PARTICIPANT_RELATION_ID=?
				AND pop.CRSO_RELATION_ID IN (".implode(',',$crsos).")
				AND pop.STATUS=?
			UNION
			SELECT 'Total Amount Confirmed Refund Requests' as TITLE,
				SUM(rrl.QUANTITY) as 'Total Quantity'
			FROM
				TBL_REFUND_REQUEST rr
			INNER JOIN TBL_REFUND_REQUEST_LINE rrl
		 		USING (REFUND_REQUEST_ID)
			WHERE
				rr.PARTICIPANT_RELATION_ID=?
				AND rr.CRSO_RELATION_ID IN (".implode(',',$crsos).")
				AND rr.STATUS = ?
			";

    	require_once(ELC_CLASSES.'RefundRequest.php');

    	$q = $this->db_app->query($sql);
    	$q->bind_values(
    		$this->if_obj->model['participant']->relation->id,
    		PROOF_OF_PAYMENT_APPROVED,
    		$this->if_obj->model['participant']->relation->id,
    		REFUND_REQUEST_CONFIRMED
    	);

    	$q->execute();

    	$tf = new TableFactory($q, $this->langIf);
    	$tf->setValueProcessor('TITLE', 'return $this->langIf->getText($value);');
    	$tf->setValueProcessor('Total Quantity', 'return is_numeric($value)?locale_nf($value,2):\'0\';');
    	$tf->columnModifier(array('Total Quantity'=>'style="text-align:right;"'));
    	$this->text = $tf->getXHTMLTable('Professional', 2);
    }

    function getFilterSelection()
    {
    	$this->script = "
			function refreshWithFilter()
			{
				document.location.href='InterfacePaymentOverview.php?f=' + $('popFilter').options[$('popFilter').selectedIndex].value + '&y=' + $('popYearFilter').options[$('popYearFilter').selectedIndex].value;
			}
		";
    	$select = new Form_Select('f');
    	$select->setID('popFilter');
		$select->addOption($this->langIf->getText('Registration Approved'), SHOW_APPROVED_POPS);
		$select->addOption($this->langIf->getText('Registration Pending'), SHOW_PENDING_POPS);
		$select->addOption($this->langIf->getText('Registration Rejected'), SHOW_REJECTED_POPS);
		$select->addOption($this->langIf->getText('All'), SHOW_ALL_POPS);
		$select->setSelectedValue($this->if_obj->model['filter']);

		// Show a year select here as showing all of them can cause timeouts
		$yearSelect = new Form_Select('y');
		$yearSelect->setID('popYearFilter');
		for($year = 2006; $year <= date('Y'); $year++) {
			$yearSelect->addOption($year, $year);
		}
		$yearSelect->addOption($this->langIf->getText('All years'), 0);
		$yearSelect->setSelectedValue($this->if_obj->model['year']);

		$this->text = '<span style="float:left; padding-right: 10px;">'.$select->getHtml().'</span>';
		$this->text .= '<span style="float:left; padding-right: 10px;">'.$yearSelect->getHtml().'</span>';

    	$this->text .= '<span style="float:left;"><input type="button" id="refresh_filter" onclick="refreshWithFilter();" value="'.$this->langIf->getText('Refresh Dataset').'" /></span><span style="clear:both;"></span>';
    }

	/**#@-*/
}
