<?php
require_once(INTERFACES.'elc/common/InterfaceParticipantRegistrationCommon.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
/**
 * @package
 * @subpackage
 * @since 26-okt-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class InterfaceProvideProducerData extends InterfaceParticipantRegistrationCommon
{
	public $noOverride = false;
	function InterfaceProvideProducerData($db, $db_app, $langIf)
	{
		$ru = new RelationUser($db_app);
		// Check for admin status on this Participant
		$ru->set($_SESSION['session_user_id'],$_SESSION['selected_participant_id']);
		if(!$ru->is_admin)
			throw new InterfacePermissionException('Unauthorized Access to Interface');

		$cp = new CRSOParticipantProduct($db_app);
		$cp->set($_REQUEST['crso']['id'],$_SESSION['selected_participant_id'],$_SESSION['selected_product_id']);
		if(!($cp->participant_type_change_request & PARTICIPANT_TYPE_CHANGE_PRODUCER_AUTHORIZED))
			throw new InterfacePermissionException('Unauthorized Access to Interface');
		elseif(!($cp->producer_exporter & PARTICIPANT_PRODUCER))
			throw new InterfacePermissionException('Unauthorized Access to Interface');

		$this->model['cp'] = $cp;
		$this->crso_id = $_REQUEST['crso']['id'];
		$this->_hideInputs = true;
		if(isset($_POST['process']))
			$this->noOverride = true;
		else
		{
			$this->script = "
				addEvent(window,'load',function(){
				".jsMessageBox(MSG_BOX_INFO, $langIf->getText('Provide Producer Data'), $langIf->getText('Please provide the Producer Data. Needed before continue is possible'))."
				});";
		}

		$this->hiddenFields = array(
			array('name'=>'participant[id]','value'=>$_SESSION['selected_participant_id']),
		//	array('name'=>'crso[id]','value'=>$_REQUEST['crso']['id']),
			array('name'=>'admission','value'=>PARTICIPANT_PRODUCER+PARTICIPANT_EXPORTER)
		);
		parent::InterfaceParticipantRegistrationCommon($db, $db_app, $langIf);

		$this->fb->removePlugin('elc_reg_participant_crso');
		$this->fb->removePlugin('elc_vat_unique'); // otherwise this page will flunk
	}

	function loadClasses($interface_id)
	{
		$this->id = $interface_id;
		$this->classPrefix = 'InterfaceProvideProducerData';
		$this->appPath = 'elc/participant/';
	}

	function init($if_id)
	{
		parent::init($if_id);

		// Check if there are errors, reset noOverride and re-init
		if(isset($this->validated)&&!$this->validated&&!isset($this->tmpVar))
		{
			$this->process = '';
			$this->noOverride = false;
			$this->tmpVar = true;
			$_POST['process'] = '';
			$this->init($if_id);
		}

		// Remove the following blocks:
		// ADMISSION , we already know this.
		if(isset($this->blocks[BLOCK_ADMISSION]))
			$this->removeChild(BLOCK_ADMISSION);
	}

	function processInput(&$thrownException)
	{
		$this->process = 'check';
		if(isset($_POST['process']))
		{
			if( isset($this->validated) && !$this->validated )
				return $thrownException .= $this->thrownException;

			if($_POST['process'] == 'check')
			{
				$this->process = 'save';
				$this->_hideInputs = true;
				$this->noOverride = true;
			}
			elseif($_POST['process'] == 'secondcheck')
			{
				$this->process = 'save';
				$this->_hideInputs = true;
				$this->noOverride = true;
			}
			elseif($_POST['process'] == 'save')
				$this->_saveInput();
		}
	}

	/**
	 */
	function _saveInput()
	{
		// start transaction
		$this->db_app->begin();

		$participant =& $this->model['cp']->participant;
		if(isset($_POST['cp']['producer_reg_no']))
		{
			$cp = new CRSOParticipant($this->db_app);
			$cp->crso =& $this->model['cp']->crso;
			$cp->participant =& $participant;
			$cp->producer_reg_no = $_POST['cp']['producer_reg_no'];
			$cp->update();
		}
		//$participant->update();

		$pb =& $this->_saveBrands($participant,$this->model['cp']->crso);
		$this->_saveProvisional($participant,$this->model['cp']->crso,$pb);
		$this->model['cp']->participant_type_change_request = DB_NULL;

		//set the date in the correct format
		$new_date = $_POST['participant']['business_start_date'];
		$check = explode("-",$new_date);
		if(strlen($check[0]) != 4)
		{
			$new_date = $check[2]."-".$check[1]."-".$check[0];
		}
		$this->model['cp']->obligation_start_date = $new_date;
		$this->model['cp']->update();

		$this->db_app->commit();

		if (!headers_sent())
			header('location: /'.SITE_PATH.'scripts/elc/participant/InterfaceProvideProducerData.php?finish=true');

		die();
	}
}

class InterfaceProvideProducerDataBlock extends InterfaceParticipantRegistrationCommonBlock
{
	function InterfaceProvideProducerDataBlock(&$db,&$db_app,&$langIf,$id=0,$blockmap=array(), $itemclass='')
	{
		if(empty($_POST['process'])&&($id==BLOCK_PROV2||$id==BLOCK_PROV1))
			$this->_hideInputs = false;
		parent::InterfaceParticipantRegistrationCommonBlock($db,$db_app,$langIf,$id,$blockmap, $itemclass);
	}
}

class InterfaceProvideProducerDataItem extends InterfaceParticipantRegistrationCommonItem
{
	function getItemValue()
	{
		$this->properties['value'] = '';
		if($this->name == 'cp[producer_reg_no]'&&!$this->if_obj->noOverride)
		{
			require_once(ELC_CLASSES.'CRSOParticipant.php');
			$cp = new CRSOParticipant($this->db_app);
			$cp->participant = $this->if_obj->model['cp']->participant;
			$cp->crso = $this->if_obj->model['cp']->crso;
			$cp->set();
			$this->properties['value'] = $cp->producer_reg_no;
			$this->_hideInput = false;
			return;
		}


		if(!isset($this->if_obj->rel))
		{
			$rel = new Relation($this->db_app,$_SESSION['selected_participant_id']);
			$this->parent->parent->rel =& $rel;
			$participant = new Participant($this->db_app,array('relation'=>$rel));
			$ru = new RelationUser($this->db_app,array('relation'=>$rel));
			$user = $ru->getRelationAdmin();

			$user->set();
			$user->info_person->set();
			$rel->set();
			$rel->customer_contact->set();
			$rel->address->set();
			$participant->set();
			$ri = new RelationInvoice($this->db_app,array('relation'=>$rel));
			$ri->set();
			$ri->info_person->set();
			$ri->address->set();
			$this->if_obj->invoice_address = (array) $ri->address;
			$this->if_obj->isSameAsCompany = ($ri->address->address_id == $rel->address->address_id);
			$this->if_obj->relation_invoice = $this->if_obj->isSameAsCompany?(array) $rel:(array) $ri;
			$this->if_obj->invoice_person = (array) $ri->info_person;

			$this->if_obj->relation = (array) $rel;
			$this->if_obj->contact = (array) $rel->customer_contact;
			$this->if_obj->address = (array) $rel->address;
			$this->if_obj->participant = (array) $participant;

			$this->if_obj->participant['business_start_date'] = '';
			$this->if_obj->user = (array) $user->info_person;
		}
		$name = str_replace('[','[\'',str_replace(']','\']',$this->name));
		eval('$this->properties[\'value\'] = isset($this->parent->parent->'.$name.')?$this->parent->parent->'.$name.':\'\';');
	}
	function getRowControlValues()
	{
		$this->if_obj->noOverride || $this->_hideInput = false;
		parent::getRowControlValues();
	}
	/**
	 * Warning: no $this->_displayItemNoInput for fancy select is overridden in the parent class
	 */
	function getBrandSelect()
	{
		$this->if_obj->noOverride || $this->_hideInput = false;
		$this->fancyHeaders = array(0=>$this->langIf->getText('Available Brands'),1=>'',2=>$this->langIf->getText('Your Selected Brands'));

		$props = array('class'=>'button');
		$this->fancybuttons = array('right'=>$props, 'left'=>$props, 'allRight'=>$props, 'allLeft'=>$props);//, 'up'=>$props, 'down'=>$props);

		$this->properties['size'] = 8;
		$this->properties['style'] = 'width: 50ex;';

		$names = $this->options = $this->selectedoptions = array();

		$used = array();

		$b = new Brand($this->db_app);
		$q = $b->getByCRSO($this->if_obj->crso_id,$_SESSION['selected_product_id']);

		while($r = $q->fetch()) // TODO: REMOVE USED ITEMS
		{
			$names[$r->BRAND_ID] = $r->NAME;
			if(isset($_REQUEST['brands'])&&in_array($r->BRAND_ID,$_REQUEST['brands']))
				continue;
			$this->options[] = array('text'=>$r->NAME,'value'=>$r->BRAND_ID);
		}

		if(isset($_REQUEST['brands']))
		{
			foreach($_REQUEST['brands'] as $value)
			{
				if(is_numeric($value))
				{
					$this->selectedoptions[] = array('text'=>$names[$value],'value'=>$value);
					$used[] = $value;
				}
				else
					$this->selectedoptions[] = array('text'=>substr($value,4),'value'=>$value);
			}
		}
		else
		{
			// get Participant brands
			$pb = new ParticipantBrand($this->db_app);
			$q = $pb->getBrands($_SESSION['selected_participant_id'],$this->if_obj->crso_id, $_SESSION['selected_product_id']);
			while($r = $q->fetch())
				$this->selectedoptions[] = array('text'=>$r->NAME,'value'=>$r->BRAND_ID);
		}

		$this->if_obj->brandNames = $names;
	}
}
?>
