<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'ProvisionalDeclaration.php');
require_once(ELC_CLASSES.'ProvisionalDeclarationHistory.php');
require_once(ELC_CLASSES.'FormBehaviourPlugins.php');
require_once(ELC_CLASSES.'User.php');
require_once(ELC_CLASSES.'Declaration.php');
require_once(ELC_CLASSES.'ParticipantBrand.php');
require_once(ELC_CLASSES.'Country.php');
require_once(ELC_CLASSES.'CRSOProductDimension.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @since 04-10-2006
 * @copyright 2006 BaseSoft B.V.
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 */
class InterfaceProvisionalForm extends InterfaceFactory
{
    /**
     * interface[id] = 149
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceProvisionalForm(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        // Verify if the user is part of the requested participant
        if(!in_array($_REQUEST['participant']['id'],$_SESSION['participant_ids']))
        {
        	alert(sprintf('User [%s] has no access at participant [%s] with product [%s]', $_SESSION['session_user_id'],$_REQUEST['participant']['id'], $_SESSION['selected_product_id']));
			throw new InterfacePermissionException(sprintf('User [%s] has no access at participant [%s] with product [%s]', $_SESSION['session_user_id'],$_REQUEST['participant']['id'], $_SESSION['selected_product_id']));
        }

        $this->hiddenFields = array(
        	array('name'=>'crso[id]','value'=>$_REQUEST['crso']['id']),
        	array('name'=>'participant[id]','value'=>$_REQUEST['participant']['id'])
        	);

        $this->model['cp'] = new CRSOParticipantProduct($this->db_app);
        $this->model['cp']->set($_REQUEST['crso']['id'],$_REQUEST['participant']['id'], $_SESSION['selected_product_id']);
        $this->model['minmonth'] = (Declaration::getLastTriggeredMonth($this->db_app,$_REQUEST['crso']['id'],$_REQUEST['participant']['id'], null, $_SESSION['selected_product_id'])+1);

        $this->fb->addPlugin('elc_reg_provdouble', 'FormBehaviourPluginElcProvDouble');
        $this->fb->register('dummy_behaviour_box','elc_reg_provdouble');

    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        if(isset($_POST['process']))
        {
        	if($_POST['process'] == 'save')
        	{
        		if(!$this->fb->validate())
					return $thrownException .= $this->langIf->getText('Please correct the form errors');

        		info(sprintf('[Provisional] - Starting Change of provisional for participant [%s] at CRSO [%s]',$this->model['cp']->participant->relation->id,$this->model['cp']->crso->relation->id));
        		// Verify if the request for change has been accepted by the CRSO
        		if($this->model['cp']->provisional_change_request_status != PROVISIONAL_CHANGE_REQUEST_AUTHORIZED)
        		{
        			// trigger error
        			alert(sprintf('User [%s] has not been authorized to change the provisional for participant [%s] at CRSO [%s]', $_SESSION['session_user_id'],$_REQUEST['participant']['id'],$_REQUEST['crso']['id']));
        			exit();
        		}

        		$this->db_app->begin();
				$date =& currentDate();

        		// ProvisionalDeclaration
        		$pd = new ProvisionalDeclaration($this->db_app,array(
					'crso'=>$this->model['cp']->crso,
					'participant'=>$this->model['cp']->participant,
					'year'=>$date->getYear(),
					'product'=>$this->model['cp']->product));
				$pd->set();
        		// Remove lines for all the open months, readd them
        		$pd->saveHistoricalProv($this->model['minmonth']);
        		$pd->removeLinesFrom($this->model['minmonth']);
        		// Eepe?
        		$pd = new ProvisionalDeclaration($this->db_app,array(
					'crso'=>$this->model['cp']->crso,
					'participant'=>$this->model['cp']->participant,
					'year'=>$date->getYear(),
					'product'=>$this->model['cp']->product));

        		foreach($_POST['provisional'] as $index => $content)
        		{
        			// IF MONTH IS ALREADY TRIGGERED, SKIP THE LINE
        			if($content['month'] < $this->model['minmonth'])
        				continue;
        			$content['year'] = $date->getYear();
					if(isset($content['product_category'])) // Create a new product category
						$content['product_category'] = new ProductCategory($this->db_app,$content['product_category']);
					if(isset($content['dimension'])) // Create a new product category
						$content['dimension'] = new CRSOProductDimension($this->db_app,$content['dimension']);
					if(isset($content['brand'])) // Add brand if not exists
						$content['brand'] = new Brand($this->db_app,$content['brand']);
					$pd->addLine($content);
        		}
        		// reinsert them
        		$pd->insert();

        		// Reset the request status to OPEN
        		$this->model['cp']->provisional_change_request_status = PROVISIONAL_CHANGE_REQUEST_OPEN;
        		$this->model['cp']->update();

        		$this->db_app->commit();
        		header('Location: /'.SITE_PATH);
        		exit();
        	}
        }
    }
}
/**
 * @package
 * @subpackage Interface
 * @since 04-10-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceProvisionalFormItem extends InterfaceItem
{
    /**
     * item[1101] =
     */
    function getProvisionalDeclaration()
    {
    	$doRequestFirst = $this->langIf->getText('You have to do a request for change first');
    	$crsoHasntApproved = $this->langIf->getText('The CRSO hasnt approved your request for change yet');
    	$noMoreMonthsToEdit = $this->langIf->getText('There are no more months left to edit for this year');
		if($this->if_obj->model['cp']->provisional_change_request_status == PROVISIONAL_CHANGE_REQUEST_OPEN)
		{
			$this->text = $doRequestFirst;
			return;
		}
		if($this->if_obj->model['cp']->provisional_change_request_status == PROVISIONAL_CHANGE_REQUEST_PENDING)
		{
			$this->text = $crsoHasntApproved;
			return;
		}
		if($this->if_obj->model['minmonth']>12)
		{
			$this->text = $noMoreMonthsToEdit;
			return;
		}

		$dt =& currentDate();
		// Create a form based on the country settings.
		// Get all the non-triggered months
		$pd = new ProvisionalDeclaration($this->db_app);
		$pd->set($this->if_obj->model['cp']->crso->relation->id,$this->if_obj->model['cp']->participant->relation->id,$dt->getYear(),null,$_SESSION['selected_product_id']);

		$lines = $pd->getLines();
		if(!count($lines))
		{
			$d = currentDate();
			$line = $this->_defLine();
			$line['remove'] = '';
			$line['month'] = $d->getMonth()+1;
			$lines = array((object)$line,(object)$line);
		}

		$ds = $this->_getLines($lines);

		$tf = new TableFactory($ds,$this->langIf);
		$tf->setValueProcessor(array('remove','month','product_category','dimension','brand','quantity','weight', 'weight_dec'),array($this,'_tfValueProcessor'));
		$this->text = '<input type="hidden" name="dummy_behaviour_box" value="1" displayname="Provisonal_Declaration" behaviour="elc_reg_provdouble" />'.$tf->getXHTMLTable('Professional',2);
		$this->text .= '<div style="text-align: right; padding-top: 2px;"><input onclick="ifHandler.duplicateParentsLastRow(document.getElementsByName(\'provisional[0][quantity]\')[0]);" type="button" value="'.$this->langIf->getText('Add Line').'" id="prov_decl_add_line" /></div>';
    }

	public function _tfValueProcessor($args)
	{
		switch($args['head'])
		{
			case 'month':
				// Create month select. Select the right index
				$opts = '';
				// ONLY NON TRIGGERED
				for($i=$this->if_obj->model['minmonth'];$i<13;$i++)
				{
					$opts .= sprintf('<option value="%s"%s>%s</option>',
						$i,
						($i==$args['value']?' selected="selected"':''),
						strftime('%B', mktime(1, 1, 1, $i, 1) ));
				}
				$rv = sprintf('<select name="provisional[%s][month]">%s</select>',$args['i'],$opts);
				return $rv;
			break;
			case 'brand':
				// Get all participant brands for this crso/participant
				$pb = new ParticipantBrand($this->db_app);
				$q = $pb->getBrands($this->if_obj->model['cp']->participant->relation->id,$this->if_obj->model['cp']->crso->relation->id, $_SESSION['selected_product_id']);
				$opts = '';
				while($r = $q->fetch())
				{
					$id = isset($args['value']->id)?$args['value']->id:false;
					$opts .= sprintf('<option value="%s"%s>%s</option>',
						$r->BRAND_ID,
						($r->BRAND_ID==$id?' selected="selected"':''),
						$r->NAME);
				}
				$rv = sprintf('<select name="provisional[%s][brand]">%s</select>',$args['i'],$opts);
				return $rv;
			break;
			case 'product_category':
				// Get all crso product categories
				$pc = new ProductCategory($this->db_app);
				$q = $pc->selectOverview(array($_SESSION['selected_product_id']),$this->if_obj->model['cp']->crso->relation->id);
				$opts = '';
				while($r = $q->fetch())
				{
					$id = isset($args['value']->id)?$args['value']->id:false;
					$opts .= sprintf('<option value="%s"%s>%s</option>',
						$r->PRODUCT_CATEGORY_ID,
						($r->PRODUCT_CATEGORY_ID==$id?' selected="selected"':''),
						$r->CATEGORY_NAME);
				}
				$rv = sprintf('<select name="provisional[%s][product_category]">%s</select>',$args['i'],$opts);
				return $rv;
				// Do something
			break;
			case 'dimension':
				// Get all crso product categories
				$pc = new CRSOProductDimension($this->db_app);
				$q = $pc->selectOverview(array($_SESSION['selected_product_id']),$this->if_obj->model['cp']->crso->relation->id);
				$opts = '';
				while($r = $q->fetch())
				{
					$opts .= sprintf('<option value="%s"%s>%s</option>',
						$r->CPD_ID,
						($r->CPD_ID==$args['value']->id?' selected="selected"':''),
						$r->DIMENSION_NAME);
				}
				$rv = sprintf('<select name="provisional[%s][dimension]">%s</select>',$args['i'],$opts);
				return $rv;
				// Do something
			break;
			case 'remove':
				if($args['i']!=0)
					return sprintf('<input id="p[%s][remove]" type="button" value=" - " onclick="ifHandler.removeParentRow(this,false);" />',$args['i']);
				return sprintf('<input id="p[%s][remove]" disabled="disabled" type="button" value=" - " onclick="ifHandler.removeParentRow(this,false);" />',$args['i']);
			break;
			case 'weight_dec':
			case 'weight':
			case 'quantity':
				return sprintf('<input type="text" value="%s" name="provisional[%s][%s]" />',$args['value'],$args['i'],$args['head']);
			break;
			default:
				return $args['value'];
		}
	}
	/**#@+
	 * @access private
	 */
	private function _getLines($pdlines)
	{
		// Filter lines on non triggered. MONTH check on TBL_DECL.
		$ds = array();
		$orgline = $this->_defLine();
		foreach($pdlines as $pdline)
		{
			if($pdline->month < $this->if_obj->model['minmonth'])
				continue;
			foreach($orgline as $index => $bitbucket)
				$line[$index] = $pdline->$index;
			$line['remove'] = '';
			$ds[] = $line;
		}
		if(!isset($_POST['provisional']))
			return $ds;
		foreach($_POST['provisional'] as $i => $line)
		{
			// Brand and ProductC
			$ds[$i] = $line;
			$ds[$i]['remove'] = '';
			if(isset($ds[$i]['brand']))
				$ds[$i]['brand'] = new Brand($this->db,$ds[$i]['brand']);
			if(isset($ds[$i]['product_category']))
				$ds[$i]['product_category'] = new ProductCategory($this->db,$ds[$i]['product_category']);
			if(isset($ds[$i]['dimension']))
				$ds[$i]['dimension'] = new CRSOProductDimension($this->db,$ds[$i]['dimension']);
		}
		return $ds;
	}

	private function _defLine()
	{
		$country = new Country($this->db_app,$this->if_obj->model['cp']->crso->getCountry(),true);

		// ALWAYS PER MONTH!!
		$line = array('month'=>'');
		if($country->has(HasDeclByCategory, $this->if_obj->model['cp']->product->id))
			$line['product_category'] = '';
		if($country->has(HasDeclarationDimension, $this->if_obj->model['cp']->product->id))
			$line['dimension'] = '';
		if($country->has(HasDeclByBrand, $this->if_obj->model['cp']->product->id))
			$line['brand'] = '';
		$line['quantity'] = '';
		if($country->has(HasDeclOfWeight, $this->if_obj->model['cp']->product->id))
			$line['weight'] = '';
		if($country->has(HasDeclOfWeightWithDecimals, $this->if_obj->model['cp']->product->id))
			$line['weight_dec'] = '';
		return $line;
	}
	/**@#-*/

    /**
     * item[1103] =
     */
    function getButtons()
    {
    	$this->_BarButtons = 0;
    	$this->_addCustomButton($this->langIf->getText('Save'),'button',array('onclick'=>
    		sprintf("if(fb.validate(\$('interface_form'))){MessageBox.prototype.Question('%s','%s',function(){\$('interface_form').submit();});}",
    			$this->langIf->getText('Prov Decl Change'),$this->langIf->getText('Are the changes correct?'))));
    }

}

/**
 * CALLBACK[getProvisionalDeclaration]
 * item[1101] = {
 *     [handle] = Item: Provisonal Declaration
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[1103] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>