<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'RemoteViewer.php');
require_once(ELC_CLASSES.'RelationUser.php');
/**
 *
 * @package
 * @subpackage Interface
 * @since 27-07-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceRVInvitation extends InterfaceFactory
{
    /**
     * interface[id] = 199
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceRVInvitation(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        $ru = new RelationUser($this->db_app);
        if(!$ru->hasClearance(ClearanceRemoteViewer, $_SESSION['session_user_id'], $_SESSION['selected_participant_id'])) {
        	throw new InterfacePermissionException('No Access for the current selected participant');
        }

        $this->model['rv'] = new RemoteViewer($this->db_app, array(
        	'participant'=> new Participant($this->db_app, array('relation'=> new Relation($this->db_app, $_SESSION['selected_participant_id']))),
        	'user' => new ELCUser($this->db_app, $_SESSION['session_user_id'])
        ));

    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        if(!isset($_POST['process'])) {
        	return;
        }

        if($_POST['process'] == 'save') {

        	$this->model['rv']->invite($_POST['crso']['id']);
        	header('Location: InterfaceRVInvitation.php');
        	exit();

        } elseif ($_POST['process'] == 'cancel') {

        	$this->model['rv']->expireAll(null,null,REMOTE_VIEWER_REQUEST_CANCELLED);
        	header('Location: InterfaceRVInvitation.php');
        	exit();
        }
    }
}
/**
 * @package
 * @subpackage Interface
 * @since 27-07-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceRVInvitationItem extends InterfaceItem
{

    /**
     * item[1570] = crso[id]
     */
    function getCRSOSelection()
    {
    	require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
	$this->options = array(array('text' => ' - '.$this->langIf->getText('Select CRSO').' -', 'value' => '' ));

    	$cp = new CRSOParticipantProduct($this->db_app);

    	$q = $cp->getParticipantCRSOs($_SESSION['selected_participant_id'],'q',false,false,$_SESSION['selected_product_id']);
    	while($r = $q->fetch())
    	{
    		$this->options[] = array('text'=>((mb_strlen($r->CRSO_NAME)>35)?mb_substr($r->CRSO_NAME,0,35).'...':$r->CRSO_NAME),
    			'value'=>$r->CRSO_RELATION_ID);
    	}
    	if(count($this->options) == 2) {
    		$this->options[1]['selected'] = 'selected';
    	}
    }

    /**
     * item[1571] =
     */
    function getButtons()
    {
    	$this->_BarButtons = 0;

    	$this->_addCustomButton($this->langIf->getText('Invite'),'button', array('onclick'=>'document.getElementById(\'interface_form\').submit();','name'=>'invite'));

    }

    /**
     * item[1572] =
     */
    function getOverview()
    {
    	$q = $this->if_obj->model['rv']->getParticipantOverview();

    	$tf = new TableFactory($q, $this->langIf);
    	$tf->hideColumns(array('Id'));
    	$tf->setValueProcessor(TF_ALL, array($this, 'tfValueProcessor'));
    	$this->text = $tf->getXHTMLTable('Professional', 2);
    }

    function tfValueProcessor($args) {

    	switch($args['head']) {
    		case 'REQUEST_STATUS':
    			list($trans, $css) = $this->if_obj->model['rv']->translateStatus($this->langIf, $args['value']);
    			return sprintf('<span class="%s">%s</span>', $css, $trans);
    		break;
    		case 'ACTION':
    			if(!in_array($args['row']['REQUEST_STATUS'], array(REMOTE_VIEWER_REQUEST_PENDING, REMOTE_VIEWER_REQUEST_ACTIVE)))
    				return '-';

    			return sprintf('<input type="button" value="%s" onclick="document.getElementsByName(\'process\')[0].value=\'cancel\'; document.getElementById(\'interface_form\').submit();">', $this->langIf->getText('Cancel'));
    		break;
    	}
    	return $args['value'];
    }

}

/**
 * CALLBACK[getCRSOSelection]
 * item[1570] = {
 *     [handle] = Item: CRSO Select
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[1571] = {
 *     [handle] = Item: Button Bar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getOverview]
 * item[1572] = {
 *     [handle] = Item: Overview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
