<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'RefundRequest.php');
require_once(ELC_CLASSES.'RefundRequestConfirmation.php');
/**
 *
 * @package WEEE
 * @subpackage Interface
 * @author Marten Koedam <marten@dalines.net>
 * @since 04-01-2007
 * @copyright 2007 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceRefundConfirmation extends InterfaceFactory
{
	var $authorize_property = array('authorize_refund_request_eu', 'authorize_refund_request_non_eu');
    /**
     * interface[id] = 161
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceRefundConfirmation(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        if(!isset($_REQUEST['rrc']['id']))
        {
        	require_once(IWARE_SCRIPTS.'PermissionDenied.php');
        	exit();
        }
        $rrc = new RefundRequestConfirmation($this->db_app,$_REQUEST['rrc']['id'],true);
		$this->model['rrc'] =& $rrc;
        if(!$rrc->isComplete()
			||($rrc->participant->relation->id!=$_SESSION['selected_participant_id'])
			||($rrc->product->id!=$_SESSION['selected_product_id'])
			||($rrc->review_status == RR_CONFIRMATION_UNREVIEWED)
			||(!in_array($rrc->auth_by_type, $this->getAuthorizeType()))
		)
        {
        	require_once(IWARE_SCRIPTS.'PermissionDenied.php');
        	exit();
        }
        if($rrc->review_status == RR_CONFIRMATION_REJECTED)
        {
        	$this->can_resubmit = true;
        }
        else
        	$this->can_resubmit = false;

        $this->model['rr'] = new RefundRequest($this->db_app);
        $this->_process = isset($_POST['process'])?$_POST['process']:'';

        $this->hiddenFields = array(array('name'=>'rrc[id]','value'=>$this->model['rrc']->id));

		$this->script = sprintf("
			function print_preview()
			{
				var nwin = new NewWindow(800, 600);
				nwin.setSource('%s');
				nwin.setName('winPrint');
				nwin.setScrollbars(true);
				var win = nwin.open(); // returns a window object

				var frm = $('interface_form');
				frm.target='winPrint';
				frm.process.value='print';
				frm.submit();

				// restore
				frm.target='_self';
				frm.process.value='save';
				enableConfirmButton();
			}

			function enableConfirmButton()
			{
				$('ifI_1181_CB_1').disabled = false;
			}

			function confirm_submit()
			{
				MessageBox.prototype.Question('%s','%s',function(){
					$('interface_form').submit();
				});

			}
		", sprintf("%s?rrc[id]=%s&print=true",
			$_SERVER['PHP_SELF'], $this->model['rrc']->id),
			$this->langIf->getText('MessageBoxTitle: ConfirmConfirmation'),
			$this->langIf->getText('MessageBoxContent: ConfirmConfirmation'));
    }

    function getAuthorizeType()
    {
		require_once(ELC_CLASSES.'RelationUser.php');
		require_once(ELC_CLASSES.'Product.php');
		$ru = new RelationUser($this->db_app);
		$ru->set($_SESSION['session_user_id'], $this->model['rrc']->participant->relation->id);

		// CRSOParticipant
		$cp = new CRSOParticipantProduct($this->db_app, array('participant'=>$this->model['rrc']->participant,
			'crso'=>$this->model['rrc']->crso,'product'=>new Product($this->db_app,$_SESSION['selected_product_id'])), true);

		$authorize_type = array();

		// check if Rui is admin (hihi) so we can authorize by statitory auditor
		if ($ru->isAdmin()) {
			foreach ($this->authorize_property as $prop)
			{
				if ($cp->{$prop} == REFUND_AUTHORIZE_STATUTORY_AUDITOR) // participant setting is auth-by-auditor
					$authorize_type[] = REFUND_AUTHORIZE_STATUTORY_AUDITOR;
			}
		}

		// also check if we can authorize by general manager
		if (isset($cp->general_manager->id) && $cp->general_manager->id == $_SESSION['session_user_id'] // this user is gm
			&& $cp->general_manager_authorized == GENERAL_MANAGER_AUTHORIZED) // gm is authorized
		{
			foreach ($this->authorize_property as $prop)
				if ($cp->{$prop} == REFUND_AUTHORIZE_GENERAL_MANAGER) // participant setting is auth-by-auditor
					$authorize_type[] = REFUND_AUTHORIZE_GENERAL_MANAGER;
		}
		return $authorize_type;
    }

    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {

        if(!isset($_POST['process']))
        	return;


        if ($_POST['process'] == 'print')
    		return;

    	// The only thing we need to do is save (overwrite) the document.

    	$this->db_app->begin();

    	$rc = new RefundRequestConfirmation($this->db_app,$this->model['rrc']->id);
    	$rc->review_status = RR_CONFIRMATION_UNREVIEWED; // Ready to be reviewed once again.

    	$item = new InterfaceItem($this->db,$this->db_app,$this->langIf);
		if ($document = $item->getDocument('document[id]')) {
    		$rc->document =& $document;
    		$rc->document->makePermanent();
		}
		$rc->update();
		$this->db_app->commit();
    	header('Location: InterfaceRefundConfirmation.php?message=true');
    	exit();
    }

	function _filter($q)
	{
		$rv = InterfaceFactory::_filter($q);
		if ($this->_process == 'print')
			unset($rv[$this->id][405]);

		if(!$this->can_resubmit)
			unset($rv[$this->id][405][1180]);
		return $rv;
    }

	function getInterfaceHeader()
	{
		$rv = parent::getInterfaceHeader();

		if ($this->_process != 'print')
			return $rv;

		$p =& $this->model['rrc']->crso->relation;
		$p->set();
		$p->address->set();

		$c =& $this->model['rrc']->participant->relation;
		$c->set();
		$c->address->set();

		require_once(ELC_CLASSES.'Country.php');
		$country = new Country($this->db_app);
		$country->set($p->address->country_id);
		$p_country = $country->name;
		$country->set($c->address->country_id);
		$c_country = $country->name;

		$d = &currentDate();
		$time = $d->getTime();

		$ids = array();
		$q = $this->model['rr']->selectOverviewByRefundConfirmation($this->model['rrc']->id);
		while($r = $q->fetch())
			$ids[] =  $r->REFUND_REQUEST_ID;

		$rv .= '
			<h1>'.$this->langIf->getText('Refund Request Confirmation Print').'</h1>
			<table class="Professional" width="100%">
				<tr>
					<td width="25%">'.$this->langIf->getText('PERIOD').'</td>
					<td width="25%">
						<strong style="font-size: 10pt !important">'.
							strftime('%B',strtotime('-1 month', $time)).' '.strftime('%Y', strtotime('-1 month', $time)).
						'</strong></td>
					<td width="50%">
						'.$this->langIf->getText('Document Checksum').':
						<strong style="font-size: 10pt !important">'.RefundRequest::generateConfirmProofOfExportChecksum($ids).'</strong>
					</td>
				</tr>
				<tr>
					<td>'.$this->langIf->getText('Print date').'</td>
					<td>
						'.$d->format('d-m-Y').'
					</td>
				</tr>
				<tr>
					<td colspan="3"><br /></td>
				</tr>
				<tr>
					<td colspan="2">'.$p->name.'</td>
					<td>'.$c->name.'</td>
				</tr>
				<tr>
					<td colspan="2">'.$p->address->address_street_name.'</td>
					<td>'.$c->address->address_street_name.'</td>
				</tr>
				<tr>
					<td colspan="2">'.$p->address->city_name.', '.$p_country.'</td>
					<td>'.$c->address->city_name.', '.$c_country.'</td>
				</tr>
				<tr>
					<td colspan="2"></td>
					<td>'.$c->address->address_zipcode .', '.$c->email.'</td>
				</tr>
				<tr>
					<td>'.$this->langIf->getText('VAT Number').'</td>
					<td>'.$p->vat_no.'</td>
					<td></td>
				</tr>
				<tr>
					<td>'.$this->langIf->getText('Bank Account Number').'</td>
					<td>'.$p->bank_account_number.'</td>
					<td></td>
				</tr>
			</table>
			<br /><br />
			';

		return $rv;
	}

	function getInterfaceFooter()
	{
		$rv = parent::getInterfaceFooter();

		if ($this->_process != 'print')
			return $rv;

		$user = new ELCUser($this->db_app, $_SESSION['session_user_id'], true);
		$user->info_person->set();

		$lines = array();
		$lines[] = $user->info_person->getFullName();
		$lines[] = $user->info_person->title;
		$lines[] = $user->info_person->position;
		$lines[] = $user->info_person->department;
		$lines[] = $user->info_person->telhouse;
		$lines[] = $user->info_person->fax;
		$lines[] = $user->info_person->email;

		$rv .= '<table class="Professional">';

		foreach ($lines as $line)
		{
			if (empty($line))
				continue;

			$rv .= '<tr>
						<td>'.$line.'</td>
					</tr>';
		}

		$rv .= '	<tr>
						<td>
							<div style="border: 1px solid black; margin: 10px; width: 200px; height: 50px;"></div>
						</td>
					</tr>
				</table>
		';

		return $rv;
	}
}
/**
 * @package WEEE
 * @subpackage Interface
 * @since 04-01-2007
 * @copyright 2007 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceRefundConfirmationItem extends InterfaceItem
{
	var $_shownColumns = array();
	var $_totalFields = array();
	var $_headType = array();
    /**
     * item[1178] =
     */
    function getRejectMessage()
    {
    	if($this->if_obj->can_resubmit)
    		$this->text = ($this->if_obj->_process!='print')? sprintf('<span class="status_forfeit">%s</span>',$this->langIf->getText('Confirmation Document was rejected, provide new one')):'';
    	else
    		$this->text = '';
    }

    /**
     * item[1179] =
     */
    function getOverview()
    {
    	$ds = $this->if_obj->model['rr']->selectOverviewByRefundConfirmation($this->if_obj->model['rrc']->id);
    	$tf = new TableFactory($ds,$this->langIf);
    	$tf->setValueProcessor(TF_ALL,array($this,'tfValueProcessor'));
    	$tf->columnModifier(array('QUANTITY'=>'style="text-align:right;"','FEE'=>'style="text-align:right;"','AMOUNT'=>'style="text-align:right;"'));
    	$tf->setFooterProcessor(array($this,'_tfFooterProcessor'));
    	$this->_shownColumns = $tf->getVisibleColumns();

    	$this->text = $tf->getXHTMLTable('Professional',2);
    }

	function tfValueProcessor($args)
	{
		switch($args['head'])
		{
			case 'PROOF_OF_EXPORT':
				if(!$args['value'])
					return '';

				require_once(ELC_CLASSES.'Document.php');
				$doc = new ELCDocument($this->db_app,$args['value'],true); // for validation.

				if ($this->if_obj->_process == 'print')
					return sprintf('<img src="%s" />', REL_IMAGES.'i-w/true.gif');
				else
					return '<a href="/'.SITE_PATH.'scripts/i-w/document/download.php?Document[id]='.$doc->id.'">'.$this->langIf->getText('Export Document').'</a>';
			break;
    		case 'QUANTITY':
    			isset($this->_totalFields[$args['head']]) || $this->_totalFields[$args['head']] = 0;
    			$this->_headType[$args['head']] = 0;
    			$this->_totalFields[$args['head']] += $args['value'];
    			return locale_nf($args['value'], 0);
    		break;
		}
		return $args['value'];
	}

    function _tfFooterProcessor()
    {
    	if(!(bool) sizeof($this->_totalFields))
    	   	return false;

    	$cols = array_flip($this->_shownColumns);
		$rv = explode(',',str_repeat(',',(sizeof($this->_shownColumns)-1)));
		foreach($this->_totalFields as $head => $val)
		{
			$pos = $cols[$head];
			$round = $this->_headType[$head];
			$rv[$pos] = '<span style="float:right; font-weight: bold;">'.locale_nf($val,$round).'</span><span style="clear:both;"></span>';
		}
		return $rv;
    }

    /**
     * item[1180] = document[id]
     */
    function getConfirmationDocument()
    {

    }

    /**
     * item[1181] =
     */
    function getButtons()
    {
    	$this->_BarButtons = 0;
    	$this->_addCustomButton($this->langIf->getText('Print Preview'),'button', array('onclick'=>"print_preview();"));
    	if($this->if_obj->can_resubmit)
		{
			$this->_addCustomButton($this->langIf->getText('Resubmit Confirmation'),'button',array('onclick'=>'confirm_submit();','disabled'=>'disabled'));
		}
    }


}

/**
 * CALLBACK[getRejectMessage]
 * item[1178] = {
 *     [handle] = Item: RejectMessage
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getOverview]
 * item[1179] = {
 *     [handle] = Item: ConfirmationOverview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getConfirmationDocument]
 * item[1180] = {
 *     [handle] = Item: ConfirmationDocument
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[1181] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>