<?php
/**
 * FileDescription
 *
 * @package PACKAGE
 * @subpackage SUBPACKAGE
 * @since Dec 15, 2006
 * @author Martijn van den Broek <mjbroek@gmail.com>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'CRSOProduct.php');

/**
 *
 * @package
 * @subpackage Interface
 * @since 26-02-2007
 * @copyright 2007 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceRefundProcessStatus extends InterfaceFactory
{
    /**
     * interface[id] = 166
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceRefundProcessStatus(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        //mark if we have selected a pid or crso
        $this->model['currentPID'] = $_SESSION['selected_participant_id'];
        $this->model['currentCID'] = isset($_REQUEST['crso']['id']) && is_numeric($_REQUEST['crso']['id']) ? (int)$_REQUEST['crso']['id'] : false;
        $this->model['currentCP'] = false;

        //now loop and match to crsos
        $this->model['cps'] = array();
        $queryObj = new CRSOParticipantProduct($this->db_app);

    	$pid = $this->model['currentPID'];
    	$cids = $queryObj->getParticipantCRSOs($pid, 'array', PARTICIPANT_EXPORTER, false, $_SESSION['selected_product_id']);

	$rv = null;
		// Check for any active Remote Viewer connections.
		if(IWRegistry::isRegistered('remote_viewer')) {
			$rv = IWRegistry::registry('remote_viewer');
		}

    	foreach($cids as $cid)
    	{
		if($rv && $rv->crso->relation->id!=$cid)
			continue;
    		$cp = new CRSOParticipantProduct($this->db_app);
    		$cp->set($cid,$pid,$_SESSION['selected_product_id']);
    		$cp->participant->relation->set();
    		$cp->crso->relation->set();
    		$this->model['cps'][] = clone $cp;

        	if($this->model['currentPID'] == $pid && $this->model['currentCID'] == $cid)
				$this->model['currentCP'] = clone $cp;
			$cp = null;
    	}


        //if we have not chosen a cid or cp, but have only one possible CRSOParticipant then use it
        if(!$this->model['currentCP'] && count($this->model['cps']) == 1)
        {
        	$cp = current($this->model['cps']);
        	$this->model['currentCID'] = $cp->crso->relation->id;
        	$this->model['currentCP'] = $cp;
        }

    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {

    }
}
/**
 * @package
 * @subpackage Interface
 * @since 26-02-2007
 * @copyright 2007 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceRefundProcessStatusItem extends InterfaceItem
{
	function getCRSOSelect()
	{
		if(sizeof($this->if_obj->model['cps']) == 1)
			$this->_hideInput = true;
		else
			$this->options[] = array('text'=>'- '.$this->langIf->getText('Select CRSO').' -', 'value'=>'');

		foreach ($this->if_obj->model['cps'] as $crsoP)
		{
			if(	$this->if_obj->model['currentPID'] && $this->if_obj->model['currentPID'] != $crsoP->participant->relation->id)
				continue;
			$this->options[] = array('text'=>$crsoP->crso->relation->name,
				'value'=>$crsoP->crso->relation->id,
				'selected'=> $this->if_obj->model['currentCID'] && ($crsoP->crso->relation->id == $this->if_obj->model['currentCID']));
		}

		$this->properties['onchange'] = sprintf("go('%s?crso[id]='+this.value);", 'InterfaceRefundProcessStatus.php');

	}

	function getCutOffDate()    {
		if(!$this->if_obj->model['currentCID'])
			$this->text = '';
		else {
			$cp = new CRSOProduct($this->db_app);
			$cp->set($this->if_obj->model['currentCID'], $_SESSION['selected_product_id']);
			$this->text = "<span class='cut-off-date'>".$this->langIf->getText('Upcoming export date').": ".$cp->getNextCutOffDateRefund()."</span>";
		}
	}


	private function whoWhen($who, $when)
	{
		// who can be two persons but if they are the same only one is printed
		if (is_array($who))
		{
			if ($who[0] == $who[1])
				$who = $who[0];
			else
				$who = sprintf('%s / %s', $who[0], $who[1]);
		}

		$ds = array();
		$ds[] = array('<b>'.$this->langIf->getText('Who?').'</b>', '<b>'.$this->langIf->getText('When?').'</b>');
		$ds[] = array($who, $when);

    	$tf = new TableFactory($ds, $this->langIf);
    	$tf->decapitate();
		return sprintf('<div style="width: 250px; float: right; margin-right: 10px; margin-top: 10px; ">%s</div> <br style="clear: both;" />',$tf->getXHTMLTable('Professional', 2)).'<br />';
	}

	function tfFixWidth($args)
	{
		$args['extra'] = ' width="50%" ';
		return $args['value'];
	}

    /**
     * item[1293] =
     */
    function getRefundSettings()
    {
		$this->text = $this->whoWhen($this->langIf->getText('Admin'), $this->langIf->getText('Once, prior to step 2'));

    	$ds = array(
    		array('setting'=>$this->langIf->getText('Item: RefundRequest EU'), 'value'=>$this->getRefundSetting('authorize_refund_request_eu')),
    		array('setting'=>$this->langIf->getText('Item: RefundRequest Non-EU'), 'value'=>$this->getRefundSetting('authorize_refund_request_non_eu')),
    		array('setting'=>$this->langIf->getText('Item: RefundRequest ProofOfPayment'), 'value'=>$this->getRefundSetting('authorize_proof_of_payment')),
    	);
		if(!$this->if_obj->model['currentCP'])
			return;
		// if rui is admin, add an empty row and a button to the refund settings
    	$rui = new RelationUser($this->db_app);
 		if ($rui->isAdmin($_SESSION['session_user_id'], $this->if_obj->model['currentCP']->participant->relation->id))
 		{
 			$btn = new Form_Button('', $this->langIf->getText('Refund Settings'), 'button');
 			$btn->onclick("go('InterfaceRefundSettings.php');");
 			$ds[] = array('setting'=>'&nbsp;');
			$ds[] = array('setting'=>$this->langIf->getText('Define refund settings'), $btn->getHtml());
 		}

    	$tf = new TableFactory($ds, $this->langIf);
    	$tf->setValueProcessor(array('0', '1'), array($this, 'tfFixWidth'));
    	$this->text .= $tf->getXHTMLTable('Professional', 2);
    }
	private function getRefundSetting($property)
	{
		// cache translation
		$this->langIf->getText('No refund setting defined');

		if (!$this->if_obj->model['currentCP'])
			return '-';

		$cp = $this->if_obj->model['currentCP'];

		// switch kees
		switch ($cp->$property)
		{
			case REFUND_AUTHORIZE_OPERATOR:
				return $this->langIf->getText('Operator');

//			case REFUND_AUTHORIZE_EMAIL:
//				return $this->langIf->getText('Importer through email');

			case REFUND_AUTHORIZE_GENERAL_MANAGER:
				$rv = $this->langIf->getText('General Manager');
				return $rv. sprintf(' (%s)', $this->getGeneralManager());

			case REFUND_AUTHORIZE_STATUTORY_AUDITOR:
				$rv = $this->langIf->getText('Statutory Auditor');

				$rui = new RelationUser($this->db_app);
				$user = $rui->getRelationAdmin($cp->participant->relation->id);
				$user->set();
				$user->info_person->set();
				return $rv . sprintf(' (%s)', $user->info_person->getFullName());

			case false:
				return sprintf('<span class="warning">%s</span>', $this->langIf->getText('No refund setting defined'));

			default:
				trigger_error('Unknown refund setting value: '.$this->if_obj->model['currentCP']->$property, E_USER_INFO);
		}
	}

	function getGeneralManager()
	{
		$cp = $this->if_obj->model['currentCP'];
		$cp->set();

		if (!$cp->general_manager)
			return false;

		$cp->general_manager->set();
		$cp->general_manager->info_person->set();

		$a = $cp->translateGmStatus($this->langIf, $cp->general_manager_authorized);
		return $cp->general_manager->info_person->getFullName() . sprintf(' - <span class="%s">%s</span>', $a[1], $a[0]);
	}

	function getIdentifyGM()
	{
		if (!$this->if_obj->model['currentCP'])
			return;

		$this->text = $this->whoWhen($this->langIf->getText('Admin'), $this->langIf->getText('Once, prior to step 2'));

		$langCurrentGm = $this->langIf->getText('The current GM is:');
		$langNoGm = $this->langIf->getText('There is no GM');

		$gm = $this->getGeneralManager();

		if (!$gm)
			$txt = $langNoGm;
		else
			$txt = $langCurrentGm.' '.$gm;

		$btn = new Form_Button('', $this->langIf->getText('User Overview'), 'button');
		$btn->onclick("go('InterfaceParticipantUserOverview.php');");

		$ds = array();
		$ds[] = array('0'=>$txt, '1'=>$btn->getHtml());

    	$tf = new TableFactory($ds, $this->langIf);
    	$tf->setValueProcessor(array('0', '1'), array($this, 'tfFixWidth'));
    	$tf->decapitate();
    	$this->text .= $tf->getXHTMLTable('Professional', 2);

	}

	function getAddPop()
	{
 		if (!$this->if_obj->model['currentCP'])
			return;
		$cp = $this->if_obj->model['currentCP'];

		$this->text = $this->whoWhen($this->langIf->getText('All'), $this->langIf->getText('At all times'));

		$ds = array();

		$btn = new Form_Button('', $this->langIf->getText('New PoP'), 'button');
		$btn->onclick(sprintf("go('InterfacePaymentForm.php?participant[id]=%s&crso[id]=%s');", $cp->participant->relation->id, $cp->crso->relation->id));
		$ds[]  = array($this->langIf->getText('Create a new PoP'), $btn->getHtml());

		$btn = new Form_Button('', $this->langIf->getText('PoP overview'), 'button');
		$btn->onclick("go('InterfacePaymentOverview.php');");
		$ds[]  = array($this->langIf->getText('Overview filed PoPs'), $btn->getHtml());

    	$tf = new TableFactory($ds, $this->langIf);
    	$tf->setValueProcessor(array('0', '1'), array($this, 'tfFixWidth'));
    	$tf->decapitate();
    	$this->text .= $tf->getXHTMLTable('Professional', 2);
	}

	function getConfirmPoP()
	{
		if (!$this->if_obj->model['currentCP'])
			return;
		$cpp = $this->if_obj->model['currentCP'];

		// get the export day from crsoproduct
		$cp = new CRSOProduct($this->db_app);
		$cp->set($cpp->crso->relation->id, $cpp->product->id);
    	$export_day = $cp->refund_export_day;


		$this->text = $this->whoWhen($this->getRefundSetting('authorize_proof_of_payment'),
			$this->langIf->getText('At all times but prior to day {EXPORT_DAY} of following month', null, array('EXPORT_DAY'=>$export_day)));


		// build table

		$ds = array();

		if ($cpp->authorize_proof_of_payment == REFUND_AUTHORIZE_OPERATOR)
		{
			$ds[] = array($this->langIf->getText('PoP Review is performed by PwC as Operator'), '');
		}
		else
		{
			$btn = new Form_Button('', $this->langIf->getText('Confirm PoP'), 'button');
			$btn->onclick(sprintf("go('InterfacePaymentConfirmation.php?participant[id]=%s&crso[id]=%s');", $cpp->participant->relation->id, $cpp->crso->relation->id));
			$ds[] = array($btn->getHtml());
		}

		$btn = new Form_Button('', $this->langIf->getText('PoP Confirm Overview'), 'button');
		$btn->onclick(sprintf("go('InterfacePaymentOverview.php?participant[id]=%s&crso[id]=%s');", $cpp->participant->relation->id, $cpp->crso->relation->id));

		$ds[] = array($this->langIf->getText('Overview of all PoPs'), $btn->getHtml());

    	$tf = new TableFactory($ds, $this->langIf);
    	$tf->setValueProcessor(array('0', '1'), array($this, 'tfFixWidth'));
    	$tf->decapitate();
    	$this->text .= $tf->getXHTMLTable('Professional', 2);
	}

	function getAddPoE()
	{
 		if (!$this->if_obj->model['currentCP'])
			return;
		$cp = $this->if_obj->model['currentCP'];

		$this->text = $this->whoWhen($this->langIf->getText('All'), $this->langIf->getText('At all times'));

		$ds = array();

		$btn = new Form_Button('', $this->langIf->getText('New PoE'), 'button');
		$btn->onclick(sprintf("go('InterfaceExportForm.php?participant[id]=%s&crso[id]=%s');", $cp->participant->relation->id, $cp->crso->relation->id));
		$ds[]  = array($this->langIf->getText('Create a new PoE'), $btn->getHtml());

		$btn = new Form_Button('', $this->langIf->getText('PoE overview'), 'button');
		$btn->onclick("go('InterfaceExportOverview.php');");
		$ds[]  = array($this->langIf->getText('Overview filed PoEs'), $btn->getHtml());

    	$tf = new TableFactory($ds, $this->langIf);
    	$tf->setValueProcessor(array('0', '1'), array($this, 'tfFixWidth'));
    	$tf->decapitate();
    	$this->text .= $tf->getXHTMLTable('Professional', 2);
	}

	function getConfirmPoE()
	{
		if (!$this->if_obj->model['currentCP'])
			return;
		$cpp = $this->if_obj->model['currentCP'];

		// get the export day from crsoproduct
		$cp = new CRSOProduct($this->db_app);
		$cp->set($cpp->crso->relation->id, $cpp->product->id);
    	$export_day = $cp->refund_export_day;

		$this->text = $this->whoWhen(array($this->getRefundSetting('authorize_refund_request_eu'), $this->getRefundSetting('authorize_refund_request_non_eu')),
			$this->langIf->getText('At all times but prior to day {EXPORT_DAY} of following month', null, array('EXPORT_DAY'=>$export_day)));

		// build table

		$ds = array();

		if ($cpp->authorize_refund_request_eu == REFUND_AUTHORIZE_OPERATOR && $cpp->authorize_refund_request_non_eu == REFUND_AUTHORIZE_OPERATOR)
		{
			$ds[] = array('PoE Review is performed by PwC as Operator', '');
		}
		else
		{
			$btn = new Form_Button('', $this->langIf->getText('Confirm PoE Overview'), 'button');
			$btn->onclick(sprintf("go('InterfaceExportConfirmationOverview.php?participant[id]=%s&crso[id]=%s');", $cpp->participant->relation->id, $cpp->crso->relation->id));
			$ds[] = array($this->langIf->getText('Go to Confirm PoE Overview'), $btn->getHtml());
		}

    	$tf = new TableFactory($ds, $this->langIf);
    	$tf->setValueProcessor(array('0', '1'), array($this, 'tfFixWidth'));
    	$tf->decapitate();
    	$this->text .= $tf->getXHTMLTable('Professional', 2);
	}

	function getReviewPoE()
	{
		if (!$this->if_obj->model['currentCP'])
			return;
		$cpp = $this->if_obj->model['currentCP'];

		// get the export day from crsoproduct
		$cp = new CRSOProduct($this->db_app);
		$cp->set($cpp->crso->relation->id, $cpp->product->id);
    	$export_day = $cp->refund_export_day;

		$this->text = $this->whoWhen(array($this->getRefundSetting('authorize_refund_request_eu'), $this->getRefundSetting('authorize_refund_request_non_eu')),
			$this->langIf->getText('At all times but prior to day {EXPORT_DAY} of following month', null, array('EXPORT_DAY'=>$export_day)));

		$crso_id = $cpp->crso->relation->id;
		$part_id = $cpp->participant->relation->id;
		$product_id = $cpp->product->id;

		// now build the virtual stock table

		require_once(ELC_CLASSES.'RefundExport.php');
		$re = new RefundExport($this->db_app);

    	$langNoStock = $this->langIf->getText('No Virtual Stock');
    	$langCurrentStock  = $this->langIf->getText('Current Virtual Stock');
    	$langVirtualStock = $this->langIf->getText('Virtual Stock');

		$lang_sufficient = $this->langIf->getText('sufficient');
		$lang_insufficient =  $this->langIf->getText('insufficient');

    	$pop = new ProofOfPayment($this->db_app);
    	$poe = new RefundRequest($this->db_app);

		$ds = array();

		//here we need to check if virtual stock is per producer
		$country = new Country($this->db_app, $cpp->crso->getCountry(), true);
		if($country->has(HasRefundInternalVAT,$cpp->product->id))
		{
			require_once(ELC_CLASSES.'ProofOfPayment.php');

			$vStock = $rrForExport = array();
	    	$producers = $pop->getProducers($cpp->crso->relation->id, $cpp->participant->relation->id, PROOF_OF_PAYMENT_APPROVED, $cpp->product->id);

	    	foreach($producers as $producerId)
	    	{
		    	$stock = $re->getVirtualStock($cpp->crso->relation->id, $cpp->participant->relation->id, $producerId, $cpp->product->id);
		    	//if (!$stock)
		    		//continue;

		    	$poe_stock = $re->getRefundStock($crso_id, $part_id, $product_id, $producerId);

		    	$producer = new Relation($this->db_app, $producerId, true);
		    	$ds[] = array(
					'PRODUCER'=>$producer->name,
		    		'POPS_APPROVED' => $pop->getCount($cpp->crso->relation->id, $cpp->participant->relation->id, $cpp->product->id, PROOF_OF_PAYMENT_APPROVED, $producerId),
		    		'POPS_PENDING' => $pop->getCount($cpp->crso->relation->id, $cpp->participant->relation->id, $cpp->product->id, PROOF_OF_PAYMENT_PENDING, $producerId),
		    		'POPS_REJECTED' => $pop->getCount($cpp->crso->relation->id, $cpp->participant->relation->id, $cpp->product->id, PROOF_OF_PAYMENT_REJECTED, $producerId),
					'VIRTUAL_STOCK'=>$stock,
					'POE_STOCK' => $poe_stock,
		    		'VSTOCK_STATUS' => $stock >= $poe_stock ? '<span class="status_verified">'.$lang_sufficient.'</span>' : '<span class="status_denied">'.$lang_insufficient.'</span>',
					);
	    	}

			if (sizeof($ds) == 0)
			{
				$this->text = $langNoStock;
				return;
			}
		}
		else // country setting not per producer
		{
	    	$stock = $re->getVirtualStock($cpp->crso->relation->id, $cpp->participant->relation->id, null, $cpp->product->id);
	    	$poe_stock = $re->getRefundStock($crso_id, $part_id, $product_id);

	    	$ds[] = array(
	    		'POPS_APPROVED' => $pop->getCount($cpp->crso->relation->id, $cpp->participant->relation->id, $cpp->product->id, PROOF_OF_PAYMENT_APPROVED),
	    		'POPS_PENDING' => $pop->getCount($cpp->crso->relation->id, $cpp->participant->relation->id, $cpp->product->id, PROOF_OF_PAYMENT_PENDING),
	    		'POPS_REJECTED' => $pop->getCount($cpp->crso->relation->id, $cpp->participant->relation->id, $cpp->product->id, PROOF_OF_PAYMENT_REJECTED),
				'VIRTUAL_STOCK'=>$stock,
				'POE_STOCK' => $poe_stock,
	    		'VSTOCK_STATUS' => $stock > $poe_stock ? '<span class="status_verified">'.$lang_sufficient.'</span>' : '<span class="status_denied">'.$lang_insufficient.'</span>',
				);
	}

		$tf = new TableFactory($ds, $this->langIf);
		$tf->setValueProcessor(array('VIRTUAL_STOCK', 'POE_STOCK'), array($this, 'tfValueProcessor'));
    	//$tf->setValueProcessor(array('PRODUCER', 'VIRTUAL_STOCK'), array($this, 'tfFixWidth'));
		$tf->setCaption($langVirtualStock);
		$this->text .= $tf->getXHTMLTable('Professional', 2);
		//$tf->decapitate(); // fixme: only decapitate for non-producer contry

		$this->text .= '<br />'.IWRegistry::registry('langIf')->getText('note: if virtualstock insufficient');

	}

	function getOverviewPoE()
	{
		if (!$this->if_obj->model['currentCP'])
			return;
		$cpp = $this->if_obj->model['currentCP'];

		// get the export day from crsoproduct
		$cp = new CRSOProduct($this->db_app);
		$cp->set($cpp->crso->relation->id, $cpp->product->id);
    	$this->text .= $this->langIf->getText('The new export will be: ') . $cp->getNextCutOffDateRefund().'<br />';

    	// Now show the exportables
		require_once(ELC_CLASSES.'RefundExport.php');
		$re = new RefundExport($this->db_app);

		//here we need to check if virtual stock is per producer
		$country = new Country($this->db_app, $cpp->crso->getCountry(), true);
		if($country->has(HasRefundInternalVAT,$cpp->product->id))
			$rrExport = $re->getExportableRefundRequestsByProducer($cpp->crso->relation->id, $cpp->participant->relation->id, $cpp->product->id);
		else
			$rrExport = $re->getExportableRefundRequests($cpp->crso->relation->id, $cpp->participant->relation->id, $cpp->product->id);

		$langNoExports = '<br /><br />'.$this->langIf->getText('No Refund Exportables found');
		if(!$rrExport)
		{
			$this->text .= $langNoExports ;
			return;
		}

		// count by country
		$d = array();
		foreach ($rrExport as $poe_id)
		{
			$poe = new RefundRequest($this->db_app, $poe_id, true);
			if (!isset($d[$poe->export_country->id]))
				$d[$poe->export_country->id]= 0;

			$d[$poe->export_country->id]++;
		}

	    // build tf dataset
    	$ds = array();
		foreach ($d as $country_id => $count)
		{
			$country = new Country($this->db_app, $country_id, true);
			$ds[] = array('COUNTRY'=>$country->name, 'POE_COUNT'=>$count);
		}

		$tf = new TableFactory($ds, $this->langIf);
    	$tf->setValueProcessor(array('COUNTRY', 'POE_COUNT'), array($this, 'tfFixWidth'));
		$tf->setCaption($this->langIf->getText('The following PoE will be exported:'));
		$this->text .= '<br/>'.$tf->getXHTMLTable('Professional', 2).'<br/>';

		// add a link to the PoE overview
		$ds = array();
		$btn = new Form_Button('', $this->langIf->getText('PoE Overview'), 'button');
		$btn->onclick(sprintf("go('InterfaceExportOverview.php?participant[id]=%s&crso[id]=%s');", $cpp->participant->relation->id, $cpp->crso->relation->id));
		$ds[] = array($this->langIf->getText('Overview PoE filed'), $btn->getHtml());
    	$tf = new TableFactory($ds, $this->langIf);
    	$tf->setValueProcessor(array('0', '1'), array($this, 'tfFixWidth'));
    	$tf->decapitate();
    	$this->text .= $tf->getXHTMLTable('Professional', 2);

    	// a total of the last export
		$tf = new TableFactory($re->selectLastExportDetails($cpp->crso->relation->id, $cpp->participant->relation->id, $cpp->product->id), $this->langIf);
		$tf->setCaption($this->langIf->getText('Last refund export details'));
		$tf->setValueProcessor(array('INVOICE_DOCUMENT', 'QUANTITY', 'AMOUNT'), array($this, 'tfValueProcessor'));
		$this->text .= '<br/>'.$tf->getXHTMLTable('Professional', 2).'<br/>';
   	}

   	function tfValueProcessor($args)
	{
		switch ($args['head'])
		{
			case 'AMOUNT':
				return locale_nf($args['value']);

			case 'VIRTUAL_STOCK':
			case 'POE_STOCK':
			case 'QUANTITY':
				return is_numeric($args['value'])?locale_nf($args['value']):$args['value'];

			case 'INVOICE_DOCUMENT':

				if (!$args['value'])
					return;
				$document = new ELCDocument($this->db_app);
				$document->set($args['value']);

				return '<a href="/'.SITE_PATH.'scripts/i-w/document/download.php?Document[id]='.$args['value'].'">'.$this->langIf->getText('document').'</a>';;
			break;
		}
		return $args['value'];
	}

	function getDownloadDocuments()
	{
		if (!$this->if_obj->model['currentCP'])
			return;
		$cpp = $this->if_obj->model['currentCP'];

		// add a link to the PoE overview
		$ds = array();
		$btn = new Form_Button('', $this->langIf->getText('Refund Overview'), 'button');
		$btn->onclick(sprintf("go('InterfaceRefundOverview.php?participant[id]=%s&crso[id]=%s');", $cpp->participant->relation->id, $cpp->crso->relation->id));
		$ds[] = array($this->langIf->getText('Overview Correcting Documents'), $btn->getHtml());
    	$tf = new TableFactory($ds, $this->langIf);
    	$tf->setValueProcessor(array('0', '1'), array($this, 'tfFixWidth'));
    	$tf->decapitate();
    	$this->text = $tf->getXHTMLTable('Professional', 2);
	}


	/*
	function getButtons()
	{
 		if (!$this->if_obj->model['currentCP'])
			return;
		$cp = $this->if_obj->model['currentCP'];

		$rui = new RelationUser($this->db_app);
		$isAdmin = $rui->isAdmin($_SESSION['session_user_id'], $this->if_obj->model['currentCP']->participant->relation->id);

		$this->_BarButtons = null;

		// always alow these as else we wont have come here
		$this->_addCustomButton($this->langIf->getText('New PoP'), 'button',
			array('onclick'=>sprintf("go('InterfacePaymentForm.php?participant[id]=%s&crso[id]=%s');", $cp->participant->relation->id, $cp->crso->relation->id)));
		$this->_addCustomButton($this->langIf->getText('New PoE'), 'button',
				array('onclick'=>sprintf("go('InterfaceExportForm.php?participant[id]=%s&crso[id]=%s');", $cp->participant->relation->id, $cp->crso->relation->id)));

		// check if authorize pop by gm and user is gm, or pop by sa and user is sa
		if (($cp->authorize_proof_of_payment == REFUND_AUTHORIZE_GENERAL_MANAGER && $cp->general_manager->id == $_SESSION['session_user_id']) ||
			($cp->authorize_proof_of_payment == REFUND_AUTHORIZE_STATUTORY_AUDITOR && $isAdmin)
		)
		{
			$this->_addCustomButton($this->langIf->getText('Confirm PoP'), 'button',
				array('onclick'=>sprintf("go('InterfacePaymentConfirmation.php?participant[id]=%s&crso[id]=%s');", $cp->participant->relation->id, $cp->crso->relation->id)));
		}

		// check if rr-eu or rr-non-eu are by by gm and user is gm or by admin
		if ( (($cp->authorize_refund_request_eu == REFUND_AUTHORIZE_GENERAL_MANAGER || $cp->authorize_refund_request_non_eu == REFUND_AUTHORIZE_GENERAL_MANAGER) && $cp->general_manager->id == $_SESSION['session_user_id']) ||
			( ($cp->authorize_refund_request_eu == REFUND_AUTHORIZE_STATUTORY_AUDITOR || $cp->authorize_refund_request_non_eu == REFUND_AUTHORIZE_STATUTORY_AUDITOR) && $isAdmin)
		)
		{
			$this->_addCustomButton($this->langIf->getText('Confirm PoE'), 'button',
				array('onclick'=>sprintf("go('InterfaceExportConfirmationOverview.php?participant[id]=%s&crso[id]=%s');", $cp->participant->relation->id, $cp->crso->relation->id)));
		}
	}
	*/

}

/**
 * CALLBACK[getRefundSettings]
 * item[1293] = {
 *     [handle] = Item: RefundSettings
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getVirtualStock]
 * item[1294] = {
 *     [handle] = Item: VirtualStock
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getNextExportDate]
 * item[1295] = {
 *     [handle] = Item: NextExportDate
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getRefundExportable]
 * item[1296] = {
 *     [handle] = Item: getRefundExportable
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
