<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @since 03-03-2006
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceRegistrationOverview extends InterfaceFactory
{
    /**
     * interface[id] = 80
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceRegistrationOverview(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {

    }
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 03-03-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceRegistrationOverviewItem extends InterfaceItem
{

    /**
     * item[730] =
     */
    function getOverview()
    {
		require_once(ELC_CLASSES.'RelationUser.php');
		require_once(ELC_CLASSES.'Participant.php');
		require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
		require_once(IWARE.'TableFactoryColumnPlugin.php');

		$ru = new RelationUser($this->db_app);
		$ds = $ru->selectRegistrationOverview($_SESSION['session_user_id']);

		$tf = new TableFactoryColumnPlugin(false,$this->langIf,$this->db,$this->db_app);
		$tf->setData($ds, 'UserRegistrationOverview',false,true);

		$tf->setRowProcessor(array($this, 'tfRowProcessor'));

		$help = ($this->parent->title != '') ? getHelp($this->langIf, $this->parent->handle, array('title'=>$this->parent->handle, 'top'=>'0px')) : '';
		$tf->setCaption($help.'<img src="/'.SITE_PATH.'images/block_arrow.png" /> '.$this->langIf->getText('User Registration Overview'), false);
		$tf->hideColumns(array('PARTICIPANT_RELATION_ID', 'CRSO_ID', 'PARTICIPANT_VAT','CRSO_VAT', 'PRODUCT_ID'));
		$tf->collapseColumns(array('PARTICIPANT_NAME','PARTICIPANT_VAT'));
		$tf->setValueProcessor(array('RU_STATUS'),'if ( isset($previousrow) && $value == $previousrow[$orighead] ){ $tdclass && $tdclass .= \'-collapse\'; return \'\';}else return ($value?\'<img src="/\'.SITE_PATH.\'images/i-w/true.gif" uniqid="\'.$row[\'PARTICIPANT_RELATION_ID\'].\'" />\':\'<img src="/\'.SITE_PATH.\'images/i-w/false.gif" uniqid="\'.$row[\'PARTICIPANT_RELATION_ID\'].\'" />\');');
		$tf->setValueProcessor(array('CP_ACTIVE','IS_ADMIN','EXPORTER','PRODUCER'),'return ($value?\'<img src="/\'.SITE_PATH.\'images/i-w/true.gif" />\':\'<img src="/\'.SITE_PATH.\'images/i-w/false.gif" />\');');
		$tf->setValueProcessor('CP_STATUS','list($trans,$class) = Participant::translateStatus($this->langIf, $value); return \'<span class="\'.$class.\'">\'.$trans.\'</span>\';');

		$tf->setValueProcessor('DECLARATION_SETTINGS', array($this, 'tfValueProcessor'));
		$tf->setValueProcessor('COMPANY_DETAILS', array($this, 'tfValueProcessor'));

		$this->text = $tf->getXHTMLTable('Professional',2);
    }

    function tfRowProcessor($args) {
	$rv = null;
		// Check for any active Remote Viewer connections.
		if(IWRegistry::isRegistered('remote_viewer')) {
			$rv = IWRegistry::registry('remote_viewer');
		}
		if(!$rv)
			return;
		// Not the participant that is the Remote Viewer, so skip this row.
		if($args['row']['PARTICIPANT_RELATION_ID']!=$rv->participant->relation->id)
			return false;
		// Not the CRSO that is the Remote Viewer, so skip this row.
		if($args['row']['CRSO_ID']!=$rv->crso->relation->id)
			return false;
    }

    function tfValueProcessor($args)
    {
    	switch ($args['head'])
    	{
    		case 'DECLARATION_SETTINGS':

    			$ru = new RelationUser($this->db_app);

    			// check for if we have clearance level and crso_participant is atleast confirmed
    			if ( $ru->hasClearance(ClearanceDeclarationSettings, $_SESSION['session_user_id'],
    				$args['row']['PARTICIPANT_RELATION_ID']) && ($args['row']['CP_STATUS'] >= PARTICIPANT_STATUS_CONFIRMED) )
    			{
	    			return sprintf('<a id="declaration_settings_%s_%s_%s" href="InterfaceDeclarationSettings.php?participant[id]=%s&crso[id]=%s&product[id]=%s">',
	    				$args['row']['PRODUCT_ID'],$args['row']['PARTICIPANT_RELATION_ID'], $args['row']['CRSO_ID'], $args['row']['PARTICIPANT_RELATION_ID'], $args['row']['CRSO_ID'], $args['row']['PRODUCT_ID']) .
	    				$this->langIf->getText('Declaration Settings').'</a>';
	    		}
	    		else
	    			return '-';

    		break;
    		case 'COMPANY_DETAILS':
    			return sprintf('<a href="InterfaceCompanyInformation.php?product[id]=%s&crso[id]=%s&participant[id]=%s">%s</a>',
    				$args['row']['PRODUCT_ID'],
    				$args['row']['CRSO_ID'],
    				$args['row']['PARTICIPANT_RELATION_ID'],
    				$this->langIf->getText('Edit Company Details')
    				);
    		break;
    	}
    }

    /**
     * item[731] =
     */
    function getButtons()
    {
	$rv = null;
		// Check for any active Remote Viewer connections.
		if(IWRegistry::isRegistered('remote_viewer')) {
			$rv = IWRegistry::registry('remote_viewer');
		}

		require_once(ELC_CLASSES.'CRSO.php');
		$crso = new CRSO($this->db_app);
		$q = $crso->selectCRSOs();
		$optsPart = $opts = '';
		while($r = $q->fetch()) {

			if($rv && $rv->crso->relation->id!=$r->CRSO_RELATION_ID)
				continue;
			$opts .= '<option value="'.$r->CRSO_RELATION_ID.'">'.((mb_strlen($r->NAME)>35)?mb_substr($r->NAME,0,35).'...':$r->NAME).'</option>';
		}

		require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
		// Get all RELATION_USERs
		$sql = "SELECT CP.PARTICIPANT_RELATION_ID, CP.STATUS, R.`NAME`, CP.CRSO_RELATION_ID, CP.PRODUCT_ID as PRODUCT_ID
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CP
			INNER JOIN TBL_RELATION as R
				ON CP.PARTICIPANT_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_RELATION_USER as RU
				USING(RELATION_ID)
			WHERE RU.USER_ID = ?
				AND CP.ACTIVE = ?
				AND RU.STATUS=1
				AND ((CLEARANCE_LEVEL & ".ClearanceApplyAtCRSO.") OR IS_ADMIN = 1)";
		$q = $this->db_app->query($sql);
		$q->bind_values($_SESSION['session_user_id'],1);
		$q->execute();
		$parts = array();
		$this->script = "var ParticipantCRSO = new Array();\n";
		while($r = $q->fetch())
		{
			if($rv && $rv->crso->relation->id!=$r->CRSO_RELATION_ID)
				continue;
			if($rv && $rv->participant->relation->id!=$r->PARTICIPANT_RELATION_ID)
				continue;
			if($r->STATUS >= PARTICIPANT_STATUS_CONFIRMED && !isset($parts[$r->PARTICIPANT_RELATION_ID]) )
			{
				$optsPart .= '<option value="'.$r->PARTICIPANT_RELATION_ID.'">'.((mb_strlen($r->NAME)>35)?mb_substr($r->NAME,0,35).'...':$r->NAME).'</option>';
				//$this->script .= sprintf("ParticipantCRSO[%d] = new Array()\n", $r->PARTICIPANT_RELATION_ID);
			}

			// little ugly but well
			$this->script .= sprintf("if (typeof ParticipantCRSO[%d] == 'undefined') ParticipantCRSO[%d] = new Array()\n", $r->PARTICIPANT_RELATION_ID, $r->PARTICIPANT_RELATION_ID);
			$this->script .= sprintf("if (typeof ParticipantCRSO[%d][%d] == 'undefined') ParticipantCRSO[%d][%d] = new Array()\n", $r->PARTICIPANT_RELATION_ID, $r->CRSO_RELATION_ID, $r->PARTICIPANT_RELATION_ID, $r->CRSO_RELATION_ID);
			$this->script .= sprintf("if (typeof ParticipantCRSO[%d][%d][%d] == 'undefined') ParticipantCRSO[%d][%d][%d] = new Array()\n", $r->PARTICIPANT_RELATION_ID, $r->CRSO_RELATION_ID, $r->PRODUCT_ID, $r->PARTICIPANT_RELATION_ID, $r->CRSO_RELATION_ID, $r->PRODUCT_ID);

			$parts[$r->PARTICIPANT_RELATION_ID][] = $r->CRSO_RELATION_ID;
			$this->script .= sprintf("ParticipantCRSO[%d][%d][%d] = [%d]\n", $r->PARTICIPANT_RELATION_ID, $r->CRSO_RELATION_ID, $r->PRODUCT_ID, $r->CRSO_RELATION_ID);
    	}

    	//$this->script .= 'alert(print_r(ParticipantCRSO));';


    	// start with javascript

		// @todo make me pretty
		$this->text = '
			<script type="text/javascript"><!--

			var crsorules = \'\';
			var crsoid = 0;
			var participantrules = \'\';
			var participantid = 0;
			var productrules = \'\';
			var productid = 0;

			function loadApplicationInterface()
			{
				if(crsorules&&participantrules&&productrules)
				{
					if(typeof ParticipantCRSO[participantid][crsoid] != \'undefined\' && typeof ParticipantCRSO[participantid][crsoid][productid] != \'undefined\')
					{
						MessageBox.prototype.Information(\''.$this->langIf->getText('Already Applied Title').'\',\''.$this->langIf->getText('Already Applied at CRSO for Participant').'\');
						return;
					}
				}
				if(fb.validate($(\'interface_form\')))
					go($(\'interfaceSelect\').value+\'.php?1=1\'+crsorules+participantrules+productrules);
			}

			function setRules(sel)
			{
				rules = \'\';
				$(\'selectCRSO\').style.display = \'none\';
				$(\'selectParticipant\').style.display = \'none\';
				$(\'selectProduct\').style.display = \'none\';
				fb.removeBehaviour($(\'selectCRSO\'),\'required\');
				fb.removeBehaviour($(\'selectParticipant\'),\'required\');
				fb.removeBehaviour($(\'selectProduct\'),\'required\');
				switch(sel.value)
				{
					case \'InterfaceUserNewParticipantRegistration\':
						$(\'selectCRSO\').style.display = \'\';
						fb.setBehaviour($(\'selectCRSO\'),\'required\');

						$(\'selectProduct\').style.display = \'\';
						fb.setBehaviour($(\'selectProduct\'),\'required\');
					break;
					case \'InterfaceParticipantNewCRSORegistration\':

						$(\'selectParticipant\').style.display = \'\';
						fb.setBehaviour($(\'selectParticipant\'),\'required\');

						$(\'selectCRSO\').style.display = \'\';
						fb.setBehaviour($(\'selectCRSO\'),\'required\');

						$(\'selectProduct\').style.display = \'\';
						fb.setBehaviour($(\'selectProduct\'),\'required\');
					break;
				}
			}

			function applyRulesCRSO(sel)
			{
				crsorules = \'&crso[id]=\'+sel.value;
				crsoid = sel.value;
			}

			function applyRulesParticipant(sel)
			{
				participantrules = \'&participant[id]=\'+sel.value;
				participantid = sel.value;
			}

			function applyRulesProduct(sel)
			{
				productrules = \'&product[id]=\'+sel.value;
				productid = sel.value;
			}

			function loadProducts()
			{
				var sel = $(\'selectProduct\');
				Select_ClearOptions(sel, 1);

				var params = new Array()
				params[\'crsoid\'] = $(\'selectCRSO\').value;

				jsbroker.call(\'getCRSOProducts\', params, function(rv)
				{
					rv.forEach(function(product_name, product_id, array)
					{
						sel.options.add(new Option(product_name, product_id));
 	                }, this);

					// automaticly select the first product if only one is available
					if (rv.count() == 1)
					{
						sel.options[1].selected = true;
						applyRulesProduct(sel);
					}
 	        	});

			}

			--></script>
			<div style="float: left;">'.$this->langIf->getText('Apply').'
				<select id="interfaceSelect" behaviour="required" onchange="setRules(this);">
					<option value="">- '.$this->langIf->getText('Please select').' -</option>';
			if(!$rv)
				$this->text .= '<option value="InterfaceUserNewParticipantRegistration">'.$this->langIf->getText('As new Participant').'</option>';
			$this->text .= '<option value="InterfaceParticipantNewCRSORegistration">'.$this->langIf->getText('At new CRSO').'</option>';
			if(!$rv)
				$this->text .= '<option value="InterfaceApplyAtParticipant">'.$this->langIf->getText('At different Participant').'</option>';

			$this->text .= '</select>
				<select style="display: none;" onchange="applyRulesParticipant(this);" id="selectParticipant">
					<option value="">- '.$this->langIf->getText('Select Participant').' -</option>
					'.$optsPart.'
				</select>
				<select style="display: none;" onchange="applyRulesCRSO(this); loadProducts();" id="selectCRSO">
					<option value="">- '.$this->langIf->getText('Select CRSO').' -</option>
					'.$opts.'
				</select>

				<select style="display: none;" onchange="applyRulesProduct(this);" id="selectProduct">
					<option value="">- '.$this->langIf->getText('Select Product').' -</option>
					<!-- filled by jsrpc -->
				</select>
			</div>
			';
			// @todo make sure there is only options available for the clearance level of the user on the participants.
		$this->_BarButtons = 0;
		$this->_addCustomButton($this->langIf->getText('Apply'),'button',array('onclick'=>"loadApplicationInterface();"));
    }

}

/**
 * CALLBACK[getOverview]
 * item[730] = {
 *     [handle] = Item: ReigstrationOverview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[731] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
