<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');

require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'Relation.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'Product.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'ProvisionalDeclaration.php');
require_once(ELC_CLASSES.'Trust.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @since 13-03-2006
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceSubrogationApplication extends InterfaceFactory
{
    /**
     * interface[id] = 88
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceSubrogationApplication(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

		$trust = new Trust($this->db_app);
		if(isset($_REQUEST['trust']['id']))
		{
			$trust->set($_REQUEST['trust']['id']);
			if($trust->approved & TRUST_TRUSTEE_APPROVED)
				throw new InterfacePermissionException('Unapproved Trustee trying to access Trust');

			$this->model['trust'] = $trust;
			$this->model['subrogator'] = $trust->subrogator;
			$this->model['crso'] = $trust->crso;
			$this->model['trustees'] = $trust->trustee;

			$this->hiddenFields = array(array('name'=>'trust[id]','value'=>$trust->id));
		}
		else
		{
			$this->model['subrogator'] = new Participant($this->db_app,array('relation'=>new Relation($this->db_app,$_REQUEST['subrogator']['id'])));
			$this->model['crso'] = new CRSO($this->db_app,array('relation'=>new Relation($this->db_app,$_REQUEST['crso']['id'])));
			$this->model['trustees'] = $trust->selectAvailableTrustees($_REQUEST['subrogator']['id'],$_REQUEST['crso']['id'],$_SESSION['selected_product_id']);
			$trustrs = $this->model['trustees']->fetchAll();
			if(!count($trustrs))
			{
				alert('Subrogator ['.$_REQUEST['subrogator']['id'].'] has no available Trustees for CRSO ['.$_REQUEST['crso']['id'].']');
				exit('No Trustees Available');
			}
			$this->model['trustees']->execute();
			$this->hiddenFields = array(
				array('name'=>'crso[id]','value'=>$this->model['crso']->relation->id),
				array('name'=>'subrogator[id]','value'=>$this->model['subrogator']->relation->id));
    	}
	$rv = null;
		// Check for any active Remote Viewer connections.
		if(IWRegistry::isRegistered('remote_viewer')) {
			$rv = IWRegistry::registry('remote_viewer');
		}
		if($rv &&  $rv->participant->relation->id!=$this->model['subrogator']->relation->id) {
			alert('['.$_SESSION['session_user_id'].'] no permission for subrogation details due to Remote Viewer for ['.$this->model['subrogator']->relation->id.']');
			throw new InterfacePermissionException('['.$_SESSION['session_user_id'].'] no permission for subrogation for ['.$this->model['subrogator']->relation->id.']');
		}


		// Check if the subrogator really is part of this user:
		if(!in_array($this->model['subrogator']->relation->id,$_SESSION['participant_ids']))
		{
			alert('User tried to add/edit subrogation details for ['.$this->model['subrogator']->relation->id.']');
			throw new InterfacePermissionException('User tried to add/edit subrogation details for ['.$this->model['subrogator']->relation->id.']');
		}

		$this->model['subrogator']->relation->set();
		setPageTitlePostfix('('.$this->model['subrogator']->relation->name.')');


		$this->model['editable'] = false;
    }
	/**
	 *
	 */
	function init($id)
	{
		//if(!empty($_POST['process']))
		if(!empty($_POST['process']))
		{
			if( !($this->validated = (int)$this->fb->validate()) )
				$this->thrownException .= $this->langIf->getText('Please correct the form errors');

			if ( !$this->validated )
				$_POST['process'] = '';

			if($_POST['process'] == 'check')
				$this->_hideInputs = true;
		}
		parent::init($id);
	}
    /**
     * @param string $thrownException
     * @todo Need to specify full paths in notification callbacks, as the notifications are now under the public scripts
     */
    function processInput(&$thrownException)
    {
    	// translation cache
    	$this->langIf->getText('{SUBROGATOR} requests a trust with {TRUSTEE} for CRSO {CRSO}');

        if(isset($_POST['process']))
		{
			if( isset($this->validated) && !$this->validated )
				return $thrownException .= $this->thrownException;

			if($_POST['process'] == 'check')
				$this->process = 'save';
			elseif($_POST['process'] == 'save')
			{
				$this->db_app->begin();
				if($this->_saveInput($thrownException))
				{
					// send notification BUT only IF new

					if(!isset($this->model['trust']))
					{
						require_once(ELC_CLASSES . 'Notification.php');
						$this->trust->subrogator->relation->set();
						$this->trust->trustee->relation->set();
						$this->trust->crso->relation->set();
						$recipients = $map = array();
						$map['sender'] = $this->trust->subrogator->relation;
						$map['type'] = NOTIFICATION_TYPE_INFO;
						$map['subject'] = $this->langIf->getText('Trust Request');
						$map['message'] = $this->langIf->getText('{SUBROGATOR} requests trust with {TRUSTEE} for {CRSO}',null,
							array('SUBROGATOR'=>$this->trust->subrogator->relation->name,
								'TRUSTEE'=>$this->trust->trustee->relation->name,
								'CRSO'=>$this->trust->crso->relation->name));
						$recipients[] = array('recipient'=>$this->trust->trustee->relation,
							'callback'=>"go('/".SITE_PATH."scripts/elc/participant/InterfaceTrustDetail.php?trust[id]=".$this->trust->id."&trustee[id]=".$this->trust->trustee->relation->id."');");
						Notification::send($this->db_app, $map, $recipients);

                                                log_elc_event(sprintf('Trust requested by [%s] with trustee [%s]',
                                                        $this->trust->subrogator->relation->name, $this->trust->trustee->relation->name),
                                                    $_SESSION['session_user_id'], $_SESSION['selected_product_id'],
                                                    $this->trust->crso->relation->id, $this->trust->subrogator->relation->id
                                                );
					}

					$this->db_app->commit();

					header('Location: InterfaceSubrogationOverview.php?subrogator[id]='.$this->model['subrogator']->relation->id);
					exit();
				}

				$this->db_app->rollback();
			}
		}
    }
	/**
	 * Check
	 * @access private
	 */
	function _saveInput(&$thrownException)
	{
		if(!isset($this->model['trust']))
		{
			$trustee = new Participant($this->db_app,array('relation'=>new Relation($this->db_app,$_REQUEST['trustee']['id'])));
			$trust = new Trust($this->db_app,array(
				'trustee'=>$trustee,
				'subrogator'=>$this->model['subrogator'],
				'product'=>new Product($this->db_app,$_SESSION['selected_product_id']),
				'crso'=>$this->model['crso'],
				'approved'=>TRUST_SUBROGATOR_APPROVED));
			$trust->insert();
		}
		else
			$trust = $this->model['trust'];

		$date = currentDate();
		$prov = new ProvisionalDeclaration($this->db_app,array(
			'year'=>$date->getYear(),
			'participant'=>$this->model['subrogator'],
			'product'=>new Product($this->db_app,$_SESSION['selected_product_id']),
			'crso'=>$this->model['crso'],
			'trust'=>$trust));
		$prov->removeSubrogation($trust);
		if(!$prov->subrogate($_POST['prov'],$_SESSION['selected_product_id']))
		{
			//This rollback will work, but then the one in processInput will fail
                        //$this->db_app->rollback();
			$thrownException .= '';
			return false;
		}

		$this->trust = &$trust;
		return true;
	}
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 13-03-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceSubrogationApplicationItem extends InterfaceItem
{

    /**
     * item[767] = 54
     */
    function getCompanyInterface()
    {
		$this->settings = array();
		$this->settings['_hideInputs'] = 'return true;';
		$this->settings['rel'] = '$relation = new Relation($this->db_app,'.$this->if_obj->model['subrogator']->relation->id.'); return $relation;';
		$this->settings['participant'] = 'return new Participant($this->db_app,array(\'relation\'=> new Relation($this->db_app,'.$this->if_obj->model['subrogator']->relation->id.')));';
		$this->settings['cp'] = 'return new CRSOParticipantProduct($this->db_app,array(\'participant\'=>new Participant($this->db_app,array(\'relation\'=> new Relation($this->db_app,'.$this->if_obj->model['subrogator']->relation->id.'))),\'crso\'=>new CRSO($this->db_app,array(\'relation\'=>new Relation($this->db_app,'.$this->if_obj->model['crso']->relation->id.'))),\'product\'=>new Product($this->db_app,$_SESSION[\'selected_product_id\'])));';
    }

    /**
     * item[768] = trustee[id]
     */
    function getTrustee()
    {
    	if($this->if_obj->model['trustees'] instanceof Participant)
    	{
    		$this->_hideInput = true;
    		$trustee = $this->if_obj->model['trustees'];
    		$trustee->relation->set();
    		$this->options = array(array('text'=>$trustee->relation->name,'value'=>'','selected'=>true));
    		return;
    	}

		$this->options = array(array('text'=>'- '.$this->langIf->getText('Select Trustee').' -','value'=>''));
		$q = $this->if_obj->model['trustees'];
		$this->script = "var trusteeVATs = new Array();\n";
		while($r = $q->fetch())
		{
			if($r->PARTICIPANT_RELATION_ID == $this->if_obj->model['subrogator']->relation->id)
				continue;
			$this->options[] = array('text'=>$r->PARTICIPANT_NAME,'value'=>$r->PARTICIPANT_RELATION_ID);
			$this->script .= "trusteeVATs[".$r->PARTICIPANT_RELATION_ID."] = '".$r->VAT_NO."';\n";
		}
		$this->script .= "
			function setTrusteeVAT(sel)
			{
				if(!sel.value)
					return $('trusteeVAT').innerHTML = '';
				$('trusteeVAT').innerHTML = trusteeVATs[sel.value];
			}
			";
		$this->properties['onclick'] = 'setTrusteeVAT(this);';
    }

    /**
     * item[769] =
     */
    function getTrusteeVAT()
    {
    	$value = '';
    	if($this->if_obj->model['trustees'] instanceof Participant)
    	{
    		$trustee = $this->if_obj->model['trustees'];
    		$trustee->relation->set();
    		$value = $trustee->relation->vat_no;
    	}
		$this->rightCol = $this->title;
		$this->text = '<div id="trusteeVAT">'.$value.'</div>';
    }

    /**
     * item[770] =
     */
    function getProvisionalDeclaration()
    {
		// Get country settings
		require_once(ELC_CLASSES.'Brand.php');
		require_once(ELC_CLASSES.'Country.php');
		require_once(ELC_CLASSES.'ProductCategory.php');
		require_once(ELC_CLASSES.'CRSOProductDimension.php');
		require_once(ELC_CLASSES.'ProvisionalDeclaration.php');
		$crso = $this->if_obj->model['crso'];
		$p = $this->if_obj->model['subrogator'];

		$pd = new ProvisionalDeclaration($this->db_app,array('crso'=>$crso, 'participant'=>$p));
		$q = $this->_getProvisional($crso,$p);

		$tf = new TableFactory($q,$this->langIf);
		$tf->brand = new Brand($this->db_app);
		$tf->product_category = new ProductCategory($this->db_app);
		$tf->fb =& $this->if_obj->fb;

		$tf->setValueProcessor('MONTH', 'return \'<input type="hidden" value="\'.$value.\'" name="prov[\'.$i.\'][month]" />\'.strftime(\'%B\', mktime(0, 0, 0, $value, 1));');
		$tf->setValueProcessor('BRAND_ID','$this->brand->set($value);return \'<input type="hidden" value="\'.$value.\'" name="prov[\'.$i.\'][brand]" />\'.$this->brand->name;');
		$tf->setValueProcessor('PRODUCT_CATEGORY_ID','$this->product_category->set($value);return \'<input type="hidden" value="\'.$value.\'" name="prov[\'.$i.\'][product_category]" />\'.$this->product_category->name;');
		$tf->setValueProcessor('CPD_ID',array($this,'tfValueProcessor'));
		$tf->setValueProcessor(array('WEIGHT','PROVISIONAL','ORIGINAL_PROV_QUANTITY', 'QUANTITY'), array($this,'tfValueProcessor'));
		$tf->columnModifier(array('PROVISIONAL'=>' style="text-align: right;"'));
		$tf->setTHProcessor('PROVISIONAL','return \'class="th-right"\';');
		$tf->hideColumns(array('TRUST_PROVISIONAL_QUANTITY','TRUST_PROVISIONAL_WEIGHT'));


		$this->text = $tf->getXHTMLTable('Professional', 2);

		$this->script = "
			function calcWeight(el,i,multiplier)
			{
				weightEl = $$('prov['+i+'][weight]');
				weightEl.value = Math.round(el.value*multiplier);
			}
			";
    }
    /**
     * @todo mkpretty
     * @param CRSO $crso
     * @param Participant $p
     */
    function _getProvisional($crso,$p)
    {
    	$country = new Country($this->db_app,$crso->getCountry(),true);

		$joinBy = '';
		$groupBy = 'PD.`YEAR`';
		if($country->has(HasProvisionalPerMonth,$_SESSION['selected_product_id']))
			$groupBy .= ',PD.`MONTH`';
		if($country->has(HasDeclByCategory,$_SESSION['selected_product_id']))
		{
			$groupBy .= ',PD.`PRODUCT_CATEGORY_ID`';
			$joinBy .= 'AND PD.`PRODUCT_CATEGORY_ID` = TP.`PRODUCT_CATEGORY_ID`';
		}
		if($country->has(HasDeclarationDimension,$_SESSION['selected_product_id']))
		{
			$groupBy .= ',PD.`CPD_ID`';
			$joinBy .= 'AND PD.`CPD_ID` = TP.`CPD_ID`';
		}
		if($country->has(HasDeclByBrand,$_SESSION['selected_product_id']))
		{
			$groupBy .= ',PD.`BRAND_ID`';
			$joinBy .= 'AND PD.`BRAND_ID` = TP.`BRAND_ID`';
		}
		$select = $groupBy;
		if($country->has(HasDeclOfWeight,$_SESSION['selected_product_id']))
			$select .= ', SUM(PD.`WEIGHT`) as `WEIGHT`';

		$join = "";
		if(isset($this->if_obj->model['trust']))
		{
			$select = '(SUM(TP.`QUANTITY`)+SUM(PD.`QUANTITY`)) as ORIGINAL_PROV_QUANTITY, '.$select;
			$select .= ', SUM(TP.`QUANTITY`) as TRUST_PROVISIONAL_QUANTITY';
			if($country->has(HasDeclOfWeight,$_SESSION['selected_product_id']))
			{
				$select .= ', SUM(TP.`WEIGHT`) as `TRUST_PROVISIONAL_WEIGHT`';
				$select = str_replace('as `WEIGHT`','as `WEIGHT`, (SUM(TP.`WEIGHT`)+SUM(PD.`WEIGHT`)) as `ORIGINAL_PROV_WEIGHT`',$select);
			}
			$join = "LEFT JOIN TBL_PROVISIONAL_DECLARATION as TP
				ON PD.`YEAR` = TP.`YEAR`
					AND PD.`MONTH` = TP.`MONTH`
					".$joinBy."
					AND TP.TRUST_ID = ".$this->if_obj->model['trust']->id;
		}

		$sql = "SELECT '' as PROVISIONAL, SUM(PD.`QUANTITY`) as QUANTITY, ".$select."
			FROM TBL_PROVISIONAL_DECLARATION as PD
			".$join."
			WHERE PD.CRSO_RELATION_ID = ?
				AND PD.PARTICIPANT_RELATION_ID = ?
				AND ISNULL(PD.TRUST_ID)
				AND PD.`YEAR` = ?
				AND PD.PRODUCT_ID = ?
			GROUP BY ".$groupBy;
		$q = $this->db_app->query($sql);
		$date = currentDate();
		$q->bind_values($crso->relation->id,$p->relation->id, $date->getYear(),$_SESSION['selected_product_id']);
		$q->execute();
		return $q;
    }

    function tfValueProcessor($args)
    {
    	switch ($args['head'])
    	{
    		case 'CPD_ID':
    			$dimension = new CRSOProductDimension($this->db_app,$args['value'],true);
    			return $dimension->name;
    		break;
    		case 'WEIGHT':
    			require_once(IWARE . 'form.php');
    			$txt = new Form_Text('prov['.($args['i']).'][weight]');
    			$txt->setReadonly();
    			$txt->setClass('weight');
    			$txt->setValue((isset($args['row']['TRUST_PROVISIONAL_WEIGHT'])?$args['row']['TRUST_PROVISIONAL_WEIGHT']:''));

    			//$hidden = new Form_Hidden('multiplier['.($args['i']).']',($args['value']/$args['row']['QUANTITY']));

    			return $txt->getHtml();
    		break;
    		case 'QUANTITY':
    			$txt = new Form_Text('prov['.$args['i'].'][quantity]');
    			$txt->setID('quantity_'.$args['i']);

    			$cmpValue = isset($args['row']['ORIGINAL_PROV_QUANTITY'])?$args['row']['ORIGINAL_PROV_QUANTITY']:$args['value'];

    			$txt->setBehaviour('integer(0,'.$cmpValue.')');
    			$txt->setClass('quantity');
    			$txt->setValue((isset($args['row']['TRUST_PROVISIONAL_QUANTITY'])?$args['row']['TRUST_PROVISIONAL_QUANTITY']:''));
    			if(isset($args['row']['WEIGHT']))
    			{
					$weight = isset($args['row']['ORIGINAL_PROV_WEIGHT'])?$args['row']['ORIGINAL_PROV_WEIGHT']:$args['row']['WEIGHT'];
    				$multiplier = (($cmpValue>0)?($weight/$cmpValue):0);
    				$calc = sprintf("calcWeight(this,%d,%f)",$args['i'],$multiplier);
    				$txt->onKeyUp($calc);
    				$txt->onchange($calc);
    			}
				return $txt->getHtml();
    		break;
    		case 'ORIGINAL_PROV_QUANTITY':
    			return locale_nf($args['row']['ORIGINAL_PROV_QUANTITY'], 0);
    		break;
    		case 'PROVISIONAL':
    			return locale_nf($args['row']['QUANTITY'], 0);
    		break;
    	}
    }

	function getButtons()
	{
		$this->_BarButtons = BUTTON_CANCEL;
		$this->_ButtonOptions[BUTTON_CANCEL] =array('onclick'=>"document.location.href='InterfaceSubrogationOverview.php?subrogator[id]=".$this->if_obj->model['subrogator']->relation->id."';");

		$this->script = "
			function confirmRequest()
			{
				if(!fb.validate($('interface_form')))
					return;
				MessageBox.prototype.Question('".$this->langIf->getText('MessageBox: Subrogation Confirmation')."','".$this->langIf->getText('MessageBox: Are Subrogation Settings Correct')."',function(){
					$('interface_form').submit();
				});
			}
			";

		$this->_addCustomButton($this->langIf->getText('Save'),'button',array('onclick'=>'confirmRequest();'));
	}
}

/**
 * CALLBACK[getCompanyInterface]
 * item[767] = {
 *     [handle] = Item: CompanyInformation
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getTrustee]
 * item[768] = {
 *     [handle] = Item: Trustee Selection
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getTrusteeVAT]
 * item[769] = {
 *     [handle] = Item: Trustee VAT
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getProvisionalDeclaration]
 * item[770] = {
 *     [handle] = Item: Provisional Declaration
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
