<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'Relation.php');
require_once(ELC_CLASSES.'RelationUser.php');
require_once(ELC_CLASSES.'Trust.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @since 10-03-2006
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceSubrogationOverview extends InterfaceFactory
{
    /**
     * interface[id] = 87
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceSubrogationOverview(&$db, &$db_app, &$langIf)
    {
		// Verify the request
		$ru = new RelationUser($db_app);
		$ru->set($_SESSION['session_user_id'],$_REQUEST['subrogator']['id']);
		if(!$ru->hasClearance(ClearanceFeeRegistration))
		{
			alert('['.$_SESSION['session_user_id'].'] no permission for subrogation for ['.$_REQUEST['subrogator']['id'].']');
			throw new InterfacePermissionException('['.$_SESSION['session_user_id'].'] no permission for subrogation for ['.$_REQUEST['subrogator']['id'].']');
		}
	$rv = null;
		// Check for any active Remote Viewer connections.
		if(IWRegistry::isRegistered('remote_viewer')) {
			$rv = IWRegistry::registry('remote_viewer');
		}
		if($rv &&  $rv->participant->relation->id!=$_REQUEST['subrogator']['id']) {
			alert('['.$_SESSION['session_user_id'].'] no permission for subrogation due to Remote Viewer for ['.$_REQUEST['subrogator']['id'].']');
			throw new InterfacePermissionException('['.$_SESSION['session_user_id'].'] no permission for subrogation for ['.$_REQUEST['subrogator']['id'].']');
		}

        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

		$ru->relation->set();
		setPageTitlePostfix('('.$ru->relation->name.')');

		$this->script = sprintf("
			function trustEditDisabled()
			{
				MessageBox.prototype.Information('%s','%s');
			}
			",
			$this->langIf->getText('MessageBox Title: Edit Disabled'),
			$this->langIf->getText('MessageBox Content: Can Not Edit Trust')
			);

    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
		$unableToRemove = $this->langIf->getText('Unable to remove trust, status might have changed');
        if(isset($_POST['process']))
		{
			if($_POST['process'] == 'delete')
			{
				$trust = new Trust($this->db_app);
				foreach($_POST['deleteTrust'] as $trust_id)
				{
					$trust->set($trust_id);
					if($_POST['subrogator']['id']!=$trust->subrogator->relation->id)
					{
						alert('['.$_SESSION['session_user_id'].'] tried to delete trust ['.$trust->id.']');
						throw new InterfacePermissionException('['.$_SESSION['session_user_id'].'] tried to delete trust ['.$trust->id.']');
					}
					if($trust->approved == TRUST_APPROVED)
					{
						$thrownException .= $unableToRemove;
						continue;
					}
					$trust->delete();
				}
			}
			$this->init($this->id);
		}
    }
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 10-03-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceSubrogationOverviewItem extends InterfaceItem
{

    /**
     * item[765] =
     */
    function getOverview()
    {
		$this->hiddenFields = array(array('name'=>'subrogator[id]', 'value'=>$_REQUEST['subrogator']['id']));
		$trust = new Trust($this->db_app);
		$ds = $trust->selectSubrogationOverview($_REQUEST['subrogator']['id'],$_SESSION['selected_product_id']);

		require_once(IWARE.'TableFactoryColumnPlugin.php');
		$tf = new TableFactoryColumnPlugin(false,$this->langIf,$this->db,$this->db_app);
		$tf->setEmptyText($this->langIf->getText('No subrogators found') );
		$tf->disableDelete = true;
		$tf->setData($ds, 'ParticipantSubrogationOverview',false,true);

		$help = ($this->parent->title != '') ? getHelp($this->langIf, $this->parent->handle, array('title'=>$this->parent->handle, 'top'=>'0px')) : '';
		$tf->setCaption($help.'<img src="/'.SITE_PATH.'images/block_arrow.png" /> '.$this->langIf->getText('Subrogation Overview'), false);
		/*
		$tf->setValueProcessor('APPROVED', '
			$erv = \'\';
			foreach(array(TRUST_SUBROGATOR_APPROVED, TRUST_TRUSTEE_APPROVED, TRUST_CRSO_APPROVED) as $approved)
				$erv .= \'<img src="/\'.SITE_PATH.\'images/i-w/\'.((($value & $approved)==$approved)?\'true\':\'false\').\'.gif" />\';
			return $erv;');
		*/
		$p = 'return \'<img src="/\'.SITE_PATH.\'images/i-w/\'.($value?\'true\':\'false\').\'.gif" />\';';
		$tf->setValueProcessor('TRUST_SUBROGATOR_APPROVED', $p);
		$tf->setValueProcessor('TRUST_TRUSTEE_APPROVED', $p);
		$tf->setValueProcessor('TRUST_CRSO_APPROVED', $p);
		$tf->setValueProcessor('TRUST_ID',array($this,'tfTrustId'));

		$tf->setValueProcessor('DETAILS', array($this,'tfDetails'));
		$tf->setValueProcessor('CHECKBOX', 'return \'<input type="checkbox" value="\'.$row[\'TRUST_ID\'].\'" name="deleteTrust[]"\'.(($row[\'APPROVED\']==TRUST_APPROVED)?\' disabled="disabled"\':\'\').\' />\';');

		$tf->setValueProcessor('AGREEMENT_DOCUMENT',array($this,'tfAgreementDocument'));

		$tf->hideColumns(array('CHECKBOX', 'TRUSTEE_RELATION_ID','CRSO_RELATION_ID', 'APPROVED'));

		$this->text = $tf->getXHTMLTable('Professional',2);
		$this->if_obj->disableDelete = $tf->disableDelete;
    }


	function tfDetails($args)
	{
		$trustDisabled = $this->langIf->getText('Trust has been terminated');
		if($args['row']['APPROVED']!=TRUST_DISABLED)
			return '<a id="trust_details_'.$args['row']['TRUSTEE_RELATION_ID'].'_'.$args['row']['CRSO_RELATION_ID'].'" href="InterfaceTrustDetail.php?trust[id]='.$args['row']['TRUST_ID'].'&subrogator[id]='.$_REQUEST['subrogator']['id'].'">'.$this->langIf->getText('Subrogator Details').'</a>';
		return $trustDisabled;
	}

	function tfAgreementDocument($args)
	{
		if(!$args['value'])
			return '<img src="/'.SITE_PATH.'images/i-w/false.gif" />';

		require_once(ELC_CLASSES.'Document.php');
		$doc = new ELCDocument($this->db_app,$args['value'],true);
		return sprintf('<a href="/%sscripts/i-w/document/download.php?Document[id]=%d">%s</a>',
			SITE_PATH,
			$args['value'],
			$doc->name);
	}

    function tfTrustId($args)
    {
    	if($args['row']['APPROVED'] == TRUST_DISABLED || ($args['row']['APPROVED'] & TRUST_TRUSTEE_APPROVED))
    		return '<a href="#" onclick="trustEditDisabled();">'.$args['value'].'</a>';
    	return sprintf("<a href=\"/%sscripts/elc/participant/InterfaceSubrogationApplication.php?trust[id]=%d\">%d</a>",SITE_PATH,$args['value'],$args['value']);
    }

    /**
     * item[766] =
     */
    function getButtons()
    {
	$rv = null;
		// Check for any active Remote Viewer connections.
		if(IWRegistry::isRegistered('remote_viewer')) {
			$rv = IWRegistry::registry('remote_viewer');
		}
		// Get all CRSO for participant

		require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
		require_once(ELC_CLASSES.'Trust.php');
		$cp = new CRSOParticipantProduct($this->db_app);
		$trust = new Trust($this->db_app);
		$q = $cp->getParticipantCRSOs($_REQUEST['subrogator']['id'],'q',false,false,$_SESSION['selected_product_id']);
		$rs = $q->fetchAll();
		// Create script
		$saveEnabled = false;
		$this->script = "
			function confirmSubrogationDelete()
			{
				MessageBox.prototype.Question('".jsescape($this->langIf->getText('Confirm delete'),"'")."', '".jsescape($this->langIf->getText('Are you sure you want to remove subrogations'),"'")."', function(){
					$$('process').value='delete';
					$('interface_form').submit();
					});
			}
			function newSubrogation()
			{
				redirurl = 'InterfaceSubrogationApplication.php?subrogator[id]=".$_REQUEST['subrogator']['id']."';
				";
		if(count($rs)==1)
		{
			$r = $rs[0];
			$t = $trust->selectAvailableTrustees($_REQUEST['subrogator']['id'],$r->CRSO_RELATION_ID,$_SESSION['selected_product_id']);
			$rs2 = $t->fetchAll();
			$saveEnabled = (bool) count($rs2);
			$this->script .= "crso = '".$r->CRSO_RELATION_ID."';";
		}
		else
			$this->script .= "crso = $('crso[id]').value;\n";
		$this->script .= "
				go(redirurl+'&crso[id]='+crso);
			}
			";
		$noCrsos = true;
		if(count($rs) > 1)
		{
			require_once(ELC_CLASSES.'Country.php');
			$country = new Country($this->db_app);
			$this->text = '<select style="float: left;" id="crso[id]">
				<option value="">- '.$this->langIf->getText('Select CRSO').' -</option>';
			foreach($rs as $r)
			{
				if($rv && $rv->crso->relation->id!=$r->CRSO_RELATION_ID)
					continue;
				$relation = new Relation($this->db_app,$r->CRSO_RELATION_ID,true);
				/// Check the country settings
				$country->set($relation->getCountry());
				if(!$country->has(HasSubrogation,$_SESSION['selected_product_id']))
					continue;
				$noCrsos = false;
				$t = $trust->selectAvailableTrustees($_REQUEST['subrogator']['id'],$r->CRSO_RELATION_ID,$_SESSION['selected_product_id']);
				$rs2 = $t->fetchAll();
				if(count($rs2))
				{
					$saveEnabled = true;
					$this->text .= '<option value="'.$r->CRSO_RELATION_ID.'">'.$r->CRSO_NAME.'</option>';
				}
			}
			if($noCrsos)
				$this->text = '';
			else
				$this->text .= '</select>';
			if(!$saveEnabled)
				$this->text = '<span style="float: left;">'.$this->langIf->getText('No Participants Available for Trust').'</span>';
		}

		$this->_BarButtons = BUTTON_DELETE;
		$disabled = $this->if_obj->disableDelete?array('disabled'=>'disabled'):array();
		$disabledNew = $saveEnabled?array(''):array('disabled'=>'disabled');
		$this->_ButtonOptions[BUTTON_DELETE] = array_merge($disabled,array('onclick'=>"confirmSubrogationDelete();"));
		$this->_addCustomButton($this->langIf->getText('New Subrogation'),'button', array_merge(array('onclick'=>"newSubrogation();"),$disabledNew));
    }

}

/**
 * CALLBACK[getOverview]
 * item[765] = {
 *     [handle] = Item: SubrogationOverview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[766] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
