<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'RelationUser.php');
require_once(ELC_CLASSES.'Trust.php');
/**
 *
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @since 15-03-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceTrustOverview extends InterfaceFactory
{
    /**
     * interface[id] = 89
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceTrustOverview(&$db, &$db_app, &$langIf)
    {
		// Verify the request
		$ru = new RelationUser($db_app);
		$ru->set($_SESSION['session_user_id'],$_REQUEST['trustee']['id']);
		if(!$ru->hasClearance(ClearanceFeeRegistration))
		{
			alert('['.$_SESSION['session_user_id'].'] no permission for trust for ['.$_REQUEST['trustee']['id'].']');
			throw new InterfacePermissionException('['.$_SESSION['session_user_id'].'] no permission for trust for ['.$_REQUEST['trustee']['id'].']');
		}
	$rv = null;
		// Check for any active Remote Viewer connections.
		if(IWRegistry::isRegistered('remote_viewer')) {
			$rv = IWRegistry::registry('remote_viewer');
		}
		if($rv &&  $rv->participant->relation->id!=$_REQUEST['trustee']['id']) {
			alert('['.$_SESSION['session_user_id'].'] no permission for trust due to Remote Viewer for ['.$_REQUEST['subrogator']['id'].']');
			throw new InterfacePermissionException('['.$_SESSION['session_user_id'].'] no permission for trust for ['.$_REQUEST['subrogator']['id'].']');
		}
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

		$ru->relation->set();
		setPageTitlePostfix('('.$ru->relation->name.')');
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {

    }
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 15-03-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceTrustOverviewItem extends InterfaceItem
{

    /**
     * item[772] =
     */
    function getOverview()
    {
		$this->hiddenFields = array(array('name'=>'trustee[id]', 'value'=>$_REQUEST['trustee']['id']));
		$trust = new Trust($this->db_app);
		$ds = $trust->selectTrustOverview($_REQUEST['trustee']['id'],$_SESSION['selected_product_id']);

		require_once(IWARE.'TableFactoryColumnPlugin.php');
		$tf = new TableFactoryColumnPlugin(false,$this->langIf,$this->db,$this->db_app);
		$tf->setEmptyText($this->langIf->getText('No trusts found') );
		$tf->setData($ds, 'ParticipantTrustOverview',false,true);

		$help = ($this->parent->title != '') ? getHelp($this->langIf, $this->parent->handle, array('title'=>$this->parent->handle, 'top'=>'0px')) : '';
		$tf->setCaption($help.'<img src="/'.SITE_PATH.'images/block_arrow.png" /> '.$this->langIf->getText('Trust Overview'), false);

		$tf->setValueProcessor('DETAILS', array($this,'tfDetails'));
		/*
		$tf->setValueProcessor('APPROVED', '
			$erv = \'\';
			foreach(array(TRUST_SUBROGATOR_APPROVED, TRUST_TRUSTEE_APPROVED, TRUST_CRSO_APPROVED) as $approved)
				$erv .= \'<img src="/\'.SITE_PATH.\'images/i-w/\'.((($value & $approved)==$approved)?\'true\':\'false\').\'.gif" />\';
			return $erv;');
		*/
		$p = 'return \'<img src="/\'.SITE_PATH.\'images/i-w/\'.($value?\'true\':\'false\').\'.gif" />\';';
		$tf->setValueProcessor('TRUST_SUBROGATOR_APPROVED', $p);
		$tf->setValueProcessor('TRUST_TRUSTEE_APPROVED', $p);
		$tf->setValueProcessor('TRUST_CRSO_APPROVED', $p);

		$tf->setValueProcessor('AGREEMENT_DOCUMENT',array($this,'tfAgreementDocument'));

		$tf->hideColumns(array('SUBROGATOR_RELATION_ID','CRSO_RELATION_ID','APPROVED'));
		$this->text = $tf->getXHTMLTable('Professional',2);
    }

	function tfDetails($args)
	{
		$trustDisabled = $this->langIf->getText('Trust has been terminated');
		if($args['row']['APPROVED']!=TRUST_DISABLED)
			return '<a id="trust_details_'.$args['row']['SUBROGATOR_RELATION_ID'].'_'.$args['row']['CRSO_RELATION_ID'].'" href="InterfaceTrustDetail.php?trust[id]='.$args['row']['TRUST_ID'].'&trustee[id]='.$_REQUEST['trustee']['id'].'">'.$this->langIf->getText('Trust Details').'</a>';
		return $trustDisabled;
	}

	function tfAgreementDocument($args)
	{
		if(!$args['value'])
			return '<img src="/'.SITE_PATH.'images/i-w/false.gif" />';

		require_once(ELC_CLASSES.'Document.php');
		$doc = new ELCDocument($this->db_app,$args['value'],true);
		return sprintf('<a href="/%sscripts/i-w/document/download.php?Document[id]=%d">%s</a>',
			SITE_PATH,
			$args['value'],
			$doc->name);
	}

}

/**
 * CALLBACK[getOverview]
 * item[772] = {
 *     [handle] = Item: TrustOverview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>