<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
//require_once(IWARE.'GPGCrypter.php');
require_once(IWARE.'encryption.php');
require_once(ELC_CLASSES.'RefundRequest.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');

require_once(ELC_CLASSES.'RefundRequestImporterToken.php');
require_once(IWARE.'VATChecker.php');

define('Block_export_button_bar',174);

/**
 *
 * @package ELC
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @since 22-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */

/**
 * Interface explanation
 *
 * The user arrives and sees a vat input form. When he submits this it is
 * validated against the token and a refund object is created to show the
 * details.
 */

class InterfaceRefundEmailAuthorize extends InterfaceFactory
{
    /**
     * interface[id] = 127
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceRefundEmailAuthorize(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

		// save the token in the interface add a hidden field for the token which will be passed on
		if (empty($_POST['token']) && empty($_GET['token']))
			throw new IWUserException('No token supplied');

		$this->token = isset($_GET['token']) ? $_GET['token'] : $_POST['token'];
		$this->hiddenFields[] = array('name'=>'token', 'value'=>$this->token);

		$this->model['rr'] = false;

		/**
		 * Check if the user submitted a vat_no so lets check if there is a
		 * token + vat_no combination.. then we set the refund object in $this-
		 * >model
		 */
    	if (!empty($_POST['importer']['vat_no']))
    	{
    		$vat_no = $_POST['importer']['vat_no'];
    		$token = $_REQUEST['token'];

			// make a hidden input for vat_no
			$this->hiddenFields[] = array('name'=>'importer[vat_no]', 'value'=>$vat_no);

			// get the rr from arguments.. this might throw a user exception
			$this->model['rr'] = $this->getImporterPoE($token, $vat_no);

			// if we get here we are successful
			info(sprintf('EmailToImporter: succesful combination of vat [%s] and token [%s] to refund [%s]', $vat_no, $token, $this->model['rr']->id));

			// just to make sure.. set some common objects.. this will be used in this class and in the nestled interfaces.. weee
			$this->model['rr']->participant->set();
			$this->model['rr']->participant->relation->set();
			$this->model['rr']->participant->relation->address->set();
			$country = new Country($this->db_app, $this->model['rr']->participant->relation->address->country_id, true);
			$this->model['rr']->participant->relation->address->country = $country;

			$this->model['rr']->crso->set();
			$this->model['rr']->crso->relation->set();
			$this->model['rr']->crso->relation->address->set();
			$country = new Country($this->db_app, $this->model['rr']->crso->relation->address->country_id, true);
			$this->model['rr']->crso->relation->address->country = $country;

			$this->model['rr']->product->set();
    	}
    }

	/**
	 * lovely.. thanks to the importer not being a normal user in the system we
	 * cannot use the controller (it does authentication).. so for now ill just
	 * put it here.
	 */
	function getImporterPoE($token, $vat_no)
	{
		$token_obj = new RefundRequestImporterToken($this->db_app);

		// try to get the token out of the db
		if (!$token_obj->setByToken($_REQUEST['token']))
			throw new IWUserException(sprintf('Token [%s] does not exist or is expired', $token));

		// if it is found lets get information about the refund request
		$token_obj->refund_request->set();

		// compare the vat of the user against the vat in the PoE
		if (VATChecker::normalizeVAT($token_obj->refund_request->importer_vat_no) != VATChecker::normalizeVAT($vat_no))
		{
			// log this error
			trigger_error(sprintf('VAT [%s] with token [%s] combination is invalid', $vat_no, $token), E_USER_NOTICE);

			// dont tell the user the combination is wrong.. just say it doesnt exist
			throw new IWUserException(sprintf('Token [%s] does not exist or is expired', $token));
		}

		// this shouldnt happen else we trow a IWException (not for user)
		$token_obj->refund_request->export_country->set();
        if(!($token_obj->refund_request->authorisation_by_type==REFUND_AUTHORIZE_EMAIL || $token_obj->refund_request->export_country->region_id == REGION_EU))
			throw new IWException(sprintf('RefundRequest [%s] is invalid for email to importer', $token_obj->refund_request->id));

        // this checks if the importer already confirmed it
		if ($token_obj->refund_request->status > REFUND_REQUEST_PENDING)
			throw new IWUserException(sprintf('RefundRequest [%s] does not require confirmation anymore', $token_obj->refund_request->id));

		return $token_obj->refund_request;
	}

	/**
	 * Here we hide the blocks if the user didnt yet validate his vat
	 * against the token. When he submitted and it is validated we just hide
	 * the vat block
	 */
    function init($id)
	{
		$this->id = $id;
		parent::init($id);

        if (!$this->model['rr'])
        {
        	unset(
        		$this->blocks[310],  // Block: Disclaimer
        		$this->blocks[308],    // Block: Legal
        		$this->blocks[315],    // Block: ExporterDetails
        		$this->blocks[311]
        	);  //Block: RefundDetails
        }
        else {
        	unset($this->blocks[465]); // Block: VAT
        }
	}

    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        if(isset($_POST['process']))
        {
        	// in the check_vat step we just return
        	if($_POST['process']=='check_vat')
        		return;

    		$vat_no = $_POST['importer']['vat_no'];
    		$token = $_REQUEST['token'];

    		// get the poe for the vat / token.. this should throw an exception on any error
			$rr = $this->getImporterPoE($token, $vat_no);

        	if($_POST['process']=='confirm')
        	{
        		$rr->importer_status = REFUND_IMPORTER_STATUS_APPROVED;
        		if($rr->authorisation_by_type==REFUND_AUTHORIZE_EMAIL)
        			$rr->status = REFUND_REQUEST_CONFIRMED;
        	}

        	elseif($_POST['process']=='reject')
        	{
        		$rr->importer_status = REFUND_IMPORTER_STATUS_REJECTED;

        		// Send notification to PwC.
        		$this->_sendOperatorNotification();
        		if($rr->authorisation_by_type==REFUND_AUTHORIZE_EMAIL)
        			$rr->status = REFUND_REQUEST_REJECTED;
        	}

        	$rr->update();
			info(sprintf('EmailToImporter: updated PoE [%s] to status [%s]', $rr->id, $rr->status));

        	header('Location: /'.SITE_PATH.'scripts/elc/public/RefundEmailAuthorizeFinish.php');
        	exit();
        }
    }

    function _sendOperatorNotification()
    {
    	$this->model['rr']->crso->relation->set();
    	$this->model['rr']->participant->relation->set();
		$langIf = IWRegistry::registry('langIf'); // FIXME: dispatcher or something
		require_once(ELC_CLASSES.'Notification.php');
		Notification::send($this->db_app,
			array('subject'=>$langIf->getText('Notification: PoE Rejected by Importer'),
				'message'=>
					$langIf->getText("Notification: Rejected PoE ({PoE_ID} at {CRSO} for {PARTICIPANT})",
					null,
					array('PoE_ID'=>$this->model['rr']->id,
						'CRSO'=>$this->model['rr']->crso->relation->name,
						'PARTICIPANT'=>$this->model['rr']->participant->relation->name))
			),
			array(
			array('recipient'=>new Relation($this->db_app,1), // operator
				'callback'=>"go('/".SITE_PATH."scripts/elc/operator/InterfaceRefundExportList.php?CRSO_RELATION_ID=".
					$this->model['rr']->crso->relation->id."&PARTICIPANT_RELATION_ID=".
					$this->model['rr']->participant->relation->id."')"))
		);
    }
}
/**
 * @package ELC
 * @subpackage Interface
 * @since 22-06-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceRefundEmailAuthorizeItem extends InterfaceItem
{
    /**
     * item[947] =
     */
    function getLegalText()
    {
		if (!$this->if_obj->model['rr'])
	    	return;

    	$rr =& $this->if_obj->model['rr'];
    	$c = new Country($this->db_app,$rr->crso->relation->address->country_id,true);

    	if($c->region_id!=REGION_EU)
    		trigger_error('The exporter country is not a EU country',E_USER_ERROR);

    	$this->text = $c->getLegalNotice();
    }

	function getDisclaimerText()
	{
		$this->text = $this->langIf->getText('RefundEmailAuthorize::Disclaimer');
	}

	function getRefundDetails()
	{
		if (!$this->if_obj->model['rr']) {
	    	$this->showInterface = false;
	    	return;
		}

		$_REQUEST['refund']['id'] = $_GET['refund']['id'] = $this->if_obj->model['rr']->id;

		$this->settings = array(
			'block_callbacks'=>array(267=>array($this,'setTitleProofOfExportDetailsBlock')),
			'rr'=>$this->if_obj->model['rr'],
			'emailtoimporterinteface'=>true,
			'unsetblocks'=>array(Block_export_button_bar));
	}

	function getCompanyInterface()
	{
		if (!$this->if_obj->model['rr']) {
	    	$this->showInterface = false;
	    	return;
		}
		require_once(ELC_CLASSES.'CRSOParticipant.php');

		$this->settings = array(
			'block_callbacks'=>array(146=>array($this,'setTitleCompanyBlock')),
			'rel'=>$this->if_obj->model['rr']->participant->relation,
			'participant'=>$this->if_obj->model['rr']->participant,
			'cp'=>new CRSOParticipant($this->db_app,array('crso'=>$this->if_obj->model['rr']->crso)),
			'_hideInputs'=>'return true;');
	}

	function setTitleCompanyBlock($block)
	{
		$block->title = $this->langIf->getText('EmailAuthorize: Exporting Company');
	}

	function setTitleProofOfExportDetailsBlock($block)
	{
		$block->title = $this->langIf->getText('EmailAuthorize: Export Details');
	}

    /**
     * item[948] =
     */
    function getButtons()
    {
    	$this->_BarButtons = 0;

		$this->script = '
			function doRequest(process)
			{
				if (fb.validate($(\'interface_form\'))){
					$(\'interface_form\').process.value = process
					$(\'interface_form\').submit();
				}
			}
			';
		if (!$this->if_obj->model['rr'])
		{
	    	$this->_addCustomButton('Next','button',array('onclick'=>'doRequest(\'check_vat\');'));
	    	return;
		}

    	$this->_addCustomButton('Confirm','button',array('onclick'=>'doRequest(\'confirm\');'));
    	$this->_addCustomButton('Reject','button',array('onclick'=>'doRequest(\'reject\');'));
    }

    /**
     * item[1351] =
     */
    function getImporterVATDescription()
    {
    	$this->text = $this->langIf->getText('Importer VAT Description');
    }

    /**
     * item[1350] = importer[vat_no]
     */
    function getImporterVAT()
    {
    }

}

/**
 * CALLBACK[getAgreementText]
 * item[947] = {
 *     [handle] = Item: AgreementText
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[948] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
