<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(ELC_CLASSES.'AccountantCRSO.php');

define('BLOCK_CLEARANCE_SELECT', 533);
/**
 *
 * @package
 * @subpackage Interface
 * @since 20-05-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceAccountantCRSORegistration extends InterfaceFactory
{
    /**
     * interface[id] = 194
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceAccountantCRSORegistration(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        $accountant = new Accountant($this->db_app, $_REQUEST['accountant'], true);
        if(!$accountant->isComplete()) {
        	header('Location: InterfaceAccountantOverview.php');
        	exit();
        }
        $this->hiddenFields[] = array('name'=>'accountant[id]', 'value'=>(int)$accountant->relation->id);

        $accountant->relation->set();
        setPageTitlePostfix(sprintf(' (%s)', $accountant->relation->name));

        $this->model['ac'] = new AccountantCRSO($this->db_app, array('accountant'=>$accountant));
        if(!empty($_REQUEST['crso']['id'])) {
        	$this->model['ac']->crso = new CRSO($this->db_app, array('relation'=> new Relation($this->db_app, $_REQUEST['crso']['id'])), true);
        	if(!$this->model['ac']->crso->isComplete()) {
        		throw new IWException('Invalid CRSO Id Provided');
        	}
        	$this->model['ac']->setByAccountantAndCRSO();
        }

    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
    	if(!isset($_POST['process']))
    		return;
        // On save, if clearance levels have changed, any non existing ones should also be removed from
        // any AccountantCRSO-User relation.
        if($_POST['process'] == 'save') {
        	// TODO: do we have to prevent changes on start/end date? Or just be careful with whatcha doing?
        	$this->model['ac']->start_date = reindex_date($_POST['ac']['start_date']);
        	$this->model['ac']->end_date = empty($_POST['ac']['end_date']) ? null : reindex_date($_POST['ac']['end_date']);
        	$this->model['ac']->available_clearance_levels = isset($_POST['ac']['available_clearance_levels']) ? array_sum($_POST['ac']['available_clearance_levels']): 0;

        	if($this->model['ac']->isComplete()) {
        		$this->model['ac']->update();
        	} else {
        		$this->model['ac']->insert();
        	}
        	header('Location: InterfaceAccountantCRSOOverview.php?accountant[id]='.$this->model['ac']->accountant->relation->id);
        	exit();
        }
    }
}
/**
 * @package
 * @subpackage Interface
 * @since 20-05-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceAccountantCRSORegistrationItem extends InterfaceItem
{

    /**
     * item[1537] = crso[id]
     */
    function getCRSOSelection() {
    	$crso = new CRSO($this->db_app);
    	$q = $crso->selectCRSOs();

		$accountant_crsos = array();
		$ac = $this->if_obj->model['ac']->getAccountantCRSOs();
		while($s=$ac->fetch())
		$accountant_crsos[] = $s->CRSO_RELATION_ID;

		$this->options = array(array('text'=>'- '.$this->langIf->getText('CRSO Selection').' -','value'=>''));
    	$i = 1;
    	while($r = $q->fetch()) {
			if(!in_array($r->CRSO_RELATION_ID, $accountant_crsos)){
				$this->options[$i] = array('text'=>$r->NAME,'value'=>$r->CRSO_RELATION_ID);
				if($this->if_obj->model['ac']->crso && ($this->if_obj->model['ac']->crso->relation->id==$r->CRSO_RELATION_ID))
					$this->options[$i]['selected'] = 'selected';
				$i++;
			}
    	}

    	$this->properties['onchange'] = '_reloadClearance(this);';

	    $this->postscript = "
		    function _reloadClearance(el)
		    {
		        var params = new Array();
		        params['accountant'] = new Array();
		        params['accountant']['id'] = ".$this->if_obj->model['ac']->accountant->relation->id.";
		    	if(el.value!=='') {
		        	params['crso'] = new Array();
		        	params['crso']['id'] = el.value;
		        }
		        params['interface_id'] = ".$this->if_obj->id."; // you MUST supply this
		        ifHandler.loadBlock( new Array(new String('".BLOCK_CLEARANCE_SELECT."')), params );
		    }";
    }

    /**
     * item[1538] =
     */
    function getSpacer()
    {
    	$this->_twoCols = true;
    }

    /**
     * item[1539] = ac[start_date]
     */
    function getStartDate()
    {
    	if(!empty($this->if_obj->model['ac']->start_date))
    		$this->properties['value'] = reindex_date($this->if_obj->model['ac']->start_date, 'YYYY-MM-DD', 'DD-MM-YYYY');
    }

    /**
     * item[1540] = ac[end_date]
     */
    function getEndDate()
    {
    	if(!empty($this->if_obj->model['ac']->end_date))
    		$this->properties['value'] = reindex_date($this->if_obj->model['ac']->end_date, 'YYYY-MM-DD', 'DD-MM-YYYY');
    }

    /**
     * item[1541] = ac[available_clearance_levels]
     */
    function getClearanceLevels() {
    	$props = array('class'=>'button');
		$this->fancybuttons = array('right'=>$props, 'left'=>$props, 'allRight'=>$props, 'allLeft'=>$props);

		$this->properties['size'] = 8;
		$this->properties['style'] = 'width: 60ex;';

		$this->selectedoptions = $this->options = array();

		foreach(Accountant::getClearanceLevels() as $clearance) {
			$value = constant($clearance);
			$opt = array('text'=>$this->langIf->getText($clearance),'value'=>$value);
			if(($this->if_obj->model['ac']->available_clearance_levels & $value) == $value) {
				$this->selectedoptions[] = $opt;
			} else {
				$this->options[] = $opt;
			}
		}
    }

    /**
     * item[1542] =
     */
    function getButtons() {
    	$this->_BarButtons = BUTTON_SAVE+BUTTON_CANCEL;
    	$this->_ButtonOptions[BUTTON_CANCEL] = array(
    		'onclick'=>sprintf("document.location='InterfaceAccountantCRSOOverview.php?accountant[id]=%s';",
    			$this->if_obj->model['ac']->accountant->relation->id)
    	);
    }

}

/**
 * CALLBACK[getCRSOSelection]
 * item[1537] = {
 *     [handle] = Item: CRSO
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getSpacer]
 * item[1538] = {
 *     [handle] = Item: Spacer
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getStartDate]
 * item[1539] = {
 *     [handle] = Item: Start Date
 *     [behaviour] = required
 *     [help] = 1
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getEndDate]
 * item[1540] = {
 *     [handle] = Item: End Date
 *     [behaviour] =
 *     [help] = 1
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getClearanceLevels]
 * item[1541] = {
 *     [handle] = Item: Available Clearance Levels
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[1542] = {
 *     [handle] = Item: Button Bar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
