<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
require_once(IWARE.'address.php');
require_once(IWARE.'InfoPerson.php');
require_once(IWARE.'nLanguage.php');
require_once(IWARE.'RelationType.php');
require_once(ELC_CLASSES.'Accountant.php');
/**
 *
 * @package
 * @subpackage Interface
 * @since 20-05-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceAccountantRegistration extends InterfaceFactory
{
    /**
     * interface[id] = 192
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceAccountantRegistration(&$db, &$db_app, &$langIf)
    {
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';
        if(isset($_POST['process']))
			$this->_setValues = true;
        parent::InterfaceFactory($db, $db_app, $langIf);

        // If we are provided with an ID, we want to load the data for this accountant.
    	if(isset($_REQUEST['accountant']['id'])) {
    		$ac = new Accountant($this->db_app, $_REQUEST['accountant']['id'], true);
    		if(!$ac->isComplete()) {
    			// Apparently not an Accountant!
    			header('Location: /'.SITE_PATH.'scripts/elc/sysadmin/InterfaceAccountantOverview.php');
				exit();
    		}
    		$ac->relation->set();
    		$ac->relation->address->set();
    		$ac->relation->customer_contact->set();

    		$this->relation = (array) $ac->relation;
    		$this->address = (array) $ac->relation->address;
    		$this->contact = (array) $ac->relation->customer_contact;

    		$this->model['isComplete'] = false;
    	}

    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
        $this->process = 'check';
        if(!$this->fb->validate())
			return $thrownException .= $this->langIf->getText('Please correct the form errors');
    	if(isset($_POST['process'])) {
    		// Confirmation.
			if($_POST['process'] == 'check') {
				$this->process = 'save';
				$this->_hideInputs = true;

				global $page_title;
				$page_title = 'Accountant Registration Confirmation';
				return;
			}
			// Real saving.
			if($_POST['process'] == 'save') {
				// Start a transaction.
				$this->db_app->begin();

				// Are we dealing with a new accountant or updating an existing one.
				$update = isset($_REQUEST['accountant']['id']);
				$ac = new Accountant($this->db_app);

				$relation = $_POST['relation'];

				$contact = $_POST['contact'];
				$contact['telhouse'] = $_POST['relation']['phone'];
				$contact['fax'] = $_POST['relation']['fax'];
				$contact['email'] = $_POST['relation']['email'];
				$contact['company_id'] = $_SESSION['session_user_company_id'];
				$contact['language_id'] = $_SESSION['session_user_language_id'];
				$relation['customer_contact'] = new InfoPerson($this->db_app,$contact);
				// Add the address settings
				$relation['address'] = new Address($this->db_app,$_POST['address']);
				// Add the relation type
				$relation['relation_type'] = new RelationType($this->db_app,RELATION_ACCOUNTANT);
				// invoice_language
				$relation['invoice_language'] = new nLanguage($this->db_app,$_SESSION['session_user_language_id']);
				$ac->relation = new Relation($this->db_app, $relation);

				if($update) {
					$relation['customer_contact']->update();
					$relation['address']->update($_POST['address']['address_id']);
					$ac->relation->update();
				} else {
					$relation['customer_contact']->insert();
					$relation['address']->insert();
					$ac->relation->insert();
					$ac->insert();
				}

				$this->db_app->commit();
				if($update) {
					// Go back to where we came from.
					header('Location: InterfaceAccountantOverview.php');
				} else {
					// New Accountant so we should probably link that to a CRSO.
					header(sprintf('Location: InterfaceAccountantCRSORegistration.php?accountant[id]=%s', $ac->relation->id));
				}
				exit();
			}
		}
    }
}
/**
 * @package
 * @subpackage Interface
 * @since 20-05-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceAccountantRegistrationItem extends InterfaceItem
{

    /**
     * item[1520] = relation[name]
     * item[1521] = contact[last_name]
     * item[1522] = address[address_street_name]
     * item[1523] = relation[phone]
     * item[1524] = address[address_zipcode]
     * item[1525] = relation[fax]
     * item[1526] = address[city_name]
     * item[1527] = relation[email]
     * item[1529] = relation[website]
     */
    function getItemValue() {
    	if(isset($_REQUEST['accountant']['id'])) {
    		if(!$this->if_obj->model['isComplete']) {

    			$this->hiddenFields = array(
    				array('name'=>'accountant[id]','value'=>$_REQUEST['accountant']['id']),
    				array('name'=>'address[address_id]','value'=>$this->if_obj->address['address_id']),
    				array('name'=>'contact[id]','value'=>$this->if_obj->contact['id']),
    				array('name'=>'relation[id]','value'=>$this->if_obj->relation['id'])
    			);

    			$this->if_obj->model['isComplete'] = true;
    		}

    		// Fill the values.
    		$name = str_replace('[','[\'',str_replace(']','\']',$this->name));
    		// NOTE: Eval, only developers have influence on the $name var.
    		eval('$this->properties[\'value\'] = isset($this->if_obj->'.$name.')?$this->if_obj->'.$name.':\'\';');
    	}
    }

    /**
     * item[1528] = address[country_id]
     */
    function getCountrySelect() {
    	require_once(ELC_CLASSES.'Country.php');
		$country = new Country($this->db_app);
		$q = $country->select();
		$this->options = array(array('text'=>'- '.$this->langIf->getText('Make Country Selection').' -','value'=>''));
		$i = 1;
		while($r = $q->fetch())
		{
			$this->options[$i] = array('text'=>$r->NAME,'value'=>$r->COUNTRY_ID);
			if(isset($this->if_obj->address['country_id']) && ($this->if_obj->address['country_id']==$r->COUNTRY_ID))
				$this->options[$i]['selected'] = 'selected';
			$i++;
		}
    }

    /**
     * item[1534] =
     */
    function getButtons()
    {
    	if(isset($_POST['process'])&& $_POST['process'] == 'check')
		{
			$this->_BarButtons = 0;
			$this->text = $this->langIf->getText('Are above settings correct?').'<br />';
			$this->_addCustomButton($this->langIf->getText('No, I would like to make changes'),'button', array('onclick'=>'document.getElementsByName(\'process\')[0].value=\'\'; document.getElementById(\'interface_form\').submit();','name'=>'signup'));
			$this->_addCustomButton($this->langIf->getText('Yes, I am ready to continue'),'submit', array('onsubmit'=>'','name'=>'signup'));
		}
		else{
			$this->_BarButtons = BUTTON_CANCEL + BUTTON_SAVE;
			$this->_ButtonOptions[BUTTON_CANCEL] = array('onclick'=>"document.location.href='InterfaceAccountantOverview.php';");
		}
    }

}

/**
 * CALLBACK[getItemValue]
 * item[1520] = {
 *     [handle] = Item: Company Name
 *     [behaviour] = companyname
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * item[1521] = {
 *     [handle] = Item: ContactName
 *     [behaviour] = personname
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[1522] = {
 *     [handle] = Item: AddressStreet
 *     [behaviour] = address
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * item[1523] = {
 *     [handle] = Item: PhoneNumber
 *     [behaviour] = phone
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[1524] = {
 *     [handle] = Item: ZipCode
 *     [behaviour] = zipcode
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * item[1525] = {
 *     [handle] = Item: FaxNumber
 *     [behaviour] = phone
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[1526] = {
 *     [handle] = Item: CityName
 *     [behaviour] = cityname
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * item[1527] = {
 *     [handle] = Item: EmailAddress
 *     [behaviour] = email
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[1529] = {
 *     [handle] = Item: WebsiteURL
 *     [behaviour] = url
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getCountrySelect]
 * item[1528] = {
 *     [handle] = Item: CountrySelect
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[1534] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
