<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');

require_once(ELC_CLASSES.'Accountant.php');
require_once(ELC_CLASSES.'AccountantCRSO.php');
/**
 *
 * @package
 * @subpackage Interface
 * @since 22-06-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceAccountantUserOverview extends InterfaceFactory
{
    /**
     * interface[id] = 196
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceAccountantUserOverview(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        $this->model['accountant'] = new Accountant($this->db_app);
        $this->model['accountant']->set($_REQUEST['accountant']['id']);
	$this->model['crsos'] = array();
	$ac = new AccountantCRSO($this->db_app);
	$q1 = $ac->getAccountantCRSOs($this->model['accountant']->relation->id);
	while($r = $q1->fetch()) {
		$this->model['crsos'][$r->CRSO_RELATION_ID] = $r->NAME;
	}

        setPageTitlePostfix(sprintf(' (%s)', $this->model['accountant']->relation->name));
    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {

    }
}
/**
 * @package
 * @subpackage Interface
 * @since 22-06-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceAccountantUserOverviewItem extends InterfaceItem
{

    /**
     * Make an overview of all users with all CRSO for this Accountant as separate columns (for clearance level editing)
     * item[1546] =
     */
    function getUserOverview()
    {
	$q = $this->if_obj->model['accountant']->selectUserOverview();

	$ds = array();
	while($r = $q->fetch()) {

		$line = array(
			'USER_ID' => $r->USER_ID,
			'USER_NAME' => $r->USER_NAME,
			'CONTACT_NAME' => $r->CONTACT_NAME,
			'USER_ACTIVE' => $r->USER_ACTIVE,
			'IS_ADMIN' => $r->IS_ADMIN
		);

			foreach($this->if_obj->model['crsos'] as $id => $name) {
				$line[$name] = '';
			}

		$ds[] = $line;
	}

	$tf = new TableFactory($ds); // Don't supply langIf, we don't want to translate CRSO names!
	$tf->setEmptyText($this->langIf->getText('No Users for this Accountant') );
	$tf->setValueProcessor(TF_ALL, array($this, '_tfValueProcessor'));
	// We set a TH processor, for translation purposes, cant use the $langIf attr of TF
	$tf->setTHProcessor(array('USER_ID', 'USER_NAME', 'USER_ACTIVE', 'IS_ADMIN', 'CONTACT_NAME'), array($this, '_tfHeaderProcessor'));

	$this->text = $tf->getXHTMLTable('Professional', 2);
	$this->script = sprintf('
			function doLogin(loginname)
			{
				MessageBox.prototype.Question(
					\'%s\',
					\'%s\', function(){
						document.location.href = "/%sscripts/elc/public/DoLogin.php?loginname="+loginname;
					});
			}',
			jstr('Login as Accountant confirm title'),
			jstr('Login as Accountant confirm'),
			SITE_PATH
			);;
    }

    function _tfHeaderProcessor($args) {
		// This is passed by reference.
		$args['headerText'] = $this->langIf->getText($args['head']);
    }

    function _tfValueProcessor($args) {

	if(in_array($args['head'], $this->if_obj->model['crsos'])) {

		$ids = array_keys($this->if_obj->model['crsos'], $args['head']);
		$crso_id = $ids[0];
		// TODO: Accountant universe compat.
		return sprintf('<a href="/%sscripts/elc/sysadmin/InterfaceUserClearanceLevels.php?accountant_id=%s&crso_id=%s&user_id=%s">%s</a>',
			SITE_PATH,
			$_REQUEST['accountant']['id'],
			$crso_id,
			$args['row']['USER_ID'],
			$this->langIf->getText('Set clearance levels')
		);
	}
	switch($args['head']) {
		case 'USER_ID':
			// TODO: make link for editing purposes.
			return sprintf('<a href="/%sscripts/elc/sysadmin/InterfaceAccountantUserRegistration.php?accountant[id]=%s&user[id]=%s">%s</a>',
				SITE_PATH,
				$_REQUEST['accountant']['id'],
				$args['value'],
				$args['value']
			);
		case 'USER_NAME':
			return sprintf('<a href="#%s" name="part_%s" onclick="doLogin(\'%s\');">%s</a>',
							$_REQUEST['accountant']['id'],$_REQUEST['accountant']['id'],$args['value'],$args['value']);
		case 'IS_ADMIN':
		case 'USER_ACTIVE':
				$img = $args['value'] ? 'true' : 'false';
				return sprintf(
					'<img src="/%simages/i-w/%s.gif" />',
					SITE_PATH,
					$img
				);
			break;
		default:
			return $args['value'];
	}
    }

    /**
     * item[1547] =
     */
    function getButtons()
    {
	$this->_BarButtons = 0;

	$this->_addCustomButton($this->langIf->getText('New'),'button', array('onclick'=>sprintf("document.location.href='InterfaceAccountantUserRegistration.php?accountant[id]=%s';",$this->if_obj->model['accountant']->relation->id)));
    }

}

/**
 * CALLBACK[getUserOverview]
 * item[1546] = {
 *     [handle] = Item: User Overview
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[1547] = {
 *     [handle] = Item: Button Bar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
