<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');

require_once(ELC_CLASSES.'AccountantCRSOUser.php');
require_once(IWARE.'GroupUser.php');
/**
 *
 * @package
 * @subpackage Interface
 * @since 22-06-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceAccountantUserRegistration extends InterfaceFactory
{
    /**
     * interface[id] = 197
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceAccountantUserRegistration(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 25;
        $this->delimiter = '<br />';

        $this->model['accountant'] = new Accountant($this->db_app);
        $this->model['accountant']->set($_REQUEST['accountant']['id']);

        setPageTitlePostfix(sprintf(' (%s)', $this->model['accountant']->relation->name));

        // Hidden field for accountant id
        $this->hiddenFields = array(array('name'=>'accountant[id]','value'=>$_REQUEST['accountant']['id']));

        if(isset($_POST['process']))
			$this->_setValues = true;


        // We are dealing with editing of a user.
        if(isset($_REQUEST['user']['id'])) {

		$this->model['user'] = new ELCUser($this->db_app, $_REQUEST['user']['id'], true);
		$this->model['user']->info_person->set();

		// Add a hidden input for posting this information.
		$this->hiddenFields[] = array('name'=>'user[id]','value'=>$_REQUEST['user']['id']);
        }

    }
    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
	// Load translations.
		$noTwoAdmins = $this->langIf->getText('This user cannot be the admin');
		$duplicateUser = $this->langIf->getText('Invalid UserName, User already exists');

		// Set to check
	$this->process = 'check';
	if(isset($_POST['process'])) {

		if($_POST['process'] == 'check')
			{
				// We just checked, set process to save.
				$this->process = 'save';
				$this->_hideInputs = true;

			} elseif($_POST['process'] == 'save') {

			// Begin transaction.

			$this->db_app->begin();

				$update = isset($_REQUEST['user']['id']);
				$orgPersonId = $orgUserName = '';
				if($update)
				{
					$user = new ELCUser($this->db_app,$_REQUEST['user']['id']);
					$user->set();
					$orgUserName = $user->name;
					$orgPersonId = $user->info_person->id;
				}

				$user = $_POST['user'];
				if (CRYPT_BLOWFISH == 1)
				{
					$u1 = sha1(uniqid());
					$salt = "\$2a\$07\$" . $u1 . "\$";
					$pass = crypt(ELCUser::generatePasswd(PASSWORD_LENGTH_CRSO),$salt);
					$user['salt'] = $salt;
				}
				else
					$pass = sha1(ELCUser::generatePasswd(PASSWORD_LENGTH_CRSO));
				$user['password'] = $pass; // generate random password. users will get a link with a login token anyway
				$user['active'] = (int) !empty($_POST['user']['active']);

				$user = new ELCUser($this->db_app,$user);
				$user->info_person = new InfoPerson($this->db_app,$_POST['person']);
				if($orgPersonId) {
					$user->info_person->id = $orgPersonId;
				}
				$user->info_person->company_id = $_SESSION['session_user_company_id'];

				if($orgUserName!=$user->name && (!$user->isUniqueUserName()))
				{
					$this->db_app->rollback();
					return $thrownException .= $duplicateUser;
				}

				$ru_vals = isset($_POST['relation_user']) ? $_POST['relation_user'] : array();
				// Add the user to right Accountant
				$ru = new RelationUser($this->db_app,$ru_vals);
				$ru->user =& $user;
				$ru->relation = new Relation($this->db_app,$_REQUEST['accountant']['id']);
				$ru->is_admin = (int) !empty($_POST['relation_user']['is_admin']);
				$ru->status = 1; // shouldnt we need defines for this
				$ru->default_relation = 1;

				if($ru->is_admin&&!$ru->validateAdmin())
				{
					$this->db_app->rollback();
					return $thrownException .= $noTwoAdmins;
				}

				if(!$update)
				{
					// Add the user to the right group
					$gu = new GroupUser($this->db_app);
					$gu->group_id = GROUP_ACCOUNTANT;
					$gu->user =& $user;

					$user->info_person->insert();
					$user->insert();
					$gu->insert();
					$ru->default_product = new Product($this->db_app,$_SESSION['selected_product_id']);
					$ru->insert();
					$this->db_app->commit();

					require_once(ELC_CLASSES.'ELCMessageHandler.php');
					$user->setPasswordInfoAndToken();
					MessageEvent::send(NEW_USER, array('user'=>$user, 'relationuser'=>$ru), 'ELCMessageHandler');
					$user->login_token = ''; // we don't want to keep the login token after it was sent
				}
				else
				{
					$user->info_person->update();
					$ru->update();
					$user->update();
					$this->db_app->commit();
				}

				log_elc_event(sprintf('Accountant user [%s] %s',
					$user->name,
					$update ? 'updated' : 'created'),
					$_SESSION['session_user_id'], $_SESSION['selected_product_id'],
					(int)$_REQUEST['accountant']['id'], null
				);
				$universe = 'sysadmin'; // TODO: can eventually be Accountant universe too?
			header(sprintf('Location: /%sscripts/elc/%s/InterfaceAccountantUserOverview.php?accountant[id]=%s',
				SITE_PATH,
				$universe,
				$_REQUEST['accountant']['id']
			));
			exit();
			}


	}
    }
}
/**
 * @package
 * @subpackage Interface
 * @since 22-06-2010
 * @copyright 2010 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceAccountantUserRegistrationItem extends InterfaceItem
{

    /**
     * item[1548] = person[first_name]
     * item[1549] = person[department]
     * item[1550] = person[middle_name]
     * item[1551] = person[telhouse]
     * item[1552] = person[last_name]
     * item[1553] = person[fax]
     * item[1554] = person[title]
     * item[1555] = person[email]
     * item[1556] = person[position]
     * item[1558] = user[name]
     * item[1559] = user[active]
     * item[1560] = user[start_page]
     * item[1561] = user[is_admin]
     */
    function getItemValue() {

	if(isset($_REQUEST['user']['id'])) {

			if(!isset($this->if_obj->userInfo)) {

				$user = $this->if_obj->model['user'];

				$ru = new RelationUser($this->db_app,array('user'=>$user, 'relation'=>new Relation($this->db_app,$_REQUEST['accountant']['id'])));
				$ru->set();

				$values = array();

				$this->if_obj->person = (array) $user->info_person;
				$this->if_obj->user = (array) $user;
				$this->if_obj->relation_user = (array) $ru;
				$this->if_obj->userInfo = true;
			}

			if($this->name == 'user[name]'&&isset($_REQUEST['user']['id']))
			{
				$this->properties['style'] = 'border: 0px; background-color: transparent;';
				$this->properties['readonly'] = 'readonly';
			}
			$name = str_replace('[','[\'',str_replace(']','\']',$this->name));
			eval('$this->properties[\'value\'] = isset($this->if_obj->'.$name.')?$this->if_obj->'.$name.':\'\';');
			if($this->type_id==IF_ITEM_CHECKBOX&& $this->properties['value'])
				$this->properties['checked'] = 'checked';
			if($this->type_id==IF_ITEM_CHECKBOX)
				$this->properties['value']=1;

	}

    }

    /**
     * item[1557] = person[language][id]
     */
    function getLanguageSelect() {
	require_once(IWARE.'nLanguage.php');
		$lang = new nLanguage($this->db_app);
		$q = $lang->get();
		$this->options = array(array('value'=>'','text'=>'- '.$this->langIf->getText('Select language').' -'));
		$i = 1;
		while($r = $q->fetch())
		{
			$this->options[$i] = array('text'=>$r->LANGUAGE,'value'=>$r->LANGUAGE_ID);
			//if() {
			if($r->LANGUAGE_ID == $_SESSION['session_user_language_id'])
				$this->options[$i]['selected'] = 'selected';
			/*}
			elseif(isset($this->parent->parent->person['language_id'])&& $this->parent->parent->person['language_id']==$r->LANGUAGE_ID)
				$this->options[$i]['selected'] = 'selected';*/
			$i++;
		}
    }

    function getButtons() {
	$this->_BarButtons = BUTTON_SAVE + BUTTON_CANCEL;

	if(isset($_POST['process']) && $_POST['process'] == 'check') {
			$this->_BarButtons = 0;
			$this->text = $this->langIf->getText('Are above settings correct?').'<br />';
			$this->_addCustomButton($this->langIf->getText('No, I would like to make changes'),'button', array('onclick'=>'document.getElementsByName(\'process\')[0].value=\'\'; document.getElementById(\'interface_form\').submit();','name'=>'signup'));
			$this->_addCustomButton($this->langIf->getText('Yes, I am ready to continue'),'submit', array('onsubmit'=>'','name'=>'signup'));
		} else {

		$this->_ButtonOptions[BUTTON_CANCEL] = array(
			'onclick'=>sprintf("document.location.href='InterfaceAccountantUserOverview.php?accountant[id]=%s'",
				$this->if_obj->model['accountant']->relation->id
			)
		);
		}
    }
}

/**
 * CALLBACK[getItemValue]
 * item[1548] = {
 *     [handle] = Item: First Name
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * item[1549] = {
 *     [handle] = Item: Department
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[1550] = {
 *     [handle] = Item: Middle Name
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * item[1551] = {
 *     [handle] = Item: Phone
 *     [behaviour] = required;phone
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[1552] = {
 *     [handle] = Item: Last Name
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * item[1553] = {
 *     [handle] = Item: Fax
 *     [behaviour] = required;phone
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[1554] = {
 *     [handle] = Item: Title
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * item[1555] = {
 *     [handle] = Item: Email
 *     [behaviour] = required;email
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[1556] = {
 *     [handle] = Item: Position
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * item[1558] = {
 *     [handle] = Item: User Name
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * item[1559] = {
 *     [handle] = Item: UserActive
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * item[1560] = {
 *     [handle] = Item: Start Page
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 0
 *     [label_pos] = 4
 *
 * item[1561] = {
 *     [handle] = Item: Administrator
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getLanguageSelect]
 * item[1557] = {
 *     [handle] = Item: Language
 *     [behaviour] = required
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
