<?php
require_once(INTERFACE_OBJ."InterfaceFactory.php");
/**
 * InterfaceAccessibilitySettings
 *
 * @package I-Ware
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @since 21-11-2005
 * @version $Id$
 */
class InterfaceAccessibilitySettings extends InterfaceFactory
{
	/**
	 * @param Database $db IWARE Database
	 * @param Database $db_app APPLICATION Database
	 * @param LanguageInterface $langIf
	 */
	function InterfaceAccessibilitySettings($db, $db_app, $langIf='')
	{
		parent::InterfaceFactory($db, $db_app, $langIf);
		$this->delimiter = '<br />';
		//$this->_IfTitle = $this->langIf->getText('Accessibility settings');
	}
	/**
	 * @param string $thrownException
	 */
	function processInput(&$thrownException)
	{
		if(isset($_POST['process'])=='save')
		{
			$this->_deletePrivileges($_POST['group_company']['id'],$_POST['interface']['id']);
			$this->_insertPrivileges($_POST['group_company']['id'],array_sum($_POST['interface'][$_POST['interface']['id']]),$_POST['interface']['id']);
			// Trash current interface settings for group_company_id
			foreach($_POST['block'] as $id => $privs)
				if(!isset($privs['useparent']))
					$this->_insertPrivileges($_POST['group_company']['id'],array_sum($privs),$_POST['interface']['id'],$id);
			foreach($_POST['item'] as $block_id => $items)
				foreach($items as $item_id => $privs)
					if(!isset($privs['useparent']))
						$this->_insertPrivileges($_POST['group_company']['id'],array_sum($privs),$_POST['interface']['id'],$block_id,$item_id);

			require_once(IWARE.'IWSync.php');
			IWSync::exportInterfacePrivs();

			$this->init(17);
		}
	}
	/**
	 *
	 */
	function setSiteDB()
	{
		if(isset($this->db_site))
			return;
		if(isset($_SESSION['current_site'])&& $_SESSION['current_site']['id']>0)
		{
			global $dba_driver;
			require_once(IQADMIN.'SiteDatabase.php');
			$siteDB = new SiteDatabase($this->db_app);

			$q = $siteDB->select($_SESSION['current_site']['id']);
			if($q->rows()==1)
			{
				$r = $q->fetch();
				$db_type_id = $r->DATABASE_TYPE_ID;
			}
			else
			{
				// ToDo
			}
			$siteDB->set($_SESSION['current_site']['id'],$db_type_id);
			$this->db_site =& $siteDB->connect($dba_driver);
			return;
		}
		$this->db_site = $this->db_app;
	}
	/**#@+
	 * @access protected
	 */
	/**
	 * @param integer $gcid GROUP_COMPANY_ID
	 * @param integer $interface_id
	 * @param integer $block_id
	 * @param integer $item_id
	 */
	function _deletePrivileges($gcid, $interface_id,$block_id='',$item_id='')
	{
		$values = array($interface_id,$gcid);
		$sql = "DELETE FROM TBL_GROUP_COMPANY_PRIVILEGES
			WHERE INTERFACE_ID = ?
				AND GROUP_COMPANY_ID = ? ";
		if($block_id)
		{
			$sql .= "AND BLOCK_ID = ? ";
			$values[] = $block_id;
		}
		if($item_id)
		{
			$sql .= "AND ITEM_ID = ? ";
			$values[] = $item_id;
		}
		$q = $this->db_site->query($sql);
		$q->bind_values($values);
		$q->execute();
	}
	/**
	 * @param integer $gcid GROUP_COMPANY_ID
	 * @param integer $privs
	 * @param integer $interface_id
	 * @param integer $block_id
	 * @param integer $item_id
	 */
	function _insertPrivileges($gcid,$privs,$interface_id,$block_id=0,$item_id=0)
	{
		$sql = "INSERT INTO TBL_GROUP_COMPANY_PRIVILEGES
			(GROUP_COMPANY_ID,INTERFACE_ID,BLOCK_ID,ITEM_ID,PRIV)
			VALUES (?, ?, ?, ?, ?)";
		$q = $this->db_site->query($sql);
		$q->bind_values(array($gcid,$interface_id,$block_id,$item_id,$privs));
		return $q->execute();
	}
	/**#@-*/
}
/**
 * InterfaceAccessibilitySettingsItem
 *
 * @package I-Ware
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @since 21-11-2005
 * @version $Id$
 */
class InterfaceAccessibilitySettingsItem extends InterfaceItem
{
	/**
	 *
	 */
	function setSiteDB()
	{
		if(isset($this->parent->parent->db_site))
			return;
		if(isset($_SESSION['current_site'])&& $_SESSION['current_site']['id']>0)
		{
			global $dba_driver;
			require_once(IQADMIN.'SiteDatabase.php');
			$siteDB = new SiteDatabase($this->db_app);

			$q = $siteDB->select($_SESSION['current_site']['id']);
			if($q->rows()==1)
			{
				$r = $q->fetch();
				$db_type_id = $r->DATABASE_TYPE_ID;
			}
			else
			{
				// ToDo
			}
			$siteDB->set($_SESSION['current_site']['id'],$db_type_id);
			$this->parent->parent->db_site =& $siteDB->connect($dba_driver);
			return;
		}
		$this->parent->parent->db_site = $this->db_app;
	}
	/**#@+
	 * Item callback functions, set in the interfaceBuilder
	 */
	/**
	 * Get all the different interfaces that are currently in use
	 */
	function getInterfaces()
	{
		$sql = "SELECT INTERFACE_ID, HANDLE, PATH
			FROM TBL_INTERFACE
			ORDER BY PATH, HANDLE";
		$q = $this->db->query($sql);
		$q->execute();
		$this->options[] = array('text'=>'---', 'value'=>'-1');
		while($r = $q->fetch())
		{
			$this->options[] = array('text'=>$r->PATH.$r->HANDLE, 'value'=>$r->INTERFACE_ID);
			if(isset($_REQUEST['interface']['id'])&& $_REQUEST['interface']['id']==$r->INTERFACE_ID)
				$this->options[(count($this->options)-1)]['selected'] = 'selected';
		}
		$this->properties['id'] = 'interfaceselect';
		$this->properties['onchange'] = 'ifHandler.loadBlock(new Array(\'56\'),\'&interface_id='.$this->parent->parent->id.'&group_company[id]=\'+document.getElementById(\'groupcompanyselect\').options[document.getElementById(\'groupcompanyselect\').selectedIndex].value+\'&interface[id]=\'+this.options[this.selectedIndex].value);';
	}
	/**
	 * Get a dropdown box with all the available group-companies
	 */
	function getGroupCompany()
	{
		$this->setSiteDB();
		$sql = "SELECT GROUP_COMPANY_ID, GROUP_NAME
			FROM TBL_GROUP_COMPANY
			INNER JOIN TBL_GROUP
				USING(GROUP_ID)
			WHERE COMPANY_ID = ?
			ORDER BY GROUP_NAME";
		$q = $this->parent->parent->db_site->query($sql);
		$q->bind_values(array($_SESSION['session_user_company_id']));
		$q->execute();
		$this->options[] = array('text'=>'---', 'value'=>'-1');
		while($r = $q->fetch())
		{
			$this->options[] = array('text'=>$r->GROUP_NAME, 'value'=>$r->GROUP_COMPANY_ID);
			if(isset($_REQUEST['group_company']['id'])&& $_REQUEST['group_company']['id']==$r->GROUP_COMPANY_ID)
				$this->options[(count($this->options)-1)]['selected'] = 'selected';
		}
		$this->properties['id'] = 'groupcompanyselect';
		$this->properties['onchange'] = 'ifHandler.loadBlock(new Array(\'56\'),\'&interface_id='.$this->parent->parent->id.'&group_company[id]=\'+this.options[this.selectedIndex].value+\'&interface[id]=\'+document.getElementById(\'interfaceselect\').options[document.getElementById(\'interfaceselect\').selectedIndex].value);';
	}
	/**
	 * Gets an overview of the settings of the items/blocks and interfaces for the selected GCID
	 */
	function getSettingOverview()
	{
		if(!isset($_REQUEST['group_company']['id'],$_REQUEST['interface']['id']))
			return;

		$this->setSiteDB();
		$priv = array();

		$sql = "SELECT gcp.INTERFACE_ID, gcp.BLOCK_ID, gcp.ITEM_ID, gcp.PRIV
			FROM TBL_GROUP_COMPANY_PRIVILEGES as gcp
			WHERE gcp.GROUP_COMPANY_ID = ?
				AND gcp.INTERFACE_ID = ?
			ORDER BY gcp.INTERFACE_ID, gcp.BLOCK_ID, gcp.ITEM_ID";
		$q = $this->parent->parent->db_site->query($sql);
		$q->bind_values(array($_REQUEST['group_company']['id'],$_REQUEST['interface']['id']));
		$q->execute();
		while($r = $q->fetch())
			$priv[$r->INTERFACE_ID][$r->BLOCK_ID][$r->ITEM_ID] = $r->PRIV;

		$inputs = '<input type="hidden" name="{name}[hidden]" value="0" />
				<input type="checkbox" name="{name}[read]" value="'.READ_ACCESS.'" {read} />
				<input type="checkbox" name="{name}[write]" value="'.WRITE_ACCESS.'" {write} />
				<input type="checkbox" name="{name}[change]" value="'.CHANGE_ACCESS.'" {change} />
				<input type="checkbox" name="{name}[delete]" value="'.DELETE_ACCESS.'" {delete} />';

		$sql = "SELECT I.INTERFACE_ID, I.HANDLE as INTERFACE_HANDLE, REPLACE('$inputs','{name}',CONCAT('interface[',I.INTERFACE_ID,']')) as INTERFACE_PRIV,
			B.BLOCK_ID, B.HANDLE as BLOCK_HANDLE,
				CONCAT('<input onclick=\"setLocalPriv(this,', B.BLOCK_ID,',\'block\')\" type=\"checkbox\" name=\"block[',B.BLOCK_ID,'][useparent]\" value=\"true\" {checked} /> ',
					REPLACE('$inputs','{name}',CONCAT('block[',B.BLOCK_ID,']'))) as BLOCK_PRIV,
			ITEM.ITEM_ID, ITEM.HANDLE as ITEM_HANDLE,
				CONCAT('<input onclick=\"setLocalPriv(this,', ITEM.ITEM_ID,',\'item\', ',B.BLOCK_ID,')\" type=\"checkbox\" name=\"item[',B.BLOCK_ID,'][',ITEM.ITEM_ID,'][useparent]\" value=\"true\" {checked} /> ',
					REPLACE('$inputs','{name}',CONCAT('item[',B.BLOCK_ID,'][',ITEM.ITEM_ID,']'))) as ITEM_PRIV
			FROM TBL_INTERFACE as I
			INNER JOIN TBL_INTERFACE_BLOCK as B
				USING(INTERFACE_ID)
			INNER JOIN TBL_INTERFACE_ITEM as ITEM
				USING(BLOCK_ID)
			WHERE I.INTERFACE_ID = ?
			ORDER BY B.POSITION, ITEM.POSITION";
		$x = $this->db->query($sql);
		$x->bind_values(array($_REQUEST['interface']['id']));
		$x->execute();

		$tf = new TableFactory($x,$this->langIf);
		$tf->collapseColumns(array('INTERFACE_ID','INTERFACE_HANDLE','INTERFACE_PRIV', 'BLOCK_ID','BLOCK_HANDLE', 'BLOCK_PRIV'));
		$tf->columnModifier(array('INTERFACE_PRIV'=>' nowrap ','BLOCK_PRIV'=>' nowrap ','ITEM_PRIV'=>' nowrap '));
		$tf->setValueProcessor(array('INTERFACE_PRIV'),
			'$VPpriv = unserialize(\''.serialize($priv).'\');
			if ( isset($previousrow) && $value == $previousrow[$orighead] ){ $tdclass && $tdclass .= \'-collapse\'; return \'\';}
			$value = str_replace("{read}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][0][0])&&($VPpriv[$row[\'INTERFACE_ID\']][0][0]&READ_ACCESS))?\'checked="checked"\':\'\'),$value);
			$value = str_replace("{write}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][0][0])&&($VPpriv[$row[\'INTERFACE_ID\']][0][0]&WRITE_ACCESS))?\'checked="checked"\':\'\'),$value);
			$value = str_replace("{change}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][0][0])&&($VPpriv[$row[\'INTERFACE_ID\']][0][0]&CHANGE_ACCESS))?\'checked="checked"\':\'\'),$value);
			$value = str_replace("{delete}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][0][0])&&($VPpriv[$row[\'INTERFACE_ID\']][0][0]&DELETE_ACCESS))?\'checked="checked"\':\'\'),$value);
			return $value;
			');
		$tf->setValueProcessor(array('BLOCK_PRIV'),
			'$VPpriv = unserialize(\''.serialize($priv).'\');
			if ( isset($previousrow) && $value == $previousrow[$orighead] ){ $tdclass && $tdclass .= \'-collapse\'; return \'\';}
			if(!isset($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0]))
			{
				$value = str_replace("{checked}", \'checked=\"checked\"\',$value);
				$value = str_replace("{read}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][0][0])&&($VPpriv[$row[\'INTERFACE_ID\']][0][0]&READ_ACCESS))?\'checked="checked"\':\'\').\' disabled="disabled"\',$value);
				$value = str_replace("{write}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][0][0])&&($VPpriv[$row[\'INTERFACE_ID\']][0][0]&WRITE_ACCESS))?\'checked="checked"\':\'\').\' disabled="disabled"\',$value);
				$value = str_replace("{change}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][0][0])&&($VPpriv[$row[\'INTERFACE_ID\']][0][0]&CHANGE_ACCESS))?\'checked="checked"\':\'\').\' disabled="disabled"\',$value);
				$value = str_replace("{delete}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][0][0])&&($VPpriv[$row[\'INTERFACE_ID\']][0][0]&DELETE_ACCESS))?\'checked="checked"\':\'\').\' disabled="disabled"\',$value);
			}
			else
			{
				$value = str_replace("{read}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0])&&($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0]&READ_ACCESS))?\'checked="checked"\':\'\'),$value);
				$value = str_replace("{write}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0])&&($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0]&WRITE_ACCESS))?\'checked="checked"\':\'\'),$value);
				$value = str_replace("{change}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0])&&($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0]&CHANGE_ACCESS))?\'checked="checked"\':\'\'),$value);
				$value = str_replace("{delete}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0])&&($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0]&DELETE_ACCESS))?\'checked="checked"\':\'\'),$value);
			}
			return $value;
			');
		$tf->setValueProcessor(array('ITEM_PRIV'),
			'$VPpriv = unserialize(\''.serialize($priv).'\');
			if ( isset($previousrow) && $value == $previousrow[$orighead] ){ $tdclass && $tdclass .= \'-collapse\'; return \'\';}
			if(!isset($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][$row[\'ITEM_ID\']]))
			{
				if(!isset($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0]))
				{
					$value = str_replace("{checked}", \'checked=\"checked\"\',$value);
					$value = str_replace("{read}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][0][0])&&($VPpriv[$row[\'INTERFACE_ID\']][0][0]&READ_ACCESS))?\'checked="checked"\':\'\').\' disabled="disabled"\',$value);
					$value = str_replace("{write}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][0][0])&&($VPpriv[$row[\'INTERFACE_ID\']][0][0]&WRITE_ACCESS))?\'checked="checked"\':\'\').\' disabled="disabled"\',$value);
					$value = str_replace("{change}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][0][0])&&($VPpriv[$row[\'INTERFACE_ID\']][0][0]&CHANGE_ACCESS))?\'checked="checked"\':\'\').\' disabled="disabled"\',$value);
					$value = str_replace("{delete}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][0][0])&&($VPpriv[$row[\'INTERFACE_ID\']][0][0]&DELETE_ACCESS))?\'checked="checked"\':\'\').\' disabled="disabled"\',$value);
				}
				else
				{
					$value = str_replace("{checked}", \'checked=\"checked\"\',$value);
					$value = str_replace("{read}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0])&&($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0]&READ_ACCESS))?\'checked="checked"\':\'\'),$value);
					$value = str_replace("{write}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0])&&($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0]&WRITE_ACCESS))?\'checked="checked"\':\'\'),$value);
					$value = str_replace("{change}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0])&&($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0]&CHANGE_ACCESS))?\'checked="checked"\':\'\'),$value);
					$value = str_replace("{delete}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0])&&($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][0]&DELETE_ACCESS))?\'checked="checked"\':\'\'),$value);
				}
			}
			else
			{
				$value = str_replace("{read}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][$row[\'ITEM_ID\']])&&($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][$row[\'ITEM_ID\']]&READ_ACCESS))?\'checked="checked"\':\'\'),$value);
				$value = str_replace("{write}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][$row[\'ITEM_ID\']])&&($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][$row[\'ITEM_ID\']]&WRITE_ACCESS))?\'checked="checked"\':\'\'),$value);
				$value = str_replace("{change}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][$row[\'ITEM_ID\']])&&($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][$row[\'ITEM_ID\']]&CHANGE_ACCESS))?\'checked="checked"\':\'\'),$value);
				$value = str_replace("{delete}", ((isset($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][$row[\'ITEM_ID\']])&&($VPpriv[$row[\'INTERFACE_ID\']][$row[\'BLOCK_ID\']][$row[\'ITEM_ID\']]&DELETE_ACCESS))?\'checked="checked"\':\'\'),$value);
			}
			return $value;
			');
		$this->text = $tf->getXHTMLTable('Professional Header',2);

		$this->postscript = "
			document.setLocalPriv = function(el,id,name,extra_id)
			{
				extra_id=extra_id?extra_id:false;
				types = new Array('read','write','change','delete');
				elDisabled = el.checked?true:false;
				for(var i=0;i<types.length;i++)
					document.getElementsByName(name+(extra_id?'['+extra_id+']':'')+'['+id+']['+types[i]+']')[0].disabled = elDisabled;
			}

		";
	}
	/**
	 * Get the buttons for the current interface
	 */
	function getButtons()
	{
		$this->hiddenFields[] = array('name'=>'process','value'=>'save');
		$this->_BarButtons = BUTTON_SAVE;
		$this->_ButtonOptions[BUTTON_SAVE] = array();
	}
	/**#@-*/
}
?>