<?php
require_once(INTERFACE_OBJ."InterfaceFactory.php");
/**
 * interfaceBuilder
 *
 * @package I-Ware
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @author Chris Wesseling <chris@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @since 13-05-2005
 * @version $Id$
 */
class InterfaceBuilder extends InterfaceFactory
{
	/**
	 * Constructor
	 * @param Database $db I_WARE_SETTINGS
	 * @param Database $db_app APPLICATION DATABASE
	 * @param LanguageInterface $langIf
	 */
	function InterfaceBuilder($db, $db_app, $langIf='')
	{
		parent::InterfaceFactory($db, $db_app, $langIf);
		$this->delimiter = '<br />';
	}
	/**
	 * Override parent class
	 * @param string $thrownException
	 * @return bool
	 */
	function processInput(&$thrownException)
	{
		$process = isset( $_POST['process'] )?$_POST['process']:'save';

		if($process == 'copy_interface')
			return $this->_copyInterface($thrownException);

		do
		{
			if ( $process == 'save' && empty($_POST['interface']['handle']) )
				break;

			if ( !$interface_id = $this->{$process.'Interface'}($_POST['interface']) )
			{
				$thrownException .= $this->langIf->getText('DB_ERROR: Unable to '.$process.' Interface');
				break;
			}

			if ( isset($_POST['block']) )
			{
				if ( empty($_POST['block']['handle']) )
					break;
				$_POST['block']['parent_id'] = $interface_id;
				$block = new InterfaceBlock($this->db, $this->db_app, $this->langIf);
				$block->preset($_POST['block']);
				if ( !$block->{$process.'Block'}($thrownException) )
					break;

				if ( isset($_POST['item']) )
				{
					if ( empty($_POST['item']['handle']) )
						break;
					$_POST['item']['parent_id'] = $block->id;
					$item = new InterfaceItem($this->db, $this->db_app, $this->langIf);
					$item->preset($_POST['item']);
					if ( !$item->{$process.'Item'}($thrownException) )
						break;
				}
			}
		}while(false);
		//reinit to make changes active
		//we are editing the current interface ;)
		require_once(IWARE.'IWSync.php');
		IWSync::exportInterfaces();
		$this->init(1);
	}
	/**
	 *
	 */
	function _copyInterface(&$thrownException)
	{
		if($_POST['interface']['id']=='new')
		{
			$thrownException .= 'Cannot copy unsaved interfaces';
			return false;
		}

		$this->db->begin();

		$q = $this->_selectInterface($_POST['interface']['id']);
		$newId = $this->saveInterface(
			array('handle'=>$_POST['interface']['copy_name'],
				'class'=>$_POST['interface']['copy_name'],
				'path'=>$_POST['interface']['path']));

		$newBlockId = $lastBlockId = -1;
		while($r = $q->fetch())
		{
			if($lastBlockId != $r->BLOCK_ID)
			{
				$lastBlockId = $r->BLOCK_ID;
				$block = new InterfaceBlock($this->db, $this->db_app, $this->langIf);
				$block->parent_id = $newId;
				$block->id = false;
				$block->position = false;
				$block->type_id = $r->BLOCK_TYPE;
				$block->handle = $r->BLOCK_HANDLE;
				$block->saveBlock($thrownException);
				$newBlockId = $block->id;
			}
			$item = new InterfaceItem($this->db, $this->db_app, $this->langIf);
			$item->parent_id = $newBlockId;
			$item->id = false;
			$item->position = false;
			$item->type_id = $r->ITEM_TYPE;
			$item->handle = $r->ITEM_HANDLE;
			$item->callback = $r->CALLBACK;
			$item->name = $r->ITEM_NAME;
			$item->close_row = $r->CLOSE_ROW;
			$item->help_enabled = $r->HELP_ENABLED;
			$item->behaviour = $r->BEHAVIOUR;
			$item->label_disposition = $r->LABEL_DISPOSITION;
			$newItemId = $item->saveItem($thrownException);
		}

		$this->db->commit();
	}
}
/**
 * interfaceItemBuilder
 *
 * @package I-Ware
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @author Chris Wesseling <chris@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @since 13-05-2005
 * @version $Id$
 */
class InterfaceBuilderItem extends InterfaceItem
{
	/**
	 * Gets all the interfaces
	 */
	function getInterfaces()
	{
		$this->parent->selectedHandles['interface'] = '';
		$this->_getSomething('interface');
	}
	/**
	 * Gets the handle of the interface
	 */
	function getInterfaceHandle()
	{
		$this->_getSomethingElse('interface');
	}
	/**
	 *
	 */
	function getInterfaceClass()
	{
		$id = (isset($_REQUEST['interface']['id']) && is_numeric($_REQUEST['interface']['id']))?$_REQUEST['interface']['id']:-1;
		$sql = "SELECT CLASS FROM TBL_INTERFACE WHERE INTERFACE_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values(array($id));
		$q->execute();
		$row = $q->fetchAll();
		if(!count($row))
			return false;
		$r = array_pop($row);
		$this->properties['value'] = $r->{'CLASS'};
	}
	/**
	 *
	 */
	function getPath()
	{
		$id = (isset($_REQUEST['interface']['id']) && is_numeric($_REQUEST['interface']['id']))?$_REQUEST['interface']['id']:-1;
		$sql = "SELECT PATH FROM TBL_INTERFACE WHERE INTERFACE_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values(array($id));
		$q->execute();
		$row = $q->fetchAll();
		if(!count($row))
			return false;
		$r = array_pop($row);
		$this->properties['value'] = $r->{'PATH'};
	}
	/**
	 * Gets all the blocks for the current interface
	 */
	function getBlocks()
	{
		$id = (isset($_REQUEST['interface']['id']) && is_numeric($_REQUEST['interface']['id']))?$_REQUEST['interface']['id']:-1;
		$this->_getSomething('block', 'WHERE INTERFACE_ID = '.$id);
		//$this->properties['size'] = 4;
		//$this->properties['ondblclick'] = 'ifHandler.moveUp(this.id, \'\');';
		//$this->properties['id'] = $this->name.'_id';
	}
	/**
	 * Gets the handle of the block
	 */
	function getBlockHandle()
	{
		$this->_getSomethingElse('block');
	}
	/**
	 * Get the different block types for the block selection
	 */
	function getBlockTypes()
	{
		foreach ( $this->parent->getTypes() as $name=>$val )
		{
			$this->options[] = array('text'=>$this->langIf->getText($name), 'value'=>$val);
			$order[$key = count($this->options)-1] = $this->options[$key]['text'];
			if ( isset($this->parent->selectedTypes['block']) && $this->parent->selectedTypes['block'] == $val )
				$this->options[$key]['selected'] = 'selected';
		}
		if ( isset($order) )
			array_multisort($order, SORT_ASC, $this->options);
		array_unshift($this->options, array('text'=>$this->langIf->getText('Select'), 'value'=>'-1'));
	}
	/**
	 * Get all the items for the current block
	 */
	function getItems()
	{
		$id = (isset($_REQUEST['block']['id']) && is_numeric($_REQUEST['block']['id']))?$_REQUEST['block']['id']:-1;
		$this->_getSomething('item', 'WHERE BLOCK_ID = '.$id);
		//$this->properties['size'] = 4;
		//$this->properties['ondblclick'] = 'ifHandler.moveUp(this.id, \'\');';
		//$this->properties['id'] = $this->name.'_id';
	}
	/**
	 * Get all the item types for the selector
	 */
	function getItemTypes()
	{
		foreach ( $this->getTypes() as $name=>$val )
		{
			$this->options[] = array('text'=>$this->langIf->getText($name), 'value'=>$val);
			$order[$key = count($this->options)-1] = $this->options[$key]['text'];
			if ( isset($this->parent->selectedTypes['item']) && $this->parent->selectedTypes['item'] == $val )
				$this->options[$key]['selected'] = 'selected';
		}
		if ( isset($order) )
			array_multisort($order, SORT_ASC, $this->options);
		array_unshift($this->options, array('text'=>$this->langIf->getText('Select'), 'value'=>'-1'));
	}
	/**
	 *
	 */
	function getOrderButton()
	{
		$type = (false!==strpos($this->handle,'BLOCK'))?'block':'item';
		$selectId = ($type=='block')?'interface':'block';
		$this->twoCols = true;
		$this->_BarButtons = 0;
		$this->_addCustomButton($this->langIf->getText('Manage Order'),'button',
			array('onclick'=>"window.open('InterfaceOrder.php?script_parent_id='+script_parent_id+'&id='+document.getElementsByName('".$selectId."[id]')[0].value+'&type=".$type."', '_order', 'width=400,height=300,scrollbars=no,status=no,toolbar=no,resizable=yes');"));
	}
	/**
	 *
	 */
	function getCopyButton()
	{
		$this->text = "
			<script type=\"text/javascript\"><!--

				function generateClassFile()
				{
					sel = document.getElementsByName('interface[id]')[0];
					if(sel.value != 'new')
						window.open('/".SITE_PATH."scripts/i-w/interface/GenerateClassFile.php?interface[id]='+sel.value, '_generate', 'width=320,height=240,resizable=yes,scrollbars=yes');
				}

			--></script>
			";
		$this->twoCols = true;
		$this->_BarButtons = 0;
		$this->_addCustomButton($this->langIf->getText('Copy Interface'),'submit', array('onclick'=>"document.forms[0].process.value='copy_interface';"));
		$this->_addCustomButton($this->langIf->getText('Generate Class File'),'button', array('onclick'=>'generateClassFile();'));
	}
	/**
	 *
	 */
	function getLabelDisposition()
	{
		$this->options = array();
		$this->options[] = array('text'=>$this->langIf->getText('Position Top'),'value'=>DISPOSITION_TOP);
		$this->options[] = array('text'=>$this->langIf->getText('Position Left'),'value'=>DISPOSITION_LEFT, 'selected'=>'selected');
		$this->options[] = array('text'=>$this->langIf->getText('Position Bottom'),'value'=>DISPOSITION_BOTTOM);
		$this->options[] = array('text'=>$this->langIf->getText('Position Right'),'value'=>DISPOSITION_RIGHT);
	}
	/**
	 *
	 */
	function getItemBehaviour()
	{
		$this->_getSomethingElse('item', 'Behaviours');
		$this->properties['size'] = 35;
	}
	/**
	 * Gets the handle of the item
	 */
	function getItemHandle()
	{
		$this->_getSomethingElse('item');
		$this->properties['size'] = 35;
	}
	/**
	 * Gets the callback function of the item
	 */
	function getItemCallback()
	{
		$this->_getSomethingElse('item', 'Callbacks');
		$this->properties['size'] = 35;
	}
	/**
	 * Gets the name of the item
	 */
	function getItemName()
	{
		$this->_getSomethingElse('item', 'Names');
		$this->properties['size'] = 35;
	}
	function getCloseRow()
	{
		(!empty($this->parent->selecteditem['Close_rows']) || !isset($this->parent->selecteditem['Close_rows'])) && $this->properties['checked'] = 'checked';
		$this->hiddenFields[] = array('name'=>$this->name, 'value'=>0);
		$this->properties['value'] = '1';
	}
	function getHelpEnabled()
	{
//		(!empty($this->parent->selecteditem['Help_enabled']) || !isset($this->parent->selecteditem['Help_enabled'])) && $this->properties['checked'] = 'checked';
		$this->hiddenFields[] = array('name'=>$this->name, 'value'=>0);
		$this->properties['value'] = '1';
	}
	/**
	 * @access private
	 * @param string $s 'interface'|'block'|'item'
	 * @param string $e what?
	 */
	function _getSomethingElse($s, $e='Handles')
	{
		isset($this->parent->{'selected'.$e}[$s]) && $this->properties['value'] = $this->parent->{'selected'.$e}[$s];
	}
	/**
	 * @todo I SMELL MORE RECURSION HERE....
	 * If TYPE would be TYPE_ID or type_id would be type, an exception could be removed.
	 * the $script .= in the foreach ( $fields ) could be moved to a function in interface.js
	 * Then that function is has to be expanded to all ITEM_TYPEs and this function will be nearly generic for all future interfaces.
	 * and hence be moved to InterfaceItem perhaps...
	 * @access private
	 * @param string $s 'interface'|'block'|'item'
	 */
	function _getSomething($s, $where='')
	{
		$selectValue = $script = '';
		$fields = array();
		$lc = strtolower($s);
		$uc = strtoupper($s);
		switch ( $uc )
		{
			case 'INTERFACE':
				$tbl = $uc;
				$fields = array('CLASS', 'PATH');
				$bn = 'new Array(2,3)'; //BlockNumbers
				$selectValue = 'PATH';
			break;
			case 'ITEM':
				$fields = array('CALLBACK', 'NAME', 'CLOSE_ROW', 'BEHAVIOUR', 'LABEL_DISPOSITION');
				$bn = false; //BlockNumbers
			case 'BLOCK':
				$tbl = 'INTERFACE_'.$uc;
				$fields[] = 'TYPE';
				isset($bn) || $bn = 'new Array(\'3\')'; //BlockNumbers
			break;
			default:
				return false;
		}
		foreach ( $fields as $field )
		{
			$script .= 'var '.$lc.ucfirst(strtolower($field)).'s = new Array();'."\n";
		}
		$id = (isset($_REQUEST[$lc]['id']) && is_numeric($_REQUEST[$lc]['id']))?$_REQUEST[$lc]['id']:-1;

		$this->options = array();

		$sql = "SELECT ${uc}_ID, HANDLE".(($void=implode(', ',$fields))?', '.$void:'')."
			FROM TBL_$tbl ".$where;
		$q = $this->db->query($sql);
		$q->execute();
		$script .= 'var '.$lc.'Handles = new Array();';
		//exit('alert("'.$where.'");');
		while ( $r = $q->fetch() )
		{
			$script .= "\n".$lc.'Handles['.$r->{$uc.'_ID'}.'] = "'.$r->HANDLE.'";';
			foreach ( $fields as $field )
				$script .= "\n".$lc.ucfirst(strtolower($field)).'s['.$r->{$uc.'_ID'}.'] = "'.$r->$field.'";';
			$this->options[] = array('text'=>($selectValue?$r->$selectValue:'').$r->HANDLE,
					'value'=>$r->{$uc.'_ID'});
			$order[$key = count($this->options)-1] = $this->options[$key]['text'];
			if ( isset($_REQUEST[$lc]['id']) && $_REQUEST[$lc]['id'] == $r->{$uc.'_ID'} )
			{
				$this->options[$key]['selected'] = 'selected';
				$this->parent->selectedHandles[$lc] = $r->HANDLE;
				foreach ( $fields as $field )
					$this->parent->{'selected'.ucfirst(strtolower($field)).'s'}[$lc] = $r->$field;
			}
		}
		if ( isset($order) )
			array_multisort($order, SORT_ASC, $this->options);
		array_unshift($this->options, array('text'=>$this->langIf->getText('New'), 'value'=>'new'));

		$this->properties['onchange'] = $lc.'Change(this);'.($bn?'ifHandler.loadBlock('.$bn.',\'&interface_id=1&'.$lc.'[id]=\'+this.options[this.selectedIndex].value);':'');
		$script .= "\ndocument.${lc}Change = function(that)
			{
				var el = document.getElementsByName('${lc}[handle]')[0];
				el.value = ${lc}Handles[that.options[that.selectedIndex].value]?${lc}Handles[that.options[that.selectedIndex].value]:'';";
		foreach ( $fields as $field )
		{
			$flc = strtolower($field); //FieldLowerCase
			$fucfp = ucfirst($flc).'s'; //FielUCFirtPlural
			if ( $field == 'TYPE' )
				$flc = 'type_id';
			$script .= "var el = document.getElementsByName('${lc}[${flc}]')[0];
				var value = ${lc}${fucfp}[that.options[that.selectedIndex].value]?${lc}${fucfp}[that.options[that.selectedIndex].value]:'';
				if ( el.type == 'text' )
					el.value = value;
				else if ( el.type == 'select-one' )
				{
					for ( var i=0 ; i < el.options.length; i++ )
						el.options[i].selected = (el.options[i].value == value);
				}
				else if ( el.type == 'hidden' )
					document.getElementsByName('${lc}[${flc}]')[1].checked = (value != '0');\n";
		}
		$script .= '};';
		$this->script = $script;
	}
}
?>
