<?php
require_once(INTERFACE_OBJ.'InterfaceFactory.php');
/**
 *
 * @package I-Ware
 * @subpackage Interface
 * @since 24-02-2006
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceLostPassword extends InterfaceFactory
{
    /**
     * interface[id] = 75
     * @param $db Database (I_WARE_SETTINGS)
     * @param $db_app Database (APPLICATION DATABASE)
     * @param $langIf LanguageInterface
     */
    function InterfaceLostPassword(&$db, &$db_app, &$langIf)
    {
        parent::InterfaceFactory($db, $db_app, $langIf);
        $this->FirstColWidth = 50;
        $this->delimiter = '<br />';

		if(isset($_REQUEST['confirm_new_pass']))
		{
			$this->script = "
				addEvent(window,'load',function(){
					".jsMessageBox(MSG_BOX_INFO, IWRegistry::registry('langIf')->getText('Forgot Password Request'), IWRegistry::registry('langIf')->getText('Your request has been processed'), 'function(){document.location.href="/'.SITE_PATH.'";}')."
				});
				";
		}

		if(isset($_REQUEST['password_request_flood']))
		{
			$this->script = "
				addEvent(window,'load',function(){
					".jsMessageBox(MSG_BOX_CRIT, IWRegistry::registry('langIf')->getText('Too many password requests'), IWRegistry::registry('langIf')->getText('You already requested a new passwd in the last {MINUTES} minutes', null, array('MINUTES'=>(int)(NEW_PASSWORD_REQUEST_TIMEOUT / 60))), 'function(){document.location.href="/'.SITE_PATH.'";}')."
				});
				";
		}

    }

    /**
     * @param string $thrownException
     */
    function processInput(&$thrownException)
    {
		$formErrors = $this->langIf->getText('Please correct the form errors');
        if(isset($_POST['process']))
		{
			if(!$this->fb->validate())
			{
				$_POST['process'] = '';
				return $thrownException .= $formErrors;
			}
			if($_POST['process'] == 'save')
			{

		        if(!$this->fb->validate())
		        	return $thrownException .= $this->langIf->getText('Please correct the form errors');

				// If both the fields are set, make sure they match.
				require_once(ELC_CLASSES.'User.php');
				require_once(ELC_CLASSES.'ELCMessageHandler.php');

				$sentTo = array();
				$sent = false;
				$user = new ELCUser($this->db_app);
				if(!empty($_POST['person']['email']))
				{
					if($users = $user->getByEmail($_POST['person']['email']))
					{
						foreach($users as $u)
						{
								if (!$u->name || !$u->password) // dont send mail if no user_name or password
									continue;

								if (!$u->canRequestNewPassword())
								{
									header('Location: InterfaceLostPassword.php?password_request_flood=true');
									exit();
								}

								$sentTo[] = $u;
						}
					}
				}

				if(!empty($_POST['user']['name']))
				{
					if($user->setByName($_POST['user']['name']))
					{
						if (!$user->canRequestNewPassword())
						{
							header('Location: InterfaceLostPassword.php?password_request_flood=true');
							exit();
						}

						$sentTo[] = $user;
					}
				}

				foreach($sentTo as $user ) {

					$user->last_login = '0000-00-00 00:00:00';
					$user->setPasswordInfoAndToken(); // mark the password as expired, create token
					MessageEvent::send(LOST_PASSWORD, array('user'=>$user), 'ELCMessageHandler');
					$user->login_token = ''; // we don't want to keep the login token after it was sent
					$user->update();
					$sent = true;

				    log_elc_event('User lost password', $user->id, null, null, null);
				}

				header('Location: InterfaceLostPassword.php?sent='.(int)$sent . '&confirm_new_pass=true');
				exit();
			}
		}
    }
}
/**
 * @package I-Ware
 * @subpackage Interface
 * @since 24-02-2006
 * @copyright 2006 BaseSoft B.V.
 * @version $Id$
 */
class InterfaceLostPasswordItem extends InterfaceItem
{

    /**
     * item[699] =
     */
    function getIntroText()
    {
		$this->twoCols = true;
		$this->text = $this->langIf->getText('Please provide us with either your username or email');
    }

    /**
     * item[700] =
     */
    function getBetweenText()
    {
		$this->twoCols = true;
		$this->text = $this->langIf->getText('And-or supply your email address');
    }

	function getUserName()
	{
		$this->properties['value'] = isset($_REQUEST['user']['name'])?$_REQUEST['user']['name']:'';
	}

    /**
     * item[702] =
     */
    function getButtons()
    {
		$this->_BarButtons = 0;
		$this->_addCustomButton($this->langIf->getText('Request'),'button', array('name'=>'request',
		       'onclick'=>'if (fb.validate($(\'interface_form\')))'.jsMessageBox(MSG_BOX_QUEST,$this->langIf->getText('Warning'),
		                               $this->langIf->getText('Send request for new password?'),
		                               "function(){\$\$('process').value='save';\$('interface_form').submit();}")));
    }

}

/**
 * CALLBACK[getIntroText]
 * item[699] = {
 *     [handle] = Item: IntroText
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getBetweenText]
 * item[700] = {
 *     [handle] = Item: BetweenText
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 * CALLBACK[getButtons]
 * item[702] = {
 *     [handle] = Item: ButtonBar
 *     [behaviour] =
 *     [help] = 0
 *     [close_row] = 1
 *     [label_pos] = 4
 *
 */
?>
