<?php
require_once(INTERFACE_OBJ."InterfaceFactory.php");
/**
 * InterfaceScriptSettings
 *
 * @package I-Ware
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @since 24-11-2005
 * @version $Id$
 */
class InterfaceScriptSettings extends InterfaceFactory
{
	/**
	 * @var int $application_id
	 */
	var $application_id = APPLICATION_ID;
	/**
	 * @param Database $db IWARE Database
	 * @param Database $db_app APPLICATION Database
	 * @param LanguageInterface $langIf
	 */
	function InterfaceScriptSettings(&$db, &$db_app, $langIf='')
	{
		$this->application_id = isset($_SESSION['current_site']['application_id'])?$_SESSION['current_site']['application_id']:APPLICATION_ID;

		parent::InterfaceFactory($db, $db_app, $langIf);
		$this->delimiter = '<br />';

		require_once(IWARE.'FormBehaviour.php');
		$fb = new FormBehaviour($this->langIf);
		$fb->register('new_script[id]','integer','Script ID');
		$fb->register('new_script[name]','alphanumeric','Script Name');
		$this->fb =& $fb;

		$this->script = "
			toggled = new Array();
			privs = new Array();
			function toggleInput(script_id)
			{
				if(toggled[script_id])
					return;
				fields = new Array('script_parent_id','script_name','script_path','script_universe','interface_id');
				names = new Array('parent_id','name','path','universe','interface_id');
				for(var i=0;i < fields.length;i++)
				{
					val = $(fields[i]+'_toggle_'+script_id).innerHTML;
					name = 'script['+script_id+']['+names[i]+']';
					extra = '';
					if(fields[i]=='script_parent_id')
						extra += '<input name=\"script['+script_id+'][old_parent_id]\" type=\"hidden\" value=\"'+val+'\" />';
					$(fields[i]+'_toggle_'+script_id).innerHTML = '<input name=\"'+name+'\" type=\"text\" value=\"'+val+'\" />'+extra;
				}

				val = parseInt($('script_menuitem_toggle_'+script_id).innerHTML);
				val = val?' checked=\"checked\"':'';
				name = 'script['+script_id+'][menu_item]';
				$('script_menuitem_toggle_'+script_id).innerHTML = '<input name=\"'+name+'\" type=\"checkbox\" value=\"1\"'+val+' />';

				access = new Array(".READ_ACCESS.",".WRITE_ACCESS.",".CHANGE_ACCESS.",".DELETE_ACCESS.");
				names = new Array('read','write','change','delete');
				val = '';
				rights = privs[script_id];

				for(var i=0;i<4;i++)
				{
					checked = (rights & access[i])?' checked=\"checked\"':'';
					val += '<input name=\"priv['+script_id+']['+names[i]+']\" type=\"checkbox\" value=\"'+access[i]+'\"'+checked+' /> ';
				}
				$('privs_toggle_'+script_id).innerHTML = val;

				toggled[script_id] = true;
			}
			function setPrivs(privileges)
			{
				privs = privileges;
			}
			function resetToggleCache()
			{
				toggled = new Array();
			}
			";


	}
	/**
	 * @param string $thrownException
	 */
	function processInput(&$thrownException)
	{
		$this->setSiteDB();
	//	print_r($_POST);
		//exit();
		if(isset($_POST['process']))
		{
			if($_POST['process'] == 'addscript')
			{
				//if($this->fb->validate())
				//	return $thrownException .= $this->langIf->getText('Please correct the form errors');
			//	print_r($_POST['new_script']);
				//exit();
				$this->_addScript($_POST['new_script']);
			}
			elseif($_POST['process'] == 'save')
			{
				foreach($_POST['script'] as $id => $values)
				{
					$this->_updateScript($id,$values);
					if(isset($_POST['priv'][$id]))
						$this->_setPrivileges($id,$_POST['group_company']['id'],$_POST['priv'][$id]);
					else
						$this->_removePrivileges($id,$_POST['group_company']['id']);
				}
			}

			if(function_exists('apc_store'))
			{
				$lookup = 'SFMenu'.$_SESSION['session_user_id'].'_'.
					(isset($_SESSION['selected_participant_id'])?$_SESSION['selected_participant_id']:0).'_'.
					(isset($_SESSION['selected_product_id'])?$_SESSION['selected_product_id']:0).APC_SUFFIX;
				apc_delete($lookup);
			}

			require_once(IWARE.'IWSync.php');
			IWSync::exportScriptPrivs();
			IWSync::exportScripts();
			$this->init(18);
		}
	}

	function setSiteDB()
	{
		if(isset($this->db_site))
			return;
		$this->db_site = $this->db_app;
	}

	/**#@+
	 * @access protected
	 */
	/**
	 * Saves the privileges for the GROUP_COMPANY
	 * @param int $script_id SCRIPT_ID
	 * @param int $gcid GROUP_COMPANY_ID
	 * @param array $privs ('read','write','change','delete')
	 * @return bool
	 */
	function _setPrivileges($script_id,$gcid,$privs)
	{
		$privs = array_sum($privs);
		$sql = "REPLACE INTO TBL_SCRIPT_GROUP
			(SCRIPT_ID,GROUP_COMPANY_ID,SCRIPT_GROUP_ACCESS)
			VALUES ( ?, ?, ? )";
		$q = $this->db_site->query($sql);
		$q->bind_values(array($script_id,$gcid,$privs));
		return $q->execute(true);
	}
	/**
	 * @param int $script_id
	 * @param int $gcid
	 * @return bool
	 */
	function _removePrivileges($script_id,$gcid)
	{
		$sql = "DELETE FROM TBL_SCRIPT_GROUP
			WHERE SCRIPT_ID = ?
				AND GROUP_COMPANY_ID = ?";
		$q = $this->db_site->query($sql);
		$q->bind_values(array($script_id,$gcid));
		return $q->execute(true);
	}
	/**
	 * Adds a new script to the I-WARE-SETTINGS database
	 * @param Array $script ('id','parent_id','name','path')
	 * @return bool
	 */
	function _addScript($script)
	{
		$pos = $this->_getMaxPosition($script['parent_id']);
		$sql = "INSERT INTO TBL_SCRIPT
			(SCRIPT_ID,SCRIPT_PARENT_ID,SCRIPT_PATH,SCRIPT_NAME,APPLICATION_ID,
				SCRIPT_POSITION,SCRIPT_MENUITEM,SCRIPT_UNIVERSE,INTERFACE_ID)
			VALUES(?,?,?,?,?,?,?,?,?)";
		$q = $this->db->query($sql);
		$q->bind_values(array($script['id'],$script['parent_id'],$script['path'],
			$script['name'],$this->application_id,$pos,(isset($script['menu_item'])?1:0),$script['universe'],$script['interface_id']));
		return $q->execute();
	}
	/**
	 * @param int $id
	 * @param array $script
	 * @param int $pos
	 * @return bool
	 */
	function _updateScript($id,$script,$pos='')
	{
		$values = array($script['parent_id'],$script['name'],$script['path'],(isset($script['menu_item'])?1:0),$script['universe'],$script['interface_id']);
		$sql = "UPDATE TBL_SCRIPT SET
			SCRIPT_PARENT_ID = ?,
			SCRIPT_NAME = ?,
			SCRIPT_PATH = ?,
			SCRIPT_MENUITEM = ?,
			SCRIPT_UNIVERSE = ?,
			INTERFACE_ID = ? ";
		if($pos)
		{
			$sql .= ", SCRIPT_POSITION = ? ";
			$values[] = $pos;
		}
		$sql .= "WHERE SCRIPT_ID = ?
			AND SCRIPT_PARENT_ID = ?";
		$values[] = $id;
		$values[] = $script['old_parent_id'];
		$q = $this->db->query($sql);
		$q->bind_values($values);
		return $q->execute(true);
	}
	/**
	 * @param int $parent_id
	 * @return int
	 */
	function _getMaxPosition($parent_id)
	{
		$sql = "SELECT MAX(SCRIPT_POSITION)+1 as POSITION FROM TBL_SCRIPT
			WHERE SCRIPT_PARENT_ID = ?
				AND APPLICATION_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values(array($parent_id,$this->application_id));
		$q->execute();
		$row = $q->fetchAll();
		if(!count($row))
			return 0;
		$r = array_pop($row);
		return $r->POSITION;
	}
	/**#@-*/
}
/**
 * InterfaceScriptSettingsItem
 *
 * @package I-Ware
 * @subpackage Interface
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright (2005) BaseSoft B.V.
 * @since 24-11-2005
 * @version $Id$
 */
class InterfaceScriptSettingsItem extends InterfaceItem
{
	public $priv;
	/**
	 *
	 */
	function setSiteDB()
	{
		if(isset($this->parent->parent->db_site))
			return;
		$this->parent->parent->db_site = $this->db_app;
	}
	/**#@+
	 * Item callback functions, set in the interfaceBuilder
	 */
	/**
	 * Gets the group companies for a select box
	 */
	function getGroupCompany()
	{
		$this->setSiteDB();
		$sql = "SELECT GROUP_COMPANY_ID, GROUP_NAME
			FROM TBL_GROUP_COMPANY
			INNER JOIN TBL_GROUP
				USING(GROUP_ID)
			ORDER BY COMPANY_ID, GROUP_NAME";
		$q = $this->parent->parent->db_site->query($sql);
//		$q->bind_values(array($_SESSION['session_user_company_id']));
		$q->execute();
		$this->options[] = array('text'=>'---', 'value'=>'-1');
		while($r = $q->fetch())
		{
			$this->options[] = array('text'=>$r->GROUP_NAME, 'value'=>$r->GROUP_COMPANY_ID);
			if(isset($_REQUEST['group_company']['id'])&& $_REQUEST['group_company']['id']==$r->GROUP_COMPANY_ID)
				$this->options[(count($this->options)-1)]['selected'] = 'selected';
		}
		$this->properties['id'] = 'groupcompanyselect';
		$this->properties['onchange'] = 'ifHandler.loadBlock(new Array(\'58\'),\'&interface_id='.$this->parent->parent->id.'&group_company[id]=\'+this.options[this.selectedIndex].value);';
	}
	/**
	 * Gets the overview of all the scripts and the privilegs for current GCID and APPLICATION_ID
	 */
	function getScriptOverview()
	{
		if(!isset($_REQUEST['group_company']['id'])||$_REQUEST['group_company']['id']<1)
			return;

		$this->setSiteDB();

		$priv = array();
		$sql = "SELECT SCRIPT_ID, SCRIPT_GROUP_ACCESS
			FROM TBL_SCRIPT_GROUP
			WHERE GROUP_COMPANY_ID = ?";
		$q = $this->parent->parent->db_site->query($sql);
		$q->bind_values(array($_REQUEST['group_company']['id']));
		$q->execute();
		while($r = $q->fetch())
			$priv[$r->SCRIPT_ID] = $r->SCRIPT_GROUP_ACCESS;

		$this->priv = $priv;
		$sql = "SELECT SCRIPT_ID,
					SCRIPT_PARENT_ID,
					IFNULL(SCRIPT_NAME, '') as SCRIPT_NAME,
					IFNULL(SCRIPT_PATH, '') as SCRIPT_PATH,
					IFNULL(SCRIPT_UNIVERSE, '') as SCRIPT_UNIVERSE,
					IFNULL(IF(INTERFACE_ID=0,'',INTERFACE_ID), '') as INTERFACE_ID,
					IFNULL(SCRIPT_MENUITEM, '') as SCRIPT_MENUITEM,
					'' as `PRIVILEGES`
			FROM TBL_SCRIPT
			WHERE APPLICATION_ID = ?
			ORDER BY SCRIPT_ID";
		$q = $this->db->query($sql);
		$this->application_id = isset($_SESSION['current_site']['application_id'])?$_SESSION['current_site']['application_id']:APPLICATION_ID;

		$q->bind_values(array($this->application_id));
		$q->execute();

		$tf = new TableFactory($q,$this->langIf);
		$tf->setValueProcessor(TF_ALL,array($this,'tfValueProcessor'));
		$tf->columnModifier(array('PRIVILEGES'=>' nowrap'));
		$this->text = $tf->getXHTMLTable('Professional Header',2);
		$this->postscript = 'setPrivs('.exportArrayToJS($this->priv).');resetToggleCache();';
	}

	public function tfValueProcessor($args)
	{
		$script_id = $args['row']['SCRIPT_ID'];
		switch($args['head'])
		{
			case 'SCRIPT_ID':
				return sprintf('<div id="toggle_%s"><a href="javascript:toggleInput(%s);">%s</a></div>',$args['value'],$args['value'],$args['value']);
			break;
			case 'SCRIPT_PARENT_ID':
			case 'SCRIPT_NAME':
			case 'SCRIPT_PATH':
			case 'SCRIPT_UNIVERSE':
			case 'INTERFACE_ID':
			case 'SCRIPT_MENUITEM':
				return sprintf('<div id="%s_toggle_%s">%s</div>',strtolower($args['head']),$script_id,$args['value']);
			break;
			case 'PRIVILEGES':
				$rv = sprintf('<div id="privs_toggle_%s">',$script_id);
				// showing the images really slows things down.
				/*foreach(array(READ_ACCESS,WRITE_ACCESS,CHANGE_ACCESS,DELETE_ACCESS) as $priv)
				{
					$img = isset($this->priv[$script_id])?((($this->priv[$script_id] & $priv)==$priv)?'true':'false'):'false';
					$rv .= sprintf('<img src="/%simages/i-w/%s.gif" /> ',SITE_PATH,$img);
				}*/
				$rv .= isset($this->priv[$script_id])?$this->priv[$script_id]:'';
				return $rv.'</div>';
			break;
		}
		return $args['value'];
	}

	/**
	 * Get the autocomplete function for the script_id's
	 */
	function getScriptParentId()
	{
		$this->properties = array('id'=>'script_parent_id','autocomplete'=>'off');
		/*$this->postscript = '
			var ac = new AutoComplete( document.getElementById(\'script_parent_id\'), \'ScriptIds\' );
			ac.maxWords = 15;
			';*/
	}
	/**
	 * Get buttons for the privileges interface
	 */
	function getButtons()
	{
		if(!isset($_REQUEST['group_company']['id'])||$_REQUEST['group_company']['id']<1)
			return $this->_BarButtons = 0;
		$this->_BarButtons = 0;
		$this->_addCustomButton($this->langIf->getText('Save'),'submit', array('name'=>'saveprivileges','style'=>'width: 100px;'));
	}
	/**
	 * Get the buttons for the script addition interface
	 */
	function getAddScriptButtons()
	{
		$this->_BarButtons = 0;
		$this->_addCustomButton($this->langIf->getText('Add Script'),'button', array('name'=>'addscript','style'=>'width: 100px;', 'onclick'=>"$$('process').value='addscript';$('interface_form').submit();"));
		$this->_addCustomButton($this->langIf->getText('Manage Order'),'button', array('style'=>'width: 100px;'));
	}
	/**#@-*/
}
?>