<?php
require_once(IWARE.'group.php');
/**
 * @package
 * @subpackage
 * @since 9-nov-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class AgeingAlarmList extends ELCDefaultList
{
	public $handle = 'AgeingAlarmList';
        private $showActuals = true;
        private $amountNotDueTotal = 0.00;
        private $amountDue0_30 = 0.00;
        private $amountDue31_60 = 0.00;
        private $amountDue61_90 = 0.00;
        private $amountDueGT_90 = 0.00;
        private $amountOutstandingTotal = 0.00;
        private $_cols = 0;

        function setEmptyText()
	{
		return $this->langIf->getText('No Ageing information found for selected filter');
	}

        // this is nonstandard as we need to call a stored proc
	function getDataSet()
	{
		$this->_filterQuery();
                $crso_id = $this->getCRSOSelect();
                if(!$crso_id)
                    $crso_id = 0;

				//get the last import date
	            $this->date = '';
	            if($crso_id)
	            {
		            $q = $this->db->query('
						SELECT MAX(IMPORT_DATE) AS LAST_IMPORT_DATE FROM TBL_AGEING WHERE CRSO_RELATION_ID = ?
					');
		            $q->bind_values($crso_id);
		            $q->execute();
		            $r = $q->fetch();
		    		if($r)
		    			$this->date = $r->LAST_IMPORT_DATE;
		    		unset($q);
		    	}
                $q = $this->db->query("CALL sp_getAgeingBreakdown(".$crso_id.")");
                $q->execute();
                return $q;
	}

	function _filterQuery()
	{
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_CRSO:
                            require_once(ELC_CLASSES.'Relation.php');
                            require_once(ELC_CLASSES.'Country.php');
                            $rel = new Relation($this->db,$_SESSION['crso_relation_id'],true);
                            $country = new Country($this->db,$rel->getCountry(),true);
                            if(!$country->has(HasCrsoDeclarationDetails,$_SESSION['selected_product_id']) ||
                                !$country->has(HasCrsoRefundDetails,$_SESSION['selected_product_id']))
                            {
                                $this->showActuals = false;
                            } else {
								// Check for CRSO universe. Set the Relation User and do a clearance level check.
					    		require_once(ELC_CLASSES.'CRSO.php');
								require_once(ELC_CLASSES.'RelationUser.php');
								$ru = new RelationUser($this->db);
								$ru->set($_SESSION['session_user_id'], $_SESSION['crso_relation_id']);
								if(!$ru->hasClearance(CRSOClearanceDeclarationDetails) || !$ru->hasClearance(CRSOClearanceRefundDetails)) {
									$this->showActuals = false;
								}
							}
			    break;
			default:
                                break;

		}

	}

	function _setWhere()
	{
		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'A.CRSO_RELATION_ID',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true,
                                'useinquery' => false));
	}

	function getProcessColumns()
	{
            return array('tfValueProcessorQuantity' => array('NOT_DUE', 'DUE_0_30', 'DUE_31_60', 'DUE_61_90', 'DUE_GT_90', 'BALANCE'));

	}

        function getColumnModifier()
	{
		return array(
			'NOT_DUE'=>'style="text-align:right;"',
			'DUE_0_30'=>'style="text-align:right;"',
                        'DUE_31_60' => 'style="text-align:right;"',
                        'DUE_61_90' => 'style="text-align:right;"',
                        'DUE_GT_90' => 'style="text-align:right;"',
                        'BALANCE' => 'style="text-align:right;"'
			);
	}

        function tfValueProcessorQuantity($args)
        {
            $this->_cols = $args['visibleColumns'];
            if($args['head'] == 'BALANCE')
            {
                $this->amountOutstandingTotal += $args['value'];
            }
            else if($args['head'] == 'NOT_DUE')
            {
                $this->amountNotDueTotal += $args['value'];
            }
            else if($args['head'] == 'DUE_0_30')
            {
                $this->amountDue0_30 += $args['value'];
            }
            else if($args['head'] == 'DUE_31_60')
            {
                $this->amountDue31_60 += $args['value'];
            }
            else if($args['head'] == 'DUE_61_90')
            {
                $this->amountDue61_90 += $args['value'];
            }
            else if($args['head'] == 'DUE_GT_90')
            {
                $this->amountDueGT_90 += $args['value'];
            }
            if(!$this->showActuals)
            {
                if($args['value'] == '0.00')
                    $args['value'] = '';
                return parent::tfBooleanIcon($args);
            }
            else
            {
                $args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
                return parent::tfValueProcessorQuantity($args);
            }
        }

        function tfSortingHat()
		{
			return array(
				'PARTICIPANT_NAME'=>"'SortVal'",
				'NOT_DUE'=>"'SortVal'",
				'DUE_0_30'=>"'SortVal'",
				'DUE_31_60'=>"'SortVal'",
				'DUE_61_90'=>"'SortVal'",
				'DUE_GT_90'=>"'SortVal'",
				'BALANCE'=>"'SortVal'",
				);
		}

        function tfFooter()
        {
            if(!$this->_cols)
                return false;

            $rv = explode(',',str_repeat(',',(sizeof($this->_cols)-1)));
            $pos = array_flip($this->_cols);
            $rv[$pos['PARTICIPANT_NAME']] = '<span style="font-weight: bold; float:right:">'.$this->getText('CRSO Total').':</span><span style="clear:both;"></span>
            <br /><br /><span style="font-weight: bold; float:left;">'.$this->getText('Date generated').': '.$this->date.'</span><span style="clear:both;"></span>';
            $rv[$pos['NOT_DUE']] = '<span style="font-weight: bold; float:right;">'.locale_nf($this->amountNotDueTotal, 2).'</span><span style="clear:both;"></span>';
            $rv[$pos['DUE_0_30']] = '<span style="font-weight: bold; float:right;">'.locale_nf($this->amountDue0_30, 2).'</span><span style="clear:both;"></span>';
            $rv[$pos['DUE_31_60']] = '<span style="font-weight: bold; float:right;">'.locale_nf($this->amountDue31_60, 2).'</span><span style="clear:both;"></span>';
            $rv[$pos['DUE_61_90']] = '<span style="font-weight: bold; float:right;">'.locale_nf($this->amountDue61_90, 2).'</span><span style="clear:both;"></span>';
            $rv[$pos['DUE_GT_90']] = '<span style="font-weight: bold; float:right;">'.locale_nf($this->amountDueGT_90, 2).'</span><span style="clear:both;"></span>';
            $rv[$pos['BALANCE']] = '<span style="font-weight: bold; float:right;">'.locale_nf($this->amountOutstandingTotal, 2).'</span><span style="clear:both;"></span>';
            return $rv;
        }
}
?>
