<?php
require_once(IWARE.'group.php');
/**
 * @package
 * @subpackage
 * @since 9-nov-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class AgeingCustomerBalanceList extends ELCDefaultList
{
	public $handle = 'AgeingCustomerBalanceList';
        private $showActuals = true;
        private $amountTotal = 0.00;
        private $amountOutstandingTotal = 0.00;
        private $currentParticipant = 0;
        private $_cols = 0;
        private $moreThanOne = false;

        function setEmptyText()
	{
		return $this->langIf->getText('No Ageing information found for selected filter');
	}

        function getDataSet()
        {
            $rv = array();
            $rowTemplate = array('CRSO_NAME' => '', 'PARTICIPANT_RELATION_ID' => '' , 'PARTICIPANT_NAME' => '',
                                'VAT_NO' => '', 'POSTING_DATE' => '', 'DOCUMENT_NO' => '', 'AMOUNT' => '',
                                'AMOUNT_OUTSTANDING' => '', 'DOCUMENT_DATE' => '', 'DUE_DATE' => '');
            $q = parent::getDataSet();
            if($q === false)
                return false;
            $rs = $q->fetchAll();
            $part_id = 0;
            $amountTotal = 0.00;
            $amountOutstandingTotal = 0.00;
            $this->moreThanOne = count($rs) > 1;
            foreach($rs as $r)
            {
                if($part_id != $r->PARTICIPANT_RELATION_ID && $this->showActuals)
                {
                    if($part_id != 0)
                    {
                        $row = $rowTemplate;
                        $row['AMOUNT'] = '<span style="font-weight: bold;">'.locale_nf($amountTotal, 2).'</span>';
                        $row['AMOUNT_OUTSTANDING'] = '<span style="font-weight: bold;">'.locale_nf($amountOutstandingTotal, 2).'</span><br /><br />';
                        $row['DOCUMENT_NO'] = '<span style="font-weight: bold;">'.$this->getText('Customer Total').':</span>';
                        $amountTotal = 0.00;
                        $amountOutstandingTotal = 0.00;
                        $rv[] = $row;
                    }
                    $part_id = $r->PARTICIPANT_RELATION_ID;
                }
                $row = $rowTemplate;
                foreach(array_keys($rowTemplate) as $field)
                {
                    $row[$field] = $r->{$field};
                }
                $amountTotal += $r->AMOUNT;
                $amountOutstandingTotal += $r->AMOUNT_OUTSTANDING;
                $rv[] = $row;
            }
            if($part_id != 0 && $this->showActuals)
            {
                $row = $rowTemplate;
                $row['AMOUNT'] = '<span style="font-weight: bold; float:right;">'.locale_nf($amountTotal, 2).'</span><span style="clear:both;"></span>';
                $row['AMOUNT_OUTSTANDING'] = '<span style="font-weight: bold; float:right;">'.locale_nf($amountOutstandingTotal, 2).'</span><span style="clear:both;"></span>';
                $row['DOCUMENT_NO'] = '<span style="font-weight: bold; float:right;">'.$this->getText('Customer Total').':</span><span style="clear:both;"></span>';
                $rv[] = $row;
            }
            return $rv;
        }


	function setQuery()
	{
		$this->_query = "SELECT CR.`NAME` as CRSO_NAME, A.PARTICIPANT_RELATION_ID, PR.`NAME` AS PARTICIPANT_NAME, PR.VAT_NO, A.POSTING_DATE,
                        A.DOCUMENT_NO, A.AMOUNT, A.AMOUNT_OUTSTANDING, A.DOCUMENT_DATE, A.DUE_DATE
                        FROM TBL_AGEING A
                        INNER JOIN TBL_RELATION CR ON A.CRSO_RELATION_ID = CR.RELATION_ID
                        INNER JOIN TBL_RELATION PR ON A.PARTICIPANT_RELATION_ID = PR.RELATION_ID ";
		$this->_orderBy = "ORDER BY A.CRSO_RELATION_ID, A.PARTICIPANT_RELATION_ID, A.POSTING_DATE";
                $this->_addHiddenCol(array('CRSO_NAME', 'PARTICIPANT_RELATION_ID'));
	}

	function _filterQuery()
	{

		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_CRSO:
                            require_once(ELC_CLASSES.'Relation.php');
                            require_once(ELC_CLASSES.'Country.php');
                            $rel = new Relation($this->db,$_SESSION['crso_relation_id'],true);
                            $country = new Country($this->db,$rel->getCountry(),true);
                            if(!$country->has(HasCrsoDeclarationDetails,$_SESSION['selected_product_id']) ||
                                !$country->has(HasCrsoRefundDetails,$_SESSION['selected_product_id']))
                            {
                                $this->showActuals = false;
                            } else {
								// Check for CRSO universe. Set the Relation User and do a clearance level check.
					    		require_once(ELC_CLASSES.'CRSO.php');
								require_once(ELC_CLASSES.'RelationUser.php');
								$ru = new RelationUser($this->db);
								$ru->set($_SESSION['session_user_id'], $_SESSION['crso_relation_id']);
								if(!$ru->hasClearance(CRSOClearanceDeclarationDetails) || !$ru->hasClearance(CRSOClearanceRefundDetails)) {
									$this->showActuals = false;
								}
							}
			    break;
			case GROUP_PARTICIPANT:
				// If a CRSO was selected, check the Country settings.
				if(!isset($_REQUEST['CRSO_RELATION_ID']) || empty($_REQUEST['CRSO_RELATION_ID']))
					return;
                require_once(ELC_CLASSES.'Relation.php');
                require_once(ELC_CLASSES.'Country.php');
                $rel = new Relation($this->db,$_REQUEST['CRSO_RELATION_ID'],true);
                $country = new Country($this->db,$rel->getCountry(),true);
                if($country->has(HasParticipantAgeingAccess, $_SESSION['selected_product_id']))
                	return $this->_query;
				throw new IWAJAXPermissionException('Permission Denied for Ageing information at this CRSO');
			break;
			default:
                                break;

		}

		return $this->_query;
	}

	function _setWhere()
	{
		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'A.CRSO_RELATION_ID',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true));

		$this->_addWhere('PARTICIPANT_RELATION_ID',
			array(
				'field'=>'A.PARTICIPANT_RELATION_ID',
				'callback' => array($this,'getParticipantSelect')));
	}

	function getProcessColumns()
	{
            return array('tfValueProcessorQuantity' => array('AMOUNT', 'AMOUNT_OUTSTANDING'),
                            'tfVat' => array('VAT_NO'),
                            'tfCollapse' => array('PARTICIPANT_NAME'));

	}

        function getColumnModifier()
	{
		return array(
			'AMOUNT'=>'style="text-align:right;"',
			'AMOUNT_OUTSTANDING'=>'style="text-align:right;"'
			);
	}

        function tfValueProcessorQuantity($args)
        {
            if($args['row']['PARTICIPANT_RELATION_ID'] != '')
            {
                $this->_cols = $args['visibleColumns'];
                if($args['head'] == 'AMOUNT')
                {
                    $this->amountTotal += $args['value'];
                }
                else
                {
                    $this->amountOutstandingTotal += $args['value'];
                }
                if(!$this->showActuals)
                {
                    if($args['value'] == '0.00')
                        $args['value'] = '';
                    return parent::tfBooleanIcon($args);
                }
                else
                    return parent::tfValueProcessorQuantity($args);
            }
            return $args['value'];
        }


        function tfCollapse($args)
        {
            if(isset($args['previousrow']) && $args['value'] == $args['previousrow'][$args['orighead']])
            {
                $args['tdclass'] && $args['tdclass'] .= '-collapse';
                return '';
            }
            else
                return $args['value'];
        }

        function tfVat($args)
        {
            if(isset($args['previousrow']) && $args['value'] == $args['previousrow'][$args['orighead']])
            {
                $args['tdclass'] && $args['tdclass'] .= '-collapse';
                return '';
            }
            else
                return parent::tfVat($args);
        }

        function tfFooter()
        {
            if(!$this->_cols || (!$this->amountTotal && !$this->amountOutstandingTotal) ||
                (isset($_REQUEST['PARTICIPANT_RELATION_ID']) && !empty($_REQUEST['PARTICIPANT_RELATION_ID'])))
                return false;
            if(!$this->showActuals && !$this->moreThanOne)
                return false;
            $rv = explode(',',str_repeat(',',(sizeof($this->_cols)-1)));
            $pos = array_flip($this->_cols);
            $rv[$pos['DOCUMENT_NO']] = '<span style="font-weight: bold; float:right:">'.$this->getText('CRSO Total').':</span><span style="clear:both;"></span>';
            $rv[$pos['AMOUNT']] = '<span style="font-weight: bold; float:right;">'.locale_nf($this->amountTotal, 2).'</span><span style="clear:both;"></span>';
            $rv[$pos['AMOUNT_OUTSTANDING']] = '<span style="font-weight: bold; float:right;">'.locale_nf($this->amountOutstandingTotal, 2).'</span><span style="clear:both;"></span>';
            return $rv;
        }
}
?>
