<?php
require_once(ELC_CLASSES.'AnnualConfirmation.php');
require_once(IWARE.'group.php');
require_once(ELC_CLASSES.'Country.php');
/**
 * @package
 * @subpackage
 * @since 9-nov-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class AnnualConfirmationList extends ELCDefaultList
{
	public $handle = 'AnnualConfirmationList';
	protected $ac;
	protected $country;
	protected $checkboxes = false;
	protected $_cols = false;

	function setQuery()
	{
		$this->ac = new AnnualConfirmation($this->db);

		$crso_id = $this->getCRSOSelect();
		if(!$crso_id)
			return ;
		require_once(ELC_CLASSES.'Relation.php');
		require_once(ELC_CLASSES.'Product.php');
		$p = new Product($this->db);
		$p->set($_SESSION['selected_product_id']);
		$this->product_name = $p->name;

		$r = new Relation($this->db);
		$this->country = new Country($this->db,$r->getCountry($crso_id),true);

		$this->_query = "SELECT AC.PARTICIPANT_RELATION_ID as CHECKBOX, AC.ANNUAL_CONFIRMATION_ID, R.`NAME` as PARTICIPANT_NAME,
				CPP.ALLOW_ANNUAL_CONFIRMATION,
				AC.`STATUS`,
				AC.ANNUAL_CONFIRMATION_ID as FORCE_CLOSE,
				IFNULL(MAX(ACD.DOCUMENT_ID),0) as AC_DOCUMENT,
				AC.CRSO_RELATION_ID, AC.LAST_UPDATED, AC.`YEAR`, '' as ALL_DECL_FILED,
				AC.PARTICIPANT_RELATION_ID, AC.CHANGE_REQUEST_STATUS as REQUESTED_CHANGE,
				AC.REVIEW_STATUS
				#SUM(DL.QUANTITY) as HASH
			FROM TBL_ANNUAL_CONFIRMATION as AC
			INNER JOIN TBL_RELATION as R
				ON AC.PARTICIPANT_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT CPP
				ON AC.CRSO_RELATION_ID = CPP.CRSO_RELATION_ID
				AND AC.PARTICIPANT_RELATION_ID =CPP.PARTICIPANT_RELATION_ID
				AND AC.PRODUCT_ID = CPP.PRODUCT_ID
			LEFT JOIN TBL_DECLARATION as D
				ON AC.PARTICIPANT_RELATION_ID = D.PARTICIPANT_RELATION_ID
					AND AC.CRSO_RELATION_ID = D.CRSO_RELATION_ID
					AND AC.PRODUCT_ID = D.PRODUCT_ID
					AND AC.`YEAR` = D.`YEAR`
			LEFT JOIN TBL_DECLARATION_LINE as DL
				ON D.DECLARATION_ID = DL.DECLARATION_ID
			LEFT JOIN TBL_ANNUAL_CONFIRMATION_DOCUMENT as ACD
				ON AC.ANNUAL_CONFIRMATION_ID = ACD.ANNUAL_CONFIRMATION_ID";
		$this->_groupBy = "GROUP BY AC.ANNUAL_CONFIRMATION_ID";
		$this->_addHiddenCol(array('PARTICIPANT_RELATION_ID','ANNUAL_CONFIRMATION_ID'));
		$this->_orderBy = "ORDER BY AC.PARTICIPANT_RELATION_ID";
	}

	function _filterQuery()
	{
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
			     break;
			case GROUP_CRSO:
			     //hide document download column in case no clearance level or country setting setting disabled
			     $ru = new RelationUser($this->db);
                 $ru->set($_SESSION['session_user_id'], $_SESSION['crso_relation_id']);
			     if($this->country->has(HasAnnualConfirmationDisabled, $_SESSION['selected_product_id']) ||
			         !$ru->hasClearance(CRSOClearanceAnnualConfirmation)) {
			         $this->_addHiddenCol(array('AC_DOCUMENT'));
			     }
			     break;
			default:
				$this->_addHiddenCol(array('HASH'));
		}

		return $this->_query;
	}

	function _setWhere()
	{
		$this->_addWhere('YEAR',
			array('field'=>'AC.`YEAR`',
				'callback' => array($this,'getYearSelect'),
				'required' => true));

		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'AC.CRSO_RELATION_ID',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true));

		$this->_addWhere('PRODUCT_ID',
			array(
				'field'=>'AC.PRODUCT_ID',
				'callback' => array($this,'getProductSelect')));
	}

	function getCRSOSelect($rv='setValue')
	{
		$setValue = !empty($_REQUEST['CRSO_RELATION_ID'])?(int)$_REQUEST['CRSO_RELATION_ID']:false;
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_CRSO:
				$filter = '';
				$selectedValue = '';
				if(isset($_REQUEST['CRSO_RELATION_ID']) && ($_REQUEST['CRSO_RELATION_ID']!=$_SESSION['crso_relation_id']))
					throw new IWAJAXPermissionException('Incorrect CRSO Id supplied');
				$_REQUEST['CRSO_RELATION_ID'] = $setValue = $_SESSION['crso_relation_id'];
				break;
			case GROUP_ACCOUNTANT:
				// get all accountant's active CRSOs for the selected product
				require_once(ELC_CLASSES.'AccountantCRSO.php');
				$crso = new AccountantCRSO($this->db);
				$q = $crso->getAccountantCRSOs($_SESSION['accountant_relation_id'], $_SESSION['selected_product_id']);
				$rs = $q->fetchAll();
			break;
			default:
				require_once(ELC_CLASSES.'CRSO.php');
				$crso = new CRSO($this->db);
				$q = $crso->selectCRSOs(1,null,$_SESSION['selected_product_id']);
				$rs = $q->fetchAll();
				break;
		}

		// only get CRSOs which do not have AC disabled as country setting
		$crsos = array();
		if(isset($rs))
			foreach($rs as $r)
			{
				$rel = new Relation($this->db);
				$country = new Country($this->db,$rel->getCountry($r->CRSO_RELATION_ID),true);
				if(!$country->has(HasAnnualConfirmationDisabled, $_SESSION['selected_product_id']))
					$crsos[] = array('id' => $r->CRSO_RELATION_ID, 'name' => $r->NAME);
			}

		if(count($crsos) == 1)
		{
			$setValue = $crsos[0]['id'];
			$_REQUEST['CRSO_RELATION_ID'] = $setValue;
		}

		if(isset($$rv))
			return $$rv;
		$selectedValue = '';

		$select = new Form_Select('CRSO_RELATION_ID');
		$select->addOption(' - '.$this->getText('CRSO Select').' - ', '', false);
		foreach($crsos as $r)
		{
			if($setValue==$r['id'])
				$selectedValue = $r['name'];
			$select->addOption(((mb_strlen($r['name'])>35)?mb_substr($r['name'],0,35).'...':$r['name']), $r['id'], false);
		}
		$select->setSelectedValue($setValue);

		$filter = $select->getHtml();

		return $$rv;
	}

	function getYearSelect($rv='setValue')
	{
		$date =& currentDate();
		return parent::getYearSelect($rv,($date->getYear()-1));
	}

	function getProcessColumns()
	{
		return array(
			'tfAllFiled'=>array('ALL_DECL_FILED'),
			'tfDocumentLink'=>array('AC_DOCUMENT'),
			'tfTranslateStatus'=>array('STATUS'),
			'tfChangeRequest'=>array('REQUESTED_CHANGE'),
			'tfTranslateReviewStatus' =>array('REVIEW_STATUS'),
			'tfCheckbox' =>array('CHECKBOX'),
			'tfBooleanLink'=>array('ALLOW_ANNUAL_CONFIRMATION'),
			'tfForceClose'=>array('FORCE_CLOSE'),
		);
	}

	function tfBooleanLink($args)
	{
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		$img = $this->tfBooleanIcon($args);
		$text = $this->langIf->getText('block');
		$message = $this->langIf->getText('The following participant will be blocked from filling its AnnualConfirmation for product').
					" ".$this->product_name.":<br />". $args['row']['PARTICIPANT_NAME'];
		if($args['value'] == 0){
			$text = $this->langIf->getText('allow');
			$message = $this->langIf->getText('The following participant will be allowed to fill its AnnualConfirmation for product').
					" ".$this->product_name.":<br />". $args['row']['PARTICIPANT_NAME'];
		}
		return $img."&nbsp;<input type=button onclick='change_ac(\"".!$args['value']."\", \"".$args['row']['PARTICIPANT_RELATION_ID']."\", \""
					.$message."\");' value='".$text."' />";
	}

	function tfForceClose($args)
	{
		if($args['row']['STATUS'] == 2) return ' - ';
		return "<input type=button onclick='force_close_ac(\"".$args['value']."\");' value='".$this->langIf->getText('Force close')."' />";
	}

	function tfCheckbox($args)
	{
		if($args['row']['STATUS'] != 2)
		{
			$this->checkboxes = true;
			return '<input type="checkbox" name="reminder[]" value='.$args['value'].' />';
		}
		return " - ";
	}

	function tfSortingHat()
	{
		return array(
			'PARTICIPANT_NAME'=>"'String'",
			'STATUS'=>"'SortVal'",
			'AC_DOCUMENT'=>"'SortVal'",
			'LAST_UPDATED'=>"'String'",
			'ALL_DECL_FILED'=>"'String'",
			'REVIEW_STATUS'=>"'String'"
			);
	}

	function tfACHash($args)
	{
		$this->_cols = $args['visibleColumns'];
		$this->totalHash .= $hash = md5($args['value'].
    		$args['row']['YEAR'].
    		str_pad($args['row']['PARTICIPANT_RELATION_ID'],8,'0',STR_PAD_LEFT).
    		str_pad($this->getCRSOSelect(),8,'0',STR_PAD_LEFT));
    	return $hash;
	}

	function tfChangeRequest($args)
	{
		// this column only shows if there is an active request (approved ones do not cound as active)
		if($args['value'] == ANNUAL_CONFIRMATION_CHANGE_APPROVED)
			$args['value'] = 0;

		if(($_SESSION['session_group_group_id']!=GROUP_CRSO)||!$args['value'])
			return $this->tfBooleanIcon($args);

		$rv = '<input type="button" id="approve_change_%s" value="%s" onclick="approveChangeRequest(\'%s\')"/>
			<input type="button" id="reject_change_%s" value="%s" onclick="rejectChangeRequest(\'%s\');" /> ';
		return sprintf($rv,
			$args['row']['ANNUAL_CONFIRMATION_ID'],
			$this->getText('Approve'),
			$args['row']['ANNUAL_CONFIRMATION_ID'],
			$args['row']['ANNUAL_CONFIRMATION_ID'],
			$this->getText('Reject'),
			$args['row']['ANNUAL_CONFIRMATION_ID']);
	}
	/**
	 * We need the ajax calls for approval/denying of the change request
	 */
	function getPostScript()
	{
		$rv = "
			function approveChangeRequest(acId)
			{
				ChangeChangeStatus('approveAnnualConfirmationChange',acId);
			}

			function rejectChangeRequest(acId)
			{
				MessageBox.prototype.Information('%s', '%s', function(){TriggerRejectChange(acId);});
			}

			function TriggerRejectChange(acId)
			{
					var params = new Array();
					params['reject_msg'] = $('change_reject_msg').value;
					ChangeChangeStatus('rejectAnnualConfirmationChange',acId, params);
			}

			function ChangeChangeStatus(callmethod,acId,params)
			{
				DisableButtons(acId);
				params = params?params:new Array();
				params['ac_id'] = acId;

				// see includes/elc/rpc/RequestChangeAnnualConfirmation.php
				jsbroker.call(callmethod, params, function(p)
				{
					MessageBox.prototype.Information('', '%s');
				});
			}

			function DisableButtons(acId)
			{
				\$('approve_change_'+acId).disabled = 'disabled';
				\$('reject_change_'+acId).disabled = 'disabled';
			}
			function selAll()
			{
				var field = \$\$s('reminder[]');
				var el = \$\$('all');
				if(el.checked)
				{
					for (i = 0; i < field.length; i++)
						field[i].checked = true ;
				}
				else
				{
					for (i = 0; i < field.length; i++)
						field[i].checked = false ;
				}
			}
			function set_process()
			{
				var is_sel = 0;
				for (i = 0; i < \$\$s('reminder[]').length; i++)
						if(\$\$s('reminder[]')[i].checked == true)
							is_sel = 1;
				if(is_sel == 1)
					MessageBox.prototype.Question('%s', '%s',
												function(){
													document.getElementsByName('process')[0].value='send';
													document.getElementById('interface_form').submit();
												},
												function(){});
				else
					MessageBox.prototype.Information('Error', '%s');
			}
			function change_ac(ac, part, message)
			{
				MessageBox.prototype.Question('%s', message,
											function(){
												var form = document.getElementById('interface_form');
												// hidden input for participant
												var phid = document.createElement('input');
												phid.type = 'hidden';
												phid.name = 'participant_id';
												phid.value = part;
												form.appendChild(phid);

												// hidden input for ac status
												var achid = document.createElement('input');
												achid.type = 'hidden';
												achid.name = 'ac';
												achid.value = ac;
												form.appendChild(achid);

												document.getElementsByName('process')[0].value='change_ac';
												form.submit();
											},
											function(){});
			}
			function reopen_acs() {
				MessageBox.prototype.Question('%s', '%s',
					function(){
						document.getElementsByName('process')[0].value='reopen';
						document.getElementById('interface_form').submit();
					},
					function(){}
				);
			}
			function force_close_ac(ac) {
				MessageBox.prototype.Question('%s', '%s',
					function(){
						var form = document.getElementById('interface_form');

						// hidden input for ac
						var achid = document.createElement('input');
						achid.type = 'hidden';
						achid.name = 'ac';
						achid.value = ac;
						form.appendChild(achid);

						document.getElementsByName('process')[0].value='force_close';
						form.submit();
					},
					function(){}
				);
			}
			";
		return sprintf($rv,
				$this->getText('Reject Annual Confirmation Change Request'),
				$this->getText('Please provide a reason of rejection').'<br /><textarea id="change_reject_msg" cols="55" rows="6"></textarea>',
				$this->getText('Message has been sent to Participant'),
				$this->langIf->getText('Warning'),
				$this->langIf->getText('Send Reminder Email to all selected participants?'),
				$this->langIf->getText('Nothing selected'),
				$this->langIf->getText('Participant Annual Confirmation'),
				$this->langIf->getText('Warning'),
				$this->langIf->getText('Are you sure you want to reopen all Annual Confirmations without documents?'),
				$this->langIf->getText('Warning'),
				$this->langIf->getText('Are you sure you want to force close this Annual Confirmation and all its Declarations?')
			);
	}

	function tfDocumentLink($args)
	{
        $args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		// Only show link if CRSO user has access to declaration details.
		if($_SESSION['session_group_group_id']==GROUP_CRSO) {
			$ru = new RelationUser($this->db);
			$ru->set($_SESSION['session_user_id'], $_SESSION['crso_relation_id']);
			if(!$ru->hasClearance(CRSOClearanceDeclarationDetails))
				return $this->tfBooleanIcon($args);
		}

		return parent::tfDocumentLink($args);
	}

	function tfTranslateStatus($args)
	{
		$args['extra'] = sprintf(' sort_val="%s" ', $args['value']);
		list($trans,$class) = AnnualConfirmation::translateStatus($this->langIf,$args['value']);
		return sprintf('<span class="%s">%s</span>',$class,$trans);
	}

	function tfTranslateReviewStatus($args)
	{
		$args['extra'] = sprintf(' sort_val="%s" ', $args['value']);
		list($trans,$class) = AnnualConfirmation::translateReviewStatus($this->langIf,$args['value']);
		return sprintf('<span class="%s">%s</span>',$class,$trans);
	}

	function tfAllFiled($args)
	{
		list($args['value'],) = $this->ac->hasNoUnfiledMonths($args['row']['PARTICIPANT_RELATION_ID'],$args['row']['CRSO_RELATION_ID'],$args['row']['YEAR'],null,$_SESSION['selected_product_id']);
		return $this->tfBooleanIcon($args);
	}

	function tfFooter() // getClass
	{
		if(!$this->_query||in_array('CHECKBOX',$this->_hiddenCols)||!$this->checkboxes)
			return false;
		$in = "<input type=checkbox name='all' value='' onclick=\"selAll();\"> ".$this->langIf->getText('select all');
		if($this->_showType==LIST_HTML_OUTPUT)
    		$rv['CHECKBOX'] = '<span style="float:left;">'.$in.'</span><span style="clear:both;"></span>';
    	else
    		$rv['CHECKBOX'] = $in;
		return $rv;
	}
	function getButtons()
	{
		$settings = parent::getButtons();
		$settings['Trigger reminder'] = array('onclick'=>"set_process();");
		$settings['Reopen all without document'] = array('onclick'=>"reopen_acs();");
	    return $settings;
	}
	function getDataSet()
	{
		if(isset($_REQUEST['process']))
		{
			switch($_SESSION['session_group_group_id'])
			{
				case GROUP_SYSADMIN:
					$crso_id = $_REQUEST['CRSO_RELATION_ID'];
					break;
				case GROUP_CRSO:
					$crso_id = $_SESSION['crso_relation_id'];
					break;
			}
			// send reminder
			if($_REQUEST['process'] == 'send' && isset($_POST['reminder'])){
				require_once(ELC_CLASSES.'CRSOParticipant.php');
				require_once(ELC_CLASSES.'ELCMessageHandler.php');
				require_once(ELC_CLASSES.'CRSO.php');
				require_once(ELC_CLASSES.'Participant.php');
				require_once(ELC_CLASSES.'User.php');
				require_once(ELC_CLASSES.'RelationUser.php');

				$crso = new CRSO($this->db);
				$crso->set($crso_id);
				foreach($_POST['reminder'] as $participant_id )
				{
					$cp = new CRSOParticipant($this->db);
					if(!$cp->set($crso_id,(int)$participant_id))
						throw new IWUserException("Permission denied");

					$ru = new RelationUser($this->db);
					$q = $ru->selectByRelation($participant_id,1);
					$res = $q->fetch();
					$user_id = $res->USER_ID;
					$part = new Participant($this->db);
					$part->set($participant_id);
					MessageEvent::send(AC_REMINDER,array(
											'user'=>new ELCUser($this->db,$user_id),
											'crso'=>$crso,
											'participant'=>$part),
									   'ELCMessageHandler');
				}
			}
			// block/allow participant annual confirmation
			if($_REQUEST['process'] == 'change_ac' && isset($_POST['participant_id']) && isset($_POST['ac']))
			{
				require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
				$cpp = new CRSOParticipantProduct($this->db);
				$cpp->set($crso_id, $_POST['participant_id'], $_SESSION['selected_product_id']);
				$cpp->allow_annual_confirmation = $_POST['ac'];
				$cpp->update();
			}
			// In case of reopen, we allow for all Annual Confirmation for the selected year?
			if($_POST['process'] == 'reopen') {


				if(!isset($_SESSION['crso_relation_id'])) {
					throw new Exception('Unauthorized access to reopen Annual Confirmations');
				}


				if(($_POST['YEAR'] != '')) {
					require_once(ELC_CLASSES.'AnnualConfirmation.php');
					$ac = new AnnualConfirmation($this->db);
					$ac->reopenAllAnnualConfirmationsWithoutDocument($_SESSION['crso_relation_id'], (int) $_POST['YEAR']);
				}

			}

			if($_POST['process'] == 'force_close') {

				if(!isset($_SESSION['crso_relation_id'])) {
					throw new Exception('Unauthorized access to reopen Annual Confirmations');
				}
				if(($_POST['ac'] != '')) {
					require_once(ELC_CLASSES.'AnnualConfirmation.php');
					$ac = new AnnualConfirmation($this->db);
					$ac->forceCloseAC($_SESSION['crso_relation_id'], (int) $_POST['ac']);
				}
			}

		}
		return parent::getDataSet();
	}
}
?>
