<?php
/**
 * @package
 * @subpackage
 * @since 26-sep-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class WEEECustomQueryDeclarationExport extends IWCustomQueryObject
{
	protected $totalHash = '';
	protected $_cols = false;
	public function isPrivileged()
	{
		require_once(IWARE.'group.php');
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
			case GROUP_ACCOUNTANT:
				return true;
		}
		return false;
	}

	public function getQuery()
	{
		$sql = "SELECT D.DECLARATION_ID,
				IF(ISNULL(dd.DECLARATION_DEBIT_ID),
						f_elcBuildInvoiceNoForInterfaces(D.CRSO_RELATION_ID,dc.`YEAR`,dc.DECLARATION_CREDIT_ID,true,false),
						f_elcBuildInvoiceNoForInterfaces(D.CRSO_RELATION_ID,dd.`YEAR`,dd.DECLARATION_DEBIT_ID,false,false)
				) as INVOICE_NR,
				IF(ISNULL(dd.DECLARATION_DEBIT_ID),
					IF(ISNULL(dc.DECLARATION_CREDIT_ID),
						'-',
						CONCAT('C',D.CRSO_RELATION_ID)),
					CONCAT('I',D.CRSO_RELATION_ID)) as JOURNAL_TEMPLATE,
				D.DOCUMENT_ID as Invoice,
				D.PARTICIPANT_RELATION_ID,
				R.`NAME` as PARTICIPANT_NAME,
				R.VAT_NO,
				D.`MONTH`,
				D.`YEAR`,
				SUM(DL.QUANTITY) as Quantity,
				BIT_OR(DL.STATUS) as STATUS,
				MD5(CONCAT(
					IF(ISNULL(dd.DECLARATION_DEBIT_ID),
							f_elcBuildInvoiceNoForInterfaces(D.CRSO_RELATION_ID,dc.`YEAR`,dc.DECLARATION_CREDIT_ID,true,false),
							f_elcBuildInvoiceNoForInterfaces(D.CRSO_RELATION_ID,dd.`YEAR`,dd.DECLARATION_DEBIT_ID,false,false)
					 ),
					CONCAT(cac.CODE,
						LPAD(D.CRSO_RELATION_ID, 8, 0),
						LPAD(D.PARTICIPANT_RELATION_ID, 8, 0)
					),
					REPLACE(IF(ISNULL(dd.DECLARATION_DEBIT_ID), f_elcGetDeclarationCreditTotalAmount(D.CRSO_RELATION_ID, dc.DECLARATION_CREDIT_ID, dc.`YEAR`), f_elcGetDeclarationDebitTotalAmount(D.CRSO_RELATION_ID, dd.DECLARATION_DEBIT_ID, dd.`YEAR`)), '.', ',')
				)) as `Hash`
			FROM TBL_DECLARATION_SET_LINE as DSL
			INNER JOIN TBL_DECLARATION as D
				ON DSL.DECLARATION_ID = D.DECLARATION_ID
			INNER JOIN TBL_DECLARATION_LINE as DL
				ON D.DECLARATION_ID = DL.DECLARATION_ID
			INNER JOIN TBL_RELATION as R
				ON D.PARTICIPANT_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_RELATION as cr
				ON cr.RELATION_ID = D.CRSO_RELATION_ID
			INNER JOIN TBL_ADDRESS as ca
				ON cr.ADDRESS_ID = ca.ADDRESS_ID
			INNER JOIN TBL_COUNTRY as cac
				ON cac.COUNTRY_ID = ca.COUNTRY_ID
			LEFT JOIN TBL_DECLARATION_DEBIT as dd
				ON DSL.DECLARATION_SET_ID = dd.DECLARATION_SET_ID
			LEFT JOIN TBL_DECLARATION_CREDIT as dc
				ON DSL.DECLARATION_SET_ID = dc.DECLARATION_SET_ID";
		return $sql;
	}

	public function tfValueProcessor($args=false)
	{
		if(!$args)
			return array($this,'tfValueProcessor');

		require_once(ELC_CLASSES.'ELCListBuilder.php');
		$list = new ELCDefaultList(IWRegistry::registry('db'),IWRegistry::registry('langIf'));
		if(!empty($_POST['export']))
			$list->setShowType(LIST_BUTTON_CSV_EXPORT);

		$this->_cols = $args['visibleColumns'];
		switch($args['head'])
		{
			case 'COMPANY_NAME':
				return $list->tfLongNames($args);
			break;
			case 'Invoice':
				$args['hidename'] = true;
				return $list->tfDocumentLink($args);
			break;
			case 'MONTH':
				return $list->tfMonth($args);
			break;
			case 'QUANTITY':
				return $list->tfValueProcessorQuantityNoDecimals($args);
			break;
			case 'STATUS':
				return $list->tfDeclarationStatus($args);
			break;
			case 'Hash':
				$this->totalHash .= $args['value'];
				return $list->tfFixedFontWidth($args);
			break;
		}
		return $args['value'];
	}

	public function getSQLGroupBy()
	{
		return " GROUP BY DSL.DECLARATION_SET_ID";
	}

	public function getSQLOrderBy()
	{
		return " ORDER BY INVOICE_NR";
	}

	public function tfAllowSorting()
	{
		return false;
	}

	public function tfFooterProcessor()
	{
		return array($this, 'tfFooter');
	}

	public function tfFooter()
	{
		if(!$this->_cols)
			return '';
		$rv = explode(',',str_repeat(',',(sizeof($this->_cols)-1)));
		$pos = array_flip($this->_cols);
		if(isset($pos['Hash']))
			$rv[$pos['Hash']] = '<span style="font-size: 9pt; font-family: monospace; font-weight: bold;">'.md5($this->totalHash).'</span>';
		return $rv;
	}

	public function getRequiredFilters()
	{
		return array('CRSO_RELATION_ID'=>'CRSO_RELATION_ID','EXPORT_DATE'=>'EXPORT_DATE');
	}

	public function registerFields(IWCustomQueryFactory $cq)
	{
		$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
			$this->getFilter('filterCRSO'),
			'CRSO_RELATION_ID','CRSO','D');
		$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
			'SELECT DISTINCT EXPORT_DATE as VAL,EXPORT_DATE as DISP FROM TBL_DECLARATION
				WHERE NOT ISNULL(EXPORT_DATE)
				ORDER BY EXPORT_DATE',
			'EXPORT_DATE','Export Date','D');

		$cq->setField('DECLARATION_DEBIT_ID','Credits',FIELD_INPUT_MULTI,array(DB_NULL=>'Show Credits'),false,'dd');
		$cq->setField('DECLARATION_CREDIT_ID','Invoices',FIELD_INPUT_MULTI,array(DB_NULL=>'Show Invoices'),false,'dc');
	}
}
?>
