<?php
/**
 * @package
 * @subpackage
 * @since 8-okt-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class WEEECustomQueryDeclarationGenericReport extends IWCustomQueryObject
{
	/**
	 * @var int
	 */
	public $limit = 50;
	protected $totals = array('Quantity'=>0,'Weight'=>0,'Amount'=>0.00,'WeightDec'=>0.00);
	protected $_cols = false;
	protected $products = array();
	protected $selectedProducts = array();
	protected $crso = false;
	protected $hasAdditionalFilterSettings = false;
	protected $viewByParticipant = false;

	public function __construct()
	{
		$this->setSettings();
	}
	/**
	 * Only Sysadmin and Operator and CRSO can view these queries
         * We never breakdown by participant, so HasCrsoDeclarationDetails
         * doesn't apply
	 * @return bool
	 */
	public function isPrivileged()
	{
		require_once(IWARE.'group.php');
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
			case GROUP_ACCOUNTANT:
			case GROUP_CRSO:
				return true;
		}
		return false;
	}

	public function tfDescription()
	{
		return IWRegistry::registry('langIf')->getText('WEEE Custom Query Generic Declaraton Report Description');
	}

	/**
	 *
	 */
	public function getQuery()
	{
		$sql = "SELECT Part.`NAME` as Participant, D.`MONTH` as Month,D.`YEAR` as Year,B.`NAME` as Brand,PC.`NAME` as Category, CPD.`NAME` as Dimension, P.`NAME` as Product,
				R.`NAME` as CRSO, SUM(DL.QUANTITY) as Quantity, SUM(DL.WEIGHT) as Weight, SUM(DL.WEIGHT_DEC) as WeightDec, SUM(DL.EXPORTED_AMOUNT) as Amount
			FROM TBL_DECLARATION as D ";
		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT) {
			$sql .= "INNER JOIN TBL_ACCOUNTANT_CRSO as ac
				ON ac.CRSO_RELATION_ID = D.CRSO_RELATION_ID ";
		}
		$sql .= "INNER JOIN TBL_DECLARATION_LINE as DL
				ON D.DECLARATION_ID = DL.DECLARATION_ID
			INNER JOIN TBL_RELATION as R
				ON D.CRSO_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_PRODUCT as P
				ON D.PRODUCT_ID = P.PRODUCT_ID
			INNER JOIN TBL_RELATION as Part
				ON D.PARTICIPANT_RELATION_ID = Part.RELATION_ID
			LEFT JOIN TBL_BRAND as B
				ON DL.BRAND_ID = B.BRAND_ID
			LEFT JOIN TBL_PRODUCT_CATEGORY as PC
				ON DL.PRODUCT_CATEGORY_ID = PC.PRODUCT_CATEGORY_ID
			LEFT JOIN TBL_CRSO_PRODUCT_DIMENSION as CPD
				ON DL.CPD_ID = CPD.CPD_ID
			";
		return $sql;
	}
	/**
	 *
	 */
	public function getSQLGroupBy()
	{
		if($this->crso && $this->hasAdditionalFilterSettings) {
			return " GROUP BY DL.BRAND_ID,DL.PRODUCT_CATEGORY_ID,DL.CPD_ID,D.`YEAR`,D.`MONTH`,D.PRODUCT_ID,D.PARTICIPANT_RELATION_ID,D.CRSO_RELATION_ID";
		}
		return " GROUP BY DL.BRAND_ID,DL.PRODUCT_CATEGORY_ID,DL.CPD_ID,D.`YEAR`,D.`MONTH`,D.PRODUCT_ID,D.CRSO_RELATION_ID";
	}
	/**
	 *
	 */
	public function getSQLOrderBy()
	{
		if($return = parent::getSQLOrderBy())
			return $return;
		return " ORDER BY CRSO,Participant,Product,Year,Month,Brand,Category,Dimension";
	}
	/**
	 *
	 */
	public function tfAllowSorting()
	{
		return true;
	}
	/**
	 *
	 */
	public function getWhere()
	{
		if($_SESSION['session_group_group_id']==GROUP_CRSO)
			return sprintf(' AND D.CRSO_RELATION_ID = %s',$_SESSION['crso_relation_id'].' ');
		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT)
			return sprintf(' AND ac.ACCOUNTANT_RELATION_ID = %s',$_SESSION['accountant_relation_id'].' ');
		return '';
	}
	/**
	 *
	 */
	public function registerFields(IWCustomQueryFactory $cq,$skipit=false)
	{
		if($_SESSION['session_group_group_id']!=GROUP_CRSO)
			$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),$this->getFilter('filterCRSO'),'CRSO_RELATION_ID','CRSO','D');

		if(!$skipit)
		{
			$cq->setField('EXPORT_DATE','Invoice Period',FIELD_INPUT_PERIOD);
			$cq->setField('MONTH','Declaration Period',FIELD_INPUT_PERIOD,false,false,false,array($this,'filterMonth'));
		}

		if(count($this->products)!=1)
		{
			$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
				'SELECT PRODUCT_ID as VAL, `NAME` as DISP FROM TBL_PRODUCT '.
					(count($this->products)?'WHERE PRODUCT_ID IN ('.implode(',',$this->products).')':'').' ORDER BY DISP',
				'PRODUCT_ID','Product','D'
				);
		}

		if($this->crso && $this->hasAdditionalFilterSettings)
		{
			$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
				sprintf('SELECT RELATION_ID as VAL, `NAME` as DISP FROM
					TBL_RELATION as R
					INNER JOIN TBL_CRSO_PARTICIPANT as CP
					ON R.RELATION_ID = CP.PARTICIPANT_RELATION_ID
					WHERE CP.CRSO_RELATION_ID = %s
						ORDER BY DISP', $this->crso),
				'PARTICIPANT_RELATION_ID','Participant','D'
			);

		}

		if(count($this->selectedProducts)!=1)
			return;
		$rv = $this->tfHideColumns();
		$div = array_diff(array('Dimension','Category','Brand'),$rv);
		foreach($div as $method)
			$this->{'register'.$method}($cq);
	}

	protected function registerDimension(IWCustomQueryFactory $cq)
	{
		$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
			sprintf('SELECT CPD_ID as VAL, `NAME` as DISP
				FROM TBL_CRSO_PRODUCT_DIMENSION WHERE PRODUCT_ID = %s %s',
				current($this->selectedProducts),
				($this->crso?' AND CRSO_RELATION_ID = '.$this->crso:'')),
			'CPD_ID','Dimension','DL');
	}

	protected function registerCategory(IWCustomQueryFactory $cq)
	{
		$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
			sprintf('SELECT PRODUCT_CATEGORY_ID as VAL, `NAME` as DISP
				FROM TBL_PRODUCT_CATEGORY WHERE PRODUCT_ID = %s %s',
				current($this->selectedProducts),
				($this->crso?' AND CRSO_RELATION_ID = '.$this->crso:'')),
			'PRODUCT_CATEGORY_ID','Category','DL');
	}

	protected function registerBrand(IWCustomQueryFactory $cq)
	{
		$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
			sprintf('SELECT BRAND_ID as VAL, `NAME` as DISP
				FROM TBL_BRAND WHERE ACTIVE = 1 AND PRODUCT_ID = %s %s',
				current($this->selectedProducts),
				($this->crso?' AND CRSO_RELATION_ID = '.$this->crso:'')),
			'BRAND_ID','Brand','DL');
	}

	public function filterMonth($cqf,$mode,$handle,$values)
	{
		$rv = "AND (";
		$src = $values['tbl']?$values['tbl'].'.'.'`'.$handle.'`':''.$handle.'';

		$andModes = array('','~','!~');
		$operator = in_array($mode,$andModes)?'AND ':' OR ';
		$ops = $val = '';

		$settings = $cqf->getSettings();

		$betweenModes = array('~','!~');
		if(in_array($mode,$betweenModes))
		{
			$rv .= $src.' '.(($mode{0}=='!')?'NOT ':'').' BETWEEN ';
			if(count($settings[$handle])>2)
				return '';
		}
		$rvYear = '';
		$valuesProvided = false;
		foreach($settings[$handle] as $value)
		{
			if(!preg_match('/^\d{4}-\d{2}-\d{2}$/',$value))
				continue;
			$valuesProvided = true;
			$year = substr($value,0,4);
			$value = substr($value,-5,2);
			$rvYear = $rvYear?$rvYear.=','.$year:' AND `YEAR` IN ('.$year;

			switch($mode)
			{
				case '': // Means EQUAL
					$ops = '=';
				case '!': // Means NOT EQUAL
					$ops || $ops = '!=';
					$rv .= sprintf("%s %s '%s' %s",$src,$ops,$value,$operator);
				break;
				case '~': // Means BETWEEN
					$ops = '=';
				case '!~': // Means NOT BETWEEN
					$ops || $ops = '!=';
					$rv .= sprintf("'%s' %s",$value,$operator);
				break;
				case '^': // Means LARGER THAN
					$ops = '>';
				case '$': // Means ENDS WITH
					$ops || $ops = '<';
					$rv .= sprintf("%s %s '%s' %s",$src,$ops,$value,$operator);
				break;
			}

		}
		if(!$valuesProvided)
			return '';
		return substr($rv,0,-4).')'.$rvYear.')';
	}

	public function getRequiredFilters()
	{
		if(!$this->crso)
			return array('CRSO_RELATION_ID'=>'CRSO_RELATION_ID');
	}

	public function tfValueProcessor($args=false)
	{
		if(!$args)
			return array($this,'tfValueProcessor');

		require_once(ELC_CLASSES.'ELCListBuilder.php');
		$list = new ELCDefaultList(IWRegistry::registry('db'),IWRegistry::registry('langIf'));
		if(!empty($_POST['export']))
			$list->setShowType(LIST_BUTTON_CSV_EXPORT);

		$this->_cols = $args['visibleColumns'];
		switch($args['head'])
		{
			case 'Quantity':
			case 'Weight':
				$args['extra'] .= ' style="text-align:right;"';
				$this->totals[$args['head']] += $args['value'];
				return $list->tfValueProcessorQuantityNoDecimals($args);
			break;
			case 'WeightDec':
			case 'Amount':
				$args['extra'] .= ' style="text-align:right;"';
				$this->totals[$args['head']] += $args['value'];
				return $list->tfValueProcessorQuantity($args);
			break;
			case 'Month':
				return $list->tfMonth($args);
			break;
		}
		return $args['value'];
	}

	public function tfHeaderProcessor($args)
	{
		$args['headerText'] = $args['head'];
		if(in_array($args['head'],array('Quantity','Weight', 'WeightDec','Amount')))
			return ' style="text-align:right;"';
	}

	public function tfHideColumns()
	{

        if($this->viewByParticipant) {
            return array('Participant'=>'Participant','CRSO'=>'CRSO','Dimension'=>'Dimension','Brand'=>'Brand','Category'=>'Category','Weight'=>'Weight','WeightDec'=>'WeightDec', 'Product' => 'Product', 'Month' => 'Month', 'Year' => 'Year');
        }
        $rv = array('');
		if(!$this->crso)
			return $rv;

		$rv = array('Participant'=>'Participant','CRSO'=>'CRSO','Dimension'=>'Dimension','Brand'=>'Brand','Category'=>'Category','Weight'=>'Weight','WeightDec'=>'WeightDec');

		$rel = new Relation(IWRegistry::registry('db'),$this->crso);
		$country = new Country(IWRegistry::registry('db'),$rel->getCountry(),true);

		if(count($this->selectedProducts)==1)
			$rv['Product'] = 'Product';

		foreach($this->selectedProducts as $pid)
		{
			if($country->has(HasDeclarationDimension,$pid))
				unset($rv['Dimension']);
			if($country->has(HasDeclByBrand,$pid))
				unset($rv['Brand']);
			if($country->has(HasDeclByCategory,$pid))
				unset($rv['Category']);
			if($country->has(HasDeclOfWeight,$pid))
				unset($rv['Weight']);
			if($country->has(HasDeclOfWeightWithDecimals,$pid))
				unset($rv['WeightDec']);
			if($country->has(HasGenericReportByParticipant, $pid))
				unset($rv['Participant']);
        }

        reset($this->selectedProducts);

        return $rv;
	}

	public function tfFooterProcessor()
	{
		return array($this, 'tfFooter');
	}

	public function tfFooter()
	{
		if(!$this->_cols)
			return '';
		$rv = explode(',',str_repeat(',',(sizeof($this->_cols)-1)));
		$pos = array_flip($this->_cols);
	    foreach($this->totals as $index => $total){
	            if($total != '0' && $total != '0.00')
	                isset($pos[$index]) && $rv[$pos[$index]] = '<div style="text-align: right; font-weight: bold;">'.locale_nf($total,($index=='Amount'?2:0)).'</div>';
	    }
	    foreach($this->totals as $index => $total)
	    {
	        $this->totals[$index] = 0;
	    }
		return $rv;
        }

        public function tfRowProcessor()
        {
            if($this->viewByParticipant){
                return array($this, 'tfRow');
            }
        }

        public function tfRow(&$params)
        {

	    $params['attr']['style'] = 'display:none;';
	    return true;

        }

	public function getRowCount($query,$where,$group_by)
	{
		$q = IWRegistry::registry('db')->query($query.$where.$group_by);
		$q->execute();
		$r = $q->fetchAll();
		return count($r);
	}

	public function getGroupedRowCount($query,$groups,$where,$group_by,$groupfield,$fields)
	{
		$rv = array();
		$src = $fields[$groupfield]['tbl']?$fields[$groupfield]['tbl'].'.'.'`'.$groupfield.'`':'`'.$groupfield.'`';
		foreach($groups['options'] as $value => $label)
		{
			$val = (($value==DB_NULL)?sprintf(' ISNULL(%s) ',$src):(($value=='NOT'.DB_NULL)?sprintf(' NOT ISNULL(%s) ',$src):sprintf('%s = %s',$src,$value)));
			$newwhere = ($where?$where.' AND ':' WHERE ').$val;
			$q = IWRegistry::registry('db')->query($query.$newwhere.$group_by);
			$q->execute();
			$r = $q->fetchAll();
			$rv[$label] = count($r);
		}
		return $rv;
	}

	private function setSettings()
	{
		$product_ids = array();
		$pids = isset($_REQUEST['PRODUCT_ID'])?$_REQUEST['PRODUCT_ID']:false;
		$crso_id = false;
		if((isset($_REQUEST['CRSO_RELATION_ID']))&&count($_REQUEST['CRSO_RELATION_ID']==1))
			$crso_id = current($_REQUEST['CRSO_RELATION_ID']);
		// Check this last, make sure they cannot access other CRSO's
		$crso_id = ($_SESSION['session_group_group_id']==GROUP_CRSO)?$_SESSION['crso_relation_id']:$crso_id;
		if($crso_id)
		{
			require_once(ELC_CLASSES.'CRSOProduct.php');
			$cp = new CRSOProduct(IWRegistry::registry('db'));
			$q = $cp->getProducts($crso_id);
			while($r = $q->fetch())
				$product_ids[] = $r->PRODUCT_ID;
		}
		$this->selectedProducts = $pids?$pids:$product_ids;
		$this->products = $product_ids;
        $this->crso = $crso_id;

        //Additional filter settings
        if(isset($_REQUEST['additional_groupby_participant'])) {
            $this->viewByParticipant = true;
        }
		if($crso_id) {
			$rel = new Relation(IWRegistry::registry('db'), $this->crso);
			$country = new Country(IWRegistry::registry('db'), $rel->getCountry(), true);
			foreach ($this->selectedProducts as $pid) {
				if (!$country->has(HasGenericReportByParticipant, $pid)) {
					$this->hasAdditionalFilterSettings = false;
					$this->viewByParticipant = false;
					return;
				}
			}
		}
        $this->hasAdditionalFilterSettings = true;
        if($this->viewByParticipant) {
            $this->showExportFunctions = false;
            $this->limit = false;
        }

    }

    public function displayAdditionalFilterSettings() {
        if(!$this->hasAdditionalFilterSettings)
            return '';

        return '<br /><label for="additional_groupby_participant">'.IWRegistry::registry('langIf')->getText('Collapse details').
            '</label>&nbsp;<input type="checkbox" value="1"'.
                ($this->viewByParticipant ? ' checked="checked" ': '').
                    'name="additional_groupby_participant" />';
    }
}
?>
