<?php
require_once(ELC_CLASSES.'Declaration.php');
/**
 * @package WEEE
 * @subpackage WEEECustomQuery
 * @since 23-jan-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class WEEECustomQueryDeclarationOverview extends IWCustomQueryObject
{
	public $limit = 50;

	public function isPrivileged()
	{
		require_once(IWARE.'group.php');
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
			case GROUP_ACCOUNTANT:
				return true;
		}
		return false;
	}

	public function getQuery()
	{
		$this->rowCntQuery = "SELECT COUNT(D.DECLARATION_ID) as CNT
			FROM TBL_DECLARATION as D ";
		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT) {
			$this->rowCntQuery .= "INNER JOIN TBL_ACCOUNTANT_CRSO as ac
				ON ac.CRSO_RELATION_ID = D.CRSO_RELATION_ID ";
		}
		$this->rowCntQuery .= "INNER JOIN TBL_RELATION as R1
				ON D.PARTICIPANT_RELATION_ID = R1.RELATION_ID
			INNER JOIN TBL_RELATION as R2
				ON D.CRSO_RELATION_ID = R2.RELATION_ID
			INNER JOIN TBL_ADDRESS as a
				ON R2.ADDRESS_ID = a.ADDRESS_ID";

		$sql = "SELECT D.DECLARATION_ID,
				IF(ISNULL(DD.DECLARATION_DEBIT_ID),
					f_elcBuildInvoiceNoForInterfaces(D.CRSO_RELATION_ID,DC.`YEAR`,DC.DECLARATION_CREDIT_ID,true,false),
					f_elcBuildInvoiceNoForInterfaces(D.CRSO_RELATION_ID,DD.`YEAR`,DD.DECLARATION_DEBIT_ID,false,false)) as `Invoice No.`,
				R2.`NAME` as CRSO_NAME, R1.`NAME` as COMPANY_NAME, R1.`VAT_NO`,
				D.`YEAR`, D.`MONTH`, '' as PERIOD, SUM(DL.QUANTITY) as QUANTITY, BIT_OR(DL.STATUS) as STATUS,
				D.DOCUMENT_ID as DOCUMENT,
				D.DECLARATION_ID as UNLINK_DOCUMENT,
				EXPORT_DATE
			FROM TBL_DECLARATION as D ";
		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT) {
			$sql .= "INNER JOIN TBL_ACCOUNTANT_CRSO as ac
				ON ac.CRSO_RELATION_ID = D.CRSO_RELATION_ID ";
		}
		$sql .= "INNER JOIN TBL_DECLARATION_LINE as DL
				ON D.DECLARATION_ID = DL.DECLARATION_ID
			INNER JOIN TBL_RELATION as R1
				ON D.PARTICIPANT_RELATION_ID = R1.RELATION_ID
			INNER JOIN TBL_RELATION as R2
				ON D.CRSO_RELATION_ID = R2.RELATION_ID
			INNER JOIN TBL_ADDRESS as a
				ON R2.ADDRESS_ID = a.ADDRESS_ID
			LEFT JOIN TBL_DECLARATION_SET_LINE as DSL
				ON D.DECLARATION_ID = DSL.DECLARATION_ID
			LEFT JOIN TBL_DECLARATION_CREDIT as DC
				ON DSL.DECLARATION_SET_ID = DC.DECLARATION_SET_ID
			LEFT JOIN TBL_DECLARATION_DEBIT as DD
				ON DSL.DECLARATION_SET_ID = DD.DECLARATION_SET_ID
			";

		return $sql;
	}

	public function getWhere()
	{
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_ACCOUNTANT:
				return " AND ac.ACCOUNTANT_RELATION_ID = ".$_SESSION['accountant_relation_id']." ";
		}
		return "";
	}

	public function getSQLGroupBy()
	{
		return " GROUP BY D.DECLARATION_ID ";
	}

	public function getSQLOrderBy()
	{
		return " ORDER BY D.DECLARATION_ID ";
	}

	public function tfColumnModifier()
	{
		return array('QUANTITY'=>'style="text-align:right;"');
	}

	public function registerFields(IWCustomQueryFactory $cq)
	{
		$cq->setField('NAME','Company',FIELD_INPUT_TEXT,false,false,'R1');
		$cq->setField('IS_EXPORTED','Exported',FIELD_INPUT_MULTI,array(DB_NULL=>'Not Exported',DECLARATION_EXPORT_PENDING=>'Pending',DECLARATION_EXPORT_EXPORTED=>'Exported'),false,'D');
		$cq->setField('DOCUMENT_ID','Document',FIELD_INPUT_MULTI,array(DB_NULL=>'Has No Document','NOT'.DB_NULL=>'Has Document'),false,'D');
		$cq->setField('YEAR','Year',FIELD_INPUT_DROPDOWN,$this->getYearList(),false,'D');
		$cq->setField('MONTH','Month',FIELD_INPUT_DROPDOWN,$this->getMonthList(),false,'D');

		$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
			$this->getFilter('filterCRSO'),
			'CRSO_RELATION_ID','CRSO','D');
		$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
			'SELECT DISTINCT IFNULL(d.EXPORT_DATE,"'.DB_NULL.'") as VAL, IFNULL(d.EXPORT_DATE,"Not Exported") as DISP
				FROM TBL_DECLARATION as d',
			'EXPORT_DATE','Export Date','D');
	}
	public function tfHideColumns()
	{
		$rv = array('YEAR','MONTH','DECLARATION_ID');
		if(!isset($_GET['groupby']))
			return $rv;
		if($_GET['groupby']=='CRSO_RELATION_ID')
			$rv[] = 'CRSO_NAME';
		return $rv;
	}
	public function tfValueProcessor($args=false)
	{
		if(!$args)
			return array($this,'tfValueProcessor');

		require_once(ELC_CLASSES.'ELCListBuilder.php');
		$list = new ELCDefaultList(IWRegistry::registry('db'),IWRegistry::registry('langIf'));
		if(!empty($_POST['export']))
			$list->setShowType(LIST_BUTTON_CSV_EXPORT);
		switch($args['head'])
		{
			case 'COMPANY_NAME':
			case 'CRSO_NAME':
				return $list->tfLongNames($args);
			break;
			case 'DOCUMENT':
				$args['hidename'] = true;
				return $list->tfDocumentLink($args);
			break;
			case 'PERIOD':
				return $list->tfMonth(array('value'=>$args['row']['MONTH'])).' '.$args['row']['YEAR'];
			break;
			case 'QUANTITY':
				return $list->tfValueProcessorQuantityNoDecimals($args);
			break;
			case 'STATUS':
				return $list->tfDeclarationStatus($args);
			break;
			case 'UNLINK_DOCUMENT':
				$args['url'] = 'scripts/elc/sysadmin/UnlinkDeclarationInvoice.php';
				return $list->tfUnlinkDocumentLink($args);
			break;
		}
		return $args['value'];
	}

	private function getYearList()
	{
		$date = currentDate();
		$year = $date->getYear();
		if($year == WEEE_APPLICATION_START_YEAR)
			return array($year=>$year);

		$rv = array();
		for($i = WEEE_APPLICATION_START_YEAR;$i<=$year;$i++)
			$rv[$i] = $i;
		return $rv;
	}

	private function getMonthList()
	{
		$rv = array();
		for($i = 1;$i<=12;$i++)
			$rv[$i] = strftime('%B', mktime(1, 1, 1, $i, 1) );
		return $rv;
	}
}
?>
