<?php
require_once(IWARE.'IWCustomQueryDynamicQuery.php');
/**
 * @package WEEE
 * @subpackage CustomQuery
 * @author Marten Koedam <marten@dalines.net>
 */
class WEEECustomQueryDynamicQuery extends IWCustomQueryObject
{
	/**
	 * @access private
	 * @var IWCustomQueryDynamicQuery
	 */
	private $obj;
	/**
	 * @var int
	 */
	public $limit = 50;
	/**
	 * @param int $report_id
	 */
	public function __construct($report_id)
	{
		$this->obj = new IWCustomQueryDynamicQuery(IWRegistry::registry('db'));
		$this->obj->set($report_id);
		$this->handle = 'WEEECustomQueryDynamicReport_'.$report_id;
		setPageTitlePostfix(' - '.$this->obj->name);
	}

	public function getRequiredFilters()
	{
		$rv = array();
		foreach($this->obj->getFilters() as $filter)
			if($filter->required == 'Yes')
				$rv[$filter->fieldname] = $filter->fieldname;
		return $rv;
	}

	/**
	 * @return string IWCustomQueryDynamicQuery->name
	 */
	public function getName()
	{
		return $this->obj->name;
	}
	/**
	 * Only Sysadmin and Operator can view these queries
	 * @return bool
	 */
	public function isPrivileged()
	{
		require_once(IWARE.'group.php');
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
				return true;
		}
		return false;
	}
	/**
	 * @return string $sql
	 */
	public function getQuery()
	{
		$sql = $this->obj->query;
		return $sql;
	}
	/**
	 * @return string $sql
	 */
	public function getWhere()
	{
		if(!$this->obj->where)
			return;
		return 'AND '.preg_replace('/^WHERE/','',$this->obj->where).' ';
	}
	/**
	 * Register the filters for the current Dynamic Report
	 */
	public function registerFields(IWCustomQueryFactory $cq)
	{
		// loop filters
		foreach($this->obj->getFilters() as $filter)
		{
			$query = (substr($filter->query,0,6)=='filter')?$this->getFilter($filter->query):$filter->query;
			$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
				$query,
				$filter->fieldname,
				$filter->displayname,
				$filter->table_alias
				);
		}
	}
	/**
	 * Get the group by
	 * @return string
	 */
	public function getSQLGroupBy()
	{
		return " ".$this->obj->group_by;
	}
	/**
	 * Default order by from the database. This will be overridden on column name click
	 * @return string
	 */
	public function getSQLOrderBy()
	{
		if($return = parent::getSQLOrderBy())
			return $return;

		return " ".$this->obj->order_by;
	}
	/**
	 * Default value processors
	 */
	public function tfValueProcessor($args=false)
	{
		if(!$args)
			return array($this,'tfValueProcessor');

		require_once(ELC_CLASSES.'ELCListBuilder.php');
		$list = new ELCDefaultList(IWRegistry::registry('db'),IWRegistry::registry('langIf'));
		if(!empty($_POST['export']))
			$list->setShowType(LIST_BUTTON_CSV_EXPORT);
		switch($args['head'])
		{
			case 'Month':
				return $list->tfMonth($args);
			break;
		}

		list(,$type) = $this->getColumnType($args['head']);
		if(!$type)
			return $args['value'];

		switch($type)
		{
			case 'Number':
				$args['extra'] .= ' style="text-align:right;"';
				return $list->tfValueProcessorQuantityNoDecimals($args);
			break;
			case 'Decimal':
				$args['extra'] .= ' style="text-align:right;"';
				return $list->tfValueProcessorQuantity($args);
			break;
			case 'Percentage':
				$args['extra'] .= ' style="text-align:right;"';
				return $list->tfPercentageNoDecimals($args);
			break;
			case 'Boolean':
				return $list->tfBooleanIcon($args);
			break;
		}
	}
	/**
	 *
	 */
	public function tfHeaderProcessor($args)
	{
		list($head,$type) = $this->getColumnType($args['head']);
		if(!$type)
			$head = $args['head'];
		$args['headerText'] = $head; // Dont translate, since those are being build on the fly
		if(in_array($type,array('Number','Decimal','Percentage')))
			return ' style="text-align:right;"';
	}
	/**
	 * Used for preset-value-processing
	 */
	private function getColumnType($head)
	{
		$rv = false;
		if(preg_match('/^(Number|Decimal|Percentage|Boolean)_(.*)$/i',$head,$matches))
		{
			$rv = $matches[1];
			$head = $matches[2];
		}
		return array($head,$rv);
	}
	/**
	 * Yes, we do allow sorting.
	 */
	public function tfAllowSorting()
	{
		return true;
	}
}
?>
