<?php
/**
 * @package
 * @subpackage
 * @author Nick Loeve <nick@basesoft.com>
 * @since 30/01/2007
 * @copyright 2006 BaseSoft B.V.
 * $Id$
 */
class WEEECustomQueryMasterDataHash extends IWCustomQueryObject
{
	public $limit = 50;
	private $_totalHash = '';

	public function isPrivileged()
	{
		require_once(IWARE.'group.php');
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
			case GROUP_ACCOUNTANT:
				return true;
		}
		return false;
	}

	public function getQuery()
	{
		$sql = "SELECT CONCAT(
			CONCAT(cac.CODE,
					LPAD(p.CRSO_RELATION_ID, 8, 0),
		   			LPAD(p.PARTICIPANT_RELATION_ID, 8, 0)
				),
			IF(ISNULL(r.NAME), '', r.NAME),
			IF(ISNULL(ca.ADDRESS_STREET_NAME), '', ca.ADDRESS_STREET_NAME),
			IF(ISNULL(ca.ADDRESS_ZIPCODE), '', ca.ADDRESS_ZIPCODE),
			IF(ISNULL(ca.CITY_NAME), '', ca.CITY_NAME),
			IF(ISNULL(cc.INFO_PERSON_LAST_NAME), '', cc.INFO_PERSON_LAST_NAME),
			IF(ISNULL(r.PHONE), '', r.PHONE),
			IF(ISNULL(r.FAX), '', r.FAX),
			IF(ISNULL(cl.NAVISION_CODE), '', cl.NAVISION_CODE),
			IF(ISNULL(cac.NAVISION_CODE), '', cac.NAVISION_CODE),
			IF(ISNULL(r.VAT_NO), '', r.VAT_NO),
			IF(ISNULL(r.EMAIL), '', r.EMAIL),
			IF(ISNULL(r.WEBSITE), '', r.WEBSITE),
			IF(ISNULL(ri.NAME), '', ri.NAME),
			IF(ISNULL(ria.ADDRESS_STREET_NAME), '', ria.ADDRESS_STREET_NAME),
			IF(ISNULL(ria.ADDRESS_ZIPCODE), '', ria.ADDRESS_ZIPCODE),
			IF(ISNULL(ria.CITY_NAME), '', ria.CITY_NAME),
			IF(ISNULL(riac.NAVISION_CODE), '', riac.NAVISION_CODE),
			IF(ISNULL(rii.INFO_PERSON_LAST_NAME), '', rii.INFO_PERSON_LAST_NAME),
			IF(ISNULL(r.BANK_ACCOUNT_NUMBER), '', r.BANK_ACCOUNT_NUMBER),
			IF(ISNULL(r.IBAN_NO), '', r.IBAN_NO),
			IF(ISNULL(r.SWIFT_BIC_CODE), '', r.SWIFT_BIC_CODE),
			IF(ISNULL(r.DD_BANK_ACCOUNT), '', r.DD_BANK_ACCOUNT),
			IF(ISNULL(r.DD_IBAN_NO), '', r.DD_IBAN_NO),
			IF(ISNULL(r.DD_SWIFT_BIC_CODE), '', r.DD_SWIFT_BIC_CODE),
			IF(ISNULL(r.DD_BANK_ACCOUNT) OR r.DD_BANK_ACCOUNT = '', 'no', 'yes'),
			IF(ISNULL(p.PRODUCER_REG_NO), '', p.PRODUCER_REG_NO),
			IF(ISNULL(r.PO_NO), '', r.PO_NO)) AS HASH,
			crso.NAME as CRSO_NAME,
			r.NAME as COMPANY_NAME,
			r.VAT_NO
		FROM TBL_CRSO_PARTICIPANT as p ";
		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT) {
			$sql .= "INNER JOIN TBL_ACCOUNTANT_CRSO as ac
				ON ac.CRSO_RELATION_ID = p.CRSO_RELATION_ID ";
		}
		$sql .= "INNER JOIN TBL_RELATION as r
			ON p.PARTICIPANT_RELATION_ID = r.RELATION_ID
		INNER JOIN TBL_PARTICIPANT as part
			ON p.PARTICIPANT_RELATION_ID = part.PARTICIPANT_RELATION_ID
		INNER JOIN TBL_RELATION as crso
			ON p.CRSO_RELATION_ID = crso.RELATION_ID
		INNER JOIN TBL_ADDRESS as crsoa
			ON crso.ADDRESS_ID = crsoa.ADDRESS_ID
		INNER JOIN TBL_LANGUAGE as cl
			ON r.INVOICE_LANGUAGE_ID = cl.LANGUAGE_ID
		INNER JOIN TBL_ADDRESS as ca
			ON r.ADDRESS_ID = ca.ADDRESS_ID
		INNER JOIN TBL_INFO_PERSON as cc
			ON r.CUSTOMER_CONTACT_ID = cc.INFO_PERSON_ID
		INNER JOIN TBL_COUNTRY as cac
			ON ca.COUNTRY_ID = cac.COUNTRY_ID
		INNER JOIN TBL_RELATION_INVOICE as ri
			ON r.RELATION_ID = ri.RELATION_ID
		INNER JOIN TBL_ADDRESS as ria
			ON ria.ADDRESS_ID = ri.ADDRESS_ID
		INNER JOIN TBL_COUNTRY as riac
			ON ria.COUNTRY_ID = riac.COUNTRY_ID
		INNER JOIN TBL_INFO_PERSON as rii
			ON ri.INFO_PERSON_ID = rii.INFO_PERSON_ID
		";

		return $sql;
	}

	public function getWhere()
	{
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_ACCOUNTANT:
				return " AND ac.ACCOUNTANT_RELATION_ID = ".$_SESSION['accountant_relation_id']." ";
		}
		return "";
	}

	public function getSQLGroupBy()
	{
		return "";
	}

	public function getSQLOrderBy()
	{
		return " ORDER BY p.CRSO_RELATION_ID,p.PARTICIPANT_RELATION_ID";;
	}

	public function registerFields(IWCustomQueryFactory $cq)
	{
		$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
			$this->getFilter('filterCRSO'),
			'CRSO_RELATION_ID','CRSO','p');
		$cq->setField('NAME','Company',FIELD_INPUT_TEXT,false,false,'r');
	}

	public function tfValueProcessor($args=false)
	{
		if(!$args)
			return array($this,'tfValueProcessor');
		require_once(ELC_CLASSES.'ELCListBuilder.php');
		$list = new ELCDefaultList(IWRegistry::registry('db'),IWRegistry::registry('langIf'));
		if(!empty($_POST['export']))
			$list->setShowType(LIST_BUTTON_CSV_EXPORT);
		switch($args['head'])
		{
			case 'COMPANY_NAME':
			case 'CRSO_NAME':
				return $list->tfLongNames($args);
			case 'HASH':
				$this->_totalHash .= md5($args['value']);
				return md5($args['value']);
			break;
			default:
		}
		return $args['value'];
	}

	public function tfHideColumns()
	{
		return array();
	}

	public function tfFooter()
	{
		return array('HASH' => '<div style="font-weight: bold;">'.md5($this->_totalHash).'</div>');
	}

	public function tfFooterProcessor()
	{
		return array($this, 'tfFooter');
	}

	public function resetInternals()
	{
		$this->_totalHash = '';
	}
}
?>