<?php
/**
 * @package WEEE
 * @subpackage WEEECustomQuery
 * @since 5-jan-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class WEEECustomQueryParticipantOverview extends IWCustomQueryObject
{
	public $limit = 50;

	public function isPrivileged()
	{
		require_once(IWARE.'group.php');
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
			case GROUP_ACCOUNTANT:
				return true;
		}
		return false;
	}

	public function getQuery()
	{
		$sql = "SELECT p.PARTICIPANT_RELATION_ID, DATE(r.CREATED) as CREATED,
				r.`NAME` as COMPANY_NAME,  rc.`NAME` as CRSO_NAME,
				pr.`NAME` as PRODUCT,
				pr.PRODUCT_ID,
				cpp.CRSO_RELATION_ID,
				r.VAT_NO, cpp.OBLIGATION_START_DATE,
				(cpp.PRODUCER_EXPORTER & 1) as PRODUCER,
				(cpp.PRODUCER_EXPORTER & 2) as EXPORTER,
				c.`NAME` as COUNTRY_NAME,
				cpp.`STATUS`, cpp.ACTIVE, u.USER_NAME as ADMIN_USER
			FROM TBL_PARTICIPANT as p
			INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT as cpp
				ON p.PARTICIPANT_RELATION_ID = cpp.PARTICIPANT_RELATION_ID
			INNER JOIN TBL_PRODUCT as pr
				ON cpp.PRODUCT_ID = pr.PRODUCT_ID ";
		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT) {
			$sql .= "INNER JOIN TBL_ACCOUNTANT_CRSO as ac
				ON ac.CRSO_RELATION_ID = cpp.CRSO_RELATION_ID ";
		}
		$sql .= "INNER JOIN TBL_RELATION as r
				ON p.PARTICIPANT_RELATION_ID = r.RELATION_ID
			INNER JOIN TBL_RELATION as rc
				ON cpp.CRSO_RELATION_ID = rc.RELATION_ID
			INNER JOIN TBL_ADDRESS as ca
				ON rc.ADDRESS_ID = ca.ADDRESS_ID
			INNER JOIN TBL_ADDRESS as a
				ON r.ADDRESS_ID = a.ADDRESS_ID
			INNER JOIN TBL_COUNTRY as c
				ON a.COUNTRY_ID = c.COUNTRY_ID
			INNER JOIN TBL_RELATION_USER as ru
				ON p.PARTICIPANT_RELATION_ID = ru.RELATION_ID
					AND ru.IS_ADMIN = 1
			INNER JOIN TBL_USER as u
				ON ru.USER_ID = u.USER_ID ";

		return $sql;
	}

	public function getWhere()
	{
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_ACCOUNTANT:
				return " AND ac.ACCOUNTANT_RELATION_ID = ".$_SESSION['accountant_relation_id']." ";
		}
		return "";
	}

	public function getSQLGroupBy()
	{
		return " GROUP BY cpp.PARTICIPANT_RELATION_ID, cpp.CRSO_RELATION_ID, cpp.PRODUCT_ID ";
	}

	public function getSQLOrderBy()
	{
		if($return = parent::getSQLOrderBy())
			return $return;
		return " ORDER BY cpp.`STATUS`,rc.NAME,r.NAME";
	}

	public function registerFields(IWCustomQueryFactory $cq)
	{
		$cq->setField('NAME','Company',FIELD_INPUT_TEXT,false,false,'r');
		$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
			'SELECT c.COUNTRY_ID as VAL, c.`NAME` as DISP
				FROM TBL_COUNTRY as c
				WHERE c.REGION_ID = 1 ORDER BY c.`NAME`',
			'COUNTRY_ID','Country','c');

		$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
			$this->getFilter('filterCRSO'),
			'CRSO_RELATION_ID','CRSO','cpp');
		// Add status filter.
		$cq->setField('STATUS','Status',FIELD_INPUT_MULTI,array(1=>'New',2=>'Verified',3=>'Authorized',4=>'Confirmed',5=>'Participant Ready'),false,'cpp');

		$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
			'SELECT p.PRODUCT_ID as VAL, p.`NAME` as DISP
				FROM TBL_PRODUCT as p
				ORDER BY p.`NAME`',
			'PRODUCT_ID','Product','pr');
	}

	public function tfValueProcessor($args=false)
	{
		if(!$args)
			return array($this,'tfValueProcessor');

		require_once(ELC_CLASSES.'ELCListBuilder.php');
		$list = new ELCDefaultList(IWRegistry::registry('db'),IWRegistry::registry('langIf'));
		if(!empty($_POST['export']))
			$list->setShowType(LIST_BUTTON_CSV_EXPORT);
		switch($args['head'])
		{
			case 'PARTICIPANT_RELATION_ID':
				$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
				if(!empty($_POST['export']))
					return $args['value'];
				return sprintf('<a href="InterfaceParticipantDetails.php?participant[id]=%s&crso[id]=%s&product[id]=%s">%s</a>',$args['value'],$args['row']['CRSO_RELATION_ID'],$args['row']['PRODUCT_ID'],$args['value']);
			case 'STATUS':
				return $list->tfParticipantStatus($args);
			case 'COMPANY_NAME':
			case 'CRSO_NAME':
				return $list->tfLongNames($args);
			case 'ACTIVE':
			case 'PRODUCER':
			case 'EXPORTER':
				return $list->tfBooleanIcon($args);
			break;
			case 'ADMIN_USER':
				if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT)
					return $args['value'];

				if(!empty($_POST['export']))
					return $args['value'];
				return sprintf('<a href="#%s" name="part_%s" onclick="doLogin(\'%s\');">%s</a>',$args['row']['PARTICIPANT_RELATION_ID'],$args['row']['PARTICIPANT_RELATION_ID'],$args['value'],$args['value']);
			break;
			default:
		}
		return $args['value'];
	}

	public function tfGetExtraInfo()
	{
		if($_SESSION['session_group_group_id'] == GROUP_ACCOUNTANT)
		{
			return sprintf('<div style="text-align: right;"><input type="button" value="%s" onclick="javascript:CQF.doExport(\'masterdata\',\'%s\');"></div>',
				IWRegistry::registry('langIf')->getText('Download csv for all participants masterdata'),md5('default'));
		}
		else
			return '';
	}

	/*
	 * Proof of concept
	 * public function tfRowProcessor($args=false)
	{
		if(!$args)
			return array($this,'tfRowProcessor');
		require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
		if($args['row']['STATUS']==PARTICIPANT_STATUS_NEW)
			$args['attr']['class'] .= ' highlite_red';
		if($args['row']['STATUS']==PARTICIPANT_STATUS_VERIFIED)
			$args['attr']['class'] .= ' highlite_yellow';
		if($args['row']['STATUS']==PARTICIPANT_STATUS_CONFIRMED)
			$args['attr']['class'] .= ' highlite_blue';
		if($args['row']['STATUS']==PARTICIPANT_STATUS_PARTICIPANT_READY)
			$args['attr']['class'] .= ' highlite_green';
		return true;
	}*/
	/**
	 * @todo FIX this, its currently broken when it loads the settings from the DB
	 */
	public function tfHideColumns()
	{
		$rv = array('CRSO_RELATION_ID','PRODUCT_ID');
		if(!isset($_GET['groupby']))
			return $rv;
		if($_GET['groupby']=='CRSO_RELATION_ID')
			$rv[] = 'CRSO_NAME';
		return $rv;
	}

	public function tfAllowSorting()
	{
		return true;
	}

	/*function tfSortingHat()
	{
		return array('STATUS'=>"'SortVal'",
			'COMPANY_NAME'=>"'String'",
			'CRSO_NAME'=>"'String'",
			'CREATED'=>"'String'",
			'VAT_NO'=>"'String'",
			'OBLIGATION_START_DATE'=>"'String'",
			'COUNTRY_NAME'=>"'String'",
			'PRODUCER'=>"'String'",
			'EXPORTER'=>"'String'",
			'ACTIVE'=>"'SortVal'",
			'PARTICIPANT_RELATION_ID'=>"'SortVal'",
			'ADMIN_USER'=>"'String'"
		);
	}*/
}
?>
