<?php
require_once(ELC_CLASSES.'RefundExport.php');
/**
 * @package WEEE
 * @subpackage CustomQuery
 * @since 24-jan-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class WEEECustomQueryRefundExportOverview extends IWCustomQueryObject
{
	public $limit = 50;

	public function isPrivileged()
	{
		require_once(IWARE.'group.php');
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
			case GROUP_ACCOUNTANT:
				return true;
		}
		return false;
	}
	public function getQuery()
	{
		$this->rowCntQuery = "SELECT COUNT(DISTINCT RE.REFUND_EXPORT_ID) as CNT
			FROM TBL_REFUND_EXPORT as RE ";
		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT) {
			$this->rowCntQuery .= "INNER JOIN TBL_ACCOUNTANT_CRSO as ac
				ON ac.CRSO_RELATION_ID = RE.CRSO_RELATION_ID ";
		}
		$this->rowCntQuery .= "INNER JOIN TBL_RELATION as R1
				ON RE.PARTICIPANT_RELATION_ID = R1.RELATION_ID
			INNER JOIN TBL_RELATION as R2
				ON RE.CRSO_RELATION_ID = R2.RELATION_ID
			INNER JOIN TBL_ADDRESS as a
				ON R2.ADDRESS_ID = a.ADDRESS_ID";

		$sql = "SELECT RE.REFUND_EXPORT_ID,
				IF(ISNULL(RRD.REFUND_REQUEST_DEBIT_ID),
					f_elcBuildInvoiceNoForInterfaces(RE.CRSO_RELATION_ID,RRC.`YEAR`,RRC.REFUND_REQUEST_CREDIT_ID,true,true),
					f_elcBuildInvoiceNoForInterfaces(RE.CRSO_RELATION_ID,RRD.`YEAR`,RRD.REFUND_REQUEST_DEBIT_ID,false,true)) as `Invoice No.`,
				R2.`NAME` as CRSO_NAME, R1.`NAME` as COMPANY_NAME, R1.`VAT_NO`,
				RE.EXPORT_DATE, SUM(RRL.QUANTITY) as QUANTITY,
				RE.INVOICE_DOCUMENT_ID as DOCUMENT,
				RE.REFUND_EXPORT_ID as UNLINK_DOCUMENT
			FROM TBL_REFUND_EXPORT as RE ";
		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT) {
			$sql .= "INNER JOIN TBL_ACCOUNTANT_CRSO as ac
				ON ac.CRSO_RELATION_ID = RE.CRSO_RELATION_ID ";
		}
		$sql .= "INNER JOIN TBL_RELATION as R1
				ON RE.PARTICIPANT_RELATION_ID = R1.RELATION_ID
			INNER JOIN TBL_RELATION as R2
				ON RE.CRSO_RELATION_ID = R2.RELATION_ID
			INNER JOIN TBL_ADDRESS as a
				ON R2.ADDRESS_ID = a.ADDRESS_ID
			LEFT JOIN TBL_REFUND_EXPORT_SET_LINE as DSL
				ON RE.REFUND_EXPORT_ID = DSL.REFUND_EXPORT_ID
			LEFT JOIN TBL_REFUND_EXPORT_CREDIT as RRC
				ON DSL.REFUND_EXPORT_SET_ID = RRC.REFUND_EXPORT_SET_ID
			LEFT JOIN TBL_REFUND_EXPORT_DEBIT as RRD
				ON DSL.REFUND_EXPORT_SET_ID = RRD.REFUND_EXPORT_SET_ID
			LEFT JOIN TBL_REFUND_REQUEST as RR
				ON RE.REFUND_EXPORT_ID = RR.REFUND_EXPORT_ID
			LEFT JOIN TBL_REFUND_REQUEST_LINE as RRL
				ON RR.REFUND_REQUEST_ID = RRL.REFUND_REQUEST_ID";

		return $sql;
	}

	public function getWhere()
	{
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_ACCOUNTANT:
				return " AND ac.ACCOUNTANT_RELATION_ID = ".$_SESSION['accountant_relation_id']." ";
		}
		return "";
	}

	public function getSQLGroupBy()
	{
		return " GROUP BY RE.REFUND_EXPORT_ID ";
	}

	public function getSQLOrderBy()
	{
		return " ORDER BY RE.REFUND_EXPORT_ID ";
	}

	public function tfHideColumns()
	{
		$rv = array('REFUND_EXPORT_ID');
		if(!isset($_GET['groupby']))
			return $rv;
		if($_GET['groupby']=='CRSO_RELATION_ID')
			$rv[] = 'CRSO_NAME';
		return $rv;
	}

	public function tfColumnModifier()
	{
		return array('QUANTITY'=>'style="text-align:right;"');
	}

	public function tfValueProcessor($args=false)
	{
		if(!$args)
			return array($this,'tfValueProcessor');

		require_once(ELC_CLASSES.'ELCListBuilder.php');
		$list = new ELCDefaultList(IWRegistry::registry('db'),IWRegistry::registry('langIf'));
		if(!empty($_POST['export']))
			$list->setShowType(LIST_BUTTON_CSV_EXPORT);
		switch($args['head'])
		{
			case 'COMPANY_NAME':
			case 'CRSO_NAME':
				return $list->tfLongNames($args);
			break;
			case 'DOCUMENT':
				$args['hidename'] = true;
				return $list->tfDocumentLink($args);
			break;
			case 'QUANTITY':
				return $list->tfValueProcessorQuantityNoDecimals($args);
			break;
			case 'UNLINK_DOCUMENT':
				$args['url'] = 'scripts/elc/sysadmin/UnlinkRefundInvoice.php';
				return $list->tfUnlinkDocumentLink($args);
			break;
		}
		return $args['value'];
	}

	public function registerFields(IWCustomQueryFactory $cq)
	{
		$cq->setField('NAME','Company',FIELD_INPUT_TEXT,false,false,'R1');
		$cq->setField('INVOICE_DOCUMENT_ID','Document',FIELD_INPUT_MULTI,array(DB_NULL=>'Has No Document','NOT'.DB_NULL=>'Has Document'),false,'RE');
//		$cq->setField('YEAR(D.EXPORT_DATE)','Year',FIELD_INPUT_DROPDOWN,$this->getYearList(),false);
//		$cq->setField('MONTH(D.EXPORT_DATE)','Month',FIELD_INPUT_DROPDOWN,$this->getMonthList(),false);

		$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
			$this->getFilter('filterCRSO'),
			'CRSO_RELATION_ID','CRSO','RE');
		$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
			'SELECT DISTINCT IFNULL(d.EXPORT_DATE,"'.DB_NULL.'") as VAL, IFNULL(d.EXPORT_DATE,"Not Exported") as DISP
				FROM TBL_REFUND_EXPORT as d',
			'EXPORT_DATE','Export Date','RE');
	}
	private function getYearList()
	{
		$date = currentDate();
		$year = $date->getYear();
		if($year == WEEE_APPLICATION_START_YEAR)
			return array($year=>$year);

		$rv = array();
		for($i = WEEE_APPLICATION_START_YEAR;$i<=$year;$i++)
			$rv[$i] = $i;
		return $rv;
	}

	private function getMonthList()
	{
		$rv = array();
		for($i = 1;$i<=12;$i++)
			$rv[$i] = strftime('%B', mktime(1, 1, 1, $i, 1) );
		return $rv;
	}
}
?>
