<?php
require_once(ELC_LISTS.'CustomQuery/WEEECustomQueryDeclarationGenericReport.php');
/**
 * @package
 * @subpackage
 * @since 10-okt-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class WEEECustomQueryRefundGenericReport extends WEEECustomQueryDeclarationGenericReport
{
	public function tfDescription()
	{
		return IWRegistry::registry('langIf')->getText('WEEE Custom Query Generic Refund Report Description');
	}

	/**
	 *
	 */
	public function getQuery()
	{
		$sql = "SELECT Part.`NAME` as Participant, MONTH(D.REFUND_EXPORT_DATE) as Month,YEAR(D.REFUND_EXPORT_DATE) as Year,
				B.`NAME` as Brand,PC.`NAME` as Category, P.`NAME` as Product,
				R2.`NAME` as Producer,
				R.`NAME` as CRSO, SUM(DL.QUANTITY) as Quantity, SUM(DL.WEIGHT) as Weight,
				SUM(DL.WEIGHT_DEC) as WeightDec, SUM(DL.EXPORTED_AMOUNT) as Amount
			FROM TBL_REFUND_REQUEST as D
			INNER JOIN TBL_REFUND_REQUEST_LINE as DL
				ON D.REFUND_REQUEST_ID = DL.REFUND_REQUEST_ID
			INNER JOIN TBL_RELATION as R
				ON D.CRSO_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_RELATION as R2
				ON DL.PRODUCER_RELATION_ID = R2.RELATION_ID
			INNER JOIN TBL_RELATION as Part
				ON D.PARTICIPANT_RELATION_ID = Part.RELATION_ID
			INNER JOIN TBL_PRODUCT as P
				ON D.PRODUCT_ID = P.PRODUCT_ID
			LEFT JOIN TBL_REFUND_EXPORT as RE
				ON D.REFUND_EXPORT_ID = RE.REFUND_EXPORT_ID
			LEFT JOIN TBL_BRAND as B
				ON DL.BRAND_ID = B.BRAND_ID
			LEFT JOIN TBL_PRODUCT_CATEGORY as PC
				ON DL.PRODUCT_CATEGORY_ID = PC.PRODUCT_CATEGORY_ID
			";
		return $sql;
	}
	/**
	 *
	 */
	public function getSQLGroupBy()
	{
		if($this->crso && $this->hasAdditionalFilterSettings) {
			return " GROUP BY DL.BRAND_ID,DL.PRODUCT_CATEGORY_ID,DL.PRODUCER_RELATION_ID,Year,Month,D.PRODUCT_ID,D.PARTICIPANT_RELATION_ID,D.CRSO_RELATION_ID";
		}
		// If we cant display by participant, we also wont display by producer.
		return " GROUP BY DL.BRAND_ID,DL.PRODUCT_CATEGORY_ID,Year,Month,D.PRODUCT_ID,D.CRSO_RELATION_ID";
	}
	public function tfHideColumns() {
		$rv = parent::tfHideColumns();
		if(!$this->crso || !$this->hasAdditionalFilterSettings) {
			$rv['Producer'] = 'Producer';
		}
		unset($rv['Dimension']); // Refunds do not use Dimensions.
		return $rv;
	}
	/**
	 *
	 */
	public function getSQLOrderBy()
	{
		if($return = IWCustomQueryObject::getSQLOrderBy())
			return $return;
		return " ORDER BY CRSO,Participant,Product,Year,Month,Producer,Brand,Category";
	}

	/**
	 *
	 */
	public function registerFields(IWCustomQueryFactory $cq)
	{
		$cq->setField('REFUND_EXPORT_DATE','Export Period',FIELD_INPUT_PERIOD,false,false,'D');
		$cq->setField('EXPORT_DATE','Invoice Period',FIELD_INPUT_PERIOD,false,false,'RE');
		parent::registerFields($cq,true);
		if($this->crso && $this->hasAdditionalFilterSettings) {
			$this->registerProducer($cq);
		}
	}

	protected function registerProducer(IWCustomQueryFactory $cq)
	{
		$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
			sprintf('SELECT RELATION_ID as VAL, `NAME` as DISP FROM
				TBL_RELATION as R
					INNER JOIN TBL_CRSO_PARTICIPANT as CP
					ON R.RELATION_ID = CP.PARTICIPANT_RELATION_ID
					WHERE CP.CRSO_RELATION_ID = %s
					ORDER BY DISP', $this->crso),
			'PRODUCER_RELATION_ID','Producer','DL');

	}

	protected function registerDimension(IWCustomQueryFactory $cq) {
		// Do nothing.
	}
}
?>
