<?php
require_once(ELC_CLASSES.'RefundRequest.php');
/**
 * @package WEEE
 * @subpackage CustomQuery
 * @since 26-sep-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class WEEECustomQueryRefundImporterStatus extends IWCustomQueryObject
{
	public $limit = 50;
	/**
	 *
	 */
	public function isPrivileged()
	{
		require_once(IWARE.'group.php');
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
			case GROUP_ACCOUNTANT:
				return true;
		}
		return false;
	}

	public function getQuery()
	{
		$sql = "SELECT RR.REFUND_REQUEST_ID as PoE_Id, R.`NAME` as RefundRequester, C.`NAME` as CRSO,
				RE.EXPORT_DATE, RR.IMPORTER_COMPANY_NAME, RR.IMPORTER_VAT_NO,
				IF(ISNULL(RED.REFUND_REQUEST_DEBIT_ID),
					IF(ISNULL(REC.REFUND_REQUEST_CREDIT_ID),
						'-',
						f_elcBuildInvoiceNoForInterfaces(RE.CRSO_RELATION_ID,REC.`YEAR`,REC.REFUND_REQUEST_CREDIT_ID,true,true)),
					f_elcBuildInvoiceNoForInterfaces(RE.CRSO_RELATION_ID,RED.`YEAR`,RED.REFUND_REQUEST_DEBIT_ID,false,true)) as `Invoice No.`,
				SUM(RRL.QUANTITY) as Quantity,
				'' as MarkAsResolved
			FROM TBL_REFUND_REQUEST as RR ";
		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT) {
			$sql .= "INNER JOIN TBL_ACCOUNTANT_CRSO as ac
				ON ac.CRSO_RELATION_ID = RR.CRSO_RELATION_ID ";
		}
		$sql .= "INNER JOIN TBL_REFUND_REQUEST_LINE as RRL
				ON RR.REFUND_REQUEST_ID = RRL.REFUND_REQUEST_ID
			INNER JOIN TBL_COUNTRY as Co
				ON RR.EXPORT_COUNTRY_ID = Co.COUNTRY_ID
			INNER JOIN TBL_RELATION as R
				ON RR.PARTICIPANT_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_RELATION as C
				ON RR.CRSO_RELATION_ID = C.RELATION_ID
			LEFT JOIN TBL_REFUND_EXPORT as RE
				ON RR.REFUND_EXPORT_ID = RE.REFUND_EXPORT_ID
			LEFT JOIN TBL_REFUND_EXPORT_SET_LINE as RESL
				ON RE.REFUND_EXPORT_ID = RESL.REFUND_EXPORT_ID
			LEFT JOIN TBL_REFUND_EXPORT_DEBIT as RED
				ON RESL.REFUND_EXPORT_SET_ID = RED.REFUND_EXPORT_SET_ID
			LEFT JOIN TBL_REFUND_EXPORT_CREDIT as REC
				ON RESL.REFUND_EXPORT_SET_ID = REC.REFUND_EXPORT_SET_ID";

		return $sql;
	}

	public function getSQLGroupBy()
	{
		return " GROUP BY RR.REFUND_REQUEST_ID";
	}

	public function getSQLOrderBy()
	{
		if($return = parent::getSQLOrderBy())
			return $return;
		return " ORDER BY RR.IMPORTER_STATUS,R.`NAME`";
	}

	public function getWhere()
	{
		$w = sprintf(" AND Co.REGION_ID = 1 AND RR.IMPORTER_STATUS = %s ", REFUND_IMPORTER_STATUS_REJECTED);
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_ACCOUNTANT:
				$w .= " AND ac.ACCOUNTANT_RELATION_ID = ".$_SESSION['accountant_relation_id']." ";
		}
		return $w;
	}

	public function registerFields(IWCustomQueryFactory $cq)
	{
		$cq->setField('IMPORTER_COMPANY_NAME','Importer',FIELD_INPUT_TEXT,false,false,'RR');
		$cq->setField('NAME','RefundRequester',FIELD_INPUT_TEXT,false,false,'R');
		$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
			$this->getFilter('filterCRSO'),
			'CRSO_RELATION_ID','CRSO','RR');
	}

	public function tfValueProcessor($args=false)
	{
		if(!$args)
			return array($this,'tfValueProcessor');

		require_once(ELC_CLASSES.'ELCListBuilder.php');
		$list = new ELCDefaultList(IWRegistry::registry('db'),IWRegistry::registry('langIf'));
		if(!empty($_POST['export']))
			$list->setShowType(LIST_BUTTON_CSV_EXPORT);
		switch($args['head'])
		{
			case 'RefundRequester':
			case 'CRSO':
			case 'IMPORTER_COMPANY_NAME':
				return $list->tfLongNames($args);
			break;
			case 'Quantity':
				return $list->tfValueProcessorQuantityNoDecimals($args);
			break;
			case 'MarkAsResolved':
				return sprintf('<a href="javascript:markAsResolved(%s);">%s</a>',
					$args['row']['PoE_Id'],
					IWRegistry::registry('langIf')->getText('Mark As Resolved'));
			break;
		}
		return $args['value'];
	}

	public function tfColumnModifier()
	{
		return array('Quantity'=>'style="text-align:right;"');
	}

	public function tfAllowSorting()
	{
		return true;
	}
}
?>
