<?php
require_once(ELC_CLASSES.'RefundExport.php');
/**
 * @package WEEE
 * @subpackage CustomQuery
 * @since 02-feb-2012
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2012 BaseSoft B.V.
 */
class WEEECustomQueryRefundYearOverview extends IWCustomQueryObject
{
	public $limit = 50;
	protected $products = array();
	protected $selectedProducts = array();
	protected $crso = false;

	public function __construct()
	{
		$this->setSettings();
	}

	public function isPrivileged()
	{
		require_once(IWARE.'group.php');
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
			case GROUP_ACCOUNTANT:
				return true;
			case GROUP_CRSO:
				return true;
		}
		return false;
	}
	public function getQuery()
	{
		$this->rowCntQuery = "SELECT COUNT(DISTINCT RE.PRODUCT_ID, YEAR(RE.EXPORT_DATE), RE.PARTICIPANT_RELATION_ID) as CNT
			FROM TBL_REFUND_EXPORT as RE ";
		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT) {
			$this->rowCntQuery .= "INNER JOIN TBL_ACCOUNTANT_CRSO as ac
				ON ac.CRSO_RELATION_ID = RE.CRSO_RELATION_ID ";
		}
		$this->rowCntQuery .= "INNER JOIN TBL_RELATION as R1
				ON RE.PARTICIPANT_RELATION_ID = R1.RELATION_ID
			INNER JOIN TBL_RELATION as R2
				ON RE.CRSO_RELATION_ID = R2.RELATION_ID
			INNER JOIN TBL_ADDRESS as a
				ON R2.ADDRESS_ID = a.ADDRESS_ID
			INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT as cpp
				ON RE.CRSO_RELATION_ID = cpp.CRSO_RELATION_ID
					AND RE.PRODUCT_ID = cpp.PRODUCT_ID
					AND RE.PARTICIPANT_RELATION_ID = cpp.PARTICIPANT_RELATION_ID";

		$sql = "SELECT R2.`NAME` as CRSO_NAME, P.`NAME` as Product,
				R1.`NAME` as COMPANY_NAME, R1.`VAT_NO`, cpp.ACTIVE,
				YEAR(RE.EXPORT_DATE) as Year, SUM(RRL.QUANTITY) as QUANTITY, SUM(RRL.WEIGHT) as WEIGHT
			FROM TBL_REFUND_EXPORT as RE ";
		if($_SESSION['session_group_group_id']==GROUP_ACCOUNTANT) {
			$sql .= "INNER JOIN TBL_ACCOUNTANT_CRSO as ac
				ON ac.CRSO_RELATION_ID = RE.CRSO_RELATION_ID ";
		}
		$sql .= "INNER JOIN TBL_RELATION as R1
				ON RE.PARTICIPANT_RELATION_ID = R1.RELATION_ID
			INNER JOIN TBL_RELATION as R2
				ON RE.CRSO_RELATION_ID = R2.RELATION_ID
			INNER JOIN TBL_ADDRESS as a
				ON R2.ADDRESS_ID = a.ADDRESS_ID
			INNER JOIN TBL_REFUND_REQUEST as RR
				ON RE.REFUND_EXPORT_ID = RR.REFUND_EXPORT_ID
			INNER JOIN TBL_REFUND_REQUEST_LINE as RRL
				ON RR.REFUND_REQUEST_ID = RRL.REFUND_REQUEST_ID
			INNER JOIN TBL_PRODUCT as P
				ON RE.PRODUCT_ID = P.PRODUCT_ID
			INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT as cpp
				ON RE.CRSO_RELATION_ID = cpp.CRSO_RELATION_ID
					AND RE.PRODUCT_ID = cpp.PRODUCT_ID
					AND RE.PARTICIPANT_RELATION_ID = cpp.PARTICIPANT_RELATION_ID";

		return $sql;
	}

	public function getWhere()
	{
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_ACCOUNTANT:
				return " AND ac.ACCOUNTANT_RELATION_ID = ".$_SESSION['accountant_relation_id']." ";
			case GROUP_CRSO:
				return " AND RE.CRSO_RELATION_ID = ".$_SESSION['crso_relation_id']." ";
		}
		return "";
	}

	public function getSQLGroupBy()
	{
		return " GROUP BY RE.PRODUCT_ID, YEAR(RE.EXPORT_DATE), RE.PARTICIPANT_RELATION_ID ";
	}

	public function getSQLOrderBy()
	{
		return " ORDER BY RE.PRODUCT_ID, YEAR(RE.EXPORT_DATE) DESC, R1.NAME ";
	}

	public function tfHideColumns()
	{
		$rv = array();
		if(!isset($_GET['groupby']))
			return $rv;
		if($_SESSION['session_group_group_id'] == GROUP_CRSO) {
			$rv[] = 'CRSO';
		}
		if($_GET['groupby']=='YEAR')
			$rv[] = 'YEAR';
		return $rv;
	}

	public function tfColumnModifier()
	{
		return array('QUANTITY'=>'style="text-align:right;"', 'WEIGHT'=>'style="text-align:right;"');
	}

	public function tfValueProcessor($args=false)
	{
		if(!$args)
			return array($this,'tfValueProcessor');

		require_once(ELC_CLASSES.'ELCListBuilder.php');
		$list = new ELCDefaultList(IWRegistry::registry('db'),IWRegistry::registry('langIf'));
		if(!empty($_POST['export']))
			$list->setShowType(LIST_BUTTON_CSV_EXPORT);
		switch($args['head'])
		{
			case 'COMPANY_NAME':
			case 'CRSO_NAME':
				return $list->tfLongNames($args);
			break;
			case 'QUANTITY':
			case 'WEIGHT':
				return $list->tfValueProcessorQuantityNoDecimals($args);
			break;
			case 'ACTIVE':
				return $list->tfBooleanIcon($args);
		}
		return $args['value'];
	}

	public function registerFields(IWCustomQueryFactory $cq)
	{
		$cq->setField('NAME','Company',FIELD_INPUT_TEXT,false,false,'R1');
		$cq->setField('YEAR(EXPORT_DATE)','Year',FIELD_INPUT_DROPDOWN,$this->getYearList(),false);

		if($_SESSION['session_group_group_id'] != GROUP_CRSO) {
			$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
				$this->getFilter('filterCRSO'),
				'CRSO_RELATION_ID','CRSO','RE');
		}

		if(count($this->products)!=1)
		{
			$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
				'SELECT PRODUCT_ID as VAL, `NAME` as DISP FROM TBL_PRODUCT '.
					(count($this->products)?'WHERE PRODUCT_ID IN ('.implode(',',$this->products).')':'').' ORDER BY DISP',
				'PRODUCT_ID','Product','RE'
				);
		}

		$cq->getFilterFromFieldInTable(IWRegistry::registry('db'),
			"SELECT `ACTIVE` as VAL, (CASE `ACTIVE` WHEN 1 THEN 'Active' ELSE 'Inactive' END) as DISP FROM TBL_CRSO_PARTICIPANT_PRODUCT ".
				"GROUP BY `ACTIVE`",
			'ACTIVE','Active','cpp'
			);

	}
	private function getYearList()
	{
		$date = currentDate();
		$year = $date->getYear();
		if($year == WEEE_APPLICATION_START_YEAR)
			return array($year=>$year);

		$rv = array();
		for($i = WEEE_APPLICATION_START_YEAR;$i<=$year;$i++)
			$rv[$i] = $i;
		return $rv;
	}

	private function setSettings()
	{
		$product_ids = array();
		$pids = isset($_REQUEST['PRODUCT_ID'])?$_REQUEST['PRODUCT_ID']:false;
		$crso_id = false;
		if((isset($_REQUEST['CRSO_RELATION_ID']))&&count($_REQUEST['CRSO_RELATION_ID']==1))
			$crso_id = current($_REQUEST['CRSO_RELATION_ID']);
		// Check this last, make sure they cannot access other CRSO's
		$crso_id = ($_SESSION['session_group_group_id']==GROUP_CRSO)?$_SESSION['crso_relation_id']:$crso_id;
		if($crso_id)
		{
			require_once(ELC_CLASSES.'CRSOProduct.php');
			$cp = new CRSOProduct(IWRegistry::registry('db'));
			$q = $cp->getProducts($crso_id);
			while($r = $q->fetch())
				$product_ids[] = $r->PRODUCT_ID;
		}
		$this->selectedProducts = $pids?$pids:$product_ids;
		$this->products = $product_ids;
		$this->crso = $crso_id;
	}
}
?>
