<?php
/**
 * This overview will also be used to re-enable users.
 *
 * @package IW
 * @subpackage CustomQuery
 * @since 19-sep-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class WEEECustomQueryUserOverview extends IWCustomQueryObject
{
	public $limit = 25;
	/**
	 *
	 */
	public function isPrivileged()
	{
		require_once(IWARE.'group.php');
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
			case GROUP_ACCOUNTANT:
				return true;
		}
		return false;
	}
	/**
	 *
	 */
	public function getQuery()
	{
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_SYSADMIN:
				$groups = GROUP_CRSO.",".GROUP_PARTICIPANT.",".GROUP_ACCOUNTANT;
				$sql = "SELECT U.USER_ID, U.USER_NAME, GU.GROUP_ID as UNIVERSE,
						R.`NAME` as COMPANY_NAME, U.USER_ACTIVE, '' as SWITCH_STATUS
					FROM TBL_USER as U
					INNER JOIN TBL_GROUP_USER as GU
						ON U.USER_ID = GU.USER_ID
							AND GU.GROUP_ID IN (".$groups.")
					INNER JOIN TBL_RELATION_USER as RU
						ON U.USER_ID = RU.USER_ID
					INNER JOIN TBL_RELATION as R
						ON RU.RELATION_ID = R.RELATION_ID";
			break;
			case GROUP_ACCOUNTANT:
				$sql = "SELECT USER_ID, USER_NAME, UNIVERSE, COMPANY_NAME, USER_ACTIVE, ACCOUNTANT_RELATION_ID
					FROM (
					(
						SELECT U.USER_ID, U.USER_NAME, GU.GROUP_ID as UNIVERSE,
							R.`NAME` as COMPANY_NAME, U.USER_ACTIVE, AC.ACCOUNTANT_RELATION_ID
						FROM TBL_USER as U
						INNER JOIN TBL_GROUP_USER as GU
							ON U.USER_ID = GU.USER_ID
								AND GU.GROUP_ID = ".GROUP_CRSO."
						INNER JOIN TBL_RELATION_USER as RU
							ON U.USER_ID = RU.USER_ID
						INNER JOIN TBL_RELATION as R
							ON RU.RELATION_ID = R.RELATION_ID
						INNER JOIN TBL_ACCOUNTANT_CRSO AC
							ON AC.CRSO_RELATION_ID=R.RELATION_ID
					)
					UNION
					(
						SELECT U.USER_ID, U.USER_NAME, GU.GROUP_ID as UNIVERSE,
							R.`NAME` as COMPANY_NAME, U.USER_ACTIVE, AC.ACCOUNTANT_RELATION_ID
						FROM TBL_USER as U
						INNER JOIN TBL_GROUP_USER as GU
							ON U.USER_ID = GU.USER_ID
								AND GU.GROUP_ID = ".GROUP_PARTICIPANT."
						INNER JOIN TBL_RELATION_USER as RU
							ON U.USER_ID = RU.USER_ID
						INNER JOIN TBL_RELATION as R
							ON RU.RELATION_ID = R.RELATION_ID
						INNER JOIN TBL_CRSO_PARTICIPANT CP
							ON CP.PARTICIPANT_RELATION_ID = R.RELATION_ID
						INNER JOIN TBL_ACCOUNTANT_CRSO AC
							ON AC.CRSO_RELATION_ID = CP.CRSO_RELATION_ID
					)
					) AS T
				";
				$this->rowCntQuery = "SELECT COUNT(*) AS CNT FROM (".$sql.") AS TT";
			break;
		}

		return $sql;
	}
	/**
	 *
	 */
	public function getSQLOrderBy()
	{
		if($return = parent::getSQLOrderBy())
			return $return;
		return " ORDER BY USER_ACTIVE,COMPANY_NAME,USER_NAME";
	}
	/**
	 *
	 */
	public function getWhere()
	{
		$w = " AND USER_NAME != '' ";
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_ACCOUNTANT:
				$w .= " AND ACCOUNTANT_RELATION_ID = ".$_SESSION['accountant_relation_id']." ";
		}
		return $w;
	}
	/**
	 *
	 */
	public function tfHideColumns()
	{
		$rv = array();
		if($_SESSION['session_group_group_id'] == GROUP_ACCOUNTANT)
			$rv[] = 'ACCOUNTANT_RELATION_ID';
		return $rv;
	}
	/**
	 *
	 */
	public function tfValueProcessor($args=false)
	{
		if(!$args)
			return array($this,'tfValueProcessor');

		require_once(ELC_CLASSES.'ELCListBuilder.php');
		$list = new ELCDefaultList(IWRegistry::registry('db'),IWRegistry::registry('langIf'));
		if(!empty($_POST['export']))
			$list->setShowType(LIST_BUTTON_CSV_EXPORT);

		switch($args['head'])
		{
			case 'UNIVERSE':
				if($args['value']==GROUP_ACCOUNTANT) // For sysadmin
					return 'Accountant';
				elseif($args['value']==GROUP_CRSO)
					return 'CRSO';
				return 'Participant';
			break;
			case 'USER_ACTIVE':
				if(!empty($_POST['export']))
					return $args['value']?'true':'false';
				$img = ($args['value']) ? 'true.gif' : 'false.gif';
    			return sprintf('<img id="img_status_%s" src="/%simages/i-w/%s" />',
    				$args['row']['USER_ID'],
    				SITE_PATH,
    				$img);
			break;
			case 'USER_NAME':
			case 'COMPANY_NAME':
				return $list->tfLongNames($args);
			break;
			case 'SWITCH_STATUS':
				return sprintf('<input id="btn_status_%s" type="button" value="%s" onclick="switchUserStatus(\'%s\');" />
					<input type="hidden" id="status_%s" name="status_%s" value="%s" />',
					$args['row']['USER_ID'],
					IWRegistry::registry('langIf')->getText((($args['row']['USER_ACTIVE']==1)?'Deactivate User':'Activate User')),
					$args['row']['USER_ID'],
					$args['row']['USER_ID'],
					$args['row']['USER_ID'],
					$args['row']['USER_ACTIVE']
					);
			break;
		}
		return $args['value'];
	}
	/**
	 *
	 */
	public function registerFields(IWCustomQueryFactory $cq)
	{
		$cq->setField('USER_NAME','User Name',FIELD_INPUT_TEXT,false,false,'');
		$cq->setField('COMPANY_NAME','Company',FIELD_INPUT_TEXT,false,false,'');
	}
	/**
	 *
	 */
	public function tfAllowSorting()
	{
		return true;
	}
}
?>
