<?php

require_once(ELC_LISTS.'DeEarAbstractList.php');

class DeEARDeclarationList extends DeEarAbstractList {
	var $handle = 'DeEARDeclarationList';

	var $exportTitle = 'DeclarationList';

	function _setWhere() {

		parent::_setWhere();

		$this -> _addWhere('CRSO_RELATION_ID', array('field' => 'D.`CRSO_RELATION_ID`', 'callback' => array($this, 'getCRSOSelect'), 'required' => false, 'related' => array(), 'groupBy' => ''));
		$this -> _addWhere('EXPORT_DATE_YEAR', array('field' => 'YEAR(D.EXPORT_DATE)', 'callback' => array($this, 'getExportYear'), 'groupBy' => '', 'required' => false));
		$this -> _addWhere('EXPORT_DATE_MONTH', array('field' => 'MONTH(D.EXPORT_DATE)', 'callback' => array($this, 'getExportMonth'), 'groupBy' => '', 'required' => false, 'useinquery' => $this -> _month_useinquery));
		$this -> _addWhere('PARTICIPANT_RELATION_ID', array('field' => 'D.PARTICIPANT_RELATION_ID', 'callback' => array($this, 'getParticipantSelect'), 'related' => array(''), 'groupBy' => ''));
		$this -> _addWhere('PRODUCT_ID', array('field' => 'D.PRODUCT_ID', 'callback' => array($this, 'getProductSelect')));
		$this -> _addWhere('IS_EXPORTED', array('field' => 'D.IS_EXPORTED', 'callback' => array($this, 'getIsExported')));

		if ($this->_has_category)
			$this -> _addWhere('PRODUCT_CATEGORY_ID', array('field' => 'DL.PRODUCT_CATEGORY_ID', 'callback' => array($this, 'getProductCategory'), 'groupBy' => '', 'required' => false, 'useinquery' => $this -> _category_useinquery));

		if ($this->_has_dimension)
			$this -> _addWhere('CPD_ID', array('field' => 'DL.CPD_ID', 'callback' => array($this, 'getDimension'), 'groupBy' => '', 'required' => false, 'useinquery' => $this -> _dimension_useinquery));

	}

	function setQuery() {

		$this -> crso_id = $this -> getCRSOSelect('setValue');

		$this -> _query = "
			SELECT
				YEAR(D.EXPORT_DATE) AS EXPORT_YEAR,
				";
		if ($this -> _month_useinquery)
			$this -> _query .= "MONTH(D.EXPORT_DATE) AS EXPORT_MONTH, ";


		$this -> _query .= "
				R.RELATION_ID AS PARTICIPANT_RELATION_ID,
				R.NAME AS PARTICIPANT_NAME,
				CP2.PRODUCER_REG_NO,
				";

		if ($this ->_category_useinquery)
			$this ->_query .= "PC.PRODUCT_CATEGORY_ID as PRODUCT_CATEGORY_ID, PC.NAME as PRODUCT_CATEGORY, ";

		if ($this ->_dimension_useinquery)
			$this ->_query .= "CPD.CPD_ID as CPD_ID, CPD.NAME as DIMENSION, ";

		$this -> _query .= "
				SUM(DL.QUANTITY) AS QUANTITY,
				SUM(DL.WEIGHT) AS WEIGHT,
				SUM(DL.WEIGHT) / SUM(DL.QUANTITY) AS AVG_WEIGHT,
				SUM(DL.WEIGHT_DEC) AS WEIGHT_DEC,
				SUM(DL.WEIGHT_DEC) / SUM(DL.QUANTITY) AS AVG_WEIGHT_DEC,
				SUM(DL.EXPORTED_AMOUNT) AS AMOUNT,
				D.EXPORT_DATE

			FROM TBL_DECLARATION D
			INNER JOIN TBL_DECLARATION_LINE DL ON D.DECLARATION_ID = DL.DECLARATION_ID
			INNER JOIN TBL_RELATION R ON D.PARTICIPANT_RELATION_ID = R.RELATION_ID

			INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT as CP
				ON R.RELATION_ID = CP.PARTICIPANT_RELATION_ID
				AND CP.CRSO_RELATION_ID = D.CRSO_RELATION_ID
				AND CP.PRODUCT_ID = D.PRODUCT_ID

			INNER JOIN TBL_CRSO_PARTICIPANT as CP2
				ON R.RELATION_ID = CP2.PARTICIPANT_RELATION_ID
				AND CP2.CRSO_RELATION_ID = D.CRSO_RELATION_ID

			LEFT JOIN TBL_PRODUCT_CATEGORY PC ON DL.PRODUCT_CATEGORY_ID = PC.PRODUCT_CATEGORY_ID
			LEFT JOIN TBL_CRSO_PRODUCT_DIMENSION CPD ON DL.CPD_ID = CPD.CPD_ID

				";

		$this -> _groupByArray = array('EXPORT_YEAR', 'PARTICIPANT_RELATION_ID');

			if (isset($_REQUEST['EXPORT_DATE_MONTH']) && $_REQUEST['EXPORT_DATE_MONTH'] != 'grouped') {
			$this -> _groupByArray[] = 'EXPORT_MONTH';
		}


		$product_category_id = isset($_REQUEST['PRODUCT_CATEGORY_ID']) ? $_REQUEST['PRODUCT_CATEGORY_ID'] : '';
		if ($product_category_id != 'grouped') {
			$this -> _groupByArray[] = 'DL.PRODUCT_CATEGORY_ID';
		}

		$cpd_id = isset($_REQUEST['CPD_ID']) ? $_REQUEST['CPD_ID'] : '';
		if ($cpd_id != 'grouped') {
			$this -> _groupByArray[] = 'DL.CPD_ID';
		}

		$this -> _orderBy = "ORDER BY EXPORT_YEAR, ";

		if ($this -> _month_useinquery)
			$this -> _orderBy .= " EXPORT_MONTH, ";


		$this -> _orderBy .= " PARTICIPANT_NAME ";

		if ($this -> _category_useinquery)
			$this -> _orderBy .= ", PRODUCT_CATEGORY ";
		if ($this -> _dimension_useinquery)
			$this -> _orderBy .= ", DIMENSION ";

	}

	function getColumnModifier() {
		$r = 'style="text-align:right;"';
		return array('QUANTITY' => $r, 'WEIGHT' => $r, 'WEIGHT_DEC' => $r, 'AMOUNT' => $r, 'AVG_WEIGHT' => $r, 'AVG_WEIGHT_DEC' => $r);
	}

	function tfHeaderProc() {
		return array('getAr' => array('QUANTITY', 'WEIGHT', 'WEIGHT_DEC', 'AMOUNT', 'AVG_WEIGHT', 'AVG_WEIGHT_DEC'));

	}

	function getProcessColumns() {
		$rv = array(
			'tfParticipantName' => array('PARTICIPANT_NAME'),
			'tfValueProcessorWeightSum' => array('WEIGHT', 'WEIGHT_DEC'),
			'tfValueProcessorAmountSum' => array('AMOUNT'),
			'tfValueProcessorWeightAvg' => array('AVG_WEIGHT', 'AVG_WEIGHT_DEC'),

			'tfMonth' => array('EXPORT_MONTH'),
			'tfQuantity' => array('QUANTITY'),

			'tfProductCategory' => array('PRODUCT_CATEGORY'),
			'tfDimension' => array('DIMENSION'),
		);

		return $rv;
	}


	function tfParticipantName($args) {

		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		if ($this -> _showType != LIST_HTML_OUTPUT)
			return $args['value'];

		return '<a href="#" onclick=" ' . '$$(\'PARTICIPANT_RELATION_ID\').value=' . $args['row']['PARTICIPANT_RELATION_ID'] . '; ' . '$$(\'EXPORT_DATE_MONTH\').value=\'\'; ' . '_LoadOverview();">' . $args['value'] . '</a>';
	}

	function tfQuantity($args) {

		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);

		$rel_type = '';
		switch($_SESSION['session_group_company_id']) {
			case GROUP_ACCOUNTANT :
				$rel_type = 'accountant';
				break;
			case GROUP_CRSO :
				$rel_type = 'crso';
				break;
			default :
				trigger_error('not implemented', E_USER_ERROR);
		}

		$this -> _maxQuantity += $args['value'];

		if ($this -> _showType != LIST_HTML_OUTPUT)
			return locale_nf($args['value'], 0);

		return '<a href="/scripts/elc/' . $rel_type . '/InterfaceDeclarationExportList.php' . '?PARTICIPANT_RELATION_ID=' . $args['row']['PARTICIPANT_RELATION_ID'] . '&YEAR=' . $args['row']['EXPORT_YEAR'] . '&EXPORT_DATE=' . $args['row']['EXPORT_DATE'] . '">' . locale_nf($args['value'], 0) . '</a>';
	}

	function getAr($args) {
		return 'class="quantity"';
	}

	function getButtons() {

		return array_merge($this -> _getCSVButton(), $this -> _getXMLButton());
	}

	function getIsExported($rv = 'setValue') {
		$filter = '';
		$setValue = '2';
		$selectedValue = '';
		return $$rv;
	}

	function getExportYear($rv = 'setValue') {

		$setValue = isset($_REQUEST['EXPORT_DATE_YEAR']) ? $_REQUEST['EXPORT_DATE_YEAR'] : false;
		$selectedValue = '';

		if (isset($$rv))
			return $$rv;

		if (isset($_REQUEST['CRSO_RELATION_ID'])) {

			$q = $this -> db -> query('SELECT DISTINCT YEAR(EXPORT_DATE) AS YEAR FROM TBL_DECLARATION WHERE IS_EXPORTED=2 AND CRSO_RELATION_ID = ? AND PRODUCT_ID = ? ORDER BY `YEAR` ASC');
			$q -> bind_values($_REQUEST['CRSO_RELATION_ID'], $_SESSION['selected_product_id']);
			$q -> execute();

			$select = new Form_Select('EXPORT_DATE_YEAR');
			$select -> addOption($this -> getText('All'), '', false);
			while ($r = $q -> fetch()) {
				$select -> addOption($r -> YEAR, $r -> YEAR, false);
				$lastValue = $r -> YEAR;
			}

			if ($setValue) {
				$select -> setSelectedValue($setValue);
				$selectedValue = $setValue;
			}

			$filter = $select -> getHtml();

		} else
			$filter = '';

		return $$rv;
	}

	function getExportMonth($rv = 'setValue') {

		$setValue = isset($_REQUEST['EXPORT_DATE_MONTH']) ? $_REQUEST['EXPORT_DATE_MONTH'] : false;
		$selectedValue = '';

		if (isset($$rv))
			return $$rv;

		if (isset($_REQUEST['CRSO_RELATION_ID'])) {

			$q = $this -> db -> query('SELECT DISTINCT MONTH(EXPORT_DATE) AS MONTH FROM TBL_DECLARATION WHERE IS_EXPORTED=2 AND CRSO_RELATION_ID = ? AND PRODUCT_ID = ? ORDER BY `MONTH` ASC');
			$q -> bind_values($_REQUEST['CRSO_RELATION_ID'], $_SESSION['selected_product_id']);
			$q -> execute();

			$select = new Form_Select('EXPORT_DATE_MONTH');
			$select -> addOption($this -> getText('All'), '', false);
			$select -> addOption($this -> getText('Grouped'), 'grouped', false);
			while ($r = $q -> fetch()) {
				$select -> addOption($this -> tfMonth(array('value' => $r -> MONTH)), $r -> MONTH, false);
				$lastValue = $r -> MONTH;
			}

			if ($setValue) {
				$selectedValue = $setValue;
				$select -> setSelectedValue($setValue);
			}

			$filter = $select -> getHtml();

		} else
			$filter = '';

		return $$rv;
	}




	function tfSortingHat() {
		return array('QUANTITY' => "'Int'", 'WEIGHT' => "'Int'", 'AMOUNT' => "'Int'", 'PARTICIPANT_NAME' => "'StringNoTags'", );
	}


	function setEmptyText() {
		return $this -> langIf -> getText('No declarations data for the current selection');
	}



}
?>
