<?php
require_once(ELC_LISTS.'DeEarAbstractList.php');


class DeEARExportList extends DeEarAbstractList {

	var $handle = 'DeEARExportList';
	var $exportTitle = 'ExportList';

	protected $_country_useinquery = true;

	function _setWhere() {

		parent::_setWhere();

		$this -> langIf -> gettext('REFUND_COUNT');
		$this -> langIf -> gettext('REFUND_REQUESTER_NAME');

		if (isset($_REQUEST['preset'])) {
			$_REQUEST['EXPORT_COUNTRY_ID'] = '__in_europe__';
		}

		if (isset($_REQUEST['EXPORT_COUNTRY_ID']) && ($_REQUEST['EXPORT_COUNTRY_ID'] == '__in_europe__' || $_REQUEST['EXPORT_COUNTRY_ID'] == '__out_europe__'))
			$this -> _country_useinquery = false;

		$this -> _addWhere('CRSO_RELATION_ID', array('field' => 'RR.`CRSO_RELATION_ID`', 'callback' => array($this, 'getCRSOSelect'), 'required' => false, 'related' => array(), 'groupBy' => ''));
		$this -> _addWhere('EXPORT_DATE_YEAR', array('field' => 'YEAR(RE.EXPORT_DATE)', 'callback' => array($this, 'getExportYear'), 'groupBy' => '', 'required' => false));
		$this -> _addWhere('EXPORT_DATE_MONTH', array('field' => 'MONTH(RE.EXPORT_DATE)', 'callback' => array($this, 'getExportMonth'), 'groupBy' => '', 'required' => false, 'useinquery' => $this -> _month_useinquery));
		$this -> _addWhere('PARTICIPANT_RELATION_ID', array('field' => 'RL.PRODUCER_RELATION_ID', 'callback' => array($this, 'getParticipantSelect'), 'related' => array(''), 'groupBy' => ''));
		$this -> _addWhere('PRODUCT_ID', array('field' => 'RR.PRODUCT_ID', 'callback' => array($this, 'getProductSelect')));
		$this -> _addWhere('IS_EXPORTED', array('field' => 'RE.IS_EXPORTED', 'callback' => array($this, 'getIsExported')));
		$this -> _addWhere('EXPORT_COUNTRY_ID', array('field' => 'RR.EXPORT_COUNTRY_ID', 'callback' => array($this, 'getCountrySelect'), 'groupBy' => '', 'useinquery' => $this -> _country_useinquery));

		if ($this->_has_category)
			$this -> _addWhere('PRODUCT_CATEGORY_ID', array('field' => 'RL.PRODUCT_CATEGORY_ID', 'callback' => array($this, 'getProductCategory'), 'groupBy' => '', 'required' => false, 'useinquery' => $this -> _category_useinquery));

		$this -> _addWhere('SHOW_REFUND_DETAILS', array('callback' => array($this, 'getShowRefundDetails'), 'useinquery' => false));

	}

	function _getWhere() {

		$where = parent::_getWhere();

		if (isset($_REQUEST['EXPORT_COUNTRY_ID'])) {

			if ($_REQUEST['EXPORT_COUNTRY_ID'] == '__in_europe__') {
				$where .= ' AND RR_C.REGION_ID = 1';
			} elseif ($_REQUEST['EXPORT_COUNTRY_ID'] == '__out_europe__') {
				$where .= ' AND ISNULL(RR_C.REGION_ID)';
			}
		}

		return $where;

	}

	function setQuery() {
		$this -> crso_id = $this -> getCRSOSelect('setValue');

		$this -> _query = "
			SELECT
				YEAR(RE.EXPORT_DATE) AS EXPORT_YEAR,
				";
		if ($this -> _month_useinquery)
			$this -> _query .= "MONTH(RE.EXPORT_DATE) AS EXPORT_MONTH, ";

		$this -> _query .= "
				R.RELATION_ID AS PARTICIPANT_RELATION_ID,
				R.NAME AS PARTICIPANT_NAME,
				CP2.PRODUCER_REG_NO,

		";

		$this -> _query .= "
			RL.REFUND_REQUEST_ID,
			RR_R.NAME AS REFUND_REQUESTER_NAME,
			RR_C.NAME AS COUNTRY,
			RR.REFUND_EXPORT_DATE,
			RR.EXPORT_DOCUMENT_ID AS EXPORT_DOCUMENT,
			RR.IMPORTER_COMPANY_NAME,
			RR.IMPORTER_EMAIL,
			RR.OFFICE,
			RR.IDENTIFICATION_NO,
		";

		if ($this ->_category_useinquery)
			$this ->_query .= "PC.PRODUCT_CATEGORY_ID, PC.NAME as PRODUCT_CATEGORY, ";

		$this -> _query .= "
			B.NAME AS BRAND,
		";
		$this -> _query .= "
			COUNT(DISTINCT RR.REFUND_REQUEST_ID) as REFUND_COUNT,
		";
		$this -> _query .= "
				SUM(RL.QUANTITY) AS QUANTITY,
				SUM(RL.WEIGHT) AS WEIGHT,
				SUM(RL.WEIGHT_DEC) AS WEIGHT_DEC,
				SUM(RL.WEIGHT) / SUM(RL.QUANTITY) AS AVG_WEIGHT,
				SUM(RL.WEIGHT_DEC) / SUM(RL.QUANTITY) AS AVG_WEIGHT_DEC,
				SUM(RL.EXPORTED_AMOUNT) AS AMOUNT

			FROM `TBL_REFUND_REQUEST` RR
			INNER JOIN `TBL_REFUND_REQUEST_LINE` RL ON RL.REFUND_REQUEST_ID = RR.REFUND_REQUEST_ID
			INNER JOIN `TBL_REFUND_EXPORT` RE ON RE.REFUND_EXPORT_ID = RR.REFUND_EXPORT_ID
			LEFT JOIN TBL_PRODUCT_CATEGORY PC ON RL.PRODUCT_CATEGORY_ID = PC.PRODUCT_CATEGORY_ID
			LEFT JOIN TBL_BRAND B ON RL.BRAND_ID = B.BRAND_ID

			-- original producer relation
			INNER JOIN TBL_RELATION R ON RL.PRODUCER_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT as CP
				ON R.RELATION_ID = CP.PARTICIPANT_RELATION_ID
				AND CP.CRSO_RELATION_ID = RR.CRSO_RELATION_ID
				AND CP.PRODUCT_ID = RR.PRODUCT_ID

			-- refund requester participant relation
			INNER JOIN TBL_RELATION RR_R ON RR.PARTICIPANT_RELATION_ID = RR_R.RELATION_ID

			-- export country
			INNER JOIN TBL_COUNTRY RR_C ON RR.EXPORT_COUNTRY_ID = RR_C.COUNTRY_ID

			-- producer registration nr
			INNER JOIN TBL_CRSO_PARTICIPANT as CP2
				ON R.RELATION_ID = CP2.PARTICIPANT_RELATION_ID
				AND CP2.CRSO_RELATION_ID = RR.CRSO_RELATION_ID

			";

		$this -> _groupByArray = array('EXPORT_YEAR', 'PARTICIPANT_RELATION_ID');

		if (isset($_REQUEST['EXPORT_DATE_MONTH']) && $_REQUEST['EXPORT_DATE_MONTH'] != 'grouped') {
			$this -> _groupByArray[] = 'EXPORT_MONTH';
		}

		$clearance = true;
		if($_SESSION['session_group_group_id']==GROUP_CRSO) {
			$ru = new RelationUser($this->db);
			$ru->set($_SESSION['session_user_id'], $_SESSION['crso_relation_id']);
			if(!$ru->hasClearance(CRSOClearanceRefundDetails))
				$clearance = false;
		}

		if ($clearance && isset($_REQUEST['SHOW_REFUND_DETAILS']) && $_REQUEST['SHOW_REFUND_DETAILS']) {
			$this -> _groupByArray[] = 'RL.REFUND_REQUEST_ID';
			$this -> _groupByArray[] = 'RL.LINE_INDEX';

			$this -> _hiddenCols[] = 'REFUND_COUNT';

		} else {

			$this -> _hiddenCols[] = 'REFUND_REQUEST_ID';
			$this -> _hiddenCols[] = 'REFUND_REQUESTER_NAME';
			$this -> _hiddenCols[] = 'COUNTRY';
			$this -> _hiddenCols[] = 'BRAND';
			$this -> _hiddenCols[] = 'REFUND_EXPORT_DATE';
			$this -> _hiddenCols[] = 'IMPORTER_COMPANY_NAME';
			$this -> _hiddenCols[] = 'IMPORTER_EMAIL';
			$this -> _hiddenCols[] = 'CUSTOM_ID_NO';
			$this -> _hiddenCols[] = 'CUSTOMS_OFFICE';
			$this -> _hiddenCols[] = 'IDENTIFICATION_NO';
			$this -> _hiddenCols[] = 'OFFICE';
			$this -> _hiddenCols[] = 'EXPORT_DOCUMENT';
			$this -> _hiddenCols[] = 'PRODUCT_CATEGORY';

		}

		// Show all columns if we do not have a specific country filter enabled
		if(isset($_REQUEST['EXPORT_COUNTRY_ID']) && $_REQUEST['EXPORT_COUNTRY_ID']) {
			if($this -> isEUCountry()) {
				$this -> _hiddenCols[] = 'IDENTIFICATION_NO';
				$this -> _hiddenCols[] = 'OFFICE';
				$this -> _hiddenCols[] = 'EXPORT_DOCUMENT';
			} else {
				$this -> _hiddenCols[] = 'IMPORTER_COMPANY_NAME';
				$this -> _hiddenCols[] = 'IMPORTER_EMAIL';
			}
		}


		$this -> _orderBy = "ORDER BY EXPORT_YEAR, ";

		if ($this -> _month_useinquery)
			$this -> _orderBy .= " EXPORT_MONTH, ";

		$this -> _orderBy .= " PARTICIPANT_NAME";
	}

	function isEUCountry() {
		if($_REQUEST['EXPORT_COUNTRY_ID'] == '__in_europe__') {
			return true;
		}
		if($_REQUEST['EXPORT_COUNTRY_ID'] == '__out_europe__') {
			return false;
		}

		$sql = "SELECT REGION_ID FROM TBL_COUNTRY WHERE COUNTRY_ID = ?";
		$q = $this->db->query($sql);
		$q->bind_values($_REQUEST['EXPORT_COUNTRY_ID']);
		$q->execute();
		$row = $q->fetch();
		return ((int)$row->REGION_ID) == 1;

	}

	function getColumnModifier() {

		$r = 'style="text-align:right;"';
		return array('QUANTITY' => $r, 'WEIGHT' => $r, 'AMOUNT' => $r, 'AVG_WEIGHT' => $r, 'WEIGHT_DEC' => $r, 'AVG_WEIGHT_DEC' => $r);
	}

	function tfHeaderProc() {
		return array('getAr' => array('QUANTITY', 'WEIGHT', 'AMOUNT', 'AVG_WEIGHT', 'WEIGHT_DEC', 'AVG_WEIGHT_DEC'));
	}

	function tfRefundId($args) {
		if ($this -> _showType != LIST_HTML_OUTPUT)
			return $args['value'];

		$rel_type = '';
		switch($_SESSION['session_group_company_id']) {
			case GROUP_ACCOUNTANT :
				$rel_type = 'accountant';
				break;
			case GROUP_CRSO :
				$rel_type = 'crso';
				break;
			default :
				trigger_error('not implemented', E_USER_ERROR);
		}

		return '<a href="#" onclick="window.open (\'/' . SITE_PATH . 'scripts/elc/' . $rel_type . '/InterfaceExportForm.php?refund[id]=' . $args['value'] . '&process=print\', \'\', \'toolbar=0,scrollbars=1,location=0,statusbar=0,menubar=0,resizable=1,width=800,height=600\'); return false;">' . $args['value'] . '</a>';

	}

	function tfDocumentLink($args) {
		$args['hidename'] = true;
		// Only show link if CRSO user has access to declaration details.
		if($_SESSION['session_group_group_id']==GROUP_CRSO) {
			$ru = new RelationUser($this->db);
			$ru->set($_SESSION['session_user_id'], $_SESSION['crso_relation_id']);
			if(!$ru->hasClearance(CRSOClearanceDocumentAccess))
				return $this->tfBooleanIcon($args);
		}

		return parent::tfDocumentLink($args);
	}

	function tfRefundCount($args) {
		if (!is_numeric($args['value']) || $this -> _showType != LIST_HTML_OUTPUT)
			return $args['value'];

		return '<a href="#" onclick=" ' . '$$(\'PARTICIPANT_RELATION_ID\').value=' . $args['row']['PARTICIPANT_RELATION_ID'] . '; ' . '$$(\'SHOW_REFUND_DETAILS\').value=\'1\'; ' . '_LoadOverview();">' . $args['value'] . '</a>';

	}

	function getAr() {
		return 'class="quantity"';
	}

	function getButtons() {
		return array_merge($this -> _getCSVButton(), $this -> _getXMLButton());
	}

	function _getZipButton()
	{
		return array('ZIP Export'=>array('onclick'=>"exportView('zip');")); // FIXME
	}

	function getShowRefundDetails($rv = 'setValue') {
		$value = isset($_REQUEST['SHOW_REFUND_DETAILS']) && $_REQUEST['SHOW_REFUND_DETAILS'] ? 1 : 0;
		$checked = $value ? 'checked="checked"' :'';
		$filter = '<label for="idShowRefundDetails">' .$this->langIf->gettext('Show details'). '</label><input id="idShowRefundDetails" type="checkbox" name="SHOW_REFUND_DETAILS" ' . $checked . ' onclick="this.checked ? $$(\'SHOW_REFUND_DETAILS\').value=1 : $$(\'SHOW_REFUND_DETAILS\').value=0;"> ';
		$setValue = '0';
		$selectedValue = $value;
		return $$rv;
	}

	function getIsExported($rv = 'setValue') {
		$filter = '';
		$setValue = '2';
		$selectedValue = '';
		return $$rv;
	}

	function getExportYear($rv = 'setValue') {

		$setValue = isset($_REQUEST['EXPORT_DATE_YEAR']) ? $_REQUEST['EXPORT_DATE_YEAR'] : false;
		$selectedValue = '';

		if (isset($$rv))
			return $$rv;

		if (isset($_REQUEST['CRSO_RELATION_ID'])) {

			$q = $this -> db -> query('SELECT DISTINCT YEAR(EXPORT_DATE) AS YEAR FROM TBL_DECLARATION WHERE IS_EXPORTED=2 AND CRSO_RELATION_ID = ? AND PRODUCT_ID = ? ORDER BY `YEAR` ASC');
			$q -> bind_values($_REQUEST['CRSO_RELATION_ID'], $_SESSION['selected_product_id']);
			$q -> execute();

			$select = new Form_Select('EXPORT_DATE_YEAR');
			$select -> addOption($this -> getText('All'), '', false);
			while ($r = $q -> fetch()) {
				$select -> addOption($r -> YEAR, $r -> YEAR, false);
				$lastValue = $r -> YEAR;
			}

			if ($setValue) {
				$select -> setSelectedValue($setValue);
				$selectedValue = $setValue;
			}

			$filter = $select -> getHtml();

		} else
			$filter = '';

		return $$rv;
	}

	function getExportMonth($rv = 'setValue') {

		$setValue = isset($_REQUEST['EXPORT_DATE_MONTH']) ? $_REQUEST['EXPORT_DATE_MONTH'] : false;
		$selectedValue = '';

		if (isset($$rv))
			return $$rv;

		if (isset($_REQUEST['CRSO_RELATION_ID'])) {

			$q = $this -> db -> query('SELECT DISTINCT MONTH(EXPORT_DATE) AS MONTH FROM TBL_DECLARATION WHERE IS_EXPORTED=2 AND CRSO_RELATION_ID = ? AND PRODUCT_ID = ? ORDER BY `MONTH` ASC');
			$q -> bind_values($_REQUEST['CRSO_RELATION_ID'], $_SESSION['selected_product_id']);
			$q -> execute();

			$select = new Form_Select('EXPORT_DATE_MONTH');
			$select -> addOption($this -> getText('All'), '', false);
			$select -> addOption($this -> getText('Grouped'), 'grouped', false);
			while ($r = $q -> fetch()) {

				$select -> addOption($this -> tfMonth(array('value' => $r -> MONTH)), $r -> MONTH, false);
				$lastValue = $r -> MONTH;
			}

			if ($setValue) {
				$selectedValue = $setValue;
				$select -> setSelectedValue($setValue);
			}

			$filter = $select -> getHtml();

		} else
			$filter = '';

		return $$rv;
	}

	function getCountrySelect($rv = 'setValue') {

		$setValue = isset($_REQUEST['EXPORT_COUNTRY_ID']) ? $_REQUEST['EXPORT_COUNTRY_ID'] : false;
		$selectedValue = '';

		if (isset($$rv))
			return $$rv;

		$q = $this -> db -> query('
			SELECT DISTINCT C.COUNTRY_ID as COUNTRY_ID, C.NAME AS COUNTRY_NAME
			FROM TBL_COUNTRY C
			INNER JOIN TBL_REFUND_REQUEST R
			ON C.COUNTRY_ID = R.EXPORT_COUNTRY_ID

		');
		$q -> bind_values($_REQUEST['CRSO_RELATION_ID'], $_SESSION['selected_product_id']);
		$q -> execute();

		$select = new Form_Select('EXPORT_COUNTRY_ID');

		$select -> addOption($this -> getText('All'), '', false);
		$select -> addOption($this -> getText('Europe'), '__in_europe__', false);
		$select -> addOption($this -> getText('Non-EU'), '__out_europe__', false);
		$select -> addOption('---', '', false);

		while ($r = $q -> fetch()) {

			$select -> addOption($r -> COUNTRY_NAME, $r -> COUNTRY_ID, false);
		}

		if ($setValue) {
			$selectedValue = $setValue;
			$select -> setSelectedValue($setValue);
		}

		$filter = $select -> getHtml();

		return $$rv;
	}

	function tfSortingHat() {
		return array('QUANTITY' => "'Int'", 'WEIGHT' => "'Int'", 'AMOUNT' => "'Int'", 'PARTICIPANT_NAME' => "'StringNoTags'", );
	}


	function setEmptyText() {
		return $this -> langIf -> getText('No export data for the current selection');
	}

	function getProcessColumns() {

		$rv = array(
			'tfParticipantName' => array('PARTICIPANT_NAME'),
			'tfValueProcessorQuantitySum' => array('QUANTITY'),
			'tfValueProcessorWeightSum' => array('WEIGHT', 'WEIGHT_DEC'),
			'tfValueProcessorAmountSum' => array('AMOUNT'),
			'tfValueProcessorWeightAvg' => array('AVG_WEIGHT', 'AVG_WEIGHT_DEC'),
			'tfMonth' => array('EXPORT_MONTH'),
			'tfRefundId' => array('REFUND_REQUEST_ID'),
			'tfRefundCount' => array('REFUND_COUNT'),
			'tfProductCategory' => array('PRODUCT_CATEGORY'),
			'tfDocumentLink' => array('EXPORT_DOCUMENT'),
		);

		return $rv;
	}

	function tfValueProcessorQuantitySum($args) {
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		if (!is_numeric($args['value']))
			return $args['value'];
		$this -> _maxQuantity += $args['value'];
		$this -> _cols = $args['visibleColumns'];
		return locale_nf($args['value'], 0);
	}


	function tfParticipantName($args) {

		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		if ($this -> _showType != LIST_HTML_OUTPUT)
			return $args['value'];

		return '<a href="#" onclick=" ' . '$$(\'PARTICIPANT_RELATION_ID\').value=' . $args['row']['PARTICIPANT_RELATION_ID'] . '; ' . '$$(\'EXPORT_DATE_MONTH\').value=\'\'; ' . '_LoadOverview();">' . $args['value'] . '</a>';

	}

}
?>
