<?php
require_once(ELC_LISTS.'DeEarAbstractList.php');

class DeEARNetResultList  extends DeEarAbstractList {
	var $handle = 'DeEARNetResultList';
	var $exportTitle = 'NetResultList';

	var $_month_useinquery = true;

	private $_maxDecQ = 0;
	private $_maxRefQ = 0;
	private $_maxQDiff = 0;

	private $_maxDecW = 0;
	private $_maxRefW = 0;
	private $_maxWDiff = 0;

	function _setWhere() {

		parent::_setWhere();

		$this -> _addWhere('CRSO_RELATION_ID', array('field' => 'D.`CRSO_RELATION_ID`', 'callback' => array($this, 'getCRSOSelect'), 'required' => false, 'related' => array(), 'groupBy' => ''));
		$this -> _addWhere('EXPORT_DATE_YEAR', array('field' => 'YEAR(D.EXPORT_DATE)', 'callback' => array($this, 'getExportYear'), 'groupBy' => '', 'required' => false));
		$this -> _addWhere('EXPORT_DATE_MONTH', array('field' => 'MONTH(D.EXPORT_DATE)', 'callback' => array($this, 'getExportMonth'), 'groupBy' => '', 'required' => false, 'useinquery' => $this -> _month_useinquery));
		$this -> _addWhere('PARTICIPANT_RELATION_ID', array('field' => 'D.PARTICIPANT_RELATION_ID', 'callback' => array($this, 'getParticipantSelect'), 'related' => array(''), 'groupBy' => ''));
		$this -> _addWhere('PRODUCT_ID', array('field' => 'D.PRODUCT_ID', 'callback' => array($this, 'getProductSelect')));
		$this -> _addWhere('IS_EXPORTED', array('field' => 'D.IS_EXPORTED', 'callback' => array($this, 'getIsExported')));

		if ($this->_has_category)
			$this -> _addWhere('PRODUCT_CATEGORY_ID', array('field' => 'DL.PRODUCT_CATEGORY_ID', 'callback' => array($this, 'getProductCategory'), 'groupBy' => '', 'required' => false, 'useinquery' => $this -> _category_useinquery));

		if ($this->_has_dimension)
			$this -> _addWhere('CPD_ID', array('field' => 'DL.CPD_ID', 'callback' => array($this, 'getDimension'), 'groupBy' => '', 'required' => false, 'useinquery' => $this -> _dimension_useinquery));

	}

	function setQuery() {

		$this -> crso_id = $this -> getCRSOSelect('setValue');

		$this -> _query = "
			SELECT
				YEAR(D.EXPORT_DATE) AS EXPORT_YEAR,
				";
		if ($this -> _month_useinquery)
			$this -> _query .= "MONTH(D.EXPORT_DATE) AS EXPORT_MONTH, ";

		$this -> _query .= "
				D.EXPORT_DATE,
				R.RELATION_ID AS PARTICIPANT_RELATION_ID,
				R.NAME AS PARTICIPANT_NAME,
				CP2.PRODUCER_REG_NO,
		";

		if ($this ->_category_useinquery)
			$this ->_query .= "PC.PRODUCT_CATEGORY_ID as PRODUCT_CATEGORY_ID, PC.NAME as PRODUCT_CATEGORY, ";

		if ($this ->_dimension_useinquery)
			$this ->_query .= "CPD.CPD_ID as CPD_ID, CPD.NAME as DIMENSION, ";


		$this->_query .= "

				SUM(DL.QUANTITY) AS DEC_Q,
				RS.REFUND_QUANTITY as REF_Q,
				SUM(DL.QUANTITY) - RS.REFUND_QUANTITY as Q_DIFF,

				SUM(DL.WEIGHT) AS DEC_W,
				SUM(DL.WEIGHT_DEC) AS DEC_W_DEC,
				RS.REFUND_WEIGHT AS REF_W,
				SUM(DL.WEIGHT) - RS.REFUND_WEIGHT as W_DIFF,
				SUM(DL.WEIGHT_DEC) - RS.REFUND_WEIGHT_DEC as W_DIFF_DEC

			FROM TBL_DECLARATION D
			INNER JOIN TBL_DECLARATION_LINE DL ON D.DECLARATION_ID = DL.DECLARATION_ID
			INNER JOIN TBL_RELATION R ON D.PARTICIPANT_RELATION_ID = R.RELATION_ID

			INNER JOIN TBL_CRSO_PARTICIPANT_PRODUCT as CP
				ON R.RELATION_ID = CP.PARTICIPANT_RELATION_ID
				AND CP.CRSO_RELATION_ID = D.CRSO_RELATION_ID
				AND CP.PRODUCT_ID = D.PRODUCT_ID

			INNER JOIN TBL_CRSO_PARTICIPANT as CP2
				ON R.RELATION_ID = CP2.PARTICIPANT_RELATION_ID
				AND CP2.CRSO_RELATION_ID = D.CRSO_RELATION_ID

			LEFT JOIN TBL_PRODUCT_CATEGORY PC ON DL.PRODUCT_CATEGORY_ID = PC.PRODUCT_CATEGORY_ID
			LEFT JOIN TBL_CRSO_PRODUCT_DIMENSION CPD ON DL.CPD_ID = CPD.CPD_ID

			LEFT JOIN (
				SELECT RL.PRODUCER_RELATION_ID, RR.CRSO_RELATION_ID, RR.PRODUCT_ID, RE.EXPORT_DATE,
					SUM(RL.QUANTITY) AS REFUND_QUANTITY, SUM(RL.WEIGHT) AS REFUND_WEIGHT, SUM(RL.WEIGHT_DEC) AS REFUND_WEIGHT_DEC
				FROM `TBL_REFUND_REQUEST_LINE` RL
				INNER JOIN `TBL_REFUND_REQUEST` RR ON RL.REFUND_REQUEST_ID = RR.REFUND_REQUEST_ID
				INNER JOIN `TBL_REFUND_EXPORT` RE ON RR.REFUND_EXPORT_ID = RE.REFUND_EXPORT_ID
				INNER JOIN TBL_COUNTRY RR_C ON RR.EXPORT_COUNTRY_ID = RR_C.COUNTRY_ID
				WHERE RR_C.REGION_ID = 1

			";

			if (isset($_REQUEST['PRODUCT_CATEGORY_ID']) && !empty($_REQUEST['PRODUCT_CATEGORY_ID']) && $_REQUEST['PRODUCT_CATEGORY_ID'] != 'grouped') {
				$this->_query .= ' AND RL.PRODUCT_CATEGORY_ID = '.(int) $_REQUEST['PRODUCT_CATEGORY_ID'];
			}

			$this->_query .= "

				GROUP BY RL.PRODUCER_RELATION_ID, RR.PRODUCT_ID, RE.EXPORT_DATE
			) AS RS
				ON RS.PRODUCER_RELATION_ID = D.PARTICIPANT_RELATION_ID
				AND RS.CRSO_RELATION_ID = D.CRSO_RELATION_ID
				AND RS.PRODUCT_ID = D.PRODUCT_ID
				AND RS.EXPORT_DATE = D.EXPORT_DATE
				";

		$this -> _groupByArray = array('EXPORT_YEAR', 'PARTICIPANT_RELATION_ID');

		if (isset($_REQUEST['EXPORT_DATE_MONTH']) && $_REQUEST['EXPORT_DATE_MONTH'] != 'grouped') {
			$this -> _groupByArray[] = 'EXPORT_MONTH';
		}

		if (isset($_REQUEST['PRODUCT_CATEGORY_ID']) && $_REQUEST['PRODUCT_CATEGORY_ID'] != 'grouped') {
			$this -> _groupByArray[] = 'DL.PRODUCT_CATEGORY_ID';
		}

		if (isset($_REQUEST['CPD_ID']) && $_REQUEST['CPD_ID'] != 'grouped') {
			$this -> _groupByArray[] = 'DL.CPD_ID';
		}


		$this -> _orderBy = "ORDER BY EXPORT_YEAR, ";

		if ($this -> _month_useinquery)
			$this -> _orderBy .= " EXPORT_MONTH, ";

		$this -> _orderBy .= " PARTICIPANT_NAME";
	}

	function getColumnModifier() {
		$r = 'style="text-align:right;"';
		return array('DEC_Q' => $r, 'REF_Q' => $r, 'Q_DIFF' => $r, 'DEC_W' => $r, 'DEC_W_DEC' => $r, 'REF_W' => $r, 'W_DIFF' => $r, 'W_DIFF_DEC' => $r);
	}

	function tfHeaderProc() {
		return array('getAr' => array('DEC_Q', 'REF_Q', 'Q_DIFF', 'DEC_W', 'REF_W', 'W_DIFF'));

	}

	function getProcessColumns() {
		$rv = array(
			'tfParticipantName' => array('PARTICIPANT_NAME'),
			'tfMonth' => array('EXPORT_MONTH'),
			'tfDecQ' => array('DEC_Q'),
			'tfRefQ' => array('REF_Q'),
			'tfQDiff' => array('Q_DIFF'),
			'tfDecW' => array('DEC_W', 'DEC_W_DEC'),
			'tfRefW' => array('REF_W'),
			'tfWDiff' => array('W_DIFF', 'W_DIFF_DEC'),
			'tfProductCategory' => array('PRODUCT_CATEGORY'),
			'tfDimension' => array('DIMENSION'),
		);

		return $rv;
	}

	function tfParticipantName($args) {

		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		if ($this -> _showType != LIST_HTML_OUTPUT)
			return $args['value'];

		return '<a href="#" onclick=" ' . '$$(\'PARTICIPANT_RELATION_ID\').value=' . $args['row']['PARTICIPANT_RELATION_ID'] . '; ' . '$$(\'EXPORT_DATE_MONTH\').value=\'\'; ' . '_LoadOverview();">' . $args['value'] . '</a>';
	}

	function tfDecQ($args) {

		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);

		if(!is_numeric($args['value']))
			return $args['value'];

		$rel_type = '';
		switch($_SESSION['session_group_company_id']) {
			case GROUP_ACCOUNTANT :
				$rel_type = 'accountant';
				break;
			case GROUP_CRSO :
				$rel_type = 'crso';
				break;
			default :
				trigger_error('not implemented', E_USER_ERROR);
		}

		$this -> _maxDecQ += $args['value'];

		$this -> _cols = $args['visibleColumns'];

		if ($this -> _showType != LIST_HTML_OUTPUT)
			return locale_nf($args['value'], 0);

		return '<a href="#" onclick="$$(\'PARTICIPANT_RELATION_ID\').value='.$args['row']['PARTICIPANT_RELATION_ID'].';$$(\'EXPORT_DATE_YEAR\').value='.$args['row']['EXPORT_YEAR'].';$$(\'EXPORT_DATE_MONTH\').value='.$args['row']['EXPORT_DATE'].';_LoadOverview();"">' . locale_nf($args['value'], 0) . '</a>';
	}

	function tfRefQ($args) {

		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);

		if(!is_numeric($args['value']))
			return $args['value'];

		$rel_type = '';
		switch($_SESSION['session_group_company_id']) {
			case GROUP_ACCOUNTANT :
				$rel_type = 'accountant';
				break;
			case GROUP_CRSO :
				$rel_type = 'crso';
				break;
			default :
				trigger_error('not implemented', E_USER_ERROR);
		}

		$this -> _maxRefQ += $args['value'];

		$this -> _cols = $args['visibleColumns'];

		if ($this -> _showType != LIST_HTML_OUTPUT)
			return locale_nf($args['value'], 0);

		return '<a href="#" onclick="$$(\'PARTICIPANT_RELATION_ID\').value='.$args['row']['PARTICIPANT_RELATION_ID'].';$$(\'EXPORT_DATE_YEAR\').value='.$args['row']['EXPORT_YEAR'].';$$(\'EXPORT_DATE_MONTH\').value='.$args['row']['EXPORT_DATE'].';_LoadOverview();"">' . locale_nf($args['value'], 0) . '</a>';
	}

	function tfQDiff($args) {
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);

		if(!is_numeric($args['value']))
			return $args['value'];

		$this -> _maxQDiff += $args['value'];

		return locale_nf($args['value'], 0);

	}

	function tfDecW($args) {
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);

		if(!is_numeric($args['value']))
			return $args['value'];

		$this -> _maxDecW += $args['value'];
		return locale_nf($args['value'], $this->show_weight_dec ? 2 : 0);
	}

	function tfRefW($args) {
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);

		if(!is_numeric($args['value']))
			return $args['value'];

		$this -> _maxRefW += $args['value'];
		return locale_nf($args['value'], $this->show_weight_dec ? 2 : 0);
	}

	function tfWDiff($args) {
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);

		if(!is_numeric($args['value']))
			return $args['value'];

		$this -> _maxWDiff += $args['value'];
		return locale_nf($args['value'], $this->show_weight_dec ? 2 : 0);
	}

	function getAr($args) {
		return 'class="quantity"';
	}

	function getButtons() {

		return array_merge($this -> _getCSVButton(), $this -> _getXMLButton());
	}

	function getIsExported($rv = 'setValue') {
		$filter = '';
		$setValue = '2';
		$selectedValue = '';
		return $$rv;
	}

	function getExportYear($rv = 'setValue') {

		$setValue = isset($_REQUEST['EXPORT_DATE_YEAR']) ? $_REQUEST['EXPORT_DATE_YEAR'] : false;
		$selectedValue = '';

		if (isset($$rv))
			return $$rv;

		if (isset($_REQUEST['CRSO_RELATION_ID'])) {

			$q = $this -> db -> query('SELECT DISTINCT YEAR(EXPORT_DATE) AS YEAR FROM TBL_DECLARATION WHERE IS_EXPORTED=2 AND CRSO_RELATION_ID = ? AND PRODUCT_ID = ? ORDER BY `YEAR` ASC');
			$q -> bind_values($_REQUEST['CRSO_RELATION_ID'], $_SESSION['selected_product_id']);
			$q -> execute();

			$select = new Form_Select('EXPORT_DATE_YEAR');
			$select -> addOption($this -> getText('All'), '', false);
			while ($r = $q -> fetch()) {
				$select -> addOption($r -> YEAR, $r -> YEAR, false);
				$lastValue = $r -> YEAR;
			}

			if ($setValue) {
				$select -> setSelectedValue($setValue);
				$selectedValue = $setValue;
			}

			$filter = $select -> getHtml();

		} else
			$filter = '';

		return $$rv;
	}

	function getExportMonth($rv = 'setValue') {

		$setValue = isset($_REQUEST['EXPORT_DATE_MONTH']) ? $_REQUEST['EXPORT_DATE_MONTH'] : false;
		$selectedValue = '';

		if (isset($$rv))
			return $$rv;

		if (isset($_REQUEST['CRSO_RELATION_ID'])) {

			$q = $this -> db -> query('SELECT DISTINCT MONTH(EXPORT_DATE) AS MONTH FROM TBL_DECLARATION WHERE IS_EXPORTED=2 AND CRSO_RELATION_ID = ? AND PRODUCT_ID = ? ORDER BY `MONTH` ASC');
			$q -> bind_values($_REQUEST['CRSO_RELATION_ID'], $_SESSION['selected_product_id']);
			$q -> execute();

			$select = new Form_Select('EXPORT_DATE_MONTH');
			$select -> addOption($this -> getText('All'), '', false);
			$select -> addOption($this -> getText('Grouped'), 'grouped', false);
			while ($r = $q -> fetch()) {
				$select -> addOption($this -> tfMonth(array('value' => $r -> MONTH)), $r -> MONTH, false);
				$lastValue = $r -> MONTH;
			}

			if ($setValue) {
				$selectedValue = $setValue;
				$select -> setSelectedValue($setValue);
			}

			$filter = $select -> getHtml();

		} else
			$filter = '';

		return $$rv;
	}


	function tfSortingHat() {
		return array(
			'DEC_Q' => "'IntNoTags'",
			'REF_Q' => "'IntNoTags'",
			'Q_DIFF' => "'Int'",
			'DEC_W' => "'Int'",
			'REF_W' => "'Int'",
			'W_DIFF' => "'Int'",
			'PARTICIPANT_NAME' => "'StringNoTags'",
		);
	}

	function tfFooter()// getClass
	{
		if (!isset($this -> _cols) || in_array('QUANTITY', $this -> _hiddenCols))
			return false;

		$rv = explode(',', str_repeat(',', (sizeof($this -> _cols) - 1)));

		$pos = array_flip($this -> _cols);
		if ($this -> _showType == LIST_HTML_OUTPUT) {
			$rv[$pos['DEC_Q']] = '<span style="float:right; font-weight: bold;">' . locale_nf($this -> _maxDecQ, 0). '</span><span style="clear:both;"></span>';
			$rv[$pos['REF_Q']] = '<span style="float:right; font-weight: bold;">' . locale_nf($this -> _maxRefQ, 0) . '</span><span style="clear:both;"></span>';
			$rv[$pos['Q_DIFF']] = '<span style="float:right; font-weight: bold;">' . locale_nf($this -> _maxQDiff, 0) . '</span><span style="clear:both;"></span>';

			if (isset($pos['DEC_W']))
				$rv[$pos['DEC_W']] = '<span style="float:right; font-weight: bold;">' . locale_nf($this -> _maxDecW, 0) . '</span><span style="clear:both;"></span>';

			if (isset($pos['DEC_W_DEC']))
				$rv[$pos['DEC_W_DEC']] = '<span style="float:right; font-weight: bold;">' . locale_nf($this -> _maxDecW, 2) . '</span><span style="clear:both;"></span>';

			$rv[$pos['REF_W']] = '<span style="float:right; font-weight: bold;">' . locale_nf($this -> _maxRefW, 0) . '</span><span style="clear:both;"></span>';

			if (isset($pos['W_DIFF']))
				$rv[$pos['W_DIFF']] = '<span style="float:right; font-weight: bold;">' . locale_nf($this -> _maxWDiff, 0) . '</span><span style="clear:both;"></span>';

			if (isset($pos['W_DIFF_DEC']))
				$rv[$pos['W_DIFF_DEC']] = '<span style="float:right; font-weight: bold;">' . locale_nf($this -> _maxWDiff, 2) . '</span><span style="clear:both;"></span>';

		} else {
			$rv[$pos['DEC_Q']] = locale_nf($this -> _maxDecQ);
			$rv[$pos['REF_Q']] = locale_nf($this -> _maxRefQ);
			$rv[$pos['Q_DIFF']] = locale_nf($this -> _maxQDiff);
			$rv[$pos['DEC_W']] = locale_nf($this -> _maxDecW);
			$rv[$pos['REF_W']] = locale_nf($this -> _maxRefW);
			$rv[$pos['W_DIFF']] = locale_nf($this -> _maxWDiff);
		}
		return $rv;
	}

	function setEmptyText() {
		return $this -> langIf -> getText('No declarations data for the current selection');
	}

}
?>
