<?php

/**
 * DeEarAbstractList provides common logic for the DeEar* reports.

 * Notice that these reports are now not only used by DE but for all CRSO's
 * that have declaration/refund details access.
 */
class DeEarAbstractList extends ELCDefaultList {

	protected $_cols = array();

	protected $_has_category = true;
	protected $_has_dimension = true;

	protected $_maxQuantity = 0;
	protected $_maxWeight = 0;
	protected $_maxAmount = 0;

	protected $_weightAvg = 0;
	protected $_weightLen = 0;

	protected $_month_useinquery = true;
	protected $_category_useinquery = true;
	protected $_dimension_useinquery = true;


	protected $show_weight = false;
	protected $show_weight_dec = false;

	function _setWhere() {

		$this -> _hiddenCols[] = 'EXPORT_DATE';
		$this -> _hiddenCols[] = 'PRODUCT_CATEGORY_ID';
		$this -> _hiddenCols[] = 'CPD_ID';

		$crso_ids = array();
		switch($_SESSION['session_group_company_id']) {
			case GROUP_ACCOUNTANT :
				require_once (ELC_CLASSES . 'AccountantCRSO.php');
				$crso = new AccountantCRSO($this -> db);
				$q = $crso -> getAccountantCRSOs($_SESSION['accountant_relation_id'], $_SESSION['selected_product_id']);
				while ($rs = $q -> fetch()) {
					$crso_ids[] = (int)$rs->CRSO_RELATION_ID;
					$this->_earCountrySettings($rs->CRSO_RELATION_ID);
				}
				break;

			case GROUP_CRSO :
				$crso_ids = array((int)$_SESSION['crso_relation_id']);
				$this->_earCountrySettings($_SESSION['crso_relation_id']);
				break;

			case GROUP_SYSADMIN:
				break;
				default :
					trigger_error('not implemented', E_USER_ERROR);
		}

		if (!$this->show_weight) {
			$this -> _hiddenCols[] = 'WEIGHT';
			$this -> _hiddenCols[] = 'AVG_WEIGHT';
			$this -> _hiddenCols[] = 'DEC_W';
			$this -> _hiddenCols[] = 'W_DIFF';
		}

		if (!$this->show_weight_dec) {
			$this -> _hiddenCols[] = 'WEIGHT_DEC';
			$this -> _hiddenCols[] = 'AVG_WEIGHT_DEC';
			$this -> _hiddenCols[] = 'DEC_W_DEC';
			$this -> _hiddenCols[] = 'W_DIFF_DEC';
		}

		if (isset($_REQUEST['preset'])) {

			$preset = $_REQUEST['preset'];

			$q = $this -> db -> query('
				SELECT
					YEAR(D.EXPORT_DATE) AS YEAR ,
					MONTH(D.EXPORT_DATE) AS MONTH
				FROM TBL_DECLARATION D
					WHERE IS_EXPORTED=2
					AND CRSO_RELATION_ID IN (' . implode($crso_ids, ',') . ')
					AND PRODUCT_ID = ?
				ORDER BY D.EXPORT_DATE DESC
				LIMIT 1');

			$q -> bind_values($_SESSION['selected_product_id']);
			$q -> execute();
			$r = $q -> fetch();

			$last_year = $r -> YEAR;
			$last_month = $r -> MONTH;

			if ($preset == 'month') {
				$_REQUEST['EXPORT_DATE_YEAR'] = $last_year;
				$_REQUEST['EXPORT_DATE_MONTH'] = $last_month;
			}

			if ($preset == 'year') {
				$_REQUEST['EXPORT_DATE_YEAR'] = $last_year;
				$_REQUEST['EXPORT_DATE_MONTH'] = 'grouped';
			}

		}

		if (isset($_REQUEST['EXPORT_DATE_MONTH']) && $_REQUEST['EXPORT_DATE_MONTH'] == 'grouped')
			$this -> _month_useinquery = false;

		if (isset($_REQUEST['PRODUCT_CATEGORY_ID']) && $_REQUEST['PRODUCT_CATEGORY_ID'] == 'grouped')
			$this -> _category_useinquery = false;

		if (isset($_REQUEST['CPD_ID']) && $_REQUEST['CPD_ID'] == 'grouped')
			$this -> _dimension_useinquery = false;
	}


	function _earCountrySettings($crso_id) {

		$r = new Relation($this->db, $crso_id);
		$c = new Country($this->db, $r->getCountry(), true);


		if ($c->has(HasDeclOfWeight,$_SESSION['selected_product_id'])) {
			$this->show_weight = true;
		}

		if ($c->has(HasDeclOfWeightWithDecimals,$_SESSION['selected_product_id'])) {
			$this->show_weight_dec = true;
		}

		if (!$c->has(HasDeclByCategory,$_SESSION['selected_product_id'])) {
			$this->_has_category = false;
			$this -> _hiddenCols[] = 'PRODUCT_CATEGORY';
		}

		if (!$c->has(HasDeclarationDimension,$_SESSION['selected_product_id'])) {
			$this->_has_dimension = false;
			$this -> _hiddenCols[] = 'DIMENSION';
		}

	}

	function getProductCategory($rv = 'setValue') {

		$setValue = isset($_REQUEST['PRODUCT_CATEGORY_ID']) ? $_REQUEST['PRODUCT_CATEGORY_ID'] : false;
		$selectedValue = '';

		if (isset($$rv))
			return $$rv;

		if (isset($_REQUEST['CRSO_RELATION_ID'])) {

			$q = $this -> db -> query('SELECT PRODUCT_CATEGORY_ID, NAME FROM TBL_PRODUCT_CATEGORY WHERE CRSO_RELATION_ID = ? AND PRODUCT_ID = ? ORDER BY `NAME`');
			$q -> bind_values($_REQUEST['CRSO_RELATION_ID'], $_SESSION['selected_product_id']);
			$q -> execute();

			$select = new Form_Select('PRODUCT_CATEGORY_ID');
			$select -> addOption($this -> getText('All'), '', false);
			$select -> addOption($this -> getText('Grouped'), 'grouped', false);
			while ($r = $q -> fetch()) {
				$select -> addOption($r -> NAME, $r -> PRODUCT_CATEGORY_ID, false);
			}

			if ($setValue) {
				$selectedValue = $setValue;
				$select -> setSelectedValue($setValue);
			}

			$filter = $select -> getHtml();

		} else
			$filter = '';

		return $$rv;
	}

	function getDimension($rv = 'setValue') {

		$setValue = isset($_REQUEST['CPD_ID']) ? $_REQUEST['CPD_ID'] : false;
		$selectedValue = '';

		if (isset($$rv))
			return $$rv;

		if (isset($_REQUEST['CRSO_RELATION_ID'])) {

			$q = $this -> db -> query('SELECT CPD_ID, NAME FROM TBL_CRSO_PRODUCT_DIMENSION WHERE CRSO_RELATION_ID = ? AND PRODUCT_ID = ? ORDER BY `NAME`');
			$q -> bind_values($_REQUEST['CRSO_RELATION_ID'], $_SESSION['selected_product_id']);
			$q -> execute();

			$select = new Form_Select('CPD_ID');
			$select -> addOption($this -> getText('All'), '', false);
			$select -> addOption($this -> getText('Grouped'), 'grouped', false);
			while ($r = $q -> fetch()) {
				$select -> addOption($r -> NAME, $r -> CPD_ID, false);
			}

			if ($setValue) {
				$selectedValue = $setValue;
				$select -> setSelectedValue($setValue);
			}

			$filter = $select -> getHtml();

		} else
			$filter = '';

		return $$rv;
	}


	function tfValueProcessorWeightSum($args) {
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		if (!is_numeric($args['value']))
			return $args['value'];
		$this -> _maxWeight += $args['value'];
		$this -> _cols = $args['visibleColumns'];
		return locale_nf($args['value'], $this->show_weight_dec ? 2 : 0);
	}


	function tfValueProcessorAmountSum($args) {
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		if (!is_numeric($args['value']))
			return $args['value'];
		$this -> _maxAmount += $args['value'];
		$this -> _cols = $args['visibleColumns'];
		return locale_nf($args['value'], 2);
	}

	function tfValueProcessorWeightAvg($args) {
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		if (!is_numeric($args['value']))
			return $args['value'];

		$this -> _weightAvg += $args['value'];
		$this -> _weightLen += 1;

		$this -> _cols = $args['visibleColumns'];
		return locale_nf($args['value'], 4);
	}

	function tfFooter()// getClass
	{
		if (!$this -> _cols || in_array('QUANTITY', $this -> _hiddenCols))
			return false;
		$rv = explode(',', str_repeat(',', (sizeof($this -> _cols) - 1)));

		$pos = array_flip($this -> _cols);

		if ($this->_weightLen != 0)
			$weightAvgAvg = $this->_weightAvg / $this->_weightLen;

		if ($this -> _showType == LIST_HTML_OUTPUT) {
			$rv[$pos['QUANTITY']] = '<span style="float:right; font-weight: bold;">' . locale_nf($this -> _maxQuantity, 0) . '</span><span style="clear:both;"></span>';

			if (isset($pos['WEIGHT']))
				$rv[$pos['WEIGHT']] = '<span style="float:right; font-weight: bold;">' . locale_nf($this -> _maxWeight, 0). '</span><span style="clear:both;"></span>';

			if (isset($pos['WEIGHT_DEC']))
				$rv[$pos['WEIGHT_DEC']] = '<span style="float:right; font-weight: bold;">' . locale_nf($this -> _maxWeight, 2). '</span><span style="clear:both;"></span>';

			if (isset($pos['AVG_WEIGHT']))
				$rv[$pos['AVG_WEIGHT']] = '<span style="float:right; font-weight: bold;">' . locale_nf($weightAvgAvg, 4) . '</span><span style="clear:both;"></span>';

			$rv[$pos['AMOUNT']] = '<span style="float:right; font-weight: bold;">' . locale_nf($this -> _maxAmount, 2) . '</span><span style="clear:both;"></span>';
		} else {
			$rv[$pos['QUANTITY']] = locale_nf($this -> _maxQuantity, 0);

			if (isset($pos['WEIGHT']))
				$rv[$pos['WEIGHT']] = locale_nf($this -> _maxWeight, 0);

			if (isset($pos['WEIGHT_DEC']))
				$rv[$pos['WEIGHT_DEC']] = locale_nf($this -> _maxWeight, 2);

			if (isset($pos['AVG_WEIGHT']))
				$rv[$pos['AVG_WEIGHT']] = locale_nf($weightAvgAvg, 4);

			$rv[$pos['AMOUNT']] = locale_nf($this -> _maxAmount, 2);
		}
		return $rv;
	}

	function getFilters($callback) {
		$html = parent::getFilters($callback);
		$html .= '&nbsp;<input type="button" onclick="document.location.reload();" value="Reset" />';
		return $html;
	}

	function tfProductCategory($args) {

		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		if ($this -> _showType != LIST_HTML_OUTPUT)
			return $args['value'];

		return '<a href="#" onclick=" ' . '$$(\'PRODUCT_CATEGORY_ID\').value=' . $args['row']['PRODUCT_CATEGORY_ID'] . '; _LoadOverview();">' . $args['value'] . '</a>';

	}
	function tfDimension($args) {

		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		if ($this -> _showType != LIST_HTML_OUTPUT)
			return $args['value'];

		return '<a href="#" onclick=" ' . '$$(\'CPD_ID\').value=' . $args['row']['CPD_ID'] . '; _LoadOverview();">' . $args['value'] . '</a>';

	}



}
?>
