<?php
/**
 * @package ELC
 * @subpackage List
 * @since 19-sep-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class DeclarationList extends ELCDefaultList
{
	public $handle = 'DeclarationList';
	private $_maxQuantity = 0;
	private $_cols = 0;
	private $_postscriptVars = '';

	function setEmptyText()
	{
		return $this->langIf->getText('No Declarations found for selected filter');
	}

	public function setQuery()
	{
		if(!isset($_REQUEST['DECLARATION_ID'])||!is_numeric($_REQUEST['DECLARATION_ID']))
		{
			$this->_query = "SELECT D.DECLARATION_ID,
					IF(ISNULL(DD.DECLARATION_DEBIT_ID),
						IF(ISNULL(DC.DECLARATION_CREDIT_ID),'-',
							f_elcBuildInvoiceNoForInterfaces(D.CRSO_RELATION_ID,DC.`YEAR`,DC.DECLARATION_CREDIT_ID,true,false)),
							f_elcBuildInvoiceNoForInterfaces(D.CRSO_RELATION_ID,DD.`YEAR`,DD.DECLARATION_DEBIT_ID,false,false)) as CREDIT_DEBIT_ID,
					R.`NAME` as PARTICIPANT_NAME,
					D.PARTICIPANT_RELATION_ID,
					D.`MONTH` as PERIOD, D.`DOCUMENT_ID`, SUM(DL.QUANTITY) as QUANTITY,
					BIT_OR(DL.`STATUS`) as STATUS
				FROM TBL_DECLARATION as D
				INNER JOIN TBL_DECLARATION_LINE as DL
					ON D.DECLARATION_ID = DL.DECLARATION_ID
				INNER JOIN TBL_RELATION as R
					ON D.PARTICIPANT_RELATION_ID = R.RELATION_ID
				LEFT JOIN TBL_DECLARATION_SET_LINE as dsl
					ON D.DECLARATION_ID = dsl.DECLARATION_ID
				LEFT JOIN TBL_DECLARATION_DEBIT as DD
					ON dsl.DECLARATION_SET_ID = DD.DECLARATION_SET_ID
						AND D.CRSO_RELATION_ID = DD.CRSO_RELATION_ID
				LEFT JOIN TBL_DECLARATION_CREDIT as DC
					ON dsl.DECLARATION_SET_ID = DC.DECLARATION_SET_ID
						AND D.CRSO_RELATION_ID = DC.CRSO_RELATION_ID";
		}
		else
		{
			$this->_query = "SELECT D.DECLARATION_ID,
					D.`MONTH` as PERIOD, PRODUCT_CATEGORY_ID as PRODUCT_CATEGORY,
					BRAND_ID,SUM(DL.QUANTITY) as QUANTITY, WEIGHT, WEIGHT_DEC,
					DL.`STATUS`,
					D.PARTICIPANT_RELATION_ID
				FROM TBL_DECLARATION as D
				INNER JOIN TBL_DECLARATION_LINE as DL
					ON D.DECLARATION_ID = DL.DECLARATION_ID";
		}
		$this->_groupBy = "GROUP BY D.`YEAR`";
	}

	function _filterQuery()
	{
		$this->_addHiddenCol(array('PRODUCT_CATEGORY','BRAND_ID','WEIGHT','WEIGHT_DEC','PARTICIPANT_RELATION_ID'));
		if(!isset($_REQUEST['MONTH'])||!is_numeric($_REQUEST['MONTH']))
		{
			$this->_groupByArray = array("D.`MONTH`");
			$this->_addHiddenCol(array('DECLARATION_ID','CREDIT_DEBIT_ID','PARTICIPANT_NAME'));
		}
		elseif(!isset($_REQUEST['DECLARATION_ID'])||!is_numeric($_REQUEST['DECLARATION_ID']))
		{
			$this->_union = " UNION (SELECT '-' as DECLARATION_ID,
					IF(ISNULL(DD.DECLARATION_DEBIT_ID),
						IF(ISNULL(DC.DECLARATION_CREDIT_ID),'-',
						f_elcBuildInvoiceNoForInterfaces(DC.CRSO_RELATION_ID,DC.`YEAR`,DC.DECLARATION_CREDIT_ID,true,false)),
						f_elcBuildInvoiceNoForInterfaces(DD.CRSO_RELATION_ID,DD.`YEAR`,DD.DECLARATION_DEBIT_ID,false,false)) as CREDIT_DEBIT_ID,
					R.`NAME` as PARTICIPANT_NAME,
					D.PARTICIPANT_RELATION_ID,
					D.`MONTH` as PERIOD, D.`DOCUMENT_ID`, '-' as QUANTITY, '-' as STATUS
				FROM TBL_REQUESTED_INVOICE as D
				INNER JOIN TBL_RELATION as R
					ON D.PARTICIPANT_RELATION_ID = R.RELATION_ID
				LEFT JOIN TBL_DECLARATION_DEBIT as DD
					ON D.REQUESTED_INVOICE_ID = DD.REQUESTED_INVOICE_ID
						AND D.CRSO_RELATION_ID = DD.CRSO_RELATION_ID
				LEFT JOIN TBL_DECLARATION_CREDIT as DC
					ON D.REQUESTED_INVOICE_ID = DC.REQUESTED_INVOICE_ID
						AND D.CRSO_RELATION_ID = DC.CRSO_RELATION_ID
				LEFT JOIN TBL_TRUST as DL
					ON D.CRSO_RELATION_ID = DL.CRSO_RELATION_ID ";
			if(($where = $this->_getWhere())!==false)
				$this->_union .= str_replace('AND  D.PRODUCT_ID  =  \''.$_SESSION['selected_product_id'].'\'','',$where).')';
			else
				$this->_union .= ')';
		}
		if(isset($_REQUEST['DECLARATION_ID'])&&is_numeric($_REQUEST['DECLARATION_ID']))
		{
			unset($this->_groupByArray);
			$this->_groupByArray = array("DL.LINE_INDEX");

			require_once(ELC_CLASSES.'CRSO.php');
			require_once(ELC_CLASSES.'Country.php');
			$crso = new CRSO($this->db,array('relation'=>new Relation($this->db,$_REQUEST['CRSO_RELATION_ID'])));
			$country = new Country($this->db,$crso->getCountry(),true);
			if($country->has(HasDeclByCategory,$_SESSION['selected_product_id']))
				$this->_removeHiddenCol('PRODUCT_CATEGORY');
			if($country->has(HasDeclByBrand,$_SESSION['selected_product_id']))
				$this->_removeHiddenCol('BRAND_ID');
			if($country->has(HasDeclOfWeight,$_SESSION['selected_product_id']))
				$this->_removeHiddenCol('WEIGHT');
			if($country->has(HasDeclOfWeightWithDecimals,$_SESSION['selected_product_id']))
				$this->_removeHiddenCol('WEIGHT_DEC');
		}
		return $this->_query;
	}

	function _setWhere()
	{

		$this->_addWhere('YEAR',
			array('field'=>'D.`YEAR`',
				'callback' => array($this,'getYearSelect'),
				'required' => true));


		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'D.CRSO_RELATION_ID',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true));
		// This cannot be the regular participant select since this will contain subrogator/trustee information as well
		$opts = array(
				'field'=>'D.`PARTICIPANT_RELATION_ID`',
				'callback' => array($this,'getParticipantSelect'),
				'related'=>array('PARTICIPANT_NAME'),
				'groupBy'=>'');
		if($_SESSION['session_group_group_id']==GROUP_PARTICIPANT)
			$opts['required'] = true;

		$this->_addWhere('PARTICIPANT_RELATION_ID',
			$opts);
		$this->_addWhere('MONTH',
			array(
				'field'=>'D.`MONTH`',
				'callback' => array($this,'getMonthSelect'),
				'related'=>array('PERIOD'),
				'groupBy'=>'D.`DECLARATION_ID`'));

		$this->_addWhere('TRUST_ID',
			array(
				'field'=>'DL.`TRUST_ID`',
				'callback' => array($this,'getTrustInput')));

		$this->_addWhere('DECLARATION_ID',
			array(
				'field'=>'D.`DECLARATION_ID`',
				'callback' => array($this,'getDeclarationInput'),
				'related'=>array('PERIOD','DOCUMENT_ID')));

		$this->_addWhere('PRODUCT_ID',
			array(
				'field'=>'D.PRODUCT_ID',
				'callback' => array($this,'getProductSelect')));
	}
	/**
	 * @todo Make work for sysadmin so you can select trusts as well.
	 */
	function getParticipantSelect($rv='setValue')
	{
		if($_SESSION['session_group_group_id']!=GROUP_PARTICIPANT)
			return parent::getParticipantSelect($rv);

		if(empty($_REQUEST['CRSO_RELATION_ID']))
			return '';
		require_once(ELC_CLASSES.'Relation.php');
		require_once(ELC_CLASSES.'Trust.php'); // Get all the subrogated stuff.

		$r = new Relation($this->db);
		$t = new Trust($this->db);

		// First list own participants, if participant has a trust, list the trustee with it in brackets.
		$setValue = isset($_REQUEST['PARTICIPANT_RELATION_ID'])?$_REQUEST['PARTICIPANT_RELATION_ID']:$_SESSION['selected_participant_id'];
		$selectedValue = '';
		$trust = (!empty($_REQUEST['TRUST_ID']))?$_REQUEST['TRUST_ID']:false;

		$select = new Form_Select('PARTICIPANT_RELATION_ID');
		$id = $_SESSION['selected_participant_id'];
		$r->set($id);
		$name = $r->name;
		$select->addOption(((mb_strlen($name)>35)?mb_substr($name,0,35).'...':$name), $id, ($setValue==$id&&!$trust),true,array('trust'=>''));
		$q = $t->selectSubrogations($id,TRUST_APPROVED,$_REQUEST['CRSO_RELATION_ID'],$_SESSION['selected_product_id']);
		while($r = $q->fetch())
		{
			$name = $r->NAME;
			$select->addOption($this->langIf->getText('Trust with ').((mb_strlen($name)>35)?mb_substr($name,0,35).'...':$name),
				$r->TRUSTEE_RELATION_ID,
				($setValue==$r->TRUSTEE_RELATION_ID),true,
				array('trust'=>$r->TRUST_ID));
		}
		$q = $t->selectTrusts($id,TRUST_APPROVED,$_REQUEST['CRSO_RELATION_ID'],$_SESSION['selected_product_id']);
		while($r = $q->fetch())
		{
			$name = $r->NAME;
			$item =& $select->addOption($this->langIf->getText('Subrogated for ').((mb_strlen($name)>35)?mb_substr($name,0,35).'...':$name),
				$id,
				($setValue==$id&& $trust==$r->TRUST_ID),true,
				array('trust'=>$r->TRUST_ID));
			$item['trust'] = $r->TRUST_ID;
		}
		$select->addAttrib('onchange','setTrust(this);');
		$filter = $select->getHtml();

		return $$rv;
	}

	function getPostScript()
	{
		return "
			function setTrust(sel)
			{
				$$('TRUST_ID').value = sel.options[sel.selectedIndex].getAttribute('trust');
			}
			";
	}

	function getTrustInput($rv='setValue')
	{
		$setValue = (!empty($_REQUEST['TRUST_ID']))?$_REQUEST['TRUST_ID']:null;
		$selectedValue = '';
		$filter = '<input type="hidden" name="TRUST_ID" value="'.$setValue.'">';
		return $$rv;
	}

	function getDeclarationInput($rv='setValue')
	{
		$setValue = isset($_REQUEST['DECLARATION_ID'])?$_REQUEST['DECLARATION_ID']:'';
		$selectedValue = '';
		$filter = '<input type="hidden" name="DECLARATION_ID" value="">';
		return $$rv;
	}

	function getProcessColumns()
	{
		$rv = array(
			'tfDeclarationLink'=>array('DECLARATION_ID'),
			'tfBrand'=>array('BRAND_ID'),
			'tfProductCategory'=>array('PRODUCT_CATEGORY'),
			'tfDeclarationStatus'=>array('STATUS'),
			'tfValueProcessorQuantity'=>array('WEIGHT', 'WEIGHT_DEC'),
			'tfValueProcessorQuantitySum'=>array('QUANTITY'),
			'tfMonthWithLink'=>array('PERIOD'),
			'tfLongNames'=>array('PARTICIPANT_NAME')
			);
		if(!isset($_REQUEST['MONTH'])||!is_numeric($_REQUEST['MONTH']))
			$rv['tfBooleanIcon'] = array('DOCUMENT_ID');
		else
			$rv['tfDocumentLink'] = array('DOCUMENT_ID');

		return $rv;
	}

	function getColumnModifier()
	{
		return array('QUANTITY'=>'style="text-align:right;"','WEIGHT'=>'style="text-align:right;"','WEIGHT_DEC'=>'style="text-align:right;"');
	}

	function tfHeaderProc() {
		return array('thAlignRight'=>array('QUANTITY', 'WEIGHT', 'WEIGHT_DEC'));
	}

	function tfSortingHat()
	{
		return array(
			'PERIOD'=>"'SortVal'",
			'DECLARATION_ID'=>"'SortVal'",
			'QUANTITY'=>"'SortVal'",
			'STATUS'=>"'String'",
			'DOCUMENT_ID'=>"'String'",
			'WEIGHT'=>"'Int'",
			'WEIGHT_DEC'=>"'SortVal'",
			'PRODUCT_CATEGORY'=>"'String'",
			'BRAND_ID'=>"'String'",
			'CREDIT_DEBIT_ID'=>"'String'"
			);
	}

	function tfDocumentLink($args)
	{
		$args['hidename'] = true;
		if($this->_showType!=LIST_HTML_OUTPUT)
			return $args['value']?'true':'false';
		return parent::tfDocumentLink($args);
	}

	function tfMonthWithLink($args)
	{
		$display = $this->tfMonth($args);
		if($this->_showType!=LIST_HTML_OUTPUT)
			return $display;
		return '<a href="#" onclick="$$(\'MONTH\').selectedIndex='.$args['value'].';_LoadOverview();">'.$display.'</a>';
	}

	function tfDeclarationLink($args)
	{
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		if(!is_numeric($args['value'])||$this->_showType!=LIST_HTML_OUTPUT)
			return $args['value'];
		return '<a href="#" onclick="$$(\'PARTICIPANT_RELATION_ID\').value='.$args['row']['PARTICIPANT_RELATION_ID'].';$$(\'DECLARATION_ID\').value='.$args['value'].';_LoadOverview();">'.$args['value'].'</a>';
	}

	function tfValueProcessorQuantitySum($args)
	{
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		if(!is_numeric($args['value']))
			return $args['value'];
		$this->_maxQuantity += $args['value'];
		$this->_cols = $args['visibleColumns'];
    	return locale_nf($args['value']);
	}

	function tfValueProcessorQuantity($args)
	{
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		if(!is_numeric($args['value']))
			return $args['value'];
		if($args['head'] == 'WEIGHT_DEC') return locale_nf($args['value'], 3);
    	return locale_nf($args['value']);
	}

	function tfDeclarationStatus($args)
	{
		if(!is_numeric($args['value']))
			return $this->langIf->getText('Correction of data');
		require_once(ELC_CLASSES.'Declaration.php');
		$d = new Declaration($this->db);
		$status = $d->getDisplayStatus($args['value']);
		list($trans, $class) = Declaration::translateStatus($this->langIf,$status);
		if($this->_showType==LIST_HTML_OUTPUT)
    		return '<span class="'.$class.'">'.$trans.'</span>';
    	return $trans;
	}

	function tfFooter() // getClass
	{
		if(!$this->_cols||in_array('QUANTITY',$this->_hiddenCols))
			return false;
		$rv = explode(',',str_repeat(',',(sizeof($this->_cols)-1)));

		$pos = array_flip($this->_cols);
		if($this->_showType==LIST_HTML_OUTPUT)
    		$rv[$pos['QUANTITY']] = '<span style="float:left; font-weight: bold;">'.$this->langIf->getText('Totals').'</span><span style="float:right; font-weight: bold;">'.locale_nf($this->_maxQuantity).'</span><span style="clear:both;"></span>';
    	else
    		$rv[$pos['QUANTITY']] = locale_nf($this->_maxQuantity);
		return $rv;
	}
}
?>
