<?php
require_once(ELC_CLASSES.'Participant.php');
/**
 * @package ELC
 * @subpackage List
 * @since 14-jul-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class DeclarationStatusList extends ELCDefaultList
{
	var $producer_exporter = PARTICIPANT_PRODUCER;
	var $handle = 'DeclarationStatusList';
	var $_maxQuantity = 0;
	var $_maxWeight = 0;
	var $_maxWeightDec = 0;
	var $_cols = false;
	protected $trustee_lists = array();

	function setQuery()
	{
		require_once(ELC_CLASSES.'Declaration.php');
		list($obj,$func) = $this->_where['CRSO_RELATION_ID']->callback;
		$crso_id = $obj->$func('setValue');

		if(!$crso_id)
			return;

		$this->_query = "SELECT
				D.`MONTH`,
				D.PARTICIPANT_RELATION_ID,
				D.CRSO_RELATION_ID,
				R1.NAME AS CRSO_NAME,
				R.NAME AS PARTICIPANT_NAME,
				IF(BIT_OR(DL.STATUS) & (".DECLARATION_DECLARED." | ".DECLARATION_NORMAL." | ".DECLARATION_RESUBMIT.") AND (SUM(DL.QUANTITY) >= 0), %s, 0) as DECLARATION_STARTED,
				IF(BIT_OR(DL.STATUS) & (".DECLARATION_NORMAL."  | ".DECLARATION_LATE." | ".DECLARATION_RESUBMIT.") AND (SUM(DL.QUANTITY) >= 0), %s, 0) as DECLARATION_CLOSED,
				IF(BIT_OR(DL.STATUS) & ".DECLARATION_RESUBMIT." = ".DECLARATION_RESUBMIT.", %s, 0) as DECLARATION_RESUBMIT,
				IF(BIT_OR(DL.STATUS) & ".DECLARATION_FORFEIT." = ".DECLARATION_FORFEIT.", %s, 0) as DECLARATION_FORFEIT,
				'' as FORFEIT_2, '' as FORFEIT_3,
				IF(BIT_OR(DL.STATUS) & ".DECLARATION_LATE." = ".DECLARATION_LATE.", %s, 0) as DECLARATION_LATE,
				SUM(DL.QUANTITY) as AGGREGATED_QUANTITY,
				SUM(DL.WEIGHT) as AGGREGATED_WEIGHT,
				SUM(DL.WEIGHT_DEC) as AGGREGATED_WEIGHT_DEC,
				COUNT(DISTINCT D.PARTICIPANT_RELATION_ID) as PARTS,
				0 as IS_TRUSTEE
				FROM TBL_DECLARATION as D
				INNER JOIN TBL_DECLARATION_LINE as DL
					ON D.DECLARATION_ID=DL.DECLARATION_ID
				INNER JOIN TBL_RELATION R
					ON D.PARTICIPANT_RELATION_ID=R.RELATION_ID
				INNER JOIN TBL_RELATION R1
					ON D.CRSO_RELATION_ID=R1.RELATION_ID";
		$this->_groupBy = "GROUP BY D.`MONTH`, D.`CRSO_RELATION_ID`";
		$this->_orderBy = "ORDER BY AGGREGATED_QUANTITY DESC";
	}

	function _filterQuery()
	{
		$d = currentDate();
		$this->_addHiddenCol(array('AGGREGATED_QUANTITY','AGGREGATED_WEIGHT', 'PARTS', 'AGGREGATED_WEIGHT_DEC'));

		if(!isset($_REQUEST['MONTH'])||!is_numeric($_REQUEST['MONTH']))
			$this->_addHiddenCol(array('IS_TRUSTEE','FORFEIT_2','FORFEIT_3'));
		elseif($_REQUEST['MONTH'] != $d->getMonth())
			$this->_addHiddenCol(array('FORFEIT_2','FORFEIT_3'));

		if(isset($_REQUEST['PARTICIPANT_RELATION_ID'])&&is_numeric($_REQUEST['PARTICIPANT_RELATION_ID']))
			$this->_addHiddenCol(array('IS_TRUSTEE'));
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_CRSO:
				require_once(ELC_CLASSES.'Country.php');
				require_once(ELC_CLASSES.'CRSO.php');
				require_once(ELC_CLASSES.'RelationUser.php');
				$rel = new Relation($this->db,$_SESSION['crso_relation_id']);
				$c = new Country($this->db,$rel->getCountry(),true);
				$ru = new RelationUser($this->db);

				// Show aggregated quantity if we are allowed to see details.
				if($c->has(HasCrsoDeclarationDetails,$_SESSION['selected_product_id'])&& $ru->hasClearance(CRSOClearanceDeclarationDetails, $_SESSION['session_user_id'], $_SESSION['crso_relation_id'])) {
					unset($this->_hiddenCols['AGGREGATED_QUANTITY']);
					if($c->has(HasDeclOfWeight,$_SESSION['selected_product_id']))
						unset($this->_hiddenCols['AGGREGATED_WEIGHT']);
					if($c->has(HasDeclOfWeightWithDecimals,$_SESSION['selected_product_id']))
						unset($this->_hiddenCols['AGGREGATED_WEIGHT_DEC']);
				}
			break;
			default:
				unset($this->_hiddenCols['AGGREGATED_QUANTITY']);
				unset($this->_hiddenCols['AGGREGATED_WEIGHT']);
				unset($this->_hiddenCols['AGGREGATED_WEIGHT_DEC']);
		}
		$sub = "(SELECT COUNT(DISTINCT D1.PARTICIPANT_RELATION_ID) FROM TBL_DECLARATION as D1
					INNER JOIN TBL_DECLARATION_LINE as DL1 ON D1.DECLARATION_ID = DL1.DECLARATION_ID
					WHERE NOT ISNULL(QUANTITY)
						AND D1.`MONTH` = D.`MONTH`
						AND D1.`YEAR` = D.`YEAR`
						AND DL1.STATUS & %s
						AND D1.CRSO_RELATION_ID = D.CRSO_RELATION_ID)";
		$started = $this->_groupByArray?'1':sprintf($sub,"(".DECLARATION_DECLARED." | ".DECLARATION_NORMAL." | ".DECLARATION_LATE." | ".DECLARATION_RESUBMIT.")");
		$closed =  $this->_groupByArray?'1':sprintf($sub,"(".DECLARATION_NORMAL." | ".DECLARATION_LATE." | ".DECLARATION_RESUBMIT.")");
		$resubmit = $this->_groupByArray?'1':sprintf($sub,DECLARATION_RESUBMIT);
		$forfeit = $this->_groupByArray?'1':sprintf($sub,DECLARATION_FORFEIT);
		$late = $this->_groupByArray?'1':sprintf($sub,DECLARATION_LATE);
		$this->_addHiddenCol('CRSO_RELATION_ID');
		if(!$this->_groupByArray)
			$this->_addHiddenCol('PARTICIPANT_NAME','IS_TRUSTEE');
		return sprintf($this->_query,$started,$closed,$resubmit,$forfeit,$late);
	}

	function _setWhere()
	{
		$this->_addWhere('YEAR',
			array('field'=>'D.`YEAR`',
				'callback' => array($this,'getYearSelect'),
				'required' => true));

		$this->_addWhere('CRSO_RELATION_ID',
				array('field'=>'D.`CRSO_RELATION_ID`',
					'callback' => array($this,'getCRSOSelect'),
					'required'=>true,
					'related'=>array('CRSO_NAME')));
		$this->_addWhere('PARTICIPANT_RELATION_ID',
			array('field'=>'D.`PARTICIPANT_RELATION_ID`',
				'callback' => array($this,'getParticipantSelect'),
				'related'=>array('PARTICIPANT_NAME'),
				'groupBy'=>'D.`PARTICIPANT_RELATION_ID`'));
		$this->_addWhere('MONTH',
			array('field'=>'D.`MONTH`',
				'callback' => array($this,'getMonthSelect'),
				'groupBy'=>'D.`PARTICIPANT_RELATION_ID`'));
		$this->_addWhere('PRODUCT_ID',
			array(
				'field'=>'D.PRODUCT_ID',
				'callback' => array($this,'getProductSelect')));
	}

	function getProcessColumns()
	{
		return array(
			'tfNumberProcessor'=>array('DECLARATION_STARTED','DECLARATION_CLOSED','DECLARATION_RESUBMIT','DECLARATION_FORFEIT','DECLARATION_LATE'),
			'tfValueProcessorQuantityNoDecimals'=>array('AGGREGATED_QUANTITY', 'AGGREGATED_WEIGHT'),
			'tfValueProcessorQuantity'=>array('AGGREGATED_WEIGHT_DEC'),
			'tfMonth'=>array('MONTH'),
			'tfLongNames'=>array('PARTICIPANT_NAME'),
			'tfTrustee'=>array('IS_TRUSTEE'),
			'tfForfeit'=>array('FORFEIT_2','FORFEIT_3')
			);
	}

	function tfForfeit($args)
	{
		$val = 0;

		$sql = "SELECT COUNT(DISTINCT D.DECLARATION_ID) as FF FROM TBL_DECLARATION as D
			INNER JOIN TBL_DECLARATION_LINE as DL
				ON D.DECLARATION_ID = DL.DECLARATION_ID
			LEFT JOIN TBL_DECLARATION as D2
				ON D.DECLARATION_ID = D2.PARENT_DECLARATION_ID
			WHERE DL.STATUS = ?
				AND ISNULL(D2.PARENT_DECLARATION_ID)
				AND D.PARTICIPANT_RELATION_ID = ?
				AND D.CRSO_RELATION_ID = ?
				AND D.PRODUCT_ID = ?
				AND D.`YEAR` = ?
				AND D.`MONTH` = ?
			GROUP BY D.DECLARATION_ID";
		$q = $this->db->query($sql);
		$q->bind_values(DECLARATION_FORFEIT,$args['row']['PARTICIPANT_RELATION_ID'],
			$args['row']['CRSO_RELATION_ID'],$_SESSION['selected_product_id'],$this->getYearSelect(),$args['row']['MONTH']);
		$q->execute();
		$row = $q->fetchAll();
		if(!count($row))
			return $this->tfBooleanIcon($val);

		$r = current($row);

		if($r->FF == 0)
			return $this->tfBooleanIcon($val);
		elseif($r->FF == 2)
			$val = 1;
		elseif(($args['head']=='FORFEIT_2')&& $r->FF==1)
			$val = 1;
		return $this->tfBooleanIcon($val);
	}

	function getColumnModifier()
	{
		return array('AGGREGATED_QUANTITY'=>'style="text-align:right;"', 'AGGREGATED_WEIGHT'=>'style="text-align:right;"', 'AGGREGATED_WEIGHT_DEC'=>'style="text-align:right;"');
	}

	function tfHeaderProc() {
		return array('thAlignRight'=>array('AGGREGATED_QUANTITY', 'AGGREGATED_WEIGHT', 'AGGREGATED_WEIGHT_DEC'));
	}

	function getHiddenColumns()
	{
		return array_merge($this->_hiddenCols,array('PARTICIPANT_RELATION_ID'));
	}

	function getPostScript()
	{
		$rv = $tmp = '';
		if(!count($this->trustee_lists))
			return '';
		$rv = 'if(typeof help != \'undefined\') {
					%s
				}';
		foreach($this->trustee_lists as $id => $text)
			$tmp .= sprintf("help.registerById('trusteeList_%s', '%s');\n",$id,$text);
		return sprintf($rv,$tmp);
	}

	function tfNumberProcessor($args)
	{
		if($args['value']<3)
			return $this->tfBooleanIcon($args);
		return $args['value'];
	}

	function tfValueProcessor($args)
	{
		switch($args['head'])
		{
			case '':
			break;
		}
	}

	function tfValueProcessorQuantity($args)
	{
		$this->_maxWeightDec += $args['value'];
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		return locale_nf($args['value'], 3);
	}

	function tfValueProcessorQuantityNoDecimals($args)
	{
		if($args['head']=='AGGREGATED_QUANTITY')
			$this->_maxQuantity += $args['value'];
		else
			$this->_maxWeight += $args['value'];
		$this->_cols = $args['visibleColumnCount'];
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
    	return locale_nf($args['value']);
	}

	function tfFooter() // getClass
	{
		if(!$this->_cols||in_array('AGGREGATED_QUANTITY',$this->_hiddenCols))
			return false;

		$rv = explode(',',str_repeat(',',($this->_cols-3)));
		$rv[] = '<span style="float:left; font-weight: bold;">Total</span><span style="float:right; font-weight: bold;">'.locale_nf($this->_maxQuantity).'</span><span style="clear:both;"></span>';
		if(!in_array('AGGREGATED_WEIGHT',$this->_hiddenCols))
			$rv[] = '<span style="float:right; font-weight: bold;">'.locale_nf($this->_maxWeight).'</span><span style="clear:right;"></span>';
		if(!in_array('AGGREGATED_WEIGHT_DEC',$this->_hiddenCols))
			$rv[] = '<span style="float:right; font-weight: bold;">'.locale_nf($this->_maxWeightDec, 3).'</span><span style="clear:right;"></span>';
		return $rv;
	}

	function tfSortingHat()
	{
		return array(
			'PARTICIPANT_NAME'=>"'String'",
			'AGGREGATED_QUANTITY'=>"'SortVal'",
			'AGGREGATED_WEIGHT'=>"'SortVal'",
			'AGGREGATED_WEIGHT_DEC'=>"'SortVal'",
			'DECLARATION_STARTED'=>"'String'",
			'DECLARATION_CLOSED'=>"'String'",
			'DECLARATION_RESUBMIT'=>"'String'",
			'DECLARATION_FORFEIT'=>"'String'",
			'FORFEIT_2'=>"'String'",
			'FORFEIT_3'=>"'String'",
			'DECLARATION_LATE'=>"'String'",
			'MONTH'=>"'SortVal'"
			);
	}
}
?>
