<?php
/**
 * @package
 * @subpackage
 * @since 27-sep-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class ImportVsPutOnMarketList extends ELCDefaultList
{
	var $handle = 'ImportVsPutOnMarketList';
	var $_totals = array('PutOnMarketQuantity'=>0,'ImportedQuantity'=>0,'ExportedQuantity'=>0);
	var $_row = array();
	function setQuery()
	{
		$this->_query = "SELECT R.`NAME` as PARTICIPANT_NAME,
				R2.`NAME` as EXPORTER_NAME,
				C.`NAME` as EXPORTER_COUNTRY,
				D.`MONTH`,
				CONCAT(ROUND((SUM(RRL.QUANTITY) / SUM(DL.QUANTITY)) * 100), ' %') as PercentageOfImportedPutOnMarket,
				SUM(DL.QUANTITY) as PutOnMarketQuantity,
				SUM(RRL.QUANTITY) as ImportedQuantity,
				SUM(RRL2.QUANTITY) as ExportedQuantity
                        FROM TBL_DECLARATION as D
			INNER JOIN TBL_DECLARATION_LINE as DL
				ON D.DECLARATION_ID = DL.DECLARATION_ID
			INNER JOIN TBL_RELATION as R
				ON D.PARTICIPANT_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_REFUND_REQUEST as RR
				ON REPLACE(LOWER(RR.IMPORTER_VAT_NO), ' ', '') = REPLACE(LOWER(R.VAT_NO),' ','')
					AND D.`MONTH` = MONTH(RR.REFUND_EXPORT_DATE)
			INNER JOIN TBL_REFUND_REQUEST_LINE as RRL
				ON RR.REFUND_REQUEST_ID = RRL.REFUND_REQUEST_ID
			INNER JOIN TBL_RELATION as R2
				ON RR.PARTICIPANT_RELATION_ID = R2.RELATION_ID
			INNER JOIN TBL_ADDRESS as A
				ON R2.ADDRESS_ID = A.ADDRESS_ID
			INNER JOIN TBL_COUNTRY as C
				ON A.COUNTRY_ID = C.COUNTRY_ID
			LEFT JOIN TBL_REFUND_REQUEST as RR2
				ON D.PARTICIPANT_RELATION_ID = RR2.PARTICIPANT_RELATION_ID
					AND D.CRSO_RELATION_ID = RR2.CRSO_RELATION_ID
					AND D.`MONTH` = MONTH(RR2.REFUND_EXPORT_DATE)
			LEFT JOIN TBL_REFUND_REQUEST_LINE as RRL2
				ON RR2.REFUND_REQUEST_ID = RRL2.REFUND_REQUEST_ID";

		$this->_groupBy = "GROUP BY D.PARTICIPANT_RELATION_ID, D.`MONTH`, RR.PARTICIPANT_RELATION_ID";
	}

	function _setWhere()
	{

		$this->_addWhere('YEAR',
			array('field'=>'D.`YEAR`',
				'callback' => array($this,'getYearSelect'),
				'required' => true));

		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'D.CRSO_RELATION_ID',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true));

		$this->_addWhere('MONTH',
			array(
				'field'=>'D.MONTH',
				'callback' => array($this,'getMonthSelect'),
				'required'=>true));
	}


        function _filterQuery()
        {

            if($_SESSION['session_group_group_id'] == GROUP_CRSO)
            {
                require_once(ELC_CLASSES.'Country.php');
                require_once(ELC_CLASSES.'CRSO.php');
                require_once(ELC_CLASSES.'RelationUser.php');
                $relation = new Relation($this->db);
                $country = new Country($this->db);
                $country->set($relation->getCountry($_SESSION['crso_relation_id']));
                $hiddenCols = array('EXPORTER_NAME', 'EXPORTER_COUNTRY');

                $ru = new RelationUser($this->db);

                if(!$country->has(HasCrsoDeclarationDetails,$_SESSION['selected_product_id'])||!$ru->hasClearance(CRSOClearanceDeclarationDetails, $_SESSION['session_user_id'], $_SESSION['crso_relation_id']))
                {
                    $hiddenCols = array_merge($hiddenCols, array('PutOnMarketQuantity', 'ImportedQuantity', 'ExportedQuantity'));
                }
                $this->_addHiddenCol($hiddenCols);
            }

            return $this->_query;

        }


	function getProcessColumns()
	{
		$rv = array(
			'tfValueProcessorQuantity'=>array('PutOnMarketQuantity','ImportedQuantity','ExportedQuantity'),
			'tfLongNames'=>array('PARTICIPANT_NAME','EXPORTER_NAME')
			);

		return $rv;
	}

	function tfValueProcessorQuantity(&$args)
	{
		$this->_row = array_keys($args['row']);
		$this->_totals[$args['head']] += $args['value'];
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);

		if(!is_numeric($args['value']))
			return 'n/d';
    	return locale_nf($args['value'], 0);
	}

	function getColumnModifier()
	{
		return array(
                        'PercentageOfImportedPutOnMarket'=>'style="text-align:right;"',
                        'PutOnMarketQuantity'=>'style="text-align:right;"',
			'ImportedQuantity'=>'style="text-align:right;"',
			'ExportedQuantity'=>'style="text-align:right;"');
	}

	function tfFooter()
	{
		$rv = array();
		$showCols = array_diff($this->_row,$this->_hiddenCols);
		foreach($showCols as $column)
			$rv[$column]=isset($this->_totals[$column])?'<div style="text-align: right; font-weight: bold;">'.(locale_nf($this->_totals[$column],0)).'</div>':'';
		return $rv;
	}
}
?>
