<?php
/**
 * @package ELC
 * @subpackage List
 * @since 04-06-2007
 * @author Nick Loeve <nick@basesoft.com>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class OperatorSummaryList extends ELCDefaultList
{
	var $handle = 'OperatorSummaryList';

	function setQuery()
	{
		//if the export month is also needed in the list, just uncomment below
		$this->_query = "
			SELECT
				R.`CRSO_NAME` AS CRSO_NAME,
				P.`NAME` AS PRODUCT_NAME,
				R.JOURNAL_TEMPLATE AS JOURNAL_CODE,
				R.EXPORT_DATE AS EXPORT_DATE,
				R.MASTERDATA_HASH AS MASTERDATA_HASH,
                                IF(R.JOURNAL_TEMPLATE IN (CONCAT('I', R.CRSO_RELATION_ID), CONCAT('C', R.CRSO_RELATION_ID)),
                                        R.DECLARATION_HASH, R.REFUND_HASH) AS EXPORT_HASH,
                                R.QUANTITY AS QUANTITY,
                                R.EXPORTED_AMOUNT AS TOTAL_FEE,
				R.DOCUMENTS_UPLOADED AS ALL_DOCUMENTS_ATTACHED
			FROM
				TBL_OPERATOR_CRSO_EXPORT_SUMMARY AS R
			INNER JOIN
				TBL_PRODUCT AS P
					ON R.PRODUCT_ID = P.PRODUCT_ID
			";

		$this->_groupBy = "";
		$this->_orderBy = "ORDER BY CRSO_NAME, EXPORT_DATE DESC, JOURNAL_TEMPLATE";

	}

	function _setWhere()
	{
		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'R.`CRSO_RELATION_ID`',
				'callback' => array($this,'getCRSOSelect')));
		$this->_addWhere('PRODUCT_ID',
			array(
				'field'=>'R.PRODUCT_ID',
				'callback' => array($this,'getDynamicProductSelect')));
                $this->_addWhere('JOURNAL_TEMPLATE',
			array(
				'field'=>'R.JOURNAL_TEMPLATE',
                                'operator' => 'LIKE',
				'callback' => array($this,'getJournalSelect')));
		$this->_addWhere('MONTH',
			array(
				'field'=>'MONTH(R.EXPORT_DATE)',
				'callback' => array($this,'getMonthSelect')));
		$this->_addWhere('YEAR',
			array(
				'field'=>'YEAR(R.EXPORT_DATE)',
				'callback' => array($this,'getYearSelect'),
				'required'=>true));
	}

        function getJournalSelect($rv='setValue')
	{
		$templates = array('Declaration Invoice' => 'I%', 'Declaration Credit' => 'C%',
                    'Refund Invoice' => 'RI%', 'Refund Credit' => 'RC%');

		$setValue = !empty($_REQUEST['JOURNAL_TEMPLATE']) ?
			$_REQUEST['JOURNAL_TEMPLATE']:false;

                if($setValue && !in_array($setValue, $templates))
                    throw new IWException('Invalid list param!');

		if(isset($$rv))
			return $$rv;
		$selectedValue = '';

		$select = new Form_Select('JOURNAL_TEMPLATE');

		$select->addOption(' - '.$this->getText('Select Journal').' - ', '', false);

		foreach($templates as $key => $value)
		{
			if($setValue==$value)
				$selectedValue = $value;
			$select->addOption($key, $value, false);
		}

		$select->setSelectedValue($setValue);
		$filter = $select->getHtml();
		return $$rv;
	}

	function getMonthSelect($rv='setValue')
	{
		$setValue = !empty($_REQUEST['MONTH'])?(int)$_REQUEST['MONTH']:false;
		return parent::getMonthSelect($rv);
	}

        function getYearSelect($rv='setValue',$maxyear=false)
	{
		$date = currentDate();
		$year = (int)($maxyear?$maxyear:$date->getYear());

		$setValue = !empty($_REQUEST['YEAR']) ?
			(int)$_REQUEST['YEAR']:false;

		if(isset($$rv))
			return $$rv;
		$selectedValue = '';

		$select = new Form_Select('YEAR');

		if($year != WEEE_APPLICATION_START_YEAR)
		{
			$select->addOption(' - '.$this->getText('Year Select').' - ', '', false);
		}
		else
		{
			$filter = '';
			$selectedValue = WEEE_APPLICATION_START_YEAR;
			return $$rv;
		}

		for($i=WEEE_APPLICATION_START_YEAR; $i<=$year; $i++)
		{
			if($setValue==$i)
				$selectedValue = $i;
			$select->addOption($i, $i, false);
		}

		$select->setSelectedValue($setValue);
		$filter = $select->getHtml();
		return $$rv;
	}

	function getProcessColumns()
	{
		$rv = array(
			'tfBooleanIcon'=>array('ALL_DOCUMENTS_ATTACHED'),
			'tfMonth'=>array('EXPORT_MONTH'),
			'tfValueProcessorQuantityNoDecimals'=>array('QUANTITY'),
			'tfValueProcessorQuantity'=>array('TOTAL_FEE'));
		return $rv;
	}

	function tfSortingHat()
	{
		return array(
			'QUANTITY'=>"'SortVal'",
			'TOTAL_FEE'=>"'SortVal'",
			'EXPORT_DATE'=>"'SortVal'");
	}

	function getColumnModifier()
	{
		return array('QUANTITY'=>'style="text-align:right;"','TOTAL_FEE'=>'style="text-align:right;"');
	}

	function setEmptyText()
	{
		return $this->langIf->getText('No Report for the current filter');
	}

        function getButtons()
        {
            if(!empty($_REQUEST['YEAR']))
                return array_merge($this->_getCSVButton(), $this->_getXMLButton());
            return array();
        }
}
?>
