<?php
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'Participant.php');
/**
 * @package ELC
 * @subpackage List
 * @since 6-sep-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class ParticipantRegistrationList extends ELCDefaultList
{
	var $handle = 'ParticipantRegistrationList';
	var $producer_exporter;

	function setQuery()
	{
		$this->_query = "SELECT P.PARTICIPANT_RELATION_ID, P.CREATED, R.`NAME` as RELATION_NAME,
				R.VAT_NO, V.STATUS AS VAT_STATUS,
                                CP.PRODUCER_REG_NO,
				CPP.OBLIGATION_START_DATE as OBLIGATION_START_DATE,
				IF((CPP.PRODUCER_EXPORTER & ".PARTICIPANT_PRODUCER."), 1, 0) as PRODUCER,
				IF((CPP.PRODUCER_EXPORTER & ".PARTICIPANT_EXPORTER."), 1, 0) as EXPORTER,
				C.`NAME` as COUNTRY_NAME,
				CPP.STATUS, CPP.ACTIVE, COUNT(RU.USER_ID) as USERS, CPP.GENERAL_MANAGER_ID as GENERAL_MANAGER, CPP.GENERAL_MANAGER_AUTHORIZED as GENERAL_MANAGER_AUTHORIZED,
				'' as REFUND_SETTINGS
			FROM TBL_CRSO_PARTICIPANT_PRODUCT as CPP
                        INNER JOIN TBL_CRSO_PARTICIPANT as CP
                            ON CPP.CRSO_RELATION_ID = CP.CRSO_RELATION_ID AND
                                CPP.PARTICIPANT_RELATION_ID = CP.PARTICIPANT_RELATION_ID
			INNER JOIN TBL_PARTICIPANT as P
				ON CPP.PARTICIPANT_RELATION_ID = P.PARTICIPANT_RELATION_ID
			INNER JOIN TBL_RELATION as R
				ON P.PARTICIPANT_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_ADDRESS as A
				USING(ADDRESS_ID)
			INNER JOIN TBL_COUNTRY as C
				USING(COUNTRY_ID)
			LEFT JOIN TBL_RELATION_USER as RU
				ON R.RELATION_ID = RU.RELATION_ID
			LEFT JOIN TBL_LANGUAGE as IL
				ON IL.LANGUAGE_ID = R.INVOICE_LANGUAGE_ID
			LEFT JOIN TBL_USER as U
				ON RU.USER_ID = U.USER_ID
			LEFT JOIN TBL_VAT V
				ON R.VAT_NO=V.VAT_NO
			";
		$this->_groupBy = "GROUP BY CPP.PARTICIPANT_RELATION_ID";
		$this->_orderBy = "ORDER BY CPP.`STATUS`, R.`NAME`";

	}

	function _setWhere()
	{
		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'CPP.`CRSO_RELATION_ID`',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true,
				'related'=>array('CRSO_NAME')));

		// TODO: Add Relation User check for refund/producer

		$this->_addWhere('PRODUCER_EXPORTER',
			array(
				'field'=>'CPP.`PRODUCER_EXPORTER`',
				'operator' => '&',
				'callback' => array($this,'getProducerExporterSelect')));



		$this->_addWhere('STATUS',
			array(
				'field'=>'CPP.`STATUS`',
				'callback' => array($this,'getStatusSelect')));
		$this->_addWhere('ACTIVE',
			array(
				'field'=>'CPP.`ACTIVE`',
				'callback' => array($this,'getActiveSelect')));
		$this->_addWhere('PRODUCT_ID',
			array(
				'field'=>'CPP.PRODUCT_ID',
				'callback' => array($this,'getProductSelect')));
	}

	function getStatusSelect($rv)
	{
		$setValue = isset($_REQUEST['STATUS'])?$_REQUEST['STATUS']:false;
		if(isset($$rv))
			return $$rv;
		$selectedValue = ' - '.$this->getText('All Status').' - ';

		$select = new Form_Select('STATUS');
		$select->addOption(' - '.$this->getText('All Status').' - ', '', false);

		$stati = array(
			PARTICIPANT_STATUS_NEW,
			PARTICIPANT_STATUS_VERIFIED,
			PARTICIPANT_STATUS_AUTHORIZED,
			PARTICIPANT_STATUS_CONFIRMED,
			PARTICIPANT_STATUS_PARTICIPANT_READY
			);

		foreach($stati as $status)
		{
			list($trans,$class) = Participant::translateStatus($this->langIf, $status);
			if($setValue==$status)
				$selectedValue = $trans;

			$select->addOption($trans, $status, false);
		}
		$select->setSelectedValue($setValue);
		$filter = $select->getHtml();
		return $$rv;
	}

	function getActiveSelect($rv)
	{
		$setValue = isset($_REQUEST['ACTIVE'])?$_REQUEST['ACTIVE']:'';
		if(isset($$rv))
			return $$rv;
		$selectedValue = ' - '.$this->getText('All Active and In-active').' - ';

		$select = new Form_Select('ACTIVE');
		$select->addOption(' - '.$this->getText('All Active and In-active').' - ', '', false);

		$active = array(
			PARTICIPANT_STATUS_ACTIVE => 'Active',
			0 => 'In-Active'
			);

		foreach($active as $id => $lang)
		{
			if($setValue===$id)
				$selectedValue = $this->getText($lang);
			$select->addOption($this->getText($lang), $id, false);
		}
		if($setValue!=='')
			$select->setSelectedValue($setValue);
		$filter = $select->getHtml();
		return $$rv;
	}

	function getProducerExporterSelect($rv)
	{
		$setValue = isset($_REQUEST['PRODUCER_EXPORTER'])?$_REQUEST['PRODUCER_EXPORTER']:false;
		if(isset($$rv))
			return $$rv;
		$selectedValue = ' - '.$this->getText('All Participant Types').' - ';

		$select = new Form_Select('PRODUCER_EXPORTER');
		$select->addOption(' - '.$this->getText('All Participant Types').' - ', '', false);

		$producer_exporter = array(
			PARTICIPANT_PRODUCER => 'Producer',
			PARTICIPANT_EXPORTER => 'Exporter'
			);

		foreach($producer_exporter as $id => $lang)
		{
			if($setValue==$id)
				$selectedValue = $this->getText($lang);
			$select->addOption($this->getText($lang), $id, false);
		}
		$select->setSelectedValue($setValue);
		$filter = $select->getHtml();
		return $$rv;
	}


	function getProcessColumns()
	{
		return array(
			'tfBooleanIcon'=> array(
				'PRODUCER',
				'EXPORTER',
				'ACTIVE'
			),
			'tfRelationIdLink'=> array('PARTICIPANT_RELATION_ID'),
			'tfUserOverviewLink'=> array('USERS'),
			'tfVatStatus'=> array('VAT_NO'),
			'tfGeneralManager'=> array('GENERAL_MANAGER'),
			'tfParticipantStatus'=> array('STATUS'),
			'tfMakePrettyDate'=> array('OBLIGATION_START_DATE'),
			'tfRefundSettings' => array('REFUND_SETTINGS')
		);
	}

	function getHiddenColumns()
	{
		return array_merge($this->_hiddenCols,array('VAT_STATUS','GENERAL_MANAGER_AUTHORIZED'));
	}

	function tfSortingHat()
	{
		return array('STATUS'=>"'SortVal'",
				'RELATION_NAME'=>"'String'",
				'CREATED'=>"'String'",
				'VAT_NO'=>"'String'",
                                'PRODUCER_REG_NO'=>"'String'",
				'OBLIGATION_START_DATE'=>"'String'",
				'COUNTRY_NAME'=>"'String'",
				'PRODUCER'=>"'String'",
				'EXPORTER'=>"'String'",
				'ACTIVE'=>"'SortVal'",
				'PARTICIPANT_RELATION_ID'=>"'SortVal'",
				'USERS'=>"'SortVal'"
				);
	}
	/**#@+
	 * Table Factory value processors
	 */
	/**
	 *
	 */
	function tfRelationIdLink($args)
	{
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		return sprintf('<a href="/%sscripts/elc/crso/InterfaceRegistrantDetails.php?participant[id]=%s">%s</a>',
			SITE_PATH,
			$args['value'],
			$args['value']
			);
	}

	function tfUserOverviewLink($args)
	{
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		return sprintf('<a href="/%sscripts/elc/crso/InterfaceRegistrantUserOverview.php?participant[id]=%s">%s</a>',
			SITE_PATH,
			$args['row']['PARTICIPANT_RELATION_ID'],
			$args['value']
			);
	}

	function tfVatStatus($args)
	{
		require_once(IWARE.'VAT.php');

		// empty status means the query didnt join tbl_vat so the vat is valid
		if ($args['row']['VAT_STATUS'] == '')
			$args['row']['VAT_STATUS'] = VAT_VALID;

		switch ($args['row']['VAT_STATUS'])
		{
			case VAT_INVALID:
				$class='warning';
			break;
			case VAT_PENDING:
				$class='warning';
			break;
			case VAT_VALID:
			default:
				$class='info';
			break;
		}
		return '<span class="'.$class.'">'.$args['value'].'</span>';
	}

	function tfGeneralManager($args)
	{
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		return sprintf('<a id="general_manager_%s" href="InterfaceRegistrantUserDetails.php?user[id]=%s&participant[id]=%s">%s</a>', $args['row']['PARTICIPANT_RELATION_ID'], $args['value'], $args['row']['PARTICIPANT_RELATION_ID'], $args['value']);
	}

	function tfRefundSettings($args)
	{
		$msg = $this->langIf->getText('Refund Settings');

		// only print link for exporters
		if ((bool) $args['row']['EXPORTER'])
			return sprintf('<a id="refund_settings_%s" href="InterfaceRefundSettings.php?&participant[id]=%s">%s</a>', $args['row']['PARTICIPANT_RELATION_ID'], $args['row']['PARTICIPANT_RELATION_ID'], $msg);
	}
	function getButtons()
	{
		$settings = parent::getButtons();
		$settings['Download csv for all participants masterdata'] = array('onclick'=>"$('interface_form').process.value='export'; $('interface_form').submit(); ");
	    return $settings;
	}
	function getDataSet()
	{
		if(isset($_REQUEST['process']) && $_REQUEST['process'] == 'export')
		{
			require_once(ELC_CLASSES.'Participant.php');
			$ids = array();
			/* - if we ever want to export only the filtered participants in the list:
			$q = parent::getDataSet();
			while($r = $q->fetch())
				$ids[] = $r->PARTICIPANT_RELATION_ID;*/
			$part = new Participant($this->db);
			if($_SESSION['session_group_group_id'] == GROUP_SYSADMIN)
			{
				$crso = '';
				$filename = "WEEE_Participants_Masterdata_Export.csv";
			}
			else if($_SESSION['session_group_group_id'] == GROUP_CRSO)
			{
				$crso = $_SESSION['crso_relation_id'];
				$filename = "Masterdata_CRSO".$crso.".csv";
			}

			$csv = $part->export_masterdata($ids,$crso, '', true); // with_active=true
			if (!headers_sent())
			header('Content-Disposition: attachment; filename="'.$filename.'"');
            echo $csv->getString();
            die();
		}
		return parent::getDataSet();
	}
	/**#@-*/
}
?>
