<?php
require_once(ELC_CLASSES.'RefundRequest.php');
/**
 * @package ELC
 * @subpackage List
 * @author Marten Koedam <marten@basesoft.com>
 * @copyright 2015 BaseSoft Solutions B.V.
 */
class PoEDocumentList extends ELCDefaultList {

	var $handle = 'PoEDocumentList';
	private $_maxQuantity = 0;
	private $_cols = 0;

	function setEmptyText() {
		return $this->langIf->getText('No refund export documents found for selected filters');
	}

	function setQuery() {
		// If a refund request id is supplied we filter only that refund request id.
		$rrId = empty($_REQUEST['REFUND_REQUEST_ID']) ? null : $_REQUEST['REFUND_REQUEST_ID'];
		if(!$rrId) {
			$this->_addHiddenCol(array('PRODUCT_CATEGORY', 'PRODUCER', 'BRAND'));
			$this->_groupBy = "GROUP BY rr.REFUND_REQUEST_ID";
		} else {
			$this->_groupBy = "GROUP BY rr.REFUND_REQUEST_ID, rrl.LINE_INDEX";
		}
		$this->_addHiddenCol(array('PARTICIPANT_RELATION_ID'));

		$this ->_query = "SELECT
			rr.REFUND_REQUEST_ID,
			rr.PARTICIPANT_RELATION_ID,
			p.NAME as PARTICIPANT_NAME,
			c.NAME as COUNTRY_NAME,
			pr.NAME as PRODUCER,
			pc.NAME as PRODUCT_CATEGORY,
			b.NAME as BRAND,
			rr.STATUS,
			rr.EXPORT_DOCUMENT_ID as DOCUMENT_ID,
			rr.REFUND_EXPORT_DATE,
			SUM(rrl.QUANTITY) as QUANTITY,
			SUM(rrl.WEIGHT) as WEIGHT,
			SUM(rrl.WEIGHT_DEC) as WEIGHT_DEC
		FROM TBL_REFUND_REQUEST as rr
		INNER JOIN TBL_RELATION as p
			ON rr.PARTICIPANT_RELATION_ID = p.RELATION_ID
		INNER JOIN TBL_COUNTRY as c
			ON rr.EXPORT_COUNTRY_ID = c.COUNTRY_ID
		INNER JOIN TBL_REFUND_REQUEST_LINE as rrl
			ON rr.REFUND_REQUEST_ID = rrl.REFUND_REQUEST_ID
		INNER JOIN TBL_RELATION as pr
			ON rrl.PRODUCER_RELATION_ID = pr.RELATION_ID
		LEFT JOIN TBL_PRODUCT_CATEGORY as pc
			ON rrl.PRODUCT_CATEGORY_ID = pc.PRODUCT_CATEGORY_ID
		LEFT JOIN TBL_BRAND as b
			ON rrl.BRAND_ID = b.BRAND_ID
		";

		if(!isset($_REQUEST['CRSO_RELATION_ID'])) {
			return;
		}
		$r = new Relation($this->db, $_REQUEST['CRSO_RELATION_ID']);
		$c = new Country($this->db, $r->getCountry(),true);

		if (!$c->has(HasDeclOfWeight,$_SESSION['selected_product_id'])) {
			$this->_addHiddenCol('WEIGHT');
		}

		if (!$c->has(HasDeclOfWeightWithDecimals,$_SESSION['selected_product_id'])) {
			$this->_addHiddenCol('WEIGHT_DEC');
		}

		if (!$c->has(HasDeclByCategory,$_SESSION['selected_product_id'])) {
			$this->_addHiddenCol('PRODUCT_CATEGORY');
		}

		if (!$c->has(HasDeclByBrand,$_SESSION['selected_product_id'])) {
			$this->_addHiddenCol('BRAND');
		}

	}

	function _setWhere() {
		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'rr.CRSO_RELATION_ID',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true
			)
		);
		$this->_addWhere('PARTICIPANT_RELATION_ID',
			array(
				'field'=>'rr.`PARTICIPANT_RELATION_ID`',
				'callback' => array($this,'getParticipantSelect'),
				'related'=>array('PARTICIPANT_NAME'),
				'groupBy'=>''
			)
		);
		$this->_addWhere('REFUND_EXPORT_YEAR',
			array(
				'field' => 'YEAR(rr.REFUND_EXPORT_DATE)',
				'callback' => array($this, 'getExportYear'),
				'groupBy' => '',
				'required' => true
			)
		);
		$this->_addWhere('STATUS',
			array(
				'field' => 'rr.STATUS',
				'callback' => array($this, 'getStatusSelect'),
				'groupBy' => '',
				'required' => false
			)
		);
		$this->_addWhere('PRODUCT_ID', array('field' => 'rr.PRODUCT_ID', 'callback' => array($this, 'getProductSelect')));
		$this->_addWhere('REFUND_REQUEST_ID', array('field' => 'rr.REFUND_REQUEST_ID', 'callback' => array($this, 'getRefundRequestInput')));
	}

	function getExportYear($rv = 'setValue') {

		$setValue = isset($_REQUEST['REFUND_EXPORT_YEAR']) ? $_REQUEST['REFUND_EXPORT_YEAR'] : false;
		$selectedValue = '';

		if (isset($$rv))
			return $$rv;

		if (isset($_REQUEST['CRSO_RELATION_ID'])) {

			$q = $this -> db->query('SELECT DISTINCT YEAR(REFUND_EXPORT_DATE) AS `YEAR` FROM TBL_REFUND_REQUEST WHERE CRSO_RELATION_ID = ? AND PRODUCT_ID = ? ORDER BY `YEAR` DESC');
			$q->bind_values($_REQUEST['CRSO_RELATION_ID'], $_SESSION['selected_product_id']);
			$q->execute();

			$select = new Form_Select('REFUND_EXPORT_YEAR');
			while ($r = $q->fetch()) {
				$select -> addOption($r->YEAR, $r->YEAR, false);
				$lastValue = $r->YEAR;
			}

			if ($setValue) {
				$select->setSelectedValue($setValue);
				$selectedValue = $setValue;
			}

			$filter = $select->getHtml();

		} else
			$filter = '';

		return $$rv;
	}

	function getStatusSelect($rv = 'setValue') {
		$filter = '';

		$stati = array(REFUND_REQUEST_PENDING,REFUND_REQUEST_APPROVED,REFUND_REQUEST_REJECTED,REFUND_REQUEST_CONFIRMED);
		if(isset($_REQUEST['STATUS'])&&!in_array(isset($_REQUEST['STATUS']),$stati))
			throw new IWAJAXPermissionException('Incorrect Status Provided');

		$setValue = isset($_REQUEST['STATUS'])?$_REQUEST['STATUS']:'';
		$selectedValue = ' - '.$this->getText('All Status').' - ';

		$select = new Form_Select('STATUS');
		$select->addOption(' - '.$this->getText('All Status').' - ', '', false);

		foreach($stati as $id)
		{
			list($trans,$class) = RefundRequest::translateStatus($this->langIf,$id);
			if($setValue==$id)
				$selectedValue = $trans;
			$select->addOption($trans, $id, false);
		}
		$select->setSelectedValue($setValue);
		$filter = $select->getHtml();

		return $$rv;
	}

	function getProcessColumns() {
		$rv = array(
			'tfRefundLink'=>array('REFUND_REQUEST_ID'),
			'tfStatus'=>array('STATUS'),
			'tfValueProcessorQuantity'=>array('WEIGHT', 'WEIGHT_DEC'), // TODO, weight dec
			'tfValueProcessorQuantitySum'=>array('QUANTITY'),
			'tfDocumentLink'=>array('DOCUMENT_ID'),
			'tfParticipantLink'=>array('PARTICIPANT_NAME'),
		);
		return $rv;
	}

	function getRefundRequestInput($rv='setValue') {
		$setValue = isset($_REQUEST['REFUND_REQUEST_ID'])?$_REQUEST['REFUND_REQUEST_ID']:'';
		$selectedValue = '';
		$filter = '<input type="hidden" name="REFUND_REQUEST_ID" value="">';
		return $$rv;
	}

	function getColumnModifier() {
		return array('QUANTITY'=>'style="text-align:right;"','WEIGHT'=>'style="text-align:right;"','WEIGHT_DEC'=>'style="text-align:right;"');
	}

	function tfHeaderProc() {
		return array('thAlignRight'=>array('QUANTITY', 'WEIGHT', 'WEIGHT_DEC'));
	}

	function tfValueProcessorQuantitySum($args) {
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		if(!is_numeric($args['value']))
			return $args['value'];
		$this->_maxQuantity += $args['value'];
		$this->_cols = $args['visibleColumns'];
		return locale_nf($args['value']);
	}

	function tfRefundLink($args) {
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		if(!is_numeric($args['value'])||$this->_showType!=LIST_HTML_OUTPUT)
			return $args['value'];
		return '<a href="#" onclick="$$(\'PARTICIPANT_RELATION_ID\').value='.$args['row']['PARTICIPANT_RELATION_ID'].';$$(\'REFUND_REQUEST_ID\').value='.$args['value'].';_LoadOverview();">'.$args['value'].'</a>';
	}

	function tfParticipantLink($args) {
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		if($this->_showType!=LIST_HTML_OUTPUT)
			return $args['value'];
		return '<a href="#" onclick="$$(\'PARTICIPANT_RELATION_ID\').value='.$args['row']['PARTICIPANT_RELATION_ID'].';_LoadOverview();">'.$args['value'].'</a>';
	}

	function tfDocumentLink($args) {
		$args['hidename'] = true;
		if($this->_showType!=LIST_HTML_OUTPUT)
			return $args['value']?'true':'false';
		return parent::tfDocumentLink($args);
	}

	function tfStatus($args) {

		list($value, $cssClass) = RefundRequest::translateStatus($this->langIf, $args['value']);
		return '<span class="'.$cssClass.'">'.$value.'</span>';
	}

	function tfFooter() {
		if(!$this->_cols)
			return false;
		$rv = explode(',',str_repeat(',',(sizeof($this->_cols)-1)));

		$pos = array_flip($this->_cols);
		if($this->_showType==LIST_HTML_OUTPUT)
			$rv[$pos['QUANTITY']] = '<span style="float:left; font-weight: bold;">'.$this->langIf->getText('Totals').'</span><span style="float:right; font-weight: bold;">'.locale_nf($this->_maxQuantity).'</span><span style="clear:both;"></span>';
		else
			$rv[$pos['QUANTITY']] = locale_nf($this->_maxQuantity);
		return $rv;
	}

}

?>