<?php
/**
 * Created by PhpStorm.
 * User: marten
 * Date: 6/7/16
 * Time: 11:17 AM
 */
require_once(ELC_CLASSES.'ProofOfPayment.php');

class PoPDetailsList extends ELCDefaultList {
	var $handle = 'PoPDetailsList';
	var $producer_exporter = PARTICIPANT_EXPORTER;
	var $_maxNum = array();
	var $_headType = array();
	var $_cols = false;

	function setQuery() {

		$this->_query = "SELECT
				pop.PROOF_OF_PAYMENT_ID,
				pop.PARTICIPANT_RELATION_ID, r.NAME as PARTICIPANT_NAME,
				pop.PRODUCER_RELATION_ID, r2.NAME as PRODUCER_NAME,
				pl.INVOICE_ID,
				pl.PAYMENT_DATE,
				pl.QUANTITY
			FROM TBL_PROOF_OF_PAYMENT as pop
			INNER JOIN TBL_PROOF_OF_PAYMENT_LINE as pl
				ON pop.PROOF_OF_PAYMENT_ID = pl.PROOF_OF_PAYMENT_ID
			INNER JOIN TBL_RELATION as r
				ON pop.PARTICIPANT_RELATION_ID = r.RELATION_ID
			INNER JOIN TBL_RELATION as r2
				ON pop.PRODUCER_RELATION_ID = r2.RELATION_ID
		";

		$this->_orderBy = "ORDER BY pop.PROOF_OF_PAYMENT_ID";

	}

	function _filterQuery() {
		$this->_addHiddenCol(array('PARTICIPANT_RELATION_ID', 'PRODUCER_RELATION_ID'));
		$this->_addGroupBy(array('pl.`LINE_INDEX`'));
		return $this->_query;
	}

	function _setWhere() {
		$this->_addWhere('CRSO_RELATION_ID',
			array('field'=>'pop.`CRSO_RELATION_ID`',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true));
		$this->_addWhere('PARTICIPANT_RELATION_ID',
			array('field'=>'pop.`PARTICIPANT_RELATION_ID`',
				'callback' => array($this,'getParticipantSelect'),
				'related'=>array('PARTICIPANT_NAME')));
		$this->_addWhere('PRODUCER_RELATION_ID',
			array('field'=>'pop.`PRODUCER_RELATION_ID`',
				'callback' => array($this,'getProducerSelect'),
				'related'=>array('PRODUCER_NAME')));
		$this->_addWhere('YEAR',
			array('field'=>'YEAR(pl.`PAYMENT_DATE`)',
				'required'=>true,
				'callback' => array($this,'getYearSelect'),
				'groupBy'=>'pop.`PROOF_OF_PAYMENT_ID`'));
		$this->_addWhere('PRODUCT_ID',
			array(
				'field'=>'pop.PRODUCT_ID',
				'callback' => array($this,'getProductSelect')));

		$this->_addWhere('STATUS',
			array(
				'field' => 'pop.STATUS',
				'callback' => array($this, 'getStatusSelect'),
				'required'=>true,
				'related' => array(),
				'groupBy' => ''
			));
	}

	function getStatusSelect($rv='setValue')
	{
		$filter = '';
		$setValue = PROOF_OF_PAYMENT_APPROVED;
		$selectedValue = '';
		return $$rv;
	}

	function getProducerSelect($rv='setValue') {
		$setValue = !empty($_REQUEST['PRODUCER_RELATION_ID'])?(int)$_REQUEST['PRODUCER_RELATION_ID']:false;

		$crso_id = isset($_REQUEST['CRSO_RELATION_ID'])?(int)$_REQUEST['CRSO_RELATION_ID']:false;
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_CRSO:
				$crso_id = $_SESSION['crso_relation_id'];
				break;
		}

		if(!is_numeric($crso_id))
			$filter = '';
		if(isset($$rv))
			return $$rv;
		$selectedValue = '';

		require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
		$p = new CRSOParticipantProduct($this->db);

		$producer_exporter = PARTICIPANT_PRODUCER;

		$is_active = null;
		$q = $p->getCRSOParticipants($crso_id,'q',$producer_exporter, $_SESSION['selected_product_id'], $is_active);

		$select = new Form_Select('PRODUCER_RELATION_ID');
		$select->addOption(' - '.$this->getText('Producer Select').' - ', '', false);
		while($r = $q->fetch())
		{
			if($setValue==$r->PARTICIPANT_RELATION_ID)
				$selectedValue = $r->PARTICIPANT_NAME;
			if($_SESSION['session_group_group_id'] == GROUP_PARTICIPANT)
			{
				if(!in_array($r->PARTICIPANT_RELATION_ID, $_SESSION['participant_ids']))
					continue;
			}
			$name = $r->PARTICIPANT_NAME;
			$select->addOption(((mb_strlen($name)>35)?mb_substr($name,0,35).'...':$name), $r->PARTICIPANT_RELATION_ID, false);
		}
		$select->setSelectedValue($setValue);

		$filter = $select->getHtml();

		return $$rv;
	}

	function getColumnModifier() {
		return array('QUANTITY'=>'style="text-align:right;"');
	}

	function tfHeaderProc() {
		return array('thAlignRight'=>array('QUANTITY'));
	}


	function getProcessColumns() {
		return array(
			'tfLongNames'=>array('PARTICIPANT_NAME', 'PRODUCER_NAME'),
			'tfValueProcessorQuantityNoDecimals'=>array('QUANTITY')
		);
	}

	function tfValueProcessorQuantityNoDecimals($args) {
		if(!isset($this->_maxNum[$args['head']]))
			$this->_maxNum[$args['head']] = 0;
		$this->_headType[$args['head']] = 0;
		$this->_maxNum[$args['head']] += $args['value'];
		$this->_cols = $args['visibleColumns'];
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		$rv = '%s';
		return sprintf($rv,locale_nf($args['value']));
	}

	function tfFooter() {
		if(!$this->_cols)
			return false;

		$cols = array_flip($this->_cols);
		$rv = explode(',',str_repeat(',',(sizeof($this->_cols)-1)));
		foreach($this->_maxNum as $head => $val) {
			$pos = $cols[$head];
			$round = $this->_headType[$head];
			$rv[$pos] = '<span style="float:right; font-weight: bold;">'.locale_nf($val,$round).'</span><span style="clear:both;"></span>';
		}
		return $rv;
	}


	function setEmptyText()	{
		return $this->langIf->getText('No Proof of Payments for current selection');
	}

	function tfSortingHat()	{
		return array(
			'PROOF_OF_PAYMENT_ID'=>"'Int'",
			'QUANTITY'=>"'SortVal'",
			'PARTICIPANT_NAME'=>"'String'",
			'PRODUCER_NAME'=>"'String'",
			'INVOICE_ID'=>"'String'",
			'PAYMENT_DATE'=>"'String'"
		);
	}

	function getButtons() {
		return array_merge($this -> _getCSVButton(), $this -> _getXMLButton());
	}

}