<?php
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'ProofOfPayment.php');
/**
 * @package
 * @subpackage
 * @since 24-okt-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class PoPList extends ELCDefaultList
{
	var $handle = 'PoPList';
	var $producer_exporter = PARTICIPANT_EXPORTER;
	var $_maxNum = array();
	var $_headType = array();
	var $_cols = false;

	function setQuery()
	{
		if(!$this->checkForParticipantOrStatus())
		{
			$this->_query = "SELECT pop.PROOF_OF_PAYMENT_ID,
					pop.PARTICIPANT_RELATION_ID, R.`NAME` as PARTICIPANT_NAME,
					SUM(popl.QUANTITY) as QUANTITY,
					%s as PENDING, %s as APPROVED, %s as REJECTED
				FROM TBL_PROOF_OF_PAYMENT as pop
				INNER JOIN TBL_PROOF_OF_PAYMENT_LINE as popl
					ON pop.PROOF_OF_PAYMENT_ID = popl.PROOF_OF_PAYMENT_ID
				INNER JOIN TBL_RELATION as R
					ON pop.PARTICIPANT_RELATION_ID = R.RELATION_ID";
			$this->_orderBy = "ORDER BY PARTICIPANT_NAME";
		}
		else
		{
			$this->_query = "SELECT pop.PROOF_OF_PAYMENT_ID,
					pop.PARTICIPANT_RELATION_ID, R.`NAME` as PARTICIPANT_NAME,
					pop.DOCUMENT_ID as ProofOfPaymentDoc, popl.QUANTITY, R2.`NAME` as PRODUCER_NAME,
					INVOICE_ID, STATUS, PAYMENT_DATE as INVOICE_DATE
				FROM TBL_PROOF_OF_PAYMENT as pop
				INNER JOIN TBL_PROOF_OF_PAYMENT_LINE as popl
					ON pop.PROOF_OF_PAYMENT_ID = popl.PROOF_OF_PAYMENT_ID
				INNER JOIN TBL_RELATION as R
					ON pop.PARTICIPANT_RELATION_ID = R.RELATION_ID
				INNER JOIN TBL_RELATION as R2
					ON pop.PRODUCER_RELATION_ID = R2.RELATION_ID";
			$this->_orderBy = "ORDER BY INVOICE_DATE";
		}
	}

	protected function checkForParticipant()
	{
		if(isset($_REQUEST['PARTICIPANT_RELATION_ID'])&&is_numeric($_REQUEST['PARTICIPANT_RELATION_ID']))
			return true;
		return false;
	}

	protected function checkForStatus()
	{
		if(isset($_REQUEST['STATUS'])&&is_numeric($_REQUEST['STATUS']))
			return true;
		return false;
	}

	protected function checkForParticipantOrStatus()
	{
		if($this->checkForStatus()||$this->checkForParticipant())
			return true;
		return false;
	}

	function _filterQuery()
	{
		$this->_addHiddenCol(array('PARTICIPANT_RELATION_ID'));
		if(!$this->checkForParticipantOrStatus())
		{
			$this->_addGroupBy(array('pop.`PARTICIPANT_RELATION_ID`'));
			$query = "(SELECT COUNT(PROOF_OF_PAYMENT_ID)
				FROM TBL_PROOF_OF_PAYMENT as pop2
				WHERE pop.PARTICIPANT_RELATION_ID = pop2.PARTICIPANT_RELATION_ID
					AND pop2.`STATUS` = %s)";
			$pending = sprintf($query,PROOF_OF_PAYMENT_PENDING);
			$approved = sprintf($query,PROOF_OF_PAYMENT_APPROVED);
			$rejected = sprintf($query,PROOF_OF_PAYMENT_REJECTED);
			$this->_addHiddenCol(array('PROOF_OF_PAYMENT_ID'));
			$this->_query = sprintf($this->_query,$pending,$approved,$rejected);
		} else {
			$this->_addGroupBy(array('popl.`LINE_INDEX`'));
		}

		return $this->_query;
	}

	function _setWhere()
	{
		$this->_addWhere('CRSO_RELATION_ID',
				array('field'=>'pop.`CRSO_RELATION_ID`',
					'callback' => array($this,'getCRSOSelect'),
					'required'=>true));
		$this->_addWhere('PARTICIPANT_RELATION_ID',
			array('field'=>'pop.`PARTICIPANT_RELATION_ID`',
				'callback' => array($this,'getParticipantSelect'),
				'related'=>array('PARTICIPANT_NAME')));
		$this->_addWhere('STATUS',
			array('field'=>'pop.`STATUS`',
				'callback' => array($this,'getStatusSelect'),
				'related'=>'STATUS',
				'groupBy'=>'pop.`PROOF_OF_PAYMENT_ID`'));
		$this->_addWhere('PRODUCT_ID',
			array(
				'field'=>'pop.PRODUCT_ID',
				'callback' => array($this,'getProductSelect')));
	}

	function getStatusSelect($rv='setValue')
	{
		$filter = '';

		$stati = array(PROOF_OF_PAYMENT_PENDING,PROOF_OF_PAYMENT_APPROVED,PROOF_OF_PAYMENT_REJECTED);
		if(isset($_REQUEST['STATUS'])&&!in_array(isset($_REQUEST['STATUS']),$stati))
			throw new IWAJAXPermissionException('Incorrect Status Provided');

		$setValue = isset($_REQUEST['STATUS'])?$_REQUEST['STATUS']:'';
		$selectedValue = ' - '.$this->getText('All Status').' - ';

		$select = new Form_Select('STATUS');
		$select->addOption(' - '.$this->getText('All Status').' - ', '', false);

		foreach($stati as $id)
		{
			list($trans,$class) = ProofOfPayment::translateStatus($this->langIf,$id);
			if($setValue==$id)
				$selectedValue = $trans;
			$select->addOption($trans, $id, false);
		}
		$select->setSelectedValue($setValue);
		$filter = $select->getHtml();

		return $$rv;
	}

	function getProcessColumns()
	{
		return array(
			'tfLongNamesCustom'=>array('PARTICIPANT_NAME'),
			'tfLongNames'=>array('PRODUCER_NAME'),
			'tfValueProcessorQuantityNoDecimalsStatus'=>array('PENDING','APPROVED','REJECTED'),
			'tfValueProcessorQuantityNoDecimals'=>array('QUANTITY'),
			'tfDocumentLink'=>array('ProofOfPaymentDoc'),
			'tfStatus'=>array('STATUS')
			);
	}

	function getColumnModifier()
	{
		return array('PENDING'=>'style="text-align:right;"',
			'APPROVED'=>'style="text-align:right;"',
			'REJECTED'=>'style="text-align:right;"',
			'QUANTITY'=>'style="text-align:right;"');
	}

	function tfHeaderProc() {
		return array('thAlignRight'=>array('QUANTITY', 'PENDING', 'APPROVED', 'REJECTED'));
	}

	function tfStatus($args)
	{
		list($trans,$class) = ProofOfPayment::translateStatus($this->langIf,$args['value']);
		return sprintf('<span class="%s">%s</span>',$class,$trans);
	}

	function tfDocumentLink($args)
	{
		$args['hidename'] = true;
		return parent::tfDocumentLink($args);
	}

	function tfLongNamesCustom($args)
	{
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		$name = parent::tfLongNames($args);
		return sprintf('<a href="#" onclick="$$(\'PARTICIPANT_RELATION_ID\').value='.$args['row']['PARTICIPANT_RELATION_ID'].';_LoadOverview();">%s</a>',$name);
	}

	function tfValueProcessorQuantityNoDecimalsStatus($args)
	{
		$args['checklink'] = true;
		return $this->tfValueProcessorQuantityNoDecimals($args);
	}

	function tfValueProcessorQuantityNoDecimals($args)
	{
		if(!isset($this->_maxNum[$args['head']]))
			$this->_maxNum[$args['head']] = 0;
		$this->_headType[$args['head']] = 0;
		$this->_maxNum[$args['head']] += $args['value'];
		$this->_cols = $args['visibleColumns'];
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		$rv = '%s';
		if(isset($args['checklink']))
			$rv = ($args['value']>0)?'<a href="#" onclick="$$(\'PARTICIPANT_RELATION_ID\').value='.$args['row']['PARTICIPANT_RELATION_ID'].';$$(\'STATUS\').value='.constant('PROOF_OF_PAYMENT_'.$args['head']).';_LoadOverview();">%s</a>':'%s';
    	return sprintf($rv,locale_nf($args['value']));
	}

	function tfFooter() // getClass
	{
		if(!$this->_cols)
			return false;

		$cols = array_flip($this->_cols);
		$rv = explode(',',str_repeat(',',(sizeof($this->_cols)-1)));
		foreach($this->_maxNum as $head => $val)
		{
			$pos = $cols[$head];
			$round = $this->_headType[$head];
			$rv[$pos] = '<span style="float:right; font-weight: bold;">'.locale_nf($val,$round).'</span><span style="clear:both;"></span>';
		}
		return $rv;
	}

	function setEmptyText()
	{
		return $this->langIf->getText('No Proof of Payments for current selection');
	}

	function tfSortingHat()
	{
		return array(
			'PROOF_OF_PAYMENT_ID'=>"'Int'",
			'QUANTITY'=>"'SortVal'",
			'APPROVED'=>"'SortVal'",
			'PENDING'=>"'SortVal'",
			'REJECTED'=>"'SortVal'",
			'PRODUCER_NAME'=>"'String'",
			'INVOICE_ID'=>"'String'",
			'STATUS'=>"'String'",
			'INVOICE_DATE'=>"'String'"
			);
	}
}
?>
