<?php
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'Country.php');
/**
 * @package WEEE
 * @subpackage List
 * @since 2-nov-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class ProvDeclChangeList extends ELCDefaultList
{
	var $handle = 'ProvDeclChangeList';
	var $producer_exporter = PARTICIPANT_PRODUCER;
	protected $crso_id = false;
	protected $participant_id = false;
	protected $change_date = false;
	protected $country;
	private $_maxQuantity = 0;
	private $_cols = 0;
	function setQuery()
	{
		$this->crso_id = $this->getCRSOSelect('setValue');
		if(!$this->crso_id)
			return ;
		$this->participant_id = $this->getParticipantSelect('setValue');
		$this->change_date = $this->getChangeDateSelect('setValue');
		$rel = new Relation($this->db,$this->crso_id);
		$c = $this->country = new Country($this->db,$rel->getCountry(),true);

		$this->_query = "SELECT R.`NAME` as PARTICIPANT_NAME,
				DATE(PDH.CHANGE_DATE) as DATE_OF_CHANGE, COUNT(DISTINCT PDH.`MONTH`) as MONTHS_AFFECTED,
				PDH.`MONTH`,
				SUM(PDH.QUANTITY) as QUANTITY,SUM(PDH.WEIGHT) as WEIGHT,SUM(PDH.WEIGHT_DEC) as WEIGHT_DEC,PDH.BRAND_ID,PDH.PRODUCT_CATEGORY_ID,
				IF(ISNULL(PD.QUANTITY),0,1) as ACTUAL_QUANTITY
			FROM TBL_PROVISIONAL_DECLARATION_HISTORY as PDH
			INNER JOIN TBL_RELATION as R
				ON PDH.PARTICIPANT_RELATION_ID = R.RELATION_ID
			LEFT JOIN TBL_PROVISIONAL_DECLARATION as PD
				ON PDH.PARTICIPANT_RELATION_ID = PD.PARTICIPANT_RELATION_ID
				AND PDH.CRSO_RELATION_ID = PD.CRSO_RELATION_ID
				AND PDH.MONTH = PD.MONTH
				AND PDH.`YEAR` = PD.`YEAR`
				AND PDH.QUANTITY = PD.QUANTITY
				AND PDH.PRODUCT_ID = PD.PRODUCT_ID
				AND (ISNULL(PDH.TRUST_ID) OR PDH.TRUST_ID = PD.TRUST_ID)";
		if($c->has(HasDeclOfWeight,$_SESSION['selected_product_id']))
			$this->_query .= " AND PDH.WEIGHT = PD.WEIGHT ";
		else
			$this->_addHiddenCol('WEIGHT');
		if($c->has(HasDeclOfWeightWithDecimals,$_SESSION['selected_product_id']))
			$this->_query .= " AND PDH.WEIGHT_DEC = PD.WEIGHT_DEC ";
		else
			$this->_addHiddenCol('WEIGHT_DEC');
		if($c->has(HasDeclByBrand,$_SESSION['selected_product_id']))
			$this->_query .= " AND PDH.BRAND_ID = PD.BRAND_ID ";
		else
			$this->_addHiddenCol('BRAND_NAME');
		if($c->has(HasDeclByCategory,$_SESSION['selected_product_id']))
			$this->_query .= " AND PDH.PRODUCT_CATEGORY_ID = PD.PRODUCT_CATEGORY_ID ";
		else
			$this->_addHiddenCol('PRODUCT_CATEGORY');

		$this->_addHiddenCol(array('BRAND_ID','WEIGHT','PRODUCT_CATEGORY_ID','ACTUAL_QUANTITY'));

		if(!($this->crso_id&& $this->participant_id&& $this->change_date))
		{
			$this->_groupBy = "GROUP BY PDH.PARTICIPANT_RELATION_ID,PDH.CHANGE_DATE";
			$this->_addHiddenCol(array('QUANTITY','MONTH'));
		}
		else
		{
			$this->_groupBy = "GROUP BY PDH.CRSO_RELATION_ID, PDH.PARTICIPANT_RELATION_ID, PDH.`MONTH`, PDH.BRAND_ID,PDH.PRODUCT_CATEGORY_ID";
			$this->_addHiddenCol(array('MONTHS_AFFECTED'));
		}

	}

	function _filterQuery()
	{
		$hasDetails = true;
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_CRSO:
				$hasDetails = $this->country->has(HasCrsoProvisionalDetails,$_SESSION['selected_product_id']);
				require_once(ELC_CLASSES.'RelationUser.php');
				$ru = new RelationUser($this->db);
				// Check for clearance.
				$hasDetails = $hasDetails && $ru->hasClearance(CRSOClearanceProvisionalDeclaration, $_SESSION['session_user_id'], $this->crso_id);
			break;
		}

		if(!$hasDetails)
			$this->_addHiddenCol('QUANTITY');

		if($this->crso_id&& $this->participant_id&& $this->change_date&& $hasDetails)
		{
			$country = $this->country;
			if($country->has(HasDeclByCategory,$_SESSION['selected_product_id']))
				$this->_removeHiddenCol('PRODUCT_CATEGORY');
			if($country->has(HasDeclByBrand,$_SESSION['selected_product_id']))
				$this->_removeHiddenCol('BRAND_ID');
			if($country->has(HasDeclOfWeight,$_SESSION['selected_product_id']))
				$this->_removeHiddenCol('WEIGHT');
		}

		// crso settings show/hide columns
		return $this->_query;
	}

	function _setWhere()
	{
		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'PDH.`CRSO_RELATION_ID`',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true));
		$this->_addWhere('PARTICIPANT_RELATION_ID',
			array(
				'field'=>'PDH.`PARTICIPANT_RELATION_ID`',
				'callback' => array($this,'getParticipantSelect'),
				'related'=>array('PARTICIPANT_NAME')));
		$this->_addWhere('CHANGE_DATE',
			array(
				'field'=>'DATE(PDH.CHANGE_DATE)',
				'callback' => array($this,'getChangeDateSelect'),
				'related'=>'DATE_OF_CHANGE'));

		$this->_addWhere('TRUST_ID',
			array(
				'field'=>'PDH.`TRUST_ID`',
				'callback' => array($this,'getTrustInput')));
		$this->_addWhere('PRODUCT_ID',
			array(
				'field'=>'PDH.PRODUCT_ID',
				'callback' => array($this,'getProductSelect')));
	}

	function getProcessColumns()
	{
		return array(
			'tfLongNames'=>array('PARTICIPANT_NAME'),
			'tfValueProcessorQuantity'=>array('QUANTITY'),
			'tfMonth'=>array('MONTH'),
			'tfBrand'=>array('BRAND_ID'),
			'tfProductCategory'=>array('PRODUCT_CATEGORY_ID'));
	}

	function tfSortingHat()
	{
		return array(
			'MONTH'=>"'SortVal'",
			'QUANTITY'=>"'SortVal'",
			'WEIGHT'=>"'Int'",
			'WEIGHT_DEC'=>"'SortVal'",
			'PRODUCT_CATEGORY'=>"'String'",
			'BRAND_ID'=>"'String'",
			'DATE_OF_CHANGE'=>"'String'",
			'MONTHS_AFFECTED'=>"'Int'",
			'PARTICIPANT_NAME'=>"'String'"
			);
	}

	function getColumnModifier()
	{
		return array('QUANTITY'=>'style="text-align:right;"','WEIGHT'=>'style="text-align:right;"','WEIGHT_DEC'=>'style="text-align:right;"');
	}

	function getTrustInput($rv='setValue')
	{
		$setValue = (!empty($_REQUEST['TRUST_ID']))?$_REQUEST['TRUST_ID']:null;
		$selectedValue = '';
		$filter = '<input type="hidden" name="TRUST_ID" value="'.$setValue.'">';
		return $$rv;
	}

	function getChangeDateSelect($rv='setValue')
	{
		$selectedValue = '- '.$this->getText('All Changes').' -';
		$setValue = isset($_REQUEST['CHANGE_DATE'])?$_REQUEST['CHANGE_DATE']:'';

		$select = new Form_Select('CHANGE_DATE');
		$select->addOption($selectedValue,'');

		$setValue && $selectedValue = $setValue;

		$filter = '';
		if(!$this->crso_id)
			return $$rv;

		$sql = "SELECT DISTINCT DATE(CHANGE_DATE) as CHG_DATE FROM TBL_PROVISIONAL_DECLARATION_HISTORY
			WHERE CRSO_RELATION_ID = ? ";
		if($this->participant_id)
			$sql .= " AND PARTICIPANT_RELATION_ID = ".$this->participant_id." ";
		$sql .= " ORDER BY DATE(CHANGE_DATE)";
		$q = $this->db->query($sql);
		$q->bind_values($this->crso_id);
		$q->execute();
		while($r = $q->fetch())
			$select->addOption($r->CHG_DATE,$r->CHG_DATE,($r->CHG_DATE==$setValue));

		$filter = $select->getHtml();

		return $$rv;
	}

	function tfValueProcessorQuantity($args)
	{
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		if(!is_numeric($args['value']))
			return $args['value'];
		$this->_maxQuantity += $args['value'];
		$this->_cols = $args['visibleColumns'];
		if($args['row']['ACTUAL_QUANTITY'])
    		return locale_nf($args['value']);
    	return '<span class="forfeit">'.locale_nf($args['value']).'</span>';
	}

	function tfFooter() // getClass
	{
		if(!$this->_cols||in_array('QUANTITY',$this->_hiddenCols))
			return false;
		$rv = explode(',',str_repeat(',',(sizeof($this->_cols)-1)));

		$pos = array_flip($this->_cols);
		$rv[$pos['QUANTITY']] = '<span style="float:left; font-weight: bold;">Total</span><span style="float:right; font-weight: bold;">'.locale_nf($this->_maxQuantity).'</span><span style="clear:both;"></span>';
		return $rv;
	}

	function getButtonBarText()
	{
		return '<span style="background-color: #D00">&nbsp;&nbsp;&nbsp;</span> = '.$this->getText('Lines that are different from Current Prov');
	}

	function setEmptyText()
	{
		return $this->getText('No Changes Found For Selection');
	}

	function tfHeaderProc() {
		return array('thAlignRight'=>array('QUANTITY', 'WEIGHT', 'WEIGHT_DEC'));
	}
}
?>
