<?php
require_once(ELC_CLASSES.'Participant.php');
/**
 * @package ELC
 * @subpackage List
 * @since 14-jul-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 * @todo Make pretty and work with Subrogation
 */
class ProvDeclCmpList extends ELCDefaultList
{
	var $producer_exporter = PARTICIPANT_PRODUCER;
	var $handle = 'ProvDeclCmpList';
	var $_totals = array('ACTUAL_QUANTITY'=>0,'PROV_QUANTITY'=>0);
	var $_row = array();
	protected $trustee_lists = array();
	protected $custom_dev = false;
	function setQuery()
	{
		$crso_id = $this->getCRSOSelect();
		if(!$crso_id)
			return ;
		require_once(ELC_CLASSES.'Country.php');
		require_once(ELC_CLASSES.'Relation.php');

		$r = new Relation($this->db);
		$id = $r->getCountry($crso_id);
		$c = new Country($this->db,$id,true);

		if(!$c->has(HasDeclOfWeight,$_SESSION['selected_product_id']))
			$this->_addHiddenCol(array('PROV_WEIGHT','ACTUAL_WEIGHT'));

		if(!$c->has(HasDeclOfWeightWithDecimals,$_SESSION['selected_product_id']))
			$this->_addHiddenCol(array('PROV_WEIGHT_DEC','ACTUAL_WEIGHT_DEC'));

		$this->_query = "SELECT R.NAME as PARTICIPANT_NAME,
				D.PARTICIPANT_RELATION_ID,
				0 as IS_TRUSTEE,D.`MONTH`,
				B.`NAME` as BRAND_NAME, PC.`NAME` as PRODUCT_CATEGORY,
				SUM(PD.WEIGHT) as PROV_WEIGHT, SUM(DL.WEIGHT) as ACTUAL_WEIGHT,
				SUM(PD.WEIGHT_DEC) as PROV_WEIGHT_DEC, SUM(DL.WEIGHT_DEC) as ACTUAL_WEIGHT_DEC,
				SUM(PD.QUANTITY) as PROV_QUANTITY, SUM(DL.QUANTITY) as ACTUAL_QUANTITY,
				IF(ISNULL(SUM(PD.QUANTITY)),
					SUM(DL.QUANTITY),
					IF((SUM(DL.QUANTITY)>SUM(PD.QUANTITY)),
						(SUM(DL.QUANTITY)-SUM(PD.QUANTITY)),
						(SUM(PD.QUANTITY)-SUM(DL.QUANTITY)))
					) as ABSOLUTE_DEVIATION,
				((SUM(DL.QUANTITY)/SUM(PD.QUANTITY)*100)-100) as DEVIATION,
				'' as TEN_PERCENT, '' as TWENTYFIVE_PERCENT, '' as FIFTY_PERCENT, '' as CUSTOM_DEVIATION
			FROM TBL_DECLARATION as D
			INNER JOIN TBL_DECLARATION_LINE as DL
				USING(DECLARATION_ID)
			INNER JOIN TBL_RELATION as R
				ON D.PARTICIPANT_RELATION_ID = R.RELATION_ID
			LEFT JOIN TBL_BRAND as B
				ON DL.BRAND_ID = B.BRAND_ID
					AND D.CRSO_RELATION_ID = B.CRSO_RELATION_ID
			LEFT JOIN TBL_PRODUCT_CATEGORY as PC
				ON DL.PRODUCT_CATEGORY_ID = PC.PRODUCT_CATEGORY_ID
					AND D.CRSO_RELATION_ID = PC.CRSO_RELATION_ID
			LEFT JOIN TBL_PROVISIONAL_DECLARATION as PD
				ON D.PARTICIPANT_RELATION_ID = PD.PARTICIPANT_RELATION_ID
				AND D.CRSO_RELATION_ID = PD.CRSO_RELATION_ID
				AND D.MONTH = PD.MONTH
				AND D.`YEAR` = PD.`YEAR`
				AND D.PRODUCT_ID = PD.PRODUCT_ID
				AND (ISNULL(DL.TRUST_ID) OR DL.TRUST_ID = PD.TRUST_ID)
				AND ISNULL(D.PARENT_DECLARATION_ID)";
		if($c->has(HasDeclByBrand,$_SESSION['selected_product_id']))
			$this->_query .= " AND DL.BRAND_ID = PD.BRAND_ID ";
		$this->_addHiddenCol('BRAND_NAME');
		if($c->has(HasDeclByCategory,$_SESSION['selected_product_id']))
			$this->_query .= " AND DL.PRODUCT_CATEGORY_ID = PD.PRODUCT_CATEGORY_ID ";
		$this->_addHiddenCol('PRODUCT_CATEGORY');
		if($c->has(HasDeclarationDimension,$_SESSION['selected_product_id']))
			$this->_query .= " AND DL.CPD_ID = PD.CPD_ID ";
		if(isset($_REQUEST['group']))
			$this->_groupBy = "GROUP BY ".$_REQUEST['group'];
		else
			$this->_groupBy = "GROUP BY D.PARTICIPANT_RELATION_ID";
		$this->_orderBy = "ORDER BY ABSOLUTE_DEVIATION DESC, PARTICIPANT_NAME, D.`MONTH`";
	}

	function _filterQuery()
	{
		$crso_id = $this->getCRSOSelect();
		$rel = new Relation($this->db,$crso_id);
		$c = new Country($this->db,$rel->getCountry(),true);
		$this->_addHiddenCol(array(
			'CUSTOM_DEVIATION','PARTICIPANT_RELATION_ID',
			'ACTUAL_WEIGHT_DEC','PROV_WEIGHT_DEC',
			'ACTUAL_WEIGHT','PROV_WEIGHT',
			'ACTUAL_QUANTITY','PROV_QUANTITY',
			'ABSOLUTE_DEVIATION'));
		require_once(ELC_CLASSES.'CRSOProduct.php');
		$cp = new CRSOProduct($this->db);
		$cp->set($crso_id, $_SESSION['selected_product_id']);
		if($cp->prov_decl_deviation_trigger_percentage) {
			$this->custom_dev = $cp->prov_decl_deviation_trigger_percentage;
			$this->_addHiddenCol(array('TEN_PERCENT','TWENTYFIVE_PERCENT','FIFTY_PERCENT'));
			unset($this->_hiddenCols['CUSTOM_DEVIATION']);
		} else {
			$this->_addHiddenCol(array('CUSTOM_DEVIATION'));
		}
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_CRSO:

				require_once(ELC_CLASSES.'CRSO.php');
				require_once(ELC_CLASSES.'RelationUser.php');

				$ru = new RelationUser($this->db);

				if($c->has(HasCrsoDeclarationDetails,$_SESSION['selected_product_id'])&& $ru->hasClearance(CRSOClearanceDeclarationDetails, $_SESSION['session_user_id'], $crso_id))
				{
					unset($this->_hiddenCols['ACTUAL_QUANTITY'],$this->_hiddenCols['PROV_QUANTITY'],$this->_hiddenCols['ABSOLUTE_DEVIATION']);
					if($c->has(HasDeclOfWeight,$_SESSION['selected_product_id']))
						unset($this->_hiddenCols['ACTUAL_WEIGHT'],$this->_hiddenCols['PROV_WEIGHT']);
					if($c->has(HasDeclOfWeightWithDecimals,$_SESSION['selected_product_id']))
						unset($this->_hiddenCols['ACTUAL_WEIGHT_DEC'],$this->_hiddenCols['PROV_WEIGHT_DEC']);
				}

			break;
			default:
				unset($this->_hiddenCols['ACTUAL_QUANTITY'],$this->_hiddenCols['PROV_QUANTITY'],$this->_hiddenCols['ABSOLUTE_DEVIATION']);
				if($c->has(HasDeclOfWeight,$_SESSION['selected_product_id']))
					unset($this->_hiddenCols['ACTUAL_WEIGHT'],$this->_hiddenCols['PROV_WEIGHT']);
				if($c->has(HasDeclOfWeightWithDecimals,$_SESSION['selected_product_id']))
					unset($this->_hiddenCols['ACTUAL_WEIGHT_DEC'],$this->_hiddenCols['PROV_WEIGHT_DEC']);
		}

		if(isset($_REQUEST['CRSO_RELATION_ID'])&&
			(!isset($_REQUEST['PARTICIPANT_RELATION_ID'])||
				!is_numeric($_REQUEST['PARTICIPANT_RELATION_ID'])))
			$this->_addHiddenCol(array('MONTH','BRAND_NAME','PRODUCT_CATEGORY'));
		if(isset($_REQUEST['group'])&& $_REQUEST['group']=='D.MONTH')
		{
			$this->_addHiddenCol(array('PARTICIPANT_NAME','IS_TRUSTEE'));
			$this->_removeHiddenCol(array('MONTH'));
		}
		if(isset($_REQUEST['PARTICIPANT_RELATION_ID'])&&is_numeric($_REQUEST['PARTICIPANT_RELATION_ID']))
			$this->_addHiddenCol(array('IS_TRUSTEE'));
		return $this->_query;
	}

	function _setWhere()
	{
		$this->_addWhere('YEAR',
			array('field'=>'D.`YEAR`',
				'callback' => array($this,'getYearSelect'),
				'required' => true));

		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'D.`CRSO_RELATION_ID`',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true,
				'related'=>array('CRSO_NAME')));
		$this->_addWhere('PARTICIPANT_RELATION_ID',
			array(
				'field'=>'D.`PARTICIPANT_RELATION_ID`',
				'callback' => array($this,'getParticipantSelect'),
				'related'=>array('PARTICIPANT_NAME'),
				'groupBy'=>array('D.`MONTH`')));
		$this->_addWhere('MONTH',
			array(
				'field'=>'D.`MONTH`',
				'callback' => array($this,'getMonthSelect'),
				'groupBy'=>array('D.`MONTH`')));
		$this->_addWhere('group',
			array(
				'useinquery'=>false,
				'field'=>'',
				'callback' => array($this,'getGroupBy')));

		$this->_addWhere('PRODUCT_ID',
			array(
				'field'=>'D.PRODUCT_ID',
				'callback' => array($this,'getProductSelect')));
	}

	function getGroupBy($rv='setValue')
	{
		$setValue = isset($_REQUEST['group'])?$_REQUEST['group']:'D.PARTICIPANT_RELATION_ID';
		$selectedValue = '';
		$filter = $this->getText('Group By')." <select name=\"group\">
			<option value=\"D.PARTICIPANT_RELATION_ID\">".$this->getText('Participant')."</option>
			<option value=\"D.MONTH\" ".(($setValue=='D.MONTH')?'selected="selected"':'').">".$this->getText('Month')."</option>
			</select>";
		return $$rv;
	}

	function getProcessColumns()
	{
		return array(
			'tfValueProcessorQuantity'=>array('PROV_QUANTITY','ACTUAL_QUANTITY','ABSOLUTE_DEVIATION'),
			'tfValueProcessorQuantityNoDecimals'=>array('PROV_WEIGHT','ACTUAL_WEIGHT'),
			'tfValueProcessorWeightWithDecimals'=>array('PROV_WEIGHT_DEC','ACTUAL_WEIGHT_DEC'),
			'tfMonth'=>array('MONTH'),
			'tfPercentageNoDecimals'=>array('DEVIATION'),
			'tfDeviationExceeds'=>array('TEN_PERCENT','TWENTYFIVE_PERCENT','FIFTY_PERCENT','CUSTOM_DEVIATION'),
			'tfTrustee'=>array('IS_TRUSTEE'),
			'tfLongNames'=>array('PARTICIPANT_NAME')
			);
	}

	function getColumnModifier()
	{
		return array(
			'ABSOLUTE_DEVIATION'=>'style="text-align:right;"',
			'DEVIATION'=>'style="text-align:right;"',
			'ACTUAL_QUANTITY'=>'style="text-align:right;"',
			'PROV_QUANTITY'=>'style="text-align:right;"',
			'ACTUAL_WEIGHT'=>'style="text-align:right;"',
			'PROV_WEIGHT'=>'style="text-align:right;"',
			'ACTUAL_WEIGHT_DEC'=>'style="text-align:right;"',
			'PROV_WEIGHT_DEC'=>'style="text-align:right;"'
		);
	}

	function tfValueProcessorQuantity(&$args)
	{
		$this->_row = array_keys($args['row']);
		if($args['head']!='ABSOLUTE_DEVIATION')
			$this->_totals[$args['head']] += $args['value'];
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);

		if(!is_numeric($args['value']))
			return 'n/d';
    	return locale_nf($args['value'], 0);
	}

	function tfDeviationExceeds($args,$getBool = false)
	{
		$check = 0;
		switch($args['head'])
		{
			case 'TEN_PERCENT':
				$check = 10;
			break;
			case 'TWENTYFIVE_PERCENT':
				$check = 25;
			break;
			case 'FIFTY_PERCENT':
				$check = 50;
			break;
			case 'CUSTOM_DEVIATION':
				$check = $this->custom_dev;
			break;
		}

		$val = $args['row']['DEVIATION'];
		if(!$val)
			return '-';
		$val = ($val<0)?($val*-1):$val;

		$bool = $val<$check;
		if($getBool)
			return $bool;

		return $this->tfBooleanIcon(array('value'=>$bool));
	}

	function tfPercentageNoDecimals($args)
	{
		$val = parent::tfPercentageNoDecimals($args);
		if(!$args['value'])
			return '-';
		if(!$this->mkHighlight($args))
			return $val;
		return sprintf('<div style="font-weight: bold;">%s</div>',$val);
	}

	function tfLongNames($args)
	{
		$val = parent::tfLongNames($args);
		if(!$this->mkHighlight($args))
			return $val;
		return sprintf('<div style="font-weight: bold;"><span style="color: #00D">*</span> %s</div>',$val);
	}

	function mkHighlight($args)
	{
		if(!$this->custom_dev)
			return false;

		$args['head'] = 'CUSTOM_DEVIATION';
		return $this->tfDeviationExceeds($args,true);
	}

	function getButtonBarText()
	{
		if(!$this->custom_dev)
			return false;
		return '<span style="color: #00D">*</span> = '.$this->getText('Exceeded the set percentage deviation from prov. decl.');
	}

	function getButtons() {
		return array_merge($this->_getCSVButton(), $this->_getXMLButton());
	}

	function tfFooter()
	{
		$rv = array();
		$this->_totals['ABSOLUTE_DEVIATION'] =
			($this->_totals['ACTUAL_QUANTITY']>$this->_totals['PROV_QUANTITY'])?
				($this->_totals['ACTUAL_QUANTITY']-$this->_totals['PROV_QUANTITY']):
				($this->_totals['PROV_QUANTITY']-$this->_totals['ACTUAL_QUANTITY']);
		if($this->_totals['PROV_QUANTITY']!=0)
		{
			$this->_totals['DEVIATION'] = round(
				((($this->_totals['ACTUAL_QUANTITY']/$this->_totals['PROV_QUANTITY'])*100)-100),0);
		}
		$showCols = array_diff($this->_row,$this->_hiddenCols);
		foreach($showCols as $column)
			$rv[$column]=isset($this->_totals[$column])?'<div style="text-align: right; font-weight: bold;">'.(is_numeric($this->_totals[$column])? ($column == 'DEVIATION' ? locale_nf($this->_totals[$column],0).'%' : locale_nf($this->_totals[$column],0)) :$this->_totals[$column]).'</div>':'';
		return $rv;
	}

	function tfSortingHat()
	{
		return array(
			'MONTH'=>"'SortVal'",
			'PARTICIPANT_NAME'=>"'String'",
			'PROV_QUANTITY'=>"'SortVal'",
			'ACTUAL_QUANTITY'=>"'SortVal'",
			'DEVIATION'=>"'SortVal'",
			'ABSOLUTE_DEVIATION'=>"'SortVal'",
			'TEN_PERCENT'=>"'String'",
			'TWENTYFIVE_PERCENT'=>"'String'",
			'FIFTY_PERCENT'=>"'String'",
			'CUSTOM_DEVIATION'=>"'String'"
			);
	}

	function tfHeaderProc()
	{
		return array(
			'tfCustomDef'=>array('CUSTOM_DEVIATION'),
			'thAlignRight'=>array('ABSOLUTE_DEVIATION', 'DEVIATION', 'ACTUAL_QUANTITY', 'PROV_QUANTITY', 'ACTUAL_WEIGHT', 'PROV_WEIGHT', 'ACTUAL_WEIGHT_DEC', 'PROV_WEIGHT_DEC')
		);
	}

	function tfCustomDef($args)
	{
		$args['headerText'] = $this->custom_dev.'%';
		return '';
	}

	function getPostScript()
	{
		$rv = $tmp = '';
		if(!count($this->trustee_lists))
			return $rv;
		$rv = 'if(typeof help != \'undefined\') {
					%s
				}';
		foreach($this->trustee_lists as $id => $text)
			$tmp .= sprintf("help.registerById('trusteeList_%s', '%s');\n",$id,$text);
		return sprintf($rv,$tmp);
	}
}
?>
