<?php
/**
 * Created on Feb 7, 2008
 *
 * @author Nicoleta Militaru <nicoleta@basesoft.com>
 **/
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'Country.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'CRSOProduct.php');

class ProvDeclDataList extends ELCDefaultList
{
	var $handle = 'ProvDeclDataList';

	function setQuery()
	{
		$crso_id = $this->getCRSOSelect();
		if(!$crso_id)
			return ;

		$r = new Relation($this->db);
		$id = $r->getCountry($crso_id);
		$c = new Country($this->db,$id,true);

		if($_SESSION['session_group_group_id'] == GROUP_CRSO) {

			require_once(ELC_CLASSES.'RelationUser.php');

			$ru = new RelationUser($this->db);
			// Check for clearance.
			if(!$c->has(HasCrsoProvisionalDetails,$_SESSION['selected_product_id'])||!$ru->hasClearance(CRSOClearanceProvisionalDeclaration, $_SESSION['session_user_id'], $crso_id))
				throw new IWUserException('Permission denied');
		}

		$this->_query = "
			SELECT PD.PARTICIPANT_RELATION_ID,R.NAME AS PARTICIPANT_NAME,PD.MONTH,
					P.NAME AS PRODUCT,PC.NAME AS PRODUCT_CATEGORY,CPD.NAME AS DIMENSION,B.NAME AS BRAND_NAME,
					SUM(IFNULL(PD.QUANTITY,0)) AS QUANTITY,SUM(IFNULL(PD.WEIGHT,0)) AS WEIGHT,SUM(IFNULL(PD.WEIGHT_DEC,0)) AS WEIGHT_DEC
			FROM TBL_PROVISIONAL_DECLARATION PD
			INNER JOIN TBL_RELATION R
				ON R.RELATION_ID = PD.PARTICIPANT_RELATION_ID
			INNER JOIN TBL_PRODUCT P
				ON PD.PRODUCT_ID = P.PRODUCT_ID
			LEFT JOIN TBL_PRODUCT_CATEGORY PC
				ON PC.PRODUCT_CATEGORY_ID = PD.PRODUCT_CATEGORY_ID
			LEFT JOIN TBL_CRSO_PRODUCT_DIMENSION CPD
				ON CPD.CPD_ID = PD.CPD_ID
			LEFT JOIN TBL_BRAND B
				ON B.BRAND_ID = PD.BRAND_ID
			";

		$this->_groupBy = "GROUP BY PD.YEAR,PD.MONTH,PD.CRSO_RELATION_ID,PD.PARTICIPANT_RELATION_ID,
				PD.PRODUCT_ID,";

		if($c->has(HasDeclByCategory,$_SESSION['selected_product_id']))
			$this->_groupBy .= "PD.PRODUCT_CATEGORY_ID,";
		else
			$this->_addHiddenCol('PRODUCT_CATEGORY');
		if($c->has(HasDeclByBrand,$_SESSION['selected_product_id']))
			$this->_groupBy .= "PD.BRAND_ID,";
		else
			$this->_addHiddenCol('BRAND_NAME');
		if($c->has(HasDeclarationDimension,$_SESSION['selected_product_id']))
			$this->_groupBy .= "PD.CPD_ID,";
		else
			$this->_addHiddenCol('DIMENSION');

		$this->_groupBy = substr($this->_groupBy,0,strlen($this->_groupBy)-1);

		if(!$c->has(HasDeclOfWeight,$_SESSION['selected_product_id']))
			$this->_addHiddenCol('WEIGHT');

		if(!$c->has(HasDeclOfWeightWithDecimals,$_SESSION['selected_product_id']))
			$this->_addHiddenCol('WEIGHT_DEC');

		$this->_orderBy = "ORDER BY PARTICIPANT_NAME, PD.MONTH ";
	}

	function _setWhere()
	{
		$this->_addWhere('YEAR',
			array('field'=>'PD.`YEAR`',
				'callback' => array($this,'getYearSelect'),
				'required' => true));
		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'PD.`CRSO_RELATION_ID`',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true));
		$this->_addWhere('PARTICIPANT_RELATION_ID',
			array(
				'field'=>'PD.`PARTICIPANT_RELATION_ID`',
				'callback' => array($this,'getParticipantSelect'),
				'related'=>array('PARTICIPANT_NAME')));
		$this->_addWhere('PRODUCT_ID',
			array(
				'field'=>'PD.PRODUCT_ID',
				'callback' => array($this,'getDynamicProductSelect')));
		if(isset($_REQUEST['PARTICIPANT_RELATION_ID']) && !empty($_REQUEST['PARTICIPANT_RELATION_ID']))
			$this->_addWhere('MONTH',
				array(
					'field'=>'PD.`MONTH`',
					'callback' => array($this,'getMonthSelect')));
		else
			$this->_addWhere('MONTH',
				array(
					'field'=>'PD.`MONTH`',
					'callback' => array($this,'getMonthSelect'),
					'required'=>true));
	}

	function getMonthSelect($rv='setValue')
	{
		$setValue = !empty($_REQUEST['MONTH'])?(int)$_REQUEST['MONTH']:false;

		if(!$setValue && !(isset($_REQUEST['PARTICIPANT_RELATION_ID']) && !empty($_REQUEST['PARTICIPANT_RELATION_ID'])))
		{
			$d = currentDate();
			$setValue = $_REQUEST['MONTH'] = $d->getMonth();
		}
		return parent::getMonthSelect($rv);
	}
	function getDynamicProductSelect($rv='setValue')
	{
		$setValue = !empty($_REQUEST['PRODUCT_ID'])?(int)$_REQUEST['PRODUCT_ID']:false;

		$crso_id = isset($_REQUEST['CRSO_RELATION_ID'])?(int)$_REQUEST['CRSO_RELATION_ID']:false;
		if(!is_numeric($crso_id))
			$filter = '';
		if(isset($$rv))
			return $$rv;
		$selectedValue = '';

		$p = new CRSOProduct($this->db);
		$q = $p->getProducts($crso_id);

		$select = new Form_Select('PRODUCT_ID');
		$select->addOption(' - '.$this->getText('Product Select').' - ', '', false);

		$count = 0;
		while($r = $q->fetch())
		{
			if($setValue==$r->PRODUCT_ID)
				$selectedValue = $r->PRODUCT_ID;
			$select->addOption($r->PRODUCT_NAME, $r->PRODUCT_ID, false);
			$product = $r->PRODUCT_ID;
			$count += 1;
		}
		$select->setSelectedValue($setValue);
		if($count == 1)
		{
			$filter = '';
			$setValue = $product;
		}
		else
			$filter = $select->getHtml();
		return $$rv;
	}

	function getProcessColumns()
	{
		return array(
			'tfValueProcessorQuantity'=>array('QUANTITY'),
			'tfValueProcessorQuantityNoDecimals'=>array('WEIGHT'),
			'tfValueProcessorWeightWithDecimals'=>array('WEIGHT_DEC'),
			'tfMonth'=>array('MONTH'),
			'tfLongNames'=>array('PARTICIPANT_NAME')
			);
	}
	function getColumnModifier()
	{
		return array('QUANTITY'=>'style="text-align:right;"','WEIGHT'=>'style="text-align:right;"','WEIGHT'=>'style="text-align:right;"');
	}
	function tfSortingHat()
	{
		return array(
			'MONTH'=>"'SortVal'",
			'PARTICIPANT_NAME'=>"'String'",
			'QUANTITY'=>"'SortVal'",
			'WEIGHT'=>"'SortVal'",
			'WEIGHT_DEC'=>"'SortVal'",
			'PRODUCT_CATEGORY'=>"'String'",
			'DIMENSION'=>"'String'",
			'PRODUCT'=>"'String'",
			);
	}
}
?>
