<?php
/**
 * Created by PhpStorm.
 * User: marten
 * Date: 21/8/17
 * Time: 11:35
 */

require_once(ELC_CLASSES.'Declaration.php');

class AbstractQuarterlyConfirmationList extends ELCDefaultList
{

	function getQuarterSelect($rv='setValue')
	{
		$date = currentDate();

		$year = isset($_REQUEST['YEAR']) ? (int) $_REQUEST['YEAR'] : $date->getYear();
		$maxQuarter = 4;
		if($year == $date->getYear()) {
			$maxQuarter = ceil(($date->getMonth() - 3) / 3);
		}

		$setValue = isset($_REQUEST['QUARTER'])?$_REQUEST['QUARTER']:1;
		$selectedValue = '';
		$select = new Form_Select('QUARTER');
		$select->addOption(' - '.$this->getText('Quarter Select').' - ', '', false);
		for($i=1;$i<($maxQuarter+1);$i++)
		{
			if($setValue == $i) {
				$selectedValue = 'Quarter '.$i;
			}
			$select->addOption('Quarter '.$i, $i, false);
		}
		$select->setSelectedValue((int) $setValue);
		$filter = $select->getHtml();
		return $$rv;
	}

	function getProcessColumns()
	{
		$rv = array(
			'tfValueProcessorQuantityNoDecimals'=>array('WEIGHT', 'WEIGHT_DEC', 'QUANTITY'),
		);
		return $rv;
	}

	function getColumnModifier()
	{
		return array('QUANTITY'=>'style="text-align:right;"','WEIGHT'=>'style="text-align:right;"');
	}

	function tfHeaderProc() {
		return array('thAlignRight'=>array('QUANTITY', 'WEIGHT'));
	}

	function tfFooter() {
		return array('Signature');
	}

	function getButtonBarText() {
		return 'Print & sign this overview and email it to Recolight Limited.';
	}

}

class QuarterlyDeclarationConfirmationList extends AbstractQuarterlyConfirmationList
{

	public $handle = 'QuarterlyDeclarationConfirmationList';

	function setQuery()
	{

		$this->_defaultQuery = "
			SELECT
				%s as QUARTER,
				P.NAME as PRODUCT_NAME,
				PC.NAME as PRODUCT_CATEGORY_NAME,
				DIM.NAME as DIMENSION_NAME,
				SUM(DL.WEIGHT) as WEIGHT,
				SUM(DL.QUANTITY) as QUANTITY
			FROM TBL_DECLARATION as D
			INNER JOIN TBL_DECLARATION_LINE as DL
				ON D.DECLARATION_ID = DL.DECLARATION_ID
				%s
			INNER JOIN TBL_PRODUCT as P
				ON D.PRODUCT_ID = P.PRODUCT_ID
			INNER JOIN TBL_PRODUCT_CATEGORY as PC
				ON DL.PRODUCT_CATEGORY_ID = PC.PRODUCT_CATEGORY_ID
			INNER JOIN TBL_CRSO_PRODUCT_DIMENSION as DIM
				ON DL.CPD_ID = DIM.CPD_ID
				%s

		";

		$this->_query = sprintf($this->_defaultQuery, '"Q1"', ' AND D.MONTH IN ('.$this->_getQuarterMonths(1).') ', ' AND DIM.NAME = "Household" ');
		$this->_groupBy = "GROUP BY D.PRODUCT_ID, DL.PRODUCT_CATEGORY_ID, DL.CPD_ID";
		$this->_orderBy = "ORDER BY QUARTER, PRODUCT_NAME, PRODUCT_CATEGORY_NAME, DIMENSION_NAME";
	}

	function _filterQuery()
	{
		if(isset($_REQUEST['QUARTER']) && $_REQUEST['QUARTER']) {
			for($i = 2; $i <= $_REQUEST['QUARTER']; $i++) {
				if($i == 4) {
					$this->_union .= "UNION (".sprintf($this->_defaultQuery, '"Q'.$i.'"', '', '').' '.$this->_getWhere().' AND ((D.MONTH IN ('.$this->_getQuarterMonths($i).') AND DIM.NAME = "Household") OR DIM.NAME = "Non-Household")'.$this->_groupBy.') ';
					continue;
				}
				$this->_union .= "UNION (".sprintf($this->_defaultQuery, '"Q'.$i.'"', ' AND D.MONTH IN ('.$this->_getQuarterMonths($i).') ', ' AND DIM.NAME = "Household" ').' '.$this->_getWhere().$this->_groupBy.') ';
			}
		}

		return $this->_query;
	}

	function _getQuarterMonths($quarter) {
		switch($quarter) {
			case 1:
				return '1, 2, 3';
			case 2:
				return '4, 5, 6';
			case 3:
				return '7, 8, 9';
			case 4:
				return '10, 11, 12';
		}
		return '';
	}

	function _getQuarterExportWhere($quarter) {

		$date = currentDate();

		$year = isset($_REQUEST['YEAR']) ? (int) $_REQUEST['YEAR'] : $date->getYear();

		switch($quarter) {
			case 1:
				return 'MONTH(D.EXPORT_DATE) <= 4 AND YEAR(D.EXPORT_DATE) = '.$year;
			case 2:
				return 'MONTH(D.EXPORT_DATE) <= 7 AND YEAR(D.EXPORT_DATE) = '.$year;
			case 3:
				return 'MONTH(D.EXPORT_DATE) <= 10 AND YEAR(D.EXPORT_DATE) = '.$year;
			case 4:
				return '((MONTH(D.EXPORT_DATE) <= 12 AND YEAR(D.EXPORT_DATE) = '.$year.') OR (MONTH(D.EXPORT_DATE) = 1 AND YEAR(D.EXPORT_DATE) = '.($year + 1).'))';
		}
	}

	function _setWhere()
	{
		$q = isset($_REQUEST['QUARTER'])?$_REQUEST['QUARTER']:1;

		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'D.CRSO_RELATION_ID',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true));

		$this->_addWhere('PARTICIPANT_RELATION_ID',
			array(
				'field'=>'D.PARTICIPANT_RELATION_ID',
				'callback' => array($this,'getParticipantSelect'),
				'required'=> true,
				'groupBy'=>''));

		$this->_addWhere('YEAR',
			array('field'=>'D.`YEAR`',
				'callback' => array($this,'getYearSelect'),
				'required' => true));

		$this->_addWhere('QUARTER',
			array(
				'field'=>'MONTH(D.LAST_UPDATED)',
				'operator' => 'IN',
				'completedStatement' => $this->_getQuarterExportWhere($q),
				'callback' => array($this,'getQuarterSelect'),
				'required'=> true,
				'groupBy'=>''));

	}

}
