<?php
/**
 * Created by PhpStorm.
 * User: marten
 * Date: 22/8/17
 * Time: 15:25
 */

require_once(ELC_LISTS.'QuarterlyDeclarationConfirmationList.php');

class QuarterlyRefundConfirmationList extends AbstractQuarterlyConfirmationList
{

	public $handle = 'QuarterlyRefundConfirmationList';

	function setQuery()
	{

		$q = isset($_REQUEST['QUARTER'])?$_REQUEST['QUARTER']:1;

		$this->_query = "
			SELECT
				Prod.NAME as PRODUCER,
				P.NAME as PRODUCT_NAME,
				PC.NAME as PRODUCT_CATEGORY_NAME,
				'Household' as DIMENSION_NAME,
				SUM(RRL.WEIGHT) as WEIGHT,
				SUM(RRL.QUANTITY) as QUANTITY
			FROM TBL_REFUND_EXPORT as RE
			INNER JOIN TBL_REFUND_REQUEST as RR
				ON RE.REFUND_EXPORT_ID = RR.REFUND_EXPORT_ID
			INNER JOIN TBL_REFUND_REQUEST_LINE as RRL
				ON RR.REFUND_REQUEST_ID = RRL.REFUND_REQUEST_ID
			INNER JOIN TBL_PRODUCT as P
				ON RE.PRODUCT_ID = P.PRODUCT_ID
			INNER JOIN TBL_PRODUCT_CATEGORY as PC
				ON RRL.PRODUCT_CATEGORY_ID = PC.PRODUCT_CATEGORY_ID
			INNER JOIN TBL_RELATION as Prod
				ON RRL.PRODUCER_RELATION_ID = Prod.RELATION_ID
		";

		$this->_groupBy = "GROUP BY RRL.PRODUCER_RELATION_ID, RE.PRODUCT_ID, RRL.PRODUCT_CATEGORY_ID";
		$this->_orderBy = "ORDER BY PRODUCT_NAME, PRODUCT_CATEGORY_NAME, DIMENSION_NAME";
	}

	function _getQuarterExportWhere($quarter) {

		$date = currentDate();

		$year = isset($_REQUEST['YEAR']) ? (int) $_REQUEST['YEAR'] : $date->getYear();

		switch($quarter) {
			case 1:
				return 'MONTH(RE.EXPORT_DATE) IN (2, 3, 4) AND YEAR(RE.EXPORT_DATE) = '.$year;
			case 2:
				return 'MONTH(RE.EXPORT_DATE) IN (5, 6, 7) AND YEAR(RE.EXPORT_DATE) = '.$year;
			case 3:
				return 'MONTH(RE.EXPORT_DATE) IN (8, 9, 10) AND YEAR(RE.EXPORT_DATE) = '.$year;
			case 4:
				return '((MONTH(RE.EXPORT_DATE) IN (11, 12) AND YEAR(RE.EXPORT_DATE) = '.$year.') OR (MONTH(RE.EXPORT_DATE) = 1 AND YEAR(RE.EXPORT_DATE) = '.($year + 1).'))';
		}
	}

	function _setWhere()
	{
		$q = isset($_REQUEST['QUARTER'])?$_REQUEST['QUARTER']:1;

		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'RE.CRSO_RELATION_ID',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true));

		$this->_addWhere('PARTICIPANT_RELATION_ID',
			array(
				'field'=>'RE.PARTICIPANT_RELATION_ID',
				'callback' => array($this,'getParticipantSelect'),
				'required'=> true,
				'groupBy'=>''));

		$this->_addWhere('YEAR',
			array('field'=>'YEAR(RE.EXPORT_DATE)',
				'callback' => array($this,'getYearSelect'),
				'required' => true));

		$this->_addWhere('QUARTER',
			array(
				'field'=>'MONTH(RE.EXPORT_DATE)',
				'operator' => 'IN',
				'completedStatement' => $this->_getQuarterExportWhere($q),
				'callback' => array($this,'getQuarterSelect'),
				'required'=> true,
				'groupBy'=>''));

	}

	function getQuarterSelect($rv='setValue')
	{
		$date = currentDate();

		$year = isset($_REQUEST['YEAR']) ? (int) $_REQUEST['YEAR'] : $date->getYear();
		$maxQuarter = 4;
		if($year == $date->getYear()) {
			$maxQuarter = ceil(($date->getMonth() - 3) / 3);
		}

		$setValue = isset($_REQUEST['QUARTER'])?$_REQUEST['QUARTER']:1;
		$selectedValue = '';
		$select = new Form_Select('QUARTER');
		$select->addOption(' - '.$this->getText('Quarter Select').' - ', '', false);
		for($i=1;$i<($maxQuarter+1);$i++)
		{
			if($setValue == $i) {
				$selectedValue = 'Quarter '.$i;
			}
			$select->addOption('Quarter '.$i, $i, false);
		}
		$select->setSelectedValue((int) $setValue);
		$filter = $select->getHtml();
		return $$rv;
	}
}