<?php
require_once(ELC_CLASSES.'RefundRequestConfirmation.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');


/**
 * @package
 * @subpackage
 * @since 3-jan-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class RefundConfirmationDocumentList extends ELCDefaultList
{
	var $handle = 'RefundConfirmationDocumentList';


	function setQuery()
	{
		if(!isset($_REQUEST['useQuery']) || $_REQUEST['useQuery'] == 'rr_query')
		{
			$this->_query = "
						SELECT
							RRC.REFUND_REQUEST_CONFIRMATION_ID AS CONFIRMATION_ID, RRC.DOCUMENT_ID,
							RRC.AUTHORISATION_BY_TYPE AS CONFIRMED_BY, RRC.REVIEW_STATUS,
							'' AS RESUBMIT_CONFIRMATION, R.AUTHORIZED_BY
						FROM
							TBL_REFUND_REQUEST_CONFIRMATION AS RRC
						LEFT JOIN
							TBL_REFUND_REQUEST AS R
							ON RRC.REFUND_REQUEST_CONFIRMATION_ID = R.REFUND_REQUEST_CONFIRMATION_ID
						LEFT JOIN
							TBL_REFUND_EXPORT_DOCUMENT_REVIEW AS REDR
							ON R.REFUND_EXPORT_ID = REDR.REFUND_EXPORT_ID
						";
			$this->_groupBy = "GROUP BY RRC.REFUND_REQUEST_CONFIRMATION_ID";
			$this->_orderBy = "ORDER BY RRC.REFUND_REQUEST_CONFIRMATION_ID";
		}
		else
		{
			$this->_query = "
						SELECT
							POPC.PROOF_OF_PAYMENT_CONFIRMATION_ID AS CONFIRMATION_ID, POPC.DOCUMENT_ID,
							POPC.AUTHORISATION_BY_TYPE AS CONFIRMED_BY, POPC.REVIEW_STATUS,
							'' AS RESUBMIT_CONFIRMATION, POP.AUTHORIZED_BY
						FROM
							TBL_PROOF_OF_PAYMENT_CONFIRMATION AS POPC
						LEFT JOIN
							TBL_PROOF_OF_PAYMENT AS POP
							ON POPC.PROOF_OF_PAYMENT_CONFIRMATION_ID = POP.PROOF_OF_PAYMENT_CONFIRMATION_ID
						LEFT JOIN
							TBL_REFUND_EXPORT_DOCUMENT_REVIEW AS REDR
							ON POP.REFUND_EXPORT_ID = REDR.REFUND_EXPORT_ID
						";
			$this->_groupBy = "GROUP BY POPC.PROOF_OF_PAYMENT_CONFIRMATION_ID";
			$this->_orderBy = "ORDER BY POPC.PROOF_OF_PAYMENT_CONFIRMATION_ID";
		}

		$this->_addHiddenCol(array('AUTHORIZED_BY'));

	}

	function _setWhere()
	{
		if(!isset($_REQUEST['useQuery']) || $_REQUEST['useQuery'] == 'rr_query')
		{
			$this->_addWhere('CRSO_RELATION_ID',
				array(
					'field'=>'RRC.`CRSO_RELATION_ID`',
					'callback' => array($this,'getCRSOSelect'),
					'required'=>true,
					'related' => array('PARTICIPANT_RELATION_ID', 'CRSO_RELATION_ID')));

			$this->_addWhere('PRODUCT_ID',
				array(
					'field'=>'RRC.PRODUCT_ID',
					'callback' => array($this,'getProductSelect'),
					'required'=>true));

			$this->_addWhere('PARTICIPANT_RELATION_ID',
				array(
					'field'=>'RRC.`PARTICIPANT_RELATION_ID`',
					'callback' => array($this,'getParticipantSelect'),
					'related'=>array('PARTICIPANT_NAME')));
		}
		else
		{
			$this->_addWhere('CRSO_RELATION_ID',
				array(
					'field'=>'POPC.`CRSO_RELATION_ID`',
					'callback' => array($this,'getCRSOSelect'),
					'required'=>true,
					'related' => array('PARTICIPANT_RELATION_ID', 'CRSO_RELATION_ID')));

			$this->_addWhere('PRODUCT_ID',
				array(
					'field'=>'POPC.PRODUCT_ID',
					'callback' => array($this,'getProductSelect'),
					'required'=>true));

			$this->_addWhere('PARTICIPANT_RELATION_ID',
				array(
					'field'=>'POPC.`PARTICIPANT_RELATION_ID`',
					'callback' => array($this,'getParticipantSelect'),
					'related'=>array('PARTICIPANT_NAME')));
		}

			$this->_addWhere('useQuery',
				array(
					'field'=>'useQuery',
					'useinquery'=>false,
					'callback' => array($this,'getQuerySelect'),
					'required'=>true));
	}

	function getQuerySelect($rv='setValue')
	{
		$array = array(
			'rr_query'=>'Refund Request Confirmation',
			'pop_query'=>'Proof Of Payment Confirmation'
			);

		$setValue = isset($_REQUEST['useQuery'])?$_REQUEST['useQuery']:'rr_query';
		if(!isset($array[$setValue]))
			throw new IWAJAXPermissionException('Incorrect Query Filter Supplied');
		if(isset($$rv))
			return $$rv;
		$selectedValue = '';

		$select = new Form_Select('useQuery');
		$select->addOption(' - '.$this->getText('Filter Select').' - ', '', false);

		foreach($array as $query => $lang)
		{
			$trans = $this->getText($lang);
			if($setValue==$query)
				$selectedValue = $trans;
			$select->addOption($trans, $query, ($setValue==$query));
		}

		$filter = $select->getHtml();
		return $$rv;
	}

	function getProcessColumns()
	{
		return array(
			'tfGoToRefundRequests'=>array('CONFIRMATION_ID'),
			'tfDocumentLink'=>array('DOCUMENT_ID'),
			'tfTranslateStatus'=>array('REVIEW_STATUS'),
			'tfTranslateAuthType'=>array('CONFIRMED_BY'),
			'tfResubmit'=>array('RESUBMIT_CONFIRMATION')
		);
	}

	function tfSortingHat()
	{
		return array(
			'CONFIRMATION_ID'=>"'SortVal'",
			'DOCUMENT_ID'=>"'Int'",
			'CONFIRMED_BY'=>"'SortVal'",
			'REVIEW_STATUS'=>"'String'"
			);
	}

	function tfDocumentLink($args)
	{
		$args['hidename'] = true;
		return parent::tfDocumentLink($args);
	}

	function tfTranslateStatus($args)
	{
		$args['extra'] = sprintf(' sort_val="%s" ', $args['value']);
		list($trans,$class) = RefundRequestConfirmation::translateStatusForParticipant($this->langIf,$args['value']);
		return sprintf('<span class="%s">%s</span>',$class,$trans);
	}

	function tfTranslateAuthType($args)
	{
		return  CRSOParticipantProduct::translateAuthTypeForParticipant($this->langIf,$args['value']);
	}

	function tfGoToRefundRequests($args)
	{
		$args['extra'] = sprintf(' sort_val="%s" ', $args['value']);
		if($args['row']['REVIEW_STATUS'] == RR_CONFIRMATION_UNREVIEWED)
		{
			return $args['value'];
		}
		if(!isset($_REQUEST['useQuery']) || $_REQUEST['useQuery'] == 'rr_query')
			return '<a href="/'.SITE_PATH.'scripts/elc/participant/InterfaceRefundConfirmation.php?rrc[id]='.$args['row']['CONFIRMATION_ID'].'">'.$args['value'].'</a>';
		else
			return '<a href="/'.SITE_PATH.'scripts/elc/participant/InterfacePaymentConfirmation.php?popc[id]='.$args['row']['CONFIRMATION_ID'].'">'.$args['value'].'</a>';
	}

	function tfResubmit($args)
	{
		if($args['row']['AUTHORIZED_BY'] == $_SESSION['session_user_id'] &&
			$args['row']['REVIEW_STATUS'] == RR_CONFIRMATION_REJECTED)
		{
			if(!isset($_REQUEST['useQuery']) || $_REQUEST['useQuery'] == 'rr_query')
				return '<a href="/'.SITE_PATH.'scripts/elc/participant/InterfaceRefundConfirmation.php?rrc[id]='.$args['row']['CONFIRMATION_ID'].'">'.$this->langIf->getText('Resubmit Confirmation Document').'</a>';
			else
				return '<a href="/'.SITE_PATH.'scripts/elc/participant/InterfacePaymentConfirmation.php?popc[id]='.$args['row']['CONFIRMATION_ID'].'">'.$this->langIf->getText('Resubmit Confirmation Document').'</a>';
		}
		return $this->tfBooleanIcon($args);
	}
}
?>
