<?php
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'Country.php');
/**
 * @package
 * @subpackage
 * @since 11-okt-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class RefundExportList extends ELCDefaultList
{
	public $handle = 'RefundExportList';
	var $producer_exporter = PARTICIPANT_EXPORTER;
	var $_maxNum = array();
	var $_headType = array();
	var $_cols = false;

	private $virtualStock = null;

	protected function verifyCRSOHasRefundDetails()
	{
		if(in_array($_SESSION['session_group_group_id'],array(GROUP_SYSADMIN)))
			return;
		$crso_id = $this->getCRSOSelect();

		$rel = new Relation($this->db,$crso_id);
		$country = new Country($this->db,$rel->getCountry(),true);

		$ru = new RelationUser($this->db);

		if(!$country->has(HasCrsoRefundDetails,$_SESSION['selected_product_id'])||!$ru->hasClearance(CRSOClearanceRefundDetails, $_SESSION['session_user_id'], $crso_id))
			$this->_addHiddenCol(array('RefundRequestCreditQuantity','VIRTUAL_STOCK','RefundRequestCreditAmount'));
	}

	function setQuery()
	{
		if(!empty($_REQUEST['EXPORT_DATE']))
			$this->__setQueryExported();
		else
			$this->__setQueryNonExported();
	}
	function _filterQuery()
	{
		$this->verifyCRSOHasRefundDetails();
		$this->_addHiddenCol(array('PARTICIPANT_RELATION_ID','PRODUCER_RELATION_ID'));
		if(!empty($_REQUEST['EXPORT_DATE']))
			return $this->__filterQueryExported();
		else
			return $this->__filterQueryNonExported();
	}
	/**#@+
	 * Stuff for non-exported refunds/document reviews
	 */

        protected function showSufficientStock()
	{
		$crso_id = $this->getCRSOSelect();

		$rel = new Relation($this->db,$crso_id);
		$country = new Country($this->db,$rel->getCountry(),true);
		if($country->has(HasRefundInternalVAT,$_SESSION['selected_product_id']) &&
                    !$country->has(HasCreditViaProducer,$_SESSION['selected_product_id']))
                    return false;
                return true;
	}

	function __setQueryNonExported()
	{
		$this->_query = "SELECT RE.REFUND_REQUEST_ID, RE.PARTICIPANT_RELATION_ID, RRL.PRODUCER_RELATION_ID, R.`NAME` as PARTICIPANT_NAME,
				DATE(RE.CREATED) as SUBMISSION_DATE,
				REFUND_EXPORT_DATE,
				COUNT(DISTINCT RE.REFUND_REQUEST_ID) as RefundRequests,
				SUM(RRL.QUANTITY) as RefundRequestCreditQuantity,
				f_elcGetVirtualStock(RE.PARTICIPANT_RELATION_ID,RE.CRSO_RELATION_ID, NULL, RE.PRODUCT_ID) as VIRTUAL_STOCK,
				'' as SufficientStock,
				(SUM(RRL.QUANTITY)*f_elcProductFee(1,RE.CRSO_RELATION_ID,RRL.PRODUCT_CATEGORY_ID,NOW())) as RefundRequestCreditAmount,
				RE.STATUS, IF(ISNULL(C.REGION_ID),NULL,RE.IMPORTER_STATUS) as IMPORTER_STATUS

			FROM TBL_REFUND_REQUEST as RE
			INNER JOIN TBL_REFUND_REQUEST_LINE as RRL
				ON RE.REFUND_REQUEST_ID = RRL.REFUND_REQUEST_ID
			INNER JOIN TBL_RELATION as R
				ON RE.PARTICIPANT_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_COUNTRY as C
				ON RE.EXPORT_COUNTRY_ID = C.COUNTRY_ID";
		$this->_groupBy = "GROUP BY R.RELATION_ID";
		$this->_orderBy = "ORDER BY PARTICIPANT_NAME, RE.REFUND_REQUEST_ID";
	}
	function __filterQueryNonExported()
	{
		$this->verifyCRSOHasRefundDetails();
		$this->_addHiddenCol(array('REFUND_REQUEST_ID'));
		if(!empty($_REQUEST['PARTICIPANT_RELATION_ID']))
		{
			$this->_addGroupBy('RE.REFUND_REQUEST_ID');
			$this->_addHiddenCol(array('RefundRequests'));
		}
		else
			$this->_addHiddenCol(array('SUBMISSION_DATE','STATUS','REFUND_EXPORT_DATE', 'IMPORTER_STATUS'));
		return $this->_query;
	}
	/**#@-*/
	/**#@+
	 * Stuff for exported refunds/document reviews etc
	 */
	function __setQueryExported()
	{
		$this->_query = "SELECT R.`NAME` as PARTICIPANT_NAME, RE.PARTICIPANT_RELATION_ID,
				DATE(RR.CREATED) as SUBMISSION_DATE,
				RR.REFUND_EXPORT_DATE,
				COUNT(DISTINCT RR.REFUND_REQUEST_ID) as RefundRequests,
				SUM(RRL.QUANTITY) as RefundRequestCreditQuantity,
				SUM(RRL.EXPORTED_AMOUNT) as RefundRequestCreditAmount,
				RR.STATUS, IF(ISNULL(C.REGION_ID),NULL,RR.IMPORTER_STATUS) as IMPORTER_STATUS
			FROM TBL_REFUND_EXPORT as RE
			INNER JOIN TBL_RELATION as R
				ON RE.PARTICIPANT_RELATION_ID = R.RELATION_ID
			LEFT JOIN TBL_REFUND_REQUEST as RR
				ON RR.REFUND_EXPORT_ID = RE.REFUND_EXPORT_ID
			LEFT JOIN TBL_REFUND_REQUEST_LINE as RRL
				ON RR.REFUND_REQUEST_ID = RRL.REFUND_REQUEST_ID
			INNER JOIN TBL_COUNTRY as C
				ON RR.EXPORT_COUNTRY_ID = C.COUNTRY_ID";
		$this->_groupBy = "GROUP BY RE.REFUND_EXPORT_ID";
	}
	function __filterQueryExported()
	{
		if(!empty($_REQUEST['PARTICIPANT_RELATION_ID']))
		{
			$this->_addGroupBy('RR.REFUND_REQUEST_ID');
			$this->_addHiddenCol(array('RefundRequests','DocumentReviews'));
		}
		else
			$this->_addHiddenCol(array('SUBMISSION_DATE','STATUS','REFUND_EXPORT_DATE', 'IMPORTER_STATUS'));
		return $this->_query;
	}
	/**#@-*/

	function getProcessColumns()
	{
		return array(
			'tfValueProcessorQuantity'=>array('DocumentReviewCost','RefundRequestCreditAmount'),
			'tfValueProcessorQuantityNoDecimals' => array('RefundRequestCreditQuantity','DocumentReviews','RefundRequests'),
			'tfParticipantName'=> array('PARTICIPANT_NAME'),
			'tfRefundRequestStatus'=> array('STATUS'),
			'tfVirtualStock'=> array('VIRTUAL_STOCK'),
			'tfSufficientStock'=>array('SufficientStock'),
			'tfImporterStatus'=>array('IMPORTER_STATUS'),
			);
	}

	function getColumnModifier()
	{
		return array(
			'DocumentReviewCost'=>'style="text-align:right;"','RefundRequestCreditAmount'=>'style="text-align:right;"',
			'DocumentReviews'=>'style="text-align:right;"','RefundRequestCreditQuantity'=>'style="text-align:right;"',
			'RefundRequests'=>'style="text-align:right;"','VIRTUAL_STOCK'=>'style="text-align:right;"'
			);
	}

	function tfSortingHat()
	{
		return array(
			'SUBMISSION_DATE'=>"'String'",
			'REFUND_EXPORT_DATE'=>"'String'",
			'RefundRequestCreditQuantity'=>"'SortVal'",
			'PARTICIPANT_NAME'=>"'String'",
			'VIRTUAL_STOCK'=>"'SortVal'",
			'SufficientStock'=>"'String'",
			'RefundRequestCreditAmount'=>"'SortVal'",
			'STATUS'=>"'String'",
			'RefundRequests'=>"'Int'",
			'DocumentReviews'=>"'Int'",
			'IMPORTER_STATUS'=>"'SortVal'",
			);
	}

	function _setWhere()
	{
		$this->_addWhere('YEAR',
			array('field'=>'YEAR(`REFUND_EXPORT_DATE`)',
				'callback' => array($this,'getYearSelect'),
				'useinquery'=>false,
				'required' => true));

		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'RE.`CRSO_RELATION_ID`',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true,
				'related'=>array(),
				'groupBy'=>''));

		$this->_addWhere('PARTICIPANT_RELATION_ID',
			array(
				'field'=>'RE.`PARTICIPANT_RELATION_ID`',
				'callback' => array($this,'getParticipantSelect'),
				'related'=>array('PARTICIPANT_NAME'),
				'groupBy'=>''));

		if(empty($_REQUEST['EXPORT_DATE']))
		{
			$this->_addWhere('REFUND_EXPORT_ID',
				array(
					'field' => 'RE.REFUND_EXPORT_ID',
					'callback' => array($this, 'getIsExported'),
					'required'=>true,
					'related' => array(),
					'groupBy' => ''

				));
		}

		$this->_addWhere('EXPORT_DATE',
			array(
				'field' => 'RE.EXPORT_DATE',
				'callback' => array($this, 'getExportDate'),
				'groupBy' => ''

			));

		$this->_addWhere('PRODUCT_ID',
			array(
				'field'=>'RE.PRODUCT_ID',
				'callback' => array($this,'getProductSelect')));
	}

	function getIsExported($rv='setValue')
	{
		$filter = '';
		$setValue=null;
		$selectedValue = '';
		return $$rv;
	}

	function getExportDate($rv='setValue')
	{
		$setValue = isset($_REQUEST['EXPORT_DATE'])?$_REQUEST['EXPORT_DATE']:false;
		if(isset($$rv))
			return $$rv;

		$selectedValue = $this->getText('Pending Exports');

		if(isset($_REQUEST['CRSO_RELATION_ID']))
		{
			$q = $this->db->query('SELECT DISTINCT EXPORT_DATE, MONTH(EXPORT_DATE) AS `MONTH`, DAY(EXPORT_DATE) AS `DAY`, YEAR(EXPORT_DATE) AS `YEAR`
				FROM TBL_REFUND_EXPORT WHERE YEAR(EXPORT_DATE) = ? AND IS_EXPORTED=2 AND CRSO_RELATION_ID = ? '.
					(empty($_REQUEST['PARTICIPANT_RELATION_ID'])?'':' AND PARTICIPANT_RELATION_ID = '.$_REQUEST['PARTICIPANT_RELATION_ID']).'
				ORDER BY `YEAR`, `MONTH`');
			$q->bind_values($this->getYearSelect('setValue'), $_REQUEST['CRSO_RELATION_ID']);
			$q->execute();

			$select = new Form_Select('EXPORT_DATE');
			$select->addOption(' - '.$this->getText('Pending Exports').' - ', '', false);
			while($r = $q->fetch())
			{
				if($setValue==$r->EXPORT_DATE)
					$selectedValue = strftime('%d, %B %Y', mktime(1, 1, 1, $r->MONTH, $r->DAY, $r->YEAR) );
				$select->addOption(strftime('%d, %B %Y', mktime(1, 1, 1, $r->MONTH, $r->DAY, $r->YEAR) ), $r->EXPORT_DATE, false);
			}
			$select->setSelectedValue($setValue);

			$filter = $select->getHtml();
		}
		else
			$filter = '';
		return $$rv;
	}

	function tfImporterStatus($args)
	{
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		if(!is_numeric($args['value']))
			return '-';
		switch($args['value'])
		{
			case REFUND_IMPORTER_STATUS_PENDING:
				$trans = $this->getText('Pending');
				$class = 'status_new';
			break;
			case REFUND_IMPORTER_STATUS_REJECTED:
				$trans = $this->getText('Rejected');
				$class = 'status_forfeit';
			break;
			case REFUND_IMPORTER_STATUS_APPROVED:
				$trans = $this->getText('Approved');
				$class = 'status_verified';
			break;
		}
		return '<span class="'.$class.'">'.$trans.'</span>';
	}

	function tfParticipantName($args)
	{
		$rv = $this->tfLongNames($args);

		return '<a href="#" onclick="$$(\'PARTICIPANT_RELATION_ID\').value='.$args['row']['PARTICIPANT_RELATION_ID'].';_LoadOverview();">'.$rv.'</a>';
	}

	function tfSufficientStock($args)
	{
		if(!$this->showSufficientStock())
                    return '<em>N/A</em>';
                if(!empty($_REQUEST['CRSO_RELATION_ID']))
		{
			require_once(ELC_CLASSES.'Country.php');
			$crso = new Relation($this->db,$_REQUEST['CRSO_RELATION_ID']);
			$country = new Country($this->db,$crso->getCountry(),true);
			// Check for HasRefundInternalVat
			if($country->has(HasRefundInternalVAT,$_SESSION['selected_product_id']))
				return $this->tfSufficientStockInternalVAT($args);
		}
		// If participant is selected, do something different;
		if(!empty($_REQUEST['PARTICIPANT_RELATION_ID']))
		{
			$value = false;
			if(is_null($this->virtualStock))
			{
				$this->virtualStock = array();
				$stock = (int) $args['row']['VIRTUAL_STOCK'];

				$sql = "SELECT RR.REFUND_REQUEST_ID,SUM(QUANTITY) as Q
					FROM TBL_REFUND_REQUEST as RR
					INNER JOIN TBL_REFUND_REQUEST_LINE as RRL
						ON RR.REFUND_REQUEST_ID = RRL.REFUND_REQUEST_ID
					WHERE RR.PARTICIPANT_RELATION_ID = ? AND CRSO_RELATION_ID = ?
						AND RR.PRODUCT_ID = ?
						AND ISNULL(RR.REFUND_EXPORT_ID)
					GROUP BY RR.REFUND_REQUEST_ID
					ORDER BY RR.REFUND_REQUEST_ID";
				$q = $this->db->query($sql);
				$q->bind_values($_REQUEST['PARTICIPANT_RELATION_ID'],$_REQUEST['CRSO_RELATION_ID'],$_SESSION['selected_product_id']);
				$q->execute();
				while($r = $q->fetch())
				{
					$quantity = (int)$r->Q;
					$this->virtualStock[$r->REFUND_REQUEST_ID] = (!($quantity>$stock));
					$stock -= $quantity;
				}
			}
			$value = $this->virtualStock[$args['row']['REFUND_REQUEST_ID']];
		}
		else
			$value = ((int)$args['row']['RefundRequestCreditQuantity']>(int)$args['row']['VIRTUAL_STOCK'])?false:true;

		return $this->tfBooleanIcon(array('value'=>$value));
	}

	protected function fillStockPerProducer($args)
	{
		$participant_id = empty($_REQUEST['PARTICIPANT_RELATION_ID'])?$args['row']['PARTICIPANT_RELATION_ID']:$_REQUEST['PARTICIPANT_RELATION_ID'];
		$vstock = array();

		require_once(ELC_CLASSES.'RefundExport.php');
		require_once(ELC_CLASSES.'ProofOfPayment.php');

		$re = new RefundExport($this->db);
		$pop = new ProofOfPayment($this->db);

		$r = $pop->getProducers($_REQUEST['CRSO_RELATION_ID'],$participant_id, PROOF_OF_PAYMENT_APPROVED, $_SESSION['selected_product_id']);
		// Get VIRTUAL STOCK per Participant per Producer
		// First get all the different Producers for this participant
		foreach($r as $producer_id)
			$vstock[$producer_id] = $re->getVirtualStock($_REQUEST['CRSO_RELATION_ID'],$participant_id, $producer_id, $_SESSION['selected_product_id']);
		$this->virtualStock[$participant_id] = $vstock;
	}

	function tfSufficientStockInternalVAT($args)
	{
		$participant_id = empty($_REQUEST['PARTICIPANT_RELATION_ID'])?$args['row']['PARTICIPANT_RELATION_ID']:$_REQUEST['PARTICIPANT_RELATION_ID'];
		if(is_null($this->virtualStock)||!isset($this->virtualStock[$participant_id]))
			$this->fillStockPerProducer($args);

		require_once(ELC_CLASSES.'RefundRequest.php');
		// Loop the whole Refund, otherwise we wont be able to check.
		// REFUND_REQUEST_ID
		$rr = new RefundRequest($this->db,$args['row']['REFUND_REQUEST_ID'],true);
		foreach($rr->lines as $line)
		{
			$producer_id = $line->producer_relation->id;
			if(!isset($this->virtualStock[$participant_id][$producer_id]))
			{
				$value = false;
				break;
			}

			$this->virtualStock[$participant_id][$producer_id]-=$args['row']['RefundRequestCreditQuantity'];
			$value = ($this->virtualStock[$participant_id][$producer_id] > 0);
			if(!$value)
				break;
		}
		return $this->tfBooleanIcon(array('value'=>$value));
	}

	function tfVirtualStock($args)
	{
		$this->_headType[$args['head']] = 0;
		if(!empty($_REQUEST['PARTICIPANT_RELATION_ID']))
		{
			$args['extra'] .= sprintf(' sort_val="%s" ', 0);
			$this->_maxNum[$args['head']] = $args['row']['VIRTUAL_STOCK'];
			return '-';
		}
		isset($this->_maxNum[$args['head']]) || $this->_maxNum[$args['head']] = 0;
		$this->_maxNum[$args['head']] += $args['value'];
		$this->_cols = $args['visibleColumns'];
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		return locale_nf($args['value']);
	}

	function tfValueProcessorQuantityNoDecimals($args)
	{
		isset($this->_maxNum[$args['head']]) || $this->_maxNum[$args['head']] = 0;
		$this->_headType[$args['head']] = 0;
		$this->_maxNum[$args['head']] += (int)$args['value'];
		$this->_cols = $args['visibleColumns'];
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
    	return locale_nf($args['value']);
	}

	function tfValueProcessorQuantity($args)
	{
		isset($this->_maxNum[$args['head']]) || $this->_maxNum[$args['head']] = 0;
		$this->_headType[$args['head']] = 2;
		$this->_maxNum[$args['head']] += $args['value'];
		$this->_cols = $args['visibleColumns'];
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
    	return locale_nf($args['value'],2);
	}

	function tfFooter() // getClass
	{
		if(!$this->_cols)
			return false;

		$cols = array_flip($this->_cols);
		$rv = explode(',',str_repeat(',',(sizeof($this->_cols)-1)));
		foreach($this->_maxNum as $head => $val)
		{
			if(!isset($cols[$head]))
				continue;
			$pos = $cols[$head];
			$round = $this->_headType[$head];
			$rv[$pos] = '<span style="float:right; font-weight: bold;">'.locale_nf($val,$round).'</span><span style="clear:both;"></span>';
		}
		return $rv;
	}

	function setEmptyText()
	{
		return $this->getText('No Refund Exports or Pending Exports found for current selection');
	}
}

?>
