<?php
require_once(ELC_CLASSES.'Participant.php');
/**
 * @package ELC
 * @subpackage List
 * @since 14-jul-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class RefundRequestList extends ELCDefaultList
{
	var $producer_exporter = PARTICIPANT_EXPORTER;
	var $handle = 'RefundRequestList';
	var $_maxQuantity = 0;
	var $_cols = false;
	function setQuery()
	{
		$this->_query = "SELECT R1.`NAME` as CRSO_NAME, R.`NAME` as PARTICIPANT_NAME,
				C.`NAME` as EXPORT_COUNTRY_NAME, C.REGION_ID as EU,
				IFNULL(IMPORTER_VAT_NO,'-') as IMPORTER_VAT, IMPORTER_COMPANY_NAME,
				 RR.STATUS, SUM(RRL.QUANTITY) as QUANTITY
			FROM TBL_REFUND_REQUEST as RR
			INNER JOIN TBL_REFUND_REQUEST_LINE as RRL
				ON RR.REFUND_REQUEST_ID = RRL.REFUND_REQUEST_ID
			INNER JOIN TBL_RELATION as R
				ON RR.PARTICIPANT_RELATION_ID = R.RELATION_ID
			INNER JOIN TBL_RELATION as R1
				ON RR.CRSO_RELATION_ID = R1.RELATION_ID
			INNER JOIN TBL_COUNTRY as C
				ON RR.EXPORT_COUNTRY_ID = C.COUNTRY_ID";
		$this->_groupBy = "GROUP BY RR.CRSO_RELATION_ID";
		$this->_orderBy = "ORDER BY CRSO_NAME, PARTICIPANT_NAME";
	}

	function _filterQuery()
	{
		if(!$this->_groupByArray)
			$this->_addHiddenCol(array('PARTICIPANT_NAME','EXPORT_COUNTRY_NAME','IMPORTER_VAT','IMPORTER_COMPANY_NAME'));
		return $this->_query;
	}

	function _setWhere()
	{
		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'RR.`CRSO_RELATION_ID`',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true,
				'related'=>array('CRSO_NAME','EXPORT_COUNTRY_NAME','IMPORTER_VAT','IMPORTER_COMPANY_NAME','EU','STATUS'),
				'groupBy'=>'RR.PARTICIPANT_RELATION_ID'));
		$this->_addWhere('PARTICIPANT_RELATION_ID',
			array(
				'field'=>'RR.`PARTICIPANT_RELATION_ID`',
				'callback' => array($this,'getParticipantSelect'),
				'related'=>array('PARTICIPANT_NAME'),
				'showCols'=>array('EXPORT_COUNTRY_NAME','IMPORTER_VAT','IMPORTER_COMPANY_NAME','EU','STATUS'),
				'groupBy'=>'RR.`REFUND_REQUEST_ID`'));
		$this->_addWhere('COUNTRY_ID',
			array(
				'field'=>'RR.EXPORT_COUNTRY_ID',
				'callback' => array($this,'getExportCountrySelect'),
				'related'=>'EXPORT_COUNTRY_NAME',
				'showCols'=>array('IMPORTER_VAT','IMPORTER_COMPANY_NAME'),
				'groupBy'=>'RR.`REFUND_REQUEST_ID`'));
		$this->_addWhere('PRODUCT_ID',
			array(
				'field'=>'RR.PRODUCT_ID',
				'callback' => array($this,'getProductSelect')));
	}

	function getProcessColumns()
	{
		return array(
			'tfValueProcessorQuantityNoDecimals'=>array('QUANTITY'),
			'tfBooleanIcon' => array('EU'),
			'tfRefundRequestStatus' => array('STATUS'),
			'tfVat' => array('IMPORTER_VAT'),
			'tfLongNames' => array('IMPORTER_COMPANY_NAME')
			);
	}

	function getColumnModifier()
	{
		return array('QUANTITY'=>'style="text-align:right;"');
	}

	function getExportCountrySelect($rv)
	{
		$setValue = isset($_REQUEST['COUNTRY_ID'])?$_REQUEST['COUNTRY_ID']:false;
		if(isset($$rv))
			return $$rv;
		$selectedValue = '';

		require_once(ELC_CLASSES.'RefundRequest.php');
		$rr = new RefundRequest($this->db);
		$q = $rr->selectExportCountrySelect(null,null,$_SESSION['selected_product_id']);
		$select = new Form_Select('COUNTRY_ID');
		$select->addOption(' - '.$this->getText('Country Select').' - ', '', false);
		while($r = $q->fetch())
		{
			if($setValue==$r->COUNTRY_ID)
				$selectedValue = $r->COUNTRY_NAME;
			$select->addOption($r->COUNTRY_NAME, $r->COUNTRY_ID, false);
		}
		$select->setSelectedValue($setValue);

		$filter = $select->getHtml();

		return $$rv;

	}

	function tfRefundRequestStatus($args)
	{
		require_once(ELC_CLASSES.'RefundRequest.php');
		$rr = new RefundRequest($this->db);
		$trans = $rr->translateStatus($this->langIf, $args['value']);
		return '<span class="'.$trans[1].'">'.$trans[0].'</span>';
	}

	function tfValueProcessorQuantityNoDecimals($args)
	{
		$this->_maxQuantity += $args['value'];
		$this->_cols = $args['visibleColumnCount'];
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
    	return locale_nf($args['value']);
	}

	function tfFooter() // getClass
	{
		if(!$this->_cols||in_array('AGGREGATED_QUANTITY',$this->_hiddenCols))
			return false;

		$rv = explode(',',str_repeat(',',($this->_cols-2)));
		$rv[] = '<span style="float:left; font-weight: bold;">Total</span><span style="float:right; font-weight: bold;">'.locale_nf($this->_maxQuantity).'</span><span style="clear:both;"></span>';
		return $rv;
	}
}
?>
