<?php
/**
 * @package
 * @subpackage
 * @since 24-apr-2007
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2007 BaseSoft B.V.
 */
class ReminderOverviewList extends ELCDefaultList
{
	var $handle = 'ReminderOverviewList';

	function setQuery()
	{
		$this->_query = "SELECT CRSO_RELATION_ID, C.`NAME` as CRSO_NAME, P.`NAME` as PARTICIPANT_NAME,
			DOCUMENT_ID as REMINDER,
			READ_DATE as DOWNLOADED, REMINDER_LEVEL, INVOICE_DATE as REMINDER_DATE
			FROM TBL_CRSO_PARTICIPANT_REMINDER as CPR
			INNER JOIN TBL_RELATION as C
				ON CPR.CRSO_RELATION_ID = C.RELATION_ID
			INNER JOIN TBL_RELATION as P
				ON CPR.PARTICIPANT_RELATION_ID = P.RELATION_ID";
		$this->_addHiddenCol(array('CRSO_RELATION_ID'));

		if($_SESSION['session_group_group_id']==GROUP_CRSO)
			$this->_addHiddenCol(array('REMINDER'));

		$this->_orderBy = "ORDER BY CRSO_RELATION_ID, READ_DATE,  IMPORT_DATE, INVOICE_DATE";
	}

	function _setWhere()
	{
		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'CPR.`CRSO_RELATION_ID`',
				'callback' => array($this,'getCRSOSelect'),
				'related'=>array('CRSO_NAME','CRSO_RELATION_ID')));

		$this->_addWhere('PARTICIPANT_RELATION_ID',
			array(
				'field'=>'CPR.`PARTICIPANT_RELATION_ID`',
				'callback' => array($this,'getParticipantSelect'),
				'related'=>array('PARTICIPANT_NAME')));

		$completedStatement = '(1=1)';
		if(isset($_REQUEST['READ_DATE']))
		{
			switch($_REQUEST['READ_DATE'])
			{
				case 'Read':
					$completedStatement = ' NOT ISNULL(READ_DATE) ';
				break;
				case 'Unread':
					$completedStatement = ' ISNULL(READ_DATE) ';
				break;
			}
		}
		$this->_addWhere('READ_DATE',
				array(
				'field'=>'READ_DATE',
				'callback' => array($this,'getReadSelect'),
				'related'=>array('DOWNLOADED'),
				'completedStatement' => $completedStatement
			));


	}

	function getReadSelect($rv='setValue')
	{
		$array = array('All','Read','Unread');

		if(isset($_REQUEST['READ_DATE'])&&!in_array($_REQUEST['READ_DATE'],$array))
			throw new IWAJAXPermissionException('Incorrect Read Filter Supplied');

		$setValue = isset($_REQUEST['READ_DATE'])?$_REQUEST['READ_DATE']:false;
		if(isset($$rv))
			return $$rv;
		$selectedValue = '';

		$select = new Form_Select('READ_DATE');

		foreach($array as $title)
		{
			$trans = $this->getText($title.'Reminders');
			if($setValue==$title)
				$selectedValue = $trans;
			$select->addOption($trans,$title);
		}

		$select->setSelectedValue($setValue);

		$filter = $select->getHtml();

		return $$rv;
	}

	function getProcessColumns()
	{
		return array(
			'tfBooleanIcon'=>array('DOWNLOADED'),
			'tfReminderLink'=>array('REMINDER')
		);
	}

	function tfReminderLink($args)
	{
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		switch($_SESSION['session_group_group_id'])
		{
			case GROUP_PARTICIPANT:
				return sprintf('<a href="/%sscripts/elc/participant/download_reminder.php?Document[id]=%s&crso[id]=%s">%s</a>',SITE_PATH,$args['value'],$args['row']['CRSO_RELATION_ID'],$this->getText('Download Reminder'));
			break;
			case GROUP_SYSADMIN:
				return sprintf('<a href="/%sscripts/i-w/document/download.php?Document[id]=%s">%s</a>',SITE_PATH,$args['value'],$this->getText('Download Reminder'));
			break;
			case GROUP_CRSO:
				return '';
			break;
		}
		return '';
	}

	function tfSortingHat()
	{
		return array(
			'CRSO_NAME'=>"'String'",
			'PARTICIPANT_NAME'=>"'String'",
			'REMINDER'=>"'SortVal'",
			'DOWNLOADED'=>"'String'",
			'INVOICE_DATE'=>"'String'",
			'IMPORT_DATE'=>"'String'"
			);
	}

}

?>
