<?php
require_once(ELC_CLASSES.'Declaration.php');
/**
 *
 * @author marten
 *
 */
class ResubmitDeviationList extends ELCDefaultList {

	public $handle = 'ResubmitDeviationList';
	var $_row = array();
	protected $custom_dev = null;
	protected $_totals = array('OriginalQuantity'=>0,'ResubmitQuantity'=>0);

	public function setQuery() {

		$this->_query = "SELECT R1.`NAME` as PARTICIPANT_NAME,
				D1.MONTH as PERIOD,
				SUM(DL1.QUANTITY) as ResubmitQuantity,
				SUM(DL2.QUANTITY) as OriginalQuantity,
				(SUM(DL1.QUANTITY)-SUM(DL2.QUANTITY)) as ABSOLUTE_DEVIATION,
				IF(
					SUM(DL1.QUANTITY)=0, -100,
					IF(
						SUM(DL2.QUANTITY)=0, 100,
						((SUM(DL1.QUANTITY)/SUM(DL2.QUANTITY)*100)-100)
					)
				) as DEVIATION,
				'' as TEN_PERCENT, '' as TWENTYFIVE_PERCENT, '' as FIFTY_PERCENT, '' as CUSTOM_DEVIATION,
				(BIT_OR(DL1.STATUS)) as DECLARATION_STATUS
			FROM TBL_DECLARATION as D1
			INNER JOIN TBL_DECLARATION_LINE as DL1
				ON D1.DECLARATION_ID = DL1.DECLARATION_ID
			INNER JOIN TBL_RELATION as R1
				ON D1.PARTICIPANT_RELATION_ID = R1.RELATION_ID
			INNER JOIN TBL_DECLARATION as D2
				ON D1.PARENT_DECLARATION_ID = D2.DECLARATION_ID
			INNER JOIN TBL_DECLARATION_LINE as DL2
				ON D2.DECLARATION_ID = DL2.DECLARATION_ID";

		$this->_groupBy = "GROUP BY D1.`DECLARATION_ID`";
		$this->_orderBy = "ORDER BY ABSOLUTE_DEVIATION DESC, R1.NAME, PERIOD";
	}

	function _filterQuery() {
		$this->_addHiddenCol(array('DECLARATION_STATUS'));
		if(isset($_REQUEST['MONTH'])&&is_numeric($_REQUEST['MONTH']))
			$this->_addHiddenCol(array('PERIOD'));

		require_once(ELC_CLASSES.'CRSOProduct.php');
		$cp = new CRSOProduct($this->db);
		$cp->set($this->getCRSOSelect(), $this->getProductSelect());
		if($cp->resubmit_deviation_trigger_percentage) {
			$this->custom_dev = $cp->resubmit_deviation_trigger_percentage;
			$this->_addHiddenCol(array('TEN_PERCENT','TWENTYFIVE_PERCENT','FIFTY_PERCENT'));
		} else {
			$this->_addHiddenCol(array('CUSTOM_DEVIATION'));
		}

		return $this->_query;
	}

	function _getHaving() {
		return ' HAVING DECLARATION_STATUS & '.DECLARATION_RESUBMIT;
	}

	function _setWhere() {
		$this->_addWhere('YEAR',
			array('field'=>'D1.`YEAR`',
				'callback' => array($this,'getYearSelect'),
				'required' => true
			)
		);

		$this->_addWhere('MONTH',
			array(
				'field'=>'D1.`MONTH`',
				'callback' => array($this,'getMonthSelect'),
				'related'=>array('PERIOD')
			)
		);

		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'D1.CRSO_RELATION_ID',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true
			)
		);

		$this->_addWhere('PRODUCT_ID',
			array(
				'field'=>'D1.PRODUCT_ID',
				'callback' => array($this,'getProductSelect'),
				'required'=>true
			)
		);
	}

	function setEmptyText()
	{
		return $this->langIf->getText('No resubmission for the current filter');
	}

	function getProcessColumns()
	{
		return array(
			'tfValueProcessorQuantity'=>array('ResubmitQuantity','OriginalQuantity','ABSOLUTE_DEVIATION'),
			'tfPercentageNoDecimals'=>array('DEVIATION'),
			'tfMonth'=>array('PERIOD'),
			'tfDeviationExceeds'=>array('TEN_PERCENT','TWENTYFIVE_PERCENT','FIFTY_PERCENT','CUSTOM_DEVIATION'),
			'tfLongNames'=>array('PARTICIPANT_NAME')
			);
	}

	function tfValueProcessorQuantity(&$args)
	{
		$this->_row = array_keys($args['row']);
		if($args['head']!='ABSOLUTE_DEVIATION')
			$this->_totals[$args['head']] += $args['value'];
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);

		if(!is_numeric($args['value']))
			return 'n/d';
    	return locale_nf($args['value'], 0);
	}

	function tfDeviationExceeds($args,$getBool = false)
	{
		$check = 0;
		switch($args['head'])
		{
			case 'TEN_PERCENT':
				$check = 10;
			break;
			case 'TWENTYFIVE_PERCENT':
				$check = 25;
			break;
			case 'FIFTY_PERCENT':
				$check = 50;
			break;
			case 'CUSTOM_DEVIATION':
				$check = $this->custom_dev;
			break;
		}

		$val = $args['row']['DEVIATION'];
		if(!$val)
			return '-';
		$val = ($val<0)?($val*-1):$val;

		$bool = $val<$check;
		if($getBool)
			return $bool;

		return $this->tfBooleanIcon(array('value'=>$bool));
	}

	function tfPercentageNoDecimals($args)
	{
		$val = parent::tfPercentageNoDecimals($args);
		if(!$args['value'])
			return '-';
		if(!$this->mkHighlight($args))
			return $val;
		return sprintf('<div style="font-weight: bold;">%s</div>',$val);
	}

	function tfLongNames($args)
	{
		$val = parent::tfLongNames($args);
		if(!$this->mkHighlight($args))
			return $val;
		return sprintf('<div style="font-weight: bold;"><span style="color: #00D">*</span> %s</div>',$val);
	}

	function tfSortingHat()
	{
		return array(
			'MONTH'=>"'SortVal'",
			'PARTICIPANT_NAME'=>"'String'",
			'ResubmitQuantity'=>"'SortVal'",
			'OriginalQuantity'=>"'SortVal'",
			'DEVIATION'=>"'SortVal'",
			'ABSOLUTE_DEVIATION'=>"'SortVal'",
			'TEN_PERCENT'=>"'String'",
			'TWENTYFIVE_PERCENT'=>"'String'",
			'FIFTY_PERCENT'=>"'String'",
			'CUSTOM_DEVIATION'=>"'String'"
			);
	}

	function mkHighlight($args)
	{
		if(!$this->custom_dev)
			return false;

		$args['head'] = 'CUSTOM_DEVIATION';
		return $this->tfDeviationExceeds($args,true);
	}

	function getColumnModifier()
	{
		return array('ABSOLUTE_DEVIATION'=>'style="text-align:right;"','DEVIATION'=>'style="text-align:right;"','ResubmitQuantity'=>'style="text-align:right;"','OriginalQuantity'=>'style="text-align:right;"','ResubmitWeight'=>'style="text-align:right;"','OriginalWeight'=>'style="text-align:right;"');
	}

	function getButtonBarText()
	{
		if(!$this->custom_dev)
			return false;
		return '<span style="color: #00D">*</span> = '.$this->getText('Exceeded the set percentage deviation from the original declaration.');
	}
}


?>