<?php
require_once(ELC_CLASSES.'Trust.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'Country.php');
require_once(IWARE.'group.php');
/**
 * This list contains all the trustees for a subrogator.
 * @package
 * @subpackage
 * @since 16-nov-2006
 * @author Marten Koedam <marten@dalines.net>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */
class SubrogationTrusteeList extends ELCDefaultList
{
	var $handle = 'SubrogationTrusteeList';
	var $type = L_SUBROGATOR;
	protected $crso_id = false;
	protected $country = false;
	protected $_totals = array('WEIGHT'=>0,'QUANTITY'=>0);
	protected $_row = array();

	function setQuery()
	{
		$this->crso_id = $this->getCRSOSelect('setValue');
		if(!$this->crso_id)
			return;

		$relation = new Relation($this->db,$this->crso_id);
		$this->country = new Country($this->db,$relation->getCountry(),true);

		$this->_query = "SELECT T.TRUST_ID,
				R1.`NAME` as SUBROGATOR, R1.VAT_NO as SUBROGATOR_VAT,
				R2.`NAME` as TRUSTEE, R2.VAT_NO as TRUSTEE_VAT,
				T.CREATED as STARTED,
				T.`APPROVED`,
				PD.`MONTH`,PRODUCT_CATEGORY_ID,BRAND_ID,
				SUM(PD.QUANTITY) as QUANTITY, SUM(WEIGHT) as WEIGHT
			FROM TBL_TRUST as T
			INNER JOIN TBL_RELATION as R1
				ON T.SUBROGATOR_RELATION_ID = R1.RELATION_ID
			INNER JOIN TBL_RELATION as R2
				ON T.TRUSTEE_RELATION_ID = R2.RELATION_ID
			INNER JOIN TBL_PROVISIONAL_DECLARATION as PD
				ON T.TRUST_ID = PD.TRUST_ID
			";

		if($this->type==L_SUBROGATOR)
			$this->_addHiddenCol('SUBROGATOR_VAT');
		else
			$this->_addHiddenCol('TRUSTEE_VAT');

		if(empty($_REQUEST['TRUST_ID']))
		{
			$this->_groupBy = "GROUP BY T.TRUST_ID";
			$this->_addHiddenCol(array('MONTH','BRAND_ID','PRODUCT_CATEGORY_ID'));
		}
		else
		{
			$this->_addHiddenCol(array('SUBROGATOR','SUBROGATOR_VAT','TRUSTEE_VAT','TRUSTEE','STARTED','APPROVED'));
			$this->_groupBy = "GROUP BY PD.MONTH";
			if(!empty($_REQUEST['MONTH']))
				$this->_groupBy = "GROUP BY PD.MONTH, PD.BRAND_ID, PD.PRODUCT_CATEGORY_ID";
			else
				$this->_addHiddenCol(array('BRAND_ID','PRODUCT_CATEGORY_ID'));
		}
		if(!$this->country->has(HasDeclByCategory,$_SESSION['selected_product_id']))
			$this->_addHiddenCol('PRODUCT_CATEGORY');
		if(!$this->country->has(HasDeclByBrand,$_SESSION['selected_product_id']))
			$this->_addHiddenCol('BRAND_ID');
		if(!$this->country->has(HasDeclOfWeight,$_SESSION['selected_product_id']))
			$this->_addHiddenCol('WEIGHT');
	}

	function _setWhere()
	{
		$this->_addWhere('YEAR',
			array('field'=>'PD.`YEAR`',
				'callback' => array($this,'getYearSelect'),
				'required' => true));
		$this->_addWhere('CRSO_RELATION_ID',
			array(
				'field'=>'T.CRSO_RELATION_ID',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true));
		$props = array(
				'field'=>'T.TRUST_ID',
				'callback' => array($this,'getTrustSelect'));
		//if($_SESSION['session_group_group_id']==GROUP_PARTICIPANT)
			//$props['required'] = true;
		$this->_addWhere('TRUST_ID',
			$props);
		$this->_addWhere('PRODUCT_ID',
			array(
				'field'=>'T.`PRODUCT_ID`',
				'callback' => array($this,'getProductSelect')));
		$this->_addWhere('MONTH',
			array(
				'field'=>'PD.`MONTH`',
				'callback' => array($this,'getMonthSelect')));
		if($_SESSION['session_group_group_id']==GROUP_PARTICIPANT)
		{
			if($this->type==L_SUBROGATOR)
			{
				$this->_addWhere('SUBROGATOR_RELATION_ID',
					array(
						'field'=>'T.`SUBROGATOR_RELATION_ID`',
						'callback' => array($this,'getSubrogatorTrusteeID')));
			}
			else
			{
				$this->_addWhere('TRUSTEE_RELATION_ID',
					array(
						'field'=>'T.`TRUSTEE_RELATION_ID`',
						'callback' => array($this,'getSubrogatorTrusteeID')));
			}
		}
	}

	function getSubrogatorTrusteeID($rv='setValue')
	{
		$selectedValue = $filter = '';
		$setValue = $_SESSION['selected_participant_id'];
		return $$rv;
	}

	function getTrustSelect($rv='setValue')
	{
		$setValue = $selectedValue = $filter = '';
		if(!$this->crso_id)
			return $$rv;

		$select = new Form_Select('TRUST_ID');
		$select->addOption(' - '.$this->getText('Trust Select').' - ', '', false);
		$setValue = empty($_REQUEST['TRUST_ID'])?'':$_REQUEST['TRUST_ID'];

		$sql = "SELECT T.TRUST_ID,
				R1.`NAME` as SUBROGATOR, R2.`NAME` as TRUSTEE
			FROM TBL_TRUST as T
			INNER JOIN TBL_RELATION as R1
				ON T.SUBROGATOR_RELATION_ID = R1.RELATION_ID
			INNER JOIN TBL_RELATION as R2
				ON T.TRUSTEE_RELATION_ID = R2.RELATION_ID
			WHERE T.CRSO_RELATION_ID = ?
				AND T.PRODUCT_ID = ? ";
		if($_SESSION['session_group_group_id']==GROUP_PARTICIPANT)
		{
			if($this->type==L_SUBROGATOR)
				$sql .= " AND T.SUBROGATOR_RELATION_ID = ".$_SESSION['selected_participant_id']." ";
			else
				$sql .= " AND T.TRUSTEE_RELATION_ID = ".$_SESSION['selected_participant_id']." ";
		}
		$q = $this->db->query($sql);
		$q->bind_values($this->crso_id,$_SESSION['selected_product_id']);
		$q->execute();
		$rs = $q->fetchAll();
		foreach($rs as $r)
		{
			if($_SESSION['session_group_group_id']==GROUP_PARTICIPANT&&(count($rs)==1))
				$setValue = $r->TRUST_ID;

			$subrogator = ((mb_strlen($r->SUBROGATOR)>20)?mb_substr($r->SUBROGATOR,0,20).'...':$r->SUBROGATOR);
			$trustee = ((mb_strlen($r->TRUSTEE)>20)?mb_substr($r->TRUSTEE,0,20).'...':$r->TRUSTEE);
			$val = ($this->type==L_SUBROGATOR)?$subrogator.' => '.$trustee:$trustee.' <= '.$subrogator;
			if($setValue==$r->TRUST_ID)
				$selectedValue = $val;
			$select->addOption($val, $r->TRUST_ID, false);
		}
		$select->setSelectedValue($setValue);
		$filter = $select->getHtml();
		return $$rv;
	}

	function getProcessColumns()
	{
		return array(
			'tfTrustApproved'=>array('APPROVED'),
			'tfMonth'=>array('MONTH'),
			'tfTrustLink'=>array('TRUST_ID'),
			'tfBrand'=>array('BRAND_ID'),
			'tfProductCategory'=>array('PRODUCT_CATEGORY_ID'),
			'tfValueProcessorQuantity'=>array('WEIGHT','QUANTITY')
			);
	}

	function tfSortingHat()
	{
		return array(
			'MONTH'=>"'SortVal'",
			'TRUSTEE'=>"'String'",
			'SUBROGATOR'=>"'String'",
			'QUANTITY'=>"'SortVal'",
			'WEIGHT'=>"'SortVal'",
			'APPROVED'=>"'String'",
			'STARTED'=>"'String'",
			'SUBROGATOR_VAT'=>"'String'",
			'TRUSTEE_VAT'=>"'String'",
			'TRUST_ID'=>"'SortVal'",
			'BRAND_ID'=>"'String'",
			'PRODUCT_CATEGORY_ID'=>"'String'"
			);
	}

	function getColumnModifier()
	{
		return array('QUANTITY'=>'style="text-align:right;"','WEIGHT'=>'style="text-align:right;"');
	}

	function tfTrustLink($args)
	{
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
		return sprintf('<a href="#" onclick="$$(\'TRUST_ID\').value='.$args['value'].';_LoadOverview();">%s</a>',$args['value']);
	}

	function tfTrustApproved($args)
	{
		$args['value'] = ($args['value']==TRUST_APPROVED);
		return $this->tfBooleanIcon($args);
	}

	function tfValueProcessorQuantity(&$args)
	{
		$this->_row = array_keys($args['row']);
		$this->_totals[$args['head']] += $args['value'];
		$args['extra'] .= sprintf(' sort_val="%s" ', $args['value']);
    	return locale_nf($args['value'], 0);
	}

	function tfFooter()
	{
		$rv = array();
		$showCols = array_diff($this->_row,$this->_hiddenCols);
		foreach($showCols as $column)
			$rv[$column]=isset($this->_totals[$column])?'<div style="text-align: right; font-weight: bold;">'.(is_numeric($this->_totals[$column])?locale_nf($this->_totals[$column],0):$this->_totals[$column]).'</div>':'';
		return $rv;
	}
}
define('L_SUBROGATOR',1);
define('L_TRUSTEE',2);
?>
