<?php
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'ProofOfPayment.php');
/**
 * User: marten
 * Date: 17/6/16
 * Time: 9:24 AM
 */

class VirtualStockList extends ELCDefaultList {

	var $handle = 'VirtualStockList';
	var $producer_exporter = PARTICIPANT_EXPORTER;

	function setQuery() {

		$this->_query = "SELECT
			REFUND_REQUESTER.NAME as PARTICIPANT_NAME,
			PRODUCER.NAME as PRODUCER_NAME,
			COUNT(APPROVED.PROOF_OF_PAYMENT_ID) as POPS_APPROVED,
			COUNT(PENDING.PROOF_OF_PAYMENT_ID) as POPS_PENDING,
			COUNT(REJECTED.PROOF_OF_PAYMENT_ID) as POPS_REJECTED,
			f_elcGetVirtualStock(REFUND_REQUESTER.RELATION_ID, pop.CRSO_RELATION_ID, PRODUCER.RELATION_ID, pop.PRODUCT_ID) as VIRTUAL_STOCK,
			f_elcGetRefundStock(REFUND_REQUESTER.RELATION_ID, pop.CRSO_RELATION_ID, pop.PRODUCT_ID, PRODUCER.RELATION_ID) as POE_STOCK,
			'' as VSTOCK_STATUS
		FROM TBL_PROOF_OF_PAYMENT as pop
		INNER JOIN TBL_RELATION as REFUND_REQUESTER
			ON pop.PARTICIPANT_RELATION_ID = REFUND_REQUESTER.RELATION_ID
		INNER JOIN TBL_RELATION as PRODUCER
			ON pop.PRODUCER_RELATION_ID = PRODUCER.RELATION_ID
		LEFT JOIN TBL_PROOF_OF_PAYMENT as APPROVED
			ON pop.PROOF_OF_PAYMENT_ID = APPROVED.PROOF_OF_PAYMENT_ID
				AND APPROVED.STATUS = ".PROOF_OF_PAYMENT_APPROVED."
		LEFT JOIN TBL_PROOF_OF_PAYMENT as PENDING
			ON pop.PROOF_OF_PAYMENT_ID = PENDING.PROOF_OF_PAYMENT_ID
				AND PENDING.STATUS = ".PROOF_OF_PAYMENT_PENDING."
		LEFT JOIN TBL_PROOF_OF_PAYMENT as REJECTED
			ON pop.PROOF_OF_PAYMENT_ID = REJECTED.PROOF_OF_PAYMENT_ID
				AND REJECTED.STATUS = ".PROOF_OF_PAYMENT_REJECTED."
		";
	}

	function _setWhere() {
		$this->_addWhere('CRSO_RELATION_ID',
			array('field'=>'pop.`CRSO_RELATION_ID`',
				'callback' => array($this,'getCRSOSelect'),
				'required'=>true));
		$this->_addWhere('PARTICIPANT_RELATION_ID',
			array('field'=>'pop.`PARTICIPANT_RELATION_ID`',
				'callback' => array($this,'getParticipantSelect'),
				'related'=>array('PARTICIPANT_NAME')));
		$this->_addWhere('PRODUCT_ID',
			array(
				'field'=>'pop.PRODUCT_ID',
				'callback' => array($this,'getProductSelect')));
	}

	function _filterQuery()
	{
		$this->_addGroupBy(array('pop.`PARTICIPANT_RELATION_ID`, pop.`PRODUCER_RELATION_ID`'));
		return $this->_query;
	}

	function getProcessColumns()
	{
		return array(
			'tfValueProcessorQuantityNoDecimals'=>array('VIRTUAL_STOCK', 'POE_STOCK', 'POPS_APPROVED','POPS_PENDING','POPS_REJECTED'),
			'tfStatus'=>array('VSTOCK_STATUS')
		);
	}

	function getColumnModifier()
	{
		return array('POPS_APPROVED'=>'style="text-align:right;"',
			'POPS_PENDING'=>'style="text-align:right;"',
			'POPS_REJECTED'=>'style="text-align:right;"',
			'VIRTUAL_STOCK'=>'style="text-align:right;"',
			'POE_STOCK'=>'style="text-align:right;"');
	}

	function tfHeaderProc() {
		return array('thAlignRight'=>array('POPS_APPROVED','POPS_PENDING','POPS_REJECTED','VIRTUAL_STOCK', 'POE_STOCK'));
	}

	function tfStatus($args)
	{
		$sufficient = $args['row']['VIRTUAL_STOCK'] >= $args['row']['POE_STOCK'];
		$class = $sufficient ? 'status_verified' : 'status_denied';
		$trans = $sufficient ? $this->langIf->getText('sufficient') : $this->langIf->getText('insufficient');
		return sprintf('<span class="%s">%s</span>',$class,$trans);
	}

	function setEmptyText()
	{
		return $this->langIf->getText('No Proof of Payments for current selection');
	}

	function tfSortingHat()
	{
		return array(
			'PARTICIPANT_NAME'=>"'String'",
			'PRODUCER_NAME'=>"'String'",
			'POPS_APPROVED'=>"'SortVal'",
			'POPS_PENDING'=>"'SortVal'",
			'POPS_REJECTED'=>"'SortVal'",
			'VIRTUAL_STOCK'=>"'SortVal'",
			'POE_STOCK'=>"'SortVal'",
			'VSTOCK_STATUS'=>"'String'"
		);
	}

}
