<?php
/**
 * File description
 *
 * @package PACKAGE
 * @subpackage SUBPACKAGE
 * @since Nov 10, 2006
 * @author Martijn van den Broek <martijn@xs4all.nl>
 * @version $Id$
 * @copyright 2006 BaseSoft B.V.
 */

$noValidate = true;
$script_id =  0;
require_once(dirname(__FILE__) . '/../default/init.php');

class FixLangIf
{
	function FixLangIf($db)
	{
		$this->db = $db;

		# import the translations first (for testing purposes)
		//$this->testData();
		//$this->fixContraints();

		$this->fixTranslations();
		//$this->createUniqueIndex();
	}


	function testData()
	{
		system('cat /Users/mjbroek/translations.sql | mysql -u root -pf00b4r IQ_ELC');
	}

	private function dump($str)
	{
		if (is_array($str))
			$str = print_r($str, true);

		echo $str . "\n";
	}

	private function fixContraints()
	{
		# FIX TBL_TRANSLATIONS

		# Delete all floating in space translations so we can add the reference
		$sql = "SELECT DISTINCT FK_TRANSLATION_HANDLE_ID FROM TBL_TRANSLATIONS WHERE FK_TRANSLATION_HANDLE_ID NOT IN (
			SELECT TRANSLATION_HANDLE_ID FROM TBL_TRANSLATION_HANDLES
		)";

		$q = $this->db->query($sql);
		$q->execute();
		while ($r = $q->fetch()) {
			$sql = "DELETE FROM TBL_TRANSLATIONS WHERE FK_TRANSLATION_HANDLE_ID=?";
			$q2 = $this->db->query($sql);
			$q2->bind_values($r->{'FK_TRANSLATION_HANDLE_ID'});
			$q2->execute();
		}

		# Apply the new contraint
		$sql ="ALTER TABLE TBL_TRANSLATIONS
				ADD CONSTRAINT `TBL_TRANSLATIONS_ibfk_1`
				FOREIGN KEY (`FK_TRANSLATION_HANDLE_ID`) REFERENCES `TBL_TRANSLATION_HANDLES` (`TRANSLATION_HANDLE_ID`)
				ON DELETE CASCADE ON UPDATE CASCADE;";
		$q = $this->db->query($sql);
		$q->execute();


		# Fix TBL_SCRIPT_HANDLES

		# Delete all floating in space script_handles so we can add the reference
		$sql = "SELECT DISTINCT FK_TRANSLATION_HANDLE_ID FROM TBL_SCRIPT_HANDLES WHERE FK_TRANSLATION_HANDLE_ID NOT IN (
			SELECT TRANSLATION_HANDLE_ID FROM TBL_TRANSLATION_HANDLES
		)";

		$q = $this->db->query($sql);
		$q->execute();
		while ($r = $q->fetch()) {
			$sql = "DELETE FROM TBL_SCRIPT_HANDLES WHERE FK_TRANSLATION_HANDLE_ID=?";
			$q2 = $this->db->query($sql);
			$q2->bind_values($r->{'FK_TRANSLATION_HANDLE_ID'});
			$q2->execute();
		}

		# Apply the new contraint
		$sql ="ALTER TABLE TBL_SCRIPT_HANDLES
				ADD CONSTRAINT `TBL_SCRIPT_HANDLES_ibfk_1`
				FOREIGN KEY (`FK_TRANSLATION_HANDLE_ID`) REFERENCES `TBL_TRANSLATION_HANDLES` (`TRANSLATION_HANDLE_ID`)
				ON DELETE CASCADE ON UPDATE CASCADE;";
		$q = $this->db->query($sql);
		$q->execute();
	}

	function fixTranslations()
	{
		# to make life easier, lets delete all translations that are empty
		/*
		$sql = "DELETE FROM TBL_TRANSLATIONS WHERE TRANSLATION=''";
		$q = $this->db->query($sql);
		$q->execute();
		$this->dump('Deleted empty translations: '.$q->affected_rows() );

		# also delete all handles that dont have a translation
		$sql = "DELETE FROM TBL_TRANSLATION_HANDLES WHERE TRANSLATION_HANDLE_ID NOT IN (
				SELECT FK_TRANSLATION_HANDLE_ID FROM TBL_TRANSLATIONS
			);";
		$q = $this->db->query($sql);
		//$q->execute();
		//$this->dump('Deleted handles without translation: '.$q->affected_rows() );
		*/


		# select all handles handle is same join itself where not handle_id,
		# order by handle_id, handle
		$sql = "
			SELECT
				CONCAT(
					h.TRANSLATION_HANDLE, '_',
					IF(ISNULL(h.FK_SCRIPT_ID), '', h.FK_SCRIPT_ID) , '_',
					IF(ISNULL(h.FK_APPLICATION_ID), '', h.FK_APPLICATION_ID), '_',
					IF(ISNULL(h.SCRIPT_NAME), '', h.SCRIPT_NAME))
					as k,

				TRANSLATION_HANDLE_ID as v
			FROM TBL_TRANSLATION_HANDLES h

		";
		$q = $this->db->query($sql);
		$q->execute();

		# this will be the array that contains all the handles with ids
		$handles = array();

		while ($r = $q->fetch())
		{
			if (!isset($handles[$r->{'k'}]))
				$handles[$r->{'k'}] = array();

			$handles[ $r->{'k'}][] = $r->{'v'};
		}

		# now we have an array with all the handles as indexes and array with id's as value
		# make sure the ids are unique
		foreach ($handles as $key=>$handle_ids)
		{
			$handles[$key] = array_unique($handle_ids);

			# we are only interested in duplicate handles
			if (sizeof($handles[$key]) <= 1)
			{
				unset($handles[$key]);
				continue;
			}
		}

		$this->dump('Handles that contain duplicates: '.sizeof($handles));
		//$this->dump('');
		//$this->dump($handles);

		foreach ($handles as $key=>$handle_ids)
		{
			# if we come here we have multiple handles in the array, assign the first to be the only unique:
			$original_handle_id = $handle_ids[0];

			# now lets process the duplicates
			foreach ($handle_ids as $handle_id)
			{
				# we want the unique to stay as it is
				if ($handle_id == $original_handle_id)
					continue;

				$this->dump('Processing duplicate '.$handle_id);

				# see if there is a translation, else trash it
				$sql = "SELECT FK_LANGUAGE_ID, TRANSLATION FROM TBL_TRANSLATIONS WHERE FK_TRANSLATION_HANDLE_ID=?";
				$q = $this->db->query($sql);
				$q->bind_values($handle_id);
				$q->execute();

				if (!($r = $q->fetch()))
				{
					$this->dump(' - no translation found for handle '.$handle_id.' .. deleting it');
					$sql = "DELETE FROM TBL_TRANSLATION_HANDLES WHERE TRANSLATION_HANDLE_ID=?";
					$q2 = $this->db->query($sql);
					$q2->bind_values($handle_id);
					$q2->execute();
				}
				else
				{
					$this->dump(' - translation found for '.$handle_id);

					# lets get the original translation so we can compare
					$sql = "SELECT TRANSLATION, FK_LANGUAGE_ID FROM TBL_TRANSLATIONS WHERE FK_TRANSLATION_HANDLE_ID=?";
					$q2 = $this->db->query($sql);
					$q2->bind_values($original_handle_id);
					$q2->execute();

					$tr_original = array();

					while ($r2 = $q2->fetch())
					{
						$language_id = $r2->{'FK_LANGUAGE_ID'};
						$translation = $r2->{'TRANSLATION'};

						$tr_original[$language_id] = $translation;
					}

					# store the duplicate translations
					do
					{
						$language_id = $r->{'FK_LANGUAGE_ID'};
						$translation = $r->{'TRANSLATION'};

						# see if there is a translation for the language
						if (isset($tr_original[$language_id]))
						{
							if ($tr_original[$language_id] != $translation)
							{
								//$this->dump(' - '.$tr_original[$language_id] .' != '. $translation . '('.$language_id.')' );
								$this->dump(' - Updating original handle with duplicate');

								# Update the original with the duplicate
								$sql = "UPDATE TBL_TRANSLATIONS SET TRANSLATION=? WHERE FK_TRANSLATION_HANDLE_ID=? AND FK_LANGUAGE_ID=?";
								$q2 = $this->db->query($sql);
								$q2->bind_values($translation, $handle_id, $language_id);
								$q2->execute();
							}

							# Delete this duplicate handle
							$this->dump(' - Deleting duplicate');
							$sql = "DELETE FROM TBL_TRANSLATIONS WHERE FK_TRANSLATION_HANDLE_ID=? AND FK_LANGUAGE_ID=?";
							$q2 = $this->db->query($sql);
							$q2->bind_values( $handle_id, $language_id );
							$q2->execute();
						}
						else
						{
							# original handle does not have the language, move it!
							$this->dump(' - Moving translation to original handle');
							$sql = "UPDATE TBL_TRANSLATIONS SET FK_TRANSLATION_HANDLE_ID=? WHERE FK_TRANSLATION_HANDLE_ID=? AND FK_LANGUAGE_ID=?";
							$q2 = $this->db->query($sql);
							$q2->bind_values($original_handle_id, $handle_id, $language_id);
							$q2->execute();
						}
					}while ($r = $q->fetch());
				}
			}
		}
	}

	function createUniqueIndex()
	{
		$sql = "CREATE UNIQUE INDEX unique_key
			ON TBL_TRANSLATION_HANDLES (FK_SCRIPT_ID, FK_APPLICATION_ID, SCRIPT_NAME)";
		$q = $this->db->query($sql);
		$q->execute();
	}
}

echo '<pre>';

new FixLangIf($db_langif);







?>