<?php
/**
 * Default I-Ware includes and requires
 *
 * @package I-Ware
 * @subpackage scripts_default
 * @version $Id$
 */
/**
 * Returns microtime
 * @return float
 */
function m()
{
   list($usec, $sec) = explode(" ", microtime());
   return ((float)$usec + (float)$sec);
}
$start_execution_time = m();

$thrownException = "";

if (! isset($site_homedir))
	$site_homedir = "../../../";

require_once ($site_homedir . "classes/i-w/IWDatabase.php");
switch($_SERVER['SERVER_NAME'])
{
	case 'cmd line':
		$require_dir = realpath(dirname(__FILE__).'/../../../').'/';
		$site_identifier = 'phpunit';
		break;

	default:
		$require_dir = $_SERVER['DOCUMENT_ROOT'];
		$site_identifier = $_SERVER['SERVER_NAME'];
}


if (substr($require_dir, -1) != '/')
	$require_dir .= '/';

// check if the browser is connecting to a 172.. address, then we detect running in docker
// in that case use the localhost.php settings file
// TODO: rename localhost -> docker
if (substr($site_identifier, 0, 4) == '172.')
	$site_identifier = 'localhost';

//Load settings
$file = $require_dir."includes/i-w/site-settings/$site_identifier.php";
if(file_exists($file))
	require_once($file);
else {
	echo sprintf('Cannot init application: require_dir: %s, site_identifier: %s.. cannot find file: %s', $require_dir, $site_identifier, $file);
	exit(1);
}

$dba_conn_string = 'mysql:host='.$sql_server.';dbname='.$dba_name;

if (defined('LANGIF_DB'))
	$dba_conn_string_langif = 'mysql:host='.(defined('IW_MYSQL_SERVER')?IW_MYSQL_SERVER:$sql_server).';dbname='.LANGIF_DB;

require_once ($site_homedir . "includes/i-w/defines.php");
require_once (IWARE_INCLUDES.'functions.php');
require_once (IWARE.'form.php');
require_once (IWARE.'sql.php');
require_once (IWARE.'date_conversion.php');
require_once (IWARE.'eventHandler.php');
require_once (IWARE.'IWSessionHandler.php');
require_once (IWARE.'IWRegistry.php');
require_once(IWARE.'IWDateTime.php');
if(!defined('CHARSET'))
	define('CHARSET','utf-8');
mb_internal_encoding("UTF-8");

/**
 * DATABASE
 */
try
{
	$db = new IWDatabase ($dba_conn_string, $dba_user, $dba_password, 0);

	$db_edit = $db;
	if( isset( $dba_conn_string_langif))
		$db_langif = new IWDatabase ($dba_conn_string_langif, 'langif', 'bRI5stia', 0);
	else
		$db_langif =& $db;
}
catch (Exception $e)
{
	header('HTTP/1.0 500 Internal Server Error', true, 500);
	syslog(LOG_ALERT, 'Unable to connect to database: '.$e->getMessage());
	echo 'Internal error: Unable to connect to database.';
	exit(1);
}

// dont let command line authenticate.. and neither clients on a local network
if(PHP_SAPI != 'cli' && (isset($_SERVER['HTTP_HOST']) && !preg_match('/^(192\.168\.)|(127\.)|(10\.)/', $_SERVER['REMOTE_ADDR']) ))
{
	$exclude_auth = array('SoapServerAccountant.php', 'WEEEBlackboxAccountant.wsdl.php');
	$request_uri = $_SERVER['REQUEST_URI'];

	$skip_auth = false;
	foreach($exclude_auth as $exclude_for_script) {
		if(strstr($request_uri, $exclude_for_script)!==false) {
			$skip_auth = true;
			break;
		}
	}
	if(!$skip_auth) {
		require_once (IWARE.'HTTPAuthenticate.php');
    	new HTTPAuthenticate($db,(isset($HTTP_AUTHENTICATE_GROUPS)?$HTTP_AUTHENTICATE_GROUPS:false));
	}
}

if(!defined('I_AM_PHPUNIT'))
{


	/**
	 * SESSION HANDLER
	 */
	if(defined('SESSION_HANDLER'))
		$SessionHandler = IWSessionHandler::createSessionHandler($db); //this should be a reference.. But php chokes.
	else
		session_start();
	/**
	 * Browser check
	 */

	if(!defined('NO_BROWSER_CHECK') && $_SESSION['browser_supported']==CHECK_FOR_BROWSER)
	{
		require_once(IWARE.'phpsniff/phpSniff.class.php');
		$sniff = new phpSniff($_SERVER['HTTP_USER_AGENT']);

		$browsers_allowed = array('mozilla'=>1.0, 'msie'=>6.0,'opera'=>9.0, 'safari'=>536, 'netscape'=>5.0); /* ie11 seems to be detected as netscape 5 */
		$javascript = 1.0;
		$gecko_ver = 1.8;

		//echo 'long name: '.$sniff->property('long_name') .'<br>';
		//echo 'version: '.$sniff->property('version') .'<br>';

		$allow = BROWSER_CHECK_DENIED;
		foreach($browsers_allowed as $long_name=>$version)
		{
			if ($sniff->property('long_name') == $long_name &&
				(int) $sniff->property('version') >= $version)
			{
					if (!$sniff->property('gecko_ver') || $sniff->property('gecko_ver') >= $gecko_ver)
						$allow = BROWSER_CHECK_APPROVED;


					if(($sniff->property('long_name') == 'msie') && ($sniff->property('version')<7.0))
						$_SESSION['ie_warning'] = true;
			}
		}
		$_SESSION['browser_supported'] = $allow;

		// check javascript version
		if ($sniff->property('javascript') < 1.0)
			header_message('javascript_unsupported');
	}


	// check browser allowed
	if(!defined('NO_BROWSER_CHECK') && $_SESSION['browser_supported']==BROWSER_CHECK_DENIED) {
		//exit('browser not supported');
		header_message('browser_unsupported');
	}



	/**
	 * ERROR HANDLER
	 */
	$errorHandler = new ErrorHandler($db, !defined('ERROR_NO_FRIENDLY') );
	$errorHandler->registerObserver('E_FATAL', array($db, 'rollbackOnError')); // FIXME
	defined('ERROR_LEVEL') && $errorHandler->setReporting( ERROR_LEVEL );

	/**
	 * EXCEPTION HANDLER
	 */
	$exceptionHandler = new ExceptionHandler($db, !defined('ERROR_NO_FRIENDLY'));

	/**
	 * LANGUAGE INTERFACE
	 */
	if (isset($langIf))
		trigger_error('LangIf already exists', E_USER_ERROR);

	require_once(IWARE."LanguageInterface.php");
	$langIf = new LanguageInterface($db, $db_langif);

	IWRegistry::register('langIf',$langIf);
	IWRegistry::register('db',$db);
	IWRegistry::register('errorHandler', $errorHandler);
	IWRegistry::register('exceptionHandler', $exceptionHandler);

	/**
	 * Mail object
	 */
	require_once(IWARE . 'phpmailer/class.phpmailer.php');
	require_once(IWARE . 'phpmailer/class.smtp.php');

	$mail = new PHPMailer();
	$mail->IsSMTP();
	$mail->SetLanguage("en", IWARE.'phpmailer/');

	if (!defined('SMTP_SERVERS'))
		trigger_error('Cannot send email: no SMTP_SERVERS defined', E_USER_ERROR);
	$mail->Host = SMTP_SERVERS; // specify main and backup server (maybe add backup server)
	$mail->SMTPKeepAlive = true;
	IWRegistry::register('mail',$mail);

	/**
	 * Suckerfish menu
	 */
	require_once(IWARE.'SuckerfishMenu.php');

	/**
	 * Script stuff
	 */
	// Default script_parent id
	$script_parent_id = isset($script_parent_id) ? (int)filter_var($script_parent_id, FILTER_SANITIZE_STRING) :
		(isset($_REQUEST['script_parent_id']) ? (int)filter_var($_REQUEST['script_parent_id'], FILTER_SANITIZE_STRING) : 0);
	$script_parent_id || $script_parent_id = 0;

	// check if a better script_parent_id is available, else try to get one ourself
	// TODO, maybe move this to Script object (if it works)
	$script_name = '';
	if (isset($script_id))
	{
		$scriptDb = $db;
		$sql = "SELECT SCRIPT_PARENT_ID, SCRIPT_NAME FROM TBL_SCRIPT WHERE SCRIPT_ID=?";
		$q = $scriptDb->query($sql);
		$q->bind_values($script_id);
		$q->execute();
		if ($row = $q->fetch())
		{

			do
			{
				$script_name = $script_name?$script_name:$row->SCRIPT_NAME;
				if(!($script_parent_id == 0 && isset($_SESSION["session_group_company_id"])))
					break;
				$script_parent_id = $row->{'SCRIPT_PARENT_ID'};
				$sql = "SELECT SCRIPT_ID FROM TBL_SCRIPT_GROUP WHERE SCRIPT_ID=? AND GROUP_COMPANY_ID=? AND SCRIPT_GROUP_ACCESS > 0";
				$q2 = $db->query($sql);
				$q2->bind_values($script_parent_id, $_SESSION["session_group_company_id"]);
				$q2->execute();
				if ($r2 = $q2->fetch()) { //
					$script_parent_id = $row->{'SCRIPT_PARENT_ID'};
					break;
				}
			}while ($row = $q->fetch());
		}
	}

	if (!(isset($noValidate) && $noValidate))
	{
		$Remoras = SuckerfishMenu::create($db, $db, $langIf);
		if (!$Remoras->hasAccess($script_parent_id, isset($script_id) ? $script_id : 0 ))
		{
				include(IWARE_SCRIPTS.'PermissionDenied.php');
				exit();
		}
	}

	/**
	 * XHTML
	 */
	if(!defined('XHTML_HEADER'))
		define('XHTML_HEADER',IWARE_SCRIPTS_DEFAULT.'xhtml_header.php');
	if(!defined('XHTML_FOOTER'))
		define('XHTML_FOOTER',IWARE_SCRIPTS_DEFAULT.'xhtml_footer.php');

	// default styles
	$show_style = array('professional', 'taalinterface', 'xtree', 'newform', 'message_box','main' );
}

/**
 * SANITIZE_OFF is now off for all interfaces. This is because the Interface
 * constructor sanitizes itself. So this is for all scripts that do not use the
 * InterfaceFactory
 */
if(!defined('SANITIZE_OFF'))
{
	//sanitize GET
	$_GET = sanitize_array_recursive($_GET);

	//sanitize POST
	$_POST = sanitize_array_recursive($_POST);

	//sanitize REQUEST
	$_REQUEST = sanitize_array_recursive($_REQUEST);
}

//if we are not cli then check server vars for tampering or XSS/code injection
if(php_sapi_name() != 'cli')
{
	/**
	 * Here we filter $_SERVER data and throw an exception if there is anything that
	 * shouldnt be there. This also filteres cookies!
	 */
	$server_vars = array('HTTP_HOST', 'HTTP_USER_AGENT', 'HTTP_ACCEPT', 'HTTP_ACCEPT_LANGUAGE',
		'HTTP_ACCEPT_ENCODING', 'HTTP_ACCEPT_CHARSET', 'HTTP_KEEP_ALIVE', 'HTTP_CONNECTION', 'HTTP_COOKIE');

	foreach ($server_vars as $server_var)
	{
		if(!isset($_SERVER[$server_var]))
			continue;
		$victem = $_SERVER[$server_var];
		if (filter_var($victem, FILTER_SANITIZE_STRING) != $victem)
		{
			// send a forbidden code
			header('HTTP/1.1 403 Forbidden');
			// make a ticket with which var is modified
			alert(sprintf('SERVER data filtered: $_SERVER[%s] = %s', $server_var, $victem));
			// present the user with a message
			throw new IWException('Security Exception: HTTP Headers were filtered. The system administrator has been notified');
		}
	}
	unset($server_vars);
}

// Remote viewer for participant
if(isset($_SESSION) && $_SESSION['session_group_group_id']==GROUP_PARTICIPANT) {
	require_once(ELC_CLASSES.'RemoteViewer.php');
	$rv = new RemoteViewer(IWRegistry::registry('db'));
	if($r = $rv->hasPendingOrActiveConnection($_SESSION['selected_participant_id'], $_SESSION['session_user_id'])) {
		// No more participant switching possible, until the RV gets cancelled/timeouts
		defined('NO_VERIFY_MULTIPLE_PARTICIPANTS') || define('NO_VERIFY_MULTIPLE_PARTICIPANTS', true);
		$rv->set($r->REMOTE_VIEWER_ID);
		IWRegistry::register('remote_viewer', $rv);
	}
}
?>
