<?php
/**
 * @package I-Ware
 * @subpackage scripts
 */
$script_id = 0;
$noValidate = true;
require_once('../default/init.php');


/**
 * XML PARSER
 */

class LangIfXmlParser
{
	var $debug = 1;

	var $filename;
	var $xml;

	var $handleObj = null;
	var $_in_script_id = false;
	var $_in_application_id = false;
	var $_in_script_name = false;
	var $_in_translation = false;

	function LangIfXmlParser($xml_file)
	{
		$this->filename = $xml_file;

		$this->xml = xml_parser_create();
		xml_set_object($this->xml, $this);
		xml_set_element_handler($this->xml, 'startHandler', 'endHandler');
		xml_set_character_data_handler($this->xml, 'dataHandler');

		$this->parse($xml_file);
	}

	function parse($xml_file)
	{
		if (!($fp = fopen($xml_file, 'r')))
			trigger_error('Cannot open XML data file: '.$xml_file, E_USER_ERROR);

		$bytes_to_parse = 512;

		while ($data = fread($fp, $bytes_to_parse))
		{
			$parse = xml_parse($this->xml, $data, feof($fp));

			if (!$parse)
			{
				trigger_error(sprintf("XML error: %s at line %d",
					xml_error_string(xml_get_error_code($this->xml)),
					xml_get_current_line_number($this->xml)), E_USER_ERROR);

				xml_parser_free($this->xml);
			}
		}

		return true;
	}

	function startHandler($parser, $name, $attributes)
	{
		if ($name == 'HANDLE')
			$this->handleObj = new LangIfHandle($attributes['NAME']);

		$this->_in_script_id = ($name == 'SCRIPT_ID');
		$this->_in_application_id = ($name == 'APPLICATION_ID');
		$this->_in_script_name = ($name == 'SCRIPT_NAME');

		if ($name == 'TRANSLATION')
		{
			$this->_in_translation = true;
			$this->_tmp_language_id = $attributes['LANGUAGE_ID'];
		}
	}

	function endHandler($parser, $name)
	{
		if ($name == 'HANDLE')
		{
			$this->handleObj->process( (isset($_POST['chkDryRun'])) );
			unset($this->handleObj);
		}

		if ($name == 'TRANSLATION')
			$this->_in_translation = false;
	}

	function dataHandler($parser, $data)
	{
		foreach (array('script_id', 'application_id', 'script_name') as $key=>$value)
		{
			if ($this->{'_in_'.$value})
				$this->handleObj->$value .= $data;
		}

		if ($this->_in_translation)
			$this->handleObj->addTranslation($this->_tmp_language_id, $data);
	}
}

class LangIfHandle
{
	var $handle = '';
	var $script_id = null;
	var $application_id = null;
	var $script_name = null;
	var $translations = array();
	var $verbose = 1;
	var $dryrun = 1;
	var $overwrite = 0;

	function LangIfHandle($handle)
	{
		global $db_iware;
		$this->db = $db_iware;
		$this->handle = $handle;

		$this->dryrun = (isset($_REQUEST['chkDryRun']));
		$this->overwrite = (isset($_REQUEST['chkOverwrite']));
		$this->verbose = (isset($_REQUEST['chkVerbose']));
	}

	function addTranslation($language_id, $translation)
	{
		if (isset($this->translations[$language_id]))
			$this->translations[$language_id] .= $translation;
		else
			$this->translations[$language_id] = $translation;
	}

	function process()
	{
		/*
		echo '<pre>';
		print_r($this);
		echo "\n";
		*/

		ob_start();

		$handle_id = $this->processHandle();

		if ($handle_id)
			$this->processTranslations($handle_id);

		$html = ob_get_clean();

		if ($this->verbose)
			echo $html;
		elseif ( (strpos($html, '<span class="warning"') !== false ||
			strpos($html, '<span class="info"') !== false ))
		{
			echo $html;
		}

	}

	/**
	 * @return int handle_id
	 */
	function processHandle()
	{
		echo '<strong>Processing handle: '. $this->handle.'</strong> <br>';

		$sql = "SELECT * FROM TBL_TRANSLATION_HANDLES WHERE TRANSLATION_HANDLE=? ";

		foreach ( array('script_id'=>'FK_SCRIPT_ID',
			'script_name'=>'SCRIPT_NAME',
			'application_id'=>'FK_APPLICATION_ID') as $key=>$value )
		{
			echo $key.': '.$this->$key.'<br />';

			if ((trim($this->$key) == ''))
				$sql .= 'AND ' . $value . ' IS NULL ';
			else
				$sql .= "AND ".$value."='".trim($this->$key)."' ";
		}

		$q = $this->db->query($sql);
		$q->bind_values($this->handle);

		$q->execute();

		if (!($row = $q->fetch()))
		{
			echo '<span class="info">This handle does not exist in database.</span> <br />';

			//-- insert handle and all translations
			$sql = new SQL_Insert('TBL_TRANSLATION_HANDLES');
			foreach ( array('script_id'=>'FK_SCRIPT_ID',
				'script_name'=>'SCRIPT_NAME',
				'application_id'=>'FK_APPLICATION_ID') as $key=>$value )
			{
				// somehow newlines got added from the export
				$sql->add($value, (trim($this->$key) == '') ? null : str_replace("\n", "", $this->$key) );
			}

			$sql->add('TRANSLATION_HANDLE', $this->handle);

			//echo $sql->getHtml();

			if ($this->dryrun)
				echo '<span class="warning">Dry run enabled, not inserting.</span> <br />';
			else
			{
				$q = $this->db->query($sql->getText());
				$q->execute();
				$handle_id = $q->lastId();

				echo '<span class="info">Creating new Handle.. ID: '.$handle_id.' .</span> <br />';
				return $handle_id;
			}
		}
		else
		{
			$handle_id = $row->{'TRANSLATION_HANDLE_ID'};

			echo 'This handle already exists in database.. handle_id: '.$handle_id.' <br>';

			return $handle_id;
		}

		echo '<br />';
	}


	function processTranslations($handle_id)
	{
		//-- check translations
		foreach ($this->translations as $language_id=>$translation)
		{
			$translation = trim($translation);

			echo "Checking for translation: '" . $translation . "'<br>";

			$sql = "SELECT *, TRIM(TRANSLATION) AS TRANSLATION FROM TBL_TRANSLATIONS WHERE FK_TRANSLATION_HANDLE_ID=? AND FK_LANGUAGE_ID=?";
			$q2 = $this->db->query($sql);
			$q2->bind_values($handle_id, $language_id);
			$q2->execute();

			if (!($row2 = $q2->fetch()))
			{
				echo '<span class="info">New translation, gonna insert it!</span> <br>';

				$sql = new SQL_Insert('TBL_TRANSLATIONS');
				$sql->add('FK_TRANSLATION_HANDLE_ID', $handle_id);
				$sql->add('FK_LANGUAGE_ID', $language_id);
				$sql->add('TRANSLATION', $translation);

				if ($this->dryrun)
					echo '<span class="warning">Dry run enabled, not executing query.</span> <br />';
				else
				{
					$q = $this->db->query($sql->getText());
					$q->execute();
					echo '<span class="info">Inserted new translation</span> <br />';
				}
			}
			else
			{

				$tr = $row2->{'TRANSLATION'};

				$reg = "/\\r(?:\\n)?/";
				if ( preg_replace($reg, "\n", trim($tr))  == preg_replace($reg, "\n", trim($translation)) )
					echo 'Translations are identical, no change needed. <br/>';
				else
				{
					echo "<span class=\"info\">Found existing different translation: '".$tr."'.</span> <br />";

					$sql = new SQL_Update('TBL_TRANSLATIONS');
					$sql->add('TRANSLATION', $translation);
					$sql->addWhere('FK_TRANSLATION_HANDLE_ID', $handle_id);
					$sql->addWhere('FK_LANGUAGE_ID', $language_id);

					if ($this->dryrun)
						echo '<span class="warning">Dry run enabled, not gonna continue!</span> <br />';
					else if (!$this->overwrite)
						echo '<span class="warning">Overwrite is off, not gonna continue!</span> <br />';
					else
					{
						$q = $this->db->query($sql->getText());
						$q->execute();
						echo '<span class="info">Updated translation</span> <br />';
					}
				}
			}
		}

		echo '<br />';
	}

}

/**
 * HTML START
 */
require_once('../default/xhtml_header.php');

$tdata = array();
$file = new Form_File('userfile');
$tdata[0][0] = 'XML File';
$tdata[0][1] = $file->getHtml();

$chk = new Form_Checkbox('chkVerbose', 'on', false);
$chk->setLabel('Be verbose');
$tdata[1][0] = '';
$tdata[1][1] = $chk->getHtml();

$chk = new Form_Checkbox('chkDryRun', 'on', true );
$chk->setLabel('Dry Run');
$tdata[2][0] = '';
$tdata[2][1] = $chk->getHtml();

$chk = new Form_Checkbox('chkOverwrite', 'on', false);
$chk->setLabel('Overwrite translations');
$tdata[3][0] = '';
$tdata[3][1] = $chk->getHtml();

$btn = new Form_Button('', 'Go', 'submit');
$btn->setWidth(80);
$tdata[4][0] = '';
$tdata[4][1] = $btn->getHtml();

?>
	<br />
	<form enctype="multipart/form-data" action="<?=$_SERVER['PHP_SELF']?>" method="POST">
	<?php
			$t = new TableFactory($tdata);
			$t->deCapitate();
			$t->setCaption('LangIf Import');
			echo $t->getXhtmlTable('Professional');
		?>
	</form>
<?php
	?>
	<table class="Professional">
		<caption>Import console</caption>
		<tr>
			<td>
				<?php
					if (isset($_FILES['userfile']))
					{
						if (is_uploaded_file($_FILES['userfile']['tmp_name']))
						{
							$file = $_FILES['userfile']['tmp_name'];
							$x = new LangIfXmlParser($file);

							echo '<br /><br /><span class="info">Done!</span><br />';
						}
						else
							die("Possible file upload attack!");
					}
				?>
			</td>
		</tr>
	</div>
<?php
	require_once('../default/xhtml_footer.php');
?>
