<?php
/**
 * Created on Jul 28, 2010
 *
 * @author Nicoleta Militaru <nicoleta@basesoft.com>
 **/

require_once(IWARE.'Task.php');
require_once(IWARE.'LanguageInterface.php');
require_once(ELC_CLASSES.'AnnualConfirmation.php');
require_once(ELC_CLASSES.'Notification.php');

class TaskCloseAnnualConfirmation extends Task
{
	/**
	 * @var Database $db
	 */
	var $db;
	/**
	 *
	 */
	var $settings = array();
	/**
	 * @param Database $db
	 */
	function TaskCloseAnnualConfirmation(&$db)
	{
		$this->db =& $db;
	}

	function execute(&$log)
	{
		$this->_load();
		$start = m();

		$date = currentDate();
		$year = $date->getYear();
		$month = $date->getMonth();
		$day = $date->getDay();

		$langIf = IWRegistry::registry('langIf');
		// get all ACs that should be closed
		$sql = "SELECT AC.ANNUAL_CONFIRMATION_ID, AC.CRSO_RELATION_ID, AC.PRODUCT_ID, AC.PARTICIPANT_RELATION_ID,
				R1.NAME AS PARTICIPANT_NAME, R2.NAME AS CRSO_NAME
				FROM TBL_ANNUAL_CONFIRMATION AC
				INNER JOIN TBL_CRSO_PRODUCT CP
					ON CP.CRSO_RELATION_ID = AC.CRSO_RELATION_ID
					AND CP.PRODUCT_ID = AC.PRODUCT_ID
				INNER JOIN TBL_RELATION R1
					ON R1.RELATION_ID = AC.PARTICIPANT_RELATION_ID
				INNER JOIN TBL_RELATION R2
					ON R2.RELATION_ID = AC.CRSO_RELATION_ID
				WHERE DATE(CONCAT('".$year."', '-', CP.AC_CUT_OFF_MONTH, '-',
					IF(CP.REFUND_EXPORT_DAY IS NULL OR CP.FORFEIT_DAY>CP.REFUND_EXPORT_DAY, CP.FORFEIT_DAY, CP.REFUND_EXPORT_DAY)))<=
					DATE(CONCAT('".$year."','-','".$month."','-','".$day."'))
				AND AC.STATUS = 1 AND CHANGE_REQUEST_STATUS=0";
		$q = $this->db->query($sql);
		$q->execute();

		$ac = new AnnualConfirmation($this->db);

		$crsos = array();
		while($r=$q->fetch()){
			$ac->set($r->ANNUAL_CONFIRMATION_ID);
			$ac->status = ANNUAL_CONFIRMATION_CLOSED;
			$ac->update();
			$log->info('[TaskRunner::TaskCloseAnnualConfirmation] closed AnnualConfirmation '.$r->ANNUAL_CONFIRMATION_ID);

			// send notification to participant
			$p_body = $langIf->getText("Your Annual confirmation has been closed at CRSO:")." ".$r->CRSO_NAME;
			$p_message = array('subject'=> $langIf->getText('Annual confirmation closed at CRSO')." ".$r->CRSO_NAME,
							   'message'=> $p_body,
							);
			Notification::send(
					$this->db,
					$p_message,
					array(array('recipient'=> new Relation($this->db, $r->PARTICIPANT_RELATION_ID))));

			if(!isset($crsos[$r->CRSO_RELATION_ID]))
				$crsos[$r->CRSO_RELATION_ID] = array();
			$crsos[$r->CRSO_RELATION_ID][] = array('id'=>$r->PARTICIPANT_RELATION_ID, 'name'=>$r->PARTICIPANT_NAME);
		}

		// send notification to crsos
		if(count($crsos) > 0){
			foreach($crsos as $key=>$value){
				$body = $langIf->getText("Annual confirmation for the following participants has been closed:")."<br /><br />";
				foreach($value as $k=>$v){
					$body .= $v['name']." (".$langIf->getText('id')." ".$v['id'].") <br />";
				}
				$message = array('subject'=> $langIf->getText('Annual confirmations closed'),
							'message'=> $body,
							);
				Notification::send(
						$this->db,
						$message,
						array(array('recipient'=> new Relation($this->db, $key))));
				}
		}

		$log->info('[TaskRunner::TaskCloseAnnualConfirmation] Done in '.round((m()-$start),3).' seconds');
	}
	/**
	 * @access private
	 */
	function _load()
	{

	}
}

?>
