<?php
require_once(IWARE.'Task.php');
require_once(ELC_CLASSES.'ELCMessageHandler.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'CRSOProduct.php');
require_once(ELC_CLASSES.'RelationUser.php');
/**
 * @package ELC
 * @subpackage Task
 * @author Nick Loeve <nick@basesoft.com>
 * @since 31/08/2006
 * @copyright 2006 BaseSoft B.V.
 * $Id$
 */
class TaskDeclarationReminder extends Task
{
	/**
	 * @var Database $db
	 */
	var $db;
	/**
	 *
	 */
	var $settings = array();


	/**
	 * @param Database $db
	 */
	function TaskDeclarationReminder(&$db)
	{
		$this->db =& $db;
		//$this->setSettings();
	}
	/**
	 *
	 */
	function setSettings()
	{
		$this->settings['db']['user'] = $this->db->user;
		$this->settings['db']['pass'] = $this->db->pass;
		$this->settings['db']['host'] = $this->db->host;
		$this->settings['db']['name'] = $this->db->name;
		$this->settings['db']['driver'] = $this->db->driver;
	}

	function execute(&$log)
	{
		$this->_load();
		$start = m();

		$date = currentDate();
		$crsoObj = new CRSO($this->db);
		$cp = new CRSOProduct($this->db);
		$crsos = $crsoObj->selectCRSOs();
		while($crso = $crsos->fetch())
		{
			$crso_id = $crso->{'CRSO_RELATION_ID'};
			$products = $cp->getProducts($crso_id);
			while($p = $products->fetch())
			{
				$product_id = $p->PRODUCT_ID;
				$cp->set($crso_id,$product_id);
				$reminder_day = $cp->getDeclarationReminderDay();
				if($reminder_day === false)
				{
					$log->info(sprintf('No DECLARATION_REMINDER_DAY set for CRSO [%s] and Product [%s] - not exporting', $crso_id, $product_id));
					continue;
				}
				if($date->getDay() != $reminder_day)
				{
					$log->info(sprintf("The declaration reminder day [%s] for CRSO [%s] and Product [%s] is not the current day.", $reminder_day, $crso_id,$product_id));
					continue;
				}

				$participants = $cp->getPartipciantsWithOpenDeclarations();

				$log->info(sprintf("Sending reminders to [%s] participants for crso [%s]", count($participants), $crso_id));
				foreach($participants as $part_id)
				{
					$ru = new RelationUser($this->db);
					// Silently fail in the edge case of a Participant with multiple Admin users (!)
					$adminUser = $ru->getRelationAdmin($part_id, true);
					if(!$adminUser)
					{
						$log->info(sprintf("No reminder sent to participant [%s] - no admin or double admin user", $part_id));
						continue;
					}
					$adminUser->set();
					if(!$adminUser->active) {
						$log->info(sprintf("No reminder sent to participant [%s] - admin user is not active", $part_id));
						continue;
					}
					MessageEvent::send(
								MESSAGE_DECLARATION_REMINDER,
								array(
									'user'=>$adminUser
								),
								'ELCMessageHandler'
					);

				}
			}
		}

		$log->info('TaskDeclarationReminder Done in '.round((m()-$start),3).' seconds');
	}


	/**
	 * @access private
	 */
	function _load()
	{
	//	$connstr = $this->settings['db']['driver'].':'.$this->settings['db']['host'].':'.$this->settings['db']['name'];
		//$this->db = new Database($connstr, $this->settings['db']['user'], $this->settings['db']['pass'], 1);
	}
}
?>