<?php
require_once(IWARE.'Task.php');
//require_once(IWARE.'IWCSVWriter.php');
require_once(IWARE.'IWXMLWriter.php');
require_once(ELC_CLASSES.'RefundExport.php');
require_once(ELC_CLASSES.'RefundRequestConfirmation.php');
require_once(ELC_CLASSES.'ProofOfPaymentConfirmation.php');
require_once(ELC_CLASSES.'CRSO.php');
require_once(ELC_CLASSES.'Participant.php');
require_once(ELC_CLASSES.'CRSOParticipantProduct.php');
require_once(ELC_CLASSES.'ELCExporter_RefundRequests.php');
/**
 * @package ELC
 * @subpackage Task
 * @copyright 2006 BaseSoft B.V.
 * @since 2006-07-27
 * @author Nick Loeve <nick@basesoft.com>
 * @version $Id$
 */
class TaskExportRefundRequest extends Task
{
	/**
	 * @var Database $db
	 */
	var $db;
	/**
	 *
	 */
	var $settings = array();
	/**
	 * @param Database $db
	 */
	function TaskExportRefundRequest(&$db)
	{
		$this->db =& $db;
	}

	function setSettings()
	{
		$this->settings['db']['user'] = $this->db->user;
		$this->settings['db']['pass'] = $this->db->pass;
		$this->settings['db']['host'] = $this->db->host;
		$this->settings['db']['name'] = $this->db->name;
		$this->settings['db']['driver'] = $this->db->driver;
	}

	/**
	 *
	 */
	function execute(&$log)
	{
		$this->_load();
		$start = m();


		### PHASE ONE ###

		require_once(ELC_CLASSES.'Country.php');
		$date =& currentDate();
		$currentDay = $date->getDay();
		$currentMonth = $date->getMonth();

		// LOOP ALL CRSO's
		$sql = "SELECT CRSO_RELATION_ID, PRODUCT_ID
			FROM TBL_CRSO_PRODUCT
			WHERE REFUND_EXPORT_DAY = ?";
		$q = $this->db->query($sql);
		$q->bind_values($currentDay);
		$q->execute();
		$rs = $q->fetchAll();
		if(!count($rs))
			$log->info('ExportRefundRequests - No Exports Scheduled for today');
		$crsos = array();
		foreach($rs as $r)
			$crsos[$r->CRSO_RELATION_ID][$r->PRODUCT_ID] = $r->PRODUCT_ID;

		$r = new Relation($this->db);
		foreach($crsos as $crso_id => $products)
		{
			$cp = new CRSOParticipantProduct($this->db);
			$r = new Relation($this->db,$crso_id);

			$groupedParticipants = $groupedProducts = array();
			$country = new Country($this->db,$r->getCountry($crso_id),true);
			foreach($products as $product_id)
			{
				//second we check if they have a fee set
				$q = $this->db->query("SELECT f_elcProductFee(?, ?, NULL, NOW()) AS FEE");
				$q->bind_values($product_id,$crso_id);
				if(!$q->execute())
					trigger_error("Error executing query", E_USER_ERROR);
				$r = $q->fetch();
				if(!is_numeric($r->FEE))
				{
					$log->info(sprintf("There is no FEE set for CRSO [%s] and Product [%s], we are not running refund export.", $crso_id,$product_id));
					continue;
				}

				if($currentMonth==1)
				{
					require_once(ELC_CLASSES.'AnnualConfirmation.php');
					$ac = new AnnualConfirmation($this->db);
					$ac->triggerNewRefundAnnualConfirmations($crso_id,$log,$product_id);
				}

				if(!$country->has(HasCrsoOneExportDay,$product_id))
				{
					$participants = $cp->getCRSOParticipants($crso_id,'array',false,$product_id);
					foreach($participants as $part_id)
						RefundExport::phaseOne($this->db, $part_id, $crso_id,array($product_id));
				}
				else
				{
					$groupedProducts[] = $product_id;
					$groupedParticipants = array_merge($cp->getCRSOParticipants($crso_id,'array',false,$product_id),$groupedParticipants);
				}

				RefundRequestConfirmation::closeOpenRefundConfirmations($this->db,$crso_id,$product_id);
				ProofOfPaymentConfirmation::closeOpenPaymentConfirmations($this->db,$crso_id,$product_id);
			}
			if(count($groupedProducts)>0)
				foreach($groupedParticipants as $part_id)
					RefundExport::phaseOne($this->db, $part_id, $crso_id,$groupedProducts);
		}


		$log->info('RefundExport:: Starting phaseTwo...');

		### PHASE TWO ###
		global $langIf;
		$filewriter = new ELCExporter_RefundRequests($this->db, $log);
		$filewriter->export();

		$log->info('[TaskRunner::TaskExportRefundRequest] Done in '.round((m()-$start),3).' seconds');
	}


	/**
	 * @access private
	 */
	function _load()
	{
		//pass
	}
}
?>
