<?php
/**
 * Created on Jan 31, 2008
 *
 * @author Nicoleta Militaru <nicoleta@basesoft.com>
 **/

require_once(IWARE.'Task.php');
require_once(IWARE.'LanguageInterface.php');
require_once(ELC_CLASSES.'uk/UKCRSOData.php');
require_once(ELC_CLASSES.'Notification.php');
require_once(ELC_CLASSES.'CRSOProduct.php');
require_once(ELC_CLASSES.'Operator.php');
/**
 * @copyright 2006 BaseSoft B.V.
 * @since 2006-02-15
 * @author Marten Koedam <marten@basesoft.com>
 * @version $Id$
 */
class TaskUKExportActuals extends Task
{
	/**
	 * @var Database $db
	 */
	var $db;
	/**
	 *
	 */
	var $settings = array();
	/**
	 * @param Database $db
	 */
	function TaskExportDeclaration(&$db)
	{
		$this->db =& $db;
	}

	function execute(&$log)
	{
		$this->_load();
		$start = m();
		$date = currentDate();

		$cp = new CRSOProduct($this->db);
		$crso_data = new UKCRSOData($this->db);
		$langIf = new LanguageInterface($this->db,$this->db);
		$langIf->setLanguage('44');

		$operators = array();
		$op = new Operator($this->db);
		$qq = $op->selectByCountry('826');
		while($rr = $qq->fetch())
			$operators[] = array('recipient'=> new Relation($this->db, $rr->RELATION_ID));

		$month = $date->getMonth();
		$day = $date->getDay();
		$quarter = ceil((int)$month / 3) - 1;
		$year = $date->getYear();
		if($quarter == 0)
		{
			$quarter = 4;
			$year = $year - 1;
		}

		echo "Year: $year , Quarter: $quarter <br />";

		//get UK CRSOs
		$sql = "SELECT R.RELATION_ID as CRSO_RELATION_ID, R.`NAME`
				FROM TBL_CRSO as C
				INNER JOIN TBL_RELATION as R
					ON C.CRSO_RELATION_ID = R.RELATION_ID
				INNER JOIN TBL_ADDRESS as A
					USING(ADDRESS_ID)
				WHERE A.COUNTRY_ID = 826
				ORDER BY NAME";
		$q = $this->db->query($sql);
		$q->execute();
		while($r = $q->fetch())
		{
			$export_day = $cp->getMaxExportDay($r->CRSO_RELATION_ID);
			$export_month = ($quarter * 3) + 1;
			if($export_month == 13) $export_month = 1;
			if(($month  != $export_month) || ($day != $export_day))
			{
				$log->info(sprintf("The Actuals Report export month/day [%s/%s] for year [%s],
                     quarter [%s] for CRSO [%s] is not the current month/day.",
                     $export_month,$export_day,$year,$quarter,$r->CRSO_RELATION_ID));
				echo sprintf("The Actuals Report export month/day [%s/%s] for year [%s],
                     quarter [%s] for CRSO [%s] is not the current month/day.",
                     $export_month,$export_day,$year,$quarter,$r->CRSO_RELATION_ID);
				continue;
			}

			//send notification to operator
			echo "Send notification to operator for CRSO ".$r->NAME."  ... <br />";
			$body = $langIf->getText(" Data for Actuals Report is available.").' <br /><b>Year</b>:'.$year.', <b>Quarter</b>:'.$quarter;

			$message = array('subject'=> $langIf->getText('Quarterly Weee Actuals Report for CRSO').' '.$r->NAME,
					'message'=> $body,
					);
			Notification::send(
					$this->db,
					$message,
					$operators);

			//send notification to CRSO in case recycle data is not available
			if(!$crso_data->set($r->CRSO_RELATION_ID,$year,$quarter))
			{
				echo "Send reminder notification to CRSO ... <br />";
				$body = $langIf->getText(" Please add recycle data data").'  <br />For: <b>Year</b>:'.$year.', <b>Quarter</b>:'.$quarter;

				$message = array('subject'=> $langIf->getText('Recycle data is not entered'),
						'message'=> $body,
						);
				Notification::send(
						$this->db,
						$message,
						array(array('recipient'=> new Relation($this->db, $r->CRSO_RELATION_ID))));
			}
		}

		$log->info('[TaskRunner::TaskUKExportActuals] Done in '.round((m()-$start),3).' seconds');
	}
	/**
	 * @access private
	 */
	function _load()
	{

	}
}

?>
