<?php
require_once(IWARE.'Task.php');
require_once(ELC_CLASSES.'RelationUser.php');
require_once(ELC_CLASSES.'ELCMessageHandler.php');
require_once(ELC_CLASSES.'User.php');
/**
 * @package ELC
 * @subpackage Task
 * @copyright 2006 BaseSoft B.V.
 * @since 2006-07-10
 * @author Nick Loeve <nick@basesoft.com>
 * @version $Id: TaskExportDeclaration.php 3231 2006-07-05 14:37:59Z Nick Loeve $
 */
class TaskUnreadNotificationOverview extends Task
{
	/**
	 * @var Database $db
	 */
	var $db;
	/**
	 *
	 */
	var $settings = array();

	/**
	 * @param Database $db
	 */
	function TaskUnreadNotificationOverview(&$db)
	{
		$this->db =& $db;
		//$this->setSettings();
	}
	/**
	 *
	 */
	function setSettings()
	{
		$this->settings['db']['user'] = $this->db->user;
		$this->settings['db']['pass'] = $this->db->pass;
		$this->settings['db']['host'] = $this->db->host;
		$this->settings['db']['name'] = $this->db->name;
		$this->settings['db']['driver'] = $this->db->driver;
	}
	/**
	 *
	 */
	function execute(&$log)
	{
		// find all unread notifications and send an overview to CRSO or participant
		$this->_load();
		$start = m();

		//build the user -> notification map
		$users = $this->buildUsersNotifications($log);

		//now we need to send a message to each user
		$this->sendReminders($log, $users);

		$log->info('Done in '.round((m()-$start),3).' seconds');
	}

	function &getMessages()
	{
		//This query should get all the unread notifications for participants and crsos
		//we only check here for messages sent to a relation
		$sql = '
				SELECT DISTINCT
					nr.NOTIFICATION_ID,
					nr.RECIPIENT_ID,
					nr.USER_ID,
					nr.CLEARANCE_LEVEL,
					n.SUBJECT,
					r.RELATION_TYPE_ID
				FROM
					TBL_NOTIFICATION_RECIPIENT nr
				INNER JOIN
					TBL_NOTIFICATION n
					ON (nr.NOTIFICATION_ID=n.NOTIFICATION_ID)
				INNER JOIN
					TBL_RELATION r
					ON( r.RELATION_ID = nr.RECIPIENT_ID)
				WHERE
					nr.STATUS=1
				AND
					DATE_SUB(NOW(), INTERVAL 24 HOUR)  <= n.SENT_DATE
				AND
				  r.RELATION_TYPE_ID IN (2,3)
				ORDER BY
					nr.RECIPIENT_ID, nr.NOTIFICATION_ID';

		$q = $this->db->query($sql);
		if($q->execute())
			return $q;
		return false;
	}

	function getRecipients(&$r, &$users)
	{
		switch($r->{'RELATION_TYPE_ID'})
		{
			//participant
			case 2:
				//we need to check clearance level
				$sql = 'SELECT
						 u.USER_ID,
						 ip.INFO_PERSON_ID
						FROM
						  TBL_USER u
						INNER JOIN
						  TBL_INFO_PERSON ip
						ON
						 (ip.INFO_PERSON_ID = u.INFO_PERSON_ID)
						LEFT JOIN
						  TBL_RELATION_USER ru
						ON
						 (ru.USER_ID = u.USER_ID)
						WHERE
						 u.USER_ACTIVE = 1
						 AND ru.STATUS = 1
						 AND ru.RELATION_ID=?';
				$q = $this->db->query($sql);
				$q->bind_values($r->{'RECIPIENT_ID'});
				$q->execute();
				while($row = $q->fetch())
				{
					$ru = new RelationUser($this->db);
					if($r->{'CLEARANCE_LEVEL'} && !$ru->hasClearance($r->{'CLEARANCE_LEVEL'}, $row->{'USER_ID'}, $r->{'RECIPIENT_ID'}))
						continue;
					$users[$row->{'USER_ID'}][] = $r->{'SUBJECT'};
				}
				break;
			//crso
			case 3:
				$sql = 'SELECT
						 u.USER_ID,
						 ip.INFO_PERSON_ID
						FROM
						  TBL_USER u
						INNER JOIN
						  TBL_INFO_PERSON ip
						ON
						 (ip.INFO_PERSON_ID = u.INFO_PERSON_ID)
						LEFT JOIN
						  TBL_RELATION_USER ru
						ON
						 (ru.USER_ID = u.USER_ID)
						WHERE
						 u.USER_ACTIVE = 1
						 AND ru.STATUS = 1
						 AND ru.RELATION_ID=?';
				$q = $this->db->query($sql);
				$q->bind_values($r->{'RECIPIENT_ID'});
				$q->execute();
				while($row = $q->fetch())
				{
					$users[$row->{'USER_ID'}][] = $r->{'SUBJECT'};
				}
				break;
		}
	}

	function buildUsersNotifications(&$log)
	{
		$users = array();
		$q = $this->getMessages();
		$rs = $q->fetchAll();
		$log->info('Sending reminders for '.count($rs).' unread notifications');
		foreach($rs as $r)
		{
			$this->getRecipients($r, $users);
		}
		return $users;
	}

	function sendReminders(&$log, $users)
	{
		foreach($users as $uid => $subjectLines)
		{
			MessageEvent::send(MESSAGE_UNREAD_NOTIFICATION_REMINDER,
								array(
									'user'=> new ELCUser($this->db, $uid),
									'subjects'=> &$subjectLines
								),
								'ELCMessageHandler');
		}
	}

	/**
	 * @access private
	 */
	function _load()
	{
	//	$connstr = $this->settings['db']['driver'].':'.$this->settings['db']['host'].':'.$this->settings['db']['name'];
	//	$this->db = new Database($connstr, $this->settings['db']['user'], $this->settings['db']['pass'], 1);
	}
}

?>