<?php
	/**
	 * @copyright Copyright 2004, BaseSoft b.v.
	 * @author Martijn van den Broek <martijn@basesoft.com>
	 * @version $Id$
	 * @package I-Ware
	 * @subpackage Wizards
	 */

	/**
	 * Class WizardTemplate
	 * The filename of this class must have the same as this class + .php
	 */
	class WizardTemplate extends Wizard
	{
		/**
		 * Constructor
		 * Responsible setting basic stuff and creating all available WizardPages.
		 * Content and processing can be filled from callbacks.
		 * Instance of this class is made by /scripts/xx/wizard.php
		 */
		function WizardTemplate($wizard_id)
		{
			//-- Call parent constructor and set title
			parent::Wizard($wizard_id, 'Wizard Title');

			//-- The roadmap is the list on the left of the wizard.
			//   Set the key to the page handle, and value to title of the page.
			$this->setRoadmap( array( 'Init'=>'Welcome',
			                          //-- more pages for roadmap here
			                          'Finish'=>'Finish' ) );

			/*
				//-- Register one or more pages here, if content is dynamic use preProcess() callback
				$this->registerPage('FirstPage');
			*/

			//-- Finish Page (optional but recommended)
			$this->registerPage('Finish');
		}
		/**
		 * Gets called after the user pressed Cancel and confirmed the warning.
		 * Should echo JavaScript code here to change document location.
		 */
		function cancelProcess()
		{
			global $callId;
			?>
				var url = '';
				jsbroker.cancel( <?=$callId;?> );
				document.location.href = url;
			<?php
		}
		/**
		 * Gets called before a page is given back to the wizard.
		 * Here u can set content, change buttons etc.
		 * @param WizardPage
		 */
		function preProcess(&$page)
		{
			switch ( $page->pageHandle )
			{
				case 'Init':
					//-- title is shown in <h1> above the page
					$page->title = 'Init Pagina';
					$page->content = $this->getInitHtml();
				break;

				/*
				case 'MyPage':
					$page->content = 'Example Content';
				break
				*/

				case 'Finish':
					//--  hide next, show finish
					$page->hideButtons(BT_NEXT);
					$page->showButtons(BT_FINISH);

					$page->content = $this->getFinishHtml();
				break;
			}
		}

		/**
		 * PostProcess gets called after every page is submitted (like after previous, next or finish)
		 * @param WizardPage
		 * @param Array
		 */
		function postProcess(&$page, $params)
		{
			$postdata = $params['POST'];
			$button = $params['BUTTON'];

			switch ($page->pageHandle)
			{
				case 'Init':
				break;

				case 'Finish':
					//-- dont save if we pressed Previous
					if ($button == BT_PREV)
						return;

					//-- database code here

					?>
						MessageBox.prototype.Information('', '<?=jsescape($this->tr('The spaces have been created') ); ?>', function()
						{
							//-- wizard has finished, go to a new document..
							document.location.href = '';
						});
					<?php
				break;
			}
		}

		function getInitHtml()
		{
			return 'Example Init Page Body';
		}

		function getFinishHtml()
		{
			return 'Example Finish Page Body';
		}
	}

?>