<?php
	/**
	 * @copyright Copyright 2004, BaseSoft b.v.
	 * @author Martijn van den Broek <martijn@basesoft.com>
	 * @version $Id$
	 * @package I-Ware
	 * @subpackage Wizards
	 */

	class WizardTest extends Wizard
	{
		function WizardTest($wizard_id)
		{
			parent::Wizard($wizard_id);

			$this->setRoadmap( array( 'Init'=>'Welkom',
			                          'FirstPage'=>'Pages doorlopen',
			                          'FormPage'=>'Form invullen',
			                          'FinishPage'=>'Afronden' ) );

			$page = new WizardPage( $this->_wizard_id, 'Init pagina');
			$this->registerPage('Init',  $page);

			$page = new WizardPage($this->_wizard_id, 'Eerste pagina');
			//$page->next_page = 'SecondPage';
			$this->registerPage('FirstPage', $page );

			$page = new WizardPage($this->_wizard_id, 'Tweede pagina');
			$this->registerPage('SecondPage', $page );

			$page = new WizardPage($this->_wizard_id, 'Derde pagina');
			$this->registerPage('ThirdPage', $page );

			$page = new WizardPage($this->_wizard_id, 'Vierde pagina');
			$this->registerPage('FourthPage', $page );

			$page = new WizardPage($this->_wizard_id, 'Vijfde pagina');
			$this->registerPage('FifthPage', $page );

			//-- we'll fill this page in preProcess, it has dynamic content
			$page = new WizardPage($this->_wizard_id);
			$this->registerPage('FormPage', $page );

			$page = new WizardPage($this->_wizard_id);
			$page->buttons[BT_NEXT][BA_DISPLAY] = 0;
			$page->buttons[BT_FINISH][BA_DISPLAY] = 1;
			$this->registerPage('FinishPage', $page );

		}

		function getTitle()
		{
			return $this->langIf->getText('Wizard Title');
		}


		function preProcess(&$page)
		{
			switch ( $page->pageHandle )
			{
				case 'Init':
					$page->content = $this->getInitHtml();
				break;

				case 'FirstPage':
					$page->content = "Hello I am Page1";
				break;

				case 'FormPage':
					$page->content = $this->getPage2Html();
				break;

				case 'FinishPage':
					$page->content = $this->getFinishHtml();
				break;
			}
		}

		function postProcess(&$page, $params)
		{
			$postdata = $params['POST'];

			switch ($page->pageHandle)
			{
				case 'Init':
					$page->next_page = $postdata['GoTo'];
				break;

				case 'FormPage':
					$this->setWizVar('Name', $postdata['txtName']);
					$this->setWizVar('Address', $postdata['txtAddress']);

					$this->setWizVar('postdata', $postdata );
				break;

				case 'Finish':
					//-- finish code here
				break;
			}
		}

		function getInitHtml()
		{
			$html = '<h1>What do you want to do today?</h1> <p>Select one of the radio buttons and press Next</p>';

			$html .= '<input type="radio" name="GoTo" id="rdGotoPage1" value="FirstPage" style="vertical-align: middle;" checked>';
			$html .= '<label for="rdGotoPage1">Take me to Page #1</label> <br>';

			$html .= '<input type="radio" name="GoTo" id="rdGotoForm" value="FormPage" style="vertical-align: middle;">';
			$html .= '<label for="rdGotoForm">Take me to the Form</label> <br>';

			return $html;
		}

		function getPage2Html()
		{
			$a = array();
			$a[0][0] = 'Name';
			$a[0][1] = '<input type="text" name="txtName" value="'.$this->getWizVar('Name', 'Default Name').'">';

			$a[1][0] = 'Address';
			$a[1][1] = '<input type="text" name="txtAddress" value="'.$this->getWizVar('Address', 'Default Address').'">';

			$a[2][0] = 'arr text1';
			$a[2][1] = '<input type="text" name="text_arr[]" value="arrv1">';

			$a[3][0] = 'arr text1';
			$a[3][1] = '<input type="text" name="text_arr[]" value="arrv2">';

			$a[4][0] = 'Select';
			$a[4][1] = '<select name="mySelect">
			            	<option value="option1">Option 1</option>
			            	<option value="option2">Option 2</option>
			            	<option value="option3" selected>Option 3</option>
			            </select>';

			$a[4][0] = 'Select Array';
			$a[4][1] = '<select name="mySelectArr[]" multiple size=4>
			            	<option value="option1">Option 1</option>
			            	<option value="option2" selected>Option 2</option>
			            	<option value="option3">Option 3</option>
			            	<option value="option4" selected>Option 4</option>
			            </select>';

			$a[4][0] = 'Radio';
			$a[4][1] = '<input type="radio" name="myRadio" value="rd1"> Radio 1 <br>
			            <input type="radio" name="myRadio" value="rd2"> Radio 2 <br>
			            <input type="radio" name="myRadio" value="rd3"> Radio 3 <br>';

			$a[5][0] = 'Checkboxes';
			$a[5][1] = '<input type="checkbox" name="myCheckbox1" value="on"> Check 1 <br>
			            <input type="checkbox" name="myCheckbox2" value="on"> Check 2 <br>';

			$t = new TableFactory($a);
			$t->decapitate();

			return $t->getXhtmlTable();
		}

		function getFinishHtml()
		{
			$html = '<p>I have collected the following data, press Finish to save.</p>';
			$html .= '<pre>' . print_r($this->getWizVars(), true) . '</pre>';

			return $html;
		}
	}

?>